#!/usr/bin/perl -w
#  This is a daemon that monitors a perforce repository for changes.
#  For each change, it takes the list of files and generates a list of manifest
#  files that need to have their versions incremented, and increments them.

#  A manifest file gets it version incremented if:
#  +it has been edited
#  +something in the src/ directory below it has been edited
#  BUT NOT IF:
#  +it has has its version changed in that change 

#set debug if we need it
my $debug="false";

use strict;
use XML::DOM;

#setup stuff
#
my $sleeptime=10; 	#wake up and run every "sleeptime" seconds
$ENV{'P4USER'} = 'bob';
$ENV{'P4PASSWD'} = 'wendy';
$ENV{'P4PORT'} = 'isl-reengkm:1666';
$ENV{'P4CLIENT'} = 'manifestwatcher';

# p4 counter used to determine the last change id
# this is fixed and it's _really bad_ if you change it
my $perforce_change_counter_name="change";
# p4 counter used to determine last thing checked
my $perforce_manifest_counter_name="xmlcheck";

#main run loop begin
while ( 1 == 1 ) {
  #get the change lists that have happened recently
  my $perforce_change_number=`p4 counter $perforce_change_counter_name`;
  my $perforce_last_manifest_check=`p4 counter $perforce_manifest_counter_name`;
  chomp($perforce_change_number);
  chomp($perforce_last_manifest_check);

  if ($perforce_last_manifest_check < $perforce_change_number) {
    my $changeid = $perforce_last_manifest_check;
    my @manifests = my @unaltered_manifests = my @changed_manifests = ();
    @manifests = &getmanifestlistforchange($changeid); 
    if ( "$manifests[0]" ne "pending") {
      if ($debug) {print "Manifests edited: \n @manifests \n";}
      @unaltered_manifests = &checkmanifestlistforversionchange($changeid, @manifests);
      if ($debug) {print "Unaltered manifests: \n @unaltered_manifests \n";}
      foreach my $manifest_edit (@unaltered_manifests) {
        &incrementmanifestfile($manifest_edit); 
      }
      if ( scalar(@unaltered_manifests) > 0 ) { #if I tried to edit any
      #create and update the change description
      my $change_description=`p4 change -o`;   
      $change_description =~ 
        s/<enter description here>/Manifest update for change number $changeid./;   
      if ($debug) { print $change_description; };
      my $pid = open(INPUT, "| p4 submit -i") or die "Couldn't run p4 submit $!\n";
      print INPUT $change_description;
      close(INPUT);
    }  
      $changeid++; 	
      if ($debug) {print "Updating manifest counter to $changeid\n"; }
      `p4 counter $perforce_manifest_counter_name $changeid`;
    }
      else { # if there's pending change
        if ($changeid == $perforce_change_number) {
          # if it's pending and it's the last change number, we need to wait
          # if it's not, we can go on. 
  	  if ($debug) {print "Sleeping for a pending change";} 
          sleep($sleeptime); 
        }
        else {
          $changeid++; 	
          if ($debug) {print "Updating manifest counter to $changeid\n"; }
          `p4 counter $perforce_manifest_counter_name $changeid`;
        }
      }
  }
  else { #if build is caught up
      sleep($sleeptime);
  }
} #main loop end

exit 0;

sub checkmanifestlistforversionchange {
  my $changeid = shift @_;
  my @given_manifests = @_;
  my @manifests_with_no_version_change;
  #print "checking for version $changeid with these manifests \n @manifests\n";
  #first we get the old and new versions of the manifests
  foreach my $manifest_file (@given_manifests) {
    my $prevchangeid = ($changeid - 1);
    #print "Comparing manifest $changeid to previous version $prevchangeid\n";
    my $oldmanifest = `p4 print -q $manifest_file\@$prevchangeid`;
    my $newmanifest = `p4 print -q $manifest_file\@$changeid`;
    #if ($debug) { print "Old manifest: \n $oldmanifest \n"; }
    #if ($debug) { print "New manifest: \n $newmanifest \n"; }
  # We only want to make the change if there's both and old version
  # and a new version, and the version number doesn't change
  # If there's no old version, manifest is new in this build
  # If there's no new version, manifest has been deleted.
  # If there's no old or new version, then there's not a manifest yet
  # If there's a version change between the two, then we leave be
    if ( ($newmanifest ne "") || ($oldmanifest ne "")) {
        my $newversion;
        my $oldversion;
        if ( $newmanifest =~ /\<plugin.+?version=\"([\d\.]+)\"/s ) {
        $newversion = $1;
	if ($debug) {print "New version number is $newversion\n"; }
        }
        if ( $oldmanifest =~ /\<plugin.+?version=\"([\d\.]+)\"/s ) {
        $oldversion = $1;
        if ($debug) { print "Old version number is $oldversion\n"; }
        }
        if ( $oldversion && $newversion && $oldversion eq $newversion ) {
          push (@manifests_with_no_version_change, $manifest_file);         
        }
      }
    }
  return @manifests_with_no_version_change;
}

#####################

sub incrementmanifestfile {
my $manifest_to_edit = shift @_; 
print "Incrementing the manifest file $manifest_to_edit\n";
`p4 sync $manifest_to_edit`;
  if ( $? ne 0) { 
  print "Error in retrieving manifest file $manifest_to_edit";
  return 1;
  }
#this gets the name of the local file from its perforce name
my $returned_path = `p4 where $manifest_to_edit`;
my @returned_path_files = split(/ /, $returned_path);
my $local_file = $returned_path_files[2];
chomp($local_file);

if ($debug) {print "Local file to be edited is $local_file\n";}
`p4 edit $local_file`;
  if ( $? ne 0) { 
  print "Error in editing manifest file $local_file";
  return 1;
}
if (-e $local_file) { 
#actual editing logic goes here
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parsefile ($local_file);
  my $nodes = $doc->getElementsByTagName ("plugin");
  my $node = $nodes->item(0);
  my $name = $nodes->item(0)->getFirstChild()->getNodeValue();
  my $id = $node->getAttributeNode ("id")->getValue();
  my $version = $node->getAttributeNode ("version")->getValue();

  my $major = my $minor = my $build = 0;
  my $stuff;
  print "Version in file is $version\n";
  if ($version =~ /([0-9]+)\.*([0-9]*)\.*([0-9]*)/) {
    $major = $major + $1;
    $minor = $minor + $2;
    $build = $build + $3;
    #print "Having parsed $local_file, the old version is\n Major: $major\n";
    #print "Minor: $minor \n Build: $build \n Stuff: $stuff\n";
    $build++; 		#the key line in the program. :)
    print "New version would be $major.$minor.$build\n";
    my $newversion = "$major.$minor.$build";
   $node->getAttributeNode ("version")->setValue($newversion);
    $doc->printToFile("$local_file");
  }  
} else {
  print "File $local_file can't be edited\n";
}
}

####################

sub getmanifestlistforchange
{
  my @uniq_manifest_list=(); #this is the return value
  if ($debug) { print "Current change list is $_[0] \n"; }
  my $changelist_number = $_[0];
  my $change_list_contents = `p4 describe -s $changelist_number`;
#if ($debug) { print $change_list_contents; }
  if ($change_list_contents =~ /\*pending\*/) { 
  push (@uniq_manifest_list, "pending");
  return @uniq_manifest_list;
  }
  my @changelisttemp = split(/Affected files \.\.\./, $change_list_contents);
# this assumes that everything following the "Files: command is potentially 
# in the list of files
  if ($changelisttemp[1] eq "") {
    print "No Files specified for this changelist\n";
  }
  if ($debug) { print $changelisttemp[1]; }

#get the list of files edited by splitting it up into words and filtering the
#words that don't contain a "\\" in them
my @change_list_file_item_list = split(/\s/, $changelisttemp[1]);
my @changelistfiles = ();

foreach my $potential_file (@change_list_file_item_list) {
  if ($potential_file =~ /\//) {
    push(@changelistfiles, $potential_file);
}
}
if ($debug) { print "Files changed: @changelistfiles"; print "\n"; } 

my @manifest_list;
#navigate through all of those to get a list of plugins that have been modified
foreach my $file (@changelistfiles) {
  my @filepieces = split(/src\//, $file);
  if ($filepieces[0] eq $file ) { # if it's the same, then there's no /src/
# to see if it's a manifest
    if ( $file =~ /(^.+modules.+manifest.xml)/) { 
       #if it's somestuff then modules, then manifest.xml
      push(@manifest_list, $1); #the $1 matches everything up til manifest.xml
    }
  }
  else { #if it's got something in /src/
         #then the manifest file is (everything to src)/manifest.xml
    my $manifest_file="$filepieces[0]manifest.xml"; 
    push(@manifest_list, $manifest_file);  
  }
}
if ($debug) { print "List of manifests\n @manifest_list \n"; }
#now we do a simple loop that uniqifies the list (from Perl Cookbook)
my %seen = ();
foreach my $item (@manifest_list) {
  push(@uniq_manifest_list, $item) unless $seen{$item}++;
}
if ($debug) { print "List of unique manifests:\n @uniq_manifest_list \n"; }
return @uniq_manifest_list;
} #end getmanifestlist
