package gov.va.med.hds.cd.ccow.internal;


import java.awt.Component;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sentillion.sdkweb.webcontextor.GeneralFailureException;
import com.sentillion.sdkweb.webcontextor.WebContextorException;

import gov.va.med.hds.cd.ccow.ContextItemNameFactory;
import gov.va.med.hds.cd.ccow.IContextItemName;
import gov.va.med.hds.cd.ccow.IContextMapper;
import gov.va.med.hds.cd.ccow.ui.CCOWContinuationDialog;
import gov.va.med.hds.util.os.OSUtils;

class CCOWHyperlinkBroker {

    public static final int SUCCESS = 0;
    public static final int USER_CANCELLED = 1;
    public static final int CPRS_UNAVAILABLE = 2;
    public static final int CCOW_LOCATOR_UNAVAILABLE = 6;
    public static final int CCOW_UNAVAILABLE = 3;
    public static final int CCOW_FAILURE = 4;
    public static final int BAD_CONTEXT = 5;

               
    

    private static final String FIRST_PASS_VALUE = "GetWindowHandle";
    static final String CUSTOM_SUBJECT_NAME = "[hds_med_va.gov]request";
    static final String CUSTOM_CONTEXT_ITEM_NAME =
        CUSTOM_SUBJECT_NAME + ".id.name";
    private static final String HYPERLINK_REPLY_PREFIX = "!@#$";

  
    private String appName;
    private String appPasscode;
    private IContextMapper mapper;  
    private CCOWParticipant participant;

    public CCOWHyperlinkBroker(String appName, String appPasscode, IContextMapper m) {
        this.appName = appName;
        this.appPasscode = appPasscode;
        this.mapper = m;
        initializeParticipant();
    }

    public void dispose() {
        if (participant != null)
            participant.dispose();
        participant = null;
    }

    private synchronized int initializeParticipant() {
        if (participant == null)
            try {
				participant = new CCOWParticipant(appName, appPasscode);
            } catch (GeneralFailureException gfe) {
                return CCOW_LOCATOR_UNAVAILABLE;
            } catch (WebContextorException wce) {
                participant = null;
                return CCOW_UNAVAILABLE;
            }
        return SUCCESS;
    }

    public int browse(Component parent, Object hyperlink) {
        Map contextHyperlink = translateHyperlink(hyperlink);
        if (contextHyperlink == null)
            return BAD_CONTEXT;

        if (mapper != null) mapper.mapSubjects(contextHyperlink);

        int result = initializeParticipant();
        if (participant == null)
            return result;

        return openHyperlink(parent, contextHyperlink);
    }

    Map translateHyperlink(Object hyperlink) {
        if (hyperlink instanceof String)
            return parseStringHyperlink((String) hyperlink);

        return null;
    }

    Map parseStringHyperlink(String hyperlink) {
        int questPos = hyperlink.indexOf('?');
        if (questPos != -1)
            hyperlink = hyperlink.substring(questPos);

        Map items = TinyWebServer.parseParams(hyperlink);
        items = InternalCCOWContext.canonicalizeContext(items);
        return items;
    }

    int openHyperlink(Component parent, Map contextHyperlink) {
        try {
            Map newContext = new HashMap(contextHyperlink);
            IContextItemName itemName =
                ContextItemNameFactory.getName(CUSTOM_CONTEXT_ITEM_NAME);

            newContext.put(itemName, FIRST_PASS_VALUE);
            ContextChangeProposalResponse response =
                participant.proposeContextChange(newContext);
            participant.cancelContextChange();

            List hyperlinkResponses = new ArrayList();
            extractHyperlinkResponses(response, hyperlinkResponses);
            long bestApp = getBestAppToDisplay(hyperlinkResponses);

            if (bestApp == -1)
                return launchAppToDisplay(parent, contextHyperlink);
            else {
                return followHyperlink(parent, contextHyperlink, bestApp);
            }

        } catch (WebContextorException wce) {
            wce.printStackTrace();
            participant.dispose();
            participant = null;
        } catch (Throwable t) {
        }
        return CCOW_FAILURE;
    }

    void extractHyperlinkResponses(
        ContextChangeProposalResponse response,
        List hyperlinkResponses) {
        // handle simplest case: no survey responses at all.
        if (response.surveyResponses == null)
            response.surveyResponses = new String[0];
        if (response.surveyResponses.length == 0)
            return;

        List ccowResponses = new ArrayList();
        Pattern p =
            Pattern.compile(
                "(.*)\\Q" + HYPERLINK_REPLY_PREFIX + "\\E(.*):(.*):(.*)");

        for (int i = 0; i < response.surveyResponses.length; i++) {
            String reply = response.surveyResponses[i];
            if (reply == null)
                continue;

            Matcher m = p.matcher(reply);
            if (m.matches()) {
                String ccowPrefix = m.group(1);
                String appID = m.group(2);
                String changeDelay = m.group(3);
                String realReply = m.group(4);

                hyperlinkResponses.add(
                    new HyperlinkResponse(appID, changeDelay));
                if (realReply.trim().length() == 0)
                    reply = null;
                else
                    reply = ccowPrefix + realReply;
            }

            if (reply != null && reply.trim().length() != 0)
                ccowResponses.add(reply.trim());
        }

        String[] surveyResponses = new String[ccowResponses.size()];
        surveyResponses = (String[]) ccowResponses.toArray(surveyResponses);
        response.surveyResponses = surveyResponses;
    }

    boolean oneMatch(List hyperlinkResponses, String target) {
        if (FIRST_PASS_VALUE.equals(target))
            return hyperlinkResponses.size() == 1;

        int count = 0;
        Iterator i = hyperlinkResponses.iterator();
        while (i.hasNext()) {
            HyperlinkResponse r = (HyperlinkResponse) i.next();
            if (target.equals(Long.toString(r.appID)))
                count++;
        }
        return (count == 1);
    }

    long getBestAppToDisplay(List hyperlinkResponses) {
        if (hyperlinkResponses.isEmpty())
            return -1;

        Collections.sort(hyperlinkResponses);
        HyperlinkResponse result =
            (HyperlinkResponse) hyperlinkResponses.get(0);
        return result.appID;
    }

    int followHyperlink(Component parent, Map newContext, long windowID) 
        throws WebContextorException 
    {
        boolean needToCancel = false;
        try {
            ContextChangeProposalResponse response =
                participant.proposeContextChange(newContext);
            needToCancel = true;
            
            // display the survey responses if necessary.
            int nextStep =
                CCOWContinuationDialog.show(parent,
                    response.surveyResponses,
                    response.noContinue,
                    true);
            
            // either commit or cancel the context transaction based on user
            // response.
            if (nextStep == CCOWContinuationDialog.CANCEL) {
                participant.cancelContextChange();
                needToCancel = false;
                return USER_CANCELLED;
            } if (nextStep == CCOWContinuationDialog.CONTINUE) {
                participant.commitContextChange();
                needToCancel = false;
                try {
                    if (windowID != -1)
                        OSUtils.setAndRestoreForegroundWindow(windowID);
                } catch (Throwable t) {
                    t.printStackTrace();
                }
                return SUCCESS;
            }
            
            participant.cancelContextChange();
            needToCancel = false;
            return CCOW_FAILURE;
        } finally {
            if (needToCancel)
                participant.cancelContextChange();
        }
    }

    int launchAppToDisplay(Component parent, Map newContext) 
        throws WebContextorException 
    {
        int result = followHyperlink(parent, newContext, -1);
        if (result != SUCCESS) return result;
        
        if (ChartLauncher.launchChart())
            return SUCCESS;
        else
            return CPRS_UNAVAILABLE;
    }

    class HyperlinkResponse implements Comparable {
        public long appID;
        public int changeDelay;
        public HyperlinkResponse(String id, String delay)
            throws NumberFormatException {
            this.appID = Long.parseLong(id);
            this.changeDelay = Integer.parseInt(delay);
        }
        public int compareTo(Object o) {
            if (o instanceof HyperlinkResponse) {
                HyperlinkResponse that = (HyperlinkResponse) o;
                if (this.changeDelay < that.changeDelay)
                    return -1;
                if (this.changeDelay > that.changeDelay)
                    return 1;
                if (this.appID < that.appID)
                    return -1;
                if (this.appID == that.appID)
                    return 0;
            }
            return 1;
        }
    }

}
