

package gov.va.med.hds.cd.ccow.internal;

import gov.va.med.hds.cd.ccow.*;
import com.sentillion.sdkweb.webcontextor.BadItemNameException;
import com.sentillion.sdkweb.webcontextor.ContextItem;


/** This class implements the {@link ContextItemName} interface for use
 * by clinician desktop components.
 *
 * The Sentillion SDK provides a class called "ContextItem" which contains
 * some of this functionality.  Although we leverage what we can from that
 * class, we do not use it outright because:<ul>
 *
 * <li>If developers had to create instances of Sentillion's ContextItem
 *    class, this would tightly couple all the code to Sentillion's
 *    implementation of the SDK, and introduce an unnecessary vendor
 *    dependency.
 *
 * <li>The Sentillion class is mutable, which makes it unusable in
 *    collections like Maps, and makes it generally dangerous for safe
 *    programming.
 *
 * <li>The Sentillion class captures both a name and its associated value,
 *    which prevents efficient reuse of context item names globally.
 *
 * </ul>
 */
public class ContextItemNameImpl implements IContextItemName {

    private String fullName;
    private String fullNameLowerCase;
    private String subject;
    private String role;
    private String prefix;
    private String suffix;

    /** Create a new ContextItemNameImpl object with the given name. */
    public ContextItemNameImpl(String fullName) {
        this.fullName = fullName;
        fullNameLowerCase = fullName.toLowerCase();

        try {
            // use the Sentillion class to parse the name
            ContextItem item = new ContextItem(fullName, "");
            subject = item.getSubject();
            role    = item.getRole();
            prefix  = item.getPrefix();
            suffix  = item.getSuffix();
        } catch (BadItemNameException bine) {}
    }

    /** @return the full name of a context item. */
    public String getName() { return fullName; }

    /** @return the subject field of a context item name. */
    public String getSubject() { return subject; }

    /** @return the role field of a context item name. */
    public String getRole() { return role; }

    /** @return the prefix field of a context item name. */
    public String getPrefix() { return prefix; }

    /** @return the suffix field of a context item name. */
    public String getSuffix() { return suffix; }

    /** Compare this object to another for equality. */
    public boolean equals(Object o) {        
        if (o instanceof ContextItemNameImpl) {
            ContextItemNameImpl that = (ContextItemNameImpl) o;
            if (this == that) return true;
            return this.fullNameLowerCase.equals(that.fullNameLowerCase);
        } else if (o == null)
            return false;
        else
            return fullName.equalsIgnoreCase(String.valueOf(o));
    }

    /** Compute the hash code for this ContextItemNameImpl object. */
    public int hashCode() {
        return fullNameLowerCase.hashCode();
    }

    public String toString() { return fullName; }

}
