package gov.va.med.hds.cd.ccow.internal;

import gov.va.med.hds.cd.ccow.IClinicalContextBroker;
import gov.va.med.hds.cd.ccow.IContextMapper;
import gov.va.med.hds.cd.ccow.IContextObserver;
import gov.va.med.hds.cd.ccow.IContextModule;

import java.awt.Component;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ContextModule implements IContextModule, IContextMapper {

    private HashMap BROKER_INSTANCES = new HashMap();
    private CCOWHyperlinkBroker hyperlinkBroker = null;
    protected List contextMappers = null;
    
    private String appName;
    private String appPasscode;
    
    
    public ContextModule(String appName, String appPasscode) {
        this.appName = appName;
        this.appPasscode = appPasscode; 
    } 

    
    public void shutdown() {
        Iterator i = BROKER_INSTANCES.values().iterator();
        while (i.hasNext())
            try {
                ((CCOWBroker) i.next()).dispose();
            } catch (Throwable t) {
            }
        BROKER_INSTANCES.clear();

        if (hyperlinkBroker != null) {
            hyperlinkBroker.dispose();
            hyperlinkBroker = null;
        }
    }

    // implementation of the IContextModule interface

    /** Get the {@link ClinicalContextBroker} instance for a given
     * clinician desktop window.
     *
     * @param forWindow The clinician desktop window for which we
     *    need a context broker.
     * @param observer The {@link ContextObserver} object to notify
     *    when the clinical context changes. (May be null)
     * @return The context broker for the given window.  If the window
     *    does not yet have a context broker, one is created
     *    automatically.  If it already has a context broker, the
     *    existing broker is returned.
     */
    public IClinicalContextBroker getBroker(
        Object forWindow,
        IContextObserver observer) {
        if (forWindow == null)
            return null;

        synchronized (BROKER_INSTANCES) {
            CCOWBroker result = (CCOWBroker) BROKER_INSTANCES.get(forWindow);

            if (result == null) {
                try {
                    result = new CCOWBroker(appName, appPasscode, this);
                } catch (Exception ioe) {
                    ioe.printStackTrace();
                    return null;
                }

                BROKER_INSTANCES.put(forWindow, result);
            }

            if (observer != null)
                result.addContextObserver(observer);

            return result;
        }
    }

    public String browseCCOWHyperlink(Object hyperlink, Component parent) {
        synchronized (this) {
            if (hyperlinkBroker == null)
				hyperlinkBroker =
					new CCOWHyperlinkBroker(appName, appPasscode, this);
        }

        switch (hyperlinkBroker.browse(parent, hyperlink)) {
            case CCOWHyperlinkBroker.SUCCESS :
                return BROWSE_SUCCESS;
            case CCOWHyperlinkBroker.USER_CANCELLED :
                return BROWSE_CANCELLED;
            case CCOWHyperlinkBroker.CPRS_UNAVAILABLE :            
                return CPRS_UNAVAILABLE_MSG;
            case CCOWHyperlinkBroker.CCOW_LOCATOR_UNAVAILABLE:
                return LOCATOR_UNAVAILABLE_MSG;
            case CCOWHyperlinkBroker.CCOW_UNAVAILABLE :
                return CCOW_UNAVAILABLE_MSG;
            default :
                return BROWSE_ERROR_MSG;
        }
    }

    private static final String CPRS_UNAVAILABLE_MSG =
        "Sorry, this feature is unavailable because you do "
            + "not have a CCOW-enabled version of CPRS installed "
            + "on your computer.";
    private static final String LOCATOR_UNAVAILABLE_MSG =
        "Sorry, this feature is unavailable because the "
            + "CCOW Desktop Components are not installed and running "
            + "on your computer.";
    private static final String CCOW_UNAVAILABLE_MSG =
        "Sorry, this feature is unavailable because the "
            + "CCOW clinical context manager could not be reached.";
    private static final String BROWSE_ERROR_MSG =
        "Sorry, this program encountered an unexpected error which "
            + "prevented it from following the hyperlink.";

    /** @link dependency */
    /*#CCOWBroker lnkCCOWBroker;*/


    /** Run all the context mappers on the given context.
     *
     * @param context a context in canonical form.
     */
    public boolean mapSubjects(Map context) {
        if (contextMappers == null) initContextMappers();
        if (contextMappers == null) return false;
        
        boolean changesMade, anyChangesMade = false;
        int iterations = contextMappers.size();
        do {
            changesMade = false;
            Iterator i = contextMappers.iterator();
            while (i.hasNext()) {
                IContextMapper mapper = (IContextMapper) i.next();
                if (mapper.mapSubjects(context))
                    changesMade = anyChangesMade = true;
            }
            iterations--;
        } while (changesMade && iterations > 0);
        return anyChangesMade;
    }


    protected void initContextMappers() {}

	public String getCcowHref(Object patient) {
        return null;
	}
    
}
