

package gov.va.med.hds.cd.ccow.internal;

import java.util.*;

import gov.va.med.hds.cd.ccow.*;


public class InternalCCOWContext {

    private Map context;
    private IContextMapper mapper;

    public InternalCCOWContext(IContextMapper mapper) { this(createEmptyContext(), mapper); }

    public InternalCCOWContext(Map initialContext, IContextMapper mapper) {
        this.context = initialContext;
        this.mapper = mapper;
    }

    public void replaceContext(Map newContext) {
        if (newContext == null)
            this.context = createEmptyContext();
        else
            this.context = newContext;
    }

    public Map getContext() { return context; }

    protected void runMappingAgent(Map unmappedContext) {
        if (mapper != null)
            mapper.mapSubjects(unmappedContext);
    }

    protected void deleteDependentItems(Map originalMap,
                                        Set changingItemNames) {
        Iterator orig = originalMap.keySet().iterator();

        while (orig.hasNext()) {
            IContextItemName origName = (IContextItemName) orig.next();
            Iterator changes = changingItemNames.iterator();

            while (changes.hasNext()) {
                IContextItemName changingName =
                    (IContextItemName) changes.next();
                if (itemIsDependent(changingName, origName)) {
                    orig.remove();
                    break;
                }
            }
        }

    }
    protected boolean itemIsDependent(IContextItemName name,
                                      IContextItemName dependentName) {
        String subj = name.getSubject();
        String dependentSubj = dependentName.getSubject();
        if ("user".equalsIgnoreCase(dependentSubj))
            return false;
        if ("patient".equalsIgnoreCase(subj))
            return true;
        if (subj.equalsIgnoreCase(dependentSubj))
            return true;

        return false;
    }

    protected Map deriveNewContext(Map originalContext, Map newContextItems) {
        Map result = new HashMap();
        result.putAll(originalContext);
        deleteDependentItems(result, newContextItems.keySet());
        result.putAll(newContextItems);
        runMappingAgent(result);

        return Collections.unmodifiableMap(result);
    }

    public synchronized Map proposeContextChange(Map newContextItems) {
        newContextItems = canonicalizeContext(newContextItems);
        return deriveNewContext(context, newContextItems);
    }


    protected static Map createEmptyContext() {
        return Collections.unmodifiableMap(new HashMap());
    }



    /** Translates a set of context item names into canonical form.
     *
     * Any objects in the set which are not of type {@link ContextItemName}
     * are translated into {@link ContextItemName} objects.
     */
    public static Set canonicalizeContextItemNames(Set contextItemNames) {
        Set result = new HashSet();
        Iterator i = contextItemNames.iterator();
        while (i.hasNext()) {
            Object name = i.next();
            if (name == null) continue;
            if (name instanceof IContextItemName) result.add(name);
            result.add(ContextItemNameFactory.getName(name.toString()));
        }
        return result;
    }


    /** Translates a context into canonical form.
     */
    public static Map canonicalizeContext(Map ctxt) {
        Map result = new HashMap();
        if (ctxt == null) return result;

        Iterator i = ctxt.entrySet().iterator();
        Map.Entry e;
        while (i.hasNext()) {
            e = (Map.Entry) i.next();
            Object k = e.getKey(), v = e.getValue();

            IContextItemName name = null;
            if (k == null) continue;
            if (k instanceof IContextItemName)
                name = (IContextItemName) k;
            else
                name = ContextItemNameFactory.getName(k.toString());

            String value = null;
            if (e.getValue() != null)
                value = e.getValue().toString();

            result.put(name, value);
        }

        return result;
    }



}
