
package gov.va.med.hds.cd.ccow.internal;

/** An object must implement this interface to receive CCOW messages from
 * the {@link CCOWParticipantServer}.
 */
public interface SimpleCCOWParticipant {


        /** This method is called by the {@link CCOWParticipantServer} to
         * indicate that a "context changes pending" message has been
         * received from the CCOW context manager.
         *
         * This method should return as quickly as possible. In particular,
         * this object is not permitted to attempt to interact with the
         * user in response to this method call.
         *
         * Also note that the context change has not yet occurred.
         *
         * @param pendingContextCoupon the context coupon of the pending
         *    context; this can be used to inspect the contents of the new
         *    context
         * @return <code>null</code> if this {@link CCOWParticipantServer}
         *    can apply the change without any adverse consequences.  If
         *    the suggested context change would cause a loss of data, this
         *    method should return a string describing the consequences of
         *    the context change.  This string will be displayed to the user,
         *    so it should be user-friendly.
         */
        public String contextChangesPending(long pendingContextCoupon);



        /** This method is called by the {@link CCOWParticipantServer}
         * to indicate that a "context changes accepted" message has
         * been received from the CCOW context manager.
         *
         * In practical terms, this means that the clinical context has
         * been changed.
         *
         * @param newContextCoupon the context coupon of the new context;
         *    this can be used to retrieve the contents of the new context.
         */
        public void contextChangesAccepted(long newContextCoupon);



        /** This method is called by the {@link CCOWParticipantServer} to
         * indicate that a "context changes canceled" message has been
         * received from the CCOW context manager.
         *
         * In practical terms, this means that the context change suggested
         * by a previous
         * {@link #contextChangesPending() contextChangesPending} method
         * call has been canceled by the user.
         *
         * @param canceledContextCoupon the coupon of the pending context
         *    which was cancelled.  This is provided for information only,
         *    as the contents of the canceled context are most likely
         *    no longer available.
         */
        public void contextChangesCanceled(long canceledContextCoupon);



        /** This method is called by the {@link CCOWParticipantServer} to
         * indicate that a "common context terminated" message has been
         * received from the CCOW context manager.
         *
         * In practical terms, this means that the clinical context is
         * no longer accessible.  */
        public void commonContextTerminated();



        /** This method is called by the {@link CCOWParticipantServer} to
         * indicate that a "ping" message has been received from the CCOW
         * context manager.
         *
         * In general, this object should simply return immediately.
         */
        public void ping();

}
