/*
 * Created on May 21, 2003
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.ccow.desktop;

import gov.va.med.hds.cd.ccow.*;
import gov.va.med.hds.cd.ccow.ContextItemNameFactory;
import gov.va.med.hds.cd.ccow.IContextItemName;
import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.util.dao.ISiteDAO;
import gov.va.med.hds.util.mumps.StringUtil;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * @author vhaisltumad
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class MPatientMapperDAO implements IMPatientMapperDAO {

	private static final String DFN_ITEM_NAME_PREFIX = "Patient.Id.MRN.DFN_";
	private static final IContextItemName DFN_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Id.MRN.DFN");
	private static IContextItemName QUALIFIED_DFN_ITEM_NAME;

	private static final IContextItemName ICN_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Id.MRN.NationalIdNumber");
	private static final IContextItemName PT_NAME_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Co.PatientName");
	private static final IContextItemName PT_DOB_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Co.DateTimeofBirth");
	private static final IContextItemName PT_SEX_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Co.Sex");
	private static final IContextItemName PT_SSN_ITEM_NAME =
		ContextItemNameFactory.getName("Patient.Co.SSN");

	private static final String ICN_TO_DFN_RPC_KEY = "CONVERT ICN TO DFN";
	private static final String DFN_LOOKUP_RPC_KEY = "SELECT";

	private IRuntimeContext runtime;
	private IMumps mumps;
	private String stationID;

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {

		this.runtime = runtime;

		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor(
				ICCowDesktopCoreConstants.PLUGIN_ID);
		mumps =
			(
				(IMumpsFactory) runtime.getPluginInstance(
					IMumpsFactory.PLUGIN_ID)).create(
				desc);

		// look up the station ID.
		IDaoRegistry daoRegistry =
			(IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
		ISiteDAO siteDao =
			(ISiteDAO) daoRegistry.getDAO(ISiteDAO.class.getName());
		stationID = siteDao.getPrimaryStationID();
        // only the first three characters of the station id are relevant
        // for CCOW synchronization.
        if (stationID != null && stationID.length() > 3)
            stationID = stationID.substring(0, 3);

		QUALIFIED_DFN_ITEM_NAME =
			ContextItemNameFactory.getName(DFN_ITEM_NAME_PREFIX + stationID);
	}

	public boolean mapSubjects(Map context) {
		boolean result = false;
		if (mapICNtoDFN(context))
			result = true;
		if (mapDFNtoMisc(context))
			result = true;

		return result;
	}

	private boolean mapICNtoDFN(Map context) {
		// if the context already has a value for the DFN, do nothing.
		if (context.containsKey(QUALIFIED_DFN_ITEM_NAME))
			return false;

		// look up the ICN.  If it is missing, do nothing.
		String icn = (String) context.get(ICN_ITEM_NAME);
		if (icn == null)
			return false;

		try {
			// translate the ICN to a DFN, and store it in the context.
			String dfn =
				mumps.callRPC(ICN_TO_DFN_RPC_KEY, new String[] { icn });
			if (dfn != null) {
				context.put(QUALIFIED_DFN_ITEM_NAME, dfn);
				return true;
			}
		} catch (MumpsException e) {
		}

		return false;
	}

	private boolean mapDFNtoMisc(Map context) {
		boolean changedDFN = false;
		if (context.containsKey(DFN_ITEM_NAME)
			&& !context.containsKey(QUALIFIED_DFN_ITEM_NAME)) {
			context.put(QUALIFIED_DFN_ITEM_NAME,
    			// discard unqualified DFN subject
    			context.remove(DFN_ITEM_NAME));
			changedDFN = true;
		}

		int contextSize = context.size();
		String dfn = (String) context.get(QUALIFIED_DFN_ITEM_NAME);
		if (dfn != null
			&& (!context.containsKey(ICN_ITEM_NAME)
				|| !context.containsKey(PT_NAME_ITEM_NAME)
				|| !context.containsKey(PT_DOB_ITEM_NAME)
				|| !context.containsKey(PT_SEX_ITEM_NAME)
				|| !context.containsKey(PT_SSN_ITEM_NAME))) {

			try {
				String results =
					mumps.callRPC(DFN_LOOKUP_RPC_KEY, new String[] { dfn });
				if (results != null) {
					String[] data = StringUtil.split(results, '^');

					storePtICN(context, data[13]);
					storePtName(context, data[0]);
					storePtDOB(context, data[2]);
					storePtSex(context, data[1]);
					storePtSSN(context, data[3]);
				}
			} catch (MumpsException e) {
				runtime.getLogger(MPatientMapperDAO.class).error("Unable to lookup information for patient DFN",e);
			}

		}

		return changedDFN || (context.size() > contextSize);
	}

	private boolean empty(String str) {
		return (str == null || str.length() == 0);
	}

	private void storePtICN(Map context, String icn) {
		if (empty(icn))
			return;
            
		context.put(ICN_ITEM_NAME, icn);
	}

	private void storePtName(Map context, String name) {
		if (empty(name))
			return;

        // FIXME this does not identify and separate middle name/initial
        int pos = name.indexOf(',');
        if (pos != -1)
            name = name.substring(0, pos) + "^" + name.substring(pos+1) + "^^^^";
        else
            name = name + "^^^^^";
		context.put(PT_NAME_ITEM_NAME, name);
	}

    private static final DateFormat DOB_FMT = new SimpleDateFormat("yyyyMMdd");
	private void storePtDOB(Map context, String dob) {
		if (empty(dob))
			return;

        try {
            Date date = StringUtil.fmDateToDate(dob);
            context.put(PT_DOB_ITEM_NAME, DOB_FMT.format(date));
        } catch (Exception e) {
        }
	}

	private void storePtSex(Map context, String sex) {
		if (empty(sex))
			return;

		context.put(PT_SEX_ITEM_NAME, sex);
	}

	private void storePtSSN(Map context, String ssn) {
		if (empty(ssn))
			return;
            
		context.put(PT_SSN_ITEM_NAME, ssn);
	}

}
