package gov.va.med.hds.cd.ccow.internal;

import gov.va.med.hds.cd.ccow.ICCowDesktopCoreConstants;
import gov.va.med.hds.cd.ccow.IContextItemName;
import gov.va.med.hds.cd.ccow.ContextItemNameFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

public class CCOWHyperlinkBrokerTest extends TestCase {

    public CCOWHyperlinkBrokerTest(String arg0) {
        super(arg0);
        EXPECTED_CONTEXT.put(CNAME1, VALUE1);
        EXPECTED_CONTEXT.put(CNAME2, VALUE2);
        EXPECTED_CONTEXT.put(CNAME3, VALUE3);
    }
    
    private CCOWHyperlinkBroker broker;

    protected void setUp() throws Exception {
        super.setUp();
		broker =
			new CCOWHyperlinkBroker(
				ICCowDesktopCoreConstants.APP_NAME,
				ICCowDesktopCoreConstants.APP_PASSCODE,
				null);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        broker.dispose();
        broker = null;
    }

    String NAME1 = "patient.id.mrn.vistaICN";
    IContextItemName CNAME1 = ContextItemNameFactory.getName(NAME1);
    String VALUE1 = "123698176591";
    
    String NAME2 = "encounter.id.visitnumber.suffix";
    IContextItemName CNAME2 = ContextItemNameFactory.getName(NAME2);
    String VALUE2 = "9872193847";
    
    String NAME3 = "observation.id.placer_order_number.suffix";
    IContextItemName CNAME3 = ContextItemNameFactory.getName(NAME3);
    String VALUE3 = "89349872";

   String TEST_HYPERLINK =
        "crl://ignored.stuff" +
        "?" + NAME2 + "=" + VALUE2 +
        "&" + NAME1 + "=" + VALUE1 +
        "&" + NAME3 + "=" + VALUE3;
        
    Map EXPECTED_CONTEXT = new HashMap();
    

    public void testTranslateHyperlink() {
        Map answer = broker.translateHyperlink(TEST_HYPERLINK);
        assertEquals(answer, EXPECTED_CONTEXT);
    }

    public void testParseStringHyperlink() {
        Map answer = broker.parseStringHyperlink(TEST_HYPERLINK);
        assertEquals(answer, EXPECTED_CONTEXT);
    }

    private static final String[] SURVEY_RESPONSES1 = {
        "Simulator#732519: Accept conditionally option set",
        "CPRS#21321: !@#$21321:9:",
        "CPRS#13212: !@#$13212:9:",
        "CPRS#32121: !@#$32121:1000:Unsaved note will be lost" };
    private static final String[] SURVEY_RESPONSES2 = {
        "Simulator#732519: Accept conditionally option set" };
    
    private ContextChangeProposalResponse getResponses1() {
        return new ContextChangeProposalResponse(SURVEY_RESPONSES1, false);
    }

    private ContextChangeProposalResponse getResponses2() {
        return new ContextChangeProposalResponse(SURVEY_RESPONSES2, false);
    }

    public void testExtractHyperlinkResponses() {
        ContextChangeProposalResponse r = getResponses1(); 
        ArrayList l = new ArrayList();
        broker.extractHyperlinkResponses(r, l);
        assertEquals(2, r.surveyResponses.length);
        assertEquals("Simulator#732519: Accept conditionally option set",
                     r.surveyResponses[0]);
        assertEquals("CPRS#32121: Unsaved note will be lost", 
                     r.surveyResponses[1]);
        assertEquals(3, l.size());
    }

    public void testMatchResolution() {
        // no survey responses? oneMatch should return false.
        ArrayList l = new ArrayList();
        assertFalse(broker.oneMatch(l, "*"));
        assertFalse(broker.oneMatch(l, "21321"));

        ContextChangeProposalResponse r = getResponses1();
        broker.extractHyperlinkResponses(r, l);
        
        assertFalse(broker.oneMatch(l, "*"));
        assertTrue(broker.oneMatch(l, "21321"));
        assertTrue(broker.oneMatch(l, "32121"));
        assertFalse(broker.oneMatch(l, "Simulator#732519"));
    }

    public void testGetBestAppToDisplay() {
        // no responses? should return null.
        assertEquals(-1, broker.getBestAppToDisplay(new ArrayList()));
        
        // test survey responses 1
        ArrayList l = new ArrayList();
        ContextChangeProposalResponse r = getResponses1();
        broker.extractHyperlinkResponses(r, l);
        long bestApp = broker.getBestAppToDisplay(l);
        assertEquals(13212, bestApp);

        // test survey responses 2
        l = new ArrayList();
        r = getResponses2();
        broker.extractHyperlinkResponses(r, l);
        assertFalse(broker.oneMatch(l, "*"));
        bestApp = broker.getBestAppToDisplay(l);
        assertEquals(-1, bestApp);
    }

}
