package gov.va.med.hds.cd.ccow.test;

import gov.va.med.hds.cd.ccow.IContextItemName;
import gov.va.med.hds.cd.ccow.ContextItemNameFactory;

import junit.framework.TestCase;

public class ContextItemNameFactoryTest extends TestCase {

    public ContextItemNameFactoryTest(String arg0) {
        super(arg0);
    }

    private static final String NAME1 = "patient.id.mpi";
    private static final String NAME2 = "pAtIeNt.iD.mPi";
    private static final String NAME3 = "patient.id.mrn.generalhospital";
    private static final String NAME4 = "user.id.logon.windows";
    private static final String NAME5 = "patient.co.patientname";
    private static final String NAME6 = "[gov.va.med.hds]hyperlink.id.target";
    private static final String NAME7 = "[gov.va.med.hds]user.id.vista";
    private static final String NAME8 = "user.id.vista";


    public void testParseName1() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME1);
        assertEquals(NAME1, itemName.getName());
        assertTrue("patient".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("mpi".equalsIgnoreCase(itemName.getPrefix()));
        String suffix = itemName.getSuffix();
        assertTrue(suffix == null || suffix.length() == 0);
    }


    public void testParseName2() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME2);
        assertEquals(NAME2, itemName.getName());
        assertTrue("patient".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("mpi".equalsIgnoreCase(itemName.getPrefix()));
        String suffix = itemName.getSuffix();
        assertTrue(suffix == null || suffix.length() == 0);
    }


    public void testParseName3() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME3);
        assertEquals(NAME3, itemName.getName());
        assertTrue("patient".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("mrn".equalsIgnoreCase(itemName.getPrefix()));
        assertTrue("generalhospital".equalsIgnoreCase(itemName.getSuffix()));
    }


    public void testParseName4() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME4);
        assertEquals(NAME4, itemName.getName());
        assertTrue("user".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("logon".equalsIgnoreCase(itemName.getPrefix()));
        assertTrue("windows".equalsIgnoreCase(itemName.getSuffix()));
    }


    public void testParseName5() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME5);
        assertEquals(NAME5, itemName.getName());
        assertTrue("patient".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("co".equalsIgnoreCase(itemName.getRole()));
        assertTrue("patientname".equalsIgnoreCase(itemName.getPrefix()));
        String suffix = itemName.getSuffix();
        assertTrue(suffix == null || suffix.length() == 0);
    }


    public void testParseName6() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME6);
        assertEquals(NAME6, itemName.getName());
        assertTrue("[gov.va.med.hds]hyperlink".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("target".equalsIgnoreCase(itemName.getPrefix()));
        String suffix = itemName.getSuffix();
        assertTrue(suffix == null || suffix.length() == 0);
    }


    public void testParseName7() {
        IContextItemName itemName = ContextItemNameFactory.getName(NAME7);
        assertEquals(NAME7, itemName.getName());
        assertTrue("[gov.va.med.hds]user".equalsIgnoreCase(itemName.getSubject()));
        assertTrue("id".equalsIgnoreCase(itemName.getRole()));
        assertTrue("vista".equalsIgnoreCase(itemName.getPrefix()));
        String suffix = itemName.getSuffix();
        assertTrue(suffix == null || suffix.length() == 0);
    }
    
    
    public void testCompareNames1() {
        IContextItemName itemName1 = ContextItemNameFactory.getName(NAME1);
        IContextItemName itemName2 = ContextItemNameFactory.getName(NAME2);
        assertEquals(itemName1, itemName2);
        assertEquals(itemName1.hashCode(), itemName2.hashCode());
    }


    public void testCompareNames2() {
        IContextItemName itemName1 = ContextItemNameFactory.getName(NAME7);
        IContextItemName itemName2 = ContextItemNameFactory.getName(NAME8);
        assertFalse(itemName1.equals(itemName2));
        assertFalse(itemName1.hashCode() == itemName2.hashCode());
    }

}
