

package gov.va.med.hds.cd.ccow.ui;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;

/** Display context change continuation dialogs as required by CCOW
 */
public class CCOWContinuationDialog {

    public static final int CONTINUE = 0;
    public static final int CANCEL = 1;
    public static final int BREAK_LINK = 2;


    private static int show(String[] surveyResponses, boolean noContinue) {
        return show(null, surveyResponses, noContinue, false);
    }

    public static int show(Component parent, String[] surveyResponses,
                           boolean noContinue, boolean noBreakLink) {

        if (surveyResponses == null || surveyResponses.length == 0)
            return CONTINUE;

        JTextArea responseList = new JTextArea();
        responseList.setPreferredSize(new Dimension(300, 200));
        responseList.setEditable(false);
        responseList.setLineWrap(true);
        int i = 0;
        while (true) {
            responseList.append(surveyResponses[i]);
            if (++i >= surveyResponses.length)
                break;
            responseList.append("\n");
        }
        JScrollPane sp = new JScrollPane(responseList);
        sp.setHorizontalScrollBarPolicy
            (JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        Object[] message;

        if (noContinue && noBreakLink) {
            message  = new Object[] {
                "There may be a problem changing the current clinical",
                "data. The following application(s) reported a problem.",
                "Because at least one application is not responding,",
                "the change cannot continue.",
                sp };
        } else {
            message = new Object[] {
                "There may be a problem changing the current clinical",
                "data. The following application(s) reported a problem.",
                "Would you like to continue with the change?",
                sp };
        }

        int numOptions = 1 + (noContinue ? 0 : 1) + (noBreakLink ? 0 : 1);
        String[] options = new String[numOptions];
        i = 0;
        if (!noContinue) options[i++] = OK_OPTION;
        options[i++] = CANCEL_OPTION;
        if (!noBreakLink) options[i++] = BREAK_LINK_OPTION;

        int opt = JOptionPane.showOptionDialog
            (parent, message, "Problem Changing Clinical Data",
             JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
             null, options, options[0]);
        responseList = null;
        sp = null;

        if (opt >= 0 && opt < numOptions) {
            String option = options[opt];
            if (option == OK_OPTION) return CONTINUE;
            if (option == CANCEL_OPTION) return CANCEL;
            if (option == BREAK_LINK_OPTION) return BREAK_LINK;
        }

        return CANCEL;
    }

    private static final String OK_OPTION = "OK";
    private static final String CANCEL_OPTION = "Cancel";
    private static final String BREAK_LINK_OPTION = "Break link";
}
