package gov.va.med.hds.cd.ccow.ui;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

public class ContextTableModel extends AbstractTableModel {

    Object[][] context;

    public ContextTableModel(Map context) {
        setNewContext(context);
    }

    public String getColumnName(int column) {
        return (column == 0 ? "Name" : "Value");
    }
    
    public int getColumnCount() {
        return 2;
    }
    
    public Class getColumnClass(int columnIndex) {
        return String.class;
    }

    public int getRowCount() {
        return context.length;
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        return context[rowIndex][columnIndex];
    }
    
    public synchronized void setNewContext(Map context) {
        int size = context.size();
        Object[][] newContext = new Object[size][2];
        Iterator i = context.entrySet().iterator();
        for (int row = 0; row < size; row++) {
            Map.Entry e = (Map.Entry) i.next();
            newContext[row][0] = e.getKey();
            newContext[row][1] = e.getValue();
        }
        Arrays.sort(newContext, ROW_SORTER);
        this.context = newContext;
        fireTableChanged(new TableModelEvent(this));
    }
    
    public Map getDisplayedPatient() {
        Map result = new HashMap();
        for (int row = getRowCount(); row-- > 0;) {
            String name = String.valueOf(getValueAt(row, 0));
            if (!name.toLowerCase().startsWith("patient."))
                continue;
            result.put(name, getValueAt(row, 1));
        }
        return result;
    }
    

    private final class RowSorter implements Comparator {
        public int compare(Object o1, Object o2) {
            String c1 = String.valueOf(((Object[]) o1)[0]);
            String c2 = String.valueOf(((Object[]) o2)[0]);
            return c1.compareToIgnoreCase(c2);
        }
    }
    final RowSorter ROW_SORTER = new RowSorter();
    
}