package gov.va.med.hds.cd.ccow.ui;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Map;
import javax.swing.*;

import gov.va.med.hds.cd.ccow.*;

public class LinkIcon extends JLabel implements IContextObserver, MouseListener {

    public static final int SMALL_SIZE  = 16;
    public static final int MEDIUM_SIZE = 32;
    public static final int LARGE_SIZE  = 48;


	IClinicalContextBroker broker = null;
	Icon connected, broken;
    


    public LinkIcon(IContextModule contextModule, Object window) {
        this(contextModule, window, MEDIUM_SIZE);
    }

    public LinkIcon(IContextModule contextModule, Object window, int size) {
        initAppearance(size);
		initBroker(contextModule, window);
	}
    protected LinkIcon() {}
    
    /** Load the image files we need to draw ourself */
    protected void initAppearance(int size) {
        switch (size) {
            case SMALL_SIZE: case MEDIUM_SIZE: case LARGE_SIZE: break;
            default: size = MEDIUM_SIZE;   
        }
        connected = new ImageIcon(LinkIcon.class.getResource("link"+size+".png"));
        broken = new ImageIcon(LinkIcon.class.getResource("broken"+size+".png"));
        updateAppearance();     
        addMouseListener(this);   
    }
    

    /** Install a different context module.
     * 
     * Added by Kyle Clarke to allow the same icon to be used with different 
     * IContextPlugin instances that an app may create/destroy (not a usual
     * use case for IContextPlugin).
     * 
     * @param contextModule
     * @param window
     */
    public void updateContextModule(IContextModule contextModule, Object window) {
        initBroker(contextModule, window);
        // initBroker calls updateAppearance
    }


    /** Initiate our connection to the context broker */ 
	protected void initBroker(IContextModule contextModule, Object window) {
        if (contextModule != null)
            broker = contextModule.getBroker(window, this);
		updateAppearance();
	}

    /** update our appearance based on the current state of the context */
	protected void updateAppearance() {
		if (broker != null && broker.isConnected()) {
			setIcon(connected);
			setToolTipText("Clinical link on");
		} else {
			setIcon(broken);
			setToolTipText("Clinical link broken");
		}
	}
    
    // Implementation of gov.va.med.hds.cd.ccow.ContextObserver

    /** Message received from the context broker - the context has changed */
    public void contextChanged() {
        updateAppearance();
    }
    
    /** We do not display any patient-specific data. */
	public Map getDisplayedPatient() {
		return null;
	}
    
    // implementation of MouseListener interface
    
    public void mouseClicked(MouseEvent e) {
        if (broker == null) return;
        
        if (broker.isConnected())
            broker.breakExternalLink();
        else
            broker.joinExternalContext(false);
    }
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }

}
