

package gov.va.med.hds.cd.ccow;

/** Encapsulates the name of an item within the clinical context.
 *
 * The general format of context item name is:
 * <pre>       Subject_label.role.Name_prefix.optional_name_suffix</pre>
 * Where: <ul>
 *  <li><code>Subject_label</code> is the name of the subject to which
 *       the item belongs.</li>
 *  <li><code>Role</code> indicates the role of the item: <ul>
 *      <li><code>Id</code> - Identifier data</li>
 *      <li><code>Co</code> - Coorborating data</li>
 *      <li><code>An</code> - Annotating data</li></ul>
 *  <li><code>Name_prefix</code> is the name of the item within the
 *       context of its subject.
 *  <li><code>optional_name_suffix</code> is optional for data items.</li></ul>
 *
 * Examples include:
 * <pre>       Patient.id.mrn.SaintElsewhereClinic</pre>
 * <pre>       User.id.logon.GeneralHospital</pre>
 *
 * Context item names are case insensitive.
 *
 * For further information, see the CCOW Specification for Subject
 * Data Definitions.
 */
public interface IContextItemName {

    /** @return the full name of a context item. */
    public String getName();

    /** @return the subject field of a context item name. */
    public String getSubject();

    /** @return the role field of a context item name. */
    public String getRole();

    /** @return the prefix field of a context item name. */
    public String getPrefix();

    /** @return the suffix field of a context item name. */
    public String getSuffix();
}
