/*
 * Created on May 20, 2003
 */
package gov.va.med.hds.cd.ccow;

import java.util.Map;

/** Interface for an object which can internally perform mapping of context
 * subjects.
 * 
 * Objects implementing this interface fill a role similar to a CCOW mapping
 * agent, with the following significant differences:<ul>
 * <li>This mapping occurs within the application, not within the context
 *     manager.
 * <li>There is no requirement that there be a one-to-one mapping between
 *     context subjects and context mappers.  One mapper can perform
 *     mappings for several subjects, and one subject can be mapped by
 *     several mappers.
 * </ul>
 */
public interface IContextMapper {

    public static final String EXTENSION_POINT_ID =
        "gov.va.med.hds.cd.ccow.contextMapper";
        

    /** Examine the given context and potentially alter its contents.
     * 
     * @param context a CCOW context in canonical form. 
     * @return true if any changes (add, delete, modify) were made to the
     *      context.
     */
    public boolean mapSubjects(Map context);

}
