
package gov.va.med.hds.cd.ccow;

import java.awt.Component;

/** Interface to the desktop context plugin.
 */
public interface IContextModule {
    
    public static final String PLUGIN_ID = "gov.va.med.hds.cd.ccow";


    /** Get the {@link ClinicalContextBroker} instance for a given
     * clinician desktop window.
     *
     * @param forWindow The clinician desktop window for which we
     *    need a context broker.
     * @param observer The {@link ContextObserver} object to notify
     *    when the clinical context changes. (May be null)
     * @return The context broker for the given window.  If the window
     *    does not yet have a context broker, one is created
     *    automatically.  If it already has a context broker, the
     *    existing broker is returned.
     */
    public IClinicalContextBroker getBroker(Object forWindow,
                                           IContextObserver observer);


    /** Create an href that can be used as a CCOW hyperlink to the chart for
     * a particular patient.
     * 
     * @param patient
     * @return an href appropriate for use in an HTML &lt;a&gt; tag.
     */
    public String getCcowHref(Object patient);                           
                                           
    /** Attempt to open a hyperlink via CCOW.
     * 
     * @return {@link #BROWSE_SUCCESS} if the hyperlink was successfully
     * opened, {@link #BROWSE_CANCELLED} if the user cancelled the operation.  
     * otherwise, returns a user-readable description of the reason the
     * hyperlink could not be opened. 
     */
    public String browseCCOWHyperlink(Object hyperlink, Component parent);
    
    public static final String BROWSE_SUCCESS = "Success";
    public static final String BROWSE_CANCELLED = "Cancelled";

}
