

package gov.va.med.hds.cd.ccow;


import java.util.Map;
import java.util.Set;

/** This interface should be implemented by any component which would
 * like to participate in context change surveys.
 */
public interface IContextParticipant extends IContextObserver {

    /** The context management plugin calls this method to inform a
     * context participant that an internal context change operation
     * is about to begin.
     *
     * If this participant needs the user's permission to perform any
     * action in preparation for the context change, the participant
     * is allowed to display modal dialog boxes to the user asking for
     * that permission. (For example, a signature object might display
     * a list of unsigned items and ask the user if they want to sign
     * the items.) These dialog boxes should include a "Cancel" button.
     *
     * @param contextItems a list of items in the context that could
     *      potentially change
     * @return false if the user selected a "Cancel" option, true
     *      otherwise.
     */
    public boolean prepareForContextChange(Set contextItems);

    /** The context management plugin calls this method to inform a
     * context participant that an external context change operation
     * has been requested.
     *
     * This participant is not allowed to display any dialogs to the
     * user at this time.
     *
     * @return a String describing the consequences of changing the
     *     context (for example, loss of unsaved data). If the
     *     proposed context changes would not have adverse
     *     consequences, this method should return null.
     */
    public String reviewContextChange(Map potentialContext);

}
