package gov.va.med.hds.clinicaldata;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * Basic patient demographics class
 */
public class PatientDemographics implements IPatientDemographics {
	/**
	 * Format for incoming dates
	 */
	protected static final String DATE_INPUT_FORMAT = "yyyyMMdd";

	/**
	 * Format for date output
	 */
	protected static final String DATE_OUTPUT_FORMAT = "dd MMM yyyy";

	/**
	 * Used to read dates from the input source.
	 */
	protected SimpleDateFormat dateInputFormat = new SimpleDateFormat(DATE_INPUT_FORMAT);
	/**
	 * Used to format the date for presentation.
	 */
	protected SimpleDateFormat dateOutputFormat = new SimpleDateFormat(DATE_OUTPUT_FORMAT);

	protected String id, name, ssn, dob;
	protected int age;

	/**
	 * Constructor
	 */
	public PatientDemographics() {
		id = "";
		name = "";
		ssn = "";
		dob = "";
		age = 0;
	}
	/**
	 * Constructor
	 */
	public PatientDemographics(String id, String name, String ssn, String dob) {
		setPatientId(id);
		setPatientName(name);
		setSsn(ssn);
		setDob(dob);
	}
	public PatientDemographics(String id, String name, String ssn, String dob, String age) {
		setPatientId(id);
		setPatientName(name);
		setSsn(ssn);
		setDob(dob);
		setAge(age);
	}
	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#setPatientName(java.lang.String)
	 */
	public void setPatientName(String patientName) {
		this.name = patientName;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#getPatientName()
	 */
	public String getPatientName() {
		return name;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#setPatientId(java.lang.String)
	 */
	public void setPatientId(String patientId) {
		this.id = patientId;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#getPatientId()
	 */
	public String getPatientId() {
		return id;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#setDob(java.lang.String)
	 */
	public void setDob(String dob) {
		// The date input format is YYYYMMDD
		Calendar dobDate = Calendar.getInstance();
		try {
			dobDate.setTime(dateInputFormat.parse(dob));
			this.dob = dateOutputFormat.format(dobDate.getTime());
			int month = dobDate.get(Calendar.MONTH);
			int year = dobDate.get(Calendar.YEAR);
			int date = dobDate.get(Calendar.DATE);
			Calendar curDate = Calendar.getInstance();
			int curMonth = curDate.get(Calendar.MONTH);
			this.age = (curDate.get(Calendar.YEAR)) - year;
			if ((month > curMonth) || (month == curMonth && date > curDate.get(Calendar.DATE)))
				this.age--;
		} catch (ParseException pe) {
			this.dob = "Unknown";
			this.age = 0;
		}
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#getDob()
	 */
	public String getDob() {
		return dob;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#setSsn(java.lang.String)
	 */
	public void setSsn(String ssn) {
		this.ssn = ssn;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#getSsn()
	 */
	public String getSsn() {
		return ssn;
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#setAge(java.lang.String)
	 */
	public void setAge(String age) {
		this.age = Integer.parseInt(age);
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.IPatientDemographics#getAge()
	 */
	public String getAge() {
		return Integer.toString(age);
	}
}
