package gov.va.med.hds.clinicaldata.dao.m;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.clinicaldata.IPatientDemographics;
import gov.va.med.hds.clinicaldata.PatientDemographics;
import gov.va.med.hds.clinicaldata.dao.IClinicalDataDaoConstants;
import gov.va.med.hds.clinicaldata.dao.IPatientDemographicsDAO;
import gov.va.med.hds.clinicaldata.dao.PatientDemographicsDAOException;
import gov.va.med.hds.util.mumps.StringUtil;

import java.util.LinkedList;

/**
 * M implementation of a patient demographics Data Access Object
 */
public class PatientDemographicsDAO implements IPatientDemographicsDAO {
	private static final String PATIENT_INFO_RPC_KEY = "PTDEMOS";
	private static final int PATIENT_DOB_PIECE = 3;
	private static final int PATIENT_AGE_PIECE = 4;
	private static final int PATIENT_SSN_PIECE = 2;
	private static final int PATIENT_NAME_PIECE = 1;
	private static final int PATIENT_INFO_RPC_REQUIRED_MIN_PIECES = 5;
	private IRuntimeContext runtime;
	private IMumps mumps;

	/**
	 * Constructor
	 */
	public PatientDemographicsDAO() {
		super();
	}

	/**
	 * @see gov.va.med.hds.clinicaldata.dao.IPatientDemographicsDAO#createPatient(java.lang.String)
	 */
	public IPatientDemographics createPatient(String patientId)
		throws PatientDemographicsDAOException {
		LinkedList params = new LinkedList();
		params.add(patientId);
		try {
			String result = mumps.callRPC(PATIENT_INFO_RPC_KEY, params);
			String[] results = StringUtil.split(result, StringUtil.DEFAULT_SEPARATOR);
			if (results.length < PATIENT_INFO_RPC_REQUIRED_MIN_PIECES)
				throw new PatientDemographicsDAOException(
					"The result for call request "
						+ PATIENT_INFO_RPC_KEY
						+ " does needs at least "
						+ PATIENT_INFO_RPC_REQUIRED_MIN_PIECES
						+ " pieces.");
			return new PatientDemographics(
				patientId,
				results[PATIENT_NAME_PIECE],
				results[PATIENT_SSN_PIECE],
				results[PATIENT_DOB_PIECE],
				results[PATIENT_AGE_PIECE]);
		} catch (Exception e) {
			throw new PatientDemographicsDAOException("Call failed to get patient demographics", e);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor(IClinicalDataDaoConstants.PLUGIN_ID);
		mumps = ((IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
	}
}
