package gov.va.med.hds.clinicaldata.ui;

import gov.va.med.hds.clinicaldata.IPatientDemographics;

import java.awt.Color;
import java.awt.GridLayout;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

/**
 *  Patient ID line component.
 */
public class PatientLine extends JLabel {
	private static PatientLine BLANK_PATIENT_LINE;
	private static String BLANK_PATIENT_LINE_HTML;
	private static final String DATE_OUTPUT_FORMAT = "dd MMM yy";
	private SimpleDateFormat dateOutputFormat = new SimpleDateFormat(DATE_OUTPUT_FORMAT);

	private boolean lastFourSSN;
	private String name = "";
	private String ssn = "";
	private String dob = "";
	private String age = "";
	private JLabel nameLabel = new JLabel();
	private JLabel idsLabel = new JLabel();

	/**
	 * Constructor
	 */
	public PatientLine() {
		super(" ");
		setLayout(new GridLayout());
		setBackground(new Color(210, 233, 204));
		setForeground(Color.black);
		setOpaque(true);
		setBorder(
			BorderFactory.createBevelBorder(
				BevelBorder.LOWERED,
				new Color(134, 174, 124),
				new Color(166, 204, 157)));
		add(nameLabel, null);
		add(idsLabel, null);
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 * @param ssn
	 * @param dob
	 * @param age
	 * @param lastFourSSN 
	 */
	public PatientLine(String name, String ssn, String dob, String age, boolean lastFourSSN) {
		this();
		this.lastFourSSN = lastFourSSN;
		setPatientLine(name, ssn, dob, age, lastFourSSN);
	}

	/**
	 * Constructor
	 * 
	 * @param demos
	 * @param lastFourSSN
	 */
	public PatientLine(IPatientDemographics demos, boolean lastFourSSN) {
		this();
		setPatientLine(
			demos.getPatientName(),
			demos.getSsn(),
			dateOutputFormat.format(demos.getDob()),
			new Integer(demos.getAge()).toString(),
			lastFourSSN);
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 * @param ssn
	 * @param dob
	 * @param age
	 * @param lastFourSSN
	 */
	public PatientLine(String name, String ssn, Date dob, int age, boolean lastFourSSN) {
		this();
		setPatientLine(
			name,
			ssn,
			dateOutputFormat.format(dob),
			new Integer(age).toString(),
			lastFourSSN);
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 * @param ssn
	 * @param dob
	 * @param age
	 * @param lastFourSSN
	 */
	public PatientLine(String name, String ssn, String dob, int age, boolean lastFourSSN) {
		this();
		setPatientLine(name, ssn, dob, new Integer(age).toString(), lastFourSSN);
	}
	/**
	 * Sets the contents of this patient line
	 * @param demos
	 * @param lastFourSSN
	 */
	public void setPatientLine(IPatientDemographics demos, boolean lastFourSSN) {
		setPatientLine(
			demos.getPatientName(),
			demos.getSsn(),
			demos.getDob(),
			demos.getAge(),
			lastFourSSN);
	}
	/**
	 * Sets the contents of this patient line
	 * 
	 * @param name
	 * @param ssn
	 * @param dob
	 * @param age
	 * @param lastFourSSN
	 */
	public void setPatientLine(
		String name,
		String ssn,
		String dob,
		String age,
		boolean lastFourSSN) {
		this.name = name;
		this.dob = dob;
		this.age = age;
		this.lastFourSSN = lastFourSSN;
		this.ssn = formatSSN(ssn);

		nameLabel.setText(name);
		nameLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		idsLabel.setText(ssn + "   " + dob + "  (" + age + ") ");
		idsLabel.setHorizontalAlignment(SwingConstants.TRAILING);
	}

	/**
	 * Gets the html equivalent of the patient line content
	 * @return
	 */
	public String getPatientLineHTML() {
		MessageFormat borderColor = new MessageFormat("border-color:rgb({0},{1},{2});");
		MessageFormat bgColor = new MessageFormat("background-color:rgb({0},{1},{2});");
		Object[] bgRGB =
			{
				new Integer(getBackground().getRed()),
				new Integer(getBackground().getGreen()),
				new Integer(getBackground().getBlue())};
		Color border = ((BevelBorder) getBorder()).getShadowOuterColor();
		Object[] borderRGB =
			{
				new Integer(border.getGreen()),
				new Integer(border.getRed()),
				new Integer(border.getBlue())};
		//border-color:rgb(166,204,157);
		//background-color:rgb(210,233,204);
		StringBuffer html = new StringBuffer();
		html.append("<table width='100%' ");
		html.append("style='margin:0 0 0 0; width:100%;");
		html.append(borderColor.format(borderRGB) + "border-style:outset; border-width:2;");
		html.append(bgColor.format(bgRGB) + "'>");
		html.append("<tr><td><b>");
		html.append(name);
		html.append("</b></td><td align='right'>");
		html.append(ssn + "&nbsp;&nbsp; ");
		html.append(dob + "&nbsp;&nbsp; (");
		html.append(age + ")&nbsp;&nbsp; ");
		html.append("</td></tr></table>");
		return html.toString();
	}

	/**
	 * Creates an empty patient line
	 * @return
	 */
	public static PatientLine emptyPatientLine() {
		if (null == BLANK_PATIENT_LINE)
			BLANK_PATIENT_LINE = new PatientLine();
		return BLANK_PATIENT_LINE;
	}

	/**
	 * Creates an html string equivalent to an empty patient line.
	 * @return
	 */
	public static String emptyPatientLineHTML() {
		return emptyPatientLine().getPatientLineHTML();
	}

	/**
	 * Clears the text for the name and id
	 */
	public void clear() {
		nameLabel.setText("");
		idsLabel.setText("");
	}

	private String formatSSN(String social) {
		String formattedSSN = "";
		if (social.matches("[0-9](3)-[0-9](2)-[0-9](4)"))
			formattedSSN = (lastFourSSN ? social.split("-")[2] : social);
		else if (social.matches("[0-9](4)")) {
			this.lastFourSSN = true;
			formattedSSN = social;
		} else if (social.matches("[0-9](9)")) {
			if (!lastFourSSN)
				formattedSSN = social.substring(0, 4) + "-" + social.substring(4, 6) + "-";
			formattedSSN += social.substring(6);
		} else
			formattedSSN = social;
		return formattedSSN;
	}

}
