package gov.va.med.hds.clinicaldata;

/**
 * Provides basic patient demographics
 */
public interface IPatientDemographics {
	/**
	   * Sets the patient's name
	   * 
	   * @param patientName The patient's name usually formatted as <pre>LastName, FirstName</pre>
	   */
	public void setPatientName(String patientName);
	/**
	   * Returns the patient's name.
	   * 
	   * @return The patient's name as a String object.
	   */
	public String getPatientName();
	/**
	   * Sets the patient's identifier.
	   * 
	   * @param patientId The patient's identifier as a String object.
	   */
	public void setPatientId(String patientId);
	/**
	   * Returns the patient's identifier.
	   * 
	   * @return The patient's identifier as a String object.
	   */
	public String getPatientId();
	/**
	   * Sets the patient's date of birth.
	   * 
	   * @param dob The patient's date of birth formatted as <pre>"YYYYMMDD"</pre> where
	   * <dl><dt>YYYY</dt><dd>is the year</dd>
	   * <dt>MM</dt><dd>is the month.  A value between 01-12 inclusive.</dd>
	   * <dt>DD</dt><dd>is the day of the month.  A value between 01-31 inclusive.</dd></dt>
	   */
	public void setDob(String dob);
	/**
	   * Returns the patient's date of birth as a formatted string.
	   * 
	   * @return The patient's date of birth as a String object formatted as <pre>
	   * "dd MMM yy"</pre> where:
	   * <dl><dt>dd</dt><dd>is the day of the month.</dd>
	   * <dt>MMM</dt><dd>is the three character abbreviation of the month</dd>
	   * <dt>yy</dt><dd>is the last two digits of the year</dd></dl>
	   */
	public String getDob();
	/**
	   * Sets the patient's Social Security Number
	   * 
	   * @param ssn The patient's Social Security Number.  No formatting is done
	   * to the input string.  It is stored as is.
	   */
	public void setSsn(String ssn);
	/**
	   * Returns the patient's Social Security Number.
	   * 
	   * @return The patient's Social Security Number as a String object. 
	   */
	public String getSsn();
	/**
	 * Sets the patient's age
	 * @param age
	 */
	public void setAge(String age);
	/**
	   * Returns the age of the patient as calculated from the patient's date of 
	   * birth. 
	   * 
	   * @return The age of the patient as a String.
	   */
	public String getAge();
}