package gov.va.med.hds.cd.config.internal;

import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.dom4j.Dom4jXPath;

import gov.va.med.hds.cd.config.ICallResultHandler;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.ParameterCategoryException;
import gov.va.med.hds.cd.config.ResultHandlerException;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfigurationCallHandler implements ICallResultHandler {
	private IConfiguration configuration = null;
	private IParameterCategory pRoot = null;
	
	public ConfigurationCallHandler(IConfiguration configuration) {
		this.configuration = configuration;
	}
	 
	/**
	 * @see gov.va.med.hds.cd.config.ICallResultHandler#processResult(java.lang.String)
	 */
	public Object processResult(String callResult) throws ResultHandlerException {
		try {
			pRoot = buildParameterTree(callResult);
		} catch (ParameterCategoryException e) {
			throw new ResultHandlerException("ParameterCategoryException Caught: " + e);
		}
		return pRoot;
	}

	private IParameterCategory buildParameterTree(String xmlTree)
		throws ParameterCategoryException {

		IParameterCategory treeNode = null;
		SAXReader reader = new SAXReader();
		Document doc;

		try {
			doc = reader.read(new StringReader(xmlTree));
		} catch (DocumentException e) {
			throw new ParameterCategoryException(
				"couldn't get reader for xmlTree: ",
				e);
		}

		// first, look for an exception raised by M Call 
		XPath xpath;
		try {
			xpath = (XPath) new Dom4jXPath("/getConfigurationCallResult/error");
			Node node = (Node) xpath.selectSingleNode(doc);
			if (node != null) {
				throw new ParameterCategoryException(node.getText());
			}
		} catch (JaxenException e) {
			throw new ParameterCategoryException(
				"JaxenException at plugin root.",
				e);
		}

		// if no exception, parse M Call result
		try {
			xpath =
				(XPath) new Dom4jXPath("/getConfigurationCallResult/configTree/pluginParameterCategory");
		} catch (JaxenException e) {
			throw new ParameterCategoryException(
				"JaxenException at plugin root.",
				e);
		}

		List results;
		try {
			results = xpath.selectNodes(doc);
		} catch (JaxenException e) {
			throw new ParameterCategoryException(
				"JaxenException at selectNodes.",
				e);
		}

		Iterator resultIter = results.iterator();

		if (!resultIter.hasNext())
			return null;

		//first, establish root node	
		Object object = resultIter.next();
		if (object instanceof Node) {
			try {
				treeNode = new ParameterCategory((Node) object, configuration);
			} catch (ParameterCategoryException e) {
				throw new ParameterCategoryException(
					"couldn't establish root node.",
					e);
			}
		}

		//then, get all descendents
		if (treeNode != null) {
			while (resultIter.hasNext()) {
				Object xmlCat = resultIter.next();
				if (xmlCat instanceof Node) {
					try {
						treeNode.insert(
							new ParameterCategory(
								(Node) xmlCat,
								treeNode,
								configuration));
					} catch (ParameterCategoryException e) {
						throw new ParameterCategoryException(
							"couldn't insert child node.",
							e);
					}
				}
			}
		}
		return treeNode;
	}

}
