/*
 * Created on Jun 23, 2003 2:17:01 PM
 */
package gov.va.med.hds.cd.config.internal;

import gov.va.med.hds.cd.config.ConfigurationChangeEvent;
import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.ConfigurationReadyEvent;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurationChangeListener;
import gov.va.med.hds.cd.config.IConfigurationEditorModel;
import gov.va.med.hds.cd.config.IConfigurationReadyListener;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IParameterDefinitionDataProvider;
import gov.va.med.hds.cd.config.IParameterTree;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.spectrum.longlist.BufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfigurationEditorModel
	implements
		IConfigurationEditorModel,
		IConfigurationReadyListener,
		IConfigurationChangeListener {

	private IBufferModel bufferModel;
	private IConfigurator configurator;
	private IRuntimeContext runtime;
	private IPluginRegistry pluginRegistry;
	private IParameterCategory retrieveNode =
		new ParameterCategory("Retrieving data...", null);
	private IParameterTree configTreeModel = null;
	private IParameterCategory configRoot = new ParameterCategory("HealtheVet Plugins", null);
	private ParameterDefinitionDataProvider dataProvider;

	/**
	 * 
	 */
	public ConfigurationEditorModel(IRuntimeContext runtime)
		throws PluginStartupException, PluginNotFoundException {
		super();
		this.runtime = runtime;
		this.bufferModel = new BufferModel(300);
		pluginRegistry = runtime.getPluginRegistry();
		configurator =
			(IConfigurator) runtime.getPluginInstance(IConfigurator.PLUGIN_ID);
		buildModel();
	}

	private void buildModel() {
		IPluginDescriptor[] pluginDescriptors =
			pluginRegistry.getPluginDescriptors();

		configRoot.insert(retrieveNode);
		configTreeModel = new ParameterTree(configRoot);

		for (int i = 0; i < pluginDescriptors.length; i++) {
			try {
				((Configurator) configurator).getMutableConfiguration(
					pluginDescriptors[i].getUniqueIdentifier(),
					this);
			} catch (ConfigurationException e) {
				runtime.getLogger(getClass()).error("Unable to get mutable configuration for " + pluginDescriptors[i], e);
			}
		}
	}

	private void buildConfigNode(IConfiguration configuration) {
		IParameterCategory pRoot = configuration.getParameterRoot();
		if ((pRoot != null) && (!pRoot.getName().equals(""))) {
			if (retrieveNode != null) {
				configTreeModel.removeNodeFromParent(retrieveNode);
				retrieveNode = null;
			}
			configTreeModel.insertNodeInto(pRoot, configRoot);
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigurationEditorModel#getParameterTree()
	 */
	public IParameterTree getParameterTree() {
		return configTreeModel;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigurationEditorModel#getDataProvider()
	 */
	public IParameterDefinitionDataProvider getDataProvider() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigurationChangeListener#configurationChanged(gov.va.med.hds.cd.config.ConfigurationChangeEvent)
	 */
	public void configurationChanged(ConfigurationChangeEvent cce) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationReady(gov.va.med.hds.cd.config.ConfigurationReadyEvent)
	 */
	public void configurationReady(ConfigurationReadyEvent cre) {
		buildConfigNode(cre.getConfiguration());
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationCallFailed(gov.va.med.hds.cd.config.ConfigurationException)
	 */
	public void configurationCallFailed(ConfigurationException ce) {
		// TODO Auto-generated method stub

	}

}
