package gov.va.med.hds.cd.config.internal;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.dom4j.Dom4jXPath;

import gov.va.med.hds.cd.config.ICallResultHandler;
import gov.va.med.hds.cd.config.ParameterDefinitionException;
import gov.va.med.hds.cd.config.ResultHandlerException;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ParameterDefinitionCallHandler implements ICallResultHandler {

	private List parameterDefinitions = new ArrayList();

	/**
	 * @see gov.va.med.hds.cd.config.ICallResultHandler#processResult(java.lang.String)
	 */
	public Object processResult(String callResult) throws ResultHandlerException {
		try {
			parseResult(callResult);
		} catch (ParameterDefinitionException e) {
			throw new ResultHandlerException("Exception parsing call result: " + e.getMessage());
		}
		return parameterDefinitions;
	}
	
	private void parseResult(String result) throws ParameterDefinitionException {
		SAXReader reader = new SAXReader();
		Document doc;
		XPath xpath;
		
		try {
			doc = reader.read(new StringReader(result));
		} catch (DocumentException e) {
			throw new ParameterDefinitionException(
				"couldn't get reader for xmlTree: ",
				e);
		}
		
		// first, look for an exception raised by M Call 
		try {
			xpath = (XPath) new Dom4jXPath("/getParameterDefinitionsCallResult/error");
			Node node = (Node) xpath.selectSingleNode(doc);
			if (node != null) {
				throw new ParameterDefinitionException(node.getText());
			}
		} catch (JaxenException e) {
			throw new ParameterDefinitionException(
				"JaxenException at plugin root.",
				e);
		}
		
		// if no exception, parse M Call result
		try {
			xpath =
				(XPath) new Dom4jXPath("/getParameterDefinitionsCallResult/parameterDefinitionList/parameterDefinition");
		} catch (JaxenException e) {
			throw new ParameterDefinitionException(
				"JaxenException at plugin root.",
				e);
		}
		
		List resultList;
		try {
			resultList = xpath.selectNodes(doc);
		} catch (JaxenException e) {
			throw new ParameterDefinitionException(
				"JaxenException at selectNodes.",
				e);
		}
		
		Iterator resultListIter = resultList.iterator();
		
		while (resultListIter.hasNext()) {
			Object pdXml = resultListIter.next();
			if (pdXml instanceof Node) {
				parameterDefinitions.add(new ParameterDefinition((Node) pdXml));
			}
		}		
	}

}
