/*
 * Created on Mar 4, 2003 8:42:45 AM
 */
package gov.va.med.hds.cd.config.internal;

import gov.va.med.hds.cd.config.ConfigDAOEvent;
import gov.va.med.hds.cd.config.ICallResultHandler;
import gov.va.med.hds.cd.config.IConfigDAO;
import gov.va.med.hds.cd.config.IConfigDAOListener;
import gov.va.med.hds.cd.config.IParameterDefinition;
import gov.va.med.hds.cd.config.IParameterDefinitionDataProvider;
import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.m.CallRequest;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ParameterDefinitionDataProvider
	implements IConfigDAOListener, IDataNeededListener, IParameterDefinitionDataProvider {
	private IConfigDAO dao;
	private ICallRequest request;
	private IBufferModel bufferModel;
	private Object filter;
	private List callParams = new ArrayList();
	private ICallResultHandler handler = new ParameterDefinitionListHandler();
	private ILogger log;

	/**
	 * 
	 */
	public ParameterDefinitionDataProvider(
		IRuntimeContext runtime,
		IBufferModel bufferModel) throws PluginStartupException, PluginNotFoundException, Exception {
		super();
		this.log = runtime.getLogger(getClass());
		this.dao =
			(IConfigDAO)
				(
					(IDaoRegistry) runtime.getPluginInstance(
						IDaoRegistry.PLUGIN_ID)).getDAO(
				IConfigDAO.INTERFACE_NAME);
		this.bufferModel = bufferModel;
		//		getDefinitions(FROM_STRING, FORWARD_DIRECTION, CALL_SIZE);
	}

	private void getDefinitions(String from, int direction, int size) {
		callParams.clear();
		callParams.add(from.toUpperCase());
		callParams.add(Integer.toString(direction));
		callParams.add(Integer.toString(size));
		request = new CallRequest("XHD GET PARAMETER DEFINITIONS", callParams);
		dao.query(request, handler, this);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAOListener#callFinished(gov.va.med.hds.cd.config.ConfigDAOEvent)
	 */
	public void callFinished(ConfigDAOEvent iCe) {
		List resultItems;
		if (iCe.getException() != null) {
			log.error("Configuration call finished due to exception", iCe.getException());
		} else {
			LinkedList items = new LinkedList();
			resultItems = (List) iCe.getResultObject();
			Iterator itIt = resultItems.iterator();
			while (itIt.hasNext()) {
				IParameterDefinition pd = (IParameterDefinition) itIt.next();

				if (pd.getFullName().equals("ORB DEFAULT RECIPIENTS")) {
					System.out.println(pd);
				} else if (pd.getFullName().equals("ORB DELETE MECHANISM")) {
					System.out.println(pd);
				}

				IBufferItem bufferItem =
					new BufferItem(pd.getFullName(), Long.toString(pd.getId()));

				items.add(bufferItem);
			}

			bufferModel.setData(items.toArray(), items.size());
			items.clear();
			resultItems.clear();
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener#getDataRequest(gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent)
	 */
	public void getDataRequest(DataNeededEvent dataNeededEvent)
		throws DataProviderException {

		if (dataNeededEvent.isDataSearch()) {
			getDefinitions(
				dataNeededEvent.getStartingAt().toString(),
				dataNeededEvent.getDirection(),
				dataNeededEvent.getLength());
			//				if (items.length<1) {
			//					items = getNext(dataNeededEvent.getStartingAt(), IBufferModel.REVERSE_DIRECTION, dataNeededEvent.getLength());
			//				}
			//				if (items.length<1) {
			//					dataNeededEvent.setDataSearch(false);
			//					dataNeededEvent.setStartingAt(null);
			//					getDataRequest(dataNeededEvent);
			//				}
			//				else {
			//					bufferModel.setData(items, dataNeededEvent.getLength());
			//				}
		} else {
			if (dataNeededEvent.getStartingAt() == null) {
				//Get TOP or BOTTOM
				if (dataNeededEvent.getDirection() == FORWARD_DIRECTION) {
					//Get TOP
					getDefinitions(
						"",
						dataNeededEvent.getDirection(),
						CALL_SIZE);
				} else {
					//Get BOTTOM
					getDefinitions(
						"",
						dataNeededEvent.getDirection(),
						CALL_SIZE);
				}
			} else {
				//Get middle part somewhere
				getDefinitions(
					dataNeededEvent.getStartingAt().toString(),
					dataNeededEvent.getDirection(),
					CALL_SIZE);
			}
		}

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener#setBufferModel(gov.va.med.hds.cd.spectrum.longlist.IBufferModel)
	 */
	public void setBufferModel(IBufferModel bufferModel) {
		this.bufferModel = bufferModel;
		this.bufferModel.addDataNeededListener(this);
		this.bufferModel.setDefaultCallSize(44);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener#setFilter(java.lang.Object)
	 */
	public void setFilter(Object filter) {
		this.filter = filter;
	}

}
