package gov.va.med.hds.cd.config.internal;

import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.config.IPreferencePage;
import gov.va.med.hds.cd.config.IPreferencePageManager;
import gov.va.med.hds.cd.config.PreferencePageException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class PreferencePageManager implements IPreferencePageManager, IPlugin {

	private Map preferencePageMap = new HashMap();
	private IConfigurationElement[] preferencePageElements;
	
	public PreferencePageManager() {
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor)
	 */
	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
		throws Exception {
		IPluginRegistry pluginRegistry = runtime.getPluginRegistry();
		preferencePageElements =
			pluginRegistry.getConfigurationElementsFor(EXTENSION_POINT_ID);

		for (int i = 0; i < preferencePageElements.length; i++) {
			String key = preferencePageElements[i].getAttribute(ID_ATTRIBUTE);
			if (!preferencePageMap.containsKey(key)) {
				preferencePageMap.put(key, preferencePageElements[i]);
			} else {
				throw new Exception(
					"The manifest "
						+ ID_ATTRIBUTE
						+ " was repeated for "
						+ key
						+ " This key must be unique");
			}
		}
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
	 */
	public void shutdown() throws Exception {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IPreferencePageManager#getPreferencePage(java.lang.String)
	 */
	public IPreferencePage getPreferencePage(String preferencePageId)
		throws PreferencePageException {
		IConfigurationElement ce =
			(IConfigurationElement) preferencePageMap.get(preferencePageId);
		IPreferencePage preferencePage = null;

		if (ce != null) {
			try {
				preferencePage =
					(IPreferencePage) ce.createExecutableExtension(
						CLASS_ATTRIBUTE);
			} catch (Exception e) {
				throw new PreferencePageException(e);
			}
		}
		return preferencePage;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IPreferencePageManager#getPreferencePageElements()
	 */
	public IConfigurationElement[] getPreferencePageElements() {
		return preferencePageElements;
	}

}
