package gov.va.med.hds.cd.config.itest;

import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.ConfigurationReadyEvent;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurationReadyListener;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IParameter;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IParameterTree;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfigurationTest extends RuntimeTestCase {
	private ISecurity securityModule = null;
	private IConfigurator configurator = null;
	private IConfiguration configuration = null;
	private IParameterTree parameterTree = null;
	private IParameterCategory parameterRoot = null;
	private ConfigurationException configurationException;

	/**
	 * Constructor for ConfigurationTest.
	 * @param arg0
	 */
	public ConfigurationTest(String arg0) {
		super(arg0);
	}

	/**
	 * @see RuntimeTestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("registry.xml");
		getRuntime().readSettings(getClass().getResourceAsStream("VistalinkSettings.xml"));

		securityModule =
			(ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		configurator =
			(IConfigurator) getRuntime().getPluginInstance(IConfigurator.PLUGIN_ID);
	}

	/**
	 * @see RuntimeTestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		configurator = null;
		configuration = null;
		securityModule.logout();
		super.tearDown();
	}

	public void testGetParameterTree() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};

		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);

		try {
			synchronized (this) {
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}

		if ((configuration != null) && (configurationException != null)) {
			fail("Call Timed Out...");
		}

		if (configurationException != null) {
			throw configurationException;
		}

		parameterTree = configuration.getParameterTree();
		assertEquals(
			"XHD PRISM PARAMETERS",
			((IParameterCategory) parameterTree.getRoot()).getFullName());
	}

	public void testGetParameterRoot() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};

		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);

		try {
			synchronized(this){
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}
		
		if ((configuration != null) && (configurationException != null)) { 
			fail("Call Timed Out...");
		}
		
		if (configurationException != null) {
			throw configurationException;
		}
		parameterRoot = configuration.getParameterRoot();
		assertEquals("Desktop", parameterRoot.getName());
	}

	/*
	 * Test for void putParameter(IParameter)
	 */
	public void testPutParameterIParameter() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};

		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);

		try {
			synchronized (this) {
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}

		if ((configuration != null) && (configurationException != null)) {
			fail("Call Timed Out...");
		}

		if (configurationException != null) {
			throw configurationException;
		}
		try {
			IParameter toParameter = configuration.getParameter("Timeout for GUI chart");
			toParameter.setValue("600");
			configuration.putParameter(toParameter);
		} catch (NonExistentParameterException e) {
			fail("NonexistentParameterException caught: " + e.getMessage());
		} catch (ConfigurationException e) {
			fail("ConfigurationException caught: " + e.getMessage());
		}
	}

//	public void testRemoveParameterCategory() {
//	}
//
//	public void testGetParameterValue() {
//	}

	/*
	 * Test for void putParameter(String, String)
	 */
//	public void testPutParameterStringString() {
//	}


	public void testGetCategoryForParameter() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};
		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);
		try {
			synchronized(this){
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}
		
		if ((configuration != null) && (configurationException != null)) { 
			fail("Call Timed Out...");
		}
		
		if (configurationException != null) {
			throw configurationException;
		}
		assertEquals(
			"Desktop",
			configuration.getCategoryForParameter("Timeout for GUI chart").toString());
	}

	public void testGetListenerCount() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};
		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);
		try {
			synchronized(this){
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}
		
		if ((configuration != null) && (configurationException != null)) { 
			fail("Call Timed Out...");
		}
		
		if (configurationException != null) {
			throw configurationException;
		}
		assertEquals(
			1,
			configuration.getListenerCount(IConfigurationReadyListener.class));
	}

	public void testGetParameter() throws Exception {
		configurationException = null;
		IConfigurationReadyListener listener =
			new IConfigurationReadyListener() {
			public void configurationReady(ConfigurationReadyEvent cre) {
				configuration = cre.getConfiguration();
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
			public void configurationCallFailed(ConfigurationException ce) {
				configurationException = ce;
				synchronized (ConfigurationTest.this) {
					ConfigurationTest.this.notify();
				}
			}
		};
		configuration =
			configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);
		try {
			synchronized(this){
				wait(5000);
			}
		} catch (InterruptedException e) {
			fail("InterruptedException caught: " + e.getMessage());
		}
		
		if ((configuration != null) && (configurationException != null)) { 
			fail("Call Timed Out...");
		}
		
		if (configurationException != null) {
			throw configurationException;
		}
		try {
			IParameter toParameter = configuration.getParameter("Timeout for GUI chart");
			assertTrue(Long.parseLong(toParameter.getValue()) > 0);
		} catch (NonExistentParameterException e) {
			fail("NonExistentParameterException caught: " + e.getMessage());
		}
	}

//	public void testUpdateParameterCategory() {
//	}

}
