package gov.va.med.hds.cd.config.itest;

import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.ConfigurationReadyEvent;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurationReadyListener;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IParameterDefinitionCallListener;
import gov.va.med.hds.cd.config.ParameterDefinitionException;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;

import java.util.List;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfiguratorTest extends RuntimeTestCase implements IParameterDefinitionCallListener {
	private IConfigurator configurator;
	private ISecurity securityModule;
	private IConfiguration configuration = null;
	/**
	 * Constructor for ConfiguratorTest.
	 * @param arg0
	 */
	public ConfiguratorTest(String arg0) {
		super(arg0);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("registry.xml");
		getRuntime().readSettings(getClass().getResourceAsStream("VistalinkSettings.xml"));

		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		configurator = (IConfigurator) getRuntime().getPluginInstance(IConfigurator.PLUGIN_ID);
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		//		securityModule.logout();
		super.tearDown();
	}

	public void testGetConfiguration() throws InterruptedException {

		try {
			IConfigurationReadyListener listener = new IConfigurationReadyListener() {
				private ConfigurationException configurationException;
				public void configurationReady(ConfigurationReadyEvent cre) {
					configuration = cre.getConfiguration();
					synchronized (ConfiguratorTest.this) {
						ConfiguratorTest.this.notify();
					}
				}
				public void configurationCallFailed(ConfigurationException ce) {
					configurationException = ce;
					synchronized (ConfiguratorTest.this) {
						ConfiguratorTest.this.notify();
					}
				}
			};
			IConfiguration configuration = configurator.getConfiguration(IPrismConstants.PLUGIN_ID, listener);
			assertEquals("gov.va.med.hds.cd.prism", configuration.toString());
		} catch (ConfigurationException e) {
			fail("ConfigurationException Caught: " + e.getMessage());
		}
	}

	public synchronized void testParameterDefinitionQuery() {
		try {
			configurator.parameterDefinitionQuery(this);
			wait();
		} catch (Exception e) {
			fail("Exception Caught: " + e.getMessage());
		}
	}
	/**
	 * @see gov.va.med.hds.cd.config.IParameterDefinitionCallListener#callSucceeded(java.util.List)
	 */
	public synchronized void callSucceeded(List definitions) {
		assertEquals("EC HFS SCRATCH", definitions.get(0).toString());
		notify();
	}
	/**
	 * @see gov.va.med.hds.cd.config.IParameterDefinitionCallListener#callFailed(gov.va.med.hds.cd.config.ParameterDefinitionException)
	 */
	public void callFailed(ParameterDefinitionException e) {
		fail("ParameterDefinitionException Caught: " + e.getMessage());
		notify();
	}

}
