/*
 * Created on Jul 1, 2003 2:02:24 PM
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.config.itest;

import gov.va.med.hds.cd.config.ConfigDAOException;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IParameter;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.internal.Parameter;
import gov.va.med.hds.cd.config.internal.ParameterCategory;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;

import java.util.List;
import java.util.Map;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ParameterCategoryTest extends RuntimeTestCase {

	private IConfigurator configurator;
	private IConfiguration configuration;
	private ISecurity securityModule;
	private IParameterCategory pCat;

	/**
	 * Constructor for ParameterCategoryTest.
	 * @param name
	 */
	public ParameterCategoryTest(String name) throws Exception {
		super(name);
	}

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("registry.xml");
		getRuntime().readSettings(
			getClass().getResourceAsStream("VistalinkSettings.xml"));

		securityModule =
			(ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		configurator =
			(IConfigurator) getRuntime().getPluginInstance(
				IConfigurator.PLUGIN_ID);
		configuration =
			(IConfiguration) configurator.getConfiguration(
				IPrismConstants.PLUGIN_ID,
				5000);
		pCat = configuration.getParameterRoot();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		pCat = null;
		configuration = null;
		configurator = null;
		securityModule.logout();
		securityModule = null;
		super.tearDown();
	}

	/*
	 * Test for void addParameter(IParameter)
	 */
	public void testAddParameterIParameter() {
		int paramcnt = pCat.parameters().size();
		IParameter param =
			new Parameter("Test addParameterIParameter", configuration, pCat);
		param.setKey("Test addParameterIParameter^User^^1");
		pCat.addParameter(param);
		assertEquals(paramcnt + 1, pCat.parameters().size());
	}

	/*
	 * Test for void addParameter(String, IParameter)
	 */
	public void testAddParameterStringIParameter() {
		int paramcnt = pCat.parameters().size();
		IParameter param =
			new Parameter(
				"Test addParameterStringIParameter",
				configuration,
				pCat);
		String key = "Test addParameterStringIParameter^User^^1";
		pCat.addParameter(key, param);
		assertEquals(paramcnt + 1, pCat.parameters().size());
	}

	/*
	 * Test for List children() 
	 */
	public void testChildren() {
		int kidSize = pCat.children().size();
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		pCat.insert(testCat1);
		List kids = pCat.children();
		assertEquals(kidSize + 2, kids.size());
	}

	public void testCompareTo() {
		IParameterCategory pCat1 = new ParameterCategory(pCat);
		pCat1.setName("Test Category");
		assertFalse(1 == ((ParameterCategory) pCat).compareTo(pCat1));
	}

	public void testGetAllowsChildren() {
		pCat.setLeaf(true);
		assertFalse(((ParameterCategory) pCat).getAllowsChildren());
	}

	public void testGetChild() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals(
			"Test Category",
			pCat.getChild("XHD TEST CATEGORY").getName());
	}

	public void testGetChildAt() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals("Test Category 1", pCat.getChildAt(1).getName());
	}

	public void testGetChildCount() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals(2, pCat.getChildCount());
	}

	public void testGetConfiguration() {
		assertEquals(
			"gov.va.med.hds.cd.prism",
			pCat.getConfiguration().toString());
	}

	public void testGetFullName() {
		assertEquals("XHD PRISM PARAMETERS", pCat.getFullName());
	}

	public void testGetId() {
		assertEquals(4, pCat.getId());
	}

	public void testGetIndex() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals(1, pCat.getIndex(testCat1));
	}

	public void testGetName() {
		assertEquals("Desktop", pCat.getName());
	}

	public void testGetParameter() {
		assertEquals(
			"ADA Section 508 Notice Acknowledged",
			((ParameterCategory) pCat)
				.getParameter("ADA Section 508 Notice Acknowledged^1412;VA(200,^325^1")
				.getName());
	}

	public void testGetParameterValue() {
		String pval =
			((ParameterCategory) pCat).getParameterValue(
				"ADA Section 508 Notice Acknowledged^1412;VA(200,^325^1");
		assertTrue(((pval.equals("true")) || (pval.equals("false"))));
	}

	public void testGetParent() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals(pCat, testCat1.getParent());
	}

	public void testGetParentId() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1);
		assertEquals(4, testCat1.getParentId());
	}

	public void testGetPath() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat.insert(testCat1);
		assertEquals(3, testCat1.getPath().length);
	}

	public void testGetPluginId() {
		assertEquals(IPrismConstants.PLUGIN_ID, pCat.getPluginId());
	}

	public void testGetPreferencePage() {
		assertEquals("prismPreferencePage", pCat.getPreferencePage());
	}

	public void testGetTransferData() {
		try {
			assertEquals(
				"Desktop",
				((ParameterCategory) pCat)
					.getTransferData(IParameterCategory.CATEGORY_FLAVOR)
					.toString());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testGetTransferDataFlavors() {
		assertEquals(
			"Parameter Category",
			((ParameterCategory) pCat)
				.getTransferDataFlavors()[0]
				.getHumanPresentableName());
	}

	/*
	 * Test for void insert(IParameterCategory)
	 */
	public void testInsertIParameterCategory() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat.insert(testCat1);
		assertEquals(1, pCat.getChildCount());
		assertEquals(1, testCat.getChildCount());
	}

	/*
	 * Test for void insert(IParameterCategory, int)
	 */
	public void testInsertIParameterCategoryint() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat, 0);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		pCat.insert(testCat1, 1);
		assertEquals(2, pCat.getChildCount());
	}

	public void testIsDataFlavorSupported() {
		assertTrue(
			((ParameterCategory) pCat).isDataFlavorSupported(
				IParameterCategory.CATEGORY_FLAVOR));
	}

	public void testIsLeaf() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		assertEquals(false, testCat.isLeaf());
		assertEquals(true, testCat1.isLeaf());
	}

	public void testIsMarkedForRemoval() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		testCat1.setMarkedForRemoval(true);
		assertFalse(testCat.isMarkedForRemoval());
		assertTrue(testCat1.isMarkedForRemoval());
	}

	public void testIsPackageRoot() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		testCat1.setMarkedForRemoval(true);
		assertFalse(testCat.isPackageRoot());
		assertTrue(pCat.isPackageRoot());
	}

	public void testIsPersistent() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		testCat1.setMarkedForRemoval(true);
		assertFalse(testCat.isPersistent());
		assertTrue(pCat.isPersistent());
	}

	/*
	 * Test for void markToRemove(int)
	 */
	public void testMarkToRemoveint() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		testCat.markToRemove(0);
		assertTrue(testCat1.isMarkedForRemoval());
	}

	/*
	 * Test for void markToRemove(IParameterCategory)
	 */
	public void testMarkToRemoveIParameterCategory() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		testCat.markToRemove(testCat1);
		assertTrue(testCat1.isMarkedForRemoval());
	}

	public void testParameters() {
		Map parameters = pCat.parameters();
		IParameter countdown =
			(IParameter) parameters.get(
				"Countdown Seconds upon Timeout^1412;VA(200,^30^1");
		assertEquals("ORWOR TIMEOUT COUNTDOWN", countdown.getFullName());
	}

	public void testParametersAsArrayList() {
		List parameters = pCat.parametersAsArrayList();
		IParameter ada508 = (IParameter) parameters.get(0);
		assertEquals("XHD PRISM ADA S508 NOTICE", ada508.getFullName());
	}

	public void testPersist() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		testCat.setPluginId(pCat.getPluginId());
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setPluginId(pCat.getPluginId());
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		try {
			pCat.persist();
			assertTrue(pCat.isPersistent());
			assertTrue(testCat.isPersistent());
			assertFalse(testCat1.isPersistent());
		} catch (ConfigDAOException e) {
			fail("ConfigDAOException caught: " + e.getMessage());
		}
		//finally, delete testCat by setting MarkedForRemoval=true and calling persist
		testCat.setMarkedForRemoval(true);
		try {
			testCat.persist();
		} catch (ConfigDAOException e1) {
			fail("ConfDAOException caught: " + e1.getMessage());
		}
	}

	/*
	 * Test for void putParameter(IParameter)
	 */
	public void testPutParameterIParameter() {
		IParameter p = new Parameter("Test Parameter", configuration, pCat);
		p.setFullName("XHD TEST PARAMETER");
		p.setDefaultValue("Here's another fine mess you've gotten me into, Ollie!");
		p.setKey("Test Parameter^USER^^1");
		pCat.putParameter(p);
		
		assertEquals("Here's another fine mess you've gotten me into, Ollie!", pCat.getParameter("Test Parameter^USER^^1").getDefaultValue());
	}

	/*
	 * Test for void putParameter(String, String)
	 */
	public void testPutParameterStringString() {
		IParameter p = pCat.getParameter("ADA Section 508 Notice Acknowledged^1412;VA(200,^325^1");
		
		pCat.putParameter(p.getKey(), "false");
		
		assertEquals("false", pCat.getParameter(p.getKey()).getValue());
	}

	/*
	 * Test for void remove(int)
	 */
	public void testRemoveint() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		testCat.setPluginId(pCat.getPluginId());
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setPluginId(pCat.getPluginId());
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		assertEquals(1, testCat.getChildCount());
		testCat.remove(0);
		assertEquals(0, testCat.getChildCount());
	}

	/*
	 * Test for void remove(IParameterCategory)
	 */
	public void testRemoveIParameterCategory() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		testCat.setPluginId(pCat.getPluginId());
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setPluginId(pCat.getPluginId());
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		assertEquals(1, testCat.getChildCount());
		testCat.remove(testCat1);
		assertEquals(0, testCat.getChildCount());
	}

	public void testRemoveFromParent() {
		IParameterCategory testCat =
			new ParameterCategory("Test Category", configuration);
		testCat.setFullName("XHD TEST CATEGORY");
		testCat.setPluginId(pCat.getPluginId());
		pCat.insert(testCat);
		IParameterCategory testCat1 =
			new ParameterCategory("Test Category 1", configuration);
		testCat1.setFullName("XHD TEST CATEGORY 1");
		testCat1.setPluginId(pCat.getPluginId());
		testCat1.setLeaf(true);
		testCat.insert(testCat1);
		assertEquals(1, pCat.getChildCount());
		testCat.removeFromParent();
		assertEquals(0, pCat.getChildCount());
	}

	public void testRemoveParameter() {
		IParameter p = new Parameter("Test Parameter", configuration, pCat);
		p.setFullName("XHD TEST PARAMETER");
		p.setDefaultValue("Here's another fine mess you've gotten me into, Ollie!");
		p.setKey("Test Parameter^USER^^1");
		pCat.putParameter(p);
		assertEquals("Here's another fine mess you've gotten me into, Ollie!", pCat.getParameter("Test Parameter^USER^^1").getDefaultValue());
		pCat.removeParameter("Test Parameter^USER^^1");
		assertNull(pCat.getParameter("Test Parameter^USER^^1"));
	}

	/*
	 * Test for String toString()
	 */
	public void testToString() {
		assertEquals("Desktop", pCat.toString());
	}

	public void testUpdateParameter() {
		IParameter p = new Parameter("Test Parameter", configuration, pCat);
		p.setFullName("XHD TEST PARAMETER");
		p.setDefaultValue("Here's another fine mess you've gotten me into, Ollie!");
		p.setKey("Test Parameter^USER^^1");
		pCat.putParameter(p);
		assertEquals("Here's another fine mess you've gotten me into, Ollie!", pCat.getParameter("Test Parameter^USER^^1").getDefaultValue());
		pCat.updateParameter("Test Parameter^USER^^1", "Test Parameter^USER^^2", "Here's another fine mess");
		assertNull(pCat.getParameter("Test Parameter^USER^^1"));
		assertEquals("Here's another fine mess", pCat.getParameter("Test Parameter^USER^^2").getValue());
	}

}
