package gov.va.med.hds.cd.config.test;

import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.ConfigurationReadyEvent;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurationReadyListener;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IParameterChangeListener;
import gov.va.med.hds.cd.config.IParameterTree;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.ParameterChangeEvent;
import junit.framework.TestCase;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfigurationTest extends TestCase implements IConfigurationReadyListener, IParameterChangeListener {
	private IConfiguration config = null;
	private int eventCount = 0;

	/**
	 * Constructor for ConfigurationTest.
	 * @param arg0
	 */
	public ConfigurationTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(ConfigurationTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		config = new MockConfigurator().getConfiguration("gov.va.med.hds.dash", this);
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testAddParameterChangeListener() {
		config.addParameterChangeListener(this);
		assertEquals(1, config.getListenerCount(IParameterChangeListener.class));
	}

	public void testRemoveParameterChangeListener() {
		config.addParameterChangeListener(this);
		assertEquals(2, config.getListenerCount(IParameterChangeListener.class));
		config.removeParameterChangeListener(this);
		assertEquals(1, config.getListenerCount(IParameterChangeListener.class));
	}

	public void testGetParameterTree() {
		IParameterTree pTree = config.getParameterTree();
		IParameterCategory pRoot = (IParameterCategory) pTree.getRoot();
		assertEquals("Dashboard", pRoot.toString());
	}

	public void testGetParameterRoot() {
		IParameterCategory pRoot = config.getParameterRoot();
		assertEquals("Dashboard", pRoot.toString());
	}

	public void testGetParameter() {
		assertEquals("XP", config.getParameterValue("Look and Feel"));
	}

	public void testPutParameter() {
		try {
			config.putParameter("Look and Feel", "Aquamarine");
			assertEquals("Aquamarine", config.getParameterValue("Look and Feel"));
			config.putParameter("Look and Feel", "XP");
		} catch (NonExistentParameterException e) {
			fail("NonExistentParameterException: " + e.getMessage());
		} catch (ConfigurationException ce) {
			fail("ConfigurationException: " + ce.getMessage());
		}
	}

	public void testGetCategoryForParameter() {
		IParameterCategory pCat = config.getCategoryForParameter("Look and Feel");
		assertEquals("Dashboard", pCat.toString());
	}
	/**
	 * @see gov.va.med.hds.cd.config.IParameterChangeListener#parameterChanged(gov.va.med.hds.cd.config.ParameterChangeEvent)
	 */
	public void parameterChanged(ParameterChangeEvent pce) {
		eventCount++;
		if (eventCount == 1){
			assertEquals("Aquamarine", pce.getModifiedParameter().getValue());
		} else if (eventCount == 2){
			assertEquals("XP", pce.getModifiedParameter().getValue());
		}
	}
	/**
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationReady(gov.va.med.hds.cd.config.ConfigurationReadyEvent)
	 */
	public void configurationReady(ConfigurationReadyEvent cre) {
	}
	/**
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationCallFailed(gov.va.med.hds.cd.config.ConfigurationException)
	 */
	public void configurationCallFailed(ConfigurationException ce) {
	}

}
