package gov.va.med.hds.cd.config.test;

import gov.va.med.hds.cd.config.ConfigDAOException;
import gov.va.med.hds.cd.config.ICallResultHandler;
import gov.va.med.hds.cd.config.IConfigDAO;
import gov.va.med.hds.cd.config.IConfigDAOListener;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IParameter;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IParameterTree;
import gov.va.med.hds.cd.config.ParameterCategoryException;
import gov.va.med.hds.cd.config.internal.ParameterCategory;
import gov.va.med.hds.cd.config.internal.ParameterTree;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.dom4j.Dom4jXPath;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
class MockConfigDAO implements IConfigDAO {
	private IParameterTree paramTree;
	private IParameterCategory paramRoot;
	private IConfiguration configuration;

	public MockConfigDAO() {

		try {
			paramRoot = buildParameterTree(getTree());
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParameterCategoryException e) {
			e.printStackTrace();
		}

		paramTree = new ParameterTree(paramRoot);

	}

	private IParameterCategory buildParameterTree(String xmlTree)
		throws ParameterCategoryException {

		IParameterCategory treeNode = null;
		SAXReader reader = new SAXReader();
		Document doc;

		try {
			doc = reader.read(new StringReader(xmlTree));
		} catch (DocumentException e) {
			throw new ParameterCategoryException(
				"couldn't get reader for xmlTree: ",
				e);
		}

		XPath xpath;
		try {
			xpath = (XPath) new Dom4jXPath("/getConfigurationCallResult/configTree/pluginParameterCategory");
		} catch (JaxenException e) {
			throw new ParameterCategoryException(
				"JaxenException at plugin root.",
				e);
		}

		List results;
		try {
			results = xpath.selectNodes(doc);
		} catch (JaxenException e) {
			throw new ParameterCategoryException(
				"JaxenException at selectNodes.",
				e);
		}

		Iterator resultIter = results.iterator();

		if (!resultIter.hasNext())
			return null;

		//first, establish root node	
		Object object = resultIter.next();
		if (object instanceof Node) {
			try {
				treeNode = new ParameterCategory((Node) object, configuration);
			} catch (ParameterCategoryException e) {
				throw new ParameterCategoryException(
					"couldn't establish root node.",
					e);
			}
		}

		//then, get all descendents
		if (treeNode != null) {
			while (resultIter.hasNext()) {
				Object xmlCat = resultIter.next();
				if (xmlCat instanceof Node) {
					try {
						treeNode.insert(
							new ParameterCategory((Node) object, treeNode, configuration));
					} catch (ParameterCategoryException e) {
						throw new ParameterCategoryException(
							"couldn't insert child node.",
							e);
					}
				}
			}
		}
		return treeNode;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#putParameterTree(gov.va.med.hds.cd.config.IParameterTree)
	 */
	public void putParameterTree(IParameterTree parameterTree) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#removeParameterCategory(gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void removeParameterCategory(IParameterCategory pc) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(
	IRuntimeContext runtime,
	IConfigurationElement config,
	String propertyName, Object data)
	throws Exception{

	}

    private static byte[] inputStreamToByteArray(InputStream in, boolean close)
        throws IOException
    {
        byte [] result = null;
        ByteArrayOutputStream slurpBuffer = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int bytesRead;
        while ((bytesRead = in.read(buffer)) != -1)
            slurpBuffer.write(buffer, 0, bytesRead);
        result = slurpBuffer.toByteArray();
        if (close) try { in.close(); } catch (IOException ioe) {}
        return result;
    }

	/**
	 * Method getTree.
	 * @return String
	 */
	private String getTree() throws IOException {
		
		return new String(inputStreamToByteArray(getClass().getResourceAsStream("MockCallResults.xml"), true));

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#insertParameterCategory(gov.va.med.hds.cd.config.IParameterCategory, gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void insertParameterCategory(
		IParameterCategory pc,
		IParameterCategory parent)
		throws ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#putParameter(java.lang.String, java.lang.String)
	 */
	public void putParameter(String key, String value)
		throws ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#updateParameterCategory(gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void updateParameterCategory(IParameterCategory pc)
		throws ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#deleteParameterCategory(gov.va.med.hds.cd.config.IParameterCategory, boolean)
	 */
	public void deleteParameterCategory(
		IParameterCategory pc,
		boolean deleteSubCategories)
		throws ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#setConfiguration(gov.va.med.hds.cd.config.IConfiguration)
	 */
	public void setConfiguration(IConfiguration configuration) {
		this.configuration = configuration;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#query(gov.va.med.hds.cd.m.ICallRequest, gov.va.med.hds.cd.config.ICallResultHandler, gov.va.med.hds.cd.config.IConfigDAOListener)
	 */
	public void query(
		ICallRequest request,
		ICallResultHandler handler,
		IConfigDAOListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#removeParameterCategory(gov.va.med.hds.cd.config.IParameterCategory, gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void removeParameterCategory(
		IParameterCategory pc,
		IParameterCategory parent)
		throws ParameterCategoryException, NumberFormatException, ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#addParameterToCategory(gov.va.med.hds.cd.config.IParameter, gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void addParameterToCategory(
		IParameter parameter,
		IParameterCategory pc)
		throws ConfigDAOException {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.config.IConfigDAO#removeParameterFromCategory(gov.va.med.hds.cd.config.IParameter, gov.va.med.hds.cd.config.IParameterCategory)
	 */
	public void removeParameterFromCategory(
		IParameter parameter,
		IParameterCategory pc)
		throws ConfigDAOException {
	}

	/**
	 * @param pluginID
	 * @return
	 */
	public IParameterTree getParameterTree(String pluginID) {
		return paramTree;
	}

	/**
	 * @param pluginID
	 * @return
	 */
	public IParameterCategory getParameterRoot(String pluginID) {
		return paramRoot;
	}


}
