package gov.va.med.hds.cd.config.dialog;

import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.FieldEditor;
import gov.va.med.hds.cd.config.FieldEditorList;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IFieldEditor;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IPreferencePanel;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.spectrum.GradientPanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
/**
 * @author Joel Russell
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 **/
public class PreferencePanel extends JPanel implements IPreferencePanel {

	private GradientPanel titlePanel;
	private MessagePanel messagePanel;
	private JPanel southPanel;
	private JPanel buttonPanel;
	private JButton restoreButton;
	private JButton applyButton;
	private PreferencePage preferencePage;
	private IParameterCategory pCat = null;
	private IConfiguration configuration = null;
	private JPanel editorPanel;
	private String title;

	/** Creates new form UiPreferencePanel */
	public PreferencePanel(String title) {
		this.title = title;
		initGUI();
	}

	/** This method is called from within the constructor to initialize the form. */
	private void initGUI() {
		editorPanel = new JPanel();
		applyButton = new JButton();
		applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((PreferencePanel.this.configuration != null)
					&& (PreferencePanel.this.preferencePage != null)) {
					FieldEditorList dirtyEditorList =
						preferencePage.getDirtyEditors();
					for (int i = 0; i < dirtyEditorList.size(); i++) {
						IFieldEditor dirtyEditor = dirtyEditorList.get(i);
						try {
							dirtyEditor.doApply();
						} catch (ConfigurationException e) {
							showErrorMessage(e.getMessage());
						}
						((FieldEditor) dirtyEditor).setDirty(false);
					}
				}
			}
		});
		restoreButton = new JButton();
		restoreButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((PreferencePanel.this.configuration != null)
					&& (PreferencePanel.this.preferencePage != null)) {
					FieldEditorList fieldEditorList =
						preferencePage.getFieldEditors();
					for (int i = 0; i < fieldEditorList.size(); i++) {
						IFieldEditor fieldEditor = fieldEditorList.get(i);
						fieldEditor.loadDefault();
						((FieldEditor) fieldEditor).invalidate();
					}
				}
			}
		});
		buttonPanel = new JPanel();
		buttonPanel.setPreferredSize(new Dimension(225, 45));
		buttonPanel.setSize(new Dimension(225, 45));
		buttonPanel.setMinimumSize(new Dimension(220, 45));
		buttonPanel.setMaximumSize(new Dimension(320, 45));
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(restoreButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(5, 21)));
		buttonPanel.add(applyButton);
		buttonPanel.add(Box.createHorizontalGlue());
		southPanel = new JPanel();
		titlePanel =
			new GradientPanel(
				title,
				"",
				new Color(0, 0, 0),
				new Color(255, 255, 255),
				GradientPanel.LEFT_TO_RIGHT);
		messagePanel = new MessagePanel();
		messagePanel.setVisible(false);
		setLayout(new BorderLayout());
		setPreferredSize(new Dimension(390, 385));
		setSize(new Dimension(390, 385));
		add(editorPanel, BorderLayout.CENTER);
		add(southPanel, BorderLayout.SOUTH);
		add(titlePanel, BorderLayout.NORTH);
		titlePanel.setMinimumSize(new Dimension(10, 20));
		titlePanel.setPreferredSize(new Dimension(10, 20));
		titlePanel.setSize(new Dimension(390, 20));
		southPanel.setPreferredSize(new Dimension(10, 45));
		southPanel.setMinimumSize(new Dimension(10, 45));
		southPanel.setLayout(new BorderLayout());
		southPanel.setMaximumSize(new Dimension(32767, 45));
		southPanel.add(buttonPanel, BorderLayout.EAST);
		restoreButton.setText("Restore Defaults");
		restoreButton.setMnemonic('D');
		restoreButton.setMargin(new java.awt.Insets(2, 5, 2, 5));
		restoreButton.setMaximumSize(new Dimension(125, 21));
		restoreButton.setMinimumSize(new Dimension(100, 21));
		restoreButton.setPreferredSize(new Dimension(115, 21));
		applyButton.setText("Apply");
		applyButton.setToolTipText("");
		applyButton.setMnemonic('A');
		applyButton.setSize(new Dimension(75, 21));
		applyButton.setMaximumSize(new Dimension(75, 21));
		applyButton.setMinimumSize(new Dimension(75, 21));
		applyButton.setPreferredSize(new Dimension(75, 21));
		editorPanel.setLayout(new BorderLayout());
	}

	public void setTitle(String title) {
		titlePanel.setTitle(title);
	}
	/**
	 * Sets the preferencePage.
	 * @param preferencePage The preferencePage to set
	 */
	public void setPrefPage(PreferencePage prefPage, IParameterCategory pCat) {
		removeAll();
		this.pCat = pCat;
		if (pCat != null) {
			configuration = pCat.getConfiguration();
		}
		title = (pCat == null ? "" : pCat.toString());
		initGUI();
		if (prefPage != null) {
			preferencePage = prefPage;
			editorPanel.add(prefPage, BorderLayout.CENTER);
		}
		invalidate();
	}

	/**
	 * Sets the pCat.
	 * @param pCat The pCat to set
	 */
	public void setParameterCategory(IParameterCategory pCat) {
		this.pCat = pCat;
		if (pCat != null) {
			this.configuration = pCat.getConfiguration();
		}
	}

	public void showErrorMessage(String messageText) {
		messagePanel.setVisible(true);
		messagePanel.setText(messageText);
		titlePanel.setVisible(false);
		this.remove(titlePanel);
		this.add(messagePanel, BorderLayout.NORTH);
		this.validate();
	}

	public void hideErrorMessage() {
		messagePanel.setText("");
		messagePanel.setVisible(false);
		titlePanel.setVisible(true);
		this.remove(messagePanel);
		this.add(titlePanel, BorderLayout.NORTH);
		this.validate();
	}

	private class MessagePanel extends JPanel {
		JLabel messageLabel = new JLabel();
		JSeparator separator = new JSeparator();

		/**
		 * Constructor for MessagePanel.
		 * @param layout
		 * @param isDoubleBuffered
		 */
		public MessagePanel(LayoutManager layout, boolean isDoubleBuffered) {
			super(layout, isDoubleBuffered);
		}

		/**
		 * Constructor for MessagePanel.
		 * @param layout
		 */
		public MessagePanel(LayoutManager layout) {
			super(layout);
		}

		/**
		 * Constructor for MessagePanel.
		 * @param isDoubleBuffered
		 */
		public MessagePanel(boolean isDoubleBuffered) {
			super(isDoubleBuffered);
		}

		/**
		 * Constructor for MessagePanel.
		 */
		public MessagePanel() {
			super();
		}

		public void setText(String messageText) {
			setMinimumSize(new Dimension(10, 20));
			setPreferredSize(new Dimension(10, 20));
			setSize(titlePanel.getSize());
			setLayout(new BorderLayout());
			messageLabel.setIcon(
				new ImageIcon(PreferencePanel.class.getResource("error_obj.gif")));
			messageLabel.setText(messageText);
			messageLabel.setIconTextGap(10);
			add(messageLabel, BorderLayout.WEST);
			add(separator, BorderLayout.SOUTH);
		}
	}

}
