package gov.va.med.hds.cd.config.dialog;

import gov.va.med.hds.cd.config.ConfigurationChangeEvent;
import gov.va.med.hds.cd.config.ConfigurationException;
import gov.va.med.hds.cd.config.ConfigurationReadyEvent;
import gov.va.med.hds.cd.config.FieldEditor;
import gov.va.med.hds.cd.config.FieldEditorList;
import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurationChangeListener;
import gov.va.med.hds.cd.config.IConfigurationReadyListener;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IFieldEditor;
import gov.va.med.hds.cd.config.IParameterCategory;
import gov.va.med.hds.cd.config.IParameterChangeListener;
import gov.va.med.hds.cd.config.IPreferencePageManager;
import gov.va.med.hds.cd.config.ParameterChangeEvent;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.config.PreferencePageException;
import gov.va.med.hds.cd.config.internal.ParameterCategory;
import gov.va.med.hds.cd.config.internal.ParameterTree;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.spectrum.EscapeDialog;
import gov.va.med.hds.cd.spectrum.OutlineRenderer;
import gov.va.med.hds.cd.spectrum.OutlineTree;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeSelectionModel;

/**
 *
 * @author Joel Russell
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class PreferencesDialog
	extends EscapeDialog
	implements
		KeyListener,
		IConfigurationReadyListener,
		IParameterChangeListener,
		IConfigurationChangeListener {
	private JPanel outlinePanel;
	private JPanel eastPanel;
	private JPanel northPanel;
	private JPanel southPanel;
	private JButton okButton;
	private JPanel buttonPanel;
	private JButton cancelButton;
	private JPanel outlinePanelWest;
	private JPanel outlinePanelSouth;
	private JPanel outlinePanelEast;
	private JPanel preferencePagePanel;
	private JScrollPane outlineScrollPane;
	private OutlineTree configTree;
	private PreferencePanel preferencePanel;
	private PreferencePageMap preferencePageMap = new PreferencePageMap();
	private IConfigurator configurator;
	private IRuntimeContext runtime;
	private IDesktop desktop;
	private IPreferencePageManager preferencePageManager;
	//	private IPluginRegistry pluginRegistry;
	private IPerspectiveRegistry perspectiveRegistry;
	private boolean isCanceledState = true;
	private URL url;
	private String errStr;
	private AudioInputStream audioInputStream;
	private double duration;
	private String keySequence = "";
	private Playback playback = new Playback();
	private IParameterCategory configRoot = new ParameterCategory("Root", null);
	private IParameterCategory retrieveNode =
		new ParameterCategory("Retrieving data...", null);
	private ParameterTree configTreeModel;

	/** Creates new form JDialog */
	public PreferencesDialog(
		Frame parent,
		boolean modal,
		IRuntimeContext runtime) {
		super(parent, modal);

		this.runtime = runtime;

		try {
			configurator =
				(IConfigurator) runtime.getPluginInstance(
					IConfigurator.PLUGIN_ID);
			preferencePageManager =
				(IPreferencePageManager) runtime.getPluginInstance(
					IPreferencePageManager.PLUGIN_ID);
		} catch (PluginStartupException e) {
			e.printStackTrace();
		} catch (PluginNotFoundException e) {
			e.printStackTrace();
		}

		//		pluginRegistry = runtime.getPluginRegistry();
		desktop = PrismUtilities.getDesktop(runtime);
		perspectiveRegistry = desktop.getPerspectiveRegistry();

		initGUI();

		pack();

		int x = parent.getX() + ((parent.getWidth() - getWidth()) / 2);
		int y = parent.getY() + ((parent.getHeight() - getHeight()) / 2);
		setLocation(x, y);

	}

	private void showErrorMessage(final Exception e) {
		class DoShowDialog implements Runnable {
			boolean proceedConfirmed;
			public void run() {
				JOptionPane.showMessageDialog(
					getRootPane(),
					e.getMessage(),
					"Data Access Error!",
					JOptionPane.ERROR_MESSAGE);
			}
		}
		DoShowDialog doShowDialog = new DoShowDialog();

		/* The invokeAndWait utility can throw an InterruptedException,
		 * which we don't bother with, and an InvocationException.
		 * The latter occurs if our Runnable throws - which would indicate
		 * a bug in DoShowDialog.  The invokeAndWait() call will not return 
		 * until the user has dismissed the modal confirm dialog.
		 */
		try {
			SwingUtilities.invokeAndWait(doShowDialog);
		} catch (java.lang.reflect.InvocationTargetException iTe) {
			iTe.printStackTrace();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}
	}

	/** This method is called from within the constructor to initialize the form. */
	private void initGUI() {
		configTree = new OutlineTree();
		outlineScrollPane = new JScrollPane();
		preferencePagePanel = new JPanel();
		preferencePanel = new PreferencePanel("Preferences");
		outlinePanelEast = new JPanel();
		outlinePanelSouth = new JPanel();
		outlinePanelWest = new JPanel();
		cancelButton = new JButton();
		okButton = new JButton();
		buttonPanel = new JPanel();
		southPanel = new JPanel();
		northPanel = new JPanel();
		eastPanel = new JPanel();
		outlinePanel = new JPanel();
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				closeDialog(evt);
			}
		});
		setSize(new Dimension(600, 480));
		setTitle("Preferences");
		setModal(true);
		getContentPane().add(preferencePagePanel, BorderLayout.CENTER);
		getContentPane().add(southPanel, BorderLayout.SOUTH);
		getContentPane().add(northPanel, BorderLayout.NORTH);
		getContentPane().add(eastPanel, BorderLayout.EAST);
		getContentPane().add(outlinePanel, BorderLayout.WEST);
		southPanel.setMaximumSize(new Dimension(32767, 60));
		southPanel.setMinimumSize(new Dimension(10, 40));
		southPanel.setPreferredSize(new Dimension(596, 60));
		southPanel.setBorder(BorderFactory.createEtchedBorder());
		southPanel.setLayout(new BorderLayout());
		southPanel.add(buttonPanel, BorderLayout.EAST);
		outlinePanel.setMaximumSize(new Dimension(200, 32767));
		outlinePanel.setMinimumSize(new Dimension(100, 200));
		outlinePanel.setPreferredSize(new Dimension(195, 385));
		outlinePanel.setLayout(new BorderLayout());
		outlinePanel.add(outlinePanelEast, BorderLayout.EAST);
		outlinePanel.add(outlinePanelSouth, BorderLayout.SOUTH);
		outlinePanel.add(outlinePanelWest, BorderLayout.WEST);
		outlinePanel.add(outlineScrollPane, BorderLayout.CENTER);
		buttonPanel.setMaximumSize(new Dimension(220, 32767));
		buttonPanel.setMinimumSize(new Dimension(180, 10));
		buttonPanel.setPreferredSize(new Dimension(200, 10));
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(okButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(5, 21)));
		buttonPanel.add(cancelButton);
		buttonPanel.add(Box.createHorizontalGlue());
		okButton.setText("OK");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				isCanceledState = false;
				doOK();
				setVisible(false);
				dispose();
			}
		});
		okButton.setMaximumSize(new Dimension(75, 21));
		okButton.setMinimumSize(new Dimension(75, 21));
		okButton.setPreferredSize(new Dimension(75, 21));
		cancelButton.setText("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				doCancel();
				setVisible(false);
				dispose();
			}
		});
		cancelButton.setMaximumSize(new Dimension(75, 21));
		cancelButton.setMinimumSize(new Dimension(75, 21));
		cancelButton.setPreferredSize(new Dimension(75, 21));
		outlinePanelEast.setMaximumSize(new Dimension(10, 32767));
		outlinePanelWest.setMaximumSize(new Dimension(10, 32767));
		northPanel.setMaximumSize(new Dimension(32767, 10));
		eastPanel.setMaximumSize(new Dimension(10, 32767));
		preferencePagePanel.setPreferredSize(new Dimension(390, 385));
		preferencePagePanel.setLayout(new BorderLayout());
		initTree();
		outlineScrollPane.getViewport().add(configTree);
		configTree.setBounds(new java.awt.Rectangle(73, 213, 74, 72));
		outlineScrollPane.setVerticalScrollBarPolicy(
			javax.swing.JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		outlineScrollPane.setHorizontalScrollBarPolicy(
			javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		preferencePagePanel.add(preferencePanel, BorderLayout.CENTER);

		configTree.addKeyListener(this);
		pack();
	}

	/**
	 * Method doOK.
	 */
	private void doOK() {

		// get all dirtyEditors from preferencePageMap
		FieldEditorList dirtyEditors = preferencePageMap.getDirtyEditors();

		// iterate through dirtyEditors and call their doApply() method
		for (int i = 0; i < dirtyEditors.size(); i++) {
			IFieldEditor dirtyEditor = dirtyEditors.get(i);

			try {
				dirtyEditor.doApply();
			} catch (ConfigurationException e) {
				e.printStackTrace();
			}
		}
	}

	private void doCancel() {

		// get all dirtyEditors from preferencePageMap
		FieldEditorList dirtyEditors = preferencePageMap.getDirtyEditors();

		// iterate through dirtyEditors and set their dirty field to false
		for (int i = 0; i < dirtyEditors.size(); i++) {
			IFieldEditor dirtyEditor = dirtyEditors.get(i);
			((FieldEditor) dirtyEditor).loadInitialValue();
			((FieldEditor) dirtyEditor).setDirty(false);
		}
	}

	private void initTree() {

		configTree.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);
		configTree.addTreeSelectionListener(new OutlineSelectionListener());

		configTree.setRootVisible(false);
		configTree.setShowsRootHandles(true);

		//Enable tool tips.
		ToolTipManager.sharedInstance().registerComponent(configTree);

		UIManager.put("ToolTip.background", new Color(255, 255, 200));

		configTree.setCellRenderer(
			new OutlineRenderer(OutlineRenderer.SUPPRESS_ICONS));

		configRoot.insert(retrieveNode);

		configTreeModel = new ParameterTree(configRoot);

		configTree.setModel(configTreeModel);

		((OutlineRenderer) (configTree.getCellRenderer())).setUseBold(false);

		buildModel();

	}

	private void buildModel() {
		IConfigurationElement[] preferencePageElements =
			preferencePageManager.getPreferencePageElements();
		IPerspectiveDescriptor[] perspectiveDescriptors =
			perspectiveRegistry.getPerspectiveDescriptors();

		for (int i = 0; i < preferencePageElements.length; i++) {
			String ppContext =
				preferencePageElements[i].getAttribute(
					IPreferencePageManager.CONTEXT_ATTRIBUTE);
			if (ppContext == null
				|| isPerspectiveActive(perspectiveDescriptors, ppContext)) {
				String pluginId =
					preferencePageElements[i].getAttribute(
						IPreferencePageManager.PLUGIN_ATTRIBUTE);
				if (pluginId != null) {
					try {
						configurator.getConfiguration(pluginId, this);
					} catch (ConfigurationException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	private boolean isPerspectiveActive(
		IPerspectiveDescriptor[] perspectiveDescriptors,
		String ppId) {
		boolean active = false;
		for (int j = 0; j < perspectiveDescriptors.length; j++) {
			String perspectiveId = perspectiveDescriptors[j].getId();
			if (ppId.equals(perspectiveId)) {
				active = true;
				break;
			}
		}
		return active;
	}

	private void buildConfigNode(IConfiguration configuration) {
		IParameterCategory pRoot = configuration.getParameterRoot();
		if ((pRoot != null)
			&& (!pRoot.getName().equals(""))
			&& (!pRoot.getPreferencePage().equals(""))) {
			if (retrieveNode != null) {
				configTreeModel.removeNodeFromParent(retrieveNode);
				retrieveNode = null;
			}
			configTreeModel.insertNodeInto(pRoot, configRoot);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.IConfigurationChangeListener#configurationChanged(gov.va.med.hds.cd.config.ConfigurationChangeEvent)
	 */
	public void configurationChanged(ConfigurationChangeEvent cce) {
	}

	/**
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationReady(gov.va.med.hds.cd.config.ConfigurationReadyEvent)
	 */
	public void configurationReady(ConfigurationReadyEvent cre) {
		IConfiguration configuration = cre.getConfiguration();
		configuration.addConfigurationChangeListener(this);
		buildConfigNode(configuration);
	}

	/**
	 * @see gov.va.med.hds.cd.config.IConfigurationReadyListener#configurationCallFailed(gov.va.med.hds.cd.config.ConfigurationException)
	 */
	public void configurationCallFailed(ConfigurationException ce) {
		showErrorMessage(ce);
	}

	/**
	 * @see gov.va.med.hds.cd.config.IParameterChangeListener#parameterChanged(gov.va.med.hds.cd.config.ParameterChangeEvent)
	 */
	public void parameterChanged(ParameterChangeEvent pce) {
	}

	private class OutlineSelectionListener implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent tSe) {

			IParameterCategory paramTreeNode =
				(IParameterCategory) configTree.getLastSelectedPathComponent();

			if ((paramTreeNode == null)
				|| !(paramTreeNode instanceof IParameterCategory))
				return;

			if (paramTreeNode != null) {

				String preferencePageId = paramTreeNode.getPreferencePage();
				PreferencePage preferencePage = null;

				if (preferencePageId != "") {
					if (preferencePageMap.containsKey(preferencePageId)) {
						preferencePage =
							preferencePageMap.get(preferencePageId);
					} else {
						try {
							preferencePage =
								(
									PreferencePage) preferencePageManager
										.getPreferencePage(
									preferencePageId);
							if (preferencePage == null) {
								throw new PreferencePageException(
									"Non-existent PreferencePage identified for category"
										+ paramTreeNode.toString());
							}
							preferencePage.setParameterCategory(paramTreeNode);
							preferencePage.setRuntime(runtime);
							preferencePage.initializeFieldEditors();
							preferencePageMap.add(
								preferencePageId,
								preferencePage);
						} catch (PreferencePageException e) {
							e.printStackTrace();
						}
					}
				}

				preferencePanel.setPrefPage(preferencePage, paramTreeNode);

			} else {
			}
		}

	}

	/** @return the return <code>true</code> if the dialog has been canceled */
	public boolean isCanceled() {
		return isCanceledState;
	}

	/** Closes the dialog */
	private void closeDialog(WindowEvent evt) {
		setVisible(false);
		dispose();
	}

	public void createAudioInputStream(URL url) {
		if (url != null) {
			try {
				this.url = url;
				errStr = null;
				audioInputStream = AudioSystem.getAudioInputStream(url);
				long milliseconds =
					(long) ((audioInputStream.getFrameLength() * 1000)
						/ audioInputStream.getFormat().getFrameRate());
				duration = milliseconds / 1000.0;
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		} else {
			System.err.println("Invalid URL.");
		}
	}

	/**
	 * Write data to the OutputChannel.
	 */
	public class Playback implements Runnable {

		SourceDataLine line;
		Thread thread;
		final int bufSize = 32767;

		public void start() {
			errStr = null;
			thread = new Thread(this);
			thread.setName("Playback");
			thread.start();
		}

		public void stop() {
			thread = null;
		}

		private void shutDown(String message) {
			if ((errStr = message) != null) {
				System.err.println(errStr);
			}
			if (thread != null) {
				thread = null;
			}
		}

		public void run() {

			// reload the file if loaded by file
			if (url != null) {
				createAudioInputStream(url);
			}

			// make sure we have something to play
			if (audioInputStream == null) {
				shutDown("No loaded audio to play back");
				return;
			}
			// reset to the beginnning of the stream
			try {
				audioInputStream.reset();
			} catch (Exception e) {
				shutDown("Unable to reset the stream\n" + e);
				return;
			}

			// get an AudioInputStream of the desired format for playback
			AudioFormat format = getFormat();
			AudioInputStream playbackInputStream =
				AudioSystem.getAudioInputStream(format, audioInputStream);

			if (playbackInputStream == null) {
				shutDown(
					"Unable to convert stream of format "
						+ audioInputStream
						+ " to format "
						+ format);
				return;
			}

			// define the required attributes for our line, 
			// and make sure a compatible line is supported.

			DataLine.Info info =
				new DataLine.Info(SourceDataLine.class, format);
			if (!AudioSystem.isLineSupported(info)) {
				shutDown("Line matching " + info + " not supported.");
				return;
			}

			// get and open the source data line for playback.

			try {
				line = (SourceDataLine) AudioSystem.getLine(info);
				line.open(format, bufSize);
			} catch (LineUnavailableException ex) {
				shutDown("Unable to open the line: " + ex);
				return;
			}

			// play back the captured audio data

			int frameSizeInBytes = format.getFrameSize();
			int bufferLengthInFrames = line.getBufferSize() / 8;
			int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
			byte[] data = new byte[bufferLengthInBytes];
			int numBytesRead = 0;

			// start the source data line
			line.start();

			while (thread != null) {
				try {
					if ((numBytesRead = playbackInputStream.read(data))
						== -1) {
						break;
					}
					int numBytesRemaining = numBytesRead;
					while (numBytesRemaining > 0) {
						numBytesRemaining
							-= line.write(data, 0, numBytesRemaining);
					}
				} catch (Exception e) {
					shutDown("Error during playback: " + e);
					break;
				}
			}
			// we reached the end of the stream.  let the data play out, then
			// stop and close the line.
			if (thread != null) {
				line.drain();
			}
			line.stop();
			line.close();
			line = null;
			shutDown(null);
		}
	} // End class Playback

	public AudioFormat getFormat() {

		AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
		float rate = 44100.0f;
		int sampleSize = 16;
		boolean bigEndian = false;
		int channels = 2;

		return new AudioFormat(
			encoding,
			rate,
			sampleSize,
			channels,
			(sampleSize / 8) * channels,
			rate,
			bigEndian);
	}

	/**
	 * @see java.awt.event.KeyListener#keyPressed(KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
	}

	/**
	 * @see java.awt.event.KeyListener#keyReleased(KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		if (e.getModifiers() == 2 && keySequence.length() < 3) {
			keySequence = keySequence + KeyEvent.getKeyText(e.getKeyCode());
		}
		if (keySequence.equals("FAV")) {
			url = getClass().getResource("favcolor.wav");

			createAudioInputStream(url);
			playback.start();
			keySequence = "";
		}
	}

	/**
	 * @see java.awt.event.KeyListener#keyTyped(KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
	}

}
