package gov.va.med.hds.cd.config;

import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class BooleanFieldEditor
	extends FieldEditor
	implements IFieldEditor, ItemListener {

	private IParameterCategory category;
	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	public static final int DEFAULT_STYLE = 0, SEPARATE_LABEL = 1;
	private int style = DEFAULT_STYLE;

	protected BooleanFieldEditor() {
		super(new JCheckBox());
		((JCheckBox) field).setFocusPainted(true);
	}

	public BooleanFieldEditor(String labelText, int style) {
		super(new JCheckBox());
		this.style = style;
		((JCheckBox) field).setFocusPainted(true);
		setLabelText(labelText);
		initGUI();
	}
	
	public BooleanFieldEditor(String labelText, int style, boolean flat) {
		super(new JCheckBox());
		this.style = style;
		((JCheckBox) field).setBorderPaintedFlat(flat);
		((JCheckBox) field).setFocusPainted(true);
		setLabelText(labelText);
		initGUI();
	}

	private void initGUI() {
		setLayout(new GridLayout(0, 2));
		labelPanel.add(label);
		add(labelPanel);
		fieldPanel.add(field);
		add(fieldPanel);

		((JCheckBox) field).addItemListener(this);
	}

	/**
	 * @see java.awt.event.ItemListener#itemStateChanged(ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		if (parameter != null) {
			setDirty(true);
			parameter.setValue(
				(e.getStateChange() == ItemEvent.SELECTED ? "true" : "false"));
		}
	}

	public void setCheckedState(boolean isChecked) {
	}

	public void setLabelText(String labelText) {
		if (style == SEPARATE_LABEL) {
			label.setText(labelText);
		} else {
			label.setText("");
			((JCheckBox) field).setText(labelText);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			((JCheckBox) field).setSelected(
				"true".equalsIgnoreCase(parameter.getDefaultValue()));
			parameter.setValue(parameter.getDefaultValue());
			setDirty(true);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			((JCheckBox) field).setSelected(
				"true".equalsIgnoreCase(parameter.getInitialValue()));
		}
	}

	/**
	 * Method setValue.
	 * @param value
	 */
	public void setValue(boolean value) {
		((JCheckBox) field).setSelected(value);
	}

	public boolean getValue() {
		return ((JCheckBox) field).isSelected();
	}

}
