package gov.va.med.hds.cd.config;

import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ComboBoxFieldEditor
	extends FieldEditor
	implements IFieldEditor, ItemListener {

	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	private String[] comboBoxModel =
		new String[] { "Metal", "Motif", "Winders" };
	private Object[] itemValues =
		new String[] {
			"javax.swing.plaf.metal.MetalLookAndFeel",
			"com.sun.java.swing.plaf.motif.MotifLookAndFeel",
			"com.sun.java.swing.plaf.windows.WindowsLookAndFeel" };
	private Map selectionMap = new HashMap();
	private ComboBoxModel model;
	private Map indexMap = new HashMap();

	public ComboBoxFieldEditor() {
		super(new JComboBox());
		label.setText("Select Item:");
		initGUI(new EmptyBorder(1, 1, 1, 1));
	}

	public ComboBoxFieldEditor(String label, Border border)
		throws ComboBoxFieldEditorException {
		super(new JComboBox());

		this.label.setText(label);

		initGUI(border);
	}

	public ComboBoxFieldEditor(String label, Object[] objects, Border border)
		throws ComboBoxFieldEditorException {
		super(new JComboBox());

		if (objects == null) {
			throw new ComboBoxFieldEditorException("No objects array passed to constructor");
		}
		this.label.setText(label);

		model = new DefaultComboBoxModel(objects);
		((JComboBox) field).setModel(model);
		initGUI(border);
	}

	public ComboBoxFieldEditor(
		String label,
		ComboBoxModel model,
		Border border) {
		super(new JComboBox(model));
		comboBoxModel = null;

		for (int i = 0; i < model.getSize(); i++) {
			selectionMap.put(
				model.getElementAt(i).toString(),
				model.getElementAt(i));
			indexMap.put(model.getElementAt(i), model.getElementAt(i).toString());
		}
		this.label.setText(label);

		initGUI(border);
	}

	public ComboBoxFieldEditor(
		String label,
		String[] comboBoxModel,
		String[] itemValues,
		Border border)
		throws ComboBoxFieldEditorException {
		super(new JComboBox(comboBoxModel));

		if (comboBoxModel.length != itemValues.length) {
			throw new ComboBoxFieldEditorException("comboBoxModel and itemValues MUST match!");
		}

		this.label.setText(label);
		this.comboBoxModel = comboBoxModel;
		this.itemValues = itemValues;
		for (int i = 0; i < comboBoxModel.length; i++) {
			selectionMap.put(comboBoxModel[i], itemValues[i]);
			indexMap.put(itemValues[i], comboBoxModel[i]);
		}
		initGUI(border);
	}

	private void initGUI(Border border) {

		((JComboBox) field).addItemListener(this);

		labelPanel.add(label);

		fieldPanel.add(field);

		setBorder(border);
		setLayout(new GridLayout(0, 2));
		add(labelPanel);
		add(fieldPanel);
	}

	public void setComboBoxModel(String[] elements) {
		setComboBoxModel(new DefaultComboBoxModel(elements));
		this.revalidate();
	}

	public void setComboBoxModel(String[] names, String[] values) {
		((JComboBox) field).setModel(new DefaultComboBoxModel(names));
		for (int i = 0; i < names.length; i++) {
			selectionMap.put(names[i], values[i]);
			indexMap.put(values[i], names[i]);
		}
		((JComboBox) field).revalidate();
	}
	
	public void setComboBoxModel(Object[] objects) {
		if (objects.length > 0) {
			setComboBoxModel(new DefaultComboBoxModel(objects));
			((JComboBox) field).revalidate();
		}
	}

	public void setComboBoxModel(ComboBoxModel model) {
		if (model.getSize() > 0) {
			((JComboBox) field).setModel(model);
			for (int i = 0; i < model.getSize(); i++) {
				selectionMap.put(
					model.getElementAt(i).toString(),
					model.getElementAt(i));
				indexMap.put(model.getElementAt(i), model.getElementAt(i).toString());
			}
			((JComboBox) field).revalidate();
		}
	}

	public void setRenderer(ListCellRenderer renderer) {
		((JComboBox) field).setRenderer(renderer);
	}
	
	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			((JComboBox) field).setSelectedItem(
				indexMap.get(parameter.getDefaultValue()));
			parameter.setValue(parameter.getDefaultValue());
			setDirty(true);
		}
	}

	/**
	 * @see java.awt.event.ItemListener#itemStateChanged(ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		if (parameter != null) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				setDirty(true);
				parameter.setValue(
					selectionMap
						.get(
							e
								.getItemSelectable()
								.getSelectedObjects()[0]
								.toString())
						.toString());
			}
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			((JComboBox) field).setSelectedItem(
				indexMap.get(parameter.getInitialValue()));
		}
	}

	public void setValue(Object value) {
		//		((JComboBox) field).setSelectedItem(indexMap.get(value));	
		((JComboBox) field).setSelectedItem(value);
	}

}
