package gov.va.med.hds.cd.config;

import java.util.EventObject;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ConfigurationChangeEvent extends EventObject {

	private IConfiguration currentConfiguration;
	private IParameterCategory subjectCategory;
	private IParameterCategory objectCategory;
	private int verb;
	
	public final static int 	CATEGORY_CHANGED = 0,
								CATEGORY_INSERTED = 1,
								CATEGORY_REMOVED = 2,
								CATEGORY_DELETED = 3;

	/**
	 * Constructor for ConfigurationChangeEvent.
	 * @param source
	 */
	public ConfigurationChangeEvent(
		IConfiguration source,
		IParameterCategory subject,
		IParameterCategory object,
		int verb) {
		super(source);
		currentConfiguration = source;
		subjectCategory = subject;
		objectCategory = object;
		this.verb = verb;
	}

	/**
	 * Returns the currentConfiguration.
	 * @return Configuration
	 */
	public IConfiguration getCurrentConfiguration() {
		return currentConfiguration;
	}

	/**
	 * Returns the subjectCategory.
	 * @return IParameterCategory
	 */
	public IParameterCategory getSubjectCategory() {
		return subjectCategory;
	}

	/**
	 * Returns the objectCategory.
	 * @return IParameterCategory
	 */
	public IParameterCategory getObjectCategory() {
		return objectCategory;
	}
	
	public int getVerb() {
		return verb;
	}

}
