package gov.va.med.hds.cd.config;

import gov.va.med.hds.cd.spectrum.date.DateComboBox;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JPanel;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class DateComboFieldEditor
	extends FieldEditor
	implements IFieldEditor, ActionListener {

	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	private Date startDate = null;
	private Date endDate = null;
	private DateComboBox dateCombo;
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
	private SimpleDateFormat sdfOut = new SimpleDateFormat("MM/dd/yyyy");

	/**
	 * Constructor for DateComboFieldEditor.
	 */
	public DateComboFieldEditor() {
		super(new DateComboBox());
		dateCombo = ((DateComboBox) this.field);
		initGUI();
	}

	public DateComboFieldEditor(String label) {
		super(new DateComboBox());
		this.label.setText(label);
		dateCombo = ((DateComboBox) this.field);
		initGUI();
	}

	public DateComboFieldEditor(String label, Date startDate) {
		super(new DateComboBox());
		this.label.setText(label);
		dateCombo = ((DateComboBox) this.field);
		dateCombo.setSelectedDate(startDate);
		initGUI();
	}

	private void initGUI() {

		labelPanel.add(label);

		fieldPanel.add(field);

		setLayout(new GridLayout(0, 2));
		add(labelPanel);
		add(fieldPanel);
		dateCombo.addActionListener(this);
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			try {
				if (parameter.getDefaultValue() != "") {
					dateCombo.setSelectedDate(
						sdf.parse(parameter.getDefaultValue()));
					setDirty(true);
				} else {
					dateCombo.setSelectedDate(new Date());
				}
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			try {
				if (parameter.getInitialValue() != "") {
					dateCombo.setSelectedDate(
						sdf.parse(parameter.getInitialValue()));
				}
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
	}

	public void setValue(Date newDate) {
		dateCombo.setSelectedDate(newDate);
	}

	public Date getValue() {
		return (Date) dateCombo.getSelectedDate();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (parameter != null) {
			parameter.setValue(
				sdf.format(
					(Date) ((DateComboBox) e.getSource()).getSelectedDate()));
			setDirty(true);
		}
	}
}
