package gov.va.med.hds.cd.config;

import java.awt.GridLayout;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerDateModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class DateSpinnerFieldEditor
	extends FieldEditor
	implements IFieldEditor, ChangeListener {

	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	private Date startDate = null;
	private Date endDate = null;
	private SpinnerDateModel spinnerDateModel =
		new SpinnerDateModel(new Date(), null, null, Calendar.DAY_OF_MONTH);
	private JSpinner dateSpinner;
	private String datePattern = "MM/dd/yyyy";
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmZ");

	/**
	 * Constructor for DateSpinnerFieldEditor.
	 */
	public DateSpinnerFieldEditor() {
		super(new JSpinner());
		dateSpinner = ((JSpinner) this.field);
		dateSpinner.setModel(spinnerDateModel);
		dateSpinner.setEditor(
			new JSpinner.DateEditor(dateSpinner, datePattern));
		dateSpinner.setBorder(new EmptyBorder(5,5,5,5));
		dateSpinner.getEditor().setBorder(new EmptyBorder(5,5,5,5));
		initGUI();
	}

	public DateSpinnerFieldEditor(String label) {
		super(new JSpinner());
		this.label.setText(label);
		dateSpinner = ((JSpinner) this.field);
		dateSpinner.setModel(spinnerDateModel);
		dateSpinner.setEditor(
			new JSpinner.DateEditor(dateSpinner, datePattern));
		initGUI();
	}

	public DateSpinnerFieldEditor(
		String label,
		Date startDate,
		Date endDate,
		String datePattern) {
		super(new JSpinner());
		this.label.setText(label);
		this.datePattern = datePattern;
		spinnerDateModel =
			new SpinnerDateModel(
				new Date(),
				startDate,
				endDate,
				Calendar.DAY_OF_MONTH);
		dateSpinner = ((JSpinner) this.field);
		dateSpinner.setModel(spinnerDateModel);
		dateSpinner.setEditor(
			new JSpinner.DateEditor(dateSpinner, datePattern));
		initGUI();
	}

	private void initGUI() {

		labelPanel.add(label);

		fieldPanel.add(field);

		setLayout(new GridLayout(0, 2));
		add(labelPanel);
		add(fieldPanel);
		dateSpinner.addChangeListener(this);
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			try {
				dateSpinner.setValue(sdf.parse(parameter.getDefaultValue()));
				setDirty(true);
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			try {
				dateSpinner.setValue(sdf.parse(parameter.getInitialValue()));
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {
		if (parameter != null) {
			parameter.setValue(sdf.format((Date) ((JSpinner) e.getSource()).getValue()));
			setDirty(true);
		}
	}

	public void setValue(Date newDate) {
		dateSpinner.setValue(newDate);
	}
	
	public Date getValue() {
		return (Date) dateSpinner.getValue();
	}
}
