package gov.va.med.hds.cd.config;

import gov.va.med.hds.util.mumps.StringUtil;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public abstract class FieldEditor extends JPanel implements IFieldEditor {

	protected IPreferencePanel	preferencePanel;
	protected JLabel				label = new JLabel();
	protected JComponent			field;
	protected String				errorText = "Invalid entry...";
	protected IParameterCategory	parameterCategory;
	protected IParameter			parameter;
	protected boolean				dirty = false;
	
	public FieldEditor(JComponent field) {
		this.field = field;
	}
	
	/**
	 * @see gov.va.med.hds.cd.config.IFieldEditor#setCategory(IParameterCategory)
	 */
	public void setCategory(IParameterCategory pCat) {
		this.parameterCategory = pCat;
	}

	/**
	 * @see gov.va.med.hds.cd.config.IFieldEditor#getCategory()
	 */
	public IParameterCategory getCategory() {
		return parameterCategory;
	}

//	/**
//	 * Returns the preferencePanel.
//	 * @return PreferencePanel
//	 */
//	public PreferencePanel getPreferencePanel() {
//		return preferencePanel;
//	}

	/**
	 * Sets the preferencePanel.
	 * @param preferencePanel The preferencePanel to set
	 */
	public void setPreferencePanel(IPreferencePanel preferencePanel) {
		this.preferencePanel = preferencePanel;
	}

	/**
	 * Returns the field.
	 * @return JComponent
	 */
	public JComponent getField() {
		return field;
	}

	/**
	 * Returns the label.
	 * @return JLabel
	 */
	public JLabel getLabel() {
		return label;
	}

	/**
	 * Sets the field.
	 * @param field The field to set
	 */
	public void setField(JComponent field) {
		this.field = field;
	}

	/**
	 * Sets the label.
	 * @param label The label to set
	 */
	public void setLabel(JLabel label) {
		this.label = label;
	}

	/**
	 * Returns the parameter.
	 * @return IParameter
	 */
	public IParameter getParameter() {
		return parameter;
	}

	/**
	 * Sets the parameter.
	 * @param parameter The parameter to set
	 */
	public void setParameter(IParameter parameter) {
		this.parameter = parameter;
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public abstract void doApply() throws ConfigurationException;
	
	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public abstract void loadDefault();

	/**
	 * Method showErrorMessage.
	 */
	protected void showErrorMessage() {
		if (preferencePanel != null)
			preferencePanel.showErrorMessage(errorText);
	}

	/**
	 * Method showErrorMessage.
	 */
	protected void hideErrorMessage() {
		if (preferencePanel != null)
			preferencePanel.hideErrorMessage();
	}

	/**
	 * Returns the dirty.
	 * @return boolean
	 */
	protected boolean isDirty() {
		return dirty;
	}

	/**
	 * Sets the dirty.
	 * @param dirty The dirty to set
	 */
	public void setDirty(boolean isDirty) {
		if (parameter != null) {
			String pKey = parameter.getKey();
			if (StringUtil.piece(StringUtil.piece(pKey, '^', 2),';',2) != "VA(200,") {
				pKey = StringUtil.setPiece(pKey, '^', 2, ";VA(200,");
			}
			parameter.setKey(pKey);
		}
		this.dirty = isDirty;
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public abstract void loadInitialValue();

}
