package gov.va.med.hds.cd.config;

import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.cd.m.ICallRequest;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public interface IConfigDAO extends IDataAccessObject {

	public static final String INTERFACE_NAME =
		"gov.va.med.hds.cd.config.IConfigDAO";

	void putParameterTree(IParameterTree parameterTree)
		throws ConfigDAOException;

	void removeParameterCategory(IParameterCategory pc)
		throws ParameterCategoryException, ConfigDAOException;

	void removeParameterCategory(
		IParameterCategory pc,
		IParameterCategory parent)
		throws ParameterCategoryException, NumberFormatException, ConfigDAOException;

	void putParameter(String key, String value) throws ConfigDAOException;

	void updateParameterCategory(IParameterCategory pc)
		throws ConfigDAOException;

	void insertParameterCategory(
		IParameterCategory pc,
		IParameterCategory parent)
		throws ConfigDAOException;

	void addParameterToCategory(IParameter parameter, IParameterCategory pc)
		throws ConfigDAOException;

	void removeParameterFromCategory(IParameter parameter, IParameterCategory pc)
		throws ConfigDAOException;

	/**
	 * Method deleteParameterCategory.
	 * Deletes Category identified by parameter pc and all its descendents
	 * from persistent store.
	 * 
	 * @param pc
	 * @throws ConfigDAOException
	 */
	void deleteParameterCategory(
		IParameterCategory pc,
		boolean deleteSubCategories)
		throws ConfigDAOException;

	/**
	 * Sets the configuration.
	 * @param configuration The IConfiguration reference for the DAO to keep
	 */
	void setConfiguration(IConfiguration configuration);

	/**
	 * Method query.
	 */
	void query(
		ICallRequest request,
		ICallResultHandler handler,
		IConfigDAOListener listener);

}
