package gov.va.med.hds.cd.config;

public interface IConfiguration {

	/**
	 * Method addConfigurationChangeListener.
	 * @param iConfigurationListener
	 */
	void addConfigurationChangeListener(IConfigurationChangeListener ccl);

	/**
	 * Method removeConfigurationChangeListener.
	 * @param iConfigurationListener
	 */
	void removeConfigurationChangeListener(IConfigurationChangeListener ccl);

	/**
	 * Method addParameterChangeListener.
	 * @param pcl
	 */
	void addParameterChangeListener(IParameterChangeListener pcl);

	/**
	 * Method removeParameterChangeListener.
	 * @param pcl
	 */
	void removeParameterChangeListener(IParameterChangeListener pcl);

	/**
	 * Method addConfigurationReadyListener.
	 * @param iConfigurationListener
	 */
	void addConfigurationReadyListener(IConfigurationReadyListener crl);

	/**
	 * Method removeConfigurationReadyListener.
	 * @param iConfigurationListener
	 */
	void removeConfigurationReadyListener(IConfigurationReadyListener crl);

	/**
	 * Method getParameterTree.
	 * @return IParameterTree
	 */
	IParameterTree getParameterTree();

	/**
	 * Method getParameterRoot.
	 * @return IParameterCategory
	 */
	IParameterCategory getParameterRoot();

	/**
	 * Method getParameterValue.
	 * @param ezKey
	 * @return String
	 */
	String getParameterValue(String ezKey);

	/**
	 * Method getParameter.
	 * @param ezKey
	 * @return IParameter
	 * @throws NonExistentParameterException
	 */
	IParameter getParameter(String ezKey) throws NonExistentParameterException;

	/**
	 * Method putParameter.
	 * @param ezKey
	 * @param value
	 * @throws NonExistentParameterException
	 * @throws ConfigurationException
	 */
	void putParameter(String ezKey, String value)
		throws NonExistentParameterException, ConfigurationException;

	/**
	 * Method putParameter.
	 * @param parameter
	 * @throws ConfigurationException
	 */
	void putParameter(IParameter parameter) throws ConfigurationException;

	/**
	 * Method getCategoryForParameter.
	 * @param ezKey
	 * @return IParameterCategory
	 */
	IParameterCategory getCategoryForParameter(String ezKey);

	/**
	 * Method getListenerCount.
	 * @return int
	 */
	int getListenerCount(Class type);

	/**
	 * Method updateParameterCategory.
	 * @param iParameterCategory
	 */
	void updateParameterCategory(IParameterCategory parameterCategory)
		throws ConfigDAOException;

	/**
	 * Method insertParameterCategory
	 * @param pc
	 * @param parent
	 * @throws ConfigDAOException
	 */
	void insertParameterCategory(
		IParameterCategory pc,
		IParameterCategory parent)
		throws ConfigDAOException;

}
