package gov.va.med.hds.cd.config;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public interface IConfigurator {

	public static final String PLUGIN_ID = "gov.va.med.hds.cd.config";

	/**
	 * Method getConfiguration. This is the <i>synchronous</i> call by which a client application can obtain an instance of its Configuration from the Configurator. The client plugin must supply a reference to a ConfigurationReadyListener.
	 * @param pluginId - the plugin ID (e.g., gov.va.med.hds.cd.prism).
	 * @param timeOut - the number of milliseconds to wait for the get configuration query to return from the server.
	 * @return IConfiguration - a reference to the interface for the plugin's Configuration.
	 * @throws ConfigurationException
	 */
	IConfiguration getConfiguration(String pluginId, int timeOut)
		throws ConfigurationException;

	/**
	 * Method getConfiguration. This is the <i>asynchronous</i> call by which a client application can obtain an instance of its Configuration from the Configurator. The client plugin must supply a reference to a ConfigurationReadyListener.
	 * @param pluginId - the plugin ID (e.g., gov.va.med.hds.cd.prism).
	 * @param crl - the Configuration Ready Listener that will be notified when the getConfiguration query returns from the server.
	 * @return IConfiguration - a reference to the interface for the plugin's Configuration.
	 * @throws ConfigurationException
	 */
	IConfiguration getConfiguration(
		String pluginId,
		IConfigurationReadyListener crl)
		throws ConfigurationException;

	/**
	 * Method parameterDefinitionQuery.
	 * @param listener
	 * @throws ConfigurationException
	 */
	void parameterDefinitionQuery(IParameterDefinitionCallListener listener)
		throws ConfigurationException;

}
