package gov.va.med.hds.cd.config;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public interface IParameter {

	/**
	 * Method getKey.
	 * @return String
	 */
	String getKey();
	
	/**
	 * Method getName.
	 * @return String
	 */
	String getName();

	/**
	 * Method getId.
	 * @return long
	 */
	long getId();
	
	/**
	 * Method getValue.
	 * @return String
	 */
	String getValue();

	/**
	 * Method getFullName.
	 * @return String
	 */
	String getFullName();
	
	/**
	 * Method getEntity.
	 * @return String
	 */
	String getEntity();
	
	/**
	 * Method getInstance.
	 * @return String
	 */
	String getInstance();
	
	/**
	 * Method setKey.
	 * @param key
	 */
	void setKey(String key);

	/**
	 * Method setName.
	 * @param name
	 */
	void setName(String name);
	
	/**
	 * Method setId.
	 * @param id
	 */
	void setId(long id);
	
	/**
	 * Method setEntity.
	 * @param entity
	 */
	void setEntity(String entity);
	
	/**
	 * Method setInstance.
	 * @param instance
	 */
	void setInstance(String instance);
	
	/**
	 * Method setValue.
	 * @param value
	 */
	void setValue(String value);
	
	/**
	 * Method setFullName.
	 * @param fullName
	 */
	void setFullName(String fullName);
	
	/**
	 * Returns the multiValued.
	 * @return boolean
	 */
	boolean isMultiValued();

	/**
	 * Returns the wordProcessing.
	 * @return boolean
	 */
	boolean isWordProcessing();

	/**
	 * Sets the multiValued.
	 * @param multiValued The multiValued to set
	 */
	void setMultiValued(boolean multiValued);

	/**
	 * Sets the wordProcessing.
	 * @param wordProcessing The wordProcessing to set
	 */
	void setWordProcessing(boolean wordProcessing);
	
	/**
	 * Returns the defaultValue of the parameter.
	 * @return String
	 */
	String getDefaultValue();
	
	/**
	 * Sets the defaultValue of the parameter.
	 **/
	void setDefaultValue(String defaultValue);
	
	/**
	 * Returns the initialValue of the parameter.
	 * @return String
	 */
	String getInitialValue();

	/**
	 * Sets the initialValue of the parameter.
	 **/
	void setInitialValue(String initialValue);
	
	/**
	 * Returns the extEntity.
	 * @return String
	 */
	String getExtEntity();
	
	/**
	 * Sets the extEntity.
	 * @param extEntity The extEntity to set
	 */
	void setExtEntity(String extEntity);

	/**
	 * Calls configuration.putParameter(this) to save parameter to backing store
	 **/
	void persist() throws ConfigurationException;

	/**
	 * Method getCategory.
	 * @return IParameterCategory
	 */
	IParameterCategory getCategory();	

	/**
	 * Sets the category.
	 * @param category The category to set
	 */
	void setCategory(IParameterCategory category);

}
