/*
 * Created on Feb 28, 2003 11:49:25 AM
 */
package gov.va.med.hds.cd.config;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
import gov.va.med.hds.cd.spectrum.longlist.LongListLookup;
import gov.va.med.hds.cd.spectrum.text.LongListField;
import gov.va.med.hds.cd.spectrum.text.StringField;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class LongListFieldEditor
	extends FieldEditor
	implements IFieldEditor, ActionListener {

	private String labelText = "Enter response: ";
	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	private int minLength = 0;
	private int maxLength = 80;

	protected LongListFieldEditor() {
		super(new LongListField(0, 80, 30, "Choose an Item", null, null));
	}

	/** Creates new form ConfigPrefFieldEditor */
	public LongListFieldEditor(
		String labelText,
		int minLength,
		int maxLength,
		int columns,
		String title,
		LongListLookup lookupBox,
		Object defaultItem) {
		super(new LongListField(minLength, maxLength, columns, title, lookupBox, defaultItem));
		this.label.setText(labelText);
		this.minLength = minLength;
		this.maxLength = maxLength;
		initGUI();
	}

	private void initGUI() {
		errorText =
			"Enter a response between "
				+ minLength
				+ " and "
				+ maxLength
				+ " in length.";
		
		field.setInputVerifier(new StringVerifier());
		
		((LongListField) field).addStringFieldActionListener(this);
		
		labelPanel.add(label);

		fieldPanel.add(field);

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		add(labelPanel);
		add(Box.createHorizontalGlue());
		add(fieldPanel);
	}

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		JTextField tf = (JTextField) e.getSource();
		boolean isValid = isStringValid(e.getActionCommand());

		if (!isValid) {
			tf.setBorder(new EtchedBorder(Color.RED, Color.RED));
			LongListFieldEditor.this.showErrorMessage();
		} else {
			tf.setBorder(new EtchedBorder());
			LongListFieldEditor.this.hideErrorMessage();

			if (parameter != null) {
				parameter.setValue(
					(isValid ? e.getActionCommand() : parameter.getValue()));
				setDirty(true);
			}
		}
	}

	private void commit() {

	}

	class StringVerifier extends InputVerifier {

		public boolean verify(JComponent input) {
			StringField tf = (StringField) input;
			return isStringValid(tf.getText());
		}

		public boolean shouldYieldFocus(JComponent input) {
			StringField tf = (StringField) input;
			boolean result;
			result = super.shouldYieldFocus(input);

			if (!result) {
				tf.setBorder(new EtchedBorder(Color.RED, Color.RED));
				LongListFieldEditor.this.showErrorMessage();
			} else {
				tf.setBorder(new EtchedBorder());
				LongListFieldEditor.this.hideErrorMessage();
				if (parameter != null) {
					parameter.setValue(tf.getText());
					setDirty(true);
				}
			}
			return result;
		}

	}

	/**
	 * validates input.
	 **/
	protected boolean isStringValid(String valueString) {
		if (valueString.length() >= minLength
			&& valueString.length() <= maxLength) {
			return true;
		} else {
			return false;
		}
	}

	public void setValidLength(int minLen, int maxLen) {
		minLength = minLen;
		maxLength = maxLen;
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			((LongListField) field).setValue(parameter.getDefaultValue());
			parameter.setValue(parameter.getDefaultValue());
			setDirty(true);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			((LongListField) field).setValue(parameter.getInitialValue());
		}
	}

	/**
	 * Method setValue.
	 * @param value
	 */
	public void setValue(String value) {
		if (isStringValid(value)) {
			((LongListField) field).setValue(value);
		}
	}
	
	/**
	 * Method getValue.
	 * @return String
	 */
	public String getValue() {
		String result = ((LongListField) field).getValue();
		if (isStringValid(result)) {
			return result;
		} else {
			return "invalid entry";
		}
	}
}
