package gov.va.med.hds.cd.config;

import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.awt.BorderLayout;
import java.awt.LayoutManager;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public abstract class PreferencePage
	extends JPanel
	implements IPreferencePage {

	protected FieldEditorList fieldEditors = new FieldEditorList();
	protected IParameterCategory parameterCategory;
	protected IConfiguration configuration;
	protected IRuntimeContext runtime;
	protected JTabbedPane tabs = new JTabbedPane(JTabbedPane.TOP);

	/**
	 * @see gov.va.med.hds.cd.config.IPreferencePage#setName(FieldEditor)
	 **/
	public abstract void setName(String name);

	/**
	 * @see gov.va.med.hds.cd.config.IPreferencePage#setName(FieldEditor)
	 **/
	public abstract String getName();

	/**
	 * @see gov.va.med.hds.cd.config.IPreferencePage#addFieldEditor(IFieldEditor)
	 */
	protected void addFieldEditor(FieldEditor pfe) {
		fieldEditors.add(pfe);
	}

	protected void addTab(String title, PreferencePage prefPage) {
		tabs.add(title, prefPage);
	}

	protected void addTab(
		String title,
		PreferencePage prefPage,
		int mnemonic) {
		tabs.add(title, prefPage);
		tabs.setMnemonicAt(tabs.getTabCount() - 1, mnemonic);
	}

	/**
	 * @see gov.va.med.hds.cd.config.IPreferencePage#removeFieldEditor(IFieldEditor)
	 */
	protected void removeFieldEditor(FieldEditor pfe) {
		fieldEditors.remove(pfe);
	}

	/**
	 * Method getFieldEditors.
	 * @return FieldEditorList
	 */
	public FieldEditorList getFieldEditors() {
		FieldEditorList feList = new FieldEditorList();
		for (int i = 0; i < fieldEditors.size(); i++) {
			feList.add(fieldEditors.get(i));
		}

		for (int i = 0; i < tabs.getTabCount(); i++) {
			PreferencePage prefPage = (PreferencePage) tabs.getComponent(i);
			FieldEditorList fe = prefPage.getFieldEditors();
			for (int j = 0; j < fe.size(); j++) {
				feList.add(fe.get(j));
			}
		}
		
		return feList;
	}

	/**
	 * Method getDirtyEditors.
	 * @return List
	 */
	public FieldEditorList getDirtyEditors() {
		FieldEditorList dirtyEditors = fieldEditors.getDirtyEditors();
		for (int i = 0; i < tabs.getTabCount(); i++) {
			PreferencePage prefPage = (PreferencePage) tabs.getComponent(i);
			FieldEditorList de = prefPage.getDirtyEditors();
			for (int j = 0; j < de.size(); j++) {
				dirtyEditors.add(de.get(j));
			}
		}
		return dirtyEditors;
	}

	/**
	 * Returns the parameterCategory.
	 * @return IParameterCategory
	 */
	protected IParameterCategory getParameterCategory() {
		return parameterCategory;
	}

	/**
	 * Sets the parameterCategory.
	 * @param parameterCategory The parameterCategory to set
	 */
	public void setParameterCategory(IParameterCategory parameterCategory) {
		this.parameterCategory = parameterCategory;
		this.configuration = parameterCategory.getConfiguration();
	}

	protected void initGUI(LayoutManager layoutManager) {
		setLayout(layoutManager);

		for (int i = 0; i < fieldEditors.size(); i++) {
			add(fieldEditors.get(i));
		}

		if (tabs.getTabCount() > 0) {
			if (layoutManager instanceof BorderLayout) {
				add(tabs, BorderLayout.CENTER);
			} else {
				add(tabs);
			}
		}

	}

	/**
	 * Returns the configuration.
	 * @return IConfiguration
	 */
	protected IConfiguration getConfiguration() {
		return configuration;
	}

	/**
	 * Sets the configuration.
	 * @param configuration The configuration to set
	 */
	protected void setConfiguration(IConfiguration configuration) {
		this.configuration = configuration;
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IPreferencePage#initializeFieldEditors()
	 */
	public abstract void initializeFieldEditors();

	/**
	 * @return
	 */
	public IRuntimeContext getRuntime() {
		return runtime;
	}

	/**
	 * @param context
	 */
	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
	}

}
