package gov.va.med.hds.cd.config;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class RadioGroupFieldEditor
	extends FieldEditor
	implements IFieldEditor, ActionListener {

	private ButtonGroup buttonGroup = new ButtonGroup();
	private TitledBorder titledBorder;
	private Map radioButtonMap = new HashMap();

	protected RadioGroupFieldEditor() {
		super(new JPanel());
	}

	/**
	 * Constructor for RadioGroupFieldEditor.
	 * @param field
	 */
	public RadioGroupFieldEditor(
		ArrayList radioButtons,
		Border border,
		String titleString) {
		super(new JPanel());

		titledBorder = BorderFactory.createTitledBorder(border, titleString);
		titledBorder.setTitleJustification(TitledBorder.LEFT);
		setBorder(titledBorder);
		setLayout(new GridLayout(radioButtons.size(), 1));

		Iterator rbIt = radioButtons.iterator();

		while (rbIt.hasNext()) {
			JRadioButton radioButton = (JRadioButton) rbIt.next();
			buttonGroup.add(radioButton);

			if (radioButton.getActionCommand() == "") {
				radioButton.setActionCommand(radioButton.getText());
			}
			radioButtonMap.put(radioButton.getText(), radioButton);
			radioButtonMap.put(radioButton.getText().toLowerCase(), radioButton);
			radioButton.addActionListener(this);
			add(radioButton);
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (parameter != null) {
			setDirty(true);
			parameter.setValue(e.getActionCommand().toLowerCase());
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			((JRadioButton) radioButtonMap.get(
					parameter.getDefaultValue())).setSelected(true);
			parameter.setValue(parameter.getDefaultValue());
			setDirty(true);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			String buttonName = parameter.getInitialValue();
			if (buttonName != "") {
				JRadioButton radioButton = (JRadioButton) radioButtonMap.get(buttonName);
				if (radioButton instanceof JRadioButton) {
					radioButton.setSelected(true);
				}
			}
		}
	}

}
