/*
 * Created on Apr 9, 2003 1:28:32 PM
 */
package gov.va.med.hds.cd.config;

import gov.va.med.hds.util.date.RelativeDateFormat;

import java.awt.GridLayout;
import java.text.ParseException;

import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class RelativeDateSpinnerFieldEditor
	extends FieldEditor
	implements IFieldEditor, ChangeListener {

	private JPanel labelPanel = new JPanel(new GridLayout(0, 1));
	private JPanel fieldPanel = new JPanel(new GridLayout(0, 1));
	private int minDate = -90;
	private int maxDate = 90;
	private JSpinner dateSpinner;
	private RelativeDateFormat relativeDateFormat;

	/**
	 * Constructor for RelativeDateSpinnerFieldEditor.
	 */
	public RelativeDateSpinnerFieldEditor() {
		super(new JSpinner(new RelativeDateSpinnerNumberModel(0, -90, 90, 1)));
		dateSpinner = ((JSpinner) this.field);
		initGUI();
	}

	public RelativeDateSpinnerFieldEditor(String label) {
		super(new JSpinner(new RelativeDateSpinnerNumberModel(0, -90, 90, 1)));
		this.label.setText(label);
		dateSpinner = ((JSpinner) this.field);
		initGUI();
	}

	public RelativeDateSpinnerFieldEditor(
		String label,
		int minDate,
		int maxDate) {
		super(new JSpinner(new RelativeDateSpinnerNumberModel(0, minDate, maxDate, 1)));
		this.label.setText(label);
		this.minDate = minDate;
		this.maxDate = maxDate;
		dateSpinner = ((JSpinner) this.field);
		initGUI();
	}
	
	private static class RelativeDateSpinnerNumberModel extends SpinnerNumberModel {
		
		public RelativeDateSpinnerNumberModel(int value, int minimum, int maximum, int stepSize) {
			super(value, minimum, maximum, stepSize);
		}

		protected void fireStateChanged() {
			super.fireStateChanged();
		}

	}

	private void initGUI() {
		JComponent editor = dateSpinner.getEditor();
		if (editor instanceof JSpinner.NumberEditor) {
			JSpinner.NumberEditor ne = (JSpinner.NumberEditor) editor;
			JFormattedTextField ftf = ne.getTextField();
			JFormattedTextField.AbstractFormatter af = ftf.getFormatter();
			relativeDateFormat = new RelativeDateFormat();
			if (af instanceof NumberFormatter) {
				((NumberFormatter) af).setFormat(relativeDateFormat);
				((RelativeDateSpinnerNumberModel) dateSpinner.getModel()).fireStateChanged();
			}
		}

		labelPanel.add(label);

		fieldPanel.add(field);

		setLayout(new GridLayout(0, 2));
		add(labelPanel);
		add(fieldPanel);
		dateSpinner.addChangeListener(this);
	}
	
	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#doApply()
	 */
	public void doApply() throws ConfigurationException {
		if (parameter != null) {
			parameter.persist();
		}
	}

	public void setText(String text) throws ParseException {
		dateSpinner.setValue(relativeDateFormat.parse(text));
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadDefault()
	 */
	public void loadDefault() {
		if (parameter != null) {
			try {
				setText(parameter.getDefaultValue().trim());
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			setDirty(true);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IFieldEditor#loadInitialValue()
	 */
	public void loadInitialValue() {
		if (parameter != null) {
			try {
				setText(parameter.getInitialValue().trim());
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {
		if (parameter != null) {
			parameter.setValue(
				relativeDateFormat.format(((JSpinner) e.getSource()).getValue()));
			setDirty(true);
		}
	}

	public void setValue(String newDate) {
		dateSpinner.setValue(newDate);
	}

	public String getValue() {
		return dateSpinner.getValue().toString();
	}
}
