package gov.va.med.hds.cd.daoregistry;

import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 * This IDAORegistry implementation provides a plugin implementation that runs
 * within the runtime framework.
 **/
public class DaoRegistry implements IDaoRegistry {

	private Map daoRegistryMap = new HashMap();
	private IRuntimeContext runtime;

	/**
	 * Constructor
	 */
	public DaoRegistry() {
		super();
	}

	/**
	 * @see gov.va.med.hds.cd.daoregistry.IDAORegistry#getDAO(String)
	 */
	public IDataAccessObject getDAO(String daoInterfaceName) throws Exception {

		IConfigurationElement ce = (IConfigurationElement) daoRegistryMap.get(daoInterfaceName);
		IDataAccessObject DAO = null;

		if (ce != null) {
			try {
				DAO =
					(IDataAccessObject) ce.createExecutableExtension(
						IDaoRegistry.CLASS_NAME_ATTRIBUTE);
			} catch (Exception e) {
				runtime.getLogger(getClass()).error(
					"Error creating executable extension for dao '" + daoInterfaceName + "'",
					e);
				throw e;
			}
		}
		return DAO;
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#startup(IRuntimeContext, IPluginDescriptor)
	 */
	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception {

		this.runtime = runtime;

		IPluginRegistry pluginRegistry = runtime.getPluginRegistry();
		IConfigurationElement[] extensions =
			pluginRegistry.getConfigurationElementsFor(IDaoRegistry.EXTENSION_POINT_ID);

		for (int i = 0; i < extensions.length; i++) {
			String key = extensions[i].getAttribute(IDaoRegistry.INTERFACE_NAME_ATTRIBUTE);
			if (!daoRegistryMap.containsKey(key)) {
				daoRegistryMap.put(key, extensions[i]);
			} else {
				runtime.getLogger(getClass()).error(
					"The data object registry already contains an entry for interface '"
						+ key
						+ "'");
			}
		}
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
	 */
	public void shutdown() throws Exception {
	}
}
