package gov.va.med.hds.cd.daoregistry.cookbook;

import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 * This class is used as an example of a Data Access Object
 * for cookbook demonstration purposes.
 * Note that setInitializationData is called by the DaoRegistry
 * plugin when this object is created.
 */
public class MyData implements IMyData {
	public MyData() {
		super();
	}

	public int getData() throws Exception {
		initMumps();
		// In all RPC calls an RPC key is passed in and the
		// real RPC name is looked up within the IMumps
		// implementation. Proper RPC option context
		// policy is applied. The actual RPC name is
		// never exposed through this interface.
		String result = mumps.callRPC("GETDATA");
		return Integer.parseInt(result);
	}
	public void setData(int data) throws Exception {
		initMumps();
		// add data as a single parameter to the call
		// Note that callRPC is overloaded and there are a couple of
		// ways to pass in parameters. callRPC can also be called 
		// asynchronously with a listener passed in.
		// An options for passing in a time limit is also available.
		// Please see the JavaDoc for details.
		mumps.callRPC("SETDATA",new Object[]{new Integer(data)});
	}
	/**
	 * This method is the entry point for initialization of an implementor
	 * of IDataAccessObject
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
	}
	private void initMumps() throws Exception{
		// Lets only create the IMumps associated with the
		// cookbook plugin once
		if(mumps != null)
			return;
			
		// Get the IMumpsFactory plugin from the runtime plugin registry
		IMumpsFactory factory = (IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID);
		
		// Find the plugin descriptor in the plugin registry
		// for "gov.va.med.hds.cd.cookbook" plugin
		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor("gov.va.med.hds.cd.daoregistry.cookbook");

		// Create a IMumps instance based on the plugin descriptor
		mumps = factory.create(desc);
	}
	private IRuntimeContext runtime;
	private IMumps mumps = null;
}
