/*
 * Created on May 19, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import java.util.ArrayList;
import java.util.List;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.dao.IDashboardEventsDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardEventItem;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardEventsDAO extends AbstractDashboardDAO implements IDashboardEventsDAO
{
  protected static final String EVENTS_BY_ID = "EVENTS BY ID";
  protected static final String EVENTS_BY_PATIENT ="EVENTS BY PATIENT";
  protected static final String ACKNOWLEDGE_EVENTS = "EVENTS ACKNOWLEDGE";
  
  private IItemEntry itemType;

  /**
   * 
   */
  public DashboardEventsDAO()
  {
    super();
    dbItemsByID = EVENTS_BY_ID;
    dbItemsByPatient = EVENTS_BY_PATIENT;
    updateByID = ACKNOWLEDGE_EVENTS;
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
	 */
	public List parseItems(Object data)
	{
    List events = new ArrayList();
    DashboardEventItem event = null;
    String[] eventData = ((String) data).split("\n");
    for (int l = 0; l < eventData.length; l++)
    {
      String[] line = eventData[l].split("=", 2);
      if ("Item".equalsIgnoreCase(line[0]))
      events.add((event = (DashboardEventItem)parseItem(line[1])));
      else if ("Text".equalsIgnoreCase(line[0]))
        parseText(event, line[1]);
      else if ("Cmnt".equalsIgnoreCase(line[0]))
      event.addComment(line[1]);
    }
    return events;
	}

  private IDashboardItem parseItem(String data)
  {
    // The ordered item looks like:
    // ORD:ID^^YYYYMMDDHHmmss
    String[] appointments = StringUtil.split(data, '^');
    IDashboardItem item = null;;
    try
    {
      item = (IDashboardItem)dbItems.get(appointments[0]);
      if (null == item)
      {
        item = (IDashboardItem)itemType.getInstance();
        item.setID(appointments[0]);
      }
      item.setDate((-1 != appointments[2].indexOf('.') ? (Object)StringUtil.fmDateToDate(appointments[2]) : appointments[2]));
      item.setShortTitle((0 != appointments[1].length() ? appointments[1] : 
        item.getFormattedDate()));
      item.setDirty(false);
      dbItems.put(item.getID(), item);
      return item;
    }
    catch (Exception e)
    {
      logger.error(e.getMessage(), e);
    }
    return item;
  }

  private void parseText(IDashboardItem item, String data)
  {
    // Text has a single line of text each row;
    StringBuffer text = (StringBuffer) item.getData();
    if (null == text)
      text = (StringBuffer)((DashboardEventItem)item)
        .createDataType();
    else
      text.append("\n");
    text.append(data);
    item.setData(text);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
   */
  public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception
  {
    super.setInitializationData(runtime, config, propertyName, null);
    itemType = itemTypes.getItemType(DashboardEventItem.ITEM_TYPE_ID);
  }

}
