package gov.va.med.hds.dashboard;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeConstants;
import gov.va.med.hds.dashboard.clinician.ClinicianDashboardPlugin;
import gov.va.med.hds.dashboard.ui.DashboardPatient;
import gov.va.med.hds.dashboard.ui.DashboardPatientList;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

/**
 * Factory for a clinician dashboard patient list.
 */
public class ClinicianDashboardPatient extends DashboardPatient {

	protected void addHelpIds() {
		component.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, IDashboardClinicianUIConstants.INTRO_HELP_ID);
	}

	protected JComponent createComponent(IVisualComponentDescriptor descriptor, IDesktop desktop) {
		JScrollPane scroller = null;
		IDashboardPlugin plugin = null;
		try {
			plugin = (IDashboardPlugin) desktop.getRuntime().getPluginInstance(ClinicianDashboardPlugin.PLUGIN_ID);

			dataModel = new DashboardPatientTableModel(plugin);
			dataModel.addDashItemMessageListener(this);
			patientList = new DashboardPatientList(dataModel, plugin);
		} catch (Exception e) {
			desktop.getRuntime().getLogger(this.getClass()).error(e.getLocalizedMessage(), e);
		}
		patientList.putClientProperty(IRuntimeConstants.PLUGIN_ID, desktop.getRuntime());
		patientList.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, "gov.va.med.hds.dashboard.intro");
		scroller = new JScrollPane(patientList);
		JLabel corner = new JLabel();
		corner.setOpaque(true);
		corner.setBackground(patientList.getSelectionBackground());
		scroller.setCorner(JScrollPane.UPPER_RIGHT_CORNER, corner);
		scroller.getViewport().setPreferredSize(patientList.getPreferredSize());
		return scroller;
	}

}
