package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JScrollPane;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.DashboardPatientTableModel;
import gov.va.med.hds.dashboard.IDashPatients;
import gov.va.med.hds.dashboard.IDashboardPlugin;
import gov.va.med.hds.dashboard.clinician.ClinicianDashboardPlugin;
import gov.va.med.hds.dashboard.dao.IDashboardEventsDAO;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.items.IDashboardEntry;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardEventItem;


/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems

 * @author vhaislschras
 *
 * 
 * @version 
 */

public class EventDetail extends AbstractItemDetail
{
  IDashboardEventsDAO dao;
  String patientID;
  
  public void service(IServletRequest request, IServletResponse response)
      throws IOException
  {
    init(request.getRuntimeContext());
    patientID = request.getParameter("patientID");
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    if (request.getParameter("display") != null)
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardEventItem.ITEM_TYPE_ID);
        dao = (IDashboardEventsDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge",
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, true, dao, display));
    out.write("</body></html>");
  }
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemFooter(gov.va.med.hds.dashboard.items.IDashboardItem, java.lang.StringBuffer)
	 */
	public void itemFooter(IDashboardItem item, StringBuffer html)
	{
//    String itemID = item.getID();
    Object patientID = null;
    try {
        patientID = ((IDashboardPatient)item.getPatient()).getPatientId();
    } catch (Exception e) {}

    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
        if (taskPlugin != null) {
            html.append("<a href=\"");    
            html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "Event", item.getID(), patientID));
            html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        }
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>&nbsp;&nbsp;&nbsp;");
        }
    }
    html.append("</td>");
/*    html.append("<td align='right'><input type='checkbox' id='ck_" + 
      itemID + "' onChange=\"acknowledge('ck_" + 
      itemID + "', '" + IDashItem.DASH_EVENT + "')\"" +
      (item.isComplete() ? " checked" : "") +
      " label='Acknowledge'>&nbsp;</td>"); */
    html.append("</tr></table>");
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#htmlBody(java.lang.String[], boolean, boolean, gov.va.med.hds.dashboard.dao.IDashboardItemDAO, boolean)
	 */
	public String htmlBody(String[] ids, boolean brief, boolean autoack, IDashboardItemDAO itemDAO, boolean display)
	{
		StringBuffer html = new StringBuffer(super.htmlBody(ids, brief, autoack, itemDAO, display));
    // This is to check for non-ADT notifications
    if (null != patientID)
  		try
  		{
  			IDashboardPlugin plugin = (IDashboardPlugin)runtime.getPluginInstance(ClinicianDashboardPlugin.PLUGIN_ID);
        IDashboardEntry def = plugin.getDashboardItemDefinition(IDashboardEventsDAO.ITEM_TYPE_ID);
        // Find the dashboard table
        DashboardPatientTableModel model = null;
        JComponent patients = plugin.getPatientListComponent();
        if (patients instanceof JScrollPane)
          patients = (JComponent)((JScrollPane)patients).getViewport().getView();
        if (patients instanceof IDashPatients)
          model = (DashboardPatientTableModel)((IDashPatients)patients).getInternalModel();
        // Get the specific patient
        if (null != model)
        {
          IDashboardPatient patient = model.getPatient(patientID);
          String url = (String)patient.visualizeDashItems(
            -def.getSequence(),
            IDashboardItem.DASH_VIEW_DETAIL);
          html.append("<div deferredsrc='" + url + "'>&nbsp;</div>"); 
        }
  		}
  		catch (Exception e)
  		{
  			runtime.getLogger(this.getClass()).error(e.getLocalizedMessage(), e);
  		}
    return html.toString();
	}

}

