package gov.va.med.hds.dashboard.ui.servlet;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardSignableDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardSignable;
import gov.va.med.hds.signature.ISignableItem;

import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Container for a signable object.
 */
public class SignableDetail extends AbstractItemDetail {
	private IDashboardSignableDAO dao;
	private boolean canSign = false;

	public void service(IServletRequest request, IServletResponse response) throws IOException {
		init(request.getRuntimeContext());
		String[] ids = request.getParameterList("id");
		boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

		boolean display = true; // default value
		boolean displaySet = false;

		if (request.getParameter("signer") != null) {
			canSign = Boolean.valueOf(request.getParameter("signer")).booleanValue();
			display = canSign;
		}

		if (request.getParameter("display") != null) {
			display = Boolean.valueOf(request.getParameter("display")).booleanValue();
			displaySet = true;
		}

		if (null == dao) {
			try {
				IDashboardItemTypes items =
					(IDashboardItemTypes) runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
				IItemEntry item = items.getItemType(DashboardSignable.ITEM_TYPE_ID);
				dao = (IDashboardSignableDAO) item.getItemDAOInstance();
			} catch (Exception pse) {
				throw new IOException(pse.toString());
			}
		}
		out = response.getWriter();
		response.setContentType("text/html");
		String[] scriptlets =
			{
				"gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
				"gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
				"gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge",
				"gov.va.med.hds.dashboard.ui.scriptlet.DisableCheckboxScriptlet" };

		out.write("<html><head>");
		out.write(htmlHeader(scriptlets));
		out.write("</head><body onload=\"disableCheckboxes()\">");
		out.write(htmlBody(ids, brief, false, dao, display, displaySet));
		out.write("</body></html>");
	}

	public String htmlBody(
		String[] ids,
		boolean brief,
		boolean autoack,
		IDashboardItemDAO itemDAO,
		boolean display,
		boolean displaySet) {
		StringBuffer html = new StringBuffer();
		List items = itemDAO.getItems(Arrays.asList(ids));
		for (Iterator iter = items.iterator(); iter.hasNext();) {
			IDashboardItem item = (IDashboardItem) iter.next();
			if (display && !displaySet)
				display = !("complete".equalsIgnoreCase(item.getStatus()));
			String itemID = item.getID();
			html.append("<div class='resultbox'>");
			itemHeader(item, (brief && display), html);
			html.append("<div " + (display ? "" : "class='displaynone'") + " id='" + itemID + "'>");
			html.append("<div>");
			itemDetail(item, brief, autoack, canSign, html);
			html.append("</div></div>");
			if (!brief)
				itemFooter(item, html);
			html.append("</div>");
			if (display)
				display = false;
		}
		if (!brief)
			pageFooter(html);
		return html.toString();
	}

	public void itemFooter(IDashboardItem item, StringBuffer html) {
		String itemID = item.getID();
		boolean requiresEnc = ((DashboardSignable) item).isRequiresEncounter();
		int chartMsgIndicator = (requiresEnc ? 1 : 0);
		String[] chartMsg = { "Go to Chart", "Go to Chart to Sign Document" };

		Object patientID = null;
		try {
			patientID = ((IDashboardPatient) item.getPatient()).getPatientId();
		} catch (Exception e) {
		}

		html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
		html.append("<td" + (requiresEnc ? "" : " width='380'") + ">&nbsp;");
		if (patientID != null) {
			if (taskPlugin != null) {
				html.append("<a href=\"");
				html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "Sign", item.getID(), patientID));
				html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
			}
			if (contextPlugin != null) {
				html.append("<a href=\"");
				html.append(contextPlugin.getCcowHref(patientID));
				html.append("\">" + chartMsg[chartMsgIndicator] + "</a>&nbsp;&nbsp;&nbsp;");
			}
		}
		html.append("</td>");
		if (requiresEnc)
			html.append("<td align='right'>Encounter data may be required&nbsp;</td>");
		else if (((ISignableItem) item).isSigned())
			html.append("<td align='right'>Signed&nbsp;</td>");
		else {
			html.append(
				"<td align='right'><input itemtype='"
					+ item.getTypeID()
					+ "' type='checkbox' id='ck_"
					+ itemID
					+ "' onChange=\"acknowledge('ck_"
					+ itemID
					+ "', '"
					+ item.getTypeID()
					+ "')\""
					+ (item.isComplete() ? " checked" : "")
					+ " label='Add to Sign List'>&nbsp;</td>");
		}
		html.append("</tr></table>");
	}

}
