/*
 * Created on Apr 17, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard;

import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.items.IDashItemChangeListener;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.itemtype.DashboardTaskItem;
import gov.va.med.hds.signature.ISignatureStatusChangeListener;
import gov.va.med.hds.signature.SignatureStatusChangeEvent;
import gov.va.med.hds.task.ITaskChangeListener;
import gov.va.med.hds.task.TaskChangeEvent;
import gov.va.med.hds.task.data.ITask;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardItemEventSource
//	extends Hashtable
implements
	IDashboardItemEventSource,
	ITaskChangeListener,
	ISignatureStatusChangeListener {
	private IRuntimeContext runtime;
	private EventListenerList listenerList = new EventListenerList();

	public void addDashItemChangeListener(IDashItemChangeListener l) {
		listenerList.add(IDashItemChangeListener.class, l);
	}

	public void removeDashItemChangeListener(IDashItemChangeListener l) {
		listenerList.remove(IDashItemChangeListener.class, l);
	}

	public synchronized void fireDashItemChange(
		IDashboardItem item,
		int event) {
		final DashItemChangeEvent e = new DashItemChangeEvent(item, event);
		IDashItemChangeListener[] listeners =
			(IDashItemChangeListener[]) listenerList.getListeners(
				IDashItemChangeListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].itemUpdated(e);
		}
	}

	private void fireDashItemChange(
		Object item,
		String itemType,
		int event) {
		final DashItemChangeEvent e =
			new DashItemChangeEvent(item, itemType, event);
		IDashItemChangeListener[] listeners =
			(IDashItemChangeListener[]) listenerList.getListeners(
				IDashItemChangeListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].itemUpdated(e);
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.task.ui.ITaskChangeListener#taskAdded(gov.va.med.hds.task.data.ITask)
	 */
	public void taskAdded(TaskChangeEvent e) {
		ITask newTask = e.getTask();
		fireDashItemChange(
			newTask,
			DashboardTaskItem.ITEM_TYPE_ID,
			DashItemChangeEvent.ITEM_ADDED);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.task.ui.ITaskChangeListener#taskChanged(gov.va.med.hds.task.data.ITask)
	 */
	public void taskChanged(TaskChangeEvent e) {
		ITask changedTask = e.getTask();
		fireDashItemChange(
			changedTask,
			DashboardTaskItem.ITEM_TYPE_ID,
			DashItemChangeEvent.ITEM_MODIFIED);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.task.ui.ITaskChangeListener#taskDeleted(gov.va.med.hds.task.data.ITask)
	 */
	public void taskDeleted(TaskChangeEvent e) {
		ITask deletedTask = e.getTask();
		fireDashItemChange(
			deletedTask,
			DashboardTaskItem.ITEM_TYPE_ID,
			DashItemChangeEvent.ITEM_DELETED);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignatureStatusChange#signatureStatusChanged(gov.va.med.hds.signature.SignatureEvent)
	 */
	public void signatureStatusChanged(SignatureStatusChangeEvent e) {
		if (e.getSource() instanceof IDashboardItem) {
			IDashboardItem item = (IDashboardItem) e.getSource();
			switch (e.getEventType()) {
				case ISignatureStatusChangeListener.SIGNATURE_SIGNED :
					fireDashItemChange(item, DashItemChangeEvent.ITEM_SIGNED);
					break;

				case ISignatureStatusChangeListener.SIGNATURE_SIGNABLE :
					fireDashItemChange(item, DashItemChangeEvent.ITEM_COMPLETED);
					break;

				case ISignatureStatusChangeListener.SIGNATURE_NOT_SIGNABLE :
					fireDashItemChange(item, DashItemChangeEvent.ITEM_MODIFIED);
					break;

				default :
					break;
			}
		}
	}

}
