package gov.va.med.hds.dashboard;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;


/**
 * @author vhaislschras
 * 
 * This class wraps the DashPatientList class to allow easier 
 * conversion from the 5 column view of the DashPatientList
 * to the 3 column view of the Clinician Dashboard Patient List perspective.
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public class DashboardTableModelWrapper extends AbstractTableModel
{
  private int numWrapperCols;
  private TableModel      model;

  /**
   * Constructor for DashboardTableModelWrapper
   * 
   * @param model the TableModel this is wrapping.
   */
  public DashboardTableModelWrapper(TableModel model, int wrapperColumns)
  {
    this.model = model;
    numWrapperCols = wrapperColumns;
    model.addTableModelListener(new ListenerAdapter());
  }

  /**
   * @see javax.swing.table.TableModel#getRowCount()
   */
  public int getRowCount()
  {
    return ((model.getRowCount() + numWrapperCols) - 1) / numWrapperCols;
  }

  /**
   * @see javax.swing.table.TableModel#getColumnCount()
   */
  public int getColumnCount()
  {
    return model.getColumnCount() * numWrapperCols;
  }

  /**
   * @see javax.swing.table.TableModel#getColumnName(int)
   */
  public String getColumnName(int columnIndex)
  {
    return model.getColumnName(columnIndex % model.getColumnCount());
  }

  /**
   * @see javax.swing.table.TableModel#getColumnClass(int)
   */
  public Class getColumnClass(int columnIndex)
  {
    return model.getColumnClass(columnIndex % model.getColumnCount());
  }

  /**
   * @see javax.swing.table.TableModel#isCellEditable(int, int)
   */
  public boolean isCellEditable(int rowIndex, int columnIndex)
  {
    return false;
  }

  /**
   * @see javax.swing.table.TableModel#getValueAt(int, int)
   */
  public Object getValueAt(int rowIndex, int columnIndex)
  {
    int numCols = model.getColumnCount();
    int multiCol = columnIndex / numCols;
    int realColumn = columnIndex % numCols;
    int numRows = getRowCount();
    int realRow = rowIndex + (numRows * multiCol);

    if (realRow >= model.getRowCount())
    {
      return null;
    }
    else
    {
      return model.getValueAt(realRow, realColumn);
    }
    
  }

  /**
   * @see javax.swing.table.TableModel#setValueAt(Object, int, int)
   */
  public void setValueAt(Object aValue, int rowIndex, int columnIndex)
  {
    // do nothing
  }
  
  public TableModel getThis()
  {
    return this;
  }
  
  private class ListenerAdapter implements TableModelListener
  {
    public void tableChanged(TableModelEvent e)
    {
      int firstRow = e.getFirstRow();
      int lastRow = e.getLastRow();
      int col = e.getColumn();
      if (e.getSource().equals(model))
      {
        if (TableModelEvent.HEADER_ROW != firstRow &&
            Integer.MAX_VALUE != lastRow)
        {
          lastRow = e.getFirstRow() % getRowCount();
          firstRow = e.getFirstRow() % getRowCount();
        }
        if (TableModelEvent.ALL_COLUMNS != e.getColumn())
        {
          int multiCol = e.getFirstRow() / getRowCount();
          col = (multiCol * model.getColumnCount()) + col;
        }
      }
      fireTableChanged(new TableModelEvent(getThis(), firstRow, lastRow, col, e.getType()));
    }
  }
  
}
