/*
 * Created on Apr 15, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import java.awt.Cursor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JRootPane;
import javax.swing.SwingUtilities;

import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.dashboard.IDashboardCommonCoreConstants;
import gov.va.med.hds.dashboard.IDashboardItemEventSource;
import gov.va.med.hds.dashboard.ResolveUserId;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public abstract class AbstractDashboardDAO implements IDashboardItemDAO {
	protected IRuntimeContext runtime;
	protected IMumps mumps = null;
	protected ILogger logger = null;
	protected JRootPane rootPane;
	protected IDashboardItemTypes itemTypes = null;

	protected Map dbItems = Collections.synchronizedMap(new Hashtable());

	protected String dbItemsByID = null;
	protected String dbItemsByPatient = null;
	protected String updateByID = null;

	protected String provider;

	private IDashboardItemEventSource itemEventSource;

	public AbstractDashboardDAO() {
		dbItems.clear();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(boolean)
	 */
	public List getItems(boolean onlyDirty) {
		List items = new ArrayList();
		synchronized (dbItems) {
			for (Iterator iter = dbItems.values().iterator(); iter.hasNext();) {
				IDashboardItem element = (IDashboardItem) iter.next();
				if (((onlyDirty && element.isDirty()) || !onlyDirty))
					items.add(element);
			}
		}
		Collections.sort(items, IDashboardItemDAO.descendingDate);
		return items;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(java.util.List)
	 */
	public List getItems(List ids) {
		List items = new ArrayList();
		List param = new ArrayList();
		for (Iterator iter = ids.iterator(); iter.hasNext();) {
			String id = (String) iter.next();
			IDashboardItem item = (IDashboardItem) dbItems.get(id);
			if (null == item || null == item.getData())
				param.add(id);
			else
				items.add(item);
		}
		if (0 != param.size()) {
			try {
				List params = new ArrayList();
				params.add(param);
				items.addAll(getDashboardItems(dbItemsByID, params, -1));
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}
		if (0 != items.size())
			Collections.sort(items, IDashboardItemDAO.descendingDate);
		return items;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItem(java.lang.Object)
	 */
	public Object getItem(Object id) {
		List singleton = getItems(Collections.singletonList(id));
		return (0 < singleton.size() ? singleton.get(0) : null);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(java.lang.Object)
	 */
	public List getItems(Object patient) {
		List items = new ArrayList();
		synchronized (dbItems) {
			for (Iterator iter = dbItems.values().iterator(); iter.hasNext();) {
				IDashboardItem item = (IDashboardItem) iter.next();
				if (patient.equals(item.getPatient()))
					items.add(item);
			}
		}
		if (0 != items.size())
			Collections.sort(items, IDashboardItemDAO.descendingDate);
		return items;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
	 */
	abstract public List parseItems(Object data);

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#putItems(java.util.List)
	 */

  protected void parseOrder(IDashboardItem item, String data)
  {
    if (null != item)
      item.setTitle(data);
  }
  
	public void putItems(List items) {
		for (Iterator iter = items.iterator(); iter.hasNext();) {
			IDashboardItem item = (IDashboardItem) iter.next();
			IDashboardItem current = (IDashboardItem) dbItems.get(item.getID());
			if (null == current || !current.isDirty())
				dbItems.put(item.getID(), item);
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#putItem(java.lang.Object)
	 */
	public void putItem(Object item) {
		putItems(Collections.singletonList(item));
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#size()
	 */
	public int size() {
		return dbItems.size();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems(boolean)
	 */
	public void updateItems(boolean flush) {
		List items = getItems(false);
		if (0 < items.size()) {
			List param = new ArrayList();
			for (Iterator iter = items.iterator(); iter.hasNext();) {
				IDashboardItem item = (IDashboardItem) iter.next();
        if (item.isDirty())
				  param.add(item.getID());
			}
			updateItems(param, flush);
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems(java.util.List, boolean)
	 */
	public void updateItems(List ids, boolean flush) {
    Cursor prevCursor = showCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    try {
  		if (0 < ids.size()) {
  			String reply;
  			List params = new ArrayList();
  			params.add(provider);
  			params.add(ids);
				reply = mumps.callRPC(updateByID, params).trim();
				if (!flush) {
					ids.clear();
					String[] itemIds = reply.split("\n");
					for (int i = 0; i < itemIds.length; i++)
						ids.add(itemIds[i].split("\\^")[0]);
					updateCache(ids);
				}
      }
      if (flush)
        dbItems.clear();
    } catch (Exception e) {
      logger.error(e.getLocalizedMessage(), e);
    }
    showCursor(prevCursor);
	}

	protected void updateCache(List ids) {
		for (Iterator iter = ids.iterator(); iter.hasNext();)
			 ((IDashboardItem) getItem(iter.next())).setDirty(false);
	}

	protected List getDashboardItems(String RPCName, List param, int timeLimit)
		throws Exception {
		String reply =
			(-1 != timeLimit
				? mumps.callRPC(RPCName, param, timeLimit)
				: mumps.callRPC(RPCName, param))
				.trim();
		return parseItems(reply);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
		ISecurity security =
			(ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		provider = ResolveUserId.resolveUser(security.getSubject());
		this.logger = runtime.getLogger(this.getClass());

		this.itemTypes =
			(IDashboardItemTypes) runtime.getPluginInstance(
				IDashboardItemTypes.PLUGIN_ID);
		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor(
				IDashboardCommonCoreConstants.PLUGIN_ID);
		mumps =
			(
				(IMumpsFactory) runtime.getPluginInstance(
					IMumpsFactory.PLUGIN_ID)).create(
				desc);

		IWorkspace workspace =
			(IWorkspace) runtime.getPluginInstance(IWorkspace.PLUGIN_ID);
		this.itemEventSource =
			(IDashboardItemEventSource) workspace
				.getScope(IWorkspaceConstants.SESSION_SCOPE)
				.getModelSpace()
				.getModel(IDashboardItemEventSource.MODEL_ID);
	}

	protected List getIdsFromItems(List items) {
		List ids = new ArrayList(items.size());
		for (Iterator iter = items.iterator(); iter.hasNext();) {
			IDashboardItem item = (IDashboardItem) iter.next();
			ids.add(item.getID());
		}
		return ids;
	}

	protected void removeFromCache(String id) {
		removeFromCache(Collections.singletonList(id));
	}

	protected void removeFromCache(List ids) {
		class doDeletion implements Runnable {
			List ids;
			public doDeletion(List ids) {
				this.ids = ids;
			}
			public void run() {
				for (Iterator iter = ids.iterator(); iter.hasNext();) {
					String id = (String) iter.next();
					IDashboardItem item = (IDashboardItem) dbItems.get(id);
					if (null != item) {
						dbItems.remove(id);
						itemEventSource.fireDashItemChange(
							item,
							DashItemChangeEvent.ITEM_DELETED);
					}
				}
			}
		}
		SwingUtilities.invokeLater(new doDeletion(ids));
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#deleteItem(java.lang.Object)
	 */
	public void deleteItem(Object item) {
		removeFromCache(((IDashboardItem) item).getID());
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#deleteItems(java.util.List)
	 */
	public void deleteItems(List items) {
		List ids = new ArrayList();
		for (Iterator iter = items.iterator(); iter.hasNext();) {
			IDashboardItem item = (IDashboardItem) iter.next();
			ids.add(item.getID());
		}
		removeFromCache(ids);
	}

  private Cursor showCursor(Cursor c) {
    IDesktopWindow activeWindow =
      PrismUtilities.getDesktop(runtime).getActiveWindow();
    if (null != activeWindow) {
      JRootPane root = activeWindow.getFrame().getRootPane();
      if (root != null) {
          Cursor previousCursor = root.getCursor();
          root.setCursor(c);
          return previousCursor;
      }
    }
    return null;
  }

}
