package gov.va.med.hds.dashboard.dao.m;

import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.dashboard.IDashboardCommonCoreConstants;
import gov.va.med.hds.dashboard.PatientParseException;
import gov.va.med.hds.dashboard.ResolveUserId;
import gov.va.med.hds.dashboard.SensitivePatient;
import gov.va.med.hds.dashboard.dao.IDashboardNotificationsDAO;
import gov.va.med.hds.dashboard.dao.IPatientListDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardNotificationItem;

import java.io.BufferedReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Generic patient list DAO for the dashboards.
 */
public class AbstractPatientDAO implements IPatientListDAO {
	protected static final String DEFAULT_PATIENT_LIST_SOURCE_RPC_KEY =
		"DFLTSRC";
	protected static final String[] PATIENT_LIST_RPC =
		new String[PATIENT_LIST_MAX];
	protected static final String[] PATIENT_LIST_SOURCES =
		new String[PATIENT_LIST_MAX];
	protected static final char[] PATIENT_LIST_TYPE =
		new char[PATIENT_LIST_MAX];

	protected static final String PATIENT_DEMO_RPC_KEY = "PTINQ";
	protected static final String DASHBOARD_PATIENTLIST_DEFAULT_RPC_KEY =
		"DASHBOARD PATIENTS";
	protected static final String LOG_SENSITIVE_RECORD_RPC_KEY =
		"SENSITIVE RECORD BULLETIN";
	protected static final String SENSITIVE_RECORD_RPC_KEY =
		"SENSITIVE RECORD ACCESS";

	protected IRuntimeContext runtime;
	protected IMumps mumps = null;
	protected String provider;
	protected ILogger logger;
	protected IItemEntry patientItem;
	protected Map itemTypes = Collections.synchronizedMap(new Hashtable());
	protected String dashboardPatientList;
	protected String dashboardType;
	protected String dashboardOptionName = null;
	protected List errorList = new ArrayList();

	private IDashboardNotificationsDAO notifyDAO = null;

	static {
		PATIENT_LIST_TYPE[PATIENT_LIST_CLINIC] = 'c';
		PATIENT_LIST_TYPE[PATIENT_LIST_PROVIDER] = 'p';
		PATIENT_LIST_TYPE[PATIENT_LIST_REGISTRY] = 'r';
		PATIENT_LIST_TYPE[PATIENT_LIST_SPECIALITY] = 's';
		PATIENT_LIST_TYPE[PATIENT_LIST_TEAM] = 't';
		PATIENT_LIST_TYPE[PATIENT_LIST_WARD] = 'w';

		PATIENT_LIST_RPC[PATIENT_LIST_DEFAULT] = "DEFAULT PATIENT LIST";
		PATIENT_LIST_RPC[PATIENT_LIST_PROVIDER] = "PROVIDER PATIENTS";
		PATIENT_LIST_RPC[PATIENT_LIST_TEAM] = "TEAM PATIENTS";
		PATIENT_LIST_RPC[PATIENT_LIST_SPECIALITY] = "SPECIALTY PATIENTS";
		PATIENT_LIST_RPC[PATIENT_LIST_CLINIC] = "CLINIC PATIENTS";
		PATIENT_LIST_RPC[PATIENT_LIST_WARD] = "BYWARD";

		PATIENT_LIST_SOURCES[PATIENT_LIST_PROVIDER] = "Provider";
		PATIENT_LIST_SOURCES[PATIENT_LIST_TEAM] = "Team";
		PATIENT_LIST_SOURCES[PATIENT_LIST_SPECIALITY] = "Speciality";
		PATIENT_LIST_SOURCES[PATIENT_LIST_CLINIC] = "Clinic";
		PATIENT_LIST_SOURCES[PATIENT_LIST_WARD] = "Ward";
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
		ISecurity security =
			(ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		setUser(security.getSubject());
		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor(
				IDashboardCommonCoreConstants.PLUGIN_ID);
		mumps =
			(
				(IMumpsFactory) runtime.getPluginInstance(
					IMumpsFactory.PLUGIN_ID)).create(
				desc);
		this.logger = runtime.getLogger(this.getClass());
	}

	private void setUser(Object user) {
		this.provider = ResolveUserId.resolveUser(user);
	}

	public String getDefaultPatientListSource() {
		String defaultSource = "";
		try {
			defaultSource = mumps.callRPC(DEFAULT_PATIENT_LIST_SOURCE_RPC_KEY);
		} catch (Exception cqe) {
			logger.error(cqe.getMessage(), cqe);
		}
		return defaultSource.trim();
	}

	protected List getPatientList(Object user, char type, List param) throws PatientParseException {
		String patients = "";
		String RPCName = PATIENT_LIST_RPC[PATIENT_LIST_DEFAULT];
		for (int i = 0; i < PATIENT_LIST_TYPE.length; i++)
			if (PATIENT_LIST_TYPE[i] == type) {
				RPCName = PATIENT_LIST_RPC[i];
				break;
			}
		try {
			patients = mumps.callRPC(RPCName, param);
		} catch (Exception cqe) {
			logger.error(cqe.getMessage(), cqe);
		}

		return parsePatientList(patients);
	}

	public void getDashboardPatientList(
		Object user,
		List sourceIds,
		IMumpsListener listener) {
		String RPCName = dashboardPatientList;
		List param = new ArrayList();
		param.add(0, null != user ? ResolveUserId.resolveUser(user) : provider);
		if (null != sourceIds && 0 < sourceIds.size()) {
			param.add(1, dashboardType);
			param.add(2, sourceIds);
			RPCName = DASHBOARD_PATIENTLIST_DEFAULT_RPC_KEY;
		}
		//		String patients = "";
		try {
			mumps.callRPC(RPCName, param, listener);
		} catch (Exception cqe) {
			logger.error(cqe.getMessage(), cqe);
		}
	}

	private List getDashboardPatientList(Object user, List sourceIds) throws PatientParseException {
		String RPCName = dashboardPatientList;
		List param = new ArrayList();
		param.add(0, null != user ? ResolveUserId.resolveUser(user) : provider);
		if (null != sourceIds && 0 < sourceIds.size()) {
			param.add(1, dashboardType);
			param.add(2, sourceIds);
			RPCName = DASHBOARD_PATIENTLIST_DEFAULT_RPC_KEY;
		}
		String patients = "";
		try {
			patients = mumps.callRPC(RPCName, param);
		} catch (Exception cqe) {
			logger.error(cqe.getMessage(), cqe);
		}
		return parsePatientList(patients);
	}

	public Object getPatientDemographics(Object patientId) {
		String demographics = "";
		if (null != patientId) {
			List params = new ArrayList(1);
			params.add(patientId);
			try {
				demographics = mumps.callRPC(PATIENT_DEMO_RPC_KEY, params);
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}
		return demographics;
	}

	public List parsePatientList(Object rawData) throws PatientParseException {
		/*
		 * The patient list is returned as:
		 * 
		 * dfn^name
		 * 
		 */
		List patientList = new ArrayList();

		String rawMData;
		if (0 != (rawMData = ((String) rawData).trim()).length()) {
			IDashboardPatient patient = null;
			BufferedReader reader = new BufferedReader(new StringReader(rawMData));
			String readLine = null;
			errorList.clear();
			try {
				while ((readLine=reader.readLine())!=null) {
					String[] line = readLine.split("=", 2);
					if (line[0].equalsIgnoreCase("Error")) {
						errorList.add(readLine.split("\\^")[1]);
					} else if (line[0].equalsIgnoreCase("Patient")) {
						if (null != patient)
							patientList.add(patient);
						patient = parsePatient(line[1]);
					} else if (null != patient)
						parseItem(patient, readLine);
				}
			}
			catch (Exception e) {
				throw new PatientParseException("Error reading patient data",e);
			}
			if (null != patient)
				patientList.add(patient);
			if (errorList!=null) {
				for (int i = 0; i < errorList.size(); i++) {
					logger.info(errorList.get(i).toString());
				}
			}
		}
		return patientList;
	}

	private void parseItem(IDashboardPatient patient, String data) {
		String[] datum = data.split("=", 2);
		String[] ids = datum[1].trim().split("\\^");
		IItemEntry itemEntry =
			(IItemEntry) itemTypes.get(datum[0].toLowerCase().trim());
		if (IDashboardNotificationsDAO
			.ITEM_TYPE_ID
			.equals(itemEntry.getItemType()))
			getNotifications(patient, datum[1], itemEntry);
		// Has no IDs - only a "0" or a "1"
		else {
			for (int i = 0; i < ids.length; i++) {
				IDashboardItem item = null;
				try {
					if (null
						!= (item = (IDashboardItem) itemEntry.getInstance())) {
						item.setPatient(patient);
						item.setID(ids[i]);
						patient.putItem(itemEntry.getID(), item);
					} else
						logger.error(
							"Unable to create type for " + itemEntry.getName());
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}
		}
	}

	private void getNotifications(
		IDashboardPatient patient,
		String hasNotifications,
		IItemEntry itemEntry) {
		if ("1".equals(hasNotifications)) {
			if (null == notifyDAO) {
				try {
					notifyDAO =
						(IDashboardNotificationsDAO) itemEntry
							.getItemDAOInstance();
				} catch (Exception e) {
					logger.error(e.getLocalizedMessage(), e);
					return;
				}
			}
			List notifications = notifyDAO.getNotifications(patient);
			if (null != notifications && 0 < notifications.size())
				for (Iterator iter = notifications.iterator();
					iter.hasNext();
					) {
					DashboardNotificationItem item =
						(DashboardNotificationItem) iter.next();
					item.setPatient(patient);
					item.setData("");
					patient.putItem(itemEntry.getID(), item);
				} else
				patient.putItem(itemEntry.getID(), null);
		}
	}

	private IDashboardPatient parsePatient(String data) {
		try {
			String[] pieces = data.trim().split("\\^");
			IDashboardPatient patient =
				(IDashboardPatient) patientItem.getInstance();
			patient.setPatient(
				pieces[0],
				pieces[1],
				pieces[2],
				pieces[3],
				pieces[4]);
			return patient;
		} catch (Exception e) {
			logger.error(
				"Exception creating new Dashboard Patient object for: " + data,
				e);
		}
		return null;
	}

	private List addUnique(List a, List b) {
		for (Iterator iter = b.iterator(); iter.hasNext();) {
			IDashboardPatient element = (IDashboardPatient) iter.next();
			if (!a.contains(element))
				a.add(element);
		}
		return a;
	}

	public boolean getSensitivePatient(Object patientId)
		throws SensitivePatient {
		List params = Collections.singletonList(patientId);

		String dgrep;
		try {
			dgrep = mumps.callRPC(SENSITIVE_RECORD_RPC_KEY, params).trim();
			if (0 < dgrep.length()) {
				String[] result = dgrep.split("\n", 2);
				int status = Integer.parseInt(result[0]);
				switch (status) {
					case -1 : // Required variable not defined
						throw new MumpsException("Required variable not defined.");

					case 0 : // Not a sensitive patient
						return false;

					default : // Sensitive patient, but user can view
						throw new SensitivePatient(
							result[1],
							Integer.parseInt(result[0]));
				}
			}
		} catch (MumpsException e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		return true;
	}


	public void logSensitivePatient(Object patientId) {
		List params;

		if (null == dashboardOptionName)
			params = Collections.singletonList(patientId);
		else {
			params = new ArrayList();
			params.add(0, patientId);
			params.add(1, "");
		}

		String result;
		try {
			result = mumps.callRPC(LOG_SENSITIVE_RECORD_RPC_KEY, params).trim();
			if (0 < result.length()) {
				if (!("1".equals(result)))
					throw new CallQueueException(
						"Unable to log sensitive patient for access of "
							+ patientId
							+ " by "
							+ provider
							+ ".");
			}
		} catch (Exception cqe) {
			logger.error(cqe.getMessage(), cqe);
		}
	}

	public List getErrorStrings() {
		List returnList = new ArrayList(errorList);
		errorList.clear();
		return returnList;
	}

}
