/*
 * Created on Jun 9, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.dao.IDashboardNotificationsDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardNotificationItem;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardNotificationsDAO extends AbstractDashboardDAO implements IDashboardNotificationsDAO
{
  protected static final String NOTIFICATIONS_BY_PATIENT_RPC_KEY = "NOTIFICATIONS BY PATIENT";

  private IItemEntry itemType;

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
	 */
	public List parseItems(Object data)
	{
    List events = new ArrayList();
    DashboardNotificationItem notification = null;
    String[] eventData = ((String) data).split("\n");
    for (int l = 0; l < eventData.length; l++)
    {
      String[] line = eventData[l].split("=", 2);
      if ("Item".equalsIgnoreCase(line[0]))
      events.add((notification = (DashboardNotificationItem)parseItem(line[1])));
      else if ("Cmnt".equalsIgnoreCase(line[0]))
      notification.addComment(line[1]);
    }
    return events;
  }

  private IDashboardItem parseItem(String data)
  {
    // The ordered item looks like:
    // ORD:ID^^YYYYMMDDHHmmss
    String[] notifications = data.split("\\^");
    IDashboardItem item = null;;
    try
    {
      item = (IDashboardItem)dbItems.get(notifications[0]);
      if (null == item)
      {
        item = (IDashboardItem)itemType.getInstance();
        item.setID(notifications[0]);
      }
      item.setDate((-1 != notifications[2].indexOf('.') ?
        (Object)StringUtil.fmDateToDate(notifications[2]) :
        notifications[2]));
      item.setShortTitle((0 != notifications[1].length() ?
        notifications[1] : 
        item.getFormattedDate()));
      item.setDirty(false);
      dbItems.put(item.getID(), item);
      return item;
    }
    catch (Exception e)
    {
      logger.error(e.getMessage(), e);
    }
    return item;
  }


	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data)
		throws Exception
	{
    super.setInitializationData(runtime, config, propertyName, null);
    itemType = itemTypes.getItemType(IDashboardNotificationsDAO.ITEM_TYPE_ID);
	}

  public List getNotifications(IDashboardPatient patient)
  {
    List notifications = new ArrayList();

    try
    {
      List params = new ArrayList(2);
      params.add(patient.getPatientId());
      params.add(provider);
      notifications = getDashboardItems(NOTIFICATIONS_BY_PATIENT_RPC_KEY, params, -1);
    }
    catch (Exception e)
    {
      logger.error(e.getLocalizedMessage(), e);
      JOptionPane.showMessageDialog(
        PrismUtilities.getDesktop(runtime).getActiveWindow().getFrame().getRootPane(),
        "*** Notification Read Failure ***\n" +
        "Unable to read notifications.  You must open the\n" +
        "patient chart to process notifications for patient\n" +
        patient.getPatientName() + ".",
        "Notification Warning",
        JOptionPane.WARNING_MESSAGE);
    }
    return notifications;
  }

}
