/*
 * Created on May 8, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.dashboard.IDashboardCommonCoreConstants;
import gov.va.med.hds.dashboard.ResolveUserId;
import gov.va.med.hds.dashboard.dao.IDashboardPermissionsDAO;

import java.util.Collections;
import java.util.List;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardPermissionsDAO implements IDashboardPermissionsDAO {
	private IRuntimeContext runtime;
	private ILogger logger;
	private IMumps mumps;
	private Object provider;

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception {
		this.runtime = runtime;
		ISecurity security = (ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		provider = ResolveUserId.resolveUser(security.getSubject());
		this.logger = runtime.getLogger(this.getClass());

		IPluginDescriptor desc = runtime.getPluginRegistry().getPluginDescriptor(IDashboardCommonCoreConstants.PLUGIN_ID);
		mumps = ((IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardPermissionsDAO#isValidKey(java.lang.String)
	 */
	public boolean isValidKey(String key) {
		String[] permission = key.split(":");
		// if the first ":"-piece of permission string equals "M", evaluate key ownership
		// if the first ":"-piece of permission string equals "A", return true
		// else return false
		if ("M".equals(permission[0])) {
			List params = Collections.singletonList(permission[1]);
			try {
				String answer = mumps.callRPC(IDashboardPermissionsDAO.VALIDATE_KEY, params);
				return ("1".equals(answer) ? true : false);
			} catch (Exception e) {
				logger.error(e.getLocalizedMessage(), e);
			}
		} else if ("A".equals(permission[0])) {
			return true;
		}
		return false;
	}

}
