/*
 * Created on Apr 12, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.dao.m;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardResultDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.itemtype.DashboardResultItem;
import gov.va.med.hds.dashboard.itemtype.IChemistryLabResult;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardResultDAO extends AbstractDashboardDAO implements IDashboardResultDAO
{
  protected static final String RESULTS_BY_PATIENT = "RESULTS BY PATIENT";
  protected static final String RESULTS_BY_ORDERID_RPC_KEY = "RESULTS BY ID";
  protected static final String RESULTS_ACK_BY_ID = "RESULTS ACKNOWLEDGE";

  public DashboardResultDAO()
  {
    super();
    dbItemsByID = RESULTS_BY_ORDERID_RPC_KEY;
    dbItemsByPatient = RESULTS_BY_PATIENT;
    updateByID = RESULTS_ACK_BY_ID;
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
   */
  public List parseItems(Object data)
  {
    ArrayList results = new ArrayList();
    DashboardResultItem result = null;
    String[] resultData = ((String) data).split("\n");
    for (int l = 0; l < resultData.length; l++)
    {
      String[] line = resultData[l].split("=", 2);
      if ("Item".equalsIgnoreCase(line[0]))
        results.add((result = (DashboardResultItem)parseItem(line[1])));
      else if ("Data".equalsIgnoreCase(line[0]))
        parseDataResult(result, line[1]);
      else if ("Text".equalsIgnoreCase(line[0]))
        parseTextResult(result, line[1]);
      else if ("Cmnt".equalsIgnoreCase(line[0]))
        result.addComment(line[1]);
    }
    return results;
  }

  private IDashboardItem parseItem(String data)
  {
    // The ordered item looks like:
    // ORD:orderID^order^YYYYMMDDHHmmss
    String[] orderInfo = data.trim().split("\\^");
    IDashboardItem item = null;;
    try
    {
      item = (IDashboardItem)dbItems.get(orderInfo[0]);
      if (null == item)
      {
        item = (IDashboardItem)itemTypes.getItemType(ITEM_TYPE_ID).getInstance();
        item.setID(orderInfo[0]);
      }
      item.setShortTitle(orderInfo[1]);
      item.setDate(orderInfo[2]);
      item.setDirty(false);
      dbItems.put(item.getID(), item);
      return item;
    }
    catch (Exception e)
    {
      logger.error(e.getMessage(), e);
    }
    return item;
  }

  private void parseTextResult(IDashboardItem item, String data)
  {
    // Text has a single line of text each row;
    StringBuffer text = (StringBuffer) item.getData();
    if (null == text)
      text = (StringBuffer)((DashboardResultItem)item)
        .createDataType(DashboardResultItem.TEXT_RESULT);
    else
      text.append("\n");
    text.append(data);
    item.setData(text);
  }

  private void parseDataResult(IDashboardItem item, String data)
  {
    Vector vData = (Vector) item.getData();
    if (null == vData) vData = new Vector();
    String[] pieces = data.trim().split("\\^");

    boolean isCritical = (5 == pieces.length && 0 != pieces[4].trim().length());
    IChemistryLabResult dataItem = (IChemistryLabResult)((DashboardResultItem)item)
      .createDataType(DashboardResultItem.DATA_RESULT);
    dataItem.setResult(pieces[0], // Result Name
      pieces[1], // Result Value
      (2 < pieces.length ? pieces[2] : ""), // Units
      (3 < pieces.length ? pieces[3] : ""), // Reference Range
     (isCritical ? pieces[4] : null)); // Critical Flag
    vData.add(dataItem);
    if (!item.isCritical())
      item.setCritical(isCritical);
    item.setData(vData);
  }

  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems(boolean)
	 */
	public void updateItems(boolean flush)
	{
    List items = getItems(false);
    if (0 < items.size())
    {
      List param = new ArrayList();
      for (Iterator iter = items.iterator(); iter.hasNext();)
      {
        IDashboardItem item = (IDashboardItem)iter.next();
        if (item.isDirty())
          param.add(item.getID() + (item.isComplete() ? "^1" : "^0"));
      }
      updateItems(param, flush);
    }
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(boolean)
	 */
	public List getItems(boolean onlyDirty)
	{
		return sortCritical(super.getItems(onlyDirty));
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(java.util.List)
	 */
	public List getItems(List ids)
	{
		return sortCritical(super.getItems(ids));
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(java.lang.Object)
	 */
	public List getItems(Object patient)
	{
		return sortCritical(super.getItems(patient));
	}
  
  private List sortCritical(List items)
  {
    Collections.sort(items, descendingPriority);
    return items;
  }

  private final Comparator descendingPriority = new Comparator(){
    public int compare(Object obj1, Object obj2)
    {
      IDashboardItem item1 = (IDashboardItem)obj1;
      IDashboardItem item2 = (IDashboardItem)obj2;

      // If either one is null, just pretend their equal
      if (null == item1 || null == item2)
        return 0;
      if ((item1.isCritical() && item2.isCritical()) ||
          (!item1.isCritical() && !item2.isCritical()))
        return IDashboardItemDAO.descendingDate.compare(obj1, obj2);
      if (item1.isCritical())
        return -1;
      return 1;
    }
  };
}
