/*
 * Created on Apr 14, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardTaskDAO;
import gov.va.med.hds.dashboard.itemtype.DashboardTaskItem;
import gov.va.med.hds.task.data.ITask;
import gov.va.med.hds.task.data.ITaskDAO;
import gov.va.med.hds.task.data.TaskDataAccessException;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardTaskDAO extends AbstractDashboardDAO implements IDashboardTaskDAO
{
  private ITaskDAO taskDAO;
  
  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#getItems(java.util.List)
   */
  public List getItems(List ids)
  {
    List items = new ArrayList();
    List taskItems = null;
    try
    {
      taskItems = taskDAO.getTasks(ids);
    }
    catch (TaskDataAccessException tae)
    {
      logger.error(tae.getMessage(), tae);
    }
    if (null != taskItems)
    {
      for (Iterator iter = taskItems.iterator(); iter.hasNext();)
      {
        ITask task = (ITask)iter.next();
        DashboardTaskItem dbItem = (DashboardTaskItem)dbItems.get(prefix + task.getID());
        if (dbItem != null) {
	      dbItem.setData(task);
	      items.add(dbItem);
        }
      }
    }
    Collections.sort(items, IDashboardItemDAO.descendingDate);
    return items;
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
	 */
	public List parseItems(Object rawData) {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems()
	 */
	public void updateItems(boolean flush) {
		List dirtyTasks = getItems(true);
		List ids = new ArrayList();
		for (Iterator iter = dirtyTasks.iterator(); iter.hasNext();)
			ids.add(((DashboardTaskItem) iter.next()).getID());
		updateItems(ids, flush);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems(java.util.List)
	 */
	public void updateItems(List ids, boolean flush) {
		if (flush)
			dbItems.clear();
		else
			for (Iterator iter = ids.iterator(); iter.hasNext();) {
				DashboardTaskItem taskItem = (DashboardTaskItem) dbItems.get(iter.next());
				taskItem.setDirty(false);
			}
	}

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
   */
  public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception
  {
    super.setInitializationData(runtime, config, propertyName, data);
    
    IDaoRegistry daoRegistry = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
    taskDAO = (ITaskDAO)daoRegistry.getDAO(ITaskDAO.class.getName());
    if (null == taskDAO)
      throw new Exception("Unable to open ITaskDAO."); 
  }

}
