/*
 * Created on Apr 11, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.itemtype;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import javax.swing.SwingUtilities;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.dashboard.IDashboardItemEventSource;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public abstract class AbstractDashboardItem implements IDashboardItem {
	protected SimpleDateFormat dateInputFormat =
		new SimpleDateFormat(IDashboardItem.INPUT_DATE_FORMAT);
	protected SimpleDateFormat dateOutputFormat =
		new SimpleDateFormat(IDashboardItem.OUTPUT_DATE_FORMAT);

	protected String itemID = null;
	protected String title = null;
  protected String shortTitle = null;
	protected boolean complete = false;
	protected boolean viewed = false;
	protected boolean dirty = false;
	protected boolean critical = false;
	protected Date date = null;
	protected String status = "unknown";

	protected Object data = null;
	protected List comments = new ArrayList();

	protected String itemTypeID = "";
	protected String dashboardModuleName;
	protected IDashboardItemTypes itemTypes;
	protected IItemEntry itemEntry = null;
	protected IDashboardPatient patient = null;

	protected IDashboardItemEventSource itemEventSource;

	protected IRuntimeContext runtime;

	public void initDashboardItem(String itemID, Object patientID) {
		setID(itemID);
		setPatient(patientID);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#isComplete()
	 */
	public boolean isComplete() {
		return complete;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#isViewed()
	 */
	public boolean isViewed() {
		return viewed;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#setComplete(boolean)
	 */
	public void setComplete(boolean complete) {
		this.complete = complete;
		setDirty(true);
		fireItemChanged(DashItemChangeEvent.ITEM_COMPLETED);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#setViewed(boolean)
	 */
	public void setViewed(boolean viewed) {
		this.viewed = viewed;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#getDate()
	 */
	public Date getDate() {
		return date;
	}

	public String getFormattedDate() {
		return (null != date ? dateOutputFormat.format(date) : "");
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardItem#setDate(java.lang.Object)
     * 
     * M's time zone handling logic appears severely flawed - the only way
     * we can parse timestamps reliably is to ignore the timezone data that
     * M provides.  This altered logic is implemented in the setDate() routine
     * below; this routine is disabled for now.
     * 
     * This problem needs to be revisited in the future for a more complete
     * solution. 
	 */
	private void old_setDate(Object itemDate) {
		if (itemDate instanceof Date)
			this.date = (Date) itemDate;
		else if (itemDate instanceof String) {
			String date = (String) itemDate;
			// Date/times come in varying strengths.  The preferred input
			// is as described above in the javadocs.  However, the timezone
			// suffix may be missing or the time could be truncated, so we
			// need to be prepared to handle these cases.

			// The first stop is to figure how much of the date time is present.
			// Split the date/time from the timezone.
			if (null != date && 0 != date.length()) {
				boolean hasTimeZone = true;
				String[] dateTime = date.split("[-+]");

				// The sign must be reversed for this timezone identifier.
				String tzDirection =
					(-1 != date.indexOf('-')
						? "+"
						: (-1 != date.indexOf('+') ? "-" : ""));

				// If a timezone is specified create it
				if ((hasTimeZone = (0 != tzDirection.length()))) {
					// Create a timezone identifier from the timezone offset and create
					// a timezone object that works for it.  All we have is the hours from GMT
					// so daylight savings time isn't accounted for.  This just makes the 
					// the date print correctly. 
					String id =
						"Etc/GMT"
							+ tzDirection
							+ Integer.parseInt(dateTime[1].substring(0, 2));
					TimeZone tz = TimeZone.getTimeZone(id);
					dateInputFormat.setTimeZone(tz);
					dateOutputFormat.setTimeZone(tz);
				}

				// If the date doesn't match the input format we need to adjust it.
				// The +4 at the end of the account for the difference in the timezone
				// specification of the input date and the format string.  The date
				// represents the timezone as [+-]HHmm and the format string represents it
				// as a "Z".
				if (date.length()
					< IDashboardItem.INPUT_DATE_FORMAT.length() + 4) {
					// The length of the date should be a multiple of 2
					int dateInputLength = (dateTime[0].length() / 2) * 2;
					String dateFormatInput =
						IDashboardItem.INPUT_DATE_FORMAT.substring(
							0,
							dateInputLength)
							+ (hasTimeZone ? "Z" : "");

					String dateFormatOutput =
						(5 > dateInputLength
							? "yyyy"
							: (7 > dateInputLength
								? "MMM, yyyy"
								: (9 > dateInputLength
									? IDashboardItem
										.OUTPUT_DATE_FORMAT
										.substring(
										0,
										dateInputLength
											+ (dateInputLength / 2))
									: IDashboardItem.OUTPUT_DATE_FORMAT)));

					// Create the time format object and set the timezone for it.
					dateInputFormat = new SimpleDateFormat(dateFormatInput);
					dateOutputFormat = new SimpleDateFormat(dateFormatOutput);
				}
				try {
					this.date = dateInputFormat.parse(date);
				} catch (ParseException pe) {
					this.date = null;
				}
			}
		}
	}

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.IDashboardItem#setDate(java.lang.Object)
     */
    public void setDate(Object itemDate) {
        if (itemDate instanceof Date)
            this.date = (Date) itemDate;
        else if (itemDate instanceof String) {
            String date = (String) itemDate;

            // Date/times come in varying strengths.  The preferred input
            // is as described above in the javadocs.
            
            if (date == null || date.length() == 0)
                this.date = null;
            else {
                // Split the date/time from the timezone.
                String[] dateTime = date.split("[-+]");
                // discard the time zone
                date = dateTime[0];

                // The first stop is to figure how much of the date time is present.

                // If the date probably won't match the input format so we'll need to adjust it.
                // The length of the date should be a multiple of 2
                int dateInputLength = (date.length() / 2) * 2;
                String dateFormatInput =
                    IDashboardItem.INPUT_DATE_FORMAT.substring(
                        0,
                        dateInputLength);

                String dateFormatOutput =
                    (5 > dateInputLength
                        ? "yyyy"
                        : (7 > dateInputLength
                            ? "MMM, yyyy"
                            : (9 > dateInputLength
                                ? IDashboardItem
                                    .OUTPUT_DATE_FORMAT
                                    .substring(
                                    0,
                                    dateInputLength
                                        + (dateInputLength / 2))
                                : IDashboardItem.OUTPUT_DATE_FORMAT)));

                // Create the time format object.
                dateInputFormat = new SimpleDateFormat(dateFormatInput);
                dateOutputFormat = new SimpleDateFormat(dateFormatOutput);

                try {
                    this.date = dateInputFormat.parse(date);
                } catch (ParseException pe) {
                    this.date = null;
                }
            }
        }
    }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#setDirty(boolean)
	 */
	public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#isCritical()
	 */
	public boolean isCritical() {
		return critical;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#setCritical(boolean)
	 */
	public void setCritical(boolean critical) {
		this.critical = critical;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#getLinkURL()
	 */
	public String getLinkURL() {
		StringBuffer result = new StringBuffer();
		result.append(
			"cdtp:///gov/va/med/hds/task/Link?linkedItemType="
				+ itemEntry.getItemShortId());
		try {
			result.append("&linkedItemID=").append(
				URLEncoder.encode(itemID, "UTF-8"));
			result.append("&patientID=").append(
				URLEncoder.encode(
					((IDashboardPatient) patient).getPatientId(),
					"UTF-8"));
		} catch (UnsupportedEncodingException e) {
		}
		return result.toString();
	}

	public void setID(String itemID) {
		if (-1 != itemID.indexOf('*'))
			setCritical(true);
		if (-1 != itemID.indexOf('!'))
			setComplete(true);
		this.itemID = itemID.replaceAll("[*!]", "");
	}

	public void setPatient(Object patient) {
		this.patient = (IDashboardPatient) patient;
	}

	public String getID() {
		return itemID;
	}

	/**
	 * @return String
	 */
	public String getTypeID() {
		return itemTypeID;
	}

	/**
	 * @return Object
	 */
	public Object getPatient() {
		return patient;
	}

	/**
	 * @return Object
	 */
	public Object getData() {
		return data;
	}

	/**
	 * Sets the data.
	 * @param data The data to set
	 */
	public void setData(Object data) {
		this.data = data;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#addComment(java.lang.Object)
	 */
	public void addComment(Object comment) {
		this.comments.add(comment);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#getComments()
	 */
	public List getComments() {
		return comments;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#getTitle()
	 */
	public String getTitle() {
		return (null == title ? shortTitle : title);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#setTitle(java.lang.String)
	 */
	public void setTitle(String title) {
		this.title = (null == this.title ? title : this.title.concat("\n" + title));
	}

	public void fireItemChanged(int eventType) {
		class itemChanged implements Runnable {
			int eventType;
			IDashboardItem item;
			public itemChanged(IDashboardItem item, int eventType) {
				this.eventType = eventType;
				this.item = item;
			}

			public void run() {
				itemEventSource.fireDashItemChange(item, eventType);
			}
		}
		SwingUtilities.invokeLater(new itemChanged(this, eventType));
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
		this.itemTypes =
			(IDashboardItemTypes) runtime.getPluginInstance(
				IDashboardItemTypes.PLUGIN_ID);
		this.itemEntry = itemTypes.getItemType(getTypeID());

		IWorkspace workspace =
			(IWorkspace) runtime.getPluginInstance(IWorkspace.PLUGIN_ID);
		this.itemEventSource =
			(IDashboardItemEventSource) workspace.getScope(IWorkspaceConstants.SESSION_SCOPE).getModelSpace().getModel(IDashboardItemEventSource.MODEL_ID);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardItem#getStatus()
	 */
	public String getStatus() {
		return status;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardItem#setStatus(java.lang.String)
	 */
	public void setStatus(String status) {
		this.status = status;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardItem#getShortTitle()
	 */
	public String getShortTitle()
	{
    return shortTitle;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardItem#setShortTitle(java.lang.String)
	 */
	public void setShortTitle(String shortTitle)
	{
		this.shortTitle = shortTitle;
	}

}
