/*
 * Created on Apr 26, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.itemtype;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.dashboard.DashboardIconView;
import gov.va.med.hds.dashboard.IDashboardPlugin;
import gov.va.med.hds.dashboard.SensitivePatient;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IPatientListDAO;
import gov.va.med.hds.dashboard.items.IDashboardDefinition;
import gov.va.med.hds.dashboard.items.IDashboardEntry;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class AbstractDashboardPatient
	extends Hashtable
	implements IDashboardPatient {
	/**
	 * Used to read dates from the input source.
	 */
	protected SimpleDateFormat dateInputFormat =
		new SimpleDateFormat(DATE_INPUT_FORMAT);
	/**
	 * Used to format the date for presentation.
	 */
	protected SimpleDateFormat dateOutputFormat =
		new SimpleDateFormat(DATE_OUTPUT_FORMAT);

	protected String patientURL;
	protected IItemEntry itemEntry;
	protected String dashboardModuleName;
	protected IDashboardPlugin plugin;

	protected String patientName;
	protected String patientId;
	protected String ssn = "";
	protected String dob;
	protected int age = -1;

	protected boolean sensitive = false;
	protected boolean allowSensitive = false;
	protected boolean sensitivityChecked = false;

	protected PatientLine patientLine = null;

	protected IRuntimeContext runtime;
	protected IDashboardDefinition dashDef;

	public AbstractDashboardPatient() {
		super();
	}

	public AbstractDashboardPatient(
		String id,
		String name,
		String ssn,
		String dob,
		String age) {
		setPatient(id, name, ssn, dob, age);
	}

	public void setPatient(
		String id,
		String name,
		String ssn,
		String dob,
		String age) {
		setPatientId(id);
		setPatientName(name);
		if (null != ssn)
			setSsn(ssn);
		if (null != dob && 0 != dob.length())
			setDob(dob);
		if (null != age && 0 != age.length())
			setAge(age);
		this.patientLine =
			(null != id
				? new PatientLine(
					patientName,
					this.ssn,
					this.dob,
					this.age,
					false)
				: PatientLine.emptyPatientLine());
	}

	private DashboardItems getTypeContainer(int seq) {
		DashboardItems dbItems = null;
		for (Iterator iter = values().iterator(); iter.hasNext();) {
			DashboardItems container = (DashboardItems) iter.next();
			if (container.getSequence() == seq) {
				dbItems = container;
				break;
			}
		}
		return dbItems;
	}

	public Object visualizeDashItems(int sequence, int view) {

		Object value = null;
		DashboardItems items = getTypeContainer(sequence);
		if (null == items)
			items = createEmptyContainer(sequence);
		switch (view) {
			case IDashboardItem.DASH_VIEW_ICON :
				DashboardIconView icon = new DashboardIconView();
				if (null != items) {
					DashboardItems alternate =
						(0 < sequence ? getTypeContainer(-sequence) : null);
					boolean hasAlternate =
						(null != alternate && 0 < alternate.getItems().size());
					List dashItems = items.getItems();
					icon.setItemType(
						items.getName()
							+ (hasAlternate ? "/" + alternate.getName() : ""));
					icon.setAcknowledged(0 < dashItems.size());
					icon.setIconValue(
						(0 < dashItems.size()
							|| hasAlternate
								? (items.hasCritical()
									? IDashboardItem.CRITICAL
									: IDashboardItem.NORMAL)
								: IDashboardItem.NOTHING));
					icon.setItemCount(
						dashItems.size() + (hasAlternate ? 1 : 0));
					for (Iterator iter = dashItems.iterator();
						iter.hasNext();
						) {
						IDashboardItem item = (IDashboardItem) iter.next();
						if (!item.isComplete()) {
							icon.setAcknowledged(false);
							break;
						}
					}
				}
				value = icon;
				break;

			case IDashboardItem.DASH_VIEW_DETAIL :
				if (null != items) {
					List dashItems = items.getItems();
					Iterator iter = dashItems.iterator();
					String rawUrl =
						setRepeaters(items.getURL(), dashItems.size());
					List variables = new ArrayList();
					variables.add(patientId);
					while (iter.hasNext()) {
						IDashboardItem item = (IDashboardItem) iter.next();
						try {
							variables.add(
								URLEncoder.encode(item.getID(), "UTF-8"));
						} catch (UnsupportedEncodingException e) {
							runtime.getLogger(getClass()).error(
								"Unable to encode URL",
								e);
						}
					}
					List permissions = items.getKeys();
					if (0 < permissions.size() && null != plugin) {
						for (Iterator iterator = permissions.iterator();
							iterator.hasNext();
							) {
							String permission = (String) iterator.next();
							variables.add(
								Boolean.valueOf(
									plugin.hasPermission(permission)));
						}
					}
					value = MessageFormat.format(rawUrl, variables.toArray());
				}
				break;
		}
		return value;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return patientName;
	}

	/**
	 * Sets the patient's name
	 * 
	 * @param patientName The patient's name usually formatted as <pre>LastName, FirstName</pre>
	 */
	public void setPatientName(String patientName) {
		this.patientName = patientName;
	}

	/**
	 * Returns the patient's name.
	 * 
	 * @return The patient's name as a String object.
	 */
	public String getPatientName() {
		return patientName;
	}

	/**
	 * Sets the patient's identifier.
	 * 
	 * @param patientId The patient's identifier as a String object.
	 */
	public void setPatientId(String patientId) {
		this.patientId = patientId;
	}

	/**
	 * Returns the patient's identifier.
	 * 
	 * @return The patient's identifier as a String object.
	 */
	public String getPatientId() {
		return patientId;
	}

	/**
	 * Sets the patient's date of birth.
	 * 
	 * @param dob The patient's date of birth formatted as <pre>"YYYYMMDD"</pre> where
	 * <dl><dt>YYYY</dt><dd>is the year</dd>
	 * <dt>MM</dt><dd>is the month.  A value between 01-12 inclusive.</dd>
	 * <dt>DD</dt><dd>is the day of the month.  A value between 01-31 inclusive.</dd></dt>
	 */
	public void setDob(String dob) {
		// The date input format is YYYYMMDD
		Calendar dobDate = Calendar.getInstance();
		try {
			dobDate.setTime(dateInputFormat.parse(dob));
			// The date output is dd MMM yy
			this.dob = dateOutputFormat.format(dobDate.getTime());

			int month = dobDate.get(Calendar.MONTH);
			int year = dobDate.get(Calendar.YEAR);
			int date = dobDate.get(Calendar.DATE);
			Calendar curDate = Calendar.getInstance();
			int curMonth = curDate.get(Calendar.MONTH);
			this.age = (curDate.get(Calendar.YEAR)) - year;
			if ((month > curMonth)
				|| (month == curMonth && date > curDate.get(Calendar.DATE)))
				this.age--;
		} catch (ParseException pe) {
			this.dob = "Unknown";
			this.age = 0;
		}
	}

	/**
	 * Returns the patient's date of birth as a formatted string.
	 * 
	 * @return The patient's date of birth as a String object formatted as <pre>
	 * "dd MMM yy"</pre> where:
	 * <dl><dt>dd</dt><dd>is the day of the month.</dd>
	 * <dt>MMM</dt><dd>is the three character abbreviation of the month</dd>
	 * <dt>yy</dt><dd>is the last two digits of the year</dd></dl>
	 */
	public String getDob() {
		return dob;
	}

	/**
	 * Sets the patient's Social Security Number
	 * 
	 * @param ssn The patient's Social Security Number.  No formatting is done
	 * to the input string.  It is stored as is.
	 */
	public void setSsn(String ssn) {
		this.ssn = ssn;
	}

	/**
	 * Returns the patient's Social Security Number.
	 * 
	 * @return The patient's Social Security Number as a String object. 
	 */
	public String getSsn() {
		return ssn;
	}

	public void setAge(String age) {
		this.age = Integer.parseInt(age);
	}
	/**
	 * Returns the age of the patient as calculated from the patient's date of 
	 * birth.  Currently, this doesn't take into account whether the patient is 
	 * dead.
	 * 
	 * @return The age of the patient as a String.
	 */
	public String getAge() {
		return Integer.toString(age);
	}

	/**
	 * Returns an object that describes how to display patient information.  The return value
	 * depends on the context.
	 * 
	 * @param context The context of the visualization.  Can be one of
	 * <ul><li>IDashItem.DASH_VIEW_ICON</li>
	 * <li>IDashItem.DASH_VIEW_DETAIL</li></ul>
	 * @return If the context is IDashItem.DASH_VIEW_ICON, the returned Object is a 
	 * javax.swing.JPanel object that displays the patient line.  If the context is
	 * IDashItem.DASH_VIEW_DETAIL the return value is the query portion of a URL as a String
	 * object.
	 */
	public Object visualizePatient(int context) {
		Object visualization = null;
		switch (context) {
			case IDashboardItem.DASH_VIEW_DETAIL :
				visualization =
					MessageFormat.format(
						patientURL,
						new Object[] { patientId });
				break;

			case IDashboardItem.DASH_VIEW_ICON :
				if (null == patientLine)
					patientLine =
						new PatientLine(
							patientName,
							ssn,
							dob,
							Integer.toString(age),
							false);
				visualization = patientLine;
				break;
		}
		return visualization;
	}

	public Object visualizeDetail(int type, int context) {
		if (!isSensitivityChecked())
			checkSensitivity();
		if ((!isSensitive() || sensitiveIsAllowed()) && null != dashDef)
			return (
				dashDef.getDefinitionByType(PATIENT_ITEM_ID).getSequence()
					== type
					? visualizePatient(context)
					: visualizeDashItems(type, context));
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if (null != obj) {
			if (obj instanceof IDashboardPatient) {
				Object pid = ((IDashboardPatient) obj).getPatientId();
				return (
					null == pid
						|| null == patientId ? false : patientId.equals(pid));
			}
			if (obj instanceof String)
				return patientId.equals(obj);
		}
		return false;
	}

	public void putItem(String itemType, IDashboardItem item) {
		DashboardItems items = (DashboardItems) get(itemType);
		if (null == items) {
			items = new DashboardItems(itemType);
			put(items.type, items);
		}
		if (null != item)
			items.putItem(item);
	}

	public IDashboardItem getItem(String itemType, Object itemID) {
		IDashboardItem item = null;
		DashboardItems items = (DashboardItems) get(itemType);
		if (null != items)
			item = items.getItem(itemID);
		return item;
	}

	private String setRepeaters(String url, int size) {
		// First separate the non-repeating portion from the repeating
		Matcher match = Pattern.compile("^(.+)(\\[.+\\])$").matcher(url);
		if (match.matches()) {
			String nonRepeatable = match.group(1);
			String repeatable = match.group(2);
			int idx = -1;
			match = Pattern.compile("\\{(\\d*)\\}").matcher(nonRepeatable);
			while (match.find()) {
				int group;
				try {
					group = Integer.parseInt(match.group(1));
				} catch (NumberFormatException e) {
					continue;
				}
				idx = (idx < group ? group : idx);
			}
			match = Pattern.compile("\\[(.+)\\]").matcher(repeatable);
			StringBuffer appendix = new StringBuffer();
			while (match.find()) {
				for (int i = 0; i < size; i++)
					appendix.append(match.group(1) + "{" + (++idx) + "}");
				nonRepeatable += appendix.toString();
			}
			return nonRepeatable.replaceFirst(
				"\\{last\\}",
				"{" + (++idx) + "}");
		}
		return url;
	}

	private IDashboardPatient getThis() {
		return this;
	}

	protected DashboardItems createEmptyContainer(int sequence) {
		if (null != dashDef) {
			Iterator iter = dashDef.iterator();
			while (iter.hasNext()) {
				IDashboardEntry entry = (IDashboardEntry) iter.next();
				if (sequence == entry.getSequence()) {
					putItem(entry.getID(), null);
					return (DashboardItems) get(entry.getID());
				}
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.runtime = runtime;
	}

	private class DashboardItems {
		int sequence;
		String type;
		String baseURL;
		String name;
		List permissions;
		IDashboardItemDAO dao;

		public DashboardItems(String type) {
			this.type = type;
			IDashboardEntry dashEntry = dashDef.getDefinition(type);
			IItemEntry item = dashEntry.getItemEntry();
			this.sequence = dashEntry.getSequence();
			this.name = item.getName();
			this.baseURL = item.getURL();
			this.permissions = item.getPermissions();
			try {
				this.dao = (IDashboardItemDAO) item.getItemDAOInstance();
			} catch (Exception e) {
				runtime.getLogger(getClass()).error(
								"Unable to get reference to instance of " + IDashboardItemDAO.class.getName(),
								e);
			}
		}

		public int getSequence() {
			return sequence;
		}

		public String getURL() {
			return baseURL;
		}

		public boolean hasCritical() {
			Iterator items = getItems().iterator();
			while (items.hasNext())
				if (((IDashboardItem) items.next()).isCritical())
					return true;
			return false;
		}

		public void putItem(IDashboardItem item) {
			dao.putItem(item);
		}

		public List getItems() {
			return getItems(false);
		}

		public IDashboardItem getItem(Object id) {
			return (IDashboardItem) dao.getItem(id);
		}

		public List getItems(List ids) {
			List items = dao.getItems(ids);
			Collections.sort(items, IDashboardItemDAO.descendingDate);
			return items;
		}

		public List getItems(boolean dirty) {
			List items = dao.getItems(getThis());
			for (Iterator iter = items.iterator(); iter.hasNext();) {
				IDashboardItem item = (IDashboardItem) iter.next();
				if (dirty && !item.isDirty())
					items.remove(item);
			}
			Collections.sort(items, IDashboardItemDAO.descendingDate);
			return items;
		}

		public String getType() {
			return type;
		}

		public String getName() {
			return name;
		}

		public List getKeys() {
			return permissions;
		}

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#isSensitive()
	 */
	public boolean isSensitive() {
		if (!isSensitivityChecked()) {

		}
		return sensitive;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#sensitiveIsAllowed()
	 */
	public boolean sensitiveIsAllowed() {
		return allowSensitive;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#setAllowSensitive(boolean)
	 */
	public void setAllowSensitive(boolean allowSensitive) {
		this.allowSensitive = allowSensitive;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#setSensitive(boolean)
	 */
	public void setSensitive(boolean sensitive) {
		this.sensitive = sensitive;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#isSensitivityChecked()
	 */
	public boolean isSensitivityChecked() {
		return sensitivityChecked;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardPatient#setSensitivityChecked(boolean)
	 */
	public void setSensitivityChecked(boolean sensitivityChecked) {
		this.sensitivityChecked = sensitivityChecked;
	}

	protected void checkSensitivity() {
		if(dashDef==null)
			return;
		IPatientListDAO dao = null;
		try {
			dao =
				(IPatientListDAO) dashDef
					.getDefinitionByType(IDashboardPatient.PATIENT_ITEM_ID)
					.getItemEntry()
					.getItemDAOInstance();
			setSensitive(dao.getSensitivePatient(getPatientId()));
		} catch (SensitivePatient sp) {
			IDesktop desktop = plugin.getDesktop();
			setSensitive(true);
			int status = sp.getStatus();
			Object[] options = new Object[2 == status ? 2 : 1];
			for (int i = 0; i < (2 == status ? 2 : 1); i++)
				options[i] = (0 == i ? (2 == status ? "Yes" : "OK") : "No");
			int answer =
				JOptionPane.showOptionDialog(
					desktop.getActiveWindow().getFrame(),
					sp.restrictedRecordMessage(),
					"Restricted Record",
					JOptionPane.OK_CANCEL_OPTION,
					(2 < status
						? JOptionPane.ERROR_MESSAGE
						: JOptionPane.WARNING_MESSAGE),
					null,
					options,
					options[options.length - 1]);
			if (JOptionPane.OK_OPTION == answer) {
				setSensitivityChecked(true);
				setAllowSensitive((2 < status ? false : true));
				if (2 == status && sensitiveIsAllowed())
					dao.logSensitivePatient(getPatientId());
			}
		} catch (Exception e) {
			runtime.getLogger(this.getClass()).error(
				e.getLocalizedMessage(),
				e);
		}
	}

}
