/*
 * Created on Apr 12, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.itemtype;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardResultItem extends AbstractDashboardItem
{
  public static final String ITEM_TYPE_ID = "dashboard.result";
  public static final int TEXT_RESULT = 1;
  public static final int DATA_RESULT = 2;

  private int resultType;
  
  public DashboardResultItem()
  {
    itemTypeID = ITEM_TYPE_ID;
  }
  
  public void setResultType(int resultType)
  {
    this.resultType = resultType;
  }

  /**
   * @return int
   */
  public int getResultType()
  {
    return resultType;
  }

  public Object createDataType(int resultType)
  {
    setResultType(resultType);
    return (TEXT_RESULT == resultType ? (Object)new StringBuffer() :
           (DATA_RESULT == resultType ? (Object)new ChemistryLabResult() :
            null));
  }

  class ChemistryLabResult implements IChemistryLabResult
  {
    private String resultName;
    private String resultValue;
    private String units;
    private String refRange;
    private String criticalFlag;

    public ChemistryLabResult()
    {
    }
 
    /**
     * Constructor for ChemLabResult
     * 
     * @param resultName the name of the analyte
     * @param resultValue the result's value
     * @param units the units for the result
     * @param refRange the reference ranges for the result
     * @param criticalFlag whether this result is critical (i.e. it falls outside the
     * reference range).
     */
    public ChemistryLabResult(String resultName, String resultValue, String units,
        String refRange, String criticalFlag)
    {
      setResult(resultName, resultValue, units, refRange, criticalFlag);
    }

    /**
     * Returns the name of the result
     * 
     * @return the name of the result as a String.
     */
    public String getResultName()
    {
      return resultName;
    }
  
    /**
     * Returns the value of the result.
     * 
     * @return the value of the result as a String.
     */
    public String getResultValue()
    {
      return resultValue;
    }
  
    /**
     * Returns the units of the result.
     * 
     * @return the units of the result as a String.
     */
    public String getUnits()
    {
      return units;
    }
  
    /**
     * Returns the reference ranges of the result.
     * 
     * @return the reference ranges of the result as a String.
     */
    public String getRefRange()
    {
      return refRange;
    }
  
    /**
     * Returns the critical flag of the result.
     * 
     * @return the critical flag of the result as a String.
     */
    public String getCriticalFlag()
    {
      return (null == criticalFlag ? " " : criticalFlag);
    }
  
    /**
     * Returns whether or not the result is critical.
     * 
     * @return true if the result is critical, otherwise false.
     */
    public boolean isCriticalResult()
    {
      return (null == criticalFlag || 0 == criticalFlag.trim().length()) ? false : true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setCriticalFlag(java.lang.String)
     */
    public void setCriticalFlag(String criticalFlag)
    {
      this.criticalFlag = criticalFlag;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setRefRange(java.lang.String)
     */
    public void setRefRange(String refRange)
    {
      this.refRange = refRange;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setResultName(java.lang.String)
     */
    public void setResultName(String resultName)
    {
      this.resultName = resultName;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setResultValue(java.lang.String)
     */
    public void setResultValue(String resultValue)
    {
      this.resultValue = resultValue;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setUnits(java.lang.String)
     */
    public void setUnits(String units)
    {
      this.units = units;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.dashboard.itemtype.IChemistryLabResult#setResult(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void setResult(String resultName, String resultValue, String units,
      String refRange, String criticalFlag)
    {
      this.resultName = resultName;
      this.resultValue = resultValue;
      this.units = units;
      this.refRange = refRange;
      this.criticalFlag = criticalFlag;
    }

  }
  
}
