package gov.va.med.hds.dashboard;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardTaskDAO;
import gov.va.med.hds.dashboard.dao.IPatientListDAO;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.items.DashItemMessageEvent;
import gov.va.med.hds.dashboard.items.IDashItemChangeListener;
import gov.va.med.hds.dashboard.items.IDashItemMessageEvent;
import gov.va.med.hds.dashboard.items.IDashItemMessageListener;
import gov.va.med.hds.dashboard.items.IDashboardEntry;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.AbstractDashboardPatient;
import gov.va.med.hds.dashboard.itemtype.DashboardTaskItem;
import gov.va.med.hds.task.data.ITask;

import java.awt.Cursor;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.event.EventListenerList;
import javax.swing.table.AbstractTableModel;

/**
 * Main table model for dashboard patients.
 */
public class DashboardPatientTableModel
	extends AbstractTableModel
	implements IMumpsListener, IDashItemChangeListener, IDashboardItemMessageSource {
	private class ascendingByName implements Comparator {
		public boolean equals(Object obj) {
			return this.equals(obj);
		}

		public int compare(Object obj1, Object obj2) {
			return ((IDashboardPatient) obj1).getPatientName().compareTo(
				((IDashboardPatient) obj2).getPatientName());
		}
	}

	private IPatientListDAO dao;
	private Map itemDAO = Collections.synchronizedMap(new Hashtable());
	private Object[] headers;
	private List data = null;
	private IDashboardPlugin plugin;
	private IDesktop desktop;
	private ILogger logger;
	private EventListenerList listenerList = new EventListenerList();

	public DashboardPatientTableModel(IDashboardPlugin plugin) {
		IRuntimeContext runtime;
		this.plugin = plugin;
		this.desktop = plugin.getDesktop();
		runtime = desktop.getRuntime();
		this.logger = runtime.getLogger(this.getClass());
		String colHeaders[] = new String[plugin.getHeaderCount()];
		try {
			Iterator types = plugin.getItemTypes();
			while (types.hasNext()) {
				IDashboardEntry entry = (IDashboardEntry) types.next();
				IItemEntry item = entry.getItemEntry();
				// Make sure this is not a silent entry (i.e. it will be displayed on the dashboard)
				if (null != entry.getLabel()) {
					colHeaders[entry.getSequence()] = entry.getLabel();
				}
				if (IPatientListDAO.ITEM_TYPE_ID.equals(item.getItemType()))
					this.dao = (IPatientListDAO) item.getItemDAOInstance();
				else
					itemDAO.put(item.getItemType(), item.getItemDAOInstance());
			}
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		setColumnHeaders(colHeaders);
	}

	public void setColumnHeaders(Object[] headers) {
		this.headers = headers;
	}

	public int getRowCount() {
		return (null == data ? 0 : data.size());
	}

	public int getColumnCount() {
		return headers.length;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		Object value = null;

		if (-1 != rowIndex && 0 < getRowCount() && rowIndex <= getRowCount()) {
			IDashboardPatient patient = (IDashboardPatient) data.get(rowIndex);
			switch (columnIndex) {
				case 0 : // Patient Name
					value = patient;

					break;

				default : // Results, Tasks, Events, Unsigned orders/notes
					value =
						patient.visualizeDashItems(
							columnIndex,
							IDashboardItem.DASH_VIEW_ICON);

					break;
			}
		}

		return value;
	}

	public void CallFailed(ICallRequest request, MumpsException e) {
		logger.error(e.getLocalizedMessage(), e);
		data =
			Collections.singletonList(
				new AbstractDashboardPatient(
					null,
					e.getLocalizedMessage(),
					null,
					null,
					null));
		fireTableDataChanged();
		IDesktopWindow dtWin = desktop.getActiveWindow();
		if (null != dtWin)
			dtWin.getFrame().getRootPane().setCursor(Cursor.getDefaultCursor());
	}

	public void CallSucceeded(ICallRequest request, String result) {
		class parsePatients extends Thread {
			String result;
			public parsePatients(String result) {
				this.result = result;
			}
			public void run() {
				IDesktopWindow dtWin = desktop.getActiveWindow();
				if (null != dtWin)
					dtWin.getFrame().getRootPane().setCursor(
						Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				try {
					data = dao.parsePatientList(result);
				} catch (PatientParseException e) {
					logger.error("Error getting patient data", e);
					data = null;
				}
				List errorStrings = dao.getErrorStrings();
				if (errorStrings.size()>0) {
					fireMessageGenerated(errorStrings);
				}
				if (null != data && 0 != data.size()) {
					Collections.sort(data, new ascendingByName());
				} else
					data.add(
						new AbstractDashboardPatient(
							null,
							"No Patients",
							null,
							null,
							null));
				fireTableDataChanged();
				if (null != dtWin)
					dtWin.getFrame().getRootPane().setCursor(
						Cursor.getDefaultCursor());
			}
		}
		new parsePatients(result).start();
	}

	public Class getColumnClass(int columnIndex) {
		return (0 == columnIndex ? String.class : JLabel.class);
	}

	public String getColumnName(int column) {
		return headers[column].toString();
	}

	private List sourceIdList;
	public void populatePatients(List sourceIds) {
		IDesktopWindow dtWin = desktop.getActiveWindow();
		if (null != dtWin)
			dtWin.getFrame().getRootPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		dao.getDashboardPatientList(null, (sourceIdList = sourceIds), this);
	}

	public String getDefaultPatientListSource() {
		return dao.getDefaultPatientListSource();
	}

	public void itemUpdated(DashItemChangeEvent e) {
		class doUpdate implements Runnable {
			DashItemChangeEvent e;

			public doUpdate(DashItemChangeEvent e) {
				this.e = e;
			}

			public void run() {
				IDashboardItem item = null;
				Object dashItem = e.getSource();
				if (!(dashItem instanceof IDashboardItem)) {
					switch (e.getEventType()) {
						case DashItemChangeEvent.ITEM_ADDED :
							try {
								item =
									(IDashboardItem) plugin
										.getDashboardItemDefinition(
											e.getItemType().toString())
										.getItemEntry()
										.getInstance();
								if (null != item) {
									if (e
										.getItemType()
										.toString()
										.equals(
											DashboardTaskItem.ITEM_TYPE_ID)) {
										item.setID(
											"TSK:"
												+ ((ITask) dashItem).getID());
										item.setData(dashItem);
										IDashboardPatient patient =
											getPatient(
												((ITask) dashItem)
													.getPatientID());
										if (null != patient) {
											item.setPatient(patient);
											(
												(
													DashboardTaskItem) item)
														.setCritical(
												((ITask) dashItem)
													.getPriority());
											patient.putItem(
												item.getTypeID(),
												item);
										} else
											item.setPatient(
												((ITask) dashItem)
													.getPatientID());
									} else
										item = null;
								}
							} catch (Exception e) {
								logger.error(e.getLocalizedMessage(), e);
							}
							break;

						case DashItemChangeEvent.ITEM_MODIFIED :
							if (e
								.getItemType()
								.toString()
								.equals(DashboardTaskItem.ITEM_TYPE_ID)) {
								IDashboardPatient patient =
									getPatient(
										((ITask) dashItem).getPatientID());
								if (null != patient) {
									item =
										patient.getItem(
											(String) e.getItemType(),
											IDashboardTaskDAO.prefix
												+ ((ITask) dashItem).getID());
									if (null != item)
										item.setData(dashItem);
								}
							} else
								item = null;
							break;

						case DashItemChangeEvent.ITEM_DELETED :
							if (e
								.getItemType()
								.toString()
								.equals(DashboardTaskItem.ITEM_TYPE_ID)) {
								IDashboardPatient patient =
									getPatient(
										((ITask) dashItem).getPatientID());
								if (null != patient) {
									item =
										patient.getItem(
											(String) e.getItemType(),
											IDashboardTaskDAO.prefix
												+ ((ITask) dashItem).getID());
									if (null != item) {
										IDashboardItemDAO dao =
											(IDashboardItemDAO) itemDAO.get(
												e.getItemType().toString());
										dao.deleteItem(item);
									}
								}
							}

					}
				} else
					item = (IDashboardItem) dashItem;

				if (null != item) {
					IDashboardEntry dbEntry =
						plugin.getDashboardItemDefinition(item.getTypeID());
					// Since all dashboards are potentially listening for item changes
					// we need to make sure this dashboard has this type of item
					if (null != dbEntry) {
						int col = dbEntry.getSequence();
						int row = getPatientRow(item.getPatient());
						if (-1 != row && data.size() > row)
							fireTableCellUpdated(row, col);
					}
				}
			}
		}
		SwingUtilities.invokeLater(new doUpdate(e));
	}

	public IDashboardPatient getPatient(Object patientID) {
		IDashboardPatient patient = null;
		Iterator iter = data.iterator();
		while (iter.hasNext()) {
			patient = (IDashboardPatient) iter.next();
			if (patient.equals(patientID))
				break;
		}
		return patient;
	}

	protected int getPatientRow(Object patient) {
		if (null != data && null != patient) {
			Object[] patients = data.toArray();
			for (int i = 0; i < patients.length; i++) {
				if (patients[i].equals(patient))
					return i;
			}
		}
		return -1;
	}
	
	private void fireMessageGenerated(List messageList) {
		String message = "Patients will be excluded for the following reason";
		if (messageList.size()>1) {
			message += "s:";
		}
		else {
			message += ":";
		}
		message += "\n\n";
		for (int i = 0; i < messageList.size(); i++) {
			message += messageList.get(i) + "\n";
		}
		fireMessageGenerated(message);
	}

	public void fireMessageGenerated(String message) {
		if (listenerList.getListenerCount()>0) {
			IDashItemMessageEvent event = new DashItemMessageEvent(this, message);
			IDashItemMessageListener[] listeners = (IDashItemMessageListener[]) listenerList.getListeners(IDashItemMessageListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].messageGenerated(event);
			}
		}
	}

	public void addDashItemMessageListener(IDashItemMessageListener l) {
		listenerList.add(IDashItemMessageListener.class, l);
	}

	public void removeDashItemMessageListener(IDashItemMessageListener l) {
		listenerList.remove(IDashItemMessageListener.class, l);
	}

}
