/*
 * Created on Apr 30, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard;

import gov.va.med.hds.cd.config.IConfiguration;
import gov.va.med.hds.cd.config.IConfigurator;
import gov.va.med.hds.cd.config.IParameter;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
//import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.util.ExceptionStackTrace;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class PatientListSources extends DefaultHandler {
	private IConfigurator configurator;
	private static final String CONFIGURATION_PARAMETER =
		"Patient Selection List";
	private static final String DASHBOARD_PLUGIN = "gov.va.med.hds.dashboard.nurse";
//	private static final int CONFIGURATION_READY_TIMEOUT_MILLISECONDS = 5000;
	private IConfiguration configuration;
	protected static final String LIST_ELEMENT = "list";
	protected static final String SOURCE_ELEMENT = "source";
	protected static final String ROOT_ELEMENT = "patientListSources";
	protected static final String XML_SOURCE_FILE =
		"./configuration/{0}.cohorts.xml";

	protected IRuntimeContext runtime;
	protected ILogger logger;

	protected IParameter patListParam;
	protected List sources = new ArrayList();
	protected List listSources;
	protected String listName;
	protected boolean defaultSource;

	public PatientListSources(IRuntimeContext runtime) {
		this.runtime = runtime;
		this.logger = runtime.getLogger(this.getClass());
		logger.debug("constructing PatientListSources()");

		try {
//			ISecurity security =
//				(ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
//			String user = ResolveUserId.resolveUser(security.getSubject());
			configurator =
				(IConfigurator) runtime.getPluginInstance(
					IConfigurator.PLUGIN_ID);
		} catch (Exception e) {
			logger.warning(
				"Unable to get dashboard configuration parameters: "
					+ ExceptionStackTrace.getStackTrace(e));
			patListParam = null;
		}
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName)
		throws SAXException {
		if (LIST_ELEMENT.equals(qName)) {
			if (defaultSource)
				sources.add(0, new PatientList(listName, listSources, true));
			else
				sources.add(new PatientList(listName, listSources));
		}
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(
		String uri,
		String localName,
		String qName,
		Attributes attributes)
		throws SAXException {
		if (LIST_ELEMENT.equals(qName)) {
			listSources = new ArrayList();
			listName = attributes.getValue("name");
			String defaultAttr = attributes.getValue("default");
			defaultSource =
				(null == defaultAttr
					? false
					: Boolean.valueOf(defaultAttr).booleanValue());
		} else if (SOURCE_ELEMENT.equals(qName))
			listSources.add(attributes.getValue("id"));
	}

	public List readConfiguration() {
		try {
			configuration =
				configurator.getConfiguration(DASHBOARD_PLUGIN, 10000);
			if (configuration != null) {
				logger.debug("readConfiguration has ONE!!!");
			} else {
				logger.debug("readConfiguration crapped out!!!");
                return null;
			}
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		return readConfiguration(configuration);
	}

	public List readConfiguration(IConfiguration configuration) {
		try {
			patListParam =
				configuration.getParameter(
					PatientListSources.CONFIGURATION_PARAMETER);
			if (patListParam == null || 
				patListParam.getValue() == null ||
				patListParam.getValue().trim().length() == 0) {
				return null;
			}
                
			SAXParser parser =
				SAXParserFactory.newInstance().newSAXParser();
			parser.parse(
				new InputSource(
					new StringReader(patListParam.getValue())),
				PatientListSources.this);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		return (0 == sources.size() ? null : sources);
	}

	public void writeConfiguration(List sources) {
		logger.debug("in writeConfiguration()");
		if (configuration == null) return;
		
		StringBuffer xml = new StringBuffer("<" + ROOT_ELEMENT + ">" + "\n");
		Iterator iter = sources.iterator();
		while (iter.hasNext())
			xml.append(((PatientList) iter.next()).toXML());
		xml.append("</" + ROOT_ELEMENT + ">" + "\n");
		try {
			configuration.putParameter(CONFIGURATION_PARAMETER, xml.toString());
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}

}
