package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.DateComboFieldEditor;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.dashboard.PatientListSources;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class ClinicianDashboardPreferencePage extends PreferencePage {

	private static final String ACTIVATION_DATE_PARAMETER_KEY = "Care Management Activation Date";
	private String name;
	private DateComboFieldEditor minDashboardDate;

	public ClinicianDashboardPreferencePage() {		
		setBorder(new EmptyBorder(10, 10, 10, 10));

		minDashboardDate = new DateComboFieldEditor("Activation Date:");

		addFieldEditor(minDashboardDate);
		

		int padTo = 35 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IPreferencePage#initializeFieldEditors()
	 */
	public void initializeFieldEditors() {
		try {
			minDashboardDate.setParameter(
				configuration.getParameter(ACTIVATION_DATE_PARAMETER_KEY));
			minDashboardDate.loadInitialValue();
		} catch (NonExistentParameterException e) {
			runtime.getLogger(getClass()).error("Unable to get paramater '" + ACTIVATION_DATE_PARAMETER_KEY + "'", e);
		}
	}

	/**
	 * @return
	 */
	private Object[] buildPatientListModel() {
		PatientListSources sources = new PatientListSources(getRuntime());
		return sources.readConfiguration(configuration).toArray();
	}

	/**
	 * @see java.awt.Component#setName(String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @see java.awt.Component#getName()
	 */
	public String getName() {
		return name;
	}

}
