/*
 * Created on Apr 9, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.ui;

import java.awt.Component;
import java.awt.Cursor;
import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

import gov.va.med.hds.cd.prism.DesktopCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopEvent;
import gov.va.med.hds.cd.prism.IDesktopListener;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;
import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.dashboard.DashTableEvent;
import gov.va.med.hds.dashboard.IDashPatients;
import gov.va.med.hds.dashboard.IDashTableListener;
import gov.va.med.hds.dashboard.ItemDAOUpdater;
import gov.va.med.hds.dashboard.items.IDashItemChangeListener;
import gov.va.med.hds.dashboard.items.IDashboardDefinition;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.task.data.ITask;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardDetailPane extends DHTMLPane
  implements IDashTableListener, IPerspectiveListener,
             IDesktopListener, IDashItemChangeListener
{
  private String perspectiveID ="";
  
  private IRuntimeContext runtime;
  private ILogger logger = null;
  private IDashboardDefinition dashboardDefinition = null;
  private int lastRow = -1, lastCol = -1;
  private ItemDAOUpdater itemUpdater;
  
  /**
   * @param runtime
   */
  public DashboardDetailPane(IRuntimeContext runtime, String dashboardID)
  {
    super(runtime);
    this.runtime = runtime;
    PrismUtilities.getDesktop(runtime).addDesktopListener(this);
    logger = runtime.getLogger(this.getClass());
    itemUpdater = new ItemDAOUpdater();
		try
		{
      dashboardDefinition = (IDashboardDefinition)((IDashboardItemTypes)runtime
			  .getPluginInstance(IDashboardItemTypes.PLUGIN_ID))
			  .getDashboardDefinition(dashboardID);
		}
		catch (Exception e)
		{
      logger.error(e.getLocalizedMessage(), e);
		}
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashTableListener#valueChanged(gov.va.med.hds.dashboard.DashTableEvent)
   */
  public void valueChanged(DashTableEvent e)
  {
    class NewSelection implements Runnable
    {
      private DashTableEvent e;
      public NewSelection(DashTableEvent e)
      {
        this.e = e;
      }
      
      public void run()
      {
        JTable source = (JTable)e.getSource();
        int col = source.getSelectedColumn();
        int row = source.getSelectedRow();
        Object value = source.getValueAt(row, col);
        if (null == value)
          clearDetail();
        else
        {
          int headersLength = ((IDashPatients)source).getNumDataCols();
          int beginCol = (col / headersLength) * headersLength;
          IDashboardPatient patient = (IDashboardPatient)source.getValueAt(source.getSelectedRow(), beginCol);

          String url = "";
          try
          {
            // Order is important here.  The visualizeDetail checks for restricted patients
            // The patient line shouldn't be displayed until it's known whether or not the
            // the user can view the data.
            url = (String)patient.visualizeDetail(col % headersLength, IDashboardItem.DASH_VIEW_DETAIL);
            Component parent = getParent();
            if (parent instanceof JViewport)
              ((JScrollPane)parent.getParent()).setColumnHeaderView((Component)patient.visualizePatient(IDashboardItem.DASH_VIEW_ICON));

            if (null != url)
            {
              Cursor oldCursor = showCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
              setPage(url);
              showCursor(oldCursor);
              updateDirtyItems();
            }
            else
              clearDetail();
          }
          catch (IOException e)
          {
            logger.error(e.getLocalizedMessage(), e);
          }
        }
      }
    }
    if (lastRow != ((JTable)e.getSource()).getSelectedRow() ||
        lastCol != ((JTable)e.getSource()).getSelectedColumn())
    {
      lastRow = ((JTable)e.getSource()).getSelectedRow();
      lastCol = ((JTable)e.getSource()).getSelectedColumn();
      SwingUtilities.invokeLater(new NewSelection(e));
    }
  }
  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashTableListener#tableChanged(gov.va.med.hds.dashboard.DashTableEvent)
   */
  public void tableChanged(DashTableEvent e)
  {
    if (lastRow != e.getRow() && lastCol != e.getColumn())
      clearDetail();
  }
  
  public void clearDetail()
  {
    Component parent = getParent();
    if (parent instanceof JViewport)
      ((JScrollPane)parent.getParent()).setColumnHeaderView(PatientLine.emptyPatientLine());
    setBlankPage();
    lastCol = lastRow = -1;
  }

  public void setReferencePerspective(String perspectiveID)
  {
    this.perspectiveID = perspectiveID;
  }
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveActivated(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveActivated(PerspectiveEvent e)
	{
		if (e.getPerspective().getDescriptor().getId().equals(perspectiveID))
    {
      URL currentURL = getPage();
      if (null != currentURL)
        try
        {
          reload();
        }
        catch (IOException ioe)
        {
          logger.error(ioe.getLocalizedMessage(), ioe);
        }
    }
    else
      updateDirtyItems();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveClosed(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveClosed(PerspectiveEvent e)
	{
    updateDirtyItems();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveOpened(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveOpened(PerspectiveEvent e)
	{
		// Do nothing
	}

  private void updateDirtyItems()
  {
    if (null != dashboardDefinition)
    {
      try
      {
        itemUpdater.updateDirtyItems(dashboardDefinition.iterator(), false);
      }
      catch (Exception e)
      {
        logger.error(e.getLocalizedMessage(), e);
      }
    }
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopClosing(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopClosing(DesktopEvent e) throws DesktopCloseVetoException
	{
		updateDirtyItems();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopOpening(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopOpening(DesktopEvent e) {} // Do Nothing

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashItemChangeListener#itemUpdated(gov.va.med.hds.dashboard.items.ItemChangeEvent)
	 */
	public void itemUpdated(DashItemChangeEvent e)
	{
    // Currently only handle adds - specifically adding tasks.
		if (DashItemChangeEvent.ITEM_ADDED == e.getEventType())
    {
			IDashboardItemTypes types = null;
			try
			{
				types = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
			}
			catch (Exception e1)
			{
				logger.error(e1.getLocalizedMessage(), e1);
			}
      if (null != types)
      {
        // Need to determine if the currently displayed page is for this type.
        // To do this compare the current URL with the URL pattern for the
        // item type.
        IItemEntry entry = types.getItemType((String)e.getItemType());
        if (null != entry)
        {
          String typeQuery = "";
          String typeURL = entry.getURL();
          Matcher match = Pattern.compile("^(.+)\\?(.*)$")
            .matcher(typeURL.subSequence(0, typeURL.length()));
          if (match.matches())
          {
            // save the query portion in case we need it.
            typeQuery = match.group(2);
            // get the non-query portion of the URL
            typeURL = match.group(1);
          }
          String currentURL = getPage().getPath();
          if (typeURL.endsWith(currentURL))
          {
            // They're the same so now we need to add the item to the URL's 
            // query and redisplay the page.
            // First find the repeating part of the query.  This should 
            // be how the page identifies items.
            match = Pattern.compile("^.*\\[(.+)\\].*$")
              .matcher(typeQuery.subSequence(0, typeQuery.length()));
            if (match.matches())
            {
              // Currently this object is only listening for changes (specifically adds)
              // of Task items.  If this assumption changes, the next line will need 
              // to be enhanced.
              try
              {
                String itemID = URLEncoder.encode("TSK:" + ((ITask)e.getSource()).getID(), "UTF-8");
                typeQuery = match.group(1);
                String query = getPage().getQuery() + typeQuery + itemID;
								setPage(typeURL + "?" + query);
							}
							catch (IOException e2)
							{
								logger.error(e2.getLocalizedMessage(), e2);
							}
            }
          }
        }
      }
    }
	}

	public void perspectiveClosing(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
    
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
    
	}
    
    private Cursor showCursor(Cursor c) {
        JRootPane root = SwingUtilities.getRootPane(this);
        if (root != null) {
            Cursor previousCursor = root.getCursor();
            root.setCursor(c);
            return previousCursor;
        } else
            return null;
    }

	public void desktopClosed(DesktopEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}
}
