package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.DashboardPatientTableModel;
import gov.va.med.hds.dashboard.IDashTableListener;
import gov.va.med.hds.dashboard.IDashboardPatientVisualComponent;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.items.IDashItemMessageEvent;
import gov.va.med.hds.dashboard.items.IDashItemMessageListener;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

/**
 * Abstract DashboardPatient visual component.  This is essecially a factory / manager of a
 * dashboard patient list component.
 */
public abstract class DashboardPatient
	implements IVisualComponent, IDashboardPatientVisualComponent, IDashItemMessageListener {
		
	public final static String ITEM_TYPE_ID = "dashboard.patient";

	protected IDesktop desktop;
	protected IVisualComponentDescriptor descriptor;
	protected JComponent component;
	protected DashboardPatientList patientList;
	protected DashboardPatientTableModel dataModel;

	public JComponent getComponent() {
		return component;
	}

	public void addDashTableListener(IDashTableListener l) {
		patientList.addDashTableListener(l);
	}

	public void removeDashTableListener(IDashTableListener l) {
		patientList.removeDashTableListener(l);
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.desktop = PrismUtilities.getDesktop(runtime);
//		this.descriptor = ((IVisualComponentDescriptor) data).getVisualComponentDescriptor();
		this.component = createComponent(descriptor, desktop);
		addHelpIds();
	}

	public void itemUpdated(DashItemChangeEvent e) {
		dataModel.itemUpdated(e);
	}

	public void messageGenerated(final IDashItemMessageEvent event) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(null, event.getMessage(),"Dashboard Message",JOptionPane.INFORMATION_MESSAGE);
			}
		});
	}
	
	protected abstract JComponent createComponent(IVisualComponentDescriptor descriptor, IDesktop desktop);
	
	protected abstract void addHelpIds();

}
