/*
 * Created on Mar 29, 2003
 *
 */
package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.prism.DesktopCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopEvent;
import gov.va.med.hds.cd.prism.IDesktopListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.ButtonComboBox;
import gov.va.med.hds.cd.spectrum.EditableTableHeader;
import gov.va.med.hds.dashboard.DashTableEvent;
import gov.va.med.hds.dashboard.DashboardPatientTableModel;
import gov.va.med.hds.dashboard.DashboardTableModelWrapper;
import gov.va.med.hds.dashboard.IDashPatients;
import gov.va.med.hds.dashboard.IDashTableListener;
import gov.va.med.hds.dashboard.IDashboardPlugin;
import gov.va.med.hds.dashboard.ItemDAOUpdater;
import gov.va.med.hds.dashboard.PatientList;
import gov.va.med.hds.dashboard.PatientListSources;
import gov.va.med.hds.dashboard.items.IDashboardDefinition;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.querytool.types.PatientSelection;
import gov.va.med.hds.querytool.ui.EditorFactoryFactory;
import gov.va.med.hds.querytool.ui.FactoryException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardPatientList
	extends JTable
	implements
		IDashPatients,
		ItemListener,
		ActionListener,
		IDesktopListener,
		IPerspectiveListener {
	static final List CPRS_DEFAULT_LIST_DEF = Collections.singletonList("x");
	public static final int NUM_WRAPPER_COLS = 2;

    private static final String PT_LIST_ACTION_KEY = "ActivatePatientList";
	private final String LIST_NAME_PREFIX = "Patients: ";
	private final String LIST_NAME_DEFAULT = "List ";
	private final int defaultNameLen =
		(LIST_NAME_PREFIX + LIST_NAME_DEFAULT).length();
	private static Cursor waitCursor =
		Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
	private static Cursor defaultCursor = Cursor.getDefaultCursor();
	private IRuntimeContext runtime;
	private IDashboardPlugin plugin;
	private ILogger logger;
  
  private boolean hasDynamic = false;

	private int counter = 0;
	private EditorFactoryFactory editorFactoryFactory;
	private DashboardPatientTableModel internalTableModel;
	private Object user;
	private String dashboardID;

	private ItemDAOUpdater itemUpdater;
	private PatientListSources sources;
	private PatientList currentPatientList;
	private Vector patientLists;

	public DashboardPatientList(TableModel dataModel, IDashboardPlugin plugin)
		throws Exception {
		super(dataModel);
		this.plugin = plugin;
		this.runtime = plugin.getDesktop().getRuntime();
		this.logger = runtime.getLogger(this.getClass());
		this.user = plugin.getUser();
		this.dashboardID = plugin.getModuleID();
    this.hasDynamic = plugin.getPluginID().endsWith("clinician");
		itemUpdater = new ItemDAOUpdater();
		plugin.getDesktop().addDesktopListener(this);
		plugin.getDesktop().getPerspectiveEventSource().addPerspectiveListener(
			this);
		jbInit();
	}

	protected void jbInit() throws Exception {
		setShowGrid(false);
		setIntercellSpacing(new Dimension(0, 0));
		setDefaultRenderer(JLabel.class, new IndicatorRendererX());
		setDefaultRenderer(String.class, new PatientNameRendererX());
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setRowSelectionAllowed(false);
		setCellSelectionEnabled(true);
		setBackground(Color.white);

		JTableHeader tableHeader =
			new EditableTableHeader(getTableHeader().getColumnModel());
		tableHeader.setReorderingAllowed(false);
		setTableHeader(tableHeader);
		((EditableTableHeader) tableHeader).setHeaderEditor(
			createPatientListSelector(),
			0);
		tableHeader.setDefaultRenderer(new HeaderRendererX());
	}
    
    private class ActivatePtListAction extends AbstractAction implements PopupMenuListener {

        private ButtonComboBox ptListSelector;
        
        public ActivatePtListAction(ButtonComboBox b) {
            ptListSelector = b;
            ptListSelector.addPopupMenuListener(this);
        }

		public void actionPerformed(ActionEvent e) {
            ptListSelector.setFocusable(true);
            ptListSelector.showPopup();
            ptListSelector.requestFocus(true);
		}

        public void popupMenuWillBecomeVisible(PopupMenuEvent e) { }
		public void popupMenuWillBecomeInvisible(PopupMenuEvent e) { defocus(); }
		public void popupMenuCanceled(PopupMenuEvent e) { defocus(); }
        private void defocus() {
            if (ptListSelector.hasFocus())
                ptListSelector.transferFocus();
            ptListSelector.setFocusable(false);
		}
    }

	public int getNumDataCols() {
		return internalTableModel.getColumnCount();
	}

	private JComponent createPatientListSelector() {
		logger.debug("creating patient list editor");
		// TODO: 
		patientLists = new Vector();
		ButtonComboBox editor = new ButtonComboBox(patientLists);
		editor.setBackground(HeaderRendererX.HEADER_COLOR);
		editor.getArrowButton().setBackground(HeaderRendererX.HEADER_COLOR);
        editor.setFocusable(false); 
		//    editor.addItemListener(this);

		// Need to get saved patient list definitions
		List editorList = getPatientListSources().readConfiguration();
		if (null == editorList) {
			editorList = new ArrayList();
		} else //  check for names of the form "Patients: List #"
			{
			String defaultName =
				(LIST_NAME_PREFIX + LIST_NAME_DEFAULT) + "\\d+$";
			for (Iterator iter = editorList.iterator(); iter.hasNext();) {
				PatientList item = (PatientList) iter.next();
				if (item.toString().matches(defaultName)) {
					int value =
						Integer.parseInt(
							item.toString().substring(defaultNameLen));
					if (value > counter)
						counter = value;
				}
			}
		}

    // Add standard lists if they're not already there
    PatientList dynList = new PatientList("Patients: Dynamic", new ArrayList(0));
    if (hasDynamic && !editorList.contains(dynList))
      editorList.add(dynList);
    else if (!hasDynamic)
      editorList.remove(dynList);
    PatientList defaultList = new PatientList("Patients: CPRS Default List",
      new ArrayList(CPRS_DEFAULT_LIST_DEF));
    if (!editorList.contains(defaultList))
      editorList.add(defaultList);

		// Always add this element last 
		editorList.add(new PatientList("Create/Edit List...", null));
		editor.addActionListener(this);
		patientLists.addAll(editorList);
		editor.setSelectedIndex(0);
		editor.setVisible(true);
		currentPatientList = (PatientList) editorList.get(0);

		this.getInputMap().put(
			KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.ALT_MASK),
			PT_LIST_ACTION_KEY);
		this.getActionMap().put(
			PT_LIST_ACTION_KEY, new ActivatePtListAction(editor));
        
		return editor;
	}

	public void refreshPatientList(PatientList ptList, boolean flush) {
   		initWaitCursor();
		try {
			IDashboardDefinition dashDef =
				(IDashboardDefinition)
					(
						(IDashboardItemTypes) runtime.getPluginInstance(
							IDashboardItemTypes
								.PLUGIN_ID))
								.getDashboardDefinition(
					dashboardID);
			itemUpdater.updateDirtyItems(dashDef.iterator(), flush);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		internalTableModel.populatePatients(ptList.listIDs);
		currentPatientList = ptList;
	}

	/* (non-Javadoc)
	 * @see javax.swing.JTable#setModel(javax.swing.table.TableModel)
	 */
	public void setModel(TableModel dataModel) {
		internalTableModel = (DashboardPatientTableModel) dataModel;
		super.setModel(
			new DashboardTableModelWrapper(
				internalTableModel,
				NUM_WRAPPER_COLS));
	}

	private PatientList showPatientListDialog() {
		if (null == editorFactoryFactory) {
			editorFactoryFactory = new EditorFactoryFactory();
			try {
				editorFactoryFactory.setRuntime(runtime);
			}
			catch (FactoryException ex) {
				logger.error(ex.getLocalizedMessage(), ex);
			}
		}
		PatientSelection patientSelect = new PatientSelection();
		IRunnableEditorPanel editor = null;
		try {
			editor =
				editorFactoryFactory
					.getFactory(PatientSelection.class.getName())
					.getEditor();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		editor.setValues(patientSelect.getValues());
		try {
			editor.init();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		JOptionPane optionPane =
			new JOptionPane(
				editor,
				JOptionPane.PLAIN_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog =
			optionPane.createDialog(
				this.getRootPane(),
				editor.getWindowHeader());

		//    show modal dialog
		dialog.show();

		//    save values in query
		Object result = optionPane.getValue();
		if ((result != null)
			&& (((Integer) result).intValue() == JOptionPane.OK_OPTION)
			&& 0 != editor.getValues().size()) {
			JTextField listName = new JTextField(LIST_NAME_DEFAULT + ++counter);
			listName.setSelectionStart(0);
			listName.setSelectionEnd(listName.getText().length());
			JLabel label = new JLabel("Enter a name for the new list: ");
			label.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
			label.setHorizontalAlignment(SwingConstants.LEFT);
			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
			panel.add(label, null);
			panel.add(listName, null);
			JOptionPane getName =
				new JOptionPane(
					panel,
					JOptionPane.PLAIN_MESSAGE,
					JOptionPane.DEFAULT_OPTION);
			getName.createDialog(this, "Patient List Name").show();
			return new PatientList(
				LIST_NAME_PREFIX + listName.getText().trim(),
				editor.getValues());
		}
		return null;
	}

	private class PatientListActionHandler implements Runnable {
		ButtonComboBox source;
		PatientList selection;
		boolean hide;

		PatientListActionHandler(
			ButtonComboBox source,
			Object selection,
			boolean hide) {
			this.source = source;
			if (selection instanceof PatientList)
				this.selection = (PatientList) selection;
            else
                this.selection = (PatientList) source.getSelectedItem();
			this.hide = hide;
		}
        
        PatientListActionHandler(ActionEvent e) {
            this((ButtonComboBox) e.getSource(), null, false);
        }
        
        PatientListActionHandler(ItemEvent e) {
            this((ButtonComboBox) e.getSource(), (PatientList) e.getItem(), true);
        }

		public void run() {
			initWaitCursor();
			if (selection == null || selection.isNull()) {
                DashboardPatientListEditor editor =
					new DashboardPatientListEditor(
						runtime,
						DashboardPatientList.this,
						patientLists);
				PatientList newList = editor.getList();
				if (null != newList)
					source.setSelectedItem(newList);
				else {
					int pos = patientLists.indexOf(currentPatientList);
					if (pos == -1)
						pos = 0;
					source.setSelectedIndex(pos);
				}
                selection = (PatientList) source.getSelectedItem();
				if (hide)
					source.setVisible(false);
			} else{
				refreshPatientList(selection, true);
			}
			TableColumn tc =
				(TableColumn) source.getClientProperty(
					EditableTableHeader.CELL_EDITOR_KEY);
			if (null != tc) {
				tc.setHeaderValue(selection);
				// force the header to repaint.
				resizeAndRepaint();
			}
			removeWaitCursor();
		}

	}
	private void removeWaitCursor() {
		JRootPane rootPane = getRootPane();
		if(rootPane != null)
			rootPane.setCursor(null);
	}

	private void initWaitCursor() {
		JRootPane rootPane = getRootPane();
		if(rootPane != null)
			rootPane.setCursor(waitCursor);
	}
	private void restoreDefaultCursor() {
		JRootPane root = getRootPane();
		if (null != root) {
			if (root.getCursor().equals(waitCursor))
				root.setCursor(defaultCursor);
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		SwingUtilities.invokeLater(new PatientListActionHandler(e));
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		if (ItemEvent.SELECTED == e.getStateChange())
			SwingUtilities.invokeLater(new PatientListActionHandler(e));
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
	 */
	public void tableChanged(TableModelEvent e) {
		class tableChangeEvent implements Runnable {
			Object source;
			public tableChangeEvent(Object source) {
				this.source = source;
			}
			public void run() {
				fireTableChanged();
				resizeAndRepaint();
				restoreDefaultCursor();
			}
		}
		super.tableChanged(e);
		if (null != runtime)
			// If runtime is null we're still being constructed
			SwingUtilities.invokeLater(new tableChangeEvent(this));
	}

	/* (non-Javadoc)
	 * @see javax.swing.JTable#changeSelection(int, int, boolean, boolean)
	 */
	public void changeSelection(
		int rowIndex,
		int columnIndex,
		boolean toggle,
		boolean extend) {
		class valueChangeEvent implements Runnable {
			Object source;
			public valueChangeEvent(Object source) {
				this.source = source;
			}
			public void run() {
				fireValueChanged();
				resizeAndRepaint();
				restoreDefaultCursor();
			}
		}

		if (rowIndex != getSelectedRow() || columnIndex != getSelectedColumn())
			SwingUtilities.invokeLater(new valueChangeEvent(this));
		super.changeSelection(rowIndex, columnIndex, toggle, extend);
	}

	public void addDashTableListener(IDashTableListener l) {
		listenerList.add(IDashTableListener.class, l);
	}

	public void removeDashTableListener(IDashTableListener l) {
		listenerList.remove(IDashTableListener.class, l);
	}

	protected void fireTableChanged() {
		final DashTableEvent e = new DashTableEvent(this);
		IDashTableListener[] listeners = (IDashTableListener[]) listenerList.getListeners(IDashTableListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].tableChanged(e);
		}
	}

	protected void fireValueChanged() {
		final DashTableEvent e = new DashTableEvent(this);
		IDashTableListener[] listeners = (IDashTableListener[]) listenerList.getListeners(IDashTableListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].valueChanged(e);
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopClosing(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopClosing(DesktopEvent e)
		throws DesktopCloseVetoException {
		EditableTableHeader header = (EditableTableHeader) getTableHeader();
		ComboBoxModel model =
			((ButtonComboBox) header.getHeaderEditor(0)).getModel();
		List editorList = new ArrayList();
		editorList.add(model.getSelectedItem());
		for (int i = 0; i < model.getSize(); i++) {
			Object item = model.getElementAt(i);
			if (!editorList.contains(item))
				editorList.add(model.getElementAt(i));
		}
		getPatientListSources().writeConfiguration(editorList);
	}

	private PatientListSources getPatientListSources() {
		if (sources == null)
			sources = new PatientListSources(runtime);
		return sources;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopOpening(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopOpening(DesktopEvent e) {
	}

	public void perspectiveActivated(PerspectiveEvent e) {
		// NOOP
	}

	public void perspectiveClosed(PerspectiveEvent e) {
		// NOOP
	}

	public void perspectiveOpened(PerspectiveEvent e) {
		if (!e.getPerspective().getDescriptor().getId().equals("dashboard.nurseDashboard")) return;
		if (currentPatientList != null) refreshPatientList(currentPatientList, false);
	}

	public void perspectiveClosing(PerspectiveEvent e) {
		// NOOP
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		// NOOP
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashPatients#getInternalModel()
	 */
	public TableModel getInternalModel()
	{
		return internalTableModel;
	}

	public void desktopClosed(DesktopEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}

}
