/*
 * Created on Jul 17, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.ui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.dashboard.PatientList;
import gov.va.med.hds.querytool.types.PatientSelection;
import gov.va.med.hds.querytool.ui.EditorFactoryFactory;
import gov.va.med.hds.querytool.ui.FactoryException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.factory.IEditorFactory;

/**
 * @author VHAISLTUMAD
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class DashboardPatientListEditor implements ListSelectionListener {

	private final String LIST_NAME_PREFIX = "Patients: ";
	private final String LIST_NAME_DEFAULT = "List ";

	private IRuntimeContext runtime;
	private DashboardPatientList parent;
	private List patientLists;
	private JTable table;
	private PatientListsTableModel tableModel;
	private JDialog dialog;
	private JButton deleteButton;
	private JButton editButton;
	private JButton createButton;
	private JButton cancelButton;
	private IEditorFactory editorFactory;
	private ILogger logger;
	private PatientList listCreated;

	public DashboardPatientListEditor(
		IRuntimeContext runtime,
		DashboardPatientList parent,
		List patientLists) {

		this.runtime = runtime;
		this.parent = parent;
		this.patientLists = patientLists;
		this.logger = runtime.getLogger(DashboardPatientListEditor.class);
		init();
	}


	private void init() {
		if (null == editorFactory) {
			try {
				EditorFactoryFactory editorFactoryFactory =
					new EditorFactoryFactory();
				editorFactoryFactory.setRuntime(runtime);
				editorFactory =
					editorFactoryFactory.getFactory(
						PatientSelection.class.getName());
			} catch (FactoryException ex) {
				logger.error(ex.getLocalizedMessage(), ex);
			}
		}

		tableModel = new PatientListsTableModel(patientLists);
		listCreated = null;
	}

	public PatientList getList() {
		if (tableModel.getRowCount() == 0)
			createNewList();
		else
			showCreateEditDialog();

		return listCreated;
	}

	private void showCreateEditDialog() {
		table = createJTable();
		JScrollPane sp = new JScrollPane(table);
		Dimension d = new Dimension(400, 200);
		sp.setMinimumSize(d);
		sp.setPreferredSize(d);
		Object[] options = createButtons();
		table.getSelectionModel().setSelectionInterval(0, 0);

		JOptionPane pane =
			new JOptionPane(
				sp,
				JOptionPane.PLAIN_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null,
				options,
				createButton);
		dialog =
			pane.createDialog(
				parent.getRootPane(),
				"Create/Edit Patient List");
		dialog.show();
	}


    private JTable createJTable() {
        JTable result = new JTable(tableModel);
        result.setRowSelectionAllowed(true);
        result.setColumnSelectionAllowed(false);
        result.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        result.getColumnModel().getColumn(0).setWidth(100);
        result.getColumnModel().getColumn(1).setWidth(300);
        result.getSelectionModel().addListSelectionListener(this);
        return result;
    }
    

	private Object[] createButtons() {
		Object[] result = new Object[4];

		editButton = new JButton("Edit List");
		editButton.setMnemonic('E');
		editButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editSelectedList();
			}
		});
		result[0] = editButton;

		deleteButton = new JButton("Delete List");
		deleteButton.setMnemonic('D');
		deleteButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				confirmDeleteSelectedList();
			}
		});
		result[1] = deleteButton;

		createButton = new JButton("New List");
		createButton.setMnemonic('N');
		createButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				createNewList();
			}
		});
		result[2] = createButton;

		cancelButton = new JButton("Close");
		cancelButton.setMnemonic('C');
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancel();
			}
		});
		result[3] = cancelButton;

		return result;
	}

	private void editSelectedList() {
		PatientList list = getSelectedList();
		if (list == null)
			return;
		editPatientList(list);
		tableModel.update(list);
	}

	private void confirmDeleteSelectedList() {
		PatientList list = getSelectedList();
		if (list == null)
			return;

		String message =
			"Are you certain you would like to delete the patient list, '"
				+ list
				+ "'?";

		if (JOptionPane.YES_OPTION
			== JOptionPane.showConfirmDialog(
				getParent(),
				message,
				"Delete List?",
				JOptionPane.YES_NO_OPTION))
			deleteList(list);
	}

	private void deleteList(PatientList list) {
		patientLists.remove(list);
		tableModel.delete(list);
	}

	private void createNewList() {
		PatientList result = createNewPatientList();
		if (result != null) {
			patientLists.add(0, result);
    		if (dialog != null)
    			dialog.dispose();
        }
	}

	private void cancel() {
		dialog.dispose();
	}

    /** Called when the list selection changes 
     */
    public void valueChanged(ListSelectionEvent e) {
        enableButtons();
    }

    private void enableButtons() {
        boolean enableButtons = (getSelectedList() != null);
        editButton.setEnabled(enableButtons);
        deleteButton.setEnabled(enableButtons);
    }

    private Component getParent() {
        if (dialog != null)
            return dialog;
        return parent.getRootPane();
    }

    private PatientList getSelectedList() {
        int row = table.getSelectedRow();
        if (row < 0 || row >= tableModel.getRowCount())
            return null;
        else
            return (PatientList) tableModel.getValueAt(row, 0);
    }


	private void editPatientList(PatientList list) {
		try {
			IRunnableEditorPanel editor = editorFactory.getEditor();
			editor.setValues(list.listIDs);
			editor.init();

			if (JOptionPane
				.showConfirmDialog(
					getParent(),
					editor,
					editor.getWindowHeader(),
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.PLAIN_MESSAGE)
				!= JOptionPane.OK_OPTION)
				return;

			if (editor.getValues().size() == 0)
				return;

			list.listIDs = editor.getValues();

		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}

	private PatientList createNewPatientList() {
		try {
			PatientSelection patientSelect = new PatientSelection();
			IRunnableEditorPanel editor = editorFactory.getEditor();
			editor.setValues(patientSelect.getValues());
			editor.init();

			if (JOptionPane
				.showConfirmDialog(
					getParent(),
					editor,
					editor.getWindowHeader(),
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.PLAIN_MESSAGE)
				!= JOptionPane.OK_OPTION)
				return null;

			if (editor.getValues().size() == 0)
				return null;

			String defaultName = getUniqueListName(editor.getDisplayText());
			JTextField listName = new JTextField(defaultName);
			listName.selectAll();
			JLabel label = new JLabel("Enter a name for the new list: ");
			label.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
			label.setHorizontalAlignment(SwingConstants.LEFT);
			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
			panel.add(label, null);
			panel.add(listName, null);
			JOptionPane getName =
				new JOptionPane(
					panel,
					JOptionPane.PLAIN_MESSAGE,
					JOptionPane.DEFAULT_OPTION);
			getName.createDialog(getParent(), "Patient List Name").show();
			listCreated =
				new PatientList(
					LIST_NAME_PREFIX + listName.getText().trim(),
					editor.getValues());
			return listCreated;

		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return null;
		}
	}


    private String getListDefinition(PatientList list) {
        try {
            IRunnableEditorPanel editor = editorFactory.getEditor();
            editor.setValues(list.listIDs);
            editor.init();
            return editor.getDisplayText();
        } catch (Exception e) {
            logger.error(e.getLocalizedMessage(), e);
            return null;
        }
    }
    

	private String getUniqueListName(String display) {
		if (display != null) {
			display = display.trim();
			if (display.length() > 0 && display.length() < 20)
				return display;
		}

		int num = 1;
		NUM : while (true) {
			String suffix = " " + num;
			Iterator i = patientLists.iterator();
			while (i.hasNext()) {
				PatientList list = (PatientList) i.next();
				if (list.listName.endsWith(suffix)) {
					num++;
					continue NUM;
				}
			}
			break;
		}

		return LIST_NAME_DEFAULT + num;
	}
    

    private class PatientListsTableModel extends AbstractTableModel {

        List data;
        List definitions;

        public PatientListsTableModel(List patientLists) {
            data = new ArrayList();
            definitions = new ArrayList();
            Iterator i = patientLists.iterator();
            while (i.hasNext()) {
                PatientList list = (PatientList) i.next();
                if (list.isNull())
                    continue; // don't include the "Create/Edit" option
                if (list.listIDs.isEmpty())
                    continue; // don't include the "Dynamic" list
                if (DashboardPatientList
                    .CPRS_DEFAULT_LIST_DEF
                    .equals(list.listIDs))
                    continue; // don't include the CPRS default list

                data.add(list);
                definitions.add(getListDefinition(list));
            }
        }

        public void delete(Object o) {
            int pos = data.indexOf(o);
            if (pos != -1) {
                data.remove(pos);
                definitions.remove(pos);
                fireTableRowsDeleted(pos, pos);
            }
        }

        public void update(PatientList list) {
            int pos = data.indexOf(list);
            if (pos != -1) {
                definitions.set(pos, getListDefinition(list));
                fireTableRowsUpdated(pos, pos);
            }
        }

        public int getRowCount() {
            return data.size();
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int columnIndex) {
            switch (columnIndex) {
                case 0 :
                    return "List Name";
                case 1 :
                    return "Definition";
            }
            return null;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
                case 0 :
                    return data.get(rowIndex);
                case 1 :
                    return definitions.get(rowIndex);
            }
            return null;
        }

    }


}
