package gov.va.med.hds.dashboard.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.accessibility.AccessibleContext;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import javax.swing.table.DefaultTableCellRenderer;

import gov.va.med.hds.dashboard.DashboardIconView;


/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class IndicatorRendererX extends DefaultTableCellRenderer {
	public static final String NORMAL_INDICATOR_ICON 				= "Blue Rnd Out LF2.png";
	public static final String NORMAL_SELECTED_INDICATOR_ICON 		= "Blue Rnd In LF3.png";
	public static final String NORMAL_ACKNOWLEDGED_INDICATOR_ICON 	= "Gray Round In LF.png";
	public static final String CRITICAL_INDICATOR_ICON 				= "Red Sq Out LF2.png";
	public static final String CRITICAL_SELECTED_ICON 				= "Red Sq In LF.png";
	public static final String CRITICAL_ACKNOWLEDGED_INDICATOR_ICON 	= "Gray Sq In LF.png";

	public static final Color EVEN_COLOR = Color.white;
	public static final Color ODD_COLOR = new Color(226, 233, 246);
	public static final Color DARK_BLUE = new Color(52, 88, 154);
    public static Color GRAY = Color.gray;    

	private ImageIcon normalIcon;
	private ImageIcon selectedNormalIcon;
	private ImageIcon acknowledgedNormalIcon;
	private ImageIcon criticalIcon;
	private ImageIcon selectedCriticalIcon;
	private ImageIcon acknowledgedCriticalIcon;  
    private boolean hasFocus;

	public IndicatorRendererX() {
		super();
		setHorizontalAlignment(CENTER);
		initIcons();
	}

	private void initIcons() {
		normalIcon = getIcon(NORMAL_INDICATOR_ICON);
		selectedNormalIcon = getIcon(NORMAL_SELECTED_INDICATOR_ICON);
		acknowledgedNormalIcon = getIcon(NORMAL_ACKNOWLEDGED_INDICATOR_ICON);

		criticalIcon = getIcon(CRITICAL_INDICATOR_ICON);
		selectedCriticalIcon = getIcon(CRITICAL_SELECTED_ICON);
		acknowledgedCriticalIcon = getIcon(CRITICAL_ACKNOWLEDGED_INDICATOR_ICON);
	}
    
    private ImageIcon getIcon(String filename) {
        return new ImageIcon(IndicatorRendererX.class.getResource(filename));
    }

	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {
        this.hasFocus = hasFocus;
		JLabel result = this;
        setColors(table, isSelected, hasFocus, row, result);
        if (value instanceof DashboardIconView) {
            DashboardIconView iconValue = (DashboardIconView) value;
            setIcon(result, isSelected, iconValue);
            setTooltip(result, iconValue);            
        } else {
            result.setToolTipText(null);
            result.setIcon(null);
        }
        return result;
	}

    public void paint(Graphics g) {
      super.paint(g);
    
      if (hasFocus) {
          g.setColor(getForeground());
          BasicGraphicsUtils.drawDashedRect(g, 0, 0, getWidth(), getHeight());
      }
    }

	private void setIcon(JLabel result, boolean isSelected, DashboardIconView iconValue) {
		Icon icon = null;

		if (iconValue.isCritical()) {
			if (iconValue.isAcknowledged()) {
				icon = acknowledgedCriticalIcon;
			} else if (isSelected) {
				icon = selectedCriticalIcon;
			} else {
				icon = criticalIcon;
			}
		} else if (iconValue.isNormal()) {
			if (iconValue.isAcknowledged()) {
				icon = acknowledgedNormalIcon;
			} else if (isSelected) {
				icon = selectedNormalIcon;
			} else {
				icon = normalIcon;
			}
		}

		result.setIcon(icon);
	}

	private void setTooltip(JLabel result, DashboardIconView iconValue) {
		String toolTip =
			Integer.toString(iconValue.getItemCount())
				+ " "
				+ iconValue.getItemType()
				+ (iconValue.getItemCount() != 1 ? "s" : "")
				+ (iconValue.isCritical()
					? " (Critical)"
					: "");
		AccessibleContext ac = result.getAccessibleContext();
		ac.setAccessibleDescription(toolTip);
		ac.setAccessibleName(iconValue.getItemType());
		result.setToolTipText(toolTip);
	}

	private boolean isRowOdd(int row) {
		return (row & 1) == 1;
	}

    private void setColors(JTable table, boolean isSelected, boolean hasFocus, int row, JLabel result) {
      if (isSelected) {
        result.setForeground(table.getSelectionForeground());      
        result.setBackground(hasFocus ? table.getSelectionBackground() : IndicatorRendererX.GRAY);
      } else {
        result.setForeground(table.getForeground());
        result.setBackground(isRowOdd(row) ? ODD_COLOR : EVEN_COLOR);
      }
    }
}
