package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.config.RelativeDateSpinnerFieldEditor;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class NurseEventDatesPreferencePage extends PreferencePage {

	private String name;
	private RelativeDateSpinnerFieldEditor minEventsDate;
	private RelativeDateSpinnerFieldEditor maxEventsDate;

	public NurseEventDatesPreferencePage() {

		setBorder(new EmptyBorder(5, 5, 5 , 5));

		addFieldEditor(new BlankFieldEditor("Date Range for Events:", 0, 10));

		minEventsDate =
			new RelativeDateSpinnerFieldEditor("Start:", -180, 0);
		minEventsDate.setBorder(new EmptyBorder(0, 0, 10, 0));

		addFieldEditor(minEventsDate);

		maxEventsDate =
			new RelativeDateSpinnerFieldEditor("Stop:", -30, 30);

		addFieldEditor(maxEventsDate);

		addFieldEditor(
			new BlankFieldEditor(BlankFieldEditor.BOTTOM_SEPARATOR, 10));

		int padTo = 30 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IPreferencePage#initializeFieldEditors()
	 */
	public void initializeFieldEditors() {
		try {
			minEventsDate.setParameter(
				configuration.getParameter("Events Min Date"));
			minEventsDate.loadInitialValue();
			maxEventsDate.setParameter(
				configuration.getParameter("Events Max Date"));
			maxEventsDate.loadInitialValue();
		} catch (NonExistentParameterException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see java.awt.Component#setName(String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @see java.awt.Component#getName()
	 */
	public String getName() {
		return name;
	}

}
