package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.config.RelativeDateSpinnerFieldEditor;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

/**
 * Allows the setting of order parameter dates for the Nursing dashboard.
 **/
public class NurseOrderDatesPreferencePage extends PreferencePage {

	private String name;
	private RelativeDateSpinnerFieldEditor minUnverifiedDate;
	private RelativeDateSpinnerFieldEditor maxUnverifiedDate;
	private RelativeDateSpinnerFieldEditor minTextOrderDate;
	private RelativeDateSpinnerFieldEditor maxTextOrderDate;

	public NurseOrderDatesPreferencePage() {

		setBorder(new EmptyBorder(5, 5, 5 , 5));

		addFieldEditor(
			new BlankFieldEditor("Date Range for Unverified Orders:", 0, 10));

		minUnverifiedDate =
			new RelativeDateSpinnerFieldEditor("Start:", -180, 0);
		minUnverifiedDate.setBorder(new EmptyBorder(0, 0, 10, 0));

		addFieldEditor(minUnverifiedDate);

		maxUnverifiedDate = new RelativeDateSpinnerFieldEditor("Stop:", -30, 30);

		addFieldEditor(maxUnverifiedDate);

		addFieldEditor(
			new BlankFieldEditor(BlankFieldEditor.BOTTOM_SEPARATOR, 10));

		addFieldEditor(new BlankFieldEditor("Date Range for Text Orders:", 10, 10));

		minTextOrderDate =
			new RelativeDateSpinnerFieldEditor("Start:", -365, 0);
		minTextOrderDate.setBorder(new EmptyBorder(0, 0, 10, 0));

		addFieldEditor(minTextOrderDate);

		maxTextOrderDate =
			new RelativeDateSpinnerFieldEditor("Stop:", -30, 180);

		addFieldEditor(maxTextOrderDate);

		addFieldEditor(
			new BlankFieldEditor(BlankFieldEditor.BOTTOM_SEPARATOR, 10));

		int padTo = 30 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	public void initializeFieldEditors() {
		try {
			minUnverifiedDate.setParameter(
				configuration.getParameter("Unverified Min Date"));
			minUnverifiedDate.loadInitialValue();
			maxUnverifiedDate.setParameter(
				configuration.getParameter("Unverified Max Date"));
			maxUnverifiedDate.loadInitialValue();
			minTextOrderDate.setParameter(
				configuration.getParameter("Text Order Min Date"));
			minTextOrderDate.loadInitialValue();
			maxTextOrderDate.setParameter(
				configuration.getParameter("Text Order Max Date"));
			maxTextOrderDate.loadInitialValue();
		} catch (NonExistentParameterException e) {
			e.printStackTrace();
		}
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

}
