package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.config.RelativeDateSpinnerFieldEditor;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class NurseResultDatesPreferencePage extends PreferencePage {

	private String name;
	private RelativeDateSpinnerFieldEditor minResultsDate;
	private RelativeDateSpinnerFieldEditor maxResultsDate;

	public NurseResultDatesPreferencePage() {

		setBorder(new EmptyBorder(5, 5, 5 , 5));

		addFieldEditor(new BlankFieldEditor("Date Range for Results:", 0, 10));

		minResultsDate =
			new RelativeDateSpinnerFieldEditor("Start:", -180, 0);
		minResultsDate.setBorder(new EmptyBorder(0, 0, 10, 0));

		addFieldEditor(minResultsDate);

		maxResultsDate =
			new RelativeDateSpinnerFieldEditor("Stop:", -30, 30);

		addFieldEditor(maxResultsDate);

		addFieldEditor(
			new BlankFieldEditor(BlankFieldEditor.BOTTOM_SEPARATOR, 10));

		int padTo = 30 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IPreferencePage#initializeFieldEditors()
	 */
	public void initializeFieldEditors() {
		try {
			minResultsDate.setParameter(
				configuration.getParameter("Results Min Date"));
			minResultsDate.loadInitialValue();
			maxResultsDate.setParameter(
				configuration.getParameter("Results Max Date"));
			maxResultsDate.loadInitialValue();
		} catch (NonExistentParameterException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see java.awt.Component#setName(String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @see java.awt.Component#getName()
	 */
	public String getName() {
		return name;
	}

}
