package gov.va.med.hds.dashboard.ui;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.config.RelativeDateSpinnerFieldEditor;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

/**
 * Allows the user setting of vital start and stop dates for the Nursing dashboard.
 **/
public class NurseVitalsDatesPreferencePage extends PreferencePage {

	private String name;
	private RelativeDateSpinnerFieldEditor minVitalsDate;
	private RelativeDateSpinnerFieldEditor maxVitalsDate;

	public NurseVitalsDatesPreferencePage() {

		setBorder(new EmptyBorder(5, 5, 5 , 5));

		addFieldEditor(new BlankFieldEditor("Date Range for Vitals:", 0, 10));

		minVitalsDate =
			new RelativeDateSpinnerFieldEditor("Start:", -180, 0);
		minVitalsDate.setBorder(new EmptyBorder(0, 0, 10, 0));

		addFieldEditor(minVitalsDate);

		maxVitalsDate =
			new RelativeDateSpinnerFieldEditor("Stop:", -30, 0);

		addFieldEditor(maxVitalsDate);

		addFieldEditor(
			new BlankFieldEditor(BlankFieldEditor.BOTTOM_SEPARATOR, 10));

		int padTo = 30 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	public void initializeFieldEditors() {
		try {
			minVitalsDate.setParameter(
				configuration.getParameter("Vitals Min Date"));
			minVitalsDate.loadInitialValue();
			maxVitalsDate.setParameter(
				configuration.getParameter("Vitals Max Date"));
			maxVitalsDate.loadInitialValue();
		} catch (NonExistentParameterException e) {
			e.printStackTrace();
		}
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

}
