/*
 * Created on Apr 18, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.accessibility.AccessibleContext;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class PatientNameRendererX extends DefaultTableCellRenderer
{
  private static final Border border = BorderFactory.createEmptyBorder(0,3,0,0);
  
  private boolean hasFocus;
  
  public PatientNameRendererX() {
    super();
    setBorder(border);
  }

  public Component getTableCellRendererComponent(
    JTable table,
    Object value,
    boolean isSelected,
    boolean hasFocus,
    int row,
    int column) {
    this.hasFocus = hasFocus;
    JLabel result = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
    setAccessibiltyProperties(value, result);
    setColors(table, isSelected, hasFocus, row, result);
    result.setBorder(border);
    return result;
  }

  private void setColors(JTable table, boolean isSelected, boolean hasFocus, int row, JLabel result) {
    if (isSelected) {
      result.setForeground(table.getSelectionForeground());      
      result.setBackground(hasFocus ? table.getSelectionBackground() : IndicatorRendererX.GRAY);
    } else {
      result.setForeground(table.getForeground());
      result.setBackground(isRowOdd(row) ? IndicatorRendererX.ODD_COLOR : IndicatorRendererX.EVEN_COLOR);
    }
  }

  private void setAccessibiltyProperties(Object value, JLabel result) {
    AccessibleContext ac = result.getAccessibleContext();
    if (null != ac) {
      if (null != value)
        ac.setAccessibleDescription("Patient is " + value.toString());
      ac.setAccessibleName("Patient Name");
    }
  }

  public void paint(Graphics g) {
    super.paint(g);
    
    int w = getWidth();
    int h = getHeight();
    
    if (hasFocus) {
        g.setColor(getForeground());
        BasicGraphicsUtils.drawDashedRect(g, 1, 0, w-1, h);
    }
    
    g.setColor(Color.black);
    g.drawLine(0, 0, 0, h);
  }
  
  private boolean isRowOdd(int row) {
    return (row & 1) == 1;
  }
}
