package gov.va.med.hds.dashboard.ui.scriptlet;

import javax.swing.JToggleButton;

import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.HTML;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.logging.ILogger;

import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptlet;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptletEvent;


import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;

/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public class Acknowledge implements IScriptlet
{
  private IRuntimeContext runtime;
  private ILogger logger;
  private IDashboardItemTypes itemTypes;
  /**
   * Key for simple acknowledging items scriptlet.  The item type must be appended
   * to this key.
   */
  public static final String CHECKBOX_SCRIPTLET_KEY = "gov.va.med.hds.dashboard.CheckBoxScriptlet.";
  
  /*
   * Public scriptlet methods
   */
  public static final String ACKNOWLEDGE = "acknowledge";
  public static final String ACKNOWLEDGE_ALL = "acknowledgeAll";   
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptlet#init(IScriptletEvent)
	 */
	public void init(IScriptletEvent event)
	{
    this.runtime = (IRuntimeContext)event.getEnvironment();
    logger = runtime.getLogger(this.getClass());
    
    try
    {
      itemTypes = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
    }
    catch (PluginStartupException pse)
    {
      logger.error(pse.getLocalizedMessage(), pse);
    }
    catch (PluginNotFoundException pfe)
    {
      logger.error(pfe.getLocalizedMessage(), pfe);
    }
    if (IScriptletEvent.SUBDOCUMENT_LOADED == event.getType())
    {
      AttributeSet attrs = event.getElement().getAttributes();
      String id = (String)attrs.getAttribute(HTML.Attribute.ID);
      String type = (String)attrs.getAttribute("itemtype");
      boolean autoAck = new Boolean((String)attrs.getAttribute("autoack")).booleanValue();
      if (autoAck)
        acknowledge(event, id, type);
    }
  }
  
  public void acknowledge(IScriptletEvent event, String id, String itemType)
  {
    String itemId = id.split("_")[1];
    Element element = event.getElement();
    AttributeSet attrs = element.getAttributes();
    IDashboardItemDAO itemsDAO = getItemDAO(event, itemType);

    if (null != itemsDAO)
    {
      // The acknowledge state is defaulted to true for those items
      // that are acknowledged by being viewed, i.e. they have no checkbox    
      boolean ackState = true;
      // If this is a checkbox then get the
      // current state of the checkbox
      if ("input".equalsIgnoreCase(element.getName()) &&
          "checkbox".equalsIgnoreCase((String)attrs.getAttribute(HTML.Attribute.TYPE)))
      {
        JToggleButton.ToggleButtonModel model =
          (JToggleButton.ToggleButtonModel) attrs.getAttribute(StyleConstants.ModelAttribute);
        ackState = model.isSelected();
      }
      // Just getting one item
      IDashboardItem item = (IDashboardItem)itemsDAO.getItem(itemId);
      item.setComplete(ackState);
      item.setViewed(true);
    }
  }

  public void acknowledgeAll(IScriptletEvent event)
  {
    // Get the current state of the checkbox
    Element allCkBox = event.getElement();
    AttributeSet allAttrs = allCkBox.getAttributes();
    JToggleButton.ToggleButtonModel allModel =
      (JToggleButton.ToggleButtonModel) allAttrs.getAttribute(StyleConstants.ModelAttribute);

    setCheckboxes(event.getDocument().getDefaultRootElement(), allModel.isSelected());
  }
  
  protected void setCheckboxes(Element node, boolean state)
  {
    if (node.isLeaf())
    {
      String name = node.getName();
      if (name.equalsIgnoreCase("input"))
      {
        AttributeSet attrs = node.getAttributes();
        if ("checkbox".equalsIgnoreCase((String)attrs.getAttribute(HTML.Attribute.TYPE)))
        {
          JToggleButton.ToggleButtonModel model = (JToggleButton.ToggleButtonModel) attrs.getAttribute(StyleConstants.ModelAttribute);
          model.setSelected(state);
        }
      }
    }
    else
    {
      for (int i=0; i<node.getElementCount(); i++)
        setCheckboxes(node.getElement(i), state);
    }
  }

  private IDashboardItemDAO getItemDAO(IScriptletEvent event, String itemType)
  {
    if (null != itemTypes)
      try
      {
        return (IDashboardItemDAO)itemTypes.getItemType(itemType).getItemDAOInstance();
      }
      catch (Exception e)
      {
        logger.error(e.getMessage(), e);
      }
    return null;
  }  
}
