package gov.va.med.hds.dashboard.ui.servlet;
import gov.va.med.hds.cd.ccow.IContextModule;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.task.ITaskPlugin;

import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public abstract class AbstractItemDetail implements IServlet
{
  protected IRuntimeContext runtime;
  protected ServletWriter out;
  protected static ITaskPlugin taskPlugin = null;
  protected static IContextModule contextPlugin = null;
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.url.IServlet#service(IServletRequest, IServletResponse)
	 */
	abstract public void service(IServletRequest request, IServletResponse response) throws IOException;
    
  protected void init(IRuntimeContext runtime) {
    this.runtime = runtime;
    
    if (taskPlugin == null) 
      try {
        taskPlugin = (ITaskPlugin) runtime.getPluginInstance(ITaskPlugin.PLUGIN_ID);
      } catch (Exception e) {
      }

    if (contextPlugin == null) 
      try {
          contextPlugin = (IContextModule) runtime.getPluginInstance(IContextModule.PLUGIN_ID);
      } catch (Exception e) {
      }
  }

  public String htmlHeader(String[] scriptlets)
  {
    StringBuffer html = new StringBuffer();
    for (int s=0; s<scriptlets.length; s++)
      html.append("<meta name='scriptlet' content='" + scriptlets[s] + "'>");
    html.append("<link rel='stylesheet' type='text/css' href='dashboard.css'>");
    return html.toString();
  }
  
  public String htmlBody(String[] ids, boolean brief, boolean autoack, IDashboardItemDAO itemDAO, boolean display)
  {
    StringBuffer html = new StringBuffer();
    List items = itemDAO.getItems(Arrays.asList(ids));
    for (Iterator iter = items.iterator(); iter.hasNext();)
    {
      IDashboardItem item = (IDashboardItem)iter.next();
      String itemID = item.getID();
      html.append("<div class='resultbox'>");
      itemHeader(item, (brief && display), html);
      html.append("<div " + (display ? "" : "class='displaynone'") +
        " id='" + itemID + "'>");
      html.append("<div>");
      itemDetail(item, brief, autoack, true, html);
      html.append("</div></div>");
      if (!brief)
        itemFooter(item, html);
      html.append("</div>");
    }
    if (!brief && 0 < ids.length)
      pageFooter(html);
    return html.toString();
  }

  public void itemHeader(IDashboardItem item, boolean hideCollapser, StringBuffer html)
  {
    String itemID = item.getID();
    html.append("<table class='resultheader' cellpadding='1' cellspacing='1'><tr>");
    if (!hideCollapser)
      html.append("<td valign='top' width='1%'><collapser forID='" + itemID +"'></td>");
    html.append("<td valign='top' align='left' style='font-weight:bold'>" + formatTitle(item) + "</td>");
    html.append("<td valign='top' align='right'>" + asHTML(item.getFormattedDate()) + "</td></tr></table>");
  }
  
  public void itemDetail(IDashboardItem item, boolean brief, boolean autoack, boolean hasPermission, StringBuffer html)
  {
    html.append("<pre style='padding-left:10;'>");
    html.append(asHTML(item.getData().toString()));
    html.append("</pre>");
    if (!brief)
      html.append("<div style='display:none'" +  (hasPermission ? " enable='true'" : "") +
        " autoack='" + autoack + "' deferredsrc='blank.html'" +  
        " itemtype='" + item.getTypeID() + "' id='d_" + item.getID() + "'></div>");
  }
  
  public void itemComments(List comments, StringBuffer html)
  {
    Iterator iter = comments.iterator();
    if (iter.hasNext())
    {
      html.append("<hr width='100%'>&nbsp;&nbsp;<b>Comments:</b><pre style='margin:0 0 0 10;'>");
      while (iter.hasNext())
      {
        String comment = asHTML((String)iter.next());
        html.append(comment + "<br>");
      }
      html.append("</pre>");
    }
  }

  public void itemFooter(IDashboardItem item, StringBuffer html)
  {
    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    html.append("<a href=''>Link Task</a>&nbsp;&nbsp;&nbsp;");
    html.append("<a href=''>Go to Chart</a>&nbsp;&nbsp;&nbsp;");
    html.append("</td>");
    html.append("</tr></table>");
  }
  
  public void pageFooter(StringBuffer html)
  {
  }
  
  protected String asHTML(String src)
  {
    return out.escapeEntities(src, false);
  }
  
  protected String formatTitle(IDashboardItem item)
  {
    String title = item.getTitle();
    if (title.equals(item.getShortTitle()))
    {
      int titleLen = Math.min(65, title.length());
      return asHTML(title.substring(0, titleLen) + (titleLen < title.length() ? "..." : ""));
    }
    else
      return asHTML(title).replaceAll("\n", "<br>");
  }
  
}
