/*
 * Created on Jun 9, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardNotificationsDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardNotificationItem;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class NotificationDetail extends AbstractItemDetail
{
  IDashboardNotificationsDAO dao = null;
  String patientID;
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.url.IServlet#service(gov.va.med.hds.cd.url.IServletRequest, gov.va.med.hds.cd.url.IServletResponse)
	 */
	public void service(IServletRequest request, IServletResponse response) throws IOException
	{
    init(request.getRuntimeContext());
    patientID = request.getParameter("patientID");
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    if (request.getParameter("display") != null)
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardNotificationItem.ITEM_TYPE_ID);
        dao = (IDashboardNotificationsDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window"
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, true, dao, display));
    out.write("</body></html>");
	}
	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#htmlBody(java.lang.String[], boolean, boolean, gov.va.med.hds.dashboard.dao.IDashboardItemDAO, boolean)
	 */
	public String htmlBody(String[] ids, boolean brief, boolean autoack, IDashboardItemDAO itemDAO, boolean display)
	{
    StringBuffer html = new StringBuffer();
    if (0 < ids.length)
    {
      List items = itemDAO.getItems(Arrays.asList(ids));
      html.append("<div class='resultbox'>");
      itemHeader(null, brief, html);
      html.append("<div " + (display ? "" : "class='displaynone'") +
        " id='NOT:" + patientID + "'>");
      html.append("<div><table width='75%' cellpadding='1' cellspacing='2'>");
      for (Iterator iter = items.iterator(); iter.hasNext();)
        itemDetail((IDashboardItem)iter.next(), brief, autoack, true, html);
      html.append("</table></div></div>");
      if (!brief)
        itemFooter(null, html);
      html.append("</div>");
    }
    else
      html.append("&nbsp;");
    return html.toString();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemDetail(gov.va.med.hds.dashboard.items.IDashboardItem, boolean, boolean, boolean, java.lang.StringBuffer)
	 */
	public void itemDetail(IDashboardItem item, boolean brief, boolean autoack, boolean hasPermission, StringBuffer html)
	{
    html.append("<tr><td>&nbsp;" + item.getTitle() +
      "</td><td align=right>" + item.getFormattedDate() + "</td></tr>");
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemHeader(gov.va.med.hds.dashboard.items.IDashboardItem, boolean, java.lang.StringBuffer)
	 */
	public void itemHeader(IDashboardItem item, boolean hideCollapser, StringBuffer html)
	{
    html.append("<table class='resultheader' cellpadding='1' cellspacing='1'><tr>");
    if (!hideCollapser)
      html.append("<td valign='top' width='1%'><collapser forID='NOT:" + patientID + "'></td>");
    html.append("<td valign='top' align='left' style='font-weight:bold'>Other Notifications</td>");
    html.append("</tr></table>");
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemFooter(gov.va.med.hds.dashboard.items.IDashboardItem, java.lang.StringBuffer)
	 */
	public void itemFooter(IDashboardItem item, StringBuffer html)
	{
    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
//        html.append("<a href=\"");    
//        html.append(Link.getLinkToTaskURL("Other Notifications", "Notification", patientID, patientID));
//        html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>&nbsp;&nbsp;&nbsp;");
        }
    }
    html.append("</td></tr></table>");
	}

}
