package gov.va.med.hds.dashboard.ui.servlet;

import gov.va.med.hds.cd.runtime.IRuntimeContext;

import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;

import gov.va.med.hds.dashboard.dao.IPatientListDAO;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;

import java.io.IOException;

/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public class PatientDetail implements IServlet
{
  IRuntimeContext runtime;
  ServletWriter out;
  IPatientListDAO patientDAO;
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.url.IServlet#service(IServletRequest, IServletResponse)
	 */
	public void service(IServletRequest request, IServletResponse response) throws IOException
	{
    runtime = request.getRuntimeContext();
    String patientId = request.getParameter("patientID");
    String itemType = request.getParameter("itemtype");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    if (null == patientDAO)
      try
      {
        IDashboardItemTypes types = (IDashboardItemTypes)runtime
          .getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        patientDAO = (IPatientListDAO)types
          .getItemType(itemType).getItemDAOInstance();
      }
      catch (Exception e)
      {
        throw new IOException(e.toString());
      }
    out = response.getWriter();
    response.setContentType("text/html");

    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.cd.ccow.ui.Hyperlink"
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(patientId, brief));
    out.write("</body></html>");
  }
  
  public String htmlHeader(String[] scriptlets)
  {
    StringBuffer html = new StringBuffer();
    for (int s=0; s<scriptlets.length; s++)
      html.append("<meta name='scriptlet' content='" + scriptlets[s] + "'>");
    html.append("<link rel='stylesheet' type='text/css' href='dashboard.css'>");
    return html.toString();
  }
  
  public String htmlBody(String patientId, boolean brief)
  {
    StringBuffer html = new StringBuffer();
    html.append("<pre>");
    html.append(out.escapeEntities(patientDAO.getPatientDemographics(patientId).toString(), false));
    html.append("</pre>");
    return html.toString();
  }

}
