/**
 * Created on Jan 16, 2003
 *
 * To change this generated comment edit the template variable "filecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of file comments go to
 * Window>Preferences>Java>Code Generation.
 */
package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;
import java.util.Vector;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.dao.IDashboardResultDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardResultItem;
import gov.va.med.hds.dashboard.itemtype.IChemistryLabResult;
import gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail;


/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class ResultDetail extends AbstractItemDetail
{
  private IDashboardResultDAO dao = null;
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.url.IServlet#service(IServletRequest, IServletResponse)
	 */
	public void service(IServletRequest request, IServletResponse response)
      throws IOException
	{
    init(request.getRuntimeContext());
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    if (request.getParameter("display") != null)
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardResultItem.ITEM_TYPE_ID);
        dao = (IDashboardResultDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge"
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, false, dao, display));
    out.write("</body></html>");
	}
  
  public void itemDetail(IDashboardItem item, boolean brief, boolean autoack, boolean hasPermission, StringBuffer html)
  {
    DashboardResultItem result = (DashboardResultItem)item;
    switch (result.getResultType())
    {
      case DashboardResultItem.DATA_RESULT :
        dataResult(result, html);
        break;

      case DashboardResultItem.TEXT_RESULT :
        textResult(result, html);
    }
    itemComments(item.getComments(), html);
  }
  
  public void itemFooter(IDashboardItem item, StringBuffer html)
  {
    String itemID = item.getID();
    Object patientID = null;
    try {
        patientID = ((IDashboardPatient)item.getPatient()).getPatientId();
    } catch (Exception e) {}

    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
        if (taskPlugin != null) {
            html.append("<a href=\"");    
            html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "NResult", item.getID(), patientID));
            html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        }
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>&nbsp;&nbsp;&nbsp;");
        }
    }
    html.append("</td>");
    html.append("<td align='right'><input type='checkbox' id='ck_" + 
      itemID + "' onChange=\"acknowledge('ck_" + 
      itemID + "', '" + DashboardResultItem.ITEM_TYPE_ID + "')\"" +
      (item.isComplete() ? " checked" : "") +
      " label='Acknowledge'>&nbsp;</td>");
    html.append("</tr></table>");
  }

  public void pageFooter(StringBuffer html)
  {
    html.append("<table class='appFont' width='98%'><tr><td align='right'>");
    html.append("<input type='checkbox' onChange='acknowledgeAll' label='Acknowledge All Results on this Page'>");
    html.append("</td></tr></table>");
  }

  protected void dataResult(DashboardResultItem result, StringBuffer html)
  {
    Vector labResults = (Vector)result.getData();

    html.append("<table style='margin:0 0 0 8;' cellpadding='0' cellspacing='0'");
    for (int idx=0; idx<labResults.size(); idx++)
    {
      IChemistryLabResult labResult = (IChemistryLabResult)labResults.elementAt(idx);
      boolean isCritical = labResult.isCriticalResult();
      html.append("<tr><td width='202' " + (isCritical ? "style='font-weight:bold;'>" : ">"));
      html.append(asHTML(labResult.getResultName()));
      html.append("</td><td width='68' align='right' " + (isCritical ? "style='font-weight:bold;color:red;'>" : ">"));
      html.append(asHTML(labResult.getResultValue()));
      html.append("</td><td width='23'>&nbsp; ");
      html.append(asHTML(labResult.getCriticalFlag()));
      html.append("</td><td width='64'>");
      html.append(asHTML(labResult.getUnits()));
      html.append("</td><td width='92' align='right'>");
      html.append(asHTML(labResult.getRefRange()));
      html.append("</td></tr>");
    }
    html.append("</table>");
  }
  
  protected void textResult(DashboardResultItem result, StringBuffer html)
  {
    html.append("<pre style='padding-left:10;'>");
    html.append(asHTML(result.getData().toString()));
    html.append("</pre>");
  }
  
  public IDashboardItem getItem(String id)
  {
    return (IDashboardItem)dao.getItem(id);
  }
  
}
