/*
 * Created on Apr 29, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard;

/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 *
 * Defines the list of valid order status for VistA.  Methods of this class will validate
 * a status and determine whether the status indicates that the order is complete. 
 */
public class OrderStatus
{
  private static Status[] statuses = {
    new Status("discontinued", true),
    new Status("complete", true), 
    new Status("hold", false),
    new Status("flagged", false),
    new Status("pending", false),
    new Status("active", false),
    new Status("expired", true),
    new Status("scheduled", false),
    new Status("partial results", false),
    new Status("delayed", false),
    new Status("unreleased", false),
    new Status("discontinued/edit", true),
    new Status("cancelled", true),
    new Status("lapsed", true),
    new Status("renewed", false),
    new Status("no status", false)
  };
  
  /**
   * The default constructor
   *
   */
  public OrderStatus(){}
  
  /**
   * Determines whether the status indicates that the order is complete or not.
   * @param status the status of the order.  A valid status is one of:
   * <ul>
   * <li>discontinued</li>
   * <li>complete</li>
   * <li>hold</li>
   * <li>flagged</li>
   * <li>pending</li>
   * <li>active</li>
   * <li>expired</li>
   * <li>scheduled</li>
   * <li>partial results</li>
   * <li>delayed</li>
   * <li>unreleased</li>
   * <li>discontinued/edit</li>
   * <li>cancelled</li>
   * <li>lapsed</li>
   * <li>renewed</li>
   * <li>no status</li>
   * </ul>
   * @return true if the status indicates the order is complete otherwise false.
   */
  public static boolean isCompleteStatus(String status)
  {
    for (int i = 0; i < statuses.length; i++)
		{
			if (status.equalsIgnoreCase(statuses[i].toString()))
        return statuses[i].isComplete();
		}
    return false;
  }

  /**
   * Determines if the status is valid for the system.
   * @param status the status to test
   * @return true if the status is valid otherwise false.
   */
  public static boolean isValidStatus(String status)
  {
    for (int i = 0; i < statuses.length; i++)
    {
      if (status.equalsIgnoreCase(statuses[i].toString()))
        return true;
    }
    return false;
  }
}

/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 *
 * Encapsulates the notion of a status. 
 */
class Status
{
  String status;
  boolean complete;
    
  /**
   * Constructor for Status
   * @param status - the status
   * @param complete - whether or not the status indicates if the order is complete
   */
  public Status (String status, boolean complete)
  {
    this.status = status;
    this.complete = complete;
  }

  /**
   * Overrides the toString() method of java.lang.Object
   * @return the text of the status
   */
  public String toString()
  {
    return status;
  }

  /**
   * Determines whether the status is a completion status
   * @return true if the status indicates the order is complete otherwise false.
   */
  public boolean isComplete()
  {
    return complete;
  }
}
