/*
 * Created on May 27, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.action;

import java.awt.event.ActionEvent;

import javax.swing.JScrollPane;
import javax.swing.JTable;

import gov.va.med.hds.cd.prism.action.AbstractAction;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;
import gov.va.med.hds.dashboard.IDashboardDetailVisualComponent;
import gov.va.med.hds.dashboard.IDashboardPatientVisualComponent;
import gov.va.med.hds.dashboard.IDashboardPlugin;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public abstract class AbstractDashboardAction extends AbstractAction implements IPerspectiveListener
{
  private String referringPLUGIN_ID = "";
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveActivated(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveActivated(PerspectiveEvent e)
	{
    reenable();
	}
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveClosing(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveClosing(PerspectiveEvent e){}
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveClosed(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveClosed(PerspectiveEvent e){}
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveOpening(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveOpening(PerspectiveEvent e){}
  
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveOpened(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveOpened(PerspectiveEvent e)
	{
    reenable();
	}
  
  protected void reenable() {
    try {
      IDashboardPlugin dashboardPlugin =
        (IDashboardPlugin) runtime.getPluginInstance(
          referringPLUGIN_ID);
      if (dashboardPlugin == null) {
        setEnabled(false);
        return;
      }
      setEnabled(shouldEnable(dashboardPlugin));
    } catch (Exception ex) {
      setEnabled(false);
    }
  }

  protected boolean shouldEnable(
    IDashboardPlugin dashboardPlugin)
    throws Exception {

    return true;
  }

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */

	public void actionPerformed(ActionEvent e)
	{
    try {
      IDashboardPlugin dashboardPlugin =
        (IDashboardPlugin) runtime.getPluginInstance(
          referringPLUGIN_ID);
      actionPerformed(e, dashboardPlugin);
    } catch (Exception ex) {
      runtime.getLogger(getClass()).error("Unable to get reference to Dashboard plugin", ex);
    }
	}

  protected abstract void actionPerformed(
    ActionEvent e,
    IDashboardPlugin dashboardPlugin
  )
    throws Exception;

  protected DHTMLPane getDashPane() {
    IPerspectiveReference perspective = getDesktop().getActiveWindow().getActivePerspective();
    IVisualComponentReference[] components = perspective.getVisualComponents();
    for (int i = 0; i < components.length; i++)
    {
      if (components[i].getVisualComponent() instanceof IDashboardDetailVisualComponent)
      {
        JScrollPane scroller = (JScrollPane)components[i].getVisualComponent().getComponent();
        return (DHTMLPane)scroller.getViewport().getView();
      }
    }
    return null;
  }

  protected JTable getPatientList() {
    IPerspectiveReference perspective = getDesktop().getActiveWindow().getActivePerspective();
    IVisualComponentReference[] components = perspective.getVisualComponents();
    for (int i = 0; i < components.length; i++)
    {
      if (components[i].getVisualComponent() instanceof IDashboardPatientVisualComponent)
      {
        JScrollPane scroller = (JScrollPane)components[i].getVisualComponent().getComponent();
        return (JTable)scroller.getViewport().getView();
      }
    }
    return null;
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data)
		throws Exception
	{
    super.setInitializationData(runtime, config, propertyName, data);
    referringPLUGIN_ID = (String)data;
    getDesktop().getPerspectiveEventSource().addPerspectiveListener(this);
	}

}
