package gov.va.med.hds.dashboard.action;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.net.URL;
import java.net.URLEncoder;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window;
import gov.va.med.hds.dashboard.DashTableEvent;
import gov.va.med.hds.dashboard.IDashPatients;
import gov.va.med.hds.dashboard.IDashTableListener;
import gov.va.med.hds.dashboard.IDashboardPlugin;
import gov.va.med.hds.dashboard.items.IDashboardPatient;

public class CreateTaskAction
	extends AbstractDashboardAction
	implements IDashTableListener, Runnable {
	private static final Integer DEFAULT_MNEMONIC = new Integer(KeyEvent.VK_C);

	public CreateTaskAction() {
		putValue(MNEMONIC_KEY, DEFAULT_MNEMONIC);
	}

	protected boolean shouldEnable(IDashboardPlugin dashboardPlugin)
		throws Exception {
		maybeRegisterForPatientChanges(dashboardPlugin);
		return getCurrentPatient(dashboardPlugin) != null;
	}

	protected void actionPerformed(
		ActionEvent e,
		IDashboardPlugin dashboardPlugin)
		throws Exception {
		IDashboardPatient patient = getCurrentPatient(dashboardPlugin);
		if (null != patient) {
			Object patientID = patient.getPatientId();
			if (patientID == null)
				return;
			String patientName = patient.getPatientName();
			StringBuffer url = new StringBuffer();
			url.append(
				"cdtp:///gov/va/med/hds/task/Display?editTaskID=new&patientID=");
			url.append(URLEncoder.encode(patientID.toString(), "UTF-8"));
			URL u = new URL(url.toString());
			DHTMLPane dashPane = getDashPane();
			Window.openWindow(
				runtime,
				dashPane,
				u,
				"Create Task for " + patientName,
				600,
				200,
				true);
		} else
			JOptionPane.showMessageDialog(
				null,
				"You must first select a patient row.",
				"Invalid Action",
				JOptionPane.INFORMATION_MESSAGE);
	}

	private boolean registeredForPatientChanges = false;
	private final void maybeRegisterForPatientChanges(IDashboardPlugin plugin) {
		if (registeredForPatientChanges)
			return;
		registeredForPatientChanges = true;
	}

	public void run() {
		reenable();
	}

	public IDashboardPatient getCurrentPatient(IDashboardPlugin plugin) {
		JTable patientListTable = getPatientList();
		int row = patientListTable.getSelectedRow();
		int col = patientListTable.getSelectedColumn();
		int realcols = ((IDashPatients) patientListTable).getNumDataCols();
		int patCol = (col / realcols) * realcols;
		return (IDashboardPatient) patientListTable.getValueAt(row, patCol);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashTableListener#tableChanged(gov.va.med.hds.dashboard.DashTableEvent)
	 */
	public void tableChanged(DashTableEvent e) {
	} // Not really interested

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashTableListener#valueChanged(gov.va.med.hds.dashboard.DashTableEvent)
	 */
	public void valueChanged(DashTableEvent e) {
		SwingUtilities.invokeLater(this);
	}

}
