/*
 * Created on Apr 22, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.items;

import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardDefinition implements IDashboardDefinition
{
  private IRuntimeContext runtime;
  private Map dashEntry = Collections.synchronizedMap(new Hashtable());
  
  private String id;

  public DashboardDefinition(IRuntimeContext runtime, IConfigurationElement defItem, IDashboardItemTypes itemTypes)
  {
    this.runtime = runtime;
    
    this.id = defItem.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_ID);
    IConfigurationElement[] dashItem = defItem.getChildren();
    for (int i = 0; i < dashItem.length; i++)
    {
      String dashId = dashItem[i].getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_ID);
      IItemEntry itemEntry = itemTypes.getItemType(dashId);
      dashEntry.put(dashId, new DashboardEntry(runtime, dashItem[i], itemEntry));
    }          
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#getID()
   */
  public String getID()
  {
    return id;
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#getDefinition(String id)
   */
  public IDashboardEntry getDefinition(String id)
  {
    return (IDashboardEntry)dashEntry.get(id);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#iterator()
   */
  public Iterator iterator()
  {
    return dashEntry.values().iterator();
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#getDefinition(int)
   */
  public IDashboardEntry getDefinition(int sequence)
  {
    Iterator iter = iterator();
    while (iter.hasNext())
    {
      IDashboardEntry item = (IDashboardEntry)iter.next();
      if (sequence == item.getSequence())
        return item;
    }
    return null;
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#size()
   */
  public int size()
  {
    return dashEntry.size();
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardDefinition#getDefinitionByType(java.lang.String)
	 */
	public IDashboardEntry getDefinitionByType(String itemType)
	{
    Iterator iter = iterator();
    while (iter.hasNext())
    {
      IDashboardEntry entry = (IDashboardEntry)iter.next();
      if (itemType.equals(entry.getItemEntry().getItemType()))
        return entry;
    }
		return null;
	}

}
