/*
 * Created on Apr 21, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.items;

import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardItemTypes implements IPlugin, IDashboardItemTypes
{
  private IRuntimeContext runtime;
  private Map itemTypeMap = Collections.synchronizedMap(new Hashtable());
  private Map dashboardMap = Collections.synchronizedMap(new Hashtable());
  
  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor)
   */
  public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception
  {

    this.runtime = runtime;
    IPluginRegistry pluginRegistry = runtime.getPluginRegistry();

    itemTypeMap.clear();
    IConfigurationElement[] entryItemExt =
      pluginRegistry.getConfigurationElementsFor(IDashboardItemTypes.DASH_ITEM_EXTENSION);

    for (int i = 0; i < entryItemExt.length; i++)
    {
      ItemEntry entry = new ItemEntry(runtime, entryItemExt[i]);
      itemTypeMap.put(entry.getID(), entry);
    }

    IConfigurationElement[] dashExt =
      pluginRegistry.getConfigurationElementsFor(IDashboardItemTypes.DASH_DEFINITION_EXTENSION);
    
    for (int i = 0; i < dashExt.length; i++)
    {
      String itemID = dashExt[i].getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_ID);
      dashboardMap.put(itemID, new DashboardDefinition(runtime, dashExt[i], this));
    }
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
   */
  public void shutdown() throws Exception
  {
    // Do nothing
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#itemIterator()
   */
  public Iterator itemIterator()
  {
    return itemTypeMap.values().iterator();
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#dashboardIterator(java.lang.String)
   */
  public Iterator dashboardIterator()
  {
    return dashboardMap.values().iterator();
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#getDashboardEntry(java.lang.String)
   */
  public IDashboardDefinition getDashboardDefinition(String id)
  {
    return (IDashboardDefinition)dashboardMap.get(id);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#getItemEntry(java.lang.String)
   */
  public IItemEntry getItemType(String id)
  {
    return (IItemEntry)itemTypeMap.get(id);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#dashboardDefinitionSize()
   */
  public int dashboardDefinitionSize()
  {
    return dashboardMap.size();
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.items.IDashboardItemTypes#itemTypeSize()
   */
  public int itemTypeSize()
  {
    return itemTypeMap.size();
  }

}
