/*
 * Created on Apr 21, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.items;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExecutableExtension;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class ItemEntry implements IItemEntry
{
  private IRuntimeContext runtime;
  private ILogger logger;
  
  private IConfigurationElement entryItem;
  private boolean signature = false;
  private String id;
  private String itemType;
  private String name;
  private String URL;
  private String DAO;
  private String shortId;
  private List permissions;
//  private String communicatorId;
  private IDataAccessObject daoObject = null;

  public ItemEntry(IRuntimeContext runtime, IConfigurationElement entry)
  {
    this.runtime = runtime;
    this.logger = runtime.getLogger(this.getClass());

    this.entryItem = entry;
    this.id = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_ID);
    this.itemType = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_ITEM_TYPE);
    if (null == this.itemType || 0 == this.itemType.length())
      this.itemType = this.id;
    this.name = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_NAME);
    this.DAO = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_DAO);
    this.URL = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_URL);
    this.shortId = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_SHORT_ID);
//    this.communicatorId = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_COMMUNICATOR);
    String keyStr = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_PERMISSIONS);
    if (null != keyStr)
      this.permissions = Arrays.asList(keyStr.split(";"));
    String signable = entry.getAttribute(IDashboardItemTypes.DASH_ITEM_ATTR_SIGNABLE);
    if (null != signable && 0 != signable.length())
      this.signature = Boolean.valueOf(signable).booleanValue(); 
  }
  
  public synchronized IExecutableExtension getInstance() throws Exception
  {
      return (IExecutableExtension)entryItem
              .createExecutableExtension(IDashboardItemTypes.DASH_ITEM_ATTR_CLASS);
  }

  public synchronized IDataAccessObject getItemDAOInstance() throws Exception
  {
    if (null == daoObject)
    {
      IDaoRegistry daoRegistry = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
      daoObject = daoRegistry.getDAO(DAO);
    }
    return daoObject;
  }
    
  public String getItemDAO()
  {
    return DAO;
  }
    
  public String getItemShortId()
  {
    return shortId;
  }
    
  public String getName()
  {
    return name;
  }
    
  public String getID()
  {
    return id;
  }
    
  public String getURL()
  {
    return URL;
  }
    
  public boolean requiresSignature()
  {
    return signature;
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IItemEntry#getCommunicatorId()
	 */
//	public String getCommunicatorId()
//	{
//		return communicatorId;
//	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IItemEntry#getItemType()
	 */
	public String getItemType()
	{
		return itemType;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IItemEntry#getKeys()
	 */
	public List getPermissions()
	{
		return (null == permissions ? new ArrayList(0) : permissions);
	}

}
