/*
 * Created on Apr 25, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.items;

import gov.va.med.hds.cd.runtime.IExecutableExtension;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public interface IDashboardPatient extends IExecutableExtension
{
  public static final String PATIENT_ITEM_ID = "dashboard.patient";

  /**
   * Format for incoming dates
   */
  public static final String DATE_INPUT_FORMAT = "yyyyMMdd";
  
  /**
   * Format for date output
   */
  public static final String DATE_OUTPUT_FORMAT = "dd MMM yyyy";

	public void setPatient(String id, String name, String ssn, String dob, String age);
	public Object visualizeDashItems(int sequence, int view);
	/**
	 * Sets the patient's name
	 * 
	 * @param patientName The patient's name usually formatted as <pre>LastName, FirstName</pre>
	 */
	public void setPatientName(String patientName);
	/**
	 * Returns the patient's name.
	 * 
	 * @return The patient's name as a String object.
	 */
	public String getPatientName();
	/**
	 * Sets the patient's identifier.
	 * 
	 * @param patientId The patient's identifier as a String object.
	 */
	public void setPatientId(String patientId);
	/**
	 * Returns the patient's identifier.
	 * 
	 * @return The patient's identifier as a String object.
	 */
	public String getPatientId();
	/**
	 * Sets the patient's date of birth.
	 * 
	 * @param dob The patient's date of birth formatted as <pre>"YYYYMMDD"</pre> where
	 * <dl><dt>YYYY</dt><dd>is the year</dd>
	 * <dt>MM</dt><dd>is the month.  A value between 01-12 inclusive.</dd>
	 * <dt>DD</dt><dd>is the day of the month.  A value between 01-31 inclusive.</dd></dt>
	 */
	public void setDob(String dob);
	/**
	 * Returns the patient's date of birth as a formatted string.
	 * 
	 * @return The patient's date of birth as a String object formatted as <pre>
	 * "dd MMM yy"</pre> where:
	 * <dl><dt>dd</dt><dd>is the day of the month.</dd>
	 * <dt>MMM</dt><dd>is the three character abbreviation of the month</dd>
	 * <dt>yy</dt><dd>is the last two digits of the year</dd></dl>
	 */
	public String getDob();
	/**
	 * Sets the patient's Social Security Number
	 * 
	 * @param ssn The patient's Social Security Number.  No formatting is done
	 * to the input string.  It is stored as is.
	 */
	public void setSsn(String ssn);
	/**
	 * Returns the patient's Social Security Number.
	 * 
	 * @return The patient's Social Security Number as a String object. 
	 */
	public String getSsn();
	public void setAge(String age);
  public void setSensitive(boolean sensitive);
  public boolean isSensitive();
  public void setAllowSensitive(boolean allowSensitive);
  public boolean sensitiveIsAllowed();
  public void setSensitivityChecked(boolean sensitivityChecked);
  public boolean isSensitivityChecked();
  
	/**
	 * Returns the age of the patient as calculated from the patient's date of 
	 * birth.  Currently, this doesn't take into account whether the patient is 
	 * dead.
	 * 
	 * @return The age of the patient as a String.
	 */
	public String getAge();
	/**
	 * Returns an object that describes how to display patient information.  The return value
	 * depends on the context.
	 * 
	 * @param context The context of the visualization.  Can be one of
	 * <ul><li>IDashItem.DASH_VIEW_ICON</li>
	 * <li>IDashItem.DASH_VIEW_DETAIL</li></ul>
	 * @return If the context is IDashItem.DASH_VIEW_ICON, the returned Object is a 
	 * javax.swing.JPanel object that displays the patient line.  If the context is
	 * IDashItem.DASH_VIEW_DETAIL the return value is the query portion of a URL as a String
	 * object.
	 */
	public Object visualizePatient(int context);
	public Object visualizeDetail(int type, int context);
	public void putItem(String itemType, IDashboardItem item);
	public IDashboardItem getItem(String itemType, Object itemID);
  public int size();
}