/*
 * Created on Apr 15, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.dao.m;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import gov.va.med.hds.dashboard.dao.IDashboardVitalsDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.itemtype.DashboardVitalsItem;
import gov.va.med.hds.dashboard.itemtype.IVitalsItem;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardVitalsDAO extends AbstractDashboardDAO implements IDashboardVitalsDAO
{
  protected static final String VITALS_BY_PATIENT = "VITALS BY PATIENT";
  protected static final String VITALS_BY_ID = "VITALS BY ID";

  public DashboardVitalsDAO()
  {
    super();
    dbItemsByID = VITALS_BY_ID;
    dbItemsByPatient = VITALS_BY_PATIENT;
    updateByID = "";
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#parseItems(java.lang.Object)
   */
  public List parseItems(Object data)
  {
    ArrayList vitals = new ArrayList();
    DashboardVitalsItem vital = null;
    String[] vitalsData = ((String) data).split("\n");
    for (int l = 0; l < vitalsData.length; l++)
    {
      String[] line = vitalsData[l].split("=", 2);
      if ("Item".equalsIgnoreCase(line[0]))
        vitals.add((vital = (DashboardVitalsItem)parseItem(line[1])));
      else if ("Data".equalsIgnoreCase(line[0]))
        parseData(vital, line[1]);
      else if ("Cmnt".equalsIgnoreCase(line[0]))
        vital.addComment(line[1]);
    }
    return vitals;
  }

  private IDashboardItem parseItem(String data)
  {
    // The ordered item looks like:
    // ORD:ID^^YYYYMMDDHHmmss
    String[] vitals = StringUtil.split(data, '^');
    IDashboardItem item = null;;
    try
    {
      item = (IDashboardItem)dbItems.get(vitals[0]);
      if (null == item)
      {
        item = (IDashboardItem)itemTypes.getItemType(ITEM_TYPE_ID).getInstance();
        item.setID(vitals[0]);
      }
      item.setDate((-1 != vitals[2].indexOf('.') ? (Object)StringUtil.fmDateToDate(vitals[2]) : vitals[2]));
      item.setShortTitle((0 != vitals[1].length() ? vitals[1] : 
        item.getFormattedDate()));
      item.setDirty(false);
      dbItems.put(item.getID(), item);
      return item;
    }
    catch (Exception e)
    {
      logger.error(e.getMessage(), e);
    }
    return item;
  }
  
  private void parseData(IDashboardItem item, String data)
  {
    // A vitals data line looks like:
    // name^value^units^metric value^metric units^bmi^s02^abnormal
    
    Vector vData = (Vector) item.getData();
    if (null == vData) vData = new Vector();
    String[] pieces = StringUtil.split(data.trim(), '^');

    IVitalsItem dataItem = (IVitalsItem)((DashboardVitalsItem)item).createDataType();
    dataItem.setVitalValues(pieces[0], // Vital name
      pieces[1], // value
      pieces[2], // units
      (3 < pieces.length  && 0 != pieces[3].length() ? pieces[3] : null), // optional metric value
      (4 < pieces.length  && 0 != pieces[4].length() ? pieces[4] : null), // optional metric units
      (5 < pieces.length  && 0 != pieces[5].length() ? pieces[5] : null), // optional BMI (only with weight)
      (6 < pieces.length  && 0 != pieces[6].length() ? pieces[6] : null), // optional SO2 (only with pulse)
      (7 < pieces.length ? pieces[7] : ""), // optional abnormal
      (8 < pieces.length ? pieces[8] : "") // optional qualifiers
    );
    vData.add(dataItem);
    if (!item.isCritical())
      item.setCritical((0 != dataItem.getAbnormal().length()) ||
                       (null != dataItem.getBmi() && -1 != dataItem.getBmi().indexOf('*')) ||
                       (null != dataItem.getSO2() && -1 != dataItem.getSO2().indexOf('*')));
    item.setData(vData);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems()
   */
  public void updateItems()
  {
    // Do nothing - Vitals aren't updated
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.dao.IDashboardItemDAO#updateItems(java.util.List)
   */
  public void updateItems(List ids)
  {
    // Do nothing - Vitals aren't updated
  }
}
