/*
 * Created on Apr 16, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.itemtype;

import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.signature.ISignableItem;
import gov.va.med.hds.signature.ISignaturePatient;
import gov.va.med.hds.signature.SignableObject;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardUnverifiedOrder
	extends AbstractDashboardItem
	implements ISignableItem {
	public static final String ITEM_TYPE_ID = "dashboard.unverified";

	private SignableObject sign = new SignableObject();

	public DashboardUnverifiedOrder() {
		itemTypeID = ITEM_TYPE_ID;
	}

	public Object createDataType() {
		return new StringBuffer();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignableItem#isReadyToSign()
	 */
	public boolean isReadyToSign() {
		return sign.isReadyToSign();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignableItem#isSigned()
	 */
	public boolean isSigned() {
		return sign.isSigned();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignableItem#setReadyToSign(boolean)
	 */
	public void setReadyToSign(boolean readyToSign) {
		sign.setReadyToSign(readyToSign);
		super.setComplete(readyToSign);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignableItem#setSigned(boolean)
	 */
	public void setSigned(boolean signed) {
		sign.setSigned(signed);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#isComplete()
	 */
	public boolean isComplete() {
		return isReadyToSign();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#setComplete(boolean)
	 */
	public void setComplete(boolean complete) {
		setReadyToSign(complete);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignableItem#getSignaturePatient()
	 */
	public ISignaturePatient getSignaturePatient() {
		return new ISignaturePatient() {
			public Object getPatientId() {
				return ((IDashboardPatient) getPatient()).getPatientId();
			}

			public PatientLine getPatientLine() {
				IDashboardPatient patient = (IDashboardPatient) getPatient();
				return new PatientLine(
					patient.getPatientName(),
					patient.getSsn(),
					patient.getDob(),
					patient.getAge(),
					false);
			}
		};
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.items.IDashboardItem#setID(java.lang.String)
	 */
	public void setID(String itemID) {
		super.setID(itemID);
		// super.setID checks the itemID for a '*' and a '!'.  The '*' indicates
		// the item is significant/abnormal.  The '!' indicates the item's status is
		// a terminal setting (e.g. complete, DC'd).
		// The only time this check should be made is from the dashboard - prior to
		// reading the detail.  Since this is a signable item, if the status is terminal
		// that means it shouldn't show up on the sign list so we need to set the signed 
		// flag to true. 
		if (isComplete())
			setSigned(true);
	}

}
