/*
 * Created on Apr 15, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.itemtype;

import java.text.SimpleDateFormat;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class DashboardVitalsItem extends AbstractDashboardItem
{
  public static final String ITEM_TYPE_ID = "dashboard.vitals";

  private static final String VITALS_OUTPUT_DATE_FORMAT = "MMM dd, yyyy@HH:mm";

  public DashboardVitalsItem()
  {
    itemTypeID = ITEM_TYPE_ID;
  }

  public Object createDataType()
  {
    return new VitalsItem();
  }
  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.itemtype.IDashboardItem#setDate(java.lang.Object)
   */
  public void setDate(Object itemDate)
  {
    super.setDate(itemDate);
    dateOutputFormat = new SimpleDateFormat(VITALS_OUTPUT_DATE_FORMAT);
  }
  
  /**
   * Department of Veterans Affairs
   * OI Field Office - Salt Lake City
   * Health Data Systems
   * 
   * @author vhaislschras
   */
  public class VitalsItem implements IVitalsItem, Comparable
  {
    private String name = "";
    private String value = "";
    private String units = "";
    private String metricValue = "";
    private String metricUnits = "";
    private String bmi = "";
    private String sO2 = "";
    private String abnormal = "";
    private String[] qualifiers = {};
    public void setVitalValues(String name, String value, String units,
      String mValue, String mUnits, String bmi, String sO2, String abnormal, String qualifiers)
    {
      setName(name);
      setValue(value);
      setMetricValue(mValue);
      setMetricUnits(mUnits);
      setBmi(bmi);
      setSO2(sO2);
      setAbnormal(abnormal);
      setQualifiers(qualifiers);
    }
  
    /**
     * @return String
     */
    public String getAbnormal()
    {
      return abnormal;
    }

    /**
     * @return String
     */
    public String getBmi()
    {
      return bmi;
    }

    /**
     * @return String
     */
    public String getMetricUnits()
    {
      return metricUnits;
    }

    /**
     * @return String
     */
    public String getMetricValue()
    {
      return metricValue;
    }

    /**
     * @return String
     */
    public String getName()
    {
      return name;
    }

    /**
     * @return String
     */
    public String getSO2()
    {
      return sO2;
    }

    /**
     * @return String
     */
    public String getUnits()
    {
      return units;
    }

    /**
     * @return String
     */
    public String getValue()
    {
      return value;
    }

    /**
     * Sets the abnormal.
     * @param abnormal The abnormal to set
     */
    public void setAbnormal(String abnormal)
    {
      this.abnormal = abnormal;
    }

    /**
     * Sets the bmi.
     * @param bmi The bmi to set
     */
    public void setBmi(String bmi)
    {
      this.bmi = bmi;
    }

    /**
     * Sets the metricUnits.
     * @param metricUnits The metricUnits to set
     */
    public void setMetricUnits(String metricUnits)
    {
      this.metricUnits = metricUnits;
    }

    /**
     * Sets the metricValue.
     * @param metricValue The metricValue to set
     */
    public void setMetricValue(String metricValue)
    {
      this.metricValue = metricValue;
    }

    /**
     * Sets the name.
     * @param name The name to set
     */
    public void setName(String name)
    {
      this.name = name;
    }

    /**
     * Sets the sO2.
     * @param sO2 The sO2 to set
     */
    public void setSO2(String sO2)
    {
      this.sO2 = sO2;
    }

    /**
     * Sets the units.
     * @param units The units to set
     */
    public void setUnits(String units)
    {
      this.units = units;
    }

    /**
     * Sets the value.
     * @param value The value to set
     */
    public void setValue(String value)
    {
      this.value = value;
    }

		/**
		 * @return
		 */
		public String[] getQualifiers()
		{
			return qualifiers;
		}

		/**
		 * @param strings
		 */
		public void setQualifiers(String qualifiers)
		{
			this.qualifiers = (0 < qualifiers.length() ? qualifiers.split(";") : new String[0]);
		}

		/* (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object o)
		{
      String[] measurementNames = {
        "Temp.",
        "Pulse",
        "Resp.",
        "B/P",
        "Ht.",
        "Wt.",
        "Pain",
        "C/P",
        "CVP",
        "PO2"
      };
        String measurement1 = getName();
        String measurement2 = ((IVitalsItem)o).getName();
        int order1;
        for (order1 = 0; order1 < measurementNames.length; order1++)
          if (measurement1.startsWith(measurementNames[order1]))
            break;
        int order2;
        for (order2 = 0; order2 < measurementNames.length; order2++)
          if (measurement2.startsWith(measurementNames[order2]))
            break;
        return (order1 < order2 ? -1 : (order1 > order2 ? 1 : 0));
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString()
		{
      return getName();
		}

  }

}
