package gov.va.med.hds.dashboard.nurse;

import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JComponent;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
//import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.dashboard.IDashboardPlugin;
import gov.va.med.hds.dashboard.ResolveUserId;
import gov.va.med.hds.dashboard.dao.IDashboardPermissionsDAO;
import gov.va.med.hds.dashboard.items.IDashboardDefinition;
import gov.va.med.hds.dashboard.items.IDashboardEntry;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class NurseDashboardPlugin implements IDashboardPlugin
{
  private IDesktop desktop;
  private IRuntimeContext runtime;
  private IPluginDescriptor plugin;
//  private IVisualComponentDescriptor patientList;
//  private IVisualComponentDescriptor detailPane;
  private IDashboardDefinition dashDef;
  
  private Object provider = null;
  private Map userPermissions = Collections.synchronizedMap(new Hashtable());
  
  public static final String PLUGIN_ID = "gov.va.med.hds.dashboard.nurse";
  public static final String PLUGIN_MODULE = "dashboard.nurse";
  public static final String PATIENT_LIST_VISUAL_COMPONENT = "dashboard.patientlist";
  public static final String DETAIL_PANE_VISUAL_COMPONENT = "dashboard.detailpane";

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor)
   */
  public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception
  {
    this.runtime = runtime;
    this.plugin = plugin;
    
    ISecurity security = (ISecurity)runtime.getPluginInstance(ISecurity.PLUGIN_ID);
    provider = ResolveUserId.resolveUser(security.getSubject());
    IDaoRegistry registry = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
    dashDef = ((IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID))
      .getDashboardDefinition(PLUGIN_MODULE);
    
    desktop = PrismUtilities.getDesktop(runtime);
    
    IDashboardPermissionsDAO permissionDAO = (IDashboardPermissionsDAO)registry.getDAO(IDashboardPermissionsDAO.class.getName());
    if (null == permissionDAO)
      throw new Exception("Unable to create IDashboardPermissionsDAO");
      
    Iterator iterator = getItemTypes();
    while (iterator.hasNext())
		{
			IDashboardEntry entry = (IDashboardEntry)iterator.next();
      List permissions = entry.getItemEntry().getPermissions();
      for (Iterator iter = permissions.iterator(); iter.hasNext();)
			{
				String permission = (String)iter.next();
        userPermissions.put(permission, Boolean.valueOf(permissionDAO.isValidKey(permission)));
			}
		}

  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
   */
  public void shutdown() throws Exception
  {
    // Do nothing
  }
  
  public JComponent getPatientListComponent()
  {
    return getComponent(PATIENT_LIST_VISUAL_COMPONENT);
  }
  
  public JComponent getDetailPaneComponent()
  {
    return getComponent(DETAIL_PANE_VISUAL_COMPONENT);
  }
  
  public Object getUser()
  {
    return provider;
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashboardPlugin#getItemTypeCount()
   */
  public int getItemTypeCount()
  {
    return dashDef.size();
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashboardPlugin#getHeaderCount()
   */
  public int getHeaderCount()
  {
    int headerCount = 0;
    for (Iterator iter = getItemTypes(); iter.hasNext();)
    {
      IDashboardEntry entry = (IDashboardEntry)iter.next();
      if (null != entry.getLabel()) headerCount++;
    }
    return headerCount;
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashboardPlugin#getItemTypes()
   */
  public Iterator getItemTypes()
  {
    return dashDef.iterator();
  }
  
  public IDashboardEntry getDashboardItemDefinition(String type)
  {
    return dashDef.getDefinition(type);
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.IDashboardPlugin#getRuntime()
   */
  public IDesktop getDesktop()
  {
    return desktop;
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardPlugin#getModuleID()
	 */
	public String getModuleID()
	{
		return NurseDashboardPlugin.PLUGIN_MODULE;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardPlugin#getPluginID()
	 */
	public String getPluginID()
	{
    return NurseDashboardPlugin.PLUGIN_ID;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.IDashboardPlugin#hasKey(java.lang.String)
	 */
	public boolean hasPermission(String permission)
	{
    Boolean value = (Boolean)userPermissions.get(permission);
    if (null != value)
      return value.booleanValue();
    return false;
	}

  protected JComponent getComponent(String componentID) {
    IPerspectiveReference perspective = desktop.getActiveWindow().getActivePerspective();
    IVisualComponentReference[] components = perspective.getVisualComponents();
    for (int i = 0; i < components.length; i++)
    {
      if (componentID.equals(components[i].getDescriptor().getId()))
        return components[i].getVisualComponent().getComponent();
    }
    return null;
  }

}
