/*
 * Created on Apr 1, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 */
package gov.va.med.hds.dashboard;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
//import gov.va.med.hds.cd.runtime.PluginNotFoundException;
//import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.dashboard.items.DashItemChangeEvent;
import gov.va.med.hds.dashboard.nurse.NurseDashboardPlugin;
import gov.va.med.hds.dashboard.ui.DashboardDetailPane;
//import gov.va.med.hds.task.ITaskPlugin;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class NurseDashboardDetail implements IVisualComponent, IDashboardDetailVisualComponent {

  private final String PERSPECTIVE_ID = "dashboard.nurseDashboard";

	private IVisualComponentDescriptor descriptor;
	private IDesktop desktop;
	private JComponent component;
	private DashboardDetailPane dhtmlPane;
	
	public JComponent getComponent() {
		return component;
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.descriptor = (IVisualComponentDescriptor) data;
		this.desktop = PrismUtilities.getDesktop(runtime);
		component = createComponent(descriptor, desktop);
		addHelpIds();
	}
	private void addHelpIds() {
		component.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, IDashboardNurseUIConstants.INTRO_HELP_ID);
	}

	private JComponent createComponent(
		IVisualComponentDescriptor descriptor,
		IDesktop desktop) {
		JScrollPane scroller = null;
//			IRuntimeContext runtime = desktop.getRuntime();

			scroller = new JScrollPane();
			PatientLine patLine = new PatientLine();
			scroller.setColumnHeaderView(patLine);
			JLabel corner = new JLabel();
			corner.setOpaque(true);
			corner.setBackground(patLine.getBackground());
			scroller.setCorner(JScrollPane.UPPER_RIGHT_CORNER, corner);
			dhtmlPane =
				new DashboardDetailPane(
					desktop.getRuntime(),
					NurseDashboardPlugin.PLUGIN_MODULE);
			dhtmlPane.setContentType("text/html");
			dhtmlPane.setBlankPage();

      		dhtmlPane.setReferencePerspective(PERSPECTIVE_ID);
			desktop.getPerspectiveEventSource().addPerspectiveListener(
				(IPerspectiveListener) dhtmlPane);
			scroller.getViewport().setView(dhtmlPane);
//		try {
//			ITaskPlugin taskPlugin =
//					   (ITaskPlugin) runtime.getPluginInstance(
//						   ITaskPlugin.PLUGIN_ID);
//			taskPlugin.registerTaskChangeListener(
//							(ITaskChangeListener) itemListener,
//							dhtmlPane);
//		} catch (PluginStartupException e) {
//			desktop.getRuntime().getLogger(this.getClass()).error(
//				e.getLocalizedMessage(),
//				e);
//		} catch (PluginNotFoundException e1) {
//			desktop.getRuntime().getLogger(this.getClass()).error(
//				e1.getLocalizedMessage(),
//				e1);
//		}
		return scroller;
	}

	public void valueChanged(DashTableEvent e) {
		dhtmlPane.valueChanged(e);
	}

	public void tableChanged(DashTableEvent e) {
		dhtmlPane.tableChanged(e);
	}

	public void itemUpdated(DashItemChangeEvent e) {
		dhtmlPane.itemUpdated(e);
	}
}
