/*
 * Created on Mar 31, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.dashboard.ui;

import java.awt.Component;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
//import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.dashboard.IDashboardPlugin;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class NurseDashPane
	extends JSplitPane
	implements IVisualComponent, TableModelListener, IPerspectiveListener {
	// Don't like having to hard code this, it would be nice if I could 
	// determine my containing plugin
	private static final String HELP_INTRO_ID =
		"gov.va.med.hds.nursing.dashboard.intro";
	private final String CONTAINER_PLUG_ID = "gov.va.med.hds.dashboard.nurse";
//	private static ListSelectionListener selectionListener = null;

	private IVisualComponentDescriptor descriptor;
	private IDesktop desktop;

	public NurseDashPane() {
		// Stack panes (i.e. top and bottom)
		setOrientation(VERTICAL_SPLIT);
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
		this.descriptor = (IVisualComponentDescriptor) data;
		this.desktop = PrismUtilities.getDesktop(runtime);
		try {
			IDashboardPlugin myContainer =
				(IDashboardPlugin) desktop.getRuntime().getPluginInstance(
					CONTAINER_PLUG_ID);
			setTopComponent(myContainer.getPatientListComponent());
			setBottomComponent(myContainer.getDetailPaneComponent());
			// Default the dividers to display all of the patient list
			setDividerLocation(-1);

			// Listen to table for size changes
			Component c = getTopComponent();
			JTable table =
				(c instanceof JScrollPane ? table =
					(JTable) ((JScrollPane) c).getViewport().getView() : (
						c instanceof JTable ? table = (JTable) c : null));
			if (null != table) {
				TableModel model = table.getModel();
				// Make sure this object is listening only once
				model.removeTableModelListener(this);
				model.addTableModelListener(this);
			}
			addHelpIds();
		} catch (PluginNotFoundException p1) {
			desktop.getRuntime().getLogger(this.getClass()).error(
				p1.getLocalizedMessage(),
				p1);
		} catch (PluginStartupException p2) {
			desktop.getRuntime().getLogger(this.getClass()).error(
				p2.getLocalizedMessage(),
				p2);
		}
	}

	private void addHelpIds() {
		this.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, HELP_INTRO_ID);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IVisualComponent#getDescriptor()
	 */
	public IVisualComponentDescriptor getDescriptor() {
		return descriptor;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IVisualComponent#getComponent()
	 */
	public JComponent getComponent() {
		return (JSplitPane) this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
	 */
	public void tableChanged(TableModelEvent e) {
		// If column == -1 and last row == Integer.MAX_VALUE it mean the whole table is changing,
		// which indicates a whole new set of patients.  The size of the table may have changed
		// so reset the splitter bar
		if (-1 == e.getColumn() && Integer.MAX_VALUE == e.getLastRow()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					Component c = getTopComponent();
					if (c instanceof JScrollPane) {
						int height = (getHeight() / 3);
						JTable table;
						JScrollPane scrollPane =
							(JScrollPane) getTopComponent();
						if (scrollPane.getViewport().getView()
							instanceof JTable)
							table = (JTable) scrollPane.getViewport().getView();
						else
							table =
								(JTable) ((JScrollPane) getBottomComponent())
									.getViewport()
									.getView();
						int rowCount = table.getRowCount();
						setDividerLocation(
							Math.min(
								height,
								(table.getTableHeader().getHeight()
									* (rowCount + 1))
									+ 2));
					}
				}
			});
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveActivated(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveActivated(PerspectiveEvent e) {
		if (!(descriptor
			.getId()
			.equals(e.getPerspective().getDescriptor().getId()))) {
			// TODO: Find the DAOs and synchronize them with the database.
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveClosed(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveClosed(PerspectiveEvent e) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveOpened(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveOpened(PerspectiveEvent e) {
	}

	public void perspectiveClosing(PerspectiveEvent e) {
	}

	public void perspectiveOpening(PerspectiveEvent e) {
	}
}
