/*
 * Created on Apr 16, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.dao.IDashboardAppointmenttDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardAppointmentItem;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class AppointmentDetail extends AbstractItemDetail
{
  IDashboardAppointmenttDAO dao = null;

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.url.IServlet#service(gov.va.med.hds.cd.url.IServletRequest, gov.va.med.hds.cd.url.IServletResponse)
   */
  public void service(IServletRequest request, IServletResponse response) throws IOException
  {
    init(request.getRuntimeContext());
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    if (request.getParameter("display") != null)
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardAppointmentItem.ITEM_TYPE_ID);
        dao = (IDashboardAppointmenttDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge",
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, false, dao, display));
    out.write("</body></html>");
  }
	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemFooter(gov.va.med.hds.dashboard.items.IDashboardItem, java.lang.StringBuffer)
	 */
	public void itemFooter(IDashboardItem item, StringBuffer html)
	{
//    String itemID = item.getID();
    Object patientID = null;
    try {
        patientID = ((IDashboardPatient)item.getPatient()).getPatientId();
    } catch (Exception e) {}

    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
        if (taskPlugin != null) {
            html.append("<a href=\"");    
            html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "Appointment", item.getID(), patientID));
            html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        }
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>&nbsp;&nbsp;&nbsp;");
        }
    }
    html.append("</td></table>");
	}

}
