/*
 * Created on Apr 16, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.OrderStatus;
import gov.va.med.hds.dashboard.dao.IDashboardItemDAO;
import gov.va.med.hds.dashboard.dao.IDashboardTxOrderDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardTxOrder;
import gov.va.med.hds.signature.ISignableItem;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class TxOrderDetail extends AbstractItemDetail
{
  private IDashboardTxOrderDAO dao = null;

  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.url.IServlet#service(gov.va.med.hds.cd.url.IServletRequest, gov.va.med.hds.cd.url.IServletResponse)
   */
  public void service(IServletRequest request, IServletResponse response) throws IOException
  {
    init(request.getRuntimeContext());
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    boolean displaySet = false;
    if (request.getParameter("display") != null) {
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();
        displaySet = true;
    }

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardTxOrder.ITEM_TYPE_ID);
        dao = (IDashboardTxOrderDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge",
      "gov.va.med.hds.dashboard.ui.scriptlet.DisableCheckboxScriptlet"
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, false, dao, display, displaySet));
    out.write("</body></html>");
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemFooter(gov.va.med.hds.dashboard.itemtype.IDashboardItem, java.lang.StringBuffer)
   */
  public void itemFooter(IDashboardItem item, StringBuffer html)
  {
    String itemID = item.getID();
    Object patientID = null;
    try {
        patientID = ((IDashboardPatient)item.getPatient()).getPatientId();
    } catch (Exception e) {}

    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
        if (taskPlugin != null) {
            html.append("<a href=\"");    
            html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "Nursing", item.getID(), patientID));
            html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        }
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>&nbsp;&nbsp;&nbsp;");
        }
    }
    html.append("</td>");
    if (((ISignableItem)item).isSigned())
      html.append("<td align='right'>Completed&nbsp;</td>");
    else if (!OrderStatus.isCompleteStatus(item.getStatus()))
    {
      html.append("<td align='right'><input itemtype='" +
        item.getTypeID() + "' type='checkbox' id='ck_" + 
        itemID + "' onChange=\"acknowledge('ck_" + 
        itemID + "', '" + item.getTypeID() + "')\"" +
        (item.isComplete() ? " checked" : "") +
        " label='Complete'>&nbsp;</td>");
    }
    html.append("</tr></table>");
  }

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemHeader(gov.va.med.hds.dashboard.items.IDashboardItem, boolean, java.lang.StringBuffer)
	 */
	public void itemHeader(IDashboardItem item, boolean hideCollapser, StringBuffer html)
	{
    String itemID = item.getID();
    html.append("<table class='resultheader' cellpadding='1' cellspacing='1'><tr>");
    if (!hideCollapser)
      html.append("<td valign='top' width='1%'><collapser forID='" + itemID +"'></td>");

    html.append("<td valign='top' align='left' style='font-weight:bold' width='350'>" + formatTitle(item) + "</td>");
    html.append("<td valign='top' align='left'style='font-weight:bold'>" + asHTML(item.getStatus().toUpperCase()));
    html.append("</td><td valign='top' align='right' width='120'>" + asHTML(item.getFormattedDate()) + "</td></tr></table>");
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#htmlBody(java.lang.String[], boolean, boolean, gov.va.med.hds.dashboard.dao.IDashboardItemDAO, boolean)
	 */
	public String htmlBody(String[] ids, boolean brief, boolean autoack, IDashboardItemDAO itemDAO, boolean display, boolean displaySet)
	{
    StringBuffer html = new StringBuffer();
    List items = itemDAO.getItems(Arrays.asList(ids));
    for (Iterator iter = items.iterator(); iter.hasNext();)
    {
      IDashboardItem item = (IDashboardItem)iter.next();
      if (display && !displaySet)
        display = !("complete".equalsIgnoreCase(item.getStatus()));
      String itemID = item.getID();
      html.append("<div class='resultbox'>");
      itemHeader(item, (brief && display), html);
      html.append("<div " + (display ? "" : "class='displaynone'") +
        " id='" + itemID + "'>");
      html.append("<div>");
      itemDetail(item, brief, autoack, true, html);
      html.append("</div></div>");
      if (!brief)
        itemFooter(item, html);
      html.append("</div>");
      if (display) display = false;
    }
    if (!brief)
      pageFooter(html);
    return html.toString();
	}

}
