/*
 * Created on Apr 15, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.dashboard.ui.servlet;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.dashboard.dao.IDashboardVitalsDAO;
import gov.va.med.hds.dashboard.items.IDashboardItem;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IDashboardPatient;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.dashboard.itemtype.DashboardVitalsItem;
import gov.va.med.hds.dashboard.itemtype.IVitalsItem;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class VitalsDetail extends AbstractItemDetail
{
  private IDashboardVitalsDAO dao = null;
  
  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.url.IServlet#service(IServletRequest, IServletResponse)
   */
  public void service(IServletRequest request, IServletResponse response)
      throws IOException
  {
    init(request.getRuntimeContext());
    String[] ids = request.getParameterList("id");
    boolean brief = Boolean.valueOf(request.getParameter("brief")).booleanValue();

    boolean display = true;  // default value
    if (request.getParameter("display") != null)
        display = Boolean.valueOf(request.getParameter("display")).booleanValue();

    if (null == dao)
    {
      try
      {
        IDashboardItemTypes items = (IDashboardItemTypes)runtime.getPluginInstance(IDashboardItemTypes.PLUGIN_ID);
        IItemEntry item = items.getItemType(DashboardVitalsItem.ITEM_TYPE_ID);
        dao = (IDashboardVitalsDAO)item.getItemDAOInstance();
      }
      catch (Exception pse)
      {
        throw new IOException(pse.toString());
      }
    }
    out = response.getWriter();
    response.setContentType("text/html");
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window",
      "gov.va.med.hds.dashboard.ui.scriptlet.Acknowledge",
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(ids, brief, false, dao, display));
    out.write("</body></html>");
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemDetail(gov.va.med.hds.dashboard.itemtype.IDashboardItem, boolean, java.lang.StringBuffer)
   */
  public void itemDetail(IDashboardItem item, boolean brief, boolean autoack, boolean hasPermission, StringBuffer html)
  {
    html.append("<table width='100%' cellspacing='0' cellpadding='0' style=\"font-family:'Courier New';\">");
    DashboardVitalsItem vitals = (DashboardVitalsItem)item;
    Vector data = sortMeasurments((Vector)vitals.getData());
    if (null == data)
      html.append("<tr><td>&nbsp;&nbsp;No Data</td></tr>");
    else
    {
      Enumeration enum = data.elements();
      while (enum.hasMoreElements())
      {
        IVitalsItem vitalsItem = (IVitalsItem)enum.nextElement();
        html.append("<tr" + bold(vitalsItem.getAbnormal()) + "><td width='15%'>&nbsp;&nbsp;");
        html.append(vitalsItem.getName() + ":</td><td width='25%'>");
        html.append(vitalsItem.getValue());
        if (0 != vitalsItem.getUnits().length())
          html.append("&nbsp;" + vitalsItem.getUnits());
        html.append(vitalsItem.getAbnormal());
        String metric = vitalsItem.getMetricValue(); 
        if (null != metric && 0 != metric.length())
        {
          html.append("&nbsp;(" + vitalsItem.getMetricValue());
          metric = vitalsItem.getMetricUnits();
          if (null != metric && 0 != metric.length())
            html.append("&nbsp;" + metric);
          html.append(vitalsItem.getAbnormal() + ")");
        }
        String[] qualifiers = vitalsItem.getQualifiers();
        html.append("</td><td>");
        for (int i = 0; i < qualifiers.length; i++)
					html.append((0 < i ? ";" : "") + qualifiers[i]);
        if (null != vitalsItem.getSO2())
        {
          if (qualifiers.length > 0)
            html.append("; ");
          html.append(vitalsItem.getSO2() + " L/Min");          
        }        
        html.append("</td></tr>");
        if (null != vitalsItem.getBmi())
        {
          html.append("<tr" + bold(vitalsItem.getBmi()) + ">");
          html.append("<td width='15%'>&nbsp;&nbsp;BMI:</td><td>" + vitalsItem.getBmi() + "</td></tr>");
          html.append("</tr>");
        }
      }
    }
    html.append("</table>");
    html.append("<div style='display:none' autoack='"+ autoack + "' deferredsrc='blank.html' " +  
      "itemtype='" + DashboardVitalsItem.ITEM_TYPE_ID + "' id='d_" + vitals.getID() + "'></div>");
  }

  public void itemFooter(IDashboardItem item, StringBuffer html)
  {
    Object patientID = null;
    try {
        patientID = ((IDashboardPatient)item.getPatient()).getPatientId();
    } catch (Exception e) {}

    html.append("<table class='resultfooter' cellpadding='1' cellspacing='2'><tr>");
    html.append("<td width='380'>&nbsp;");
    if (patientID != null) {
        if (taskPlugin != null) {
            html.append("<a href=\"");    
            html.append(taskPlugin.getLinkToTaskURL(item.getTitle(), "Vital", item.getID(), patientID));
            html.append("\">Link Task</a>&nbsp;&nbsp;&nbsp;");
        }
        if (contextPlugin != null) {
            html.append("<a href=\"");
            html.append(contextPlugin.getCcowHref(patientID));
            html.append("\">Go to Chart</a>");
        }
        //html.append("&nbsp;&nbsp;&nbsp;Patient:");
    }
    html.append("</td>");
//    html.append("<td align='right'><input type='checkbox' id='ck_" + 
//      itemID + "' onChange=\"acknowledge('ck_" + 
//      itemID + "', '" + DashboardVitalsItem.ITEM_TYPE_ID + "')\"" +
//      (item.isComplete() ? " checked" : "") +
//      " label='Acknowledge'>&nbsp;</td>");
    html.append("</tr></table>");
  }

  /* (non-Javadoc)
   * @see gov.va.med.hds.dashboard.ui.servlet.AbstractItemDetail#itemHeader(gov.va.med.hds.dashboard.itemtype.IDashboardItem, boolean, java.lang.StringBuffer)
   */
  public void itemHeader(IDashboardItem item, boolean hideCollapser, StringBuffer html)
  {
    String itemID = item.getID();
    html.append("<table class='resultheader' cellpadding='1' cellspacing='1'><tr>");
    if (!hideCollapser)
      html.append("<td valign='top' width='1%'><collapser forID='" + itemID +"'></td>");
    html.append("<td valign='top' align='left' style='font-weight:bold'>" + formatTitle(item) + "</td>");
    if (!(item.getTitle().equals(item.getFormattedDate())))
      html.append("<td valign='top' align='right'>" + item.getFormattedDate() + "</td>");
    html.append("</tr></table>");
  }
  
  private String bold(String item)
  {
    String boldStyle = "";
    if (-1 != item.indexOf('*'))
    boldStyle = " style='font-weight:bold;color:red;'";
    return boldStyle;
  }
  
  private Vector sortMeasurments(Vector measurements)
  {
    if (null != measurements)
    {
      List measList = new ArrayList(measurements);
      Collections.sort(measList, new Comparator(){
        public int compare(Object obj1, Object obj2){
           return (((Comparable)obj1).compareTo(obj2));
      }});
      return new Vector(measList);
    }
    return measurements;
  }

}
