package gov.va.med.hds.cd.help.internal;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.util.Hashtable;
import java.util.Locale;

import javax.help.HelpModel;
import javax.help.HelpSet;
import javax.help.TOCView;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.UIManager;

/**
 * Provides a custom table of contents view for help that allows the window icon to be 
 * changed. This CustomTOCView will override the JavaHelp TOCView automatically.
 */
public class CustomTOCView extends TOCView implements HierarchyListener {

	/**
	 * Constructor
	 * 
	 * @param hs
	 * @param name
	 * @param label
	 * @param params
	 */
	public CustomTOCView(HelpSet hs, String name, String label, Hashtable params) {
		super(hs, name, label, params);
	}

	/**
	 * Constructor 
	 * 
	 * @param hs
	 * @param name
	 * @param label
	 * @param locale
	 * @param params
	 */
	public CustomTOCView(HelpSet hs, String name, String label, Locale locale, Hashtable params) {
		super(hs, name, label, locale, params);
	}

	/**
	 * @see javax.help.NavigatorView#createNavigator(javax.help.HelpModel)
	 */
	public Component createNavigator(HelpModel model) {
		Component result = super.createNavigator(model);
		result.addHierarchyListener(this);
		return result;
	}

	/**
	 * @see java.awt.event.HierarchyListener#hierarchyChanged(java.awt.event.HierarchyEvent)
	 */
	public void hierarchyChanged(HierarchyEvent e) {
		Component c = e.getComponent();
		while (c != null) {
			if (c instanceof Frame) {
				((Frame) c).setIconImage(getIcon());
				return;
			}
			c = c.getParent();
		}
	}

	private Image getIcon() {
		Icon i = UIManager.getDefaults().getIcon("InternalFrame.icon");
		if (i instanceof ImageIcon)
			return ((ImageIcon) i).getImage();
		else
			return null;
	}
}
