package gov.va.med.hds.cd.help.internal;

import gov.va.med.hds.cd.help.CouldNotFindHelpSetException;
import gov.va.med.hds.cd.help.HelpConfigException;
import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.loader.IDelegatingClassLoader;
import gov.va.med.hds.cd.mum.boot.loader.LinkedPluginLoader;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.awt.Component;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.net.URL;

import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.help.HelpSetException;
import javax.help.CSH.DisplayHelpAfterTracking;
import javax.help.CSH.DisplayHelpFromSource;
import javax.help.event.HelpSetEvent;
import javax.help.event.HelpSetListener;
import javax.swing.JRootPane;

/**
 * Help plugin default implementation
 */
public class Help implements IHelp, HelpSetListener {
	/**
	 * Extension point id for the helpSet extension point
	 */
	public static final String HELPSET_EXTENSION_POINT_ID = "gov.va.med.hds.cd.help.helpSet";
	/**
	 * Help Set name tag for the helpSet extension point
	 */
	public static final String CLASSPATH_HELPSET_NAME_TAG = "classPathHelpSetName";

	private static final String ROOT_HELPSET_RESOURCE_NAME =
		"/gov/va/med/hds/cd/help/internal/help/rootHelp/roothelp.hs";
	private static final String ROOT_HELPID = "top";

	/**
	 * Default constructor
	 */
	public Help() {
		super();
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor)
	 */
	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception {
		this.runtime = runtime;
		this.plugin = plugin;
	}

	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
	 */
	public void shutdown() throws Exception {
		masterHelpSet = null;
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#displayHelpWindow(java.awt.event.ActionEvent)
	 */
	public void displayHelpWindow(ActionEvent e) {
		createHelpBroker();
		displayHelpListener.actionPerformed(e);
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#launchDirectHelp(java.awt.event.ActionEvent)
	 */
	public void launchDirectHelp(ActionEvent e) {
		createHelpBroker();
		displayDirectHelpListener.actionPerformed(e);
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#setRootPane(javax.swing.JRootPane)
	 */
	public void setRootPane(JRootPane pane) {
		createHelpBroker();
		helpBroker.enableHelpKey(pane, ROOT_HELPID, null);
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#showHelp(java.awt.Component)
	 */
	public void showHelp(Component component) {
		createHelpBroker();
		helpBroker.setCurrentID(CSH.getHelpIDString(component));
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#showHelp(java.awt.MenuItem)
	 */
	public void showHelp(MenuItem menuItem) {
		createHelpBroker();
		helpBroker.setCurrentID(CSH.getHelpIDString(menuItem));
	}
	/**
	 * @see gov.va.med.hds.cd.help.IHelp#showHelp(java.lang.String)
	 */
	public void showHelp(String helpId) {
		createHelpBroker();
		helpBroker.setCurrentID(helpId);
	}
	/**
	 * @see javax.help.event.HelpSetListener#helpSetAdded(javax.help.event.HelpSetEvent)
	 */
	public void helpSetAdded(HelpSetEvent e) {
		//		System.out.println(e.toString());
	}
	/**
	 * @see javax.help.event.HelpSetListener#helpSetRemoved(javax.help.event.HelpSetEvent)
	 */
	public void helpSetRemoved(HelpSetEvent e) {
		//		System.out.println(e.toString());
	}
	private void buildHelpSetTree()
		throws CouldNotFindHelpSetException, HelpSetException, HelpConfigException {
		URL rootHSUrl = Help.class.getResource(ROOT_HELPSET_RESOURCE_NAME);
		if (rootHSUrl == null) {
			throw new CouldNotFindHelpSetException(
				"Could not find root help set: "
					+ ROOT_HELPSET_RESOURCE_NAME
					+ " in the resource tree");
		}
		masterHelpSet = new HelpSet(Help.class.getClassLoader(), rootHSUrl);
		masterHelpSet.addHelpSetListener(this);
		IPluginRegistry moduleRegistry = runtime.getPluginRegistry();
		IConfigurationElement[] pluginExtensionConfigs =
			moduleRegistry.getConfigurationElementsFor(HELPSET_EXTENSION_POINT_ID);

		for (int i = 0; i < pluginExtensionConfigs.length; i++) {
			insertHelpSet(masterHelpSet, pluginExtensionConfigs[i]);
		}
	}
	private void createHelpBroker() {
		if (helpBroker == null) {
			try {
				buildHelpSetTree();
			} catch (Exception e) {
				runtime.getLogger(getClass().getName()).error(
					"Couldn't create the help set tree",
					e);
			}
			helpBroker = masterHelpSet.createHelpBroker();
			displayHelpListener = new DisplayHelpFromSource(helpBroker);
			displayDirectHelpListener = new DisplayHelpAfterTracking(helpBroker);
		}
	}
	private void insertHelpSet(HelpSet helpSet, IConfigurationElement element)
		throws HelpConfigException {

		if (element.getName().equals(CLASSPATH_HELPSET_NAME_TAG)) {
			if (element.getValue() == null)
				throw new HelpConfigException(
					"The '" + CLASSPATH_HELPSET_NAME_TAG + "' cannot have a null element value");

			insertClassPathHelpSet(
				helpSet,
				element.getValue(),
				element
					.getDeclaringExtension()
					.getDeclaringPluginDescriptor()
					.getPluginClassLoader());
		} else {
			throw new HelpConfigException(
				"The \""
					+ HELPSET_EXTENSION_POINT_ID
					+ "\" extension point must contain a \""
					+ CLASSPATH_HELPSET_NAME_TAG
					+ "\" tag.");
		}
	}
	private void insertClassPathHelpSet(HelpSet helpSet, String name, ClassLoader classLoader)
		throws HelpConfigException {
		URL url = HelpSet.findHelpSet(classLoader, name);
		try {
			IDelegatingClassLoader delegatingClassLoader = ClassLoaderManager.getDelegatingLoader();
			ClassLoader loader = Help.class.getClassLoader();
			if (loader instanceof LinkedPluginLoader)
				delegatingClassLoader.addLoader((LinkedPluginLoader) loader);
			if (classLoader instanceof LinkedPluginLoader)
				delegatingClassLoader.addLoader((LinkedPluginLoader) classLoader);

			HelpSet subHelpSet = new HelpSet((ClassLoader) delegatingClassLoader, url);
			masterHelpSet.add(subHelpSet);
		} catch (HelpSetException e) {
			throw new HelpConfigException(
				"Couldn't find the help set named " + name + " in the classpath.");
		}
	}
	private IRuntimeContext runtime;
	private IPluginDescriptor plugin;
	private HelpSet masterHelpSet;
	private HelpBroker helpBroker;
	private DisplayHelpFromSource displayHelpListener;
	private DisplayHelpAfterTracking displayDirectHelpListener;
}
