unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, uLauncher, uLogger;

type
  TfrmMain = class(TForm)
    procedure FormCreate(Sender: TObject);
  private
    FLauncher: TLauncher;
    FLogger: TLogger;
    procedure Abort(Text: String);
    procedure WaitForJava;
    procedure FlushAndTerminateLog;
    function GetFileNameFromParam: string;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses fSplash, uFnSystem, fLinks;

{$R *.DFM}

const
  TC_MIN_VER  = 'Java Virtual Machine Required';
  TX_MIN_VER1 = 'The necessary Java Virtual Machine was not found.' + #13#10 +
                'This application requires version ';
  TX_MIN_VER2 = ' or greater.';
  TX_SILENT   = '"-silent"';
  TX_LOG  = '-log';

var
  uProcess: TProcessInformation;
  uFound: boolean;
  uForeground: boolean = false;

procedure TfrmMain.Abort(Text: String);
begin
  if Pos(TX_SILENT, LowerCase(FLauncher.RunParams)) = 0 then ShowMessage(Text);
  FLogger.Error(self, Text);
  FlushAndTerminateLog;
  Application.Terminate;
  raise EAbort.Create('Aborted');
end;

procedure TfrmMain.FlushAndTerminateLog;
begin
  FLogger.Debug(self, 'Terminating...');
  if (Pos(TX_LOG, LowerCase(FLauncher.RunParams)) > 0) then
  begin
    FLogger.saveToFile(GetFileNameFromParam);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FLogger := TLogger.Create;
  FLogger.Debug(self, 'Started application / created logger');
  FLogger.Debug(self, 'TLauncher.Create start');
  FLauncher := TLauncher.Create;
  FLauncher.Logger := FLogger;
  FLogger.Debug(self, 'TLauncher.Create end');
  if not FLauncher.UpdateSelf then
  begin
    if (FLauncher.URL = '') and (Pos(TX_SILENT, LowerCase(FLauncher.RunParams)) = 0)
      then FLauncher.URL := ChooseURL(FLauncher.URLChoices);
    if FLauncher.URL = '' then Abort('Missing URL');
    if not FLauncher.FilesAreUpdated then Abort('Unable to update files');
    if FLauncher.JREPath = '' then Abort(TX_MIN_VER1 + FLauncher.JREMinimum + TX_MIN_VER2);
    uProcess := FLauncher.RunJava;
    if Pos(TX_SILENT, LowerCase(FLauncher.RunParams)) = 0 then WaitForJava;
  end
  else
  begin
    FLogger.Debug(self, 'Running self update program');
    RunProgram(FLauncher.UpdateCmd);
  end;
  FlushAndTerminateLog;
  Application.Terminate;
end;

function GetJavaSplashScreen(Handle: HWND; Info: Pointer): BOOL; stdcall;
var
  OwningProcess: Pointer;
  Buffer: array[0..255] of Char;
  WindowName: String;

begin
  GetWindowThreadProcessID(Handle, @OwningProcess);
  Result := TRUE;
  if(DWORD(OwningProcess) = uProcess.dwProcessId) then
  begin
    if(uForeground) then
    begin
      SetForegroundWindow(Handle);
      uForeground := FALSE;
    end;
    SetString(WindowName, Buffer, GetWindowText(Handle, Buffer, SizeOf(Buffer)));
    if(WindowName = 'HealtheVet Desktop') then
    begin
      SetForegroundWindow(Handle);
      uFound := TRUE;
      Result := FALSE;
    end;
  end;
end;

function TfrmMain.GetFileNameFromParam: string;
var
  index, endIndex: integer;
begin
  result := 'session.log';
  index := Pos(TX_LOG, LowerCase(FLauncher.RunParams));
  if (index>0) then
  begin
    index := index + Length(TX_LOG) + 1;
    endIndex := index;
    while ((endIndex<Length(FLauncher.RunParams)) and (Copy(FLauncher.RunParams, endIndex, 1)<>' ')) do
      endIndex := endIndex + 1;
    if (index<>endIndex) then
      result := Copy(FLauncher.RunParams, index, endIndex-index);
  end;
end;

procedure TfrmMain.WaitForJava;
{ waits for Java to start in order to know how long to leave the splash screen up }
const
  TIMEOUT = 45.0;

var
  Done: boolean;
  code: DWORD;
  startTime: TDateTime;

begin
  FLogger.Debug(self, 'WaitForJava start');
  if(uProcess.hProcess = 0) then exit;
  startTime := Now;
  repeat
    if((SecsPerDay*(Now - startTime)) > TIMEOUT) then
      begin
      Done := TRUE;
      FLogger.Error(self, 'Timed out waiting for JVM startup.');
      end
    else
    begin
      GetExitCodeProcess(uProcess.hProcess, code);
      if(code <> STILL_ACTIVE) then
        Done := TRUE
      else
      begin
        uFound := FALSE;
        EnumWindows(@GetJavaSplashScreen, 0);
        Done := uFound;
        if(not Done) then
        begin
          Application.ProcessMessages; // Allow for Screen repaint if needed
          sleep(100);
        end;
      end;
    end;
  until Done;
  FLogger.Debug(self, 'WaitForJava end');
end;

end.











