unit uFnString;
{ common string functions }

interface

uses
  SysUtils;

function CharAt(const x: string; APos: Integer): Char;
function CompareVersionJava(A, B: string): Integer;
function Piece(const S: string; Delim: char; PieceNum: Integer): string;
function StringBetween(const SrcStr, SubA, SubB: string): string;

implementation

function CharAt(const x: string; APos: Integer): Char;
{ returns a character at a given position in a string or the null character if past the end }
begin
  if Length(x) < APos then Result := #0 else Result := x[APos];
end;

function CompareVersionJava(A, B: string): Integer;
{ Result=1 if A>B, Result=-1 if A<B, Result=0 if A=B }
const
  OFFSET_MAJOR = $1000000000000;
  OFFSET_MINOR = $100000000;
  OFFSET_MICRO = $10000;
var
  NumA, NumB: Int64;
  UpdateA, UpdateB: Integer;
begin
  UpdateA := StrToIntDef(Piece(A, '_', 2), 0);  A := Piece(A, '_', 1);
  UpdateB := StrToIntDef(Piece(B, '_', 2), 0);  B := Piece(B, '_', 1);
  NumA := (StrToIntDef(Piece(A, '.', 1), 0) * OFFSET_MAJOR) +
          (StrToIntDef(Piece(A, '.', 2), 0) * OFFSET_MINOR) +
          (StrToIntDef(Piece(A, '.', 3), 0) * OFFSET_MICRO) + UpdateA;
  NumB := (StrToIntDef(Piece(B, '.', 1), 0) * OFFSET_MAJOR) +
          (StrToIntDef(Piece(B, '.', 2), 0) * OFFSET_MINOR) +
          (StrToIntDef(Piece(B, '.', 3), 0) * OFFSET_MICRO) + UpdateB;
  Result := 0;
  if NumA - NumB > 0 then Result := 1;
  if NumA - NumB < 0 then Result := -1;
end;

function Piece(const S: string; Delim: char; PieceNum: Integer): string;
{ returns the Nth piece (PieceNum) of a string delimited by Delim }
var
  i: Integer;
  Strt, Next: PChar;
begin
  i := 1;
  Strt := PChar(S);
  Next := StrScan(Strt, Delim);
  while (i < PieceNum) and (Next <> nil) do
  begin
    Inc(i);
    Strt := Next + 1;
    Next := StrScan(Strt, Delim);
  end;
  if Next = nil then Next := StrEnd(Strt);
  if i < PieceNum then Result := '' else SetString(Result, Strt, Next - Strt);
end;

function StringBetween(const SrcStr, SubA, SubB: string): string;
var
  IndexA, IndexB: Integer;
begin
  IndexA := Pos(SubA, SrcStr);
  if(IndexA <= 0) then
    Result := ''
  else
  begin
    inc(IndexA, Length(SubA));
    IndexB := Pos(SubB, copy(SrcStr, IndexA, MaxInt)) + IndexA - 1;
    Result := Copy(SrcStr, IndexA, IndexB-IndexA);
  end;
end;

end.
