package gov.va.med.hds.cd.m;
import gov.va.med.hds.cd.m.internal.BrokerCallQueue;
import gov.va.med.hds.cd.m.internal.Mumps;
import gov.va.med.hds.cd.m.internal.MumpsConnectionSettings;
import gov.va.med.hds.cd.m.internal.VistalinkCallQueue;
import gov.va.med.hds.cd.m.internal.XmlCacheCallQueue;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.InvalidConfigurationElementException;

import java.util.HashMap;
/**
 * Provides an implementation for MUMPS RPC call connectivity based on the RPC Shared Broker
 */
public class MumpsFactory implements IMumpsFactory, IPlugin {
	public static final String BROKER = "broker";
	public static final String VISTALINK = "vistalink";
	public static final String CACHE = "cache";
	public static final String DEFAULT_SERVER_NAME = "NOT CONNECTED TO SERVER: RUNNING FROM CACHE";
	public static final String RPC_ROOT_TAG = "rpc";
	public static final String RPC_KEY_TAG = "key";
	public static final String RPC_VALUE_TAG = "value";
	public static final String RPC_EXTENSTION_POINT = "gov.va.med.hds.cd.m." + RPC_ROOT_TAG;
	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
	 */
	public void shutdown() {
	}
	/**
	 * @see gov.va.med.hds.cd.runtime.IPlugin#startup(IRuntimeContext, IPluginDescriptor)
	 */
	public void startup(IRuntimeContext runtime, IPluginDescriptor descriptor) throws Exception {
		this.runtime = runtime;
	}
	/**
	 * @see gov.va.med.hds.cd.m.IMumpsFactory#getServerName()
	 */
	public String getServerName() {
		return serverName;
	}
	/**
	 * @see gov.va.med.hds.cd.m.IMumpsFactory#getServerPort()
	 */
	public int getServerPort() {
		return serverPort;
	}
	/**
	 * @see gov.va.med.hds.cd.m.IMumpsFactory#create(gov.va.med.hds.cd.runtime.IPluginDescriptor)
	 */
	public IMumps create(IPluginDescriptor descriptor) throws MumpsFactoryException {
		if (queue == null)
			try {
				createQueue();
			} catch (Exception e) {
				throw new MumpsFactoryException("Could not create a call queue", e);
			}
		HashMap rpcKeyValues = new HashMap();
		createRpcKeyValues(rpcKeyValues, descriptor);
		return new Mumps(queue, new OptionContext(descriptor), rpcKeyValues);
	}
	private void createRpcKeyValues(HashMap rpcKeyValues, IPluginDescriptor descriptor)
		throws PluginDescriptorException {
		IExtension[] extensions = descriptor.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IExtension ext = extensions[i];
			if (ext.getExtensionPointUniqueIdentifier().equals(RPC_EXTENSTION_POINT)) {
				IConfigurationElement[] elems = ext.getConfigurationElements();
				for (int j = 0; j < elems.length; j++) {
					IConfigurationElement elem = elems[j];
					if (elem.getName().equals(RPC_ROOT_TAG)) {
						String key = elem.getAttribute(RPC_KEY_TAG);
						if (key == null)
							throw new PluginDescriptorException(
								"In the plugin +"
									+ descriptor.getUniqueIdentifier()
									+ " the element "
									+ RPC_ROOT_TAG
									+ " must contain a "
									+ RPC_KEY_TAG
									+ " attribute.");

						String value = elem.getAttribute((RPC_VALUE_TAG));
						if (value == null)
							throw new PluginDescriptorException(
								"In the plugin +"
									+ descriptor.getUniqueIdentifier()
									+ " the element "
									+ RPC_ROOT_TAG
									+ " must contain a "
									+ RPC_VALUE_TAG
									+ " attribute.");
						rpcKeyValues.put(key, value);
					}
				}
			}
		}
	}
	private void createQueue()
		throws CallQueueException, CallQueueConfigException, InvalidConfigurationElementException {
		MumpsConnectionSettings settings;
		try {
			settings = new MumpsConnectionSettings(runtime.getSettings());
		} catch (InvalidSettingsException e) {
			throw new InvalidConfigurationElementException(e);
		}
		if (settings.getDriver().equals(IMumpsFactory.SETTINGS_CACHE)) {
			queue = new XmlCacheCallQueue(runtime, settings);
		} else if (settings.getDriver().equals(IMumpsFactory.SETTINGS_BROKER)) {
			queue = new BrokerCallQueue(runtime, settings);
			serverName = settings.getServer();
			serverPort = settings.getPort();
		} else {
			queue = new VistalinkCallQueue(runtime, settings);
			serverName = settings.getServer();
			serverPort = settings.getPort();
		}
		if(settings.isRpcTimeoutOverride()){
			queue.enableTimeLimitOverride(settings.getTimeoutOverride());
		}
	}

	private IRuntimeContext runtime;
	private ICallQueue queue = null;
	private String serverName = DEFAULT_SERVER_NAME;
	private int serverPort = 0;
}
