package gov.va.med.hds.cd.m.internal;

import gov.va.med.hds.cd.m.IMumpsConnectionSettings;
import gov.va.med.hds.cd.m.InvalidSettingsException;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.security.IPlugin;
import gov.va.med.hds.cd.security.PluginException;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

/**
 * Abstracts a security plugin for M based security
 */
abstract public class AbstractPlugin implements IPlugin {
	/**
	 * Default constructor
	 *
	 */
	public AbstractPlugin() {
	}
	/**
	 * Constructor
	 * @param runtime
	 * @param settings
	 * @throws InvalidSettingsException
	 */
	public AbstractPlugin(IRuntimeContext runtime, IMumpsConnectionSettings settings)
		throws InvalidSettingsException {
		server = settings.getServer();
		port = settings.getPort();
	}
	/**
	 * Creates a login context
	 * @see gov.va.med.hds.cd.security.IPlugin#createLoginContext()
	 */
	public LoginContext createLoginContext() throws PluginException {
		ClassLoader old = null;
		try {
			old = Thread.currentThread().getContextClassLoader();
			Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
			return new LoginContext(getLoginModuleKeyName());
		} catch (LoginException e) {
			throw new PluginException(e);
		} finally {
			if (old != null)
				Thread.currentThread().setContextClassLoader(old);
		}
	}
	abstract protected String getLoginModuleKeyName();

	protected String server;
	protected int port;
}
