package gov.va.med.hds.cd.m.internal;
import gov.va.med.foundations.utilities.FoundationsException;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.ConnectException;
import gov.va.med.hds.cd.m.CreateException;
import gov.va.med.hds.cd.m.EncryptException;
import gov.va.med.hds.cd.m.IBroker;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IConnectRequest;
import gov.va.med.hds.cd.m.SetContextException;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.security.UserCancelledAuthenticationException;
/**
 * Provides an RPC Shared Broker based implementation of ICallQueue.
 */
public class BrokerCallQueue extends CallQueueBase {
	private static final String BROKER_CALL_QUEUE_CLIENT_NAME = "BrokerCallQueue";

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#CallQueueBase(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement)
	 */
	public BrokerCallQueue(IRuntimeContext runtime, MumpsConnectionSettings settings)
		throws CallQueueException {
		super(runtime, settings);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#decrypt(java.lang.String)
	 */
	public String decrypt(String data) {
		return VistaKernelHash.decrypt(data);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#encrypt(java.lang.String)
	 */
	public String encrypt(String data) throws EncryptException {
		try {
			return VistaKernelHash.encrypt(data, false);
		} catch (FoundationsException e) {
			throw new EncryptException(e);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processConnectInit()
	 */
	protected void processConnectInit() throws CallQueueProcessConnectInitException {
		try {
			broker = Broker.getInstance();
		} catch (Exception e) {
			throw new CallQueueProcessConnectInitException(e);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processRequest(gov.va.med.hds.cd.m.ICallRequest)
	 */
	protected String processRequest(ICallRequest request) throws Exception {
		broker.setRPCTimeLimit(request.getCallTimeLimit());
		setContext(request.getContext());
		String result = broker.call(request.getCallName(), request.getParams());
		return (result == null ? "" : result);
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processConnect(gov.va.med.hds.cd.m.IConnectRequest)
	 */
	protected void processConnect(IConnectRequest connectRequest)
		throws ConnectException, UserCancelledAuthenticationException {
		BrokerConnectRequest request = (BrokerConnectRequest) connectRequest;
		if (broker == null)
			try {
				broker = Broker.getInstance();
			} catch (CreateException e) {
				throw new ConnectException(e);
			}
		if (request.getLogin() != null) {
			broker.setLogin(request.getLogin());
			broker.setPassword(request.getPassword());
		}
		broker.setClientName(BROKER_CALL_QUEUE_CLIENT_NAME);
		broker.setServerName(request.getServerName());
		broker.setServerPort(request.getServerPort());
		broker.setWantDebug(request.getWantDebug());
		broker.setConnectTimeLimit(connectRequest.getTimeLimit());
		broker.connect();
		setContext(DEFAULT_RPC_CONTEXT);
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processDisconnect()
	 */
	protected void processDisconnect() throws Exception {
		if (broker != null) {
			broker.dispose();
			broker = null;
		}
	}
	private void setContext(String context) throws SetContextException {
		if (context.equals(""))
			context = DEFAULT_RPC_CONTEXT;
		if (!lastContext.equals(context)) {
			broker.setContext(context);
			lastContext = context;
		}
	}
	private String lastContext = "";

	private IBroker broker = null;

}
