package gov.va.med.hds.cd.m.internal;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.MumpsFactoryException;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.security.ConnectionFailedException;
import gov.va.med.hds.cd.security.UserCancelledAuthenticationException;
import gov.va.med.hds.util.ExceptionStackTrace;
/**
 * Extends BrokerLoginModuleBase providing a connection where the SharedBroker asks
 * for user credentials
 */
public class BrokerLoginModule extends BrokerLoginModuleBase {
	/**
	 * Creates M connection
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#createConnections()
	 */
	protected void createConnections()
		throws ConnectionFailedException, UserCancelledAuthenticationException {
		try {
			IPluginDescriptor desc =
				runtime.getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
			IMumpsFactory factory =
				(IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID);
			mumps = factory.create(desc);
		} catch (MumpsFactoryException e) {
			throw new ConnectionFailedException(
				"Problem with mumps connection.\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginStartupException e) {
			throw new ConnectionFailedException(
				"Plugin would not startup.\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginNotFoundException e) {
			throw new ConnectionFailedException(
				"Plugin was not found.\n" + ExceptionStackTrace.getStackTrace(e));
		}
		try {
			((BrokerCallQueue) ((Mumps) mumps).getCallQueue()).connect(
				new BrokerConnectRequest(serverName, serverPort, wantProgrammerDebug));
		} catch (Exception e) {
			throw new ConnectionFailedException(
				e.getMessage() + "\n" + ExceptionStackTrace.getStackTrace(e));
		}
	}
}
