package gov.va.med.hds.cd.m.internal;

import gov.va.med.hds.cd.m.DisconnectException;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.security.AuthenticationFailedException;
import gov.va.med.hds.cd.security.DisconnectFailedException;
import gov.va.med.hds.util.ExceptionStackTrace;

import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;

import org.dom4j.DocumentException;
import org.jaxen.JaxenException;

/**
 * Abstract M based login module for Shared Broker implementations
 */
public abstract class BrokerLoginModuleBase extends LoginModuleBase {
	private static final String USER_DEMOGRAPHICS_RPC_NAME = "USER DEMOS";
	public static final String PROGRAMMER_DEBUG_KEY = "programmerDebug";

	/**
	 * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject, javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
	 */
	public void initialize(
		Subject subject,
		CallbackHandler callbackHandler,
		Map sharedState,
		Map options) {
		super.initialize(subject, callbackHandler, sharedState, options);
		Boolean want = (Boolean) options.get(PROGRAMMER_DEBUG_KEY);
		if (want != null)
			wantProgrammerDebug = want.booleanValue();
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#authenticate()
	 */
	protected boolean authenticate() throws AuthenticationFailedException {
		try {
			setupPrincipal();
		} catch (JaxenException e) {
			throw new AuthenticationFailedException(
				"XPath parsing failure\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (MumpsException e) {
			throw new AuthenticationFailedException(
				"Call queue problem\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (DocumentException e) {
			throw new AuthenticationFailedException(
				"XML Document problem\n" + ExceptionStackTrace.getStackTrace(e));
		}
		return true;
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#shutdownConnections()
	 */
	protected void shutdownConnections() throws DisconnectFailedException {
		if (mumps != null) {
			try {
				((BrokerCallQueue) (((Mumps) mumps).getCallQueue())).disconnect();
			} catch (DisconnectException e) {
				throw new DisconnectFailedException(
					"Disconnect Failure\n" + ExceptionStackTrace.getStackTrace(e));
			}
			mumps = null;
		}
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#getPrincipalClass()
	 */
	protected Class getPrincipalClass() {
		return BrokerMumpsPrincipal.class;
	}

	/**
	 * Method setupPrincipal.
	 * @throws Exception
	 */
	private void setupPrincipal() throws MumpsException, JaxenException, DocumentException {
		String result = mumps.callRPC(USER_DEMOGRAPHICS_RPC_NAME);
		BrokerMumpsPrincipal principal = new BrokerMumpsPrincipal(result);
		subject.getPrincipals().add(principal);
	}
	protected boolean wantProgrammerDebug = false;

}
