package gov.va.med.hds.cd.m.internal;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.MumpsFactoryException;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.security.ConnectionFailedException;
import gov.va.med.hds.cd.security.UserCancelledAuthenticationException;
import gov.va.med.hds.util.ExceptionStackTrace;
/**
 * Extends BrokerLoginModuleBase providing a silent access/verify code for integration testing
 */
public class IntegrationTestBrokerLoginModule extends BrokerLoginModuleBase {
	private static final String CODE_A = "jorel1";
	private static final String CODE_B = "5big.man";

	protected void createConnections()
		throws ConnectionFailedException, UserCancelledAuthenticationException {
		try {
			IPluginDescriptor desc =
				runtime.getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
			IMumpsFactory factory =
				(IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID);
			mumps = factory.create(desc);
		} catch (MumpsFactoryException e) {
			throw new ConnectionFailedException(
				"Problem with mumps connection\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginStartupException e) {
			throw new ConnectionFailedException(
				"Plugin would not startup\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginNotFoundException e) {
			throw new ConnectionFailedException(
				"Plugin was not found\n" + ExceptionStackTrace.getStackTrace(e));
		}
		try {
			((BrokerCallQueue) ((Mumps) mumps).getCallQueue()).connect(
				new BrokerConnectRequest(
					serverName,
					serverPort,
					CODE_A,
					CODE_B,
					wantProgrammerDebug));
		} catch (Exception e) {
			throw new ConnectionFailedException(
				"Attempt to connect failed\n" + ExceptionStackTrace.getStackTrace(e));
		}
	}
}
