package gov.va.med.hds.cd.m.internal;

import gov.va.med.hds.cd.m.IMumpsConnectionSettings;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.InvalidSettingsException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;

/**
 * Constructs a class form the runtime settings
 */
public class MumpsConnectionSettings implements IMumpsConnectionSettings {
	/**
	 * Constructor
	 * @param settings
	 * @throws InvalidSettingsException
	 */
	public MumpsConnectionSettings(IConfigurationElement[] settings)
		throws InvalidSettingsException {
		boolean foundIt = false;
		for (int i = 0; i < settings.length; i++) {
			if (settings[i].getName().equals(IMumpsFactory.SETTINGS_ROOT_ELEMENT_NAME)) {
				construct(settings[i]);
				foundIt = true;
				break;
			}
		}
		if (!foundIt)
			throw new InvalidSettingsException(
				"Missing " + IMumpsFactory.SETTINGS_ROOT_ELEMENT_NAME + " node in settings");
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#isDebug()
	 */
	public boolean isDebug() {
		return debug;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#getDriver()
	 */
	public String getDriver() {
		return driver;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#getPort()
	 */
	public int getPort() {
		return port;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#getServer()
	 */
	public String getServer() {
		return server;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#isSilent()
	 */
	public boolean isSilent() {
		return silent;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#isRecordAppend()
	 */
	public boolean isRecordAppend() {
		return recordAppend;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#getRecordFileName()
	 */
	public String getRecordFileName() {
		return recordFileName;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#isRecord()
	 */
	public boolean isRecord() {
		return record;
	}
	/**
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#isRpcTimeoutOverride()
	 */
	public boolean isRpcTimeoutOverride() {
		return useRpcTimeoutOverride;
	}
	/**
	 * @see gov.va.med.hds.cd.m.IMumpsConnectionSettings#getTimeoutOverride()
	 */
	public int getTimeoutOverride() {
		return rpcTimeoutOverride;
	}
	public boolean isUseRecordedCacheLatencies() {
		return useRecordedCacheLatencies;
	}
	public String[] getCacheResourceNames() {
		return cacheResourceNames;
	}
	private void construct(IConfigurationElement element) throws InvalidSettingsException {
		setupDriver(element);
		setupServer(element);
		setupPort(element);
		setupRecordParams(element);
		setupCacheParams(element);
		setupRpcTimeoutOverrideParams(element);
	}
	private void setupRpcTimeoutOverrideParams(IConfigurationElement rootElem)
		throws InvalidSettingsException {
		IConfigurationElement[] timeouts =
			rootElem.getChildren(IMumpsFactory.SETTINGS_RPC_TIMEOUT_OVERRIDE);
		if (timeouts.length == 0)
			return;
		if (timeouts.length > 1)
			throw new InvalidSettingsException(
				"There must be 1 or none "
					+ IMumpsFactory.SETTINGS_RPC_TIMEOUT_OVERRIDE
					+ " node in settings. There were "
					+ timeouts.length);
		String tval = timeouts[0].getValue();
		if (tval != null && tval.length() > 0) {
			int time = Integer.parseInt(timeouts[0].getValue());
			if (time > 0) {
				useRpcTimeoutOverride = true;
				rpcTimeoutOverride = time;
			}
		}
	}
	private void setupCacheParams(IConfigurationElement rootElem) throws InvalidSettingsException {
		IConfigurationElement[] params =
			rootElem.getChildren(IMumpsFactory.SETTINGS_CACHEPARAMS_ELEMENT_NAME);
		if (params == null || params.length == 0)
			return;
		if (params.length != 1)
			throw new InvalidSettingsException(
				"There must no more than 1 "
					+ IMumpsFactory.SETTINGS_CACHEPARAMS_ELEMENT_NAME
					+ " node in settings. There were "
					+ params.length);

		setupUseCacheLatencyFlag(params[0]);
		setupCacheResourceNames(params[0]);
	}
	private void setupCacheResourceNames(IConfigurationElement rootElem)
		throws InvalidSettingsException {
		IConfigurationElement[] resources =
			rootElem.getChildren(IMumpsFactory.SETTINGS_RESOURCES_ELEMENT_NAME);
		if (resources.length != 1)
			throw new InvalidSettingsException(
				"There must one and only 1 "
					+ IMumpsFactory.SETTINGS_RESOURCES_ELEMENT_NAME
					+ " node in settings. There were "
					+ resources.length);

		IConfigurationElement[] resourceNames =
			resources[0].getChildren(IMumpsFactory.SETTINGS_RESOURCE_NAME_ELEMENT_NAME);
		if (resources.length == 0)
			throw new InvalidSettingsException(
				"There must one or more "
					+ IMumpsFactory.SETTINGS_RESOURCE_NAME_ELEMENT_NAME
					+ " node in settings. There were "
					+ resourceNames.length);

		cacheResourceNames = new String[resourceNames.length];
		for (int i = 0; i < resourceNames.length; i++) {
			cacheResourceNames[i] = resourceNames[i].getValue();
		}
	}
	private void setupUseCacheLatencyFlag(IConfigurationElement element)
		throws InvalidSettingsException {
		IConfigurationElement[] latencyFlags =
			element.getChildren(IMumpsFactory.SETTINGS_USE_CALL_LATENCY_ELEMENT_NAME);
		if (latencyFlags == null || latencyFlags.length == 0)
			throw new InvalidSettingsException(
				"Missing "
					+ IMumpsFactory.SETTINGS_USE_CALL_LATENCY_ELEMENT_NAME
					+ " node in settings");
		if (latencyFlags.length != 1)
			throw new InvalidSettingsException(
				"There must be 1 and only 1 "
					+ IMumpsFactory.SETTINGS_USE_CALL_LATENCY_ELEMENT_NAME
					+ " node in settings. There were "
					+ latencyFlags.length);
		useRecordedCacheLatencies = Boolean.valueOf(latencyFlags[0].getValue()).booleanValue();
	}
	private void setupRecordParams(IConfigurationElement rootElem)
		throws InvalidSettingsException {
		IConfigurationElement[] params =
			rootElem.getChildren(IMumpsFactory.SETTINGS_RECORDPARAMS_ELEMENT_NAME);
		if (params == null || params.length == 0)
			return;
		if (params.length != 1)
			throw new InvalidSettingsException(
				"There must no more than 1 "
					+ IMumpsFactory.SETTINGS_RECORDPARAMS_ELEMENT_NAME
					+ " node in settings. There were "
					+ params.length);
		record = validateRecord(params[0]);
		recordFileName = validateFileName(params[0]);
		recordAppend = validateAppend(params[0]);
	}
	private boolean validateAppend(IConfigurationElement element) throws InvalidSettingsException {
		IConfigurationElement[] appends =
			element.getChildren(IMumpsFactory.SETTINGS_APPEND_ELEMENT_NAME);
		if (appends == null || appends.length == 0)
			throw new InvalidSettingsException(
				"Missing " + IMumpsFactory.SETTINGS_APPEND_ELEMENT_NAME + " node in settings");
		if (appends.length != 1)
			throw new InvalidSettingsException(
				"There must be 1 and only 1 "
					+ IMumpsFactory.SETTINGS_APPEND_ELEMENT_NAME
					+ " node in settings. There were "
					+ appends.length);
		return Boolean.valueOf(appends[0].getValue()).booleanValue();
	}
	private String validateFileName(IConfigurationElement element)
		throws InvalidSettingsException {
		IConfigurationElement[] filenames =
			element.getChildren(IMumpsFactory.SETTINGS_FILENAME_ELEMENT_NAME);
		if (filenames == null || filenames.length == 0)
			throw new InvalidSettingsException(
				"Missing " + IMumpsFactory.SETTINGS_FILENAME_ELEMENT_NAME + " node in settings");
		if (filenames.length != 1)
			throw new InvalidSettingsException(
				"There must be 1 and only 1 "
					+ IMumpsFactory.SETTINGS_FILENAME_ELEMENT_NAME
					+ " node in settings. There were "
					+ filenames.length);
		return filenames[0].getValue();
	}
	private boolean validateRecord(IConfigurationElement element) throws InvalidSettingsException {
		IConfigurationElement[] records =
			element.getChildren(IMumpsFactory.SETTINGS_RECORD_ELEMENT_NAME);
		if (records == null || records.length == 0)
			throw new InvalidSettingsException(
				"Missing " + IMumpsFactory.SETTINGS_RECORD_ELEMENT_NAME + " node in settings");
		if (records.length != 1)
			throw new InvalidSettingsException(
				"There must be 1 and only 1 "
					+ IMumpsFactory.SETTINGS_RECORD_ELEMENT_NAME
					+ " node in settings. There were "
					+ records.length);
		return Boolean.valueOf(records[0].getValue()).booleanValue();
	}
	private void setupPort(IConfigurationElement rootElem) throws InvalidSettingsException {
		IConfigurationElement[] ports =
			rootElem.getChildren(IMumpsFactory.SETTINGS_PORT_ELEMENT_NAME);
		if (ports.length == 0)
			return;
		if (ports.length > 1)
			throw new InvalidSettingsException(
				"There must be 1 or none "
					+ IMumpsFactory.SETTINGS_PORT_ELEMENT_NAME
					+ " node in settings. There were "
					+ ports.length);
		port = Integer.parseInt(ports[0].getValue());
	}
	private void setupServer(IConfigurationElement rootElem) throws InvalidSettingsException {
		IConfigurationElement[] servers =
			rootElem.getChildren(IMumpsFactory.SETTINGS_SERVER_ELEMENT_NAME);
		if (servers.length == 0)
			return;
		if (servers.length > 1)
			throw new InvalidSettingsException(
				"There must be 1 or none "
					+ IMumpsFactory.SETTINGS_SERVER_ELEMENT_NAME
					+ " node in settings. There were "
					+ servers.length);
		server = servers[0].getValue();
	}
	private void setupDriver(IConfigurationElement rootElem) throws InvalidSettingsException {
		IConfigurationElement[] drivers =
			rootElem.getChildren(IMumpsFactory.SETTINGS_DRIVER_ELEMENT_NAME);
		if (drivers == null || drivers.length == 0)
			throw new InvalidSettingsException(
				"Missing " + IMumpsFactory.SETTINGS_DRIVER_ELEMENT_NAME + " node in settings");
		if (drivers.length != 1)
			throw new InvalidSettingsException(
				"There must be 1 and only 1 "
					+ IMumpsFactory.SETTINGS_DRIVER_ELEMENT_NAME
					+ " node in settings. There were "
					+ drivers.length);
		IConfigurationElement driverElem = drivers[0];
		driver =
			validateDriverAttr(
				driverElem.getAttribute(IMumpsFactory.SETTINGS_CONNECTION_ATTR_NAME));
		silent = validateSilentAttr(driverElem.getAttribute(IMumpsFactory.SETTINGS_TEST_ATTR_NAME));
		debug = validateDebugAttr(driverElem.getAttribute(IMumpsFactory.SETTINGS_DEBUG_ATTR_NAME));
	}
	private boolean validateDebugAttr(String debugStr) {
		if (debugStr == null)
			return false;
		else
			return Boolean.valueOf(debugStr).booleanValue();
	}
	private boolean validateSilentAttr(String silentStr) {
		if (silentStr == null)
			return false;
		else
			return Boolean.valueOf(silentStr).booleanValue();
	}
	private String validateDriverAttr(String driverVal) throws InvalidSettingsException {
		if (driverVal.equals(IMumpsFactory.SETTINGS_VISTALINK)
			|| driverVal.equals(IMumpsFactory.SETTINGS_BROKER)
			|| driverVal.equals(IMumpsFactory.SETTINGS_CACHE))
			return driverVal;
		else
			throw new InvalidSettingsException(
				"The "
					+ IMumpsFactory.SETTINGS_CONNECTION_ATTR_NAME
					+ " attribute of the "
					+ IMumpsFactory.SETTINGS_DRIVER_ELEMENT_NAME
					+ " must be a \""
					+ IMumpsFactory.SETTINGS_VISTALINK
					+ "\", \""
					+ IMumpsFactory.SETTINGS_TEST_ATTR_NAME
					+ "\", or a \""
					+ IMumpsFactory.SETTINGS_DEBUG_ATTR_NAME
					+ "\"");
	}
	private boolean record = false;
	private boolean recordAppend = false;
	private boolean useRecordedCacheLatencies = false;
	private String[] cacheResourceNames = null;
	private String recordFileName = "";
	private String server = "";
	private int port = 0;
	private String driver = "";
	private boolean silent = false;
	private boolean debug = false;
	private boolean useRpcTimeoutOverride = false;
	private int rpcTimeoutOverride = 0;
}
