package gov.va.med.hds.cd.m.internal;

import gov.va.med.hds.cd.m.IMumpsConnectionSettings;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.InvalidSettingsException;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.security.IPlugin;
import gov.va.med.hds.cd.security.IPluginFactory;
import gov.va.med.hds.cd.security.PluginCreationFailedException;

/**
 * Satisfies the security extension 
 */
public class SecurityPluginFactory implements IPluginFactory {
	public IPlugin create(IRuntimeContext runtime) throws PluginCreationFailedException {
		try {
			IMumpsConnectionSettings settings = new MumpsConnectionSettings(runtime.getSettings());
			return createPlugin(runtime, settings);
		} catch (InvalidSettingsException e) {
			throw new PluginCreationFailedException(e);
		}

	}

	/**
	 * Creates the security plugin
	 * @param runtime
	 * @param settings
	 * @return
	 * @throws InvalidSettingsException
	 */
	private IPlugin createPlugin(IRuntimeContext runtime, IMumpsConnectionSettings settings)
		throws InvalidSettingsException {
		if (settings.getDriver().equals(IMumpsFactory.SETTINGS_CACHE)) {
			return new SilentXmlCachePlugin(runtime, settings);
		} else if (settings.getDriver().equals(IMumpsFactory.SETTINGS_BROKER)) {
			if (settings.isSilent()) {
				return new IntegrationTestBrokerPlugin(runtime, settings);
			} else {
				return new BrokerPlugin(runtime, settings);
			}
		} else {
			if (settings.isSilent()) {
				return new IntegrationTestVistalinkPlugin(runtime, settings);
			} else {
				return new VistalinkPlugin(runtime, settings);
			}
		}
	}
}
