package gov.va.med.hds.cd.m.internal;

import java.awt.ContainerOrderFocusTraversalPolicy;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

/**
 * User credentials collecting dialog
 */
public class UserCredentialsDialog extends JDialog {

	private static final String DEFAULT_TITLE = "VistA Sign-on";
	private static final String ACCESS_LABEL = "Access Code:";
	private static final String VERIFY_LABEL = "Verify Code:";
	private static final int CODE_FIELD_COLUMNS = 13;
	private static final String VA_LOGO = "VAlogo.gif";
	private static final String OK_BUTTON_LABEL = "OK";
	private static final char OK_BUTTON_MNEMONIC = 'O';
	private static final String CANCEL_BUTTON_LABEL = "Cancel";
	private static final char CANCEL_BUTTON_MNEMONIC = 'C';
	private static final String PORT_LABEL = "Port:";
	private static final String UCI_LABEL = "UCI:";
	private static final String VOLUME_LABEL = "Volume:";
	private static final String SERVER_LABEL = "Server:";
	private static final int PREFERRED_TEXT_AREA_HEIGHT = 135;

	/**
	 * Shows the user credentials dialog
	 * @param parent
	 * @param serverInfo
	 * @return
	 */
	public static UserCredentials showUserCredentialsDialog(Frame parent, ServerInfo serverInfo) {
		UserCredentialsDialog dialog = new UserCredentialsDialog(parent, serverInfo);
		dialog.show();
		return dialog.getUserCredentials();
	}

	private JLabel server;
	private JLabel volume;
	private JLabel uci;
	private JLabel port;
	private JPasswordField access = new JPasswordField("", CODE_FIELD_COLUMNS);
	private JPasswordField verify = new JPasswordField("", CODE_FIELD_COLUMNS);
	private boolean credentialsSet = false;

	private UserCredentialsDialog(Frame parent, ServerInfo serverInfo) {
		super(parent, DEFAULT_TITLE, true);
		createServerInfoLabels(serverInfo);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add(createInfoPane());
		getContentPane().add(createContentPane());
		getContentPane().add(createServerInfoPanel());
		pack();
		centerDialog();
		access.requestFocus();
	}
	private void createServerInfoLabels(ServerInfo serverInfo) {
		if (serverInfo == null) {
			server = new JLabel("TBD");
			volume = new JLabel("TBD");
			uci = new JLabel("TBD");
			port = new JLabel("TBD");
		} else {
			server = new JLabel(serverInfo.getServer());
			volume = new JLabel(serverInfo.getVolume());
			uci = new JLabel(serverInfo.getUci());
			port = new JLabel(serverInfo.getPort());
		}
	}

	private void centerDialog() {
		Dimension screenSize = getToolkit().getScreenSize();
		int x = (screenSize.width - getWidth()) / 2;
		int y = (screenSize.height - getHeight()) / 2;
		setLocation(x, y);
	}

	private JComponent createInfoPane() {
		JTextArea info = new JTextArea("\nTBD: Need the text returned from the connection here\n");
		info.setEditable(false);
		JScrollPane scrollPane = new JScrollPane(info);
		Border bevelOut = BorderFactory.createBevelBorder(BevelBorder.RAISED);
		Border bevelIn = BorderFactory.createBevelBorder(BevelBorder.LOWERED);
		scrollPane.setBorder(BorderFactory.createCompoundBorder(bevelOut, bevelIn));
		Dimension prefSize = scrollPane.getPreferredSize();
		prefSize.height = PREFERRED_TEXT_AREA_HEIGHT;
		scrollPane.setPreferredSize(prefSize);
		return scrollPane;
	}

	private JComponent createContentPane() {
		GridBagLayout gridbag = new GridBagLayout();

		JPanel p = new JPanel();
		p.setFocusCycleRoot(true);
		p.setFocusTraversalPolicy(new ContainerOrderFocusTraversalPolicy());
		p.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(6, 6, 6, 6);
		c.ipadx = 3;
		c.ipady = 3;

		c.gridx = c.gridy = 0;
		c.gridheight = 2;
		p.add(createLogoLabel(), c);

		c.gridheight = 1;
		c.gridx = 1;
		c.gridy = 0;
		p.add(createAccessLabel(), c);

		c.gridy = 1;
		p.add(createVerifyLabel(), c);

		c.gridx = 2;
		c.gridy = 0;
		p.add(access, c);

		c.gridy = 1;
		p.add(verify, c);

		c.gridx = 3;
		c.gridy = 0;
		p.add(createOKButton(), c);

		c.gridy = 1;
		p.add(createCancelButton(), c);

		return p;
	}

	private JButton createCancelButton() {
		JButton cancel = new JButton(new CancelAction());
		cancel.setFont(cancel.getFont().deriveFont(Font.BOLD));
		cancel.setDefaultCapable(false);
		return cancel;
	}

	private JButton createOKButton() {
		JButton ok = new JButton(new OKAction());
		ok.setFont(ok.getFont().deriveFont(Font.BOLD));
		this.getRootPane().setDefaultButton(ok);
		return ok;
	}

	private JLabel createVerifyLabel() {
		JLabel l = new JLabel(VERIFY_LABEL);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		l.setFont(l.getFont().deriveFont(Font.BOLD));
		l.setFocusable(false);
		return l;
	}

	private JLabel createAccessLabel() {
		JLabel l = new JLabel(ACCESS_LABEL);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		l.setFont(l.getFont().deriveFont(Font.BOLD));
		l.setFocusable(false);
		return l;
	}

	private JComponent createServerInfoPanel() {
		JPanel p = new JPanel();
		p.setLayout(new GridLayout(1, 4));
		p.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		p.add(createServerInfoSubPanel(SERVER_LABEL, server));
		p.add(createServerInfoSubPanel(VOLUME_LABEL, volume));
		p.add(createServerInfoSubPanel(UCI_LABEL, uci));
		p.add(createServerInfoSubPanel(PORT_LABEL, port));
		return p;
	}

	private JPanel createServerInfoSubPanel(String label, JLabel valueLabel) {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
		JLabel l = new JLabel(label);
		valueLabel.setFont(valueLabel.getFont().deriveFont(Font.BOLD));
		valueLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		p.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		p.add(l);
		p.add(valueLabel);
		return p;
	}

	private UserCredentials getUserCredentials() {
		if (!credentialsSet)
			return null;
		return new UserCredentials(
			new String(access.getPassword()),
			new String(verify.getPassword()));
	}

	private JLabel createLogoLabel() {
		JLabel l = new JLabel(new ImageIcon(UserCredentialsDialog.class.getResource(VA_LOGO)));
		l.setFocusable(false);
		return l;
	}

	private class OKAction extends AbstractAction {

		public OKAction() {
			putValue(Action.NAME, OK_BUTTON_LABEL);
			putValue(Action.MNEMONIC_KEY, new Integer(OK_BUTTON_MNEMONIC));
		}

		public void actionPerformed(ActionEvent e) {
			credentialsSet = true;
			hide();
		}

	}

	private class CancelAction extends AbstractAction {

		public CancelAction() {
			putValue(Action.NAME, CANCEL_BUTTON_LABEL);
			putValue(Action.MNEMONIC_KEY, new Integer(CANCEL_BUTTON_MNEMONIC));
		}

		public void actionPerformed(ActionEvent e) {
			credentialsSet = false;
			hide();
		}

	}
}
