package gov.va.med.hds.cd.m.internal;
import gov.va.med.foundations.adapter.cci.VistaLinkConnection;
import gov.va.med.foundations.adapter.record.VistaLinkFaultException;
import gov.va.med.foundations.rpc.RpcRequest;
import gov.va.med.foundations.rpc.RpcRequestFactory;
import gov.va.med.foundations.rpc.RpcResponse;
import gov.va.med.foundations.security.vistalink.VistaKernelPrincipalImpl;
import gov.va.med.foundations.utilities.FoundationsException;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.EncryptException;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IConnectRequest;
import gov.va.med.hds.cd.m.SetContextException;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
/**
 * Implements an ICallQueue with the VistALink Mumps services
 */
public class VistalinkCallQueue extends CallQueueBase {

	/**
	 * Constructor
	 * @param settings
	 * @throws CallQueueException
	 */
	public VistalinkCallQueue(IRuntimeContext runtime, MumpsConnectionSettings settings)
		throws CallQueueException {
		super(runtime, settings);
	}
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#decrypt(java.lang.String)
	 */
	public String decrypt(String data) {
		return VistaKernelHash.decrypt(data);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#encrypt(java.lang.String)
	 */
	public String encrypt(String data) throws EncryptException {
		try {
			return VistaKernelHash.encrypt(data, false);
		} catch (FoundationsException e) {
			throw new EncryptException(e);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processConnectInit()
	 */
	protected void processConnectInit() {
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processRequest(gov.va.med.hds.cd.m.ICallRequest)
	 */
	protected String processRequest(ICallRequest request) throws Exception {
		String result = null;
		RpcRequest vistaRequest = null;
		setContext(request.getContext());
		vistaRequest = RpcRequestFactory.getRpcRequest(lastContext, request.getCallName());
		// M can only deal with seconds		
		vistaRequest.setRpcClientTimeOut((int) request.getCallTimeLimit() / 1000);

		if (request.getParams() != null)
			vistaRequest.setParams(request.getParams());

		RpcResponse rpcResponse;
		try {
			rpcResponse = connection.executeRPC(vistaRequest);
		} catch (VistaLinkFaultException e) {
			throw e;
		} catch (FoundationsException e) {
			throw e;
		}
		return rpcResponse.getResults();
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processConnect(gov.va.med.hds.cd.m.IConnectRequest)
	 */
	protected void processConnect(IConnectRequest connectRequest)
		throws SetContextException { //throws ConnectException, UserCancelledAuthenticationException, {
		principal = ((VistalinkConnectRequest) connectRequest).getPrincipal();
		connection = principal.getAuthenticatedConnection();
		setContext(DEFAULT_RPC_CONTEXT);
	}

	/**
	 * @see gov.va.med.hds.cd.m.internal.CallQueueBase#processDisconnect()
	 */
	protected void processDisconnect() {
		connection = null;
	}
	private void setContext(String context) {
		if (context.equals(""))
			context = DEFAULT_RPC_CONTEXT;
		if (!lastContext.equals(context)) {
			lastContext = context;
		}
	}
	private VistaLinkConnection connection;
	private VistaKernelPrincipalImpl principal;
	private String lastContext = "";
}
