package gov.va.med.hds.cd.m.internal;

import java.io.StringReader;
import java.security.Principal;

import gov.va.med.foundations.security.vistalink.VistaKernelPrincipal;
import gov.va.med.hds.cd.m.IMumpsUserInfo;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jaxen.JaxenException;
import org.jaxen.dom4j.Dom4jXPath;

/**
 * Vistalink based security principal
 */
class VistalinkMumpsPrincipal implements Principal, IMumpsUserInfo {
	VistalinkMumpsPrincipal(String data) throws DocumentException, JaxenException {
		SAXReader reader = new SAXReader();
		Document doc = reader.read(new StringReader(data));
		duz =
			((Node) (new Dom4jXPath("result/userInfo/uniqueId")).selectSingleNode(doc))
				.getStringValue();
		first =
			((Node) (new Dom4jXPath("result/userInfo/firstName")).selectSingleNode(doc))
				.getStringValue();
		last =
			((Node) (new Dom4jXPath("result/userInfo/lastName")).selectSingleNode(doc))
				.getStringValue();
		middle =
			((Node) (new Dom4jXPath("result/userInfo/middleInitial")).selectSingleNode(doc))
				.getStringValue();
		title =
			((Node) (new Dom4jXPath("result/userInfo/title")).selectSingleNode(doc))
				.getStringValue();
	}
	VistalinkMumpsPrincipal(VistaKernelPrincipal principal) {
		duz = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_DUZ);
		first = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_GIVENFIRST);
		last = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_FAMILYLAST);
		middle = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_MIDDLE);
		title = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_TITLE);
		divisionIEN = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_DIVISION_IEN);
		divisionStationName =
			principal.getUserDemographicValue(VistaKernelPrincipal.KEY_DIVISION_STATION_NAME);
		divisionStationNumber =
			principal.getUserDemographicValue(VistaKernelPrincipal.KEY_DIVISION_STATION_NUMBER);
		dTime = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_DTIME);
		newPerson01 = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_NEWPERSON01);
		prefix = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_PREFIX);
		suffix = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_NAME_SUFFIX);
		language = principal.getUserDemographicValue(VistaKernelPrincipal.KEY_LANGUAGE);
		serviceSection =
			principal.getUserDemographicValue(VistaKernelPrincipal.KEY_SERVICE_SECTION);
	}
	/**
	 * 
	 * @see java.security.Principal#getName()
	 */
	public String getName() {
		return name;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getDuz()
	 */
	public String getDuz() {
		return duz;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getFirstName()
	 */
	public String getFirstName() {
		return first;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getLastName()
	 */
	public String getLastName() {
		return last;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getMiddleName()
	 */
	public String getMiddleName() {
		return middle;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getTitle()
	 */
	public String getTitle() {
		return title;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getDivisionIEN()
	 */
	public String getDivisionIEN() {
		return divisionIEN;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getDivisionStationName()
	 */
	public String getDivisionStationName() {
		return divisionStationName;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getDivisionStationNumber()
	 */
	public String getDivisionStationNumber() {
		return divisionStationNumber;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getDTime()
	 */
	public String getDTime() {
		return dTime;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.m.IMumpsUserInfo#getNewPerson01()
	 */
	public String getNewPerson01() {
		return newPerson01;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getPrefix()
	 */
	public String getPrefix() {
		return prefix;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getSuffix()
	 */
	public String getSuffix() {
		return suffix;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getLanguage()
	 */
	public String getLanguage() {
		return language;
	}
	/**
	 * 
	 * @see gov.va.med.hds.cd.security.IUserInfo#getServiceSection()
	 */
	public String getServiceSection() {
		return serviceSection;
	}
	private String name = VistalinkMumpsPrincipal.class.getName();
	private String duz, first, last, middle, title;
	private String divisionIEN, divisionStationName;
	private String divisionStationNumber, dTime, newPerson01;
	private String prefix, suffix, language, serviceSection;
}
