package gov.va.med.hds.cd.m.internal;
import gov.va.med.foundations.utilities.FoundationsException;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.hds.cd.m.CallDoesNotExistInCacheException;
import gov.va.med.hds.cd.m.CallQueueConfigException;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.CallRequestCacheException;
import gov.va.med.hds.cd.m.EncryptException;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IConnectRequest;
import gov.va.med.hds.cd.m.IMumpsUserInfo;
import gov.va.med.hds.cd.m.TimedResult;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

/**
 * Implements an ICallQueue with the XmlCache services
 */
public class XmlCacheCallQueue extends CallQueueBase {
	public static final String CACHE_PARAMS_ELEMENT_NAME = "cacheParams";
	public static final String RESOURCE_NAME_ELEMENT_NAME = "resourceName";
	public static final String WAIT_ON_CALL_TIMES_ATTR_NAME = "waitOnCallTimes";

	/**
	 * Constructor
	 * 
	 * @param runtime
	 * @param settings
	 * @throws CallQueueException
	 * @throws CallQueueConfigException
	 */
	public XmlCacheCallQueue(IRuntimeContext runtime, MumpsConnectionSettings settings)
		throws CallQueueException, CallQueueConfigException {
		super(runtime, settings);
		setupCache(settings);
	}
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#decrypt(java.lang.String)
	 */
	public String decrypt(String data) {
		return VistaKernelHash.decrypt(data);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.m.ICallQueue#encrypt(java.lang.String)
	 */
	public String encrypt(String data) throws EncryptException {
		try {
			return VistaKernelHash.encrypt(data, false);
		} catch (FoundationsException e) {
			throw new EncryptException(e);
		}
	}

	IMumpsUserInfo getMumpsUserInfo() {
		return cache.retrieveUserInfo();
	}
	private void setupCache(MumpsConnectionSettings settings) throws CallQueueConfigException {
		try {
			cache = new CallRequestCache(runtime);
		} catch (CallRequestCacheException e) {
			throw new CallQueueConfigException(e);
		}
		waitOnCallTimes = settings.isUseRecordedCacheLatencies();

		if (settings.getCacheResourceNames() != null) {
			String resourcePath = null;
			for (int i = 0; i < settings.getCacheResourceNames().length; i++) {
				resourcePath = settings.getCacheResourceNames()[i];
				//				Need to fix this validation					
				//				try {
				//					InputStream xml = getClass().getResourceAsStream(resourcePath);
				//					cache.validate(xml);
				//				} catch (ValidationFailedException e) {
				//					throw new CallQueueConfigException(
				//						"The resource "
				//							+ resourcePath
				//							+ " does not validate.",
				//						e);
				//				}
				try {
					cache.load(getClass().getResourceAsStream(resourcePath));
				} catch (CallRequestCacheException e) {
					throw new CallQueueConfigException(e);
				}
			}
		}
	}
	protected void processConnectInit() {
	}
	protected String processRequest(ICallRequest request) throws Exception {
		TimedResult result = cache.getResult(request);
		if (result == null) {
			throw new CallDoesNotExistInCacheException(
				"The call " + request.getCallName() + " does not exist in the cache");
		}
		if (waitOnCallTimes && result.elapsedTime > 0)
			synchronized (this) {
				wait(result.elapsedTime);
			}
		return result.result;
	}
	protected void processConnect(IConnectRequest connectRequest) {
	}
	protected void processDisconnect() {
	}

	private CallRequestCache cache = null;
	private boolean waitOnCallTimes = false;
	private IRuntimeContext runtime;
}
