package gov.va.med.hds.cd.m.internal;

import gov.va.med.hds.cd.m.DisconnectException;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.MumpsFactoryException;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.security.ConnectionFailedException;
import gov.va.med.hds.cd.security.DisconnectFailedException;
import gov.va.med.hds.util.ExceptionStackTrace;

/**
 * Abstract implementation for XML Cache based login modules
 */
abstract class XmlCacheLoginModuleBase extends LoginModuleBase {
	private static final int CONNECT_TIME_LIMIT_IN_MILLIS = 2000;
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#shutdownConnections()
	 */
	protected void shutdownConnections() throws DisconnectFailedException {
		if (mumps != null) {
			try {
				((XmlCacheCallQueue) (((Mumps) mumps).getCallQueue())).disconnect();
			} catch (DisconnectException e) {
				throw new DisconnectFailedException(
					"Disconnect Failure.\n" + ExceptionStackTrace.getStackTrace(e));
			}
			mumps = null;
		}
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#createConnections()
	 */
	protected void createConnections() throws ConnectionFailedException {
		try {
			IPluginDescriptor desc =
				runtime.getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
			IMumpsFactory factory =
				(IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID);
			mumps = factory.create(desc);
		} catch (MumpsFactoryException e) {
			throw new ConnectionFailedException(
				"Problem with mumps connection.\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginStartupException e) {
			throw new ConnectionFailedException(
				"Plugin would not startup.\n" + ExceptionStackTrace.getStackTrace(e));
		} catch (PluginNotFoundException e) {
			throw new ConnectionFailedException(
				"Plugin was not found.\n" + ExceptionStackTrace.getStackTrace(e));
		}
		try {
			((XmlCacheCallQueue) ((Mumps) mumps).getCallQueue()).connect(
				new XmlCacheConnectRequest(CONNECT_TIME_LIMIT_IN_MILLIS));
		} catch (Exception e) {
			throw new ConnectionFailedException(
				"Attempt to connect failed.\n" + ExceptionStackTrace.getStackTrace(e));
		}
	}
	/**
	 * @see gov.va.med.hds.cd.m.internal.LoginModuleBase#getPrincipalClass()
	 */
	protected Class getPrincipalClass() {
		return XmlCachePrincipal.class;
	}
	/**
	 * Makes a RPC call which does an authenticate. The user identifier (DUZ)
	 * value is stripped out of a successfull authenticate call and set in the
	 * XmlCallQueue.
	 * @param login
	 * @param password
	 * @return boolean
	 */
	protected boolean callAuthenticate(String login, String password) {
		subject.getPrincipals().add(
			new XmlCachePrincipal(
				((XmlCacheCallQueue) ((Mumps) mumps).getCallQueue()).getMumpsUserInfo()));
		return true;
	}
}
