#include <jni.h>
#include "Connector.h"
#include <objbase.h>
#include <assert.h>
#include <winerror.h>
#include <comdef.h>

#import "RPCSharedBrokerSessionMgr.tlb"
using namespace RPCSharedBrokerSessionMgr1;

char* ErrorCodeToText(long errorCode);

static const char* ConnectorExceptionClassName    = "gov/va/med/hds/cd/m/internal/broker/jni/ConnectorException";
static const char* ComExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/COMException";
static const char* GeneralFailureExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/GeneralFailureException";
static const char* UniqueRpcIdDoesNotExistExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/UniqueRpcIdDoesNotExistException";
static const char* UniqueClientIdDoesNotExistExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/UniqueClientIdDoesNotExistException";
static const char* RpcHistoryIndexOutOfRangeExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/RpcHistoryIndexOutOfRangeException";
static const char* RpcHistoryNotEnabledExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/RpcHistoryNotEnabledException";
static const char* CouldNotSetContextExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/CouldNotSetContextException";
static const char* CouldNotConnectExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/CouldNotConnectException";
static const char* ClientIndexOutOfRangeExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/ClientIndexOutOfRangeException";
static const char* NilClientPointerExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/NilClientPointerException";
static const char* ConnectionIndexOutOfRangeExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/ConnectionIndexOutOfRangeException";
static const char* ConnectorNotInitializedExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/ConnectorNotInitializedException";
static const char* ConnectorNotConnectedExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/ConnectorNotConnectedException";
static const char* UserCancelledAuthenticationExceptionClassName = "gov/va/med/hds/cd/m/internal/broker/jni/UserCancelledAuthenticationException";
static const BSTR UserCancelledMessageStr = L"User Cancelled Login Process";

static ISharedBrokerPtr pSharedBroker=NULL;
static bool isConnected = false;
static bool isInitialized = false;

void ThrowConnectorException(JNIEnv *env,char *message)
{
	jclass expClass = env->FindClass(ConnectorExceptionClassName);
	if(expClass != NULL)
		env->ThrowNew(expClass,message);
	env->DeleteLocalRef(expClass);
}
void ThrowConnectorNotInitializedException(JNIEnv *env,char *message)
{
	jclass expClass = env->FindClass(ConnectorNotInitializedExceptionClassName);
	if(expClass != NULL)
		env->ThrowNew(expClass,message);
	env->DeleteLocalRef(expClass);
}
void ThrowConnectorNotConnectedException(JNIEnv *env,char *message)
{
	jclass expClass = env->FindClass(ConnectorNotConnectedExceptionClassName);
	if(expClass != NULL)
		env->ThrowNew(expClass,message);
	env->DeleteLocalRef(expClass);
}

void ThrowCOMException(JNIEnv *env, char *excStr, HRESULT hresult)
{
	char message[256];
	sprintf(message,"%s HRESULT: %X",excStr,hresult);
	jclass expClass = env->FindClass(ComExceptionClassName);
	if(expClass != NULL)
		env->ThrowNew(expClass,message);
	env->DeleteLocalRef(expClass);
}
void ThrowOnConnectError(JNIEnv *env,BSTR loginStr,long errorCode){
	char *exceptionClassName = NULL;

	switch(errorCode)
	{
	case CouldNotConnect :
		if(wcsstr(loginStr,UserCancelledMessageStr)!=NULL){
			exceptionClassName = (char*)UserCancelledAuthenticationExceptionClassName;
		}else{
			exceptionClassName = (char*)CouldNotConnectExceptionClassName;
		}
		break;
	default:
		exceptionClassName = (char*)GeneralFailureExceptionClassName;
		break;
	}
	if(errorCode != Success)
	{
		char* errorCodeStr = ErrorCodeToText(errorCode);
		char* mess = new char[strlen(errorCodeStr)+40];
		sprintf(mess,"ErrorCode %s",errorCodeStr);
		jclass expClass = env->FindClass(exceptionClassName);
		if(expClass != NULL)
			env->ThrowNew(expClass,mess);
		env->DeleteLocalRef(expClass);
		delete mess;
	}
}
void ThrowOnError(JNIEnv *env,char *message,long errorCode)
{
	char *exceptionClassName = NULL;

	switch(errorCode)
	{
	case UniqueRpcIdDoesNotExist:
		exceptionClassName = (char*)UniqueRpcIdDoesNotExistExceptionClassName;
		break;
	case UniqueClientIdDoesNotExist:
		exceptionClassName = (char*)UniqueClientIdDoesNotExistExceptionClassName;
		break;
	case RpcHistoryIndexOutOfRange:
		exceptionClassName = (char*)RpcHistoryIndexOutOfRangeExceptionClassName;
		break;
	case RpcHistoryNotEnabled:
		exceptionClassName = (char*)RpcHistoryNotEnabledExceptionClassName;
		break;
	case CouldNotSetContext:
		exceptionClassName = (char*)CouldNotSetContextExceptionClassName;
		break;
	case ClientIndexOutOfRange:
		exceptionClassName = (char*)ClientIndexOutOfRangeExceptionClassName;
		break;
	case NilClientPointer:
		exceptionClassName = (char*)NilClientPointerExceptionClassName;
		break;
	case ConnectionIndexOutOfRange:
		exceptionClassName = (char*)ConnectionIndexOutOfRangeExceptionClassName;
		break;
	default:
		exceptionClassName = (char*)GeneralFailureExceptionClassName;
		break;
	}
	if(errorCode != Success)
	{
		char* errorCodeStr = ErrorCodeToText(errorCode);
		char* mess = new char[strlen(message)+strlen(errorCodeStr)+40];
		sprintf(mess,"%s, %s",errorCodeStr,message);
		jclass expClass = env->FindClass(exceptionClassName);
		if(expClass != NULL)
			env->ThrowNew(expClass,mess);
		env->DeleteLocalRef(expClass);
		delete mess;
	}
}
void printHRESULT(BSTR message,HRESULT hr)
{
	wprintf(L"MESSAGE: %s HRESULT: %X   SUCCEEDED: %d   CODE: %X   FACILITY: %X   SEVERITY: %X\n",
		message,
		hr,
		SUCCEEDED(hr),
		HRESULT_CODE(hr),
		HRESULT_FACILITY(hr),
		HRESULT_SEVERITY(hr));
}
char* ErrorCodeToText(long errorCode)
{
	switch(errorCode)
	{
	case GeneralFailure:
		return "General Failure";
	case Success:
		return "Success";
	case UniqueRpcIdDoesNotExist:
		return "Unique Rpc Id Does Not Exist";
	case UniqueClientIdDoesNotExist:
		return "Unique Client Id Does Not Exist";
	case RpcHistoryIndexOutOfRange:
		return "Rpc History Index Out Of Range";
	case RpcHistoryNotEnabled:
		return "Rpc History Not Enabled";
	case CouldNotSetContext:
		return "Could Not Set OPTION";
	case CouldNotConnect :
		return "Could Not Connect";
	case ClientIndexOutOfRange:
		return "Client Index Out Of Range";
	case NilClientPointer:
		return "Null Client Pointer";
	case ConnectionIndexOutOfRange:
		return "Connection Index Out Of Range";
	default:
		return "Invalid Error Code";
	}
}

void ReportAdditionalConnectResults(JNIEnv *env, jobject obj, int uniqueClientId)
{
	jclass cls = env->GetObjectClass(obj);
	jmethodID mid = env->GetMethodID(cls, "ReportAdditionalConnectResults", "(I)V");
	if (mid == 0) ThrowConnectorException(env,"Could not find ReportAdditionalConnectResults method on Connector class\n");
	env->CallVoidMethod(obj, mid,uniqueClientId);
}

void ReportAdditionalCallResults(JNIEnv *env, jobject obj, long uniqueRPCId)
{
	jclass cls = env->GetObjectClass(obj);
	jmethodID mid = env->GetMethodID(cls, "ReportAdditionalCallResults", "(I)V");
	if (mid == 0) ThrowConnectorException(env,"Could not find ReportAdditionalCallResults method on Connector class\n");
	env->CallVoidMethod(obj, mid, uniqueRPCId);
}
JNIEXPORT void JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_initialize
(JNIEnv *env, jobject)
{
	HRESULT hr;

	hr = CoInitialize(NULL);
	if(!SUCCEEDED(hr))
		ThrowCOMException(env,"CoInitialize failed",hr);
	LPOLESTR lpolestr;
	StringFromCLSID(__uuidof(SharedBroker), &lpolestr);
	pSharedBroker.CreateInstance(lpolestr);
	CoTaskMemFree(lpolestr);
	isInitialized = true;
}
JNIEXPORT void JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_unInitialize
(JNIEnv *env, jobject)
{
	if(!isInitialized)
	{
		ThrowConnectorNotInitializedException(env,"Should have been initialized first");
		return;
	}
	long errorCode = pSharedBroker->BrokerDisconnect();
	if(errorCode != Success)
		ThrowOnError(env,"unInitialize error",errorCode);
	pSharedBroker.Release();
	CoUninitialize();
	isInitialized = false;
}
JNIEXPORT void JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_connect
(JNIEnv *env, jobject obj, jstring clientName, jstring serverPort, jint timeLimit, jboolean kernelLogin, jstring loginStr, jboolean wantDebug)
{
	if(!isInitialized)
	{
		ThrowConnectorNotInitializedException(env,"Should have been initialized first");
		return;
	}
	if(isConnected)
		Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_disconnect(env, obj);

	BSTR name = NULL;
	BSTR servPort = NULL;
	BSTR loginString = NULL;

	if(clientName != NULL && env->GetStringLength(clientName) > 0)
		name = (BSTR)env->GetStringChars(clientName,0);
	if(serverPort != NULL && env->GetStringLength(serverPort) > 0)
		servPort = (BSTR)env->GetStringChars(serverPort,0);
	if(loginStr != NULL && env->GetStringLength(loginStr) > 0)
	{
		const jchar *ls = env->GetStringChars(loginStr,0);
		SysReAllocString(&loginString, ls);
		env->ReleaseStringChars(loginStr, (jchar*)ls);
	}

	BSTR errorMsg = NULL;
	int uniqueClientId = 0;

	const bool allowShared = true;
	const long showErrorMsg = isemQuiet;

	long errorCode;
	try{
		errorCode = pSharedBroker->BrokerConnect(name,0,servPort,wantDebug,allowShared,kernelLogin,showErrorMsg,timeLimit,&loginString,&uniqueClientId,&errorMsg);
	}catch(...){
		ThrowConnectorException(env,"There was an unexplained error with COM call BrokerConnect");

		env->ReleaseStringChars(clientName, (jchar*)name);
		env->ReleaseStringChars(serverPort, (jchar*)servPort);

		if(loginString != NULL)
			::SysFreeString(loginString);
		if(errorMsg != NULL)
			::SysFreeString(errorMsg);
		return;
	}
	if(errorCode != Success)
	{
		ThrowOnConnectError(env,loginString,errorCode);
	}
	else
	{
		isConnected = true;
		ReportAdditionalConnectResults(env,obj,uniqueClientId);
	}
	env->ReleaseStringChars(clientName, (jchar*)name);
	env->ReleaseStringChars(serverPort, (jchar*)servPort);

	if(loginString != NULL)
		::SysFreeString(loginString);
	if(errorMsg != NULL)
		::SysFreeString(errorMsg);
}
JNIEXPORT void JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_disconnect
(JNIEnv *env, jobject)
{
	if(!isInitialized)
	{
		ThrowConnectorNotInitializedException(env,"Should have been initialized first");
		return;
	}
	if(isConnected)
	{
		isConnected = false;
		long errorCode;
		try{
			errorCode = pSharedBroker->BrokerDisconnect();
		}catch(...){
			ThrowConnectorException(env,"There was an unexplained error with COM call BrokerDisconnect");
			return;
		}
		if(errorCode != Success)
			ThrowOnError(env,"disconnect failed",errorCode);
	}
}
JNIEXPORT void JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_setContext
(JNIEnv *env, jobject, jstring option)
{
	if(!isInitialized)
	{
		ThrowConnectorNotInitializedException(env,"Should have been initialized first");
		return;
	}
	if(!isConnected)
	{
		ThrowConnectorNotConnectedException(env,"Should have been connected first");
		return;
	}
	BSTR optionName = (BSTR)env->GetStringChars(option, 0);
	long errorCode = GeneralFailure;

	try{
		errorCode = pSharedBroker->BrokerSetContext(optionName);
	}catch(...){
		ThrowConnectorException(env,"There was an unexplained error with COM call BrokerSetContext");
		return ;
	}
	if(errorCode != Success){
		//char* opName = NULL;
		//int len = _bstr_t(optionName).length();
		//if(len > 0){
		//	opName = new char[len];
		//	strcpy(opName,(char*)_bstr_t(optionName));
		//}else{
		//	opName = new char[1];
		//	opName[0] = '\0';
		//}
		ThrowOnError(env,(char*)_bstr_t(optionName),errorCode);
		//delete [] opName;
	}
	env->ReleaseStringChars(option, (jchar*)optionName);
}
JNIEXPORT jstring JNICALL Java_gov_va_med_hds_cd_m_internal_broker_jni_Connector_call
(JNIEnv *env, jobject obj, jstring name, jstring params, jint timeLimit)
{
	if(!isInitialized)
	{
		ThrowConnectorNotInitializedException(env,"Should have been initialized first");
		return NULL;
	}
	if(!isConnected)
	{
		ThrowConnectorNotConnectedException(env,"Should have been connected first");
		return NULL;
	}
	BSTR rpcName = (BSTR)env->GetStringChars(name,0);
	BSTR rpcParams = (BSTR)env->GetStringChars(params,0);

	BSTR rpcResults = NULL;
	long uniqueRpcCallId = 0;
	long errorCode = GeneralFailure;
	try{
		errorCode = pSharedBroker->BrokerCall(rpcName,rpcParams,timeLimit,&rpcResults,&uniqueRpcCallId);
	}catch(...){
		char mess[256];
		sprintf(mess,"There was an unexplained error with COM call BrokerCall: \"%s\"",rpcName);
		ThrowConnectorException(env,mess);
		return NULL;
	}
	if(errorCode != Success)
		ThrowOnError(env,"call failed",errorCode);
	else
		ReportAdditionalCallResults(env, obj,uniqueRpcCallId);
	env->ReleaseStringChars(name,(jchar*)rpcName);
	env->ReleaseStringChars(params,(jchar*)rpcParams);

	jstring result = NULL;
	if(rpcResults != NULL)
	{
		result = env->NewString(rpcResults,::SysStringLen(rpcResults));
		::SysFreeString(rpcResults);
	}

	return result;
}

