package gov.va.med.hds.cd.m.internal.broker.itest;
import gov.va.med.hds.cd.m.CallRequestCacheEntry;
import gov.va.med.hds.cd.m.CallRequestExpectedResultPair;
import gov.va.med.hds.cd.m.ICallRequestCache;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.test.CallFactory;
import gov.va.med.hds.cd.m.test.TestConstants;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.runtime.test.TestRuntime;
import gov.va.med.hds.cd.security.ISecurity;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.jaxen.dom4j.Dom4jXPath;

/**
 * Tests Shared Broker based call queue recording
 */
public class BrokerCallQueueRecordAppendTest extends RuntimeTestCase {
	public BrokerCallQueueRecordAppendTest(String arg0) {
		super(arg0);
	}
	final String baseFileName = "callcache.xml";
	public void testAppend() throws Exception {
		writeFirstTime();
		setRuntime(new TestRuntime());
		parseRegistryResource(TestConstants.DEFAULT_REGISTRY_PATH);
		getRuntime().readSettings(getClass().getResourceAsStream("BrokerRecordAppendSettings.xml"));
		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);

		CallRequestExpectedResultPair secondRequest =
			CallFactory.getInstance().create(CallFactory.NO_PARAM_SIMPLE);
		mumps.callRPC(secondRequest.request.getCallName(), secondRequest.request.getParams());
		securityModule.logout();
		getRuntime().shutdown();
		setRuntime(null);

		SAXReader reader = new SAXReader();
		Document document = reader.read(new FileInputStream(new File(baseFileName)));

		CallRequestExpectedResultPair firstRequest =
			CallFactory.getInstance().create(CallFactory.PARAM_COLLECTION);

		final String callEntryXpath =
			"/"
				+ ICallRequestCache.CALL_CACHE_ROOT_ELEMENT_NAME
				+ "/"
				+ ICallRequestCache.CALLS_ELEMENT_NAME
				+ "/"
				+ CallRequestCacheEntry.CALL_ENTRY_ROOT_ELEMENT_NAME;
		List nodes =
			(
				new Dom4jXPath(
					callEntryXpath
						+ "[name='"
						+ firstRequest.request.getCallName()
						+ "']")).selectNodes(
				document);
		assertTrue(
			"Appended cache did not contain the call '" + firstRequest.request.getCallName() + "'",
			nodes.size() == 1);

		nodes =
			(
				new Dom4jXPath(
					callEntryXpath
						+ "[name='"
						+ secondRequest.request.getCallName()
						+ "']")).selectNodes(
				document);
		assertTrue(
			"Appended cache did not contain the call '" + secondRequest.request.getCallName() + "'",
			nodes.size() == 1);
	}
	private void writeFirstTime() throws Exception {
		parseRegistryResource(TestConstants.DEFAULT_REGISTRY_PATH);
		getRuntime().readSettings(getClass().getResourceAsStream("BrokerRecordSettings.xml"));
		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);

		CallRequestExpectedResultPair firstRequest =
			CallFactory.getInstance().create(CallFactory.PARAM_COLLECTION);

		mumps.callRPC(firstRequest.request.getCallName(), firstRequest.request.getParams());
		securityModule.logout();
		getRuntime().shutdown();
		setRuntime(null);
	}
	private IMumps mumps;
	private ISecurity securityModule;
}
