package gov.va.med.hds.cd.m.internal.broker.itest;
import gov.va.med.hds.cd.m.CallRequestCacheEntry;
import gov.va.med.hds.cd.m.CallRequestExpectedResultPair;
import gov.va.med.hds.cd.m.ICallRequestCache;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.test.CallFactory;
import gov.va.med.hds.cd.m.test.TestConstants;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.jaxen.dom4j.Dom4jXPath;
/**
 * Tests the Shared Broker based call queue
 */
public class BrokerCallQueueRecordTest extends RuntimeTestCase {
	public BrokerCallQueueRecordTest(String arg0) {
		super(arg0);
	}
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource(TestConstants.DEFAULT_REGISTRY_PATH);
		getRuntime().readSettings(getClass().getResourceAsStream("BrokerRecordSettings.xml"));
	}
	public void testCreateVistalinkCallQueue() throws Exception {
		final String baseFileName = "callcache.xml";
		final String expectedResultFilePath =
			"/gov/va/med/hds/cd/m/internal/broker/itest/ExpectedResult1.xml";

		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);

		CallRequestExpectedResultPair request =
			CallFactory.getInstance().create(CallFactory.PARAM_COLLECTION);

		mumps.callRPC(request.request.getCallName(), request.request.getParams());
		securityModule.logout();

		SAXReader reader = new SAXReader();
		ICallRequestCache duh;
		Document document = reader.read(new FileInputStream(new File(baseFileName)));
		final String callEntryXpath =
			"/"
				+ ICallRequestCache.CALL_CACHE_ROOT_ELEMENT_NAME
				+ "/"
				+ ICallRequestCache.CALLS_ELEMENT_NAME
				+ "/"
				+ CallRequestCacheEntry.CALL_ENTRY_ROOT_ELEMENT_NAME;
		List nodes =
			(
				new Dom4jXPath(
					callEntryXpath
						+ "[name='"
						+ request.request.getCallName()
						+ "']")).selectNodes(
				document);
		assertEquals(
			"The Cache is missing the RPC call '" + request.request.getCallName() + "'",
			1,
			nodes.size());

		nodes = (new Dom4jXPath(callEntryXpath + "/elapsedTime")).selectNodes(document);
		assertTrue(
			"The Cache is missing elapsed time for the RPC call '"
				+ request.request.getCallName()
				+ "'",
			nodes.size() >= 1);
	}
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	private IMumps mumps;
	private ISecurity securityModule;
}
