package gov.va.med.hds.cd.m.internal.test;

import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.IRPCLogListener;
import gov.va.med.hds.cd.m.RPCLogEventObject;
import gov.va.med.hds.cd.m.test.TestConstants;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;

import java.util.ArrayList;

/**
 * Tests call logging.
 */
public class CallLoggingTest extends RuntimeTestCase {

	public CallLoggingTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource(TestConstants.DEFAULT_REGISTRY_PATH);
		getRuntime().readSettings(getClass().getResourceAsStream("CacheSettings.xml"));
		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);
	}
	static final String SUCCESS_CALL_KEY = "ECHO";
	static final String SUCCESS_CALL_VALUE = "XWB EXAMPLE ECHO STRING";
	static final String SUCCESS_PARAM_NAME = "This is an echo string.";
	static final String SUCCESS_CALL_RESULT = SUCCESS_PARAM_NAME;
	boolean pickedUpLog;
	public void testLogListenerSuccess() throws Exception {
		ArrayList params = new ArrayList();
		params.add(SUCCESS_PARAM_NAME);
		SuccessLogListener listener = new SuccessLogListener();
		mumps.addLogListener(listener);
		pickedUpLog = false;
		String result = mumps.callRPC(SUCCESS_CALL_KEY, params);
		synchronized (this) {
			wait(1000);
		}
		assertTrue("Should have picked up a log item", pickedUpLog);
		assertEquals(SUCCESS_CALL_RESULT, result);
		mumps.removeLogListener(listener);
	}
	private class SuccessLogListener implements IRPCLogListener {
		public void callExecuted(RPCLogEventObject object) {
			assertNotNull(object);
			assertEquals(SUCCESS_CALL_RESULT, object.result.result);
			assertEquals(SUCCESS_PARAM_NAME, (String) object.request.getParams().get(0));
			assertEquals(SUCCESS_CALL_VALUE, object.request.getCallName());
			assertTrue(object.result.elapsedTime > 0);
			synchronized (this) {
				pickedUpLog = true;
				notify();
			}
		}
	};
	static final String FAIL_CALL_NAME = "INVALID_RPC";
	protected void tearDown() throws Exception {
		super.tearDown();
		securityModule.logout();
	}
	ISecurity securityModule;
	IMumps mumps;
}
