package gov.va.med.hds.cd.m.internal.test;
import gov.va.med.hds.cd.m.CallFailedException;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.CallRequestException;
import gov.va.med.hds.cd.m.DisconnectException;
import gov.va.med.hds.cd.m.ICallQueue;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.IRPCLogListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.util.DisposeException;

import java.util.List;

import junit.framework.TestCase;

import org.dom4j.Element;
/**
 * Tests the call queue
 */
public class CallQueueTest extends TestCase implements IMumpsListener {
	private boolean done = false;
	private String succeededResult = "";
	private MumpsException failedException = null;
	public CallQueueTest(String arg0) {
		super(arg0);
	}
	protected void setUp() throws Exception {
		super.setUp();
	}
	public void testGetInstance() {
		ICallQueue queue = MockCallQueue.getInstance();
		assertNotNull(queue);
	}
	public void testCallSuccess()
		throws DisposeException, MumpsException, DisconnectException, InterruptedException {
		ICallQueue queue = MockCallQueue.getInstance();
		assertNotNull(queue);
		queue.add(new MockCallRequest(MockCallQueue.MOCK_RPC_NAME_SUCCESS), this);
		synchronized (this) {
			wait(1000);
		}
		assertEquals(MockCallQueue.MOCK_RPC_SUCCESS_RESULT, succeededResult);
	}
	public void testCallFail()
		throws DisposeException, MumpsException, DisconnectException, InterruptedException {
		ICallQueue queue = MockCallQueue.getInstance();
		assertNotNull(queue);
		queue.add(new MockCallRequest(MockCallQueue.MOCK_RPC_NAME_FAIL), this);
		synchronized (this) {
			wait(1000);
		}
		assertTrue(failedException instanceof CallFailedException);
	}
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	public void CallFailed(ICallRequest request, MumpsException e) {
		done = true;
		failedException = e;
		synchronized (this) {
			notify();
		}
	}
	public void CallSucceeded(ICallRequest request, String result) {
		done = true;
		succeededResult = result;
		synchronized (this) {
			notify();
		}
	}
	class MockCallRequest implements ICallRequest {
		private String callName = null;
		private List params = null;
		public MockCallRequest(String callName) {
			setCallName(callName);
		}
		public MockCallRequest(String callName, List params) {
			setCallName(callName);
			setParams(params);
		}
		public String getCallName() {
			return callName;
		}
		public int getCallTimeLimit() {
			return 0;
		}
		public String getContext() {
			return null;
		}
		public List getParams() {
			return params;
		}
		public void setCallName(String name) {
			callName = name;
		}
		public void setCallTimeLimit(int timeLimitInMillis) {
		}
		public void setContext(String context) {
		}
		public void setParams(List params) {
			this.params = params;
		}
		public boolean getResultsWillBeXml() {
			return false;
		}
		public void setResultsWillBeXml(boolean willBe) {
		}
		public void write(Element entry) {
		}
		public ICallRequest copy() throws CallRequestException {
			return null;
		}

	}
}
class MockCallQueue implements ICallQueue {
	public static final String MOCK_RPC_NAME_SUCCESS = "TEST RPC SUCCESS";
	public static final String MOCK_RPC_NAME_FAIL = "TEST RPC FAIL";
	public static final String MOCK_RPC_SUCCESS_RESULT = "SUCCESSFUL RESULT";
	protected MockCallQueue() {
	}
	public void add(ICallRequest request, IMumpsListener listener) {
		if (request.getCallName() == MOCK_RPC_NAME_SUCCESS) {
			listener.CallSucceeded(request, MOCK_RPC_SUCCESS_RESULT);
		} else if (request.getCallName() == MOCK_RPC_NAME_FAIL) {
			listener.CallFailed(request, new CallFailedException());
		} else {
			listener.CallFailed(request, new CallFailedException());
		}
	}
	public String add(ICallRequest request) throws CallQueueException {
		return null;
	}
	public static ICallQueue getInstance() {
		if (instance == null)
			instance = new MockCallQueue();
		return instance;
	}
	public void dispose() throws DisposeException {
	}
	private static ICallQueue instance = null;
	public void addLogListener(IRPCLogListener listener) throws CallQueueException {
	}
	public void removeLogListener(IRPCLogListener listener) throws CallQueueException {
	}
	public String decrypt(String data) {
		return data;
	}
	public String encrypt(String data) {
		return data;
	}
	/**
	 * @see gov.va.med.hds.cd.m.ICallQueue#enableTimeLimitOverride(int)
	 */
	public void enableTimeLimitOverride(int rpcTimeoutOverride) {
	}
	/**
	 * @see gov.va.med.hds.cd.m.ICallQueue#disableTimeLimitOverride()
	 */
	public void disableTimeLimitOverride() {
	}
}
