package gov.va.med.hds.cd.m.internal.test;
import gov.va.med.hds.cd.m.CallRequest;
import gov.va.med.hds.cd.m.CallRequestCacheEntry;
import gov.va.med.hds.cd.m.CallRequestCacheException;
import gov.va.med.hds.cd.m.ICallRequestCache;
import gov.va.med.hds.cd.m.TimedResult;
import gov.va.med.hds.cd.m.ValidationFailedException;
import gov.va.med.hds.cd.m.internal.CallRequestCache;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.custommonkey.xmlunit.XMLTestCase;
import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.jaxen.dom4j.Dom4jXPath;

/**
 * Tests the call request cache
 */
public class CallRequestCacheTest extends XMLTestCase {
	private static final String FILE_POSTFIX = ".xml";
	public CallRequestCacheTest(String arg0) {
		super(arg0);
	}
	protected void setUp() throws Exception {
		super.setUp();
	}
	public void testCreate() throws Exception {
		CallRequestCache cache = new CallRequestCache(null);
		assertNotNull(cache);
		cache = null;
	}
	public void testRecordOverwrite() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();
		FileOutputStream fileStream = new FileOutputStream(file);

		CallRequestCache cache = new CallRequestCache(null);
		String testResult = "TEST RPC RESULT";
		cache.add(new CallRequest("TEST RPC"), new TimedResult(testResult, 0));
		cache.write(fileStream);
		fileStream.flush();
		fileStream.close();

		SAXReader reader = new SAXReader();
		Document document = reader.read(new FileInputStream(file));
		final String callEntryXpath =
			"/"
				+ ICallRequestCache.CALL_CACHE_ROOT_ELEMENT_NAME
				+ "/"
				+ ICallRequestCache.CALLS_ELEMENT_NAME
				+ "/"
				+ CallRequestCacheEntry.CALL_ENTRY_ROOT_ELEMENT_NAME;

		Node node = (Node) (new Dom4jXPath(callEntryXpath + "/name")).selectSingleNode(document);
		assertEquals("TEST RPC", node.getText());
		node = (Node) (new Dom4jXPath(callEntryXpath + "/result")).selectSingleNode(document);
		assertEquals("TEST RPC RESULT", node.getText());
	}
	public void testListParamType() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();
		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc4(), new TimedResult(TEST_RPC4_RESULT, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		cache = new CallRequestCache(null);
		cache.load(new FileInputStream(file));
		TimedResult result = cache.getResult(createRpc4());
		assertNotNull(result);
		assertEquals(TEST_RPC4_RESULT, result.result);
	}
	public void testMapParamType() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();
		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc5(), new TimedResult(TEST_RPC5_RESULT, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		cache = new CallRequestCache(null);
		cache.load(new FileInputStream(file));
		TimedResult result = cache.getResult(createRpc5());
		assertXMLEqual(TEST_RPC5_RESULT, result.result);
	}
	public void testCachingVariousParamTypes() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();
		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		cache.add(createRpc2(), new TimedResult(TEST_RPC2_RESULT, 0));
		cache.add(createRpc3(), new TimedResult(TEST_RPC3_RESULT, 0));
		cache.add(createRpc4(), new TimedResult(TEST_RPC4_RESULT, 0));
		cache.add(createRpc5(), new TimedResult(TEST_RPC5_RESULT, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		cache = new CallRequestCache(null);
		cache.load(new FileInputStream(file));
		TimedResult result = cache.getResult(createRpc1());
		assertEquals(TEST_RPC1_RESULT, result.result);
		result = cache.getResult(createRpc2());
		assertEquals(TEST_RPC2_RESULT, result.result);
		result = cache.getResult(createRpc3());
		assertEquals(TEST_RPC3_RESULT, result.result);
		result = cache.getResult(createRpc4());
		assertEquals(TEST_RPC4_RESULT, result.result);
		result = cache.getResult(createRpc5());
		assertEquals(TEST_RPC5_RESULT, result.result);
	}
	public void testRecordAppend() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();

		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		String testResult = "TEST RPC RESULT";
		cache.add(new CallRequest("TEST RPC"), new TimedResult(testResult, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		FileInputStream input = new FileInputStream(file);
		cache = new CallRequestCache(null);
		cache.load(input);
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		cache.add(createRpc2(), new TimedResult(TEST_RPC2_RESULT, 0));
		stream = new FileOutputStream(file);
		cache.write(stream);
		stream.flush();
		stream.close();

		SAXReader reader = new SAXReader();
		Document document = reader.read(new FileInputStream(file));
		final String callEntryXpath =
			"/"
				+ ICallRequestCache.CALL_CACHE_ROOT_ELEMENT_NAME
				+ "/"
				+ ICallRequestCache.CALLS_ELEMENT_NAME
				+ "/"
				+ CallRequestCacheEntry.CALL_ENTRY_ROOT_ELEMENT_NAME;

		Node node =
			(Node) (new Dom4jXPath(callEntryXpath + "[name='TEST RPC']")).selectSingleNode(
				document);
		assertNotNull(node);
		node =
			(Node) (new Dom4jXPath(callEntryXpath + "[name='TEST RPC2']")).selectSingleNode(
				document);
		assertNotNull(node);
		node =
			(Node) (new Dom4jXPath(callEntryXpath + "[name='TEST RPC1']")).selectSingleNode(
				document);
		assertNotNull(node);
	}
	public void testLoadMultipleCacheFiles() throws Exception {
		File file1 = File.createTempFile(getClass().getName() + 1, FILE_POSTFIX);
		file1.deleteOnExit();
		File file2 = File.createTempFile(getClass().getName() + 2, FILE_POSTFIX);
		file2.deleteOnExit();
		File file3 = File.createTempFile(getClass().getName() + 3, FILE_POSTFIX);
		file3.deleteOnExit();

		FileOutputStream out = new FileOutputStream(file1);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		cache.write(out);
		out.flush();
		out.close();

		out = new FileOutputStream(file2);
		cache = new CallRequestCache(null);
		cache.add(createRpc2(), new TimedResult(TEST_RPC2_RESULT, 0));
		cache.write(out);
		out.flush();
		out.close();

		out = new FileOutputStream(file3);
		cache = new CallRequestCache(null);
		cache.add(createRpc3(), new TimedResult(TEST_RPC3_RESULT, 0));
		cache.write(out);
		out.flush();
		out.close();

		cache = new CallRequestCache(null);

		cache.load(new FileInputStream(file1));
		TimedResult result = cache.getResult(createRpc1());
		assertEquals(TEST_RPC1_RESULT, result.result);
		result = cache.getResult(createRpc2());
		assertNull(result);
		result = cache.getResult(createRpc3());
		assertNull(result);

		cache.load(new FileInputStream(file2));
		result = cache.getResult(createRpc1());
		assertEquals(TEST_RPC1_RESULT, result.result);
		result = cache.getResult(createRpc2());
		assertEquals(TEST_RPC2_RESULT, result.result);
		result = cache.getResult(createRpc3());
		assertNull(result);

		cache.load(new FileInputStream(file3));
		result = cache.getResult(createRpc1());
		assertEquals(TEST_RPC1_RESULT, result.result);
		result = cache.getResult(createRpc2());
		assertEquals(TEST_RPC2_RESULT, result.result);
		result = cache.getResult(createRpc3());
		assertEquals(TEST_RPC3_RESULT, result.result);
	}
	public void testAddDuplicateOk() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);
		file.deleteOnExit();

		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		// Since already cached it is just ignored.
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		FileInputStream input = new FileInputStream(file);
		cache = new CallRequestCache(null);
		cache.load(input);
		// Since already cached it is just ignored
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
	}
	public void testSchemaValidationSuccess() throws Exception {
		File file = File.createTempFile(getClass().getName(), FILE_POSTFIX);

		FileOutputStream stream = new FileOutputStream(file);
		CallRequestCache cache = new CallRequestCache(null);
		cache.add(createRpc1(), new TimedResult(TEST_RPC1_RESULT, 0));
		cache.write(stream);
		stream.flush();
		stream.close();

		cache = new CallRequestCache(null);
		// need to fix the validation
		//		cache.validate(new FileInputStream(file));
		cache.load(new FileInputStream(file));
		TimedResult result = cache.getResult(createRpc1());
		assertEquals(TEST_RPC1_RESULT, result.result);
		file.deleteOnExit();
	}
	public void testSchemaValidationEmptyFail() throws Exception {
		final String EMPTY_FILE_FAIL = "CallRequestEmptyFileFail.xml";
		CallRequestCache cache = new CallRequestCache(null);
		cache = new CallRequestCache(null);

		try {
			cache.validate(getClass().getResourceAsStream(EMPTY_FILE_FAIL));
			fail("validating an empty file should have failed.");
		} catch (ValidationFailedException e) {
			// Should have failed
		}
		try {
			cache.load(getClass().getResourceAsStream(EMPTY_FILE_FAIL));
			fail("loading an empty file should have failed.");
		} catch (CallRequestCacheException e) {
			// loading an empty file (contains no markup) should fail
		}

	}
	public void testSchemaValidationFail() throws Exception {
		CallRequestCache cache = new CallRequestCache(null);
		final String FAIL_FILE = "CallRequestCacheTestFail.xml";
		try {
			cache.validate(getClass().getResourceAsStream(FAIL_FILE));
			fail("Invalid XML cache file should have failed validation test");
		} catch (ValidationFailedException e) {
			// invalid XML file should fail validation
		}
		try {
			cache.load(getClass().getResourceAsStream(FAIL_FILE));
			fail("Invalid XML should fail to load.");
		} catch (CallRequestCacheException e) {
			// invalid XML should cause an exception.
		}
	}
	public void testSchemaValidationFail2()
		throws CallRequestCacheException, ValidationFailedException {
		CallRequestCache cache = new CallRequestCache(null);
		final String FAIL_FILE = "CallRequestCacheTestFail2.xml";
		try {
			cache.validate(getClass().getResourceAsStream(FAIL_FILE));
			fail("Invalid XML cache file should have failed validation test");
		} catch (ValidationFailedException e) {
			// invalid XML file should fail validation
		}
		//		try {
		//			cache.load(getClass().getResourceAsStream(FAIL_FILE));
		//			fail("Invalid XML should fail to load.");
		//		} catch (CallRequestCacheException e) {
		//			// invalid XML should cause an exception.
		//		}
	}
	private static final String TEST_RPC1_RESULT = "Test Result 1";
	private CallRequest createRpc1() {
		return new CallRequest("TEST RPC1");
	}
	private static final String TEST_RPC2_RESULT = "Test Result 2";
	private CallRequest createRpc2() {
		return new CallRequest("TEST RPC2");
	}
	private static final String TEST_RPC3_RESULT =
		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
			+ "<testRoot>"
			+ "<someTag>some Data</someTag>"
			+ "</testRoot>";
	private CallRequest createRpc3() {
		ArrayList params = new ArrayList();
		params.add("first param");
		params.add(new Integer(100));
		params.add(new Float(101.2f));
		CallRequest request = new CallRequest("TEST RPC3", params);
		return request;
	}
	private static final String TEST_RPC4_RESULT =
		"LINE^1^HAT^DELIMITED\n"
			+ "LINE^2^HAT^DELIMITED\n"
			+ "LINE^2^HAT^DELIMITED^VARIABLE\n"
			+ "LINE^2^HAT^DELIMITED^VARIABLE^ODD*()&6%$#@!^CHARS\n";
	private CallRequest createRpc4() {
		ArrayList params = new ArrayList();
		params.add(new Integer(100));
		ArrayList secondParam = new ArrayList();
		secondParam.add("list1");
		secondParam.add("list2");
		secondParam.add("list3");
		secondParam.add("list4");
		secondParam.add("list5");
		params.add(secondParam);
		ArrayList thirdParam = new ArrayList();
		thirdParam.add(new Integer(5));
		thirdParam.add(new Float(11.0f));
		thirdParam.add(new Double(1234567891011121314151617181920.0));
		thirdParam.add("Other stuff");
		thirdParam.add(new Long(3276823));
		params.add(thirdParam);
		params.add(new Float(101.2f));
		CallRequest request = new CallRequest("TEST RPC4", params);
		return request;
	}
	private static final String TEST_RPC5_RESULT =
		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
			+ "<test5Result>\n"
			+ "<someTag><![CDATA[Another Result]]></someTag>\n"
			+ "<someTag2>45</someTag2>\n"
			+ "</test5Result>\n";
	private CallRequest createRpc5() {
		final String value1 =
			"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
				+ "<test5Value1>\n"
				+ "<valueTag>some value</valueTag>\n"
				+ "</test5Value1>\n";
		final String value2 =
			"<test5Value2>\n" + "<valueTag><![CDATA[Some Data]]></valueTag>\n" + "</test5Value2>\n";

		ArrayList params = new ArrayList();
		Map param1 = new HashMap();
		param1.put("key1", value1);
		param1.put("key2", value2);
		params.add(param1);
		params.add(param1);
		ArrayList param3 = new ArrayList();
		param3.add("list1");
		param3.add("list2");
		param3.add("list3");
		param3.add("list4");
		params.add(param3);
		params.add("fourth param");
		CallRequest request = new CallRequest("TEST RPC5", params);
		return request;
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
}
