package gov.va.med.hds.cd.m.internal.test;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import gov.va.med.hds.cd.m.CallRequest;

import junit.framework.TestCase;

/**
 * Tests a CallRequest
 */
public class CallRequestTest extends TestCase {
	public CallRequestTest(String arg0) {
		super(arg0);
	}
	public void testHashCodeEquivalenceForSimpleCallRequests() {
		CallRequest cr1 = new CallRequest("TEST RPC1");
		CallRequest cr2 = new CallRequest("TEST RPC1");
		assertEquals(cr1.hashCode(), cr2.hashCode());
	}
	public void testHashCodeEquivalenceForSingleParamCallRequests() {
		ArrayList cr1Params = new ArrayList();
		cr1Params.add("First");
		CallRequest cr1 = new CallRequest("TEST RPC1", cr1Params);
		ArrayList cr2Params = new ArrayList();
		cr2Params.add("First");
		CallRequest cr2 = new CallRequest("TEST RPC1", cr2Params);
		assertEquals(cr1.hashCode(), cr2.hashCode());
	}
	public void testHashCodeEquivalenceForListParamCallRequests() {
		ArrayList cr1Params = new ArrayList();
		ArrayList list1 = new ArrayList();
		list1.add("l1");
		list1.add("l2");
		list1.add("l3");
		list1.add("l4");
		cr1Params.add(list1);
		CallRequest cr1 = new CallRequest("TEST RPC1", cr1Params);
		ArrayList cr2Params = new ArrayList();
		ArrayList list2 = new ArrayList();
		list2.add("l1");
		list2.add("l2");
		list2.add("l3");
		list2.add("l4");
		cr2Params.add(list2);
		CallRequest cr2 = new CallRequest("TEST RPC1", cr2Params);
		assertEquals(cr1.hashCode(), cr2.hashCode());
	}
	public void testHashCodeEquivalenceForMapParamCallRequests() {
		ArrayList cr1Params = new ArrayList();
		Map map1 = new TreeMap();
		map1.put("key1", "first val");
		map1.put("key2", "second val");
		map1.put("key3", "third val");
		cr1Params.add(map1);
		CallRequest cr1 = new CallRequest("TEST RPC1", cr1Params);
		ArrayList cr2Params = new ArrayList();
		Map map2 = new TreeMap();
		map2.put("key1", "first val");
		map2.put("key2", "second val");
		map2.put("key3", "third val");
		cr2Params.add(map2);
		CallRequest cr2 = new CallRequest("TEST RPC1", cr2Params);
		assertEquals(cr1.hashCode(), cr2.hashCode());
	}
	public void testHashCodeEquivalenceForMixedParamCallRequests() {
		ArrayList cr1Params = new ArrayList();
		Map map1 = new TreeMap();
		map1.put("key1", "first val");
		map1.put("key2", "second val");
		map1.put("key3", "third val");
		cr1Params.add(map1);
		ArrayList list1 = new ArrayList();
		list1.add("l1");
		list1.add("l2");
		list1.add("l3");
		cr1Params.add(list1);
		cr1Params.add(new Float(10.0f));
		cr1Params.add(new Integer(100));
		cr1Params.add(new String("Yo"));
		cr1Params.add("test");
		CallRequest cr1 = new CallRequest("TEST RPC1", cr1Params);
		ArrayList cr2Params = new ArrayList();
		Map map2 = new TreeMap();
		map2.put("key1", "first val");
		map2.put("key2", "second val");
		map2.put("key3", "third val");
		cr2Params.add(map2);
		ArrayList list2 = new ArrayList();
		list2.add("l1");
		list2.add("l2");
		list2.add("l3");
		cr2Params.add(list2);
		cr2Params.add(new Float(10.0f));
		cr2Params.add(new Integer(100));
		cr2Params.add(new String("Yo"));
		cr2Params.add("test");
		CallRequest cr2 = new CallRequest("TEST RPC1", cr2Params);
		assertEquals(cr1.hashCode(), cr2.hashCode());
	}
}
