package gov.va.med.hds.cd.m.internal.test;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.CallRequest;
import gov.va.med.hds.cd.m.ConnectException;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IConnection;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.IRPCLogListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IExtensionPoint;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginPrerequisite;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.util.DisposeException;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * Tests the IMumps interface
 */
public class MumpsTest extends TestCase implements IMumpsListener {
	private static final String TEST_CALL_NAME = "TEST CALL";

	private boolean done = true;
	private String callResult = "";

	public MumpsTest(String arg0) {
		super(arg0);
	}
	protected void setUp() throws Exception {
		super.setUp();
		factory = new MockMumpsFactory();
		factory.startup(null, new MockModuleDescriptor());
	}
	public void testCreate() throws Exception {
		MockMumpsFactory conn = new MockMumpsFactory();
		conn.startup(null, new MockModuleDescriptor());
		conn.shutdown();
	}
	public void testConnect() throws ConnectException, DisposeException, MumpsException {
	}
	public void testCall()
		throws
			ConnectException,
			DisposeException,
			MumpsException,
			CallQueueException,
			InterruptedException {
		IMumps mumps = factory.create(null);
		mumps.callRPC(TEST_CALL_NAME, this);
		while (!done) {
			Thread.sleep(100);
		};
		assertEquals(TEST_CALL_NAME, callResult);
	}
	public void CallFailed(ICallRequest request, MumpsException e) {
	}
	public void CallSucceeded(ICallRequest request, String result) {
		done = true;
		callResult = result;
	}
	protected void tearDown() throws Exception {
		super.tearDown();
		factory.shutdown();
	}
	private MockMumpsFactory factory = null;

	class MockMumpsFactory implements IMumpsFactory, IPlugin {
		public void shutdown() throws Exception {
		}
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception {
		}
		class MockMumps implements IMumps {
			private IRPCLogListener listener;
			public void addLogListener(IRPCLogListener listener) throws MumpsException {
				this.listener = listener;
			}
			public void removeLogListener(IRPCLogListener listener) throws MumpsException {
				this.listener = null;
			}
			public String decrypt(String data) {
				return data;
			}
			public String encrypt(String data) {
				return data;
			}
			public String callRPC(String rpcName) throws MumpsException {
				return null;
			}
			public String callRPC(String rpcName, Object[] params) throws MumpsException {
				return null;
			}
			public String callRPC(String rpcName, List params) throws MumpsException {
				return null;
			}
			public void callRPC(String rpcName, IMumpsListener listener) throws MumpsException {
				listener.CallSucceeded(new CallRequest(rpcName), rpcName);
			}
			public void callRPC(String rpcName, Object[] args, IMumpsListener listener)
				throws MumpsException {
				ArrayList params = new ArrayList(args.length);
				for (int i = 0; i < args.length; i++)
					params.add(args[i]);
				listener.CallSucceeded(new CallRequest(rpcName, params), rpcName);
			}
			public void callRPC(String rpcName, List params, IMumpsListener listener)
				throws MumpsException {
				listener.CallSucceeded(new CallRequest(rpcName, params), rpcName);
			}
			public String callRPC(String rpcName, int timeLimit) throws MumpsException {
				return null;
			}
			public String callRPC(String rpcName, Object[] params, int timeLimit)
				throws MumpsException {
				return null;
			}
			public String callRPC(String rpcName, List params, int timeLimit)
				throws MumpsException {
				return null;
			}
			public void callRPC(String rpcName, IMumpsListener listener, int timeLimit)
				throws MumpsException {
			}
			public void callRPC(
				String rpcName,
				Object[] params,
				IMumpsListener listener,
				int timeLimit)
				throws MumpsException {
			}
			public void callRPC(
				String rpcName,
				List params,
				IMumpsListener listener,
				int timeLimit)
				throws MumpsException {
			}
		}
		public String getServerName() {
			return "MockServer";
		}
		public int getServerPort() {
			return -1;
		}
		public IMumps create(IPluginDescriptor descriptor) {
			return new MockMumps();
		}
	}
	class MockConnection implements IConnection {
		public void connect() {
		}
		public void dispose() throws DisposeException {
		}
	}

	class MockModuleDescriptor implements IPluginDescriptor {
		public String getClassName() {
			return null;
		}
		public IExtension getExtension(String extensionName) {
			return null;
		}
		public IExtensionPoint getExtensionPoint(String extensionPointId) {
			return null;
		}
		public IExtensionPoint[] getExtensionPoints() {
			return null;
		}
		public IExtension[] getExtensions() {
			return null;
		}
		public String getLabel() {
			return null;
		}
		public IPluginPrerequisite[] getPluginPrerequisites() {
			return null;
		}
		public String getProviderName() {
			return null;
		}
		public String getUniqueIdentifier() {
			return null;
		}
		public Version getVersion() {
			return null;
		}
		public boolean isModuleActivated() {
			return false;
		}
		public IRuntimeContext getRuntimeContext() {
			return null;
		}
		public ClassLoader getPluginClassLoader() {
			return null;
		}
	}

}
