package gov.va.med.hds.cd.m.internal.test;

import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.EncryptException;
import gov.va.med.hds.cd.m.ICallQueue;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IOptionContext;
import gov.va.med.hds.cd.m.IRPCLogListener;
import gov.va.med.hds.cd.m.OptionContext;
import gov.va.med.hds.cd.m.internal.InvalidContextException;
import gov.va.med.hds.cd.m.internal.OptionContextValidator;
import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.mum.VersionMatch;
import gov.va.med.hds.cd.mum.boot.IPluginConstants;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IExtensionPoint;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginPrerequisite;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.util.List;

import javax.security.auth.login.LoginException;

import junit.framework.TestCase;

/**
 * Tests the OptionContext validation
 */
public class OptionContextValidatorTest extends TestCase implements ICallQueue, IPluginConstants {
	private IOptionContext callingContext =
		new OptionContext(
			"XHDXC DESKTOP",
			new Version("0.1.0"),
			null,
			VersionMatch.GREATER_OR_EQUAL);
	public void testValidationPass() throws LoginException {
		assertNull(params);
		validator.validate(contexts, this, callingContext);
		assertNotNull(params);
		assertEquals(1, params.size());
		Object obj = params.get(0);
		assertNotNull(obj);
		assertTrue(obj instanceof List);
		List list = (List) obj;
		assertEquals(4, list.size());
		assertTrue(list.contains("ZZTESTING FRED"));
		assertTrue(list.contains("ZZTESTING HARRY"));
		assertTrue(list.contains("ZZTESTING GEORGE"));
		assertTrue(list.contains("ZZTESTING FRANK"));
	}

	public void testValidationFail() throws Exception {
		assertFail(fredID, "0.5", fred);
		assertFail(fredID, "3.0", fred2);
		assertFail(fredID, "2.7", fred3);
		assertFail(georgeID, "3.2", george);
		assertFail(georgeID, "4.2", george);
		assertFail(harryID, "4.6", harry);
		assertFail(harryID, "4.4", harry);
		assertFail(frankID, "5.6.7.7", frank);
		assertFail(frankID, "5.6.7.whatever", frank);
		assertFail(-1, null, george, georgeID);
		assertFail(harryID, null, harry);

		IOptionContext missing =
			new TestContext("ZZTESTING MISSING OPTION", "7.8.9", null, descriptor);
		try {
			validator.validate(new IOptionContext[] { missing }, this, callingContext);
			fail("Should have thrown InvalidContextException");
		} catch (InvalidContextException e) {
			checkError(e, missing);
			assertTrue(e.getMessage().indexOf("No such context exists on the server") > 0);
		}
	}

	//********************
	//*** end of tests ***
	//********************
	private void assertFail(int index, String version, IOptionContext context) throws Exception {
		assertFail(index, version, context, -1);
	}

	private void assertFail(int index, String version, IOptionContext context, int skip)
		throws Exception {
		updateRPCResult(index, version, skip);
		try {
			validator.validate(contexts, this, callingContext);
			fail("Should have thrown InvalidContextException");
		} catch (InvalidContextException e) {
			int i4 = checkError(e, context);
			if ((skip < 0) && (version != null)) {
				int i5 = e.getMessage().indexOf(version.toString());
				assertTrue(i4 < i5);
			}
		}
	}

	private int checkError(InvalidContextException e, IOptionContext context) {
		int i1 = e.getMessage().indexOf(context.getPluginDescriptor().getUniqueIdentifier());
		int i2 = e.getMessage().indexOf(context.getPluginDescriptor().getVersion().toString());
		int i3 = e.getMessage().indexOf(context.getName());
		int i4 = e.getMessage().indexOf(context.getVersion().toString());
		assertTrue(i1 >= 0);
		assertTrue(i1 < i2);
		assertTrue(i2 < i3);
		assertTrue(i3 < i4);
		return i4;
	}

	private OptionContextValidator validator;
	private List params;
	private IPluginDescriptor descriptor;
	private IOptionContext fred;
	private IOptionContext fred2;
	private IOptionContext fred3;
	private IOptionContext george;
	private IOptionContext harry;
	private IOptionContext frank;
	private IOptionContext[] contexts;
	private String rpcResults;
	private String[] optionNames =
		{ "ZZTESTING GEORGE", "ZZTESTING FRED", "ZZTESTING FRANK", "ZZTESTING HARRY" };
	private String[] optionVersions = { "3.7", "2.5", "5.6.7.8", "4.5.99" };
	private int georgeID = 0;
	private int fredID = 1;
	private int frankID = 2;
	private int harryID = 3;

	public OptionContextValidatorTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		validator = new OptionContextValidator();
		descriptor = new TestPluginDescriptor("my.plugin", "1.2.3");
		fred =
			new TestContext(
				optionNames[fredID],
				"1.0",
				GREATER_OR_EQUAL_MATCH_CONDITION,
				descriptor);
		fred2 = new TestContext(optionNames[fredID], "2.0", COMPATIBLE_MATCH_CONDITION, descriptor);
		fred3 = new TestContext(optionNames[fredID], "2.5", EQUIVALENT_MATCH_CONDITION, descriptor);
		george =
			new TestContext(optionNames[georgeID], "3.4", COMPATIBLE_MATCH_CONDITION, descriptor);
		harry =
			new TestContext(optionNames[harryID], "4.5.6", EQUIVALENT_MATCH_CONDITION, descriptor);
		frank =
			new TestContext(optionNames[frankID], "5.6.7.8", PERFECT_MATCH_CONDITION, descriptor);
		params = null;
		contexts = new IOptionContext[] { fred, george, harry, frank, fred2, fred3 };
		updateRPCResult(-1, null, -1);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	private void updateRPCResult(int idx, String version, int skip) {
		StringBuffer buf = new StringBuffer();
		boolean first = true;
		for (int i = 0; i < optionNames.length; i++) {
			if (i == skip)
				continue;
			if (first)
				first = false;
			else
				buf.append("\n");
			buf.append(optionNames[i]);
			buf.append('^');
			if (i == idx)
				buf.append(version);
			else
				buf.append(optionVersions[i]);
		}
		rpcResults = buf.toString();
	}

	public void add(ICallRequest request, IMumpsListener listener) throws CallQueueException {

	}

	public String add(ICallRequest request) throws CallQueueException {
		assertEquals("XHDXC DESKTOP", request.getContext());
		assertEquals("XHDX VERSRV", request.getCallName());
		params = request.getParams();
		return rpcResults;
	}

	public String encrypt(String data) throws EncryptException {
		return null;
	}

	public String decrypt(String data) throws EncryptException {
		return null;
	}
	public void addLogListener(IRPCLogListener listener) throws CallQueueException {
	}
	public void removeLogListener(IRPCLogListener listener) throws CallQueueException {
	}
	private static class TestContext implements IOptionContext {

		private VersionMatch match;
		private IPluginDescriptor descriptor;
		private Version version;
		private String name;

		public TestContext(
			String name,
			String version,
			String match,
			IPluginDescriptor descriptor) {
			super();
			this.name = name;
			this.version = new Version(version);
			this.match = VersionMatch.getVersionMatch(match);
			this.descriptor = descriptor;
		}

		public String getName() {
			return name;
		}

		public Version getVersion() {
			return version;
		}

		public IPluginDescriptor getPluginDescriptor() {
			return descriptor;
		}

		public VersionMatch getVersionMatch() {
			return match;
		}
	}

	private static class TestPluginDescriptor implements IPluginDescriptor {

		private Version version;
		private String id;

		public TestPluginDescriptor(String id, String version) {
			this.id = id;
			this.version = new Version(version);
		}

		public IExtension getExtension(String extensionName) {
			return null;
		}

		public IExtensionPoint getExtensionPoint(String extensionPointId) {
			return null;
		}

		public IExtensionPoint[] getExtensionPoints() {
			return null;
		}

		public IExtension[] getExtensions() {
			return null;
		}

		public String getLabel() {
			return null;
		}

		public IPluginPrerequisite[] getPluginPrerequisites() {
			return null;
		}

		public String getProviderName() {
			return null;
		}

		public String getUniqueIdentifier() {
			return id;
		}

		public Version getVersion() {
			return version;
		}

		public String getClassName() {
			return null;
		}

		public IRuntimeContext getRuntimeContext() {
			return null;
		}

		public ClassLoader getPluginClassLoader() {
			return null;
		}
	}

	/**
	 * @see gov.va.med.hds.cd.m.ICallQueue#enableTimeLimitOverride(int)
	 */
	public void enableTimeLimitOverride(int rpcTimeoutOverride) {
	}

	/**
	 * @see gov.va.med.hds.cd.m.ICallQueue#disableTimeLimitOverride()
	 */
	public void disableTimeLimitOverride() {
	}
}
