package gov.va.med.hds.cd.m.internal.xmlcache.itest;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.test.CallFactory;
import gov.va.med.hds.cd.m.test.TestConstants;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.runtime.test.TestRuntime;
import gov.va.med.hds.cd.security.ISecurity;

/**
 * Tests the XML Cache call queue
 */
public class XmlCacheCallQueueTest extends RuntimeTestCase {
	private static final boolean RECORD_DATA = false;

	public XmlCacheCallQueueTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
		if (RECORD_DATA) {
			recordCacheData1();
			recordCacheData2();
		}
		// These need to be copied to the resource directory specified in the registry file
		parseRegistryResource(TestConstants.DEFAULT_REGISTRY_PATH);
		getRuntime().readSettings(getClass().getResourceAsStream("CacheSettings.xml"));
		securityModule = (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);
	}
	private void recordCacheData1() throws Exception {
		TestRuntime runtime = new TestRuntime();
		IHandler recordHandler;
		runtime.getLogger(
			RuntimeTestCase.RUNTIME_PACKAGE).addHandler(recordHandler = new IHandler() {
			public void close() {
			}
			public void publish(ILogRecord logRecord) {
				fail(logRecord.getMessage());
			}

		});
		runtime.readPluginDescriptors(
			getClass().getResourceAsStream(TestConstants.DEFAULT_REGISTRY_PATH));
		runtime.readSettings(getClass().getResourceAsStream("Cache1Settings.xml"));
		ISecurity securityModule = (ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();
		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);

		CallFactory cfactory = CallFactory.getInstance();

		for (int i = 0; i < 3; i++) {
			ICallRequest r = cfactory.create(i).request;
			mumps.callRPC(r.getCallName(), r.getParams());
		}
		securityModule.logout();
		runtime.shutdown();
		runtime = null;
		log.removeHandler(recordHandler);
	}
	private void recordCacheData2() throws Exception {
		TestRuntime runtime = new TestRuntime();
		IHandler recordHandler;
		runtime.getLogger(
			RuntimeTestCase.RUNTIME_PACKAGE).addHandler(recordHandler = new IHandler() {
			public void close() {
			}
			public void publish(ILogRecord logRecord) {
				fail(logRecord.getMessage());
			}

		});
		runtime.readPluginDescriptors(
			getClass().getResourceAsStream(TestConstants.DEFAULT_REGISTRY_PATH));
		runtime.readSettings(getClass().getResourceAsStream("Cache2Settings.xml"));
		ISecurity securityModule = (ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		securityModule.login();

		IPluginDescriptor desc =
			getRuntime().getPluginRegistry().getPluginDescriptor(IMumpsFactory.PLUGIN_ID);
		IMumpsFactory factory =
			(IMumpsFactory) getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
		mumps = factory.create(desc);

		CallFactory cfactory = CallFactory.getInstance();

		for (int i = 3; i < cfactory.getCount(); i++) {
			ICallRequest r = cfactory.create(i).request;
			mumps.callRPC(r.getCallName(), r.getParams());
		}

		securityModule.logout();
		runtime.shutdown();
		runtime = null;
		runtime.getLogger(RuntimeTestCase.RUNTIME_PACKAGE).removeHandler(recordHandler);
	}
	public void testCallSuite() throws Exception {
		CallFactory.getInstance().executeCallSuite(mumps);
	}
	protected void tearDown() throws Exception {
		securityModule.logout();
		super.tearDown();
	}
	private IMumps mumps;
	private ISecurity securityModule;
}
