package gov.va.med.hds.cd.m;

import gov.va.med.hds.util.mumps.StringUtil;

import java.io.Serializable;

import org.dom4j.Element;
import org.dom4j.Node;
import org.jaxen.dom4j.Dom4jXPath;

/**
 * A Mumps Call Request entry for use in an RPC call cache
 */
public class CallRequestCacheEntry implements Serializable {
	public static final String CALL_ENTRY_RESULT_ELEMENT_NAME = "result";
	public static final String CALL_ENTRY_RESULT_ELAPSED_TIME_NAME = "elapsedTime";
	public static final String CALL_ENTRY_ROOT_ELEMENT_NAME = "callEntry";
	/**
	 * Constructor
	 * @param request Call request
	 * @param result Result from the call
	 * @throws CallRequestCacheException
	 */
	public CallRequestCacheEntry(ICallRequest request, TimedResult result)
		throws CallRequestCacheException {
		if (request == null)
			throw new CallRequestCacheException("The CallRequest cannot be null");
		this.request = request;
		this.result = result;
	}
	/**
	 * Constructs from an XML cache node
	 * @param node
	 * @throws CallRequestCacheException
	 */
	public CallRequestCacheEntry(Node node) throws CallRequestCacheException {
		try {
			Node resultNode =
				(Node) (new Dom4jXPath(CALL_ENTRY_RESULT_ELEMENT_NAME)).selectSingleNode(node);
			String r = resultNode.getText();
			String resultTemp = StringUtil.restoreCDATATags(r);
			Node timeNode =
				(Node) (new Dom4jXPath(CALL_ENTRY_RESULT_ELAPSED_TIME_NAME)).selectSingleNode(node);

			long elapsedTime = 0;
			if (timeNode != null) {
				elapsedTime = Long.parseLong(timeNode.getText());
			}
			result = new TimedResult(resultTemp, elapsedTime);
			request = new CallRequest(node);
		} catch (Exception e) {
			throw new CallRequestCacheException(e);
		}
	}
	/**
	 * Constructs from and XML element
	 * @param parent
	 */
	public void construct(Element parent) {
		Element entry = parent.addElement(CALL_ENTRY_ROOT_ELEMENT_NAME);
		request.write(entry);
		Element resultElement = entry.addElement(CALL_ENTRY_RESULT_ELEMENT_NAME);
		if (result != null && !result.equals("")) {
			resultElement.addCDATA(StringUtil.replaceCDATATags(result.result));
		}

		Element timeElement = entry.addElement(CALL_ENTRY_RESULT_ELAPSED_TIME_NAME);
		timeElement.addText(String.valueOf(result.elapsedTime));
	}
	/**
	 * Gets the call request
	 * @return
	 */
	public ICallRequest getRequest() {
		return request;
	}
	/**
	 * Gets the timed result of the call
	 * @return
	 */
	public TimedResult getResult() {
		return result;
	}
	private ICallRequest request = null;
	private TimedResult result = null;
}
