package gov.va.med.hds.cd.m;
import java.util.Collection;
import gov.va.med.hds.cd.security.UserCancelledAuthenticationException;
/**
 * Provides an abstraction for a wrapper around the RPC Shared Broker
 */
public interface IBroker extends IConnection {
	/**
	 * Sets the login string. If a login string is supplied then
	 * silent login is enabled.
	 * 
	 * @param login
	 */
	void setLogin(String login);
	/**
	 * Sets the password string.
	 * 
	 * @param password
	 */
	void setPassword(String password);
	/**
	 * Sets the division string
	 * 
	 * @param division
	 */
	void setDivision(String division);
	/**
	 * Enables silent login
	 * 
	 * @param silent set to true to enable silent login
	 */
	void setSilentLogin(boolean silent);
	/**
	 * Sets the time out in seconds to establish a connection. If connection takes longer
	 * then the timeout will fail.
	 * 
	 * @param connectTimeLimitInMillis
	 */
	void setConnectTimeLimit(int connectTimeLimitInMillis);
	/**
	 * Sets the MUMPS context.
	 * 
	 * @param context
	 * @throws SetContextException
	 */
	void setContext(String context) throws SetContextException;
	/**
	 * Sets the name of the MUMPS server
	 * 
	 * @param serverName
	 */
	void setServerName(String serverName);
	/**
	 * Sets the port to connect to on the MUMPS server
	 * 
	 * @param serverPort
	 */
	void setServerPort(int serverPort);
	/**
	 * Sets the name of the client for debugging purposes.
	 * 
	 * This name is arbitrary
	 * 
	 * @param clientName
	 */
	void setClientName(String clientName);
	/**
	 * @see gov.va.med.hds.cd.m.IConnection#connect()
	 */
	void connect() throws ConnectException, UserCancelledAuthenticationException;
	/**
	 * Sets the time out in seconds for RPC calls.
	 * 
	 * @param rpcTimeLimitInMillis
	 */
	void setRPCTimeLimit(int rpcTimeLimitInMillis);
	/**
	 * Enables programmer level debugging against a MUMPS server.
	 * 
	 * @param wantDebug
	 */
	void setWantDebug(boolean wantDebug);
	/**
	 * Returns programmer level debugging state
	 * 
	 * @return boolean
	 */
	boolean getWantDebug();
	/**
	 * Calls a MUMPS RPC.
	 * The rpcName and params are marshalled properly for the underlying MUMPS connection.
	 * 
	 * @param rpcName name of the RPC
	 * @param params each entry in params can be a Collection, Map or single value
	 * @return String result of the call
	 * @throws CallException
	 */
	String call(String rpcName, Collection params) throws CallException;
}
