package gov.va.med.hds.cd.m;

/**
 * Provides an abstraction for queue based requests.
 */
public interface ICallQueue {
	/**
	 * Adds a request to the connected request processing queue
	 * <p>
	 * This is an <b>asyncronous</b> call.
	 * </p>
	 * 
	 * @param request - connect request to be processed
	 * @param listener - listener to be notified once the request has been processed
	 * @throws MumpsException
	 */
	void add(ICallRequest request, IMumpsListener listener) throws MumpsException;
	/**
	 * Adds a request to the connected request processing queue
	 * <p>
	 * This is an <b>syncronous</b> call.
	 * </p>
	 * 
	 * @param request - request to be processed
	 * @return String - result of the request
	 * @throws MumpsException
	 */
	String add(ICallRequest request) throws MumpsException;

	/**
	 * Adds a log listener to the call queue. The log listener is notified when
	 * a call is completed
	 * 
	 * @param listener
	 * @throws MumpsException
	 */
	void addLogListener(IRPCLogListener listener) throws MumpsException;

	/**
	 * Removes a log listener from the call queue.
	 * 
	 * @param listener
	 * @throws MumpsException
	 */
	void removeLogListener(IRPCLogListener listener) throws MumpsException;

	/**
	 * Encrypts data based on underlying implementation
	 * 
	 * @param data
	 * @return String
	 * @throws EncryptException
	 */
	String encrypt(String data) throws EncryptException;

	/**
	 * Decrypts data based on underlying implementation
	 * 
	 * @param data
	 * @return String
	 * @throws EncryptException
	 */
	String decrypt(String data) throws EncryptException;
	
	/**
	 * Enables a global rpc time limit override
	 * @param rpcTimeoutOverride time in milliseconds
	 */
	void enableTimeLimitOverride(int rpcTimeoutOverride);
	
	/**
	 * Disables the effect of calling enableTimeLimitOverride
	 */
	void disableTimeLimitOverride();
}
