package gov.va.med.hds.cd.m;
import java.util.List;

import org.dom4j.Element;
/**
 * Abstracts a call request
 */
public interface ICallRequest {
	/**
	 * Sets the time limit (in milliseconds) after which the call request will fail
	 * @param timeLimitInMillis the time limit of the call
	 */
	public void setCallTimeLimit(int timeLimitInMillis);
	/**
	 * Gets the current call request time out in millis.
	 * @return long time limit in millis
	 */
	public int getCallTimeLimit();
	/**
	 * Sets the name of the call request.
	 * @param name call request name
	 */
	public void setCallName(String name);
	/**
	 * Gets the name of the call request.
	 * @return String call request name
	 */
	public String getCallName();
	/**
	 * Sets the parameters to be passed to the call request.
	 * @param params parameters
	 */
	public void setParams(List params);
	/**
	 * Gets the parameters of the call request.
	 * @return Collection parameters
	 */
	public List getParams();
	/**
	 * Sets the context for the call request.
	 * @param context context for the call request
	 */
	public void setContext(String context);
	/**
	 * Gets the context for the call request
	 * @return String context for the call request
	 */
	public String getContext();
	
	/**
	 * Writes the call request to an XML element
	 * @param entry
	 */
	public void write(Element entry);
	
	/**
	 * Makes a deep copy of this instance.
	 * @return ICallRequest
	 * @throws CallRequestException
	 */
	ICallRequest copy() throws CallRequestException;

}
