package gov.va.med.hds.cd.m;

import java.util.List;

/**
 * Abstracts a MUMPS connection enabling RPC calles and access to server information
 */
public interface IMumps {
	/**
	 * Makes a synchronous RPC call with no params
	 * @param rpcKey Name of the RPC
	 * @return result As a String
	 */
	public String callRPC(String rpcKey) throws MumpsException;
	
	/**
	 * Makes a synchronous RPC call with no params
	 * @param rpcKey
	 * @param timeLimit
	 * @return
	 * @throws MumpsException
	 */
	public String callRPC(String rpcKey,int timeLimit) throws MumpsException;
	
	/**
	 * Makes a synchronous RPC call
	 * @param rpcKey Name of the RPC
	 * @param params Arguments for the RPC
	 * @return result As a String
	 */
	public String callRPC(String rpcKey, Object[] params) throws MumpsException;
	
	/**
	 * Makes a synchronous RPC call
	 * @param rpcKey
	 * @param params
	 * @param timeLimit
	 * @return
	 * @throws MumpsException
	 */
	public String callRPC(String rpcKey, Object[] params, int timeLimit) throws MumpsException;

	/**
	 * @param rpcKey
	 * @param params
	 * @return
	 * @throws MumpsException
	 */
	public String callRPC(String rpcKey, List params) throws MumpsException;
	
	/**
	 * @param rpcKey
	 * @param params
	 * @param timeLimit
	 * @return
	 * @throws MumpsException
	 */
	public String callRPC(String rpcKey, List params, int timeLimit) throws MumpsException;

	/**
	 * Makes an asynchronous RPC call with no params
	 * 
	 * @param rpcKey Name of the RPC
	 * @param listener Listner to receive success or failure
	 */
	public void callRPC(String rpcKey,IMumpsListener listener) throws MumpsException;
	
	/**
	 * Makes an asynchronous RPC call with no params
	 * 
	 * @param rpcKey Name of the RPC
	 * @param listener Listner to receive success or failure
	 * @param timeLimit
	 * @throws MumpsException
	 */
	public void callRPC(String rpcKey,IMumpsListener listener,int timeLimit) throws MumpsException;
	
	/**
	 * Makes an asynchronous RPC call
	 * 
	 * @param rpcKey Name of the RPC
	 * @param params Arguments for the RPC
	 * @param listener Listner to receive success or failure
	 */
	public void callRPC(String rpcKey, Object[] params, IMumpsListener listener) throws MumpsException;
	
	/**
	 * Makes an asynchronous RPC call
	 * 
	 * @param rpcKey Name of the RPC
	 * @param params Arguments for the RPC
	 * @param listener Listener to receive success or failure
	 * @param timeLimit
	 * @throws MumpsException
	 */
	public void callRPC(String rpcKey, Object[] params, IMumpsListener listener, int timeLimit) throws MumpsException;
	
	/**
	 * @param rpcKey
	 * @param params
	 * @param listener
	 * @throws MumpsException
	 */
	public void callRPC(String rpcKey, List params, IMumpsListener listener) throws MumpsException;
	
	/**
	 * @param rpcKey
	 * @param params
	 * @param listener
	 * @param timeLimit
	 * @throws MumpsException
	 */
	public void callRPC(String rpcKey, List params, IMumpsListener listener, int timeLimit) throws MumpsException;
	
	/**
	 * Adds a log listener to the call queue. The log listener is notified when
	 * a call is completed
	 * 
	 * @param listener
	 * @throws MumpsException
	 */	
	void addLogListener(IRPCLogListener listener) throws MumpsException;
	
	/**
	 * Removes a log listener from the call queue.
	 * 
	 * @param listener
	 * @throws MumpsException
	 */
	void removeLogListener(IRPCLogListener listener) throws MumpsException;
	
	/**
	 * Encrypts data based on underlying implementation
	 * 
	 * @param data
	 * @return String
	 * @throws EncryptException
	 */
	String encrypt(String data) throws EncryptException;
	
	/**
	 * Decrypts data based on underlying implementation
	 * 
	 * @param data
	 * @return String
	 * @throws EncryptException
	 */
	String decrypt(String data)throws EncryptException;
}
