package gov.va.med.hds.cd.m;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.mum.VersionMatch;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;

/**
 * Implements a basic OPTION based on MUMPS
 */
public class OptionContext implements IOptionContext {
	public static final String EXTENSION_POINT_ID = "gov.va.med.hds.cd.m.optionContext";
	public static final String ROOT_TAG = "optionContext";
	public static final String NAME_ATTR = "name";
	public static final String VERSION_ATTR = "version";
	public static final String MATCH_ATTR = "match";
	private String name;
	private Version version;
	private IPluginDescriptor plugin;
	private VersionMatch condition;
	
	/**
	 * Constructor
	 */
	public OptionContext(String name, Version version, IPluginDescriptor plugin, VersionMatch condition) {
		this.name = name;
		this.version = version;
		this.plugin = plugin;
		this.condition = condition;
	}

	/**
	 * Constructor
	 */
	public OptionContext(IPluginDescriptor plugin) throws OptionContextExtensionConfigurationException, NoOptionContextInPluginDescriptorException{
		IExtension[] extensions = plugin.getExtensions();
		IExtension ext;
		boolean foundOne = false;
		for(int i=0;i<extensions.length;i++){
			ext = extensions[i];
			if(ext.getExtensionPointUniqueIdentifier().equals(EXTENSION_POINT_ID)){
				IConfigurationElement[] elems = ext.getConfigurationElements();
				for(int j=0;j<elems.length;j++){
					if(elems[j].getName().equals(ROOT_TAG)){
						if(foundOne)
							throw new OptionContextExtensionConfigurationException("The plugin "+plugin.getUniqueIdentifier()+" has more than one extension "+EXTENSION_POINT_ID+".\n Only one maximum is allowed per plugin.");
						String val = elems[j].getAttribute(NAME_ATTR);
						if(val != null)
							name = val;
						else
							throw new OptionContextExtensionConfigurationException("The plugin "+plugin.getUniqueIdentifier()+" extension "+EXTENSION_POINT_ID+" is missing the "+NAME_ATTR+" attribute");

						val = elems[j].getAttribute(VERSION_ATTR);
						if(val != null)
							version = new Version(val);
						else
							throw new OptionContextExtensionConfigurationException("The plugin "+plugin.getUniqueIdentifier()+" extension "+EXTENSION_POINT_ID+" is missing the "+VERSION_ATTR+" attribute");
							
						val = elems[j].getAttribute(MATCH_ATTR);
						if(val != null)
							condition = VersionMatch.getVersionMatch(val);
						else
							condition = VersionMatch.GREATER_OR_EQUAL;
						
						this.plugin = plugin;
						foundOne = true;
					}
				}
			}
		}
		if(!foundOne)
			throw new NoOptionContextInPluginDescriptorException("No extension "+EXTENSION_POINT_ID+" was found in plugin "+plugin.getUniqueIdentifier()+"\nAt least one is required.");		
	}

	/**
	 * @see gov.va.med.hds.cd.m.IOptionContext#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * @see gov.va.med.hds.cd.m.IOptionContext#getVersion()
	 */
	public Version getVersion() {
		return version;
	}

	/**
	 * @see gov.va.med.hds.cd.m.IOptionContext#getPluginDescriptor()
	 */
	public IPluginDescriptor getPluginDescriptor() {
		return plugin;
	}

	/**
	 * @see gov.va.med.hds.cd.m.IOptionContext#getVersionMatch()
	 */
	public VersionMatch getVersionMatch() {
		return condition;
	}
}
