package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.InvalidPluginException;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

/**
 * Used to create IConfigPlugin instances.
 */
public class ConfigPluginFactory {

	/**
	 * Creates an IConfigPlugin instance based on the passed in manifest URL.
     * @param cache
     * @param url
     * @return
     * @throws InvalidPluginException
     */
	public static IConfigPlugin newConfigPlugin(ClassDataCache cache, URL url)
		throws InvalidPluginException {
		if (enabled) {
			if (alternateClass != null) {
				try {
					return (IConfigPlugin) alternateClass.newInstance(
						new Object[] { url });
				} catch (InstantiationException e) {
				} catch (IllegalAccessException e) {
				} catch (InvocationTargetException e) {
				}
			}
			return new ConfigPlugin(cache, url);
		} else
			return null;
	}

	/**
	 * Returns the enabled status.  When disabled, newConfigPlugin returns null
	 * @return boolean
	 */
	public static boolean isEnabled() {
		return enabled;
	}

	/**
	 * Sets the enabled status.  When disabled, newConfigPlugin returns null.
	 * @param enabled - The enabled status to set
	 */
	public static void setEnabled(boolean enabled) {
		ConfigPluginFactory.enabled = enabled;
	}

	/**
	 * Assigns a new IConfigPlugin class to instantiate when newConfigPlugin
	 * is callse.  Used primarily in unit testing.
	 * @param cls
	 */
	public static void assignAlternateConfigPluginClass(Class cls) {
		if (cls == null)
			alternateClass = null;
		else {
			try {
				alternateClass = cls.getConstructor(new Class[] { URL.class });
			} catch (NoSuchMethodException e) {
				alternateClass = null;
			}
		}
	}

	private static boolean enabled = true;
	private static Constructor alternateClass;
}
