package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Used to locate plugins in the gold directory
 */
public class ConfigPluginLocator implements IConfigPluginLocator, IConfigConstants {

	/**
     * Constructor for ConfigPluginLocator 
     */
	protected ConfigPluginLocator(File rootDir, ClassDataCache cache) {
		super();
		this.rootDir = rootDir;
		this.cache = cache;
	}

	/**
     * @see gov.va.med.hds.cd.mum.config.IConfigPluginLocator#createDistributionPlugins(gov.va.med.hds.cd.mum.boot.utils.ListMap)
     */
    public void createDistributionPlugins(ListMap list)
		throws PluginLocatorException {
		if (rootDir == null)
			throw new PluginLocatorException(LOCATOR_NOT_INITIALIZED);
		File pluginDir = getPluginDir(rootDir);
		if (pluginDir == null)
			throw new PluginLocatorException(PLUGIN_DIRECTORY_NOT_FOUND);
		findFiles(list, pluginDir);
	}

	private File getPluginDir(File rootDir) {
		if (rootDir.isDirectory()) {
			if (rootDir.getName().equals(PLUGIN_DIRECTORY_NAME))
				return rootDir;
			File[] files = rootDir.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].getName().equals(PLUGIN_DIRECTORY_NAME))
					return files[i];
			}
		}
		return null;
	}

	private void findFiles(ListMap list, File pluginDir)
		throws PluginLocatorException {
		File[] files = pluginDir.listFiles();
		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory()) {
				File manifest = new File(files[i], IBootConstants.MANIFEST_FILE);
				if (manifest.exists())
					try {
						URL url = manifest.toURL();
						try {
							IConfigPlugin plugin = ConfigPluginFactory.newConfigPlugin(cache, url);
							list.add(plugin.getID(), plugin);
						} catch (InvalidPluginException e) {
							throw new PluginLocatorException(e);
						}
					} catch (MalformedURLException e) {
					}
			}
		}
	}

	private File rootDir;
	private ClassDataCache cache;
}
