package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.IPluginExceptionManager;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.utils.MUMChecksum;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * Analyzes all class files in a directory, and determines various information
 * about classes in the directory.
 */
public class DirectoryAnalyzer extends ClassCollectionAnalyzer {

    private SimpleList list;
    private MUMChecksum chksumSize;
    private MUMChecksum chksumOther;
    private long count;

    /**
     * Constructor for DirectoryAnalyzer.
     * @param errorManager
     */
    public DirectoryAnalyzer(ClassDataCache cache, IPluginExceptionManager errorManager) {
        super(cache, errorManager);
        list = new SimpleList();
        chksumSize = new MUMChecksum();
        chksumOther = new MUMChecksum();
    }
    
    /**
     * @see gov.va.med.hds.cd.mum.config.ClassCollectionAnalyzer#reset()
     */
    protected void reset() {
        super.reset();
        list.clear();
        chksumSize.reset();
        chksumOther.reset();
        count = 0;
    }

    private void update(File file) {
        count++;
        chksumSize.update(file.length());
        chksumOther.update(file.toString());
        chksumOther.update(file.lastModified());
        if (!file.isDirectory())
            list.add(file);
    }

    /**
     * Analyzes all the class files in the the directory and any sub-
     * directories, and updates itself accordingly.
     * @param dir - the directory to search
     * @throws InvalidPluginException
     */
    public void analyzeDirectory(File dir) throws InvalidPluginException {
        reset();
        if (dir != null) {
            searchDirectory(dir);
            chksumSize.update(count);
            if (needsClassData(ClassDataCache.Utils.getID(dir),
                chksumSize.getValue(),
                chksumOther.getValue())) {
                try {
                    for (int i = 0; i < list.size(); i++) {
                        File file = (File) list.get(i);
                        InputStream stream = new BufferedInputStream(new FileInputStream(file));
                        analyzeClass(stream, true);
                    }
                    saveClassData();
                } catch (FileNotFoundException e) {
                    error(e);
                } catch (InvalidFileStreamException e) {
                    error(e);
                }
            }
        }
        validateClasses();
    }

    private void searchDirectory(File dir) {
        update(dir);
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].isDirectory())
                searchDirectory(files[i]);
            else {
                if (files[i].getName().endsWith(IBootConstants.CLASS_EXT)) {
                    update(files[i]);
                }
            }
        }
    }
}
