package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.KeyedList;

import java.io.PrintStream;

/**
 * This class maintains a list of Error instances, representing all
 * configuration errors for a given configuration.
 */
public class ErrorManager implements IConfigConstants {

    private KeyedList errors;
    private long errorMask;

    /**
     * Constructor for ErrorManager 
     */
    public ErrorManager() {
        super();
        errors = new KeyedList();
    }

    /**
     * Returns the list of Error instances.
     * @return KeyedList
     */
    public KeyedList getErrors() {
        return errors;
    }

    /**
     * Returns true if there are any configuration errors.
     * @return boolean
     */
    public boolean hasErrors() {
        return (errorMask != 0);
    }

    /**
     * Returns true if there are any errors with the specified error code.
     * @param errorCode
     * @return boolean
     */
    public boolean hasError(Error.ErrorType errorType) {
        return ((errorMask & errorType.getErrorCode()) != 0);
    }

    /**
     * Resets the error manager to contain no errors.
     */
    public void clear() {
        errors.clear();
        errorMask = 0;
    }

    /**
     * //JMTODO - JAVADOC!!!
     * @param errorType
     * @return
     */
    public Error addError(Error.ErrorType errorType) {
        return addError(errorType, null, null, null, null);
    }

    public Error addError(
        Error.ErrorType errorType,
        IConfigPlugin plugin1,
        IConfigPlugin plugin2) {
        return addError(errorType, null, plugin1, plugin2, null);
    }

    public Error addError(
        Error.ErrorType errorType,
        String message,
        IConfigPlugin plugin1,
        IConfigPlugin plugin2) {
        return addError(errorType, message, plugin1, plugin2, null);
    }

    public Error addError(Error.ErrorType errorType, IConfigPlugin plugin) {
        return addError(errorType, null, plugin, null, null);
    }

    public Error addError(Error.ErrorType errorType, String message) {
        return addError(errorType, message, null, null, null);
    }

    public Error addError(Error.ErrorType errorType, String message, IConfigPlugin plugin) {
        return addError(errorType, message, plugin, null, null);
    }

    public Error addError(
        Error.ErrorType errorType,
        String message,
        IConfigPlugin plugin1,
        IConfigPlugin plugin2,
        IConfigPlugin foundInPlugin) {
        errorMask = errorMask | errorType.getErrorCode();
        Error error = null;
        boolean addError;
        String key = getKey(errorType, message);
        if (errorType.shareMessage()) {
            error = (Error) errors.get(key);
            addError = (error == null);
        } else
            addError = true;
        if (addError) {
            error = errorType.createError();
            errors.add(key, error);
        }
        if ((message != null) && (message.length() > 0) && (error instanceof Error.MessageError))
             ((Error.MessageError) error).setMessage(message);
        error.addPlugin(plugin1);
        error.addPlugin(plugin2);
        error.addFound(foundInPlugin);
        return error;
    }

    public void addError(Error error) {
        errorMask = errorMask | error.getErrorType().getErrorCode();
        String message;
        boolean addError;
        if (error instanceof Error.MessageError)
            message = error.getMessage();
        else
            message = null;
        String key = getKey(error.getErrorType(), message);
        if (error.getErrorType().shareMessage())
            addError = (errors.get(key) == null);
        else
            addError = true;
        if (addError)
            errors.add(key, error);
    }

    public void addErrors(ErrorManager manager) {
        for (int i = 0; i < manager.getErrors().size(); i++) {
            addError((Error) manager.getErrors().get(i));
        }
    }

    private static StringBuffer buffer;

    private static String getKey(Error.ErrorType errorType, String message) {
		if (buffer == null)
			buffer = new StringBuffer();
		else
			buffer.setLength(0);
        if (errorType.shareMessage()) {
            buffer.append(Integer.toString(errorType.getErrorNumber()));
            buffer.append('*');
            buffer.append(message);
        } else {
        	buffer.append(".*.*.");
        	messageID++;
			buffer.append(Long.toString(messageID));
        }
		return buffer.substring(0, buffer.length());
    }

    private static long messageID = 0;

    /**
     * Prints all errors to the specified output stream.
     * @param out
     */
    public void printErrors(PrintStream out) {
        for (int i = 0; i < errors.size(); i++) {
            Error error = (Error) errors.get(i);
            out.print(error.toString());
            out.print(IBootConstants.EOL);
        }
    }
}
