package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.IDependentPlugin;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;

/**
 * Represents an existing plugin found in the gold directory.  Combines
 * information from the actual, installed plugin, and from the plugin's
 * manifest.xml file.
 */
public interface IConfigPlugin extends IDependentPlugin {
	
	/**
	 * Returns a list of Strings representing the classes this plugin
	 * implements;
	 * @return Collection
	 */
	SimpleList getClasses();
	
	/**
	 * Returns a URL to the plugin file location on the server
	 * @param index
	 * @return URL
	 */
	ConfigFileEntry getFile(int index);
	
	/**
	 * Returns an ignore code based on optional xml in the manifest file
	 * directing the plugin to ignore confuguration error conditions.  If
	 * the plugin specifies an ignore="unavailable" attribute, this method
	 * will return IConfigConstants.IGNORE_UNAVAILABLE, otherwise it will return
	 * IConfigConstants.IGNORE_NONE.  This funationality is intended only for
	 * those third party libraries that ship with class dependencies that are
	 * not provided by the library, and are not provided in another plugin.
	 * @return int
	 */
	int getIgnoreCode();
	
	/**
	 * The name of the class implementing the IStartup interface.  If the plugin
	 * does not have any classes implementing the IStartup interface, this
	 * returns null.  Only one plugin can have a startup class for any given
	 * configuration.
	 * @return String
	 */
	String getStartupClassName();

	/**
	 * Returns a list of Strings representing the classes this plugin is
	 * dependent upon.
	 * @return Collection
	 */
	SimpleList getClassDependencies();

	/**
	 * Returns a list of PluginDepencendy objects representing the plugins this plugin is
	 * dependent upon.
	 * @return Collection
	 */
	ListMap getPluginDependencies();	
	
	/**
     * Returns a list of String values representing the id's of declared extension points
     */
    SimpleList getExtensionPoints();
	
	/**
	 * Returns a list of String values representing the id's of extended extension points
	 */
	SimpleList getExtensions();
	
	/**
     * Returns a list of Strings representing the classes specified in the manifest 
     * as being required.  This includes the plugin class activated by the Runtime, 
     * as well as classes declared in extenions that are activated by the Runtime for
     * the plugin implementing the extension point.  
     */
    SimpleList getDeclaredClassDependencies();
}
