package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;

import java.io.File;
import java.util.List;

/**
 * Locates all available plugins found in the gold directory.  It is
 * subsequently used by Configurations to provide a list of available plugins.
 */
public interface IPluginManager {
	/**
	 * Assigns the gold directory used to locate plugins.
	 * @param goldDir
	 */
	void assignGoldDirectory(File goldDir);

	/**
	 * Populates the plugin list with plugins in the gold directory.
	 * @throws PluginLocatorException
	 */
	void load() throws PluginLocatorException;

	/**
	 * Clears out the plugin list.
	 */
	void reset();

	/**
	 * Returns a list of owned IConfigPlugin objects, representing all
	 * available plugins.  Only valid after load() has been called.
	 * @return List
	 */
	ListMap getPlugins();

	/**
	 * Returns true if plugins have been loaded into the plugin list.
	 * @return boolean
	 */
	boolean isLoaded();

	/**
	 * Loads all the RootElements for all plugins into a single element list.
	 */
	void loadPluginInformation();

	/**
	 * Returns a list of all classes for all plugins, after loadPluginInformation() has
	 * been called.
	 * @return Collection
	 */
	SimpleList getClasses();

	/**
	 * Returns true if any of the plugins in the plugin list return true from
	 * the PluginID.supports() method for the passed in plugin.  This method is
	 * used when a plugin is validating it's plugin dependencies, and an
	 * unresolved dependency was found.  If this method returns true, a
	 * ConfigError.UNRESOLVED_DEPENDENT_PLUGIN_ERROR error is created.  If it
	 * returns false, a ConfigError.UNAVAILABLE_DEPENDENT_PLUGIN_ERROR is
	 * created.
	 * @param plugin
	 * @return boolean
	 */

	boolean satisfiesDependency(PluginDependency dependency);

	/**
	 * Returns all IConfigPlugin objects that implements the passed in class, added to the .
	 * passed in list
	 * @param element
	 * @return IConfigPlugin
	 */
	void getClassPlugins(List list, String cls);
	
	/**
     * Returns the plugins implementing the specified extension point
     */
    List getExtensionPointPlugins(String extensionPoint);
}
