package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.boot.IBootConstants;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * JVMAnalyzer is used to detect all the classes in the core java libraries.
 * This is needed to filter class dependencies detected in a plugin. JVMAnalyzer
 * is a singleton that only analyzes the JVM pacakges once.
 */
public class JVMAnalyzer implements IConfigConstants {

    /**
     * Constructor for JVMAnalyzer is private, to enforce singleton pattern.
     */
    private JVMAnalyzer() {
        super();
        analyzeJVM();
    }

    private static JVMAnalyzer singleton;

    /**
     * Returns the collection of classes found in the JVM.
     * @return Collection
     */
    public static Collection getClasses() {
        if (testCollection != null)
            return testCollection;

        if (singleton == null) {
            singleton = new JVMAnalyzer();
        }
        return singleton.classes;
    }

    private static Collection testCollection;

    /**
     * Used to replate the JVM class list with a simple list used in unit
     * testing.  This method should never be used in production code.
     * @param col
     */
    public static void assignTestCollection(Collection col) {
        testCollection = col;
    }

    private HashSet classes = new HashSet();

    private void analyzeJVM() {
        String bootpath = System.getProperty(JVM_PATH_PROPERTY).replace('\\', '/');
        String delim = File.pathSeparator;
        String[] path = bootpath.split(delim);
        int len = IBootConstants.CLASS_EXT.length();
        for (int i = 0; i < path.length; i++) {
            if (path[i].toLowerCase().endsWith(IBootConstants.JAR_EXT)) {
                File file = new File(path[i]);
                if (file.exists() && (!file.isDirectory())) {
                    try {
                        ZipFile zip = new ZipFile(file);
                        try {
                            Enumeration enum = zip.entries();
                            while (enum.hasMoreElements()) {
                                String name = ((ZipEntry) enum.nextElement()).getName();
                                if (name.endsWith(IBootConstants.CLASS_EXT)) {
                                    classes.add(
                                        name.substring(0, name.length() - len).replace('/', '.'));
                                }
                            }
                        } finally {
                            zip.close();
                        }
                    } catch (ZipException e) {
                    } catch (IOException e) {
                    }
                }
            }
        }
    }
}