package gov.va.med.hds.cd.mum.config;

import gov.va.med.hds.cd.mum.IPluginExceptionManager;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.Utils;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Analyzes all class files in a jar, and determines various information about
 * classes in the jar.
 */
public class JarFileAnalyzer extends ClassCollectionAnalyzer implements IConfigConstants {

    /**
     * Constructor for JarFileAnalyzer.
     */
    public JarFileAnalyzer(ClassDataCache cache, IPluginExceptionManager errorManager) {
        super(cache, errorManager);
    }

    /**
     * Analyzes the jar file pointed to by the passed in URL, and updates itself
     * accordingly.
     * @param url - the url of the jar file to analyze
     * @param checkClasses - determines if the class file analysis will look for
     * classes that implement the IStartup or IApplication interface
     * @throws InvalidPluginException
     */
    public void analyzeJar(URL url) throws InvalidPluginException {
        reset();
        if (url != null) {
            try {
                URLConnection conn = url.openConnection();
                conn.connect();
                if (needsClassData(ClassDataCache.Utils.getID(url),
                    conn.getContentLength(),
                    conn.getLastModified())) {
                    InputStream in = new BufferedInputStream(conn.getInputStream());
                    ZipInputStream zipStream = new ZipInputStream(in);
                    try {
                        analyzeJarEntries(zipStream);
                    } finally {
                        zipStream.close();
                    }
                    saveClassData();
                }
            } catch (Exception e) {
                error(e);
            }
        }
        validateClasses();
    }

    private void analyzeJarEntries(ZipInputStream zipStream)
        throws IOException, InvalidFileStreamException {
        ZipEntry entry;
        while ((entry = zipStream.getNextEntry()) != null) {
            if (!entry.isDirectory()) {
                String entryName = entry.getName().toLowerCase();
                if (entryName.endsWith(IBootConstants.CLASS_EXT)) {
                    analyzeClass(zipStream, false);
                } else
                    addPackage(Utils.getPackageFromZipEntry(entry));
            }
        }
    }
}
