package gov.va.med.hds.cd.mum.boot.test;

import gov.va.med.hds.cd.mum.boot.Boot;
import gov.va.med.hds.cd.mum.boot.BootFailureException;
import gov.va.med.hds.cd.mum.boot.IAbortManager;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.IncompatibleVersionException;
import gov.va.med.hds.cd.mum.boot.StatusNotifierFactory;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;

import java.net.URL;

import junit.framework.TestCase;

/**
 * enclosing_type
 */
public class BootTest extends TestCase implements IAbortManager, IBootConstants, IStatusNotifier {

    public void testBadJVMVersion() throws BootFailureException {
        assertNull(error);
        String oldVersion = System.getProperty(JVM_VERSION_PROPERTY);
        try {
            error = null;
            System.setProperty(JVM_VERSION_PROPERTY, "0.5.5_");
            Boot.main(getClass().getClassLoader(), null);
            assertTrue(error.getClass().equals(IncompatibleVersionException.class));
            error = null;
            System.setProperty(JVM_VERSION_PROPERTY, "1.2.5");
            Boot.main(getClass().getClassLoader(), null);
            assertTrue(error.getClass().equals(IncompatibleVersionException.class));
            error = null;
            System.setProperty(JVM_VERSION_PROPERTY, "1.4.0_dfsaf");
            Boot.main(getClass().getClassLoader(), null);
            assertTrue(error.getClass().equals(IncompatibleVersionException.class));
        } finally {
            System.setProperty(JVM_VERSION_PROPERTY, oldVersion);
        }
    }

    public void testBootOutsideJar() {
        URL url =
            this.getClass().getClassLoader().getResource(
                this.getClass().getName().replace('.', '/').concat(CLASS_EXT));
        boolean inJar = url.getProtocol().equals("jar");
        if (inJar) {
        } else {
            Boot.main(getClass().getClassLoader(), null);
            assertNotNull(error);
            assertEquals(BootFailureException.class, error.getClass());
            assertEquals(NO_STARTUP_JAR_FOUND, error.getMessage());
        }
    }

    public void testSplashCloseOnError() throws Throwable {
        Boot.setDoSplashScreen(true);
        StatusNotifierFactory.assignStatusNotifier(this);
        try {
            closedBeforeError = false;
            Boot.main(getClass().getClassLoader(), null);
            assertNotNull(error);
            assertTrue(closedBeforeError);
            assertTrue(splashCalled);
        } finally {
            StatusNotifierFactory.assignStatusNotifier(null);
        }
    }

    public void testNoSplashOnSilentUpdate() {
        Boot.setDoSplashScreen(true);
        StatusNotifierFactory.assignStatusNotifier(this);
        try {
            Boot.main(getClass().getClassLoader(), new String[] { SILENT_UPDATE_TAG });
            assertNotNull(error);
            assertFalse(splashCalled);
        } finally {
            StatusNotifierFactory.assignStatusNotifier(null);
        }
    }

    //********************
    //*** end of tests ***
    //********************

    /**
     * Constructor for BootTest.
     * @param arg0
     */
    public BootTest(String arg0) {
        super(arg0);
    }

    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        error = null;
        ClassLoaderManager.resetSharedLoader();
        Boot.assignAbortManager(this);
        Boot.setDoSplashScreen(false);
        splashCalled = false;
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        Boot.assignAbortManager(null);
        Boot.setDoSplashScreen(true);
    }

    private Throwable error;
    private boolean splashCalled;
    private boolean closedBeforeError;

    public void abort(Throwable e, boolean warnUser) {
        error = e;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#showProgressBar(long, java.lang.String)
     */
    public void showProgressBar(long maxValue, String message) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressBar(long)
     */
    public void updateProgressBar(long value) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressBarMax(long)
     */
    public void updateProgressBarMax(long value) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressMessage(java.lang.String)
     */
    public void updateProgressMessage(String message) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#hideProgressBar()
     */
    public void hideProgressBar() {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#displayMessage(java.lang.String)
     */
    public void displayMessage(String message) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.ISplashScreen#close()
     */
    public void close() {
        splashCalled = true;
        if (error == null)
            closedBeforeError = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setVersion(java.lang.String)
     */
    public void setVersion(String version) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setShowCancelButton(boolean)
     */
    public void showCancelButton(boolean value) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#wasCancelButtonPressed()
     */
    public boolean isCanceled() {
        splashCalled = true;
        return false;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setCancelMessage(java.lang.String)
     */
    public void setCancelMessage(String message) {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#beginDisplayUpdate()
     */
    public void beginDisplayUpdate() {
        splashCalled = true;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#endDisplayUpdate()
     */
    public void endDisplayUpdate() {
        splashCalled = true;
    }
}
