package gov.va.med.hds.cd.mum.boot.test;

import gov.va.med.hds.cd.mum.boot.Boot;
import gov.va.med.hds.cd.mum.boot.BootFailureException;
import gov.va.med.hds.cd.mum.boot.IAbortManager;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.IStartupInfoFactory;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.util.io.FileUtilities;

import java.io.File;
import java.net.URL;

import junit.framework.TestCase;

/**
 * enclosing_type
 */
public class BootTest2
    extends TestCase
    implements IAbortManager, IStartupInfoFactory, IBootConstants {

    public void testStartupLaunchErrors() throws Throwable {
        testCount++;
        Boot.assignStartupClass("a.b.c.NonExistantClass");
        Boot.main(getClass().getClassLoader(), null);
        assertFailure(ClassNotFoundException.class);
        Boot.assignStartupClass("mumtest.internal.boot.BadStartupClass");
        Boot.main(getClass().getClassLoader(), null);
        assertFailure(ClassCastException.class);
        Boot.assignStartupClass("mumtest.internal.boot.BadStartupClass2");
        Boot.main(getClass().getClassLoader(), null);
        assertFailure(InstantiationException.class);
    }

    public void testStartupLaunch() throws Throwable {
        testCount++;
        File flag = new File(startupOverride, "StartupWasRun.Test");
        assertFalse(flag.exists());
        Boot.assignStartupClass("mumtest.internal.boot.StartupTest");
        Boot.main(getClass().getClassLoader(), null);
        if (error != null)
            throw error;
        assertTrue(flag.exists());
    }

    //********************
    //*** end of tests ***
    //********************

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(BootTest2.class);

    /**
     * Constructor for BootTest2.
     * @param arg0
     */
    public BootTest2(String arg0) {
        super(arg0);
    }

    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.initRootTestDirs();
        }
        rootOverride = TestFileBuilder.getDestFile();
        pluginsOverride = new File(rootOverride, PLUGINS_DIRECTORY);
        startupOverride = new File(pluginsOverride, TEST_MUM_DIR);
        if (testCount == 0) {
            startupOverride.mkdirs();
            File newMUM = new File(startupOverride, STARTUP_JAR);
            URL oldMUM =
                this.getClass().getClassLoader().getResource(
                    "gov/va/med/hds/cd/mum/boot/test/testdata/mum.jar");
            FileUtilities.copy(oldMUM, newMUM);
        }
        ClassLoaderManager.resetSharedLoader();
        Boot.assignAbortManager(this);
        Boot.assignStartupInfoFactory(this);
        Boot.setDoSplashScreen(false);
    }

    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
        Boot.assignAbortManager(null);
        Boot.assignStartupInfoFactory(null);
        Boot.setDoSplashScreen(true);
    }

    private static final String TEST_MUM_DIR = "gov.va.med.hds.cd.mum_0.0.0.unitTestVersionOnly";
    private Throwable error;
    private File rootOverride;
    private File pluginsOverride;
    private File startupOverride;

    /**
     * @see gov.va.med.hds.cd.mum.boot.IAbortManager#abort(java.lang.Throwable, boolean)
     */
    public void abort(Throwable e, boolean warnUser) {
        error = e;
    }
    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfoFactory#createStartupInfo(java.lang.String[])
     */
    public IStartupInfo createStartupInfo(String[] args) {
        TestStartupInfo info = new TestStartupInfo(args);
        if (rootOverride != null)
            info.setRootDirectory(rootOverride);
        if (pluginsOverride != null)
            info.setPluginsDirectory(pluginsOverride);
        if (startupOverride != null)
            info.setStartupDirectory(startupOverride);
        return info;
    }

    private void assertFailure(Class cause) {
        assertNotNull(error);
        assertEquals(BootFailureException.class, error.getClass());
        assertEquals(UNEXPECTED_STARTUP_FAILURE, error.getMessage());
        if (cause == null) {
            assertNull(error.getCause());
        } else {
            assertNotNull(error.getCause());
            assertEquals(cause, error.getCause().getClass());
        }
        error = null;
    }
}
