/*
 * Created on May 27, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.mum.boot.utils.test;

import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.Settings;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;

import junit.framework.TestCase;

/**
 * @author vhaislmerrij
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SettingsTests extends TestCase implements IMUMTestConstants {

    public void testRawSettings() throws TreeDataException, IOException {
        ((ISavableTreeData) settings.data()).save(writer);
        assertEquals(settingsXML, writer.toString());
        settings = new Settings(null);
        int root = settings.data().getRootID();
        assertEquals(IBootConstants.SETTINGS_ELEMENT, settings.data().getName(root));
        assertEquals(0, settings.data().getChildCount(root));
        assertEquals(0, settings.data().getAttributeCount(root));
    }

    public void testMergeArgs() {
        int root = settings.data().getRootID();
        int node1 = settings.data().findChildID(root, "one");
        int node2 = settings.data().findChildID(node1, "two");
        assertNull(settings.data().getValue(node2));
        settings.mergeWithCommandLineParameters(new String[] { "-TeSt=  ccccc  " });
        assertEquals("  ccccc  ", settings.data().getValue(node2));
        settings.mergeWithCommandLineParameters(new String[] { "-a=aaaaa" });
        assertEquals("aaaaa", settings.data().getValue(node2));
        settings.mergeWithCommandLineParameters(new String[] { "-B=bbbbb" });
        assertEquals("bbbbb", settings.data().getValue(node2));
        int node4 = settings.data().findChildID(node1, "four");
        int node5 = settings.data().findChildID(node4, "five");
        int node6 = settings.data().findChildID(node5, "six");
        int node7 = settings.data().findChildID(node5, "seven");
        int node8 = settings.data().findChildID(node5, "eight");
        assertEquals("66666", settings.data().getValue(node6));
        assertEquals("77777", settings.data().getValue(node7));
        assertEquals("88888", settings.data().getValue(node8));
        settings.mergeWithCommandLineParameters(
            new String[] { "-asdfg=whatever", "-whatever=google", "-ARRGG=AAARRRRGGGG" });
        assertEquals("google", settings.data().getValue(node6));
        assertEquals("77777", settings.data().getValue(node7));
        assertEquals("AAARRRRGGGG", settings.data().getValue(node8));
        assertEquals("bbbbb", settings.data().getValue(node2));
    }

    public void testMergeSettings() {
        int root = settings.data().getRootID();
        int node1 = settings.data().findChildID(root, "one");
        int node2 = settings.data().findChildID(node1, "two");
        assertNull(settings.data().getValue(node2));
        settings.mergeWithSettings(new StringReader(" some thing =  one two three four five  "));
        assertEquals("one two three four five", settings.data().getValue(node2));
        int node4 = settings.data().findChildID(node1, "four");
        int node5 = settings.data().findChildID(node4, "five");
        int node6 = settings.data().findChildID(node5, "six");
        int node7 = settings.data().findChildID(node5, "seven");
        int node8 = settings.data().findChildID(node5, "eight");
        assertEquals("66666", settings.data().getValue(node6));
        assertEquals("77777", settings.data().getValue(node7));
        assertEquals("88888", settings.data().getValue(node8));
        settings.mergeWithSettings(
            new StringReader(
                "Smile, you're on Candid CAMERA= cheese"
                    + WIN_EOL
                    + "fdjskalfjkdsaf"
                    + WIN_EOL
                    + "  This is a Test   =gREETINGS, eARTHLING"));
        assertEquals("66666", settings.data().getValue(node6));
        assertEquals("cheese", settings.data().getValue(node7));
        assertEquals("gREETINGS, eARTHLING", settings.data().getValue(node8));
        assertEquals("one two three four five", settings.data().getValue(node2));
    }

    //********************
    //*** end of tests ***
    //********************

    private Settings settings;
    private Writer writer;

    public SettingsTests(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        super.setUp();
        ISavableTreeData data = TreeDataFactory.newSimpleXMLTreeData();
        data.load(new StringReader(config1));
        settings = new Settings(data);
        writer = new StringWriter();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    private static String config1 =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<configuration crc=\"1870539677\">"
            + WIN_EOL
            + "  <startup plugin=\"one.two.three.four.five_1.0.0\"/>"
            + WIN_EOL
            + "  <settings>"
            + WIN_EOL
            + "    <one>"
            + WIN_EOL
            + "      <two clparam=\"test,a,b\" setting=\"Some Thing\"/>"
            + WIN_EOL
            + "      <three>whatever</three>"
            + WIN_EOL
            + "      <four>"
            + WIN_EOL
            + "        <five>"
            + WIN_EOL
            + "          <six clparam=\"Whatever\">66666</six>"
            + WIN_EOL
            + "          <seven setting=\"  Smile, you&apos;re on candid camera  \">77777</seven>"
            + WIN_EOL
            + "          <eight setting=\"this is a test\" clparam=\"arrgg\">88888</eight>"
            + WIN_EOL
            + "        </five>"
            + WIN_EOL
            + "      </four>"
            + WIN_EOL
            + "    </one>"
            + WIN_EOL
            + "  </settings>"
            + WIN_EOL
            + "  <plugin directory=\"one.two.three.four.five_1.0.0\" module=\"one.two.three\" source=\"testing1\"/>"
            + WIN_EOL
            + "  <plugin directory=\"one.two.three.four.five_2.0.0\" module=\"one.two.three\" source=\"testing2\"/>"
            + WIN_EOL
            + "  <plugin directory=\"one.two.three.four.five_3.0.0\" module=\"one.two.three\" source=\"testing3\"/>"
            + WIN_EOL
            + "</configuration>";

    private static String settingsXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<settings>"
            + WIN_EOL
            + "  <one>"
            + WIN_EOL
            + "    <two clparam=\"test,a,b\" setting=\"Some Thing\"/>"
            + WIN_EOL
            + "    <three>whatever</three>"
            + WIN_EOL
            + "    <four>"
            + WIN_EOL
            + "      <five>"
            + WIN_EOL
            + "        <six clparam=\"Whatever\">66666</six>"
            + WIN_EOL
            + "        <seven setting=\"  Smile, you&apos;re on candid camera  \">77777</seven>"
            + WIN_EOL
            + "        <eight setting=\"this is a test\" clparam=\"arrgg\">88888</eight>"
            + WIN_EOL
            + "      </five>"
            + WIN_EOL
            + "    </four>"
            + WIN_EOL
            + "  </one>"
            + WIN_EOL
            + "</settings>";
}
