package gov.va.med.hds.cd.mum.boot.utils.test;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.IntList;
import gov.va.med.hds.cd.mum.boot.utils.TreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;

import junit.framework.TestCase;

/**
 * enclosing_type
 */
public class TreeDataTests extends TestCase implements IMUMConstants, IBootConstants {

    public void testXMLData() throws FileNotFoundException, TreeDataException {
        internalXMLTest(TreeDataFactory.newSAXTreeData());
        internalXMLTest(TreeDataFactory.newSimpleXMLTreeData());
    }

    private void internalXMLTest(ILoadableTreeData data)
        throws FileNotFoundException, TreeDataException {
        assertFalse(data.hasRoot());
        data.load(new StringReader(xmlData1));
        assertTrue(data.hasRoot());
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/sampleXML.xml");
        data.load(url);
        int root = data.getRootID();
        assertEquals("root", data.getName(root));
        assertEquals(1, data.getAttributeCount(root));
        assertEquals("attrib0", data.getAttributeName(root, 0));
        assertEquals("data0", data.getAttributeValue(root, 0));
        assertEquals("data0", data.getAttribute(root, "attrib0"));
        assertEquals(8, data.getChildCount(root));
        int child = data.getChildID(root, 0);
        assertEquals("child1", data.getName(child));
        assertEquals(2, data.getAttributeCount(child));
        assertEquals("attrib1", data.getAttributeName(child, 0));
        assertEquals("attrib2", data.getAttributeName(child, 1));
        assertEquals("data1", data.getAttributeValue(child, 0));
        assertEquals("data2", data.getAttributeValue(child, 1));
        assertEquals("data1", data.getAttribute(child, "attrib1"));
        assertEquals("data2", data.getAttribute(child, "attrib2"));
        assertEquals(0, data.getChildCount(child));
        child = data.getChildID(root, 1);
        assertEquals("child2", data.getName(child));
        assertEquals("  test &&'\"><& value  ", data.getValue(child));
        assertEquals(0, data.getAttributeCount(child));
        assertEquals(0, data.getChildCount(child));
        child = data.getChildID(root, 2);
        assertEquals("child3", data.getName(child));
        assertEquals(3, data.getAttributeCount(child));
        assertEquals("attrib3", data.getAttributeName(child, 0));
        assertEquals("attrib4", data.getAttributeName(child, 1));
        assertEquals("attrib5", data.getAttributeName(child, 2));
        assertEquals("data3", data.getAttributeValue(child, 0));
        assertEquals("data4", data.getAttributeValue(child, 1));
        assertEquals("data5", data.getAttributeValue(child, 2));
        assertEquals("data3", data.getAttribute(child, "attrib3"));
        assertEquals("data4", data.getAttribute(child, "attrib4"));
        assertEquals("data5", data.getAttribute(child, "attrib5"));
        assertEquals("this is a test", data.getValue(child));
        assertEquals(0, data.getChildCount(child));
        child = data.getChildID(root, 3);
        assertEquals("child4", data.getName(child));
        assertEquals(1, data.getAttributeCount(child));
        assertEquals("name", data.getAttributeName(child, 0));
        assertEquals("bi>lly", data.getAttributeValue(child, 0));
        assertEquals("bi>lly", data.getAttribute(child, "name"));
        assertEquals(3, data.getChildCount(child));
        int grandchild = data.getChildID(child, 0);
        assertEquals("item", data.getName(grandchild));
        assertEquals(0, data.getAttributeCount(grandchild));
        assertEquals("one", data.getValue(grandchild));
        assertEquals(0, data.getChildCount(grandchild));
        grandchild = data.getChildID(child, 1);
        assertEquals("item", data.getName(grandchild));
        assertEquals(0, data.getAttributeCount(grandchild));
        assertEquals("two", data.getValue(grandchild));
        assertEquals(0, data.getChildCount(grandchild));
        grandchild = data.getChildID(child, 2);
        assertEquals("item", data.getName(grandchild));
        assertEquals(1, data.getAttributeCount(grandchild));
        assertEquals("number", data.getAttributeName(grandchild, 0));
        assertEquals("3", data.getAttributeValue(grandchild, 0));
        assertEquals("3", data.getAttribute(grandchild, "number"));
        assertEquals("three", data.getValue(grandchild));
        assertEquals(0, data.getChildCount(grandchild));
        child = data.getChildID(root, 4);
        assertEquals("child5", data.getName(child));
        assertNull(data.getValue(child));
        assertEquals(0, data.getAttributeCount(child));
        child = data.getChildID(root, 5);
        assertEquals("child6", data.getName(child));
        assertNull(data.getValue(child));
        assertEquals(0, data.getAttributeCount(child));
        child = data.getChildID(root, 6);
        assertEquals("child7", data.getName(child));
        assertNull(data.getValue(child));
        assertEquals(1, data.getAttributeCount(child));
        assertEquals("bbb", data.getAttribute(child, "aaa"));
        child = data.getChildID(root, 7);
        assertEquals("child8", data.getName(child));
        assertNull(data.getValue(child));
        assertEquals(1, data.getAttributeCount(child));
        assertEquals("ddd", data.getAttribute(child, "ccc"));
    }

    public void testBadXML() throws FileNotFoundException, TreeDataException {
        internalTestBadXML(TreeDataFactory.newSimpleXMLTreeData());
        internalTestBadXML(TreeDataFactory.newSAXTreeData());
    }

    private void internalTestBadXML(ILoadableTreeData data)
        throws TreeDataException, FileNotFoundException {
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/badXML.xml");
        try {
            data.load(url);
            fail("Should have thrown TreeDataException");
        } catch (TreeDataException e) {
        }
        url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/badXML2.xml");
        try {
            data.load(url);
            fail("Should have thrown TreeDataException");
        } catch (TreeDataException e) {
        }
    }

    public void testIgnoreWhitespace() throws FileNotFoundException, TreeDataException {
        internalTestIgnoreWhitespace(TreeDataFactory.newSimpleXMLTreeData());
        internalTestIgnoreWhitespace(TreeDataFactory.newSAXTreeData());
    }

    private void internalTestIgnoreWhitespace(ILoadableTreeData data)
        throws FileNotFoundException, TreeDataException {
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/sampleXML.xml");

        data.ignoreWhitespace(false);
        data.load(url);
        int root = data.getRootID();
        assertNotNull(data.getValue(root));
        assertNull(data.getValue(data.getChildID(root, 0)));
        assertNotNull(data.getValue(data.getChildID(root, 1)));
        assertEquals("  test &&'\"><& value  ", data.getValue(data.getChildID(root, 1)));
        assertNotNull(data.getValue(data.getChildID(root, 2)));
        assertNotNull(data.getValue(data.getChildID(root, 3)));
        data.ignoreWhitespace(true);
        data.load(url);
        root = data.getRootID();
        assertNull(data.getValue(root));
        assertNull(data.getValue(data.getChildID(root, 0)));
        assertNotNull(data.getValue(data.getChildID(root, 1)));
        assertEquals("  test &&'\"><& value  ", data.getValue(data.getChildID(root, 1)));
        assertNotNull(data.getValue(data.getChildID(root, 2)));
        assertNull(data.getValue(data.getChildID(root, 3)));
    }

    public void testIntLists() {
        IntList list = new IntList(1);
        assertEquals(0, list.size());
        try {
            list.get(0);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        list.add(1);
        list.add(2);
        list.add(7);
        assertEquals(3, list.size());
        assertEquals(1, list.get(0));
        assertEquals(2, list.get(1));
        assertEquals(7, list.get(2));
        list.set(5, 123);
        assertEquals(6, list.size());
        assertEquals(0, list.get(3));
        assertEquals(0, list.get(4));
        assertEquals(123, list.get(5));
        try {
            list.get(6);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        try {
            list.get(-1);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        list.clear();
        assertEquals(0, list.size());
        list.set(3, 456);
        list.add(789);
        list.set(7, 12345);
        assertEquals(0, list.get(0));
        assertEquals(0, list.get(1));
        assertEquals(0, list.get(2));
        assertEquals(456, list.get(3));
        assertEquals(789, list.get(4));
        assertEquals(0, list.get(5));
        assertEquals(0, list.get(6));
        assertEquals(12345, list.get(7));
        assertEquals(8, list.size());
        list.remove(3);
        list.set(2, 47328);
        assertEquals(7, list.size());
        assertEquals(0, list.get(0));
        assertEquals(0, list.get(1));
        assertEquals(47328, list.get(2));
        assertEquals(789, list.get(3));
        assertEquals(0, list.get(4));
        assertEquals(0, list.get(5));
        assertEquals(12345, list.get(6));

    }

    public void testIntListCache() {
        TestTreeData.TestIntListCache cache = new TestTreeData.TestIntListCache(1);
        assertEquals(0, cache.size());
        IntList list0 = cache.get(0);
        IntList list3 = cache.get(3);
        assertNotNull(list0);
        assertNotNull(list3);
        assertEquals(4, cache.size());
        list0.add(123);
        list3.add(1);
        list3.add(2);
        list3.add(3);
        assertEquals(1, cache.get(0).size());
        assertEquals(0, cache.get(1).size());
        assertEquals(0, cache.get(2).size());
        assertEquals(3, cache.get(3).size());
        cache.clear();
        assertEquals(0, list0.size());
        assertEquals(0, list3.size());
        assertEquals(4, cache.size());
        cache.get(2).add(1);
        assertEquals(1, cache.get(2).size());
        cache.get(3).add(7);
        assertEquals(1, cache.get(3).size());
        cache.get(4).add(3);
        cache.get(4).add(12);
        assertEquals(2, cache.get(4).size());
        cache.clear(3);
        assertEquals(1, cache.get(2).size());
        assertEquals(0, cache.get(3).size());
        assertEquals(2, cache.get(4).size());
    }

    public void testBufList() {
        TreeData.BufList list = new TreeData.BufList(1);
        try {
            list.get(-1);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        try {
            list.get(0);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        StringBuffer buf2 = list.ensureBuffer(2);
        StringBuffer buf1 = list.get(1);
        assertEquals(3, list.size());
        assertNotNull(buf2);
        assertNull(buf1);
        assertFalse(list.hasData(0));
        assertFalse(list.hasData(1));
        assertFalse(list.hasData(2));
        buf2.append("testing");
        assertTrue(list.hasData(2));
        list.clear();
        assertFalse(list.hasData(2));
        assertEquals(3, list.size());
        assertEquals(0, buf2.length());
    }

    public void testAddingXML() throws TreeDataException {
        internalTestAddingXML(TreeDataFactory.newSimpleXMLTreeData());
        internalTestAddingXML(TreeDataFactory.newSAXTreeData());
    }

    public void internalTestAddingXML(ITreeData data) throws TreeDataException {
        assertFalse(data.hasRoot());
        int root = data.createRoot("testing");
        assertTrue(data.hasRoot());
        assertEquals("testing", data.getName(root));
        try {
            data.createRoot("testing2");
            fail("Should have thrown TreeDataException");
        } catch (RuntimeException e) {
            assertEquals(TreeDataException.class, e.getCause().getClass());
        }
        try {
            data.createChild(-1, "testing2");
            fail("Should have thrown TreeDataException");
        } catch (RuntimeException e) {
            assertEquals(TreeDataException.class, e.getCause().getClass());
        }
        assertEquals(0, data.getChildCount(root));
        int child = data.createChild(root, "whatever");
        assertEquals("whatever", data.getName(child));
        assertEquals(1, data.getChildCount(root));
        assertEquals(child, data.getChildID(root, 0));
        assertEquals(0, data.getAttributeCount(root));
        data.setAttribute(root, "attrib1", "oops");
        data.setAttribute(root, "attrib3", "oops");
        data.setAttribute(root, "attrib3", null);
        data.setAttribute(root, "attrib4", null);
        data.setAttribute(root, "attrib1", "value1");
        data.setAttribute(root, "attrib2", "value2");
        assertEquals(2, data.getAttributeCount(root));
        assertEquals("attrib1", data.getAttributeName(root, 0));
        assertEquals("attrib2", data.getAttributeName(root, 1));
        assertEquals("value1", data.getAttributeValue(root, 0));
        assertEquals("value2", data.getAttributeValue(root, 1));
        assertEquals("value1", data.getAttribute(root, "attrib1"));
        assertEquals("value2", data.getAttribute(root, "attrib2"));
        assertNull(data.getAttribute(root, "smile"));
        data.setValue(root, "test12345");
        assertEquals("test12345", data.getValue(root));
        data.setValue(child, "12345");
        assertEquals("12345", data.getValue(child));
        data.setValue(root, null);
        assertNull(data.getValue(root));
        assertEquals("12345", data.getValue(child));
        int child1 = data.createChild(root, "testing1");
        data.setAttribute(child1, "one", "1");
        int child2 = data.createChild(root, "testing2");
        data.setAttribute(child2, "two", "2");
        int child3 = data.createChild(root, "testing3");
        data.setAttribute(child3, "three", "3");
        assertEquals(4, data.getChildCount(root));
        assertEquals(child3, data.getChildID(root, 3));
        data.removeChild(root, child2);
        assertEquals(3, data.getChildCount(root));
        assertEquals("whatever", data.getName(data.getChildID(root, 0)));
        assertEquals("testing1", data.getName(data.getChildID(root, 1)));
        assertEquals("testing3", data.getName(data.getChildID(root, 2)));
        assertEquals(child3, data.getChildID(root, 2));
        data.removeChild(-1, root);
        try {
            data.getRootID();
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
        assertBadTag(data, "&");
        assertBadTag(data, "a1&");
        assertBadTag(data, "a1.a");
        assertBadTag(data, "a;a");
        assertBadTag(data, ".");
        assertBadTag(data, ">sadfa");
        assertBadTag(data, "z[a");
        assertBadTag(data, "z]a");
        assertBadTag(data, "1");
        assertBadTag(data, "");
    }

    public void testFactorySingleton() throws FileNotFoundException, TreeDataException {
        ILoadableTreeData data1 = TreeDataFactory.getSimpleXMLTreeDataInstance();
        ILoadableTreeData data2 = TreeDataFactory.getSimpleXMLTreeDataInstance();
        assertEquals(data1, data2);
        TreeDataFactory.releaseSingleton();
        ILoadableTreeData data3 = TreeDataFactory.getSimpleXMLTreeDataInstance();
        assertFalse(data1 == data3);
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/emptyFilename.xml");
        data3.load(url);
        int root = data3.getRootID();
        assertEquals("startup", data3.getName(root));
        ITreeData data4 = TreeDataFactory.getSimpleXMLTreeDataInstance();
        assertEquals(data3, data4);
        try {
            data4.getName(root);
            fail("Should have thrown IndexOutOfBoundsException");
        } catch (IndexOutOfBoundsException e) {
        }
    }

    public void testFindChild() throws FileNotFoundException, TreeDataException {
        internalFindChildTest(TreeDataFactory.newSimpleXMLTreeData());
        internalFindChildTest(TreeDataFactory.newSAXTreeData());
    }

    private void internalFindChildTest(ILoadableTreeData data)
        throws FileNotFoundException, TreeDataException {
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/sampleXML.xml");
        data.load(url);
        int root = data.getRootID();
        int child4 = data.findChildID(root, "child4");
        assertEquals("child4", data.getName(child4));
        assertEquals("bi>lly", data.getAttribute(child4, "name"));
        int item = data.findChildID(child4, "item");
        assertEquals("item", data.getName(item));
        assertEquals("one", data.getValue(item));
        item = data.findChildID(child4, "item", item);
        assertEquals("item", data.getName(item));
        assertEquals("two", data.getValue(item));
        item = data.findChildID(child4, "item", item);
        assertEquals("item", data.getName(item));
        assertEquals("three", data.getValue(item));
        assertEquals(-1, data.findChildID(child4, "item", root));
        assertEquals(-1, data.findChildID(child4, "item", child4));
        item = data.findChildID(child4, "item", -1);
        assertEquals("item", data.getName(item));
        assertEquals("one", data.getValue(item));
    }

    public void testCDataRead() throws FileNotFoundException, TreeDataException {
        URL url =
            getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/utils/testdata/advancedXML.xml");
        ILoadableTreeData data = TreeDataFactory.newSAXTreeData();
        data.load(url);
        int root = data.getRootID();
        assertEquals("testing", data.getName(root));
        int child = data.findChildID(root, "item");
        assertEquals("item", data.getName(child));
        assertEquals("Item 1", data.getValue(child));
        child = data.findChildID(root, "item", child);
        assertEquals("item", data.getName(child));
        assertEquals("Item 2", data.getValue(child));
        child = data.findChildID(root, "item", child);
        assertEquals("item", data.getName(child));
        assertEquals("    this is a test    \nwhatever\nsmile", data.getValue(child));
    }

    public void testCopy() throws TreeDataException, IOException {
        ISavableTreeData data1 = TreeDataFactory.newSimpleXMLTreeData();
        data1.load(new StringReader(xmlData1));
        ILoadableTreeData data2 = TreeDataFactory.newSimpleXMLTreeData();
        data2.load(new StringReader(xmlData2));
        int root = data1.getRootID();
        int prereq = data1.getChildID(root, 0);
        int plugin = data1.getChildID(prereq, 2);
        data1.createChildCopy(data2, data2.getRootID(), plugin);
        StringWriter output = new StringWriter(xmlMerged.length());
        data1.save(output);
        String out = output.toString();
        assertEquals(out, xmlMerged, out);
    }

    public void testClone() throws TreeDataException, IOException {
        ISavableTreeData data1 = TreeDataFactory.newSimpleXMLTreeData();
        data1.load(new StringReader(xmlMerged));
        ISavableTreeData data2 = TreeDataFactory.newSimpleXMLTreeData();
        data2.ignoreWhitespace(true);
        data2.createChildCopy(data1, data1.getRootID(), -1);
        StringWriter output1 = new StringWriter(xmlData1.length());
        StringWriter output2 = new StringWriter(xmlData1.length());
        data1.save(output1);
        data2.save(output2);
        assertEquals(output1.toString(), output2.toString());
        assertEquals(xmlMerged, output2.toString());
    }

    public void testXMLSave() throws TreeDataException, IOException {
        TestFileBuilder.initRootTestDirs();
        try {

            File file = new File(TestFileBuilder.getDestFile(), "test.xml");
            ISavableTreeData data = TreeDataFactory.newSimpleXMLTreeData();
            int root = data.createRoot("testXML");
            int child1 = data.createChild(root, "one");
            data.createChild(root, "two");
            int childx = data.createChild(root, "x");
            int child3 = data.createChild(root, "three");
            int child4 = data.createChild(root, "four");
            int child5 = data.createChild(root, "five");
            data.setAttribute(child1, "attrib", " <&value&> ");
            data.setValue(child3, "Some Value < > & ' \" <><>");
            data.setAttribute(child4, "a", "1");
            data.setAttribute(child4, "b", "2");
            data.setAttribute(child4, "c", "7");
            data.setAttribute(child4, "c", "3");
            data.setAttribute(child4, "d", null);
            data.setAttribute(child5, "george", "harry");
            data.setValue(child4, "smile");
            int grandchild = data.createChild(child4, "whatever");
            data.setValue(grandchild, "hi there");
            grandchild = data.createChild(child5, "ralph");
            data.setValue(grandchild, " fantasy land ");
            data.setAttribute(grandchild, "country", "USA");
            data.removeChild(root, childx);
            data.save(file);
            BufferedReader reader = new BufferedReader(new FileReader(file));
            try {
                String line;
                int idx = 0;
                while ((line = reader.readLine()) != null) {
                    if (!line.equals(xml[idx])) {
                        System.out.println(line);
                        System.out.println(xml[idx]);
                    }
                    assertEquals(xml[idx], line);
                    idx++;
                }
                assertEquals(xml.length, idx);
            } finally {
                reader.close();
            }
        } finally {
            TestFileBuilder.eraseTestDirectories(true);
        }
    }

    //********************
    //*** end of tests ***
    //********************
    /**
     * Constructor for TreeDataTests.
     * @param arg0
     */
    public TreeDataTests(String arg0) {
        super(arg0);
    }

    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        TreeDataFactory.releaseSingleton();
    }
    private void assertBadTag(ITreeData data, String tag) {
        data.clear();
        try {
            data.createRoot(tag);
            fail("Should have thrown RuntimeException");
        } catch (RuntimeException e) {
            assertEquals(INVALID_TAG_NAME_ERROR, e.getMessage());
        }
        int root = data.createRoot("test");
        try {
            data.createChild(root, tag);
            fail("Should have thrown RuntimeException");
        } catch (RuntimeException e) {
            assertEquals(INVALID_TAG_NAME_ERROR, e.getMessage());
        }
        try {
            data.setAttribute(root, tag, null);
            fail("Should have thrown RuntimeException");
        } catch (RuntimeException e) {
            assertEquals(INVALID_TAG_NAME_ERROR, e.getMessage());
        }
    }

    private String xmlData1 =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<plugin id=\"gov.va.med.hds.cd.mum.testing.some.plugin\" name=\"Test Plugin\" version=\"1.2.3.four\" class=\"do.not.care\" provider-name=\"do not care\" module=\"one.two.three.MyModule\">"
            + WIN_EOL
            + "	<prerequisites>"
            + WIN_EOL
            + "		<requires plugin=\"ug1\" version=\"1\"/>"
            + WIN_EOL
            + "		<requires plugin=\"ug2\" version=\"2\"/>"
            + WIN_EOL
            + "		<requires plugin=\"ug3\" version=\"3\"/>"
            + WIN_EOL
            + "		<requires plugin=\"ug4\" version=\"4\"/>"
            + WIN_EOL
            + "		<requires plugin=\"ug5\" version=\"5\"/>"
            + WIN_EOL
            + "	</prerequisites>"
            + WIN_EOL
            + "</plugin>";

    private String xmlData2 =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<insert attrib=\"Testing\">"
            + WIN_EOL
            + "  <child>"
            + WIN_EOL
            + "		<grandchild plugin=\"ug6\" version=\"6\"/>"
            + WIN_EOL
            + "		<grandchild plugin=\"ug7\" version=\"7\"/>"
            + WIN_EOL
            + "		<grandchild plugin=\"ug8\" version=\"8\"/>"
            + WIN_EOL
            + "		<grandchild plugin=\"ug9\" version=\"9\">some test data</grandchild>"
            + WIN_EOL
            + "	</child>"
            + WIN_EOL
            + "</insert>";

    private String xmlMerged =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<plugin id=\"gov.va.med.hds.cd.mum.testing.some.plugin\" name=\"Test Plugin\" version=\"1.2.3.four\" class=\"do.not.care\" provider-name=\"do not care\" module=\"one.two.three.MyModule\">"
            + WIN_EOL
            + "  <prerequisites>"
            + WIN_EOL
            + "    <requires plugin=\"ug1\" version=\"1\"/>"
            + WIN_EOL
            + "    <requires plugin=\"ug2\" version=\"2\"/>"
            + WIN_EOL
            + "    <requires plugin=\"ug3\" version=\"3\">"
            + WIN_EOL
            + "      <insert attrib=\"Testing\">"
            + WIN_EOL
            + "        <child>"
            + WIN_EOL
            + "          <grandchild plugin=\"ug6\" version=\"6\"/>"
            + WIN_EOL
            + "          <grandchild plugin=\"ug7\" version=\"7\"/>"
            + WIN_EOL
            + "          <grandchild plugin=\"ug8\" version=\"8\"/>"
            + WIN_EOL
            + "          <grandchild plugin=\"ug9\" version=\"9\">some test data</grandchild>"
            + WIN_EOL
            + "        </child>"
            + WIN_EOL
            + "      </insert>"
            + WIN_EOL
            + "    </requires>"
            + WIN_EOL
            + "    <requires plugin=\"ug4\" version=\"4\"/>"
            + WIN_EOL
            + "    <requires plugin=\"ug5\" version=\"5\"/>"
            + WIN_EOL
            + "  </prerequisites>"
            + WIN_EOL
            + "</plugin>";
    private String[] xml =
        {
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
            "<testXML>",
            "  <one attrib=\" &lt;&amp;value&amp;&gt; \"/>",
            "  <two/>",
            "  <three>Some Value &lt; &gt; &amp; &apos; &quot; &lt;&gt;&lt;&gt;</three>",
            "  <four a=\"1\" b=\"2\" c=\"3\">smile<whatever>hi there</whatever>",
            "  </four>",
            "  <five george=\"harry\">",
            "    <ralph country=\"USA\"> fantasy land </ralph>",
            "  </five>",
            "</testXML>" };

    private static class TestTreeData extends TreeData {
        private static class TestIntListCache extends TreeData.IntListCache {

            public TestIntListCache() {
                super();
            }

            public TestIntListCache(int initialCapacity) {
                super(initialCapacity);
            }
        }
    }
}
