package gov.va.med.hds.cd.mum.boot.utils.test;

import gov.va.med.hds.cd.mum.boot.utils.Utils;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

public class UtilsTest extends TestCase {

	public void testHasArgument() {
		String[] args = { "-test1", "-TesT2", "-TEST3", "test4" };
		assertArgument(args, "-test1", 0);
		assertArgument(args, "-test2", 1);
		assertArgument(args, "-TEST1", 0);
		assertArgument(args, "-test3", 2);
		assertArgument(args, "-test5", -1);
		assertArgument(args, null, 3);
		assertArgument(args, "test4", 3);
		args = new String[] { "-test1", "-TEST3" };
		assertArgument(args, null, -1);
		assertArgument(args, "test4", -1);
		assertArgument(args, "-test2", -1);
		assertArgument(args, "-TEST1", 0);
		assertArgument(args, "-TEST3", 1);
		assertArgument(args, "-test3", 1);
		assertArgument(null, "-test1", -1);
	}

	public void testArgumentValues() {
		String[] args = { "-test1=ONE", "-test2=tWo", "-test3=three", "four" };
		assertArgumentValue(args, "-test1=", "ONE", 0);
		assertArgumentValue(args, "-TEST1=", "ONE", 0);
		assertArgumentValue(args, "-TEST7=", null, -1);
		assertArgumentValue(args, "-TEST2=", "tWo", 1);
		assertArgumentValue(args, "-TEST3=", "three", 2);
		assertArgumentValue(args, null, "four", 3);
		assertArgumentValue(null, "-test1", null, -1);
		assertArgumentValue(null, null, null, -1);
		args = new String[] { "-test5=five" };
		assertArgumentValue(args, "-TeSt5=", "five", 0);
		assertArgumentValue(args, null, null, -1);
	}

	public void testExtractArgument() {
		String[] args = "1,2,3,4,5".split(",");
		String[] result = Utils.extractArguemnt(args, 0);
		assertArrayEquals("2,3,4,5", result);
		result = Utils.extractArguemnt(result, 1);
		assertArrayEquals("2,4,5", result);
		result = Utils.extractArguemnt(args, 4);
		assertArrayEquals("1,2,3,4", result);
		result = Utils.extractArguemnt(args, 3);
		assertArrayEquals("1,2,3,5", result);
		try {
			result = Utils.extractArguemnt(args, -1);
			fail("Should have thrown ArrayIndexOutOfBoundsException");
		} catch (ArrayIndexOutOfBoundsException e) {
		}
		try {
			result = Utils.extractArguemnt(args, 5);
			fail("Should have thrown ArrayIndexOutOfBoundsException");
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	public void testInsertArgument() {
		String[] args = "1,2,3,4".split(",");
		String[] result = Utils.insertArgument(args, "x", 0);
		assertArrayEquals("x,1,2,3,4", result);
		result = Utils.insertArgument(args, "x", 1);
		assertArrayEquals("1,x,2,3,4", result);
		result = Utils.insertArgument(args, "x", 2);
		assertArrayEquals("1,2,x,3,4", result);
		result = Utils.insertArgument(args, "x", 4);
		assertArrayEquals("1,2,3,4,x", result);
		result = Utils.insertArgument(new String[0], "x", 0);
		assertArrayEquals("x", result);
		result = Utils.insertArgument((String[]) null, "x", 0);
		assertArrayEquals("x", result);
		try {
			result = Utils.insertArgument(args, "x", -1);
			fail("Should have thrown ArrayIndexOutOfBoundsException");
		} catch (ArrayIndexOutOfBoundsException e) {
		}
		try {
			result = Utils.insertArgument(args, "x", 5);
			fail("Should have thrown ArrayIndexOutOfBoundsException");
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	public void testInsertArgumentFromList() {
		List args = new ArrayList();
		args.add("1");
		args.add("2");
		args.add("3");
		args.add("4");
		String[] result = Utils.insertArgument(args, "x", 0);
		assertArrayEquals("x,1,2,3,4", result);
		result = Utils.insertArgument(args, "x", 1);
		assertArrayEquals("1,x,2,3,4", result);
		result = Utils.insertArgument(args, "x", 2);
		assertArrayEquals("1,2,x,3,4", result);
		result = Utils.insertArgument(args, "x", 4);
		assertArrayEquals("1,2,3,4,x", result);
		result = Utils.insertArgument(new String[0], "x", 0);
		assertArrayEquals("x", result);
		result = Utils.insertArgument((String[]) null, "x", 0);
		assertArrayEquals("x", result);
		try {
			result = Utils.insertArgument(args, "x", -1);
			fail("Should have thrown IndexOutOfBoundsException");
		} catch (IndexOutOfBoundsException e) {
		}
		try {
			result = Utils.insertArgument(args, "x", 5);
			fail("Should have thrown IndexOutOfBoundsException");
		} catch (IndexOutOfBoundsException e) {
		}
	}
	
	public void testGetArguementValue() {
		assertEquals("testing", Utils.getArgumentValue("-ug=testing", "-ug="));
		assertEquals("hello", Utils.getArgumentValue("-smile=hello", "-smile="));		
		assertEquals("smile", Utils.getArgumentValue("smile", null));
	}

	//********************
	//*** end of tests ***
	//********************

	/**
	 * Constructor for BootUtilsTest.
	 * @param arg0
	 */
	public UtilsTest(String arg0) {
		super(arg0);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	private void assertArgument(String[] args, String tag, int idx) {
		assertEquals(idx, Utils.getArgumentIndex(args, tag));
		if (idx < 0)
			assertFalse(Utils.hasArgument(args, tag));
		else
			assertTrue(Utils.hasArgument(args, tag));
	}

	private void assertArgumentValue(
		String[] args,
		String tag,
		String value,
		int idx) {
		assertEquals(idx, Utils.getArgumentIndex(args, tag));
		if (idx < 0)
			assertNull(Utils.getArgument(args, tag));
		else
			assertEquals(value, Utils.getArgument(args, tag));
	}

	private void assertArrayEquals(String string, String[] result) {
		String[] test = string.split(",");
		assertEquals(test.length, result.length);
		for (int i = 0; i < test.length; i++) {
			assertEquals(test[i], result[i]);
		}
	}

}
