/*
 * Created on May 15, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.config.ClassData;
import gov.va.med.hds.cd.mum.config.ClassDataCache;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;

import junit.framework.TestCase;

/**
 * @author vhaislmerrij
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ClassDataCacheTest extends TestCase implements IBootConstants {

    public void testClassData() {
        ClassData data = new ClassData();
        assertEmpty(data);

        setValues(data);
        assertValues(data);

        ClassData data1 = new ClassData();
        data1.copy(data);
        ClassData data2 = data.createClone();

        data.clear();
        assertEmpty(data);

        assertFalse(data == data1);
        assertFalse(data == data2);
        assertFalse(data1 == data2);
        assertValues(data1);
        assertValues(data2);
    }

    public void testClassDataToAndFromXML() throws TreeDataException, IOException {
        ClassData data = new ClassData();
        setValues(data);
        ISavableTreeData xml = TreeDataFactory.newSimpleXMLTreeData();
        int root = xml.createRoot("testXML");
        int node = data.save(xml, root);
        StringWriter writer = new StringWriter();
        xml.save(writer);
        assertEquals(valueXML, writer.toString());
        data.clear();
        assertEmpty(data);
        data.load(xml, node);
        assertValues(data);
    }

    public void testCache() throws IOException {
        ClassDataCache cache = new ClassDataCache(null);
        ClassData data1 = new ClassData();
        ClassData data2 = new ClassData();

        assertEmpty(data1);
        assertTrue(cache.needsClassData("one", 123, 456, data1));
        assertEquals("one", data1.getID());
        assertEquals(123, data1.getSize());
        assertEquals(456, data1.getLastModified());

        data1.setStartupClass("testing");
        data1.setApplicationClass("some.other.application.class");
        data1.setMultipleStartupClasses();
        data1.getClasses().add("something");
        cache.cacheClassData(data1);
        data1.clear();
        assertEmpty(data2);
        assertFalse(cache.needsClassData("one", 123, 456, data2));
        assertEquals("testing", data2.getStartupClass());
        assertEquals("some.other.application.class", data2.getApplicationClass());
        assertTrue(data2.hasMultipleStartupClasses());
        assertEquals("one", data2.getID());
        assertEquals(123, data2.getSize());
        assertEquals(456, data2.getLastModified());
        assertEquals(1, data2.getClasses().size());
        assertEquals("something", data2.getClasses().get(0));

        assertTrue(cache.needsClassData("two", 123, 456, data2));
        data2.setID(null);
        data2.setSize(0);
        data2.setLastModified(0);
        assertEmpty(data2);
        assertFalse(cache.needsClassData("one", 123, 456, data1));
        assertTrue(cache.needsClassData("one", 789, 456, data2));
        assertTrue(cache.needsClassData("one", 123, 567, data2));

        assertTrue(cache.needsClassData(null, 123, 456, data1));
        cache.cacheClassData(data1);
        assertTrue(cache.needsClassData(null, 123, 456, data1));
    }

    public void testCacheToAndFromXML()
        throws IOException, FileNotFoundException, TreeDataException {
        ClassData data = new ClassData();
        ClassDataCache cache = new ClassDataCache(null);

        cache.loadCache(cacheXML);
        assertFalse(cache.needsClassData("my.id", 123456789, 987654321, data));
        assertValues(data);
        assertFalse(cache.needsClassData("aaa.bbb.ccc", 12345, 54321, data));
        assertEquals("app.cls", data.getApplicationClass());
        data.setID(null);
        data.setSize(0);
        data.setLastModified(0);
        data.setApplicationClass(null);
        assertEmpty(data);
    }

    //********************
    //*** end of tests ***
    //********************

    public ClassDataCacheTest(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    private void assertEmpty(ClassData data) {
        assertEquals(0, data.getClasses().size());
        assertEquals(0, data.getPackages().size());
        assertEquals(0, data.getDependencies().size());
        assertNull(data.getApplicationClass());
        assertNull(data.getStartupClass());
        assertFalse(data.hasMultipleApplicationClasses());
        assertFalse(data.hasMultipleStartupClasses());
        assertNull(data.getID());
        assertEquals(0, data.getSize());
        assertEquals(0, data.getLastModified());
    }

    private void setValues(ClassData data) {
        data.setMultipleApplicationClasses();
        data.setMultipleStartupClasses();
        data.setStartupClass("some.startup.Class");
        data.setApplicationClass("some.application.Class");
        data.getClasses().add("some.class");
        data.getClasses().add("some.other.class");
        data.getPackages().add("some.package");
        data.getPackages().add("some.other.package");
        data.getDependencies().add("some.dependency");
        data.getDependencies().add("some.other.dependency");
        data.setID("my.id");
        data.setSize(123456789);
        data.setLastModified(987654321);
    }

    private void assertValues(ClassData data) {
        assertEquals(2, data.getClasses().size());
        assertEquals("some.class", data.getClasses().get(0));
        assertEquals("some.other.class", data.getClasses().get(1));
        assertEquals(2, data.getPackages().size());
        assertEquals("some.package", data.getPackages().get(0));
        assertEquals("some.other.package", data.getPackages().get(1));
        assertEquals(2, data.getDependencies().size());
        assertEquals("some.dependency", data.getDependencies().get(0));
        assertEquals("some.other.dependency", data.getDependencies().get(1));
        assertEquals("some.application.Class", data.getApplicationClass());
        assertEquals("some.startup.Class", data.getStartupClass());
        assertTrue(data.hasMultipleApplicationClasses());
        assertTrue(data.hasMultipleStartupClasses());
        assertEquals("my.id", data.getID());
        assertEquals(123456789, data.getSize());
        assertEquals(987654321, data.getLastModified());
    }

    private static final String valueXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<testXML>"
            + WIN_EOL
            + "  <data id=\"my.id\" size=\"123456789\" date=\"987654321\" startupClass=\"some.startup.Class\" appClass=\"some.application.Class\" multipleStartups=\"true\" multipleApps=\"true\">"
            + WIN_EOL
            + "    <classes>*some.class*some.other.class*</classes>"
            + WIN_EOL
            + "    <packages>*some.package*some.other.package*</packages>"
            + WIN_EOL
            + "    <requires>*some.dependency*some.other.dependency*</requires>"
            + WIN_EOL
            + "  </data>"
            + WIN_EOL
            + "</testXML>";

    private static final String cacheXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<cache>"
            + WIN_EOL
            + "  <data id=\"my.id\" size=\"123456789\" date=\"987654321\" startupClass=\"some.startup.Class\" appClass=\"some.application.Class\" multipleStartups=\"true\" multipleApps=\"true\">"
            + WIN_EOL
            + "    <classes>*some.class*some.other.class*</classes>"
            + WIN_EOL
            + "    <packages>*some.package*some.other.package*</packages>"
            + WIN_EOL
            + "    <requires>*some.dependency*some.other.dependency*</requires>"
            + WIN_EOL
            + "  </data>"
            + WIN_EOL
            + "  <data id=\"aaa.bbb.ccc\" size=\"12345\" date=\"54321\" appClass=\"app.cls\"/>"
            + WIN_EOL
            + "</cache>";
}
