package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.IApplication;
import gov.va.med.hds.cd.mum.IPluginSource;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.ISplashScreen;
import gov.va.med.hds.cd.mum.boot.IStartup;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.utils.IIgnoreStartup;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.config.ClassFileAnalyzer;
import gov.va.med.hds.cd.mum.config.InvalidFileStreamException;
import gov.va.med.hds.cd.mum.utils.IIgnoreApplication;
import gov.va.med.hds.util.io.StreamUtilities;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;

import junit.framework.TestCase;

public class ClassFileAnalyzerTest
    extends TestCase
    implements IStartup, IIgnoreStartup, IApplication, IIgnoreApplication, IBootConstants {

    public void testClassName() throws InvalidFileStreamException {
        assertEquals("", analyzer.getClassName());
        doThisClass();
        assertEquals(
            "gov.va.med.hds.cd.mum.config.test.ClassFileAnalyzerTest",
            analyzer.getClassName());

        doSuperslass();
        assertEquals("junit.framework.TestCase", analyzer.getClassName());

        doClass(testClass);
        assertEquals("mumtest.configdata.XTestClass", analyzer.getClassName());

        doClass(startupClass);
        assertEquals("mumtest.configdata.XStartupClass", analyzer.getClassName());

        doClass(startupInterface);
        assertEquals("mumtest.configdata.XIStartupDescendant", analyzer.getClassName());

        doClass(applicationInterface);
        assertEquals("mumtest.configdata.XIApplicationDescendant", analyzer.getClassName());

        doClass(abstractStartup);
        assertEquals("mumtest.configdata.XAbstractStartup", analyzer.getClassName());

        doClass(abstractApplication);
        assertEquals("mumtest.configdata.XAbstractApplication", analyzer.getClassName());
    }

    public void testDependencies() throws InvalidFileStreamException {
        assertDependencyCount(0);

        doSuperslass();
        assertDependencyCount(4);
        assertDependency("junit.framework.Assert");
        assertDependency("junit.framework.Test");
        assertDependency("junit.framework.TestCase");
        assertDependency("junit.framework.TestResult");

        doClass(testClass);
        assertDependencyCount(2);
        assertDependency("gov.va.med.hds.cd.mum.IApplication");
        assertDependency("mumtest.configdata.XTestClass");

        doClass(startupClass);
        assertDependencyCount(1);
        assertDependency("mumtest.configdata.XStartupClass");

        doClass(startupInterface);
        assertDependencyCount(1);
        assertDependency("mumtest.configdata.XIStartupDescendant");

        doClass(applicationInterface);
        assertDependencyCount(2);
        assertDependency("mumtest.configdata.XIApplicationDescendant");
        assertDependency("gov.va.med.hds.cd.mum.IApplication");

        doClass(abstractStartup);
        assertDependencyCount(1);
        assertDependency("mumtest.configdata.XAbstractStartup");

        doClass(abstractApplication);
        assertDependencyCount(2);
        assertDependency("mumtest.configdata.XAbstractApplication");
        assertDependency("gov.va.med.hds.cd.mum.IApplication");

    }

    public void testIsStartup() throws InvalidFileStreamException {
        assertFalse(analyzer.isStartup());
        doThisClass();

        ClassFileAnalyzer.SetUseIgnoreInterfaces(false);
        try {
            assertTrue(analyzer.isStartup());
        } finally {
            ClassFileAnalyzer.SetUseIgnoreInterfaces(true);
        }
        assertFalse(analyzer.isStartup());

        doSuperslass();
        assertFalse(analyzer.isStartup());

        doClass(testClass);
        assertFalse(analyzer.isStartup());

        doClass(startupClass);
        assertTrue(analyzer.isStartup());

        doClass(startupInterface);
        assertFalse(analyzer.isStartup());

        doClass(applicationInterface);
        assertFalse(analyzer.isStartup());

        doClass(abstractStartup);
        assertFalse(analyzer.isStartup());

        doClass(abstractApplication);
        assertFalse(analyzer.isStartup());
    }

    public void testIsApplication() throws InvalidFileStreamException {
        assertFalse(analyzer.isApplication());
        doThisClass();

        ClassFileAnalyzer.SetUseIgnoreInterfaces(false);
        try {
            assertTrue(analyzer.isApplication());
        } finally {
            ClassFileAnalyzer.SetUseIgnoreInterfaces(true);
        }
        assertFalse(analyzer.isApplication());

        doSuperslass();
        assertFalse(analyzer.isApplication());

        doClass(testClass);
        assertTrue(analyzer.isApplication());

        doClass(startupClass);
        assertFalse(analyzer.isApplication());

        doClass(startupInterface);
        assertFalse(analyzer.isApplication());

        doClass(applicationInterface);
        assertFalse(analyzer.isApplication());

        doClass(abstractStartup);
        assertFalse(analyzer.isApplication());

        doClass(abstractApplication);
        assertFalse(analyzer.isApplication());
    }

    //********************
    //*** end of tests ***
    //********************

    public ClassFileAnalyzerTest(String arg0) {
        super(arg0);
    }

    ClassFileAnalyzer analyzer;

    protected void setUp() throws Exception {
        super.setUp();
        analyzer = new ClassFileAnalyzer();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    private void doClass(String cls) throws InvalidFileStreamException {
        doClass(cls, false);
    }

    private void doClass(String res, boolean bump) throws InvalidFileStreamException {
        URL url = this.getClass().getClassLoader().getResource(res);
        if (url == null)
            fail("should have found class resource " + res);
        try {
            ByteArrayInputStream stream =
                new ByteArrayInputStream(
                    StreamUtilities.convertStreamToByteArray(url.openStream()));
            if (bump)
                stream.skip(1);
            analyzer.analyzeClass(stream);
        } catch (IOException e) {
            fail("error reading stream");
        }
    }

    private void doClass(Class cls) throws InvalidFileStreamException {
        doClass(cls, false);
    }

    private void doClass(Class cls, boolean bump) throws InvalidFileStreamException {
        doClass(cls.getName().replace('.', '/') + ".class", bump);
    }

    private String testClass = "gov/va/med/hds/cd/mum/config/testdata/XTestClass.cls";

    private String startupClass = "gov/va/med/hds/cd/mum/config/testdata/XStartupClass.cls";

    private String startupInterface =
        "gov/va/med/hds/cd/mum/config/testdata/XIStartupDescendant.cls";

    private String applicationInterface =
        "gov/va/med/hds/cd/mum/config/testdata/XIApplicationDescendant.cls";

    private String abstractStartup = "gov/va/med/hds/cd/mum/config/testdata/XAbstractStartup.cls";

    private String abstractApplication =
        "gov/va/med/hds/cd/mum/config/testdata/XAbstractApplication.cls";

    private void doThisClass() throws InvalidFileStreamException {
        doClass(this.getClass());
    }

    private void doSuperslass() throws InvalidFileStreamException {
        doClass(this.getClass().getSuperclass());
    }

    private void outputDependencies(StringBuffer buf) {
        ArrayList list = new ArrayList();
        for (int i = 0; i < analyzer.getDependencies().size(); i++) {
            list.add(analyzer.getDependencies().get(i));
        }
        Collections.sort(list);
        for (int i = 0; i < list.size(); i++) {
            buf.append((String) list.get(i));
            buf.append(WIN_EOL);
        }
    }

    private void assertDependency(String pkg) {
        if (!analyzer.getDependencies().contains(pkg)) {
            ensureBuffer();
            buf.append("Should have had dependency ");
            buf.append(pkg);
            buf.append(WIN_EOL);
            outputDependencies(buf);
            fail(buf.toString());
        }
    }

    private void assertDependencyCount(int count) {
        if (analyzer.getDependencies().size() != count) {
            ensureBuffer();
            buf.append("Expected ");
            buf.append(Integer.toString(count));
            buf.append(" dependencies, found ");
            buf.append(Integer.toString(analyzer.getDependencies().size()));
            buf.append(WIN_EOL);
            outputDependencies(buf);
            fail(buf.toString());
        }
    }

    public void run(
        String[] args,
        ITreeData configXML,
        IPluginSource[] plugins,
        ISplashScreen splash) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartup#run(gov.va.med.hds.cd.mum.boot.IStartupInfo)
     */
    public void run(IStartupInfo info) {
    }
    private void ensureBuffer() {
        if (buf == null)
            buf = new StringBuffer();
        else
            buf.setLength(0);
    }
    private StringBuffer buf;
}
