package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.IApplication;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartup;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.config.CommandLineUtility;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.config.IConfigPluginLocator;
import gov.va.med.hds.cd.mum.config.IPluginManager;
import gov.va.med.hds.cd.mum.config.PluginDependency;
import gov.va.med.hds.cd.mum.config.PluginLocatorException;
import gov.va.med.hds.cd.mum.config.PluginManagerFactory;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;
import java.net.URL;
import java.util.List;

import junit.framework.TestCase;

public class CommandLineUtilityTest
    extends TestCase
    implements IConfigConstants, IPluginManager, IBootConstants, IMUMTestConstants {

    public void testHelp() {
        CommandLineUtility.main(new String[] { CommandLineUtility.HELP_ARG });
        assertEquals(CommandLineUtility.HELP_TEXT, out.toString());
    }

    public void testDefaultPath() {
        PluginManagerFactory.assignPluginManager(this);
        CommandLineUtility.main(null);
        assertEquals(
            System.getProperty(CommandLineUtility.USER_CURRENT_DIR_PROPERTY),
            goldDir.getPath());
        assertEquals(errorMessage, out.toString());
    }

    public void testPassedInPath() {
        PluginManagerFactory.assignPluginManager(this);
        CommandLineUtility.main(new String[] { CommandLineUtility.PATH_ARG + "this is a test" });
        assertEquals("this is a test", goldDir.getPath());
        assertEquals(errorMessage, out.toString());
    }

    public void testBadPath() {
        CommandLineUtility.main(new String[] { getBadPathArg()});
        assertTrue(out.toString().indexOf(PLUGIN_DIRECTORY_NOT_FOUND) >= 0);
    }

    //********************
    //*** end of tests ***
    //********************

    public CommandLineUtilityTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        out = new ByteArrayOutputStream();
        old = System.out;
        System.setOut(new PrintStream(out));
        goldDir = null;
        emptyList = new ListMap();
        PluginManagerFactory.assignPluginManager(null);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        System.setOut(old);
        PluginManagerFactory.assignPluginManager(null);
    }

    private ByteArrayOutputStream out;
    private PrintStream old;

    private String rootDir = "gov/va/med/hds/cd/mum/config/cmdutil/testdata/";
    private String badDir = "baddir";

    private String getBadPathArg() {
        return CommandLineUtility.PATH_ARG + getURL(badDir).getPath();
    }

    private URL getURL(String path) {
        return this.getClass().getClassLoader().getResource(rootDir + path);
    }

    private String errorMessage =
        "Error: MissingStartupPlugin"
            + EOL
            + "Cause: Configuration is missing a startup plugin."
            + EOL
            + "  No plugin implements the "
            + IStartup.class.getName()
            + " interface"
            + EOL
            + EOL
            + "Error: MissingApplicationPlugin"
            + EOL
            + "Cause: Configuration is missing an application plugin."
            + EOL
            + "  No plugin implements the "
            + IApplication.class.getName()
            + " interface"
            + EOL
            + EOL;

    // plugin manager calls
    private File goldDir;
    private ListMap emptyList;

    public void assignGoldDirectory(File goldDir) {
        this.goldDir = goldDir;
    }
    public void load() throws PluginLocatorException {
    }
    public void reset() {
    }
    public ListMap getPlugins() {
        return emptyList;
    }
    public boolean isLoaded() {
        return false;
    }
    public void loadPluginInformation() {
    }
    public void assignPluginLocator(IConfigPluginLocator locator) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.config.IPluginManager#getClasses()
     */
    public SimpleList getClasses() {
        return null;
    }
    /**
     * @see gov.va.med.hds.cd.mum.config.IPluginManager#getPluginIDs()
     */
    public List getPluginIDs() {
        return null;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.config.IPluginManager#supportsPlugin(gov.va.med.hds.cd.mum.PluginDependency)
     */
    public boolean satisfiesDependency(PluginDependency dependency) {
        return false;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.config.IPluginManager#getClassPlugins(java.util.List, java.lang.String)
     */
    public void getClassPlugins(List list, String cls) {

    }

    public List getExtensionPointPlugins(String extensionPoint) {
        return null;
    }
}
