package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.config.CommandLineUtility;
import gov.va.med.hds.cd.mum.config.ConfigPluginLocatorFactory;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.utils.ChecksumCalculator;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;

import junit.framework.TestCase;

public class CommandLineUtilityTest2 extends TestCase implements IMUMTestConstants {

    public void testPassthroughNoSettings() throws IOException {
        testCount++;
        assertFalse(configFile.exists());
        CommandLineUtility.main(new String[] { configFile.getPath()});
        assertEquals("", out.toString());
        assertTrue(configFile.exists());
        String[] xml = new String[rawLen];
        xml[0] = rawXML1;
        xml[1] = noSettingsCRC;
        int idx = addXML(xml, startupXML, 2);
        addXML(xml, rawXML, idx);
        assertXML(xml);
    }

    public void testPassthroughWithSettings() throws IOException {
        testCount++;
        assertFalse(configFile.exists());
        CommandLineUtility.main(
            new String[] {
                configFile.getPath(),
                CommandLineUtility.SETTINGS_ARG + getURL(settings).toString()});
        assertEquals("", out.toString());
        assertTrue(configFile.exists());
        String[] xml = new String[rawLen + settingsXML.length];
        xml[0] = rawXML1;
        xml[1] = settingsCRC;
        int idx = addXML(xml, startupXML, 2);
        idx = addXML(xml, settingsXML, idx);
        addXML(xml, rawXML, idx);
        assertXML(xml);
    }

    public void testError() {
        testCount++;
        locator.addPlugin(getURL(badManifest));
        CommandLineUtility.main(new String[] { configFile.getPath()});
        assertFalse(configFile.exists());
        String result = out.toString();
        if ((!result.equals(errorMessage1a)) && (!result.equals(errorMessage1b)))
            assertEquals(errorMessage1a, result);
    }

    public void testErrorWithIgnore() {
        testCount++;
        locator.addPlugin(getURL(badManifest2));
        CommandLineUtility.main(new String[] { configFile.getPath()});
        assertFalse(configFile.exists());
        assertEquals(errorMessage2, out.toString());
    }

    //********************
    //*** end of tests ***
    //********************
    private ByteArrayOutputStream out;
    private PrintStream old;
    private File configFile;
    private RedirectedPluginLocator locator;

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(CommandLineUtilityTest2.class);

    public CommandLineUtilityTest2(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.initRootTestDirs();
        }
        out = new ByteArrayOutputStream();
        old = System.out;
        System.setOut(new PrintStream(out));
        locator = new RedirectedPluginLocator();
        locator.addPlugin(getURL(fredManifest));
        locator.addPlugin(getURL(georgeManifest));
        locator.addPlugin(getURL(harryManifest));
        ConfigPluginLocatorFactory.assignConfigPluginLocator(locator);
        configFile =
            new File(TestFileBuilder.getGoldFile().getPath() + File.separator + "testConfig.xml");
        if (configFile.exists())
            configFile.delete();
        MockJVM.Init();
        ChecksumCalculator.setJarTestMode(true);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
        System.setOut(old);
        MockJVM.ShutDown();
        ChecksumCalculator.setJarTestMode(false);
        ConfigPluginLocatorFactory.assignConfigPluginLocator(null);
    }

    private String rootDir = "gov/va/med/hds/cd/mum/config/cmdutil/testdata/";
    private String pluginDir = "plugins/";
    private String settings = "settings.xml";
    private String badManifest = pluginDir + "bad/" + MANIFEST_FILE;
    private String badManifest2 = pluginDir + "bad/" + "manifest2.xml";
    private String fredManifest = pluginDir + "fred/" + MANIFEST_FILE;
    private String georgeManifest = pluginDir + "george/" + MANIFEST_FILE;
    private String harryManifest = pluginDir + "harry/" + MANIFEST_FILE;
//    private String tempSettings = "tempSettings.xml";
//
//    private String getPathArg(String path) {
//        return CommandLineUtility.PATH_ARG + getURL(path).getPath();
//    }

    private URL getURL(String path) {
        return this.getClass().getClassLoader().getResource(rootDir + path);
    }

    private String rawXML1 = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";

    private String noSettingsCRC = "<configuration crc=\"1579692586\">";

    private String settingsCRC = "<configuration crc=\"3663834335\">";

    private String error1 =
        "Error: UnavailableDependentPlugin"
            + EOL
            + "Cause: No plugin satisfies required plugin dependency."
            + EOL
            + "  Plugin dependency: mumtest.cmdutil.fred v5.0.0"
            + EOL
            + "  Plugin requiring dependency:"
            + EOL
            + "    mumtest.cmdutil.bad v1.0.0"
            + EOL
            + EOL;

    private String error2 =
        "Error: UnavailableDependentClass"
            + EOL
            + "Cause: No plugin implements required class dependency."
            + EOL
            + "  Class dependency: mumtest.plugin.one.TestClassFour"
            + EOL
            + "  Plugin requiring dependency:"
            + EOL
            + "    mumtest.cmdutil.bad v1.0.0"
            + EOL
            + EOL;

    private String error3 =
        "Error: UnpublishedDependentClass"
            + EOL
            + "Cause: Required class available but not published by the providing plugin."
            + EOL
            + "  Class: mumtest.cmdutil.george.SomeOtherClass"
            + EOL
            + "  Plugin requiring dependency:"
            + EOL
            + "    mumtest.cmdutil.bad v1.0.0"
            + EOL
            + "  Plugin satisfying dependency if class were published:"
            + EOL
            + "    mumtest.cmdutil.george v1.0.0"
            + EOL
            + EOL;

    private String errorMessage1a = error1 + error2 + error3;
    private String errorMessage1b = error1 + error3 + error2;
    private String errorMessage2 = error1 + error3;

    private String[] rawXML =
        {
            "  <plugin directory=\"mumtest.cmdutil.fred_1.0.0\" source=\"fred\">",
            "    <file filename=\"manifest.xml\" crc=\"3375330463\" size=\"367\"/>",
            "    <file filename=\"boot.jar\" crc=\"0\" size=\"0\"/>",
            "    <file filename=\"mum.jar\" crc=\"4253358179\" size=\"1285\">",
            "      <package>mumtest.cmdutil.fred</package>",
            "    </file>",
            "    <requires plugin=\"mumtest.cmdutil.george_1.0.0\"/>",
            "    <publish class=\"mumtest.cmdutil.fred.Child\"/>",
            "  </plugin>",
            "  <plugin directory=\"mumtest.cmdutil.george_1.0.0\" source=\"george\">",
            "    <file filename=\"manifest.xml\" crc=\"2613926290\" size=\"283\"/>",
            "    <file filename=\"george1.jar\" crc=\"759828660\" size=\"774\">",
            "      <package>mumtest.cmdutil.george</package>",
            "    </file>",
            "    <file filename=\"george2.jar\" crc=\"3172995416\" size=\"797\">",
            "      <package>mumtest.cmdutil.george</package>",
            "    </file>",
            "    <publish class=\"mumtest.cmdutil.george.Parent\"/>",
            "  </plugin>",
            "  <plugin directory=\"mumtest.cmdutil.harry_1.0.0\" source=\"harry\">",
            "    <application class=\"mumtest.cmdutil.harry.App\"/>",
            "    <file filename=\"manifest.xml\" crc=\"2943570524\" size=\"259\"/>",
            "    <file filename=\"harry.jar\" crc=\"1267477729\" size=\"1774\">",
            "      <package>mumtest.cmdutil.harry</package>",
            "      <package>gov.va.med.hds.cd.mum</package>",
            "    </file>",
            "    <requires plugin=\"mumtest.cmdutil.fred_1.0.0\"/>",
            "  </plugin>",
            "</configuration>" };

    private String[] settingsXML =
        {
            "  <settings>",
            "    <connection port=\"1234\" server=\"12.34.56.78\"/>",
            "    <something/>",
            "    <more stuff=\"data\">",
            "      <children/>",
            "      <test>This is a Test</test>",
            "    </more>",
            "    <mumpsConnection>",
            "      <server>isl-reengkm</server>",
            "      <port>9000</port>",
            "    </mumpsConnection>",
            "  </settings>" };

    private String[] startupXML =
        {
            "  <startup plugin=\"mumtest.cmdutil.fred_1.0.0\" run=\"boot.jar\" source=\"fred\">",
            "    <files>",
            "      <file>manifest.xml</file>",
            "      <file>boot.jar</file>",
            "      <file>mum.jar</file>",
            "    </files>",
            "  </startup>" };

    private int rawLen = rawXML.length + startupXML.length + 2;

    private int addXML(String[] xml, String[] addition, int idx) {
        for (int i = 0; i < addition.length; i++) {
            xml[i + idx] = addition[i];
        }
        return addition.length + idx;
    }

    private void assertXML(String[] xml) throws FileNotFoundException, IOException {
        ArrayList list = new ArrayList();
        BufferedReader reader = new BufferedReader(new FileReader(configFile));
        try {
            String line;
            while ((line = reader.readLine()) != null) {
                list.add(line);
            }
        } finally {
            reader.close();
        }
        if (list.size() != xml.length)
            xmlFailure(list, xml, "XML Length does not match");
        for (int i = 0; i < list.size(); i++) {
            String line = (String) list.get(i);
            if (!xml[i].equals(line)) {
                xmlFailure(
                    list,
                    xml,
                    "Should have beed:" + WIN_EOL + xml[i] + WIN_EOL + "but was" + WIN_EOL + line);
            }
        }
    }

    private void xmlFailure(ArrayList list, String[] xml, String message) {
        StringBuffer buf = new StringBuffer(message);
        buf.append(WIN_EOL);
        buf.append("Should have been:");
        buf.append(WIN_EOL);
        for (int i = 0; i < xml.length; i++) {
            buf.append(xml[i]);
            buf.append(WIN_EOL);
        }
        buf.append(WIN_EOL);
        buf.append("but was:");
        buf.append(WIN_EOL);
        for (int i = 0; i < list.size(); i++) {
            buf.append(list.get(i));
            buf.append(WIN_EOL);
        }
        fail(buf.toString());
    }
}