package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.IApplication;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.PluginID;
import gov.va.med.hds.cd.mum.VersionFormatException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.config.ConfigFileEntry;
import gov.va.med.hds.cd.mum.config.ConfigPlugin;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.config.IConfigPlugin;
import gov.va.med.hds.cd.mum.config.PluginDependency;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.utils.IIgnoreApplication;

import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;

import junit.framework.TestCase;

public class ConfigPluginTest extends TestCase implements IConfigConstants, IBootConstants {

    public void testID() throws InvalidPluginException {
        nextPlugin();
        assertEquals("gov.va.med.hds.cd.mum.testing.some.plugin", plugin.getID());
        nextPlugin();
        assertEquals("fred", plugin.getID());
        nextPlugin();
        assertEquals("test.one", plugin.getID());
        nextPlugin();
        assertEquals("test.two", plugin.getID());
        nextPlugin();
        assertEquals("test.three", plugin.getID());
        nextPlugin();
        assertEquals("test.four", plugin.getID());
    }

    public void testVersion() throws InvalidPluginException {
        nextPlugin();
        assertEquals("1.2.3.four", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("12.1.0", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("1.0.0", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("1.1.0", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("2.0.0", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("1.2.0", plugin.getVersion().toString());
    }

    public void testDirectory() throws InvalidPluginException {
        nextPlugin();
        assertEquals("gov.va.med.hds.cd.mum.testing.some.plugin_1.2.3.four", plugin.getDirectory());
        nextPlugin();
        assertEquals("fred_12.1.0", plugin.getDirectory());
        nextPlugin();
        assertEquals("test.one_1.0.0", plugin.getDirectory());
        nextPlugin();
        assertEquals("test.two_1.1.0", plugin.getDirectory());
        nextPlugin();
        assertEquals("test.three_2.0.0", plugin.getDirectory());
        nextPlugin();
        assertEquals("test.four_1.2.0", plugin.getDirectory());
    }

    public void testSourceDirectory() throws InvalidPluginException {
        nextPlugin();
        assertEquals("manifests", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("manifests", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("one", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("two", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("three", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("four", plugin.getSourceDirectory());
    }

    public void testModule() throws InvalidPluginException {
        nextPlugin();
        assertEquals("one.two.three.MyModule", plugin.getModule());
        nextPlugin();
        assertEquals("fred", plugin.getModule());
        nextPlugin();
        assertEquals("testModule", plugin.getModule());
        nextPlugin();
        assertEquals("testModule", plugin.getModule());
        nextPlugin();
        assertEquals("someOtherModule", plugin.getModule());
        nextPlugin();
        assertEquals("test.four", plugin.getModule());
    }

    public void testDistributionFiles() throws InvalidPluginException {
        assertManifestFileAfterLoad();
        assertEquals(1, plugin.getFileCount());

        assertManifestFileAfterLoad();
        assertEquals(1, plugin.getFileCount());

        String root = assertManifestFileAfterLoad();
        assertEquals(2, plugin.getFileCount());
        entry = getFileEntry(root + "one.jar");
        assertEquals(3, entry.getPackages().size());
        assertTrue(entry.getPackages().contains("mumtest.plugin.one"));
        assertTrue(entry.getPackages().contains("mumtest.plugin.one.published"));
        assertTrue(entry.getPackages().contains("mumtest.plugin.one.resources"));

        root = assertManifestFileAfterLoad();
        assertEquals(3, plugin.getFileCount());
        entry = getFileEntry(root + "two7.jar");
        assertEquals(1, entry.getPackages().size());
        assertTrue(entry.getPackages().contains("mumtest.plugin.two"));
        entry = getFileEntry(root + "two8.jar");
        assertEquals(1, entry.getPackages().size());
        assertTrue(entry.getPackages().contains("mumtest.plugin.two.whatever"));

        root = assertManifestFileAfterLoad();
        assertEquals(2, plugin.getFileCount());
        entry = getFileEntry(root + "three.jar");
        assertEquals(2, entry.getPackages().size());
        assertTrue(entry.getPackages().contains("mumtest.plugin.three"));
        assertTrue(entry.getPackages().contains(DEFAULT_PACKAGE));

        assertManifestFileAfterLoad();
        assertEquals(1, plugin.getFileCount());
    }

    public void testClasses() throws InvalidPluginException {
        nextPlugin();
        assertEquals(0, plugin.getClasses().size());

        nextPlugin();
        assertEquals(0, plugin.getClasses().size());

        nextPlugin();
        assertEquals(11, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.published.PublicClassTwo"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestIgnoreStartupClass"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestIgnoreApplicationClass"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestStartupClass"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestClassFour"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestApplicationClass"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.published.PublicInterfaceOne"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestClassTwo"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.published.PublicClassOne"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestClassThree"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.one.TestClassOne"));

        nextPlugin();
        assertEquals(2, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("mumtest.plugin.two.TestClassSeven"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.two.whatever.IAppDescendant"));

        nextPlugin();
        assertEquals(7, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("DefaultPkgCls"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.TestInterfaceTwo"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.TestClassSix"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.Application"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.TestInterfaceOne"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.IStartupDescendant"));
        assertTrue(plugin.getClasses().contains("mumtest.plugin.three.TestClassFive"));

        nextPlugin();
        assertEquals(0, plugin.getClasses().size());
    }

    public void testPlugins() throws InvalidPluginException {
        assertPluginID("gov.va.med.hds.cd.mum.testing.some.plugin v1.2.3.four");
        assertPluginID("fred v12.1.0");
        assertPluginID("test.one v1.0.0");
        assertPluginID("test.two v1.1.0");
        assertPluginID("test.three v2.0.0");
        assertPluginID("test.four v1.2.0");
    }

    private boolean[] isStartupValues = { false, false, true, true, false, false };
    private boolean[] isApplicationValues = { false, false, true, false, true, false };

    public void testIsStartupOrApplication() throws InvalidPluginException {
        while (manifestsAvailable()) {
            nextPlugin();
            assertEquals(
                "Invalid startup value for " + manifests[manifestIndex],
                isStartupValues[manifestIndex],
                (plugin.getStartupClassName() != null));
            assertEquals(
                "Invalid application value for " + manifests[manifestIndex],
                isApplicationValues[manifestIndex],
                (plugin.getApplicationClassName() != null));
        }
    }

    public void testDependentClasses() throws InvalidPluginException {
        nextPlugin();
        assertEquals(0, plugin.getClassDependencies().size());

        nextPlugin();
        assertEquals(0, plugin.getClassDependencies().size());

        nextPlugin();
        assertEquals(13, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestStartupClass"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.one.TestApplicationClass"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestClassOne"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestClassTwo"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestClassThree"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestClassFour"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.one.TestIgnoreStartupClass"));
        assertTrue(
            plugin.getClassDependencies().contains(
                "mumtest.plugin.one.TestIgnoreApplicationClass"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.one.published.PublicClassTwo"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.one.published.PublicClassOne"));
        assertTrue(
            plugin.getClassDependencies().contains(
                "mumtest.plugin.one.published.PublicInterfaceOne"));
        assertTrue(plugin.getClassDependencies().contains(IApplication.class.getName()));
        assertTrue(plugin.getClassDependencies().contains(IIgnoreApplication.class.getName()));

        nextPlugin();
        assertEquals(3, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.two.TestClassSeven"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.two.whatever.IAppDescendant"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.IApplication"));

        nextPlugin();
        assertEquals(8, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.three.TestInterfaceOne"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.three.TestInterfaceTwo"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.three.Application"));
        assertTrue(
            plugin.getClassDependencies().contains("mumtest.plugin.three.IStartupDescendant"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.three.TestClassFive"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.three.TestClassSix"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.plugin.one.TestClassOne"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.IApplication"));

        nextPlugin();
        assertEquals(0, plugin.getClassDependencies().size());
    }

    public void testDependentPlugins() throws InvalidPluginException {
        nextPlugin();
        assertEquals(2, plugin.getPluginDependencies().size());
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("one.two.three.plugin", "0.1.0"),
                    null,
                    false)));
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("some.other.module", "2.3.4"),
                    null,
                    true)));
        assertFalse(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("some.other.module", "2.3.4"),
                    null,
                    false)));

        nextPlugin();
        assertEquals(0, plugin.getPluginDependencies().size());

        nextPlugin();
        assertEquals(2, plugin.getPluginDependencies().size());
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("test.two", "1.2.3"),
                    null,
                    false)));
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("some.other.module", "0.1.0"),
                    null,
                    false)));

        nextPlugin();
        assertEquals(1, plugin.getPluginDependencies().size());
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("test.three", "0.0.0"),
                    null,
                    false)));

        nextPlugin();
        assertEquals(0, plugin.getPluginDependencies().size());

        nextPlugin();
        assertEquals(3, plugin.getPluginDependencies().size());
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("test.one", "0.1.0"),
                    null,
                    false)));
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("test.two", "0.2.0"),
                    null,
                    false)));
        assertTrue(
            plugin.getPluginDependencies().containsValue(
                PluginDependency.getDependency(
                    PluginID.getPluginID("test.three", "0.3.0"),
                    null,
                    false)));
    }

    public void testPublishedClasses() throws InvalidPluginException {
        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
        assertFalse(plugin.publishAllClasses());

        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
        assertFalse(plugin.publishAllClasses());

        nextPlugin();
        assertEquals(4, plugin.getPublished().size());
        assertPublished("mumtest.plugin.one.TestClassThree");
        assertPublished("mumtest.plugin.one.TestApplicationClass");
        assertPublished("mumtest.plugin.one.published.PublicClassOne");
        assertPublished("mumtest.plugin.one.published.PublicClassTwo");
        assertFalse(plugin.publishAllClasses());

        nextPlugin();
        assertEquals(1, plugin.getPublished().size());
        assertPublished("mumtest.plugin.two.TestClassSeven");
        assertFalse(plugin.publishAllClasses());

        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
        assertTrue(plugin.publishAllClasses());

        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
        assertFalse(plugin.publishAllClasses());

    }

    public void testErrorConditions() {
        try {
            plugin = new ConfigPlugin(null, (URL) null);
            fail("Should have thrown an NullPointerException");
        } catch (NullPointerException e) {
        } catch (InvalidPluginException e) {
            fail(e.getMessage());
        }
        try {
            plugin = new ConfigPlugin(null, new URL(nonExistantFile));
            fail("Should have thrown an InvalidPluginException");
        } catch (IllegalArgumentException e) {
            fail(e.getMessage());
        } catch (MalformedURLException e) {
            fail(e.getMessage());
        } catch (InvalidPluginException e) {
            assertEquals(FileNotFoundException.class, e.getCause().getClass());
        }

        assertTrue(
            getManifestError(nonExistantFileManifest).getCause() instanceof FileNotFoundException);

        assertManifestError(INVALID_MANIFEST, badRootManifest);

        assertManifestError(MISSING_PLUGIN_ID, missingPluginIDManifest);

        assertTrue(
            getManifestError(badRootVersionManifest).getCause() instanceof VersionFormatException);

        assertTrue(
            getManifestError(missingRootVersionManifest).getCause()
                instanceof VersionFormatException);

        assertManifestError(MULTIPLE_DISTRIBUTION_SECTIONS, duplicateDistributionSectionsManifest);

        assertManifestError(INVALID_FILE_ENTRY, missingFileNameManifest);

        assertManifestError(MULTIPLE_PREREQUISITES_SECTIONS, duplicatePrerequesiteSectionsManifest);

        assertManifestError(INVALID_PREREQUISITES_ENTRY, badPrerequesiteManifest);

        Exception e = getManifestError(badPrerequisiteVersionManifest);
        utils.assertManifestError(e, INVALID_PREREQUISITES_ENTRY);
        assertTrue(e.getCause() instanceof VersionFormatException);

        assertManifestError(MULTIPLE_PUBLISHED_SECTIONS, duplicatePublishedSectionsManifest);

        assertManifestError(INVALID_PUBLISHED_ENTRY, missingPublishedClassManifest);

        e = getManifestError(classNotFoundManifest);
        assertTrue(e.getCause() instanceof ClassNotFoundException);
        utils.assertManifestError(e, "a.b.c.SomeClass" + CLASS_NOT_FOUND);

        assertManifestError(INVALID_WILDCARD, badClassWildcardManifest);

        assertManifestError(NO_CLASSES_FOUND + "non.existant.files.*", wildcardNotFoundManifest);

        assertManifestError(TOO_MANY_PUBLISHED_ENTRIES, tooManyPublishedEntries1Manifest);

        assertManifestError(TOO_MANY_PUBLISHED_ENTRIES, tooManyPublishedEntries2Manifest);

        assertManifestError(DUPLICATE_DEPENDENCY_TXT, dupDependencyManifest);

        assertManifestError(SELF_DEPENDENCY_TXT, selfDependencyManifest);

        try {
            loadPlugin(plugin5);
            fail("Should have thrown an InvalidPluginException");
        } catch (InvalidPluginException ex) {
            utils.assertManifestError(ex, MULTIPLE_STARTUP_CLASSES);
        }
        try {
            loadPlugin(plugin6);
            fail("Should have thrown an InvalidPluginException");
        } catch (InvalidPluginException ex) {
            utils.assertManifestError(ex, MULTIPLE_APPLICATION_CLASSES);
        }
        try {
            loadPlugin(plugin7);
            fail("Should have thrown an InvalidPluginException");
        } catch (InvalidPluginException ex) {
            utils.assertManifestError(ex, MULTIPLE_APPLICATION_CLASSES);
        }
        try {
            loadPlugin(plugin8);
            fail("Should have thrown an InvalidPluginException");
        } catch (InvalidPluginException ex) {
            utils.assertManifestError(ex, MULTIPLE_STARTUP_CLASSES);
        }
    }

    public void testIgnores() throws InvalidPluginException {
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        nextPlugin();
        assertEquals(IGNORE_NONE, plugin.getIgnoreCode());
        loadPlugin(ignore1);
        assertEquals(IGNORE_UNAVAILABLE, plugin.getIgnoreCode());
    }

    public void testBadStartup() {
        URL url =
            this.getClass().getClassLoader().getResource(dataRoot + "plugins/startup/manifest.xml");
        assertNotNull(url);
        try {
            new ConfigPlugin(null, url);
            fail("Should have thrown InvalidPluginException");
        } catch (InvalidPluginException e) {
            assertTrue(e.getMessage().indexOf(ILLEGAL_BOOT_DEPENDENCY_START) >= 0);
            assertTrue(e.getMessage().indexOf(ILLEGAL_BOOT_DEPENDENCY_END) >= 0);
        }
    }

    public void testExtensionsAndExtensionPoints() throws InvalidPluginException {
        URL url =
            this.getClass().getClassLoader().getResource(
                dataRoot + "manifests/extensionPoint1.xml");
        assertNotNull(url);
        ConfigPlugin plugin = new ConfigPlugin(null, url);
        assertNotNull(plugin.getExtensionPoints());
        assertEquals(2, plugin.getExtensionPoints().size());
        assertTrue(plugin.getExtensionPoints().contains("a.b.c.ep1.aaa"));
        assertTrue(plugin.getExtensionPoints().contains("a.b.c.ep1.bbb"));
        assertNotNull(plugin.getExtensions());
        assertEquals(0, plugin.getExtensions().size());
        url =
            this.getClass().getClassLoader().getResource(
                dataRoot + "manifests/extensionPoint2.xml");
        assertNotNull(url);
        plugin = new ConfigPlugin(null, url);
        assertNotNull(plugin.getExtensionPoints());
        assertEquals(3, plugin.getExtensionPoints().size());
        assertTrue(plugin.getExtensionPoints().contains("d.e.f.ep2.ccc"));
        assertTrue(plugin.getExtensionPoints().contains("d.e.f.ep2.ddd"));
        assertTrue(plugin.getExtensionPoints().contains("d.e.f.ep2.eee"));
        assertNotNull(plugin.getExtensions());
        assertEquals(3, plugin.getExtensions().size());
        assertTrue(plugin.getExtensions().contains("a.b.c.ep1.aaa"));
        assertTrue(plugin.getExtensions().contains("d.e.f.ep2.ddd"));
        assertTrue(plugin.getExtensions().contains("a.b.c.ep1.eee"));
        url =
            this.getClass().getClassLoader().getResource(
                dataRoot + "manifests/extensionPoint3.xml");
        assertNotNull(url);
        plugin = new ConfigPlugin(null, url);
        assertEquals(0, plugin.getExtensionPoints().size());
        assertEquals(4, plugin.getExtensions().size());
        assertTrue(plugin.getExtensions().contains("a.b.c.ep1.aaa"));
        assertTrue(plugin.getExtensions().contains("d.e.f.ep2.ddd"));
        assertTrue(plugin.getExtensions().contains("a.b.c.ep1.eee"));
        assertTrue(plugin.getExtensions().contains("r.s.t.uvw.xyz"));

    }

    public void testPluginNames() throws InvalidPluginException {
        nextPlugin();
        assertEquals("Test Plugin", plugin.getName());
        nextPlugin();
        assertEquals("Empty Plugin", plugin.getName());
        nextPlugin();
        assertEquals("Plugin ONE", plugin.getName());
        nextPlugin();
        assertNull(plugin.getName());
        nextPlugin();
        assertEquals("Test Plugin", plugin.getName());
    }

    public void testDeclaredClasses() throws InvalidPluginException {
        nextPlugin();
        assertEquals(1, plugin.getDeclaredClassDependencies().size());
        assertTrue(plugin.getDeclaredClassDependencies().contains("do.not.care"));
        nextPlugin();
        assertEquals(0, plugin.getDeclaredClassDependencies().size());
    }

    //********************
    //*** end of tests ***
    //********************

    public ConfigPluginTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        manifestIndex = -1;
        MockJVM.Init();
        utils = new TestUtils(this);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        MockJVM.ShutDown();
    }

    private IConfigPlugin plugin;

    private String dataRoot = "gov/va/med/hds/cd/mum/config/testdata/";

    private String nonExistantFile = "file:/fjdsklafjkdlsafj783219yfhdsjfh";

    private String nonExistantFileManifest = "manifests/nonexistantFile.xml";
    private String badRootManifest = "manifests/badRoot.xml";
    private String badRootVersionManifest = "manifests/badRootVersion.xml";
    private String missingPluginIDManifest = "manifests/missingPluginID.xml";
    private String missingRootVersionManifest = "manifests/missingRootVersion.xml";
    private String duplicateDistributionSectionsManifest =
        "manifests/duplicateDistributionSections.xml";
    private String missingFileNameManifest = "manifests/missingFileName.xml";
    private String badPrerequisiteVersionManifest = "manifests/badPrerequisiteVersion.xml";
    private String badPrerequesiteManifest = "manifests/badPrerequesite.xml";
    private String missingPublishedClassManifest = "manifests/missingPublishedClass.xml";
    private String classNotFoundManifest = "manifests/classNotFound.xml";
    private String duplicatePublishedSectionsManifest = "manifests/duplicatePublishedSections.xml";
    private String duplicatePrerequesiteSectionsManifest =
        "manifests/duplicatePrerequesiteSections.xml";
    private String badClassWildcardManifest = "manifests/badClassWildcard.xml";
    private String wildcardNotFoundManifest = "manifests/wildcardNotFound.xml";
    private String tooManyPublishedEntries1Manifest = "manifests/tooManyPublishedEntries1.xml";
    private String tooManyPublishedEntries2Manifest = "manifests/tooManyPublishedEntries2.xml";
    private String dupDependencyManifest = "manifests/dupDependency.xml";
    private String selfDependencyManifest = "manifests/selfDependency.xml";

    private String ignore1 = "manifests/ignoreManifest1.xml";

    private String[] manifests =
        {
            "manifests/manifest1.xml",
            "manifests/manifest2.xml",
            "plugins/one/manifest.xml",
            "plugins/two/manifest.xml",
            "plugins/three/manifest.xml",
            "plugins/four/manifest.xml" };

    private String plugin5 = "plugins/five/manifest.xml";
    private String plugin6 = "plugins/six/manifest.xml";
    private String plugin7 = "plugins/seven/manifest.xml";
    private String plugin8 = "plugins/eight/manifest.xml";

    private int manifestIndex;

    private URL nextPlugin() throws InvalidPluginException {
        manifestIndex++;
        URL result = loadPlugin(manifests[manifestIndex]);
        return result;
    }

    private boolean manifestsAvailable() {
        return (manifestIndex < (manifests.length - 1));
    }
    private static HashMap map = new HashMap();

    private URL loadPlugin(String manifestName) throws InvalidPluginException {
        String resource = dataRoot + manifestName;
        URL url = this.getClass().getClassLoader().getResource(resource);
        plugin = (IConfigPlugin) map.get(resource);
        if (plugin == null) {
            plugin = new ConfigPlugin(null, url);
            map.put(resource, plugin);
        }
        return url;
    }

    private String assertManifestFileAfterLoad() throws InvalidPluginException {
        URL url = nextPlugin();
        String root = url.toString();
        int idx = root.lastIndexOf("/");
        root = root.substring(0, idx + 1);
        entry = getFileEntry(root + getManifestFile());
        assertNull(entry.getPackages());
        return root;
    }

    private String getManifestFile() {
        String result = manifests[manifestIndex];
        int idx = result.lastIndexOf("/");
        return result.substring(idx + 1, result.length());
    }

    private ConfigFileEntry getFileEntry(String url) {
        for (int i = 0; i < plugin.getFileCount(); i++) {
            ConfigFileEntry entry = plugin.getFile(i);
            if (entry.getURL().toString().equals(url))
                return entry;
        }
        for (int i = 0; i < plugin.getFileCount(); i++)
            System.out.println(plugin.getFile(i).getURL().toString());
        fail("Should have had file " + url);
        return null;
    }

    private InvalidPluginException getManifestError(String manifestFile) {
        InvalidPluginException result = null;
        try {
            loadPlugin(manifestFile);
            fail("Should have thrown an InvalidPluginException");
        } catch (InvalidPluginException e) {
            result = e;
        }
        return result;
    }

    private void assertManifestError(String message, String manifest) {
        utils.assertManifestError(getManifestError(manifest), message);
    }

    public void assertPluginID(String id) throws InvalidPluginException {
        nextPlugin();
        assertEquals(id, plugin.getPluginID().toString());
    }

    private void assertPublished(String name) {
        assertTrue("should have had published class " + name, plugin.getPublished().contains(name));
    }

    TestUtils utils;
    ConfigFileEntry entry;
}
