package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.config.ClassFileAnalyzer;
import gov.va.med.hds.cd.mum.config.ConfigPluginFactory;
import gov.va.med.hds.cd.mum.config.ConfigPluginLocatorFactory;
import gov.va.med.hds.cd.mum.config.Configuration;
import gov.va.med.hds.cd.mum.config.IConfigPlugin;
import gov.va.med.hds.cd.mum.config.IConfigPluginLocator;
import gov.va.med.hds.cd.mum.config.IPluginManager;
import gov.va.med.hds.cd.mum.config.JVMAnalyzer;
import gov.va.med.hds.cd.mum.config.PluginDependency;
import gov.va.med.hds.cd.mum.config.PluginLocatorException;
import gov.va.med.hds.cd.mum.config.PluginManagerFactory;
import gov.va.med.hds.cd.mum.test.AssertionList;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.utils.ChecksumCalculator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.net.URL;

import junit.framework.TestCase;

public class DummyConfigFileTests extends TestCase implements IMUMTestConstants {

    public void testPluginLocator() throws PluginLocatorException {
        testCount++;
        ConfigPluginFactory.assignAlternateConfigPluginClass(MockConfigPlugin.class);
        try {
            IConfigPluginLocator locator =
                ConfigPluginLocatorFactory.newConfigPluginLocator(
                    TestFileBuilder.getGoldFile(),
                    null);
            ListMap list = new ListMap();
            locator.createDistributionPlugins(list);
            assertEquals(21, list.size());
            for (int i = 0; i < MOCKJAR_DATA.length; i++) {
                assertHasManifest(list, i);
            }
        } finally {
            ConfigPluginFactory.assignAlternateConfigPluginClass(null);
        }
    }

    public void testPlugins() throws PluginLocatorException {
        testCount++;
        IPluginManager manager = PluginManagerFactory.newPluginManager();
        manager.assignGoldDirectory(TestFileBuilder.getGoldFile());
        JVMAnalyzer.getClasses();
        manager.load();
        assertEquals(21, manager.getPlugins().size());
        IConfigPlugin plugin;

        plugin = getPlugin(manager.getPlugins(), mjEditor1);
        assertEquals("NoteEditor", plugin.getID());
        assertEquals("1.0.0", plugin.getVersion().toString());
        assertEquals("MoreNotes", plugin.getModule());
        assertEquals(MOCKJAR_DATA[mjEditor1][miDir], plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjEditor1][miGoldDir], plugin.getSourceDirectory());
        assertNull(plugin.getApplicationClassName());
        assertEquals("mumtest.mum.TestStartup", plugin.getStartupClassName());
        assertEquals(2, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("tiu.notes.editor.DummyClass"));
        assertTrue(plugin.getClasses().contains("mumtest.mum.TestStartup"));
        assertEquals(5, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("tiu.notes.editor.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("tiu.notes.internal.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("something.not.found.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.mum.TestStartup"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.update.Startup"));
        assertTrue(plugin.getPluginID().toString().equals("NoteEditor v1.0.0"));
        assertEquals(3, plugin.getPluginDependencies().size());
        assertNeedsPlugin(plugin, "NoteCore v1.0.0");
        assertNeedsPlugin(plugin, "NoteTest v0.7534.0");
        assertNeedsPlugin(plugin, "Runtime v1.0.0");
        assertTrue(plugin.publishAllClasses());
        assertEquals(0, plugin.getPublished().size());

        plugin = getPlugin(manager.getPlugins(), mjEditor3);
        assertEquals("VendorNoteEditor", plugin.getID());
        assertEquals("0.0.1", plugin.getVersion().toString());
        assertEquals("VendorNoteEditor", plugin.getModule());
        assertEquals(MOCKJAR_DATA[mjEditor3][miDir], plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjEditor3][miGoldDir], plugin.getSourceDirectory());
        assertNull(plugin.getApplicationClassName());
        assertEquals("mumtest.mum.AnotherTestStartup", plugin.getStartupClassName());
        assertEquals(4, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("vendor.notes.DummyClass"));
        assertTrue(plugin.getClasses().contains("vendor.notes.editor.DummyClass"));
        assertTrue(plugin.getClasses().contains("vendor.notes.utils.DummyClass"));
        assertTrue(plugin.getClasses().contains("mumtest.mum.AnotherTestStartup"));
        assertEquals(6, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("vendor.notes.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("vendor.notes.editor.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("vendor.notes.utils.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.mum.AnotherTestStartup"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.update.Startup"));
        assertTrue(plugin.getClassDependencies().contains("tiu.notes.internal.DummyClass"));
        assertTrue(plugin.getPluginID().toString().equals("VendorNoteEditor v0.0.1"));
        assertEquals(2, plugin.getPluginDependencies().size());
        assertNeedsPlugin(plugin, "NoteCore v1.0.0");
        assertNeedsPlugin(plugin, "Runtime v0.5.0");
        assertFalse(plugin.publishAllClasses());
        assertEquals(1, plugin.getPublished().size());
        assertTrue(plugin.getPublished().contains("mumtest.mum.AnotherTestStartup"));

        plugin = getPlugin(manager.getPlugins(), mjTemplateFields2);
        assertEquals("TemplateFields", plugin.getID());
        assertEquals("1.5.0", plugin.getVersion().toString());
        assertEquals("Templates", plugin.getModule());
        assertEquals(MOCKJAR_DATA[mjTemplateFields2][miDir], plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjTemplateFields2][miGoldDir], plugin.getSourceDirectory());
        assertNull(plugin.getApplicationClassName());
        assertNull(plugin.getStartupClassName());
        assertEquals(3, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("tiu.templates.DummyClass"));
        assertTrue(plugin.getClasses().contains("tiu.templates.fields.DummyClass"));
        assertTrue(plugin.getClasses().contains("tiu.templates.fields.internal.DummyClass"));
        assertEquals(4, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("tiu.templates.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("tiu.templates.fields.DummyClass"));
        assertTrue(
            plugin.getClassDependencies().contains("tiu.templates.fields.internal.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("tiu.templates.internal.DummyClass"));
        assertTrue(plugin.getPluginID().toString().equals("TemplateFields v1.5.0"));
        assertEquals(1, plugin.getPluginDependencies().size());
        assertNeedsPlugin(plugin, "TemplateCore v1.0.0");
        assertFalse(plugin.publishAllClasses());
        assertEquals(2, plugin.getPublished().size());
        assertTrue(plugin.getPublished().contains("tiu.templates.DummyClass"));
        assertTrue(plugin.getPublished().contains("tiu.templates.fields.DummyClass"));

        plugin = getPlugin(manager.getPlugins(), mjEncounters);
        assertEquals("PCE", plugin.getID());
        assertEquals("1.0.0", plugin.getVersion().toString());
        assertEquals("PCE", plugin.getModule());
        assertEquals(MOCKJAR_DATA[mjEncounters][miDir], plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjEncounters][miGoldDir], plugin.getSourceDirectory());
        assertNull(plugin.getApplicationClassName());
        assertNull(plugin.getStartupClassName());
        assertEquals(2, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("pce.DummyClass"));
        assertTrue(plugin.getClasses().contains("pce.internal.DummyClass"));
        assertEquals(3, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("pce.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("pce.internal.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("util.two.DummyClass"));
        assertTrue(plugin.getPluginID().toString().equals("PCE v1.0.0"));
        assertEquals(1, plugin.getPluginDependencies().size());
        assertNeedsPlugin(plugin, "MyLib v0.75.0");
        assertFalse(plugin.publishAllClasses());
        assertEquals(1, plugin.getPublished().size());
        assertTrue(plugin.getPublished().contains("pce.DummyClass"));

        plugin = getPlugin(manager.getPlugins(), mjNoteCore1);
        assertEquals("NoteCore", plugin.getID());
        assertEquals("2.0.0", plugin.getVersion().toString());
        assertEquals("Notes", plugin.getModule());
        assertEquals("NoteCore_2.0.0", plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjNoteCore1][miDir], plugin.getDirectory());
        assertEquals(MOCKJAR_DATA[mjNoteCore1][miGoldDir], plugin.getSourceDirectory());
        assertEquals("mumtest.mum.TestApplication", plugin.getApplicationClassName());
        assertEquals("mumtest.mum.TestStartup", plugin.getStartupClassName());
        assertEquals(4, plugin.getClasses().size());
        assertTrue(plugin.getClasses().contains("tiu.notes.DummyClass"));
        assertTrue(plugin.getClasses().contains("tiu.notes.internal.DummyClass"));
        assertTrue(plugin.getClasses().contains("mumtest.mum.TestApplication"));
        assertTrue(plugin.getClasses().contains("mumtest.mum.TestStartup"));
        assertEquals(6, plugin.getClassDependencies().size());
        assertTrue(plugin.getClassDependencies().contains("tiu.notes.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("tiu.notes.internal.DummyClass"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.mum.TestApplication"));
        assertTrue(plugin.getClassDependencies().contains("mumtest.mum.TestStartup"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.update.Startup"));
        assertTrue(plugin.getClassDependencies().contains("gov.va.med.hds.cd.mum.IApplication"));

        assertTrue(plugin.getPluginID().toString().equals("NoteCore v2.0.0"));
        assertEquals(1, plugin.getPluginDependencies().size());
        assertNeedsPlugin(plugin, "Runtime v0.0.0");
        assertFalse(plugin.publishAllClasses());
        assertEquals(2, plugin.getPublished().size());
        assertTrue(plugin.getPublished().contains("mumtest.mum.TestApplication"));
        assertTrue(plugin.getPublished().contains("mumtest.mum.TestStartup"));
    }

    public void testPasthrough() throws Throwable {
        testCount++;
        IPluginManager manager = PluginManagerFactory.newPluginManager();
        manager.assignGoldDirectory(TestFileBuilder.getGoldFile());
        Configuration config = new Configuration(manager);
        int idx = 0;
        while (idx < config.getPlugins().size()) {
            IConfigPlugin plugin = (IConfigPlugin) config.getPlugins().get(idx);
            boolean ok = false;
            for (int i = 0; i < GOOD_CONFIG.length; i++) {
                if (plugin.getDirectory().equals(MOCKJAR_DATA[GOOD_CONFIG[i]][miDir])) {
                    ok = true;
                    break;
                }
            }
            if (ok)
                idx++;
            else
                config.getPlugins().remove(plugin.getID(), plugin);
        }
        config.validate();
        for (int i = 0; i < config.errors().getErrors().size(); i++) {
            Error error = (Error) config.errors().getErrors().get(i);
            System.out.println(error.toString());
        }
        ConfigurationTest.assertNoErrors(config.errors());
        URL url =
            this.getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/update/testdata/configurations/settings-internal.xml");
        File xml = new File(TestFileBuilder.getGoldFile(), TEST_CONFIG_FILE);
        assertFalse(xml.exists());
        config.saveXML(xml, url, false, false);
        assertTrue(xml.exists());
        BufferedReader reader = new BufferedReader(new FileReader(xml));
        StringBuffer expected = new StringBuffer();
        StringBuffer was = new StringBuffer();
        int index = 0;
        AssertionList al = new AssertionList();
        try {
            try {
                String line;
                while ((line = reader.readLine()) != null) {
                    was.append(line);
                    was.append(WIN_EOL);
                    if (index < CONFIG_XML.length) {
                        expected.append(CONFIG_XML[index]);
                        expected.append(WIN_EOL);
                    }
                    al.assertEquals(line, CONFIG_XML[index], line);
                    index++;
                }
            } finally {
                reader.close();
            }
            String wasText = was.toString();
            al.assertEquals(expected.toString(), wasText);
            ChecksumCalculator crc = new ChecksumCalculator();
            crc.calcConfigFile(new FileInputStream(xml));
            al.assertEquals(CONFIG_XML_CRC, crc.getValue());
        } catch (Throwable e) {
            al.addError(e);
        } finally {
            al.showErrors();
        }
        
    }

    //********************
    //*** end of tests ***
    //********************

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(DummyConfigFileTests.class);

    public DummyConfigFileTests(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.ensureTestDirectories(false);
        }
        ClassFileAnalyzer.SetUseIgnoreInterfaces(false);
        ChecksumCalculator.setJarTestMode(true);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
        ClassFileAnalyzer.SetUseIgnoreInterfaces(true);
        ChecksumCalculator.setJarTestMode(false);
    }

    private void assertHasManifest(ListMap list, int idx) {
        String manifest =
            PLUGIN_DIRECTORY + "/" + (String) MOCKJAR_DATA[idx][miGoldDir] + "/" + MANIFEST_FILE;
        for (int i = 0; i < list.size(); i++) {
            IConfigPlugin plugin = (IConfigPlugin) list.get(i);
            if (plugin.getID().endsWith(manifest)) {
                return;
            }
        }
        for (int i = 0; i < list.size(); i++) {
            IConfigPlugin plugin = (IConfigPlugin) list.get(i);
            System.out.println(plugin.getID());
        }
        fail("Should have had manifest " + manifest);
    }

    private IConfigPlugin getPlugin(ListMap list, int index) {
        for (int i = 0; i < list.size(); i++) {
            IConfigPlugin plugin = (IConfigPlugin) list.get(i);
            if (plugin.getDirectory().equals(MOCKJAR_DATA[index][miDir]))
                return plugin;
        }
        fail("Should have had " + MOCKJAR_DATA[index][miDir]);
        return null;
    }

    private void assertNeedsPlugin(IConfigPlugin plugin, String pluginName) {
        for (int i = 0; i < plugin.getPluginDependencies().size(); i++) {
            PluginDependency id = (PluginDependency) plugin.getPluginDependencies().get(i);
            if (id.getPluginID().toString().equals(pluginName))
                return;
        }
        fail("Should have needed plugin " + pluginName);
    }
}