package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.config.Error;
import gov.va.med.hds.cd.mum.config.ErrorManager;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.config.IConfigPlugin;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import junit.framework.TestCase;

public class ErrorTesting extends TestCase implements IConfigConstants, IMUMTestConstants {

    public void testErrorManager() {
        assertFalse(mgr.hasErrors());
        IConfigPlugin plugin1 = new MockConfigPlugin(mjEditor1);
        IConfigPlugin plugin2 = new MockConfigPlugin(mjEditor2);
        IConfigPlugin plugin3 = new MockConfigPlugin(mjEditor3);
        mgr.addError(Error.MULTIPLE_STARTUP_PLUGINS, "Something", plugin1, plugin2);
        assertTrue(mgr.hasErrors());
        assertTrue(mgr.hasError(Error.MULTIPLE_STARTUP_PLUGINS));
        assertFalse(mgr.hasError(Error.MULTIPLE_APPLICATION_PLUGINS));
        mgr.addError(Error.MULTIPLE_STARTUP_PLUGINS, "Something", plugin2, plugin3);
        assertEquals(1, mgr.getErrors().size());
        Error error = (Error) mgr.getErrors().get(0);
        assertEquals(3, error.getPluginList().size());
        assertEquals(Error.MULTIPLE_STARTUP_PLUGINS, error.getErrorType());
        assertTrue(error.getPluginList().contains(plugin1));
        assertTrue(error.getPluginList().contains(plugin2));
        assertTrue(error.getPluginList().contains(plugin3));
        mgr.addError(Error.MULTIPLE_APPLICATION_PLUGINS, plugin1, plugin3);
        assertEquals(2, mgr.getErrors().size());
        assertTrue(mgr.hasError(Error.MULTIPLE_APPLICATION_PLUGINS));
        assertFalse(mgr.hasError(Error.UNAVAILABLE_DEPENDENT_CLASS));
        mgr.addError(Error.MULTIPLE_APPLICATION_PLUGINS, plugin3, null);
        assertEquals(2, mgr.getErrors().size());
        error = (Error) mgr.getErrors().get(1);
        assertEquals(Error.MULTIPLE_APPLICATION_PLUGINS, error.getErrorType());
        assertEquals(2, error.getPluginList().size());
        assertTrue(error.getPluginList().contains(plugin1));
        assertTrue(error.getPluginList().contains(plugin3));

        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "smile", plugin1);
        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "smile", plugin1);
        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "smile", plugin2);
        assertEquals(3, mgr.getErrors().size());
        error = (Error) mgr.getErrors().get(2);
        assertEquals(Error.UNAVAILABLE_DEPENDENT_CLASS, error.getErrorType());
        assertEquals(2, error.getPluginList().size());
        assertTrue(error.getPluginList().contains(plugin1));
        assertTrue(error.getPluginList().contains(plugin2));
        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "fred", plugin1);
        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "ralph", plugin1);
        assertEquals(5, mgr.getErrors().size());
        assertTrue(mgr.hasError(Error.UNAVAILABLE_DEPENDENT_CLASS));
        mgr.clear();
        assertEquals(0, mgr.getErrors().size());
        assertFalse(mgr.hasErrors());
        assertFalse(mgr.hasError(Error.MULTIPLE_STARTUP_PLUGINS));
        assertFalse(mgr.hasError(Error.MULTIPLE_APPLICATION_PLUGINS));
        assertFalse(mgr.hasError(Error.UNAVAILABLE_DEPENDENT_CLASS));
    }

    public void testAddingExistingErrors() {
        ErrorManager mgr2 = new ErrorManager();
        mgr.addError(Error.MISSING_STARTUP_PLUGIN);
        mgr.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "some.class");
        mgr2.addError(Error.MISSING_APPLICATION_PLUGIN);
        mgr2.addError(Error.UNRESOLVED_DEPENDENT_PLUGIN);
        assertTrue(mgr.hasError(Error.MISSING_STARTUP_PLUGIN));
        assertTrue(mgr.hasError(Error.UNAVAILABLE_DEPENDENT_CLASS));
        assertTrue(mgr2.hasError(Error.MISSING_APPLICATION_PLUGIN));
        assertTrue(mgr2.hasError(Error.UNRESOLVED_DEPENDENT_PLUGIN));
        assertFalse(mgr2.hasError(Error.MISSING_STARTUP_PLUGIN));
        assertFalse(mgr2.hasError(Error.UNAVAILABLE_DEPENDENT_CLASS));
        assertFalse(mgr.hasError(Error.MISSING_APPLICATION_PLUGIN));
        assertFalse(mgr.hasError(Error.UNRESOLVED_DEPENDENT_PLUGIN));
        assertEquals(2, mgr.getErrors().size());
        mgr.addErrors(mgr2);
        assertTrue(mgr.hasError(Error.MISSING_APPLICATION_PLUGIN));
        assertTrue(mgr.hasError(Error.UNRESOLVED_DEPENDENT_PLUGIN));
        assertEquals(4, mgr.getErrors().size());
        mgr2.addError(Error.UNAVAILABLE_DEPENDENT_CLASS, "some.other.class");
        mgr.addErrors(mgr2);
        assertEquals(5, mgr.getErrors().size());
        ErrorManager mgr3 = new ErrorManager();
        Error error1 = mgr3.addError(Error.UNPUBLISHED_DEPENDENT_CLASS, "one");
        Error error2 = mgr3.addError(Error.UNPUBLISHED_DEPENDENT_CLASS, "two");
        Error error3 = mgr3.addError(Error.UNPUBLISHED_DEPENDENT_CLASS, "two");
        assertEquals(error2, error3);
        assertFalse(error1 == error2);
        mgr.addError(error1);
        assertEquals(6, mgr.getErrors().size());
        mgr.addError(error2);
        assertEquals(7, mgr.getErrors().size());
        mgr.addError(error3);
        assertEquals(7, mgr.getErrors().size());
        assertEquals(3, mgr2.getErrors().size());
        mgr2.addErrors(mgr);
        assertEquals(8, mgr2.getErrors().size());
    }

    public void testTooManyErrorCodes() {
        TestErrorType.startErrorCountTest();
        try {
            int i = 0;
            long errorCode;
            long lastErrorCode = 0;
            TestErrorType errorType = null;
            boolean done = false;
            do {
                try {
                    errorType = new TestErrorType();
                    errorCode = errorType.getErrorCode();
                    if (errorCode <= lastErrorCode)
                        fail("Should have thrown IllegalStateException");
                    lastErrorCode = errorCode;
                    i++;
                    if(i > 200)
						fail("Should have thrown IllegalStateException");
                } catch (IllegalStateException e) {
                    done = true;
                }
            } while (!done);

        } finally {
            TestErrorType.stopErrorCountTest();
        }
    }

    //********************
    //*** end of tests ***
    //********************

    public ErrorTesting(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        mgr = new ErrorManager();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    ErrorManager mgr;

    private static class TestErrorType extends Error.ErrorType {

        public TestErrorType() {
            super();
        }

        protected Error createError() {
            return null;
        }

        public int getErrorNumber() {
            return super.getErrorNumber();
        }

        public long getErrorCode() {
            return super.getErrorCode();
        }
        public static void startErrorCountTest() {
            Error.ErrorType.startErrorCountTest();
        }

        public static void stopErrorCountTest() {
            Error.ErrorType.stopErrorCountTest();
        }
    }
}
