package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.IApplication;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.config.IClassListener;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.config.JarFileAnalyzer;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;

public class JarFileAnalyzerTest
    extends TestCase
    implements IClassListener, IConfigConstants, IBootConstants {

    public void testClassesAndPackages() throws InvalidPluginException {
        assertClassCount(0);
        assertPackageCount(0);

        doJar(1);
        assertClassCount(7);
        assertClass("DefaultPkgCls");
        assertClass("mumtest.configdata.XTestClass");
        assertClass("mumtest.configdata.XAbstractStartup");
        assertClass("mumtest.configdata.XStartupClass");
        assertClass("mumtest.configdata.XIStartupDescendant");
        assertClass("mumtest.configdata.XIApplicationDescendant");
        assertClass("mumtest.configdata.XAbstractApplication");
        assertPackageCount(3);
        assertPackage("mumtest.configdata");
        assertPackage("mumtest.configdata.resource");
        assertPackage(DEFAULT_PACKAGE);

        doJar(2);
        assertClassCount(7);
        assertClass("gov.va.med.hds.cd.config.Configuration");
        assertClass("gov.va.med.hds.querytool.Constraint");
        assertClass("gov.va.med.hds.querytool.HTMLSelectionModel$HTMLObjectItem");
        assertClass("gov.va.med.hds.cd.ccow.internal.ContextModule");
        assertClass("gov.va.med.hds.common.IDisposable");
        assertClass("javax.swing.JCheckBoxMenuItem");
        assertClass("EmptyClass");
        assertPackageCount(6);
        assertPackage("gov.va.med.hds.cd.config");
        assertPackage("gov.va.med.hds.querytool");
        assertPackage("gov.va.med.hds.cd.ccow.internal");
        assertPackage("gov.va.med.hds.common");
        assertPackage("javax.swing");
        assertPackage(DEFAULT_PACKAGE);

        doJar(5);
        assertClassCount(6);
        assertClass("mumtest.configdata.XTestClass");
        assertClass("mumtest.configdata.XAbstractStartup");
        assertClass("mumtest.configdata.XStartupClass");
        assertClass("mumtest.configdata.XIStartupDescendant");
        assertClass("mumtest.configdata.XIApplicationDescendant");
        assertClass("mumtest.configdata.XAbstractApplication");
        assertPackageCount(2);
        assertPackage("mumtest.configdata");
        assertPackage(DEFAULT_PACKAGE);
    }

    public void testDependencies() throws InvalidPluginException {
        assertDependencyCount(0);
        doJar(1);
        assertDependencyCount(7);
        assertDependency(IApplication.class.getName());
        assertDependency("mumtest.configdata.XAbstractApplication");
        assertDependency("mumtest.configdata.XAbstractStartup");
        assertDependency("mumtest.configdata.XIApplicationDescendant");
        assertDependency("mumtest.configdata.XIStartupDescendant");
        assertDependency("mumtest.configdata.XStartupClass");
        assertDependency("mumtest.configdata.XTestClass");

        doJar(2);
        assertDependencyCount(39);
        assertDependency("gov.va.med.hds.cd.ccow.ContextObserver");
        assertDependency("gov.va.med.hds.cd.ccow.IContextModule");
        assertDependency("gov.va.med.hds.cd.ccow.internal.CCOWBroker");
        assertDependency("gov.va.med.hds.cd.ccow.internal.CCOWHyperlinkBroker");
        assertDependency("gov.va.med.hds.cd.ccow.internal.ContextModule");
        assertDependency("gov.va.med.hds.cd.config.Configuration");
        assertDependency("gov.va.med.hds.cd.config.ConfigurationChangeEvent");
        assertDependency("gov.va.med.hds.cd.config.ConfigurationException");
        assertDependency("gov.va.med.hds.cd.config.IConfiguration");
        assertDependency("gov.va.med.hds.cd.config.IConfigurationChangeListener");
        assertDependency("gov.va.med.hds.cd.config.IParameterCategory");
        assertDependency("gov.va.med.hds.cd.config.IParameterTree");
        assertDependency("gov.va.med.hds.cd.config.NonExistentParameterException");
        assertDependency("gov.va.med.hds.cd.config.ParameterCategory");
        assertDependency("gov.va.med.hds.cd.config.ParameterCategoryException");
        assertDependency("gov.va.med.hds.cd.config.configDAO.ConfigDAOException");
        assertDependency("gov.va.med.hds.cd.config.configDAO.IConfigDAO");
        assertDependency("gov.va.med.hds.cd.dao.DAORegistryException");
        assertDependency("gov.va.med.hds.cd.dao.IDAORegistry");
        assertDependency("gov.va.med.hds.cd.dao.IDataAccessObject");
        assertDependency("gov.va.med.hds.cd.runtime.IModule");
        assertDependency("gov.va.med.hds.cd.runtime.IModuleDescriptor");
        assertDependency("gov.va.med.hds.cd.runtime.IRuntimeContext");
        assertDependency("gov.va.med.hds.cd.runtime.ModuleNotFoundException");
        assertDependency("gov.va.med.hds.cd.runtime.ModuleStartupException");
        assertDependency("gov.va.med.hds.common.DisposeException");
        assertDependency("gov.va.med.hds.common.IDisposable");
        assertDependency("gov.va.med.hds.querytool.Constraint");
        assertDependency("gov.va.med.hds.querytool.Constraint$ItemChangedListener");
        assertDependency("gov.va.med.hds.querytool.CriteriaList");
        assertDependency("gov.va.med.hds.querytool.HTML");
        assertDependency("gov.va.med.hds.querytool.HTMLSelectionModel");
        assertDependency("gov.va.med.hds.querytool.HTMLSelectionModel$HTMLObjectItem");
        assertDependency("gov.va.med.hds.querytool.HTMLSelectionModel$ObjectID");
        assertDependency("gov.va.med.hds.querytool.IConstraint");
        assertDependency("gov.va.med.hds.querytool.ICriteriaList");
        assertDependency("gov.va.med.hds.querytool.ICriterion");
        assertDependency("gov.va.med.hds.querytool.IValue");
        assertDependency("gov.va.med.hds.util.mumps.StringUtil");
    }

    public void testApplicationClass() throws InvalidPluginException {
        assertNull(analyzer.getApplicationClass());
        doJar(1);
        assertEquals("mumtest.configdata.XTestClass", analyzer.getApplicationClass());
        doJar(2);
        assertNull(analyzer.getApplicationClass());
    }

    public void testStartupClass() throws InvalidPluginException {
        assertNull(analyzer.getStartupClass());
        doJar(1);
        assertEquals("mumtest.configdata.XStartupClass", analyzer.getStartupClass());
        doJar(2);
        assertNull(analyzer.getStartupClass());
    }

    public void testClassLists() throws InvalidPluginException {
        classes = new ArrayList();
        analyzer.assignClassListener(this);
        doJar(1);
        assertEquals(7, classes.size());
        assertTrue(classes.contains("mumtest.configdata.XIApplicationDescendant"));
        assertTrue(classes.contains("mumtest.configdata.XIStartupDescendant"));
        assertTrue(classes.contains("mumtest.configdata.XStartupClass"));
        assertTrue(classes.contains("mumtest.configdata.XTestClass"));
        assertTrue(classes.contains("mumtest.configdata.XAbstractStartup"));
        assertTrue(classes.contains("mumtest.configdata.XAbstractApplication"));
        assertTrue(classes.contains("DefaultPkgCls"));

        classes.clear();
        doJar(2);
        assertEquals(7, classes.size());
        assertTrue(classes.contains("gov.va.med.hds.cd.config.Configuration"));
        assertTrue(classes.contains("gov.va.med.hds.querytool.Constraint"));
        assertTrue(classes.contains("gov.va.med.hds.cd.ccow.internal.ContextModule"));
        assertTrue(classes.contains("EmptyClass"));
        assertTrue(classes.contains("gov.va.med.hds.querytool.HTMLSelectionModel$HTMLObjectItem"));
        assertTrue(classes.contains("gov.va.med.hds.common.IDisposable"));
        assertTrue(classes.contains("javax.swing.JCheckBoxMenuItem"));
    }

    public void testJarErrors() {
        try {
            doJar(3);
            fail("Should have thrown InvalidPluginException");
        } catch (InvalidPluginException e) {
            assertEquals(MULTIPLE_APPLICATION_CLASSES, e.getMessage());
        }
        try {
            doJar(4);
            fail("Should have thrown InvalidPluginException");
        } catch (InvalidPluginException e) {
            assertEquals(MULTIPLE_STARTUP_CLASSES, e.getMessage());
        }
    }

    //********************
    //*** end of tests ***
    //********************

    private JarFileAnalyzer analyzer;

    public JarFileAnalyzerTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        analyzer = new JarFileAnalyzer(null, null);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    private String jarPath = "gov/va/med/hds/cd/mum/config/testdata/";

    private String testJar1 = jarPath + "TestJar1.jar";
    private String testJar2 = jarPath + "TestJar2.jar";
    private String testJar3 = jarPath + "TestJar3.jar";
    private String testJar4 = jarPath + "TestJar4.jar";
    private String testJar5 = jarPath + "TestJar5.jar";

    private String[] jars = { testJar1, testJar2, testJar3, testJar4, testJar5 };

    private void doJar(int idx) throws InvalidPluginException {
        doJar(jars[idx - 1]);
    }

    private void doJar(String jar) throws InvalidPluginException {
        URL url = this.getClass().getClassLoader().getResource(jar);
        assertNotNull("Should have found " + jar, url);
        analyzer.analyzeJar(url);
    }

    private void assertDependencyCount(int count) {
        if (analyzer.getDependencies().size() != count) {
            showValues(analyzer.getDependencies());
            fail(
                "Expected dependency count of "
                    + Integer.toString(count)
                    + " but was "
                    + Integer.toString(analyzer.getDependencies().size()));
        }
    }

    private void assertDependency(String pkg) {
        if (!analyzer.getDependencies().contains(pkg)) {
            showValues(analyzer.getDependencies());
            fail("Should have had dependency " + pkg);
        }
    }

    private void showValues(SimpleList strings) {
        List sorted = new ArrayList(strings.getList());
        Collections.sort(sorted);
        for (int i = 0; i < sorted.size(); i++) {
            System.out.println((String) sorted.get(i));
        }
    }

    private void assertClassCount(int count) {
        if (analyzer.getClasses().size() != count) {
            showValues(analyzer.getClasses());
            fail(
                "Expected class count of "
                    + Integer.toString(count)
                    + " but was "
                    + Integer.toString(analyzer.getClasses().size()));
        }
    }

    private void assertClass(String cls) {
        if (!analyzer.getClasses().contains(cls)) {
            showValues(analyzer.getClasses());
            fail("Should have had class " + cls);
        }
    }

    private void assertPackageCount(int count) {
        if (analyzer.getPackages().size() != count) {
            showValues(analyzer.getPackages());
            fail(
                "Expected pacakge count of "
                    + Integer.toString(count)
                    + " but was "
                    + Integer.toString(analyzer.getPackages().size()));
        }
    }

    private void assertPackage(String cls) {
        if (!analyzer.getPackages().contains(cls)) {
            showValues(analyzer.getPackages());
            fail("Should have had package " + cls);
        }
    }

    private ArrayList classes;

    public void classFound(String className) {
        if ((classes != null) && (!classes.contains(className)))
            classes.add(className);
    }
}
