package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.PluginID;
import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.KeyedList;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.config.ConfigFileEntry;
import gov.va.med.hds.cd.mum.config.IConfigPlugin;
import gov.va.med.hds.cd.mum.config.PluginDependency;
import gov.va.med.hds.cd.mum.config.PluginPublicationManager;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestUtils;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class MockConfigPlugin implements IConfigPlugin, IMUMTestConstants, IBootConstants {

    private static boolean noStartupJar = false;

    private static PluginPublicationManager publications =
        new PluginPublicationManager(new MockConfigPlugin());
    private String id;
    private String sourceDirectory;
    private Version version;
    private SimpleList published;
    private SimpleList classes;
    private boolean publishAllClasses;
    private SimpleList classDependencies;
    private ListMap pluginDependencies;
    private String directory;
    private PluginID plugin;
    private URL manifest;
    private List files;
    private String startupClass;
    private String applicationClass;
    private String module;
    private KeyedList dependentPlugins;
    private SimpleList extensions;
    private SimpleList extensionPoints;
    private String name;
    private SimpleList declaredClassDependencies;

    private MockConfigPlugin() {
    }

    public MockConfigPlugin(URL url) {
        super();
        id = url.getPath();
    }

    public MockConfigPlugin(int index) {
        dependentPlugins = new KeyedList();
        extensions = new SimpleList();
        extensionPoints = new SimpleList();
        declaredClassDependencies = new SimpleList();
        String nameAndVersion = (String) MOCKJAR_DATA[index][miDir];
        sourceDirectory = (String) MOCKJAR_DATA[index][miGoldDir];
        classes = new SimpleList();
        startupClass = null;
        for (int i = 0; i < STARTUP_PACKAGES.length; i++) {
            if (index == STARTUP_PACKAGES[i]) {
                startupClass = "mumtest.Startup";
                classes.add("mumtest.mum.TestStartup");
                break;
            }
        }
        applicationClass = null;
        for (int i = 0; i < APPLICATION_PACKAGES.length; i++) {
            if (index == APPLICATION_PACKAGES[i]) {
                applicationClass = "mumtest.Application";
                classes.add("mumtest.mum.TestApplication");
                break;
            }
        }
        Object[] data = MOCKJAR_DATA[index];
        module = (String) data[miModule];
        id = getName(nameAndVersion);
        String ver = getVersion(nameAndVersion);
        version = new Version(ver);
        classDependencies = new SimpleList();
        pluginDependencies = new ListMap();
        published = new SimpleList();
        plugin = PluginID.getPluginID(id, ver);
        String[] ary = (String[]) data[miRequiredPlugins];
        for (int i = 0; i < ary.length; i++) {
            PluginDependency dependency =
                PluginDependency.getDependency(
                    PluginID.getPluginID(getName(ary[i]), getVersion(ary[i])),
                    null,
                    false);
            pluginDependencies.add(dependency.getPluginID().getID(), dependency);
        }
        ary = (String[]) data[miPackages];
        URL url;
        if (startupClass == null)
            url = dummyURL(id.toLowerCase() + JAR_EXT);
        else
            url = dummyURL(STARTUP_JAR);
        ConfigFileEntry entry = new ConfigFileEntry(url);
        for (int i = 0; i < ary.length; i++) {
            String pkg = ary[i] + ".DummyClass";
            classes.add(pkg);
        }
        SimpleList list = new SimpleList();
        for (int i = 0; i < ary.length; i++) {
            list.add(ary[i]);
        }
        entry.setPackages(list);
        ary = (String[]) data[miRequiredPackages];
        for (int i = 0; i < ary.length; i++)
            classDependencies.add(ary[i] + ".DummyClass");
        directory = id + "_" + version.toString();
        manifest = dummyURL("manifest.xml");
        files = new ArrayList();
        files.add(new ConfigFileEntry(manifest));
        if ((startupClass != null) && (!noStartupJar)) {
            files.add(new ConfigFileEntry(dummyURL(BOOT_JAR)));
        }
        files.add(entry);
        try {
            publications.reset();
            String[] publishedEntries = (String[]) data[miPublishedStatements];
            for (int i = 0; i < publishedEntries.length; i++)
                publications.addPublication(publishedEntries[i]);
            publications.resolvePublications(classes, published);
            publishAllClasses = publications.publishAllClasses();
        } catch (InvalidPluginException e) {
        }
        name = TestUtils.id2Name(id);
    }

    private URL dummyURL(String fileName) {
        try {
            return new URL("file:/somedir/someOtherDir/" + fileName);
        } catch (MalformedURLException e) {
            return null;
        }
    }

    private String getName(String s) {
        int idx = s.indexOf(VERSION_SEPARATOR);
        return s.substring(0, idx);
    }

    private String getVersion(String s) {
        int idx = s.indexOf(VERSION_SEPARATOR);
        return s.substring(idx + 1);
    }

    public String getID() {
        return id;
    }

    public Version getVersion() {
        return version;
    }

    public String getModule() {
        return module;
    }

    public String getDirectory() {
        return directory;
    }

    public URL getManifest() {
        return manifest;
    }

    public SimpleList getPublished() {
        return published;
    }

    public String toString() {
        if (version == null)
            return id;
        else
            return id + " v" + version.toString();
    }
    public String getStartupClassName() {
        return startupClass;
    }

    public String getApplicationClassName() {
        return applicationClass;
    }

    public boolean publishAllClasses() {
        return publishAllClasses;
    }

    public ConfigFileEntry getFile(int index) {
        return (ConfigFileEntry) files.get(index);
    }
    public int getFileCount() {
        return files.size();
    }

    public int getIgnoreCode() {
        return 0;
    }
    public String getSourceDirectory() {
        return sourceDirectory;
    }
    public void badPlugin(String message, Throwable cause) throws InvalidPluginException {
    }
    public void badPlugin(Throwable cause) throws InvalidPluginException {
    }
    public void badPlugin(String message) throws InvalidPluginException {
    }
    public void badPlugin() throws InvalidPluginException {
    } /**
           * @see gov.va.med.hds.cd.mum.IDependentPlugin#getPluginID()
           */
    public PluginID getPluginID() {
        return plugin;
    } /**
               * @see gov.va.med.hds.cd.mum.IDependentPlugin#getClassDependencies()
               */
    public SimpleList getClassDependencies() {
        return classDependencies;
    } /**
               * @see gov.va.med.hds.cd.mum.IDependentPlugin#getPluginDependencies()
               */
    public ListMap getPluginDependencies() {
        return pluginDependencies;
    } /**
               * @see gov.va.med.hds.cd.mum.config.IConfigPlugin#getClasses()
               */
    public SimpleList getClasses() {
        return classes;
    } /**
               * @param b
               */
    public static void setNoStartupJar(boolean b) {
        noStartupJar = b;
    }

    public KeyedList requiredPlugins() {
        return dependentPlugins;
    }

    public SimpleList getExtensionPoints() {
        return extensionPoints;
    }

    public SimpleList getExtensions() {
        return extensions;
    }

    public String getName() {
        return name;
    }

    public SimpleList getDeclaredClassDependencies() {
        return declaredClassDependencies;
    }

}
