package gov.va.med.hds.cd.mum.config.test;

import gov.va.med.hds.cd.mum.PluginID;
import gov.va.med.hds.cd.mum.config.ConfigPluginLocatorFactory;
import gov.va.med.hds.cd.mum.config.IConfigPlugin;
import gov.va.med.hds.cd.mum.config.IPluginManager;
import gov.va.med.hds.cd.mum.config.PluginDependency;
import gov.va.med.hds.cd.mum.config.PluginLocatorException;
import gov.va.med.hds.cd.mum.config.PluginManagerFactory;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;

import java.util.List;

import junit.framework.TestCase;

public class PluginManagerTest extends TestCase implements IMUMTestConstants {

    public void testUnitDetection() throws PluginLocatorException {
        assertEquals(0, mgr.getPlugins().size());
        assertFalse(mgr.isLoaded());
        mgr.load();
        assertEquals(21, mgr.getPlugins().size());
        assertTrue(mgr.isLoaded());
        for (int i = 0; i < MOCKJAR_DATA.length; i++) {
            boolean found = false;
            for (int j = 0; j < mgr.getPlugins().size(); j++) {
                IConfigPlugin plugin = (IConfigPlugin) mgr.getPlugins().get(j);
                if (plugin.getDirectory().equals(MOCKJAR_DATA[i][miDir])) {
                    found = true;
                    break;
                }
            }
            if (!found)
                fail("Should have had plugin " + MOCKJAR_DATA[i][miDir]);
        }
    }

    public void testClasses() throws PluginLocatorException {
        mgr.load();
        mgr.loadPluginInformation();
        assertEquals(31, mgr.getClasses().size());
        assertTrue(mgr.getClasses().contains("mumtest.mum.TestStartup"));
        assertTrue(mgr.getClasses().contains("mumtest.mum.TestApplication"));
        assertTrue(mgr.getClasses().contains("mum.common.DummyClass"));
        assertTrue(mgr.getClasses().contains("gov.va.med.hds.cd.mum.update.DummyClass"));
        assertTrue(mgr.getClasses().contains("gov.va.med.hds.cd.mum.DummyClass"));
        assertTrue(mgr.getClasses().contains("orders.DummyClass"));
        assertTrue(mgr.getClasses().contains("orders.inpatient.DummyClass"));
        assertTrue(mgr.getClasses().contains("orders.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("orders.meds.DummyClass"));
        assertTrue(mgr.getClasses().contains("orders.outpatient.DummyClass"));
        assertTrue(mgr.getClasses().contains("pce.DummyClass"));
        assertTrue(mgr.getClasses().contains("pce.forms.DummyClass"));
        assertTrue(mgr.getClasses().contains("pce.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("reminders.DummyClass"));
        assertTrue(mgr.getClasses().contains("reminders.dialogs.DummyClass"));
        assertTrue(mgr.getClasses().contains("reminders.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.notes.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.notes.editor.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.notes.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.notes.utilities.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.templates.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.templates.core.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.templates.fields.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.templates.fields.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("tiu.templates.internal.DummyClass"));
        assertTrue(mgr.getClasses().contains("util.one.DummyClass"));
        assertTrue(mgr.getClasses().contains("util.three.DummyClass"));
        assertTrue(mgr.getClasses().contains("util.two.DummyClass"));
        assertTrue(mgr.getClasses().contains("vendor.notes.DummyClass"));
        assertTrue(mgr.getClasses().contains("vendor.notes.editor.DummyClass"));
        assertTrue(mgr.getClasses().contains("vendor.notes.utils.DummyClass"));
    }

    public void testPlugins() throws PluginLocatorException {
        mgr.load();
        mgr.loadPluginInformation();
        assertEquals(MOCKJAR_DATA.length, mgr.getPlugins().size());
        for (int i = 0; i < MOCKJAR_DATA.length; i++) {
            String dir = (String) MOCKJAR_DATA[i][miDir];
            int idx = dir.indexOf(VERSION_SEPARATOR);
            String name = dir.substring(0, idx);
            String ver = dir.substring(idx + 1);
            List temp = mgr.getPlugins().get(name);
            boolean found = false;
            PluginID id = PluginID.getPluginID(name, ver);
            for (int j = 0; j < temp.size(); j++) {
                IConfigPlugin plugin = (IConfigPlugin) temp.get(j);
                if (plugin.getPluginID() == id)
                    found = true;
            }
            assertTrue(found);
        }
    }

    public void testPluginSupport() throws PluginLocatorException {
        mgr.load();
        mgr.loadPluginInformation();
        assertSupports(true, "InpatientMedOrders", "0.5");
        assertSupports(false, "InpatientMedOrders", "1.5");
        assertSupports(false, "Whatever", "0.0");
        assertSupports(true, "NoteEditor", "0.5");
        assertSupports(true, "NoteEditor", "1.0");
        assertSupports(true, "NoteEditor", "2.5");
        assertSupports(true, "NoteEditor", "3.0.0");
        assertSupports(false, "NoteEditor", "3.0.1");
        assertSupports(false, "NoteEditor", "3.0.0.a");
    }

    public void testExtensionPoints() throws PluginLocatorException {
        mgr.load();
        IConfigPlugin plugin1 = (IConfigPlugin) mgr.getPlugins().get(0);
        plugin1.getExtensionPoints().add("one.two.three");
        plugin1.getExtensionPoints().add("four.five.six");
        IConfigPlugin plugin2 = (IConfigPlugin) mgr.getPlugins().get(1);
        plugin2.getExtensionPoints().add("seven.eight.nine");
        plugin2.getExtensionPoints().add("four.five.six");

        List list = mgr.getExtensionPointPlugins("one.two.three");
        assertEquals(0, list.size());
        list = mgr.getExtensionPointPlugins("four.five.six");
        assertEquals(0, list.size());
        list = mgr.getExtensionPointPlugins("seven.eight.nine");
        assertEquals(0, list.size());
        list = mgr.getExtensionPointPlugins("ten.eleven.twelve");
        assertEquals(0, list.size());

        mgr.loadPluginInformation();

        list = mgr.getExtensionPointPlugins("one.two.three");
        assertEquals(1, list.size());
        assertEquals(plugin1, list.get(0));
        list = mgr.getExtensionPointPlugins("four.five.six");
        assertEquals(2, list.size());
        assertTrue(list.contains(plugin1));
        assertTrue(list.contains(plugin2));
        list = mgr.getExtensionPointPlugins("seven.eight.nine");
        assertEquals(1, list.size());
        assertEquals(plugin2, list.get(0));
        list = mgr.getExtensionPointPlugins("ten.eleven.twelve");
        assertEquals(0, list.size());
    }

    //********************
    //*** end of tests ***
    //********************

    IPluginManager mgr;

    public PluginManagerTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        PluginID.reset();
        ConfigPluginLocatorFactory.assignConfigPluginLocator(new MockPluginLocator());
        mgr = PluginManagerFactory.newPluginManager();
        mgr.reset();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        PluginID.reset();
        ConfigPluginLocatorFactory.assignConfigPluginLocator(null);
    }

    private void assertSupports(boolean support, String name, String version) {
        assertEquals(
            support,
            mgr.satisfiesDependency(
                PluginDependency.getDependency(PluginID.getPluginID(name, version), null, false)));
    }
}
