package gov.va.med.hds.cd.mum.loader.test;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IPluginConstants;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.loader.IDesktopClassLoader;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.update.ApplicationLauncher;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.UpdateManager;
import gov.va.med.hds.cd.mum.update.UpdateManagerFactory;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import junit.framework.TestCase;

public class ClassLoaderManagerTest
    extends TestCase
    implements IClassLoaderTest, IMUMConstants, IBootConstants, IPluginConstants {

    public void testClassLoaderManager() throws Exception {
        testCount++;
        ClassLoaderManager.assignClassLoaders(info.getLoader(), plugins);
        assertClasses(0, utils.CLASSES[0]);
        assertClasses(0, utils.CLASSES[1]);
        assertNoClasses(0, utils.CLASSES[2]);

        assertNoClasses(1, utils.CLASSES[0]);
        assertClasses(1, utils.CLASSES[1]);
        assertClasses(1, utils.CLASSES[2]);

        assertNoClasses(2, utils.CLASSES[0]);
        assertNoClasses(2, utils.CLASSES[1]);
        assertClasses(2, utils.CLASSES[2]);

        assertClasses(3, utils.PUBLISHED[0]);
        assertClasses(3, utils.PUBLISHED[1]);
        assertClasses(3, utils.PUBLISHED[2]);
        assertNoClasses(3, utils.NON_PUBLISHED[0]);
        assertNoClasses(3, utils.NON_PUBLISHED[1]);
        assertNoClasses(3, utils.NON_PUBLISHED[2]);
    }

    public void testSharedLoader() throws Exception {
        testCount++;
        info.setArgs(new String[] { IBootConstants.LOAD_ALL_COMMAND_TAG });
        ClassLoaderManager.setForceSharedLoader(false);
        ClassLoaderManager.init(info.getArgs());
        startupSpec.setClassLoader(null);
        ClassLoaderManager.assignClassLoaders(info.getLoader(), plugins);
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                assertClasses(i, utils.CLASSES[j]);
            }
        }
    }

    //********************
    //*** end of tests ***
    //********************

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(ClassLoaderManagerTest.class);

    public ClassLoaderManagerTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        old = System.err;
        System.setErr(new PrintStream(new ByteArrayOutputStream()));
        if (testCount == 0) {
            TestFileBuilder.ensureJarDir();
        }
        ClassLoaderManager.resetSharedLoader();
        origional = ClassLoaderManager.getForcedSharedLoader();
        ClassLoaderManager.setForceSharedLoader(false);
        ClassLoaderManager.init(null);
        utils = new ClassLoaderUtils(this);
        utils.setNestedManager(utils);
        info = new TestStartupInfo();
        ITreeData xml = utils.getDocument(utils.getURL(utils.LOADER_CONFIG));
        int root = xml.getRootID();
        plugins = new ListMap();
        specs = new IPluginSpec[xml.getChildCount(root)];
        for (int i = 0; i < xml.getChildCount(root); i++) {
            int child = xml.getChildID(root, i);
            if (PLUGIN_ELEMENT.equals(xml.getName(child))) {
                specs[i] =
                    new TestPluginSpec(
                        xml,
                        child,
                        utils.getTestManfiestURL(i),
                        utils.getTestPluginFiles(i));
                plugins.add(specs[i].getID(), specs[i]);
                if (xml.findChildID(child, STARTUP_ELEMENT) >= 0) {
                    startupSpec = specs[i];
                    IDesktopClassLoader loader =
                        ClassLoaderManager.getClassLoader(this.getClass().getClassLoader());
                    info.setLoader((ClassLoader) loader);
                    loader.assignPlugin(startupSpec);
                }
            }
        }
        ((UpdateManager) UpdateManagerFactory.newUpdateManager()).findRequiredPlugins(plugins);
        (new ApplicationLauncher()).assignPluginDependencies(plugins, info);
    }

    PrintStream old;

    protected void tearDown() throws Exception {
        super.tearDown();
        System.setErr(old);
        ClassLoaderManager.setForceSharedLoader(origional);
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
    }

    private boolean origional;
    private TestStartupInfo info;
    private IPluginSpec startupSpec;
    private ListMap plugins;
    private IPluginSpec[] specs;
    private ClassLoaderUtils utils;

    public String getLoaderName(ClassLoader loader) {
        for (int i = 0; i < specs.length; i++) {
            if (specs[i].getClassLoader() == loader)
                return "Class loader for " + specs[i].getDirectory();
        }
        return "Unknown class loader ";
    }

    private void assertClasses(int idx, String[] classes) {
        utils.assertClasses((ClassLoader) specs[idx].getClassLoader(), classes);
    }

    private void assertNoClasses(int idx, String[] classes) {
        utils.assertNoClasses((ClassLoader) specs[idx].getClassLoader(), classes);
    }
}
