package gov.va.med.hds.cd.mum.loader.test;

import gov.va.med.hds.cd.mum.test.TestUtils;
import junit.framework.TestCase;

public class ClassLoaderUtils extends TestUtils {

    private TestCase tester;

    public ClassLoaderUtils(TestCase tester) {
        super(tester);
        if (!(tester instanceof IClassLoaderTest))
            throw new IllegalArgumentException();
        this.tester = tester;
    }

    private static String dpc = "DefaultPkgCls";
    private static String ctc1 = "mumtest.plugin.one.TestClassOne";
    private static String ctc2 = "mumtest.plugin.one.TestClassTwo";
    private static String ctc3 = "mumtest.plugin.one.TestClassThree";
    private static String ctc4 = "mumtest.plugin.one.TestClassFour";
    private static String cpi1 = "mumtest.plugin.one.published.PublicInterfaceOne";
    private static String cpc1 = "mumtest.plugin.one.published.PublicClassOne";
    private static String cpc2 = "mumtest.plugin.one.published.PublicClassTwo";
    private static String ctsc = "mumtest.plugin.one.TestStartupClass";
    private static String ctac = "mumtest.plugin.one.TestApplicationClass";

    private static String ctc7 = "mumtest.plugin.two.TestClassSeven";
    private static String ciad = "mumtest.plugin.two.whatever.IAppDescendant";

    private static String ctc5 = "mumtest.plugin.three.TestClassFive";
    private static String ctc6 = "mumtest.plugin.three.TestClassSix";
    private static String cti1 = "mumtest.plugin.three.TestInterfaceOne";
    private static String cti2 = "mumtest.plugin.three.TestInterfaceTwo";
    private static String cisd = "mumtest.plugin.three.IStartupDescendant";
    private static String capp = "mumtest.plugin.three.Application";

    public static String resourceOne = "mumtest/plugin/one/resources/test.xml";

    public static final String[][] CLASSES =
        { { ctc1, ctc2, ctc3, ctc4, cpi1, cpc1, cpc2, ctsc, ctac }, {
            ctc7, ciad }, {
            dpc, ctc5, ctc6, cti1, cti2, cisd, capp }, {
        }
    };

    public static final String[][] PUBLISHED = { { cpc1, cpc2, ctac, ctc3 }, {
            ctc7 }, {
            dpc, ctc5, ctc6, cti1, cti2, cisd, capp }, {
        }
    };

    public static final String[][] NON_PUBLISHED = { { ctc1, ctc2, ctc4, cpi1, ctsc }, {
            ciad }, {
        }, {
        }
    };

    public void assertNoClass(ClassLoader loader, String name) {
        try {
            loader.loadClass(name);
            tester.fail(
                ((IClassLoaderTest) tester).getLoaderName(loader)
                    + " should not have found class "
                    + name);
        } catch (ClassNotFoundException e) {
        }
    }

    public void assertClass(ClassLoader loader, String name) {
        boolean fail = false;
        try {
            fail = (loader.loadClass(name) == null);
        } catch (ClassNotFoundException e) {
            fail = true;
        }
        if (fail) {
            tester.fail(
                ((IClassLoaderTest) tester).getLoaderName(loader)
                    + " should have had class "
                    + name);
        }
    }

    public void assertClasses(ClassLoader loader, String[] classList) {
        for (int i = 0; i < classList.length; i++) {
            assertClass(loader, classList[i]);
        }
    }

    public void assertNoClasses(ClassLoader loader, String[] classList) {
        for (int i = 0; i < classList.length; i++) {
            assertNoClass(loader, classList[i]);
        }
    }

}
