package gov.va.med.hds.cd.mum.loader.test;

import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderException;
import gov.va.med.hds.cd.mum.boot.loader.DelegatingClassLoader;
import gov.va.med.hds.cd.mum.boot.loader.LinkedPluginLoader;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.test.WinUtils;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import junit.framework.TestCase;

public class LibraryTest extends TestCase implements IConfigConstants, IMUMTestConstants {

    public void testLib1()
        throws
            ClassNotFoundException,
            InstantiationException,
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            ClassLoaderException {
        testCount++;
        if (WinUtils.notWindows())
            return;
        addURL(jar);
        addURL(dll);

        //        Class cls = loader.loadClass(testLibClass);
        //        Object obj = cls.newInstance();
        //        String param = "testing 1..2..3..";
        //        Method method = cls.getMethod(methodName, new Class[] { param.getClass()});
        //        assertEquals(param, method.invoke(obj, new Object[] { param }));

        DelegatingClassLoader dlgLoader =
            new TestDelegatingClassLoader(this.getClass().getClassLoader().getSystemClassLoader());
        dlgLoader.addLoader(loader);
        Class cls = dlgLoader.loadClass(testLibClass);
        Object obj = cls.newInstance();
        String param = "testing 1..2..3..";
        Method method = cls.getMethod(methodName, new Class[] { param.getClass()});
        assertEquals(param, method.invoke(obj, new Object[] { param }));
    }

    //********************
    //*** end of tests ***
    //********************

    private String root = "gov/va/med/hds/cd/mum/loader/test/testdata/libtest/";
    private String jar = "TestJar.jar";
    private String dll = "simpledll.dll";
    private String testLibClass = "mumtest.dlltester.jni.simpledll.SimpleDll";
    private String methodName = "call";

    private LinkedPluginLoader loader;

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(LibraryTest.class);

    public LibraryTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.ensureJarDir();
        }
        utils = new TestUtils(this);
        utils.setNestedManager(utils);
        loader =
            new TestLinkedPluginLoader(this.getClass().getClassLoader().getSystemClassLoader());
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
    }

    private TestUtils utils;

    private void addURL(String fileName) throws ClassLoaderException {
        File file = utils.getFile(root + fileName);
        assertNotNull(file);
        MockDesktopPlugin plugin = new MockDesktopPlugin();
        plugin.addFile(file);
        loader.assignPlugin(plugin);
    }
}
