package gov.va.med.hds.cd.mum.loader.test;

import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderException;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.loader.DelegatingClassLoader;
import gov.va.med.hds.cd.mum.boot.loader.LinkedPluginLoader;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;

import junit.framework.TestCase;

public class LinkedClassLoaderTest extends TestCase implements IClassLoaderTest {

    public void testNoLinkage() throws ClassLoaderException {
        testCount++;
        initLoaders(differentModules, null);
        for (int i = 0; i < utils.CLASSES.length; i++) {
            utils.assertClasses(loaders[i], utils.CLASSES[i]);
        }

        for (int i = 0; i < utils.CLASSES.length; i++) {
            for (int j = 0; j < utils.CLASSES.length; j++) {
                if (i != j) {
                    utils.assertNoClasses(loaders[i], utils.CLASSES[j]);
                }
            }
        }
    }

    public void testModuleLink() throws ClassLoaderException {
        testCount++;
        initLoaders(sameModules, tree1);
        utils.assertClasses(loaders[0], utils.CLASSES[1]);
        utils.assertNoClasses(loaders[1], utils.CLASSES[0]);
        utils.assertClasses(loaders[3], utils.CLASSES[0]);
        utils.assertNoClasses(loaders[3], utils.CLASSES[1]);
    }

    public void testPublishedLink() throws ClassLoaderException {
        testCount++;
        initLoaders(differentModules, tree1);
        utils.assertClasses(loaders[0], utils.PUBLISHED[1]);
        utils.assertNoClasses(loaders[0], utils.NON_PUBLISHED[1]);
        utils.assertClasses(loaders[3], utils.PUBLISHED[0]);
        utils.assertNoClasses(loaders[3], utils.NON_PUBLISHED[0]);
        utils.assertNoClasses(loaders[3], utils.CLASSES[1]);
    }

    public void testDelegatingClassLoader() throws ClassLoaderException {
        testCount++;
        initLoaders(differentModules, null);
        DelegatingClassLoader loader =
            new TestDelegatingClassLoader(this.getClass().getClassLoader());
        for (int i = 0; i < utils.CLASSES.length; i++) {
            loader.addLoader(loaders[i]);
        }
        for (int i = 0; i < utils.CLASSES.length; i++) {
            utils.assertClasses(loader, utils.CLASSES[i]);
        }
    }

    public void testResourceLoad()
        throws ClassLoaderException, FileNotFoundException, TreeDataException {
        testCount++;
        initLoaders(differentModules, tree1);
        URL url = loaders[0].getResource(utils.resourceOne);
        assertNotNull(url);
        ILoadableTreeData xml = TreeDataFactory.getSimpleXMLTreeDataInstance();
        xml.load(url);
        assertTrue(xml.hasRoot());
        assertEquals("testing", xml.getName(xml.getRootID()));
        url = loaders[1].getResource(utils.resourceOne);
        assertNull(url);
        url = loaders[3].getResource(utils.resourceOne);
        assertNotNull(url);
        DelegatingClassLoader loader =
            new TestDelegatingClassLoader(this.getClass().getClassLoader());
        loader.addLoader(loaders[3]);
        url = loader.getResource(utils.resourceOne);
        assertNotNull(url);
        xml.clear();
        xml.load(url);
        assertTrue(xml.hasRoot());
        assertEquals("testing", xml.getName(xml.getRootID()));
    }

    //********************
    //*** end of tests ***
    //********************

    public LinkedClassLoaderTest(String arg0) {
        super(arg0);
    }

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(LinkedClassLoaderTest.class);

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.ensureJarDir();
        }
        ClassLoaderManager.resetSharedLoader();
        utils = new ClassLoaderUtils(this);
        utils.setNestedManager(utils);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
    }

    private ClassLoaderUtils utils;
    private LinkedPluginLoader[] loaders = new LinkedPluginLoader[4];
    private MockDesktopPlugin[] plugins = new MockDesktopPlugin[4];
    private String[] differentModules = { "A", "B", "C", "D" };
    private String[] sameModules = { "X", "X", "X", "X" };
    private int[][] tree1 = { { 1 }, {
        }, {
        }, {
            0 }
    };

    private void initLoaders(String[] mods, int[][] tree) throws ClassLoaderException {
        for (int i = 0; i < 4; i++) {
            initLoader(i, mods[i]);
        }
        if (tree != null) {
            for (int i = 0; i < tree.length; i++) {
                int[] links = tree[i];
                for (int j = 0; j < links.length; j++) {
                    plugins[i].getLinkedPlugins().add(Integer.toString(j), plugins[links[j]]);
                }
            }
            for (int i = 0; i < 4; i++)
                loaders[i].establishLink();
        }
    }

    private LinkedPluginLoader initLoader(int idx, String module) throws ClassLoaderException {
        LinkedPluginLoader loader = new TestLinkedPluginLoader(this.getClass().getClassLoader());
        loaders[idx] = loader;
        plugins[idx] = new MockDesktopPlugin();
        plugins[idx].setModule(module);
        String[] pubCls = utils.PUBLISHED[idx];
        for (int i = 0; i < pubCls.length; i++) {
            plugins[idx].getPublishedClasses().add(pubCls[i]);
        }
        File[] files = utils.getTestPluginFiles(idx);
        for (int i = 0; i < files.length; i++) {
            plugins[idx].addFile(files[i]);
        }
        loaders[idx].assignPlugin(plugins[idx]);
        return loader;
    }

    public String getLoaderName(ClassLoader loader) {
        String result = "Unknown Loader";
        for (int i = 0; i < loaders.length; i++) {
            if (loaders[i] == loader) {
                result = "loader" + Integer.toString(i);
                break;
            }
        }
        return result;
    }
}
