package gov.va.med.hds.cd.mum.loader.test;

import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.loader.FileLoader;
import gov.va.med.hds.cd.mum.boot.loader.IDataBuffer;
import gov.va.med.hds.cd.mum.boot.loader.IFileLoader;
import gov.va.med.hds.cd.mum.boot.loader.IFileLoaderFactory;
import gov.va.med.hds.cd.mum.boot.loader.SharedClassLoader;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.config.IConfigConstants;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.test.WinUtils;
import gov.va.med.hds.util.io.FileUtilities;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * enclosing_type
 */
public class SharedClassLoaderTest
    extends TestCase
    implements IFileLoaderFactory, IConfigConstants, IMUMTestConstants, IBootConstants {

    public void testInvalidClassLoader() throws FileNotFoundException {
        testCount++;
        try {
            (new TestSharedClassLoader(this.getClass().getClassLoader())).registerFile(null);
            fail("Should have thrown a NullPointerException");
        } catch (NullPointerException e) {
        }
        File file = new File("arweq/rweq/rqwe/rweq/rbfjdkfjdksla");
        try {
            (new TestSharedClassLoader(this.getClass().getClassLoader())).registerFile(file);
            fail("Should have thrown a FileNotFound exception");
        } catch (FileNotFoundException e) {
        }
    }

    public void testSingleJar() throws ClassNotFoundException, FileNotFoundException {
        testCount++;
        newLoader();
        loader.registerFile(jar1);
        assertEquals(1, fileLoaders.size());
        String[] classes = ClassLoaderUtils.CLASSES[0];
        for (int j = 0; j < classes.length; j++) {
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
        }
        classes = ClassLoaderUtils.CLASSES[1];
        for (int j = 0; j < classes.length; j++) {
            try {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                loader.loadClass(classes[j]);
                fail("Should not have found class " + classes[j]);
            } catch (ClassNotFoundException e) {
                assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            }
        }
    }

    public void testMultipleJarsAutoDetect() throws ClassNotFoundException, FileNotFoundException {
        testCount++;
        newLoader();
        loader.registerFile(jar1);
        loader.registerFile(jar2);
        assertEquals(2, fileLoaders.size());
        String[] classes = ClassLoaderUtils.CLASSES[0];
        for (int j = 0; j < classes.length; j++) {
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
        }
        classes = ClassLoaderUtils.CLASSES[2];
        for (int j = 0; j < classes.length; j++) {
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
        }
        classes = ClassLoaderUtils.CLASSES[1];
        for (int j = 0; j < classes.length; j++) {
            try {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
                loader.loadClass(classes[j]);
                fail("Should not have found class " + classes[j]);
            } catch (ClassNotFoundException e) {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            }
        }
    }

    public void testPackageLists() throws ClassNotFoundException, FileNotFoundException {
        testCount++;
        List packages = new ArrayList();
        newLoader();
        loader.registerFile(jar1, packages);
        assertEquals(1, fileLoaders.size());
        String[] classes = ClassLoaderUtils.CLASSES[0];
        for (int j = 0; j < classes.length; j++) {
            try {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                loader.loadClass(classes[j]);
                fail("Should not have found class " + classes[j]);
            } catch (ClassNotFoundException e) {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            }
        }

        newLoader();
        packages.add(pkg1);
        loader.registerFile(jar1, packages);
        assertEquals(1, fileLoaders.size());
        for (int j = 0; j < classes.length; j++) {
            int idx = classes[j].lastIndexOf('.');
            if (pkg1.equals(classes[j].substring(0, idx))) {
                Class cls = loader.loadClass(classes[j]);
                assertEquals(classes[j], cls.getName());
                assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            } else {
                try {
                    loader.loadClass(classes[j]);
                    fail("Should not have found class " + classes[j]);
                } catch (ClassNotFoundException e) {
                    assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                }
            }
        }
        newLoader();
        loader.registerFile(jar1, packages);
        packages.clear();
        packages.add(pkg1pub);
        loader.registerFile(jar1, packages);
        assertEquals(2, fileLoaders.size());
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            int idx = classes[j].lastIndexOf('.');
            if (pkg1.equals(classes[j].substring(0, idx))) {
                assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            } else {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            }
        }
        newLoader();
        packages.clear();
        packages.add(pkg2);
        packages.add(DEFAULT_PACKAGE);
        loader.registerFile(jar1, packages);
        loader.registerFile(jar2, packages);
        loader.registerFile(jar3, packages);
        assertEquals(3, fileLoaders.size());
        classes = ClassLoaderUtils.CLASSES[2];
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            assertFalse(((TestFileLoader) fileLoaders.get(2)).wasCalled());
        }
        newLoader();
        loader.registerFile(jar3, packages);
        loader.registerFile(jar1, packages);
        loader.registerFile(jar2, packages);
        assertEquals(3, fileLoaders.size());
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(2)).wasCalled());
        }
        newLoader();
        packages.clear();
        packages.add(pkg3);
        packages.add(pkg4);
        loader.registerFile(jar3, packages);
        loader.registerFile(jar4, packages);
        loader.registerFile(jar1, packages);
        loader.registerFile(jar2, packages);
        assertEquals(4, fileLoaders.size());
        classes = ClassLoaderUtils.CLASSES[1];
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            boolean inThree = (classes[j].indexOf("whatever") < 0);
            assertEquals(inThree, ((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(2)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(3)).wasCalled());
        }
    }

    public void testMixed() throws ClassNotFoundException, FileNotFoundException {
        testCount++;
        newLoader();
        loader.registerFile(jar1);
        assertEquals(1, fileLoaders.size());
        String[] classes = ClassLoaderUtils.CLASSES[0];
        for (int j = 0; j < classes.length; j++) {
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
        }
        classes = ClassLoaderUtils.CLASSES[2];
        for (int j = 0; j < classes.length; j++) {
            try {
                assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
                loader.loadClass(classes[j]);
                fail("Should not have found class " + classes[j]);
            } catch (ClassNotFoundException e) {
                assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            }
        }
        List packages = new ArrayList();
        packages.add(pkg2);
        packages.add(DEFAULT_PACKAGE);
        newLoader();
        loader.registerFile(jar1);
        loader.registerFile(jar2, packages);
        assertEquals(2, fileLoaders.size());
        classes = ClassLoaderUtils.CLASSES[0];
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertTrue(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertFalse(((TestFileLoader) fileLoaders.get(1)).wasCalled());
        }
        classes = ClassLoaderUtils.CLASSES[2];
        for (int j = 0; j < classes.length; j++) {
            Class cls = loader.loadClass(classes[j]);
            assertEquals(classes[j], cls.getName());
            assertFalse(((TestFileLoader) fileLoaders.get(0)).wasCalled());
            assertTrue(((TestFileLoader) fileLoaders.get(1)).wasCalled());
        }
    }

    public void testDLLLoad()
        throws
            ClassNotFoundException,
            InstantiationException,
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            MalformedURLException,
            FileNotFoundException {
        testCount++;
        if (WinUtils.notWindows())
            return;
        newLoader();
        loader.registerFile(dllJar);
        loader.registerFile(dll);
        Class cls = loader.loadClass(testLibClass);
        Object obj = cls.newInstance();
        String param = "testing 1..2..3..";
        Method method = cls.getMethod(dllMethodName, new Class[] { param.getClass()});
        assertEquals(param, method.invoke(obj, new Object[] { param }));
        URL url = loader.getResource(dll.toURL().getPath());
        assertNotNull(url);
        assertEquals(url.toString(), dll.toURL().toString());
    }

    public void testResourceLocation()
        throws MalformedURLException, FileNotFoundException, TreeDataException {
        testCount++;
        List packages = new ArrayList();
        packages.add(pkg1);
        packages.add(pkg1pub);
        packages.add(pkg1res);
        packages.add(DEFAULT_PACKAGE);
        newLoader();
        loader.registerFile(jar2);
        URL url = loader.getResource("mumtest/plugin/three/TestClassFive.class");
        assertNotNull(url);
        url = loader.getResource("DefaultPkgCls.class");
        assertNotNull(url);
        url = loader.getResource("DummyResource.txt");
        assertNotNull(url);
        loader.registerFile(jar1, packages);
        url = loader.getResource("mumtest/plugin/one/TestClassOne.class");
        assertNotNull(url);
        url = loader.getResource("mumtest/plugin/one/resources/test.xml");
        assertNotNull(url);
        ILoadableTreeData data = TreeDataFactory.newSimpleXMLTreeData();
        data.load(url);
        int root = data.getRootID();
        assertEquals("testing", data.getName(root));
        assertEquals(3, data.getChildCount(root));
        assertEquals("one", data.getName(data.getChildID(root, 0)));
        assertEquals("two", data.getName(data.getChildID(root, 1)));
        assertEquals("three", data.getName(data.getChildID(root, 2)));
        url = loader.getResource("mumtest/plugin/three/TestClassSix.class");
        assertNotNull(url);
        loader.registerFile(jar2, packages);
        loader.registerFile(manifest);
        url = loader.getResource(manifest.toURL().getPath());
        assertNotNull(url);
        assertEquals(manifest.toURL().toString(), url.toString());
        data = TreeDataFactory.newSimpleXMLTreeData();
        data.load(url);
        root = data.getRootID();
        assertEquals("plugin", data.getName(root));
        assertEquals(2, data.getChildCount(root));
        assertEquals("prerequisites", data.getName(data.getChildID(root, 0)));
        assertEquals("extension-points", data.getName(data.getChildID(root, 1)));
    }

    //********************
    //*** end of tests ***
    //********************

    private String pkg1 = "mumtest.plugin.one";
    private String pkg1pub = "mumtest.plugin.one.published";
    private String pkg1res = "mumtest.plugin.one.resources";
    private String pkg2 = "mumtest.plugin.three";
    private String pkg3 = "mumtest.plugin.two";
    private String pkg4 = "mumtest.plugin.two.whatever";

    /**
     * Constructor for FileClassLoaderTest.
     * @param arg0
     */
    public SharedClassLoaderTest(String arg0) {
        super(arg0);
    }

    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(SharedClassLoaderTest.class);

    private String root = "gov/va/med/hds/cd/mum/";
    private String rootDir = root + "config/testdata/plugins/";
    private String testLibClass = "mumtest.dlltester.jni.simpledll.SimpleDll";
    private String libRoot = root + "loader/test/testdata/libtest/";
    private String dllName = "simpledll.dll";
    private String dllJarName = "TestJar.jar";
    private String dllJarPath = libRoot + dllJarName;
    private String dllPath = libRoot + dllName;
    private String dllMethodName = "call";
    private String manifestPath = root + "config/testdata/manifests/manifest1.xml";

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.ensureJarDir();
        }
        ClassLoaderManager.resetSharedLoader();
        jar1 = new File(TestFileBuilder.getJarDir(), "sortedA.jar");
        jar2 = new File(TestFileBuilder.getJarDir(), "sortedB.jar");
        jar3 = new File(TestFileBuilder.getJarDir(), "sortedD.jar");
        jar4 = new File(TestFileBuilder.getJarDir(), "sortedC.jar");
        dll = new File(TestFileBuilder.getJarDir(), dllName);
        dllJar = new File(TestFileBuilder.getJarDir(), dllJarName);
        manifest = new File(TestFileBuilder.getJarDir(), MANIFEST_FILE);
        if (testCount == 0) {
            ClassLoader loader = getClass().getClassLoader();
            URL url = loader.getResource(rootDir + "one/one.jar");
            FileUtilities.copy(url, jar1);
            url = loader.getResource(rootDir + "three/three.jar");
            FileUtilities.copy(url, jar2);
            url = loader.getResource(rootDir + "two/two7.jar");
            FileUtilities.copy(url, jar3);
            url = loader.getResource(rootDir + "two/two8.jar");
            FileUtilities.copy(url, jar4);
            url = loader.getResource(dllPath);
            FileUtilities.copy(url, dll);
            url = loader.getResource(dllJarPath);
            FileUtilities.copy(url, dllJar);
            url = loader.getResource(manifestPath);
            FileUtilities.copy(url, manifest);
            jar1.deleteOnExit();
            jar2.deleteOnExit();
            jar3.deleteOnExit();
            jar4.deleteOnExit();
            dll.deleteOnExit();
            dllJar.deleteOnExit();
            manifest.deleteOnExit();
        }
        fileLoaders = new ArrayList();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if (loader != null)
            loader.dispose();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
    }

    private File jar1;
    private File jar2;
    private File jar3;
    private File jar4;
    private File dll;
    private File dllJar;
    private File manifest;
    private List fileLoaders;

    private static class TestFileLoader extends FileLoader {
        public TestFileLoader(File file) {
            super(file);
        }

        public IDataBuffer getData(String name) throws IOException {
            called = true;
            return super.getData(name);
        }

        private boolean called;

        boolean wasCalled() {
            boolean result = called;
            called = false;
            return result;
        }
    }

    private SharedClassLoader loader;

    private void newLoader() {
        if (loader != null)
            loader.dispose();
        loader = new TestSharedClassLoader(this.getClass().getClassLoader(), this);
        fileLoaders.clear();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IFileLoaderFactory#getNewLoader(java.io.File)
     */
    public IFileLoader getNewLoader(File file) {
        IFileLoader loader = new TestFileLoader(file);
        fileLoaders.add(loader);
        return loader;
    }

}
