package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.utils.ChecksumCalculator;

import java.io.ByteArrayInputStream;
import java.net.URL;

import junit.framework.TestCase;

public class ChecksumCalculatorTest extends TestCase {

	public void testCRCUpdate() {
		assertEquals(0, calc.getValue());
		calc.update("12345");
		assertEquals(2621456L, calc.getValue());
		calc.update("Testing 1..2..3..4..5..");
		assertEquals(1762200497L, calc.getValue());

		calc.reset();
		assertEquals(0, calc.getValue());

		calc.update("Testing 1..2..3..4..5..");
		assertEquals(1736969122L, calc.getValue());
		calc.update("12345");
		assertEquals(2715093937L, calc.getValue());
	}

	public void testFileCalc() {
		calcFile(testJar2);
		assertEquals(3569448340L, calc.getValue());

		calcFile(manifest);
		assertEquals(3622548631L, calc.getValue());

		calcFile(dll);
		assertEquals(1765739385L, calc.getValue());
	}

	public void testExcludeCRCCalculation() {
		long value = getConfigFileCRCFor(crcStream1);
		assertEquals(value, getConfigFileCRCFor(crcStream2));
		assertEquals(value, getConfigFileCRCFor(crcStream3));
		assertEquals(value, getConfigFileCRCFor(crcStream4));
		assertEquals(value, getNormalCRCFor(crcStream1));
		assertTrue(getNormalCRCFor(crcStream2) != value);
		assertTrue(getNormalCRCFor(crcStream3) != value);
		assertTrue(getNormalCRCFor(crcStream4) != value);
	}
	
	//********************
	//*** end of tests ***
	//********************

	public ChecksumCalculatorTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
		calc = new ChecksumCalculator();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	ChecksumCalculator calc;

	private String testJar2 =
		"gov/va/med/hds/cd/mum/config/testdata/TestJar2.jar";

	private String manifest =
		"gov/va/med/hds/cd/mum/config/testdata/manifests/manifest1.xml";

	private String dll = "gov/va/med/hds/cd/mum/loader/test/testdata/libtest/simpledll.dll";

	private void calcFile(String file) {
		URL url = this.getClass().getClassLoader().getResource(file);
		assertNotNull("Should have found " + file, url);
		calc.calcCRC(url);
	}

	private String crcStreamStart = "<This is a test crc=\"";
	private String crcStreamEnd = "\" I hope it works! crc=\"12345\">";
	private String crcStream1 = crcStreamStart + crcStreamEnd;
	private String crcStream2 = crcStreamStart + "47238147" + crcStreamEnd;
	private String crcStream3 = crcStreamStart + "0" + crcStreamEnd;
	private String crcStream4 =
		crcStreamStart + "Something else to ignore" + crcStreamEnd;

	private long getConfigFileCRCFor(String name) {
		ByteArrayInputStream in = new ByteArrayInputStream(name.getBytes());
		calc.calcConfigFile(in);
		return calc.getValue();
	}

	private long getNormalCRCFor(String name) {
		calc.reset();
		calc.update(name);
		return calc.getValue();
	}
}
