package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.boot.ExceptionDialog;

import java.awt.Window;

import javax.swing.JButton;
import javax.swing.JTextArea;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.JFCTestHelper;
import junit.extensions.jfcunit.TestHelper;
import junit.extensions.jfcunit.eventdata.MouseEventData;

public class ExceptionDialogTest extends JFCTestCase {
	private static final String SHORT_MESSAGE = "This is a single sentence short message.";
	private static final String VERY_SHORT_MESSAGE = "Very short";
	private static final String DIALOG_NAME = "Test Box";
	private static final String LONG_MESSAGE = 
	"This is the beginning of a long message. "+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the continuation of a long mesage."+
	"This is the end of a long message";	

	private Thread modalThread = null;
	private Exception lastException = null;
	private TestHelper helper = null;
	private boolean normalExit = false;
	private volatile boolean started = false;
	private Window win = null;
	static public void main(String[] args){
		Exception e = new Exception(LONG_MESSAGE);
		ExceptionDialog.showExceptionDialog(DIALOG_NAME, e.getMessage(), e);
	}
	public ExceptionDialogTest(String name) {
		super(name);
	}
	protected void setUp() throws Exception {
		super.setUp();
		helper = new JFCTestHelper();
		lastException = null;
	}
	
	public void testVeryShortMessage() throws Exception{
		win = createDialog(VERY_SHORT_MESSAGE);
		assertNotNull("Dialog "+DIALOG_NAME+" not found:", win);
		
		JTextArea field = (JTextArea) TestHelper.findComponent(JTextArea.class, win, 0);
		assertNotNull("Could not find message field", field);

		assertEquals("message field text was wrong",field.getText(),VERY_SHORT_MESSAGE);
	}
	public void testShortMessage() throws Exception{
		win = createDialog(SHORT_MESSAGE);
		assertNotNull("Dialog "+DIALOG_NAME+" not found:", win);

		JTextArea field = (JTextArea) TestHelper.findComponent(JTextArea.class, win, 0);
		assertNotNull("Could not find message field", field);

		assertEquals("message field text was wrong",field.getText(),SHORT_MESSAGE);
	}
	public void testLongerMessage() throws Exception{
		win = createDialog(LONG_MESSAGE);
		assertNotNull("Dialog "+DIALOG_NAME+" not found:", win);

		JTextArea field = (JTextArea) TestHelper.findComponent(JTextArea.class, win, 0);
		assertNotNull("Could not find message field", field);

		assertEquals("message field text was wrong",field.getText(),LONG_MESSAGE);
	}
	public void testDetails() throws Exception{
		win = createDialog(VERY_SHORT_MESSAGE);
		assertNotNull("Dialog "+DIALOG_NAME+" not found:", win);
		
		JButton details = ( JButton ) JFCTestHelper.findNamedComponent(JButton.class, "Details", win, 0 );
		assertNotNull( "Could not find the Details button", details );

		// Open details
		helper.enterClickAndLeave(new MouseEventData(this,details));
		// Close details
		helper.enterClickAndLeave(new MouseEventData(this,details));
	}
	protected void tearDown() throws Exception {
		JButton okButton = ( JButton ) JFCTestHelper.findNamedComponent(JButton.class, "OK", win, 0 );
		assertNotNull( "Could not find the OK button", okButton );
		
		helper.enterClickAndLeave(new MouseEventData(this,okButton));

		if (lastException != null) {
			throw lastException;
		}
		if (modalThread.isAlive()) {
			modalThread.interrupt();
		}
		modalThread = null;
		helper = null;
		JFCTestHelper.cleanUp( this );
		super.tearDown();
	}
	private void checkException() throws Exception {
		if (lastException != null) {
			throw lastException;
		}
	}
	private Window createDialog(String message) throws InterruptedException, Exception {
		modalThread = new ModalThread("Modal Thread",new Exception(message));
		modalThread.start();
		Thread.yield();
		
		// Wait for the thread to start
		while (!started) {Thread.sleep(50);}
		
		// Give a little extra time for the painting/construction
		Thread.sleep(2000);
		flushAWT();
		checkException();
		
		Window[] wins = TestHelper.getAllWindows();
		Window win = null;
		for(int i=0;i<wins.length;i++){
			if(wins[i].getName().equals(DIALOG_NAME)){
				win = wins[i];
				break;
			}
		}
		return win;
	}

	class ModalThread extends Thread{
		private Exception exception;
		public ModalThread(String threadName,Exception e){
			this.setName(threadName);
			this.exception = e;
		}
		public void run(){
			try {
				lastException = null;
				started = true;
				ExceptionDialog.showExceptionDialog(DIALOG_NAME, exception.getMessage(), exception);
				normalExit = true;
			} catch (Exception e) {
				lastException = e;
			}
		}
	}
}
