package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.config.ClassDataCache;
import gov.va.med.hds.cd.mum.config.ClassFileAnalyzer;
import gov.va.med.hds.cd.mum.config.ConfigPlugin;
import gov.va.med.hds.cd.mum.config.ConfigPluginLocatorFactory;
import gov.va.med.hds.cd.mum.config.Configuration;
import gov.va.med.hds.cd.mum.config.IConfigPluginLocator;
import gov.va.med.hds.cd.mum.config.IPluginManager;
import gov.va.med.hds.cd.mum.config.PluginLocatorException;
import gov.va.med.hds.cd.mum.config.PluginManagerFactory;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

public class ManifestValidationTest
    extends TestCase
    implements IConfigPluginLocator, IMUMConstants {

    private String[] roots = { "modules", "lib" };

    public void testValidConfiguration() throws PluginLocatorException, IOException {
        String ok = System.getProperty("MUMIgnoreTest");
        if ("ignore".equals(ok))
            return;
        assignRootDirectory();
        if (root == null)
            return;
        IPluginManager manager = PluginManagerFactory.newPluginManager();
        Configuration config = new Configuration(manager);
        cache.save();
        if (config.errors().hasErrors()) {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(out);
            config.errors().printErrors(ps);
            ps.println();
            ps.println("Plugin's found");
            for (int i = 0; i < plugins.size(); i++) {
                ps.println(plugins.get(i));
            }
            fail(out.toString());
        }
    }

    //********************
    //*** end of tests ***
    //********************

    /**
     * Constructor for ManifestValidationTest.
     * @param arg0
     */
    public ManifestValidationTest(String arg0) {
        super(arg0);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        plugins = new ArrayList();
        ConfigPluginLocatorFactory.assignConfigPluginLocator(this);
        ClassFileAnalyzer.SetUseIgnoreInterfaces(true);
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        ConfigPluginLocatorFactory.assignConfigPluginLocator(null);
    }

    private File root;
    private List plugins;
    private ClassDataCache cache;

    private void assignRootDirectory() {
        URL url =
            this.getClass().getClassLoader().getResource(
                this.getClass().getName().replace('.', '/') + ".class");
        if (url.getProtocol().equals("jar"))
            return;
        StringBuffer buf = new StringBuffer(url.getPath());
        for (int i = 0; i < 12; i++) {
            int idx = buf.lastIndexOf("/");
            if (idx >= 0)
                buf.setLength(idx);
            if (i == 0)
                cache = new ClassDataCache(new File(buf.toString()));
        }
        root = new File(buf.toString());
    }

    /**
     * @see gov.va.med.hds.cd.mum.config.IConfigPluginLocator#createDistributionPlugins(java.util.List)
     */
    public void createDistributionPlugins(ListMap list) throws PluginLocatorException {
        for (int i = 0; i < roots.length; i++) {
            File file = new File(root, roots[i]);
            if (file.exists() && file.isDirectory()) {
                searchDirectory(list, file, 0);
            }
        }
    }

    private void searchDirectory(ListMap list, File file, int count)
        throws PluginLocatorException {
        if (count > 2)
            return;
        File[] files = file.listFiles();
        boolean notFound = true;
        for (int i = 0; i < files.length; i++) {
            String fileName = files[i].getName();
            if (fileName.equals(IBootConstants.MANIFEST_FILE) || fileName.equals("module.xml")) {
                notFound = false;
                boolean parseDir = fileName.equals(IBootConstants.MANIFEST_FILE);
                try {
                    plugins.add(files[i].toString());
                    ConfigPlugin plugin = new ConfigPlugin(cache, files[i], parseDir);
                    list.add(plugin.getID(), plugin);
                } catch (InvalidPluginException e) {
                    throw new PluginLocatorException(e);
                }
            }
        }
        if (notFound) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory())
                    searchDirectory(list, files[i], count + 1);
            }
        }
    }
}
