package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.PluginID;
import gov.va.med.hds.cd.mum.Version;
import junit.framework.TestCase;

/**
 * enclosing_type
 */
public class PluginIDTest extends TestCase {

	public void testNewPluginID() {
		id = PluginID.getPluginID("Testing", "1.2.3.4");
		assertEquals("Testing v1.2.3.4", id.toString());
		assertEquals("Testing", id.getID());
		assertEquals("1.2.3.4", id.getVersion().toString());
		id2 = PluginID.getPluginID("Testing", "1.2.3.4");
		assertTrue(id == id2);
		id2 = PluginID.getPluginID("Testing", "1.2.0");
		assertTrue(id != id2);
		id = PluginID.getPluginID("Testing", "1.2");
		assertTrue(id != id2);

		Version ver = new Version("2.3.4");
		id = PluginID.getPluginID("Testing", ver);
		id2 = PluginID.getPluginID("Testing", "2.3.4");
		assertTrue(id == id2);
		assertTrue(ver == id2.getVersion());
		
		ver = new Version("3.4.0");
		id = PluginID.getPluginID("Testing", "3.4.0");
		id2 = PluginID.getPluginID("Testing", ver);
		assertTrue(id == id2);
		assertTrue(ver != id2.getVersion());
	}

	public void testErrors() {
		assertError(null, null);
		assertError(null, "1.2.3");
		assertError("whatever", null);
		assertError("", "");
		assertError("abc", "");
		assertError("", "123");
		assertError("abc", "x");
		assertError("abc", "1.x");
		assertError("abc", "1.2.x");
		assertError("abc", ".");
		assertError("abc", ".1");
		assertError("abc", "1.");
		assertError("abc", "1.2.");
	}

	//********************
	//*** end of tests ***
	//********************

	private void assertError(String name, String version) {
		try {
			PluginID.getPluginID(name, version);
			fail("Should have thrown an exception");
		} catch (Exception e) {
		}
	}
	/**
	 * Constructor for PluginIDTest.
	 * @param arg0
	 */
	public PluginIDTest(String arg0) {
		super(arg0);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		PluginID.reset();
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
		PluginID.reset();
	}

	private PluginID id;
	private PluginID id2;
}
