package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.util.io.CopyFailedException;
import gov.va.med.hds.util.io.FileUtilities;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.jar.JarFile;
import java.util.jar.JarOutputStream;
import java.util.zip.ZipEntry;

/**
 * @author vhaislmerrij
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class TestFileBuilder implements IMUMTestConstants {

	public static final long forcedDate = 1043616048000L;
	private static URL goldURL;
	private static URL destURL;
	private static File root, gold, dest;
	private static final String tempFilePrefix = "temp mum ";
	private static final String tempFileSuffix = " test.1789123";
	private static final String USER_HOME_DIR_PROPERTY = "user.home";
	private static final String testPlugin = "test.plugin_1.2.3";

	private static String jarDir = "testJars";
	private static File jarFile;

	private static String testDataDir = "gov/va/med/hds/cd/mum/test/testdata/";
	private static String realDataDir = "mumtest/mum/";
	private static String classExt = ".class";
	private static String clsExt = ".cls";

	public static void initRootTestDirs() {
		try {
			File dir = new File(System.getProperty(USER_HOME_DIR_PROPERTY));
			root = dir.createTempFile(tempFilePrefix, tempFileSuffix, dir);
			root.delete();
			root.mkdirs();
			root.deleteOnExit();
			gold = new File(root.toString() + File.separator + "Gold");
			gold.mkdir();
			gold.deleteOnExit();
			dest = new File(root.toString() + File.separator + "PC");
			dest.mkdir();
			dest.deleteOnExit();
			goldURL = gold.toURL();
			destURL = dest.toURL();
		} catch (IOException e) {
			System.out.println("ERROR: " + e.getMessage());
		}
	}

	public static void eraseTestDirectories(boolean all) {
		writeDummy = true;
		if (!all)
			System.out.println(
				"WARNING: MUM test not erasing all test directories");
		System.gc();
		if (root != null) {
			if (all) {
				deleteFiles(root);
				root.delete();
			}
			File parent = root.getParentFile();
			if (parent != null) {
				File[] files = parent.listFiles();
				for (int i = 0; i < files.length; i++) {
					File file = files[i];
					if (file.isDirectory()
						&& (!file.getPath().equals(root.getPath()))
						&& file.getName().startsWith(tempFilePrefix)
						&& file.getName().endsWith(tempFileSuffix)) {
						deleteFiles(file);
						file.delete();
					}
				}
			}
			root = null;
			goldURL = null;
			destURL = null;
			gold = null;
			dest = null;
		}
	}

	private static void deleteFiles(File dir) {
		File[] files = dir.listFiles();
		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory())
				deleteFiles(files[i]);
			files[i].delete();
		}
	}

	public static int numTests(Class cls) {
		Method[] methods = cls.getMethods();
		int count = 0;
		for (int i = 0; i < methods.length; i++) {
			if (methods[i].getName().startsWith("test"))
				count++;
		}
		return count;
	}

	public static File createTestJar(
		boolean goldDir,
		String name,
		String[] classes,
		Class startClass,
		String[] resources,
		String[] names)
		throws FileNotFoundException, IOException, ClassNotFoundException {
		File root;
		if (goldDir)
			root = gold;
		else
			root = dest;
		File result =
			new File(
				root.toString()
					+ File.separator
					+ PLUGINS_DIRECTORY
					+ File.separator
					+ testPlugin
					+ File.separator
					+ name);
		File parent = result.getParentFile();
		parent.mkdirs();
		parent.deleteOnExit();
		JarOutputStream jar =
			new JarOutputStream(
				new BufferedOutputStream(new FileOutputStream(result)));
		BufferedOutputStream out;
		ZipEntry entry;

		entry = new ZipEntry(JarFile.MANIFEST_NAME);
		jar.putNextEntry(entry);
		OutputStreamWriter writer = new OutputStreamWriter(jar);
		writer.write("Manifest-Version: 1.0");
		writer.write(WIN_EOL);
		writer.flush();
		entry.setTime(forcedDate);
		jar.closeEntry();

		byte[] buf = new byte[1024];
		int start;
		if (startClass != null)
			start = -1;
		else
			start = 0;
		if (classes != null) {
			for (int i = start; i < classes.length; i++) {
				Class cls;
				if (i == -1)
					cls = startClass;
				else
					cls = TestFileBuilder.class.getClassLoader().loadClass("gov.va.med.hds.cd." + classes[i]);
				String className = cls.getName().replace('.', '/') + ".class";
				URL url = cls.getClassLoader().getResource(className);
				entry = new ZipEntry(className);
				jar.putNextEntry(entry);
				out = new BufferedOutputStream(jar);
				InputStream in = url.openConnection().getInputStream();
				int bytesRead;
				while ((bytesRead = in.read(buf)) != -1)
					out.write(buf, 0, bytesRead);
				out.flush();
				entry.setTime(forcedDate);
				jar.closeEntry();
			}
		}
		if ((resources != null)
			&& (names != null)
			&& (resources.length == names.length)) {
			for (int i = 0; i < resources.length; i++) {
				URL url =
					TestFileBuilder.class.getClassLoader().getResource(
						resources[i]);
				entry = new ZipEntry(names[i].replace('.', '/') + ".class");
				jar.putNextEntry(entry);
				out = new BufferedOutputStream(jar);
				InputStream in = url.openStream();
				try {
					int bytesRead;
					while ((bytesRead = in.read(buf)) != -1)
						out.write(buf, 0, bytesRead);
				} finally {
					in.close();
				}
				out.flush();
				entry.setTime(forcedDate);
				jar.closeEntry();
			}
		}
		jar.close();
		result.setLastModified(forcedDate);
		result.deleteOnExit();
		return result;
	}

	public static File getGoldFile() {
		return gold;
	}

	public static File getDestFile() {
		return dest;
	}

	public static URL getGoldURL() {
		return goldURL;
	}

	public static URL getDestURL() {
		return destURL;
	}

	public static void ensureTestDirectories(boolean doDest) {
		initRootTestDirs();
		buildJars(getGoldFile(), ALL_PLUGINS, true);
		if (doDest)
			buildJars(getDestFile(), EXISTING_CONFIG, false);
	}

	public static void ensureJarDir() {
		initRootTestDirs();
		jarFile = new File(getDestFile().getParent(), jarDir);
		jarFile.mkdirs();
		jarFile.deleteOnExit();
	}

	public static File getJarDir() {
		return jarFile;
	}

	private static void buildJars(
		File base,
		int[] config,
		boolean stripVersion) {
		boolean isStartup = false;
		boolean isApplication = false;
		File pluginDir = new File(base, PLUGIN_DIRECTORY);
		for (int i = 0; i < config.length; i++) {
			Object[] data = MOCKJAR_DATA[config[i]];
			isStartup = false;
			for (int j = 0; j < STARTUP_PACKAGES.length; j++) {
				if (config[i] == STARTUP_PACKAGES[j]) {
					isStartup = true;
					break;
				}
			}
			isApplication = false;
			for (int j = 0; j < APPLICATION_PACKAGES.length; j++) {
				if (config[i] == APPLICATION_PACKAGES[j]) {
					isApplication = true;
					break;
				}
			}
			String temp = (String) data[miDir];
			int idx = temp.indexOf(VERSION_SEPARATOR);
			String name = temp.substring(0, idx);
			Version version = new Version(temp.substring(idx + 1));
			File root;
			if (stripVersion)
				root = new File(pluginDir, (String) data[miGoldDir]);
			else
				root = new File(pluginDir, name + "_" + version.toString());
			buildPlugin(root, temp, data, isStartup, isApplication);
		}
	}

	private static void buildPlugin(
		File root,
		String plugin,
		Object[] data,
		boolean isStartup,
		boolean isApplication) {
		root.mkdirs();
		File manifest = new File(root, MANIFEST_FILE);
		FileWriter writer = null;
		String jarName;
		if (isStartup) {
			jarName = STARTUP_JAR;
			try {
				File boot = new File(root, BOOT_JAR);
				boot.createNewFile();
				boot.deleteOnExit();
			} catch (IOException e) {
			}
		} else
			jarName =
				"tempjar_" + plugin.substring(0, plugin.indexOf("_")) + ".jar";
		try {
			String module = (String) data[miModule];
			if (module.length() > 0)
				module = " " + MODULE_ATTRIBUTE + "=\"" + module + "\"";
			writer = new FileWriter(manifest);
			writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			writer.write(WIN_EOL);
			writer.write(toXMLFormat(plugin, false, module));
			String[] requirements = (String[]) data[miRequiredPlugins];
			if (requirements.length > 0) {
				writer.write("  <" + PREREQUISITES_ELEMENT + ">" + WIN_EOL);
				for (int i = 0; i < requirements.length; i++) {
					writer.write("    ");
					writer.write(toXMLFormat(requirements[i], true, ""));
				}
				writer.write("  </" + PREREQUISITES_ELEMENT + ">" + WIN_EOL);
			}
			writer.write("  <" + DISTRIBUTION_ELEMENT + ">" + WIN_EOL);
			if (isStartup)
				writer.write(
					"    <"
						+ FILE_ELEMENT
						+ " "
						+ FILENAME_ATTRIBUTE
						+ "=\""
						+ BOOT_JAR
						+ "\"/>"
						+ WIN_EOL);
			writer.write(
				"    <"
					+ FILE_ELEMENT
					+ " "
					+ FILENAME_ATTRIBUTE
					+ "=\""
					+ jarName
					+ "\"/>"
					+ WIN_EOL);
			writer.write("  </" + DISTRIBUTION_ELEMENT + ">" + WIN_EOL);
			String[] published = (String[]) data[miPublishedStatements];
			if (published.length > 0) {
				writer.write("  <" + PUBLISHED_ELEMENT + ">" + WIN_EOL);
				for (int i = 0; i < published.length; i++)
					writer.write(
						"    <"
							+ PUBLISH_ELEMENT
							+ " "
							+ CLASS_ATTRIBUTE
							+ "=\""
							+ published[i]
							+ "\"/>"
							+ WIN_EOL);
				writer.write("  </" + PUBLISHED_ELEMENT + ">" + WIN_EOL);
			}
			writer.write("</" + PLUGIN_ELEMENT + ">" + WIN_EOL);
		} catch (IOException e) {
		} finally {
			try {
				writer.close();
				manifest.deleteOnExit();
			} catch (IOException e) {
			}
		}

		String[] pkgs = (String[]) data[miPackages];
		String[] reqPkgs = (String[]) data[miRequiredPackages];
		String[] additional = (String[]) data[miAdditionalClasses];
		File jarFile = new File(root, jarName);
		JarOutputStream jar = null;
		byte[] b4 = new byte[4];
		byte[] b2 = new byte[2];
		try {
			jar =
				new JarOutputStream(
					new BufferedOutputStream(new FileOutputStream(jarFile)));
			DataOutputStream out;
			ZipEntry entry;
			int cnt = pkgs.length + additional.length;
			int adlCnt = 0;
			for (int i = 0; i < cnt; i++) {
				String clsName;
				String pkg;
				if (i < pkgs.length) {
					clsName = "DummyClass";
					entry =
						new ZipEntry(
							pkgs[i].replace('.', '/')
								+ "/"
								+ clsName
								+ ".class");
					pkg = pkgs[i];
				} else {
					int tmp = additional[adlCnt].lastIndexOf('.');
					if (tmp >= 0) {
						clsName = additional[adlCnt].substring(tmp + 1);
						pkg = additional[adlCnt].substring(0, tmp);
					} else {
						clsName = additional[adlCnt];
						pkg = "";
					}
					entry =
						new ZipEntry(
							additional[adlCnt].replace('.', '/')
								+ "/"
								+ clsName
								+ ".class");
					adlCnt++;
				}
				jar.putNextEntry(entry);
				out = new DataOutputStream(jar);
				b4[0] = (byte) 0xCA;
				b4[1] = (byte) 0xFE;
				b4[2] = (byte) 0xBA;
				b4[3] = (byte) 0xBE;
				out.write(b4); // magic (class file signature)
				out.write(b4); // version
				int count = 1;
				if (i == 0)
					count += reqPkgs.length;
				out.writeShort((count * 2) + 1);
				for (int j = 0; j < count; j++) {
					out.write((byte) 1);
					if (j == 0) {
						writePackage(out, clsName, pkg);
					} else {
						writePackage(out, "DummyClass", reqPkgs[j - 1]);
					}
					out.write((byte) 7);
					out.writeShort(j * 2 + 1);
				}
				b2[0] = 0;
				b2[1] = 0;
				out.write(b2);
				out.writeShort(1);
				out.write(b2);
				out.writeShort(0);
				out.flush();
				entry.setTime(forcedDate);
				jar.closeEntry();
			}
			if (isStartup) {
				if (plugin.equals(MOCKJAR_DATA[mjEditor3][miDir]))
					writeTestFile(jar, "AnotherTestStartup");
				else
					writeTestFile(jar, "TestStartup");
			}
			if (isApplication) {
				writeTestFile(jar, "TestApplication");
			}
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		} finally {
			if (jar != null)
				try {
					jar.close();
					jarFile.setLastModified(forcedDate);
					jarFile.deleteOnExit();
				} catch (IOException e) {
				}
		}
	}

	private static void writePackage(
		DataOutputStream out,
		String clsName,
		String pkg)
		throws IOException {
		out.writeUTF(pkg.replace('.', '/') + "/" + clsName);
	}

	private static void writeDataToJar(
		JarOutputStream jar,
		String className,
		URL url)
		throws IOException {
		BufferedInputStream in = new BufferedInputStream(url.openStream());
		ZipEntry entry = new ZipEntry(className);
		jar.putNextEntry(entry);
		BufferedOutputStream bout = new BufferedOutputStream(jar);
		byte[] buf = new byte[1024];
		int i = 0;
		while ((i = in.read(buf)) != -1)
			bout.write(buf, 0, i);
		bout.flush();
		entry.setTime(forcedDate);
		jar.closeEntry();
	}

	private static void writeTestFile(JarOutputStream jar, String fileName)
		throws IOException {
		URL url =
			TestFileBuilder.class.getClassLoader().getResource(
				testDataDir + fileName + clsExt);
		writeDataToJar(jar, realDataDir + fileName + classExt, url);
	}

	private static boolean writeDummy = true;

	public static String dummyXMLFileName() {
		return getGoldURL().getPath() + TEST_CONFIG_FILE;
	}

	public static void writeDummyXMLFile() {
		if (writeDummy) {
			File xmlFile = new File(dummyXMLFileName());

			FileWriter out = null;
			try {
				out = new FileWriter(xmlFile);
				for (int i = 0; i < CONFIG_XML.length; i++) {
					String line = CONFIG_XML[i];
					out.write(line);
					out.write(WIN_EOL);
				}
			} catch (IOException e) {
			} finally {
				try {
					if (out != null)
						out.close();
				} catch (IOException e) {
				}
			}
			xmlFile.deleteOnExit();
			writeDummy = false;
		}
	}

	private static String toXMLFormat(
		String in,
		boolean requirement,
		String additional) {
		StringBuffer buf = new StringBuffer();
		buf.append("<");
		if (requirement) {
			buf.append(PREREQUISITE_ELEMENT);
			buf.append(" ");
			buf.append(PLUGIN_ELEMENT);
		} else {
			buf.append(PLUGIN_ELEMENT);
			buf.append(" ");
			buf.append(NAME_ATTRIBUTE);
		}
		int idx = in.indexOf(VERSION_SEPARATOR);
		String name = in.substring(0, idx);
		String version = in.substring(idx + 1);
		if(!requirement) {
			buf.append("=\"");
			buf.append(TestUtils.id2Name(name));
			buf.append("\" ");
			buf.append(ID_ATTRIBUTE);
		}
		buf.append("=\"");
		buf.append(name);
		buf.append("\" ");
		buf.append(VERSION_ATTRIBUTE);
		buf.append("=\"");
		buf.append(version);
		buf.append("\"");
		if (additional.length() > 0)
			buf.append(additional);
		if (requirement)
			buf.append("/");
		buf.append(">");
		buf.append(WIN_EOL);
		return buf.toString();
	}

	public static void copyStartupAsLauncher(int startup)
		throws CopyFailedException {
		File from =
			new File(
				getGoldFile(),
				(PLUGIN_DIRECTORY
					+ '/'
					+ (String) MOCKJAR_DATA[startup][miGoldDir]));
		File to =
			new File(
				getDestFile(),
				(PLUGIN_DIRECTORY
					+ '/'
					+ (String) MOCKJAR_DATA[startup][miDir]));
		to.mkdirs();
		to.deleteOnExit();
		File[] files = from.listFiles();
		for (int i = 0; i < files.length; i++) {
			File file = new File(to, files[i].getName());
			FileUtilities.copy(files[i], file);
			file.deleteOnExit();
		}
	}
}