package gov.va.med.hds.cd.mum.test;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;

import junit.framework.TestCase;

import org.w3c.dom.Document;

public class TestUtils implements IMUMConstants, IBootConstants {

    private TestCase test;

    private TestUtils nestedManager;

    public void setNestedManager(TestUtils nestedManager) {
        this.nestedManager = nestedManager;
    }

    public TestUtils(TestCase testCase) {
        super();
        this.test = testCase;
    }

    private static String basePath = "gov/va/med/hds/cd/mum/";
    private String rootPath = basePath + "update/testdata/";
    private String configPath = rootPath + "configurations/";
    private String manifestPath = rootPath + "manifests/";

    public static final String LOADER_CONFIG = basePath + "loader/test/testdata/config.xml";

    private static final String MANIFEST = "manifest";
    private static final String XML = ".xml";
    public static final String MANIFEST1 = "manifest1.xml";
    public static final String MANIFEST2 = "manifest2.xml";
    public static final String MANIFEST3 = "manifest3.xml";
    public static final String NO_PLUGIN = "NoPlugin.xml";

    public static final String CLPARAM = "clparam.xml";
    public static final String CONFIG1 = "config1.xml";
    public static final String CONFIG2 = "config2.xml";
    public static final String BAD_CONFIG = "badConfig.xml";
    public static final String MISSING_ROOT = "missingRoot.xml";
    public static final String INVALID_ROOT = "invalidRoot.xml";
    public static final String MISSING_CRC = "missingCRC.xml";
    public static final String INVALID_CRC = "invalidCRC.xml";
    public static final String ZERO_cRC = "zeroCRC.xml";
    public static final String MULTIPLE_SETTINGS = "multipleSettings.xml";
    public static final String INCORRECT_CRC = "incorrectCRC.xml";
    public static final String MISSING_STARTUP = "missingStartup.xml";
    public static final String MISSING_APP = "missingApp.xml";
    public static final String MISSING_MANIFEST = "missingManifest.xml";
    public static final String MULTIPLE_STARTUPS = "multipleStartups.xml";
    public static final String MULTIPLE_APPS = "multipleApps.xml";

    public static final int pluginOne = 0;
    public static final int pluginTwo = 1;
    public static final int pluginThree = 2;
    public static final int pluginFour = 3;

    private static final String pluginPath = "gov/va/med/hds/cd/mum/config/testdata/plugins/";

    private static final String[][] jarFiles = { { "one/one.jar" }, {
            "two/two7.jar", "two/two8.jar" }, {
            "three/three.jar" }, {
        }
    };

    private static final String[] manifests =
        { "one/manifest.xml", "two/manifest.xml", "three/manifest.xml", "four/manifest.xml" };

    public File[] getTestPluginFiles(int idx) {
        String[] jars = jarFiles[idx];
        File[] files = new File[jars.length];
        for (int i = 0; i < jars.length; i++) {
            files[i] = getFile(pluginPath, jars[i]);
        }
        return files;
    }

    public URL getTestManfiestURL(int idx) {
        return getURL(pluginPath, manifests[idx]);
    }

    public URL getConfigURL(String config) {
        return getURL(configPath, config);
    }

    public URL getManifestURL(String manifest) {
        return getURL(manifestPath, manifest);
    }

    public URL getManifestURL(int idx) {
        return getManifestURL(MANIFEST + Integer.toString(idx + 1) + XML);
    }

    public ITreeData getConfigDocument(String config)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        return getDocument(configPath, config);
    }

    public ITreeData getManifestDocument(String manifest)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        return getDocument(manifestPath, manifest);
    }

    public ITreeData getManifestXML(String manifest)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        return getManifestDocument(manifest);
    }

    public ITreeData getManifestXML(URL manifest)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        return getDocument(manifest);
    }

    private URL getURL(String path, String file) {
        return getURL(path + file);
    }

    private File getFile(String path, String file) {
        return getFile(path + file);
    }

    public URL getURL(String path) {
        URL url = null;
        try {
            File file = getFile(path, false);
            if (file.exists())
                url = getFile(path).toURL();
            else {
                url = this.getClass().getClassLoader().getResource(path);
                //System.out.println(url.toString());		
                if ((url.getProtocol().equals("jar"))
                    && ((url.getPath().endsWith(JAR_EXT)) || (url.getPath().endsWith(".dll")))
                    && (nestedManager != null))
                    url = nestedManager.manageNestedJar(url);
                if (test != null)
                    test.assertNotNull(url);
            }
        } catch (MalformedURLException e) {
        }
        return url;
    }

    public File getFile(String path) {
        return getFile(path, true);
    }

    private File getFile(String path, boolean doTests) {
        File file = null;
        URL url = this.getClass().getClassLoader().getResource(path);
        //System.out.println(url.toString());		
        if ((url.getProtocol().equals("jar"))
            && ((url.getPath().endsWith(JAR_EXT)) || (url.getPath().endsWith(".dll")))
            && (nestedManager != null))
            file = nestedManager.manageNestedFile(url);
        else
            file = new File(url.getPath());
        if ((test != null) && doTests) {
            test.assertNotNull(file);
            test.assertTrue(file.exists());
        }
        return file;
    }

    private ITreeData getDocument(String path, String manifest)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        return getDocument(getURL(path, manifest));
    }

    public ITreeData getDocument(URL url)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        URLConnection conn;
        InputStream in = null;
        ILoadableTreeData doc = null;
        conn = url.openConnection();
        conn.connect();
        in = new BufferedInputStream(conn.getInputStream());
        try {
            doc = TreeDataFactory.newSimpleXMLTreeData();
            doc.load(in);
        } finally {
            if (in != null) {
                in.close();
            }
        }
        if (test != null)
            test.assertNotNull(doc);
        return doc;
    }

    public URL manageNestedJar(URL url) {
        //        File root = TestFileBuilder.getJarDir();
        //        String temp = url.getPath();
        //        if (temp.startsWith("file:"))
        //            temp = temp.substring(5);
        //        int idx = temp.indexOf("!/");
        //        String subjar = temp.substring(idx + 2);
        //        temp = temp.substring(0, idx);
        //        idx = subjar.lastIndexOf('/');
        //        String subjarName = subjar.substring(idx);
        //        File out = new File(root, subjarName);
        //        File jarFile = new File(temp);
        //        try {
        //            JarFile jar = new JarFile(jarFile);
        //            JarEntry entry = jar.getJarEntry(subjar);
        //            InputStream fis = jar.getInputStream(entry);
        //            FileOutputStream fos = new FileOutputStream(out);
        //            byte[] buf = new byte[1024];
        //            int i = 0;
        //            while ((i = fis.read(buf)) != -1) {
        //                fos.write(buf, 0, i);
        //            }
        //            fis.close();
        //            fos.close();
        //            url = out.toURL();
        //        } catch (IOException e) {
        //        }
        //        //		System.out.println(url.toString());
        //        return url;
        try {
            return manageNestedFile(url).toURL();
        } catch (MalformedURLException e) {
            return null;
        }
    }

    public File manageNestedFile(URL url) {
        File root = TestFileBuilder.getJarDir();
        String temp = url.getPath();
        if (temp.startsWith("file:"))
            temp = temp.substring(5);
        int idx = temp.indexOf("!/");
        String subjar = temp.substring(idx + 2);
        temp = temp.substring(0, idx);
        idx = subjar.lastIndexOf('/');
        String subjarName = subjar.substring(idx);
        File out = new File(root, subjarName);
        File jarFile = new File(temp);
        try {
            JarFile jar = new JarFile(jarFile);
            JarEntry entry = jar.getJarEntry(subjar);
            InputStream fis = jar.getInputStream(entry);
            FileOutputStream fos = new FileOutputStream(out);
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) {
                fos.write(buf, 0, i);
            }
            fis.close();
            fos.close();
        } catch (IOException e) {
        }
        //		System.out.println(url.toString());
        return out;

    }

    public void assertManifestError(Exception e, String message) {
        String error = e.getMessage();
        test.assertTrue(
            "\"" + error + "\" should have started with \"" + PLUGIN_ERROR + "\"",
            error.startsWith(PLUGIN_ERROR));
        test.assertTrue(
            "\"" + error + "\" should have contained \"" + message + "\"",
            error.indexOf(message) > 0);
    }

    /**
     * Method saves Document data to a file
     * @param xml
     * @param doc
     * @throws FileNotFoundException
     * @throws TransformerFactoryConfigurationError
     * @throws TransformerException
     */
    public static void saveXMLFile(File xml, Document doc)
        throws FileNotFoundException, TransformerFactoryConfigurationError, TransformerException {
        if (xml.exists())
            xml.delete();
        xml.getParentFile().mkdirs();
        OutputStream data = new BufferedOutputStream(new FileOutputStream(xml));
        saveXMLData(data, doc);
    }

    /**
     * Saves Document data to an output stream
     * @param data
     * @param doc
     * @throws FileNotFoundException
     * @throws TransformerFactoryConfigurationError
     * @throws TransformerException
     */
    public static void saveXMLData(OutputStream data, Document doc)
        throws FileNotFoundException, TransformerFactoryConfigurationError, TransformerException {
        try {
            Transformer t = TransformerFactory.newInstance().newTransformer();
            t.transform(
                new javax.xml.transform.dom.DOMSource(doc),
                new javax.xml.transform.stream.StreamResult(data));
        } finally {
            try {
                data.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * Saves Document data to a file, trapping and ignoring any errors
     * @param xml
     * @param doc
     */
    public static void saveXMLFileIgnoreErrors(File xml, Document doc) {
        try {
            saveXMLFile(xml, doc);
        } catch (FileNotFoundException e) {
        } catch (TransformerFactoryConfigurationError e) {
        } catch (TransformerException e) {
        }
    }
    
    public static String id2Name(String id) {
		StringBuffer buf = new StringBuffer(id);
		int idx = buf.indexOf("_");
		if(idx >= 0)
			buf.setLength(idx);
		idx = 1;
		while (idx < (buf.length() - 1)) {
			if ((Character.isUpperCase(buf.charAt(idx))
				&& (!Character.isUpperCase(buf.charAt(idx + 1))))) {
				buf.insert(idx, ' ');
				idx++;
			}
			idx++;
		}
		return buf.toString();
    }

}
