package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.IApplication;
import gov.va.med.hds.cd.mum.IPluginSource;
import gov.va.med.hds.cd.mum.boot.ISplashScreen;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.Settings;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.loader.test.TestSharedClassLoader;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.update.ApplicationLauncher;
import gov.va.med.hds.cd.mum.update.IApplicationLauncher;
import gov.va.med.hds.cd.mum.update.IUpdateConstants;
import gov.va.med.hds.cd.mum.utils.IIgnoreApplication;

import java.io.StringWriter;
import java.net.URL;
import java.util.Arrays;

import junit.framework.TestCase;

public class ApplicationLauncherTest
    extends TestCase
    implements IApplication, IIgnoreApplication, IMUMTestConstants, IUpdateConstants {

    public void testLaunch() throws Throwable {
        assertNull(passedArgs);
        assertNull(passedPlugins);
        assertNull(passedSplash);
        launcher.launchApplication(null, specs, info, startupSpec);
        assertEquals(args, passedArgs);
        assertNotNull(passedPlugins);
        assertEquals(3, passedPlugins.length);
        assertEquals(startupSpec.getClassLoader(), passedPlugins[0].getClassLoader());
        assertEquals(
            "one.two.three.four.five",
            passedPlugins[0].getManifest().getAttribute(
                passedPlugins[0].getManifest().getRootID(),
                ID_ATTRIBUTE));
        assertNotNull(passedSplash);

        assertEquals(appSpec.getClassLoader(), passedPlugins[1].getClassLoader());
        assertEquals(
            "gov.va.med.hds.cd.mum.testing.some.plugin",
            passedPlugins[1].getManifest().getAttribute(
                passedPlugins[1].getManifest().getRootID(),
                ID_ATTRIBUTE));

        assertEquals(otherSpec.getClassLoader(), passedPlugins[2].getClassLoader());
        assertEquals(
            "my.plugin",
            passedPlugins[2].getManifest().getAttribute(
                passedPlugins[2].getManifest().getRootID(),
                ID_ATTRIBUTE));
    }

    public void testConfigXML() throws Throwable {
        launcher.launchApplication(utils.getConfigURL(utils.CONFIG1), specs, info, startupSpec);
        assertEmptySettings(passedConfigXML);
        ILoadableTreeData xml = TreeDataFactory.newSimpleXMLTreeData();
        URL url = utils.getConfigURL(utils.CONFIG2);
        xml.load(url);
        info.setSettings(new Settings(xml));
        launcher.launchApplication(url, specs, info, startupSpec);
        StringWriter writer = new StringWriter();
        ((ISavableTreeData) passedConfigXML).save(writer);
        String result = writer.toString();
        assertEquals(result, mergedXML, result);
    }

    public void testJUnitNotFound() throws Throwable {
        try {
            info.setArgs(new String[] { TEST_TEXT_COMMAND_TAG + "whatever" });
            launcher.launchApplication(utils.getConfigURL(utils.CONFIG1), specs, info, startupSpec);
        } catch (IllegalArgumentException e) {
            assertEquals(JUNIT_PLUGIN_NOT_FOUND, e.getMessage());
        }
    }

    public void testCommandLineParamMapping() throws Throwable {
        assertCommandLineParameters(null, null);
        String[] args = new String[9];
        String[] values = new String[9];
        Arrays.fill(args, null);
        Arrays.fill(values, null);
        assertCommandLineParameters(args, values);
        args[0] = "-FiVe=12345";
        values[5] = "12345";
        values[7] = "12345";
        assertCommandLineParameters(args, values);
        args[1] = "-T=teSTing";
        values[0] = "teSTing";
        assertCommandLineParameters(args, values);
        args[1] = "-t=testing one more timE";
        values[0] = "testing one more timE";
        assertCommandLineParameters(args, values);
        args[1] = "-testing=testing again";
        values[0] = "testing again";
        args[2] = "-node2=whatever";
        values[2] = "whatever";
        assertCommandLineParameters(args, values);
        args[3] = "-harry=monkey";
        values[6] = "monkey";
        assertCommandLineParameters(args, values);
        args[4] = "-fred=";
        values[3] = "";
        assertCommandLineParameters(args, values);
        args[5] = "-node1=smile";
        values[1] = ">smile</node1>";
        args[6] = "-four=greetings";
        values[4] = ">greetings</node4>";
        args[2] = null;
        values[2] = null;
        assertCommandLineParameters(args, values);
        args[8] = "-ta1=whatever";
        values[8] = "whatever";
		assertCommandLineParameters(args, values);
		values[8] = "ZYAHSADFASD fhdj fhjk fhsdja";
		args[8] = "-zxt=" + values[8];		
		assertCommandLineParameters(args, values);
    }

    //********************
    //*** end of tests ***
    //********************

    public ApplicationLauncherTest() {
        super("");
    }

    public ApplicationLauncherTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        utils = new TestUtils(this);
        launcher = new ApplicationLauncher();
        args = new String[] { "something" };
        specs = new ListMap();
        info = new TestStartupInfo();
        startupSpec = new MockPluginSpec(mjRuntime);
        appSpec = new MockPluginSpec(mjRemCore);
        otherSpec = new MockPluginSpec(mjNoteCore1);
        specs.add(startupSpec.getID(), startupSpec);
        specs.add(appSpec.getID(), appSpec);
        specs.add(otherSpec.getID(), otherSpec);
        appSpec.setAppClass(this.getClass().getName());
        for (int i = 0; i < specs.size(); i++) {
            MockPluginSpec spec = (MockPluginSpec) specs.get(i);
            spec.setClassLoader(new TestSharedClassLoader(this.getClass().getClassLoader()));
            spec.setManifestURL(utils.getManifestURL(i));
        }
        info.setArgs(args);
        info.setLoader(startupSpec.getClassLoader());
        info.setSplashScreen(new DummySplashScreen());
        info.setSettings(new Settings(null));
        passedArgs = null;
        passedConfigXML = null;
        passedPlugins = null;
        passedSplash = null;
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void run(
        String[] args,
        ITreeData configXML,
        IPluginSource[] plugins,
        ISplashScreen splash) {
        passedArgs = args;
        passedConfigXML = configXML;
        passedPlugins = plugins;
        passedSplash = splash;
    }

    private static String[] passedArgs;
    private static ITreeData passedConfigXML;
    private static ISplashScreen passedSplash;
    private static IPluginSource[] passedPlugins;

    private ListMap specs;
    private TestStartupInfo info;
    private String[] args;
    private IApplicationLauncher launcher;
    private MockPluginSpec startupSpec;
    private MockPluginSpec appSpec;
    private MockPluginSpec otherSpec;

    private TestUtils utils;

    private static class DummySplashScreen implements ISplashScreen {
        /**
         * @see gov.va.med.hds.cd.mum.utils.ISplashScreen#close()
         */
        public void close() {
        }
    }

    private void assertCommandLineParameters(String[] args, String[] commands) throws Throwable {
        info.setArgs(args);
        ILoadableTreeData xml = TreeDataFactory.newSimpleXMLTreeData();
        URL url = utils.getConfigURL(utils.CLPARAM);
        xml.load(url);
        info.setSettings(new Settings(xml));
        launcher.launchApplication(utils.getConfigURL(utils.CLPARAM), specs, info, startupSpec);
        assertNotNull(passedConfigXML);
        int len = 0;
        StringBuffer buf = new StringBuffer(paramXML);
        if (commands != null)
            len = commands.length;
        for (int i = 0; i < paramVars.length; i++) {
            String value;
            if ((i < len) && (commands[i] != null))
                value = commands[i];
            else
                value = paramDefaults[i];
            int idx = buf.indexOf(paramVars[i]);
            buf.delete(idx, idx + paramVars[i].length());
            buf.insert(idx, value);
        }
        StringWriter writer = new StringWriter();
        ((ISavableTreeData) passedConfigXML).save(writer);
        assertEquals(buf.toString(), writer.toString());
    }

    private String mergedXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<settings>"
            + WIN_EOL
            + "  <whatever setting=\"WHATEVER\">Hello</whatever>"
            + WIN_EOL
            + "  <merged c=\"d\">"
            + WIN_EOL
            + "    <testing e=\"f\">"
            + WIN_EOL
            + "      <zzz setting=\"nested\">InTernaL</zzz>"
            + WIN_EOL
            + "    </testing>"
            + WIN_EOL
            + "    <something setting=\"ug\">else</something>"
            + WIN_EOL
            + "    <smile setting=\"smile, you&apos;re on candid camera\">G r e e t i n g s</smile>"
            + WIN_EOL
            + "  </merged>"
            + WIN_EOL
            + "  <d2>smile</d2>"
            + WIN_EOL
            + "</settings>";

    private String[] paramVars = { "%1%", "%2%", "%3%", "%4%", "%5%", "%6%", "%7%", "%8%", "%9%" };
    private String[] paramDefaults =
        {
            "bbb",
            "/>",
            "some value",
            "george",
            ">44444</node4>",
            "12345",
            "beetle",
            "55555",
            "test attribute setting value" };

    private String paramXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<settings>"
            + WIN_EOL
            + "  <test clparam=\"testing,t,TST\">%1%</test>"
            + WIN_EOL
            + "  <test2>"
            + WIN_EOL
            + "    <node1 clparam=\"noDe1\"%2%"
            + WIN_EOL
            + "    <node2 clparam=\"node2\">%3%</node2>"
            + WIN_EOL
            + "    <node3>"
            + WIN_EOL
            + "      <child testAttrib=\"%9%\" testAttrib-clparam=\"zxt,ta1\" testAttrib-setting=\"ta2\">"
            + WIN_EOL
            + "        <grandchild clparam=\"fred\">%4%</grandchild>"
            + WIN_EOL
            + "      </child>"
            + WIN_EOL
            + "    </node3>"
            + WIN_EOL
            + "    <node4 clparam=\"four\" setting=\"four\"%5%"
            + WIN_EOL
            + "    <node5 clparam=\"five\" setting=\"five\">%6%</node5>"
            + WIN_EOL
            + "    <node6>"
            + WIN_EOL
            + "      <child>"
            + WIN_EOL
            + "        <grandchild clparam=\"harry\" setting=\"six\">%7%</grandchild>"
            + WIN_EOL
            + "      </child>"
            + WIN_EOL
            + "    </node6>"
            + WIN_EOL
            + "  </test2>"
            + WIN_EOL
            + "  <test3 clparam=\"five\">%8%</test3>"
            + WIN_EOL
            + "</settings>";

    private void assertEmptySettings(ITreeData xml) {
        assertNotNull(xml);
        assertTrue(xml.hasRoot());
        int root = xml.getRootID();
        assertEquals(0, xml.getChildCount(root));
        assertEquals(0, xml.getAttributeCount(root));
    }

}