package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.IBasePlugin;
import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.VersionFormatException;
import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.UpdateAuthorizationFailureException;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.test.AssertionList;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.update.ConfigReaderFactory;
import gov.va.med.hds.cd.mum.update.ExistingFileEntry;
import gov.va.med.hds.cd.mum.update.ExistingPlugin;
import gov.va.med.hds.cd.mum.update.IConfigReader;
import gov.va.med.hds.cd.mum.update.IExistingPlugin;
import gov.va.med.hds.cd.mum.update.IPluginLocator;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.IPluginUpdater;
import gov.va.med.hds.cd.mum.update.IPluginUsageTracker;
import gov.va.med.hds.cd.mum.update.IUpdateConstants;
import gov.va.med.hds.cd.mum.update.IUpdateListener;
import gov.va.med.hds.cd.mum.update.InvalidPluginDirectoryException;
import gov.va.med.hds.cd.mum.update.PluginClassFactory;
import gov.va.med.hds.cd.mum.update.SpecFileEntry;
import gov.va.med.hds.cd.mum.update.UnauthorizedUpdateException;
import gov.va.med.hds.cd.mum.update.UpdateException;
import gov.va.med.hds.cd.mum.utils.ChecksumCacheInfo;
import gov.va.med.hds.cd.mum.utils.ChecksumCalculator;
import gov.va.med.hds.cd.mum.utils.MUMUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class DummyUpdateTests
    extends TestCase
    implements IStatusNotifier, IUpdateListener, IUpdateConstants, IMUMTestConstants {

    public void testNonPluginDirs() throws FileNotFoundException {
        testCount++;
        try {
            PluginClassFactory.newExistingPlugin(configFile);
            fail("Should have thrown InvalidPluginDirectoryException");
        } catch (InvalidPluginDirectoryException e) {
            assertEquals(FILE_NOT_DIRECTORY, e.getMessage());
        }
        assertBadDir("tests/test1");
        assertBadDir("tests/_test2");
        assertBadDir("tests/test3_");
        assertBadVersion("tests/test4_x");
        assertBadVersion("tests/test4_1.x");
        assertBadVersion("tests/test4_1.0.x");
        assertBadPlugin("tests/empty.Plugin_1.0.0", null, MISSING_MANIFEST_FILE);
        File file = new File(TestFileBuilder.getDestFile(), "tests");
        IPluginLocator locator = PluginClassFactory.newPluginLocator(false);
        try {
            locator.locatePlugins(file, new ListMap());
            fail("Should have thrown InvalidPluginDirectoryException");
        } catch (InvalidPluginDirectoryException e) {
            assertEquals(NO_PLUGINS_DIRECTORY_FOUND, e.getMessage());

        }
    }

    public void testPluginLocatorAndExistingPlugins() throws Throwable {
        testCount++;
        AssertionList al = new AssertionList();
        try {
            IPluginLocator locator = PluginClassFactory.newPluginLocator(false);
            al.assertEquals(0, list.size());
            locator.locatePlugins(pluginsDir, list);
            al.assertEquals(8, list.size());
            IExistingPlugin plugin = getExistingPlugin(list, "VendorNoteEditor_0.0.1");
            al.assertNotNull(plugin);
            al.assertEquals("VendorNoteEditor", plugin.getID());
            al.assertEquals("0.0.1", plugin.getVersion().toString());
            al.assertEquals(3, plugin.getFileCount());
            ExistingFileEntry entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(1428330817L, entry.getCRC());
            al.assertEquals(447, entry.getSize());
            entry = getFileEntry(plugin, STARTUP_JAR);
            al.assertEquals(2060524882L, entry.getCRC());
            al.assertEquals(1280, entry.getSize());
            entry = getFileEntry(plugin, BOOT_JAR);
            al.assertEquals(0, entry.getCRC());
            al.assertEquals(0, entry.getSize());

            plugin = getExistingPlugin(list, "NoteCore_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("NoteCore", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(1110675973L, entry.getCRC());
            al.assertEquals(415, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(4238744751L, entry.getCRC());
            al.assertEquals(1819, entry.getSize());

            plugin = getExistingPlugin(list, "MyLib_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("MyLib", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(1429690339L, entry.getCRC());
            al.assertEquals(238, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(3855682574L, entry.getCRC());
            al.assertEquals(407, entry.getSize());

            plugin = getExistingPlugin(list, "PCE_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("PCE", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(693991820L, entry.getCRC());
            al.assertEquals(343, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(2744322793L, entry.getCRC());
            al.assertEquals(426, entry.getSize());

            plugin = getExistingPlugin(list, "PCEForms_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("PCEForms", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(3742397585L, entry.getCRC());
            al.assertEquals(330, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(2222459563L, entry.getCRC());
            al.assertEquals(413, entry.getSize());

            plugin = getExistingPlugin(list, "ReminderCore_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("ReminderCore", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(4155603745L, entry.getCRC());
            al.assertEquals(361, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(4187687909L, entry.getCRC());
            al.assertEquals(462, entry.getSize());

            plugin = getExistingPlugin(list, "RemDialogs_2.0.0");
            assertNotNull(plugin);
            al.assertEquals("RemDialogs", plugin.getID());
            al.assertEquals("2.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            assertNotNull(entry);
            al.assertEquals(1357935845L, entry.getCRC());
            al.assertEquals(347, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(2731083426L, entry.getCRC());
            al.assertEquals(254, entry.getSize());

            plugin = getExistingPlugin(list, "Runtime_1.0.0");
            al.assertNotNull(plugin);
            al.assertEquals("Runtime", plugin.getID());
            al.assertEquals("1.0.0", plugin.getVersion().toString());
            al.assertEquals(2, plugin.getFileCount());
            entry = getFileEntry(plugin, MANIFEST_FILE);
            al.assertNotNull(entry);
            al.assertEquals(473336326L, entry.getCRC());
            al.assertEquals(374, entry.getSize());
            entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
            al.assertEquals(851538084L, entry.getCRC());
            al.assertEquals(2340, entry.getSize());
        } catch (Throwable e) {
            al.addError(e);
        } finally {
            al.showErrors();
        }
    }

    public void testPluginComparisons()
        throws
            InvalidConfigurationException,
            InvalidPluginDirectoryException,
            SAXException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        testCount++;
        IConfigReader reader = ConfigReaderFactory.newConfigReader();
        assertEquals(0, list.size());
        reader.readConfiguration(configFile.toURL(), list);
        assertEquals(15, list.size());
        ListMap plugins = new ListMap();
        IPluginLocator locator = PluginClassFactory.newPluginLocator(false);
        locator.locatePlugins(pluginsDir, plugins);
        List matches = new ArrayList();
        IPluginSpec ordersPlugin = null;
        for (int i = 0; i < list.size(); i++) {
            IPluginSpec spec = (IPluginSpec) list.get(i);
            if ((ordersPlugin == null) && (spec.getDirectory().equals("OrdersCore_2.0.0")))
                ordersPlugin = spec;
            for (int j = 0; j < plugins.size(); j++) {
                IExistingPlugin plugin = (IExistingPlugin) plugins.get(j);
                if (matchesPlugin(spec, plugin))
                    matches.add(spec);
            }
        }
        assertEquals(5, matches.size());
        assertHasSpecPlugin(matches, mjEncounters);
        assertHasSpecPlugin(matches, mjEncounterForms);
        assertHasSpecPlugin(matches, mjRemCore);
        assertHasSpecPlugin(matches, mjRuntime);
        assertHasSpecPlugin(matches, mjLib1);

        assertNotNull(ordersPlugin);

        String name = "OrdersCore_2.0.0";
        long manifestCRC = 1163353978L;
        long manifestSize = 285;
        String jarName = "tempjar_OrdersCore.jar";
        String jarCRC = ",386273718";
        String jarSize = ",632";

        assertTrue(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC,
                    manifestSize,
                    jarName + jarCRC + jarSize)));

        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC,
                    manifestSize,
                    jarName + jarCRC + jarSize + "1")));

        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC,
                    manifestSize + 1,
                    jarName + jarCRC + jarSize)));

        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    "OrdersCore_2.0.1",
                    manifestCRC,
                    manifestSize,
                    jarName + jarCRC + jarSize)));
        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name.toLowerCase(),
                    manifestCRC,
                    manifestSize,
                    jarName + jarCRC + jarSize)));
        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC + 1,
                    manifestSize,
                    jarName + jarCRC + jarSize)));
        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC,
                    manifestSize,
                    jarName.toLowerCase() + jarCRC + jarSize)));
        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(
                    name,
                    manifestCRC,
                    manifestSize,
                    jarName + jarCRC + "1" + jarSize)));

        assertFalse(
            matchesPlugin(
                ordersPlugin,
                new MockExistingPlugin(name, manifestCRC, manifestSize, null)));
    }

    public void testCRCCache()
        throws
            InvalidPluginDirectoryException,
            SAXException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError,
            FileNotFoundException,
            ClassNotFoundException,
            TransformerFactoryConfigurationError,
            TransformerException {
        testCount++;
        ListMap plugins = new ListMap();
        File crcCache = new File(pluginsDir.getParentFile(), CRC_CACHE_FILENAME);
        IPluginLocator locator = PluginClassFactory.newPluginLocator(false);
        locator.locatePlugins(pluginsDir, plugins);
        assertFalse(crcCache.exists());
        locator = PluginClassFactory.newPluginLocator(true);
        boolean pass = false;
        boolean working = true;
        boolean toggle = true;
        HashMap map = new HashMap();
        do {
            plugins.clear();
            locator.locatePlugins(pluginsDir, plugins);
            assertTrue(crcCache.exists());
            ObjectInputStream cacheIn = new ObjectInputStream(new FileInputStream(crcCache));
            Map cachedData = (Map) cacheIn.readObject();
            cacheIn.close();
            assertNotNull(cachedData);
            for (int i = 0; i < plugins.size(); i++) {
                IExistingPlugin plugin = (IExistingPlugin) plugins.get(i);
                for (int j = 0; j < plugin.getFileCount(); j++) {
                    ExistingFileEntry entry = plugin.getFile(j);
                    String name = entry.getFile().getCanonicalPath();
                    ExistingFileEntry oldEntry = (ExistingFileEntry) map.get(name);
                    if (oldEntry == null)
                        map.put(name, entry);
                    else {
                        int bump = 0;
                        if (!oldEntry.getFile().getName().equals(MANIFEST_FILE))
                            bump = 10;
                        assertEquals(oldEntry.getCRC() + bump, entry.getCRC());
                    }
                    
                    ChecksumCacheInfo fileInfo = (ChecksumCacheInfo) cachedData.get(name);
                    assertNotNull(fileInfo);
                    assertEquals(entry.getCRC(), fileInfo.crc);
                    assertEquals(entry.getFile().length(), entry.getSize());
                    assertEquals(entry.getSize(), fileInfo.size);
                    assertEquals(entry.getFile().lastModified(), fileInfo.modDate);

                    if (!pass) {
                        fileInfo.crc = entry.getCRC() + 10;
                        if (entry.getFile().getName().equals(MANIFEST_FILE)) {
                            toggle = !toggle;
                            if (toggle)
                                fileInfo.size = entry.getSize() + 1;
                            else
                                fileInfo.modDate = entry.getFile().lastModified() + 1;
                        }
                    }
                }
            }
            if (pass)
                working = false;
            else {
                ObjectOutputStream cacheOut = new ObjectOutputStream(new FileOutputStream(crcCache));
                cacheOut.writeObject(cachedData);
                cacheOut.close();
                pass = true;
            }
        } while (working);
    }

    public void testPluginUpdate()
        throws
            InvalidConfigurationException,
            MalformedURLException,
            FileNotFoundException,
            InvalidPluginDirectoryException,
            UpdateException,
            UpdateAuthorizationFailureException {
        testCount++;
        IConfigReader reader = ConfigReaderFactory.newConfigReader();
        reader.readConfiguration(configFile.toURL(), list);
        IPluginSpec spec = getSpecPlugin(list, "OrdersCore_2.0.0");
        assertNotNull(spec);
        IPluginSpec spec2 = getSpecPlugin(list, "Runtime_1.0.0");
        assertNotNull(spec2);

        IPluginUpdater updater = PluginClassFactory.newPluginUpdater();
        File dir = new File(TestFileBuilder.getDestFile(), "tests/" + PLUGIN_DIRECTORY);
        URL goldRoot = new URL(TestFileBuilder.getGoldURL(), PLUGIN_DIRECTORY);
        dir.mkdirs();
        IPluginLocator locator = PluginClassFactory.newPluginLocator(false);
        list.clear();
        locator.locatePlugins(dir, list);
        assertEquals(0, list.size());
        updater.beginUpdate(TestFileBuilder.getDestFile(), null, null, 0);
        File[] files;
        for (int i = 0; i < spec.getFileCount(); i++)
            spec.getFile(i).setUpdate(true);
        try {
            files = updater.update(goldRoot, dir, spec);
        } finally {
            updater.endUpdate(null);
        }
        locator.locatePlugins(dir, list);
        assertEquals(1, list.size());
        IExistingPlugin plugin = (IExistingPlugin) list.get(0);
        assertEquals("OrdersCore_2.0.0", plugin.getDirectory());
        assertEquals("OrdersCore", plugin.getID());
        assertEquals("2.0.0", plugin.getVersion().toString());
        assertEquals(2, plugin.getFileCount());
        assertEquals(2, files.length);
        for (int i = 0; i < files.length; i++) {
            boolean found = false;
            for (int j = 0; j < plugin.getFileCount(); j++) {
                ExistingFileEntry entry = plugin.getFile(j);
                if (entry.getFile().toString().equals(files[i].toString())) {
                    found = true;
                    break;
                }
            }
            assertTrue(files[i].toString(), found);
        }
        ExistingFileEntry entry = getFileEntry(plugin, MANIFEST_FILE);
        assertNotNull(entry);
        SpecFileEntry specEntry = getSpecEntry(spec, MANIFEST_FILE);
        assertNotNull(specEntry);
        assertEquals(specEntry.getCRC(), entry.getCRC());
        assertEquals(specEntry.getSize(), entry.getSize());
        entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
        assertNotNull(entry);
        specEntry = getSpecEntry(spec, tempjar(plugin.getDirectory()));
        assertNotNull(specEntry);
        assertEquals(specEntry.getCRC(), entry.getCRC());
        assertEquals(specEntry.getSize(), entry.getSize());

        dir = new File(TestFileBuilder.getDestFile(), "test2/" + PLUGIN_DIRECTORY);
        dir.mkdirs();
        list.clear();
        locator.locatePlugins(dir, list);
        assertEquals(0, list.size());
        updater.beginUpdate(TestFileBuilder.getDestFile(), null, null, 0);
        for (int i = 0; i < spec2.getFileCount(); i++)
            spec2.getFile(i).setUpdate(true);
        try {
            updater.update(goldRoot, dir, spec2);
        } finally {
            updater.endUpdate(null);
        }
        locator.locatePlugins(dir, list);
        assertEquals(1, list.size());
        plugin = (IExistingPlugin) list.get(0);
        assertEquals("Runtime_1.0.0", plugin.getDirectory());
        assertEquals("Runtime", plugin.getID());
        assertEquals("1.0.0", plugin.getVersion().toString());
        assertEquals(2, plugin.getFileCount());
        entry = getFileEntry(plugin, MANIFEST_FILE);
        assertNotNull(entry);
        specEntry = getSpecEntry(spec2, MANIFEST_FILE);
        assertNotNull(specEntry);
        assertEquals(specEntry.getCRC(), entry.getCRC());
        assertEquals(specEntry.getSize(), entry.getSize());
        entry = getFileEntry(plugin, tempjar(plugin.getDirectory()));
        assertNotNull(entry);
        specEntry = getSpecEntry(spec2, tempjar(plugin.getDirectory()));
        assertNotNull(specEntry);
        assertEquals(specEntry.getCRC(), entry.getCRC());
        assertEquals(specEntry.getSize(), entry.getSize());
    }

    public void testIntermediateUpdate()
        throws
            InvalidConfigurationException,
            MalformedURLException,
            UpdateException,
            UpdateAuthorizationFailureException {
        testCount++;
        internalTestCopyToUpdateFolder(false, true);
    }

    public void testAutoCorrectFromCorruptCopy()
        throws
            InvalidConfigurationException,
            MalformedURLException,
            UpdateException,
            UpdateAuthorizationFailureException {
        testCount++;
        clearCorrupt = true;
        internalTestCopyToUpdateFolder(true, true);
    }

    public void testCorruptCopyFailure()
        throws
            UpdateException,
            InvalidConfigurationException,
            MalformedURLException,
            UpdateAuthorizationFailureException {
        testCount++;
        clearCorrupt = false;
        try {
            internalTestCopyToUpdateFolder(true, true);
            fail("should have thrown UpdateException");
        } catch (UpdateException e) {
            if ((!e.getMessage().startsWith(COPY_FAILED_ERROR))
                || (!e.getMessage().endsWith(COPY_FAILED_REASON)))
                throw e;
        }
    }

    public void testDuplicateCopy()
        throws
            IOException,
            InvalidConfigurationException,
            MalformedURLException,
            UpdateException,
            UpdateAuthorizationFailureException {
        testCount++;
        internalTestCopyToUpdateFolder(false, true);
        File garbage =
            new File(
                TestFileBuilder.getDestFile(),
                PLUGINS_DIRECTORY + "/" + ordersFolder + "/garbageFile.txt");
        garbage.createNewFile();
        assertTrue(garbage.exists());
        copiedPlugin = null;
        copiedSize = 0;
        internalTestCopyToUpdateFolder(false, false);
        assertFalse(garbage.exists());
    }

    private void internalTestCopyToUpdateFolder(
        boolean testBadCRC,
        boolean deleteTargetFolderFirst)
        throws
            InvalidConfigurationException,
            MalformedURLException,
            UpdateException,
            UpdateAuthorizationFailureException {
        String ordersJar = "tempjar_OrdersCore.jar";
        updateDir = new File(TestFileBuilder.getDestFile(), UPDATE_DIRECTORY);
        File ordersUpdateDir = new File(updateDir, ordersFolder);
        File manifest = new File(ordersUpdateDir, MANIFEST_FILE);
        File jar = new File(ordersUpdateDir, ordersJar);

        File sourceDir = new File(TestFileBuilder.getGoldFile(), PLUGINS_DIRECTORY);
        File sourceOrdersDir = new File(sourceDir, "Orders");
        File sourceManifest = new File(sourceOrdersDir, MANIFEST_FILE);
        File sourceJar = new File(sourceOrdersDir, ordersJar);
        long size = sourceManifest.length() + sourceJar.length();

        long crc = 0;
        ChecksumCalculator calc = null;
        if (testBadCRC) {
            fileToCorrupt = jar;
            calc = new ChecksumCalculator();
            calc.calcCRC(sourceJar.toURL());
            crc = calc.getValue();
        }

        updateFiles = new File[] { ordersUpdateDir, manifest, jar };
        deleteAllFiles(updateDir);
        assertFalse(updateDir.exists());
        File targetDir =
            new File(TestFileBuilder.getDestFile(), PLUGINS_DIRECTORY + "/" + ordersFolder);
        if (deleteTargetFolderFirst) {
            deleteAllFiles(targetDir);
            assertFalse(targetDir.exists());
        }

        IConfigReader reader = ConfigReaderFactory.newConfigReader();
        reader.readConfiguration(configFile.toURL(), list);
        IPluginSpec spec = getSpecPlugin(list, ordersFolder);
        assertNotNull(spec);

        IPluginUpdater updater = PluginClassFactory.newPluginUpdater();
        updater.assignUpdateListener(this);
        updater.beginUpdate(TestFileBuilder.getDestFile(), "testing", this, size);
        File[] files = null;
        try {
            assertNull(copiedPlugin);
            for (int i = 0; i < spec.getFileCount(); i++)
                spec.getFile(i).setUpdate(true);
            URL from = new URL(TestFileBuilder.getGoldURL(), PLUGINS_DIRECTORY);
            File to = new File(TestFileBuilder.getDestFile(), PLUGINS_DIRECTORY);
            files = updater.update(from, to, spec);
        } finally {
            updater.endUpdate(null);
        }
        assertEquals(spec, copiedPlugin);
        assertEquals(2, files.length);
        assertFalse(updateDir.exists());
        assertTrue(targetDir.exists());
        for (int i = 0; i < updateFiles.length; i++)
            assertFalse(updateFiles[i].exists());

        long expected = size;
        if (testBadCRC)
            expected *= 2;
        assertEquals(expected, maxSize);
        assertEquals(expected, copiedSize);

        if (testBadCRC) {
            calc.reset();
            assertEquals(0, calc.getValue());
            URL newJar =
                new URL(
                    TestFileBuilder.getDestURL(),
                    PLUGINS_DIRECTORY + "/" + ordersFolder + "/" + ordersJar);
            calc.calcCRC(newJar);
            assertEquals(crc, calc.getValue());
        }
    }

    public void testPluginUsageTracker()
        throws
            SAXException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError,
            FileNotFoundException,
            InvalidPluginDirectoryException {
        testCount++;
        IPluginUsageTracker tracker =
            PluginClassFactory.newPluginUsageTracker(TestFileBuilder.getDestFile());
        for (int i = 0; i < EXISTING_CONFIG.length; i++) {
            if (EXISTING_CONFIG[i] != mjRemCore) {
                MockPluginSpec spec = new MockPluginSpec(EXISTING_CONFIG[i]);
                list.add(spec.getID(), spec);
            }
        }
        File usageFile = new File(TestFileBuilder.getDestFile(), PLUGIN_USAGE_FILENAME);
        assertFalse(usageFile.exists());
        tracker.trackPluginUsage(list);
        assertTrue(usageFile.exists());

        Element pceNode =
            validateUsageFile(usageFile, list, (String) MOCKJAR_DATA[mjEncounters][miDir]);
        assertNotNull(pceNode);

        pceNode.setAttribute(
            LAST_USED_ATTRIBUTE,
            Integer.toString(MUMUtils.getDaysSince1970() - PLUGIN_USAGE_DAYS_BEFORE_DELETE - 5));
        TestUtils.saveXMLFileIgnoreErrors(usageFile, pceNode.getOwnerDocument());
        tracker = PluginClassFactory.newPluginUsageTracker(TestFileBuilder.getDestFile());
        list.clear();
        IExistingPlugin pceEP = null;
        File rootDir = new File(TestFileBuilder.getDestFile(), PLUGINS_DIRECTORY);
        for (int i = 0; i < EXISTING_CONFIG.length; i++) {
            File file = new File(rootDir, (String) MOCKJAR_DATA[EXISTING_CONFIG[i]][miDir]);
            assertTrue(file.exists());
            IExistingPlugin ep = PluginClassFactory.newExistingPlugin(file);
            list.add(ep.getID(), ep);
            if (EXISTING_CONFIG[i] == mjEncounters)
                pceEP = ep;
        }
        tracker.deleteUnusedPlugins(list);
        for (int i = 0; i < list.size(); i++) {
            ExistingPlugin ep = (ExistingPlugin) list.get(i);
            boolean shouldExist = true;
            if (ep == pceEP)
                shouldExist = false;
            for (int j = 0; j < ep.getFileCount(); j++) {
                assertEquals(
                    existMsg(shouldExist, ep.getFile(j)),
                    shouldExist,
                    ep.getFile(j).getFile().exists());
            }
        }
        validateUsageFile(usageFile, list, null);
    }

    public void testUpdatesInProgress() throws UpdateException {
        testCount++;
        long currentTime = System.currentTimeMillis();
        File rootDir = TestFileBuilder.getDestFile();
        File updateDir = new File(rootDir, UPDATE_DIRECTORY);
        File flagFile = new File(updateDir, UPDATE_PROGRESS_FILE);
        assertFalse(flagFile.exists());
        IPluginUpdater updater1 = PluginClassFactory.newPluginUpdater();
        IPluginUpdater updater2 = PluginClassFactory.newPluginUpdater();
        updater1.beginUpdate(rootDir, null, null, 0);
        assertTrue(flagFile.exists());
        try {
            updater2.beginUpdate(rootDir, null, null, 0);
            fail("Should have thrown UnauthorizedUpdateException");
        } catch (UnauthorizedUpdateException e) {
            assertEquals(UPDATE_IN_PROGRESS_ERROR, e.getMessage());
        }
        flagFile.setLastModified(currentTime - UPDATE_TIMEOUT - 1);
        updater2.beginUpdate(rootDir, null, null, 0);
    }

    //********************
    //*** end of tests ***
    //********************
    private static int testCount = 0;
    private static int maxTestCount = TestFileBuilder.numTests(DummyUpdateTests.class);

    public DummyUpdateTests(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        if (testCount == 0) {
            TestFileBuilder.ensureTestDirectories(true);
            TestFileBuilder.writeDummyXMLFile();
            File file = new File(TestFileBuilder.getDestFile(), PLUGIN_DIRECTORY);
            (new File(file, "badPlugin1")).mkdir();
            (new File(file, "badPlugin2_")).mkdir();
            (new File(file, "_badPlugin3")).mkdir();
            (new File(file, "badPlugin4_x")).mkdir();
            (new File(file, "badPlugin5_1.x")).mkdir();
            (new File(file, "badPlugin6_1.2.x")).mkdir();
            (new File(file, "emptyPlugin_1.0.0")).mkdir();
            configFile = new File(TestFileBuilder.dummyXMLFileName());
            pluginsDir = new File(TestFileBuilder.getDestFile(), PLUGIN_DIRECTORY);
        }
        list = new ListMap();
        copiedPlugin = null;
        fileToCorrupt = null;
        clearCorrupt = true;
        copiedSize = 0;
        maxSize = 0;
        ChecksumCalculator.setJarTestMode(true);
    }

    private String ordersFolder = "OrdersCore_2.0.0";

    protected void tearDown() throws Exception {
        super.tearDown();
        if (testCount == maxTestCount) {
            TestFileBuilder.eraseTestDirectories(true);
            testCount = 0;
        }
        ChecksumCalculator.setJarTestMode(false);
    }

    private IExistingPlugin getExistingPlugin(ListMap list, String dir) {
        for (int i = 0; i < list.size(); i++) {
            IExistingPlugin plugin = (IExistingPlugin) list.get(i);
            if (plugin.getDirectory().equals(dir))
                return plugin;
        }
        return null;
    }

    private IPluginSpec getSpecPlugin(ListMap list, String dir) {
        for (int i = 0; i < list.size(); i++) {
            IPluginSpec plugin = (IPluginSpec) list.get(i);
            if (plugin.getDirectory().equals(dir))
                return plugin;
        }
        return null;
    }

    private ExistingFileEntry getFileEntry(IExistingPlugin plugin, String fileName) {
        for (int i = 0; i < plugin.getFileCount(); i++) {
            ExistingFileEntry entry = plugin.getFile(i);
            if (entry.getFile().getName().equals(fileName))
                return entry;
        }
        return null;
    }

    private SpecFileEntry getSpecEntry(IPluginSpec plugin, String fileName) {
        for (int i = 0; i < plugin.getFileCount(); i++) {
            SpecFileEntry entry = plugin.getFile(i);
            if (entry.getFileName().equals(fileName))
                return entry;
        }
        return null;
    }

    private String tempjar(String dir) {
        return "tempjar_" + dir.substring(0, dir.indexOf("_")) + ".jar";
    }

    private void assertBadDir(String dirName) {
        assertBadPlugin(dirName, null, INVALID_DIRECTORY_ENTRY);
    }

    private void assertBadVersion(String dirName) {
        assertBadPlugin(dirName, verError, INVALID_PLUGIN_VERSION);
    }

    private void assertBadPlugin(String dirName, Class causeClass, String message) {
        File file = new File(TestFileBuilder.getDestFile(), dirName);
        file.mkdirs();
        try {
            PluginClassFactory.newExistingPlugin(file);
            fail("Should have thrown InvalidPluginDirectoryException");
        } catch (InvalidPluginDirectoryException e) {
            assertEquals(message, e.getMessage());
            if (causeClass != null)
                assertTrue(e.getCause().getClass() == causeClass);
        } catch (FileNotFoundException e) {
            fail(e.getMessage());
        }
    }

    private void assertHasSpecPlugin(List list, int index) {
        String dir = (String) MOCKJAR_DATA[index][miDir];
        for (int i = 0; i < list.size(); i++) {
            IPluginSpec plugin = (IPluginSpec) list.get(i);
            if (plugin.getDirectory().equals(dir))
                return;
        }
        fail("Should have had plugin " + dir);
    }

    private static final Class verError = VersionFormatException.class;
    private static File configFile;
    private static File pluginsDir;
    private ListMap list;

    private void assertCacheDataEquals(URL in, URL cache) throws IOException {
        assertCacheDataEquals(in.openConnection().getInputStream(), cache);
    }

    private void assertCacheDataEquals(InputStream in, URL cache) throws IOException {
        assertNotNull("Cache not found", cache);
        try {
            InputStream cachedStream = cache.openConnection().getInputStream();
            try {
                assertEquals(in.available(), cachedStream.available());
                int bytes;
                byte[] buffer1 = new byte[1024];
                byte[] buffer2 = new byte[1024];
                while ((bytes = in.read(buffer1)) != -1) {
                    assertEquals(bytes, cachedStream.read(buffer2));
                    for (int i = 0; i < buffer1.length; i++) {
                        assertEquals(
                            "cached config doesn't match origional",
                            buffer1[i],
                            buffer2[i]);
                    }
                }
            } finally {
                cachedStream.close();
            }
        } finally {
            in.close();
        }
    }

    private void deleteAllFiles(File file) {
        if (file.exists()) {
            if (file.isDirectory()) {
                File[] files = file.listFiles();
                for (int i = 0; i < files.length; i++)
                    deleteAllFiles(files[i]);
            }
            file.delete();
        }
    }

    private IPluginSpec copiedPlugin;
    private File[] updateFiles;
    private File fileToCorrupt;
    private boolean clearCorrupt;
    private long copiedSize;
    private long maxSize;
    private File updateDir;
    /**
     * @see gov.va.med.hds.cd.mum.update.IUpdateListener#copiedToUpdateFolder(gov.va.med.hds.cd.mum.update.IPluginSpec)
     */
    public void copiedToUpdateFolder(IPluginSpec plugin) {
        assertTrue(updateDir.exists());
        copiedPlugin = plugin;
        for (int i = 0; i < updateFiles.length; i++)
            assertTrue(updateFiles[i].exists());
        if (fileToCorrupt != null) {
            RandomAccessFile rand = null;
            try {
                rand = new RandomAccessFile(fileToCorrupt, "rw");
                long offset = rand.getFilePointer();
                int value = rand.readInt();
                rand.seek(offset);
                value++;
                rand.writeInt(value);
            } catch (Exception e) {
                fail(e.getMessage());
            } finally {
                if (rand != null) {
                    try {
                        rand.close();
                    } catch (IOException e) {
                    }
                }
            }
            if (clearCorrupt)
                fileToCorrupt = null;
        }
    } /**
                            																																																																				 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#showProgressBar(long)
                            																																																																				 */
    public void updateProgressBar(long value) {
        copiedSize += value;
    } /**
                            																																																																					 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressMessage(java.lang.String)
                            																																																																					 */
    public void updateProgressMessage(String message) {
    } /**
                            																																																																				 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#hideProgressBar()
                            																																																																				 */
    public void hideProgressBar() {
    } /**
                            																																																																				 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#displayMessage(java.lang.String)
                            																																																																				 */
    public void displayMessage(String message) {
    } /**
                            																																																																				 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#isProgressBarDisplayed()
                            																																																																				 */
    public boolean isProgressBarDisplayed() {
        return false;
    } /**
                            																																																																					 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#getProgress()
                            																																																																					 */
    public long getProgress() {
        return 0;
    } /**
                            																																																																					 * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#getProgressMax()
                            																																																																					 */
    public long getProgressMax() {
        return 0;
    } /**
                            																																																																					 * @see gov.va.med.hds.cd.mum.utils.ISplashScreen#close()
                            																																																																					 */
    public void close() {
    }

    private String existMsg(boolean shouldExist, ExistingFileEntry entry) {
        StringBuffer buf = new StringBuffer();
        if (shouldExist)
            buf.append("Should have found file ");
        else
            buf.append("Should not have found file ");
        buf.append(entry.getFile().toString());
        return buf.toString();
    }

    private Element validateUsageFile(File usageFile, ListMap list, String returnNodeDir)
        throws SAXException, IOException, ParserConfigurationException, FactoryConfigurationError {
        Element result = null;
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(usageFile);
        Element root = doc.getDocumentElement();
        assertEquals(PLUGINS_ELEMENT, root.getNodeName());
        NodeList children = root.getChildNodes();
        int today = MUMUtils.getDaysSince1970();
        assertTrue(today > (30 * 365)); // make sure it's greater than 30 years
        for (int i = 0; i < children.getLength(); i++) {
            Node node = children.item(i);
            if (node instanceof Element) {
                assertEquals(PLUGIN_ELEMENT, node.getNodeName());
                String dir = ((Element) node).getAttribute(DIRECTORY_ATTRIBUTE);
                IBasePlugin plugin = null;
                for (int j = 0; j < list.size(); j++) {
                    IBasePlugin temp = (IBasePlugin) list.get(j);
                    if (temp.getDirectory().equals(dir)) {
                        plugin = temp;
                        break;
                    }
                }
                assertNotNull(plugin);
                assertEquals(
                    today,
                    Integer.parseInt((String) ((Element) node).getAttribute(LAST_USED_ATTRIBUTE)));
                if ((returnNodeDir != null) && (plugin.getDirectory().equals(returnNodeDir)))
                    result = (Element) node;
            }
        }
        return result;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#showProgressBar(long, java.lang.String)
     */
    public void showProgressBar(long maxValue, String message) {
        maxSize = maxValue;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressBarMax(long)
     */
    public void updateProgressBarMax(long value) {
        maxSize += value;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setVersion(java.lang.String)
     */
    public void setVersion(String version) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setShowCancelButton(boolean)
     */
    public void showCancelButton(boolean value) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#wasCancelButtonPressed()
     */
    public boolean isCanceled() {
        return false;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setCancelMessage(java.lang.String)
     */
    public void setCancelMessage(String message) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#beginDisplayUpdate()
     */
    public void beginDisplayUpdate() {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#endDisplayUpdate()
     */
    public void endDisplayUpdate() {
    }

    private boolean matchesPlugin(IPluginSpec spec, IExistingPlugin plugin) {
        boolean matches = spec.samePlugin(plugin);
        ;
        if (matches) {
            spec.assignNeededUpdates(plugin);
            matches = !spec.hasUpdates();
        }
        return matches;
    }
}
