package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.PluginID;
import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.KeyedList;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.update.ExistingFileEntry;
import gov.va.med.hds.cd.mum.update.IExistingPlugin;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.SpecFileEntry;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

public class MockPluginSpec implements IPluginSpec, IMUMTestConstants {

    private String directory;
    private String startupClass;
    private String appClass;
    private ITreeData xml;
    private ClassLoader loader;
    private URL manifestURL;
    private List dependencies;
    private String module;
    private String id;
    private SpecFileEntry manfiestFileEntry;
    private SpecFileEntry jarFileEntry;
    private long jarCRC = 98765;
    private KeyedList requiredPlugins;

    public MockPluginSpec(int index) {
        this();
        directory = (String) MOCKJAR_DATA[index][miDir];
        int idx = directory.indexOf("_");
        id = directory.substring(0, idx);
        module = directory;
    }

    public MockPluginSpec() {
        super();
        dependencies = new ArrayList();
        requiredPlugins = new KeyedList();
    }

    public MockPluginSpec(ITreeData element, ClassLoader loader) {
        this();
        this.xml = element;
        this.loader = loader;
    }

    public void setIDAndLoader(String id, ClassLoader loader) {
        this.id = id;
        this.loader = loader;
    }

    public SpecFileEntry getFile(int index) {
        if (index == 0) {
            if (manfiestFileEntry == null)
                manfiestFileEntry = new SpecFileEntry(MANIFEST_FILE, 12345, 70);
            return manfiestFileEntry;
        } else {
            if (jarFileEntry == null)
                jarFileEntry = new SpecFileEntry("test.jar", jarCRC, 1000);
            return jarFileEntry;
        }
    }

    public String getModule() {
        return module;
    }

    public String getStartupClassName() {
        return startupClass;
    }

    public String getApplicationClassName() {
        return appClass;
    }

    public SimpleList getPublished() {
        return null;
    }

    public boolean publishAllClasses() {
        return true;
    }

    public String getID() {
        return id;
    }

    public Version getVersion() {
        return null;
    }

    public String getDirectory() {
        return directory;
    }

    public int getFileCount() {
        return 2;
    }

    public void setStartupClass(String cls) {
        startupClass = cls;
    }

    public void setAppClass(String cls) {
        appClass = cls;
    }

    public File[] getFiles(File rootDirectory) {
        return null;
    }

    public ClassLoader getClassLoader() {
        return loader;
    }

    public List getElements() {
        return null;
    }

    public Collection getDependencies() {
        return dependencies;
    }

    public void loadManifest(URL manifest) throws InvalidPluginException {
        TestUtils utils = new TestUtils(null);
        try {
            xml = utils.getManifestXML(manifest);
        } catch (IOException e) {
            throw new InvalidPluginException(e);
        } catch (ParserConfigurationException e) {
            throw new InvalidPluginException(e);
        } catch (FactoryConfigurationError e) {
            throw new InvalidPluginException(e);
        } catch (TreeDataException e) {
            throw new InvalidPluginException(e);
        }
    }

    public URL getManifestURL() {
        return manifestURL;
    }

    public void setManifestURL(URL url) {
        manifestURL = url;
    }

    public void setClassLoader(ClassLoader classLoader) {
        this.loader = classLoader;
    }

    public ITreeData getManifestXML() {
        return xml;
    }

    public URL[] getURLs(File rootDirectory) {
        return null;
    }
    public String getSourceDirectory() {
        return null;
    }
    public void badPlugin(String message, Throwable cause) throws InvalidPluginException {
    }
    public void badPlugin(Throwable cause) throws InvalidPluginException {
    }
    public void badPlugin(String message) throws InvalidPluginException {
    }
    public void badPlugin() throws InvalidPluginException {
    }

    public void setJarCRC(long crc) {
        jarCRC = crc;
    }
    /**
     * @see gov.va.med.hds.cd.mum.IDependentPlugin#getPluginID()
     */
    public PluginID getPluginID() {
        return null;
    }
    /**
     * @see gov.va.med.hds.cd.mum.IDependentPlugin#getClassDependencies()
     */
    public SimpleList getClassDependencies() {
        return null;
    }
    /**
     * @see gov.va.med.hds.cd.mum.IDependentPlugin#getPluginDependencies()
     */
    public ListMap getPluginDependencies() {
        return null;
    }

    public KeyedList requiredPlugins() {
        return requiredPlugins;
    }

    public File getPhysicalFile(int index) {
        return null;
    }

    public SimpleList getFilePackages(int index) {
        return null;
    }

    public SimpleList getPublishedClasses() {
        return null;
    }

    public KeyedList getLinkedPlugins() {
        return null;
    }

    public boolean samePlugin(IExistingPlugin plugin) {
        return directory.equals(plugin.getDirectory());
    }

    public void assignNeededUpdates(IExistingPlugin plugin) {
        for (int i = 0; i < getFileCount(); i++) {
            SpecFileEntry specEntry = getFile(i);
            specEntry.setUpdate(true);
            if (plugin != null) {
                for (int j = 0; j < plugin.getFileCount(); j++) {
                    ExistingFileEntry entry = plugin.getFile(j);
                    if (specEntry.getFileName().equals(entry.getFile().getName())) {
                        if ((entry.getCRC() == specEntry.getCRC())
                            && (entry.getSize() == specEntry.getSize()))
                            specEntry.setUpdate(false);
                        break;
                    }
                }
            }
        }
    }

    public boolean hasUpdates() {
        for (int i = 0; i < getFileCount(); i++) {
            SpecFileEntry entry = getFile(i);
            if (entry.getUpdate())
                return true;
        }
        return false;
    }

    public String getName() {
        return null;
    }

}
