package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.boot.Boot;
import gov.va.med.hds.cd.mum.boot.IAbortManager;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.IStartupInfoFactory;
import gov.va.med.hds.cd.mum.boot.SilentAbortException;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;
import gov.va.med.hds.cd.mum.test.WinUtils;
import gov.va.med.hds.cd.mum.update.IPluginUpdater;
import gov.va.med.hds.cd.mum.update.IUpdateConstants;
import gov.va.med.hds.cd.mum.update.IUpdateNotifier;
import gov.va.med.hds.cd.mum.update.PluginClassFactory;
import gov.va.med.hds.cd.mum.update.UnauthorizedUpdateException;
import gov.va.med.hds.cd.mum.update.UpdateManagerFactory;
import gov.va.med.hds.cd.mum.utils.ChecksumCalculator;
import gov.va.med.hds.util.io.FileUtilities;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.net.URL;
import java.net.URLClassLoader;

import junit.framework.TestCase;

public class PassthroughTest
    extends TestCase
    implements
        IBootConstants,
        IMUMTestConstants,
        IUpdateConstants,
        IAbortManager,
        IUpdateNotifier,
        IStartupInfoFactory {

    public void testCompletePassthrough() throws Throwable {
//		If this crashes on Linux, uncommend the next to lines!
//        if (WinUtils.notWindows()) 
//            return;
        internalTestUpdateInProgress();
        intenralCompletePassthroughTest();
    }

    public void testCompletePassthroughUNC() throws Throwable {
        if (WinUtils.notWindows())
            return;
        String pc = "ISL-MERRILLJ2";
        if (!pc.equals(WinUtils.getMachineName()))
            return;
        StringBuffer buf = new StringBuffer(args[1]);
        int idx = buf.indexOf(":");
        buf.delete(0, idx + 1);
        args[1] = "\\\\ISL-MERRILLJ2\\C Drive" + buf.toString().replace('/', '\\');
        File file = new File(args[1]);
        if (file.exists())
            intenralCompletePassthroughTest();
    }

    private void internalTestUpdateInProgress() throws Throwable {
        error = null;
        IPluginUpdater updater = PluginClassFactory.newPluginUpdater();
        updater.beginUpdate(TestFileBuilder.getDestFile(), null, null, 0);
        try {
            Boot.main(parentLoader, args);
            assertNotNull(error);
            assertEquals(SilentAbortException.class, error.getClass());
            assertNotNull(error.getCause());
            assertEquals(UnauthorizedUpdateException.class, error.getCause().getClass());
            assertEquals(UPDATE_IN_PROGRESS_ERROR, error.getCause().getMessage());
            assertEquals(UPDATE_IN_PROGRESS_ERROR, fatalError);
            assertEquals(UPDATE_ABORTED_TITLE, fatalTitle);
        } finally {
            updater.endUpdate(null);
        }
    }

    public void intenralCompletePassthroughTest() throws Throwable {
        error = null;
        assertPlugin(mjEditor3);
        assertPlugin(mjNoteCore1);
        assertPlugin(mjLib1);
        assertPlugin(mjEncounters);
        assertPlugin(mjEncounterForms);
        assertPlugin(mjRemCore);
        assertPlugin(mjRemDialogs1);
        assertPlugin(mjRuntime);
        assertPlugin(mjNoteCore1);

        assertNoPlugin(mjEditor4);
        assertNoPlugin(mjLib2);
        assertNoPlugin(mjTemplates1);
        assertNoPlugin(mjTemplatesCore);
        assertNoPlugin(mjTemplateFields2);
        assertNoPlugin(mjRemDialogs2);
        assertNoPlugin(mjInMedOrders);
        assertNoPlugin(mjOutMedOrders);
        assertNoPlugin(mjMedCore);

        assertFalse(appFile.exists());

        Boot.main(parentLoader, args);
        if (error != null)
            throw error;

        assertPlugin(mjEditor4);
        assertPlugin(mjLib2);
        assertPlugin(mjTemplates1);
        assertPlugin(mjTemplatesCore);
        assertPlugin(mjTemplateFields2);
        assertPlugin(mjRemDialogs2);
        assertPlugin(mjInMedOrders);
        assertPlugin(mjOutMedOrders);
        assertPlugin(mjMedCore);

        assertTrue(appFile.exists());
        StringBuffer buf = new StringBuffer();
        BufferedReader reader = new BufferedReader(new FileReader(appFile));
        try {
            char[] buffer = new char[1024];
            int count;
            while ((count = reader.read(buffer)) != -1) {
                buf.append(buffer, 0, count);
            }
        } finally {
            reader.close();
        }
        assertEquals(mergedXML, buf.toString());
    }

    //********************
    //*** end of tests ***
    //********************

    public PassthroughTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        TestFileBuilder.ensureTestDirectories(true);
        TestFileBuilder.writeDummyXMLFile();
        TestFileBuilder.copyStartupAsLauncher(mjNoteCore1);
        appFile =
            new File(
                TestFileBuilder.getDestFile().getPath()
                    + File.separator
                    + "application.was.run.flag");
        if (appFile.exists())
            appFile.delete();
        error = null;
        Boot.setDoSplashScreen(false);
        Boot.assignAbortManager(this);
        Boot.assignStartupInfoFactory(this);
        Boot.assignStartupClass("mumtest.mum.AnotherTestStartup");
        oldForce = ClassLoaderManager.getForcedSharedLoader();
        ClassLoaderManager.setForceSharedLoader(false);
        if ((serverList != null) && (serverList.exists()))
            serverList.delete();
        ChecksumCalculator.setJarTestMode(true);
        parentLoader = new URLClassLoader(new URL[0], this.getClass().getClassLoader());
        UpdateManagerFactory.assignUpdateNotifier(this);
        fatalError = null;
        fatalTitle = null;
        root = TestFileBuilder.getDestFile();
        plugins = new File(root, PLUGIN_DIRECTORY);
        startup = new File(plugins, "VendorNoteEditor_0.0.1");
        args = new String[2];
        args[0] = TESTFILE_COMMAND_TAG + appFile.getPath();
        args[1] = TestFileBuilder.dummyXMLFileName();
        String settingsFileName = TestFileBuilder.dummyXMLFileName();
        int i = settingsFileName.lastIndexOf('.');
        settingsFileName = settingsFileName.substring(0, i) + SETTINGS_EXT;
        File settings = new File(settingsFileName);
        URL url =
            this.getClass().getClassLoader().getResource(
                "gov/va/med/hds/cd/mum/update/testdata/configurations/settings.xml");
        FileUtilities.copy(url, settings);
    }

    private File root;
    private File plugins;
    private File startup;
    private boolean oldForce;

    protected void tearDown() throws Exception {
        super.tearDown();
        TestFileBuilder.eraseTestDirectories(true);
        ChecksumCalculator.setJarTestMode(false);
        UpdateManagerFactory.assignUpdateNotifier(null);
        Boot.setDoSplashScreen(true);
        Boot.assignAbortManager(null);
        Boot.assignStartupInfoFactory(null);
        Boot.assignStartupClass(null);
        ClassLoaderManager.setForceSharedLoader(oldForce);
    }

    private String[] args;
    private Throwable error;
    private File appFile;
    private static File serverList;
    private ClassLoader parentLoader;

    public void abort(Throwable e, boolean warnUser) {
        error = e;
    }

    private void assertPlugin(int index) {
        assertTrue(getPluginFile(index).exists());
    }

    private void assertNoPlugin(int index) {
        assertFalse(getPluginFile(index).exists());
    }

    private File getPluginFile(int index) {
        return new File(
            TestFileBuilder.getDestFile().getPath()
                + File.separator
                + this.PLUGIN_DIRECTORY
                + File.separator
                + MOCKJAR_DATA[index][miDir]
                + File.separator
                + MANIFEST_FILE);
    }

    /**
     * @see gov.va.med.hds.cd.mum.update.IUpdateNotifier#fatalErrorMessage(java.lang.String, java.lang.String)
     */
    public void fatalErrorMessage(String message, String title) {
        fatalError = message;
        fatalTitle = title;
    }

    private String fatalError;
    private String fatalTitle;
    /**
     * @see gov.va.med.hds.cd.mum.update.IUpdateNotifier#fatalErrorMessage(java.lang.String)
     */
    public void fatalErrorMessage(String message) {
        fatalErrorMessage(message, UPDATE_ABORTED_TITLE);
    }
    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfoFactory#createStartupInfo(java.lang.String[])
     */
    public IStartupInfo createStartupInfo(String[] args) {
        TestStartupInfo info = new TestStartupInfo(args);
        info.setRootDirectory(root);
        info.setPluginsDirectory(plugins);
        info.setStartupDirectory(startup);
        return info;
    }

    private static final String mergedXML =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
            + WIN_EOL
            + "<settings>"
            + WIN_EOL
            + "  <testSetting1>one</testSetting1>"
            + WIN_EOL
            + "  <testSetting2 setting=\"two\">22222</testSetting2>"
            + WIN_EOL
            + "  <testSetting3 setting=\"three\">three</testSetting3>"
            + WIN_EOL
            + "</settings>";

}
