package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.VersionFormatException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IPluginConstants;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.loader.test.TestLinkedPluginLoader;
import gov.va.med.hds.cd.mum.loader.test.TestSharedClassLoader;
import gov.va.med.hds.cd.mum.test.TestUtils;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.IUpdateConstants;
import gov.va.med.hds.cd.mum.update.PluginClassFactory;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

public class PluginSpecTest
    extends TestCase
    implements IUpdateConstants, IPluginConstants, IBootConstants, IMUMConstants {

    public void testName()
        throws
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError,
            TreeDataException,
            InvalidConfigurationException {
        loadGoodConfig();
        nextPlugin();
        assertEquals("One Two Three Four Five", plugin.getName());
        nextPlugin();
        assertNull(plugin.getName());
        nextPlugin();
        assertEquals("Testing", plugin.getName());
        nextPlugin();
        assertNull(plugin);
    }

    public void testDirectory()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals("one.two.three.four.five_1.0.0", plugin.getDirectory());
        nextPlugin();
        assertEquals("one.two.three.four.five.six_0.0.0.whatever", plugin.getDirectory());
        nextPlugin();
        assertEquals("this.is.a.test_1.2.3.4", plugin.getDirectory());
        nextPlugin();
        assertNull(plugin);
    }

    public void testSourceDirectory()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals("testing", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("one.two.three.four.five.six_0.0.0.whatever", plugin.getSourceDirectory());
        nextPlugin();
        assertEquals("This Is A Test", plugin.getSourceDirectory());
    }

    public void testID()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals("one.two.three.four.five", plugin.getID());
        nextPlugin();
        assertEquals("one.two.three.four.five.six", plugin.getID());
        nextPlugin();
        assertEquals("this.is.a.test", plugin.getID());
    }

    public void testVersion()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals("1.0.0", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("0.0.0.whatever", plugin.getVersion().toString());
        nextPlugin();
        assertEquals("1.2.3.4", plugin.getVersion().toString());
    }

    public void testModule()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals("one.two.three", plugin.getModule());
        nextPlugin();
        assertEquals("some.other.module", plugin.getModule());
        nextPlugin();
        assertEquals("this.is.a.test", plugin.getModule());
    }

    public void testApplicationClass()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertNull(plugin.getApplicationClassName());
        nextPlugin();
        assertNull(plugin.getApplicationClassName());
        nextPlugin();
        assertEquals("some.pacakge.Application", plugin.getApplicationClassName());
    }

    public void testPublishAllClasses()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertFalse(plugin.publishAllClasses());
        nextPlugin();
        assertTrue(plugin.publishAllClasses());
        nextPlugin();
        assertFalse(plugin.publishAllClasses());
    }

    public void testPublished()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals(2, plugin.getPublished().size());
        assertTrue(plugin.getPublished().contains("this.is.a.test.SomeClass"));
        assertTrue(plugin.getPublished().contains("this.is.a.test.SomeOtherClass"));
        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
        nextPlugin();
        assertEquals(0, plugin.getPublished().size());
    }

    public void testFileEntries()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        assertEquals(2, plugin.getFileCount());
        assertFile(1232101, "manifest.xml", null);
        assertFile(
            1232102,
            "one.jar",
            new String[] {
                "pkg.one.two.three",
                "pkg.one.two.three.four",
                "pkg.one.two.four",
                "pkg.whatever" });
        nextPlugin();
        assertEquals(4, plugin.getFileCount());
        assertFile(1232103, "manifest.xml", null);
        assertFile(1232104, "one.jar", new String[] { "pkg.one.two.three", "pkg.this.is.a.test" });
        assertFile(1232105, "two.jar", null);
        assertFile(
            1232106,
            "three.jar",
            new String[] { "testing.default.package", DEFAULT_PACKAGE });
        nextPlugin();
        assertEquals(1, plugin.getFileCount());
        assertFile(1232107, "manifest.xml", null);
    }

    public void testRequiredPlugins()
        throws
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError,
            TreeDataException,
            InvalidConfigurationException {
        loadGoodConfig();
        nextPlugin();
        assertEquals(2, plugin.requiredPlugins().size());
        assertTrue(plugin.requiredPlugins().containsKey("some.other.plugin_7.34.16.whatever"));
        assertTrue(plugin.requiredPlugins().containsKey("third.party.plugin_1.0.0"));

        nextPlugin();
        assertEquals(0, plugin.requiredPlugins().size());

        nextPlugin();
        assertEquals(5, plugin.requiredPlugins().size());
        assertTrue(plugin.requiredPlugins().containsKey("ug1_1.0.0"));
        assertTrue(plugin.requiredPlugins().containsKey("ug2_2.0.0"));
        assertTrue(plugin.requiredPlugins().containsKey("ug3_3.0.0"));
        assertTrue(plugin.requiredPlugins().containsKey("ug4_4.0.0"));
        assertTrue(plugin.requiredPlugins().containsKey("ug5_5.0.0"));
    }

    public void testGetFilesAndURLs()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        File root = new File("/dir/subdir");
        loadGoodConfig();
        nextPlugin();
        File[] files = plugin.getFiles(root);
        assertFilesMatchURLs(files, plugin.getURLs(root));
        assertEquals(2, files.length);
        assertHasFile(files, "/dir/subdir/plugins/one.two.three.four.five_1.0.0/manifest.xml");
        assertHasFile(files, "/dir/subdir/plugins/one.two.three.four.five_1.0.0/one.jar");
        nextPlugin();
        files = plugin.getFiles(root);
        assertFilesMatchURLs(files, plugin.getURLs(root));
        assertEquals(4, files.length);
        assertHasFile(
            files,
            "/dir/subdir/plugins/one.two.three.four.five.six_0.0.0.whatever/manifest.xml");
        assertHasFile(
            files,
            "/dir/subdir/plugins/one.two.three.four.five.six_0.0.0.whatever/one.jar");
        assertHasFile(
            files,
            "/dir/subdir/plugins/one.two.three.four.five.six_0.0.0.whatever/two.jar");
        assertHasFile(
            files,
            "/dir/subdir/plugins/one.two.three.four.five.six_0.0.0.whatever/three.jar");
        nextPlugin();
        files = plugin.getFiles(root);
        assertFilesMatchURLs(files, plugin.getURLs(root));
        assertEquals(1, files.length);
        assertHasFile(files, "/dir/subdir/plugins/this.is.a.test_1.2.3.4/manifest.xml");
    }

    public void testSetClassLoader()
        throws
            InvalidConfigurationException,
            TreeDataException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError {
        loadGoodConfig();
        nextPlugin();
        try {
            plugin.setClassLoader(ClassLoader.getSystemClassLoader());
            fail("Should have thrown IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            assertEquals(INVALID_CLASS_LOADER, e.getMessage());
        }
        plugin.setClassLoader(new TestLinkedPluginLoader(null));
        plugin.setClassLoader(new TestSharedClassLoader(null));
    }

    public void testLoadManifests()
        throws
            InvalidConfigurationException,
            InvalidPluginException,
            IOException,
            ParserConfigurationException,
            FactoryConfigurationError,
            TreeDataException {
        loadGoodConfig();
        nextPlugin();
        assertNull(plugin.getPluginID());
        assertNull(plugin.getManifestXML());
        plugin.loadManifest(utils.getManifestURL(utils.MANIFEST1));
        ITreeData xml = plugin.getManifestXML();
        assertNotNull(xml);
        assertEquals(plugin.getID(), xml.getAttribute(xml.getRootID(), ID_ATTRIBUTE));
        assertEquals("one.two.three.four.five v1.0.0", plugin.getPluginID().toString());
        assertEquals("one", xml.getAttribute(xml.getRootID(), "testing"));

        try {
            plugin.loadManifest(utils.getManifestURL(utils.NO_PLUGIN));
            fail("Should have thrown InvalidPluginException");
        } catch (InvalidPluginException e) {
            utils.assertManifestError(e, INVALID_MANIFEST);
        }

        plugin.loadManifest(utils.getManifestURL(utils.MANIFEST2));
        xml = plugin.getManifestXML();
        assertEquals("two", xml.getAttribute(xml.getRootID(), "testing"));

        plugin.loadManifest(utils.getManifestURL(utils.MANIFEST3));
        xml = plugin.getManifestXML();
        assertEquals("three", xml.getAttribute(xml.getRootID(), "testing"));

    }

    public void testErrorConditions()
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        try {
            plugin = PluginClassFactory.newPluginSpec(null, -1);
            fail("Should have thrown an InvalidConfigurationException");
        } catch (InvalidConfigurationException e) {
            assertEquals(PLUGIN_NODE_MISSING, e.getMessage());
        }

        loadBadConfig();
        assertError(MISSING_DIRECTORY, noDirectory);
        assertError(INVALID_DIRECTORY_ENTRY, missingDirectory);
        assertError(INVALID_DIRECTORY_ENTRY, missingVersion);
        assertError(INVALID_DIRECTORY_ENTRY, badVersion1);
        Class verEx = VersionFormatException.class;
        assertError(INVALID_PLUGIN_VERSION, verEx, badVersion2);
        assertError(INVALID_PLUGIN_VERSION, verEx, badVersion3);
        assertError(INVALID_PLUGIN_VERSION, verEx, badVersion4);
        assertError(INVALID_MODULE_ENTRY, badModule);
        assertError(MULTIPLE_APPLICATION_ENTRIES, badApp1);
        assertError(MISSING_APPLICATION_CLASS, emptyAppClass);
        assertError(MISSING_APPLICATION_CLASS, missingAppClass);
        assertError(TOO_MANY_PUBLISHED_ENTRIES, badPublish1);
        assertError(NO_WILDCARD_ENTRIES, badPublish2);
        assertError(NO_WILDCARD_ENTRIES, badPublish3);
        assertError(NO_WILDCARD_ENTRIES, badPublish4);
        assertError(DUPLICATE_CLASS_ENTRIES, badPublish5);
        assertError(MISSING_PUBLISHED_CLASS, badPublish6);
        assertError(MISSING_PUBLISHED_CLASS, badPublish7);
        assertError(MISSING_MANIFEST_FILE, noFiles);
        assertError(MISSING_MANIFEST_FILE, noManifest);
        assertError(INVALID_FILE_ENTRY, noFileCRC);
        assertError(INVALID_FILE_ENTRY, emptyFileCRC);
        assertError(INVALID_FILE_ENTRY, noFilename);
        assertError(INVALID_FILE_ENTRY, emptyFilename);
        Class numEx = NumberFormatException.class;
        assertError(INVALID_FILE_ENTRY, numEx, badCRC1);
        assertError(INVALID_FILE_ENTRY, numEx, badCRC2);
        assertError(INVALID_FILE_ENTRY, numEx, badCRC3);
    }

    //********************
    //*** end of tests ***
    //********************

    public PluginSpecTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        utils = new TestUtils(this);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    private void loadBadConfig()
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        loadConfig(utils.BAD_CONFIG);
    }

    private void loadGoodConfig()
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        loadConfig(utils.CONFIG1);
    }

    private void loadConfig(String configFileName)
        throws IOException, ParserConfigurationException, FactoryConfigurationError, TreeDataException {
        xml = utils.getConfigDocument(configFileName);
        xmlRoot = xml.getRootID();
        pluginIndex = -1;
    }

    private void nextPlugin() throws InvalidConfigurationException {
        pluginIndex = xml.findChildID(xmlRoot, PLUGIN_ELEMENT, pluginIndex);
        if (pluginIndex >= 0)
            plugin = PluginClassFactory.newPluginSpec(xml, pluginIndex);
        else
            plugin = null;
    }

    private void getPlugin(int index) throws InvalidConfigurationException {
        if (index < xml.getChildCount(xmlRoot))
            plugin = PluginClassFactory.newPluginSpec(xml, xml.getChildID(xmlRoot, index));
        else
            plugin = null;
    }

    private int noDirectory = 0;
    private int missingDirectory = 1;
    private int missingVersion = 2;
    private int badVersion1 = 3;
    private int badVersion2 = 4;
    private int badVersion3 = 5;
    private int badVersion4 = 6;
    private int badModule = 7;
    private int badApp1 = 8;
    private int badPublish1 = 9;
    private int badPublish2 = 10;
    private int badPublish3 = 11;
    private int badPublish4 = 12;
    private int badPublish5 = 13;
    private int badPublish6 = 14;
    private int badPublish7 = 15;
    private int emptyAppClass = 16;
    private int missingAppClass = 17;
    private int noFiles = 18;
    private int noManifest = 19;
    private int noFileCRC = 20;
    private int emptyFileCRC = 21;
    private int noFilename = 22;
    private int emptyFilename = 23;
    private int badCRC1 = 24;
    private int badCRC2 = 25;
    private int badCRC3 = 26;

    private IPluginSpec plugin;
    private ITreeData xml;
    private int xmlRoot;
    private int pluginIndex;
    private TestUtils utils;

    private void assertError(int index) {
        assertError("", null, index);
    }

    private void assertError(String message, int index) {
        assertError(message, null, index);
    }

    private void assertError(String message, Class exceptionClass, int index) {
        try {
            getPlugin(index);
            fail("Should have thrown an InvalidConfigurationException");
        } catch (InvalidConfigurationException e) {
            if (message.length() > 0)
                assertEquals(message, e.getMessage());
            if (exceptionClass != null)
                assertTrue(e.getCause().getClass() == exceptionClass);
        }
    }

    public void assertFile(long crc, String fileName, String[] pkgs) {
        assertFile(plugin, crc, fileName, pkgs);
    }

    public static void assertFile(IPluginSpec plugin, long crc, String fileName, String[] pkgs) {
        for (int i = 0; i < plugin.getFileCount(); i++) {
            if ((plugin.getFile(i).getCRC() == crc)
                && (plugin.getFile(i).getFileName().equals(fileName))) {
                if (pkgs == null)
                    assertNull(plugin.getFile(i).getPackages());
                else {
                    assertNotNull(plugin.getFile(i).getPackages());
                    assertEquals(pkgs.length, plugin.getFile(i).getPackages().size());
                    for (int j = 0; j < pkgs.length; j++) {
                        assertTrue(
                            "Should have had package " + pkgs[j],
                            plugin.getFile(i).getPackages().contains(pkgs[j]));
                    }
                }
                return;
            }
        }
        fail("Should have had file " + fileName + " with a CRC of " + Long.toString(crc));
    }

    private void assertHasFile(File[] files, String fileName) {
        String file = fileName.replace('/', File.separatorChar);
        for (int i = 0; i < files.length; i++) {
            if (files[i].getPath().equals(file))
                return;
        }
        for (int i = 0; i < files.length; i++)
            System.out.println(files[i].toString());
        fail("Should have had file " + file);
    }

    private void assertFilesMatchURLs(File[] files, URL[] urls) throws MalformedURLException {
        assertEquals(files.length, urls.length);
        for (int i = 0; i < files.length; i++) {
            assertEquals(files[i].toURL().toString(), urls[i].toString());
        }
    }

}
