package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartup;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.SilentAbortException;
import gov.va.med.hds.cd.mum.boot.StatusNotifierFactory;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.update.ApplicationLauncherFactory;
import gov.va.med.hds.cd.mum.update.IApplicationLauncher;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.IPluginUsageTracker;
import gov.va.med.hds.cd.mum.update.IUpdateManager;
import gov.va.med.hds.cd.mum.update.PluginClassFactory;
import gov.va.med.hds.cd.mum.update.Startup;
import gov.va.med.hds.cd.mum.update.UpdateManagerFactory;

import java.net.URL;

import junit.framework.TestCase;

public class StartupTest
    extends TestCase
    implements IUpdateManager, IApplicationLauncher, IPluginUsageTracker, IBootConstants {

    public void testNormalStartup() throws Throwable {
        TestStartupInfo info = new TestStartupInfo();
        startup.run(info);
        assertTrue(updated);
        assertTrue(applicationStarted);
        assertEquals(target, trackedPlugins);
        assertEquals(current, pluginsToDelete);
    }

    public void testSilentUpdate() throws Throwable {
        TestStartupInfo info = new TestStartupInfo(new String[] { SILENT_UPDATE_TAG });
        try {
            startup.run(info);
            fail("Should have thrown a SilentAbortException");
        } catch (SilentAbortException e) {
        }
        assertTrue(updated);
        assertFalse(applicationStarted);
        assertEquals(target, trackedPlugins);
        assertEquals(current, pluginsToDelete);
    }

    public void testAbort() throws Throwable {
        abort = true;
        try {
            startup.run(null);
            fail("Should have thrown InvalidConfigurationException");
        } catch (InvalidConfigurationException e) {
        }
        assertTrue(updated);
        assertFalse(applicationStarted);
    }

    //********************
    //*** end of tests ***
    //********************

    public StartupTest(String arg0) {
        super(arg0);
    }

    private IStartup startup;
    private boolean abort;
    private boolean selfUpdate;
    private boolean updated;

    protected void setUp() throws Exception {
        super.setUp();
        updated = false;
        abort = false;
        selfUpdate = false;
        applicationStarted = false;
        startup = new Startup();
        UpdateManagerFactory.assignUpdateManager(this);
        ApplicationLauncherFactory.assignApplicationLauncher(this);
        PluginClassFactory.assignPluginUsageTracker(this);
        trackedPlugins = null;
        pluginsToDelete = null;
        target = new ListMap();
        current = new ListMap();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        UpdateManagerFactory.assignUpdateManager(null);
        ApplicationLauncherFactory.assignApplicationLauncher(null);
        StatusNotifierFactory.assignStatusNotifier(null);
        PluginClassFactory.assignPluginUsageTracker(null);
    }

    public void update(IStartupInfo info) throws InvalidConfigurationException {
        updated = true;
        if (abort)
            throw new InvalidConfigurationException();
    }

    private boolean applicationStarted;

    public URL getConfigFileURL(IStartupInfo info) throws InvalidConfigurationException {
        return null;
    }
    public ListMap getSpecs() {
        return target;
    }
    public IPluginSpec getStartupSpec() {
        return null;
    }

    public void launchApplication(
        URL configFile,
        ListMap specs,
        IStartupInfo info,
        IPluginSpec startupSpec)
        throws Throwable {
        applicationStarted = true;
    }

    public void close() {
    }

    public ListMap getCurrentPlugins() {
        return current;
    }

    private ListMap trackedPlugins;
    private ListMap pluginsToDelete;
    private ListMap target;
    private ListMap current;
    /**
     * @see gov.va.med.hds.cd.mum.update.IPluginUsageTracker#trackPluginUsage(java.util.List)
     */
    public void trackPluginUsage(ListMap plugins) {
        trackedPlugins = plugins;
    }
    /**
     * @see gov.va.med.hds.cd.mum.update.IPluginUsageTracker#deleteUnusedPlugins(java.util.List)
     */
    public void deleteUnusedPlugins(ListMap plugins) {
        pluginsToDelete = plugins;
    }
}
