package gov.va.med.hds.cd.mum.update.test;

import gov.va.med.hds.cd.mum.boot.IAbortManager;
import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.test.IMUMTestConstants;
import gov.va.med.hds.cd.mum.update.ConfigReaderFactory;
import gov.va.med.hds.cd.mum.update.IExistingPlugin;
import gov.va.med.hds.cd.mum.update.IPluginSpec;
import gov.va.med.hds.cd.mum.update.IPluginUpdater;
import gov.va.med.hds.cd.mum.update.IUpdateConstants;
import gov.va.med.hds.cd.mum.update.IUpdateListener;
import gov.va.med.hds.cd.mum.update.IUpdateManager;
import gov.va.med.hds.cd.mum.update.PluginClassFactory;
import gov.va.med.hds.cd.mum.update.UpdateException;
import gov.va.med.hds.cd.mum.update.UpdateManagerFactory;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

public class UpdateManagerTest
    extends TestCase
    implements IUpdateConstants, IPluginUpdater, IAbortManager, IStatusNotifier, IMUMTestConstants {

    public void testFullPassthrough() throws Throwable {
        initNormalTest();
        assertEquals(0, updatePendingCallCount);
        reader.setStartupPlugin(mjEncounters);
        manager.update(info);
        if (error != null)
            throw error;
        assertEquals(1, updatePendingCallCount);
        assertEquals(PLUGIN_UPDATE_MESSAGE, updatePendingMessage);
        assertEquals(this, updatePendingNotifier);
        assertEquals(10700, updatePendingSize);
        assertEquals(10, updatedPlugins.size());
        assertTrue(updateStarted);
        assertTrue(updateStopped);
        assertPlugin(mjEditor4);
        assertPlugin(mjNoteCore1);
        assertPlugin(mjLib2);
        assertPlugin(mjTemplates1);
        assertPlugin(mjTemplatesCore);
        assertPlugin(mjTemplateFields2);
        assertPlugin(mjRemDialogs2);
        assertPlugin(mjInMedOrders);
        assertPlugin(mjOutMedOrders);
        assertPlugin(mjMedCore);
        assertTrue(manager.getConfigFileURL(info).getPath().endsWith("SomeConfigFile.xml"));
        assertEquals(GOOD_CONFIG.length, manager.getSpecs().size());
        IPluginSpec startup = manager.getStartupSpec();
        for (int i = 0; i < manager.getSpecs().size(); i++) {
            boolean bad = true;
            IPluginSpec spec = (IPluginSpec) manager.getSpecs().get(i);
            for (int j = 0; j < GOOD_CONFIG.length; j++) {
                if (spec.getDirectory().equals(MOCKJAR_DATA[GOOD_CONFIG[j]][miDir])) {
                    bad = false;
                    break;
                }
            }
            if (bad)
                fail("Should have had " + spec.getDirectory());
        }
        assertEquals(startup, manager.getStartupSpec());

        assertEquals(EXISTING_CONFIG.length, manager.getCurrentPlugins().size());
        for (int i = 0; i < manager.getCurrentPlugins().size(); i++) {
            boolean bad = true;
            IExistingPlugin plugin = (IExistingPlugin) manager.getCurrentPlugins().get(i);
            for (int j = 0; j < EXISTING_CONFIG.length; j++) {
                if (plugin.getDirectory().equals(MOCKJAR_DATA[EXISTING_CONFIG[j]][miDir])) {
                    bad = false;
                    break;
                }
            }
            if (bad)
                fail("Should have had " + plugin.getDirectory());
        }

        for (int i = 0; i < manager.getSpecs().size(); i++) {
            IPluginSpec spec = (IPluginSpec) manager.getSpecs().get(i);
            List keys = spec.requiredPlugins().getKeys();
            for (int j = 0; j < keys.size(); j++) {
                String dir = (String) keys.get(j);
                Object obj = spec.requiredPlugins().get(dir);
                assertTrue(obj instanceof IPluginSpec);
                assertEquals(dir, ((IPluginSpec) obj).getDirectory());
            }
        }

        assertNotNull(info.getSettings());
        assertNotNull(info.getSettings().data());
        assertEquals(
            "settings",
            info.getSettings().data().getName(info.getSettings().data().getRootID()));
    }

    //********************
    //*** end of tests ***
    //********************

    public UpdateManagerTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        error = null;
        locator = new MockPluginLocator();
        reader = new MockConfigReader();
        manager = UpdateManagerFactory.newUpdateManager();
        ConfigReaderFactory.assignConfigReader(reader);
        PluginClassFactory.assignPluginLocator(locator);
        PluginClassFactory.assignPluginUpdater(this);
        updatedPlugins = new ArrayList();
        info = new TestStartupInfo();
        info.setRootDirectory(new File("somedir"));
        info.setSplashScreen(this);
        updatePendingCallCount = 0;
        updatePendingMessage = null;
        updatePendingNotifier = null;
        updatePendingSize = 0;
        statusMessageHidden = false;
        statusBarHidden = false;
        updateStarted = false;
        updateStopped = false;
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        ConfigReaderFactory.assignConfigReader(null);
        PluginClassFactory.assignPluginLocator(null);
        PluginClassFactory.assignPluginUpdater(null);
    }

    private void initNormalTest() {
        info.setArgs(new String[] { OPTION_PREFIX + "CommandUnknownToTheMUM", configFile });
    }

    private TestStartupInfo info;
    private MockPluginLocator locator;
    private MockConfigReader reader;
    private IUpdateManager manager;
    private List updatedPlugins;

    public File[] update(URL fromPluginDir, File toPluginDir, IPluginSpec plugin)
        throws UpdateException {
        updatedPlugins.add(plugin.getDirectory());
        File[] temp = new File[plugin.getFileCount()];
        for (int i = 0; i < plugin.getFileCount(); i++) {
            temp[i] = new File(plugin.getFile(i).getFileName());
        }
        return temp;
    }

    public void beginUpdate(
        File rootDirectory,
        String overrideMessage,
        IStatusNotifier notifier,
        long size) {
        updatePendingCallCount++;
        updatePendingMessage = overrideMessage;
        updatePendingNotifier = notifier;
        updatePendingSize = size;
        updateStarted = true;
    }
    private int updatePendingCallCount;
    private boolean updateStarted;
    private boolean updateStopped;
    private String updatePendingMessage;
    private IStatusNotifier updatePendingNotifier;
    private long updatePendingSize;
    private boolean statusMessageHidden;
    private boolean statusBarHidden;
    private Throwable error;
    public void abort(Throwable e, boolean warnUser) {
        error = e;
    }

    private void assertPlugin(int index) {
        assertTrue(updatedPlugins.contains(MOCKJAR_DATA[index][miDir]));
    }

    private String configFile = "SomeConfigFile.xml";

    private String linkPath = "gov/va/med/hds/cd/mum/update/testdata/links/";

    private String getLinkPath(String fileName) {
        URL url = this.getClass().getClassLoader().getResource(linkPath + fileName + XML_EXT);
        String result = url.getPath();
        try {
            url = new URL(result);
            result = url.getPath();
        } catch (MalformedURLException e) {
        }
        return result;
    }

    private int select;
    private String selectedName;
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#showProgressBar(int)
     */
    public void showProgressBar(long maxValue) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressBar(int)
     */
    public void resetProgressBar(long value) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressBar(int, java.lang.String)
     */
    public void resetProgressBar(long value, String message) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#hideProgressBar()
     */
    public void hideProgressBar() {
        statusBarHidden = true;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#displayMessage(java.lang.String)
     */
    public void displayMessage(String message) {
        statusMessageHidden = (message == null);
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#isProgressBarDisplayed()
     */
    public boolean isProgressBarDisplayed() {
        return false;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#getProgress()
     */
    public long getProgress() {
        return 0;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#getProgressMax()
     */
    public long getProgressMax() {
        return 0;
    }
    /**
     * @see gov.va.med.hds.cd.mum.utils.ISplashScreen#close()
     */
    public void close() {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressBar(long)
     */
    public void updateProgressBar(long value) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressMessage(java.lang.String)
     */
    public void updateProgressMessage(String message) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.update.IPluginUpdater#assignUpdateListener(gov.va.med.hds.cd.mum.update.IUpdateListener)
     */
    public void assignUpdateListener(IUpdateListener listener) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.update.IPluginUpdater#endUpdate(gov.va.med.hds.cd.mum.status.IStatusNotifier)
     */
    public void endUpdate(IStatusNotifier notifier) {
        updateStopped = true;
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#showProgressBar(long, java.lang.String)
     */
    public void showProgressBar(long maxValue, String message) {
    }
    /**
     * @see gov.va.med.hds.cd.mum.status.IStatusNotifier#updateProgressBarMax(long)
     */
    public void updateProgressBarMax(long value) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setVersion(java.lang.String)
     */
    public void setVersion(String version) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setShowCancelButton(boolean)
     */
    public void showCancelButton(boolean value) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#wasCancelButtonPressed()
     */
    public boolean isCanceled() {
        return false;
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setCancelMessage(java.lang.String)
     */
    public void setCancelMessage(String message) {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#beginDisplayUpdate()
     */
    public void beginDisplayUpdate() {
    }

    /* (non-Javadoc)
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#endDisplayUpdate()
     */
    public void endDisplayUpdate() {
    }
}