package gov.va.med.hds.cd.mum.utils.test;

import gov.va.med.hds.cd.mum.boot.Boot;
import gov.va.med.hds.cd.mum.boot.IAbortManager;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartup;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.IStartupInfoFactory;
import gov.va.med.hds.cd.mum.boot.loader.SharedClassLoader;
import gov.va.med.hds.cd.mum.boot.test.TestStartupInfo;
import gov.va.med.hds.cd.mum.boot.utils.IIgnoreStartup;
import gov.va.med.hds.cd.mum.loader.test.TestSharedClassLoader;
import gov.va.med.hds.cd.mum.test.TestFileBuilder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;

import junit.framework.TestCase;

public class DummyJarTests
	extends TestCase
	implements
		IAbortManager,
		IStartup,
		IIgnoreStartup,
		IBootConstants,
		IStartupInfoFactory {

	public void testStartupClassLoader()
		throws
			NoSuchMethodException,
			FileNotFoundException,
			IOException,
			ClassNotFoundException {
		testCount++;
		initJar();

		File jar2 =
			TestFileBuilder.createTestJar(
				true,
				"AnotherTestJar.jar",
				null,
				null,
				new String[] { update2ClassFile },
				new String[] { update2Class });

		loader = new TestSharedClassLoader(this.getClass().getClassLoader());
		loader.registerFile(jarFile);
		loader.registerFile(jar2);

		assertTrue(jarFile.exists());
		assertNotNull(loader);
		Class cls = loader.loadClass(dummyClass);
		assertNotNull(cls);

		Method meth = cls.getMethod("TestMethod", null);
		assertNotNull(meth);

		cls = loader.loadClass(updateClass);
		assertNotNull(cls);
		assertHasStartup(cls);

		cls = loader.loadClass(update2Class);
		assertNotNull(cls);
		assertHasStartup(cls);
	}

	public void testPassThrough() throws Throwable {
		testCount++;
		initJar();
		Boot.assignAbortManager(this);
		try {
			assertNull(error);
			assertFalse(flagFile.exists());
			Boot.assignStartupClass(updateClass);
			Boot.main(
				this.getClass().getClassLoader(),
				new String[] { "-commandUnknowToTheBoot", });
			if (error != null)
				throw error;
			assertTrue(flagFile.exists());
		} finally {
			Boot.assignAbortManager(null);
			Boot.assignStartupClass(null);
		}
	}

	//********************
	//*** end of tests ***
	//********************

	public DummyJarTests(String arg0) {
		super(arg0);
	}

	private File jarFile = null;
	private SharedClassLoader loader;

	private static int testCount = 0;
	private static int maxTestCount =
		TestFileBuilder.numTests(DummyJarTests.class);

	protected void setUp() throws Exception {
		super.setUp();
		Boot.setDoSplashScreen(false);
		Boot.assignStartupInfoFactory(this);
		if (testCount == 0) {
			TestFileBuilder.initRootTestDirs();
		}
		error = null;
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		if (testCount == maxTestCount) {
			TestFileBuilder.eraseTestDirectories(true);
			testCount = 0;
		}
		Boot.setDoSplashScreen(true);
		Boot.assignStartupInfoFactory(null);
	}

	private File flagFile;

	private String jarName = "mum.jar";

	private String dummyClass = "mumtest.boot.Dummy";
	private String updateClass = "mumtest.boot.Update";
	private String update2Class = "mumtest.boot.Update2";

	private String rootPath = "gov/va/med/hds/cd/mum/utils/test/testfiles/";
	private String dummyClassFile = rootPath + "Dummy.cls";
	private String updateClassFile = rootPath + "Update.cls";
	private String update2ClassFile = rootPath + "Update2.cls";

	private String[] names = new String[] { dummyClass, updateClass };

	private String[] resources =
		new String[] { dummyClassFile, updateClassFile };

	private void initJar()
		throws IOException, ClassNotFoundException, FileNotFoundException {
		if (jarFile == null) {
			jarFile =
				TestFileBuilder.createTestJar(
					true,
					jarName,
					null,
					null,
					resources,
					names);
			flagFile = new File(jarFile.getParentFile(), "UpdateRun.flag");
		}
	}

	private Throwable error;

	private void assertHasStartup(Class cls) {
		Class[] interfaces = cls.getInterfaces();
		boolean boom = true;
		for (int i = 0; i < interfaces.length; i++) {
			if (interfaces[i].getName().equals(IStartup.class.getName())) {
				boom = false;
				break;
			}
		}
		if (boom)
			fail(
				cls
					+ "should have shown "
					+ IStartup.class.getName()
					+ " implementation");
	}

	public void abort(Throwable e, boolean warnUser) {
		error = e;
	}

	public void run(IStartupInfo info) {
	}

	/**
	 * @see gov.va.med.hds.cd.mum.boot.IStartupInfoFactory#createStartupInfo(java.lang.String[])
	 */
	public IStartupInfo createStartupInfo(String[] args) {
		TestStartupInfo info = new TestStartupInfo(args);
		info.setStartupDirectory(jarFile.getParentFile());
		return info;
	}
}