package gov.va.med.hds.cd.mum;

import gov.va.med.hds.cd.mum.boot.utils.KeyedList;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;

/**
 * The core specification for plugin classes needing dependency information.
 */
public interface IDependentPlugin extends IBasePlugin, IPluginExceptionManager {

	/**
	 * The name of the plugin's module.  When one plugin tries to load classes from another 
	 * plugin, and both plugins have the same module name, all classes are accessible to that
	 * plugin (i.e. the published section of the manifest is ignored).  
	 * @return String
	 */
	String getModule();

	/**
	 * The name of the class implementing the IApplication interface.  If the
	 * plugin does not have any classes implementing the IApplication interface,
	 * this returns null.  Only one plugin can have an application class for any
	 * given configuration.
	 * @return String
	 */
	String getApplicationClassName();

	/**
	 * Returns a collection of Strings representing class names published by the plugin. Other 
	 * plugins with dependencies on this plugin can access only those classes contained within
	 * the Published list.  Plugins of the same module ignore this list.  
	 * @return Collection
	 */
	SimpleList getPublished();

	/**
	 * Returns true when all classes in the plugin are published.  This
	 * should generally be used only for third party libraries where access to
	 * all classes in the plugin is expected.
	 * @return boolean
	 */
	boolean publishAllClasses();

	/**
	 * Returns the PluginID associated with this plugin.
	 * @return PluginID
	 */
	PluginID getPluginID();
	
	/**
	 * The directory where the plugin is located in the file server's plugins directory.
	 * @return String
	 */
	String getSourceDirectory();
	
	/**
     * Returns the actual list of IDependentPlugin objects that satisfy all plugin
     * dependencies, including those inherited from exports 
     */
    KeyedList requiredPlugins();
    
	/**
	 * Returns the human readable plugin name
	 */
	String getName();
}
