package gov.va.med.hds.cd.mum;

import java.util.HashMap;
import java.util.Map;

/**
 * Represents the base information about a plugin - used for dependency chacking
 */
public class PluginID {

	private static Map map = new HashMap();
	private static StringBuffer buf = new StringBuffer();
	private String id;
	private String uniqueID;
	private Version version;

	/**
	 * Constructor for PluginID.
	 */
	private PluginID(String name, String id, String version) {
		this(name, id, new Version(version));
	}

	/**
	 * Constructor for PluginID.
	 */
	private PluginID(String name, String id, Version version) {
		super();
		this.id = name;
		this.uniqueID = id;
		this.version = version;
	}

	/**
	 * Used to retrieve the correct PluginID object.  Ensures that there is only
	 * one PluginID object for any give plugin name and specific version.
	 * @param name
	 * @param version
	 * @return PluginID
	 */
	public static PluginID getPluginID(String name, String version) {
		String id = getUniqueID(name, version);
		PluginID result = (PluginID) map.get(id);
		if (result == null) {
			result = new PluginID(name, id, version);
			map.put(id, result);
		}
		return result;
	}

	/**
	 * Used to retrieve the correct PluginID object.  Ensures that there is only
	 * one PluginID object for any give plugin name and specific version.
	 * @param name
	 * @param version
	 * @return PluginID
	 */
	public static PluginID getPluginID(String name, Version version) {
		String id = getUniqueID(name, version.toString());
		PluginID result = (PluginID) map.get(id);
		if (result == null) {
			result = new PluginID(name, id, version);
			map.put(id, result);
		}
		return result;
	}

	private static String getUniqueID(String name, String version) {
		if ((name == null) || (version == null))
			throw new NullPointerException();
		if ((name.length() == 0) || (version.length() == 0))
			throw new IllegalArgumentException();
		buf.setLength(0);
		buf.append(name);
		buf.append(" v");
		buf.append(version);
		return buf.substring(0, buf.length());
	}

	/**
	 * Clears the list of existing PluginID objects - used mainly for unit
	 * testing.
	 */
	public static void reset() {
		map.clear();
	}

	/**
	 * Returns the plugin's name
	 * @return String
	 */
	public String getID() {
		return id;
	}

	/**
	 * Returns the plugin version
	 * @return Version
	 */
	public Version getVersion() {
		return version;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return uniqueID;
	}
}
