/*
 * Created on May 2, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.mum;

import gov.va.med.hds.cd.mum.boot.IPluginConstants;


/**
 * @author vhaislmerrij
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public abstract class VersionMatch {

	public static final VersionMatch GREATER_OR_EQUAL;
	public static final VersionMatch COMPATIBLE;
	public static final VersionMatch EQUIVALENT;
	public static final VersionMatch PERFECT;
	public static final VersionMatch DEFAULT_MATCH_CONDITION;

	private static final VersionMatch[] conditions;

	static {
		GREATER_OR_EQUAL = new GreaterOrEqualsVersionMatch();
		COMPATIBLE = new CompatibleVersionMatch();
		EQUIVALENT = new EquilaventVersionMatch();
		PERFECT = new PerfectVersionMatch();

		DEFAULT_MATCH_CONDITION = GREATER_OR_EQUAL;

		conditions =
			new VersionMatch[] {
				GREATER_OR_EQUAL,
				COMPATIBLE,
				EQUIVALENT,
				PERFECT};
	}

	private String condition;

	private VersionMatch(String condition) {
		this.condition = condition;
	}

	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		if (!(obj instanceof VersionMatch))
			return false;
		return condition.equals(((VersionMatch) obj).condition);
	}

	public int hashCode() {
		return condition.hashCode();
	}

	public String toString() {
		return condition;
	}

	public static VersionMatch getVersionMatch(String condition) {
		if ((condition == null) || (condition.length() == 0))
			return DEFAULT_MATCH_CONDITION;
		for (int i = 0; i < conditions.length; i++) {
			if (conditions[i].condition.equals(condition))
				return conditions[i];
		}
		throw new VersionFormatException(
			IMUMConstants.UNKNOWN_VERSION_MATCH_ERROR + condition + "'");
	}

	public abstract boolean matches(Version source, Version matching);

	private static class GreaterOrEqualsVersionMatch extends VersionMatch {

		private GreaterOrEqualsVersionMatch() {
			super(IPluginConstants.GREATER_OR_EQUAL_MATCH_CONDITION);
		}

		public boolean matches(Version source, Version matching) {
			return source.isGreaterOrEqualTo(matching);
		}
	}

	private static class CompatibleVersionMatch extends VersionMatch {

		private CompatibleVersionMatch() {
			super(IPluginConstants.COMPATIBLE_MATCH_CONDITION);
		}

		public boolean matches(Version source, Version matching) {
			return source.isCompatibleWith(matching);
		}
	}

	private static class EquilaventVersionMatch extends VersionMatch {

		private EquilaventVersionMatch() {
			super(IPluginConstants.EQUIVALENT_MATCH_CONDITION);
		}

		public boolean matches(Version source, Version matching) {
			return source.isEquivalentTo(matching);
		}
	}

	private static class PerfectVersionMatch extends VersionMatch {

		private PerfectVersionMatch() {
			super(IPluginConstants.PERFECT_MATCH_CONDITION);
		}

		public boolean matches(Version source, Version matching) {
			return source.isPerfect(matching);
		}
	}
}
