package gov.va.med.hds.cd.mum.boot;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.LookAndFeelLauncher;

/**
 * AbortManager is used when a fatal error has occured that will result in the
 * terminition of the program.
 */
public class AbortManager implements IAbortManager {

    /**
     * @see gov.va.med.hds.cd.mum.boot.IAbortManager#abort(java.lang.Throwable, boolean)
     * @param e - if this error is an instance of SilentAbortException, no error
     * message will be displayed to the user.
     */
    public void abort(Throwable e, boolean warnUser) {
        if ((warnUser) && (!(e instanceof SilentAbortException))) {
            LookAndFeelLauncher.initDefaultLookAndFeel();
            String message = e.getMessage();
            if (message == null)
                message = "An unexpected error occurred.";
            message = showAbortDialog(e, message);
        }
        System.exit(0);
    }


    private String showAbortDialog(Throwable e, String message) {
        if ((message == null) || (message.length() == 0)) {
            if (e.getCause() != null)
                message = e.getCause().getMessage();
        }
        ExceptionDialog.showExceptionDialog(IBootConstants.ABORT_MESSAGE, message, e);
        return message;
    }

}
