package gov.va.med.hds.cd.mum.boot;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;

/**
 * ExceptionDialog is used to display errors to the user.
 */
public class ExceptionDialog extends JDialog {
    private static final int DIALOG_MIN_W = 400;
    private static final int MSG_PNL_MIN_H = 50;
    private static final int MSG_PNL_PREF_H = 50;
    private static final int MSG_PNL_MIN_W = DIALOG_MIN_W;
    private static final int DTLS_PNL_MIN_H = 50;
    private static final int DTLS_PNL_MIN_W = DIALOG_MIN_W;
    private static final int DIALOG_MIN_H = MSG_PNL_MIN_H + 20;
    private static final String HIDE_DETAILS_TXT = "Hide Details";
    private static final String SHOW_DETAILS_TXT = "Show Details";
    private static final String DETAILS_BTN_TEXT = "Details";
    private static final String DETAILS_LBL_TEXT = "Message Details";
    private static final String OK_BTN_TEXT = "OK";
    private static final String DEFAULT_DETAILS_MESSAGE = "No stacktrace available";
    private static HiddenFrame hiddenFrame = new HiddenFrame();
    private static ExceptionDialog defaultDialog;
    private UnselectableTextArea txtMessage = new UnselectableTextArea();
    private JPanel btnPnl = new JPanel();
    private JPanel messagePnl = new JPanel();
    private JPanel detailPnl = new JPanel();
    private JScrollPane scrollPane;
    private JTextArea txtDetails = new JTextArea();
    private JButton btnOK = new JButton();
    private JButton btnDetails = new JButton();
    private JLabel detailsLbl = new JLabel(DETAILS_LBL_TEXT);
    private String message;
    private Throwable exception;
    private String[] detailText = { SHOW_DETAILS_TXT, HIDE_DETAILS_TXT };
    private boolean[] detailEnabled = { false, true };
    private int detailIdx = 0;

    private ExceptionDialog(String title) {
        super(hiddenFrame, true);
        hiddenFrame.setTitle(title);
        hiddenFrame.setName(title);
        jbInit();
    }

    public static void showExceptionDialog(String title, String message, Throwable e) {
        if (defaultDialog == null) {
            defaultDialog = new ExceptionDialog(title);
        }

        defaultDialog.setTitle(title);
        defaultDialog.setName(title);
        defaultDialog.setMessage(message);
        defaultDialog.setException(e);
        defaultDialog.setVisible(true);
    }

    public String getMessage() {
        return message;
    }

    public Throwable getException() {
        return exception;
    }

    public void setMessage(String message) {
        this.message = message;
        txtMessage.setText(message);
    }

    public void setException(Throwable e) {
        this.exception = e;
        txtDetails.setText(readFromException(e));
    }

    private String readFromException(Throwable e) {
        String text = DEFAULT_DETAILS_MESSAGE;
        try {
            StringWriter writer = new StringWriter();
            PrintWriter pwriter = new PrintWriter(writer);
            e.printStackTrace(pwriter);
            text = writer.toString();
            text =  text.replaceAll("\r","");
            writer.close();
        } catch (Exception ex) {
            //	  Log.info("ExceptionDialog: exception suppressed.");
        }

        return text;
    }

    private void jbInit() {
        BorderLayout borderLayout = new BorderLayout();
        this.getContentPane().setLayout(borderLayout);

        Dimension maxDim = getToolkit().getScreenSize();
        maxDim.setSize(maxDim.getWidth() * 0.9, maxDim.getHeight() * 0.9);

        Dimension msgDimMax = new Dimension((int) maxDim.getWidth(),
                                            (int) (maxDim.getHeight() * 0.2));
        Dimension msgDimMin = new Dimension(MSG_PNL_MIN_W, MSG_PNL_MIN_H);
        Dimension msgDimPref = new Dimension((int) (msgDimMin.getWidth()), MSG_PNL_PREF_H);

        Dimension dtlsDimMax = new Dimension((int) maxDim.getWidth(),
                                             (int) (maxDim.getHeight() * 0.6));
        Dimension dtlsDimMin = new Dimension(DTLS_PNL_MIN_W, DTLS_PNL_MIN_H);
        Dimension dtlsDimPref = new Dimension((int) (dtlsDimMin.getWidth()), MSG_PNL_MIN_H * 2);

        setupMessagePanel(msgDimMin, msgDimMax, msgDimPref);
        setupDetailsPanel(dtlsDimMin, dtlsDimMax, dtlsDimPref);
        setupButtonPanel();

        this.getContentPane().add(messagePnl, BorderLayout.NORTH);
        this.getContentPane().add(detailPnl, BorderLayout.CENTER);
        this.getContentPane().add(btnPnl, BorderLayout.SOUTH);

        toggleDetails();
        toggleDetails();
    }

    private void setupDetailsPanel(Dimension min, Dimension max, Dimension preferred) {
        detailPnl.setMinimumSize(min);
        detailPnl.setMaximumSize(max);
        detailPnl.setPreferredSize(preferred);
        txtDetails.setText(DEFAULT_DETAILS_MESSAGE);
        txtDetails.setEditable(false);

        detailPnl.setLayout(new BorderLayout());
        detailPnl.add(detailsLbl, BorderLayout.NORTH);
        scrollPane = new JScrollPane(txtDetails);
        detailPnl.add(scrollPane, BorderLayout.CENTER);
    }

    private void setupButtonPanel() {
        getRootPane().setDefaultButton(btnOK);
        btnOK.setText(OK_BTN_TEXT);
        btnOK.setName(OK_BTN_TEXT);
        btnOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    btnOK_actionPerformed(e);
                }
            });
        btnDetails.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    btnDetails_actionPerformed(e);
                }
            });
        btnPnl.setLayout(new FlowLayout(FlowLayout.RIGHT));
        btnPnl.add(btnOK, null);
        btnPnl.add(btnDetails, null);
    }

    private void setupMessagePanel(Dimension min, Dimension max, Dimension preferred) {
        messagePnl.setMinimumSize(min);
        messagePnl.setMaximumSize(max);
        messagePnl.setPreferredSize(preferred);
        txtMessage.setLineWrap(true);
        txtMessage.setWrapStyleWord(true);

        JLabel icon = new JLabel(UIManager.getIcon("OptionPane.errorIcon"));
		icon.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));

        txtMessage.setFont(UIManager.getFont("Label.font"));
        txtMessage.setBackground(UIManager.getColor("Label.background"));
        txtMessage.setEditable(false);
		txtMessage.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));

        messagePnl.setLayout(new BoxLayout(messagePnl, BoxLayout.X_AXIS));
//		messagePnl.setLayout(new BorderLayout());
        messagePnl.add(icon);
        icon.setAlignmentY(Component.CENTER_ALIGNMENT);
        messagePnl.add(Box.createHorizontalStrut(6));
        messagePnl.add(txtMessage);
        txtMessage.setAlignmentY(Component.CENTER_ALIGNMENT);
    }

    private void toggleDetails() {
        detailIdx = 1 - detailIdx;
        btnDetails.setText(detailText[detailIdx]);
        btnDetails.setName(DETAILS_BTN_TEXT);
        detailPnl.setVisible(detailEnabled[detailIdx]);
        this.pack();
        this.setLocationRelativeTo(null);
    }

    void btnOK_actionPerformed(ActionEvent e) {
        hide();
    }

    void btnDetails_actionPerformed(ActionEvent e) {
        toggleDetails();
    }

    private static class HiddenFrame extends JFrame {
        public HiddenFrame() {
            setIconImage(DesktopLookAndFeel.getDesktopIconImage());
        }
    }

    private static class UnselectableTextArea extends JTextArea {
        public UnselectableTextArea() {
            setEditable(false);
            setFocusable(false);
        }

        protected void processMouseEvent(MouseEvent e) {
            // NOOP
        }

        protected void processMouseMotionEvent(MouseEvent e) {
            // NOOP
        }

        protected void processMouseWheelEvent(MouseWheelEvent e) {
            // NOOP
        }
    }
}
