package gov.va.med.hds.cd.mum.boot;

import gov.va.med.hds.cd.mum.boot.utils.Settings;

import java.io.File;

/**
 * Information collected, used, and shared throughout the startup process. 
 */
public interface IStartupInfo {

	/**
     * Returns the directory where the boot and startup jars are located
     */
    File getStartupDirectory();
    
	/**
     * Returns the class loader used by the boot to load the startup jar
     */
    ClassLoader getLoader();
    
	/**
     * Returns the plugins directory 
     */
    File getPluginsDirectory();
    
	/**
     * Returns the root directory where the launcher executable and the plugins directory reside.
     */
    File getRootDirectory();
    
	/**
     * Returns the instance of the splash screen.
     */
    ISplashScreen getSplashScreen();
    
	/**
     * Used to set the class loader that loads the startup jar
     */
    void setLoader(ClassLoader loader);
    
	/**
     * Used to assign the splash screen instance
     */
    void setSplashScreen(ISplashScreen splashScreen);
    
	/**
     * Returns the command line arguments passed to the application
     */
    String[] getArgs();
    
	/**
     * Returns the Settings instance updated by various parts of the startup process.
     */
    Settings getSettings();
    
	/**
     * Used to assign the Settings instance.
     */
    void setSettings(Settings settings);
}