package gov.va.med.hds.cd.mum.boot;

/**
 * Used to provide status notification to the user on the splash screen. 
 */
public interface IStatusNotifier extends ISplashScreen {

	/**
	 * When this character is used in a message sent to the updateProgressMessage method,
	 * the message will be split into two lines, with the line split point being the location 
	 * of this character.  
	 */
	char PROGRESS_MESSAGE_SEPERATOR = '\r';

	/**
	 * Displays the progress bar on the splash screen, and initializes it's
	 * maximum value.  If the progress bar is already displayed it simply
	 * updates the maximum value.
	 * @param maxValue
	 * @param message - an optional message to display
	 */
	void showProgressBar(long maxValue, String message);

	/**
	 * Adds the passed in value to the progress value of the progress bar.
	 * @param value
	 */
	void updateProgressBar(long value);
	
	/**
	 * Adds the passed in valid to the progress value maximum.
	 * @param value
	 */
	void updateProgressBarMax(long value);

	/**
	 * Updates the sub-message that appears above the progress bar.  Supports one or 
	 * two lines of text.  The presense of a PROGRESS_MESSAGE_SEPERATOR character in
	 * the message indicates the point at which the message is split into two lines.  
	 * @param value
	 */
	void updateProgressMessage(String message);

	/**
	 * Hides the progress bar and the progress bar messages, if any.
	 */
	void hideProgressBar();

	/**
	 * Displays the indicated text on the splash screen, above the progress bar and progress
	 * bar message.  This message is in a larger font that the progress bar message.
	 * @param message
	 */
	void displayMessage(String message);
	
	/**
	 * Sets the version displayed on the splash screen
	 * @param version
	 */
	void setVersion(String version);	
	
	/**
	 * Turns the display of the cancel button on or off.  Each call to this method
	 * sets the wasCancelButtonPressed method to false;
	 * @param value
	 */
	void showCancelButton(boolean value);
	
	/**
	 * Returns true if the cancel button has been pressed;
	 */
	boolean isCanceled();
	
	/**
	 * Defines the message to display when the cancel button is pressed.
	 * @param message
	 */
	void setCancelMessage(String message);
	
	/**
	 * Disables screen refresh until endDisplayUpdate is called.  Called when 
	 * multiple display updates are about to be made to the splash screen. 
	 */
	void beginDisplayUpdate();
	
	/**
	 * Reenables screen refresh and refreshed the screen 
	 */
	void endDisplayUpdate();
}
