package gov.va.med.hds.cd.mum.boot;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

/**
 * The actual splash screen
 */
public class Splash extends JFrame implements IStatusNotifier, IBootConstants {

    /**
     * Constructor for Splash 
     */
    public Splash() {
        super();
        setUndecorated(true);
        getContentPane().setBackground(Color.WHITE);
        splash = new PaintLabel(getSplashImage());
        ImageIcon icon = getSplashIcon();
        setIconImage(icon.getImage());
        cancel = new JButton("Cancel");
        cancel.setBounds(buttonX, buttonY, buttonWidth, buttonHeight);
        cancel.setVisible(false);
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                button_actionPerformed(e);
            }
        });
        splash.add(cancel);
        getContentPane().add(splash);
        pack();
        Dimension screen = getToolkit().getScreenSize();
        setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
        setVisible(true);
        setTitle(SPLASH_SCREEN_TITLE);
    }

    public static ImageIcon getSplashImage() {
        return new ImageIcon(Splash.class.getResource("SplashScreen.jpg"));
    }

    public static ImageIcon getSplashIcon() {
        return new ImageIcon(Splash.class.getResource("SplashIcon.gif"));
    }

    public class PaintLabel extends JLabel {

        public PaintLabel(Icon image) {
            super(image);
        }

        protected void paintComponent(Graphics graphics) {
            super.paintComponent(graphics);
            Graphics2D g = (Graphics2D) graphics;

            if (size == null)
                size = this.getSize();
            if (barWidth == 0)
                barWidth = size.width - (barGap * 2);

            g.setColor(Color.white);
            //			g.drawRect(0, 0, size.width - 1, size.height - 1);

            if (normalFont == null) {
                normalFont = new Font(FONT_NAME, Font.PLAIN, SMALL_FONT_SIZE);
                boldFont = new Font(normalFont.getName(), Font.BOLD, LARGE_FONT_SIZE);
            }

            if (version != null) {
                if (versionFont == null)
                    versionFont = new Font(VERSION_FONT_NAME, Font.BOLD, VERSION_FONT_SIZE);
                g.setFont(versionFont);
                if (!version.equals(lastVersion)) {
                    Rectangle2D versionRect = g.getFontMetrics().getStringBounds(version, g);
                    versionX = size.width - versionDX - (int) versionRect.getWidth();
                    versionY = size.height - versionDY - (int) versionRect.getHeight();
                    lastVersion = version;
                }
                g.drawString(version, versionX, versionY);
            }
            if (message != null) {
                int y;
                if (max > 0)
                    y = messageY;
                else
                    y = messageY3;
                g.setFont(boldFont);
                try {
                    int x = (size.width - g.getFontMetrics().stringWidth(message)) / 2;
                    g.drawString(message, x, y);
                } finally {
                    g.setFont(normalFont);
                }
            }
            if (max > 0) {
                if (message2 != null) {
                    int x = (size.width - g.getFontMetrics().stringWidth(message2)) / 2;
                    int y = messageY2;
                    if (message3 == null)
                        y += twoMessageOffset;
                    g.drawString(message2, x, y);
                    if (message3 != null) {
                        x = (size.width - g.getFontMetrics().stringWidth(message3)) / 2;
                        g.drawString(message3, x, messageY4);
                    }
                }
                if (max > 0) {
                    g.drawRect(barGap, barY, barWidth + 1, barHeight);
                    long pcentLong = (progress * 100) / max;
                    long xLong = (progress * barWidth) / max;
                    int x = (int) xLong;
                    int xx = (int) pcentLong;
                    String pcnt = Long.toString(xx) + "%";
                    if (progress >= max)
                        x = barWidth;
                    if (x > 0) {
                        if (gp == null)
                            gp =
                                new GradientPaint(
                                    barGap + 1,
                                    barY + 1,
                                    statusBarColorLeft,
                                    barWidth + 1,
                                    barHeight - 1,
                                    statusBarColorRight);
                        g.setPaint(gp);
                        Rectangle2D rect =
                            new Rectangle2D.Double(barGap + 1, barY + 1, x, barHeight - 1);

                        g.fill(rect);
                    }
                    try {
                        g.setColor(Color.white);
                        g.setFont(boldFont);
                        x = (barWidth - g.getFontMetrics().stringWidth(pcnt)) / 2 + barGap + 1;
                        g.drawString(pcnt, x, barY + (barHeight / 2) + 7);
                    } finally {
                        g.setPaintMode();
                        g.setFont(normalFont);
                    }
                }
            }
        }
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#showProgressBar(long, java.lang.String)
     */
    public void showProgressBar(long maxValue, String message) {
        this.max = maxValue;
        this.message = message;
        progress = 0;
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressBar(long)
     */
    public void updateProgressBar(long value) {
        progress += value;
        if (progress < 0)
            progress = 0;
        else if (progress > max)
            progress = max;
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressBarMax(long)
     */
    public void updateProgressBarMax(long value) {
        max += value;
        if (max < 0)
            max = 0;
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#updateProgressMessage(java.lang.String)
     */
    public void updateProgressMessage(String message) {
        int idx = message.indexOf(PROGRESS_MESSAGE_SEPERATOR);
        if (idx >= 0) {
            message2 = message.substring(0, idx);
            message3 = message.substring(idx + 1);
        } else {
            message2 = message;
            message3 = null;
        }
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#hideProgressBar()
     */
    public void hideProgressBar() {
        max = 0;
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#displayMessage(java.lang.String)
     */
    public void displayMessage(String message) {
        this.message = message;
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setVersion(java.lang.String)
     */
    public void setVersion(String version) {
        if (version == null) {
            if (this.version != null) {
                this.version = null;
                redraw();
            }
        } else if (!version.equals(this.version)) {
            this.version = version;
            redraw();
        }
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#showCancelButton(boolean)
     */
    public void showCancelButton(boolean value) {
        cancel.setVisible(value);
        redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#isCanceled()
     */
    public boolean isCanceled() {
        return canceled;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.ISplashScreen#close()
     */
    public void close() {
        this.setVisible(false);
        this.dispose();
    }

    private void redraw() {
        if (redrawEnabled)
            splash.repaint();
    }

    private boolean redrawEnabled = true;
    GradientPaint gp;
    String message;
    String lastVersion;
    String version;
    String message2;
    String message3;
    String cancelMessage;
    private static final String FONT_NAME = "Century Gothic";
    private static final int LARGE_FONT_SIZE = 18;
    private static final int SMALL_FONT_SIZE = 14;
    private static final String VERSION_FONT_NAME = "Arial";
    private static final int VERSION_FONT_SIZE = 16;
    Font versionFont;
    Font boldFont;
    Font normalFont;
    Dimension size;
    Color statusBarColorLeft = new Color(226, 233, 245);
    Color statusBarColorRight = new Color(62, 30, 105);
    private final int offsetY = 210;
    private final int messageY = offsetY + 22;
    private final int messageY2 = messageY + 20;
    private final int messageY4 = messageY2 + 20;
    private final int messageY3 = messageY2 + 6;
    private final int barY = offsetY + 74;
    private final int twoMessageOffset = 10;
    private final int barGap = 34;
    private final int barHeight = 23;
    private final int versionDX = 49;
    private final int versionDY = 22;
    private final int buttonHeight = 24;
    private final int buttonWidth = 75;
    private final int buttonX = 514;
    private final int buttonY = 255;
    int versionX;
    int versionY;
    int barWidth = 0;
    long max;
    long progress;
    private JLabel splash;
    private JButton cancel;
    private boolean canceled;

    void button_actionPerformed(ActionEvent e) {
        canceled = true;
        message = cancelMessage;
        message2 = null;
        message3 = null;
        max = 0;
        redraw();
    }

    private void setRedrawEnabled(boolean b) {
        redrawEnabled = b;
        if (redrawEnabled)
            redraw();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#beginDisplayUpdate()
     */
    public void beginDisplayUpdate() {
        setRedrawEnabled(false);
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#endDisplayUpdate()
     */
    public void endDisplayUpdate() {
        setRedrawEnabled(true);
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStatusNotifier#setCancelMessage(java.lang.String)
     */
    public void setCancelMessage(String message) {
        cancelMessage = message;
    }
}
