package gov.va.med.hds.cd.mum.boot;

import gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.KeyedList;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.mum.boot.utils.Utils;

import java.io.File;
import java.io.FileNotFoundException;

/**
 * This class is used to provide plugin information to the class loaders for the startup plugin. 
 */
public class StartupDesktopPlugin implements IDesktopPlugin {

    private File jar;
    private File manifest;
    private String module;
    private ClassLoader loader;
    private SimpleList emptyList;
    private SimpleList packages;
    private SimpleList published;
    private KeyedList emptyKeyedList;
    private File[] files;

    /**
     * Constructor for StartupDesktopPlugin 
     */
    public StartupDesktopPlugin(File jar) {
        super();
        this.jar = jar;
        emptyList = new SimpleList();
        emptyKeyedList = new KeyedList();
        packages = new SimpleList();
        published = new SimpleList();
        manifest = new File(jar.getParentFile(), IBootConstants.MANIFEST_FILE);
        if (jar.exists()) {
            Utils.getPackages(jar, packages);
        }
        if (manifest.exists()) {
            ILoadableTreeData xml = null;
            try {
                xml = TreeDataFactory.newSAXTreeData();
                xml.load(manifest);
            } catch (TreeDataException e) {
            } catch (FileNotFoundException e) {
            }
            if (xml != null) {
                int root = xml.getRootID();
                module = xml.getAttribute(root, IPluginConstants.MODULE_ATTRIBUTE);
                if (module == null)
                    module = xml.getAttribute(root, IPluginConstants.ID_ATTRIBUTE);
                int pubList = xml.findChildID(root, IPluginConstants.PUBLISHED_ELEMENT);
                if (pubList >= 0) {
                    int node = -1;
                    do {
                        node = xml.findChildID(pubList, IPluginConstants.PUBLISH_ELEMENT, node);
                        if (node >= 0) {
                            published.add(xml.getAttribute(node, IPluginConstants.CLASS_ATTRIBUTE));
                        }
                    } while (node >= 0);
                }
            }
            files = new File[2];
            files[0] = manifest;
        } else {
            manifest = null;
            files = new File[1];
        }
        files[files.length - 1] = jar;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getFileCount()
     */
    public int getFileCount() {
        return files.length;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getPhysicalFile(int)
     */
    public File getPhysicalFile(int index) {
        return files[index];
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getFilePackages(int)
     */
    public SimpleList getFilePackages(int index) {
        if (files[index] == jar)
            return packages;
        return emptyList;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getClassLoader()
     */
    public ClassLoader getClassLoader() {
        return loader;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#setClassLoader(java.lang.ClassLoader)
     */
    public void setClassLoader(ClassLoader loader) {
        this.loader = loader;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#publishAllClasses()
     */
    public boolean publishAllClasses() {
        return false;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getPublishedClasses()
     */
    public SimpleList getPublishedClasses() {
        return emptyList;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getLinkedPlugins()
     */
    public KeyedList getLinkedPlugins() {
        return emptyKeyedList;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin#getModule()
     */
    public String getModule() {
        return module;
    }

}
