package gov.va.med.hds.cd.mum.boot;

import gov.va.med.hds.cd.mum.boot.utils.Settings;
import gov.va.med.hds.cd.mum.boot.utils.Utils;

import java.io.File;

/**
 * Startup information used througout the startup process
 */
public class StartupInfo implements IBootConstants, IStartupInfo {

    private Settings settings;
    private File rootDirectory;
    private File startupDirectory;
    private File pluginsDirectory;
    private ClassLoader loader;
    private ISplashScreen splashScreen;
    private String[] args;

    /**
     * Constructor for StartupInfo 
     */
    public StartupInfo(String[] args) {
        super();
        this.args = args;
        detectDirectories();
    }

    private void detectDirectories() {
        startupDirectory = new File(Utils.getPhysicalDirectory(Boot.class));
        pluginsDirectory = startupDirectory.getParentFile();
        rootDirectory = pluginsDirectory.getParentFile();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getStartupDirectory()
     */
    public File getStartupDirectory() {
        return startupDirectory;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getLoader()
     */
    public ClassLoader getLoader() {
        return loader;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getPluginsDirectory()
     */
    public File getPluginsDirectory() {
        return pluginsDirectory;
    }
    
    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getRootDirectory()
     */
    public File getRootDirectory() {
        return rootDirectory;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getSplashScreen()
     */
    public ISplashScreen getSplashScreen() {
        return splashScreen;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#setLoader(java.lang.ClassLoader)
     */
    public void setLoader(ClassLoader loader) {
        this.loader = loader;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#setSplashScreen(gov.va.med.hds.cd.mum.boot.ISplashScreen)
     */
    public void setSplashScreen(ISplashScreen splashScreen) {
        this.splashScreen = splashScreen;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getArgs()
     */
    public String[] getArgs() {
        return args;
    }

    /**
     * Sets the startupDirectory - used mainly in unit testing
     */
    protected void setStartupDirectory(File startupDirectory) {
        this.startupDirectory = startupDirectory;
    }

    /**
     * Sets the pluginsDirectory - used mainly in unit testing
     */
    protected void setPluginsDirectory(File pluginsDirectory) {
        this.pluginsDirectory = pluginsDirectory;
    }

    /**
     * Sets the rootDirectory - used mainly in unit testing
     */
    protected void setRootDirectory(File rootDirectory) {
        this.rootDirectory = rootDirectory;
    }

    /**
     * Sets the args - used mainly in unit testing
     */
    protected void setArgs(String[] args) {
        this.args = args;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#getSettings()
     */
    public Settings getSettings() {
        return settings;
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartupInfo#setSettings(gov.va.med.hds.cd.mum.boot.utils.Settings)
     */
    public void setSettings(Settings settings) {
        this.settings = settings;
    }
}
