package gov.va.med.hds.cd.mum.boot.loader;

import gov.va.med.hds.cd.mum.boot.utils.KeyedList;

import java.util.List;

/**
 * The DelegatingClassLoader class is used when a single class loader is needed to load classes and
 * other resources from multiple plugins, such as when passing a class loader to a third party
 * library.
 */
public class DelegatingClassLoader extends LinkedClassLoader implements IDelegatingClassLoader {

	protected DelegatingClassLoader(ClassLoader parent) {
        super(parent);
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.loader.IDelegatingClassLoader#addLoader(gov.va.med.hds.cd.mum.boot.loader.LinkedPluginLoader)
     */
    public void addLoader(LinkedPluginLoader loader) {
        List keys = loader.getLinks().getKeys();
        for (int i = 0; i < keys.size(); i++) {
            String key = (String) keys.get(i);
            List list = loader.getLinks().get(key);
            for (int j = 0; j < list.size(); j++) {
                LinkedPluginLoader child = (LinkedPluginLoader) list.get(j);
                LoaderData data = new LoaderData(loader, child);
                getLinks().add(key, data);
            }
        }
        KeyedList list = loader.getOtherFiles();
        if ((list != null) && (list.size() > 0)) {
            ensureOtherFileList();
            keys = list.getKeys();
            for (int i = 0; i < keys.size(); i++) {
                String key = (String) keys.get(i);
                Object value = list.get(key);
                getOtherFiles().add(key, value);
            }
        }
    }

    LinkedPluginLoader getRequestor(Object obj) {
        return ((LoaderData) obj).getRequestor();
    }

    LinkedPluginLoader getLoader(Object obj) {
        return ((LoaderData) obj).getLoader();
    }

    static class LoaderData {
        private LinkedPluginLoader loader;
        private LinkedPluginLoader requestor;

        public LoaderData(LinkedPluginLoader requestor, LinkedPluginLoader loader) {
            super();
            this.loader = loader;
            this.requestor = requestor;
        }

        public LinkedPluginLoader getLoader() {
            return loader;
        }

        public LinkedPluginLoader getRequestor() {
            return requestor;
        }

        public boolean equals(Object obj) {
            if (obj instanceof LoaderData) {
                LoaderData data = (LoaderData) obj;
                return (
                    (data.getLoader() == this.loader) && (data.getRequestor() == this.requestor));
            }
            return false;
        }

        public int hashCode() {
            return loader.hashCode() ^ requestor.hashCode();
        }

    }
}
