package gov.va.med.hds.cd.mum.boot.loader;

/** Interface to globally listen for notifications that a class/resource was
 * not found by a {@link LinkedClassLoader}. 
 * 
 * @author vhaisltumad
 */
public interface IClassLoaderErrorListener {
    
    public static final boolean ENABLED = true;

    /** Called when a class is not found.
     * 
     * @param className the fully qualified name of the class which could
     *    not be loaded.
     * @param requestorKey a string uniquely identifying the plugin that
     *    attempted to load this class.
     * @param moduleName the name of the module containing that plugin
     */
    public void classNotFound(String className,
        String requestorKey,
        String moduleName);
        


    /** Called when a resource is not found.
     * 
     * @param resourceName the name of the resource which could
     *    not be located.
     * @param requestorKey a string uniquely identifying the plugin that
     *    attempted to load this resource.
     * @param moduleName the name of the module containing that plugin
     */
    public void resourceNotFound(String resourceName,
        String requestorKey,
        String moduleName);

}
