package gov.va.med.hds.cd.mum.boot.loader;

import gov.va.med.hds.cd.mum.boot.utils.KeyedList;
import gov.va.med.hds.cd.mum.boot.utils.SimpleList;

import java.io.File;

/**
 * Interface used by the class loaders to gather plugin information 
 */
public interface IDesktopPlugin {
	
    /**
     * Returns the number of physical files in the plugin 
     */
    int getFileCount();
    
    /**
     * Returns the physical File object associated with the plugin file.
     */
    File getPhysicalFile(int index);
    
	/**
	 * When the physical file is jar, the returned list is a list of class and resource packages 
	 * available in that jar.  Other files return an empty list. 
     */
    SimpleList getFilePackages(int index);
    
	/**
     * Returns the class loader associated with the plugin, if it has been assigned by the 
     * setClassLoader method.
     */
    ClassLoader getClassLoader();
    
    /**
     * Assignes the class loader used to load classes for this plugin 
     */
    void setClassLoader(ClassLoader loader);
    
	/**
     * Returns true if all classes in the plugin are published.
     */
    boolean publishAllClasses();
    
	/**
     * Returns a list of Strings representing published classes.  If all classes are published,
     * this list is empty.
     */
    SimpleList getPublishedClasses();
    
	/**
     * Returns a list of plugins that this plugin is linked to.
     */
    KeyedList getLinkedPlugins();
    
	/**
     * Return the name of the plugin module, used to bypass the published list.
     */
    String getModule();
}
