package gov.va.med.hds.cd.mum.boot.loader;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.zip.ZipFile;

/**
 * Used internally by the linked class loader to load data from jar files
 */
public interface IFileLoader {
	
	/**
     * Returns the data associated with the file in the jar file.
     * @param name - the name of the data needed
     * @throws IOException
     */
    IDataBuffer getData(String name) throws IOException;
    
	/**
     * Returns true if the data exists in the jar file.
     * @param name
     */
    boolean hasData(String name);
    
	/**
     * Returns the ZipFile instance associated with the jar file.
     */
    ZipFile getZipFile();
    
	/**
     * Returns the File instance associated with the jar file. 
     */
    File getFile();
    
	/**
     * Returns the URL of the jar file.
     */
    URL getURL();
    
	/**
     * Returns the absolute path of the jar file.
     */
    String getAbsolutePath();
    
	/**
     * Return true if the file is a jar or zip file.
     */
    boolean isZipFile();
}
