package gov.va.med.hds.cd.mum.boot.loader;

import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;

import java.net.URL;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * Root class loader used to manage class loader / package maps 
 */
public abstract class LinkedClassLoader extends LibraryClassLoader implements Comparator {

    private ListMap links;
    
    LinkedClassLoader(ClassLoader parent) {
        super(parent);
        links = new ListMap();
    }

    ListMap getLinks() {
        return links;
    }

    /**
     * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
     */
    protected synchronized Class loadClass(String name, boolean resolve)
        throws ClassNotFoundException {
        int idx = name.lastIndexOf('.');
        String pkg;
        if (idx > 0)
            pkg = name.substring(0, idx);
        else
            pkg = IBootConstants.DEFAULT_PACKAGE;
        List list = getLinks().get(pkg);
        Class result = null;
        if (list.size() == 0)
            return super.loadClass(name, resolve);

        for (int i = 0; i < list.size(); i++) {
            Object obj = list.get(i);
            LinkedPluginLoader loader = getLoader(obj);
            LinkedPluginLoader requestor = getRequestor(obj);
            try {
                result = loader.loadPluginClass(requestor, name, resolve);
                if (result != null)
                    return result;
            } catch (ClassNotFoundException e) {
            }
        }
        throw new ClassNotFoundException(name);
    }

    /**
     * @see java.lang.ClassLoader#findResource(java.lang.String)
     */
    public URL findResource(String name) {
        int idx = name.lastIndexOf('/');
        String pkg;
        if (idx > 0)
            pkg = name.substring(0, idx).replace('/', '.');
        else
            pkg = IBootConstants.DEFAULT_PACKAGE;
        List list = getLinks().get(pkg);
        for (int i = 0; i < list.size(); i++) {
            LinkedPluginLoader loader = getLoader(list.get(i));
            URL result = loader.findPluginResource(name);
            if (result != null)
                return result;
        }
        return null;
    }

    Class internalLoadClass(String name, boolean resolve) throws ClassNotFoundException {
        return super.loadClass(name, resolve);
    }

    URL internalFindResource(String name) {
        return super.findResource(name);
    }

    void ensureLinkOrder() {
        List keys = getLinks().getKeys();
        for (int i = 0; i < keys.size(); i++) {
            List list = getLinks().get(keys.get(i));
            if (list.size() > 1) {
                Collections.sort(list, this);
            }
        }
    }

    public int compare(Object o1, Object o2) {
        return ((LinkedPluginLoader) o1).getSortKey().compareTo(
            ((LinkedPluginLoader) o2).getSortKey());
    }

    abstract LinkedPluginLoader getRequestor(Object obj);
    abstract LinkedPluginLoader getLoader(Object obj);
}
