package gov.va.med.hds.cd.mum.boot.plaf.desktop;

import java.awt.Component;
import java.awt.Graphics;
import java.io.Serializable;

import javax.swing.AbstractButton;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.plaf.UIResource;

public class DesktopIconFactory {

	private static Icon checkBoxMenuItemIcon;
	private static Icon radioButtonMenuItemIcon;
	private static Icon menuItemCheckIcon;
	
	public static Icon getMenuItemCheckIcon() {
		if (menuItemCheckIcon == null) {
			menuItemCheckIcon = new MenuItemCheckIcon();
		}
		return menuItemCheckIcon;
	}

	public static Icon getCheckBoxMenuItemIcon() {
		if (checkBoxMenuItemIcon == null) {
			checkBoxMenuItemIcon = new CheckBoxMenuItemIcon();
		}
		return checkBoxMenuItemIcon;
	}

	public static Icon getRadioButtonMenuItemIcon() {
		if (radioButtonMenuItemIcon == null) {
			radioButtonMenuItemIcon = new RadioButtonMenuItemIcon();
		}
		return radioButtonMenuItemIcon;
	}

	private static class CheckBoxMenuItemIcon implements Icon, UIResource, Serializable {
		
		private static int OFFSET = 4;
		
		public void paintIcon(Component c, Graphics g, int x, int y) {
			AbstractButton b = (AbstractButton) c;
			ButtonModel model = b.getModel();
			boolean isSelected = model.isSelected();
			if (isSelected) {
				x += OFFSET;
				y += OFFSET;
				g.drawLine(x + 7, y + 1, x + 7, y + 3);
				g.drawLine(x + 6, y + 2, x + 6, y + 4);
				g.drawLine(x + 5, y + 3, x + 5, y + 5);
				g.drawLine(x + 4, y + 4, x + 4, y + 6);
				g.drawLine(x + 3, y + 5, x + 3, y + 7);
				g.drawLine(x + 2, y + 4, x + 2, y + 6);
				g.drawLine(x + 1, y + 3, x + 1, y + 5);
				x -= OFFSET;
				y -= OFFSET;
			}
		}
		public int getIconWidth() {
			return 16;
		}
		public int getIconHeight() {
			return 16;
		}

	} // End class CheckBoxMenuItemIcon

	private static class RadioButtonMenuItemIcon implements Icon, UIResource, Serializable {
		
		private static final int DIAMETER = 6;
		
		public void paintIcon(Component c, Graphics g, int x, int y) {
			AbstractButton b = (AbstractButton) c;
			ButtonModel model = b.getModel();
			if (b.isSelected() == true) {
				int originx = x + ((getIconWidth() - DIAMETER) / 2);
				int originy = y + ((getIconHeight() - DIAMETER) / 2);
				g.fillOval(originx, originy, DIAMETER, DIAMETER);
			}
		}
		public int getIconWidth() {
			return 16;
		}
		public int getIconHeight() {
			return 16;
		}

	} // End class RadioButtonMenuItemIcon

	private static class MenuItemCheckIcon implements Icon, UIResource, Serializable {
		public void paintIcon(Component c, Graphics g, int x, int y) {
		}
		public int getIconWidth() {
			return 16;
		}
		public int getIconHeight() {
			return 16;
		}

	} // End class MenuItemCheckIcon
}
