package gov.va.med.hds.cd.mum.boot.plaf.desktop;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicListUI;

public class DesktopListUI extends BasicListUI {

	/**
	 * Returns a new instance of DesktopListUI.  DesktopListUI delegates are
	 * allocated one per JList.
	 *
	 * @return A new ListUI implementation for the Desktop look and feel.
	 */
	public static ComponentUI createUI(JComponent list) {
		return new DesktopListUI();
	}

	protected MouseInputListener createMouseInputListener() {
		return new WindowsStyleMouseInputHandler();
	}

	public int locationToIndex(JList list, Point location) {
		int index = super.locationToIndex(list, location);
		if (index >= list.getModel().getSize() - 1) {
			Rectangle cellBounds = getCellBounds(list, index, index);
			if (cellBounds != null && !cellBounds.contains(location))
				return -1;
		}
		return index;
	}

	public class WindowsStyleMouseInputHandler extends MouseInputHandler {

		private boolean selectedOnPress;

		public void mousePressed(MouseEvent e) {
			if (e.isConsumed()) {
				selectedOnPress = false;
				return;
			}
			selectedOnPress = true;
			adjustFocusUndSelection(e);
		}

		public void mouseReleased(MouseEvent e) {
			if (selectedOnPress) {
				if (!SwingUtilities.isLeftMouseButton(e)) {
					return;
				}

				list.setValueIsAdjusting(false);
			} else {
				adjustFocusUndSelection(e);
			}
		}

		public void mouseDragged(MouseEvent e) {
			if (e.isConsumed()) {
				return;
			}
			if (!SwingUtilities.isLeftMouseButton(e)) {
				return;
			}
			if (!list.isEnabled()) {
				return;
			}
			if (e.isShiftDown() || e.isControlDown()) {
				return;
			}

			int index = locationToIndex(DesktopListUI.this.list, e.getPoint());
			if (index != -1) {
				Rectangle cellBounds = getCellBounds(list, index, index);
				if (cellBounds != null) {
					list.scrollRectToVisible(cellBounds);
					list.setSelectionInterval(index, index);
				}
			}
		}

		void adjustFocusUndSelection(MouseEvent e) {
			if (!SwingUtilities.isLeftMouseButton(e)) {
				return;
			}

			if (!list.isEnabled()) {
				return;
			}

			if (!list.hasFocus() && list.isRequestFocusEnabled()) {
				list.requestFocus();
			}

			int index = locationToIndex(DesktopListUI.this.list, e.getPoint());
			if (index != -1) {
				boolean adjusting =
					(e.getID() == MouseEvent.MOUSE_PRESSED) ? true : false;
				list.setValueIsAdjusting(adjusting);
				int anchorIndex = list.getAnchorSelectionIndex();
				if (e.isControlDown()) {
					if (list.isSelectedIndex(index)) {
						list.removeSelectionInterval(index, index);
					} else {
						list.addSelectionInterval(index, index);
					}
				} else if (e.isShiftDown() && (anchorIndex != -1)) {
					list.setSelectionInterval(anchorIndex, index);
				} else {
					list.setSelectionInterval(index, index);
				}
			}
		}
	}
}
