package gov.va.med.hds.cd.mum.boot.plaf.desktop;

import java.awt.Image;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.InsetsUIResource;

import com.sun.java.swing.plaf.windows.WindowsLookAndFeel;

public class DesktopLookAndFeel extends WindowsLookAndFeel {

	private static final String DESKTOP_ICON = "icons/CPRS_LOGO Icon 32.gif";
	private static final String DESKTOP = "HealtheVet Desktop";

	private static ImageIcon clinicianDesktopIcon;
	private static IDesktopTheme currentTheme = new DefaultDesktopTheme();

    private ClassLoader plafClassLoader;

	static {
		UIManager.installLookAndFeel(DESKTOP, DesktopLookAndFeel.class.getName());
	}

	public DesktopLookAndFeel(ClassLoader loader) {
		super();
        plafClassLoader = loader;
	}

	public String getID() {
		return DESKTOP;
	}

	public String getName() {
		return DESKTOP;
	}

	public String getDescription() {
		return "The " + DESKTOP + " Look and Feel";
	}

	protected void initClassDefaults(UIDefaults table) {
		super.initClassDefaults(table);

		Object[] uiDefaults =
			{
				"ClassLoader",
				plafClassLoader,
				"ButtonUI",
				DesktopButtonUI.class.getName(),
				"ListUI",
				DesktopListUI.class.getName(),
				"MenuItemUI",
				DesktopMenuItemUI.class.getName(),
				"CheckBoxMenuItemUI",
				DesktopCheckBoxMenuItemUI.class.getName(),
				"RadioButtonMenuItemUI",
				DesktopRadioButtonMenuItemUI.class.getName(),
				"TreeUI",
				DesktopTreeUI.class.getName()};
		table.putDefaults(uiDefaults);
	}

	/**
	* Load the SystemColors into the defaults table.  The keys
	* for SystemColor defaults are the same as the names of
	* the public fields in SystemColor.  If the table is being
	* created on a native Windows platform we use the SystemColor
	* values, otherwise we create color objects whose values match
	* the defaults Windows95 colors.
	*/
	protected void initSystemColorDefaults(UIDefaults table) {
		Object[] systemColors =
			{ "desktop", getDesktopColor(), /* Color of the desktop background */
			"activeCaption", getWindowTitleBackground(),
			/* Color for captions (title bars) when they are active. */
			"activeCaptionText", getWindowTitleForeground(),
			/* Text color for text in captions (title bars). */
			"activeCaptionBorder", getPrimaryControlShadow(),
			/* Border color for caption (title bar) window borders. */
			"inactiveCaption", getWindowTitleInactiveBackground(),
			/* Color for captions (title bars) when not active. */
			"inactiveCaptionText", getInactiveSystemTextColor(),
			/* Text color for text in inactive captions (title bars). */
			"inactiveCaptionBorder", getControlShadow(),
			/* Border color for inactive caption (title bar) window borders. */
			"window", getWindowBackground(), /* Default color for the interior of windows */
			"windowBorder", getControl(), /* ??? */
			"windowText", getWindowTitleForeground(), /* ??? */
			"menu", getMenuBackground(), /* Background color for menus */
			"menuPressedItemB", "#000080", /* LightShadow of menubutton highlight */
			"menuPressedItemF", "#FFFFFF", /* Default color for foreground "text" in menu item */
			"menuText", getMenuForeground(), /* Text color for menus  */
			"text", getControl(), /* Text background color */
			"textText", getUserTextColor(), /* Text foreground color */
			"textHighlight", getTextHighlightColor(), /* Text background color when selected */
			"textHighlightText", getHighlightedTextColor(), /* Text color when selected */
			"textInactiveText", getInactiveUserTextColor(), /* Text color when disabled */
			"control", getControl(), /* Default color for controls (buttons, sliders, etc) */
			"controlText", getControlTextColor(), /* Default color for text in controls */
			"controlHighlight",
				getControlLight(),
			/* Specular highlight (opposite of the shadow) */
			"controlLtHighlight", getControlHighlight(), /* Highlight color for controls */
			"controlShadow", getControlShadow(), /* Shadow color for controls */
			"controlDkShadow", getControlDarkShadow(), /* Dark shadow color for controls */
			"scrollbar", getScrollbarBackground(), /* Scrollbar background (usually the "track") */
			"info", getInfoBackground(), /* ToolTip background */
			"infoText", getInfoForeground() /* ToolTip text */
		};

		for (int i = 0; i < systemColors.length; i += 2) {
			table.put((String) systemColors[i], systemColors[i + 1]);
		}
	}

	protected void initComponentDefaults(UIDefaults table) {
		super.initComponentDefaults(table);

		// menu item value objects
		Object menuItemCheckIcon =
			new UIDefaults.ProxyLazyValue(
				DesktopIconFactory.class.getName(),
				"getMenuItemCheckIcon");

		Object checkBoxMenuItemIcon =
			new UIDefaults.ProxyLazyValue(
				DesktopIconFactory.class.getName(),
				"getCheckBoxMenuItemIcon");

		Object radioButtonMenuItemIcon =
			new UIDefaults.ProxyLazyValue(
				DesktopIconFactory.class.getName(),
				"getRadioButtonMenuItemIcon");

		// SplitPane value objects
		Object splitPaneBorder =
			new UIDefaults.ProxyLazyValue(DesktopBorders.class.getName(), "getSplitPaneBorder");
		Object splitPaneDividerBorder =
			new UIDefaults.ProxyLazyValue(
				DesktopBorders.class.getName(),
				"getSplitPaneDividerBorder");

		// Tree Value Objects
		Object treeExpandedIcon = DesktopTreeUI.TwistyExpandedIcon.createExpandedIcon();
		Object treeCollapsedIcon = DesktopTreeUI.TwistyCollapsedIcon.createCollapsedIcon();

		Object[] defaults = {
			// Button
			"Button.showMnemonics",
				Boolean.TRUE,
				"Button.font",
				getControlTextFont(),
				"Button.background",
				getControl(),
				"Button.foreground",
				getControlTextColor(),
				"Button.shadow",
				getControlShadow(),
				"Button.darkShadow",
				getControlDarkShadow(),
				"Button.light",
				getControlLight(),
				"Button.highlight",
				getControlHighlight(),
				"Button.disabledForeground",
				getInactiveControlTextColor(),
				"Button.disabledShadow",
				getControlHighlight(),
				"Button.focus",
				getFocusColor(),
				"Button.margin",
				new InsetsUIResource(0, 2, 0, 2),

			// CheckBox
			"CheckBox.font",
				getControlTextFont(),
				"CheckBox.interiorBackground",
				getWindowBackground(),
				"CheckBox.background",
				getControl(),
				"CheckBox.foreground",
				getControlTextColor(),
				"CheckBox.shadow",
				getControlShadow(),
				"CheckBox.darkShadow",
				getControlDarkShadow(),
				"CheckBox.light",
				getControlLight(),
				"CheckBox.highlight",
				getControlHighlight(),
				"CheckBox.focus",
				getFocusColor(),

			// CheckBoxMenuItem
			"CheckBoxMenuItem.font",
				getMenuTextFont(),
				"CheckBoxMenuItem.background",
				getMenuBackground(),
				"CheckBoxMenuItem.foreground",
				getMenuForeground(),
				"CheckBoxMenuItem.selectionForeground",
				getMenuSelectedForeground(),
				"CheckBoxMenuItem.selectionBackground",
				getMenuSelectedBackground(),
				"CheckBoxMenuItem.acceleratorForeground",
				getMenuForeground(),
				"CheckBoxMenuItem.acceleratorSelectionForeground",
				getMenuSelectedForeground(),
				"CheckBoxMenuItem.checkIcon",
				checkBoxMenuItemIcon,

			// ComboBox
			"ComboBox.font",
				getControlTextFont(),
				"ComboBox.background",
				getWindowBackground(),
				"ComboBox.foreground",
				getUserTextColor(),
				"ComboBox.buttonBackground",
				getControl(),
				"ComboBox.buttonShadow",
				getControlShadow(),
				"ComboBox.buttonDarkShadow",
				getControlDarkShadow(),
				"ComboBox.buttonHighlight",
				getControlHighlight(),
				"ComboBox.selectionBackground",
				getTextHighlightColor(),
				"ComboBox.selectionForeground",
				getHighlightedTextColor(),
				"ComboBox.disabledBackground",
				getControl(),
				"ComboBox.disabledForeground",
				getInactiveControlTextColor(),
				"Desktop.background",
				getDesktopColor(),

			// EditorPane
			"EditorPane.font",
				getControlTextFont(),
				"EditorPane.background",
				getWindowBackground(),
				"EditorPane.foreground",
				getUserTextColor(),
				"EditorPane.selectionBackground",
				getTextHighlightColor(),
				"EditorPane.selectionForeground",
				getHighlightedTextColor(),
				"EditorPane.caretForeground",
				getUserTextColor(),
				"EditorPane.inactiveForeground",
				getInactiveControlTextColor(),

			"FormattedTextField.font", getControlTextFont(),
			"FormattedTextField.background", getWindowBackground(),
			"FormattedTextField.foreground", getUserTextColor(),
			"FormattedTextField.inactiveForeground", getInactiveControlTextColor(),
			"FormattedTextField.inactiveBackground", getControl(),
			"FormattedTextField.selectionBackground", getTextHighlightColor(),
			"FormattedTextField.selectionForeground", getHighlightedTextColor(),

			// InternalFrame
			"InternalFrame.titleFont",
				getWindowTitleFont(),
				"InternalFrame.borderColor",
				getControl(),
				"InternalFrame.borderShadow",
				getControlShadow(),
				"InternalFrame.borderDarkShadow",
				getControlDarkShadow(),
				"InternalFrame.borderHighlight",
				getControlHighlight(),
				"InternalFrame.borderLight",
				getControlLight(),
				"InternalFrame.minimizeIconBackground",
				getControl(),
				"InternalFrame.resizeIconHighlight",
				getControlHighlight(),
				"InternalFrame.resizeIconShadow",
				getControlShadow(),
				"InternalFrame.icon", 
				getClinicianDesktopIcon(),

			// Label
			"Label.font",
				getControlTextFont(),
				"Label.background",
				getControl(),
				"Label.foreground",
				getControlTextColor(),
				"Label.disabledForeground",
				getInactiveControlTextColor(),
				"Label.disabledShadow",
				getControlHighlight(),

			// List
			"List.font",
				getControlTextFont(),
				"List.background",
				getWindowBackground(),
				"List.foreground",
				getUserTextColor(),
				"List.selectionBackground",
				getTextHighlightColor(),
				"List.selectionForeground",
				getHighlightedTextColor(),
				"List.focusCellBorderColor",
				getControlHighlight(),

			// PopupMenu
			"PopupMenu.font",
				getMenuTextFont(),
				"PopupMenu.background",
				getMenuBackground(),
				"PopupMenu.foreground",
				getMenuForeground(),

			// Menus
			"Menu.font",
				getMenuTextFont(),
				"Menu.foreground",
				getMenuForeground(),
				"Menu.background",
				getMenuBackground(),
				"Menu.selectionForeground",
				getMenuSelectedForeground(),
				"Menu.selectionBackground",
				getMenuSelectedBackground(),
				"Menu.acceleratorForeground",
				getMenuForeground(),
				"Menu.acceleratorSelectionForeground",
				getMenuSelectedForeground(),

			// MenuBar.
			"MenuBar.font",
				getMenuTextFont(),
				"MenuBar.background",
				getMenuBackground(),
				"MenuBar.foreground",
				getMenuForeground(),
				"MenuBar.shadow",
				getControlShadow(),
				"MenuBar.highlight",
				getControlHighlight(),

			// MenuItem
			"MenuItem.font",
				getMenuTextFont(),
				"MenuItem.acceleratorFont",
				getMenuTextFont(),
				"MenuItem.foreground",
				getMenuForeground(),
				"MenuItem.background",
				getMenuBackground(),
				"MenuItem.selectionForeground",
				getMenuSelectedForeground(),
				"MenuItem.selectionBackground",
				getMenuSelectedBackground(),
				"MenuItem.disabledForeground",
				getInactiveControlTextColor(),
				"MenuItem.acceleratorForeground",
				getMenuForeground(),
				"MenuItem.acceleratorSelectionForeground",
				getMenuSelectedForeground(),
				"MenuItem.checkIcon",
				menuItemCheckIcon,

			// RadioButton
			"RadioButton.font",
				getControlTextFont(),
				"RadioButton.interiorBackground",
				getWindowBackground(),
				"RadioButton.background",
				getControl(),
				"RadioButton.foreground",
				getControlTextColor(),
				"RadioButton.shadow",
				getControlShadow(),
				"RadioButton.darkShadow",
				getControlDarkShadow(),
				"RadioButton.light",
				getControlLight(),
				"RadioButton.highlight",
				getControlHighlight(),
				"RadioButton.focus",
				getFocusColor(),

			// RadioButtonMenuItem
			"RadioButtonMenuItem.font",
				getMenuTextFont(),
				"RadioButtonMenuItem.foreground",
				getMenuForeground(),
				"RadioButtonMenuItem.background",
				getMenuBackground(),
				"RadioButtonMenuItem.selectionForeground",
				getMenuSelectedForeground(),
				"RadioButtonMenuItem.selectionBackground",
				getMenuSelectedBackground(),
				"RadioButtonMenuItem.disabledForeground",
				getInactiveControlTextColor(),
				"RadioButtonMenuItem.acceleratorForeground",
				getMenuForeground(),
				"RadioButtonMenuItem.acceleratorSelectionForeground",
				getMenuSelectedForeground(),
				"RadioButtonMenuItem.checkIcon",
				radioButtonMenuItemIcon,

			// OptionPane.
			"OptionPane.font",
				getControlTextFont(),
				"OptionPane.messageFont",
				getControlTextFont(),
				"OptionPane.buttonFont",
				getControlTextFont(),
				"OptionPane.background",
				getControl(),
				"OptionPane.foreground",
				getControlTextColor(),
				"OptionPane.messageForeground",
				getControlTextColor(),

			// Panel
			"Panel.font",
				getControlTextFont(),
				"Panel.background",
				getControl(),
				"Panel.foreground",
				getControlTextColor(),

			// PasswordField
			"PasswordField.font",
				getUserTextFont(),
				"PasswordField.background",
				getWindowBackground(),
				"PasswordField.foreground",
				getUserTextColor(),
				"PasswordField.inactiveForeground",
				getInactiveControlTextColor(),
				"PasswordField.inactiveBackground",
				getControl(),
				"PasswordField.selectionBackground",
				getTextHighlightColor(),
				"PasswordField.selectionForeground",
				getHighlightedTextColor(),
				"PasswordField.caretForeground",
				getUserTextColor(),

			// ProgressBar
			"ProgressBar.font",
				getControlTextFont(),
				"ProgressBar.foreground",
				getTextHighlightColor(),
				"ProgressBar.background",
				getControl(),
				"ProgressBar.shadow",
				getControlShadow(),
				"ProgressBar.highlight",
				getControlHighlight(),
				"ProgressBar.selectionForeground",
				getControl(),
				"ProgressBar.selectionBackground",
				getTextHighlightColor(),

			// ScrollBar.
			"ScrollBar.background", getScrollbarBackground(), "ScrollBar.foreground", getControl(),
			//			 "ScrollBar.track", white,
			"ScrollBar.trackForeground", getScrollbarBackground(),
			//			 "ScrollBar.trackHighlight", black,
			//			 "ScrollBar.trackHighlightForeground", scrollBarTrackHighlight,
			"ScrollBar.thumb",
				getControl(),
				"ScrollBar.thumbHighlight",
				getControlHighlight(),
				"ScrollBar.thumbDarkShadow",
				getControlDarkShadow(),
				"ScrollBar.thumbShadow",
				getControlShadow(),

			// ScrollPane.
			"ScrollPane.font",
				getControlTextFont(),
				"ScrollPane.background",
				getControl(),
				"ScrollPane.foreground",
				getControlTextColor(),

			// Separator
			"Separator.background",
				getControlHighlight(),
				"Separator.highlight",
				getControlHighlight(),
				"Separator.foreground",
				getControlShadow(),
				"Separator.shadow",
				getControlShadow(),

			// Slider.
			"Slider.foreground",
				getControl(),
				"Slider.background",
				getControl(),
				"Slider.highlight",
				getControlHighlight(),
				"Slider.shadow",
				getControlShadow(),
				"Slider.focus",
				getControlDarkShadow(),

			// Spinner
			"Spinner.font", getControlTextFont(),

			// SplitPane
			"SplitPane.background",
				getControl(),
				"SplitPane.highlight",
				getControlHighlight(),
				"SplitPane.shadow",
				getControlShadow(),
				"SplitPane.darkShadow",
				getControlDarkShadow(),
				"SplitPane.border",
				splitPaneBorder,
				"SplitPane.dividerSize",
				new Integer(4),
				"SplitPaneDivider.border",
				splitPaneDividerBorder,

			// TabbedPane
			"TabbedPane.font",
				getControlTextFont(),
				"TabbedPane.background",
				getControl(),
				"TabbedPane.foreground",
				getControlTextColor(),
				"TabbedPane.highlight",
				getControlHighlight(),
				"TabbedPane.light",
				getControlLight(),
				"TabbedPane.shadow",
				getControlShadow(),
				"TabbedPane.darkShadow",
				getControlDarkShadow(),
				"TabbedPane.focus",
				getControlTextColor(),

			// Table
			"Table.font", getControlTextFont(), "Table.foreground", getControlTextColor(),
			// cell text color
			"Table.background", getWindowBackground(), // cell background color
			"Table.highlight",
				getControlHighlight(),
				"Table.light",
				getControlLight(),
				"Table.shadow",
				getControlShadow(),
				"Table.darkShadow",
				getControlShadow(),
				"Table.selectionForeground",
				getHighlightedTextColor(),
				"Table.selectionBackground",
				getTextHighlightColor(),
			//				"Table.gridColor",
			//				gray,
			// grid line color
			"Table.focusCellBackground",
				getWindowBackground(),
				"Table.focusCellForeground",
				getControlTextColor(),
				"TableHeader.font",
				getControlTextFont(),
				"TableHeader.foreground",
				getControlTextColor(),
			// header text color
			"TableHeader.background", getControl(), // header background

			// TextArea
			"TextArea.font",
				getUserTextFont(),
				"TextArea.background",
				getWindowBackground(),
				"TextArea.foreground",
				getUserTextColor(),
				"TextArea.inactiveForeground",
				getInactiveControlTextColor(),
				"TextArea.selectionBackground",
				getTextHighlightColor(),
				"TextArea.selectionForeground",
				getHighlightedTextColor(),
				"TextArea.caretForeground",
				getWindowBackground(),

			// TextField
			"TextField.font",
				getControlTextFont(),
				"TextField.background",
				getWindowBackground(),
				"TextField.foreground",
				getControlTextColor(),
				"TextField.shadow",
				getControlShadow(),
				"TextField.darkShadow",
				getControlDarkShadow(),
				"TextField.light",
				getControlLight(),
				"TextField.highlight",
				getControlHighlight(),
				"TextField.inactiveForeground",
				getInactiveControlTextColor(),
				"TextField.inactiveBackground",
				getControl(),
				"TextField.selectionBackground",
				getTextHighlightColor(),
				"TextField.selectionForeground",
				getHighlightedTextColor(),
				"TextField.caretForeground",
				getControlTextColor(),

			// TextPane
			"TextPane.font",
				getControlTextFont(),
				"TextPane.background",
				getWindowBackground(),
				"TextPane.foreground",
				getControlTextColor(),
				"TextPane.selectionBackground",
				getTextHighlightColor(),
				"TextPane.selectionForeground",
				getHighlightedTextColor(),
				"TextPane.caretForeground",
				getWindowBackground(),

			// TitledBorder
			"TitledBorder.font",
				getControlTextFont(),
				"TitledBorder.titleColor",
				getControlTextColor(),

			// ToggleButton
			"ToggleButton.font",
				getControlTextFont(),
				"ToggleButton.background",
				getControl(),
				"ToggleButton.foreground",
				getControlTextColor(),
				"ToggleButton.shadow",
				getControlShadow(),
				"ToggleButton.darkShadow",
				getControlDarkShadow(),
				"ToggleButton.light",
				getControlLight(),
				"ToggleButton.highlight",
				getControlHighlight(),
				"ToggleButton.focus",
				getControlTextColor(),

			// ToolBar
			"ToolBar.font",
				getMenuTextFont(),
				"ToolBar.background",
				getControl(),
				"ToolBar.foreground",
				getControlTextColor(),
				"ToolBar.shadow",
				getControlShadow(),
				"ToolBar.darkShadow",
				getControlDarkShadow(),
				"ToolBar.light",
				getControlLight(),
				"ToolBar.highlight",
				getControlHighlight(),
				"ToolBar.dockingBackground",
				getControl(),
			//			"ToolBar.dockingForeground", red,
			"ToolBar.floatingBackground", getControl(),
			//			"ToolBar.floatingForeground", darkGray,

			// ToolTip
			"ToolTip.font",
				getInfoFont(),
				"ToolTip.background",
				getInfoBackground(),
				"ToolTip.foreground",
				getInfoForeground(),

			// Tree
			"Tree.font",
				getControlTextFont(),
				"Tree.background",
				getWindowBackground(),
				"Tree.foreground",
				getControlTextColor(),
			//				"Tree.hash",
			//				gray,
			"Tree.textForeground",
			getControlTextColor(),
				"Tree.textBackground",
				getWindowBackground(),
				"Tree.selectionForeground",
				getHighlightedTextColor(),
				"Tree.selectionBackground",
				getTextHighlightColor(),
			//				"Tree.selectionBorderColor",
			//				yellow,
			"Tree.expandedIcon",
				treeExpandedIcon,
				"Tree.collapsedIcon",
				treeCollapsedIcon,
				"Tree.openIcon",
				null,
				"Tree.closedIcon",
				null,
				"Tree.leafIcon",
				null,

			// Viewport
			"Viewport.font",
				getControlTextFont(),
				"Viewport.background",
				getControl(),
				"Viewport.foreground",
				getControlTextColor(),
				};

		table.putDefaults(defaults);
	}

	public static Image getDesktopIconImage() {
		if (clinicianDesktopIcon == null)
			getClinicianDesktopIcon();
		return clinicianDesktopIcon.getImage();
	}

	public static Icon getClinicianDesktopIcon() {
		if (clinicianDesktopIcon == null) {
			clinicianDesktopIcon =
				new ImageIcon(DesktopLookAndFeel.class.getResource(DESKTOP_ICON));
		}
		return clinicianDesktopIcon;
	}

	public static void setCurrentTheme(IDesktopTheme theme) {
		if (theme == null)
			return;
		if (currentTheme == theme)
			return;
		currentTheme = theme;
	}

	public static IDesktopTheme getCurrentTheme() {
		return currentTheme;
	}

	public static FontUIResource getControlTextFont() {
		return getCurrentTheme().getControlTextFont();
	}
	public static FontUIResource getSystemTextFont() {
		return getCurrentTheme().getSystemTextFont();
	}
	public static FontUIResource getUserTextFont() {
		return getCurrentTheme().getUserTextFont();
	}
	public static FontUIResource getMenuTextFont() {
		return getCurrentTheme().getMenuTextFont();
	}
	public static FontUIResource getWindowTitleFont() {
		return getCurrentTheme().getWindowTitleFont();
	}
	public static FontUIResource getInfoFont() {
		return getCurrentTheme().getInfoFont();
	}
	public static ColorUIResource getFocusColor() {
		return getCurrentTheme().getFocusColor();
	}
	public static ColorUIResource getControl() {
		return getCurrentTheme().getControl();
	}
	public static ColorUIResource getControlShadow() {
		return getCurrentTheme().getControlShadow();
	}
	public static ColorUIResource getControlDarkShadow() {
		return getCurrentTheme().getControlDarkShadow();
	}
	public static ColorUIResource getControlLight() {
		return getCurrentTheme().getControlLight();
	}
	public static ColorUIResource getControlHighlight() {
		return getCurrentTheme().getControlHighlight();
	}
	public static ColorUIResource getControlDisabled() {
		return getCurrentTheme().getControlDisabled();
	}
	public static ColorUIResource getDesktopColor() {
		return getCurrentTheme().getDesktopColor();
	}
	public static ColorUIResource getPrimaryControl() {
		return getCurrentTheme().getPrimaryControl();
	}
	public static ColorUIResource getPrimaryControlShadow() {
		return getCurrentTheme().getPrimaryControlShadow();
	}
	public static ColorUIResource getPrimaryControlDarkShadow() {
		return getCurrentTheme().getPrimaryControlDarkShadow();
	}
	public static ColorUIResource getPrimaryControlLight() {
		return getCurrentTheme().getPrimaryControlLight();
	}
	public static ColorUIResource getPrimaryControlHighlight() {
		return getCurrentTheme().getPrimaryControlHighlight();
	}
	public static ColorUIResource getSystemTextColor() {
		return getCurrentTheme().getSystemTextColor();
	}
	public static ColorUIResource getInactiveSystemTextColor() {
		return getCurrentTheme().getInactiveSystemTextColor();
	}
	public static ColorUIResource getUserTextColor() {
		return getCurrentTheme().getUserTextColor();
	}
	public static ColorUIResource getInactiveUserTextColor() {
		return getCurrentTheme().getInactiveUserTextColor();
	}
	public static ColorUIResource getControlTextColor() {
		return getCurrentTheme().getControlTextColor();
	}
	public static ColorUIResource getInactiveControlTextColor() {
		return getCurrentTheme().getInactiveControlTextColor();
	}
	public static ColorUIResource getTextHighlightColor() {
		return getCurrentTheme().getTextHighlightColor();
	}
	public static ColorUIResource getHighlightedTextColor() {
		return getCurrentTheme().getHighlightedTextColor();
	}
	public static ColorUIResource getWindowBackground() {
		return getCurrentTheme().getWindowBackground();
	}
	public static ColorUIResource getWindowTitleBackground() {
		return getCurrentTheme().getWindowTitleBackground();
	}
	public static ColorUIResource getWindowTitleForeground() {
		return getCurrentTheme().getWindowTitleForeground();
	}
	public static ColorUIResource getWindowTitleInactiveBackground() {
		return getCurrentTheme().getWindowTitleInactiveBackground();
	}
	public static ColorUIResource getWindowTitleInactiveForeground() {
		return getCurrentTheme().getWindowTitleInactiveForeground();
	}
	public static ColorUIResource getMenuBackground() {
		return getCurrentTheme().getMenuBackground();
	}
	public static ColorUIResource getMenuForeground() {
		return getCurrentTheme().getMenuForeground();
	}
	public static ColorUIResource getMenuSelectedBackground() {
		return getCurrentTheme().getMenuSelectedBackground();
	}
	public static ColorUIResource getMenuSelectedForeground() {
		return getCurrentTheme().getMenuSelectedForeground();
	}
	public static ColorUIResource getMenuDisabledForeground() {
		return getCurrentTheme().getMenuDisabledForeground();
	}
	public static ColorUIResource getAcceleratorForeground() {
		return getCurrentTheme().getAcceleratorForeground();
	}
	public static ColorUIResource getAcceleratorSelectedForeground() {
		return getCurrentTheme().getAcceleratorSelectedForeground();
	}
	public static ColorUIResource getScrollbarBackground() {
		return getCurrentTheme().getScrollbarBackground();
	}
	public static ColorUIResource getInfoBackground() {
		return getCurrentTheme().getInfoBackground();
	}
	public static ColorUIResource getInfoForeground() {
		return getCurrentTheme().getInfoForeground();
	}
}
