package gov.va.med.hds.cd.mum.boot.plaf.desktop;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;

import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;

import com.sun.java.swing.plaf.windows.WindowsGraphicsUtils;
import com.sun.java.swing.plaf.windows.WindowsLookAndFeel;

public class DesktopMenuItemRenderer {

	static final String MAX_ACC_WIDTH = "maxAccWidth";
	static final String MAX_TEXT_WIDTH = "maxTextWidth";

	static Rectangle zeroRect = new Rectangle(0, 0, 0, 0);
	static Rectangle iconRect = new Rectangle();
	static Rectangle textRect = new Rectangle();
	static Rectangle acceleratorRect = new Rectangle();
	static Rectangle checkIconRect = new Rectangle();
	static Rectangle arrowIconRect = new Rectangle();
	static Rectangle viewRect = new Rectangle(Short.MAX_VALUE, Short.MAX_VALUE);
	static Rectangle r = new Rectangle();

	protected Color selectionBackground;
	protected Color selectionForeground;
	protected Color disabledForeground;
	protected Color acceleratorForeground;
	protected Color acceleratorSelectionForeground;
	protected Font acceleratorFont;

	private String acceleratorDelimiter;

	DesktopMenuItemRenderer(String prefix) {
		installDefaults(prefix);
	}

	protected void installDefaults(String prefix) {
		acceleratorFont = UIManager.getFont("MenuItem.acceleratorFont");

		// MenuItem specific defaults
		if (selectionBackground == null || selectionBackground instanceof UIResource) {
			selectionBackground = UIManager.getColor(prefix + ".selectionBackground");
		}
		if (selectionForeground == null || selectionForeground instanceof UIResource) {
			selectionForeground = UIManager.getColor(prefix + ".selectionForeground");
		}
		if (disabledForeground == null || disabledForeground instanceof UIResource) {
			disabledForeground = UIManager.getColor(prefix + ".disabledForeground");
		}
		if (acceleratorForeground == null || acceleratorForeground instanceof UIResource) {
			acceleratorForeground = UIManager.getColor(prefix + ".acceleratorForeground");
		}
		if (acceleratorSelectionForeground == null
			|| acceleratorSelectionForeground instanceof UIResource) {
			acceleratorSelectionForeground =
				UIManager.getColor(prefix + ".acceleratorSelectionForeground");
		}
		// Get accelerator delimiter
		acceleratorDelimiter = UIManager.getString("MenuItem.acceleratorDelimiter");
		if (acceleratorDelimiter == null) {
			acceleratorDelimiter = "+";
		}
	}

	private void resetRects() {
		iconRect.setBounds(zeroRect);
		textRect.setBounds(zeroRect);
		acceleratorRect.setBounds(zeroRect);
		checkIconRect.setBounds(zeroRect);
		arrowIconRect.setBounds(zeroRect);
		viewRect.setBounds(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);
		r.setBounds(zeroRect);
	}

	protected void paintMenuItem(
		Graphics g,
		JComponent c,
		Icon checkIcon,
		Icon arrowIcon,
		Color background,
		Color foreground,
		int defaultTextIconGap) {
		JMenuItem b = (JMenuItem) c;
		ButtonModel model = b.getModel();

		//   Dimension size = b.getSize();
		int menuWidth = b.getWidth();
		int menuHeight = b.getHeight();
		Insets i = c.getInsets();

		resetRects();

		viewRect.setBounds(0, 0, menuWidth, menuHeight);

		viewRect.x += i.left;
		viewRect.y += i.top;
		viewRect.width -= (i.right + viewRect.x);
		viewRect.height -= (i.bottom + viewRect.y);

		Font holdf = g.getFont();
		Font f = c.getFont();
		g.setFont(f);
		FontMetrics fm = g.getFontMetrics(f);
		FontMetrics fmAccel = g.getFontMetrics(acceleratorFont);

		// get Accelerator text
		KeyStroke accelerator = b.getAccelerator();
		String acceleratorText = "";
		if (accelerator != null) {
			int modifiers = accelerator.getModifiers();
			if (modifiers > 0) {
				acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
				//acceleratorText += "-";
				acceleratorText += acceleratorDelimiter;
			}

			int keyCode = accelerator.getKeyCode();
			if (keyCode != 0) {
				acceleratorText += KeyEvent.getKeyText(keyCode);
			} else {
				acceleratorText += accelerator.getKeyChar();
			}
		}

		// layout the text and icon
		String text =
			layoutMenuItem(
				b,
				fm,
				b.getText(),
				fmAccel,
				acceleratorText,
				b.getIcon(),
				checkIcon,
				arrowIcon,
				b.getVerticalAlignment(),
				b.getHorizontalAlignment(),
				b.getVerticalTextPosition(),
				b.getHorizontalTextPosition(),
				viewRect,
				iconRect,
				textRect,
				acceleratorRect,
				checkIconRect,
				arrowIconRect,
				b.getText() == null ? 0 : defaultTextIconGap,
				defaultTextIconGap);

		paintBackground(g, b, background);

		Color holdc = g.getColor();

		paintCheck(g, c, checkIcon, foreground, model, holdc);
		paintIcon(g, c, b, model);
		paintText(g, c, b, text);
		paintAcceleratorText(g, c, b, model, fmAccel, acceleratorText);
		paintArrow(g, c, arrowIcon, foreground, model);

		g.setColor(holdc);
		g.setFont(holdf);
	}
	/**
			 * Method which renders the text of the current menu item.
			 * <p>
			 * @param g Graphics context
			 * @param menuItem Current menu item to render
			 * @param textRect Bounding rectangle to render the text.
			 * @param text String to render
			 */
	protected void paintText(Graphics g, JMenuItem menuItem, Rectangle textRect, String text) {
		// Note: This method is almost identical to the same method in WindowsMenuUI
		ButtonModel model = menuItem.getModel();

		if (!model.isEnabled()) {
			// *** paint the text disabled
			WindowsGraphicsUtils.paintText(g, menuItem, textRect, text, 0);
		} else {
			FontMetrics fm = g.getFontMetrics();
			int mnemonicIndex = menuItem.getDisplayedMnemonicIndex();
			// W2K Feature: Check to see if the Underscore should be rendered.
			if (WindowsLookAndFeel.isMnemonicHidden() == true) {
				mnemonicIndex = -1;
			}

			Color oldColor = g.getColor();

			// *** paint the text normally
			if (model.isArmed() || (menuItem instanceof JMenu && model.isSelected())) {
				g.setColor(selectionForeground); // Uses protected field.
			}
			BasicGraphicsUtils.drawStringUnderlineCharAt(
				g,
				text,
				mnemonicIndex,
				textRect.x,
				textRect.y + fm.getAscent());
			g.setColor(oldColor);
		}
	}

	protected void paintCheck(
		Graphics g,
		JComponent c,
		Icon checkIcon,
		Color foreground,
		ButtonModel model,
		Color holdc) {
		if (checkIcon != null) {
			if (model.isArmed() || (c instanceof JMenu && model.isSelected())) {
				g.setColor(foreground);
			} else {
				g.setColor(holdc);
			}
			if (useCheckAndArrow((JMenuItem) c) /*&& ((JMenuItem) c).getIcon() != null*/) {
				checkIcon.paintIcon(c, g, checkIconRect.x, checkIconRect.y);
			}
			g.setColor(holdc);
		}
	}

	protected void paintIcon(Graphics g, JComponent c, JMenuItem b, ButtonModel model) {
		if (b.getIcon() != null) {
			Icon icon;
			if (!model.isEnabled()) {
				icon = (Icon) b.getDisabledIcon();
			} else if (model.isPressed() && model.isArmed()) {
				icon = (Icon) b.getPressedIcon();
				if (icon == null) {
					// Use default icon
					icon = (Icon) b.getIcon();
				}
			} else {
				icon = (Icon) b.getIcon();
			}

			if (icon != null)
				icon.paintIcon(c, g, iconRect.x, iconRect.y);
		}
	}

	protected void paintArrow(
		Graphics g,
		JComponent c,
		Icon arrowIcon,
		Color foreground,
		ButtonModel model) {
		if (arrowIcon != null) {
			if (model.isArmed() || (c instanceof JMenu && model.isSelected()))
				g.setColor(foreground);
			if (useCheckAndArrow((JMenuItem) c))
				arrowIcon.paintIcon(c, g, arrowIconRect.x, arrowIconRect.y);
		}
	}

	protected void paintAcceleratorText(
		Graphics g,
		JComponent c,
		JMenuItem b,
		ButtonModel model,
		FontMetrics fmAccel,
		String acceleratorText) {
		if (acceleratorText != null && !acceleratorText.equals("")) {

			//Get the maxAccWidth from the parent to calculate the offset.
			int accOffset = 0;
			Container parent = c.getParent();
			if (parent != null && parent instanceof JComponent) {
				JComponent p = (JComponent) parent;
				Integer maxValueInt = (Integer) p.getClientProperty(MAX_ACC_WIDTH);
				int maxValue = maxValueInt != null ? maxValueInt.intValue() : acceleratorRect.width;

				//Calculate the offset, with which the accelerator texts will be drawn with.
				accOffset = maxValue - acceleratorRect.width;
			}

			g.setFont(acceleratorFont);
			if (!model.isEnabled()) {
				// *** paint the acceleratorText disabled
				if (disabledForeground != null) {
					g.setColor(disabledForeground);
					BasicGraphicsUtils.drawString(
						g,
						acceleratorText,
						0,
						acceleratorRect.x - accOffset,
						acceleratorRect.y + fmAccel.getAscent());
				} else {
					g.setColor(b.getBackground().brighter());
					BasicGraphicsUtils.drawString(
						g,
						acceleratorText,
						0,
						acceleratorRect.x - accOffset,
						acceleratorRect.y + fmAccel.getAscent());
					g.setColor(b.getBackground().darker());
					BasicGraphicsUtils.drawString(
						g,
						acceleratorText,
						0,
						acceleratorRect.x - accOffset - 1,
						acceleratorRect.y + fmAccel.getAscent() - 1);
				}
			} else {
				// *** paint the acceleratorText normally
				if (model.isArmed() || (c instanceof JMenu && model.isSelected())) {
					g.setColor(acceleratorSelectionForeground);
				} else {
					g.setColor(acceleratorForeground);
				}
				BasicGraphicsUtils.drawString(
					g,
					acceleratorText,
					0,
					acceleratorRect.x - accOffset,
					acceleratorRect.y + fmAccel.getAscent());
			}
		}
	}

	protected void paintText(Graphics g, JComponent c, JMenuItem b, String text) {
		if (text != null) {
			View v = (View) c.getClientProperty(BasicHTML.propertyKey);
			if (v != null) {
				v.paint(g, textRect);
			} else {
				paintText(g, b, textRect, text);
			}
		}
	}

	protected void paintBackground(Graphics g, JMenuItem menuItem, Color bgColor) {
		ButtonModel model = menuItem.getModel();
		Color oldColor = g.getColor();
		int menuWidth = menuItem.getWidth();
		int menuHeight = menuItem.getHeight();

		if (menuItem.isOpaque()) {
			if (model.isArmed() || (menuItem instanceof JMenu && model.isSelected())) {
				g.setColor(bgColor);
				g.fillRect(0, 0, menuWidth, menuHeight);
			} else {
				g.setColor(menuItem.getBackground());
				g.fillRect(0, 0, menuWidth, menuHeight);
			}
			g.setColor(oldColor);
		}
	}

	/** 
	 * Compute and return the location of the icons origin, the 
	 * location of origin of the text baseline, and a possibly clipped
	 * version of the compound labels string.  Locations are computed
	 * relative to the viewRect rectangle. 
	 */
	private String layoutMenuItem(
		JMenuItem menuItem,
		FontMetrics fm,
		String text,
		FontMetrics fmAccel,
		String acceleratorText,
		Icon icon,
		Icon checkIcon,
		Icon arrowIcon,
		int verticalAlignment,
		int horizontalAlignment,
		int verticalTextPosition,
		int horizontalTextPosition,
		Rectangle viewRect,
		Rectangle iconRect,
		Rectangle textRect,
		Rectangle acceleratorRect,
		Rectangle checkIconRect,
		Rectangle arrowIconRect,
		int textIconGap,
		int menuItemGap) {

		// layout as if no icon specified
		SwingUtilities.layoutCompoundLabel(
			menuItem,
			fm,
			text,
			null,
			verticalAlignment,
			horizontalAlignment,
			verticalTextPosition,
			horizontalTextPosition,
			viewRect,
			iconRect,
			textRect,
			textIconGap);

		initializeAcceleratorTextBounds(fmAccel, acceleratorText, acceleratorRect);
		initializeCheckAndArrowIconBounds(menuItem, checkIcon, arrowIcon, checkIconRect, arrowIconRect);

		if (menuItem.getComponentOrientation().isLeftToRight()) {
			textRect.x += menuItemGap;

			// Position the Accelerator text rect
			acceleratorRect.x =
				viewRect.x
					+ viewRect.width
					- arrowIconRect.width
					- menuItemGap
					- acceleratorRect.width;

			// Position the Check and Arrow Icons 
			if (useCheckAndArrow(menuItem)) {
				checkIconRect.x = viewRect.x;
				textRect.x += menuItemGap + checkIconRect.width;
				arrowIconRect.x = viewRect.x + viewRect.width - menuItemGap - arrowIconRect.width;
			}
		} else {
			textRect.x -= menuItemGap;

			// Position the Accelerator text rect
			acceleratorRect.x = viewRect.x + arrowIconRect.width + menuItemGap;

			// Position the Check and Arrow Icons 
			if (useCheckAndArrow(menuItem)) {
				checkIconRect.x = viewRect.x + viewRect.width - checkIconRect.width;
				textRect.x -= menuItemGap + checkIconRect.width;
				arrowIconRect.x = viewRect.x + menuItemGap;
			}
		}

		verticallyAlignAcceleratorTextAndIcons(
		menuItem,
			viewRect,
			acceleratorRect,
			checkIconRect,
			arrowIconRect);

		// position iconRect where checkIconRect is (they should be the same size)
		iconRect.x = checkIconRect.x;
		iconRect.y = checkIconRect.y;

		//		System.out.println("Layout: text="+menuItem.getText()+"\n\tv="
		//						   +viewRect+"\n\tc="+checkIconRect+"\n\ti="
		//						   +iconRect+"\n\tt="+textRect+"\n\tacc="
		//						   +acceleratorRect+"\n\ta="+arrowIconRect+"\n");

		return text;
	}

	/* Initialize the acceelratorText bounds rectangle textRect.  If a null 
	 * or and empty String was specified we substitute "" here 
	 * and use 0,0,0,0 for acceleratorTextRect.
	 */
	private void initializeAcceleratorTextBounds(
		FontMetrics fmAccel,
		String acceleratorText,
		Rectangle acceleratorRect) {
		if ((acceleratorText == null) || acceleratorText.equals("")) {
			acceleratorRect.width = acceleratorRect.height = 0;
			acceleratorText = "";
		} else {
			acceleratorRect.width = SwingUtilities.computeStringWidth(fmAccel, acceleratorText);
			acceleratorRect.height = fmAccel.getHeight();
		}
	}

	private void initializeCheckAndArrowIconBounds(
		JMenuItem menuItem,
		Icon checkIcon,
		Icon arrowIcon,
		Rectangle checkIconRect,
		Rectangle arrowIconRect) {
		if (useCheckAndArrow(menuItem)) {
			if (checkIcon != null) {
				checkIconRect.width = checkIcon.getIconWidth();
				checkIconRect.height = checkIcon.getIconHeight();
			} else {
				checkIconRect.width = checkIconRect.height = 0;
			}

			/* Initialize the arrowIcon bounds rectangle width & height.
			 */

			if (arrowIcon != null) {
				arrowIconRect.width = arrowIcon.getIconWidth();
				arrowIconRect.height = arrowIcon.getIconHeight();
			} else {
				arrowIconRect.width = arrowIconRect.height = 0;
			}
		}
	}

	private void verticallyAlignAcceleratorTextAndIcons(
		JMenuItem menuItem,
		Rectangle viewRect,
		Rectangle acceleratorRect,
		Rectangle checkIconRect,
		Rectangle arrowIconRect) {
		acceleratorRect.y = viewRect.y + (viewRect.height / 2) - (acceleratorRect.height / 2);
		if (useCheckAndArrow(menuItem)) {
			arrowIconRect.y = viewRect.y + (viewRect.height / 2) - (arrowIconRect.height / 2);
			checkIconRect.y = viewRect.y + (viewRect.height / 2) - (checkIconRect.height / 2);
		}
	}

	/*
	 * Returns false if the component is a JMenu and it is a top
	 * level menu (on the menubar).
	 */
	private boolean useCheckAndArrow(JMenuItem menuItem) {
		boolean b = true;
		if ((menuItem instanceof JMenu) && (((JMenu) menuItem).isTopLevelMenu())) {
			b = false;
		}
		return b;
	}
}
