package gov.va.med.hds.cd.mum.boot.plaf.desktop;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

/**
 * This class is used to isolate the loading of the look and feel from the rest of the Boot, 
 * with the idea being that it be called on the same thread that starts up the Splash Screen,
 * Thus helping to load more of swing in a seperate thread.
 */
public class LookAndFeelLauncher {

	
	/**
	 * Used to initialize the default look and feel.  This method may be called from more than
	 * one thread, and may be called more than one time, thus it is synchronized and will only
	 * set the look and feel once.
	 */
	public synchronized static void initDefaultLookAndFeel() {
		if (initialize) {
			initialize = false;
			if (!DesktopLookAndFeel
				.class
				.equals(UIManager.getLookAndFeel().getClass())) {
                LookAndFeel laf =
                    new DesktopLookAndFeel(
                        DesktopLookAndFeel.class.getClassLoader());
				if (laf.isSupportedLookAndFeel()) {
					try {
						UIManager.setLookAndFeel(laf);
						LookAndFeelUtilities.updateFrames();
					} catch (Exception e) {
						e.printStackTrace();
					}
				} else {
					try {
						UIManager.setLookAndFeel(
							UIManager.getSystemLookAndFeelClassName());
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	private static boolean initialize = true;
}
