package gov.va.med.hds.cd.mum.boot.utils;

/**
 * This class manages a list of interger values without using the Interger class 
 */
public class IntList {

    /**
     * Constructor for IntList 
     */
    public IntList() {
        this(10);
    }

    public IntList(int initialCapacity) {
        super();
        if (initialCapacity < 0)
            throw new IllegalArgumentException("Illegal Capacity: " + initialCapacity);
        data = new int[initialCapacity];
    }

    /**
     * Clears the list.
     */
    public void clear() {
        size = 0;
    }

    /**
     * Returns the value at the specifiec location in the list.
     * @param index
     */
    public int get(int index) {
        rangeCheck(index);
        return data[index];
    }

    /**
     * Assigns a valud to the specified index in the list.  If the list is smaller than the specified
     * index, the list is expanded to a size of index+1, with new indexes being set to a value of zero.  
     * @param index
     * @param value
     */
    public void set(int index, int value) {
        ensureCapacity(index + 1);
        if (size <= index) {
            for (int i = size; i < index; i++)
                data[i] = 0;
            size = index + 1;
        }
        data[index] = value;
    }

    /**
     * Adds a value to the end of the list
     * @param value
     */
    public void add(int value) {
        ensureCapacity(size + 1);
        data[size] = value;
        size++;
    }

    /**
     * Removes a value from the specified index, shrinking the list accordingly
     * @param index
     */
    public void remove(int index) {
        rangeCheck(index);
        int numMoved = size - index - 1;
        if (numMoved > 0)
            System.arraycopy(data, index + 1, data, index, numMoved);
        size--;
    }

    private transient int data[];
    private int size;
    private void ensureCapacity(int minCapacity) {
        int oldCapacity = data.length;
        if (minCapacity > oldCapacity) {
            int oldData[] = data;
            int newCapacity = (oldCapacity * 3) / 2 + 1;
            if (newCapacity < minCapacity)
                newCapacity = minCapacity;
            data = new int[newCapacity];
            System.arraycopy(oldData, 0, data, 0, size);
        }
    }

    /**
     * Returns the size of the list.
     */
    public int size() {
        return size;
    }

    private void rangeCheck(int index) {
        if ((index < 0) || (index >= size)) {
            throw new IndexOutOfBoundsException("Index: " + index + ", Size: " + size);
        }
    }
}