package gov.va.med.hds.cd.mum.boot.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Stores values in a list through key/value pairs, but also allows access to the list by index. 
 */
public class KeyedList {

    private ArrayList list;
    private ArrayList keys;
    private HashMap map;

    /**
     * Constructor for KeyedList 
     */
    public KeyedList() {
        super();
        list = new ArrayList();
        map = new HashMap();
        keys = new ArrayList();
    }

    /**
     * Clears the list. 
     */
    public void clear() {
        list.clear();
        map.clear();
        keys.clear();
    }

    /**
     * Returns true if the list contains the specified key value
     * @param key
     */
    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }

    /**
     * Returns true 
     * @param value
     */
    public boolean containsValue(Object value) {
        return list.contains(value);
    }

    /**
     * Returns the list of keys
     */
    public List getKeys() {
        return keys;

    }

    /**
     * Returns the list of values
     */
    public List getList() {
        return list;
    }

    /**
     * Returns the value associated with the specified key
     * @param key
     */
    public Object get(Object key) {
        if (key == null)
            return null;
        return map.get(key);
    }

    /**
     * Returns the value at the specified index
     * @param index
     */
    public Object get(int index) {
        return list.get(index);
    }

    /**
     * Return the size of the list.
     */
    public int size() {
        return list.size();
    }

    /**
     * Adds a value to the list with the specifed key, if that key has not already been used.  The
     * key and value are also added to the end of the list when accessing the list by index.
     * @param key
     * @param value
     */
    public void add(Object key, Object value) {
        if ((key == null) || (value == null))
            return;
        Object data = map.get(key);
        if (data == null) {
            map.put(key, value);
            keys.add(key);
            list.add(value);
        }
    }

    /**
     * Removes the key and it's value from the list 
     * @param key
     */
    public void remove(Object key) {
        if (key == null)
            return;
        Object data = map.get(key);
        if (data != null) {
            list.remove(data);
            map.remove(key);
            keys.remove(key);
        }
    }

    /**
     * Replaces the value associated with a key, if that key has already been added to the list.
     * @param key
     * @param value
     */
    public void update(Object key, Object value) {
        if (key == null)
            return;
        Object data = map.get(key);
        if (data != null) {
            list.remove(data);
            list.add(value);
            map.put(key, value);
        }
    }

}
