package gov.va.med.hds.cd.mum.boot.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * This class is similar to a hash map, except that multiple values can be assocaited with a single
 * key.  It also allows indexed access to values in the list.
 */
public class ListMap {

    private ArrayList list;
    private ArrayList keys;
    private HashMap map;
    private static final List emptyList = new ArrayList(0);

    /**
     * Constructor for ListMap 
     */
    public ListMap() {
        super();
        list = new ArrayList();
        map = new HashMap();
        keys = new ArrayList();
    }

    /**
     * Clears the list 
     */
    public void clear() {
        list.clear();
        map.clear();
        keys.clear();
    }

    /**
     * Returns true if the list contains one or more values associated with the specified key
     * @param key
     */
    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }

    /**
     * Returns true if the list contains the specified value.
     * @param value
     */
    public boolean containsValue(Object value) {
        return list.contains(value);
    }

    /**
     * Returns the list of keys used in the list that have at least one value associated with them.
     */
    public List getKeys() {
        return keys;

    }

    /**
     * Returns a list containing zero or more values associated with the specified key.
     * @param key
     */
    public List get(Object key) {
        if (key == null)
            return emptyList;
        Object data = map.get(key);
        if (data == null)
            return emptyList;
        return (List) data;
    }

    /**
     * Returns the unique value at the specified index in the list.
     * @param index
     */
    public Object get(int index) {
        return list.get(index);
    }

    /**
     * Returns the size of the list.
     */
    public int size() {
        return list.size();
    }

    /**
     * Adds the value to the list of values associated with the specified key, if it has not already
     * been added to the list. 
     * @param key
     * @param value
     */
    public List add(Object key, Object value) {
        if (value instanceof List)
            throw new IllegalArgumentException();
        List data = (List) map.get(key);
        boolean add = true;
        if (data == null) {
            data = new ArrayList();
            data.add(value);
            map.put(key, data);
            keys.add(key);
        } else {
            if (data.contains(value))
                add = false;
            else
                data.add(value);
        }
        if (add)
            list.add(value);
        return data;
    }

    /**
     * Removes the specified value from the list of values associated with the key.
     * @param key
     * @param value
     */
    public void remove(Object key, Object value) {
        List data = (List) map.get(key);
        if (data != null) {
            if (data.contains(value)) {
                data.remove(value);
                list.remove(value);
                if (data.size() == 0) {
                    keys.remove(key);
                    map.remove(key);
                }
            }
        }
    }

    /**
     * Adds all the entries passed in ListMap to this ListMap.  Any key/value pairs already in the
     * list are not added again.
     * @param mappedList
     */
    public void addAll(ListMap mappedList) {
        List keys = mappedList.getKeys();
        for (int i = 0; i < keys.size(); i++) {
            Object key = keys.get(i);
            List tempList = mappedList.get(key);
            for (int j = 0; j < tempList.size(); j++) {
                add(key, tempList.get(j));
            }
        }
    }

    /**
     * Removes all values associated with the specified key.
     * @param key
     */
    public void removeList(Object key) {
        List data = (List) map.get(key);
        if (data != null) {
        	for (int i = 0; i < data.size(); i++) {
        		Object value = data.get(i);
				list.remove(value);
            }
			keys.remove(key);
			map.remove(key);
        }
    }
}
