package gov.va.med.hds.cd.mum.boot.utils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;

import org.xml.sax.InputSource;

/**
 * Base class used by ITreeData classes that load data from a resource.
 */
public abstract class LoadableTreeData extends TreeData implements ILoadableTreeData {

    /**
     * Constructor for LoadableTreeData 
     */
    public LoadableTreeData() {
        super();
    }

	/**
     * @see gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData#load(java.io.File)
     */
    public void load(File file)
		throws FileNotFoundException, TreeDataException {
		load(new BufferedInputStream(new FileInputStream(file)));
	}

	/**
     * @see gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData#load(java.net.URL)
     */
    public void load(URL url) throws FileNotFoundException, TreeDataException {
		try {
			load(url.openConnection().getInputStream());
		} catch (IOException e) {
			if (e instanceof FileNotFoundException)
				throw (FileNotFoundException) e;
			else
				throw new TreeDataException(e);
		}
	}

	/**
     * @see gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData#load(java.io.InputStream)
     */
    public void load(InputStream stream) throws TreeDataException {
		internalLoad(new InputStreamReader(stream));
	}

	/**
     * @see gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData#load(org.xml.sax.InputSource)
     */
    public void load(InputSource source) throws TreeDataException {
		internalLoad(source.getCharacterStream());
	}
	
	/**
     * @see gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData#load(java.io.Reader)
     */
    public void load(Reader reader) throws TreeDataException {
		internalLoad(reader);
	}

	private void internalLoad(Reader reader) throws TreeDataException {
		clear();
		try {
			parse(reader);
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				throw new TreeDataException(e);
			}
		}
	}
	
	/**
	 * Must be implemented by subclasses to parse the data.
     * @param reader
     * @throws TreeDataException
     */
    protected abstract void parse(Reader reader) throws TreeDataException;
}
