package gov.va.med.hds.cd.mum.boot.utils;

import java.io.Reader;

import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * TreeData class that parses XML using SAX
 */
public class SAXTreeData extends LoadableTreeData {

    int level;
    int count;

    /**
     * Constructor for SAXTreeData.
     */
    public SAXTreeData() {
        super();
        ignoreWhitespace(false);
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.ITreeData#clear()
     */
    public void clear() {
        super.clear();
        count = 0;
        level = -1;
    }
    
    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.LoadableTreeData#parse(java.io.Reader)
     */
    protected void parse(Reader reader) throws TreeDataException {
        try {
            SAXParserFactory.newInstance().newSAXParser().parse(
                new InputSource(reader),
                new dataHandler());
        } catch (Exception e) {
            if (e instanceof TreeDataException)
                throw (TreeDataException) e;
            else
                throw new TreeDataException(e);
        }
        if (elementIndex.size() <= 0)
            error();
    }

    private class dataHandler extends DefaultHandler {
        public void startElement(String uri, String localName, String qName, Attributes attributes)
            throws SAXException {
            if (level >= 0)
                children.get(current.get(level)).add(count);
            elementIndex.set(count, data.size());
            data.add(qName);
            attribs.clear(count);
            children.clear(count);
            for (int i = 0; i < attributes.getLength(); i++) {
                attribs.get(count).add(data.size());
                data.add(attributes.getQName(i));
                data.add(attributes.getValue(i));
            }
            level++;
            current.set(level, count);
            count++;
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            if (strbuf.hasData(level)) {
                dataIndex.set(current.get(level), data.size());
                data.add(strbuf.get(level).substring(0, strbuf.get(level).length()));
                strbuf.get(level).setLength(0);
            } else
                dataIndex.set(current.get(level), -1);
            level--;
        }

        public void characters(char ch[], int start, int length) throws SAXException {
            if (length > 0) {
                if ((getIgnoreWhitespace()) && (isAllWhitespace(ch, start, length)))
                    return;
                strbuf.ensureBuffer(level).append(ch, start, length);
            }
        }

        private boolean isAllWhitespace(char[] ch, int start, int length) {
            for (int i = 0; i < length; i++) {
                if (ch[start + i] > ' ')
                    return false;
            }
            return true;
        }

    }
}
