package gov.va.med.hds.cd.mum.boot.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * This is a simple list class that uses in internal hash map speed up calls to the contains method.
 * It also prevents the same value from being added to the list twice. 
 */
public class SimpleList {

    /**
     * Constructor for SimpleList 
     */
    public SimpleList() {
        super();
        list = new ArrayList();
        map = new HashMap();
    }

    /**
     * Constructor for SimpleList 
     */
    public SimpleList(SimpleList copy) {
        this();
        addAll(copy);
    }

    private ArrayList list;
    private HashMap map;
    private static final Object obj = new Object();

    /**
     * Clears the list 
     */
    public void clear() {
        list.clear();
        map.clear();
    }

    /**
     * Returns true if the list contains the specified value.
     * @param value
     */
    public boolean contains(Object value) {
        return map.containsKey(value);
    }

    /**
     * Returns the object at the specified index.
     * @param index
     */
    public Object get(int index) {
        return list.get(index);
    }

    /**
     * Returns the size of the list
     */
    public int size() {
        return list.size();
    }

    /**
     * Adds the value to the end of the list
     * @param data
     */
    public void add(Object data) {
        if ((data != null) && (!map.containsKey(data))) {
            map.put(data, obj);
            list.add(data);
        }
    }

    /**
     * Removes the value from the list
     * @param data
     */
    public void remove(Object data) {
        if ((data != null) && (map.containsKey(data))) {
            map.remove(data);
            list.remove(data);
        }
    }

    /**
     * Adds all values in the specified list that are not already part of the existing list to the
     * list.
     * @param temp
     */
    public void addAll(SimpleList temp) {
        if (temp != null) {
            for (int i = 0; i < temp.size(); i++) {
                add(temp.get(i));
            }
        }
    }

    /**
     * Returns the internal list.  Do NOT modify this list, it is for reading purposes only
     */
    public List getList() {
        return list;
    }

    /**
     * Removes the data at the specified index.
     * @param idx
     */
    public void remove(int idx) {
        Object obj = list.get(idx);
        list.remove(idx);
        map.remove(obj);
    }

    /**
     * Allows several items to be truncated from the end of the list.
     * @param length
     * @throws IndexOutOfBoundsException - thrown if you try to set the list to larger than it's 
     * current size
     */
    public void setLength(int length) {
        if (length > list.size())
            throw new IndexOutOfBoundsException();
        while (list.size() > length)
            remove(list.size() - 1);
    }
}
