package gov.va.med.hds.cd.mum.boot.utils;

import gov.va.med.hds.cd.mum.boot.IBootConstants;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

/**
 * Simple implementation of the ITreeData interface
 */
public class SimpleXMLTreeData extends LoadableTreeData implements ISavableTreeData {

    private Parser parser;
    private StringBuffer temp;
    
    /**
     * Constructor for SimpleXMLTreeData.
     */
    public SimpleXMLTreeData() {
        super();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.ITreeData#clear()
     */
    public void clear() {
        super.clear();
        if (parser != null)
            parser.clear();
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData#save(java.io.File)
     */
    public void save(File file) throws IOException {
        file.getParentFile().mkdirs();
        Writer writer = new BufferedWriter(new FileWriter(file));
        save(writer);
    }

    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.ISavableTreeData#save(java.io.Writer)
     */
    public void save(Writer writer) throws IOException {
        if (temp == null)
            temp = new StringBuffer();
        try {
            writer.write(xmlHeader);
            writer.write(IBootConstants.WIN_EOL);
            writeData(writer, getRootID(), 0);
        } finally {
            writer.close();
        }
    }

    private void writeData(Writer writer, int id, int indent) throws IOException {
        writer.write("<");
        writer.write(getName(id));
        for (int i = 0; i < getAttributeCount(id); i++) {
            writer.write(' ');
            writer.write(getAttributeName(id, i));
            writer.write("=\"");
            addEncodedData(writer, getAttributeValue(id, i));
            writer.write('"');
        }
        String value = getValue(id);
        if ((value == null) && (getChildCount(id) == 0))
            writer.write("/>");
        else {
            boolean addWhitespace = true;
            writer.write('>');
            if (value != null) {
                addEncodedData(writer, value);
                addWhitespace = false;
            }
            if (getChildCount(id) > 0) {
                indent++;
                if (addWhitespace)
                    writer.write(IBootConstants.WIN_EOL);
                for (int i = 0; i < getChildCount(id); i++) {
                    if (addWhitespace) {
                        for (int j = 0; j < indent; j++)
                            writer.write("  ");
                    }
                    writeData(writer, getChildID(id, i), indent);
                    writer.write(IBootConstants.WIN_EOL);
                    addWhitespace = true;
                }
                indent--;
            }
            if (addWhitespace) {
                for (int j = 0; j < indent; j++)
                    writer.write("  ");
            }
            writer.write("</");
            writer.write(getName(id));
            writer.write(">");
        }
    }

    private static final String xmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";

    /**
     * @see gov.va.med.hds.cd.mum.boot.utils.LoadableTreeData#parse(java.io.Reader)
     */
    protected void parse(Reader reader) throws TreeDataException {
        if (parser == null)
            parser = new Parser();
        parser.parse(reader, this);
    }

    private void addEncodedData(Writer writer, String value) throws IOException {
        temp.setLength(0);
        temp.append(value);
        Utils.encodedData(temp);
        for (int i = 0; i < temp.length(); i++)
            writer.write(temp.charAt(i));
    }

    static class Parser {

        private TreeData treeData;
        private StringBuffer temp = new StringBuffer();
        private StringBuffer buf = new StringBuffer(2048);
        private char[] buffer = new char[1024];
        private final TreeData.BufList strbuf = new TreeData.BufList();
        private Reader reader;
        private int index;
        private int index2;
        private int level;
        private int count;
        private char ch;

        /**
         * Constructor for Parser.
         */
        public Parser() {
            super();
        }

        /**
         * @see gov.va.med.hds.cd.mum.utils.ITreeData#clear()
         */
        public void clear() {
            count = 0;
            level = -1;
            temp.setLength(0);
            buf.setLength(0);
        }

        public void parse(Reader reader, ITreeData data) throws TreeDataException {
            try {
                this.reader = reader;
                this.treeData = (TreeData) data;
                treeData.clear();
                getInitialDataFromReader();
                char c;
                boolean gettingAttributes;
                boolean reading = true;
                int level = 0;
                String tag, attrib;
                while (reading) {
                    getIndexOf("<");
                    if (index2 < 0) {
                        reading = false;
                        if (level != 0)
                            error();
                    } else {
                        if (level > 0) {
                            decodeData(index, index2);
                            characters(temp);
                        }
                        index2++;
                        index = index2;
                        index = nextChar(index);
                        if (ch == '/') {
                            index2 = eatLabel(index2 + 1);
                            tag = buf.substring(index + 1, index2);
                            index = eatWhitespace(index2);
                            index = nextChar(index);
                            if (ch != '>')
                                error();
                            endElement(tag);
                            level--;
                            index++;
                        } else {
                            index2 = eatLabel(index2);
                            if (index2 > index) {
                                tag = buf.substring(index, index2);
                                newElementStart(tag);
                                level++;
                                gettingAttributes = true;
                                while (gettingAttributes) {
                                    index = eatWhitespace(index2);
                                    index = nextChar(index);
                                    c = ch;
                                    switch (c) {
                                        case '/' :
                                            index++;
                                            index = nextChar(index);
                                            if (ch != '>')
                                                error();
                                            index++;
                                            newElementEnd();
                                            endElement(tag);
                                            level--;
                                            gettingAttributes = false;
                                            break;
                                        case '>' :
                                            newElementEnd();
                                            index++;
                                            gettingAttributes = false;
                                            break;
                                        default :
                                            if (!Utils.validTagChar(c))
                                                error();
                                            index2 = index + 1;
                                            index2 = eatLabel(index2);
                                            attrib = buf.substring(index, index2);
                                            index = eatWhitespace(index2);
                                            index = nextChar(index);
                                            if (ch != '=')
                                                error();
                                            index = eatWhitespace(index + 1);
                                            index = nextChar(index);
                                            if (ch != '"')
                                                error();
                                            index++;
                                            index2 = index;
                                            do {
                                                index2 = nextChar(index2);
                                                if (ch != '"')
                                                    index2++;
                                            } while (ch != '"');
                                            newAttribute(attrib, getEncodedString(index, index2));
                                            index2++;
                                            break;
                                    }
                                }
                            }
                        }
                    }
                }
            } catch (Exception e) {
                if (e instanceof TreeDataException)
                    throw (TreeDataException) e;
                else
                    throw new TreeDataException(e);
            } finally {
                if (treeData.elementIndex.size() <= 0)
                    error();
                treeData = null;
                if (reader != null) {
                    try {
                        reader.close();
                    } catch (IOException e1) {
                        throw new TreeDataException(e1);
                    } finally {
                        reader = null;
                    }
                }
            }
        }

        private int eatLabel(int idx2) throws TreeDataException {
            boolean eating = true;
            do {
                idx2 = nextChar(idx2);
                if (Utils.validTagChar(ch))
                    idx2++;
                else
                    eating = false;
            } while (eating);
            return idx2;
        }

        private int eatWhitespace(int idx) throws TreeDataException {
            boolean eating = true;
            do {
                idx = nextChar(idx);
                if (ch <= ' ')
                    idx++;
                else
                    eating = false;
            } while (eating);
            return idx;
        }

        private int nextChar(int idx) throws TreeDataException {
            if (idx >= buf.length()) {
                int tempIdx = index;
                updateBuffer();
                if (tempIdx != index)
                    idx -= tempIdx;
                if (idx >= buf.length())
                    error();
            }
            ch = buf.charAt(idx);
            return idx;
        }

        private void getInitialDataFromReader() throws TreeDataException {
            buf.setLength(0);
            index = 0;
            index2 = 0;
            updateBuffer();
        }

        private void getIndexOf(String search) throws TreeDataException {
            do {
                index2 = buf.indexOf(search, index);
                if (index2 < 0)
                    updateBuffer();
            } while ((index2 < 0) && (reader != null));
        }

        private void updateBuffer() throws TreeDataException {
            if (reader == null)
                return;
            if (index > 0) {
                buf.delete(0, index);
                index2 -= index;
                index = 0;
            }
            try {
                int chars = reader.read(buffer);
                if (chars == -1) {
                    reader.close();
                    reader = null;
                } else
                    buf.append(buffer, 0, chars);
            } catch (IOException e) {
                try {
                    reader.close();
                } catch (IOException e1) {
                }
                reader = null;
                throw new TreeDataException(e);
            }
        }

        private String getEncodedString(int start, int end) {
            decodeData(start, end);
            return temp.substring(0, temp.length());
        }

        private void decodeData(int start, int end) {
            temp.setLength(0);
            for (int i = start; i < end; i++)
                temp.append(buf.charAt(i));
            Utils.decodeData(temp);
        }

        private void newElementStart(String name) {
            if (level >= 0)
                treeData.children.get(treeData.current.get(level)).add(count);
            treeData.elementIndex.set(count, treeData.data.size());
            treeData.data.add(name);
            treeData.attribs.clear(count);
            treeData.children.clear(count);
        }

        private void newAttribute(String name, String value) {
            treeData.attribs.get(count).add(treeData.data.size());
            treeData.data.add(name);
            treeData.data.add(value);
        }

        private void newElementEnd() {
            level++;
            treeData.current.set(level, count);
            count++;
        }

        private void endElement(String tag) throws TreeDataException {
            if (strbuf.hasData(level)) {
                treeData.dataIndex.set(treeData.current.get(level), treeData.data.size());
                treeData.data.add(strbuf.get(level).substring(0, strbuf.get(level).length()));
                strbuf.get(level).setLength(0);
            } else
                treeData.dataIndex.set(treeData.current.get(level), -1);
            if (!tag
                .equals(treeData.data.get(treeData.elementIndex.get(treeData.current.get(level)))))
                error();
            level--;
        }

        private void characters(StringBuffer temp) {
            if (temp.length() > 0) {
                if ((treeData.getIgnoreWhitespace()) && (isAllWhitespace(temp)))
                    return;
                strbuf.ensureBuffer(level).append(temp);
            }
        }

        private boolean isAllWhitespace(StringBuffer temp) {
            for (int i = 0; i < temp.length(); i++) {
                if (temp.charAt(i) > ' ')
                    return false;
            }
            return true;
        }

        protected void error() throws TreeDataException {
            throw new TreeDataException("Invalid Tree Data Format");
        }
    }
}
