package gov.va.med.hds.cd.mum.boot.utils;

/**
 * Used to create ITreeData instances
 */
public class TreeDataFactory {

	private static Class saxXMLClass = SAXTreeData.class;
	private static Class simpleXMLClass = SimpleXMLTreeData.class;

	/**
	 * Returns an object implementing the ISavableTreeData interface.
	 * @return ISavableTreeData
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public static ISavableTreeData newSimpleXMLTreeData() throws TreeDataException {
		try {
			return (ISavableTreeData) simpleXMLClass.newInstance();
		} catch (InstantiationException e) {
			throw new TreeDataException(e);
		} catch (IllegalAccessException e) {
			throw new TreeDataException(e);
		}
	}

	/**
	 * Returns an object implementing the ITreeData interface, using SAX as the
	 * underlying xml parser.
	 * @return ITreeData
	 */
	public static ILoadableTreeData newSAXTreeData() throws TreeDataException {
		try {
			return (ILoadableTreeData) saxXMLClass.newInstance();
		} catch (InstantiationException e) {
			throw new TreeDataException(e);
		} catch (IllegalAccessException e) {
			throw new TreeDataException(e);
		}
	}

	/**
	 * Allows reassignment of the class that is instantiated and returned by
	 * calls to newSimpleXMLData().
	 * @param xmlClass
	 */
	public static void assignSimpleXMLTreeDataClass(Class xmlClass) {
		simpleXMLClass = xmlClass;
	}

	/**
	 * Allows reassignment of the class that is instantiated and returned by
	 * calls to newSAXXMLData().
	 * @param xmlClass
	 */
	public static void assignSAXTreeDataClass(Class xmlClass) {
		saxXMLClass = xmlClass;
	}

	/**
	 * Returns a singleton instance of the ISavableTreeData interface
	 * @return ISavableTreeData
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public static ISavableTreeData getSimpleXMLTreeDataInstance() throws TreeDataException {
		if (singleton == null)
			singleton = newSimpleXMLTreeData();
		else
			singleton.clear();
		return singleton;
	}

	/**
	 * Releases the singleton instance, making it available for garbage
	 * collection.
	 */
	public static void releaseSingleton() {
		singleton = null;
	}

	private static ISavableTreeData singleton;
}
