package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataException;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;

import java.io.InputStream;

/**
 * Base class implementing the code needed to read and parse configuration files
 * for the update process (not as extensive as that needed by the MUM.Config).
 */
public abstract class ConfigParser implements IUpdateConstants {

    /**
     * Constructor for ConfigParser 
     */
    public ConfigParser() {
        super();
    }

    /**
     * Common method that throws InvalidConfigurationException.
     * @param cause
     * @throws InvalidConfigurationException
     */
    protected void badConfig(Throwable cause) throws InvalidConfigurationException {
        throw new InvalidConfigurationException(cause);
    }

    /**
     * Common method that throws InvalidConfigurationException.
     * @param message
     * @param cause
     * @throws InvalidConfigurationException
     */
    protected void badConfig(String message, Throwable cause)
        throws InvalidConfigurationException {
        throw new InvalidConfigurationException(message, cause);
    }

    /**
     * Common method that throws InvalidConfigurationException.
     * @param message
     * @throws InvalidConfigurationException
     */
    protected void badConfig(String message) throws InvalidConfigurationException {
        throw new InvalidConfigurationException(message);
    }

    /**
     * Returns the XML Document of the configuration file.
     * @param input - the input stream of the manifest file.
     * @return Document
     * @throws InvalidConfigurationException
     */
    protected ITreeData getDocument(InputStream input) throws InvalidConfigurationException {
        ILoadableTreeData xml = null;
        try {
            xml = TreeDataFactory.getSimpleXMLTreeDataInstance();
            xml.load(input);
        } catch (TreeDataException e) {
            badConfig(e);
        }
        return xml;
    }

    /**
     * Returns the XML Element of the settings section of the configuration
     * file.
     * @param root - the root element of the configuration file.
     * @return Element
     * @throws InvalidConfigurationException
     */
    protected void validateSettings(ITreeData xml) throws InvalidConfigurationException {
        int settings = xml.findChildID(xml.getRootID(), IBootConstants.SETTINGS_ELEMENT);
        if (settings < 0)
            return;
        if (xml.findChildID(xml.getRootID(), IBootConstants.SETTINGS_ELEMENT, settings) >= 0)
            badConfig(MULTIPLE_SETTINGS);
    }
}
