package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.boot.utils.Settings;

import java.net.URL;

/**
 * Interface represeting the object that reads the configuration files.
 */
public interface IConfigReader {
	/**
	 * Reads the configuration file, and populates the plugins list with
	 * IPluginSpec instances, one for each plugin in the configuration file.
	 * @param configFile - the URL to the configuration file.
	 * @param plugins - the list to populate with IPluginSpec objects.
	 * @throws InvalidConfigurationException
	 */
	void readConfiguration(URL configFile, ListMap plugins)
		throws InvalidConfigurationException;

	/**
	 * Returns the CRC of the configuration file.
	 * @return long
	 */
	long getCRC();
	
	/**
	 * Returns the plugin spec implementing the IStartup interface
     */
    IPluginSpec getStartupPlugin();
	
	/**
	 * Returns the Settings object associated with the configuration.
     */
    Settings getSettings();
}
