package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.IDependentPlugin;
import gov.va.med.hds.cd.mum.InvalidPluginException;
import gov.va.med.hds.cd.mum.boot.loader.IDesktopPlugin;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;

import java.io.File;
import java.net.URL;

/**
 * Represents the plugin specification as detailed in the configuration file.
 */
public interface IPluginSpec extends IDependentPlugin, IDesktopPlugin {
	
    /**
     * Returns a SpecFileEntry for the specified file.
     * @param index
     * @return SpecFileEntry
     */
    SpecFileEntry getFile(int index);

    /**
     * Indicates if this plugin matches an plugin already installed on the
     * workstation.
     * @param plugin
     * @return boolean
     */
    boolean samePlugin(IExistingPlugin plugin);
    
	/**
     * Sets the update flag in the SpecFileEntries on those files that need to be 
     * updated to the workstation 
     */
    void assignNeededUpdates(IExistingPlugin plugin);
    
    /**
     * Returns true if any of the SpecFileEntries have the update flag set
     */
    boolean hasUpdates();

    /**
     * Returns the file instances for the plugin, calculated from the
     * specifications in the configuration, relative to the passed in root
     * directory.
     * @param rootDirectory
     * @return File[]
     */
    File[] getFiles(File rootDirectory);

    /**
     * Returns the URL instances for the plugin, calculated from the
     * specifications in the configuration, relative to the passed in root
     * directory.
     * @param rootDirectory
     * @return URL[]
     */
    URL[] getURLs(File rootDirectory);

    /**
     * Returns the URL to the plugin's manifest.xml file.  This is only valid if
     * the getFiles or getURLs has been called, since it must be relative to a
     * given directory.
     * @return URL
     */
    URL getManifestURL();

    /**
     * Returns the class loader used to load classes in this plugin.
     * @return ClassLoader
     */
    ClassLoader getClassLoader();

    /**
     * Sets the plugin class loader.
     * @param classLoader
     */
    void setClassLoader(ClassLoader classLoader);

    /**
     * Assigns plugin dependencies based on the actual plugin manifest.  This
     * method only works after the plugin has actually been installed on the
     * workstation, representing an existing manifest.
     * @param manifest
     * @throws InvalidPluginException
     */
    void loadManifest(URL manifest) throws InvalidPluginException;

    /**
     * Returns the XML of the manifest file 
     * @return Element
     */
    ITreeData getManifestXML();
}
