package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.UpdateAuthorizationFailureException;

import java.io.File;
import java.net.URL;

/**
 * Used to update plugins from the server to the workstation.
 */
public interface IPluginUpdater {

	/**
	 * Updates a plugin from the server to the workstation, returning a list of files installed for 
	 * the plugin.
     * @param fromPluginDir
     * @param toPluginDir
     * @param plugin
     * @throws UpdateException
     * @throws UpdateAuthorizationFailureException
     */
    File[] update(URL fromPluginDir, File toPluginDir, IPluginSpec plugin)
		throws UpdateException, UpdateAuthorizationFailureException;

	/**
	 * Informs the plugin updater that one or more updates are about to be
	 * performed.  This allows the updater to inform the status notifer of
	 * progress made during plugin updates, and to lock updates on the
	 * workstation to avoid duplicate update collisions.  A call to beginUpdate
	 * is required before updates can begin, and a call to endUpdate is required
	 * following all calls to beginUpdate.
	 * @param rootDirectory - this required parameter must contain the directory
	 * on the destination workstation that contains the plugins directory
	 * @param message - a message to display in the status notifier.
	 * @param notifier - the status notifier to inform about updates
	 * @param size - the maximum size of all updates about to be performed
	 * @throws UpdateException
	 */
	void beginUpdate(
		File rootDirectory,
		String message,
		IStatusNotifier notifier,
		long size)
		throws UpdateException;

	/**
	 * Informs the plugin updater that updates have completed, and allows the
	 * updater to unlock updates on the workstaiton.  The status notifier is
	 * also told to hide the progress bar and to clear out the displayed
	 * message
	 * @param notifier
	 */
	void endUpdate(IStatusNotifier notifier);

	/**
	 * Assigns a listener to be notified when plugin files have been copied to
	 * the temporary updates directory. 
	 * @param dummyUpdateTests
	 */
	void assignUpdateListener(IUpdateListener listener);
}
