package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.UpdateAuthorizationFailureException;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;

import java.io.FileNotFoundException;
import java.net.URL;

/**
 * Manages the update of plugins to the workstation.
 */
public interface IUpdateManager {

	/**
	 * Updates plugins on the workstation as needed.
     * @param info
     * @throws InvalidConfigurationException
     * @throws FileNotFoundException
     * @throws InvalidPluginDirectoryException
     * @throws UpdateException
     * @throws UpdateAuthorizationFailureException
     */
	void update(IStartupInfo info)
		throws
			InvalidConfigurationException,
			FileNotFoundException,
			InvalidPluginDirectoryException,
			UpdateException,
			UpdateAuthorizationFailureException;

	/**
	 * Returns the URL of the configuration file.
	 * @param info
	 * @return URL
	 * @throws InvalidConfigurationException
	 */
	URL getConfigFileURL(IStartupInfo info)
		throws InvalidConfigurationException;

	/**
	 * Returns the list of IPluginSpec objects, one for each plugin.
	 */
	ListMap getSpecs();

	/**
	 * Returns a list of IExitingPlugin objects representing the plugins
	 * installed  on the workstation prior to the plugin update.
	 */
	ListMap getCurrentPlugins();

	/**
	 * Returns the startup IPluginSpec object.
	 * @return IPluginSpec
	 */
	IPluginSpec getStartupSpec();
}
