package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.IPluginSource;
import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.utils.ITreeData;

import java.io.File;
import java.io.FileNotFoundException;

/**
 * Creates instances of various Plugin Interface Classes
 */
public class PluginClassFactory {

	/**
	 * Returns an IExistingPlugin instance for the specified plugin directory
     * @param pluginDir
     * @throws FileNotFoundException
     * @throws InvalidPluginDirectoryException
     */
    public static IExistingPlugin newExistingPlugin(File pluginDir)
		throws FileNotFoundException, InvalidPluginDirectoryException {
		return new ExistingPlugin(pluginDir);
	}

	/**
	 * Returns a new instance of the IPluginLocator interface
	 * @return IPluginLocator
	 */
	public static IPluginLocator newPluginLocator(boolean useCache) {
		if (locator == null)
			return new PluginLocator(useCache);
		else
			return locator;
	}

	/**
	 * Allows subsequent calls to newPluginLocator to return a predetermined
	 * instance of the IPluginLocator interface.  This is used primarily in unit
	 * testing.
	 * @param pluginLocator
	 */
	public static void assignPluginLocator(IPluginLocator pluginLocator) {
		locator = pluginLocator;
	}

	/**
	 * Returns an instance of the IPluginSource interface.
	 * @param spec
	 * @return IPluginSource
	 */
	public static IPluginSource newPluginSource(IPluginSpec spec) {
		return new PluginSource(spec);
	}

	/**
	 * Returns an instance of the IPluginSpec interface.
     * @param xml - the config file xml
     * @param plugin - the node in the config file representing the plugin
     * @throws InvalidConfigurationException
     */
	public static IPluginSpec newPluginSpec(ITreeData xml, int plugin)
		throws InvalidConfigurationException {
		return new PluginSpec(xml, plugin);
	}

	/**
	 * Returns an instance of the IPluginUpdater interface.
	 * @return IPluginUpdater
	 */
	public static IPluginUpdater newPluginUpdater() {
		if (updater == null)
			return new PluginUpdater();
		else
			return updater;
	}

	/**
	 * Allows subsequent calls to newPluginUpdater to return a predetermined
	 * instance of the IPluginUpdater interface.  This is used primarily in unit
	 * testing.
	 * @param pluginUpdater
	 */
	public static void assignPluginUpdater(IPluginUpdater pluginUpdater) {
		updater = pluginUpdater;
	}

    /**
	 * Returns an instance of the IPluginUsageTracker interface.
	 * @param root
	 */
	public static IPluginUsageTracker newPluginUsageTracker(File root) {
		if (tracker == null)
			return new PluginUsageTracker(root);
		else
			return tracker;
	}

	/**
	 * Allows subsequent calls to newPluginUsageTracker to return a
	 * predetermined instance of the IPluginUsageTracker interface.  This is
	 * used primarily in unit testing.
	 * @param pluginUpdater
	 */
	public static void assignPluginUsageTracker(IPluginUsageTracker pluginTracker) {
		tracker = pluginTracker;
	}

	private static IPluginLocator locator;
	private static IPluginUpdater updater;
	private static IPluginUsageTracker tracker;

}
