package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.IMUMConstants;
import gov.va.med.hds.cd.mum.boot.utils.ListMap;
import gov.va.med.hds.cd.mum.utils.ChecksumCache;

import java.io.File;
import java.io.FileNotFoundException;

/**
 * Locates plugins currently installed on the workstation.
 */
public class PluginLocator implements IPluginLocator, IUpdateConstants {

	private boolean useCache;
	protected PluginLocator(boolean useCache) {
		super();
		this.useCache = useCache;
	}

    /**
     * @see gov.va.med.hds.cd.mum.update.IPluginLocator#locatePlugins(java.io.File, gov.va.med.hds.cd.mum.boot.utils.ListMap)
     */
    public void locatePlugins(File root, ListMap list)
		throws FileNotFoundException, InvalidPluginDirectoryException {
		if ((root == null) || (list == null))
			throw new NullPointerException();
		if (!root.exists())
			throw new FileNotFoundException();
		if (!root.getName().equals(IMUMConstants.PLUGIN_DIRECTORY_NAME))
			throw new InvalidPluginDirectoryException(NO_PLUGINS_DIRECTORY_FOUND);
		if (useCache)
			ChecksumCache.enable(root.getParentFile());
		else
			ChecksumCache.disable();
		try {
			processPlugins(root, list);
		} finally {
			if (useCache) {
				ChecksumCache.saveCache();
				ChecksumCache.disable();
			}
		}
	}

	private void processPlugins(File root, ListMap list) {
		File[] files = root.listFiles();
		for (int i = 0; i < files.length; i++) {
			try {
				IExistingPlugin plugin =
					PluginClassFactory.newExistingPlugin(files[i]);
				list.add(plugin.getID(), plugin);
			} catch (FileNotFoundException e) {
			} catch (InvalidPluginDirectoryException e) {
			}
		}
	}

}
