package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.boot.utils.SimpleList;

/**
 * Simple data class representing a plugin file, as specified in the
 * configuration file.  Contains both the string name of the file, it's size, and the CRC
 * value of the file as specified in the configuration file.
 */
public class SpecFileEntry extends FileData {

    /**
     * Constructor for SpecFileEntry 
     * @param fileName
     * @param crc
     */
    public SpecFileEntry(String fileName, long crc, long size) {
        super(crc, size);
        this.file = fileName;
    }

    /**
     * Returns the file name
     * @return String
     */
    public String getFileName() {
        return file;
    }

    /**
     * Returns a list of packages added to the addPackage method
     * @return List
     */
    public SimpleList getPackages() {
        return packages;
    }

    /**
     * Adds a package to the package list
     * @param pkg
     */
    public void addPackage(String pkg) {
        if (pkg == null)
            return;
        if (packages == null)
            packages = new SimpleList();
        packages.add(pkg);
    }

    private String file;
    private SimpleList packages;
    private boolean update;
    
    /**
     * Sets a flag indicating that the file needs to be updated on the workstation
     * @param update
     */
    public void setUpdate(boolean update) {
    	this.update = update;
    }
    
    /**
     * Returns the value of the update flag
     */
    public boolean getUpdate() {
    	return update;
    }
}

