package gov.va.med.hds.cd.mum.update;

import gov.va.med.hds.cd.mum.InvalidConfigurationException;
import gov.va.med.hds.cd.mum.boot.IBootConstants;
import gov.va.med.hds.cd.mum.boot.IStartup;
import gov.va.med.hds.cd.mum.boot.IStartupInfo;
import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.SilentAbortException;
import gov.va.med.hds.cd.mum.boot.utils.Utils;

import java.io.File;

/**
 * The initial class executed from the Boot.  This class is responsible for
 * executing the MUM update process, and launching the application plugin.
 */
public class Startup implements IStartup, IUpdateConstants {

    public Startup() {
        super();
    }

    private IUpdateManager manager;
    private IPluginUsageTracker tracker;

    /**
     * @see gov.va.med.hds.cd.mum.boot.IStartup#run(gov.va.med.hds.cd.mum.boot.IStartupInfo)
     */
    public void run(IStartupInfo info) throws Throwable {
        init(info);
        manager.update(info);
        finish(info);
    }

    private void finish(IStartupInfo info) throws Throwable {
        tracker.trackPluginUsage(manager.getSpecs());
        tracker.deleteUnusedPlugins(manager.getCurrentPlugins());
        notify(info, PLUGINS_UPDATED_MESSAGE);
        if (Utils.hasArgument(info.getArgs(), IBootConstants.SILENT_UPDATE_TAG))
            throw new SilentAbortException();
        launchApplication(info);
    }

    private void launchApplication(IStartupInfo info)
        throws InvalidConfigurationException, Throwable {
        ApplicationLauncherFactory.newApplicationLauncher().launchApplication(
            manager.getConfigFileURL(info),
            manager.getSpecs(),
            info,
            manager.getStartupSpec());
    }

    /**
     * Sends a message to the splash screen.
     * @param info
     * @param message
     */
    public static void notify(IStartupInfo info, String message) {
        if ((info.getSplashScreen() != null)
            && (info.getSplashScreen() instanceof IStatusNotifier)) {
            ((IStatusNotifier) info.getSplashScreen()).displayMessage(message);
        }
    }

    private void init(IStartupInfo info) {
        File root;
        if (info != null) {
            root = info.getRootDirectory();
        } else
            root = null;
        tracker = PluginClassFactory.newPluginUsageTracker(root);
        manager = UpdateManagerFactory.newUpdateManager();
    }
}
