package gov.va.med.hds.cd.mum.update;

/**
 * Creates an instance of the IUpdateManager and IUpdateNotifier interfaces.
 */
public class UpdateManagerFactory {

	private static IUpdateManager manager;

	/**
	 * Returns a new instance of the IUpdateManager
	 * @return IUpdateManager
	 */
	public static IUpdateManager newUpdateManager() {
		if (manager == null)
			return new UpdateManager();
		else
			return manager;
	}

	/**
	 * Allows subsequent calls to newUpdateManager to return a predetermined
	 * instance of the IUpdateManager interface.  This is used primarily
	 * in unit testing.
	 * @param updateManager
	 */
	public static void assignUpdateManager(IUpdateManager updateManager) {
		manager = updateManager;
	}

	private static IUpdateNotifier notifier;

	/**
	 * Returns a new instance of the IUpdateNotifier
	 * @return IUpdateNotifier
	 */
	public static IUpdateNotifier newUpdateNotifier() {
		if (notifier == null)
			return new UpdateNotifier();
		else
			return notifier;
	}

	/**
	 * Allows subsequent calls to newUpdateNotifier to return a predetermined
	 * instance of the IUpdateNotifier interface.  This is used primarily
	 * in unit testing.
	 * @param updateManager
	 */
	public static void assignUpdateNotifier(IUpdateNotifier updateNotifier) {
		notifier = updateNotifier;
	}
}