package gov.va.med.hds.cd.mum.utils;

import gov.va.med.hds.cd.mum.boot.IBootConstants;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Contains Miscelaneous utilities for the Module Update Manager
 */
public class MUMUtils {

    /**
     * Returns the number of days since 1970, using the current date. 
     * @return int
     */
    public static int getDaysSince1970() {
        ensureToday();
        return today;

    }

    /**
     * Deletes the passed in file, and, if the file is a directory, all files
     * and subdirectories in that directory.
     * @param file
     */
    public static void deleteTree(File file) {
        deleteTree(file, null);
    }

    /**
     * Deletes the passed in file, and, if the file is a directory, all files
     * and subdirectories in that directory.  Files in the exclude list are not deleted.
     * @param file
     * @param exclude
     */
    public static void deleteTree(File file, File[] exclude) {
        if (file.exists()) {
            boolean process = (exclude == null);
            if (!process) {
                process = true;
                for (int i = 0; i < exclude.length; i++) {
                    if (exclude[i] != null) {
                        if (file.compareTo(exclude[i]) == 0) {
                            process = false;
                            break;
                        }
                    }
                }
            }
            if (process) {
                if (file.isDirectory()) {
                    File[] files = file.listFiles();
                    for (int i = 0; i < files.length; i++)
                        deleteTree(files[i], exclude);
                }
                file.delete();
            }
        }
    }

    /**
     * Creates a new URL based on a path and the parent URL
     * @param parent
     * @param path
     */
    public static URL newURL(String path) throws MalformedURLException {
        try {
            if (path.startsWith(IBootConstants.FILE_PROTOCOL))
                return new File(path.substring(IBootConstants.FILE_PROTOCOL.length())).toURL();
            return new URL(path);
        } catch (MalformedURLException e) {
            return new File(path).toURL();
        }
    }

    private static final int firstYear = 1970;
    private static int today = 0;

    private static void ensureToday() {
        if (today == 0) {
            GregorianCalendar cal = new GregorianCalendar();
            today = cal.get(Calendar.DAY_OF_YEAR);
            int thisYear = cal.get(Calendar.YEAR);
            cal.set(Calendar.DAY_OF_MONTH, 1);
            for (int year = firstYear; year < thisYear; year++) {
                cal.set(Calendar.YEAR, year);
                today += cal.getActualMaximum(Calendar.DAY_OF_YEAR);
            }
        }
    }

    /**
     * Returns the class name of the passed in string, excluding a .class or the jar subpath
     * @param name
     */
    public static String getBaseClassName(String name) {
        int idx = name.lastIndexOf('$');
        if (idx < 0)
            idx = name.lastIndexOf('.');
        if (idx < 0)
            return name;
        return name.substring(idx + 1);
    }
}
