package gov.va.med.hds.cd.prism.action.internal;

import gov.va.med.hds.cd.print.IPrint;
import gov.va.med.hds.cd.print.IPrintActionEnablingListener;
import gov.va.med.hds.cd.print.IPrintActionHandler;
import gov.va.med.hds.cd.print.IPrintActionHandlerProvider;
import gov.va.med.hds.cd.print.PrintActionEnablingEvent;
import gov.va.med.hds.cd.print.PrintHandlerException;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.action.AbstractAction;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JComponent;
abstract public class AbstractPrintAction extends AbstractAction implements IPrintActionEnablingListener,IPerspectiveListener {

	public AbstractPrintAction() {
		super();
		setEnabled(false);
	}

	/**
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveActivated(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveActivated(PerspectiveEvent e){
		perspectiveActivatedDetected(e);
	}
	public void perspectiveClosing(PerspectiveEvent e){}
	/**
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveClosed(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveClosed(PerspectiveEvent e){}
	public void perspectiveOpening(PerspectiveEvent e) {}
	/**
	 * @see gov.va.med.hds.cd.prism.perspective.IPerspectiveListener#perspectiveOpened(gov.va.med.hds.cd.prism.perspective.PerspectiveEvent)
	 */
	public void perspectiveOpened(PerspectiveEvent e){}
	/**
	 *  @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object) 
	 **/
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception {
		this.runtime = runtime;
		super.setInitializationData(runtime, config, propertyName, data);
	}

	/**
	 * @see gov.va.med.hds.cd.print.IPrintActionEnablingListener#printEnabled(gov.va.med.hds.cd.print.PrintActionEnablingEvent)
	 */
	public void enable(PrintActionEnablingEvent e) {
		setEnabled(e.isEnabled());
		if(e.isEnabled())
			this.addPrintHandler(e.getHandler());
		else
			this.removePrintHandler(e.getHandler());
	}

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if(e instanceof InitPrintActionEvent){
			IDesktopWindow window = ((InitPrintActionEvent)e).getWindow();
			if(e.getID() == InitPrintActionEvent.ADD){
				IPerspectiveReference activePerspective = window.getActivePerspective();
				if (activePerspective == null) return;
				unhookPreviousPrintHandlerProviders(activePerspective);
				hookupPrintHandlerProviders(activePerspective);
				window.addPerspectiveListener(this);
			} else if(e.getID() == InitPrintActionEvent.REMOVE)
				window.removePerspectiveListener(this);
			else
				runtime.getLogger(getClass().getName()).error("Invalid "+InitPrintActionEvent.class.getName()+" ID of "+e.getID());
		}else{
			Iterator it = handlers.iterator();
			while (it.hasNext()) {
				IPrintActionHandler handler = (IPrintActionHandler) it.next();
				try {
					handler.doAction();
				} catch (PrintHandlerException e1) {
					runtime.getLogger(getClass().getName()).error("Error running print action for action: "+getClass().getName(), e1);
				}
			}
		}
	}

	protected void addPrintHandler(IPrintActionHandler handler) {
		if (handler != null)
			if (!handlers.contains(handler))
				handlers.add(handler);
	}
	protected void removePrintHandler(IPrintActionHandler handler) {
		if (handler != null)
			handlers.remove(handler);
	}
	abstract protected void removeActionEnablingListener(IPrintActionHandlerProvider php);
	abstract protected void addActionEnablingListener(IPrintActionHandlerProvider php);
	
	protected void perspectiveActivatedDetected(PerspectiveEvent e){
		this.enable(new PrintActionEnablingEvent(this, false, null));
		hookupPrintHandlerProviders(e.getPerspective());
	}
	private void unhookPreviousPrintHandlerProviders(IPerspectiveReference perspective) {
		IVisualComponentReference[] comps = perspective.getVisualComponents();
		for (int i = 0; i < comps.length; i++) {
			JComponent comp = comps[i].getVisualComponent().getComponent();
			unhookPreviousPrintHandlerProviders(comp);
			if (comp instanceof IPrintActionHandlerProvider) {
				IPrintActionHandlerProvider php = (IPrintActionHandlerProvider) comp;
				this.removeActionEnablingListener(php);
			}
		}
	}
	private void unhookPreviousPrintHandlerProviders(Component comp) {
		if (comp instanceof IPrintActionHandlerProvider) {
			IPrintActionHandlerProvider php = (IPrintActionHandlerProvider) comp;
			this.removeActionEnablingListener(php);
		}
		if(comp instanceof Container){
			Component[] comps = ((Container)comp).getComponents();
			for(int i=0;i<comps.length;i++)
			unhookPreviousPrintHandlerProviders(comps[i]);
		}
	}

	private void hookupPrintHandlerProviders(IPerspectiveReference perspective) {
		if (prevPerspective != null) {
			unhookPreviousPrintHandlerProviders(prevPerspective);
		}

		IVisualComponentReference[] comps = perspective.getVisualComponents();
		for (int i = 0; i < comps.length; i++) {
			JComponent comp = comps[i].getVisualComponent().getComponent();
			hookupPrintHandlerProviders(comp);
		}
		prevPerspective = perspective;
	}
	private void hookupPrintHandlerProviders(Component comp) {
		if (comp instanceof IPrintActionHandlerProvider) {
			IPrintActionHandlerProvider php = (IPrintActionHandlerProvider) comp;
			this.addActionEnablingListener(php);
		}
		if(comp instanceof Container){
			Component[] comps = ((Container)comp).getComponents();
			for(int i=0;i<comps.length;i++)
				hookupPrintHandlerProviders(comps[i]);
		}
	}

	private IPerspectiveReference prevPerspective = null;

	protected IPrint printPlugin;
	protected IRuntimeContext runtime;
	protected ArrayList handlers = new ArrayList();
}
