package gov.va.med.hds.cd.prism.action.internal;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.ActionAvailableEvent;
import gov.va.med.hds.cd.prism.action.IActionAvailableListener;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IPluginActivationListener;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginEvent;
import gov.va.med.hds.cd.workspace.ListenerOfEventDescriptor;
import gov.va.med.hds.cd.workspace.SourceOfEventDescriptor;

public class ActionDescriptor implements IActionDescriptor, IPrismConstants {
	
	private EventListenerList listenerList = new EventListenerList();
	IPluginDescriptor plugin;
	IRuntimeContext runtime;
	private IConfigurationElement actionElement;
	private Map properties = new HashMap();
	private Map sourceOfEvents = new HashMap();
	private Map listenerOfEvents = new HashMap();

	public ActionDescriptor(IConfigurationElement actionElement) {
		this.actionElement = actionElement;
		IExtension ext = actionElement.getDeclaringExtension();
		if (ext != null) {this.plugin = ext.getDeclaringPluginDescriptor();
			this.runtime = plugin.getRuntimeContext();
		}
		putProperty(Action.NAME, actionElement.getAttribute(NAME_ATTRIBUTE));
		putProperty(IActionConstants.ENABLED, Boolean.TRUE);
		if (!isActionAvailable() && runtime != null) {
			this.runtime.addPluginActivationListener(new IPluginActivationListener() {
				public void pluginActivated(PluginEvent e) {
					if (!e.getPluginDescriptor().equals(plugin)) return;
					fireActionAvailable();
					runtime.removePluginActivationListener(this);
				}
				public void pluginDeactivated(PluginEvent e) {
				}
			});
		}
	}

	public String getId() {
		return actionElement.getAttribute(ID_ATTRIBUTE);
	}

	public String getActionClassName() {
		return actionElement.getAttribute(CLASS_ATTRIBUTE);
	}

	public boolean isActionAvailable() {
		if (runtime == null) return false;
		return runtime.isPluginActive(plugin.getUniqueIdentifier(), plugin.getVersion());
	}

	public Action createAction() throws Exception {
		return (Action) actionElement.createExecutableExtension(CLASS_ATTRIBUTE, this);
	}

	public Map getProperties() {
		return Collections.unmodifiableMap(properties);
	}
	
	public void putProperty(String key, Object value) {
		properties.put(key, value);
	}
	
	public String toString() {
		return getId() + " (" + getActionClassName() + ")";
	}

	public void addActionAvailableListener(IActionAvailableListener l) {
		listenerList.add(IActionAvailableListener.class, l);
	}
	
	public void removeActionAvailableListener(IActionAvailableListener l) {
		listenerList.remove(IActionAvailableListener.class, l);
	}
	
	protected void fireActionAvailable() {
		final ActionAvailableEvent e = new ActionAvailableEvent(this);
		IActionAvailableListener[] listeners = (IActionAvailableListener[]) listenerList.getListeners(IActionAvailableListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].actionAvailable(e);
		}
	}
	
	public void addEventSourceDescriptor(String eventClassName) {
		sourceOfEvents.put(eventClassName, new SourceOfEventDescriptor(eventClassName));
	}

	public void removeEventSourceDescriptor(String eventClassName) {
		sourceOfEvents.remove(eventClassName);
	}

	public void addEventListenerDescriptor(String eventClassName) {
		listenerOfEvents.put(eventClassName, new ListenerOfEventDescriptor(eventClassName));
	}

	public void removeEventListenerDescriptor(String eventClassName) {
		listenerOfEvents.remove(eventClassName);
	}

	public SourceOfEventDescriptor[] getSourceOfEventDescriptors() {
		return (SourceOfEventDescriptor[]) sourceOfEvents.values().toArray(new SourceOfEventDescriptor[sourceOfEvents.size()]);
	}

	public ListenerOfEventDescriptor[] getListenerOfEventDescriptors() {
		return (ListenerOfEventDescriptor[]) listenerOfEvents.values().toArray(new ListenerOfEventDescriptor[listenerOfEvents.size()]);
	}
}
