package gov.va.med.hds.cd.prism.action.internal;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.prism.action.ActionAvailableEvent;
import gov.va.med.hds.cd.prism.action.IActionAvailableListener;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;

public class ActionProxy extends AbstractAction {

	private static final String ERROR_CREATING_ACTION = "Error creating action delegate";

	private EventListenerList listenerList = new EventListenerList();
	private ILogger log;
	private IActionDescriptor descriptor;
	private Action delegate;

	public ActionProxy(
		ILoggingCapable logging,
		IActionDescriptor actionDescriptor) {
		this.log = logging.getLogger(getClass());

		this.descriptor = actionDescriptor;
		putValues();
		if (descriptor.isActionAvailable()) {
			try {
				createDelegate();
			} catch (Exception e) {
				log.error(ERROR_CREATING_ACTION, e);
			}
		} else {
			actionDescriptor.addActionAvailableListener(
				new ActionAvailabilityListener());
		}
	}

	public void addActionProxyDelegateListener(IActionProxyDelegateListener l) {
		listenerList.add(IActionProxyDelegateListener.class, l);
	}

	public void removeActionProxyDelegateListener(IActionProxyDelegateListener l) {
		listenerList.remove(IActionProxyDelegateListener.class, l);
	}

	protected void fireDelegateCreated(Action delegate) {
		final ActionProxyDelegateEvent e = new ActionProxyDelegateEvent(this, delegate);
		IActionProxyDelegateListener[] listeners = (IActionProxyDelegateListener[]) listenerList.getListeners(IActionProxyDelegateListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].delegateCreated(e);
		}
	}

	public IActionDescriptor getDescriptor() {
		return descriptor;
	}

	private void putValues() {
		Map m = descriptor.getProperties();
		for (Iterator i = m.keySet().iterator(); i.hasNext();) {
			String key = (String) i.next();
			if (IActionConstants.ENABLED.equals(key)) {
				setEnabled(((Boolean) m.get(key)).booleanValue());
			} else {
				putValue(key, m.get(key));
			}
		}
	}

	private void syncPropertyValues(Action a) {
		Object[] keys = this.getKeys();
		if (keys == null)
			return;
		for (int i = 0; i < keys.length; i++) {
			String key = (String) keys[i];
			a.putValue(key, getValue(key));
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (isDelegating()) {
			delegate.actionPerformed(e);
		} else {
			try {
				createDelegate();
				delegate.actionPerformed(e);
			} catch (Exception ex) {
				log.error(ERROR_CREATING_ACTION, ex);
			}
		}
	}

	private void createDelegate() throws Exception {
		if (isDelegating())
			return;

		Action a = descriptor.createAction();
		syncPropertyValues(a);
		delegate = a;
		firePropertyChange(
			"enabled",
			new Boolean(super.isEnabled()),
			new Boolean(isEnabled()));
		delegate.addPropertyChangeListener(
			new DelegatePropertyChangeListener());
		fireDelegateCreated(a);
	}

	public boolean isEnabled() {
		if (isDelegating())
			return delegate.isEnabled();
		return super.isEnabled();
	}

	public Object getValue(String key) {
		if (isDelegating())
			return delegate.getValue(key);
		return super.getValue(key);
	}

	public void putValue(String key, Object newValue) {
		if (isDelegating())
			delegate.putValue(key, newValue);
		else
			super.putValue(key, newValue);
	}

	public void setEnabled(boolean newValue) {
		if (isDelegating())
			delegate.setEnabled(newValue);
		else
			super.setEnabled(newValue);
	}

	private boolean isDelegating() {
		return delegate != null;
	}

	private class DelegatePropertyChangeListener
		implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			firePropertyChange(
				evt.getPropertyName(),
				evt.getOldValue(),
				evt.getNewValue());
		}
	}

	private class ActionAvailabilityListener
		implements IActionAvailableListener {
		public void actionAvailable(ActionAvailableEvent e) {
			try {
				createDelegate();
				descriptor.removeActionAvailableListener(this);
			} catch (Exception ex) {
				log.error(ERROR_CREATING_ACTION, ex);
			}
		}
	}
}
