package gov.va.med.hds.cd.prism.action.internal;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.InvalidConfigurationElementException;

public class ActionRegistryReader
	implements IPrismConstants, IActionConstants {

	private static final String ACTION_CREATION_WARNING =
		"unable to create action from configuration element";

	public void addRegisteredActions(
		ActionRegistry actionRegistry,
		IRuntimeContext runtime) {
		readRegisteredActions(runtime, actionRegistry);
	}

	public ActionRegistry readRegisteredActionRegistry(IRuntimeContext runtime) {
		ActionRegistry result = new ActionRegistry();
		readRegisteredActions(runtime, result);
		return result;
	}

	private void readRegisteredActions(
		IRuntimeContext runtime,
		ActionRegistry actionRegistry) {
		IConfigurationElement[] elements =
			runtime.getPluginRegistry().getConfigurationElementsFor(
				UNIQUE_ACTION_EXTENSION_POINT_ID);
		for (int i = 0; i < elements.length; i++) {
			try {
				IActionDescriptor action = readAction(elements[i]);
				actionRegistry.put(action);
			} catch (Exception e) {
				runtime.getLogger(getClass()).error(ACTION_CREATION_WARNING, e);
				continue;
			}
		}
	}

	private IActionDescriptor readAction(IConfigurationElement element)
		throws
			InvalidConfigurationElementException,
			ClassNotFoundException,
			InstantiationException,
			IllegalAccessException {
		validateConfigurationElement(element);
		ActionDescriptor action = new ActionDescriptor(element);
		IConfigurationElement[] children = element.getChildren();
		for (int i = 0; i < children.length; ++i) {
			if (PROPERTY_CONFIGURATION_ELEMENT.equals(children[i].getName())) {
				setActionProperty(children[i], action);
			} else if (SOURCE_CONFIGURATION_ELEMENT.equals(children[i].getName())) {
				addSourceOfDescriptor(children[i], action);
			} else if (LISTENER_CONFIGURATION_ELEMENT.equals(children[i].getName())) {
				addListenerOfDescriptor(children[i], action);
			}
		}
		return action;
	}

	private void validateConfigurationElement(IConfigurationElement element)
		throws InvalidConfigurationElementException {
		if (!element.getName().equals(ACTION_CONFIGURATION_ELEMENT)) {
			throw new InvalidConfigurationElementException(
				"\""
					+ element.getName()
					+ "\" is an unrecognized configuration element name for extension point \""
					+ UNIQUE_ACTION_EXTENSION_POINT_ID
					+ "\"");
		}
		String id = element.getAttribute(ID_ATTRIBUTE);
		if (id == null) {
			throw new InvalidConfigurationElementException(
				"expected \""
					+ ID_ATTRIBUTE
					+ "\" attribute in \""
					+ ACTION_CONFIGURATION_ELEMENT
					+ "\" configuration element contributed by \""
					+ element
						.getDeclaringExtension()
						.getDeclaringPluginDescriptor()
					+ "\" for extension point \""
					+ UNIQUE_ACTION_EXTENSION_POINT_ID
					+ "\"");
		}
		String name = element.getAttribute(NAME_ATTRIBUTE);
		if (name == null) {
			throw new InvalidConfigurationElementException(
				"expected \""
					+ NAME_ATTRIBUTE
					+ "\" attribute in \""
					+ ACTION_CONFIGURATION_ELEMENT
					+ "\" configuration element contributed by \""
					+ element
						.getDeclaringExtension()
						.getDeclaringPluginDescriptor()
					+ "\" for extension point \""
					+ UNIQUE_ACTION_EXTENSION_POINT_ID
					+ "\"");
		}
		String className = element.getAttribute(CLASS_ATTRIBUTE);
		if (className == null) {
			throw new InvalidConfigurationElementException(
				"expected \""
					+ CLASS_ATTRIBUTE
					+ "\" attribute in \""
					+ ACTION_CONFIGURATION_ELEMENT
					+ "\" configuration element contributed by \""
					+ element
						.getDeclaringExtension()
						.getDeclaringPluginDescriptor()
					+ "\" for extension point \""
					+ UNIQUE_ACTION_EXTENSION_POINT_ID
					+ "\"");
		}
	}

	private void setActionProperty(
		IConfigurationElement element,
		ActionDescriptor action) {
		String propertyName = element.getAttribute(NAME_ATTRIBUTE);
		String propertyValue = element.getAttribute(VALUE_ATTRIBUTE);
		if (propertyName.equalsIgnoreCase(ENABLED)) {
			action.putProperty(ENABLED, new Boolean("true".equalsIgnoreCase(propertyValue)));
		} else if (propertyName.equals(Action.ACCELERATOR_KEY)) {
			action.putProperty(
				propertyName,
				KeyStroke.getKeyStroke(propertyValue));
		} else if (propertyName.equals(Action.MNEMONIC_KEY)) {
			if (propertyValue.length() != 1)
				return;
			char keyChar = propertyValue.charAt(0);
			action.putProperty(propertyName, new Integer(keyChar));
		} else if (propertyName.equals(Action.SMALL_ICON)) {
			ClassLoader loader = element.getDeclaringExtension().getDeclaringPluginDescriptor().getPluginClassLoader();
			ImageIcon icon = new ImageIcon(loader.getResource(propertyValue));
			if (icon != null) action.putProperty(propertyName, icon);
		} else {
			action.putProperty(propertyName, propertyValue);
		}
	}
	
	private void addListenerOfDescriptor(IConfigurationElement element, ActionDescriptor action) {
		action.addEventListenerDescriptor(element.getAttribute(EVENT_ATTRIBUTE));
	}

	private void addSourceOfDescriptor(IConfigurationElement element, ActionDescriptor action) {
		action.addEventSourceDescriptor(element.getAttribute(EVENT_ATTRIBUTE));
	}
}
