package gov.va.med.hds.cd.prism.action.internal;

import java.util.Arrays;
import java.util.Comparator;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;

public class RegistryTableModel extends AbstractTableModel implements TableModel {

	public static final int PLUGIN_LABEL_COLUMN = 0;
	public static final int PLUGIN_VERSION_COLUMN = 1;
	public static final int PLUGIN_PROVIDER_COLUMN = 2;
	public static final int PLUGIN_ID_COLUMN = 3;

	private IPluginDescriptor[] plugins;

	public RegistryTableModel(IPluginRegistry registry) {
		super();
		this.plugins = registry.getPluginDescriptors();
		Arrays.sort(this.plugins, new Comparator() {
			public int compare(Object o1, Object o2) {
				IPluginDescriptor d1 = (IPluginDescriptor) o1;
				IPluginDescriptor d2 = (IPluginDescriptor) o2;
				return d1.getUniqueIdentifier().compareTo(d2.getUniqueIdentifier());
			}
		});
	}

	public int getRowCount() {
		return plugins.length;
	}

	public int getColumnCount() {
		return 4;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case PLUGIN_LABEL_COLUMN :
				return plugins[rowIndex].getLabel();
			case PLUGIN_VERSION_COLUMN :
				return plugins[rowIndex].getVersion().toString();
			case PLUGIN_PROVIDER_COLUMN :
				return plugins[rowIndex].getProviderName();
			case PLUGIN_ID_COLUMN :
				return plugins[rowIndex].getUniqueIdentifier();
			default :
				return null;
		}
	}

	public String getColumnName(int columnIndex) {
		switch (columnIndex) {
			case PLUGIN_LABEL_COLUMN :
				return "Plug-in Name";
			case PLUGIN_VERSION_COLUMN :
				return "Version";
			case PLUGIN_PROVIDER_COLUMN :
				return "Provider";
			case PLUGIN_ID_COLUMN :
				return "Plug-in Id";
			default :
				return null;
		}
	}

}
