package gov.va.med.hds.cd.prism.internal;

import gov.va.med.hds.cd.prism.DesktopWindowAdapter;
import gov.va.med.hds.cd.prism.DesktopWindowEvent;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IDesktopWindowEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveEventSourceSupport;

public class DesktopPerspectiveEventSource extends PerspectiveEventSourceSupport {

	private IDesktopWindowEventSource windowEventSource;

	public DesktopPerspectiveEventSource(IDesktopWindowEventSource windowEventSource) {
		this.windowEventSource = windowEventSource;
		this.windowEventSource.addWindowListener(new WindowListener());
	}

	public void addPerspectiveListener(IPerspectiveListener l) {
		super.addPerspectiveListener(l);
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w != null) w.addPerspectiveListener(l);
	}

	public IPerspectiveReference getActivePerspective() {
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w == null) return null;
		return w.getActivePerspective();
	}

	public void removePerspectiveListener(IPerspectiveListener l) {
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w != null) w.removePerspectiveListener(l);
		super.removePerspectiveListener(l);
	}

	private void addListeners(IDesktopWindow window) {
		IPerspectiveListener[] listeners = (IPerspectiveListener[]) listenerList.getListeners(IPerspectiveListener.class);
		for (int i = 0; i < listeners.length; i++) {
			window.addPerspectiveListener(listeners[i]);
		}
	}
	
	private void removeListeners(IDesktopWindow window) {
		IPerspectiveListener[] listeners = (IPerspectiveListener[]) listenerList.getListeners(IPerspectiveListener.class);
		for (int i = 0; i < listeners.length; i++) {
			window.removePerspectiveListener(listeners[i]);
		}
	}
	
	private class WindowListener extends DesktopWindowAdapter {
		
		public void windowActivated(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			IPerspectiveReference activePerspective = w.getActivePerspective();
			if (activePerspective != null) firePerspectiveActivated(activePerspective, w);
		}

		public void windowClosed(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			removeListeners(w);
			IPerspectiveReference[] perspectives = w.getPerspectives();
			for (int i = 0; i < perspectives.length; i++) {
				firePerspectiveClosed(perspectives[i], w);	
			}
		}

		public void windowOpened(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			addListeners(w);
			IPerspectiveReference activePerspective = w.getActivePerspective();
			if (activePerspective == null) return;
			firePerspectiveOpening(activePerspective.getDescriptor(), w);
			firePerspectiveOpened(activePerspective, w);
			firePerspectiveActivated(activePerspective, w);
		}
	}
}
