package gov.va.med.hds.cd.prism.internal;

import gov.va.med.hds.cd.prism.DesktopWindowAdapter;
import gov.va.med.hds.cd.prism.DesktopWindowEvent;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IDesktopWindowEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentListener;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentEventSourceSupport;

public class DesktopVisualComponentEventSource extends VisualComponentEventSourceSupport {

	private IDesktopWindowEventSource windowEventSource;

	public DesktopVisualComponentEventSource(IDesktopWindowEventSource windowEventSource) {
		this.windowEventSource = windowEventSource;
		this.windowEventSource.addWindowListener(new WindowListener());
	}

	public void addVisualComponentListener(IVisualComponentListener l) {
		super.addVisualComponentListener(l);
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w != null) w.getVisualComponentEventSource().addVisualComponentListener(l);
	}

	public void removePerspectiveListener(IVisualComponentListener l) {
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w != null) w.getVisualComponentEventSource().removeVisualComponentListener(l);
		super.removeVisualComponentListener(l);
	}

	public IVisualComponentReference getActiveVisualComponent() {
		IDesktopWindow w = windowEventSource.getActiveWindow();
		if (w == null) return null;
		return w.getVisualComponentEventSource().getActiveVisualComponent();
	}

	private void addListeners(IDesktopWindow window) {
		IVisualComponentListener[] listeners = (IVisualComponentListener[]) listenerList.getListeners(IVisualComponentListener.class);
		for (int i = 0; i < listeners.length; i++) {
			window.getVisualComponentEventSource().addVisualComponentListener(listeners[i]);
		}
	}
	
	private void removeListeners(IDesktopWindow window) {
		IVisualComponentListener[] listeners = (IVisualComponentListener[]) listenerList.getListeners(IVisualComponentListener.class);
		for (int i = 0; i < listeners.length; i++) {
			window.getVisualComponentEventSource().removeVisualComponentListener(listeners[i]);
		}
	}

	private class WindowListener extends DesktopWindowAdapter {
		
		public void windowActivated(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			fireComponentActivated(w.getVisualComponentEventSource().getActiveVisualComponent());
		}

		public void windowClosed(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			removeListeners(w);
			IPerspectiveReference activePerspective = w.getActivePerspective();
			if (activePerspective == null) return;
			IVisualComponentReference[] components = activePerspective.getVisualComponents();
			for (int i = 0; i < components.length; i++) {
				fireComponentClosed(components[i]);
			}
		}

		public void windowDeactivated(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			fireComponentDeactivated(w.getVisualComponentEventSource().getActiveVisualComponent());
		}

		public void windowOpened(DesktopWindowEvent e) {
			IDesktopWindow w = e.getWindow();
			addListeners(w);
			IPerspectiveReference activePerspective = w.getActivePerspective();
			if (activePerspective == null) return;
			IVisualComponentReference[] components = activePerspective.getVisualComponents();
			for (int i = 0; i < components.length; i++) {
				fireComponentOpened(components[i]);
			}
		}
	}
}
