package gov.va.med.hds.cd.prism.internal;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionMapBuilder;
import gov.va.med.hds.cd.prism.action.internal.ActionProxyDelegateEvent;
import gov.va.med.hds.cd.prism.action.internal.IActionProxyDelegateListener;
import gov.va.med.hds.cd.prism.menu.internal.IMenuFactory;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveAdapter;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.perspective.internal.ActivePerspectiveModel;
import gov.va.med.hds.cd.prism.perspective.internal.IActivePerspectiveModel;
import gov.va.med.hds.cd.prism.perspective.internal.IPerspectiveSelector;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectivePanel;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveSelector;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentEventSource;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.cd.security.IUserInfo;
import gov.va.med.hds.cd.workspace.EventRegistrar;
import gov.va.med.hds.cd.workspace.IMulticaster;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.cd.workspace.IWorkspaceScope;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JSplitPane;
import javax.swing.WindowConstants;
import javax.swing.border.BevelBorder;

public class DesktopWindow extends JFrame implements IDesktopWindow {
	private static final int PREFERRED_WIDTH = 800;
	private static final int PREFERRED_HEIGHT = 600;
	private static final int SELECTOR_MIN_WIDTH = 21;
	private static final String ICON = "prismIcon.png";
	private static final String PRISM_INTRO_HELPID = "gov.va.med.hds.cd.prism.intro";

	private IPerspectiveEventSource perspectiveEventSource;
	private IVisualComponentEventSource visualComponentEventSource;

	private IDesktop desktop;
	private IActivePerspectiveModel perspectiveModel;
	private JComponent contentPanel;
	private PerspectivePanel perspectivePanel;

	private ActionMap actions;

	private IMenuFactory menuFactory;
	private JMenuBar menuBar;
	private MenuBarManager menuBarManager = new MenuBarManager();

	public DesktopWindow(IDesktop desktop, IMenuFactory menuFactory) {
		this(desktop, menuFactory, null);
	}

	public DesktopWindow(IDesktop desktop, IMenuFactory menuFactory, String perspectiveId) {
		super();
		this.desktop = desktop;
		this.menuFactory = menuFactory;
		createPerspectiveModel(perspectiveId);
		createActionMap();
		perspectiveModel.addPerspectiveListener(menuBarManager);
		getContentPane().add(createMainPanel());
		IPerspectiveDescriptor activePerspective = perspectiveModel.getActivePerspective();
		setJMenuBar(
			menuBarManager.getMenuBar(
				(activePerspective == null ? null : activePerspective.getId())));
		setVisualComponentEventSource(new WindowVisualComponentEventSource(this));
		setPerspectiveEventSource(perspectivePanel);
		addPerspectiveListener(new CursorPerspectiveListener());
		setWindowProperties();
		setVisible(true);
	}

	private void createActionMap() {
		ActionEventRegistrar registrar = new ActionEventRegistrar();
		ActionMapBuilder builder = new ActionMapBuilder(desktop.getRuntime());
		actions = builder.buildActionMap(getDesktop().getActionRegistry(), registrar);
	}

	private void setWindowProperties() {
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setIconImage(DesktopLookAndFeel.getDesktopIconImage());
		setTitleProperty();
		centerFrame();
	}

	private void setTitleProperty() {
		StringBuffer title = new StringBuffer();
		title.append(IPrismConstants.DESKTOP_NAME);
		String userDisplayName = getUserDisplayName();
		title.append((userDisplayName != null) ? (" in use by: " + userDisplayName) : "");
		String serverAndPort = getServerAndPort();
		title.append((serverAndPort != null) ? " (" + serverAndPort + ")" : "");
		this.setTitle(title.toString());
	}
	private String getUserDisplayName() {
		try {
			ISecurity securityPlugin = getSecurity();
			IUserInfo userInfo = securityPlugin.getUserInfo();
			return userInfo.getLastName() + "," + userInfo.getFirstName();
		} catch (Exception e) {
			return null;
		}
	}
	private String getServerAndPort() {
		try {
			IMumpsFactory m = getM();
			return m.getServerName() + ":" + Integer.toString(m.getServerPort());
		} catch (Exception e) {
			return null;
		}
	}

	private ISecurity getSecurity() throws PluginStartupException, PluginNotFoundException {
		return (ISecurity) desktop.getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
	}

	private IMumpsFactory getM() throws PluginStartupException, PluginNotFoundException {
		return (IMumpsFactory) desktop.getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID);
	}

	protected JSplitPane createMainPanel() {
		JPanel selectionPanel = createSelectionPanel();
		this.contentPanel = createContentPanel();
		JSplitPane split =
			new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, selectionPanel, contentPanel);
		addHelpIds(split);

		return split;
	}
	private void addHelpIds(JComponent component) {
		component.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, PRISM_INTRO_HELPID);
	}

	protected JPanel createSelectionPanel() {
		JPanel selectionPanel = new JPanel();
		selectionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

		selectionPanel.setLayout(new BorderLayout());

		IPerspectiveSelector selector = createPerspectiveSelector();
		selector.setModel(this.perspectiveModel);
		selectionPanel.add((JComponent) selector, BorderLayout.CENTER);

		Dimension minSize = selectionPanel.getMinimumSize();
		minSize.width = SELECTOR_MIN_WIDTH;
		selectionPanel.setMinimumSize(minSize);

		return selectionPanel;
	}

	protected JPanel createContentPanel() {
		JPanel contentPanel = new JPanel();

		contentPanel.setLayout(new BorderLayout());
		if (this.menuBar != null)
			contentPanel.add(this.menuBar, BorderLayout.NORTH);
		this.perspectivePanel = new PerspectivePanel(this, this.perspectiveModel);
		contentPanel.add(perspectivePanel, BorderLayout.CENTER);
		setPerspectiveEventSource(perspectivePanel);
		return contentPanel;
	}

	protected void createPerspectiveModel(String defaultPerspectiveId) {
		this.perspectiveModel = new ActivePerspectiveModel();

		IPerspectiveDescriptor[] perspectives =
			desktop.getPerspectiveRegistry().getPerspectiveDescriptors();
		for (int i = 0; i < perspectives.length; i++) {
			this.perspectiveModel.add(perspectives[i]);
		}

		setActivePerspective(defaultPerspectiveId);
	}

	public void setActivePerspective(String activePerspectiveId) {
		if (activePerspectiveId != null && activePerspectiveId.length() != 0) {
			this.perspectiveModel.setActivePerspective(activePerspectiveId);
		}

		if (this.perspectiveModel.getActivePerspective() == null) {
			setFirstPerspectiveActive();
		}
	}

	private void setFirstPerspectiveActive() {
		if (perspectiveModel.size() == 0)
			return;
		IPerspectiveDescriptor[] descriptors = perspectiveModel.getDescriptors();
		if (descriptors.length == 0)
			return;
		perspectiveModel.setActivePerspective(descriptors[0]);
	}

	protected IPerspectiveSelector createPerspectiveSelector() {
		return new PerspectiveSelector();
	}

	private void centerFrame() {
		Dimension d = new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT);
		setSize(d);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(
			(screenSize.width / 2) - (d.width / 2),
			(screenSize.height / 2) - (d.height / 2));
	}

	/**
	 * @see gov.va.med.hds.cd.prism.IDesktopWindow#getDesktop()
	 */
	public IDesktop getDesktop() {
		return desktop;
	}

	/**
	 * @see gov.va.med.hds.cd.prism.IDesktopWindow#getFrame()
	 */
	public JFrame getFrame() {
		return this;
	}

	/**
	 * @see gov.va.med.hds.cd.prism.IDesktopWindow#getActionMap()
	 */
	public ActionMap getActionMap() {
		return actions;
	}

	/**
	* @see gov.va.med.hds.cd.prism.IDesktopWindow#getVisualComponentEventSource()
	*/
	public IVisualComponentEventSource getVisualComponentEventSource() {
		return this.visualComponentEventSource;
	}

	/**
	* @see gov.va.med.hds.cd.prism.IPerspectiveEventSource.getActivePerspective()
	*/
	public IPerspectiveReference getActivePerspective() {
		return getPerspectiveEventSource().getActivePerspective();
	}

	/**
	 * @see gov.va.med.hds.cd.prism.IPerspectiveEventSource.addPerspectiveListener()
	 */
	public void addPerspectiveListener(IPerspectiveListener l) {
		getPerspectiveEventSource().addPerspectiveListener(l);
	}

	/**
	 * @see gov.va.med.hds.cd.prism.IPerspectiveEventSource.removePerspectiveListener()
	 */
	public void removePerspectiveListener(IPerspectiveListener l) {
		getPerspectiveEventSource().removePerspectiveListener(l);
	}

	public IPerspectiveReference[] getPerspectives() {
		return perspectivePanel.getPerspectives();
	}

	public JMenuBar getJMenuBar() {
		return menuBar;
	}

	public void setJMenuBar(JMenuBar menubar) {
		if (this.menuBar != null && this.menuBar.getParent() == contentPanel)
			contentPanel.remove(this.menuBar);

		this.menuBar = menubar;

		if (menuBar != null)
			contentPanel.add(menuBar, BorderLayout.NORTH);

		contentPanel.revalidate();
		contentPanel.repaint();
	}

	private IPerspectiveEventSource getPerspectiveEventSource() {
		return perspectiveEventSource;
	}

	private void setPerspectiveEventSource(IPerspectiveEventSource source) {
		this.perspectiveEventSource = source;
	}

	private void setVisualComponentEventSource(IVisualComponentEventSource visualComponentEventSource) {
		this.visualComponentEventSource = visualComponentEventSource;
	}

	private class MenuBarManager extends PerspectiveAdapter {

		private Map menuBars = new HashMap();
		private JMenuBar currentMenuBar;

		public void perspectiveActivated(PerspectiveEvent e) {
			JMenuBar menuBar = getMenuBar(e.getPerspective().getDescriptor().getId());
			menuBar.updateUI();
			setJMenuBar(menuBar);
		}

		public JMenuBar getMenuBar(String perspectiveId) {
			if (perspectiveId == null)
				perspectiveId = "null";
			JMenuBar menuBar = (JMenuBar) menuBars.get(perspectiveId);
			if (menuBar == null) {
				menuBar = createMenuBar(perspectiveId);
				menuBars.put(perspectiveId, menuBar);
			}
			return menuBar;
		}

		private JMenuBar createMenuBar(String perspectiveId) {
			return menuFactory.createMenuBar(perspectiveId, getActionMap());
		}
	}

	private class ActionEventRegistrar
		extends EventRegistrar
		implements IActionProxyDelegateListener {

		private IWorkspace workspace;

		public ActionEventRegistrar() {
			try {
				workspace =
					(IWorkspace) getDesktop().getRuntime().getPluginInstance(
						IWorkspaceConstants.PLUGIN_ID);
			} catch (Exception e) {
				getDesktop().getRuntime().getLogger(getClass()).error(
					"unable to get workspace reference",
					e);
			}
		}

		public void delegateCreated(ActionProxyDelegateEvent e) {
			IActionDescriptor d = e.getProxy().getDescriptor();
			connect(d, e.getDelegateAction());
		}

		private void connect(IActionDescriptor d, Action action) {
			IPerspectiveDescriptor[] perspectives = getPerspectivesFor(d);
			for (int i = 0; i < perspectives.length; i++) {
				String scopeId =
					PrismUtilities.getPerspectiveScopeId(DesktopWindow.this, perspectives[i]);
				IWorkspaceScope perspectiveScope = workspace.getScope(scopeId);
				if (perspectiveScope == null) {
					workspace.addScope(scopeId);
					perspectiveScope = workspace.getScope(scopeId);
				}
				IMulticaster caster = perspectiveScope.getMulticaster();
				try {
					connectSources(caster, action, d.getSourceOfEventDescriptors());
					connectListeners(caster, action, d.getListenerOfEventDescriptors());
				} catch (Exception ex) {
					getDesktop().getRuntime().getLogger(getClass()).error(
						"unable to connect events for action " + d,
						ex);
				}
			}
		}

		private IPerspectiveDescriptor[] getPerspectivesFor(IActionDescriptor action) {
			List perspectives = new LinkedList();
			IPerspectiveDescriptor[] persps = perspectiveModel.getDescriptors();
			for (int i = 0; i < persps.length; i++) {
				String actionId = action.getId();
				String perspectiveId = persps[i].getId();
				boolean menuContains = menuFactory.menuBarContains(perspectiveId, actionId);
				if (menuContains) {
					perspectives.add(persps[i]);
				}
			}
			return (IPerspectiveDescriptor[]) perspectives.toArray(
				new IPerspectiveDescriptor[perspectives.size()]);
		}
	}

	private class CursorPerspectiveListener extends PerspectiveAdapter {
		public void perspectiveOpened(PerspectiveEvent e) {
			JRootPane rootPane = getRootPane();
			if (rootPane != null)
				rootPane.setCursor(Cursor.getDefaultCursor());
		}

		public void perspectiveOpening(PerspectiveEvent e) {
			JRootPane rootPane = getRootPane();
			if (rootPane != null)
				rootPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}
	}
}
