package gov.va.med.hds.cd.prism.internal;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.prism.DesktopWindowCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopWindowEvent;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IDesktopWindowEventSource;
import gov.va.med.hds.cd.prism.IDesktopWindowListener;

public class DesktopWindowManager implements IDesktopWindowEventSource {

	private List windows = new ArrayList();
	private IDesktopWindow activeWindow;
	private EventListenerList listenerList = new EventListenerList();
	private SwingWindowListener windowListener = new SwingWindowListener();

	public void open(IDesktopWindow window) {
		if (windows.contains(window))
			return;
		fireWindowOpening(window);
		windows.add(window);
		fireWindowOpened(window);
		window.getFrame().addWindowListener(this.windowListener);
		setActiveWindow(window);
	}

	public void close(IDesktopWindow window) {
		if (!windows.contains(window))
			return;
		try {
			fireWindowClosing(window);
		} catch (DesktopWindowCloseVetoException e) {
			if (getWindowCount() == 1) return; // only the last window can be vetoed
		}
		windows.remove(window);
		window.getFrame().removeWindowListener(this.windowListener);
		if (getActiveWindow().equals(window)) {
			if (getWindowCount() > 0) {
				setActiveWindow((IDesktopWindow) windows.get(0));
			} else {
				setActiveWindow(null);
			}
		}
		window.getFrame().setVisible(false);
		fireWindowClosed(window);
	}

	public void addWindowListener(IDesktopWindowListener l) {
		listenerList.add(IDesktopWindowListener.class, l);
	}

	public void removeWindowListener(IDesktopWindowListener l) {
		listenerList.remove(IDesktopWindowListener.class, l);
	}

	public IDesktopWindow getActiveWindow() {
		return activeWindow;
	}

	public void setActiveWindow(IDesktopWindow window) {
		if (getActiveWindow() != null && getActiveWindow().equals(window))
			return;
		if (window != null && !isWindowOpen(window))
			open(window);
		if (getActiveWindow() != null)
			fireWindowDeactivated(activeWindow);
		this.activeWindow = window;
		if (getActiveWindow() != null) this.activeWindow.getFrame().requestFocus();
		if (getActiveWindow() != null)
			fireWindowActivated(activeWindow);
	}

	public boolean isWindowOpen(IDesktopWindow window) {
		return windows.contains(window);
	}

	public IDesktopWindow[] getWindows() {
		IDesktopWindow[] returnVal = new IDesktopWindow[windows.size()];
		windows.toArray(returnVal);
		return returnVal;
	}

	public int getWindowCount() {
		return windows.size();
	}

	protected void fireWindowActivated(IDesktopWindow w) {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowActivated(event);
		}
	}

	protected void fireWindowDeactivated(IDesktopWindow w) {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowDeactivated(event);
		}
	}

	protected void fireWindowOpening(IDesktopWindow w) {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowOpening(event);
		}
	}

	protected void fireWindowOpened(IDesktopWindow w) {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowOpened(event);
		}
	}

	protected void fireWindowClosed(IDesktopWindow w) {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowClosed(event);
		}
	}

	protected void fireWindowClosing(IDesktopWindow w) throws DesktopWindowCloseVetoException {
		final DesktopWindowEvent event = new DesktopWindowEvent(this, w);
		IDesktopWindowListener[] listeners =
			(IDesktopWindowListener[]) listenerList.getListeners(IDesktopWindowListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].windowClosing(event);
		}
	}

	private IDesktopWindow findWindowFromFrame(JFrame frame) {
		for (Iterator i = windows.iterator(); i.hasNext();) {
			IDesktopWindow window = (IDesktopWindow) i.next();
			if (frame.equals(window.getFrame()))
				return window;
		}
		return null;
	}

	private class SwingWindowListener implements WindowListener {

		public void windowActivated(WindowEvent e) {
			if (!(e.getWindow() instanceof JFrame))
				return;
			IDesktopWindow window = findWindowFromFrame((JFrame) e.getWindow());
			if (window != null)
				setActiveWindow(window);
		}

		public void windowClosing(WindowEvent e) {
			if (!(e.getWindow() instanceof JFrame))
				return;
			IDesktopWindow window = findWindowFromFrame((JFrame) e.getWindow());
			close(window);
		}

		public void windowClosed(WindowEvent e) {
			// NOOP
		}

		public void windowDeactivated(WindowEvent e) {
			// NOOP
		}

		public void windowDeiconified(WindowEvent e) {
			// NOOP
		}

		public void windowIconified(WindowEvent e) {
			// NOOP
		}

		public void windowOpened(WindowEvent e) {
			// NOOP
		}
	}
}
