package gov.va.med.hds.cd.prism.internal;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.Timer;

import gov.va.med.hds.util.os.OSUtils;

public class InactivityDialog extends JDialog {

	public static int CONTINUE = 42;
	public static int CLOSE = 23;

	private static final String DESKTOP = "Clinician Desktop";
	private static final String TITLE = DESKTOP + " Timeout";
	private static final String LINE_1 = DESKTOP + " has been idle and will close!";
	private static final String LINE_2 = "Press the button to continue working with " + DESKTOP +".";
	private static final String BUTTON_CAPTION = "Don't close " + DESKTOP;

	public static int showInactivityDialog(JFrame parent, int countDownSeconds) {
		InactivityDialog dialog = new InactivityDialog(parent, countDownSeconds);
		return dialog.getResult();
	}

	private Timer countDownTimer;
	private int secondsRemaining;
	private int result = CLOSE;
	private JLabel remainingLabel;
	private JButton dontCloseButton;
    private long parentWindowHandle;

	public InactivityDialog(JFrame owner, int countDownSeconds) {
		super(owner, TITLE, true);
		this.secondsRemaining = countDownSeconds;

		countDownTimer = new Timer(1000, new Countdown());
		countDownTimer.setRepeats(true);

		init();
		
		getToolkit().beep();
		countDownTimer.start();
		show();
	}

	private void init() {
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		createSecondsRemainingLabel();
		createDontCloseButton();
		
		getContentPane().add(createMainPanel());
		
		getRootPane().setDefaultButton(dontCloseButton);
		pack();
		
		Dimension ownerSize = getOwner() != null && getOwner().isShowing() ? getOwner().getSize() : getToolkit().getScreenSize();
		int x = (getOwner() != null ? getOwner().getX() : 0) + (ownerSize.width - getSize().width) / 2;
		int y = (getOwner() != null ? getOwner().getY() : 0) + (ownerSize.height - getSize().height) / 2;
		setLocation(x, y);
        
        parentWindowHandle = OSUtils.getParentWindowHandle(this);
        OSUtils.displayAndStayOnTop(parentWindowHandle);
	}

	private void createDontCloseButton() {
		dontCloseButton = new JButton(BUTTON_CAPTION);
		dontCloseButton.addActionListener(new ContinueWorking());
	}

	private void createSecondsRemainingLabel() {
		remainingLabel = new JLabel(String.valueOf(secondsRemaining));
		remainingLabel.setFont(remainingLabel.getFont().deriveFont(Font.BOLD, 20));
	}

	public int getResult() {
		return result;
	}

	private Container createMainPanel() {
		Box b = Box.createVerticalBox();
		b.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
		b.add(createLineOneLabel());
		b.add(Box.createVerticalStrut(6));
		b.add(createLineTwoLabel());
		b.add(Box.createVerticalStrut(6));
		b.add(createLowPanel());
		return b;
	}

	private JLabel createLineTwoLabel() {
		JLabel l = new JLabel(LINE_2);
		l.setHorizontalAlignment(SwingConstants.CENTER);
		return l;
	}
	private Component createLineOneLabel() {
		JLabel l = new JLabel(LINE_1);
		l.setHorizontalAlignment(SwingConstants.CENTER);
		l.setFont(l.getFont().deriveFont(Font.BOLD));
		return l;
	}

	private JPanel createLowPanel() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
		p.add(Box.createHorizontalStrut(12));
		p.add(remainingLabel);
		p.add(Box.createHorizontalGlue());
		p.add(dontCloseButton);
		p.add(Box.createHorizontalStrut(12));
		return p;
	}

	private void updateDisplay() {
		remainingLabel.setText(String.valueOf(secondsRemaining));
	}

	private void close(int result) {
		countDownTimer.stop();
		this.result = result;
		hide();
        OSUtils.setStayOnTop(parentWindowHandle, false);
	}

	private class Countdown implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			--secondsRemaining;
			updateDisplay();
			if (secondsRemaining <= 0) {	
				close(CLOSE);
			}
		}
	}

	private class ContinueWorking implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			close(CONTINUE);
		}
	}
}
