package gov.va.med.hds.cd.prism.internal;

import java.awt.AWTEvent;
import java.awt.event.AWTEventListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.Timer;

import gov.va.med.hds.cd.prism.IDesktop;

public class InactivityListener implements AWTEventListener {

	public static final long ACTIVITY_MASK =
		AWTEvent.MOUSE_EVENT_MASK
			| AWTEvent.MOUSE_MOTION_EVENT_MASK
			| AWTEvent.MOUSE_WHEEL_EVENT_MASK
			| AWTEvent.KEY_EVENT_MASK;

	private Timer inactivityTimer;
	private IDesktop desktop;
	private ActionListener exitListener;
	private int timeoutSeconds;
	private int countdownSeconds;
	
	public InactivityListener(
		IDesktop desktop,
		int timeoutSeconds,
		int countdownSeconds,
		ActionListener exitListener) {
		this.exitListener = exitListener;
		this.desktop = desktop;
		this.timeoutSeconds = timeoutSeconds;
		this.countdownSeconds = countdownSeconds;
		inactivityTimer = new Timer(timeoutSeconds * 1000, new Timeout());
		inactivityTimer.setRepeats(false);
		inactivityTimer.start();
	}

	public void eventDispatched(AWTEvent event) {
		userActivityDetected();
	}

	private void userActivityDetected() {
		if (inactivityTimer.isRunning()) {
			inactivityTimer.restart();
		}
	}

	private class Timeout implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			inactivityTimer.stop();
			JFrame activeWindow = (desktop != null ? desktop.getActiveWindow().getFrame() : null);
			if (activeWindow != null) {
				int result = InactivityDialog.showInactivityDialog(activeWindow, countdownSeconds);
				if (result == InactivityDialog.CONTINUE) {
					inactivityTimer.restart();
					return;
				}
			}
			exitListener.actionPerformed(e);
		}
	}
}
