package gov.va.med.hds.cd.prism.internal;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;

public class LogWindow extends JFrame implements IPrismConstants {
    private static Document document = new PlainDocument();
    private static JFrame logWindow;
    private static LogOutputCollector collector;

	private JTextArea textArea;

    public LogWindow(Document document) {
        super(DESKTOP_NAME + " Log Window");
        setIconImage(DesktopLookAndFeel.getDesktopIconImage());
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        JMenuBar menubar = new JMenuBar();
        setJMenuBar(menubar);

        JMenuItem item = new JMenuItem("Copy");
        item.setMnemonic('C');
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    textArea.selectAll();
                    textArea.copy();
                }
            });
        menubar.add(item);

        item = new JMenuItem("Clear");
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    textArea.setText(null);
                }
            });
        menubar.add(item);

        textArea = new JTextArea(document);
        textArea.setEditable(false);
        textArea.setCaretColor(textArea.getBackground());
        getContentPane().add(new JScrollPane(textArea));
        setSize(new Dimension(600, 300));
        Dimension screenSize = getToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);
    }

    public static void setLogWindowVisible(boolean b) {
        if (b) {
            try {
                if (logWindow == null) {
                    logWindow = new LogWindow(document);
                }

                logWindow.setVisible(true);
                logWindow.toFront();
            } catch (Throwable t) {
                // don't print anything!  If we're in panic mode, it will
                // cause a StackOverflowError.
            }
        } else {
            if (logWindow != null) {
                logWindow.setVisible(false);
            }

            logWindow = null;
        }
    }

    public static void install(ILoggingCapable logging) {
        if (isInstalled()) {
            return;
        }

        if (collector == null) {
            collector = new LogOutputCollector();
        }

        logging.getRootLogger().addHandler(collector);
    }

    public static void uninstall(ILoggingCapable logging) {
        if (!isInstalled()) {
            return;
        }

        logging.getRootLogger().removeHandler(collector);
    }

    private static boolean isInstalled() {
        return collector != null;
    }

    private static class LogOutputCollector implements IHandler {
        public void publish(ILogRecord logRecord) {
            append(logRecord.getMessage());
        }

        public void close() {
            // NOOP
        }

        private void append(String s) {
            try {
                s = s.replaceAll("\r", "");

                int docLength = document.getLength();
                document.insertString(docLength, s, null);
            } catch (BadLocationException e) {
            }
        }
    }
}
