package gov.va.med.hds.cd.prism.internal;

import java.awt.Toolkit;
import java.io.IOException;

import javax.swing.Action;
import javax.swing.UIManager;
import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.help.HelpUtilities;
import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.mum.boot.ISplashScreen;
import gov.va.med.hds.cd.mum.boot.IStatusNotifier;
import gov.va.med.hds.cd.mum.boot.loader.ClassLoaderManager;
import gov.va.med.hds.cd.mum.boot.loader.IDelegatingClassLoader;
import gov.va.med.hds.cd.mum.boot.loader.LinkedPluginLoader;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.LookAndFeelUtilities;
import gov.va.med.hds.cd.prism.DesktopCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopEvent;
import gov.va.med.hds.cd.prism.DesktopWindowAdapter;
import gov.va.med.hds.cd.prism.DesktopWindowCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopWindowEvent;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.IDesktopListener;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IDesktopWindowListener;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionRegistry;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistryReader;
import gov.va.med.hds.cd.prism.action.internal.InitPrintActionEvent;
import gov.va.med.hds.cd.prism.menu.internal.IMenuFactory;
import gov.va.med.hds.cd.prism.menu.internal.MenuFactory;
import gov.va.med.hds.cd.prism.menu.internal.MenuTree;
import gov.va.med.hds.cd.prism.menu.internal.MenuTreeReader;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveRegistryReader;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentEventSource;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentRegistry;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistry;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistryReader;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IMainPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntime;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;

public class Prism implements IMainPlugin, IDesktop {

	private class DesktopWindowListener extends DesktopWindowAdapter implements IDesktopWindowListener {
		public void windowClosing(DesktopWindowEvent e)
			throws DesktopWindowCloseVetoException {
			if (windowManager.getWindowCount() == 1) {
				try {
					close();
				} catch (DesktopCloseVetoException ex) {
					throw new DesktopWindowCloseVetoException(e, ex);
				}
			}
		}
	}

	private static final String MISSING_ACTION_WARNING =
		"missing default action: ";
	private ActionRegistry actions;
	private VisualComponentRegistry components;
	private InactivityListener inactivityListener;

	private boolean isClosed = false;
	private EventListenerList listenerList = new EventListenerList();
	private MenuTree menus;
	private IMenuFactory menuFactory;

	private IPrismParameters parameters;
	private IPerspectiveEventSource perspectiveEventSource;
	private PerspectiveRegistry internalPerspectives;
	private PerspectiveRegistry perspectives;
	private IPluginDescriptor prismDescriptor;

	private IRuntime runtime;
	private IRuntimeContext runtimeContext;
	private IVisualComponentEventSource visualComponentEventSource;

	private DesktopWindowManager windowManager;
	private Section508Dialog section508Dialog;
	private IWorkspace workspace;

	public Prism() {
		this.windowManager = createWindowManager();
		this.perspectiveEventSource = createPerspectiveEventSource();
		this.visualComponentEventSource =
			createVisualComponentEventSource();

		this.windowManager.addWindowListener(new DesktopWindowListener());
		// XXX: this is a Camp CPRS 2003 HACK: REMOVE
		this.windowManager.addWindowListener(new SillyPrintWindowListener());
	}

	public void addDesktopListener(IDesktopListener l) {
		listenerList.add(IDesktopListener.class, l);
	}

	public void addWindowListener(IDesktopWindowListener l) {
		windowManager.addWindowListener(l);
	}

	public synchronized void close() throws DesktopCloseVetoException {
		if (isClosed)
			return;
		fireDesktopClosing();
		isClosed = true;
		runtime.shutdown();
	}

	private void createActionRegistry() {
		ActionRegistryReader reader = new ActionRegistryReader();
		this.actions = reader.readRegisteredActionRegistry(runtimeContext);
		validateDefaultActions();
	}

	private void createInactivityListener() {
		this.inactivityListener =
			new InactivityListener(
				this,
				parameters.getTimeoutSeconds(),
				parameters.getCountdownSeconds(),
				getActiveWindow().getActionMap().get(
					IActionConstants.EXIT_ACTION));
		Toolkit.getDefaultToolkit().addAWTEventListener(
			this.inactivityListener,
			InactivityListener.ACTIVITY_MASK);
	}

	protected IMenuFactory getMenuFactory() {
		return menuFactory;
	}

	private void createMenuTree() {
		MenuTreeReader reader = new MenuTreeReader(this.runtimeContext);
		this.menus =
			reader.readMenuTree(
				this.runtimeContext.getPluginRegistry(),
				actions);
		this.menuFactory = new MenuFactory(this.menus);
	}

	protected IPerspectiveEventSource createPerspectiveEventSource() {
		return new DesktopPerspectiveEventSource(this);
	}

	private void createInternalPerspectiveRegistry() {
		PerspectiveRegistryReader reader =
			new PerspectiveRegistryReader(runtimeContext);
		this.internalPerspectives =
			reader.readPerspectiveRegistry(runtimeContext.getPluginRegistry());
	}

	protected IVisualComponentEventSource createVisualComponentEventSource() {
		return new DesktopVisualComponentEventSource(this);
	}

	private void createVisualComponentRegistry() {
		VisualComponentRegistryReader reader =
			new VisualComponentRegistryReader(runtimeContext);
		this.components =
			reader.readComponentRegistry(runtimeContext.getPluginRegistry());
	}

	protected DesktopWindowManager createWindowManager() {
		return new DesktopWindowManager();
	}

	private void disposeInactivityListener() {
		if (this.inactivityListener == null)
			return;
		Toolkit.getDefaultToolkit().removeAWTEventListener(
			this.inactivityListener);
		inactivityListener = null;
	}

	protected void fireDesktopClosing() throws DesktopCloseVetoException {
		if (listenerList.getListenerCount(IDesktopListener.class) == 0)
			return;
		final DesktopEvent event = new DesktopEvent(this);
		IDesktopListener[] listeners =
			(IDesktopListener[]) listenerList.getListeners(
				IDesktopListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].desktopClosing(event);
		}
	}

	protected void fireDesktopOpening() {
		if (listenerList.getListenerCount(IDesktopListener.class) == 0)
			return;
		final DesktopEvent event = new DesktopEvent(this);
		IDesktopListener[] listeners =
			(IDesktopListener[]) listenerList.getListeners(
				IDesktopListener.class);
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].desktopOpening(event);
		}
	}

	public IActionRegistry getActionRegistry() {
		return actions;
	}

	public IDesktopWindow getActiveWindow() {
		return windowManager.getActiveWindow();
	}

	private void fetchParameters() {
		parameters = new PrismParameters(getRuntime());
	}

	public IPerspectiveEventSource getPerspectiveEventSource() {
		return perspectiveEventSource;
	}

	public IPerspectiveRegistry getPerspectiveRegistry() {
		return perspectives;
	}

	public IRuntimeContext getRuntime() {
		return runtimeContext;
	}

	public IVisualComponentEventSource getVisualComponentEventSource() {
		return visualComponentEventSource;
	}

	public IVisualComponentRegistry getVisualComponentRegistry() {
		return components;
	}

	public IDesktopWindow[] getWindows() {
		return windowManager.getWindows();
	}

	private void notifyHelpOfNewWindow(IDesktopWindow window) {
		IHelp helpPlugin = HelpUtilities.getHelp(runtimeContext);
		if (helpPlugin != null)
			helpPlugin.setRootPane(window.getFrame().getRootPane());
	}

	public IDesktopWindow openWindow(String perspectiveId) {
		DesktopWindow window =
			new DesktopWindow(this, getMenuFactory(), perspectiveId);
		notifyHelpOfNewWindow(window);
		windowManager.open(window);
		return window;
	}

	public void removeDesktopListener(IDesktopListener l) {
		listenerList.remove(IDesktopListener.class, l);
	}

	public void removeWindowListener(IDesktopWindowListener l) {
		windowManager.removeWindowListener(l);
	}

	public void run(IRuntime runtime, ISplashScreen splash, String[] args) {
		this.runtime = runtime;
		installLogHandler();
		setDefaultLookAndFeel();
		updateSplash(
			splash,
			"Fetching " + IPrismConstants.DESKTOP_NAME + " parameters");
		fetchParameters();
		updateSplash(splash, "Opening " + IPrismConstants.DESKTOP_NAME);
		fireDesktopOpening();
		createPerspectiveRegistry();
		if (!parameters.getSection508Notice()) {
			addWindowListener(new Section508WindowListener());
		}
		IDesktopWindow window = openWindow(parameters.getDefaultPerspective());
		splash.close();
		createInactivityListener();
	}
	
	private void installLogHandler() {
		LogWindow.install(runtimeContext);
	}

	private class Section508WindowListener extends DesktopWindowAdapter {

		/* (non-Javadoc)
		 * @see gov.va.med.hds.cd.prism.IDesktopWindowListener#windowOpened(gov.va.med.hds.cd.prism.DesktopWindowEvent)
		 */
		public void windowOpened(DesktopWindowEvent e) {
			try {
				section508Dialog = new Section508Dialog(e.getWindow().getFrame());
				section508Dialog.setVisible(true);
				parameters.setSection508Notice();
			} catch (IOException e1) {
				ILogger logger = runtimeContext.getLogger(getClass().getName());
				logger.warning("Unable to load 508 notice: ", e1);
			}
		}
	}

	private void createPerspectiveRegistry() {
		this.perspectives = new PerspectiveRegistry();
		String[] perspectiveIds = parameters.getPerspectiveIds();
		for (int i = 0; i < perspectiveIds.length; i++) {
			IPerspectiveDescriptor descriptor = internalPerspectives.get(perspectiveIds[i]);
			if (descriptor != null) perspectives.add(descriptor);
		}
	}

	private void setDefaultLookAndFeel() {		
		try {
            UIManager.setLookAndFeel(new DesktopLookAndFeel(createPlafClassLoader()));
            LookAndFeelUtilities.updateFrames();
		} catch (Exception e) {
            e.printStackTrace();
			getLogger().warning(
				"Unable to set the "
					+ IPrismConstants.DESKTOP_NAME
					+ " look and feel",
				e);
		}
	}
    
    private ClassLoader createPlafClassLoader() {
        IDelegatingClassLoader result = ClassLoaderManager.getDelegatingLoader();
        IExtension[] exts =
            runtimeContext.getPluginRegistry().getExtensions(
                IPrismConstants.UNIQUE_PLAF_PROVIDER_EXTENSION_POINT_ID);

        for (int i = 0; i < exts.length; i++) {
            ClassLoader loader = exts[i].getDeclaringPluginDescriptor().getPluginClassLoader();
            if (loader instanceof LinkedPluginLoader)
                result.addLoader((LinkedPluginLoader) loader);
        }
        
        return (ClassLoader) result;
    }

	public void shutdown() {
		try {
			close();
		} catch (DesktopCloseVetoException e) {
			getLogger().error(
				"Unexpected veto during desktop close on prism shutdown",
				e);
		}
		disposeInactivityListener();
		actions = null;
	}

	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception {
		this.runtimeContext = runtime;
		this.prismDescriptor = plugin;
		this.workspace = (IWorkspace) runtime.getPluginInstance(IWorkspaceConstants.PLUGIN_ID);
		createActionRegistry();
		createMenuTree();
		createVisualComponentRegistry();
		createInternalPerspectiveRegistry();
		createWorkspaceRegistrars();
	}

	private void createWorkspaceRegistrars() {
		addDesktopListener(new SessionScopeRegistrar(getWorkspace()));
		addWindowListener(new PatientScopeRegistrar(getWorkspace()));
		getPerspectiveEventSource().addPerspectiveListener(new PerspectiveScopeRegistrar(getWorkspace()));	
		getPerspectiveEventSource().addPerspectiveListener(new VisualComponentEventRegistrar(getRuntime(), getWorkspace()));	
	}

	private void validateDefaultActions() {
		if (!this.actions.contains(IActionConstants.EXIT_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.EXIT_ACTION);
		if (!this.actions.contains(IActionConstants.UNDO_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.UNDO_ACTION);
		if (!this.actions.contains(IActionConstants.REDO_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.REDO_ACTION);
		if (!this.actions.contains(IActionConstants.CUT_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.CUT_ACTION);
		if (!this.actions.contains(IActionConstants.COPY_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.COPY_ACTION);
		if (!this.actions.contains(IActionConstants.PASTE_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.PASTE_ACTION);
		if (!this.actions.contains(IActionConstants.DELETE_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.DELETE_ACTION);
		if (!this.actions.contains(IActionConstants.ABOUT_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.ABOUT_ACTION);
		if (!this.actions.contains(IActionConstants.PRINT_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.PRINT_ACTION);
		if (!this.actions.contains(IActionConstants.PRINT_PREVIEW_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.PRINT_PREVIEW_ACTION);
		if (!this.actions.contains(IActionConstants.PAGE_SETUP_ACTION))
			getLogger().error(
				MISSING_ACTION_WARNING + IActionConstants.PAGE_SETUP_ACTION);
	}

	private void updateSplash(ISplashScreen splash, String message) {
		getRuntime().getLogger(getClass()).info(message);
		if (!(splash instanceof IStatusNotifier))
			return;
		IStatusNotifier notifier = (IStatusNotifier) splash;
		notifier.displayMessage(message);
	}

	private ILogger getLogger() {
		return getRuntime().getLogger(getClass());
	}

	private IWorkspace getWorkspace() {
		return workspace;
	}

	// XXX: remove this camp CPRS 2003 PRINT HACK
	private static class SillyPrintWindowListener
		extends DesktopWindowAdapter implements IDesktopWindowListener {

		/* @see gov.va.med.hds.cd.prism.IDesktopWindowListener#windowOpened(gov.va.med.hds.cd.prism.DesktopWindowEvent) */
		public void windowOpened(DesktopWindowEvent e) {
			createPrintListeners(e.getWindow());
		}

		/* @see gov.va.med.hds.cd.prism.IDesktopWindowListener#windowClosing(gov.va.med.hds.cd.prism.DesktopWindowEvent) */
		public void windowClosing(DesktopWindowEvent e)
			throws DesktopWindowCloseVetoException {
			disposePrintListeners(e.getWindow());
		}

		private void createPrintListeners(IDesktopWindow window) {
			InitPrintActionEvent event =
				new InitPrintActionEvent(
					window,
					this,
					InitPrintActionEvent.ADD);
			Action action =
				window.getActionMap().get(IActionConstants.PRINT_ACTION);
			if (action != null)
				action.actionPerformed(event);

			action =
				window.getActionMap().get(
					IActionConstants.PRINT_PREVIEW_ACTION);
			if (action != null)
				action.actionPerformed(event);

			action =
				window.getActionMap().get(IActionConstants.PAGE_SETUP_ACTION);
			if (action != null)
				action.actionPerformed(event);
		}
		private void disposePrintListeners(IDesktopWindow window) {
			InitPrintActionEvent event =
				new InitPrintActionEvent(
					window,
					this,
					InitPrintActionEvent.REMOVE);
			Action action =
				window.getActionMap().get(IActionConstants.PRINT_ACTION);
			if (action != null)
				action.actionPerformed(event);

			action =
				window.getActionMap().get(
					IActionConstants.PRINT_PREVIEW_ACTION);
			if (action != null)
				action.actionPerformed(event);

			action =
				window.getActionMap().get(IActionConstants.PAGE_SETUP_ACTION);
			if (action != null)
				action.actionPerformed(event);
		}
	}
}
