package gov.va.med.hds.cd.prism.internal;

import java.util.Arrays;

import javax.swing.BoxLayout;
import javax.swing.border.EmptyBorder;

import gov.va.med.hds.cd.config.BlankFieldEditor;
import gov.va.med.hds.cd.config.ComboBoxFieldEditor;
import gov.va.med.hds.cd.config.ComboBoxFieldEditorException;
import gov.va.med.hds.cd.config.IPreferencePage;
import gov.va.med.hds.cd.config.NonExistentParameterException;
import gov.va.med.hds.cd.config.PreferencePage;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.PerspectiveDescriptorComparator;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class PrismPreferencePage extends PreferencePage implements IPreferencePage {

	private String name;
	private ComboBoxFieldEditor defaultPerspective;
	private IDesktop desktop;
	private IPerspectiveRegistry perspectiveRegistry;
	private IPerspectiveDescriptor[] perspectives;
	private String[] names;
	private String[] values;
	
	public PrismPreferencePage() {
			
		setBorder(new EmptyBorder(10, 10, 10, 10));

		try {
			defaultPerspective =
				new ComboBoxFieldEditor(
					"Default Perspective:",
					new EmptyBorder(0, 0, 0, 0));
		} catch (ComboBoxFieldEditorException e) {
			e.printStackTrace();
		}
		addFieldEditor(defaultPerspective);

		int padTo = 30 - fieldEditors.size();
		for (int i = 0; i < padTo; i++) {
			addFieldEditor(new BlankFieldEditor());
		}

		initGUI(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	/**
	 * @see gov.va.med.hds.cd.config.dialog.IPreferencePage#initializeFieldEditors()
	 */
	public void initializeFieldEditors() {
		desktop = PrismUtilities.getDesktop(runtime);
		perspectiveRegistry = desktop.getPerspectiveRegistry();
		perspectives = perspectiveRegistry.getPerspectiveDescriptors();
		Arrays.sort(perspectives, new PerspectiveDescriptorComparator());
		String[] names = new String[perspectives.length];
		String[] values = new String[perspectives.length];

		for (int i=0; i < perspectives.length; i++) {
			names[i] = perspectives[i].getName();
			values[i] = perspectives[i].getId();			
		}
		
		defaultPerspective.setComboBoxModel(names, values);
		
		try {
			defaultPerspective.setParameter(
				configuration.getParameter("Default Perspective"));
			defaultPerspective.loadInitialValue();
		} catch (NonExistentParameterException e) {
			e.printStackTrace();
		}

	}

	/**
	 * @see java.awt.Component#setName(String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @see java.awt.Component#getName()
	 */
	public String getName() {
		return name;
	}

}
