package gov.va.med.hds.cd.prism.internal;

import gov.va.med.hds.cd.prism.perspective.IPerspectiveEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveAdapter;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentListener;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentEventSourceSupport;

public class WindowVisualComponentEventSource extends VisualComponentEventSourceSupport {

	private IPerspectiveEventSource perspectiveEventSource;
	private IPerspectiveReference currentActivePerspective;

	public WindowVisualComponentEventSource(IPerspectiveEventSource perspectiveEventSource) {
		this.perspectiveEventSource = perspectiveEventSource;
		this.perspectiveEventSource.addPerspectiveListener(new PerspectiveListener());
		this.currentActivePerspective = perspectiveEventSource.getActivePerspective();
	}

	public IVisualComponentReference getActiveVisualComponent() {
		IPerspectiveReference p = perspectiveEventSource.getActivePerspective();
		if (p == null) return null;
		return p.getActiveVisualComponent();
	}

	public void addVisualComponentListener(IVisualComponentListener l) {
		super.addVisualComponentListener(l);
		IPerspectiveReference p = perspectiveEventSource.getActivePerspective();
		if (p != null) p.addVisualComponentListener(l);
	}

	public void removeVisualComponentListener(IVisualComponentListener l) {
		IPerspectiveReference p = perspectiveEventSource.getActivePerspective();
		if (p != null) p.removeVisualComponentListener(l);
		super.removeVisualComponentListener(l);
	}

	private void addListeners(IPerspectiveReference perspective) {
		IVisualComponentListener[] listeners = (IVisualComponentListener[]) listenerList.getListeners(IVisualComponentListener.class);
		for (int i = 0; i < listeners.length; i++) {
			perspective.addVisualComponentListener(listeners[i]);
		}
	}
	
	private void removeListeners(IPerspectiveReference perspective) {
		IVisualComponentListener[] listeners = (IVisualComponentListener[]) listenerList.getListeners(IVisualComponentListener.class);
		for (int i = 0; i < listeners.length; i++) {
			perspective.removeVisualComponentListener(listeners[i]);
		}
	}

	private class PerspectiveListener extends PerspectiveAdapter {
		
		public void perspectiveActivated(PerspectiveEvent e) {
			if (currentActivePerspective != null) {
				fireComponentDeactivated(currentActivePerspective.getActiveVisualComponent());
			}
			fireComponentActivated(e.getPerspective().getActiveVisualComponent());
			currentActivePerspective = e.getPerspective();
		}

		public void perspectiveClosed(PerspectiveEvent e) {
			IPerspectiveReference p = e.getPerspective();
			removeListeners(p);
			fireComponentDeactivated(p.getActiveVisualComponent());
			IVisualComponentReference[] components = p.getVisualComponents();
			for (int i = 0; i < components.length; i++) {
				fireComponentClosed(components[i]);
			}
		}

		public void perspectiveOpened(PerspectiveEvent e) {
			IPerspectiveReference p = e.getPerspective();
			addListeners(p);
			IVisualComponentReference[] components = p.getVisualComponents();
			for (int i = 0; i < components.length; i++) {
				fireComponentOpened(components[i]);
			}
		}
	}
}
