package gov.va.med.hds.cd.prism.menu.internal;

import java.util.Iterator;
import java.util.Stack;

import javax.swing.ActionMap;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

public class CreateMenuBarVisitor implements IMenuTreeNodeVisitor {

	private String contextId;
	private ActionMap actions;
	private JMenuBar menuBar;
	private Stack menuStack;

	public CreateMenuBarVisitor(String contextId, ActionMap actions) {
		this.contextId = contextId;
		this.actions = actions;
		this.menuBar = new JMenuBar();
		this.menuStack = new Stack();
	}

	public void visit(ActionNode node) {
		JMenu menu = (JMenu) menuStack.peek();
		JMenuItem item = new JMenuItem();
		item.setAction(actions.get(node.getId()));
		menu.add(item);

	}

	public void visit(MenuNode node) {
		JMenu newMenu = createMenu(node);
		if (menuStack.isEmpty()) {
			menuBar.add(newMenu);
		} else {
			JMenu menu = (JMenu) menuStack.peek();
			menu.add(newMenu);
		}

		menuStack.push(newMenu);
		for (Iterator i = node.getChildren().iterator(); i.hasNext();) {
			MenuTreeNode n = (MenuTreeNode) i.next();
			n.accept(this);
		}
		menuStack.pop();
	}

	public void visit(SlotNode node) {
		for (Iterator i = node.getChildren().iterator(); i.hasNext();) {
			MenuTreeNode n = (MenuTreeNode) i.next();
			n.accept(this);
		}
	}

	public void visit(SeparatorNode separatorNode) {
		JMenu menu = (JMenu) menuStack.peek();
		JSeparator separator = new JSeparator();
		menu.add(separator);
	}

	public void visit(ContextNode node) {
		if (contextId != null && contextId.equals(node.getContextId()))
			node.getNode().accept(this);
	}

	public JMenuBar getMenuBar() {
		return menuBar;
	}

	private JMenu createMenu(MenuNode node) {
		if (node.hasCustomMenuClass()) {
			try {
				return node.createMenu();
			} catch (Exception e) {
				return new JMenu("Error creating menu " + node.getId());
			}
		}

		JMenu menu = new JMenu(node.getName());
		if (node.getMnemonic() != null) {
			menu.setMnemonic(node.getMnemonic().charValue());
		}
		return menu;
	}
}
