package gov.va.med.hds.cd.prism.menu.internal;

import java.util.Iterator;

public class FindNodeVisitor implements IMenuTreeNodeVisitor {

	private final String startPath;
	private String path;
	private String contextId;
	private MenuTreeNode node;

	public FindNodeVisitor(String path, String contextId) throws InvalidMenuPathException {
		if (path.equals("")) throw new InvalidMenuPathException(path);
		if (path.endsWith(MenuTree.PATH_SEPARATOR))
			path = path.substring(0, path.length() - 1);
		if (!path.startsWith(MenuTree.PATH_SEPARATOR))
			path = MenuTree.PATH_SEPARATOR + path;
		this.startPath = path;
		this.path = path;
		this.contextId = contextId;
		this.node = null;
	}

	public void visit(MenuNode node) {
		visitNode(node);
		visitChildren(node);
	}

	public void visit(SlotNode node) {
		visitNode(node);
		visitChildren(node);
	}

	public void visit(ActionNode node) {
		visitNode(node);
	}

	public void visit(SeparatorNode node) {
		visitNode(node);
	}

	public void visit(ContextNode node) {
		if (contextId != null && contextId.equals(node.getContextId())) node.getNode().accept(this);
	}

	public MenuTreeNode getNode() throws InvalidMenuPathException {
		if (isDone()) {
			return node;
		} else {
			throw new InvalidMenuPathException(startPath);
		}
	}

	private void visitNode(MenuTreeNode node) {
		this.node = node;
		if (path.equals(node.getId())) {
			path = "";
		} else if (node.getId().equals(MenuTree.ROOT_ID)) {
			path = path.substring(1);
		} else if (path.startsWith(node.getId())) {
			path = path.substring(node.getId().length() + 1);
		}
	}

	private boolean isDone() {
		return path.equals("");
	}

	private void visitChildren(ParentNode node) {
		if (isDone())
			return;
		for (Iterator i = node.getChildren().iterator(); i.hasNext();) {
			((MenuTreeNode) i.next()).accept(this);
			if (isDone()) return;
		}
	}
}
