package gov.va.med.hds.cd.prism.menu.internal;

import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;

public class MenuTree implements Cloneable {

	static final String PATH_SEPARATOR = "/";
	static final String ROOT_ID = "root";
	static final String ROOT_PATH = PATH_SEPARATOR;

	private ParentNode root;
	private ActionRegistry actions;

	public MenuTree(ActionRegistry actions) {
		this.root = new SlotNode(ROOT_ID);
		setActions(actions);
	}

	public ActionRegistry getActions() {
		return actions;
	}

	public void setActions(ActionRegistry actions) {
		this.actions = actions;
	}

	public void addMenu(String menuId, String path, String name)
		throws InvalidMenuPathException {
		addMenu(null, menuId, path, name);
	}

	public void addMenu(String contextId, String menuId, String path, String name)
		throws InvalidMenuPathException {
		addMenu(contextId, menuId, path, name, null);
	}

	public void addMenu(
			String menuId,
			String path,
			String name,
			Character mnemonic)
			throws InvalidMenuPathException {
		addMenu(null, menuId, path, name, mnemonic);
	}

	public void addMenu(
		String contextId,
		String menuId,
		String path,
		String name,
		Character mnemonic)
		throws InvalidMenuPathException {
		MenuTreeNode n = getNode(path);
		if (!(n instanceof ParentNode))
			throw new InvalidMenuPathException("unable to add menu to specified path: ", path);
		ParentNode node = (ParentNode) n;
		addChild(contextId, node, new MenuNode(menuId, name, mnemonic));
	}

	public void addMenu(String menuId, String path, IConfigurationElement menuElement)
		throws InvalidMenuPathException {
		addMenu(null, menuId, path, menuElement);
	}

	public void addMenu(String contextId, String menuId, String path, IConfigurationElement menuElement)
		throws InvalidMenuPathException {
		addMenu(contextId, menuId, path, menuElement, null);
	}

	public void addMenu(String menuId, String path, IConfigurationElement menuElement, Character mnemonic)
		throws InvalidMenuPathException {
		addMenu(null, menuId, path, menuElement, mnemonic);
	}

	public void addMenu(
		String contextId,
		String menuId,
		String path,
		IConfigurationElement menuElement,
		Character mnemonic)
		throws InvalidMenuPathException {
		MenuTreeNode n = getNode(path);
		if (!(n instanceof ParentNode))
			throw new InvalidMenuPathException("unable to add menu to specified path: ", path);
		ParentNode node = (ParentNode) n;
		addChild(contextId, node, new MenuNode(menuId, menuElement, mnemonic));
	}

	public void addSlot(String slotId, String path) throws InvalidMenuPathException {
		addSlot(null, slotId, path);
	}

	public void addSlot(String contextId, String slotId, String path)
		throws InvalidMenuPathException {
		MenuTreeNode n = getNode(path, contextId);
		if (!(n instanceof ParentNode))
			throw new InvalidMenuPathException("unable to add slot to specified path: ", path);
		ParentNode node = (ParentNode) n;
		addChild(contextId, node, new SlotNode(slotId));
	}

	public void addAction(String actionId, String path)
		throws InvalidMenuPathException, ActionNotFoundException {
		addAction(null, actionId, path);
	}

	public void addAction(String contextId, String actionId, String path)
		throws InvalidMenuPathException, ActionNotFoundException {
		if (!actions.contains(actionId))
			throw new ActionNotFoundException(actionId);
		MenuTreeNode n = getNode(path, contextId);
		if (!(n instanceof ParentNode))
			throw new InvalidMenuPathException("unable to add action to specified path: ", path);
		ParentNode node = (ParentNode) n;
		addChild(contextId, node, new ActionNode(actionId));
	}

	public void addSeparator(String separatorId, String path) throws InvalidMenuPathException {
		addSeparator(null, separatorId, path);
	}

	public void addSeparator(String contextId, String separatorId, String path)
		throws InvalidMenuPathException {
		MenuTreeNode n = getNode(path, contextId);
		if (!(n instanceof ParentNode))
			throw new InvalidMenuPathException("unable to add separator to specified path: ", path);
		ParentNode node = (ParentNode) n;
		addChild(contextId, node, new SeparatorNode(separatorId));
	}

	public MenuTreeNode getNode(String path) throws InvalidMenuPathException {
		return getNode(path, null);
	}

	public MenuTreeNode getNode(String path, String contextId) throws InvalidMenuPathException {
		FindNodeVisitor findNodeVisitor = new FindNodeVisitor(path, contextId);
		root.accept(findNodeVisitor);
		return findNodeVisitor.getNode();
	}

	public ParentNode getRoot() {
		return root;
	}

	public Object clone() {
		MenuTree menuTree = new MenuTree(actions);
		menuTree.root = (ParentNode) this.root.clone();
		return menuTree;
	}

	private void addChild(String contextId, ParentNode parent, MenuTreeNode node) {
		if (contextId == null || contextId.length() == 0)
			parent.addChild(node);
		else
			parent.addChild(new ContextNode(node, contextId));
	}
}
