package gov.va.med.hds.cd.prism.perspective.internal;

import java.util.EventListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveDescriptorComparator;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;

public class ActivePerspectiveModel implements IActivePerspectiveModel {
	
	protected EventListenerList listeners = new EventListenerList();
	protected SortedSet descriptorsByName = new TreeSet(new PerspectiveDescriptorComparator());
	protected Map descriptorsById = new HashMap();
	protected IPerspectiveDescriptor activePerspective = null;
	
	public void addPerspectiveListener(IPerspectiveListener p) {
		listeners.add(IPerspectiveListener.class, p);
	}
	
	public void removePerspectiveListener(IPerspectiveListener p) {
		listeners.remove(IPerspectiveListener.class, p);
	}
	
	public void add(IPerspectiveDescriptor p) {
		if (descriptorsById.containsKey(p.getId())) return;
		descriptorsByName.add(p);
		descriptorsById.put(p.getId(), p);
		firePerspectiveOpened(p);
	}
	
	public void remove(IPerspectiveDescriptor p) {
		if (descriptorsById.containsKey(p.getId())) {
			if (activePerspective != null && activePerspective.equals(p)) {
				if (descriptorsByName.size() == 1) {
					activePerspective = null;
					firePerspectiveActivated();
				}
			}
			descriptorsByName.remove(p);
			descriptorsById.remove(p.getId());
			firePerspectiveClosed(p);
		}
	}
	
	public IPerspectiveDescriptor getActivePerspective() {
		return activePerspective;
	}
	
	public void setActivePerspective(IPerspectiveDescriptor p) {
		if (p != null && !descriptorsByName.contains(p)) add(p);
		if (p != activePerspective) {
			activePerspective = p;
			firePerspectiveActivated();
		}
	}
	
	public void setActivePerspective(String id) {
		if (!descriptorsById.containsKey(id)) return;
		activePerspective = (IPerspectiveDescriptor) descriptorsById.get(id);
		firePerspectiveActivated();
	}
	
	public int size() {
		return descriptorsByName.size();
	}

	public Iterator iterator() {
		return descriptorsByName.iterator();
	}
	
	protected void firePerspectiveActivated() {
		final PerspectiveEvent event = new PerspectiveEvent(this, new DescriptorOnlyPerspective(getActivePerspective()), null);
		
		EventListener[] lstnrs = listeners.getListeners(IPerspectiveListener.class);
		for (int i = 0; i < lstnrs.length; i++) {
			IPerspectiveListener pl = (IPerspectiveListener) lstnrs[i];
			pl.perspectiveActivated(event);
		}
	}
	
	protected void firePerspectiveOpened(IPerspectiveDescriptor p) {
		final PerspectiveEvent event = new PerspectiveEvent(this, new DescriptorOnlyPerspective(p), null);
		
		EventListener[] lstnrs = listeners.getListeners(IPerspectiveListener.class);
		for (int i = 0; i < lstnrs.length; i++) {
			IPerspectiveListener pl = (IPerspectiveListener) lstnrs[i];
			pl.perspectiveOpened(event);
		}
	}
	
	protected void firePerspectiveClosed(IPerspectiveDescriptor p) {
		final PerspectiveEvent event = new PerspectiveEvent(this, new DescriptorOnlyPerspective(p), null);
		
		EventListener[] lstnrs = listeners.getListeners(IPerspectiveListener.class);
		for (int i = 0; i < lstnrs.length; i++) {
			IPerspectiveListener pl = (IPerspectiveListener) lstnrs[i];
			pl.perspectiveClosed(event);
		}
	}
	
	public IPerspectiveDescriptor[] getDescriptors() {
		return (IPerspectiveDescriptor[]) descriptorsByName.toArray(new IPerspectiveDescriptor[descriptorsByName.size()]);
	}

}
