package gov.va.med.hds.cd.prism.perspective.internal;

import java.awt.Component;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;

import gov.va.med.hds.cd.prism.perspective.ILayoutNode;
import gov.va.med.hds.cd.spectrum.MultiSplitPane;
import gov.va.med.hds.cd.spectrum.ScalarSpring;

public class Box extends JPanel {

	private BoxNode boxNode;
	private boolean firstLayout = true;

	public Box(BoxNode boxNode) {
		super();
		this.boxNode = boxNode;
		this.setLayout(new SpringLayout());
		//		box.setBorder(BorderFactory.createLineBorder(Color.BLUE));
	}

	public void doLayout() {
		super.doLayout();
		if (firstLayout) {
			setupSpringLayout();
			firstLayout = false;
			doLayout();
		}
	}

	private void setupSpringLayout() {
		SpringLayout layout = (SpringLayout) getLayout();
		SpringLayout.Constraints boxConstraints = layout.getConstraints(Box.this);
		
		ILayoutNode[][] children = boxNode.split();
		if (children.length != getComponentCount()) throw new RuntimeException();
		
//		ILayoutNode[] childNodes = boxNode.getChildren();
		for (int i = 0; i < getComponentCount(); ++i) {
			Component component = getComponent(i);
			SpringLayout.Constraints constraints = layout.getConstraints(component);
			
			if (i == 0) {
				anchorToContainer(boxConstraints, constraints);
			} else {
				anchorToPrecedingComponent(layout, i, constraints);
			}
			if (boxNode.getAlign() == ILayoutNode.STRETCH) {
				if (boxNode.getOrientation() == SwingConstants.VERTICAL) {
					constraints.setWidth(boxConstraints.getWidth());
				} else {
					constraints.setHeight(boxConstraints.getHeight());
				}
			}
			
			if (component instanceof MultiSplitPane || component instanceof JSplitPane) {
				if (ContainerNode.getFlexTotal(children[i]) != 0) {
					if (boxNode.getOrientation() == SwingConstants.VERTICAL) {
						constraints.setHeight(
							new ScalarSpring(
								(double) ContainerNode.getFlexTotal(children[i])
									/ (double) boxNode.getFlexTotal(),
								boxConstraints.getHeight()));
					} else {
						constraints.setWidth(
							new ScalarSpring(
								(double) ContainerNode.getFlexTotal(children[i])
									/ (double) boxNode.getFlexTotal(),
								boxConstraints.getWidth()));
					}
				}
			} else {
				if (children[i][0].getFlex() != 0) {
					if (boxNode.getOrientation() == SwingConstants.VERTICAL) {
						constraints.setHeight(
							new ScalarSpring(
								(double) children[i][0].getFlex()
									/ (double) boxNode.getFlexTotal(),
								boxConstraints.getHeight()));
					} else {
						constraints.setWidth(
							new ScalarSpring(
								(double) children[i][0].getFlex()
									/ (double) boxNode.getFlexTotal(),
								boxConstraints.getWidth()));
					}
				}
			}
		}
	}

	private void anchorToPrecedingComponent(
		SpringLayout layout,
		int i,
		SpringLayout.Constraints constraints) {
		SpringLayout.Constraints precedingConstraints = layout.getConstraints(getComponent(i - 1));
		if (boxNode.getOrientation() == SwingConstants.VERTICAL) {
			constraints.setY(precedingConstraints.getConstraint(SpringLayout.SOUTH));
		} else {
			constraints.setX(precedingConstraints.getConstraint(SpringLayout.EAST));
		}
	}

	private void anchorToContainer(
		SpringLayout.Constraints boxConstraints,
		SpringLayout.Constraints constraints) {
		if (boxNode.getOrientation() == SwingConstants.VERTICAL) {
			constraints.setY(boxConstraints.getConstraint(SpringLayout.NORTH));
		} else {
			constraints.setX(boxConstraints.getConstraint(SpringLayout.WEST));
		}
	}
}
