package gov.va.med.hds.cd.prism.perspective.internal;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import gov.va.med.hds.cd.prism.perspective.ILayoutNode;

public class BoxNode extends ContainerNode {
    
    private int orientation;
    private int align = STRETCH;

    public BoxNode() {
        this(HORIZONTAL);
    }

    public BoxNode(int orientation) {
        setOrientation(orientation);
    }

    public int getAlign() {
        return align;
    }

    public int getOrientation() {
        return orientation;
    }

    public void setAlign(int align) {
        if ((align != START) && (align != CENTER) && (align != END) && (align != BASELINE) &&
                (align != STRETCH)) {
            throw new IllegalArgumentException();
        }

        this.align = align;
    }

    public void setOrientation(int orientation) {
        if ((orientation != HORIZONTAL) && (orientation != VERTICAL)) {
            throw new IllegalArgumentException();
        }

        this.orientation = orientation;
    }
    
    public ILayoutNode[][] split() {
		List groups = new ArrayList();
		SplitterGroup currentGroup = null;
		ILayoutNode[] children = getChildren();
		for (int i = 0; i < children.length; i++) {
			if (currentGroup == null) {
				if (!(children[i] instanceof SplitterNode))
					currentGroup = new SplitterGroup(children[i]);
			} else {
				if (!(children[i] instanceof SplitterNode)) {
					if ((children[i - 1] instanceof SplitterNode)) {
						currentGroup.add(children[i]);
					} else {
						groups.add(currentGroup);
						currentGroup = new SplitterGroup(children[i]);
					}
				}
			}
		}
		groups.add(currentGroup);
		
		ILayoutNode[][] returnVal = new ILayoutNode[groups.size()][];
		for (int i = 0; i < returnVal.length; i++) {
			returnVal[i] = ((SplitterGroup) groups.get(i)).getNodes();
		}
		return returnVal;
	}
	
	private static class SplitterGroup {

		private List nodes = new LinkedList();
		public SplitterGroup(ILayoutNode firstNode) {
			add(firstNode);
		}

		public void add(ILayoutNode node) {
			if (node instanceof SplitterNode)
				throw new IllegalArgumentException();
			nodes.add(node);
		}

		public ILayoutNode getFirstNode() {
			return (ILayoutNode) nodes.get(0);
		}

		public ILayoutNode[] getNodes() {
			return (ILayoutNode[]) nodes.toArray(new ILayoutNode[nodes.size()]);
		}

		public int size() {
			return nodes.size();
		}

	}
}
