package gov.va.med.hds.cd.prism.perspective.internal;

import javax.swing.SwingConstants;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.InvalidConfigurationElementException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;
import gov.va.med.hds.util.ExceptionStackTrace;

public class LayoutReader implements IPrismConstants {

	private ILogger log;

	public LayoutReader(ILoggingCapable logging) {
		log = logging.getLogger(getClass());
	}

	public PerspectiveLayout readLayout(IConfigurationElement perspectiveElement) {
		PerspectiveLayout layout = new PerspectiveLayout();
		IConfigurationElement[] nodes = perspectiveElement.getChildren();
		for (int i = 0; i < nodes.length; i++) {
			try {
				LayoutNode node = readNodeElement(nodes[i]);
				layout.add(node);
			} catch (InvalidConfigurationElementException e) {
				log.error("error reading perspective layout",e);
				continue;
			}
		}
		return layout;
	}

	private LayoutNode readNodeElement(IConfigurationElement nodeElement)
		throws InvalidConfigurationElementException {
		if (BOX_CONFIGURATION_ELEMENT.equals(nodeElement.getName())
			|| VERTICAL_BOX_CONFIGURATION_ELEMENT.equals(nodeElement.getName())
			|| HORIZONTAL_BOX_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			return readBoxElement(nodeElement);
		} else if (COMPONENT_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			return readComponentElement(nodeElement);
		} else if (SPACER_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			return readSpacerElement(nodeElement);
		} else if (SPLITTER_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			return readSplitterElement(nodeElement);
		} else {
			throw new InvalidConfigurationElementException(
				"unexpected element \"" + nodeElement.getName() + "\"");
		}
	}

	private LayoutNode readBoxElement(IConfigurationElement nodeElement) throws InvalidConfigurationElementException{
		BoxNode box = createBoxNode(nodeElement);
		readNodeAttributes(box, nodeElement);
		IConfigurationElement[] children = nodeElement.getChildren();
		for (int i = 0; i < children.length; i++) {
			try {
				LayoutNode node = readNodeElement(children[i]);
				box.add(node);
			} catch (InvalidConfigurationElementException e) {
				log.warning(ExceptionStackTrace.getStackTrace(e));
				continue;
			}
		}
		return box;
	}

	private BoxNode createBoxNode(IConfigurationElement nodeElement) {
		int orientation = SwingConstants.HORIZONTAL;
		if (BOX_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			if (nodeElement.getAttribute(ORIENTATION_ATTRIBUTE).equals("vertical"))
				orientation = SwingConstants.VERTICAL;
		} else if (VERTICAL_BOX_CONFIGURATION_ELEMENT.equals(nodeElement.getName())) {
			orientation = SwingConstants.VERTICAL;
		}
		return new BoxNode(orientation);
	}

	private LayoutNode readComponentElement(IConfigurationElement nodeElement) throws InvalidConfigurationElementException {
		ComponentNode component = new ComponentNode(nodeElement.getAttribute(ID_ATTRIBUTE));
		readNodeAttributes(component, nodeElement);
		return component;
	}

	private LayoutNode readSpacerElement(IConfigurationElement nodeElement) {
		SpacerNode spacer = new SpacerNode();
		readNodeAttributes(spacer, nodeElement);
		return spacer;
	}

	private LayoutNode readSplitterElement(IConfigurationElement nodeElement) {
		return new SplitterNode();
	}

	private void readNodeAttributes(LayoutNode node, IConfigurationElement nodeElement) {
		if (nodeElement.getAttribute(FLEX_ATTRIBUTE) != null) node.setFlex(Integer.parseInt(nodeElement.getAttribute(FLEX_ATTRIBUTE)));
		if (nodeElement.getAttribute(HEIGHT_ATTRIBUTE) != null) node.setHeight(Integer.parseInt(nodeElement.getAttribute(HEIGHT_ATTRIBUTE)));
		if (nodeElement.getAttribute(WIDTH_ATTRIBUTE) != null) node.setWidth(Integer.parseInt(nodeElement.getAttribute(WIDTH_ATTRIBUTE)));
		if (nodeElement.getAttribute(MIN_HEIGHT_ATTRIBUTE) != null) node.setMinHeight(Integer.parseInt(nodeElement.getAttribute(MIN_HEIGHT_ATTRIBUTE)));
		if (nodeElement.getAttribute(MAX_HEIGHT_ATTRIBUTE) != null) node.setMaxHeight(Integer.parseInt(nodeElement.getAttribute(MAX_HEIGHT_ATTRIBUTE)));
		if (nodeElement.getAttribute(MIN_WIDTH_ATTRIBUTE) != null) node.setMinWidth(Integer.parseInt(nodeElement.getAttribute(MIN_WIDTH_ATTRIBUTE)));
		if (nodeElement.getAttribute(MAX_WIDTH_ATTRIBUTE) != null) node.setMaxWidth(Integer.parseInt(nodeElement.getAttribute(MAX_WIDTH_ATTRIBUTE)));
	}

}
