package gov.va.med.hds.cd.prism.perspective.internal;

import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;

public class PerspectiveManager extends PerspectiveEventSourceSupport {

	private Map perspectives = new HashMap();
	private IPerspectiveReference activePerspective;
	private IDesktopWindow window;
	private IPerspectiveFactory factory;

	public PerspectiveManager(IPerspectiveFactory factory, IDesktopWindow w) {
		this.factory = factory;
		this.window = w;	
	}

	public IPerspectiveReference open(IPerspectiveDescriptor descriptor) {
		if (contains(descriptor)) return (IPerspectiveReference) perspectives.get(descriptor.getId());
		firePerspectiveOpening(descriptor, window);
		IPerspectiveReference p = factory.createPerspective(descriptor);
		open(p);
		return p;
	}

	private void open(IPerspectiveReference p) {
		if (contains(p)) return;
		perspectives.put(p.getDescriptor().getId(), p);
		firePerspectiveOpened(p, window);
		setActivePerspective(p);
	}

	public void close(IPerspectiveReference p) {
		if (!contains(p)) return;
		firePerspectiveClosing(p, window);
		perspectives.remove(p.getDescriptor().getId());
		if (getActivePerspective().equals(p)) {
			if (getPerspectiveCount() > 0) {
				setActivePerspective((IPerspectiveReference) perspectives.values().iterator().next());
			} else {
				setActivePerspective(null);
			}
		}
		firePerspectiveClosed(p, window);
	}

	public void setActivePerspective(IPerspectiveReference p) {
		if (getActivePerspective() != null && getActivePerspective().equals(p)) return;
		if (p != null && !isPerspectiveOpen(p)) open(p);
		this.activePerspective = p;
		if (getActivePerspective() != null) firePerspectiveActivated(activePerspective, window);
	}

	public IPerspectiveReference getActivePerspective() {
		return activePerspective;
	}

	public boolean isPerspectiveOpen(IPerspectiveReference perspective) {
		return contains(perspective);
	}

    public IPerspectiveReference[] getPerspectives() {
       return (IPerspectiveReference[]) perspectives.values().toArray(new IPerspectiveReference[perspectives.size()]);
    }

	public int getPerspectiveCount() {
		return perspectives.size();
	}

	private boolean contains(IPerspectiveReference p) {
		return contains(p.getDescriptor());
	}
	
	private boolean contains(IPerspectiveDescriptor d) {
		return perspectives.containsKey(d.getId());
	}
}
