package gov.va.med.hds.cd.prism.perspective.internal;

import java.awt.CardLayout;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveAdapter;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentRegistry;

public class PerspectivePanel extends JPanel implements IPerspectiveEventSource {

	private static final String NONE = "null";

	private Map openPerspectives;
	private Map openingPerspectives;
	private CardLayout cardLayout;
	private NoPerspectivesPane none;
	private String activePerspectiveId;
	private PerspectiveManager perspectiveManager;

	public PerspectivePanel(IDesktopWindow window, IActivePerspectiveModel model) {		
		this.openPerspectives = new HashMap();
		this.openingPerspectives = new HashMap();
		this.cardLayout = new CardLayout();
		this.setLayout(cardLayout);
		IVisualComponentRegistry components = window.getDesktop().getVisualComponentRegistry();
		PerspectiveFactory factory = new PerspectiveFactory(components);
		this.perspectiveManager = new PerspectiveManager(factory, window);
		if (model.size() == 0) {
			none = new NoPerspectivesPane();
			this.add(none, NONE);
		} else {
			setActivePerspective(model.getActivePerspective());
		}
		model.addPerspectiveListener(new PerspectiveModelListener());
	}

	public void addPerspectiveListener(IPerspectiveListener l) {
		perspectiveManager.addPerspectiveListener(l);
	}

	public void removePerspectiveListener(IPerspectiveListener l) {
		perspectiveManager.removePerspectiveListener(l);
	}

	public IPerspectiveReference getActivePerspective() {
		return perspectiveManager.getActivePerspective();
	}

	public IPerspectiveReference[] getPerspectives() {
		return perspectiveManager.getPerspectives();
	}

	private void setActivePerspective(IPerspectiveDescriptor p) {
		if (!isPerspectiveOpen(p)) {
			openPerspective(p);
		}
		this.activePerspectiveId = p.getId();
		cardLayout.show(this, this.activePerspectiveId);
		perspectiveManager.setActivePerspective((IPerspectiveReference) openPerspectives.get(activePerspectiveId));
	}

	private void openPerspective(IPerspectiveDescriptor descriptor) {
		IPerspectiveReference p = perspectiveManager.open(descriptor);
		add(p.getComponent(), descriptor.getId());
		openPerspectives.put(descriptor.getId(), p);
	}

	private void closePerspective(IPerspectiveDescriptor descriptor) {
		IPerspectiveReference p = (IPerspectiveReference) openPerspectives.get(descriptor.getId());
		this.remove(p.getComponent());
		perspectiveManager.close(p);
	}

	private boolean isPerspectiveOpen(String id) {
		return openPerspectives.containsKey(id);
	}

	private boolean isPerspectiveOpen(IPerspectiveDescriptor p) {
		return isPerspectiveOpen(p.getId());
	}
	
	private class PerspectiveModelListener extends PerspectiveAdapter {

		public void perspectiveActivated(PerspectiveEvent e) {
			setActivePerspective(e.getPerspective().getDescriptor());
		}

		public void perspectiveClosed(PerspectiveEvent e) {
			if (!isPerspectiveOpen(e.getPerspective().getDescriptor())) return;
			closePerspective(e.getPerspective().getDescriptor());
		}
	}
	
	private static class NoPerspectivesPane extends JLabel {
		
		private static final String NO_PERSPECTIVES_MESSAGE = "No perspectives configured for this user.";
		
		public NoPerspectivesPane() {
			super(NO_PERSPECTIVES_MESSAGE, UIManager.getIcon("OptionPane.warningIcon"), SwingConstants.CENTER);
		}
	}
	
	private static class PerspectiveLoadingPane extends JLabel {
		public PerspectiveLoadingPane(IPerspectiveDescriptor descriptor) {
			super();
			Object[] args = new Object[] { descriptor.getName(), descriptor.getId() };
			setText(MessageFormat.format("Loading {0} perspective ({1})", args));
		}
				
	}
}
