package gov.va.med.hds.cd.prism.perspective.internal;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveLayout;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.InvalidConfigurationElementException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;

public class PerspectiveRegistryReader implements IPrismConstants {
    private static final String PERSPECTIVE_CREATION_WARNING = "unable to create perspective definition from configuration element:\n";
    private ILogger log;
    private LayoutReader layoutReader;

	public PerspectiveRegistryReader(ILoggingCapable logging) {
		layoutReader = new LayoutReader(logging);
		log = logging.getLogger(getClass());
	}

    public PerspectiveRegistry readPerspectiveRegistry(IPluginRegistry pluginRegistry) {
        PerspectiveRegistry result = new PerspectiveRegistry();
        readPerspectiveDescriptors(result, pluginRegistry);
        return result;
    }

    private void readPerspectiveDescriptors(PerspectiveRegistry result,
                                            IPluginRegistry pluginRegistry) {
        IConfigurationElement[] elements = pluginRegistry.getConfigurationElementsFor(UNIQUE_PERSPECTIVE_EXTENSION_POINT_ID);
        for (int i = 0; i < elements.length; i++) {
            try {
                PerspectiveDescriptor perspective = readPerspectiveDescriptor(elements[i]);
                result.add(perspective);
            } catch (Exception e) {
                log.error(PERSPECTIVE_CREATION_WARNING);
                continue;
            }
        }
    }

    private PerspectiveDescriptor readPerspectiveDescriptor(IConfigurationElement element)
        throws InvalidConfigurationElementException, ClassNotFoundException {
        validatePerspectiveDescriptorElement(element);

        String id = element.getAttribute(ID_ATTRIBUTE);
        String name = element.getAttribute(NAME_ATTRIBUTE);
        IPerspectiveLayout layout = layoutReader.readLayout(element);
        return new PerspectiveDescriptor(id, name, layout);
    }

    private void validatePerspectiveDescriptorElement(IConfigurationElement element)
        throws InvalidConfigurationElementException {
        if (element.getAttribute(ID_ATTRIBUTE) == null) {
            throw new InvalidConfigurationElementException("expected \"" +
                                                           ID_ATTRIBUTE +
                                                           "\" attribute in \"" +
                                                           PERSPECTIVE_CONFIGURATION_ELEMENT +
                                                           "\" configuration element for extension point \"" +
                                                           UNIQUE_PERSPECTIVE_EXTENSION_POINT_ID +
                                                           "\"");
        }

        if (element.getAttribute(NAME_ATTRIBUTE) == null) {
            throw new InvalidConfigurationElementException("expected \"" +
                                                           NAME_ATTRIBUTE +
                                                           "\" attribute in \"" +
                                                           PERSPECTIVE_CONFIGURATION_ELEMENT +
                                                           "\" configuration element for extension point \"" +
                                                           UNIQUE_PERSPECTIVE_EXTENSION_POINT_ID +
                                                           "\"");
        }
    }
}
