package gov.va.med.hds.cd.prism.perspective.internal;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Rectangle2D;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveAdapter;
import gov.va.med.hds.cd.prism.perspective.PerspectiveDescriptorListCellRenderer;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;

public class PerspectiveSelector extends JPanel implements IPerspectiveSelector {
	private static final int SPACER = 42;
	private IActivePerspectiveModel perspectiveModel;
	private IPerspectiveListener perspectiveModelListener;
	private DefaultListModel listModel;
	private JList list;
	private int rolloverIndex = -1;
	private boolean updatingSelection = false;

	public PerspectiveSelector() {
		super();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.perspectiveModelListener = new PerspectiveModelListener();
	}

	public IActivePerspectiveModel getModel() {
		return perspectiveModel;
	}

	public void setModel(IActivePerspectiveModel model) {
		if (this.perspectiveModel != null) {
			this.perspectiveModel.removePerspectiveListener(perspectiveModelListener);
		}		
		if (this.getComponentCount() > 0) {
			this.removeAll();
		}
		
		this.perspectiveModel = model;
		this.perspectiveModel.addPerspectiveListener(perspectiveModelListener);
		
		this.listModel = new DefaultListModel();

		IPerspectiveDescriptor[] descriptors = model.getDescriptors();
		for (int i = 0; i < descriptors.length; i++) {
			listModel.addElement(descriptors[i]);
		}

		add(Box.createVerticalStrut(SPACER));
		createJList();
		this.add(list);
	}

	private void createJList() {
		list = new JList(this.listModel);
		list.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.setSelectedValue(this.perspectiveModel.getActivePerspective(), true);
		list.addListSelectionListener(new SwingSelectionListener());
		list.setCellRenderer(new PerspectiveSelectorCellRenderer());
		list.setOpaque(false);
		list.setFont(list.getFont().deriveFont(Font.BOLD));
		Dimension maxSize = list.getMaximumSize();
		maxSize.width = Short.MAX_VALUE;
		list.setMaximumSize(maxSize);
		this.list.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				int index = list.locationToIndex(e.getPoint());
				setRolloverIndex(index);
			}
		});
		this.list.addMouseListener(new MouseAdapter() {
			public void mouseExited(MouseEvent e) {
				setRolloverIndex(-1);
			}
		});
	}

	private int getRolloverIndex() {
		return rolloverIndex;
	}

	private void setRolloverIndex(int rolloverIndex) {
		if (this.rolloverIndex != rolloverIndex) {
			this.rolloverIndex = rolloverIndex;
			repaint();
		};
	}

	protected void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		GradientPaint gp =
			new GradientPaint(
				(float) getWidth() / 2f,
				0,
				getTopGradientColor(),
				(float) getWidth() / 2f,
				getGradientHeight(),
				getBottomGradientColor());
		g2.setPaint(gp);

		Rectangle2D rect = new Rectangle2D.Double(0, 0, getWidth(), getHeight());
		g2.fill(rect);
		paintChildren(g);
	}

	private float getGradientHeight() {
		return SPACER + 10;
	}

	private Color getTopGradientColor() {
		return DesktopLookAndFeel.getPrimaryControlDarkShadow();
	}

	private Color getBottomGradientColor() {
		return DesktopLookAndFeel.getPrimaryControl();
	}

	private class PerspectiveModelListener extends PerspectiveAdapter {
		public void perspectiveActivated(PerspectiveEvent e) {
			list.setSelectedValue(perspectiveModel.getActivePerspective(), true);
		}
	}

	private class SwingSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent e) {
			if (updatingSelection) return;
			if (e.getValueIsAdjusting()) return;
			JList list = (JList) e.getSource();
			perspectiveModel.setActivePerspective(
			(PerspectiveDescriptor) listModel.get(list.getSelectedIndex()));
		}
	}

	public class PerspectiveSelectorCellRenderer extends PerspectiveDescriptorListCellRenderer {

        private static final int MARGIN = 3;

		protected Border noneFocusBorder;
		protected Border rolloverBorder;
		protected Border rolloverBorderWithSeparator;
		protected Border selectedBorder;
		protected Border selectedBorderWithSeparator;
		protected Border straightBorder;
		protected Border straightBorderWithSeparator;
        
        boolean hasFocus = false;

		public PerspectiveSelectorCellRenderer() {
			super();
            noneFocusBorder = BorderFactory.createEmptyBorder(
                MARGIN, MARGIN, MARGIN, MARGIN);

			rolloverBorder = createRolloverBorder();
			rolloverBorderWithSeparator = createSeparatorBorder(rolloverBorder);

			selectedBorder = createSelectedBorder();
			selectedBorderWithSeparator = createSeparatorBorder(selectedBorder);
            
            Insets i = selectedBorder.getBorderInsets(this);
            straightBorder = BorderFactory.createEmptyBorder(i.top+1, i.left+1, i.bottom-1, i.right-1);
            straightBorderWithSeparator = createSeparatorBorder(straightBorder);
		}

		private Border createSeparatorBorder(Border innerBorder) {
			return BorderFactory.createCompoundBorder(new SeparatorBorder(), innerBorder);
		}

		private Border createRolloverBorder() {
			Border bevel =
				BorderFactory.createBevelBorder(
					BevelBorder.RAISED,
					getBottomGradientColor().brighter(),
					getBottomGradientColor().darker());
			return BorderFactory.createCompoundBorder(bevel, noneFocusBorder);
		}

		private Border createSelectedBorder() {
			Border bevel =
				BorderFactory.createBevelBorder(
					BevelBorder.LOWERED,
					getBottomGradientColor().brighter(),
					getBottomGradientColor().darker());
			return BorderFactory.createCompoundBorder(bevel, noneFocusBorder);
		}

		public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {
			JLabel l = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			l.setComponentOrientation(list.getComponentOrientation());
            hasFocus = cellHasFocus;

			if (isSelected) {
				l.setOpaque(true);
				l.setBackground(list.getSelectionBackground());
				l.setForeground(list.getSelectionForeground());
			} else {
				l.setOpaque(false);
				l.setForeground(list.getForeground());
			}

			l.setBorder(getBorder(isSelected, index));
			l.setEnabled(list.isEnabled());
			return l;
		}
        
        public void paint(Graphics g) {
            super.paint(g);
            
            if (hasFocus) {
                Border border = getBorder();
                Insets i = border.getBorderInsets(this);
                g.setColor(getForeground());
                BasicGraphicsUtils.drawDashedRect(
                    g,
                    i.left - 2,
                    i.top - 2,
                    getWidth() - i.left - i.right + 4,
                    getHeight() - i.top - i.bottom + 4);
            }
        }

		private Border getBorder(boolean isSelected, int index) {
			if (isSelected) {
				if (index == 0) {
					return selectedBorder;
				} else {
					return selectedBorderWithSeparator;
				}
			} else {
				if (getRolloverIndex() != -1 && getRolloverIndex() == index) {
					if (index == 0) {
						return rolloverBorder;
					} else {
						return rolloverBorderWithSeparator;
					}
				} else {
					if (index == 0) {
						return straightBorder;
					} else {
						return straightBorderWithSeparator;
					}
				}
			}
		}

		private class SeparatorBorder implements Border {

			private static final int HEIGHT = 3;

			private Insets insets = new Insets(HEIGHT, 0, 0, 0);

			public Insets getBorderInsets(Component c) {
				return insets;
			}

			public boolean isBorderOpaque() {
				return true;
			}

			public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
				Graphics2D g2 = (Graphics2D) g;

				GradientPaint gp =
					new GradientPaint(
				getMargin(),
						1,
						getBottomGradientColor(),
						width - getMargin(),
						1,
						getTopGradientColor());
				g2.setPaint(gp);
				g2.fillRect(getMargin(), 0, width - getMargin(), HEIGHT);

				g2.setColor(getBottomGradientColor());
				g2.fillRect(0, 0, getMargin(), HEIGHT);
				g2.fillRect(width - getMargin(), 0, getMargin(), HEIGHT);

				g2.setColor(getBottomGradientColor().brighter());
				g2.drawLine(getMargin(), HEIGHT, width - getMargin(), HEIGHT);
				g2.drawLine(width - getMargin(), 0, width - getMargin(), HEIGHT);
			}
			
			private int getMargin() {
				return MARGIN * 2;
			}
		}
	}
}
