package gov.va.med.hds.cd.prism.visualcomponent.internal;

import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.workspace.ListenerOfEventDescriptor;
import gov.va.med.hds.cd.workspace.SourceOfEventDescriptor;

public class VisualComponentDescriptor implements IVisualComponentDescriptor, IPrismConstants {

	private IConfigurationElement element;
	private Map sourceOfEvents = new HashMap();
	private Map listenerOfEvents = new HashMap();

	public VisualComponentDescriptor(IConfigurationElement element) {
		this.element = element;
	}

	public String getId() {
		return element.getAttribute(ID_ATTRIBUTE);
	}

	public String getName() {
		return element.getAttribute(NAME_ATTRIBUTE);
	}

	public IVisualComponent createVisualComponent() {
		try {
			return (IVisualComponent) element.createExecutableExtension(CLASS_ATTRIBUTE, this);
		} catch (Exception e) {
			return new VisualComponentErrorPlaceholder(this, e);
		}
	}

	public void addEventSourceDescriptor(String eventClassName, String scope) {
		sourceOfEvents.put(eventClassName, new SourceOfEventDescriptor(eventClassName, scope));
	}

	public void removeEventSourceDescriptor(String eventClassName) {
		sourceOfEvents.remove(eventClassName);
	}

	public void addEventListenerDescriptor(String eventClassName, String scope) {
		listenerOfEvents.put(eventClassName, new ListenerOfEventDescriptor(eventClassName, scope));
	}

	public void removeEventListenerDescriptor(String eventClassName) {
		listenerOfEvents.remove(eventClassName);
	}

	public SourceOfEventDescriptor[] getSourceOfEventDescriptors() {
		return (SourceOfEventDescriptor[]) sourceOfEvents.values().toArray(new SourceOfEventDescriptor[sourceOfEvents.size()]);
	}

	public ListenerOfEventDescriptor[] getListenerOfEventDescriptors() {
		return (ListenerOfEventDescriptor[]) listenerOfEvents.values().toArray(new ListenerOfEventDescriptor[listenerOfEvents.size()]);
	}
}
