package gov.va.med.hds.cd.prism.itest;

import javax.swing.JPanel;
import javax.swing.JSplitPane;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.JFCTestHelper;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionRegistry;
import gov.va.med.hds.cd.prism.action.internal.ActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.prism.action.internal.ExitAction;
import gov.va.med.hds.cd.prism.internal.DesktopWindow;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.perspective.internal.ComponentNode;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveLayout;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveSelector;
import gov.va.med.hds.cd.prism.test.MockClinicianDesktop;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentRegistry;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistry;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;

public class DesktopWindowTest extends JFCTestCase implements IPerspectiveListener {

	private MockClinicianDesktop desktop = new MockClinicianDesktop();
	private IPerspectiveDescriptor descriptor1;
	private IPerspectiveDescriptor descriptor2;
	private boolean activated = false;
	private boolean opened = false;
	private boolean closed = false;

	protected void setUp() throws Exception {
		super.setUp();
		setHelper(new JFCTestHelper());
		
		PerspectiveLayout layout = new PerspectiveLayout();
		layout.add(new ComponentNode("testComponent"));
		descriptor1 = new PerspectiveDescriptor("aTestPerspective", "Test", layout);
		descriptor2 = new PerspectiveDescriptor("aSecondTestPerspective", "Test 2", layout);
		
		desktop.setExpectedGetPerspectiveRegistryCalls(1);
		desktop.setupGetPerspectiveRegistry(createPerspectiveRegistry());
		
		IVisualComponentRegistry registry = createVisualComponentRegistry();
		desktop.setupGetVisualComponentRegistry(registry);
		
		desktop.setupGetActionRegistry(createActionRegistry());
		
		desktop.setupGetRuntime(null);
		
		activated = false;
		opened = false;
		closed = false;
	}

	protected void tearDown() throws Exception {
		JFCTestHelper.cleanUp(this);
		super.tearDown();
	}

	private IVisualComponentDescriptor createVisualComponentDescriptor(String id, String name) {
		return null;
	}

	private IActionRegistry createActionRegistry() {
		ActionRegistry registry = new ActionRegistry();
		registry.put(createActionDescriptor(IActionConstants.EXIT_ACTION, ExitAction.class));
		return registry;
	}

	private ActionDescriptor createActionDescriptor(String id, Class c) {
		ConfigurationElement element = new ConfigurationElement(IPrismConstants.ACTION_CONFIGURATION_ELEMENT);
		element.addAttribute(IPrismConstants.CLASS_ATTRIBUTE, c.getName());
		element.addAttribute(IPrismConstants.ID_ATTRIBUTE, id);
		ActionDescriptor a = new ActionDescriptor(element);
		return a;
	}

	private IPerspectiveRegistry createPerspectiveRegistry() {
		PerspectiveRegistry registry = new PerspectiveRegistry();
		registry.add(descriptor1);
		registry.add(descriptor2);
		return registry;
	}

	private IVisualComponentRegistry createVisualComponentRegistry() {
		VisualComponentRegistry registry = new VisualComponentRegistry();
		registry.add(createVisualComponentDescriptor("testComponent", "Test"));
		return registry;
	}	

	public DesktopWindowTest(String name) {
		super(name);
	}

	public void testPerspectiveEventProduction() {
		DesktopWindow window = new DesktopWindow(desktop, null);
		window.addPerspectiveListener(this);
		
		JSplitPane splitPane = (JSplitPane) window.getContentPane().getComponent(0);
		JPanel selectionPanel = (JPanel) splitPane.getLeftComponent();
		
		PerspectiveSelector selector = (PerspectiveSelector) selectionPanel.getComponent(0);
		selector.getModel().setActivePerspective(descriptor2);
		
		assertTrue(opened);
		assertTrue(activated);
		
		desktop.verify();
	}
	
	public void perspectiveActivated(PerspectiveEvent e) {
		activated = true;
	}

	public void perspectiveClosed(PerspectiveEvent e) {
		closed = true;
	}

	public void perspectiveOpened(PerspectiveEvent e) {
		opened = true;
	}

	public void perspectiveClosing(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}
}
