package gov.va.med.hds.cd.prism.itest;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.JFCTestHelper;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveLayout;
import gov.va.med.hds.cd.prism.perspective.internal.Box;
import gov.va.med.hds.cd.prism.perspective.internal.BoxNode;
import gov.va.med.hds.cd.prism.perspective.internal.ComponentNode;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveFactory;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveLayout;
import gov.va.med.hds.cd.prism.perspective.internal.SpacerNode;
import gov.va.med.hds.cd.prism.perspective.internal.SplitterNode;
import gov.va.med.hds.cd.prism.test.MockPluginDescriptor;
import gov.va.med.hds.cd.prism.test.MockVisualComponent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentDescriptor;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentDescriptor;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.internal.Extension;
import gov.va.med.hds.cd.spectrum.MultiSplitPane;

public class PerspectiveFactoryTest
	extends JFCTestCase implements IPrismConstants {

	private JFCTestHelper helper;
	private JFrame frame;
	private VisualComponentRegistry visualComponents;

	public PerspectiveFactoryTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		createVisualComponentRegistry();
		helper = new JFCTestHelper();
		frame = new JFrame(getClass().getName() + ": " + getName());
		frame.setSize(640, 480);
	}

	protected void tearDown() throws Exception {
		visualComponents = null;
		frame = null;
		JFCTestHelper.cleanUp(this);
	}

	private IExtension createExtension() {
		MockPluginDescriptor plugin = new MockPluginDescriptor(null);
		return new Extension(plugin, IPrismConstants.UNIQUE_VISUAL_COMPONENT_EXTENSION_POINT_ID);
	}

	private IConfigurationElement createConfigurationElement(String id) {
		return new ComponentConfigurationElement(id);
	}

	private IVisualComponentDescriptor createVisualComponentDescriptor(String id) {
		VisualComponentDescriptor descriptor = new VisualComponentDescriptor(createConfigurationElement(id));
		return descriptor;
	}

	private void createVisualComponentRegistry() {
		visualComponents = new VisualComponentRegistry();
		visualComponents.add(createVisualComponentDescriptor("test1"));
		visualComponents.add(createVisualComponentDescriptor("test2"));
		visualComponents.add(createVisualComponentDescriptor("test3"));
		visualComponents.add(createVisualComponentDescriptor("test4"));
		visualComponents.add(createVisualComponentDescriptor("test5"));
		visualComponents.add(createVisualComponentDescriptor("test6"));
		visualComponents.add(createVisualComponentDescriptor("test7"));
		visualComponents.add(createVisualComponentDescriptor("test8"));
	}

	private ComponentNode createComponentNode(String id) {
		ComponentNode n = new ComponentNode(id);
		n.setFlex(1);
		return n;
	}

	public void testFixedSizes() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor("testPersp1", "A Test Perspective", createLayoutFixedSizes());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();
		Box vbox = (Box) root.getComponent(0);
		JLabel test1 = (JLabel) vbox.getComponent(0);
		JLabel test2 = (JLabel) vbox.getComponent(1);
		JLabel test3 = (JLabel) vbox.getComponent(2);

		assertEquals(30, test1.getMinimumSize().height);
		assertEquals(35, test1.getMinimumSize().width);
		assertEquals(30, test1.getPreferredSize().height);
		assertEquals(35, test1.getPreferredSize().width);
		assertEquals(30, test1.getMaximumSize().height);
		assertEquals(35, test1.getMaximumSize().width);

		assertEquals(40, test2.getMinimumSize().height);
		assertEquals(45, test2.getMinimumSize().width);

		assertEquals(50, test3.getMaximumSize().height);
		assertEquals(55, test3.getMaximumSize().width);
	}

	private IPerspectiveLayout createLayoutFixedSizes() {
		PerspectiveLayout layout = new PerspectiveLayout();
		BoxNode vbox = new BoxNode(BoxNode.VERTICAL);
		ComponentNode n = new ComponentNode("test1");
		n.setHeight(30);
		n.setWidth(35);
		vbox.add(n);
		n = new ComponentNode("test2");
		n.setMinHeight(40);
		n.setMinWidth(45);
		vbox.add(n);
		n = new ComponentNode("test3");
		n.setMaxHeight(50);
		n.setMaxWidth(55);
		vbox.add(n);
		layout.add(vbox);
		return layout;
	}

	public void testVerticalBoxFlex() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor(
				"testPersp1",
				"A Test Perspective",
				createLayoutVerticalBoxFlex());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();
		frame.getContentPane().add(root, BorderLayout.CENTER);
		frame.setVisible(true);

		Box vbox = (Box) root.getComponent(0);
		JLabel test1 = (JLabel) vbox.getComponent(0);
		JLabel test2 = (JLabel) vbox.getComponent(1);
		JLabel test3 = (JLabel) vbox.getComponent(2);

		int availableWidth = getAvailableWidth(vbox);
		assertEquals(availableWidth, test1.getSize().width);
		assertEquals(availableWidth, test2.getSize().width);
		assertEquals(availableWidth, test3.getSize().width);

		int availableHeight = getAvailableHeight(vbox);
		assertPlusOrMinusOne(availableHeight / 6, test1.getSize().height);
		assertPlusOrMinusOne(availableHeight * 2 / 6, test2.getSize().height);
		assertPlusOrMinusOne(availableHeight * 3 / 6, test3.getSize().height);

		assertPlusOrMinusOne(
			availableHeight,
			test1.getSize().height + test2.getSize().height + test3.getSize().height);
	}

	private IPerspectiveLayout createLayoutVerticalBoxFlex() {
		PerspectiveLayout layout = new PerspectiveLayout();
		BoxNode vbox = new BoxNode(BoxNode.VERTICAL);
		vbox.setFlex(1);
		ComponentNode n = new ComponentNode("test1");
		n.setFlex(1);
		vbox.add(n);
		n = new ComponentNode("test2");
		n.setFlex(2);
		vbox.add(n);
		n = new ComponentNode("test3");
		n.setFlex(3);
		vbox.add(n);
		layout.add(vbox);
		return layout;
	}

	public void testHorizontalBoxFlex() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor(
				"testPersp1",
				"A Test Perspective",
				createLayoutHorizontalBoxFlex());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();
		frame.getContentPane().add(root, BorderLayout.CENTER);
		frame.setVisible(true);

		Box hbox = (Box) root.getComponent(0);
		JLabel test1 = (JLabel) hbox.getComponent(0);
		JLabel test2 = (JLabel) hbox.getComponent(1);
		JLabel test3 = (JLabel) hbox.getComponent(2);

		int availableHeight = getAvailableHeight(hbox);
		assertEquals(availableHeight, test1.getSize().height);
		assertEquals(availableHeight, test2.getSize().height);
		assertEquals(availableHeight, test3.getSize().height);

		int availableWidth = getAvailableWidth(hbox);
		assertPlusOrMinusOne(availableWidth * 3 / 23, test1.getSize().width);
		assertPlusOrMinusOne(availableWidth * 7 / 23, test2.getSize().width);
		assertPlusOrMinusOne(availableWidth * 13 / 23, test3.getSize().width);

		assertPlusOrMinusOne(
			availableWidth,
			test1.getSize().width + test2.getSize().width + test3.getSize().width);
	}

	private IPerspectiveLayout createLayoutHorizontalBoxFlex() {
		PerspectiveLayout layout = new PerspectiveLayout();
		BoxNode hbox = new BoxNode(BoxNode.HORIZONTAL);
		hbox.setFlex(1);
		ComponentNode n = new ComponentNode("test1");
		n.setFlex(3);
		hbox.add(n);
		n = new ComponentNode("test2");
		n.setFlex(7);
		hbox.add(n);
		n = new ComponentNode("test3");
		n.setFlex(13);
		hbox.add(n);
		layout.add(hbox);
		return layout;
	}

	public void testResizeOneComponent() {
		PerspectiveLayout layout = new PerspectiveLayout();
		ComponentNode n = new ComponentNode("test1");
		n.setFlex(1);
		layout.add(n);
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor("testPersp1", "A Test Perspective", layout);
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();
		frame.getContentPane().add(root, BorderLayout.CENTER);
		frame.setVisible(true);

		JComponent c = (JComponent) root.getComponent(0);
		c.setPreferredSize(new Dimension(640 + 100, 480 + 100));

		assertEquals(
			getAvailableWidth(root),
			c.getSize().width);
		assertEquals(
			getAvailableHeight(root),
			c.getSize().height);

		// resize frame smaller	
		frame.setSize(frame.getSize().width / 2, frame.getSize().height / 2);
		frame.validate();
		assertEquals(
			getAvailableWidth(root),
			c.getSize().width);
		assertEquals(
			getAvailableHeight(root),
			c.getSize().height);

		// resize frame larger
		frame.setSize(frame.getSize().width * 3, frame.getSize().height * 3);
		frame.validate();
		assertEquals(
			getAvailableWidth(root),
			c.getSize().width);
		assertEquals(
			getAvailableHeight(root),
			c.getSize().height);
	}

	public void testCreateComponentTree() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor("testPersp1", "A Test Perspective", createLayoutTree());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		JComponent c = factory.createPerspective(perspective).getComponent();
		assertNotNull(c);
		assertTrue(c instanceof Box);
		Component[] children = c.getComponents();
		assertEquals(1, children.length);
		assertTrue(children[0] instanceof Box);
		Box vbox = (Box) children[0];

		children = vbox.getComponents();
		assertEquals(3, children.length);
		assertTrue(children[0] instanceof Box);

		children = ((Box) vbox.getComponents()[0]).getComponents();
		assertEquals(3, children.length);
		assertTrue(children[0] instanceof JLabel);
		assertEquals("test1", ((JLabel) children[0]).getText());
		assertTrue(children[1] instanceof JLabel);
		assertEquals("test2", ((JLabel) children[1]).getText());
		assertTrue(children[2] instanceof JLabel);
		assertEquals("test3", ((JLabel) children[2]).getText());

		children = ((Box) vbox.getComponents()[1]).getComponents();
		assertEquals(2, children.length);
		assertTrue(children[0] instanceof JLabel);
		assertEquals("test4", ((JLabel) children[0]).getText());
		assertTrue(children[1] instanceof JLabel);
		assertEquals("test5", ((JLabel) children[1]).getText());

		children = ((Box) vbox.getComponents()[2]).getComponents();
		assertEquals(3, children.length);
		assertTrue(children[0] instanceof JLabel);
		assertEquals("test6", ((JLabel) children[0]).getText());
		assertTrue(children[1] instanceof JLabel);
		assertEquals("test7", ((JLabel) children[1]).getText());
		assertTrue(children[2] instanceof JLabel);
		assertEquals("test8", ((JLabel) children[2]).getText());
	}

	private IPerspectiveLayout createLayoutTree() {
		PerspectiveLayout layout = new PerspectiveLayout();
		BoxNode vbox = new BoxNode(BoxNode.VERTICAL);
		vbox.setFlex(1);
		BoxNode hbox1 = new BoxNode();
		hbox1.setFlex(1);
		hbox1.add(createComponentNode("test1"));
		hbox1.add(createComponentNode("test2"));
		hbox1.add(createComponentNode("test3"));
		BoxNode hbox2 = new BoxNode();
		hbox2.setFlex(2);
		hbox2.add(createComponentNode("test4"));
		hbox2.add(createComponentNode("test5"));
		BoxNode hbox3 = new BoxNode();
		hbox3.setFlex(1);
		hbox3.add(createComponentNode("test6"));
		hbox3.add(createComponentNode("test7"));
		hbox3.add(createComponentNode("test8"));
		vbox.add(hbox1);
		vbox.add(hbox2);
		vbox.add(hbox3);
		layout.add(vbox);
		return layout;
	}

	public void testSpacer() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor("testPersp1", "A Test Perspective", createSpacerLayout());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();
		frame.getContentPane().add(root, BorderLayout.CENTER);
		frame.setVisible(true);

		assertTrue(root.getComponent(1) instanceof JPanel);
		assertTrue(root.getComponent(3) instanceof JPanel);

		assertEquals(23, root.getComponent(1).getSize().width);

		int availableHeight = getAvailableHeight(root);
		assertEquals(availableHeight, root.getComponent(1).getSize().height);
		assertEquals(availableHeight, root.getComponent(3).getSize().height);
	}

	private IPerspectiveLayout createSpacerLayout() {
		PerspectiveLayout layout = new PerspectiveLayout();
		ComponentNode n = new ComponentNode("test1");
		layout.add(n);
		SpacerNode s = new SpacerNode();
		s.setWidth(23);
		layout.add(s);
		n = new ComponentNode("test2");
		layout.add(n);
		s = new SpacerNode();
		s.setFlex(3);
		layout.add(s);
		n = new ComponentNode("test3");
		layout.add(n);
		return layout;
	}
	
	public void testSplitter() {
		PerspectiveDescriptor perspective =
			new PerspectiveDescriptor("testPersp1", "A Test Perspective", createSplitterLayout());
		PerspectiveFactory factory =
			new PerspectiveFactory(visualComponents);
		Box root = (Box) factory.createPerspective(perspective).getComponent();

		assertTrue(root.getComponent(0) instanceof MultiSplitPane);
// JSplitPane workaround for MultiSplitPane
//		assertTrue(root.getComponent(1) instanceof MultiSplitPane);
        assertTrue(root.getComponent(1) instanceof JSplitPane);
		
		MultiSplitPane splitPane1 = (MultiSplitPane) root.getComponent(0);
		assertTrue(splitPane1.getComponent(0) instanceof JLabel);
		assertTrue(splitPane1.getComponent(1) instanceof JLabel);
		assertTrue(splitPane1.getComponent(2) instanceof JLabel);
		
// JSplitPane workaround for MultiSplitPane
//		MultiSplitPane splitPane2 = (MultiSplitPane) root.getComponent(1);
//		assertTrue(splitPane2.getComponent(0) instanceof JLabel);
//		assertTrue(splitPane2.getComponent(1) instanceof JLabel);        
        JSplitPane splitPane2 = (JSplitPane) root.getComponent(1);
        assertTrue(splitPane2.getLeftComponent() instanceof JLabel);
        assertTrue(splitPane2.getRightComponent() instanceof JLabel);
		
		frame.getContentPane().add(root);
		frame.setVisible(true);
        
		int availableHeight = getAvailableHeight(root);
		assertEquals(availableHeight, splitPane1.getSize().height);
		assertEquals(availableHeight, splitPane2.getSize().height);
		
		int availableWidth = getAvailableWidth(root);
		assertEquals(availableWidth * 4 / 10, splitPane1.getSize().width);
		assertEquals(availableWidth * 6 / 10, splitPane2.getSize().width);
		
		availableWidth = getAvailableWidth(splitPane1) - 2 * splitPane1.getDividerSize();
		assertPlusOrMinusOne(availableWidth * 1 / 4, splitPane1.getComponent(0).getSize().width);
		assertPlusOrMinusOne(availableWidth * 2 / 4, splitPane1.getComponent(1).getSize().width);
		assertPlusOrMinusOne(availableWidth * 1 / 4, splitPane1.getComponent(2).getSize().width);
		
		availableWidth = getAvailableWidth(splitPane2) - splitPane2.getDividerSize();
// JSplitPane workaround for MultiSplitPane
//		assertPlusOrMinusOne(availableWidth * 4 / 6, splitPane2.getComponent(0).getSize().width);
//		assertPlusOrMinusOne(availableWidth * 2 / 6, splitPane2.getComponent(1).getSize().width);
        // JSplitPane's calculations take more factors into account than MultiSplitPane, making
        // it much more difficult to precisely estimate how many pixels each component will be
        // allocated.  Here we just verify that it's within about 5% of our expectation. 
        assertPlusOrMinusOne(10, splitPane2.getLeftComponent().getSize().width * 15 / availableWidth);
        assertPlusOrMinusOne(5, splitPane2.getRightComponent().getSize().width * 15 / availableWidth);
	}

	private IPerspectiveLayout createSplitterLayout() {
		PerspectiveLayout layout = new PerspectiveLayout();
		ComponentNode n = new ComponentNode("test1");
		n.setFlex(1);
		layout.add(n);
		layout.add(new SplitterNode());
		n = new ComponentNode("test2");
		n.setFlex(2);
		layout.add(n);
		layout.add(new SplitterNode());
		n = new ComponentNode("test3");
		n.setFlex(1);
		layout.add(n);
		n = new ComponentNode("test4");
		n.setFlex(4);
		layout.add(n);
		layout.add(new SplitterNode());
		n = new ComponentNode("test5");
		n.setFlex(2);
		layout.add(n);
		return layout;
	}

	public void assertPlusOrMinusOne(int expected, int actual) {
		assertTrue(
			"expected integer between "
				+ Integer.toString(expected - 1)
				+ " and "
				+ Integer.toString(expected + 1)
				+ ", but was "
				+ actual,
			actual >= (expected - 1) && actual <= (expected + 1));
	}
	
	public int getAvailableWidth(JComponent c) {
		return c.getSize().width - c.getInsets().left - c.getInsets().right;
	}
	
	public int getAvailableHeight(JComponent c) {
		return c.getSize().height - c.getInsets().top - c.getInsets().bottom;
	}
	
	private static class ComponentConfigurationElement implements IConfigurationElement {

		private String id;

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#createExecutableExtension(java.lang.String) */
		public Object createExecutableExtension(String propertyName) throws Exception {
			return createExecutableExtension(propertyName, null);
		}
		
		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#createExecutableExtension(java.lang.String, java.lang.Object) */
		public Object createExecutableExtension(String propertyName, Object data) throws Exception {
			MockVisualComponent component = new MockVisualComponent();
			component.setInitializationData(null, this, IPrismConstants.CLASS_ATTRIBUTE, data);
			component.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			component.setName(id);
			return component;
		}
		
		public ComponentConfigurationElement(String id) {
			this.id = id;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getAttribute(java.lang.String) */
		public String getAttribute(String name) {
			if (name.equals(IPrismConstants.ID_ATTRIBUTE)) return id;
			if (name.equals(IPrismConstants.NAME_ATTRIBUTE)) return id;
			if (name.equals(IPrismConstants.CLASS_ATTRIBUTE)) return MockVisualComponent.class.getName();
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getAttributeAsIs(java.lang.String) */
		public String getAttributeAsIs(String name) {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getAttributeNames() */
		public String[] getAttributeNames() {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getChildren() */
		public IConfigurationElement[] getChildren() {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getChildren(java.lang.String) */
		public IConfigurationElement[] getChildren(String name) {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getDeclaringExtension() */
		public IExtension getDeclaringExtension() {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getName() */
		public String getName() {
			return IPrismConstants.COMPONENT_CONFIGURATION_ELEMENT;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getValue() */
		public String getValue() {
			return null;
		}

		/* @see gov.va.med.hds.cd.runtime.IConfigurationElement#getValueAsIs() */
		public String getValueAsIs() {
			return null;
		}
	}
}
