package gov.va.med.hds.cd.prism.test;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.KeyStroke;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.internal.ActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionMapBuilder;
import gov.va.med.hds.cd.prism.action.internal.ActionProxy;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.internal.Extension;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;

public class ActionMapBuilderTest extends TestCase {

	private IRuntimeContext runtime;
	private ILogRecord logRecord;

	public ActionMapBuilderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		this.runtime = new MockRuntime();
	}

	private ActionDescriptor createActionDescriptor(String id, Class c) {
		return createActionDescriptor(id, c.getName());
	}

	private ActionDescriptor createActionDescriptor(
		String id,
		String className) {
		Extension ext =
			new Extension(
				new MockPluginDescriptor(runtime),
				"a.test.extension.point");
		ConfigurationElement element =
			new ConfigurationElement(
				IPrismConstants.ACTION_CONFIGURATION_ELEMENT);
		element.setDeclaringExtension(ext);
		element.addAttribute(IPrismConstants.CLASS_ATTRIBUTE, className);
		element.addAttribute(IPrismConstants.ID_ATTRIBUTE, id);
		element.addAttribute(IPrismConstants.NAME_ATTRIBUTE, id);
		ActionDescriptor a = new ActionDescriptor(element);
		return a;
	}

	public void testBuildActionMap() {
		ActionRegistry registry = new ActionRegistry();
		registry.put(createActionDescriptor("action1", MockAction.class));
		registry.put(createActionDescriptor("action2", MockAction.class));
		ActionDescriptor action3desc =
			createActionDescriptor("action3", MockAction.class);
		action3desc.putProperty(Action.NAME, "Test Action");
		action3desc.putProperty(Action.LONG_DESCRIPTION, "A test description");
		action3desc.putProperty(
			Action.ACCELERATOR_KEY,
			KeyStroke.getKeyStroke("ctrl T"));
		action3desc.putProperty(Action.MNEMONIC_KEY, new Integer('A'));
		registry.put(action3desc);

		ActionMapBuilder builder = new ActionMapBuilder(runtime);
		ActionMap actions = builder.buildActionMap(registry, null);

		assertTrue(actions.get("action1") instanceof ActionProxy);
		assertEquals(MockAction.class.getName(),((ActionProxy) actions.get("action1")).getDescriptor().getActionClassName());
		assertTrue(actions.get("action2") instanceof ActionProxy);
		assertEquals(MockAction.class.getName(),((ActionProxy) actions.get("action2")).getDescriptor().getActionClassName());

		Action action3 = actions.get("action3");
		assertTrue(action3 instanceof ActionProxy);
		assertEquals(MockAction.class.getName(),((ActionProxy) actions.get("action3")).getDescriptor().getActionClassName());
		assertEquals("Test Action", action3.getValue(Action.NAME));
		assertEquals(
			"A test description",
			action3.getValue(Action.LONG_DESCRIPTION));
		KeyStroke ks = KeyStroke.getKeyStroke("ctrl T");
		assertEquals(ks, action3.getValue(Action.ACCELERATOR_KEY));
		assertEquals(new Integer('A'), action3.getValue(Action.MNEMONIC_KEY));
	}

//	public void testActionClassNotFound() {
//		ILoggingCapable logging = new Log4JLogging();
//		ActionRegistry registry = new ActionRegistry();
//		registry.put(
//			createActionDescriptor(
//				"action1",
//				"gov.va.med.hds.ANonExistantClass"));
//		ActionMapBuilder builder = new ActionMapBuilder(logging);
//		logging.getLogger(ActionMapBuilder.class).addHandler(new TestHandler());
//		logging.getLogger(ActionMapBuilder.class).setUseParentHandlers(false);
//		ActionMap actions = builder.buildActionMap(registry);
//		assertNotNull(logRecord);
//		assertEquals(Level.ERROR, logRecord.getLevel());
//	}
//
//	public void testActionIllegalAccessException() {
//		ILoggingCapable logging = new Log4JLogging();
//		ActionRegistry registry = new ActionRegistry();
//		registry.put(
//			createActionDescriptor(
//				"action1",
//				"gov.va.med.hds.cd.prism.action.test.InternalTestAction"));
//		ActionMapBuilder builder = new ActionMapBuilder(logging);
//		logging.getLogger(ActionMapBuilder.class).addHandler(new TestHandler());
//		logging.getLogger(ActionMapBuilder.class).setUseParentHandlers(false);
//		ActionMap actions = builder.buildActionMap(registry);
//		assertNotNull(logRecord);
//		assertEquals(Level.ERROR, logRecord.getLevel());
//	}
//
//	public void testActionInstantiationException() {
//		ILoggingCapable logging = new Log4JLogging();
//		ActionRegistry registry = new ActionRegistry();
//		registry.put(
//			createActionDescriptor(
//				"action1",
//				"gov.va.med.hds.cd.prism.action.test.ActionRegistryFactoryTest$NoDefaultConstructorTestAction"));
//		ActionMapBuilder builder = new ActionMapBuilder(logging);
//		logging.getLogger(ActionMapBuilder.class).addHandler(new TestHandler());
//		logging.getLogger(ActionMapBuilder.class).setUseParentHandlers(false);
//		ActionMap actions = builder.buildActionMap(registry);
//		assertNotNull(logRecord);
//		assertEquals(Level.ERROR, logRecord.getLevel());
//	}

	private class TestHandler implements IHandler {
		public void publish(ILogRecord record) {
			logRecord = record;
		}
		public void close() {
		}
	}

	public static class NoDefaultConstructorTestAction extends MockAction {
		public NoDefaultConstructorTestAction(String arg0) {
		}
	}
}

class InternalTestAction extends MockAction {
}
