package gov.va.med.hds.cd.prism.test;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;

import javax.swing.Action;
import javax.swing.KeyStroke;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistryReader;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginActivationListener;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.InvalidExtensionException;
import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginRegistry;
import gov.va.med.hds.cd.runtime.internal.RuntimeDescriptor;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;
import gov.va.med.hds.cd.runtime.logging.Level;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;

public class ActionRegistryReaderTest
	extends TestCase
	implements IPrismConstants, IActionConstants {

	protected MockRuntime runtime;
	protected PluginRegistry registry;
	protected ActionRegistryReader reader;
	protected ILogRecord logRecord;

	public ActionRegistryReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		runtime = new MockRuntime();
		reader = new ActionRegistryReader();
		registry = new PluginRegistry(runtime);
		runtime.getLogger(ActionRegistryReader.class).setUseParentHandlers(
			false);
	}

	private PluginDescriptor createPrismDescriptor(ConfigurationElement[] configElements)
		throws InvalidExtensionException {
		PluginDescriptor prism =
			new PluginDescriptor(
				runtime,
				getClass().getClassLoader(),
				PLUGIN_ID,
				"Prism",
				new Version("0.1"),
				MockPrism.class.getName());
		prism.addPrerequisite(
			RuntimeDescriptor.PLUGIN_ID,
			RuntimeDescriptor.VERSION);
		prism.addExtensionPoint(ACTION_EXTENSION_POINT_ID, "Action");
		prism.addExtension(
			PLUGIN_ID + "." + ACTION_EXTENSION_POINT_ID,
			configElements);
		return prism;
	}

	private ConfigurationElement[] createValidActionConfigurationElements() {
		ConfigurationElement[] result = new ConfigurationElement[3];
		result[0] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		result[0].addAttribute(ID_ATTRIBUTE, "gov.va.med.hds.cd.prism.action1");
		result[0].addAttribute(NAME_ATTRIBUTE, "Action 1");
		result[0].addAttribute(CLASS_ATTRIBUTE, MockAction.class.getName());
		result[1] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		result[1].addAttribute(ID_ATTRIBUTE, "gov.va.med.hds.cd.prism.action2");
		result[1].addAttribute(NAME_ATTRIBUTE, "Action 2");
		result[1].addAttribute(CLASS_ATTRIBUTE, MockAction.class.getName());
		ConfigurationElement source =
			new ConfigurationElement(SOURCE_CONFIGURATION_ELEMENT);
		source.addAttribute(
			EVENT_ATTRIBUTE,
			PropertyChangeEvent.class.getName());
		result[1].addChild(source);
		ConfigurationElement listener =
			new ConfigurationElement(LISTENER_CONFIGURATION_ELEMENT);
		listener.addAttribute(EVENT_ATTRIBUTE, ActionEvent.class.getName());
		result[1].addChild(listener);
		result[2] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		result[2].addAttribute(ID_ATTRIBUTE, "gov.va.med.hds.cd.prism.action3");
		result[2].addAttribute(NAME_ATTRIBUTE, "Action 3");
		result[2].addAttribute(CLASS_ATTRIBUTE, MockAction.class.getName());
		ConfigurationElement property1 =
			new ConfigurationElement(PROPERTY_CONFIGURATION_ELEMENT);
		property1.addAttribute(NAME_ATTRIBUTE, Action.LONG_DESCRIPTION);
		property1.addAttribute(VALUE_ATTRIBUTE, "A test description");
		ConfigurationElement property2 =
			new ConfigurationElement(PROPERTY_CONFIGURATION_ELEMENT);
		property2.addAttribute(NAME_ATTRIBUTE, Action.ACCELERATOR_KEY);
		property2.addAttribute(VALUE_ATTRIBUTE, "ctrl T");
		ConfigurationElement property3 =
			new ConfigurationElement(PROPERTY_CONFIGURATION_ELEMENT);
		property3.addAttribute(NAME_ATTRIBUTE, Action.MNEMONIC_KEY);
		property3.addAttribute(VALUE_ATTRIBUTE, "T");
		ConfigurationElement property4 =
			new ConfigurationElement(PROPERTY_CONFIGURATION_ELEMENT);
		property4.addAttribute(NAME_ATTRIBUTE, Action.SMALL_ICON);
		property4.addAttribute(VALUE_ATTRIBUTE, "junit/swingui/icons/ok.gif");
		result[2].addChild(property1);
		result[2].addChild(property2);
		result[2].addChild(property3);
		result[2].addChild(property4);
		return result;
	}

	public void testCreateFromValidElements()
		throws InvalidPluginDescriptorException, InvalidExtensionException {

		registry.add(
			createPrismDescriptor(createValidActionConfigurationElements()));

		ActionRegistry actions = reader.readRegisteredActionRegistry(runtime);

		assertEquals(3, actions.size());
		assertEquals(
			MockAction.class.getName(),
			actions
				.get("gov.va.med.hds.cd.prism.action1")
				.getActionClassName());
		assertEquals(
			"Action 1",
			actions.get("gov.va.med.hds.cd.prism.action1").getProperties().get(
				Action.NAME));

		IActionDescriptor a = actions.get("gov.va.med.hds.cd.prism.action2");
		assertEquals(MockAction.class.getName(), a.getActionClassName());
		assertEquals("Action 2", a.getProperties().get(Action.NAME));
		assertEquals(1, a.getSourceOfEventDescriptors().length);
		assertEquals(1, a.getListenerOfEventDescriptors().length);
		assertEquals(PropertyChangeEvent.class.getName(), a.getSourceOfEventDescriptors()[0].getEventClassName());
		assertEquals(ActionEvent.class.getName(), a.getListenerOfEventDescriptors()[0].getEventClassName());

		
		a = actions.get("gov.va.med.hds.cd.prism.action3");
		assertEquals(MockAction.class.getName(), a.getActionClassName());
		assertEquals("Action 3", a.getProperties().get(Action.NAME));
		assertEquals(
			"A test description",
			a.getProperties().get(Action.LONG_DESCRIPTION));
		KeyStroke ks = KeyStroke.getKeyStroke("ctrl T");
		assertEquals(ks, a.getProperties().get(Action.ACCELERATOR_KEY));
		assertEquals(
			new Integer('T'),
			a.getProperties().get(Action.MNEMONIC_KEY));
		assertNotNull(a.getProperties().get(Action.SMALL_ICON));
	}

	public void testInvalidConfigurationElementName()
		throws InvalidPluginDescriptorException, InvalidExtensionException {
		ConfigurationElement[] config = new ConfigurationElement[1];
		config[0] = new ConfigurationElement("someNonActionTestElement");
		registry.add(createPrismDescriptor(config));

		runtime.getLogger(ActionRegistryReader.class).addHandler(
			new TestHandler());
		ActionRegistry actions = reader.readRegisteredActionRegistry(runtime);
		assertNotNull(logRecord);
		assertEquals(Level.ERROR, logRecord.getLevel());
	}

	public void testInvalidConfigurationElementMissingIdAttribute()
		throws InvalidPluginDescriptorException, InvalidExtensionException {
		ConfigurationElement[] config = new ConfigurationElement[1];
		config[0] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		config[0].addAttribute(NAME_ATTRIBUTE, "Test Action");
		config[0].addAttribute(CLASS_ATTRIBUTE, MockAction.class.getName());
		registry.add(createPrismDescriptor(config));

		runtime.getLogger(ActionRegistryReader.class).addHandler(
			new TestHandler());
		ActionRegistry actions = reader.readRegisteredActionRegistry(runtime);
		assertNotNull(logRecord);
		assertEquals(Level.ERROR, logRecord.getLevel());
	}

	public void testInvalidConfigurationElementMissingNameAttribute()
		throws InvalidPluginDescriptorException, InvalidExtensionException {
		ConfigurationElement[] config = new ConfigurationElement[1];
		config[0] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		config[0].addAttribute(ID_ATTRIBUTE, "someTestActionId");
		config[0].addAttribute(CLASS_ATTRIBUTE, MockAction.class.getName());
		registry.add(createPrismDescriptor(config));

		runtime.getLogger(ActionRegistryReader.class).addHandler(
			new TestHandler());
		ActionRegistry actions = reader.readRegisteredActionRegistry(runtime);
		assertNotNull(logRecord);
		assertEquals(Level.ERROR, logRecord.getLevel());
	}

	public void testInvalidConfigurationElementMissingClassAttribute()
		throws InvalidPluginDescriptorException, InvalidExtensionException {
		ConfigurationElement[] config = new ConfigurationElement[1];
		config[0] = new ConfigurationElement(ACTION_CONFIGURATION_ELEMENT);
		config[0].addAttribute(ID_ATTRIBUTE, "someTestActionId");
		config[0].addAttribute(NAME_ATTRIBUTE, "Action 1");
		registry.add(createPrismDescriptor(config));

		runtime.getLogger(ActionRegistryReader.class.getName()).addHandler(
			new TestHandler());
		ActionRegistry actions = reader.readRegisteredActionRegistry(runtime);
		assertNotNull(logRecord);
		assertEquals(Level.ERROR, logRecord.getLevel());
	}

	public class MockRuntime implements IRuntimeContext {

		private ILoggingCapable logging = new Log4JLogging();

		public Object getPluginInstance(String id)
			throws PluginStartupException, PluginNotFoundException {
			return null;
		}

		public Object getPluginInstance(String pluginId, Version version)
			throws PluginStartupException, PluginNotFoundException {
			return null;
		}

		public IPluginRegistry getPluginRegistry() {
			return registry;
		}

		public boolean isPluginActive(String pluginId) {
			return true;
		}

		public IConfigurationElement[] getSettings() {
			return null;
		}

		public ILogger getLogger(Class c) {
			return logging.getLogger(c);
		}

		public ILogger getLogger(String name) {
			return logging.getLogger(name);
		}

		public ILogger getRootLogger() {
			return logging.getRootLogger();
		}

		/* @see gov.va.med.hds.cd.runtime.IRuntimeContext#isPluginActive(java.lang.String, gov.va.med.hds.cd.mum.Version) */
		public boolean isPluginActive(String pluginId, Version version) {
			return true;
		}

		/* @see gov.va.med.hds.cd.runtime.IRuntimeContext#isPluginActivating(java.lang.String, gov.va.med.hds.cd.mum.Version) */
		public boolean isPluginActivating(String pluginId, Version version) {
			return true;
		}

		/* @see gov.va.med.hds.cd.runtime.IRuntimeContext#addPluginActivationListener(gov.va.med.hds.cd.runtime.IPluginActivationListener) */
		public void addPluginActivationListener(IPluginActivationListener l) {

		}

		/* @see gov.va.med.hds.cd.runtime.IRuntimeContext#removePluginActivationListener(gov.va.med.hds.cd.runtime.IPluginActivationListener) */
		public void removePluginActivationListener(IPluginActivationListener l) {
		}
	}

	public class TestHandler implements IHandler {
		public void publish(ILogRecord record) {
			logRecord = record;
		}

		public void flush() {
		}

		public void close() {
		}

	}
}
