package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.perspective.IContainerNode;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveLayout;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.perspective.internal.ActivePerspectiveModel;
import gov.va.med.hds.cd.prism.perspective.internal.IActivePerspectiveModel;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveDescriptor;

public class ActivePerspectiveModelTest
	extends TestCase
	implements IPerspectiveListener {

	private IActivePerspectiveModel model;
	private IPerspectiveDescriptor pd = null;

	public ActivePerspectiveModelTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		model = new ActivePerspectiveModel();
		model.addPerspectiveListener(this);
	}

	public void testAdd() {
		PerspectiveDescriptor testDescriptor =
			new PerspectiveDescriptor("testAdd", "testAdd", new MockLayout());
		assertEquals(0, model.size());
		model.add(testDescriptor);
		assertEquals(testDescriptor, pd);
		assertEquals(1, model.size());
		assertNull(model.getActivePerspective());
	}

	public void testAddDuplicate() {
		PerspectiveDescriptor testDescriptor =
			new PerspectiveDescriptor("testAddDuplicate", "testAddDuplicate", new MockLayout());
		model.add(testDescriptor);
		assertEquals(testDescriptor, pd);
		pd = null;
		model.add(testDescriptor);
		assertEquals(1, model.size());
		assertNull(pd);
	}

	public void testRemove() {
		PerspectiveDescriptor testDescriptor =
			new PerspectiveDescriptor(
				"testRemove",
				"testRemove",
				new MockLayout());
		model.add(testDescriptor);
		model.setActivePerspective(testDescriptor);
		pd = null;
		assertEquals(1, model.size());
		model.remove(testDescriptor);
		assertEquals(0, model.size());
		assertEquals(testDescriptor, pd);
		assertNull(model.getActivePerspective());
	}

	public void testActivePerspective() {
		PerspectiveDescriptor testDescriptor1 =
			new PerspectiveDescriptor("test1", "test1", new MockLayout());
		PerspectiveDescriptor testDescriptor2 =
			new PerspectiveDescriptor("test2", "test2", new MockLayout());
		model.add(testDescriptor1);
		model.add(testDescriptor2);
		pd = null;
		model.setActivePerspective(testDescriptor1);
		assertEquals(testDescriptor1, model.getActivePerspective());
		assertEquals(testDescriptor1, pd);
		model.setActivePerspective(testDescriptor2);
		assertEquals(testDescriptor2, model.getActivePerspective());
		assertEquals(testDescriptor2, pd);
		model.setActivePerspective((IPerspectiveDescriptor) null);
		assertNull(model.getActivePerspective());
		assertNull(pd);
	}

	public void perspectiveActivated(PerspectiveEvent e) {
		pd = e.getPerspective().getDescriptor();
	}

	public void perspectiveClosed(PerspectiveEvent e) {
		pd = e.getPerspective().getDescriptor();
	}

	public void perspectiveOpened(PerspectiveEvent e) {
		pd = e.getPerspective().getDescriptor();
	}

	public void perspectiveClosing(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}
}

class MockLayout implements IPerspectiveLayout {

	public IContainerNode getRoot() {
		return null;
	}

}