package gov.va.med.hds.cd.prism.test;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.action.AbstractAction;
import gov.va.med.hds.cd.prism.action.ActionAvailableEvent;
import gov.va.med.hds.cd.prism.action.IActionAvailableListener;
import gov.va.med.hds.cd.prism.action.IActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionProxy;
import gov.va.med.hds.cd.workspace.ListenerOfEventDescriptor;
import gov.va.med.hds.cd.workspace.SourceOfEventDescriptor;

public class DelegatingActionTest extends TestCase {
	
	private MockRuntime runtime;
	
	public DelegatingActionTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		runtime = new MockRuntime();
	}

	public void testActionEventDelegation() {
		IActionDescriptor actionDesc = new MockActionDescriptor("eventDelegationTest", TestAction.class, false);
		ActionProxy a = new ActionProxy(runtime, actionDesc);
		ActionEvent ae = new ActionEvent(this, 0, "theActionCommand");
		a.actionPerformed(ae);
		assertEquals(ae, TestAction.result);
	}
	
	public void testActionPropertyDelegation() {
		IActionDescriptor actionDesc = new MockActionDescriptor("eventDelegationTest", TestAction.class, false);
		actionDesc.getProperties().put(Action.NAME, "A test action");
		actionDesc.getProperties().put(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_A));
		ActionProxy a = new ActionProxy(runtime, actionDesc);
		assertEquals("A test action", a.getValue(Action.NAME));
		assertEquals(new Integer(KeyEvent.VK_A), a.getValue(Action.MNEMONIC_KEY));
		a.actionPerformed(new ActionEvent(this, 0, "theActionCommand"));
		assertEquals("yoda", a.getValue("jedi"));
		assertEquals("A test action", TestAction.action.getValue(Action.NAME));
		assertEquals(new Integer(KeyEvent.VK_A), TestAction.action.getValue(Action.MNEMONIC_KEY));
	}
	
	public void testActionAvailableNotification() {
		MockActionDescriptor actionDesc = new MockActionDescriptor("eventDelegationTest", TestAction.class, false);
		ActionProxy a = new ActionProxy(runtime, actionDesc);
		assertNull(a.getValue("jedi"));
		actionDesc.listener.actionAvailable(new ActionAvailableEvent(actionDesc));
		assertEquals("yoda", a.getValue("jedi"));
	}
	
	private static class TestAction extends AbstractAction {
		
		private static ActionEvent result;
		private static Action action;
		
		public TestAction() {
			putValue("jedi", "yoda");
		}
		
		public void actionPerformed(ActionEvent e) {
			action = this;
			result = e;
		}
	}
	
	private static class MockActionDescriptor implements IActionDescriptor {

		private boolean available;
		private Class actionClass;
		private String id;
		private Map props;
		private IActionAvailableListener listener;

		public MockActionDescriptor(String id, Class actionClass, boolean available) {
			this.id = id;
			this.actionClass = actionClass;
			this.props = new HashMap();
			this.available = available;
		}

		public String getId() {
			return id;
		}

		public String getActionClassName() {
			return actionClass.getName();
		}

		public Map getProperties() {
			return props;
		}

		public Action createAction() throws Exception {
			return (Action) actionClass.newInstance();
		}

		public boolean isActionAvailable() {
			return available;
		}

		public void addActionAvailableListener(IActionAvailableListener l) {
			listener = l;
		}

		public void removeActionAvailableListener(IActionAvailableListener l) {
		}

		public SourceOfEventDescriptor[] getSourceOfEventDescriptors() {
			return new SourceOfEventDescriptor[0];
		}

		public ListenerOfEventDescriptor[] getListenerOfEventDescriptors() {
			return new ListenerOfEventDescriptor[0];
		}
	}
}
