package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.internal.DesktopPerspectiveEventSource;
import gov.va.med.hds.cd.prism.internal.DesktopWindowManager;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;

public class DesktopPerspectiveEventSourceTest extends TestCase implements IPerspectiveListener {

	private DesktopWindowManager manager = new DesktopWindowManager();
	private IDesktopWindow window1 = new MockWindow();
	private IDesktopWindow window2 = new MockWindow();

	private boolean activated = false;
	private boolean opening = false;
	private boolean opened = false;
	private boolean closed = false;

	public DesktopPerspectiveEventSourceTest(String name) {
		super(name);
	}

	public void testPerspectiveEventProduction() {
		DesktopPerspectiveEventSource source = new DesktopPerspectiveEventSource(manager);
		source.addPerspectiveListener(this);
		manager.open(window1);
		assertTrue(opening);
		assertTrue(opened);
		assertTrue(activated);
		reset();
		manager.open(window2);
		assertTrue(opening);
		assertTrue(opened);
		assertTrue(activated);
		reset();
		manager.setActiveWindow(window1);
		assertTrue(activated);
		reset();
		manager.close(window2);
		assertTrue(closed);
		reset();
		manager.close(window1);
		assertTrue(closed);
	}

	private void reset() {
		activated = false;
		opening = false;
		opened = false;
		closed = false;
	}
	
	public void perspectiveActivated(PerspectiveEvent e) {
		activated = true;
	}

	public void perspectiveClosed(PerspectiveEvent e) {
		closed = true;
	}

	public void perspectiveOpened(PerspectiveEvent e) {
		opened = true;
	}

	public void perspectiveClosing(PerspectiveEvent e) {
		// TODO_SB: Auto-generated method stub	
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		opening = true;
	}

}
