package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.internal.DesktopVisualComponentEventSource;
import gov.va.med.hds.cd.prism.internal.DesktopWindowManager;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentListener;
import gov.va.med.hds.cd.prism.visualcomponent.VisualComponentEvent;

public class DesktopVisualComponentEventSourceTest extends TestCase implements IVisualComponentListener {

	private DesktopWindowManager manager = new DesktopWindowManager();
	private IDesktopWindow window1 = new MockWindow();
	private IDesktopWindow window2 = new MockWindow();

	private boolean activated = false;
	private boolean deactivated = false;
	private boolean opened = false;
	private boolean closed = false;

	public DesktopVisualComponentEventSourceTest(String name) {
		super(name);
	}
	
	public void testPerspectiveEventProduction() {
		DesktopVisualComponentEventSource source = new DesktopVisualComponentEventSource(manager);
		source.addVisualComponentListener(this);
		manager.open(window1);
		assertTrue(opened);
		assertTrue(activated);
		reset();
		manager.open(window2);
		assertTrue(deactivated);
		assertTrue(opened);
		assertTrue(activated);
		reset();
		manager.setActiveWindow(window1);
		assertTrue(deactivated);
		assertTrue(activated);
		reset();
		manager.close(window2);
		assertTrue(closed);
		reset();
		manager.close(window1);
		assertTrue(closed);
	}

	private void reset() {
		activated = false;
		deactivated = false;
		opened = false;
		closed = false;
	}
	
	public void componentActivated(VisualComponentEvent e) {
		activated = true;
	}

	public void componentDeactivated(VisualComponentEvent e) {
		deactivated = true;
	}

	public void componentOpened(VisualComponentEvent e) {
		opened = true;
	}
	
	public void componentClosed(VisualComponentEvent e) {
		closed = true;
	}
}
