package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.DesktopWindowEvent;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.IDesktopWindowListener;
import gov.va.med.hds.cd.prism.internal.DesktopWindowManager;

public class DesktopWindowManagerTest extends TestCase implements IDesktopWindowListener {

	private IDesktopWindow activated = null;
	private IDesktopWindow deactivated = null;
	private IDesktopWindow opening = null;
	private IDesktopWindow opened = null;
	private IDesktopWindow closed = null;
	private IDesktopWindow closing = null;

	public DesktopWindowManagerTest(String name) {
		super(name);
	}

	public void testWindowLifecycleEvents() {
		DesktopWindowManager windowManager = new DesktopWindowManager();
		windowManager.addWindowListener(this);
		MockWindow window1 = new MockWindow();
		MockWindow window2 = new MockWindow();
		
		windowManager.open(window1);
		assertNotNull(opening);
		assertTrue(window1.equals(opening));
		assertNotNull(opened);
		assertTrue(window1.equals(opened));
		assertNotNull(activated);
		assertTrue(window1.equals(activated));
		assertEquals(1, windowManager.getWindowCount());
		assertTrue(window1.equals(windowManager.getActiveWindow()));
		
		reset();
		windowManager.open(window2);
		assertNotNull(opened);
		assertTrue(window2.equals(opened));
		assertNotNull(deactivated);
		assertTrue(window1.equals(deactivated));
		assertNotNull(activated);
		assertTrue(window2.equals(activated));
		assertEquals(2, windowManager.getWindowCount());
		assertTrue(window2.equals(windowManager.getActiveWindow()));
		
		reset();
		windowManager.setActiveWindow(window1);
		assertNotNull(activated);
		assertTrue(window1.equals(activated));
		assertNotNull(deactivated);
		assertTrue(window2.equals(deactivated));
		assertTrue(window1.equals(windowManager.getActiveWindow()));
		
		reset();
		windowManager.close(window1);
		assertNotNull(deactivated);
		assertTrue(window1.equals(deactivated));
		assertNotNull(closed);
		assertTrue(window1.equals(closed));
		assertNotNull(activated);
		assertTrue(window2.equals(activated));
		assertTrue(window2.equals(windowManager.getActiveWindow()));
		
		reset();
		windowManager.close(window2);
		assertNotNull(deactivated);
		assertTrue(window2.equals(deactivated));
		assertNotNull(closed);
		assertTrue(window2.equals(closed));
		assertNull(windowManager.getActiveWindow());
	}
	
	public void windowActivated(DesktopWindowEvent e) {
		activated = e.getWindow();
	}

	public void windowClosed(DesktopWindowEvent e) {
		closed = e.getWindow();
	}

	public void windowDeactivated(DesktopWindowEvent e) {
		deactivated = e.getWindow();
	}

	public void windowOpened(DesktopWindowEvent e) {
		opened = e.getWindow();
	}
	
	private void reset() {
		activated = null;
		deactivated = null;
		opening = null;
		opened = null;
		closing = null;
		closed = null;
	}
	
	public void windowClosing(DesktopWindowEvent e) {
		closing = e.getWindow();
	}

	public void windowOpening(DesktopWindowEvent e) {
		opening = e.getWindow();
	}
}
