package gov.va.med.hds.cd.prism.test;

import javax.swing.ActionMap;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JSeparator;

import junit.framework.Test;
import junit.framework.TestSuite;

import gov.va.med.hds.cd.prism.action.internal.ActionMapBuilder;
import gov.va.med.hds.cd.prism.menu.internal.MenuFactory;
import gov.va.med.hds.cd.spectrum.LookAndFeelMenu;

public class MenuFactoryTest extends MenuTreeTest {

	public static Test suite() {
		TestSuite suite = new TestSuite();
		suite.addTest(new MenuFactoryTest("testCreateMainMenuBar"));
		suite.addTest(new MenuFactoryTest("testCreatePerspectiveMenuBar"));
		return suite;
	}

	public MenuFactoryTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();

		menuTree.addMenu("file", "/", "File", new Character('F'));
		menuTree.addMenu("sub1", "file", "New");
		menuTree.addAction("actionId1", "file");
		menuTree.addAction("actionId1", "file/sub1");
		menuTree.addSlot("slot2", "file/sub1");
		menuTree.addAction("actionId4", "file/sub1");
		menuTree.addAction("actionId2", "file/sub1/slot2");
		menuTree.addAction("actionId3", "file/sub1/slot2");

		menuTree.addMenu("edit", "/", "Edit");
		menuTree.addAction("actionId2", "edit");
		menuTree.addSlot("slot1", "edit");
		menuTree.addAction("actionId3", "edit");
		menuTree.addSeparator("separator1", "edit");
		menuTree.addAction("actionId4", "edit");
		menuTree.addAction("actionId5", "/edit/slot1");

		menuTree.addSlot("perspectiveSlot", "/");
		menuTree.addMenu("aTestPerspective", "action", "perspectiveSlot", "Action");
		menuTree.addAction("aTestPerspective", "actionId3", "perspectiveSlot/action");

		menuTree.addMenu("help", "/", "Help");
		menuTree.addAction("actionId4", "help");

		menuTree.addMenu("lnf", "/", createMenuConfigurationElement(LookAndFeelMenu.class));
	}

	public void testCreateMainMenuBar() {
		ActionMapBuilder builder = new ActionMapBuilder(new MockRuntime());
		ActionMap actions = builder.buildActionMap(menuTree.getActions(), null);
		MenuFactory factory = new MenuFactory(menuTree);
		JMenuBar menuBar = factory.createMenuBar(null, actions);
		assertEquals(4, menuBar.getMenuCount());
		assertEquals("File", menuBar.getMenu(0).getText());
		assertEquals('F', menuBar.getMenu(0).getMnemonic());
		assertEquals("Edit", menuBar.getMenu(1).getText());
		assertEquals("Help", menuBar.getMenu(2).getText());
		assertTrue(menuBar.getMenu(3) instanceof LookAndFeelMenu);

		assertEquals(2, menuBar.getMenu(0).getItemCount());
		assertEquals("New", menuBar.getMenu(0).getItem(0).getText());
		assertEquals("actionId1", menuBar.getMenu(0).getItem(1).getText());
		JMenu menu = (JMenu) menuBar.getMenu(0).getItem(0);
		assertEquals(4, menu.getItemCount());
		assertEquals("actionId1", menu.getItem(0).getText());
		assertEquals("actionId2", menu.getItem(1).getText());
		assertEquals("actionId3", menu.getItem(2).getText());
		assertEquals("actionId4", menu.getItem(3).getText());

		assertEquals(5, menuBar.getMenu(1).getMenuComponentCount());
		assertEquals("actionId2", menuBar.getMenu(1).getItem(0).getText());
		assertEquals("actionId5", menuBar.getMenu(1).getItem(1).getText());
		assertEquals("actionId3", menuBar.getMenu(1).getItem(2).getText());
		assertTrue(menuBar.getMenu(1).getMenuComponent(3) instanceof JSeparator);
		assertEquals("actionId4", menuBar.getMenu(1).getItem(4).getText());

		assertEquals(1, menuBar.getMenu(2).getItemCount());
		assertEquals("actionId4", menuBar.getMenu(2).getItem(0).getText());

	}

	public void testCreatePerspectiveMenuBar() {
		ActionMapBuilder builder = new ActionMapBuilder(new MockRuntime());
		ActionMap actions = builder.buildActionMap(menuTree.getActions(), null);
		MenuFactory factory = new MenuFactory(menuTree);
		JMenuBar menuBar = factory.createMenuBar("aTestPerspective", actions);
		assertEquals(5, menuBar.getMenuCount());
		assertEquals("File", menuBar.getMenu(0).getText());
		assertEquals('F', menuBar.getMenu(0).getMnemonic());
		assertEquals("Edit", menuBar.getMenu(1).getText());
		assertEquals("Action", menuBar.getMenu(2).getText());
		assertEquals("Help", menuBar.getMenu(3).getText());
		assertTrue(menuBar.getMenu(4) instanceof LookAndFeelMenu);

		assertEquals(2, menuBar.getMenu(0).getItemCount());
		assertEquals("New", menuBar.getMenu(0).getItem(0).getText());
		assertEquals("actionId1", menuBar.getMenu(0).getItem(1).getText());
		JMenu menu = (JMenu) menuBar.getMenu(0).getItem(0);
		assertEquals(4, menu.getItemCount());
		assertEquals("actionId1", menu.getItem(0).getText());
		assertEquals("actionId2", menu.getItem(1).getText());
		assertEquals("actionId3", menu.getItem(2).getText());
		assertEquals("actionId4", menu.getItem(3).getText());

		assertEquals(5, menuBar.getMenu(1).getMenuComponentCount());
		assertEquals("actionId2", menuBar.getMenu(1).getItem(0).getText());
		assertEquals("actionId5", menuBar.getMenu(1).getItem(1).getText());
		assertEquals("actionId3", menuBar.getMenu(1).getItem(2).getText());
		assertTrue(menuBar.getMenu(1).getMenuComponent(3) instanceof JSeparator);
		assertEquals("actionId4", menuBar.getMenu(1).getItem(4).getText());

		assertEquals(1, menuBar.getMenu(3).getItemCount());
		assertEquals("actionId4", menuBar.getMenu(3).getItem(0).getText());
	}
	
	public void testMenuBarContains() {
		ActionMapBuilder builder = new ActionMapBuilder(new MockRuntime());
		ActionMap actions = builder.buildActionMap(menuTree.getActions(), null);
		MenuFactory factory = new MenuFactory(menuTree);
		assertTrue(factory.menuBarContains(null, "actionId1"));
		assertFalse(factory.menuBarContains(null, "actionId6"));
		assertTrue(factory.menuBarContains("aTestPerspective", "actionId3"));
		assertTrue(factory.menuBarContains(null, "actionId3"));
	}
}
