package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.prism.IMenuConstants;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.action.IActionConstants;
import gov.va.med.hds.cd.prism.action.internal.ActionDescriptor;
import gov.va.med.hds.cd.prism.action.internal.ActionRegistry;
import gov.va.med.hds.cd.prism.menu.internal.InvalidMenuPathException;
import gov.va.med.hds.cd.prism.menu.internal.MenuNode;
import gov.va.med.hds.cd.prism.menu.internal.MenuTree;
import gov.va.med.hds.cd.prism.menu.internal.MenuTreeNode;
import gov.va.med.hds.cd.prism.menu.internal.MenuTreeReader;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginRegistry;
import gov.va.med.hds.cd.runtime.internal.RuntimeDescriptor;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.spectrum.LookAndFeelMenu;

public class MenuTreeReaderTest
	extends TestCase
	implements IPrismConstants, IMenuConstants, IActionConstants {

	private PluginRegistry pluginRegistry = new PluginRegistry(null);

	public MenuTreeReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		pluginRegistry.add(createPrismDescriptor(createMenuConfigurationElements()));
	}

	private ActionDescriptor createActionDescriptor(String id, Class c) {
		ConfigurationElement element = new ConfigurationElement(IPrismConstants.ACTION_CONFIGURATION_ELEMENT);
		element.addAttribute(IPrismConstants.CLASS_ATTRIBUTE, c.getName());
		element.addAttribute(IPrismConstants.ID_ATTRIBUTE, id);
		element.addAttribute(IPrismConstants.NAME_ATTRIBUTE, id);
		ActionDescriptor a = new ActionDescriptor(element);
		return a;
	}

	private PluginDescriptor createPrismDescriptor(ConfigurationElement[] configElements)
		throws Exception {
		PluginDescriptor prism =
			new PluginDescriptor(
				null,
				getClass().getClassLoader(),
				PLUGIN_ID,
				"Prism",
				new Version("0.1"),
				MockPrism.class.getName());
		prism.addPrerequisite(RuntimeDescriptor.PLUGIN_ID, RuntimeDescriptor.VERSION);
		prism.addExtensionPoint(MAIN_MENU_EXTENSION_POINT_ID, "Menu");
		prism.addExtension(UNIQUE_MENU_EXTENSION_POINT_ID, configElements);
		return prism;
	}

	private ConfigurationElement[] createMenuConfigurationElements() {
		ConfigurationElement[] result = new ConfigurationElement[9];
		result[0] = new ConfigurationElement(MENU_CONFIGURATION_ELEMENT);
		result[0].addAttribute(ID_ATTRIBUTE, "file");
		result[0].addAttribute(NAME_ATTRIBUTE, "File");
		result[0].addAttribute(MNEMONIC_ATTRIBUTE, "F");
		result[0].addAttribute(PATH_ATTRIBUTE, "/");
		result[1] = new ConfigurationElement(MENU_CONFIGURATION_ELEMENT);
		result[1].addAttribute(ID_ATTRIBUTE, "help");
		result[1].addAttribute(NAME_ATTRIBUTE, "Help");
		result[1].addAttribute(PATH_ATTRIBUTE, "/");
		result[2] = new ConfigurationElement(MENU_ITEM_CONFIGURATION_ELEMENT);
		result[2].addAttribute(ACTION_ATTRIBUTE, "exit");
		result[2].addAttribute(PATH_ATTRIBUTE, "file");
		result[3] = new ConfigurationElement(SLOT_CONFIGURATION_ELEMENT);
		result[3].addAttribute(ID_ATTRIBUTE, "helpSlot1");
		result[3].addAttribute(PATH_ATTRIBUTE, "help");
		result[4] = new ConfigurationElement(SEPARATOR_CONFIGURATION_ELEMENT);
		result[4].addAttribute(ID_ATTRIBUTE, "separator1");
		result[4].addAttribute(PATH_ATTRIBUTE, "help");
		result[5] = new ConfigurationElement(MENU_ITEM_CONFIGURATION_ELEMENT);
		result[5].addAttribute(ACTION_ATTRIBUTE, "about");
		result[5].addAttribute(PATH_ATTRIBUTE, "help");
		result[6] = new ConfigurationElement(MENU_ITEM_CONFIGURATION_ELEMENT);
		result[6].addAttribute(ACTION_ATTRIBUTE, "about");
		result[6].addAttribute(PATH_ATTRIBUTE, "help/helpSlot1");
		result[7] = new ConfigurationElement(MENU_CONFIGURATION_ELEMENT);
		result[7].addAttribute(ID_ATTRIBUTE, "lnf");
		result[7].addAttribute(CLASS_ATTRIBUTE, LookAndFeelMenu.class.getName());
		result[7].addAttribute(PATH_ATTRIBUTE, "/");
		result[8] = new ConfigurationElement(MENU_CONFIGURATION_ELEMENT);
		result[8].addAttribute(ID_ATTRIBUTE, "actionId");
		result[8].addAttribute(NAME_ATTRIBUTE, "Action");
		result[8].addAttribute(PATH_ATTRIBUTE, "/");
		result[8].addAttribute(CONTEXT_ATTRIBUTE, "aTestPerspectiveId");
		return result;
	}

	public void testCreateMenuRegistry() {
		MockRuntime runtime = new MockRuntime();
		runtime.getLogger(MenuTreeReader.class).addHandler(new TestHandler());
		ActionRegistry actions = new ActionRegistry();
		actions.put(createActionDescriptor("exit", MockAction.class));
		actions.put(createActionDescriptor("about", MockAction.class));
		MenuTreeReader reader = new MenuTreeReader(runtime);

		MenuTree mainTree = reader.readMenuTree(pluginRegistry, actions);
		try {
			MenuTreeNode node = mainTree.getNode("/file");
			assertNotNull(node);
			assertTrue(node instanceof MenuNode);
			assertEquals('F', ((MenuNode) node).getMnemonic().charValue());
			assertNotNull(mainTree.getNode("/file/exit"));
			assertNotNull(mainTree.getNode("/help"));
			assertNotNull(mainTree.getNode("/help/helpSlot1"));
			assertNotNull(mainTree.getNode("/help/separator1"));
			assertNotNull(mainTree.getNode("/help/about"));
			assertNotNull(mainTree.getNode("/help/helpSlot1/about"));
			assertNotNull(mainTree.getNode("/lnf"));
			assertNotNull(mainTree.getNode("/actionId", "aTestPerspectiveId"));
		} catch (InvalidMenuPathException e) {
			fail(e.toString());
		}
		try {
			assertNull(mainTree.getNode("/actionId"));
			fail("expected InvalidMenuPathException");
		} catch (InvalidMenuPathException e1) {
		}
	}

	public class TestHandler implements IHandler {
		public void publish(ILogRecord record) {
			fail(record.getMessage());
		}

		public void flush() {
		}

		public void close() {
		}

	}
}
