package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.perspective.internal.IPerspectiveFactory;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveManager;

public class PerspectiveManagerTest extends TestCase implements IPerspectiveFactory, IPerspectiveListener {
	
	private PerspectiveManager manager;
	private MockPerspectiveDescriptor perspective1 = new MockPerspectiveDescriptor("testPerspective1");
	private MockPerspectiveDescriptor perspective2 = new MockPerspectiveDescriptor("testPerspective2");
	private MockPerspective p1 = new MockPerspective();
	private MockPerspective p2 = new MockPerspective();
	
	private IPerspectiveReference activated = null;
	private IPerspectiveReference opening = null;
	private IPerspectiveReference opened = null;
	private IPerspectiveReference closing = null;
	private IPerspectiveReference closed = null;

	public PerspectiveManagerTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		manager = new PerspectiveManager(this, new MockWindow());
		manager.addPerspectiveListener(this);
	}

	public void testPerspectiveLifeCycleEvents() {
		IPerspectiveReference p1 = manager.open(perspective1);
		assertNotNull(opening);
		assertNotNull(opened);
		assertTrue(p1.equals(opened));
		assertNotNull(activated);
		assertTrue(p1.equals(activated));
		assertEquals(1, manager.getPerspectiveCount());
		assertTrue(p1.equals(manager.getActivePerspective()));
		
		reset();
		IPerspectiveReference p2 = manager.open(perspective2);
		assertNotNull(opening);
		assertNotNull(opened);
		assertTrue(p2.equals(opened));
		assertNotNull(activated);
		assertTrue(p2.equals(activated));
		assertEquals(2, manager.getPerspectiveCount());
		assertTrue(p2.equals(manager.getActivePerspective()));
		
		reset();
		manager.setActivePerspective(p1);
		assertNotNull(activated);
		assertTrue(p1.equals(activated));
		assertTrue(p1.equals(manager.getActivePerspective()));
		
		reset();
		manager.close(p1);
		assertNotNull(closing);
		assertTrue(p1.equals(closing));
		assertNotNull(closed);
		assertTrue(p1.equals(closed));
		assertNotNull(activated);
		assertTrue(p2.equals(activated));
		assertTrue(p2.equals(manager.getActivePerspective()));
		
		reset();
		manager.close(p2);
		assertNotNull(closing);
		assertTrue(p2.equals(closing));
		assertNotNull(closed);
		assertTrue(p2.equals(closed));
		assertNull(manager.getActivePerspective());
	}
	
	public IPerspectiveReference createPerspective(IPerspectiveDescriptor descriptor) {
		MockPerspective p = new MockPerspective();
		p.setDescriptor(descriptor);
		return p;
	}
	
	public void perspectiveActivated(PerspectiveEvent e) {
		activated = e.getPerspective();
	}

	public void perspectiveClosed(PerspectiveEvent e) {
		closed = e.getPerspective();
	}

	public void perspectiveOpened(PerspectiveEvent e) {
		opened = e.getPerspective();
	}
	
	public void perspectiveClosing(PerspectiveEvent e) {
		closing = e.getPerspective();
	}

	public void perspectiveOpening(PerspectiveEvent e) {
		opening = e.getPerspective();
	}
	
	private void reset() {
		activated = null;
		opening = null;
		opened = null;
		closing = null;
		closed = null;
	}
	
}
