package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveRegistry;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveRegistryReader;
import gov.va.med.hds.cd.runtime.InvalidExtensionException;
import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginRegistry;
import gov.va.med.hds.cd.runtime.internal.RuntimeDescriptor;

public class PerspectiveRegistryReaderTest extends TestCase implements IPrismConstants {

	private PluginRegistry pluginRegistry;

	public PerspectiveRegistryReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		this.pluginRegistry = new PluginRegistry(null);
	}

	private PluginDescriptor createPrismDescriptor(ConfigurationElement[] configElements) {
		PluginDescriptor prism = new PluginDescriptor(null, getClass().getClassLoader(), PLUGIN_ID, "Prism", new Version("0.1"), "gov.va.med.hds.cd.prism.Prism");
		prism.addPrerequisite(RuntimeDescriptor.PLUGIN_ID, RuntimeDescriptor.VERSION);
		try {
			prism.addExtensionPoint(PERSPECTIVE_EXTENSION_POINT_ID, "Perspective");
			prism.addExtension(UNIQUE_PERSPECTIVE_EXTENSION_POINT_ID, configElements);
		} catch (InvalidExtensionException e) {}
		return prism;
	}

	private ConfigurationElement[] createValidPerspectiveElements() {
		ConfigurationElement[] elements = new ConfigurationElement[3];
		elements[0] = new ConfigurationElement(PERSPECTIVE_CONFIGURATION_ELEMENT);
		elements[0].addAttribute(ID_ATTRIBUTE, "testId1");
		elements[0].addAttribute(NAME_ATTRIBUTE, "testName1");
		ConfigurationElement componentElement = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		componentElement.addAttribute(ID_ATTRIBUTE, "testComponentId1");
		elements[0].addChild(componentElement);
		
		elements[1] = new ConfigurationElement(PERSPECTIVE_CONFIGURATION_ELEMENT);
		elements[1].addAttribute(ID_ATTRIBUTE, "testId2");
		elements[1].addAttribute(NAME_ATTRIBUTE, "testName2");
		componentElement = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		componentElement.addAttribute(ID_ATTRIBUTE, "testComponentId2");
		elements[1].addChild(componentElement);
		
		elements[2] = new ConfigurationElement(PERSPECTIVE_CONFIGURATION_ELEMENT);
		elements[2].addAttribute(ID_ATTRIBUTE, "testId3");
		elements[2].addAttribute(NAME_ATTRIBUTE, "testName3");
		componentElement = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		componentElement.addAttribute(ID_ATTRIBUTE, "testComponentId3");
		elements[2].addChild(componentElement);
		
		return elements;
	}

	public void testCreatePerspectiveRegistry() {
		try {
			this.pluginRegistry.add(createPrismDescriptor(createValidPerspectiveElements()));
		} catch (InvalidPluginDescriptorException e) {
			fail(e.toString());
		}
		
		MockRuntime runtime = new MockRuntime();
		PerspectiveRegistryReader factory = new PerspectiveRegistryReader(runtime);
		PerspectiveRegistry perspectives = factory.readPerspectiveRegistry(pluginRegistry);
		assertEquals(3, perspectives.size());
		assertNotNull(perspectives.get("testId1"));
		assertEquals("testName1", ((PerspectiveDescriptor) perspectives.get("testId1")).getName());
		assertNotNull(perspectives.get("testId2"));
		assertEquals("testName2", ((PerspectiveDescriptor) perspectives.get("testId2")).getName());
		assertNotNull(perspectives.get("testId3"));
		assertEquals("testName3", ((PerspectiveDescriptor) perspectives.get("testId3")).getName());
	}
}
