package gov.va.med.hds.cd.prism.test;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.BevelBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.prism.internal.InactivityListener;
import gov.va.med.hds.cd.prism.perspective.internal.ActivePerspectiveModel;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveSelector;
import gov.va.med.hds.cd.spectrum.LookAndFeelMenu;
import gov.va.med.hds.cd.spectrum.OutlineTree;

public class PerspectiveSelectorTester {

	public static void main(String[] args) {
		try {
            UIManager.setLookAndFeel(
                new DesktopLookAndFeel(
                    PerspectiveSelectorTester.class.getClassLoader()));
		} catch (UnsupportedLookAndFeelException e) {
		}

		JFrame f = new JFrame("PerspectiveSelector Test");
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JPanel selectorPanel = new JPanel();
		selectorPanel.setLayout(new BorderLayout());

		PerspectiveSelector selector = new PerspectiveSelector();
		selector.setModel(createActivePerspectiveModel());
		
		selectorPanel.add(selector, BorderLayout.CENTER);
		selectorPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		Dimension minSize = selectorPanel.getMinimumSize();
		minSize.width = 21;
		selectorPanel.setMinimumSize(minSize);
		

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(selectorPanel);
		splitPane.setRightComponent(createContentPanel());

		f.getContentPane().add(splitPane, BorderLayout.CENTER);

		f.getToolkit().addAWTEventListener(new InactivityListener(null, 20, 5, new ExitListener()), InactivityListener.ACTIVITY_MASK);

		f.setSize(640, 480);
		f.setVisible(true);
	}

	private static ActivePerspectiveModel createActivePerspectiveModel() {
		ActivePerspectiveModel model = new ActivePerspectiveModel();
		model.add(new PerspectiveDescriptor("gov.va.med.hds.query.QueryTool", "Query Tool", null));
		model.add(
			new PerspectiveDescriptor("gov.va.med.hds.dashboard.Dashboard", "Dashboard", null));
		model.add(new PerspectiveDescriptor("gov.va.med.hds.task.TaskEditor", "Tasks", null));
		return model;
	}

	private static Component createContentPanel() {
		JPanel content = new JPanel();
		content.setLayout(new BorderLayout());
		content.add(createMenuBar(), BorderLayout.NORTH);
		JLabel label = new JLabel("Some Content");
		label.setHorizontalAlignment(SwingConstants.CENTER);
		content.add(label, BorderLayout.CENTER);

		OutlineTree tree = new OutlineTree(createTreeModel());
		content.add(tree, BorderLayout.EAST);

		JPanel p = new JPanel();
		p.add(new JButton("A"));
		p.add(new JButton("Short"));
		p.add(new JButton("A Very Long Description Or Title"));
		content.add(p, BorderLayout.SOUTH);

		return content;
	}

	private static JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		fileMenu.add(new ActionWithIcon());
		fileMenu.add(new ActionWithoutIcon());
		fileMenu.add(new JCheckBoxMenuItem("A checkable item"));
		fileMenu.add(new JRadioButtonMenuItem("A radio item"));
		menuBar.add(fileMenu);
		menuBar.add(new JMenu("Edit"));
		menuBar.add(new JMenu("View"));
		menuBar.add(new JMenu("Action"));
		menuBar.add(new JMenu("Tools"));
		menuBar.add(new LookAndFeelMenu());
		menuBar.add(new JMenu("Help"));
		return menuBar;
	}

	private static TreeModel createTreeModel() {
		DefaultMutableTreeNode yoda = new DefaultMutableTreeNode("Yoda");
		DefaultMutableTreeNode obiwan = new DefaultMutableTreeNode("Obi-wan");
		DefaultMutableTreeNode dooku = new DefaultMutableTreeNode("Dooku");
		DefaultMutableTreeNode quigonn = new DefaultMutableTreeNode("Qui-gonn");
		DefaultMutableTreeNode luke = new DefaultMutableTreeNode("Luke");
		DefaultMutableTreeNode anakin = new DefaultMutableTreeNode("Anakin");
		yoda.add(dooku);
		dooku.add(quigonn);
		quigonn.add(obiwan);
		obiwan.add(anakin);
		obiwan.add(luke);
		return new DefaultTreeModel(yoda);
	}

	private static class ActionWithIcon extends AbstractAction {
		public ActionWithIcon() {
			putValue(NAME, "ItemWithIcon");
			putValue(SMALL_ICON, UIManager.getIcon("InternalFrame.icon"));
		}

		public void actionPerformed(ActionEvent e) {
		}
	}

	private static class ActionWithoutIcon extends AbstractAction {
		public ActionWithoutIcon() {
			putValue(NAME, "ItemWithoutIcon");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
		}
	}
	
	private static class ExitListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}
	}
}
