package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.ISelection;
import gov.va.med.hds.cd.prism.ISelectionListener;
import gov.va.med.hds.cd.prism.ISelectionSource;
import gov.va.med.hds.cd.prism.SelectionEvent;
import gov.va.med.hds.cd.prism.internal.Selection;
import gov.va.med.hds.cd.prism.internal.SelectionTracker;

public class SelectionTrackerTest
	extends TestCase {

	private SelectionTracker caster;
	
	public SelectionTrackerTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		caster = new SelectionTracker();
	}	

	public void testTracker() {
		// set up sources
		MockSourceAndSink mockSource1 = new MockSourceAndSink("source1");
		MockSourceAndSink mockSource2 = new MockSourceAndSink("source2");
		mockSource1.addSelectionListener(caster);
		mockSource2.addSelectionListener(caster);
		
		// set up sinks
		MockSourceAndSink mockSink1 = new MockSourceAndSink("sink1");
		MockSourceAndSink mockSink2 = new MockSourceAndSink("sink2");
		caster.addSelectionListener(mockSink1);
		caster.addSelectionListener(mockSink2);
	
		mockSource1.fireSelection();
		assertEquals("source1", caster.getSelection().getFirstElement());
		assertEquals("source1", mockSink1.getSelection().getFirstElement());
		assertEquals(mockSource1, mockSink1.getSelectionProvider());
		assertEquals("source1", mockSink2.getSelection().getFirstElement());
		assertEquals(mockSource1, mockSink2.getSelectionProvider());

		mockSink1.clearSelection();
		mockSink2.clearSelection();
		mockSource1.fireSelection();
		assertEquals("source1", caster.getSelection().getFirstElement());
		assertNull(mockSink1.getSelection());
		assertNull(mockSink1.getSelection());

		mockSource2.fireSelection();
		assertEquals("source2", caster.getSelection().getFirstElement());
		assertEquals("source2", mockSink1.getSelection().getFirstElement());
		assertEquals(mockSource2, mockSink1.getSelectionProvider());
		assertEquals("source2", mockSink2.getSelection().getFirstElement());
		assertEquals(mockSource2, mockSink2.getSelectionProvider());		
	}

	public void testCircularSubscription() {
		MockSourceAndSink s = new MockSourceAndSink("test");
		s.addSelectionListener(caster);
		caster.addSelectionListener(s);
		s.fireSelection();
		assertEquals("test", caster.getSelection().getFirstElement());
		assertNull(s.getSelection());
		assertNull(s.getSelectionProvider());
	}

	class MockSourceAndSink implements ISelectionSource, ISelectionListener {
		
		private ISelection selection;
		private ISelectionSource provider;
		private ISelectionListener listener;
		private String message;
		
		public MockSourceAndSink(String message) {
			this.message = message;
		}
		
		public void addSelectionListener(ISelectionListener listener) {
			this.listener = listener;
		}
		
		public void removeSelectionListener(ISelectionListener listener) {
		}
		
		public ISelection getSelection() {
			return selection;
		}
		
		public ISelectionSource getSelectionProvider() {
			return provider;
		}
		
		public void selectionChanged(SelectionEvent event) {
			selection = event.getSelection();
			provider = event.getSelectionProvider();
		}
		
		public void fireSelection() {
			this.listener.selectionChanged(new SelectionEvent(this, new Selection(message)));
		}
		
		public void clearSelection() {
			selection = null;
		}
	}
}
