package gov.va.med.hds.cd.prism.test;

import javax.swing.SwingConstants;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.perspective.IContainerNode;
import gov.va.med.hds.cd.prism.perspective.ILayoutNode;
import gov.va.med.hds.cd.prism.perspective.internal.BoxNode;
import gov.va.med.hds.cd.prism.perspective.internal.ComponentNode;
import gov.va.med.hds.cd.prism.perspective.internal.LayoutReader;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveLayout;
import gov.va.med.hds.cd.prism.perspective.internal.SpacerNode;
import gov.va.med.hds.cd.prism.perspective.internal.SplitterNode;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;

public class ValidLayoutReaderTest extends TestCase implements IPrismConstants {

	private LayoutReader reader;
	private ConfigurationElement perspectiveElement;

	public ValidLayoutReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		MockRuntime runtime = new MockRuntime();
		runtime.getLogger(LayoutReader.class).setUseParentHandlers(false);
		runtime.getLogger(LayoutReader.class).addHandler(new IHandler() {
			public void publish(ILogRecord record) {
				fail(record.getMessage());
			}

			public void flush() {
			}

			public void close() {
			}
		});

		reader = new LayoutReader(runtime);
		perspectiveElement = new ConfigurationElement(PERSPECTIVE_CONFIGURATION_ELEMENT);

		ConfigurationElement vbox = new ConfigurationElement(BOX_CONFIGURATION_ELEMENT);
		vbox.addAttribute(ORIENTATION_ATTRIBUTE, "vertical");
		vbox.addAttribute(FLEX_ATTRIBUTE, "1");
		vbox.addAttribute(WIDTH_ATTRIBUTE, "246");
		vbox.addAttribute(MIN_WIDTH_ATTRIBUTE, "90");
		vbox.addAttribute(MAX_WIDTH_ATTRIBUTE, "31456");
		vbox.addAttribute(HEIGHT_ATTRIBUTE, "3");
		vbox.addAttribute(MIN_HEIGHT_ATTRIBUTE, "1");
		vbox.addAttribute(MAX_HEIGHT_ATTRIBUTE, "5");

		ConfigurationElement top = new ConfigurationElement(HORIZONTAL_BOX_CONFIGURATION_ELEMENT);
		top.addAttribute(FLEX_ATTRIBUTE, "1");
		top.addChild(createComponentElement("test1"));
		top.addChild(createSpacerElement());
		top.addChild(createComponentElement("test3"));

		ConfigurationElement middle = new ConfigurationElement(VERTICAL_BOX_CONFIGURATION_ELEMENT);
		middle.addAttribute(FLEX_ATTRIBUTE, "1");
		middle.addChild(createComponentElement("test4"));
		middle.addChild(createComponentElement("test5"));

		ConfigurationElement bottom = new ConfigurationElement(BOX_CONFIGURATION_ELEMENT);
		bottom.addAttribute(ORIENTATION_ATTRIBUTE, "horizontal");
		bottom.addAttribute(FLEX_ATTRIBUTE, "1");
		bottom.addChild(createComponentElement("test6"));
		bottom.addChild(createSplitterElement());
		bottom.addChild(createComponentElement("test7"));
		bottom.addChild(createSplitterElement());
		bottom.addChild(createComponentElement("test8"));

		vbox.addChild(top);
		vbox.addChild(middle);
		vbox.addChild(bottom);
		perspectiveElement.addChild(vbox);
	}

	private ConfigurationElement createComponentElement(String id) {
		ConfigurationElement component = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		component.addAttribute("id", id);
		component.addAttribute("flex", "1");
		return component;
	}

	private ConfigurationElement createSpacerElement() {
		ConfigurationElement component = new ConfigurationElement(SPACER_CONFIGURATION_ELEMENT);
		component.addAttribute("flex", "5");
		return component;
	}

	private ConfigurationElement createSplitterElement() {
		ConfigurationElement component = new ConfigurationElement(SPLITTER_CONFIGURATION_ELEMENT);
		return component;
	}

	public void testReadLayout() {
		PerspectiveLayout layout = reader.readLayout(perspectiveElement);

		assertTrue(layout.getRoot() instanceof BoxNode);
		assertEquals(BoxNode.HORIZONTAL, ((BoxNode) layout.getRoot()).getOrientation());

		ILayoutNode[] children = layout.getRoot().getChildren();
		assertEquals(1, children.length);
		assertTrue(children[0] instanceof BoxNode);
		assertEquals(BoxNode.VERTICAL, children[0].getOrientation());
		assertEquals(1, children[0].getFlex());
		assertEquals(246, children[0].getWidth());
		assertEquals(90, children[0].getMinWidth());
		assertEquals(31456, children[0].getMaxWidth());
		assertEquals(3, children[0].getHeight());
		assertEquals(1, children[0].getMinHeight());
		assertEquals(5, children[0].getMaxHeight());

		children = ((IContainerNode) children[0]).getChildren();
		assertEquals(3, children.length);
		assertTrue(children[0] instanceof BoxNode);
		assertEquals(SwingConstants.HORIZONTAL, children[0].getOrientation());
		assertEquals(1, children[0].getFlex());
		assertTrue(children[1] instanceof BoxNode);
		assertEquals(BoxNode.VERTICAL, children[1].getOrientation());
		assertEquals(1, children[1].getFlex());
		assertTrue(children[2] instanceof BoxNode);
		assertEquals(BoxNode.HORIZONTAL, children[2].getOrientation());
		assertEquals(1, children[2].getFlex());

		children =
			((IContainerNode) ((IContainerNode) layout.getRoot().getChildren()[0])
				.getChildren()[0])
				.getChildren();
		assertEquals(3, children.length);
		assertTrue(children[0] instanceof ComponentNode);
		assertEquals("test1", ((ComponentNode) children[0]).getVisualComponentId());
		assertTrue(children[1] instanceof SpacerNode);
		assertEquals(5, children[1].getFlex());
		assertTrue(children[2] instanceof ComponentNode);
		assertEquals("test3", ((ComponentNode) children[2]).getVisualComponentId());

		children =
			((IContainerNode) ((IContainerNode) layout.getRoot().getChildren()[0])
				.getChildren()[1])
				.getChildren();
		assertEquals(2, children.length);
		assertTrue(children[0] instanceof ComponentNode);
		assertEquals("test4", ((ComponentNode) children[0]).getVisualComponentId());
		assertTrue(children[1] instanceof ComponentNode);
		assertEquals("test5", ((ComponentNode) children[1]).getVisualComponentId());

		children =
			((IContainerNode) ((IContainerNode) layout.getRoot().getChildren()[0])
				.getChildren()[2])
				.getChildren();
		assertEquals(5, children.length);
		assertTrue(children[0] instanceof ComponentNode);
		assertEquals("test6", ((ComponentNode) children[0]).getVisualComponentId());
		assertTrue(children[1] instanceof SplitterNode);
		assertTrue(children[2] instanceof ComponentNode);
		assertEquals("test7", ((ComponentNode) children[2]).getVisualComponentId());
		assertTrue(children[3] instanceof SplitterNode);
		assertTrue(children[4] instanceof ComponentNode);
		assertEquals("test8", ((ComponentNode) children[4]).getVisualComponentId());
	}
}
