package gov.va.med.hds.cd.prism.test;

import javax.swing.event.ChangeEvent;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.internal.VisualComponentEventRegistrar;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;
import gov.va.med.hds.cd.workspace.IModelSpace;
import gov.va.med.hds.cd.workspace.IMulticaster;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceListener;
import gov.va.med.hds.cd.workspace.IWorkspaceScope;
import gov.va.med.hds.cd.workspace.InvalidEventListenerException;
import gov.va.med.hds.cd.workspace.InvalidEventSourceException;


public class VisualComponentEventRegistrarTest extends TestCase implements IWorkspace, IWorkspaceScope, IMulticaster {

	private MockWindow w;
	private MockPerspective p;
	private int sourceCount = 0;
	private int listenerCount = 0;
	private VisualComponentEventRegistrar r;

	public VisualComponentEventRegistrarTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		w = new MockWindow();
		p = new MockPerspective();
		p.setDescriptor(new MockPerspectiveDescriptor("scopeTestPerspective"));
		r = new VisualComponentEventRegistrar(new Log4JLogging(), this);
	}

	public void testIntraPerspectiveEventConnection() {
		p.addSourceVisualComponent("sourceOfChangeEvents", ChangeEvent.class.getName());
		p.addListeningVisualComponent("listenerOfChangeEvents", ChangeEvent.class.getName());
		assertEquals(0, sourceCount);
		assertEquals(0, listenerCount);
		r.perspectiveOpened(new PerspectiveEvent(this, p, w));
		assertEquals(1, sourceCount);
		assertEquals(1, listenerCount);
		r.perspectiveClosing(new PerspectiveEvent(this, p, w));
		assertEquals(0, sourceCount);
		assertEquals(0, listenerCount);
	}

	public IWorkspaceScope getScope(String scope) {
		return this;
	}

	public void addScope(String scope) {
		// NOOP
	}

	public void removeScope(String scope) {
		// NOOP
	}

	public void addSource(Class eventType, Object eventSource) throws InvalidEventSourceException {
		assertEquals(eventType, ChangeEvent.class);
		sourceCount++;
	}

	public void removeSource(Class eventType, Object eventSource) {
		assertEquals(eventType, ChangeEvent.class);
		sourceCount--;
	}

	public void addListener(Class eventType, Object listener) throws InvalidEventListenerException {
		assertEquals(eventType, ChangeEvent.class);
		listenerCount++;
	}

	public void removeListener(Class eventType, Object listener) {
		assertEquals(eventType, ChangeEvent.class);
		listenerCount--;
	}

	public String getId() {
		return "ftang";
	}

	public IMulticaster getMulticaster() {
		return this;
	}

	public void addWorkspaceListener(IWorkspaceListener l) {
		// TODO_SB: Auto-generated method stub
		
	}

	public void removeWorkspaceListener(IWorkspaceListener l) {
		// TODO_SB: Auto-generated method stub
		
	}

	public IModelSpace getModelSpace() {
		// TODO_SB: Auto-generated method stub
		return null;
	}

	public IWorkspaceScope getScope() {
		// TODO_SB: Auto-generated method stub
		return null;
	}
}
