package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentListener;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentReference;
import gov.va.med.hds.cd.prism.visualcomponent.VisualComponentEvent;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentManager;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentReference;

public class VisualComponentManagerTest extends TestCase implements IVisualComponentListener {

	private IVisualComponentReference activated = null;
	private IVisualComponentReference deactivated = null;
	private IVisualComponentReference opened = null;
	private IVisualComponentReference closed = null;

	public VisualComponentManagerTest(String name) {
		super(name);
	}

	public void testVisualComponentLifeCycleEvents() {
		VisualComponentManager manager = new VisualComponentManager();
		manager.addVisualComponentListener(this);
		MockVisualComponent comp1 = new MockVisualComponent();
		MockVisualComponent comp2 = new MockVisualComponent();
		IVisualComponentReference component1 = new VisualComponentReference(null, comp1);
		IVisualComponentReference component2 = new VisualComponentReference(null, comp2);
		
		manager.open(component1);
		assertNotNull(opened);
		assertTrue(component1.equals(opened));
		assertNotNull(activated);
		assertTrue(component1.equals(activated));
		assertEquals(1, manager.getVisualComponentCount());
		assertTrue(component1.equals(manager.getActiveVisualComponent()));
		
		reset();
		manager.open(component2);
		assertNotNull(opened);
		assertTrue(component2.equals(opened));
		assertNotNull(deactivated);
		assertTrue(component1.equals(deactivated));
		assertNotNull(activated);
		assertTrue(component2.equals(activated));
		assertEquals(2, manager.getVisualComponentCount());
		assertTrue(component2.equals(manager.getActiveVisualComponent()));
		
		reset();
		manager.setActiveVisualComponent(component1);
		assertNotNull(activated);
		assertTrue(component1.equals(activated));
		assertNotNull(deactivated);
		assertTrue(component2.equals(deactivated));
		assertTrue(component1.equals(manager.getActiveVisualComponent()));
		
		reset();
		manager.close(component1);
		assertNotNull(deactivated);
		assertTrue(component1.equals(deactivated));
		assertNotNull(closed);
		assertTrue(component1.equals(closed));
		assertNotNull(activated);
		assertTrue(component2.equals(activated));
		assertTrue(component2.equals(manager.getActiveVisualComponent()));
		
		reset();
		manager.close(component2);
		assertNotNull(deactivated);
		assertTrue(component2.equals(deactivated));
		assertNotNull(closed);
		assertTrue(component2.equals(closed));
		assertNull(manager.getActiveVisualComponent());
	}
	
	public void componentActivated(VisualComponentEvent e) {
		activated = e.getVisualComponent();
	}

	public void componentClosed(VisualComponentEvent e) {
		closed = e.getVisualComponent();
	}

	public void componentDeactivated(VisualComponentEvent e) {
		deactivated = e.getVisualComponent();
	}

	public void componentOpened(VisualComponentEvent e) {
		opened = e.getVisualComponent();
	}
	
	private void reset() {
		activated = null;
		deactivated = null;
		opened = null;
		closed = null;
	}
}
