package gov.va.med.hds.cd.prism.test;

import java.beans.PropertyChangeEvent;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.prism.IPrismConstants;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistry;
import gov.va.med.hds.cd.prism.visualcomponent.internal.VisualComponentRegistryReader;
import gov.va.med.hds.cd.runtime.InvalidExtensionException;
import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginRegistry;
import gov.va.med.hds.cd.runtime.internal.RuntimeDescriptor;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;

public class VisualComponentRegistryReaderTest extends TestCase implements IPrismConstants {

	protected PluginRegistry pluginRegistry;

	public VisualComponentRegistryReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		this.pluginRegistry = new PluginRegistry(null);
	}

	private PluginDescriptor createPrismDescriptor(ConfigurationElement[] configElements) {
		PluginDescriptor prism = new PluginDescriptor(null, getClass().getClassLoader(), PLUGIN_ID, "Prism", new Version("0.1"), "gov.va.med.hds.cd.prism.Prism");
		prism.addPrerequisite(RuntimeDescriptor.PLUGIN_ID, RuntimeDescriptor.VERSION);
		try {
			prism.addExtensionPoint(VISUAL_COMPONENT_EXTENSION_POINT_ID, "Visual Component");
			prism.addExtension(UNIQUE_VISUAL_COMPONENT_EXTENSION_POINT_ID, configElements);
		} catch (InvalidExtensionException e) {}
		return prism;
	}

	private ConfigurationElement[] createValidComponentElements() {
		ConfigurationElement[] elements = new ConfigurationElement[3];
		elements[0] = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		elements[0].addAttribute(ID_ATTRIBUTE, "testId1");
		elements[0].addAttribute(NAME_ATTRIBUTE, "testName1");
		elements[0].addAttribute(CLASS_ATTRIBUTE, MockVisualComponent.class.getName());
		elements[1] = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		elements[1].addAttribute(ID_ATTRIBUTE, "testId2");
		elements[1].addAttribute(NAME_ATTRIBUTE, "testName2");
		elements[1].addAttribute(CLASS_ATTRIBUTE, MockVisualComponent.class.getName());
		ConfigurationElement source = new ConfigurationElement(SOURCE_CONFIGURATION_ELEMENT);
		source.addAttribute(EVENT_ATTRIBUTE, PropertyChangeEvent.class.getName());
		elements[1].addChild(source);
		elements[2] = new ConfigurationElement(COMPONENT_CONFIGURATION_ELEMENT);
		elements[2].addAttribute(ID_ATTRIBUTE, "testId3");
		elements[2].addAttribute(NAME_ATTRIBUTE, "testName3");
		elements[2].addAttribute(CLASS_ATTRIBUTE, MockVisualComponent.class.getName());
		ConfigurationElement listener = new ConfigurationElement(LISTENER_CONFIGURATION_ELEMENT);
		listener.addAttribute(EVENT_ATTRIBUTE, PropertyChangeEvent.class.getName());
		elements[2].addChild(listener);
		return elements;
	}
	
	public void testCreateVisualComponentRegistry() {
		try {
			this.pluginRegistry.add(createPrismDescriptor(createValidComponentElements()));
		} catch (InvalidPluginDescriptorException e) {
			fail(e.toString());
		}
		
		VisualComponentRegistryReader registryReader = new VisualComponentRegistryReader(new Log4JLogging());
		VisualComponentRegistry components = registryReader.readComponentRegistry(pluginRegistry);
		
		assertEquals(3, components.size());
		assertNotNull(components.get("testId1"));
		assertEquals("testId1", components.get("testId1").getId());
		assertEquals("testName1", components.get("testId1").getName());
		assertNotNull(components.get("testId2"));
		assertEquals("testId2", components.get("testId2").getId());
		assertEquals("testName2", components.get("testId2").getName());
		assertEquals(PropertyChangeEvent.class.getName(), components.get("testId2").getSourceOfEventDescriptors()[0].getEventClassName());
		assertNotNull(components.get("testId3"));
		assertEquals("testId3", components.get("testId3").getId());
		assertEquals("testName3", components.get("testId3").getName());
		assertEquals(PropertyChangeEvent.class.getName(), components.get("testId3").getListenerOfEventDescriptors()[0].getEventClassName());
	}
}
