package gov.va.med.hds.cd.prism.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.prism.internal.WindowVisualComponentEventSource;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.internal.IPerspectiveFactory;
import gov.va.med.hds.cd.prism.perspective.internal.PerspectiveManager;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentListener;
import gov.va.med.hds.cd.prism.visualcomponent.VisualComponentEvent;

public class WindowVisualComponentEventSourceTest extends TestCase implements IPerspectiveFactory, IVisualComponentListener {

	private PerspectiveManager manager;

	private boolean activated = false;
	private boolean deactivated = false;
	private boolean opened = false;
	private boolean closed = false;
	
	public WindowVisualComponentEventSourceTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		manager = new PerspectiveManager(this, new MockWindow());
	}

	public IPerspectiveReference createPerspective(IPerspectiveDescriptor descriptor) {
		MockPerspective p = new MockPerspective();
		p.setDescriptor(descriptor);
		return p;
	}

	public void testVisualComponentEventProduction() {
		WindowVisualComponentEventSource source = new WindowVisualComponentEventSource(manager);
		source.addVisualComponentListener(this);
		IPerspectiveReference perspective1 = manager.open(new MockPerspectiveDescriptor("testPerspective1"));
		assertTrue(opened);
		assertTrue(activated);
		reset();
		IPerspectiveReference perspective2 = manager.open(new MockPerspectiveDescriptor("testPerspective2"));
		assertTrue(deactivated);
		assertTrue(opened);
		assertTrue(activated);
		reset();
		manager.setActivePerspective(perspective1);
		assertTrue(deactivated);
		assertTrue(activated);
		reset();
		manager.close(perspective2);
		assertTrue(deactivated);
		assertTrue(closed);
		reset();
		manager.close(perspective1);
		assertTrue(deactivated);
		assertTrue(closed);
	}
	
	private void reset() {
		activated = false;
		deactivated = false;
		opened = false;
		closed = false;
	}
	
	public void componentActivated(VisualComponentEvent e) {
		activated = true;
	}

	public void componentDeactivated(VisualComponentEvent e) {
		deactivated = true;
	}

	public void componentClosed(VisualComponentEvent e) {
		closed = true;
	}

	public void componentOpened(VisualComponentEvent e) {
		opened = true;
	}

}
