package gov.va.med.hds.cd.prism;

/**
 * An <code>DesktopCloseVetoException</code> is thrown when a proposed closing
 * of the desktop occurs at an unacceptable time.
 */
public class DesktopCloseVetoException extends Exception {

	private DesktopEvent event;

	/**
	 * Constructs <code>DesktopCloseVetoException</code>.
	 * @param e an <code>DesktopEvent</code> describing the close that was
	 * vetoed.
	 */
	public DesktopCloseVetoException(DesktopEvent e) {
		super();
		event = e;
	}

	/**
	 * Constructs <code>DesktopCloseVetoException</code> with a detailed message.
	 * @param message a descriptive message.
	 * @param e an <code>DesktopEvent</code> describing the close that was
	 * vetoed.
	 */
	public DesktopCloseVetoException(String message, DesktopEvent e) {
		super(message);
		event = e;
	}
	
	/**
	 * Gets the vetoed <code>DesktopEvent</code>.
	 * @return A <code>DesktopEvent</code> describing the vetoed close.

	 */
	public DesktopEvent getDesktopEvent() {
		return event;
	}
}
