package gov.va.med.hds.cd.prism;

/**
 * An abstract adapter class for receiving window events. The methods in this class are empty. 
 * This class exists as convenience for creating listener objects.
 * 
 * Extend this class to create a DesktopWindowEvent listener and override the
 * methods for the events of interest. (If you implement the IDesktopWindowListener 
 * interface, you have to define all of the methods in it. This abstract class defines null
 * methods for them all, so you can only have to define methods for events you care about.)
 * 
 * Create a listener object using the extended class and then register it with a Window using
 * the window's <code>addWindowListener()</code> method. When the window's status changes by 
 * virtue of being opened, closed, activated or deactivated, the relevant method 
 * in the listener object is invoked, and the WindowEvent is passed to it. 
 *
 * @see gov.va.med.hds.cd.prism.IDesktopWindowListener
 *.@see gov.va.med.hds.cd.prism.IDesktopWindowEventSource
 */
public class DesktopWindowAdapter implements IDesktopWindowListener {

	public void windowActivated(DesktopWindowEvent e) {
		// NOOP
	}

	public void windowDeactivated(DesktopWindowEvent e) {
		// NOOP
	}

	public void windowOpening(DesktopWindowEvent e) {
		// NOOP
	}

	public void windowOpened(DesktopWindowEvent e) {
		// NOOP
	}

	public void windowClosing(DesktopWindowEvent e)
		throws DesktopWindowCloseVetoException {
		// NOOP
	}

	public void windowClosed(DesktopWindowEvent e) {
		// NOOP
	}
}
