package gov.va.med.hds.cd.prism;

/**
 * A <code>DesktopWindowCloseVetoException</code> is thrown when a
 * proposed closing of a window represents an unacceptable event.
 * For instance, there may be actions that a user must take before
 * closing the desktop or one of its windows.
 * 
 * @see gov.va.med.hds.cd.prism.DesktopCloseVetoException
 */
public class DesktopWindowCloseVetoException extends Exception {
	
	private DesktopWindowEvent event;
	
	/**
	 * Constructs a <code>DesktopWindowCloseVetoException</code>.
	 * @param e A <code>DesktopWindowEvent</code> describing the vetoed window close.
	 */
	public DesktopWindowCloseVetoException(DesktopWindowEvent e) {
		super();
		event = e;
	}

	/**
	 * Constructs a <code>DesktopWindowCloseVetoException</code> with a detailed message.
	 * @param e A <code>DesktopWindowEvent</code> describing the vetoed window close.
	 * @param message A detailed message
	 */
	public DesktopWindowCloseVetoException(DesktopWindowEvent e, String message) {
		super(message);
		event = e;
	}

	/**
	 * Constructs a <code>DesktopWindowCloseVetoException</code> with a detailed message and
	 * an exception that is the cause of the veto.
	 * @param e A <code>DesktopWindowEvent</code> describing the vetoed window close.
	 * @param message A detailed message
	 * @param cause the cause (which is saved for later retrieval by the Throwable.getCause() method). 
	 * 			(A null value is permitted, and indicates that the cause is nonexistent or unknown.)
	 */
	public DesktopWindowCloseVetoException(DesktopWindowEvent e, String message, Throwable cause) {
		super(message, cause);
		event = e;
	}

	/**
	 * Constructs a <code>DesktopWindowCloseVetoException</code> with an exception that 
	 * is the cause of the veto.
	 * @param e A <code>DesktopWindowEvent</code> describing the vetoed window close.
	 * @param cause the cause (which is saved for later retrieval by the Throwable.getCause() method). 
	 * 			(A null value is permitted, and indicates that the cause is nonexistent or unknown.)
	 */
	public DesktopWindowCloseVetoException(DesktopWindowEvent e, Throwable cause) {
		super(cause);
		event = e;
	}

	/**
	 * Gets the vetoed <code>DesktopWindowEvent</code>.
	 * @return A <code>DesktopWindowEvent</code> describing the vetoed window close.
	 */
	public DesktopWindowEvent getDesktopWindowEvent() {
		return event;
	}
}
