package gov.va.med.hds.cd.prism;

import java.util.EventObject;

/**
 * An event that indicates that a desktop window has changed its status. This event
 * is generated by the desktop when it is opened, closed, activated, or deactivated.
 * The event is passed to every <code>IDesktopWindowListener</code> or 
 * <code>DesktopWindowAdapter</code> object which registered to receive such events
 * using the desktop's addDesktopWindowListener() method. (<code>DesktopWindowAdapter</code>
 * objects implement the <code>IDesktopWindowListener</code> interface.) 
 * Each such listener object gets this <code>DesktopWindowEvent</code> when the event occurs.
 * 
 * @see gov.va.med.hds.cd.prism.IDesktop
 * @see gov.va.med.hds.cd.prism.IDesktopWindowEventSource
 * @see gov.va.med.hds.cd.prism.IDesktopWindowListener
 * @see gov.va.med.hds.cd.prism.DesktopWindowAdapter
 */
public class DesktopWindowEvent extends EventObject {
    private IDesktopWindow window;

	/**
	 * Constructs a <code>DesktopWindowEvent</code> object.
	 *
	 * @param source the object that originated the event
	 * @param window the <code>IDesktopWindow</code> object that this event pertains to.
	 */
    public DesktopWindowEvent(Object source, IDesktopWindow window) {
        super(source);
        this.window = window;
    }

	/**
	 * Returns the window this event pertains to.
	 * 
	 * @return the <code>IDesktopWindow</code> object that this event pertains to.
	 */
    public IDesktopWindow getWindow() {
        return window;
    }
}
