package gov.va.med.hds.cd.prism;

import java.util.EventListener;
/**
 * A <code>DesktopExitEvent</code> event gets fired when the desktop is about
 * to exit. You can register a <code>IDesktopExitListener</code> with the
 * desktop so as to be notified.  If there is something pending that needs
 * attention before the exit, throw an <code>DesktopCloseVetoException</code> to prevent
 * the exit from occurring.
 */
public interface IDesktopListener extends EventListener {
	
	/**
	 * This method is called when the desktop is about to open.
	 * @param e a <code>DesktopEvent</code> object describing the open event.
	 */
	void desktopOpening(DesktopEvent e);
	
	/**
	 * This method is called when the desktop is about to close.
	 * @param e a <code>DesktopEvent</code> object describing the closing event.
	 * @throws DesktopCloseVetoException if the recipient wishes the close to be
	 * prevented.
	 */
	void desktopClosing(DesktopEvent e) throws DesktopCloseVetoException;
	
	/**
	 * This method is called when the desktop is finished closing and is closed.
	 * @param e a <code>DesktopEvent</code> object describing the closed event.
	 */
	void desktopClosed(DesktopEvent e);
}
