package gov.va.med.hds.cd.prism;

import javax.swing.ActionMap;
import javax.swing.JFrame;

import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveEventSource;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponentEventSource;

/**
 * A desktop window is a top level window in an instance of the clinician desktop. Visually, 
 * a desktop window has a menubar, a perspective selection area, and a main area for displaying
 * a single perspective consisting of a collection of visual components. 
 * 
 * Each desktop window has a collection of 1 or more perspectives; the active perspective is
 * the one that is being presented to the end user; at most one perspective is active in a
 * window at a time. 
 * 
 * This interface is not intended to be implemented by clients. 
 */
public interface IDesktopWindow extends IPerspectiveEventSource {
	/**
	 * Returns the desktop instance for this window.
	 * 
	 * @return the desktop instance
	 */
	IDesktop getDesktop();
	
	/**
	 * Returns the action map for this window.
	 * 
	 * @return ActionMap
	 */
	ActionMap getActionMap();
	
	/**
	 * Returns the visual component event source for this window.
	 * 
	 * Add an <code>IVisualComponentListener</code> to this if
	 * you want to be notified of visual component lifecycle events
	 * for all visual components inside this window.
	 * 
	 * @return IVisualComponentEventSource
	 */
	IVisualComponentEventSource getVisualComponentEventSource();	
	/**
	 * Returns a list of the open perspectives in this desktop window. 
	 * 
	 * Note that each window has its own perspectives; perspective instances are never shared
	 * between different windows.
	 * 
	 * @return a list of pages
	 */
	IPerspectiveReference[] getPerspectives();
	/**
	 * Returns this desktop window's frame.
	 * 
	 * @return the frame containing this window's components
	 */
	JFrame getFrame();
	/**
	 * Sets the active perspective in this window.
	 *
	 * @param perspectiveId the unique identifier of the perspective to make active in this window
	 */
	void setActivePerspective(String perspectiveId);
}
