package gov.va.med.hds.cd.prism;

/**
 * A desktop window event source tracks desktop window lifecycle events.
 * 
 * A listener that wants to be notified of desktop window events
 * must implement the <code>IDesktopWindowListener</code> interface.
 * 
 * This interface is not intended to be implemented by clients. 
 */
public interface IDesktopWindowEventSource {
	/**
	 * Returns the currently active window as provided by this source.
	 * 
	 * @return the active desktop window, or null if the currently active window is not
	 * a desktop window
	 */
	IDesktopWindow getActiveWindow();
	/**
	 * Adds a window listener.
	 * @param l the window listener
	 */
	void addWindowListener(IDesktopWindowListener l);

	/**
	 * Removes a window listener.
	 * @param l the window listener
	 */
	void removeWindowListener(IDesktopWindowListener l);
}
