package gov.va.med.hds.cd.prism;

import java.util.Iterator;
import java.util.List;

/**
 * Interface for a selection.
 *
 * @see ISelectionSource
 * @see ISelectionListener
 * @see SelectionEvent
 */
public interface ISelection {
	/**
	 * Returns whether this selection is empty.
	 * 
	 * @return <code>true</code> if this selection is empty,
	 *   and <code>false</code> otherwise
	 */
	public boolean isEmpty();
	
	/**
	 * Returns the first element in this selection, or <code>null</code>
	 * if the selection is empty.
	 *
	 * @return an element, or <code>null</code> if none
	 */
	public Object getFirstElement();
	/**
	 * Returns an iterator over the elements of this selection.
	 *
	 * @return an iterator over the selected elements
	 */
	public Iterator iterator();
	/**
	 * Returns the number of elements selected in this selection.
	 *
	 * @return the number of elements selected
	 */
	public int size();
	/**
	 * Returns the elements in this selection as an array.
	 *
	 * @return the selected elements as an array
	 */
	public Object[] toArray();
	/**
	 * Returns the elements in this selection as a <code>List</code>.
	 *
	 * @return the selected elements as a list
	 */
	public List toList();
}
