package gov.va.med.hds.cd.prism;

/**
 * Interface common to all objects that provide a selection.
 *
 * @see ISelection
 * @see ISelectionListener
 * @see SelectionEvent
 */
public interface ISelectionSource {
	/**
	 * Adds a listener for selection changes in this selection provider.
	 * Has no effect if an identical listener is already registered.
	 *
	 * @param listener a selection changed listener
	 */
	public void addSelectionListener(ISelectionListener listener);
	/**
	 * Removes the given selection change listener from this selection provider.
	 * Has no affect if an identical listener is not registered.
	 *
	 * @param listener a selection changed listener
	 */
	public void removeSelectionListener(ISelectionListener listener);
	/**
	 * Returns the current selection for this provider.
	 * 
	 * @return the current selection
	 */
	public ISelection getSelection();
	/**
	 * Sets the current selection for this selection provider.
	 *
	 * @param selection the new selection
	 */
//	public void setSelection(ISelection selection);
}
