package gov.va.med.hds.cd.prism;

import gov.va.med.hds.cd.prism.perspective.IPerspectiveDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;

/**
 * A collection of utility methods for the desktop user interface (also known as "Prism").
 */
public class PrismUtilities implements IPrismConstants {
	/**
	 * Convenience method that returns the IDesktop in the specified runtime context.
	 * 
	 * @param runtime the runtime
	 * @return a reference to <code>IDesktop</code> given the specified runtime context.
	 */
	public static IDesktop getDesktop(IRuntimeContext runtime) {
		try {
			return (IDesktop) runtime.getPluginInstance(PLUGIN_ID);
		} catch (PluginStartupException e) {
			runtime.getLogger(PrismUtilities.class).fatal(
				"error getting reference to the desktop",
				e);
			return null;
		} catch (PluginNotFoundException e) {
			runtime.getLogger(PrismUtilities.class).fatal(
				"error getting reference to the desktop",
				e);
			return null;
		}
	}

	/**
	 * Gets the workspace perspective scope id for a given perspective in a given window.
	 * 
	 * The convention for constructing a perspective workspace scope id is:
	 * "perspective.window&lt;hashCode&gt;.&lt;perspectiveId&gt;"
	 * 
	 * @param w the <code>IDesktopWindow</code> in which the perspective will live.
	 * @param d the <code>IPerspectiveDescriptor</code> for the perspective of interest.
	 * @return a string containing the perspective scope id.
	 * 
	 * @see gov.va.med.hds.cd.workspace.IWorkspace
	 * @see gov.va.med.hds.cd.workspace.IWorkspaceScope
	 */
	public static String getPerspectiveScopeId(
		IDesktopWindow w,
		IPerspectiveDescriptor d) {
		return "perspective.window"
			+ Integer.toString(w.hashCode())
			+ "."
			+ d.getId();
	}

	/**
	 * Gets the workspace patient scope id for a given window.
	 * 
	 * The convention for constructing a patient workspace scope id is:
	 * "perspective.window&lt;hashCode&gt;"
	 * 
	 * @param w the <code>IDesktopWindow</code> in which the perspective will live.
	 * @return a string containing the patient scope id.
	 * 
	 * @see gov.va.med.hds.cd.workspace.IWorkspace
	 * @see gov.va.med.hds.cd.workspace.IWorkspaceScope
	 */
	public static String getPatientScopeId(IDesktopWindow w) {
		return "patient.window" + Integer.toString(w.hashCode());
	}
}
