package gov.va.med.hds.cd.prism;

import java.util.EventObject;


/**
 * Event object describing a selection change. The source of these
 * events is a selection provider.
 *
 * @see ISelection
 * @see ISelectionSource
 * @see ISelectionListener
 */
public class SelectionEvent extends EventObject {

	protected ISelection selection;

	/**
	 * Creates a new event for the given source and selection.
	 *
	 * @param source the selection provider
	 * @param selection the selection
	 */
	public SelectionEvent(
		ISelectionSource source,
		ISelection selection) {
		super(source);
		this.selection = selection;
	}
	/**
	 * Returns the selection.
	 *
	 * @return the selection
	 */
	public ISelection getSelection() {
		return selection;
	}
	/**
	 * Returns the selection provider that is the source of this event.
	 *
	 * @return the originating selection provider
	 */
	public ISelectionSource getSelectionProvider() {
		return (ISelectionSource) getSource();
	}
}
