package gov.va.med.hds.cd.prism.perspective;

/**
 * An </code>IPerspectiveEventSource<code> tracks the perspective lifecycle
 * events within a desktop window. 
 * 
 * This interface is not intended to be implemented by clients. 
 */
public interface IPerspectiveEventSource {
	/**
	 * Returns the active perspective.
	 *
	 * @return the active perspective, or <code>null</code> if no perspective is currently active
	 */
	public IPerspectiveReference getActivePerspective();
	
	/**
	 * Adds the given listener for a page's perspective lifecycle events.
	 * Has no effect if an identical listener is already registered.
	 *
	 * @param listener a perspective listener
	 */
	public void addPerspectiveListener(IPerspectiveListener l);
	
	/**
	 * Removes the given page's perspective listener.
	 * Has no affect if an identical listener is not registered.
	 *
	 * @param listener a perspective listener
	 */
	public void removePerspectiveListener(IPerspectiveListener l);
}
