/*
 * Initially Created on Apr 7, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.event.EventListenerList;

/**
 * Abstract IQueryList implementation.  This implementation only allows ISoftListItem's 
 * to be added and removed, and will always maintain the list sorted by the ISoftListItem's
 * toString method.  If anything other than an ISoftListItem is used, the methods will 
 * throw a SoftListException.
 */
public class AbstractQueryList implements IQueryList {
	private List list = new ArrayList();
	private EventListenerList listenerList = new EventListenerList();
	private IHardReferenceManager queryReferenceManager = new HardReferenceManager();

	public AbstractQueryList() {
		super();
	}

	public void removeElementAt(int index) throws QueryListException {
		list.remove(index);
		fireDataEvent(IQueryListEvent.INTERVAL_REMOVED, index, index);
	}

	public int getSize() {
		return list.size();
	}

	public void addListener(IQueryListListener l) {
		listenerList.add(IQueryListListener.class, l);
	}

	public void removeListener(IQueryListListener l) {
		listenerList.remove(IQueryListListener.class, l);
	}

	public List getAsList() {
		return Collections.unmodifiableList(list);
	}
	
	protected void fireDataEvent(final int listEventID, final int start, final int end) {
		if (listenerList.getListenerCount()>0) {
			IQueryListEvent event = new QueryListEvent(this, listEventID, start, end);
			IQueryListListener[] listeners = (IQueryListListener[])listenerList.getListeners(IQueryListListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].listChanged(event);
			}
		}
	}

	public int indexOf(ISoftListItem item) {
		return list.indexOf(item);
	}

	public void addElement(IQuery query) throws QueryListException {
		ISoftListItem item = new QueryListItem(query, queryReferenceManager);
		addItem(item);
	}
	
	public void addElement(IQueryID queryID, String displayName) throws QueryListException {
		ISoftListItem item = new QueryListItem(queryID, displayName, queryReferenceManager);
		addItem(item);
	}

	private void addItem(ISoftListItem item) throws QueryListException {
		if (list.contains(item)) {
			throw new QueryListException("Query already exists in list.");
		}
		else {
			int index = Collections.binarySearch(list, item);
			if (index<0) {
				index = (index*-1) - 1;
			}
			list.add(index, item);
			fireDataEvent(IQueryListEvent.INTERVAL_ADDED, index, index);
		}
	}

	public void removeElement(IQuery query) throws QueryListException {
		removeElement(query.getQueryID());
	}

	public void removeElement(IQueryID queryID) throws QueryListException {
		int index;
		if ((index=indexOf(queryID))>-1) {
			removeElementAt(index);
		}
	}

	public void removeElement(ISoftListItem listItem) throws QueryListException {
		int index = list.indexOf(listItem);
		if (index>-1) {
			removeElementAt(index);
		}
	}

	public int indexOf(IQueryID queryID) {
		for (int i = 0; i < list.size(); i++) {
			if (((ISoftListItem)list.get(i)).getID().equals(queryID)) {
				return i;
			}
		}
		return -1;
	}

	public int indexOf(IQuery query) {
		return indexOf(query.getQueryID());
	}

	public int indexOf(String displayName) {
		for (int i = 0; i < list.size(); i++) {
			if (((ISoftListItem)list.get(i)).getDisplayName().equalsIgnoreCase(displayName)) {
				return i;
			}
		}
		return -1;
	}

	public IQuery getQueryAt(int index) {
		if ((index>-1) && (index<list.size())) {
			return (IQuery)((ISoftListItem)list.get(index)).getItem();
		}
		return null;
	}

	public ISoftListItem getItemAt(int index) {
		if ((index>-1) && (index<list.size())) {
			return (ISoftListItem)list.get(index);
		}
		return null;
	}

}
