/*
 * Initially Created on Jul 17, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.dao.IFieldDefnDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.event.EventListenerList;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * Model containing the fields available to be saved in a query based upon the current query criteria.
 */
public class AvailableColumnsList implements IAvailableColumnsList {
	private List fieldList = new ArrayList();
	private EventListenerList listenerList = new EventListenerList();
	private IQuery query;
	private IFieldDefnDAO fieldDefnDAO;
	private ILogger logger;
	private Map fieldReferenceMap = new HashMap();
	private PropertyChangeListener changeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			updateFromModel();
		}
	};

	public AvailableColumnsList() {
		super();
	}

	public AvailableColumnsList(IRuntimeContext runtime) {
		super();
		setRuntime(runtime);
	}

	public void addListDataListener(ListDataListener listener) {
		listenerList.add(ListDataListener.class, listener);
	}

	public void removeListDataListener(ListDataListener listener) {
		listenerList.remove(ListDataListener.class, listener);
	}

	private void fireChange(int eventID, int from, int to) {
		if (listenerList.getListenerCount()>0) {
			ListDataEvent event = new ListDataEvent(this, eventID, from, to);
			ListDataListener[] listeners = (ListDataListener[])listenerList.getListeners(ListDataListener.class);
			for (int i = 0; i < listeners.length; i++) {
				switch (eventID) {
					case ListDataEvent.CONTENTS_CHANGED: {
						listeners[i].contentsChanged(event);
						break;
					}
					case ListDataEvent.INTERVAL_ADDED: {
						listeners[i].intervalAdded(event);
						break;
					}
					case ListDataEvent.INTERVAL_REMOVED: {
						listeners[i].intervalRemoved(event);
						break;
					}
				}
			}
		}
	}

	public IDisplayField get(int index) {
		if (index>-1 && index<fieldList.size()) {
			return (IDisplayField)fieldList.get(index);
		}
		return null;
	}

	public int size() {
		return fieldList.size();
	}

	synchronized public void setModel(IQuery query) {
		if (this.query==query) return;
		if (this.query!=null) {
			this.query.removePropertyChangeListener(changeListener);
		}
		this.query = query;
		if (query!=null) {
			query.addPropertyChangeListener(changeListener);
		}
		updateFromModel();
	}

	private void updateFromModel() {
		List newTypeList = new ArrayList();
		
		if (query!=null) {
			newTypeList.addAll(query.getDisplayFieldTypes());
		}
		
		List newFieldList = new ArrayList();
		for (Iterator i = newTypeList.iterator(); i.hasNext();) {
			List items = getFieldsFromReference(i.next());
			if (!newFieldList.containsAll(items)) {
				newFieldList.addAll(items);
			}
		}
		
		if (fieldList.equals(newFieldList)) return;
		fieldList = newFieldList;
		int size = fieldList.size()>0 ? fieldList.size()-1 : 0;
		fireChange(ListDataEvent.CONTENTS_CHANGED, 0, size);
	}

	private List getFieldsFromReference(Object reference) {
		if (!fieldReferenceMap.containsKey(reference)) {
			List referenceList = new ArrayList();
			referenceList.add(reference);
			try {
				List newFieldList = fieldDefnDAO.getFields(referenceList);
				fieldReferenceMap.put(reference, newFieldList);
			}
			catch (QueryDAOException e) {
				logger.error("Error updating the available query display fields:"+e.getMessage(), e);
				return null;
			}
		}
		return (List)fieldReferenceMap.get(reference);
	}

	public void setRuntime(IRuntimeContext runtime) {
		logger = runtime.getLogger(this.getClass());
		try {
			IDaoRegistry reg = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
			fieldDefnDAO = (IFieldDefnDAO)reg.getDAO(IFieldDefnDAO.class.getName());
		}
		catch (Exception e) {
			logger.error("Error retrieving the DaoRegistry to look up available query fields:"+e.getMessage(), e);
			return;
		}
	}

	public int indexOf(IDisplayField field) {
		return fieldList.indexOf(field);
	}

	public IDisplayField getMutable(int index) {
		if (index<0 || index>fieldList.size()-1) {
			return null;
		}
		IDisplayField field = get(index);
		IDisplayField newField = new DisplayField();
		newField.setColumnWidth(field.getColumnWidth());
		newField.setDisplayText(field.getDisplayText());
		newField.setID(field.getID());
		newField.setListDisplayName(field.getListDisplayName());
		newField.setSampleData(field.getSampleData());
		newField.setSortType(field.getSortType());
		return newField;
	}
	
}
