package gov.va.med.hds.querytool;

import gov.va.med.hds.util.date.RelativeDate;
import gov.va.med.hds.util.mumps.StringUtil;

import java.util.Date;

/**
 * @author vhaislmonsos
 *
 * Feb 27, 2003 8:11:15 AM
 */
public class DateRange implements IDateRange, Comparable {
	private Date fromDate;
	private Date toDate;
	private String otherDate;
	private Date referenceDate;
	private String range;
	private final char sep = ':';

	private DateRange() {
	}
	
	public DateRange(Date referenceDate, String text) {
		this.referenceDate = referenceDate;
		init(text);
	}

	private void init(String text) {
		String s = null;
		if (text.indexOf(sep)>-1) {
			range = text;
			if ((s=StringUtil.piece(text, sep, 1)).length()>0) {
				fromDate = RelativeDate.getRelativeToDate(referenceDate, s);
			}
			if ((s=StringUtil.piece(text, sep, 2)).length()>0) {
				toDate = RelativeDate.getRelativeToDate(referenceDate, s);
			}
			otherDate = null;
		}
		else {
			otherDate = text;
			fromDate = null;
			toDate = null;
			range = null;
		}
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IDateRange#setConcreteRange(java.util.Date, java.util.Date)
	 */
	public void setConcreteRange(Date from, Date to) {
		fromDate = from;
		toDate = to;
		range = this.toString();
		otherDate = null;
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IDateRange#setOtherRange(java.lang.String)
	 */
	public void setRange(String text) {
		init(text);
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IDateRange#isConcreteRange()
	 */
	public boolean isConcreteRange() {
		return (range!=null);
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (isConcreteRange()) {
			return (RelativeDate.getRelativeDateString(referenceDate, fromDate) + sep + RelativeDate.getRelativeDateString(referenceDate, toDate));
		}
		else {
			return otherDate;
		}
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IDateRange#getReferenceDate()
	 */
	public Date getReferenceDate() {
		return referenceDate;
	}
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		return o.toString().compareTo(this.toString());
	}
	
}
