package gov.va.med.hds.querytool;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.event.EventListenerList;

/**
 * Display field class, contained by a IDisplayFieldList for a given query.
 */
public class DisplayField implements IDisplayField {
	private int columnWidth = 80;
	private String displayText = "";  //HeaderName (or short name)
	private String id = "";	//ColumnName    InternalName
	private String listDisplayName = "";	//List box name
	private String sortType = "";
	private String[] sampleData = new String[2];
	private EventListenerList listeners = new EventListenerList();
	
	public DisplayField() {
		super();
	}
	
	public void setColumnWidth(int columnWidth) {
		int oldWidth = this.columnWidth;
		this.columnWidth = columnWidth;
		firePropertyChanged(new PropertyChangeEvent(this,COL_WIDTH_PROPERTY,new Integer(oldWidth),new Integer(columnWidth)));
	}

	public void setID(Object id) {
		String oldVal = this.id;
		this.id = id.toString();
		firePropertyChanged(new PropertyChangeEvent(this,ID_PROPERTY,oldVal, id));
	}

	public void setDisplayText(String displayText) {
		String oldVal = this.displayText;
		this.displayText = displayText;
		firePropertyChanged(new PropertyChangeEvent(this, DISPLAY_TEXT_PROPERTY, oldVal, displayText));
	}

	public String getDisplayText() {
		return displayText;
	}

	public Object getID() {
		return id;
	}

	public int getColumnWidth() {
		return columnWidth;
	}

	public String getListDisplayName() {
		return listDisplayName;
	}

	public String[] getSampleData() {
		return sampleData;
	}

	public String getSortType() {
		return sortType;
	}

	public void setListDisplayName(String listDisplayName) {
		this.listDisplayName = listDisplayName;
	}

	public void setSampleData(String[] sampleData) {
		this.sampleData = sampleData;
	}

	public void setSortType(String sortType) {
		this.sortType = sortType;
	}

	public String getCategory() {
		if (id!=null && id.length()>0) {
			return id.substring(0, 1);
		}
		return "";
	}

	public boolean equals(Object o) {
		return (o instanceof DisplayField) && (((DisplayField)o).id.equals(id));
	}
	
	public String toString() {
		return listDisplayName;
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		listeners.add(PropertyChangeListener.class, listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		listeners.remove(PropertyChangeListener.class, listener);
	}

	public void firePropertyChanged(PropertyChangeEvent e) {
		if (listeners.getListenerCount()>0) {
			PropertyChangeListener[] list = (PropertyChangeListener[])listeners.getListeners(PropertyChangeListener.class);
			for (int i = 0; i < list.length; i++) {
				list[i].propertyChange(e);
			}
		}
	}

}
