/*
 * Initially Created on Mar 18, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import java.lang.ref.SoftReference;
import java.util.EventObject;

/**
 * Implementation of an ISoftListItem to use SoftReferences for storage of IQuery's.
 * Also allows users of IQueryList to use these items in ListModel's.
 */
public class QueryListItem implements ISoftListItem {
	private SoftReference queryReference;
	private String displayName;
	private IQueryID queryID;
	private IFinalizeListener queryFinalListener = new IFinalizeListener() {
		public void preprocessFinalization(EventObject event) {
			fireQueryPreFinalization(event);
		}
	};
	private IHardReferenceManager refManager;

	public QueryListItem() {
		super();
	}
	
	public QueryListItem(Object id, String displayName, IHardReferenceManager refManager) {
		super();
		this.refManager = refManager;
		init(id, displayName);
	}

	public QueryListItem(Object id, Object item, String displayName, IHardReferenceManager refManager) {
		super();
		this.refManager = refManager;
		setDisplayName(displayName);
		setID(id);
		setItem(item);
	}
	
	public QueryListItem(Object item, IHardReferenceManager refManager) {
		super();
		this.refManager = refManager;
		init(item);
	}
	
	private void init(Object item) {
		IQuery query = (IQuery) item;
		setDisplayName(query.getDisplayName());
		setID(query.getQueryID());
		setItem(query);
	}
	
	private void init(Object id, String displayName) {
		setDisplayName(displayName);
		setID(id);
	}

	public Object getItem() {
		if ((queryReference!=null) && (queryReference.get()!=null)) {
			return queryReference.get();
		}
		return null;
	}

	public void setItem(Object item) {
		if (queryReference!=null && queryReference.get()!=null) {
			IQuery q = (IQuery)queryReference.get();
			try {
				refManager.removeItem(q);
			}
			catch (HardReferenceManagerException e) {
			}
			q.removeFinalizeListener(queryFinalListener);
			setValuesFromQuery(q);
		}
		if (item==null) {
			queryReference = null;
			return;
		}
		IQuery query = (IQuery)item;
		try {
			refManager.addItem(query);
		}
		catch (HardReferenceManagerException e) {
		}
		queryReference = new SoftReference(query);
		query.addFinalizeListener(queryFinalListener);
		setValuesFromQuery(query);
	}

	public String getDisplayName() {
		if ((queryReference!=null) && (queryReference.get()!=null)) {
			return ((IQuery)queryReference.get()).getDisplayName();
		}
		else {
			return displayName;
		}
	}

	public void setDisplayName(String name) {
		this.displayName = name;
	}

	public Object getID() {
		if ((queryReference!=null) && (queryReference.get()!=null)) {
			return ((IQuery)queryReference.get()).getQueryID();
		}
		else {
			return queryID;
		}
	}

	public void setID(Object queryID) {
		this.queryID = (IQueryID)queryID;
	}

	public String toString() {
		return getDisplayName();
	}

	public boolean equals(Object o) {
		return (o instanceof ISoftListItem) && 
				(((ISoftListItem)o).getID().equals(this.getID())) &&
				(((ISoftListItem)o).getDisplayName().equalsIgnoreCase(this.getDisplayName()));
	}

	public int compareTo(Object o) {
		return displayName.compareToIgnoreCase(((ISoftListItem)o).getDisplayName());
	}
	
	private void fireQueryPreFinalization(EventObject event) {
		IQuery q = (IQuery)event.getSource();
		if (refManager!=null) {
			try {
				refManager.removeItem(q);
			}
			catch (HardReferenceManagerException e) {
			}
		}
		setValuesFromQuery(q);
	}

	private void setValuesFromQuery(IQuery q) {
		displayName = new String(q.getDisplayName());
		queryID = new QueryID(q.getQueryID().toString());
	}

	public void setReferenceManager(IHardReferenceManager referenceManager) {
		this.refManager = referenceManager;
	}

	public IHardReferenceManager getReferenceManager() {
		return refManager;
	}

}
