/*
 * Initially Created on Jun 23, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.dao.ISensitiveDataDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * Container for the sensitive data models.
 */
public class SensitiveDataModel implements ISensitiveDataModel {
	private ISensitiveDataDAO dataDAO;
	private Document document = new PlainDocument();
	private ISensitiveList sensitiveList;
	private IRuntimeContext runtime;
	private ILogger logger;
	private IDetailReceivable detailReceivable = new IDetailReceivable() {
		public void setDetail(String detail) {
			setMessage(detail);
		}
	};

	public SensitiveDataModel() {
		super();
	}

	public SensitiveDataModel(IRuntimeContext runtime, ISensitiveDataDAO dataDAO) {
		super();
		setRuntime(runtime);
		setSensitiveDataDAO(dataDAO);
	}

	public ISensitiveList getSensitivePtList() {
		if (sensitiveList==null) {
			sensitiveList = new SensitiveList();
			setupList();
		}
		return sensitiveList;
	}

	public synchronized void setSensitiveMessagePatient(ISensitiveListItem item) throws SensitivePatientException {
		try {
			document.remove(0, document.getLength());
			if (item!=null) {
				document.insertString(0, "Retrieving message...", null);
			}
		}
		catch (Exception e) {
			throw new SensitivePatientException(e);
		}
		if (item!=null && dataDAO!=null) {
			try {
				dataDAO.getSensitiveMessage(item.getPatientID(), detailReceivable);
			}
			catch (QueryDAOException e) {
				throw new SensitivePatientException(e);
			}
		}
	}

	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
		logger = runtime.getLogger(this.getClass());
	}
	
	public Document getSensitiveMessageModel() {
		return document;
	}

	private synchronized void setMessage(String text) {
		try {
			document.remove(0, document.getLength());
			document.insertString(0, text, null);
		}
		catch (BadLocationException e) {
			logger.error("Error setting the detail message text for a sensitive patient.", e);
		}
	}
	
	private void setupList() {
		Thread setupListThread = new Thread(new Runnable() {
			public void run() {
				sensitiveList.setChanging(true);
				sensitiveList.clearList();
				sensitiveList.setChanging(false);
				dataDAO.getList(sensitiveList);
			}
		}, "Query_Setup_Sensitive_List");
		setupListThread.start();
	}

	public void setSensitiveDataDAO(ISensitiveDataDAO dataDAO) {
		this.dataDAO = dataDAO;
	}

	public void remove(Object patientKey) throws SensitivePatientException {
		try {
			dataDAO.remove(patientKey);
		}
		catch (QueryDAOException e) {
			throw new SensitivePatientException(e);
		}
	}

}
