/*
 * Initially Created on Jun 5, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.EventListener;
import java.util.List;

import javax.swing.event.EventListenerList;

/**
 * Implements the sensitive patient list interface.
 */
public class SensitiveList implements ISensitiveList {
	private List list = new ArrayList();
	private EventListenerList eventListeners = new EventListenerList();
	private boolean changing = false;

	public SensitiveList() {
		super();
	}

	public void add(ISensitiveListItem item) {
		list.add(item);
		fireChanged();
	}

	public ISensitiveListItem getItem(int index) {
		if (index>-1 && index<list.size()) {
			return (ISensitiveListItem)list.get(index);
		}
		else {
			return null;
		}
	}

	public int getSize() {
		return list.size();
	}
	
	public void checkAll() {
		changeSelection(true);
	}

	private void changeSelection(boolean selection) {
		for (int i = 0; i < list.size(); i++) {
			((ISensitiveListItem)list.get(i)).setChecked(selection);
		}
		fireChanged();
	}
	
	public void clearAllChecks() {
		changeSelection(false);
	}
	
	public void addActionListener(ActionListener listener) {
		eventListeners.add(ActionListener.class, listener);
	}

	public void removeActionListener(ActionListener listener) {
		eventListeners.remove(ActionListener.class, listener);
	}

	private void fireChanged() {
		if (!changing && eventListeners.getListenerCount()>0) {
			ActionEvent event = new ActionEvent(this,0,null);
			ActionListener[] ala = (ActionListener[])eventListeners.getListeners(ActionListener.class);
			for (int i = 0; i < ala.length; i++) {
				ala[i].actionPerformed(event);
			}
		}
	}

	public void removeAllListeners() {
		while (eventListeners.getListenerCount()>0) {
			Object[] list = eventListeners.getListenerList();
			Object type = list[0];
			Object listener = list[1];
			eventListeners.remove((Class)type, (EventListener)listener);
		}
	}

	public void clearList() {
		list.clear();
		fireChanged();
	}

	public void setChanging(boolean changing) {
		boolean oldChanging = this.changing;
		this.changing = changing;
		if (!changing && oldChanging) {
			fireChanged();
		}
	}

	public boolean isChanging() {
		return changing;
	}

}
