/*
 * Initially Created on Jun 3, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.actions;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.dao.IExportAccessDAO;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Builds the querytool actions.
 */
public class QueryToolActionFactory {

	public static final Map buildActionSet(IQueryTool queryTool, ILogger logger) {
		Map actionMap = new HashMap();
		
		actionMap.put(IQueryTool.SELECT_ACTION, new Select(queryTool));
		actionMap.put(IQueryTool.UNSELECT_ACTION, new Unselect(queryTool));
		actionMap.put(IQueryTool.HOME_ACTION, new Home(queryTool));
		actionMap.put(IQueryTool.EDIT_ACTION, new Edit(queryTool));
		actionMap.put(IQueryTool.CREATE_ACTION, new Create(queryTool));
		actionMap.put(IQueryTool.EDIT_PARM_ACTION, new EditParameter(queryTool));
		actionMap.put(IQueryTool.EDIT_PARM_COMPLETE_ACTION, new EditParmComplete(queryTool));
		actionMap.put(IQueryTool.RUN_ACTION, new RunStart(queryTool));
		actionMap.put(IQueryTool.DELETE_ACTION, new Delete(queryTool));
		actionMap.put(IQueryTool.DELETE_CONFIRMED_ACTION, new DeleteConfirmed(queryTool));
		actionMap.put(IQueryTool.RENAME_ACTION, new Rename(queryTool));
		actionMap.put(IQueryTool.SAVEAS_ACTION, new SaveAs(queryTool));
		actionMap.put(IQueryTool.SAVEAS_CONFIRMED_ACTION, new SaveAsConfirmed(queryTool));
		actionMap.put(IQueryTool.EDIT_COMPLETE_SAVE_ACTION, new EditCompleteSave(queryTool));
		actionMap.put(IQueryTool.EDIT_COMPLETE_NOSAVE_ACTION, new EditCompleteNoSave(queryTool));
		actionMap.put(IQueryTool.EDIT_CANCEL_ACTION, new EditCancel(queryTool));
		actionMap.put(IQueryTool.STOP_ACTION, new Stop(queryTool));
		actionMap.put(IQueryTool.RUN_CONTINUE_ACTION, new RunContinue(queryTool));
		actionMap.put(IQueryTool.RUN_COMPLETE_ACTION, new RunComplete(queryTool));
		actionMap.put(IQueryTool.RUN_COMPLETE_MESSAGE_ACTION, new RunCompleteMessage(queryTool));
		actionMap.put(IQueryTool.RUN_COMPLETE_ERROR_ACTION, new RunCompleteError(queryTool));
		actionMap.put(IQueryTool.RUN_COMPLETE_NON_SENSITIVE_ACTION, new RunCompleteNonSensitive(queryTool));
		actionMap.put(IQueryTool.EXPORT_ACTION, new Export(queryTool));
		actionMap.put(IQueryTool.PRINT_ACTION, new Print(queryTool));
		actionMap.put(IQueryTool.SORT_COLUMN_ACTION, new SortColumn(queryTool));
		actionMap.put(IQueryTool.SELECT_ALL_ACTION, new SelectAllSensitive(queryTool));
		actionMap.put(IQueryTool.SELECT_NONE_ACTION, new SelectNoneSensitive(queryTool));
		actionMap.put(IQueryTool.SELECT_SENSITIVE_ACTION, new SelectSensitive(queryTool));
		
		for (Iterator i = actionMap.values().iterator(); i.hasNext();) {
			((IQueryAction)i.next()).setLogger(logger);
		}
		
		return actionMap;
	}
	
	public static final Map getAuthorizedActions(IRuntimeContext runtime) {
		Map map = new HashMap();
		try {
			IDaoRegistry reg = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
			IExportAccessDAO exportAccessDAO = (IExportAccessDAO)reg.getDAO(IExportAccessDAO.class.getName());
			map.put(IQueryTool.EXPORT_ACTION, Boolean.valueOf(exportAccessDAO.getResult()));
			map.put(IQueryTool.PRINT_ACTION, Boolean.valueOf(exportAccessDAO.getResult()));
		}
		catch (Exception e) {
			runtime.getLogger(QueryToolActionFactory.class).error("Error setting up authorized actions table.", e);
		}
		
		return map;
	}
	
}
