/*
 * Initially Created on Apr 22, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.dao.factory;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.dao.IDetailDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

/**
 * Returns a specifiec detail DAO based on the detail DAO mappings in the registry.
 */
public class DetailDAOFactory {

	/**
	 * Returns a specifiec detail DAO based on the detail DAO mappings in the registry.
	 * @param daoID
	 * @return detailDAO
	 */
	public static IDetailDAO getDetailDAO(IRuntimeContext runtime, Object daoID) throws QueryDAOException {
		IDetailDAO detailDAO = null;
		IPluginRegistry pluginRegistry = runtime.getPluginRegistry();
		IConfigurationElement[] extensions =
			pluginRegistry.getConfigurationElementsFor(IQueryTool.DETAIL_DAO_MAPS_EXTENSION_POINT_ID);

		for (int i = 0; i < extensions.length; i++) {
			String key = extensions[i].getAttribute(IQueryTool.ID_ATTRIBUTE);
			if (key!=null && key.equals(daoID)) {
				String interfaceName = extensions[i].getAttribute(IQueryTool.INTERFACE_ATTRIBUTE);
				if (interfaceName!=null && interfaceName.length()>0) {
					try {
						IDaoRegistry daoRegistry = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
						detailDAO = (IDetailDAO)daoRegistry.getDAO(interfaceName);
					}
					catch (Exception e) {
						throw new QueryDAOException("Interface not found: " + interfaceName);
					}
				}
				else {
					throw new QueryDAOException("Interface not found: " + interfaceName);
				}
			}
		}

		return detailDAO;
	}
	
}
