/*
 * Initially Created on Apr 7, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.AbstractQueryList;
import gov.va.med.hds.querytool.IQueryID;
import gov.va.med.hds.querytool.IQueryList;
import gov.va.med.hds.querytool.ISoftListItem;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.QueryListException;
import gov.va.med.hds.querytool.dao.IQueryListDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.collections.ListUtil;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Abstract M based class for processing query lists.
 */
public abstract class AbstractQueryListDAO extends AbstractMDAO implements IQueryListDAO {
	private IQueryList queryList;
	private static final String RPTLST_RPC_KEY = "ORRHCR RPTLST";
	protected final char SEPARATOR = '^';

	public AbstractQueryListDAO() {
		super();
	}

	protected void getQueryList() throws QueryDAOException {
		if (queryList==null) {
			queryList = new AbstractQueryList();
		}
		try {
			mumps.callRPC(RPTLST_RPC_KEY,new IMumpsListener() {
				public void CallSucceeded(ICallRequest request, String result) {
					processData(result);
				}
				public void CallFailed(ICallRequest request, MumpsException ex) {
					logError("M call queue error on calling RPC " + RPTLST_RPC_KEY + ":"+ex.getMessage());
				}
			});
		}
		catch (Exception e) {
			logError("M call queue error on RPC " + RPTLST_RPC_KEY + ":" + e.getMessage());
		}
	}

	protected abstract void processData(String result);
	
	protected void removeUnfoundQueries(Map queryMap) {
		List removeList = new ArrayList();
		ListUtil.copy(queryList.getAsList(), removeList);
		int i = 0;
		while (i<removeList.size()) {
			ISoftListItem item = (ISoftListItem)removeList.get(i);
			if (queryMap.containsKey(item.getID().toString())) {
				removeList.remove(i);
			}
			else {
				if (!QueryID.isValid((IQueryID)item.getID())) {
					removeList.remove(i);
				}
				else {
					i++;
				}
			}
		}
		for (Iterator j = removeList.iterator(); j.hasNext();) {
			ISoftListItem item = (ISoftListItem)j.next();
			try {
				queryList.removeElement(item);
			}
			catch (QueryListException e) {
				logError("Could not remove item from a query list:" + item.toString());
			}
		}
	}

	protected void addUnfoundQueries(Map queryMap) {
		for (int i = 0; i < queryList.getSize(); i++) {
			ISoftListItem item = (ISoftListItem)queryList.getItemAt(i);
			if (queryMap.containsKey(item.getID().toString())) {
				queryMap.remove(item.getID().toString());
			}
		}
		for (Iterator i = queryMap.keySet().iterator(); i.hasNext();) {
			String key = (String)i.next();
			try {
				queryList.addElement(new QueryID(key), (String)queryMap.get(key));
			}
			catch (QueryListException e) {
				logError("Error creating new query list item:"+key);
			}
		}
	}
	
	protected void updateQueries(Map queryMap) {
		for (int i = 0; i < queryList.getSize(); i++) {
			ISoftListItem item = (ISoftListItem)queryList.getItemAt(i);
			String string = null;
			if (((string=(String)queryMap.get(item.getID().toString()))!=null) && (!string.equals(item.getDisplayName()))) {
				item.setDisplayName(string);
			}
		}
	}

	protected void logError(String message) {
		ILogger logger = runtime.getLogger(this.getClass());
		logger.error(message);
	}
	
	public void refresh(IQueryList list) throws QueryDAOException {
		queryList = list;
		getQueryList();
	}

}
