/*
 * Initially Created on Jun 5, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.clinicaldata.IPatientDemographics;
import gov.va.med.hds.clinicaldata.dao.IPatientDemographicsDAO;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IPatDetailReceivable;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IReportRow;
import gov.va.med.hds.querytool.dao.ICompleteQueryDataDAO;
import gov.va.med.hds.querytool.dao.IDetailDAO;
import gov.va.med.hds.querytool.dao.IPatDetailDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.querytool.dao.factory.DetailDAOFactory;
import gov.va.med.hds.querytool.dao.formatter.IFormatter;
import gov.va.med.hds.util.mumps.NameValuePair;
import gov.va.med.hds.util.mumps.StringUtil;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.SwingUtilities;

/**
 * Complete query data DAO for Mumps.
 */
public class CompleteQueryDataDAO extends AbstractMDAO implements ICompleteQueryDataDAO {
	private static final int MAX_RPC_TIME = 300000;
	private static final String GET_REPORT_ROW_CALL = "SUBDTA";
	private static final String NO_DETAIL_AVAILABLE = "No detail available.";
	private Map formatters;
	private IQuery query;
	private List parms = new ArrayList();
	private IDaoRegistry daoRegistry;
	private Map detailDaoMap = new HashMap();
	private IPatDetailDAO patDetailDAO;

	public CompleteQueryDataDAO() {
		super();
	}

	public IReportRow[] getReportRow(int fromIndex, int toIndex)
		throws QueryDAOException {
		parms.clear();
		parms.add(Integer.toString(fromIndex + 1));
		parms.add(Integer.toString(toIndex + 1));
		String result = null;
		try {
			result = mumps.callRPC(GET_REPORT_ROW_CALL, parms);
		}
		catch (Exception e) {
			throw new QueryDAOException("Could not query rows from database.");
		}

		IReportRow[] rows = new IReportRow[Math.abs(fromIndex - toIndex) + 1];
		BufferedReader reader = new BufferedReader(new StringReader(result));
		String line = null;
		NameValuePair pair = null;
		int i = -1;
		int col = -1;
		Object key = null;
		try {
			while ((line = reader.readLine()) != null) {
				pair = new NameValuePair(line);
				if (pair.getLeft().equals("RowItemID")) {
					i++;
					rows[i] = new ReportRow(pair.getRight());
					col = -1;
				}
				else {
					col++;
					if ((query != null)
						&& (formatters
							.containsKey(key=query.getDisplayFields().get(col).getID()))) {
						rows[i].addItem(
							((IFormatter)formatters.get(key)).format(
								pair.getRight()));
					}
					else {
						rows[i].addItem(pair.getRight());
					}
				}
			}
		}
		catch (IOException e) {
			throw new QueryDAOException(e);
		}
		return rows;
	}

	public void detailResultsRequest(IReportRow row,
									IDetailReceivable dataUser)
									throws QueryDAOException {
		if (row==null || row.getRowID()==null) {
			dataUser.setDetail(NO_DETAIL_AVAILABLE);
			return;
		}
		String id = StringUtil.piece(row.getRowID().toString(), ':', 1);
		String dataID = null;
		IDetailDAO detailDAO = null;
		if (detailDaoMap.get(id)==null) {
			detailDaoMap.put(id, DetailDAOFactory.getDetailDAO(runtime, id));
			if (detailDaoMap.get(id)==null) {
				dataUser.setDetail(NO_DETAIL_AVAILABLE);
				return;
			}
		}
		detailDAO = (IDetailDAO)detailDaoMap.get(id);
		dataID = StringUtil.piece(row.getRowID().toString(), ':', 2);
		if (detailDAO != null) {
			detailDAO.getDetailRequest(dataID, dataUser);
		}

	}

	public void patientInfoRequest(IReportRow row,
									final IPatDetailReceivable dataUser)
									throws QueryDAOException {
		if (row==null || row.getRowID()==null || row.getRowID().toString().length()<1) return;
		String rowID = row.getRowID().toString();
		rowID = rowID.replaceAll("CST", "ORD");
		if (patDetailDAO==null) {
			try {
				patDetailDAO = (IPatDetailDAO)daoRegistry.getDAO(IPatDetailDAO.class.getName());
			}
			catch (Exception e) {
				throw new QueryDAOException(e);
			}
		}
		patDetailDAO.getDetailRequest(rowID, new IDetailReceivable() {
			public void setDetail(String text) {
				setPatientData(dataUser, text);
			}
		});
	}

	private void setPatientData(final IPatDetailReceivable dataUser,
									final String text) {
		if ((text == null) || (text.length() < 1)) return;
		
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					setPatientData(dataUser, text);
				}
			});
		}
		else {
			try {
				IPatientDemographicsDAO patientDao = (IPatientDemographicsDAO)daoRegistry.getDAO(IPatientDemographicsDAO.class.getName());
				IPatientDemographics patient = patientDao.createPatient(text);
				dataUser.setPatient(patient);
			} catch (Exception e) {
				runtime.getLogger(this.getClass().getName()).error("Couldn't get an instance of "+IPatientDemographics.class.getName()+" from DaoRegistry.",e);
			}
		}
	}

	public void setSortField(IDisplayField field, boolean ascending) throws QueryDAOException {
		parms.clear();
		parms.add(field.getID());
		parms.add(ascending ? "1" : "0");
		try {
			mumps.callRPC("ORRHCQ SORTBY", parms,MAX_RPC_TIME);
		}
		catch (Exception e) {
			throw new QueryDAOException(e);
		}
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName,
		Object data)
		throws Exception {
			
		super.setInitializationData(runtime, config, propertyName, data);

		daoRegistry = (IDaoRegistry)runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
	}

	private void loadLineFormatters() {
		if (formatters == null) {
			formatters = new HashMap();
		}
		formatters.clear();
		for (int i = 0; i < query.getDisplayFields().size(); i++) {
			String fieldName = query.getDisplayFields().get(i).getID().toString();
			int pos = -1;
			if ((pos = fieldName.indexOf('.')) > -1) {
				String formatName =
					fieldName.substring(0, pos).toLowerCase()
						+ ".I"
						+ fieldName.substring(pos + 1);
				if (formatters.containsKey(fieldName)) {
					continue;
				}
				IFormatter formatter = null;
				try {
					formatter =
						(IFormatter)daoRegistry.getDAO(
							IFormatter.PROJECT_PREFIX + "." + formatName);
				}
				catch (Exception e) {
					throw new RuntimeException(
						"Error retrieving data formatter for " + fieldName);
				}
				if (formatter != null) {
					formatters.put(fieldName, formatter);
				}
			}
		}
	}
	
	public void setQuery(IQuery query) {
		this.query = query;
		loadLineFormatters();
	}

}
