package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.dao.IDetailDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.util.LinkedList;
import java.util.List;

import javax.swing.SwingUtilities;

/**
 * Abstract M detail DAO class.  Provides a detail string to listeners based on an
 * identifier.
 */
public abstract class MAbstractDetailDAO extends AbstractMDAO implements IDetailDAO {
	protected List parms = new LinkedList();
	protected IDetailReceivable detailReceivable;
	protected ILogger logger;

	/**
	 * Constructor for MAbstractDetailDAO.
	 */
	public MAbstractDetailDAO() {
		super();
	}

	public void getDetailRequest(Object id, IDetailReceivable target) throws QueryDAOException {
		this.detailReceivable = target;
		parms.clear();
		parms.add(id);
//		System.out.println(id.toString());
		try {
			mumps.callRPC(getCallString(),parms,new IMumpsListener() {
				public void CallSucceeded(ICallRequest request, String result) {
					callDone(request, result);
				}
				public void CallFailed(ICallRequest request, MumpsException e) {
					callFailed(request, e);
				}
			});
		}
		catch (Exception e) {
			throw new QueryDAOException(e);
		}
	}

	private void callDone(final ICallRequest request, final String result) {
		if (SwingUtilities.isEventDispatchThread()) {
			callSucceeded(request, result);
		}
		else {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					callSucceeded(request, result);
				}
			});
		}
	}
	
	public abstract String getCallString();
	
	public void callSucceeded(ICallRequest request, String result) {
		this.detailReceivable.setDetail(result);
	}
	
	public void callFailed(ICallRequest request, MumpsException e) {
		logger.fatal("Error getting detail data on " + request.getCallName(), e);
	}
	
	public void setInitializationData(IRuntimeContext runtime,
										IConfigurationElement config,
										String propertyName, Object data) throws Exception {
		super.setInitializationData(runtime, config, propertyName, data);											
		logger = runtime.getLogger(getClass());
	}

}
