package gov.va.med.hds.querytool.m;

import gov.va.med.hds.querytool.dao.INameValuePairDAO;
import gov.va.med.hds.querytool.dao.QueryDataDAOException;
import gov.va.med.hds.util.mumps.StringUtil;

import java.io.BufferedReader;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author vhaislmonsos
 *
 * Dec 17, 2002 12:42:26 PM
 */
public abstract class MAbstractNameValueDAO extends AbstractMDAO implements INameValuePairDAO {
	private static final String ID2EXT_RPC_KEY = "ID2EXT";
	private TreeMap tree = new TreeMap();

	/**
	 * @see gov.va.med.hds.querytool.INameValuePairDAO#getKey()
	 */
	public Object getKey(Object text) throws QueryDataDAOException {
		return StringUtil.piece(text.toString(), '^', 1);
	}

	/**
	 * @see gov.va.med.hds.querytool.INameValuePairDAO#getValue()
	 */
	public Object getValue(Object text) throws QueryDataDAOException {
		return StringUtil.piece(text.toString(), '^', 2);
	}

	/**
	 * @see gov.va.med.hds.querytool.dao.INameValuePairDAO#getSortedMap(Map)
	 */
	public TreeMap get(Map items) throws QueryDataDAOException {
		if (items.size() > 0) {
			String result = null;
			List parms = new LinkedList();
			parms.add(getFileNumString());
			parms.add(items.keySet());
			try {
				result = mumps.callRPC(ID2EXT_RPC_KEY, parms);
			}
			catch (Exception e) {
				throw new QueryDataDAOException(e);
			}
			BufferedReader reader = new BufferedReader(new StringReader(result));
			items.clear();
			String line = null;
			try {
				while ((line=reader.readLine())!=null) {
					String value = getValue(line).toString();
					String key = getKey(line).toString();
					if ((value.length() > 0) && (key.length()>0)) {
						items.put(key,value);
						tree.put(value+key,new Entry(key,value));
					}
				}
			}
			catch (Exception e) {
				throw new QueryDataDAOException("Invalid database data.",e);
			}
		}
		return tree;
	}

	protected abstract String getFileNumString();
}
