package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;

import java.io.BufferedReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.LinkedList;

/**
 * General buffer list DAO for M long list data routines.
 */
public class MBufferListDAO implements IDataNeededListener {
	public static final int FORWARD_DIRECTION = 1;
	public static final int REVERSE_DIRECTION = -1;
	private ArrayList parms;
	private IMBufferListCall mBufferListCall;
	private IBufferModel bufferModel;
	private Object filter;
	private IRuntimeContext runtime;
	private IMumps dbConnection;
	
	private MBufferListDAO() {
		super();
	}
	
	public MBufferListDAO(IMBufferListCall call, IMumps mumps, IRuntimeContext runtime) {
		this();
		this.runtime = runtime;
		mBufferListCall = call;
		this.dbConnection = mumps;
	}

	private Object[] getCallResults(int size, String startString, int direction) throws DataProviderException {
		String result = null;
		LinkedList allResults = new LinkedList();
		parms = new ArrayList();
		int totalResults = 0;
		
		while (totalResults < size) {
			mBufferListCall.clearParms();
			mBufferListCall.setStartParm(startString.toUpperCase());
			mBufferListCall.setDirectionParm(direction);
			mBufferListCall.setFilterParm(filter);
			parms.clear();
			parms.addAll(mBufferListCall.getParms());
			try {
				result = dbConnection.callRPC(mBufferListCall.getCallName(), parms);
			}
			catch (Exception e) {
				throw new DataProviderException(e);
			}
			BufferedReader reader = new BufferedReader(new StringReader(result));
			IBufferItem item = null;
			String line;
			int lineCount = 0;
			try {
				while ((line=reader.readLine())!=null) {
					item = new BufferItem(mBufferListCall.getValue(line),mBufferListCall.getKey(line));
					allResults.addLast(item);
					totalResults += 1;
					lineCount++;
				}
			}
			catch (Exception e) {
				throw new DataProviderException("Error in database data"+result,e);
			}
			if (lineCount < mBufferListCall.getExpectedCallSize()) {
				break;
			}
			startString = item.getValue().toString();
		}
		return allResults.toArray();
	}

	private Object[] getTop(int size) throws DataProviderException {
		return getCallResults(size,"\n",FORWARD_DIRECTION);
	}

	private Object[] getBottom(int size) throws DataProviderException {
		return getCallResults(size,"~",REVERSE_DIRECTION);
	}

	private Object[] getNext(Object itemStart, int direction, int count) throws DataProviderException {
		Object[] results = getCallResults(count,itemStart.toString(),direction);
		return results;
	}

	/**
	 * Returns the mBufferListCall.
	 * @return IMBufferListCall
	 */
	public IMBufferListCall getMBufferListCall() {
		return mBufferListCall;
	}

	/**
	 * Sets the mBufferListCall.
	 * @param mBufferListCall The mBufferListCall to set
	 */
	public void setMBufferListCall(IMBufferListCall mBufferListCall) {
		this.mBufferListCall = mBufferListCall;
	}

	public void getDataRequest(DataNeededEvent event) throws DataProviderException {
		final DataNeededEvent e = event;
		Thread daoThread = new Thread(new Runnable() {
			public void run() {
				try {
					processRequest(e);
				}
				catch (DataProviderException e) {
					runtime.getLogger(this.getClass()).error(mBufferListCall.getCallName()+" M call failed.", e);
				}
			}
		}, "Querytool_MBufferListDAO_Call");
		daoThread.start();
	}
		
	private void processRequest(DataNeededEvent event) throws DataProviderException {
		Object[] items = null;
		if (event.isDataSearch()) {
			String searchText = (event.getStartingAt().toString().toUpperCase());
			while (searchText!=null && searchText.length()>0) {
				items = getNext(searchText, IBufferModel.FORWARD_DIRECTION, 1);
				if (items.length<1) {
					event.setDataSearch(false);
					event.setStartingAt(null);
					processRequest(event);
					items = null;
					break;
				}
				if (items[0].toString().toUpperCase().startsWith(searchText)) {
					break;
				}
				if (items[0].toString().toUpperCase().compareTo(searchText)>0 && items[items.length-1].toString().toUpperCase().compareTo(searchText)<0) {
					int matchingIndex = findCloseIndex(items, searchText);
					if (items.length-matchingIndex<event.getLength()) {
						event.setDataSearch(false);
						event.setStartingAt(items[matchingIndex]);
						processRequest(event);
						items = null;
					}
					break;
				}
				
				searchText = searchText.substring(0, searchText.length()-1);
			}
			
			if (searchText!=null && searchText.length()<1) {
				event.setDataSearch(false);
				event.setStartingAt(null);
				processRequest(event);
				items = null;
			}
			
		}
		else {
			if (event.getStartingAt()==null) {
				//Get TOP or BOTTOM
				if (event.getDirection()>0) {
					//Get TOP
					items = getTop(event.getLength());
				}
				else {
					//Get BOTTOM
					items = getBottom(event.getLength());
				}
			}
			else {
				//Get middle part somewhere
				items = getNext(event.getStartingAt(),event.getDirection(),event.getLength());
			}
		}
		if (items != null) {
			bufferModel.setData(items,event.getLength());
		}
	}

	private int findCloseIndex(Object[] items, String searchText) {
		for (int i = 0; i < items.length; i++) {
			if (items[i].toString().compareToIgnoreCase(searchText)>=0) {
				if (i>0) {
					return i-1;
				}
				else {
					return i;
				}
			}
		}
		return items.length-1;
	}

	public void setBufferModel(IBufferModel bufferModel) {
		this.bufferModel = bufferModel;
		this.bufferModel.addDataNeededListener(this);
	}

	public void setFilter(Object filter) {
		this.filter = filter;
	}

}
