package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.querytool.dao.IDisplayGroupTreeDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.mumps.StringUtil;

import java.util.TreeMap;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

/**
 * Builds the query constraint group tree from the M data source
 */
public class MDisplayGroupTreeDAO extends AbstractMDAO implements IDisplayGroupTreeDAO {
	private static final String ALLTREE_RPC_KEY = "ALLTREE";

	/**
	 * @see gov.va.med.hds.querytool.dao.ITreeModelDAO#getTree()
	 */
	public TreeModel getTree() throws QueryDAOException {
		TreeMap nodeMap = new TreeMap();
		String callResult = null;
		try {
			callResult = mumps.callRPC(ALLTREE_RPC_KEY);
		} catch (Exception e) {
			throw new QueryDAOException(e);
		}
		String[] results = null;
		if (callResult.length() > 0) {
			results = callResult.split("\\r\\n|\\n");
		} else {
			results = new String[0];
		}

		TextNode textNode = new TextNode(results[0]);
		IBufferItem item = new BufferItem(textNode.getValue(), textNode.getNodeID());
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(item);
		nodeMap.put(textNode.getNodeID(), root);
		TreeModel result = new DefaultTreeModel(root);
		for (int i = 1; i < results.length; i++) {
			textNode = new TextNode(results[i]);
			item = new BufferItem(textNode.getValue(), textNode.getNodeID());
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(item);
			nodeMap.put(textNode.getNodeID(), node);
			DefaultMutableTreeNode parent =
				(DefaultMutableTreeNode) nodeMap.get(textNode.getParentID());
			parent.add(node);
		}
		return result;
	}

	private class TextNode {
		private String nodeID;
		private String value;
		private String parentID;

		private TextNode() {
		}

		public TextNode(String text) {
			this.nodeID = StringUtil.piece(text, '^', 1);
			this.value = StringUtil.piece(text, '^', 2);
			this.parentID = StringUtil.piece(text, '^', 3);
		}

		public String getNodeID() {
			return nodeID;
		}

		public String getParentID() {
			return parentID;
		}

		public String getValue() {
			return value;
		}

	}

}
