package gov.va.med.hds.querytool.m;

import gov.va.med.hds.querytool.DisplayField;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.dao.IFieldDefnDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.mumps.NameValuePair;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * M implementation of the field definition DAO.
 */
public class MFieldDefnDAO extends AbstractMDAO implements IFieldDefnDAO {
	private static final String DFLDS_RPC_KEY = "DFLDS";

	private List getListFromLines(final String result)
		throws IllegalArgumentException, NullPointerException {
		BufferedReader reader = new BufferedReader(new StringReader(result));
		String line;
		final List results = new ArrayList();
		IDisplayField field = null;
		int sampleCount = 0;
		try {
			while ((line=reader.readLine())!=null) {
				NameValuePair pair = new NameValuePair(line);
				if (pair.getLeft().equals("DisplayName")) {
					if (field!=null) {
						sampleCount = 0;
						results.add(field);
					}
					field = new DisplayField();
					field.setListDisplayName(pair.getRight());
				}
				else if (pair.getLeft().equals("InternalName")) {
					field.setID(pair.getRight());
				}
				else if (pair.getLeft().equals("HeaderName")) {
					field.setDisplayText(pair.getRight());
				}
				else if (pair.getLeft().equals("SortType")) {
					field.setSortType(pair.getRight());
				}
				else if (pair.getLeft().equals("SampleData")) {
					field.getSampleData()[sampleCount++] = pair.getRight();
				}
			}
			if (field!=null) {
				results.add(field);
			}
		}
		catch (IOException e) {
		}
		return results;
	}

	public List getFields(Collection fieldTypes) throws QueryDAOException {
		String request = new String();
		for (Iterator i = fieldTypes.iterator(); i.hasNext();) {
			request += i.next().toString().substring(0, 1);
		}
		List parms = new LinkedList();
		parms.add(request);
		String results = null;
		try {
			results = mumps.callRPC(DFLDS_RPC_KEY,parms);
		}
		catch (Exception e) {
			throw new QueryDAOException("M call 'ORRHCU DFLDS' failed: parameter - " + parms,e);
		}
		return getListFromLines(results);
	}
	
}
