package gov.va.med.hds.querytool.m;

import gov.va.med.hds.querytool.dao.IRunDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.mumps.StringUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * M implementation of the calls to run a query from a given query definition.
 */
public abstract class MRunDAO extends AbstractMDAO implements IRunDAO {
	private static final int M_RPC_TIMEOUT = 300000;
	protected String iterator;
	protected boolean finished = false;
	protected boolean patientSelected = false;
	protected int recordCount = 0;
	private List parms = new ArrayList();

	public MRunDAO() {
		super();
	}

	public void getNextIteration() throws QueryDAOException {
		parms.clear();
		parms.add(iterator);
		String result = null;
		try {
//						System.out.println("Iterator:" + iterator);
			result = mumps.callRPC(getIteratorCall(), parms,  M_RPC_TIMEOUT);
		}
		catch (Exception e) {
			throw new QueryDAOException(
				"Error iterating through the query: RPC:" +getIteratorCall()+"("+iterator+")" ,e);
		}
		iterator = StringUtil.piece(result, '^', 3);
		
		if (iterator.length() < 1) {
			finished = true;
			patientSelected = false;
			recordCount = 0;
		}
		
		if (StringUtil.piece(result, '^', 1).equals("1")) {
			patientSelected = true;
		}
		else {
			patientSelected = false;
		}
		
		if ((result == null) || (result.length() < 1)) {
			recordCount = 0;
		}
		else {
			String countString = StringUtil.piece(result, '^', 2);
			if (countString.length() < 1) {
				recordCount = 0;
			}
			else {
				recordCount = Integer.parseInt(countString);
			}
		}
	}

	protected abstract String getIteratorCall();

	public boolean isPatientSelected() {
		return patientSelected;
	}

	public int getSize() {
		return recordCount;
	}

	public boolean isReportFinished() {
		return finished;
	}

	public void setInitialIterator(Object iterator) {
		this.iterator = iterator.toString();
		if ((this.iterator != null) && (this.iterator.length() < 1)) {
			finished = true;
		}
		else {
			finished = false;
		}
		recordCount = 0;
		patientSelected = false;
	}
	
}
