/*
 * Initially Created on Jun 5, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.m;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.IPrivacyLevel;
import gov.va.med.hds.querytool.ISensitiveList;
import gov.va.med.hds.querytool.ISensitiveListItem;
import gov.va.med.hds.querytool.PrivacyLevel;
import gov.va.med.hds.querytool.SensitiveListItem;
import gov.va.med.hds.querytool.dao.ISensitiveDataDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.mumps.StringUtil;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Sensitive patient cohort DAO for a query run in Mumps.
 */
public class SensitiveDataDAO extends AbstractMDAO implements ISensitiveDataDAO {
	private static final String GET_SENSITIVE_MESSAGE		= "ORRHCQ1 GETSLN";
	private static final String REMOVE_SENSITIVE_PATIENT		= "ORRHCQ1 DELSEN";
	private static final String GET_SENSITIVE_PATIENT_INFO	= "ORRHCQ1 GETSPT";
	private static final String GET_SENSITIVE_SIZE			= "ORRHCQ1 GETCNT";
	private IDetailReceivable messageReceiver;
	private IMumpsListener messageProcessor = new IMumpsListener() {
		public void CallFailed(ICallRequest request, MumpsException e) {
			runtime.getLogger(SensitiveDataDAO.class).error("Error getting sensitive message from DB:"+request.toString(),e);
		}

		public void CallSucceeded(ICallRequest request, String result) {
			if (messageReceiver!=null) {
				messageReceiver.setDetail(result);
			}
		}
	};

	public int getSize() throws QueryDAOException {
		int size = 0;
		try {
			String result = mumps.callRPC(GET_SENSITIVE_SIZE);
			size = Integer.parseInt(result.trim());
		}
		catch (Exception e) {
			throw new QueryDAOException("Error getting the count of sensitive patients for this query.");
		}
		return size;
	}

	private ISensitiveListItem[] getItems(ISensitiveListItem start, int size) throws QueryDAOException {
		List parms = new ArrayList();
		parms.add(start.getName() + "!" + start.getPatientID().toString());
		parms.add(Integer.toString(size));
		String result = null;
		try {
			result = mumps.callRPC(GET_SENSITIVE_PATIENT_INFO, parms);
		}
		catch (Exception e) {
			throw new QueryDAOException("Error retrieving the sensitive patient list, starting from " + start.getName() + start.getPatientID().toString(), e);
		}
		
		BufferedReader reader = new BufferedReader(new StringReader(result));
		
		String line = null;
		List list = new ArrayList();
		try {
			while ((line=reader.readLine())!=null) {
				ISensitiveListItem item = new SensitiveListItem();
				item.setName(StringUtil.piece(line, '^', 2));
				item.setPatientID(StringUtil.piece(line, '^', 1));
				int levelCode = Integer.parseInt(StringUtil.piece(line, '^', 3));
				String level = null;
				switch (levelCode) {
					case 0: {level = IPrivacyLevel.MINIMAL; break;}
					case 1: {level = IPrivacyLevel.LEVEL_1; break;}
					case 2: {level = IPrivacyLevel.LEVEL_2; break;}
					case 3: {level = IPrivacyLevel.EMPLOYEE; break;}
					case 4: {level = IPrivacyLevel.NO_SSN; break;}
					default: {level = IPrivacyLevel.MINIMAL;}
				}
				item.setPrivacyLevel(new PrivacyLevel(level));
				list.add(item);
			}
		}
		catch (IOException ex) {
			throw new QueryDAOException("Error processing DB call for the sensitive patient result string.");
		}
		
		return (ISensitiveListItem[])list.toArray(new ISensitiveListItem[0]);
	}

	public void remove(Object patientKey) throws QueryDAOException {
		String[] parm = new String[1];
		parm[0] = patientKey.toString();
		int code = 0;
		try {
			String result = mumps.callRPC(REMOVE_SENSITIVE_PATIENT, parm);
			code = Integer.parseInt(result.trim());
			if (code!=1) {
				throw new Exception("RPC returned an error code.");
			}
		}
		catch (Exception e) {
			throw new QueryDAOException("RPC error removing a sensitive patient from the query list.",e);
		}
	}

	public void getSensitiveMessage(Object patientKey,IDetailReceivable receivable)
			throws QueryDAOException {
				
		messageReceiver = receivable;
		String[] parm = new String[1];
		parm[0] = patientKey.toString();
		try {
			mumps.callRPC(GET_SENSITIVE_MESSAGE, parm, messageProcessor);
		}
		catch (Exception e) {
			throw new QueryDAOException("Error setting up database call for a patient sensitive message.");
		}
	}

	public void getList(ISensitiveList list) {
		try {
			int chunkSize = 50;
			ISensitiveListItem[] items = new ISensitiveListItem[1];
			items[0] = new SensitiveListItem("","",null,false);
			while (items.length>0) {
				items = getItems(items[items.length-1],chunkSize);
				for (int i = 0; i < items.length; i++) {
					list.add(items[i]);
				}
			}
		}
		catch (QueryDAOException e) {
			runtime.getLogger(this.getClass()).error("Error getting sensitive patient list from database.", e);
		}
	}

}
