package gov.va.med.hds.querytool.types;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.IValueFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * Stores database value ID's to IValue type mappings, as well as the editor type
 * information for a given IValue type.
 */
public class TypeFactory implements ITypeFactory {
	private IRuntimeContext runtime;
	private Map typeMap = new HashMap();
	private ILogger logger;

	public TypeFactory() {
	}
	
	public TypeFactory(IRuntimeContext runtime) {
		setRuntime(runtime);
	}
	
	public IValue getValue(Object editorID) {
		IValue value = null;
		if (typeMap.containsKey(editorID)) {
			Class valueClass = (Class)typeMap.get(editorID);
			Object reference = null;
			try {
				reference = valueClass.newInstance();
			}
			catch (Exception e) {
				logger.error("Could not create IValue for ID" + editorID.toString(), e);
				return null;
			}
			if (reference instanceof IValueFactory) {
				value = ((IValueFactory)reference).getValue(runtime);
			}
			else {
				value = (IValue)reference;
			}
			value.setEditorID(editorID);
		}
		return value;
	}

	public void addValueType(Object key, Class value) {
		typeMap.put(key, value);
	}

	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
		logger = runtime.getLogger(this.getClass());
		typeMap.clear();
		IConfigurationElement[] extensions = runtime.getPluginRegistry().
			getConfigurationElementsFor(IQueryTool.TYPES_EXTENSION_POINT_ID);

		for (int i = 0; i < extensions.length; i++) {
			String key = extensions[i].getAttribute(IQueryTool.ID_ATTRIBUTE);
			if (key!=null && key.length()>0) {
                ClassLoader classLoader =
                    extensions[i]
                        .getDeclaringExtension()
                        .getDeclaringPluginDescriptor()
                        .getPluginClassLoader();
				String typeName = extensions[i].getAttribute(IQueryTool.TYPE_ATTRIBUTE);
				if (typeName!=null && typeName.length()>0) {
					Class type = null;
					try {
						type = Class.forName(typeName, true, classLoader);
					}
					catch (ClassNotFoundException e) {
						logger.error("Type not found: " + typeName, e);
					}
					Object typeInstance = null;
					try {
						typeInstance = type.newInstance();
					}
					catch (Exception e2) {
						logger.error("Error creating type: " + type.getName(), e2);
					}
					if (typeInstance instanceof IValueFactory) {
						type = (((IValueFactory)typeInstance).getValue(runtime)).getClass();
					}
					typeMap.put(key, type);
				}
			}
		}
	}
	
}
