/*
 * Initially Created on Apr 8, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.querytool.AbstractQueryList;
import gov.va.med.hds.querytool.IQueryList;
import gov.va.med.hds.querytool.IQueryListEvent;
import gov.va.med.hds.querytool.IQueryListListener;
import gov.va.med.hds.querytool.ISoftListItem;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.QueryListException;
import gov.va.med.hds.querytool.SoftListException;
import junit.framework.TestCase;

/**
 * Test suite for the AbstractQueryList.
 */
public class AbstractQueryListTest extends TestCase {
	private IQueryList queryList;
	private int addedCount;
	private int removedCount;
	private int changedCount;
	private IQueryListListener listener = new IQueryListListener() {
		public void listChanged(IQueryListEvent event) {
			switch(event.getEventID()) {
				case (IQueryListEvent.INTERVAL_ADDED):{
					addedCount++;
					break;
				}
				case (IQueryListEvent.INTERVAL_REMOVED):{
					removedCount++;
					break;
				}
				case (IQueryListEvent.INTERVAL_CHANGED):{
					changedCount++;
					break;
				}
			}
		}
	};
	
	private void resetCounts() {
		addedCount=0;
		removedCount=0;
		changedCount=0;
	}
	
	public AbstractQueryListTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AbstractQueryListTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		queryList = new AbstractQueryList();
	}

//	public void testAddElementAt() {
//	}

	public void testAddElement() throws SoftListException, QueryListException {
		queryList.addElement(new QueryID("id3"), "zelda");
		queryList.addElement(new QueryID("id2"), "about");
		queryList.addElement(new QueryID("id1"), "descr");
		assertEquals("AbstractQueryList.addElement failed:",3,queryList.getSize());
		ISoftListItem testItem = (ISoftListItem)queryList.getItemAt(0);
		assertEquals("AbstractQueryList.addElement failed:","about",testItem.getDisplayName());
		testItem = (ISoftListItem)queryList.getItemAt(1);
		assertEquals("AbstractQueryList.addElement failed:","descr",testItem.getDisplayName());
		testItem = (ISoftListItem)queryList.getItemAt(2);
		assertEquals("AbstractQueryList.addElement failed:","zelda",testItem.getDisplayName());
	}

	public void testRemoveElementAt() throws SoftListException, QueryListException {
		testAddElement();
		queryList.removeElementAt(0);
		queryList.removeElementAt(1);
		ISoftListItem item = (ISoftListItem)queryList.getItemAt(0);
		assertEquals("AbstractQueryList.addElement failed:","descr",item.getDisplayName());
	}

	public void testRemoveElement() throws SoftListException, QueryListException {
		testAddElement();
		queryList.removeElement(new QueryID("id1"));
		assertEquals("AbstractQueryList.removeElement failed:",2,queryList.getSize());
		ISoftListItem testItem = (ISoftListItem)queryList.getItemAt(0);
		assertEquals("AbstractQueryList.addElement failed:","about",testItem.getDisplayName());
		testItem = (ISoftListItem)queryList.getItemAt(1);
		assertEquals("AbstractQueryList.addElement failed:","zelda",testItem.getDisplayName());
	}

	public void testAddListDataListener() throws SoftListException, InterruptedException, QueryListException {
		resetCounts();
		queryList.addListener(listener);
		testAddElement();
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(500);
		}
		assertEquals("AbstractQueryList.addListDataListener failed:",3,addedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",0,removedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",0,changedCount);
		
		resetCounts();
		queryList.removeElementAt(0);
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(500);
		}
		assertEquals("AbstractQueryList.addListDataListener failed:",0,addedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",1,removedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",0,changedCount);
		
		queryList.removeListener(listener);
		resetCounts();
		queryList.removeElementAt(0);
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(500);
		}
		assertEquals("AbstractQueryList.addListDataListener failed:",0,addedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",0,removedCount);
		assertEquals("AbstractQueryList.addListDataListener failed:",0,changedCount);
		
	}
	
	public void testIndexOf() throws SoftListException, QueryListException {
		queryList.addElement(new QueryID("34"), "query one");
		assertEquals("QueryListItem.indexOf failed:",0,queryList.indexOf(new QueryID("34")));
		queryList.addElement(new QueryID("44"), "one query");
		assertEquals("QueryListItem.indexOf failed:",1,queryList.indexOf(new QueryID("34")));
		assertEquals("QueryListItem.indexOf failed:",0,queryList.indexOf(new QueryID("44")));
	}

}
