package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.runtime.internal.Runtime;
import gov.va.med.hds.cd.runtime.itest.IntegrationTestRuntimeContextHolder;
import gov.va.med.hds.cd.runtime.test.TestRuntime;
import gov.va.med.hds.cd.security.ConnectionFailedException;
import gov.va.med.hds.cd.security.ISecurity;
import junit.extensions.TestSetup;
import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * Querytool integration tests (tests that take a long time, DAO tests, and 
 * JFCUnit tests.)
 */
public class AllTests {
	private static ISecurity securityPlugin = null;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AllTests.class);
	}

	public static Test suite() {
		TestSuite suite =
			new TestSuite("Test for gov.va.med.hds.querytool.itest");
		//$JUnit-BEGIN$
		suite.addTest(new TestSuite(DateRangePanelTest.class));
		suite.addTest(new TestSuite(HTMLSelectionModelFactoryTest.class));
		suite.addTest(new TestSuite(MClinicRefDAOTest.class));
		suite.addTest(new TestSuite(MClinicianListDAOFactoryTest.class));
		suite.addTest(new TestSuite(MConstraintDAOTest.class));
		suite.addTest(new TestSuite(MDisplayFieldListDAOTest.class));
		suite.addTest(new TestSuite(MDisplayGroupTreeDAOTest.class));
		suite.addTest(new TestSuite(MDocClassRefDAOTest.class));
		suite.addTest(new TestSuite(MDocDetailDAOTest.class));
		suite.addTest(new TestSuite(MDocStatusListDAOTest.class));
		suite.addTest(new TestSuite(MDocTitleRefDAOTest.class));
		suite.addTest(new TestSuite(MFieldDefnDAOTest.class));
		suite.addTest(new TestSuite(MHepCRegistryAccessDAOTest.class));
		suite.addTest(new TestSuite(MNameListDAOTest.class));
		suite.addTest(new TestSuite(MOrderDetailDAOTest.class));
		suite.addTest(new TestSuite(MOrderItemListDAOFactoryTest.class));
		suite.addTest(new TestSuite(MOrderItemRefDAOTest.class));
		suite.addTest(new TestSuite(MOrderSetRefDAOTest.class));
		suite.addTest(new TestSuite(MOrderSigStatusListDAOTest.class));
		suite.addTest(new TestSuite(MOrderStatusListDAOTest.class));
		suite.addTest(new TestSuite(MPatDetailDAOTest.class));
		suite.addTest(new TestSuite(MProcedureListDAOFactoryTest.class));
		suite.addTest(new TestSuite(MQueryDAOTest.class));
		suite.addTest(new TestSuite(MRegistryRefDAOTest.class));
		suite.addTest(new TestSuite(MSpecialtyRefDAOTest.class));
		suite.addTest(new TestSuite(MTeamRefDAOTest.class));
		suite.addTest(new TestSuite(MWardRefDAOTest.class));
		suite.addTest(new TestSuite(PredefinedQueryListDAOTest.class));
		suite.addTest(new TestSuite(QueryPanelFactoryJFCTest.class));
		suite.addTest(new TestSuite(TypeFactoryTest.class));
		suite.addTest(new TestSuite(UserQueryListDAOTest.class));
		suite.addTest(new TestSuite(AbstractQueryListTest.class));
		suite.addTest(new TestSuite(QueryUtilTest.class));
		suite.addTest(new TestSuite(CombinedQueryCollectionTest.class));
		suite.addTest(new TestSuite(RunSetupDAOTest.class));
		suite.addTest(new TestSuite(RunMainIterationTest.class));
		suite.addTest(new TestSuite(CompleteQueryDataDAOTest.class));
		suite.addTest(new TestSuite(QueryToolTest.class));
		suite.addTest(new TestSuite(EditorFactoryFactoryTest.class));
		suite.addTest(new TestSuite(ConsultServiceAllDAOTest.class));
		suite.addTest(new TestSuite(AvailableColumnListTest.class));
		suite.addTest(new TestSuite(SampleTableModelTest.class));
		suite.addTest(new TestSuite(AvailableColumnsListTest.class));
		suite.addTest(new TestSuite(QueryExecuteMainTest.class));
		suite.addTest(new TestSuite(SelectAndEditTest.class));
		suite.addTest(new TestSuite(SensitiveReviewTest.class));
		suite.addTest(new TestSuite(ViewCompleteStateTest.class));
		//$JUnit-END$

		TestSetup wrapper = new TestSetup(suite) {
			protected void setUp()
				throws Exception, ConnectionFailedException {
				oneTimeSetUp();
			}
			protected void tearDown() throws Exception, SecurityException {
				oneTimeTearDown();
			}
		};

		return wrapper;
	}

	public static void oneTimeSetUp()
		throws Exception, ConnectionFailedException {
		TestRuntime runtime = new TestRuntime();
		runtime.readPluginDescriptors(
			AllTests.class.getResourceAsStream("registry.xml"));
		runtime.readSettings(
			AllTests.class.getResourceAsStream("CacheSettings.xml"));
		//		runtime.readSettings(
		//			AllTests.class.getResourceAsStream("RecordSettings.xml"));
		IntegrationTestRuntimeContextHolder.setInstance(runtime);
		securityPlugin =
			(ISecurity)runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		securityPlugin.login();
	}
	
	public static void oneTimeTearDown() throws SecurityException, Exception {
		securityPlugin.logout();
		((Runtime)IntegrationTestRuntimeContextHolder.getInstance()).shutdown();
	}

}
