/*
 * Created on Jul 17, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.DisplayField;
import gov.va.med.hds.querytool.IAvailableColumnsList;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.ui.AvailableColumnList;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * 
 */
public class AvailableColumnListTest extends MAbstractTestCase implements IAvailableColumnsList {
	private AvailableColumnList columnList;
	private List list;
	private Thread testThread;
	private ListDataListener listener;
	private List eventList;
	private ListDataListener dummyListener;

	public AvailableColumnListTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AvailableColumnListTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		columnList = new AvailableColumnList();
		columnList.setAvailableColumnModel(this);
		list = new ArrayList();
		testThread = Thread.currentThread();
	}

	public void testGetSize() {
		assertEquals("AvailableColumnList.getSize() failed:",0,columnList.getSize());
		setupRows(1);
		assertEquals("AvailableColumnList.getSize() failed:",1,columnList.getSize());
		setupRows(9);
		assertEquals("AvailableColumnList.getSize() failed:",10,columnList.getSize());
	}

	private void setupRows(int count) {
		for (int i = 0; i < count; i++) {
			IDisplayField field = new DisplayField();
			field.setColumnWidth(100+i);
			field.setDisplayText("displayText"+Integer.toString(i));
			field.setID("wow"+Integer.toString(i));
			field.setListDisplayName("listDisplayName"+Integer.toString(i));
			field.setSampleData(new String[]{"fred"+Integer.toString(i),"jane"+Integer.toString(i)});
			field.setSortType("aaa"+Integer.toString(i));
			list.add(field);
		}
	}

	public void testGetElementAt() {
		setupRows(1);
		assertEquals("AvailableColumnList.getElementAt() failed:","displayText0",((IDisplayField)columnList.getElementAt(0)).getDisplayText());
		setupRows(5);
		assertEquals("AvailableColumnList.getElementAt() failed:","displayText4",((IDisplayField)columnList.getElementAt(5)).getDisplayText());
	}
	
	public void testChangedEvent() throws InterruptedException {
		eventList = new ArrayList();
		listener = new ListDataListener() {
			public void intervalAdded(ListDataEvent e) {
				saveEvent(e);
			}

			public void intervalRemoved(ListDataEvent e) {
				saveEvent(e);
			}

			public void contentsChanged(ListDataEvent e) {
				saveEvent(e);
			}

			private void saveEvent(ListDataEvent e) {
				eventList.add(e);
				if (!SwingUtilities.isEventDispatchThread()) {
					fail("AvailableColumnList listener failed: not using the swing thread.");
				}
			}
		};
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("AvailableColumnList listener add/remove failed:",0,eventList.size());
		columnList.addListDataListener(listener);
		ListDataEvent dummyEvent = new ListDataEvent(this,ListDataEvent.CONTENTS_CHANGED,12345,54321);
		dummyListener.contentsChanged(dummyEvent);
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("AvailableColumnList listener add/remove failed:",1,eventList.size());
		assertEquals("AvailableColumnList listener add/remove failed:",12345,((ListDataEvent)eventList.get(0)).getIndex0());
		assertEquals("AvailableColumnList listener add/remove failed:",54321,((ListDataEvent)eventList.get(0)).getIndex1());
		assertEquals("AvailableColumnList listener add/remove failed:",ListDataEvent.CONTENTS_CHANGED,((ListDataEvent)eventList.get(0)).getType());
		columnList.removeListDataListener(listener);
		eventList.clear();
		dummyEvent = new ListDataEvent(this,ListDataEvent.INTERVAL_ADDED,12345,54321);
		dummyListener.contentsChanged(dummyEvent);
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("AvailableColumnList listener add/remove failed:",0,eventList.size());
	}
	
	public void addListDataListener(ListDataListener listener) {
		dummyListener = listener;
	}
	public void removeListDataListener(ListDataListener listener) {}
	
	public IDisplayField get(int index) {
		if (index<0 || index>list.size()-1) return null;
		return (IDisplayField)list.get(index);
	}

	public int size() {
		return list.size();
	}

	public void setModel(IQuery query) {}
	public void setRuntime(IRuntimeContext runtime) {}
	public int indexOf(IDisplayField field) {return 0;}
	
	public IDisplayField getMutable(int index) {
		return get(index);
	}

}
