/*
 * Initially Created on Jul 21, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.querytool.AvailableColumnsList;
import gov.va.med.hds.querytool.IAvailableColumnsList;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.Query;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.types.ConsultService;
import gov.va.med.hds.querytool.types.DocumentStatus;
import gov.va.med.hds.querytool.types.OrderStatus;
import gov.va.med.hds.querytool.types.PatientSelection;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * Tests for AvailableColumnsList model provided by querytool.core.
 */
public class AvailableColumnsListTest extends MAbstractTestCase {
	private IAvailableColumnsList columnsList;
	private IQuery query;
	private List eventList;
	private ListDataListener listener = new ListDataListener() {
		public void intervalAdded(ListDataEvent e) {
			eventList.add(e);
		}

		public void intervalRemoved(ListDataEvent e) {
			eventList.add(e);
		}

		public void contentsChanged(ListDataEvent e) {
			eventList.add(e);
		}
	};

	public AvailableColumnsListTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AvailableColumnsListTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		columnsList = new AvailableColumnsList();
		columnsList.setRuntime(runtime);
		query = new Query(new QueryID("1234"),"test");
		query.getConstraints().addRelationship("0=1000");
		query.getConstraints().addRelationship("1000=2000");
		query.getConstraints().addRelationship("1000=3000");
		query.getConstraints().addRelationship("1000=4000");
		query.getConstraints().addRelationship("1000=5000");
		eventList = new ArrayList();
	}

	public void testSize() {
		columnsList.setModel(query);
		assertEquals("AvailableColumnsList.size() failed:",0,columnsList.size());
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		assertEquals("AvailableColumnsList.size() failed:",4,columnsList.size());
	}

	public void testGet() {
		columnsList.setModel(query);
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		assertEquals("AvailableColumnsList.get() failed:",4,columnsList.size());
		assertEquals("AvailableColumnsList.get() failed:","Patient.Name",columnsList.get(0).getID().toString());
		assertEquals("AvailableColumnsList.get() failed:","Patient.Last4",columnsList.get(1).getID().toString());
		assertEquals("AvailableColumnsList.get() failed:","Patient.Age",columnsList.get(2).getID().toString());
		assertEquals("AvailableColumnsList.get() failed:","Patient.Ward",columnsList.get(3).getID().toString());
	}
	
	public void testGetMutable() {
		columnsList.setModel(query);
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		assertFalse("AvailableColumnsList.getMutable() failed: new object not returned.",columnsList.get(0)==columnsList.getMutable(0));
		assertNull("AvailableColumnsList.getMutable() failed: object returned.",columnsList.getMutable(-1));
		assertNull("AvailableColumnsList.getMutable() failed: object returned.",columnsList.getMutable(Integer.MAX_VALUE));
	}

	public void testAddListDataListener() {
		columnsList.addListDataListener(listener);
		columnsList.setModel(query);
		assertEquals("AvailableColumnsList listeners/events failure:",0,eventList.size());
		eventList.clear();
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		assertEquals("AvailableColumnsList listeners/events failure:",1,eventList.size());
		eventList.clear();
		columnsList.removeListDataListener(listener);
		query.getConstraints().clearAll();
		assertEquals("AvailableColumnsList listeners/events failure:",0,eventList.size());
	}
	
	public void testClearModel() {
		columnsList.setModel(query);
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		columnsList.setModel(null);
		assertEquals("AvailableColumnsList.setModel failed:",0,columnsList.size());
	}
	
	public void testIndexOf() {
		columnsList.setModel(query);
		IValue value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		for (int i = 0; i < 4; i++) {
			assertEquals("AvailableColumnsList.indexOf failed on column"+Integer.toString(i)+":",i,columnsList.indexOf(columnsList.get(i)));
		}
	}
	
	public void testUniqueFields() {
		columnsList.setModel(query);
		
		IValue value = new OrderStatus();
		value.setID("3000");
		value.setConstraintName("Order.status");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("9");
		value.setDisplayFieldCategory("O");
		query.getConstraints().addValue(value);
		
		value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		
		value = new ConsultService();
		value.setID("4000");
		value.setConstraintName("Consult.service");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("9");
		value.setDisplayFieldCategory("C");
		query.getConstraints().addValue(value);
		
		assertEquals("AvailableColumnsList setup failed:",12,columnsList.size());
	}
	
	public void testFieldOrder() {
		columnsList.setModel(query);
		
		IValue value = new OrderStatus();
		value.setID("3000");
		value.setConstraintName("Order.status");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("9");
		value.setDisplayFieldCategory("O");
		query.getConstraints().addValue(value);
		
		value = new PatientSelection();
		value.setID("2000");
		value.setConstraintName("Patient.list");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("10");
		value.setDisplayFieldCategory("P");
		query.getConstraints().addValue(value);
		
		value = new DocumentStatus();
		value.setID("4000");
		value.setConstraintName("Document.status");
		value.setDefaultDisplayText("nothing here");
		value.setDisplayText("or here...");
		value.setEditorID("8");
		value.setDisplayFieldCategory("D");
		query.getConstraints().addValue(value);
		
		assertEquals("AvailableColumnsList field order failed:",17,columnsList.size());
		assertEquals("AvailableColumnsList field order failed:","Patient.Name",columnsList.get(0).getID().toString());
		assertEquals("AvailableColumnsList field order failed:","Patient.Ward",columnsList.get(3).getID().toString());
		assertEquals("AvailableColumnsList field order failed:","Order.Datetime",columnsList.get(4).getID().toString());
		assertEquals("AvailableColumnsList field order failed:","Order.Finding",columnsList.get(11).getID().toString());
		assertEquals("AvailableColumnsList field order failed:","Document.Reference",columnsList.get(12).getID().toString());
		assertEquals("AvailableColumnsList field order failed:","Document.Cosigner",columnsList.get(16).getID().toString());
	}
	
	public void testFullQueryUpdates() throws Exception {
		IQueryDAO queryDAO = (IQueryDAO)daoRegistry.getDAO(IQueryDAO.class.getName());
		query = queryDAO.getAsync(new QueryID("88"), "");
		columnsList.setModel(query);
		query.init();
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(200);
		}
		assertEquals("AvailableColumnsList full query update failed:",20,columnsList.size());
	}

}
