/*
 * Initially Created on Apr 28, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.querytool.CombinedQueryCollection;
import gov.va.med.hds.querytool.ICombinedQueryCollection;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryCollectionEvent;
import gov.va.med.hds.querytool.IQueryCollectionListener;
import gov.va.med.hds.querytool.ISoftListItem;
import gov.va.med.hds.querytool.QueryCollectionException;
import gov.va.med.hds.querytool.QueryListItem;
import gov.va.med.hds.querytool.dao.IQueryDAO;

/**
 * CombinedQueryCollection tests.
 */
public class CombinedQueryCollectionTest extends MAbstractTestCase {
	private ICombinedQueryCollection collection;
	private IQueryCollectionListener l = new IQueryCollectionListener() {
		public void collectionChanged(IQueryCollectionEvent event) {
			eventCount++;
		}
	};
	private int eventCount;

	public CombinedQueryCollectionTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(CombinedQueryCollectionTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		collection = new CombinedQueryCollection();
		IQueryDAO queryDAO = (IQueryDAO)daoRegistry.getDAO(IQueryDAO.class.getName());
		collection.setQueryDAO(queryDAO);
		collection.setRuntime(runtime);
	}
	
	public void testContainsDisplayName() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("There and Back Again"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Everything Report"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Abnormal Results"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Consult Follow-up"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Incomplete Orders"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Recent Activity"));
		assertTrue("CombinedQueryCollection.containsDisplayName failed: expected TRUE.",collection.containsDisplayName("Scheduled/Due Activity"));
	}

	private void setupRefresh()
		throws QueryCollectionException, InterruptedException {
		collection.refresh();
		synchronized (Thread.currentThread()) {
			Thread.yield();
			Thread.currentThread().wait(300);
		}
	}

	public void testGetUserQueryItem() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		assertEquals("CombinedQueryCollection.getUserQueryItem failed:","Everything Report",collection.getUserQueryItem(0).getDisplayName());
		assertEquals("CombinedQueryCollection.getUserQueryItem failed:","There and Back Again",collection.getUserQueryItem(1).getDisplayName());
	}

	public void testGetPredefinedQueryItem() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		assertEquals("CombinedQueryCollection.getPredefinedQueryItem failed:","Abnormal Results",collection.getPredefinedQueryItem(0).getDisplayName());
		assertEquals("CombinedQueryCollection.getPredefinedQueryItem failed:","Consult Follow-up",collection.getPredefinedQueryItem(1).getDisplayName());
		assertEquals("CombinedQueryCollection.getPredefinedQueryItem failed:","Incomplete Orders",collection.getPredefinedQueryItem(2).getDisplayName());
		assertEquals("CombinedQueryCollection.getPredefinedQueryItem failed:","Recent Activity",collection.getPredefinedQueryItem(3).getDisplayName());
		assertEquals("CombinedQueryCollection.getPredefinedQueryItem failed:","Scheduled/Due Activity",collection.getPredefinedQueryItem(4).getDisplayName());
	}

	public void testIsPredefinedItem() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		ISoftListItem item = collection.getPredefinedQueryItem(0);
		assertEquals("CombinedQueryCollection.isPredefinedItem failed:",true,collection.isPredefinedItem(item));
		item = collection.getUserQueryItem(0);
		assertEquals("CombinedQueryCollection.isPredefinedItem failed:",false,collection.isPredefinedItem(item));
		item = new QueryListItem();
		assertEquals("CombinedQueryCollection.isPredefinedItem failed:",false,collection.isPredefinedItem(item));
	}

	public void testIsUserQuery() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		ISoftListItem item = collection.getUserQueryItem(0);
		assertEquals("CombinedQueryCollection.isUserQuery failed:",true,collection.isUserQuery(item));
		item = collection.getPredefinedQueryItem(0);
		assertEquals("CombinedQueryCollection.isUserQuery failed:",false,collection.isUserQuery(item));
		item = new QueryListItem();
		assertEquals("CombinedQueryCollection.isUserQuery failed:",false,collection.isUserQuery(item));
	}

//	public void testRemoveUserItem() {
//	}
//
//	public void testReloadChangedQuery() {
//	}
//
	public void testSizes() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		assertEquals("CombinedQueryCollection.getUserDefinedQuerySize failed:",2,collection.getUserDefinedQuerySize());
		assertEquals("CombinedQueryCollection.getPredefinedQuerySize failed:",5,collection.getPredefinedQuerySize());
		assertEquals("CombinedQueryCollection.getSize failed:",7,collection.getSize());
	}

	public void testIsQueryItem() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		
		assertEquals("CombinedQueryCollection.isQueryItem failed:",false,collection.isQueryItem(null));
		assertEquals("CombinedQueryCollection.isQueryItem failed:",false,collection.isQueryItem(new Object()));
		assertEquals("CombinedQueryCollection.isQueryItem failed:",false,collection.isQueryItem(new String()));
		assertEquals("CombinedQueryCollection.isQueryItem failed:",false,collection.isQueryItem(new QueryListItem()));
		assertEquals("CombinedQueryCollection.isQueryItem failed:",true,collection.isQueryItem(collection.getPredefinedQueryItem(0)));
		assertEquals("CombinedQueryCollection.isQueryItem failed:",true,collection.isQueryItem(collection.getUserQueryItem(0)));
	}

	public void testGetQuery() throws QueryCollectionException, InterruptedException {
		setupRefresh();
		IQuery query = collection.getQuery(collection.getUserQueryItem(0));
		assertNotNull("CombinedQueryCollection.getQuery failed: item null.",query);
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(300);
		}
		assertNotNull("CombinedQueryCollection.getQuery failed: item null.",collection.getUserQueryItem(0).getItem());
	}

//	public void testSaveQuery() {
//	}
//

	public void testAddChangeListener() throws QueryCollectionException, InterruptedException {
		collection.addChangeListener(l);
		setupRefresh();
		assertEquals("CombinedQueryCollection.addChangeListener failed:",7,eventCount);
	}


}
