/*
 * Initially Created on Jun 9, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.clinicaldata.IPatientDemographics;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.IPatDetailReceivable;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IReportRow;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.ICompleteQueryDataDAO;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.querytool.m.ReportRow;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Tests for the completeQueryData m DAO.
 */
public class CompleteQueryDataDAOTest extends MAbstractTestCase {
	private IQuery query;
	private IQueryDAO queryDAO;
	private ICompleteQueryDataDAO dataDAO;

	/**
	 * Constructor for CompleteQueryDataDAOTest.
	 * @param name
	 */
	public CompleteQueryDataDAOTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(CompleteQueryDataDAOTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		queryDAO = (IQueryDAO)daoRegistry.getDAO(IQueryDAO.class.getName());
		dataDAO = (ICompleteQueryDataDAO)daoRegistry.getDAO(ICompleteQueryDataDAO.class.getName());
	}

	public void testGetReportRow() throws QueryDAOException, InterruptedException {
		query = queryDAO.getAsync(new QueryID("88"),"blaaaah");
		query.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized(CompleteQueryDataDAOTest.this) {
					CompleteQueryDataDAOTest.this.notify();
				}
			}
		});
		query.init();
		synchronized(this) {
			this.wait(5000);
		}
		
		IReportRow[] rows = dataDAO.getReportRow(0,9);
		assertEquals("MRunDAOTest.getReportRow failed:",10,rows.length);
	}

	String text1;	
	public void testDetailResultsRequest() throws InterruptedException, QueryDAOException {
		IReportRow row = new ReportRow("ORD:7091655;1");
		text1 = null;
		dataDAO.detailResultsRequest(row,new IDetailReceivable() {
			public void setDetail(String text) {
				text1 = text;
				callNotify();
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertEquals("MRunDAO.detailResultsRequest failed:","line 1\nline 2\nline 3\n",text1);
	}

	public void testDetailResultsRequest_empty() throws InterruptedException, QueryDAOException {
		IReportRow row = new ReportRow("");
		text1 = null;
		dataDAO.detailResultsRequest(row,new IDetailReceivable() {
			public void setDetail(String text) {
				text1 = text;
				callNotify();
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertEquals("MRunDAO.detailResultsRequest failed:","No detail available.",text1);
	}
	
	public void testDetailResultsRequest_null() throws InterruptedException, QueryDAOException {
		IReportRow row = new ReportRow();
		text1 = null;
		dataDAO.detailResultsRequest(row,new IDetailReceivable() {
			public void setDetail(String text) {
				text1 = text;
				callNotify();
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertEquals("MRunDAO.detailResultsRequest failed:","No detail available.",text1);
	}
	
	public void testDetailResultsRequest_null2() throws InterruptedException, QueryDAOException {
		IReportRow row = null;
		text1 = null;
		dataDAO.detailResultsRequest(row,new IDetailReceivable() {
			public void setDetail(String text) {
				text1 = text;
				callNotify();
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertEquals("MRunDAO.detailResultsRequest failed:","No detail available.",text1);
	}
	
	private void callNotify() {
		synchronized(this) {
			this.notify();
		}
	}
	
	IPatientDemographics demos;
	public void testPatientInfoRequest() throws QueryDAOException, InterruptedException {
		IReportRow row = new ReportRow("ORD:7091661;1");
		demos = null;
		dataDAO.patientInfoRequest(row,new IPatDetailReceivable() {
			public void setPatient(IPatientDemographics patient) {
				demos = patient;
				synchronized(CompleteQueryDataDAOTest.this) {
					CompleteQueryDataDAOTest.this.notify();
				}
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertNotNull("MRunDAO.patientInfoRequest should not time out.",demos);
		assertEquals("MRunDAO.patientInfoRequest failed:","75",demos.getPatientId());
		assertEquals("MRunDAO.patientInfoRequest failed:","AAA,PATIENT",demos.getPatientName());
		assertEquals("MRunDAO.patientInfoRequest failed:","52",demos.getAge());
		assertEquals("MRunDAO.patientInfoRequest failed:","13 Jul 1950",demos.getDob());
		assertEquals("MRunDAO.patientInfoRequest failed:","701-31-9898",demos.getSsn());
	}

	public void testPatientInfoRequest_empty() throws QueryDAOException, InterruptedException {
		IReportRow row = new ReportRow("");
		demos = null;
		dataDAO.patientInfoRequest(row,new IPatDetailReceivable() {
			public void setPatient(IPatientDemographics patient) {
				demos = patient;
				synchronized(CompleteQueryDataDAOTest.this) {
					CompleteQueryDataDAOTest.this.notify();
				}
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertNull("MRunDAO.patientInfoRequest should not time out.",demos);
	}

	public void testPatientInfoRequest_null() throws QueryDAOException, InterruptedException {
		IReportRow row = new ReportRow();
		demos = null;
		dataDAO.patientInfoRequest(row,new IPatDetailReceivable() {
			public void setPatient(IPatientDemographics patient) {
				demos = patient;
				synchronized(CompleteQueryDataDAOTest.this) {
					CompleteQueryDataDAOTest.this.notify();
				}
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertNull("MRunDAO.patientInfoRequest should not time out.",demos);
	}
	
	public void testPatientInfoRequest_null2() throws QueryDAOException, InterruptedException {
		IReportRow row = null;
		demos = null;
		dataDAO.patientInfoRequest(row,new IPatDetailReceivable() {
			public void setPatient(IPatientDemographics patient) {
				demos = patient;
				synchronized(CompleteQueryDataDAOTest.this) {
					CompleteQueryDataDAOTest.this.notify();
				}
			}
		});
		synchronized(this) {
			this.wait(1000);
		}
		assertNull("MRunDAO.patientInfoRequest should not time out.",demos);
	}
	
}
