package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.itest.IntegrationTestRuntimeContextHolder;
import gov.va.med.hds.cd.spectrum.longlist.BufferModelException;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.DataRequest;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.IListPositionListener;
import gov.va.med.hds.querytool.dao.IClinicianListDAOFactory;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.event.ListDataListener;
import junit.framework.TestCase;

/**
 * @author vhaislmonsos
 *
 * Dec 30, 2002 9:29:22 AM
 */
public class MClinicianListDAOFactoryTest extends TestCase implements IBufferModel {
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private TestHandler testHandler = new TestHandler();
	private IClinicianListDAOFactory mClinBufferListDAOFactory;
	private IDataNeededListener dataProvider;
	private Object[] items;
	private int expected = -1;
	private Thread testThread;

	/**
	 * Constructor for MClinBufferListDAOFactoryTest.
	 * @param arg0
	 */
	public MClinicianListDAOFactoryTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MClinicianListDAOFactoryTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		Logger.getLogger(MClinicianListDAOFactoryTest.class.getName()).setUseParentHandlers(false);
		Logger.getLogger(MClinicianListDAOFactoryTest.class.getName()).addHandler(testHandler);
		runtime = IntegrationTestRuntimeContextHolder.getInstance();
		daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
		mClinBufferListDAOFactory = (IClinicianListDAOFactory)daoRegistry.getDAO(IClinicianListDAOFactory.class.getName());
		items = null;
		expected = -1;
		dataProvider = null;
		testThread = Thread.currentThread();
	}
	
	public void testGetBufferModelDAO() throws BufferModelException {
		dataProvider = mClinBufferListDAOFactory.getBufferModelDAO();
		assertNotNull("MClinBufferListDAOFactory.getBufferModelDAO failed: null value returned.",dataProvider);
	}
	
	public void testGetTop() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",44,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Ackerman,Nien-Chin",items[0].toString());
		assertEquals("MBufferListDAO.setTop failed:","Lynch,Kathy",items[43].toString());
	}

	public void testGetBottom() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",44,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Ztest,Iost",items[0].toString());
		assertEquals("MBufferListDAO.setTop failed:","Killdare,Michael",items[43].toString());
	}
	
	public void testGetNext_1() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ACKERMAN,NIEN-CHIN",IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",44,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Ackerman,Shirley",items[0].toString());
		assertEquals("MBufferListDAO.setTop failed:","Macimposter,Cary",items[43].toString());
	}
	
	public void testGetNext_2() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ZAAAAAA",IBufferModel.FORWARD_DIRECTION,20);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",1,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Ztest,Iost",items[0].toString());
	}
	
	public void testGetNext_3() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ZTEST,IOST",IBufferModel.FORWARD_DIRECTION,20);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",0,items.length);
	}
	
	public void testGetPrev_1() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ZTEST,IOST",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",44,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Welby,Marcus",items[0].toString());
		assertEquals("MBufferListDAO.setTop failed:","Insley,Marcia L",items[43].toString());
	}
	
	public void testGetPrev_2() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ACKERMAN,NIEN-CHIO",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",1,items.length);
		assertEquals("MBufferListDAO.setTop failed:","Ackerman,Nien-Chin",items[0].toString());
	}
	
	public void testGetPrev_3() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"ACKERMAN,NIEN-CHIN",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(10000);
		}
		assertNotNull("MBufferListDAO.setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO.setTop failed:",0,items.length);
	}
	
	class TestHandler extends Handler {
		public void close() throws SecurityException {
		}
		public void flush() {
		}
		public void publish(LogRecord record) {
			fail(record.getMessage());
		}
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addDataNeededListener(IDataNeededListener)
	 */
	public void addDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListDataListener(IListPositionListener)
	 */
	public void addListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getBufferFull()
	 */
	public Object[] getBufferFull() {
		return null;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getInitSize()
	 */
	public int getDefaultCallSize() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getMaxSize()
	 */
	public int getCapacity() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtBottom()
	 */
	public boolean isAtBottom() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtTop()
	 */
	public boolean isAtTop() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isBoundryKnown()
	 */
	public boolean isBoundryKnown() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeDataNeededListener(IDataNeededListener)
	 */
	public void removeDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListDataListener(IListPositionListener)
	 */
	public void removeListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setInitSize(int)
	 */
	public void setDefaultCallSize(int initSize) {
	}

	/**
	 * @see javax.swing.ListModel#addListDataListener(ListDataListener)
	 */
	public void addListDataListener(ListDataListener l) {
	}

	/**
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		return null;
	}

	/**
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return 0;
	}

	/**
	 * @see javax.swing.ListModel#removeListDataListener(ListDataListener)
	 */
	public void removeListDataListener(ListDataListener l) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getElement(Object, int)
	 */
	public int getElement(Object item, int size) throws BufferModelException {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getDataRequest(DataRequest)
	 */
	public boolean getDataRequest(DataRequest dataRequest) {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setData(Object[], int)
	 */
	public void setData(Object[] items, int sizeExpected) {
		this.items = items;
		this.expected = sizeExpected;
		synchronized(testThread) {
			testThread.notify();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListPositionListener(IListPositionListener)
	 */
	public void addListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListPositionListener(IListPositionListener)
	 */
	public void removeListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isCallInProgress()
	 */
	public boolean isCallInProgress() {
		return false;
	}

}
