package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.Query;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.ConstraintDAOException;
import gov.va.med.hds.querytool.dao.IConstraintDAO;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Tests for the M implementation of the Constraint DAO.
 */
public class MConstraintDAOTest extends MAbstractTestCase {
	private IConstraintDAO constraintDAO = null;
	private IQueryDAO queryDAO = null;

	/**
	 * Constructor for MConstraintDAOTest.
	 * @param arg0
	 */
	public MConstraintDAOTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MConstraintDAOTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		constraintDAO = (IConstraintDAO)daoRegistry.getDAO(IConstraintDAO.class.getName());
		queryDAO = (IQueryDAO)daoRegistry.getDAO(IQueryDAO.class.getName());
	}

	public void testGet() throws ConstraintDAOException {
		IQuery q = new Query(new QueryID("77"),null);
		String[] reportStrings =
			new String[] {
				"Name=RPTU123456789065 EVERYTHING REPORT",
				"DisplayText=Everything Report",
				"QueryText=1",
				"CriterionTag=1000500",
				"Caption=Patient Lists for Search:",
				"QueryText=2",
				"CriterionTag=1000500",
				"Caption= Clinic: 1 Cary's Clinic and Clinic: Cardiology and Clinic: Diabetic Ede",
				"ConstraintName=Patient.ListSource",
				"EditorID=1",
				"Value=c:7:T",
				"Value=c:27:T",
				"Value=c:21:T",
				"Value=w:6:",
				"Value=w:1:",
				"Value=w:3:",
				"Value=t:172:",
				"Value=t:169:",
				"Value=t:150:",
				"Value=t:184:",
				"Value=s:7:",
				"Value=s:10:",
				"Value=s:3:",
				"Value=s:1:",
				"Value=p:7:",
				"Value=p:27:",
				"Value=p:21:",
				"Value=p:15:",
				"QueryText=3",
				"CriterionTag=1500500",
				"Caption=Date Range for Search:",
				"QueryText=4",
				"CriterionTag=1500500",
				"Caption=during the Past Month",
				"ConstraintName=Search.DateRange",
				"EditorID=2",
				"Value=3000201:T",
				"QueryText=5",
				"CriterionTag=1011000",
				"Caption=include only patients with inpt/outpt/pharmacy visits during",
				"ConstraintName=Patient.AnyVisit",
				"EditorID=12",
				"QueryText=6",
				"CriterionTag=1011000",
				"Caption=during the Current Year",
				"ConstraintName=Patient.DateRange",
				"EditorID=2",
				"Value=YF",
				"QueryText=7",
				"CriterionTag=1011500",
				"Caption=where the primary outpatient provider is",
				"QueryText=8",
				"CriterionTag=1011500",
				"Caption=ACKERMAN,NIEN-CHIN or ANDREWS,BOB or BARNES,LENORA or BAYLIS,RANDALL orE",
				"ConstraintName=Patient.Primary",
				"EditorID=8",
				"Value=1323",
				"Value=1117",
				"Value=123456789031",
				"Value=1312",
				"Value=1360",
				"QueryText=9",
				"CriterionTag=2000500",
				"Caption=Find Orders",
				"ConstraintName=Order.Search",
				"EditorID=12",
				"Value=1",
				"QueryText=10",
				"CriterionTag=2012000",
				"Caption=where the service category (display group) is",
				"QueryText=11",
				"CriterionTag=2012000",
				"Caption=LABORATORY or DIET ORDERS or CONSULTS or ACTIVITY or SURGERY or CONDITIN",
				"ConstraintName=Order.DisplayGroup",
				"EditorID=4",
				"Value=3",
				"Value=19",
				"Value=30",
				"Value=43",
				"Value=27",
				"Value=58",
				"QueryText=12",
				"CriterionTag=2013500",
				"Caption=where the text of the order contains",
				"QueryText=13",
				"CriterionTag=2013500",
				"Caption=tHIS IS fREE tEXT..D",
				"ConstraintName=Order.Text",
				"EditorID=9",
				"Value=tHIS IS fREE tEXT..D",
				"QueryText=14",
				"CriterionTag=2014000",
				"Caption=where the order has abnormal results or significant findings",
				"ConstraintName=Order.Abnormal",
				"EditorID=12",
				"Value=1",
				"QueryText=15",
				"CriterionTag=2014500",
				"Caption=where the order status is",
				"QueryText=16",
				"CriterionTag=2014500",
				"Caption=ACTIVE or DELAYED or DISCONTINUED/EDIT or HOLD or PARTIAL RESULTS or RED",
				"ConstraintName=Order.Status",
				"EditorID=6",
				"Value=6",
				"Value=10",
				"Value=12",
				"Value=3",
				"Value=9",
				"Value=15",
				"Value=11",
				"QueryText=17",
				"CriterionTag=2015000",
				"Caption=where the signature status is",
				"QueryText=18",
				"CriterionTag=2015000",
				"Caption=ON CHART w/written orders or NOT SIGNED or ON CHART w/printed orders orD",
				"ConstraintName=Order.SignStatus",
				"EditorID=7",
				"Value=0",
				"Value=2",
				"Value=4",
				"Value=7",
				"QueryText=19",
				"CriterionTag=2015500",
				"Caption=where the requesting clinician is",
				"QueryText=20",
				"CriterionTag=2015500",
				"Caption=ACKERMAN,SHIRLEY or ANWER,MOHAMED or BARNACLE,BILL or BROWN,STANLEY A oN",
				"ConstraintName=Order.Requestor",
				"EditorID=8",
				"Value=1307",
				"Value=123456789017",
				"Value=12",
				"Value=1314",
				"Value=123456789032",
				"QueryText=21",
				"CriterionTag=3000500",
				"Caption=Find Consults",
				"ConstraintName=Consult.Search",
				"EditorID=12",
				"Value=1",
				"QueryText=22",
				"CriterionTag=3011500",
				"Caption=where the consulting service is",
				"QueryText=23",
				"CriterionTag=3011500",
				"Caption=TEST CONSULT TYPE or GMRCTVTEST or VTEST or ZZTEST CONSULT",
				"ConstraintName=Consult.DisplayGroup",
				"EditorID=14",
				"Value=3350",
				"Value=3354",
				"Value=3355",
				"Value=3357",
				"QueryText=24",
				"CriterionTag=3013000",
				"Caption=where the text of the consult request contains",
				"QueryText=25",
				"CriterionTag=3013000",
				"Caption=mORE FREE TESXT",
				"ConstraintName=Consult.Text",
				"EditorID=9",
				"Value=mORE FREE TESXT",
				"QueryText=26",
				"CriterionTag=3013500",
				"Caption=where the consult has significant findings",
				"ConstraintName=Consult.Abnormal",
				"EditorID=12",
				"Value=1",
				"QueryText=27",
				"CriterionTag=3014000",
				"Caption=where the consult status is",
				"QueryText=28",
				"CriterionTag=3014000",
				"Caption=ACTIVE or DISCONTINUED or EXPIRED or LAPSED or PENDING or SCHEDULED",
				"ConstraintName=Consult.Status",
				"EditorID=6",
				"Value=6",
				"Value=1",
				"Value=7",
				"Value=14",
				"Value=5",
				"Value=8",
				"QueryText=29",
				"CriterionTag=3014500",
				"Caption=where the signature status is",
				"QueryText=30",
				"CriterionTag=3014500",
				"Caption=ON CHART w/written orders or ELECTRONIC or NOT REQUIRED or ON CHART w/pD",
				"ConstraintName=Consult.SignStatus",
				"EditorID=7",
				"Value=0",
				"Value=1",
				"Value=3",
				"Value=4",
				"Value=7",
				"QueryText=31",
				"CriterionTag=3015000",
				"Caption=where the requesting clinician is",
				"QueryText=32",
				"CriterionTag=3015000",
				"Caption=ARCENEAUX,CHARLES or EATON,DENIS or MALMROSE,CARY or MAUSETH,PATRICIA A",
				"ConstraintName=Consult.Requestor",
				"EditorID=8",
				"Value=1299",
				"Value=1273",
				"Value=5",
				"Value=123456789025",
				"QueryText=33",
				"CriterionTag=4000500",
				"Caption=Find Documents",
				"ConstraintName=Document.Search",
				"EditorID=12",
				"Value=1",
				"QueryText=34",
				"CriterionTag=4011000",
				"Caption=where the document class is",
				"QueryText=35",
				"CriterionTag=4011000",
				"Caption=Clinical Procedures or Discharge Summaries or Rad/Nuc Med Reports",
				"ConstraintName=Document.Class",
				"EditorID=13",
				"Value=1143",
				"Value=244",
				"Value=964",
				"QueryText=36",
				"CriterionTag=4011500",
				"Caption=where the title is",
				"QueryText=37",
				"CriterionTag=4011500",
				"Caption=519 or  or  or",
				"ConstraintName=Document.Title",
				"EditorID=10",
				"Value=8",
				"Value=723",
				"Value=1138",
				"Value=1091",
				"QueryText=38",
				"CriterionTag=4012000",
				"Caption=where the document status is",
				"QueryText=39",
				"CriterionTag=4012000",
				"Caption=ACTIVE or COMPLETED or INACTIVE or RETRACTED or UNCOSIGNED or UNRELEASED",
				"ConstraintName=Document.Status",
				"EditorID=11",
				"Value=11",
				"Value=7",
				"Value=13",
				"Value=15",
				"Value=6",
				"Value=3",
				"Value=5",
				"Value=2",
				"QueryText=40",
				"CriterionTag=4012500",
				"Caption=where the author is",
				"QueryText=41",
				"CriterionTag=4012500",
				"Caption=BROWN,CLINICIAN or MISSING,USER or MORGAN,BRAD or MODERSITZKI,MARC or MY",
				"ConstraintName=Document.Author",
				"EditorID=8",
				"Value=1352",
				"Value=1121",
				"Value=1363",
				"Value=1343",
				"Value=1324",
				"QueryText=42",
				"CriterionTag=4013000",
				"Caption=where the visit location is",
				"QueryText=43",
				"CriterionTag=4013000",
				"Caption=ULTRASOUND CLINIC or REPORT OF CONTACTS or MARCIA or GENERAL MEDICINE oC",
				"ConstraintName=Document.Location",
				"EditorID=3",
				"Value=20",
				"Value=38",
				"Value=30",
				"Value=15",
				"Value=21",
				"Value=27",
				"Value=7",
				"QueryText=44",
				"CriterionTag=5000500",
				"Caption=Find Appointments/Visits",
				"ConstraintName=Visit.Search",
				"EditorID=12",
				"Value=1",
				"QueryText=45",
				"CriterionTag=5011000",
				"Caption=where the visit is for",
				"QueryText=46",
				"CriterionTag=5011000",
				"Caption=XRAY CLINIC or PULMONARY CLINIC or Metal Health or DIABETIC EDUCATION-IC",
				"ConstraintName=Visit.Location",
				"EditorID=3",
				"Value=19",
				"Value=16",
				"Value=36",
				"Value=21",
				"Value=7",
				"ColumnName=Patient.Name",
				"ColumnHeader=Name",
				"ColumnWidth=130",
				"ColumnName=Document.Author",
				"ColumnHeader=Author",
				"ColumnWidth=80",
				"ColumnName=Document.Cosigner",
				"ColumnHeader=Expected Cosigner",
				"ColumnWidth=80",
				"ColumnName=Document.Reference",
				"ColumnHeader=Note Date/Time",
				"ColumnWidth=80",
				"ColumnName=Document.Status",
				"ColumnHeader=Status",
				"ColumnWidth=80",
				"ColumnName=Document.Title",
				"ColumnHeader=Title",
				"ColumnWidth=80",
				"ColumnName=Order.Abnormal",
				"ColumnHeader=Abnormal",
				"ColumnWidth=80",
				"ColumnName=Order.Datetime",
				"ColumnHeader=Date",
				"ColumnWidth=80",
				"ColumnName=Order.Finding",
				"ColumnHeader=Finding",
				"ColumnWidth=80",
				"ColumnName=Order.Provider",
				"ColumnHeader=Requestor",
				"ColumnWidth=80",
				"ColumnName=Order.DisplayGroup",
				"ColumnHeader=Service",
				"ColumnWidth=80",
				"ColumnName=Order.Signature",
				"ColumnHeader=Signature",
				"ColumnWidth=80",
				"ColumnName=Order.Status",
				"ColumnHeader=Status",
				"ColumnWidth=80",
				"ColumnName=Order.Text",
				"ColumnHeader=Order/Request",
				"ColumnWidth=80",
				"ColumnName=Patient.Age",
				"ColumnHeader=Age",
				"ColumnWidth=80",
				"ColumnName=Patient.Last4",
				"ColumnHeader=SSN",
				"ColumnWidth=80",
				"ColumnName=Patient.Ward",
				"ColumnHeader=Ward",
				"ColumnWidth=80",
				"ColumnName=Visit.Datetime",
				"ColumnHeader=Visit Date/Time",
				"ColumnWidth=80",
				"ColumnName=Visit.Location",
				"ColumnHeader=Clinic",
				"ColumnWidth=80",
				"ColumnName=Visit.Status",
				"ColumnHeader=Status",
				"ColumnWidth=80" };
		constraintDAO.get(q,reportStrings);
		assertNotNull("MConstraintDAO.get failed:",q.getConstraints().getCriterion("1000500"));
		IValue value = (IValue)((List)q.getConstraints().getCriterion("1500500")).get(0);
		assertEquals("MConstraintDAO.get failed:","T-1246:T",value.getValues().get(0).toString());
	}
	
	public void testGetTag() throws QueryDAOException, InterruptedException {
		IQuery q = queryDAO.getAsync(new QueryID("88"),"wow");
		q.init();
		q.getConstraints().clearAll();
		q.getConstraints().setTag("5000500");
		Thread.yield();
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if ((e.getNewValue()!=null) && (e.getNewValue().toString().equals("1010500"))) {
					synchronized(MConstraintDAOTest.this) {
						MConstraintDAOTest.this.notify();
					}
				}
			}
		});
		q.getConstraints().setTag("1010500");
		synchronized(this) {
			this.wait(10000);
		}
		assertNotNull("Constraint.getTag failed:",q.getConstraints().getCriterion("5000500"));
		assertNotNull("Constraint.getTag failed:",q.getConstraints().getCriterion("1010500"));
		List list = new ArrayList();
		q.getConstraints().getSetTags(list);
		assertTrue("Constraint.getTag failed:tag not found.",list.contains("1010500"));
		synchronized(this) {
			this.wait(1000);
		}
		List crit = q.getConstraints().getCriterion("1010500");
		assertEquals("Constraint.getTag failed:",2,crit.size());
	}

}
