package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.itest.IntegrationTestRuntimeContextHolder;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.BufferModelException;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.DataRequest;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.IListPositionListener;
import gov.va.med.hds.querytool.dao.IOrderItemListDAOFactory;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.event.ListDataListener;
import junit.framework.TestCase;

/**
 * @author vhaislmonsos
 *
 * Dec 30, 2002 9:29:22 AM
 */
public class MOrderItemListDAOFactoryTest extends TestCase implements IBufferModel {
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private TestHandler testHandler = new TestHandler();
	private IOrderItemListDAOFactory factory;
	private IDataNeededListener dataProvider;
	private Object[] items;
	private int expected = -1;
	private Thread testThread;

	/**
	 * Constructor for MClinBufferListDAOFactoryTest.
	 * @param arg0
	 */
	public MOrderItemListDAOFactoryTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MOrderItemListDAOFactoryTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		Logger.getLogger(MOrderItemListDAOFactoryTest.class.getName()).setUseParentHandlers(false);
		Logger.getLogger(MOrderItemListDAOFactoryTest.class.getName()).addHandler(testHandler);
		runtime = IntegrationTestRuntimeContextHolder.getInstance();
		daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
		factory = (IOrderItemListDAOFactory)daoRegistry.getDAO(IOrderItemListDAOFactory.class.getName());
		items = null;
		expected = -1;
		dataProvider = null;
		testThread = Thread.currentThread();
	}
	
	public void testGetBufferModelDAO() throws BufferModelException {
		dataProvider = factory.getBufferModelDAO();
		assertNotNull("MOrderItemListDAOFactory.getBufferModelDAO failed: null value returned.",dataProvider);
	}
	
	public void testGetTop() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).setTop failed:",44,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).setTop failed:","   <Cbc & Morphology (With Diff) Blood>",items[0].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).setTop failed:","2hr.Gtt (Urine)",items[43].toString());
	}

	public void testGetBottom() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).setBottom failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).setBottom failed:","Wat",items[0].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).setBottom failed:","Zarontin  <Ethosuximide Cap,Oral >",items[43].toString());
	}
	
	public void testGetNext_1() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,".45NS  <0.45NS INJ (IV use)>",IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getNext failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:",44,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:",".9ns  <0.9ns Inj >",items[0].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:","3 Day Platelets  <Platelets,20-24 C, 3 Day Exp.>",items[43].toString());
	}
	
	public void testGetNext_2() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"was",IBufferModel.FORWARD_DIRECTION,20);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getNext failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:",1,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:","Wat",items[0].toString());
	}
	
	public void testGetNext_3() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"wat",IBufferModel.FORWARD_DIRECTION,20);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getNext failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext failed:",0,items.length);
	}
	
	public void testGetNext_duplicate() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,new BufferItem("0.45NS  <0.45NS INJ (IV use)>",""),FORWARD_DIRECTION,40,false);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(1000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed:",44,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed:","0.45ns Inj (Iv Use)",items[0].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed:","1044",((IBufferItem)items[0]).getId());
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed:","0.45ns Inj (Iv Use)",items[1].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).getNext(duplicate) failed:","3821",((IBufferItem)items[1]).getId());
	}
	
	public void testGetPrev_1() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"wat",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getPrev failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:",46,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:","Ventolin  <Albuterol Inhaler Inhalant >",items[0].toString());
		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:","Zantac  <Ranitidine Liquid Syrup >",items[45].toString());
	}
	
	public void testGetPrev_2() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"   <CBC & MORPHOLOGY (WITH DIFF) BLOOE>",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getPrev failed: null result set returned.",items);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:",1,items.length);
		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:","   <Cbc & Morphology (With Diff) Blood>",items[0].toString());
	}
	
	public void testGetPrev_3() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"   <CBC & MORPHOLOGY (WITH DIFF) BLOOA>",IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(5000);
		}
		assertNotNull("MBufferListDAO(MOrderItemListDAO).getPrev failed: null result set returned.",items);
//		assertEquals("MBufferListDAO(MOrderItemListDAO).getPrev failed:",0,items.length);
//		TODO: This test fails because of a problem with the M call.
	}
	
	class TestHandler extends Handler {
		public void close() throws SecurityException {
		}
		public void flush() {
		}
		public void publish(LogRecord record) {
			fail(record.getMessage());
		}
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addDataNeededListener(IDataNeededListener)
	 */
	public void addDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListDataListener(IListPositionListener)
	 */
	public void addListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getBufferFull()
	 */
	public Object[] getBufferFull() {
		return null;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getInitSize()
	 */
	public int getDefaultCallSize() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getMaxSize()
	 */
	public int getCapacity() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtBottom()
	 */
	public boolean isAtBottom() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtTop()
	 */
	public boolean isAtTop() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isBoundryKnown()
	 */
	public boolean isBoundryKnown() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeDataNeededListener(IDataNeededListener)
	 */
	public void removeDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListDataListener(IListPositionListener)
	 */
	public void removeListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setInitSize(int)
	 */
	public void setDefaultCallSize(int initSize) {
	}

	/**
	 * @see javax.swing.ListModel#addListDataListener(ListDataListener)
	 */
	public void addListDataListener(ListDataListener l) {
	}

	/**
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		return null;
	}

	/**
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return 0;
	}

	/**
	 * @see javax.swing.ListModel#removeListDataListener(ListDataListener)
	 */
	public void removeListDataListener(ListDataListener l) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getDataRequest(DataRequest)
	 */
	public boolean getDataRequest(DataRequest dataRequest) {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setData(Object[], int)
	 */
	public void setData(Object[] items, int sizeExpected) {
		this.items = items;
		this.expected = sizeExpected;
		synchronized(testThread) {
			testThread.notify();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListPositionListener(IListPositionListener)
	 */
	public void addListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListPositionListener(IListPositionListener)
	 */
	public void removeListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isCallInProgress()
	 */
	public boolean isCallInProgress() {
		return false;
	}

}
