package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.itest.IntegrationTestRuntimeContextHolder;
import gov.va.med.hds.cd.spectrum.longlist.BufferModelException;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.DataRequest;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.IListPositionListener;
import gov.va.med.hds.querytool.dao.IProcedureListDAOFactory;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.event.ListDataListener;
import junit.framework.TestCase;

/**
 * @author vhaislmonsos
 *
 * Dec 30, 2002 9:29:22 AM
 */
public class MProcedureListDAOFactoryTest extends TestCase implements IBufferModel {
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private TestHandler testHandler = new TestHandler();
	private IProcedureListDAOFactory mProcedureListDAOFactory;
	private IDataNeededListener dataProvider;
	private Object[] items;
	private int expected = -1;
	private Thread testThread;

	/**
	 * Constructor for MClinBufferListDAOFactoryTest.
	 * @param arg0
	 */
	public MProcedureListDAOFactoryTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MProcedureListDAOFactoryTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		Logger.getLogger(MProcedureListDAOFactoryTest.class.getName()).setUseParentHandlers(false);
		Logger.getLogger(MProcedureListDAOFactoryTest.class.getName()).addHandler(testHandler);
		runtime = IntegrationTestRuntimeContextHolder.getInstance();
		daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
		mProcedureListDAOFactory = (IProcedureListDAOFactory)daoRegistry.getDAO(IProcedureListDAOFactory.class.getName());
		items = null;
		expected = -1;
		dataProvider = null;
		testThread = Thread.currentThread();
	}
	
	public void testGetBufferModelDAO() throws BufferModelException {
		dataProvider = mProcedureListDAOFactory.getBufferModelDAO();
		assertNotNull("MProcedureListDAOFactory.getBufferModelDAO failed: null value returned.",dataProvider);
	}
	
	public void testGetTop() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(3000);
		}
		assertNotNull("MBufferListDAO(ProcedureListDAO).setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:",44,items.length);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","A-Lead Imp  <Atrial Lead Implant>",items[0].toString());
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","Pace Surv  <Pacemaker Surveillance>",items[43].toString());
	}

	public void testGetBottom() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(3000);
		}
		assertNotNull("MBufferListDAO(ProcedureListDAO).setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:",44,items.length);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","Ventrical Lead Implant",items[0].toString());
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","Cath  <Catheterization>",items[43].toString());
	}
	
	public void testGetNext_1() throws BufferModelException, DataProviderException, InterruptedException {
		testGetBufferModelDAO();
		dataProvider.setBufferModel(this);
		DataNeededEvent event = new DataNeededEvent(this,"A-LEAD IMP  <Atrial Lead Implant>",IBufferModel.FORWARD_DIRECTION,10);
		dataProvider.getDataRequest(event);
		synchronized(testThread) {
			testThread.wait(3000);
		}
		assertNotNull("MBufferListDAO(ProcedureListDAO).setTop failed: null result set returned.",items);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:",44,items.length);
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","Abg  <Arterial Blood Gases>",items[0].toString());
		assertEquals("MBufferListDAO(ProcedureListDAO).setTop failed:","Pacemaker Surveillance",items[43].toString());
	}
	
	
	class TestHandler extends Handler {
		public void close() throws SecurityException {
		}
		public void flush() {
		}
		public void publish(LogRecord record) {
			fail(record.getMessage());
		}
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addDataNeededListener(IDataNeededListener)
	 */
	public void addDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListDataListener(IListPositionListener)
	 */
	public void addListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getBufferFull()
	 */
	public Object[] getBufferFull() {
		return null;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getInitSize()
	 */
	public int getDefaultCallSize() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getMaxSize()
	 */
	public int getCapacity() {
		return 0;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtBottom()
	 */
	public boolean isAtBottom() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtTop()
	 */
	public boolean isAtTop() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isBoundryKnown()
	 */
	public boolean isBoundryKnown() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeDataNeededListener(IDataNeededListener)
	 */
	public void removeDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListDataListener(IListPositionListener)
	 */
	public void removeListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setInitSize(int)
	 */
	public void setDefaultCallSize(int initSize) {
	}

	/**
	 * @see javax.swing.ListModel#addListDataListener(ListDataListener)
	 */
	public void addListDataListener(ListDataListener l) {
	}

	/**
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		return null;
	}

	/**
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return 0;
	}

	/**
	 * @see javax.swing.ListModel#removeListDataListener(ListDataListener)
	 */
	public void removeListDataListener(ListDataListener l) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getDataRequest(DataRequest)
	 */
	public boolean getDataRequest(DataRequest dataRequest) {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setData(Object[], int)
	 */
	public void setData(Object[] items, int sizeExpected) {
		this.items = items;
		this.expected = sizeExpected;
		synchronized(testThread) {
			testThread.notify();
		}
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListPositionListener(IListPositionListener)
	 */
	public void addListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListPositionListener(IListPositionListener)
	 */
	public void removeListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isCallInProgress()
	 */
	public boolean isCallInProgress() {
		return false;
	}

}
