package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.querytool.m.MQueryDAO;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Tests the M implementation of the Query DAO.
 */
public class MQueryDAOTest extends MAbstractTestCase {
	private IQueryDAO queryDAO = null;
	private Thread testThread;
	private int eventCount;
	private boolean queryFired;
	private boolean parmFired;
	private boolean dispFired;

	/**
	 * Constructor for MQueryDAOTest.
	 * @param arg0
	 */
	public MQueryDAOTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MQueryDAOTest.class);
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		queryDAO = (IQueryDAO)daoRegistry.getDAO(IQueryDAO.class.getName());
		testThread = Thread.currentThread();
	}

	public void testGetCurrentReport() throws QueryDAOException, InterruptedException {
		IQuery q = null;
		q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					testThread.notify();
					eventCount++;
				}
			}
		});
		eventCount = 0;
		assertNotNull("MQueryDAO.getLightweight returned an unexpected null value.",q);
		assertEquals("MQueryDAO.getName failed:","",q.getName());
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(3000);
		}
		assertEquals("MQueryDAO.getName failed:","RPTU123456789065 EVERYTHING REPORT",q.getName());
		assertEquals("MQueryDAO failed: wrong number of events fired:",1,eventCount);
		
	}
	
	public void testGetParms() throws InterruptedException, QueryDAOException {
		IQuery q = null;
		q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					testThread.notify();
				}
			}
		});
		q.init();
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(3000);
			if (q.getName().equals("")) {
				testThread.wait(3000);
			}
		}
		int size = 0;
		for (Iterator i = q.getConstraints().iterator(); i.hasNext();) {
			i.next();
			size++;
		}
		assertEquals("MQueryDAO.getChildren failed:",31,size);
	}
	
	public void testGetDisplay() throws InterruptedException, QueryDAOException {
		IQuery q = null;
		q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					testThread.notify();
				}
			}
		});
		q.init();
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(3000);
			if (q.getName().equals("")) {
				testThread.wait(3000);
			}
		}
		eventCount = 0;
		assertEquals("MQueryDAO.getDisplayField failed: wrong number of fields returned:",4,q.getDisplayFields().size());
	}
	
	public void testGetAsync() throws InterruptedException, QueryDAOException {
		final IQuery q;
		q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					synchronized (testThread) {
						queryFired=true;
						if (queryFired&&dispFired&&parmFired) {
							testThread.notify();
						}
					}
				}
			}
		});
		q.init();
		synchronized(testThread) {
			testThread.wait(3000);
		}
		assertEquals("MQueryDAO getting all data failed:","Everything Report",q.getDisplayName());
		int size = 0;
		for (Iterator i = q.getConstraints().iterator(); i.hasNext();) {
			i.next();
			size++;
		}
		assertEquals("MQueryDAO getting all data failed:",31,size);
		
	}
	
	public void testGetReportStrings() throws QueryDAOException, InterruptedException {
		IQuery q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					testThread.notify();
				}
			}
		});
		q.init();
		synchronized(testThread) {
			testThread.wait(3000);
		}
		List strings = queryDAO.getReportStrings(q);
		assertNotNull("MQueryDAO.getReportStrings failed: null return strings.",strings);
		assertTrue("MQueryDAO.getReportStrings failed: string count is 0.",strings.size()>0);
//		System.out.println("=====Report Strings=====");
//		for (int i = 0; i < strings.size(); i++) {
//			System.out.println(strings.get(i));
//		}
	}
	
	public void testGetSync() throws QueryDAOException {
		IQuery q = queryDAO.getSync(new QueryID("88"));
		assertEquals("QueryDAO.getSync failed:","Everything Report",q.getDisplayName());
		assertEquals("QueryDAO.getSync failed:","RPTU123456789065 EVERYTHING REPORT",q.getName());
		assertEquals("QueryDAO.getSync failed:",4,q.getDisplayFields().size());
		IDisplayField field = q.getDisplayFields().get(0);
		assertEquals("QueryDAO.getSync failed:","Name",field.getDisplayText());
		field = q.getDisplayFields().get(3);
		assertEquals("QueryDAO.getSync failed:","Ward",field.getDisplayText());
		List list = new ArrayList();
		for (Iterator i = q.getConstraints().iterator(); i.hasNext();) {
			list.add(i.next());
		}
		assertEquals("QueryDAO.getSync failed:",31,list.size());
		assertEquals("QueryDAO.getSync failed:","1",((IValue)list.get(0)).getEditorID());
		assertEquals("QueryDAO.getSync failed:","Patient.ListSource",((IValue)list.get(0)).getConstraintName());
		
		assertEquals("QueryDAO.getSync failed:","12",((IValue)list.get(30)).getEditorID());
		assertEquals("QueryDAO.getSync failed:","Visit.NegativeSearch",((IValue)list.get(30)).getConstraintName());
		
	}
	
	public void testGetDefnStrings() throws NumberFormatException, QueryDAOException, InterruptedException {
		IQuery q = queryDAO.getAsync(new QueryID("88"), "test");
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				synchronized (testThread) {
					testThread.notify();
				}
			}
		});
		q.init();
		synchronized(testThread) {
			testThread.wait(3000);
		}
		List strings = ((MQueryDAO)queryDAO).getDefnStrings(q);
		assertEquals("MQueryDAO.getDefnStrings failed:","IEN=88",strings.get(0).toString());
		assertEquals("MQueryDAO.getDefnStrings failed:","Name=RPTU123456789065 EVERYTHING REPORT",strings.get(1).toString());
		assertEquals("MQueryDAO.getDefnStrings failed:","DisplayText=Everything Report",strings.get(2).toString());
		int columnStart = -1;
		int queryTextCount = 0;
		for (int i = 3; i < strings.size(); i++) {
			if (strings.get(i).toString().startsWith("ColumnName=")) {
				columnStart = i;
				break;
			}
			if (strings.get(i).toString().startsWith("QueryText=")) {
				queryTextCount++;
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+1).toString() + "' in incorrect position.",strings.get(i+1).toString().startsWith("CriterionTag="));
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+2).toString() + "' in incorrect position.",strings.get(i+2).toString().startsWith("Caption="));
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+3).toString() + "' in incorrect position.",strings.get(i+3).toString().startsWith("ConstraintName="));
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+4).toString() + "' in incorrect position.",strings.get(i+4).toString().startsWith("EditorID="));
				if (strings.get(i+5).toString().startsWith("Value=")) {
					int j = i+5;
					String val = null;
					while ((val=strings.get(j).toString()).startsWith("Value=")) {
						assertTrue("MQueryDAO.getDefnStrings failed: Value field is empty.",val.length()>6);
						j++;
					}
				}
			}
		}
		assertTrue("MQueryDAO.getDefnStrings failed: No QueryText strings found.",queryTextCount>0);
		
		int columnDefCount = 0;
		for (int i = columnStart; i < strings.size(); i++) {
			if (strings.get(i).toString().startsWith("Column=")) {
				columnDefCount++;
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+1).toString() + "' in incorrect position.",strings.get(i+1).toString().startsWith("ColumnName="));
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+2).toString() + "' in incorrect position.",strings.get(i+2).toString().startsWith("ColumnHeader="));
				assertTrue("MQueryDAO.getDefnStrings failed: '" + strings.get(i+3).toString() + "' in incorrect position.",strings.get(i+3).toString().startsWith("ColumnWidth="));
			}
		}
		assertTrue("MQueryDAO.getDefnStrings failed: No column definition strings found.",columnDefCount>0);
//		System.out.println("=====Query Save Strings=====");
//		for (int i = 0; i < strings.size(); i++) {
//			System.out.println(strings.get(i));
//		}
	}
	
}
