/*
 * Initially Created on Jul 21, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.itest;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.DisplayField;
import gov.va.med.hds.querytool.IAvailableColumnsList;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.ui.SampleTableModel;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

/**
 * Tests for the SampleTableModel model.
 */
public class SampleTableModelTest extends MAbstractTestCase implements IAvailableColumnsList {
	private SampleTableModel model;
	private List list;
	private ListDataListener dummyListener;
	private List eventList;

	public SampleTableModelTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(SampleTableModelTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		model = new SampleTableModel();
		model.setAvailableColumnsModel(this);
		list = new ArrayList();
		eventList = new ArrayList();
	}

	public void testGetRowCount() {
		assertEquals("SampleTableModel.getRowCount() failed:",0,model.getRowCount());
		setupRows(10);
		assertEquals("SampleTableModel.getRowCount() failed:",2,model.getRowCount());
	}

	public void testGetColumnCount() {
		assertEquals("SampleTableModel.getColumnCount() failed:",0,model.getColumnCount());
		setupRows(10);
		assertEquals("SampleTableModel.getColumnCount() failed:",10,model.getColumnCount());
	}

	public void testGetValueAt() {
		assertNull("SampleTableModel.getValueAt() failed: expected NULL",model.getValueAt(0, 0));
		assertNull("SampleTableModel.getValueAt() failed: expected NULL",model.getValueAt(Integer.MAX_VALUE, Integer.MAX_VALUE));
		setupRows(8);
		assertNotNull("SampleTableModel.getValueAt() failed: expected non-NULL",model.getValueAt(0, 0));
		assertNotNull("SampleTableModel.getValueAt() failed: expected non-NULL",model.getValueAt(1, 7));
		assertEquals("SampleTableModel.getValueAt() failed:","jane7",model.getValueAt(1, 7));
		assertEquals("SampleTableModel.getValueAt() failed:","fred0",model.getValueAt(0, 0));
	}
	
	public void testListeners() throws InterruptedException {
		TableModelListener modelListener = new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				eventList.add(e);
				if (!SwingUtilities.isEventDispatchThread()) {
					fail("SampleTableModel listener processing failed: events not using the Swing event thread.");
				}
			}
		};
		assertNotNull("SampleTableModel listener failed: listener not registered",dummyListener);
		model.addTableModelListener(modelListener);
		ListDataEvent event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,111,222);
		dummyListener.contentsChanged(event);
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(100);
		}
		assertEquals("SampleTableModel listener failed:",1,eventList.size());
		TableModelEvent tableEvent = (TableModelEvent)eventList.get(0);
		assertEquals("SampleTableModel listener failed:",TableModelEvent.UPDATE,tableEvent.getType());
		assertEquals("SampleTableModel listener failed:",-1,tableEvent.getColumn());
		assertEquals("SampleTableModel listener failed:",-1,tableEvent.getFirstRow());
		assertEquals("SampleTableModel listener failed:",-1,tableEvent.getLastRow());

		eventList.clear();
		model.removeTableModelListener(modelListener);
		event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,1111,2222);
		dummyListener.contentsChanged(event);
		assertEquals("SampleTableModel listener failed:",0,eventList.size());
	}

	public void addListDataListener(ListDataListener listener) {
		dummyListener = listener;
	}
	
	public void removeListDataListener(ListDataListener listener) {
		dummyListener = null;
	}

	public IDisplayField get(int index) {
		if (index>-1 && index<list.size()) {
			return (IDisplayField)list.get(index);
		}
		return null;
	}

	public int size() {
		return list.size();
	}

	public void setModel(IQuery query) {}
	public void setRuntime(IRuntimeContext runtime) {}
	public int indexOf(IDisplayField field) {return 0;}

	private void setupRows(int count) {
		for (int i = 0; i < count; i++) {
			IDisplayField field = new DisplayField();
			field.setColumnWidth(100+i);
			field.setDisplayText("displayText"+Integer.toString(i));
			field.setID("wow"+Integer.toString(i));
			field.setListDisplayName("listDisplayName"+Integer.toString(i));
			field.setSampleData(new String[]{"fred"+Integer.toString(i),"jane"+Integer.toString(i)});
			field.setSortType("aaa"+Integer.toString(i));
			list.add(field);
		}
	}

	public IDisplayField getMutable(int index) {
		return get(index);
	}

	
}
