/*
 * Initially Created on Mar 14, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.test;

import gov.va.med.hds.querytool.ui.ColumnListModel;
import gov.va.med.hds.querytool.ui.IColumnListModel;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.table.TableColumn;

import junit.framework.TestCase;

/**
 * Tests the ColumnListModel which provides a TableColumnModel and ListModel to 
 * facilitate the selection of query display fields.
 */
public class ColumnListModelTest extends TestCase {
	private IColumnListModel model;
	private int addEventCount;
	private int removeEventCount;
	private int changeEventCount;

	public ColumnListModelTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(ColumnListModelTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		model = new ColumnListModel();
		TableColumn col = new TableColumn(0);
		col.setHeaderValue("a1");
		col.setIdentifier("fred");
		col.setWidth(80);
		col.setResizable(true);
		model.addColumn(col);
	}

	public void testGetSize() {
		assertEquals("ColumnListModel.getSize failed:",1,model.getSize());
		assertEquals("ColumnListModel.getSize failed:",1,model.getColumnCount());
	}

	public void testGetElementAt() {
		assertEquals("ColumnListModel.getElementAt failed:","fred",model.getElementAt(0).toString());
		assertTrue("ColumnListModel.getElementAt failed:",model.getElementAt(0) instanceof String);
	}

	public void testListDataListener() {
		ListDataListener d = new ListDataListener() {
			public void intervalAdded(ListDataEvent e) {
				addEventCount++;
			}
			public void intervalRemoved(ListDataEvent e) {
				removeEventCount++;
			}
			public void contentsChanged(ListDataEvent e) {
				changeEventCount++;
			}
		};
		model.addListDataListener(d);
		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		TableColumn col = new TableColumn(0);
		col.setHeaderValue("b2");
		col.setIdentifier("jill");
		col.setWidth(800);
		col.setResizable(false);
		model.addColumn(col);
		assertEquals("ColumnListModel.addListDataListener failed:",1,addEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",0,removeEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",0,changeEventCount);
		
		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		model.moveColumn(0, 1);
		assertEquals("ColumnListModel.addListDataListener failed:",0,addEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",0,removeEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",1,changeEventCount);
		
		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		model.removeColumn(model.getColumn(0));
		assertEquals("ColumnListModel.addListDataListener failed:",0,addEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",1,removeEventCount);
		assertEquals("ColumnListModel.addListDataListener failed:",0,changeEventCount);

		model.removeListDataListener(d);
		col = new TableColumn(0);
		col.setHeaderValue("c3");
		col.setIdentifier("henry");
		col.setWidth(8000);
		col.setResizable(false);
		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		model.addColumn(col);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,addEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,removeEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,changeEventCount);

		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		model.moveColumn(0, 1);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,addEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,removeEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,changeEventCount);
		
		addEventCount = 0;
		removeEventCount = 0;
		changeEventCount = 0;
		model.removeColumn(model.getColumn(1));
		assertEquals("ColumnListModel.removeListDataListener failed:",0,addEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,removeEventCount);
		assertEquals("ColumnListModel.removeListDataListener failed:",0,changeEventCount);
		
	}

}
