package gov.va.med.hds.querytool.test;

import gov.va.med.hds.querytool.DisplayField;
import gov.va.med.hds.querytool.DisplayFieldList;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IDisplayFieldList;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import junit.framework.TestCase;

/**
 * Tests for the display field list model.
 */
public class DisplayFieldListTest extends TestCase {
	private IDisplayFieldList list;
	private int displayedListenerCalled = 0;
	private ListDataListener displayedListener = null;
	private int availableListenerCalled = 0;
	private ListDataListener availableListener = null;
	private Object lock = new Object();
	private int callCount;
	private int callSize;
	private Thread testThread;
	private ListDataEvent event;

	/**
	 * Constructor for DisplayFieldListTest.
	 * @param name
	 */
	public DisplayFieldListTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(DisplayFieldListTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		testThread = Thread.currentThread();
		list = new DisplayFieldList();
		super.setUp();
		displayedListener = new ListDataListener() {
			public void intervalRemoved(ListDataEvent e) {
				displayedListenerCalled = 1;
				event = e;
				clearLock();
			}
			public void contentsChanged(ListDataEvent e) {
				displayedListenerCalled = 2;
				event = e;
				clearLock();
			}
			public void intervalAdded(ListDataEvent e) {
				displayedListenerCalled = 4;
				event = e;
				clearLock();
			}
		};
		availableListener = new ListDataListener() {
			public void intervalRemoved(ListDataEvent e) {
				availableListenerCalled = 1;
				clearLock();
			}
			public void contentsChanged(ListDataEvent e) {
				availableListenerCalled = 2;
				clearLock();
			}
			public void intervalAdded(ListDataEvent e) {
				availableListenerCalled = 4;
				clearLock();
			}
		};
	}
	
	private void clearLock() {
		synchronized(lock) {
			lock.notify();
		}
	}

	public void testSizeAdd() {
		IDisplayField displayField = new DisplayField();
		displayField.setDisplayText("Test column");
		displayField.setColumnWidth(80);
		displayField.setID("Test.Column1");
		list.add(displayField);
		assertEquals("DisplayFieldList.getSize or add failed: Wrong number of display fields:",1,list.size());
		displayField = new DisplayField();
		displayField.setDisplayText("Test column2");
		displayField.setColumnWidth(40);
		displayField.setID("Test.Column2");
		list.add(displayField);
		assertEquals("DisplayFieldList.getSize or add failed: Wrong number of display fields:",2,list.size());
		displayField = new DisplayField();
		displayField.setDisplayText("Test column3");
		displayField.setColumnWidth(40);
		displayField.setID("Test.Column3");
		list.add(displayField);
		assertEquals("DisplayFieldList.getSize or add failed: Wrong number of display fields:",3,list.size());
		displayField = new DisplayField();
		displayField.setDisplayText("Test column4");
		displayField.setColumnWidth(40);
		displayField.setID("Test.Column4");
		list.add(displayField);
		assertEquals("DisplayFieldList.getSize or add failed: Wrong number of display fields:",4,list.size());
	}

	public void testAddListDataListener() throws InterruptedException {
		list.addListDataListener(displayedListener);
		displayedListenerCalled = 0;
		testSizeAdd();
		if (displayedListenerCalled==0) {
			synchronized(lock) {
				lock.wait();
			}
		}
		assertEquals("DisplayFieldList listeners were not called:",4,displayedListenerCalled);
		
		displayedListenerCalled = 0;
		event = null;
		list.moveUp(1);
		assertEquals("DisplayFieldList listeners were not called:",2,displayedListenerCalled);
		assertNotNull("DisplayFieldList listener failed: event object is null.",event);
		assertEquals("DisplayFieldList listener failed:",1,event.getIndex0());
		assertEquals("DisplayFieldList listener failed:",0,event.getIndex1());
		
		displayedListenerCalled = 0;
		event = null;
		list.moveDown(2);
		assertEquals("DisplayFieldList listeners were not called:",2,displayedListenerCalled);
		assertNotNull("DisplayFieldList listener failed: event object is null.",event);
		assertEquals("DisplayFieldList listener failed:",2,event.getIndex0());
		assertEquals("DisplayFieldList listener failed:",3,event.getIndex1());
	}

	public void testGetElementAt() {
		testSizeAdd();
		assertEquals("DisplayFieldList.getElementAt:","Test column",list.get(0).getDisplayText());
		assertEquals("DisplayFieldList.getElementAt:","Test.Column2",list.get(1).getID());
		assertEquals("DisplayFieldList.getElementAt:","Test column3",list.get(2).getDisplayText());
	}
	
	public void testClearAll() throws InterruptedException {
		callCount = 0;
		callSize = 0;
		list.addListDataListener(new ListDataListener() {
			public void intervalAdded(ListDataEvent e) {}
			public void contentsChanged(ListDataEvent e) {}
			public void intervalRemoved(ListDataEvent e) {
				callCount++;
				callSize = e.getIndex1()-e.getIndex0()+1;
				notifyThread();
			}
		});
		testSizeAdd();
		assertEquals("DisplayFieldList ClearAll test failed:",4,list.size());
		list.clearAll();
		synchronized (testThread) {
			testThread.wait(1000);
		}
		assertEquals("DisplayFieldList.clearAll failed:",0,list.size());
		assertEquals("DisplayFieldList.clearAll failed:",1,callCount);
		assertEquals("DisplayFieldList.clearAll failed:",4,callSize);
	}
	
	public void testMoveDown() {
		IDisplayField f1 = new DisplayField();
		f1.setDisplayText("f1");
		f1.setID("f1");
		IDisplayField f2 = new DisplayField();
		f2.setDisplayText("f2");
		f2.setID("f2");
		list.add(f1);
		list.add(f2);
		list.moveDown(1);
		assertEquals("DisplayFieldList.moveDown failed:",2,list.size());
		assertEquals("DisplayFieldList.moveDown failed:","f1",list.get(0).getDisplayText());
		assertEquals("DisplayFieldList.moveDown failed:","f2",list.get(1).getDisplayText());
		list.moveDown(0);
		assertEquals("DisplayFieldList.moveDown failed:","f2",list.get(0).getDisplayText());
		assertEquals("DisplayFieldList.moveDown failed:","f1",list.get(1).getDisplayText());
	}
	
	public void testMoveUp() {
		IDisplayField f1 = new DisplayField();
		f1.setDisplayText("f1");
		f1.setID("f1");
		IDisplayField f2 = new DisplayField();
		f2.setDisplayText("f2");
		f2.setID("f2");
		list.add(f1);
		list.add(f2);
		list.moveUp(0);
		assertEquals("DisplayFieldList.moveDown failed:",2,list.size());
		assertEquals("DisplayFieldList.moveDown failed:","f1",list.get(0).getDisplayText());
		assertEquals("DisplayFieldList.moveDown failed:","f2",list.get(1).getDisplayText());
		list.moveUp(1);
		assertEquals("DisplayFieldList.moveDown failed:","f2",list.get(0).getDisplayText());
		assertEquals("DisplayFieldList.moveDown failed:","f1",list.get(1).getDisplayText());
	}
	
	private void notifyThread() {
		synchronized (testThread) {
			testThread.notifyAll();
		}
	}

}
