package gov.va.med.hds.querytool.test;

import java.awt.event.ActionEvent;

import gov.va.med.hds.cd.spectrum.linkpanel.HTMLSelectionModel;
import gov.va.med.hds.cd.spectrum.linkpanel.ISelectionEventListener;
import gov.va.med.hds.querytool.types.NoEditor;

import junit.framework.TestCase;

/**
 * @author vhaislmonsos
 *
 * Nov 20, 2002 11:11:15 PM
 */
public class HTMLSelectionModelTest extends TestCase {
	private HTMLSelectionModel model = null;
	private int count = 0;
	/**
	 * Constructor for HTMLSelectionModelTest.
	 * @param arg0
	 */
	public HTMLSelectionModelTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(HTMLSelectionModelTest.class);
	}
	
	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		model = new HTMLSelectionModel();
		count = 0;
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
		model = null;
	}

	public void testAddHTMLTest() {
		model.addHTMLText("fred");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred",model.getHTML());
		
		model.addHTMLText(" is a very wierd person.");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred is a very wierd person.",model.getHTML());
		
		model.addHTMLText("  That's what I said!");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred is a very wierd person.  That's what I said!",model.getHTML());
	}
	
	public void testAddSelectableObject1() {
		model.addHTMLText("fred ");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred ",model.getHTML());
		
		int objectID = model.addSelectableObject(new NoEditor());
		model.setDescription("ow", objectID);
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" >ow</a>",model.getHTML());
		
		model.addHTMLText("jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" >ow</a>jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ",model.getHTML());
		
	}

	public void testAddSelectableObject2() {
		model.addHTMLText("jane ");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","jane ",model.getHTML());
		
		model.addSelectableObject(new NoEditor(),"ClIcK hErR nOw","style=\"color: #808080\"");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","jane <a href=\"http://.1.\" style=\"color: #808080\">ClIcK hErR nOw</a>",model.getHTML());
		
	}
	
	public void testSetters() {
		model.addHTMLText("fred ");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred ",model.getHTML());
		
		int objectID = model.addSelectableObject(new NoEditor());
		model.setDescription("why",objectID);
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" >why</a>",model.getHTML());
		
		model.addHTMLText("jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ");
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" >why</a>jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ",model.getHTML());
		
		model.setDescription("<PLEASE CLICK HERE NOW>",objectID);
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" >&lt;PLEASE CLICK HERE NOW&gt;</a>jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ",model.getHTML());
		
		model.setStyle("style=\"color: #800080\"",objectID);
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" style=\"color: #800080\">&lt;PLEASE CLICK HERE NOW&gt;</a>jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ",model.getHTML());
		
		model.setObject(new String("WOW"),objectID);
		model.setDescription("",objectID);
		assertEquals("HTMLSelectionModel.addHTMLText failed:","fred <a href=\"http://.1.\" style=\"color: #800080\">WOW</a>jdks gioerjgoewihjprehjjpoiuehrshpoireshyiushdfkhlkjhlksh hjkhkhdh sfl ",model.getHTML());
		
	}
	
	public void testTextChangeListeners() {
		model.addHTMLText("wewewew");
		model.addEventListener(new ISelectionEventListener() {
			public void itemSelected(ActionEvent ae) {
				if (ae.getActionCommand().equals(HTMLSelectionModel.TEXT_CHANGED)) {
					assertNotNull("HTMLSelectionModel ActionEvent generator failed to return the source.",ae.getSource());
					assertEquals("HTMLSelectionModel ActionEvent generator failed to return the right event ID:",HTMLSelectionModel.TEXT_CHANGED,ae.getActionCommand());
					count++;
				}
			}
		});
		model.addHTMLText("line2");
		assertEquals("HTMLSelectionModel listeners not called.",1,count);
	}
	
	public void testTextChangeListeners2() {
		model.addHTMLText("wewewew");
		model.addEventListener(new ISelectionEventListener() {
			public void itemSelected(ActionEvent ae) {
				if (ae.getActionCommand().equals(HTMLSelectionModel.TEXT_CHANGED)) {
					assertNotNull("HTMLSelectionModel ActionEvent generator failed to return the source.",ae.getSource());
					assertEquals("HTMLSelectionModel ActionEvent generator failed to return the right event ID:",HTMLSelectionModel.TEXT_CHANGED,ae.getActionCommand());
					count++;
				}
			}
		});
		model.clear();
		assertEquals("HTMLSelectionModel listeners not called.",1,count);
	}
	
	public void testEventListeners() {
		model.addHTMLText("yoyoyoyoyoyo");
		String obj = new String("high their");
		model.addEventListener(new ISelectionEventListener() {
			public void itemSelected(ActionEvent ae) {
				if (ae.getActionCommand().equals(HTMLSelectionModel.ITEM_SELECTED)) {
					Object source = ae.getSource();
					assertNotNull("HTMLSelectionModel ActionEvent generator failed to return the source.",ae.getSource());
					assertEquals("HTMLSelectionModel ActionEvent generator failed to return the correct object:",String.class,source.getClass());
					assertEquals("HTMLSelectionModel ActionEvent generator failed to return the right event ID:",HTMLSelectionModel.ITEM_SELECTED,ae.getActionCommand());
					count++;
				}
			}
		});
		model.addHTMLText("line2");
		model.addSelectableObject(obj);
		assertEquals("HTMLSelectionModel listeners unexpectedly called.",0,count);
		model.fireObjectSelected(1);
		assertEquals("HTMLSelectionModel listeners not called.",1,count);
		
	}
	
}
