package gov.va.med.hds.querytool.test;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.CriteriaItem;
import gov.va.med.hds.querytool.ICriteriaItem;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.dao.ConstraintDAOException;
import gov.va.med.hds.querytool.dao.IConstraintDAO;
import gov.va.med.hds.querytool.types.ITypeFactory;
import gov.va.med.hds.querytool.types.TypeFactory;
import gov.va.med.hds.util.mumps.NameValuePair;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * Mock constraint DAO for testing the Query class.
 */
public class MockMConstraintDAO implements IConstraintDAO {
	private IRuntimeContext runtime;
	private ICriteriaItem priorItem;
	private String[] constraintTree;
	private ITypeFactory typeFactory;
	private Map displayFieldCategoryMap = new HashMap();

	public MockMConstraintDAO() {
		super();
		constraintTree = getConstraintTree();
		Arrays.sort(constraintTree);
	}

	public void get(IQuery query, String[] constraintData) {
		query.setChanging(true);
		query.getConstraints().clearAll();
		if (constraintData != null) {
			for (int i = 0; i < constraintTree.length; i++) {
				query.getConstraints().addRelationship(constraintTree[i]);
			}
			ICriteriaItem cItem = new CriteriaItem();
			boolean hasValue = false;
			for (int i = 0; i < constraintData.length; i++) {
				NameValuePair line = new NameValuePair(constraintData[i]);
				String left = line.getLeft();
				String right = line.getRight();
				if (left.compareTo("QueryText") == 0) {
					if (hasValue) {
						saveItem(cItem, query);
						cItem = new CriteriaItem();
					} else {
						hasValue = true;
					}
					cItem.setOrder(Integer.parseInt(right));
				} else if (left.compareTo("CriterionTag") == 0) {
					cItem.setTag(right);
				} else if (left.compareTo("Caption") == 0) {
					cItem.setCaption(right);
				} else if (left.compareTo("ConstraintName") == 0) {
					cItem.setConstraintName(right);
				} else if (left.compareTo("EditorID") == 0) {
					cItem.setEditorID(right);
				} else if (left.compareTo("Value") == 0) {
					cItem.getValues().add(right);
				}
			}
			if (hasValue) {
				saveItem(cItem, query);
			}
		}
		query.setChanging(false);
	}

	private void saveItem(ICriteriaItem cItem, IQuery query) {
		//		System.out.println("saveItem");
		//if cItem is a header value
		if (!isValue(cItem)) {
			if (priorItem != null) {
				saveValue(priorItem, null, priorItem.getTag(), query);
			}
			priorItem = cItem;
			cItem = null;
		}
		//	if the previous is a header, save previous as a value; make cItem the previous
		//}

		if (cItem != null) {
			if ((priorItem != null) && (cItem.getTag().compareTo(priorItem.getTag()) == 0)) {
				saveValue(priorItem, cItem, cItem.getTag(), query);
				priorItem = null;
				cItem = null;
			}
			if (priorItem != null) {
				saveValue(priorItem, null, priorItem.getTag(), query);
				priorItem = null;
			}
			if (cItem != null) {
				saveValue(null, cItem, cItem.getTag(), query);
				cItem = null;
			}
		}
		//if cItem is not a header
		//	if cItem and previous have the same id number, save both as one value
		//	else, save previous and cItem as separate values
		//	clear previous.
		//}

	}

	private void saveValue(ICriteriaItem item1, ICriteriaItem item2, String tag, IQuery query) {
		//		System.out.println("saveValue");

		IValue value = null;
		if ((item1 != null) && (item2 != null)) {
			value = typeFactory.getValue(item2.getEditorID());
			value.setDisplayText(item1.getCaption());
			value.setDefaultDisplayText(item2.getCaption());
			value.setValues(item2.getValues());
			value.setConstraintName(item2.getConstraintName());
			item1 = null;
			item2 = null;
		}

		if (item1 != null) {
			value = typeFactory.getValue("12");
			value.setDefaultDisplayText(item1.getCaption());
		}

		if (item2 != null) {
			value = typeFactory.getValue(item2.getEditorID());
			value.setDefaultDisplayText(item2.getCaption());
			value.setValues(item2.getValues());
			value.setConstraintName(item2.getConstraintName());
		}

		value.setID(tag);
		if (value.getConstraintName()!=null && value.getConstraintName().length()>0) {
			value.setDisplayFieldCategory(displayFieldCategoryMap.get(value.getConstraintName().substring(0,1)));
		}
		query.getConstraints().addValue(value);

	}

	private boolean isValue(ICriteriaItem item) {
		//		System.out.println("isValue");
		return (item.getEditorID() != CriteriaItem.NULL_EDITOR);
	}

	private String[] getConstraintTree() {
		//		System.out.println("getConstraintTree");
		String[] result = MockSharedBroker.getCriteriaHierarchy();
		Arrays.sort(result);
		return result;
	}

	private void getDisplayFieldCategoryMap() throws ConstraintDAOException {
		displayFieldCategoryMap.put("P","P");
		displayFieldCategoryMap.put("O","O");
		displayFieldCategoryMap.put("V","V");
		displayFieldCategoryMap.put("C","O");
		displayFieldCategoryMap.put("D","D");
	}

	public void setInitializationData(
					IRuntimeContext runtime,
					IConfigurationElement config,
					String propertyName, Object data)
					throws Exception {

		getDisplayFieldCategoryMap();
		this.runtime = runtime;
		typeFactory = new TypeFactory(runtime);
	}

	public void getTag(IQuery query, String tag) throws ConstraintDAOException {
	}

}
