package gov.va.med.hds.querytool.test;

import java.util.TreeMap;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.querytool.dao.IDisplayGroupTreeDAO;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 * Mock m display group tree DAO implementation.
 */
public class MockMDisplayGroupTreeDAO implements IDisplayGroupTreeDAO {

	/**
	 * @see gov.va.med.hds.querytool.dao.ITreeModelDAO#getTree()
	 */
	public TreeModel getTree() {
		String[] results = getBrokerData();
		TreeMap nodeMap = new TreeMap();
		TextNode textNode = new TextNode(results[0]);
		IBufferItem item = new BufferItem(textNode.getValue(), textNode.getNodeID());
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(item);
		nodeMap.put(textNode.getNodeID(), root);
		TreeModel result = new DefaultTreeModel(root);
		for (int i = 1; i < results.length; i++) {
			textNode = new TextNode(results[i]);
			item = new BufferItem(textNode.getValue(), textNode.getNodeID());
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(item);
			nodeMap.put(textNode.getNodeID(), node);
			DefaultMutableTreeNode parent =
				(DefaultMutableTreeNode) nodeMap.get(textNode.getParentID());
			parent.add(node);
		}
		return result;
	}

	private class TextNode {
		private String nodeID;
		private String value;
		private String parentID;

		private TextNode() {
		}

		public TextNode(String text) {
			this.nodeID = StringUtil.piece(text, '^', 1);
			this.value = StringUtil.piece(text, '^', 2);
			this.parentID = StringUtil.piece(text, '^', 3);
		}
		/**
		 * Returns the nodeID.
		 * @return String
		 */
		public String getNodeID() {
			return nodeID;
		}

		/**
		 * Returns the parentID.
		 * @return String
		 */
		public String getParentID() {
			return parentID;
		}

		/**
		 * Returns the value.
		 * @return String
		 */
		public String getValue() {
			return value;
		}

	}

	/**
	 * @see gov.va.med.hds.cd.daoregistry.IDataAccessObject#init(IRuntimeContext)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName, Object data)
		throws Exception {
		//Setup CallQueue here for the real M...DAO
	}

	/**
	 * Method getBrokerData.
	 * @return String[]
	 */
	private String[] getBrokerData() {
		return new String[] {
			"1^ALL SERVICES^0^+",
			"2^PHARMACY^1^+",
			"24^INPATIENT MEDICATIONS^2^+",
			"31^UNIT DOSE MEDICATIONS^24",
			"32^IV MEDICATIONS^24^+",
			"61^PARENTERAL NUTRITION^32",
			"25^OUTPATIENT MEDICATIONS^2",
			"3^LABORATORY^1^+",
			"8^CHEMISTRY^3",
			"9^HEMATOLOGY^3",
			"10^MICROBIOLOGY^3",
			"53^BLOOD BANK^3",
			"54^ANATOMIC PATHOLOGY^3",
			"34^ELECTRON MICROSCOPY^3",
			"36^SURGICAL PATHOLOGY^3",
			"33^AUTOPSY^3",
			"63^CYTOLOGY^3",
			"13^IMAGING^1^+",
			"46^CT SCAN^13",
			"45^GENERAL RADIOLOGY^13",
			"47^MAGNETIC RESONANCE IMAGING^13",
			"48^ANGIO/NEURO/INTERVENTIONAL^13^+",
			"49^CARDIOLOGY STUDIES (NUC MED)^13^+",
			"50^NUCLEAR MEDICINE^13",
			"51^ULTRASOUND^13",
			"52^VASCULAR LAB^13",
			"62^MAMMOGRAPHY^13",
			"14^DIETETICS^1^+",
			"19^DIET ORDERS^14",
			"20^TUBEFEEDINGS^14",
			"21^DIETETIC CONSULTS^14",
			"22^DIET ADDITIONAL ORDERS^14",
			"23^EARLY/LATE TRAYS^14",
			"60^PRECAUTIONS^14",
			"30^CONSULTS^1^+",
			"55^PROCEDURES^1",
			"18^VITALS/MEASUREMENTS^1",
			"26^NURSING^1^+",
			"43^ACTIVITY^26",
			"27^SURGERY^1",
			"28^M.A.S.^1^+",
			"57^DIAGNOSIS^28",
			"58^CONDITION^28",
			"16^OTHER HOSPITAL SERVICES^1",
			"56^ALLERGIES^1",
			"59^SUPPLIES/DEVICES^1" };
	}

}
