package gov.va.med.hds.querytool.test;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.cd.m.CallQueueException;
import gov.va.med.hds.cd.m.CallRequest;
import gov.va.med.hds.cd.m.ICallQueue;
import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.IDataNeededListener;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryID;
import gov.va.med.hds.querytool.Query;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.ConstraintDAOException;
import gov.va.med.hds.querytool.dao.IConstraintDAO;
import gov.va.med.hds.querytool.dao.IDisplayFieldListDAO;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.util.mumps.NameValuePair;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 * @author vhaislmonsos
 *
 * Nov 12, 2002 11:03:57 AM
 */
public class MockMQueryDAO implements IQueryDAO {
	private final int QUERY_CALL = 0;
	private final int PARM_CALL = 1;
	private final int DISP_CALL = 2;
	private IDaoRegistry daoRegistry;
	private IRuntimeContext runtime;
	private ICallQueue queue;
	private Map queryObjectQueue = new HashMap();
	private IDisplayFieldListDAO displayFieldListDAO;
	private IConstraintDAO constraintDAO;
	private IDataNeededListener dataNeededListener = new IDataNeededListener() {
		public void actionPerformed(ActionEvent e) {
			fireDataRequest(e);
		}
	};
	private Map queryLines = new HashMap();
	private Map parmLines = new HashMap();
	private Map dispLines = new HashMap();

	/**
	 * Constructor for MQueryDAO.
	 */
	public MockMQueryDAO() {
		super();
	}

	/**
	 * @see gov.va.med.hds.querytool.IQueryDAO#getLightweight(QueryID, IUserContext)
	 */
	public IQuery getAsync(IQueryID queryID, String displayText) {
		IQuery q = null;
		if ((q = (IQuery) queryObjectQueue.get(queryID)) != null) {
			return q;
		}

		q = new Query(queryID, null);
		q.setDisplayName(displayText);
		q.setChanged(false);
		queryObjectQueue.put(queryID, q);
		setQuery(q);
		return q;
	}

	public IQuery getSync(IQueryID queryID) throws QueryDAOException {
		IQuery q = null;
		if ((q = (IQuery) queryObjectQueue.get(queryID)) != null) {
			return q;
		}

		String[] results = MockSharedBroker.getQuery(queryID);
		if (results.length > 1) {
			q = new Query(queryID, null);
			NameValuePair reportLine1 = new NameValuePair((String) results[1]);
			q.setDisplayName(reportLine1.getRight());
			NameValuePair reportLine2 = new NameValuePair((String) results[0]);
			q.setName(reportLine2.getRight());
			hydrateFull(q, results);
			queryObjectQueue.put(queryID, q);
			setQuery(q);
			q.init();
		}

		return q;
	}

	public IQuery createNew(IRuntimeContext runtime) {
		IQuery query = new Query(QueryID.nullID, null);
		setQuery(query);
		return query;
	}

	public boolean hydrate(IQuery query, IRuntimeContext runtime) throws QueryDAOException {
		boolean result = false;
		if (query == null) {
			throw new QueryDAOException("MQueryDAO: Input query is null.");
		}

		if (query.isNullQueryID()) {
			return result;
		}

		if (!queryObjectQueue.containsValue(query)) {
			throw new QueryDAOException("MQueryDAO: Query created outside of the data store.");
		}

		String brokerResult = null;
		ArrayList parms = new ArrayList();
		String parm1 = new String(query.getQueryID().toString());
		parms.add(parm1);
		try {
			brokerResult = queue.add(new CallRequest("ORRHCR RPTDEF", parms));
		} catch (Exception e) {
			throw new QueryDAOException(e);
		}

		String[] resultList = brokerResult.split("\n");
		if (resultList.length > 0) {
			NameValuePair line1 = new NameValuePair(resultList[0]);
			query.setName(line1.getRight());
			hydrateFull(query, resultList);
			result = true;
		}
		return result;
	}

	/**
	 * @see gov.va.med.hds.querytool.IQueryDAO#save(IQuery)
	 */
	public void save(IQuery query) {
	}

	/**
	 * @see gov.va.med.hds.querytool.IQueryDAO#delete(IQuery)
	 */
	public boolean delete(IQuery query) {
		return false;
	}

	private void hydrateFull(IQuery q, String[] inputStrings) throws QueryDAOException {
		if ((constraintDAO == null) || (displayFieldListDAO == null)) {
			try {
				IDaoRegistry daoRegistry =
					(IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
				IDataAccessObject dao = daoRegistry.getDAO(IConstraintDAO.class.getName());
				constraintDAO = (IConstraintDAO) dao;
				dao = daoRegistry.getDAO(IDisplayFieldListDAO.class.getName());
				displayFieldListDAO = (IDisplayFieldListDAO) dao;
			} catch (Exception e) {
				throw new QueryDAOException(e);
			}
		}
		try {
			//			q.setDisplayFields(displayFieldListDAO.get(inputStrings));
			//			q.setQueryParameters(constraintDAO.get(inputStrings));
		} catch (Exception e) {
			throw new QueryDAOException(e);
		}
		q.setChanged(false);
	}

	/**
	 * @see gov.va.med.hds.cd.daoregistry.IDataAccessObject#init(IRuntimeContext)
	 */
	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName, Object data)
		throws Exception {
		this.runtime = runtime;
		try {
			daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
			constraintDAO = (IConstraintDAO) daoRegistry.getDAO(IConstraintDAO.class.getName());
			displayFieldListDAO =
				(IDisplayFieldListDAO) daoRegistry.getDAO(IDisplayFieldListDAO.class.getName());
			//			IMumpsFactory mumpsConnections = (IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID);
			//			queue = mumpsConnections.getCallQueue();
		} catch (Exception e) {
			throw e;
		}
	}

	/**
	 * @see gov.va.med.hds.querytool.dao.IQueryDAO#clearQuery(IQuery)
	 */
	public void clearQuery(IQuery query) {
		query.removeDataNeededListener(dataNeededListener);
	}

	/**
	 * @see gov.va.med.hds.querytool.dao.IQueryDAO#setQuery(IQuery)
	 */
	public void setQuery(IQuery query) {
		query.addDataNeededListener(dataNeededListener);
	}

	/**
	 * Method fireQueryDataRequest.
	 * @param e
	 */
	private synchronized void fireQueryDataRequest(final ActionEvent event) {
		IQuery q = (IQuery) event.getSource();
		final String key;
		if (!QueryID.isValid(q.getQueryID())) {
			key = "22";
		}
		else {
			key = q.getQueryID().toString();
		}
		
		if ((queryLines.containsKey(key))
			&& (parmLines.containsKey(key))
			&& (dispLines.containsKey(key))) {
			q.setChanging(true);
			String[] lines = (String[]) queryLines.remove(key);
			q.setDisplayName(lines[1].substring(12));
			q.setName(lines[0].substring(5));

			lines = (String[]) parmLines.remove(key);
			try {
				constraintDAO.get(q, lines);
			} catch (ConstraintDAOException e) {
				throw new RuntimeException(e);
			}

			lines = (String[]) dispLines.remove(key);
			try {
				displayFieldListDAO.get(q, lines);
			} catch (QueryDAOException e) {
				throw new RuntimeException(e);
			}
			q.setChanging(false);

		} else {
			IQueryID id = q.getQueryID();
			if (!QueryID.isValid(q.getQueryID())) {
				id = new QueryID("22");
			}
			String[] results = MockSharedBroker.getQuery(id);
			if (results == null) {
				callFailed(null, new CallQueueException("Query ID not found."));
			}
			else {
				reportCallSucceeded(QUERY_CALL, key, event, null, results);
			}
		}
	}

	private synchronized void reportCallSucceeded(
		int callType,
		String key,
		ActionEvent event,
		ICallRequest request,
		String[] results) {
		//		String[] results;
		//		if ((result!=null) && (result.length>0)) {
		//			results = result.split("\\n");
		//		}
		//		else {
		//			results = new String[0];
		//		}
		List parmList = new LinkedList();
		List qList = new LinkedList();
		List dispList = new LinkedList();
		for (int i = 0; i < results.length; i++) {
			if (results[i].startsWith("Name=")) {
				qList.add(results[i]);
			} else if (results[i].startsWith("Displ")) {
				qList.add(results[i]);
			} else if (results[i].startsWith("Colum")) {
				dispList.add(results[i]);
			} else {
				parmList.add(results[i]);
			}
		}
		queryLines.put(key, (String[]) qList.toArray(new String[0]));
		parmLines.put(key, parmList.toArray(new String[0]));
		dispLines.put(key, dispList.toArray(new String[0]));
		switch (callType) {
			case QUERY_CALL :
				{
					fireQueryDataRequest(event);
					break;
				}
			case PARM_CALL :
				{
					try {
						fireParmDataRequest(event);
					} catch (ConstraintDAOException ex) {
						throw new RuntimeException(ex);
					}
					break;
				}
			case DISP_CALL :
				{
					fireDispDataRequest(event);
					break;
				}
		}
	}

	private void callFailed(ICallRequest request, CallQueueException e) {
		throw new RuntimeException(e);
	}

	/**
	 * Method fireParmDataRequest.
	 * @param e
	 */
	private synchronized void fireParmDataRequest(final ActionEvent e)
		throws ConstraintDAOException {
		throw new RuntimeException("Parameter data request should not run.");
		//		IQuery q = (IQuery)e.getSource();
		//		final String key;
		//		if (q.getQueryID().equals(QueryID.nullID)) {
		//			key = "22";
		//		}
		//		else {
		//			key = q.getQueryID().toString();
		//		}
		//		if (parmLines.containsKey(key)) {
		//			String[] lines = (String[])parmLines.get(key);
		//			parmLines.remove(key);
		//			//Process data
		//			try {
		//				constraintDAO.get(q.getQueryParameters(),lines);
		//			}
		//			catch (Exception ex) {
		//				throw new ConstraintDAOException(ex);
		//			}
		//		}
		//		else {
		//			try {
		//				List parms = new LinkedList();
		//				parms.add(key);
		//				queue.add(new CallRequest("ORRHCR RPTDEF",parms),new ICallQueueListener() {
		//					public void CallSucceeded(ICallRequest request, String result) {
		//						reportCallSucceeded(PARM_CALL, key, e, request, result);
		//					}
		//					public void CallFailed(ICallRequest request, CallQueueException e) {
		//						callFailed(request,e);
		//					}
		//				});
		//			}
		//			catch (CallQueueException ex) {
		//				callFailed(null,null);
		//			}
		//		}

	}

	/**
	 * Method fireDispDataRequest.
	 * @param e
	 */
	private synchronized void fireDispDataRequest(final ActionEvent e) {
		throw new RuntimeException("Display data request should not run.");
		//		IQuery q = (IQuery)e.getSource();
		//		final String key;
		//		if (q.getQueryID().equals(QueryID.nullID)) {
		//			key = "22";
		//		}
		//		else {
		//			key = q.getQueryID().toString();
		//		}
		//		if (dispLines.containsKey(key)) {
		//			String[] lines = (String[])dispLines.get(key);
		//			dispLines.remove(key);
		//			//Process data
		//			displayFieldListDAO.get(q.getDisplayFields(),lines);
		//		}
		//		else {
		//			try {
		//				List parms = new LinkedList();
		//				parms.add(key);
		//				ICallRequest request = new CallRequest("ORRHCR RPTDEF",parms);
		//				queue.add(request,new ICallQueueListener() {
		//					public void CallSucceeded(ICallRequest request, String result) {
		//						reportCallSucceeded(DISP_CALL, key, e, request, result);
		//					}
		//					public void CallFailed(ICallRequest request, CallQueueException e) {
		//						callFailed(request,e);
		//					}
		//				});
		//			}
		//			catch (CallQueueException ex) {
		//				callFailed(null,null);
		//			}
		//		}

	}

	/**
	 * Method fireDataRequest.
	 * @param e
	 */
	private synchronized void fireDataRequest(ActionEvent e) {
		String object = StringUtil.piece(e.getActionCommand(), '.', 1);
		if (object.equals("query")) {
			fireQueryDataRequest(e);
		} else if (object.equals("parm")) {
			try {
				fireParmDataRequest(e);
			} catch (ConstraintDAOException ex) {
				throw new RuntimeException("Query constraint data request failure:" + ex);
			}
		} else if (object.equals("disp")) {
			fireDispDataRequest(e);
		} else {
			throw new RuntimeException("Data request not registered.");
		}
	}

	/**
	 * @see gov.va.med.hds.querytool.dao.IQueryDAO#getDefnStrings(IQuery)
	 */
	public List getDefnStrings(IQuery query) {
		return null;
	}

	/**
	 * @see gov.va.med.hds.querytool.dao.IQueryDAO#getReportStrings(IQuery)
	 */
	public List getReportStrings(IQuery query) {
		return null;
	}

	public IQuery getNew(String displayText) {
		return null;
	}

	public boolean delete(IQueryID queryID) throws QueryDAOException {
		return false;
	}

	public void saveAs(IQuery currentQuery, String name) throws QueryDAOException {
	}

}
