package gov.va.med.hds.querytool.test;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.querytool.DisplayField;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.Query;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.dao.IQueryDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;
import gov.va.med.hds.querytool.types.Empty;
import gov.va.med.hds.querytool.types.NoEditor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Query object tests.
 */
public class QueryTest extends RuntimeTestCase {
	private IQuery query = null;
	private boolean testProperty = false;
	private Object source;
	private IQueryDAO queryDao;
	private int eventCount;
	
	/**
	 * Constructor for QueryTest.
	 * @param name
	 */
	public QueryTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(QueryTest.class);
	}

	public void testQuery() {
		query = new Query(QueryID.nullID,null);
		assertNotNull("Query constructor failed.",query);
	}

	public void testGetUserContext() {
		query = new Query(QueryID.nullID,null);
		Object userContext = query.getUserContext();
		assertNull("Query.getUserContext failed: null user context expected.",userContext);
	}

	public void testSetGetQueryID() {
		query = new Query(QueryID.nullID,null);
		query.setQueryID(new QueryID("1111"));
		QueryID queryID = new QueryID("1111");
		assertEquals("Query.getQueryID failed:",queryID.toString(),query.getQueryID().toString());
	}

	public void testSetGetDisplayName() {
		query = new Query(QueryID.nullID,null);
		query.setDisplayName("ThIs Is FrEd.!");
		assertEquals("Query.setDisplayName or getDisplayName failed:","ThIs Is FrEd.!",query.getDisplayName());
	}

	public void testSetGetQueryParameters() {
		query = new Query(new QueryID("12345"), null);
		int size = 0;
		for (Iterator i = query.getConstraints().iterator(); i.hasNext();) {
			i.next();
			size++;
		}
		assertEquals("Query.getQueryParameters failed:",1,size);
	}

	public void testSetGetDisplayFields() {
		query = new Query(new QueryID("12345"), null);
		assertNotNull("Query.getDisplayFields failed: null displayFields",query.getDisplayFields());
		assertEquals("Query.getDisplayFields failed:",0,query.getDisplayFields().size());
	}

	public void testSetGetName() {
		query = new Query(new QueryID("40"), null);
		assertNotNull("Query.getName failed: Name is null.",query.getName());
		assertEquals("Query.getName failed: Name is not blank:","",query.getName());
		query.setName("fred:fred");
		assertEquals("Query.setName failed:","fred:fred",query.getName());
	}

	public void testAddParameterChangeListener() {
		query = new Query(QueryID.nullID, null);
		PropertyChangeListener listener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				source = e.getSource();
				testProperty = true;
			}
		};
		query.addPropertyChangeListener(listener);

		testProperty = false;
		source = null;
		query.setDisplayName("fred");
		assertTrue("Query.setDisplayName failed: Listener not fired.",testProperty);
		assertTrue("Query.setDisplayName failed: Listener not fired.",source==query);

		testProperty = false;
		source = null;
		query.setQueryID(new QueryID("1111"));
		assertTrue("Query.setQueryID failed: Listener not fired.",testProperty);
		assertTrue("Query.setQueryID failed: Listener not fired.",source==query);

		testProperty = false;
		source = null;
		NoEditor noEditor = new NoEditor();
		noEditor.setID("0");
		query.getConstraints().addValue(noEditor);
		assertTrue("Query.createConstraintValue failed: Listener not fired.",testProperty);
		assertTrue("Query.createConstraintValue failed: Listener not fired.",source==query);

		testProperty = false;
		List list = new ArrayList();
		list.add(new String("one"));
		noEditor.setValues(list);
		assertTrue("Query.setQueryParameters failed: Listener not fired.",testProperty);

		testProperty = false;
		query.setName("fred");
		assertTrue("Query.setName failed: Listener not fired.",testProperty);
	}

	public void testRemoveParameterChangeListener() {
		query = new Query(QueryID.nullID, null);
		PropertyChangeListener listener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				testProperty = true;
			}
		};
		query.addPropertyChangeListener(listener);
		query.removePropertyChangeListener(listener);

		testProperty = false;
		query.setDisplayName("fred");
		assertFalse("Listener fired.",testProperty);

		testProperty = false;
		query.setQueryID(new QueryID("1111"));
		assertFalse("Listener fired.",testProperty);

		testProperty = false;
		query.getDisplayFields().add(new DisplayField());
		assertFalse("Listener fired.",testProperty);

		testProperty = false;
		query.setName("fred");
		assertFalse("Listener fired.",testProperty);
	}

	public void testIsQueryChanged() {
		query = new Query(new QueryID("12345"), null);
		assertFalse("Query change is incorrect.",query.isQueryChanged());

		query.setChanged(false);
		query.setDisplayName("fred");
		assertTrue("Query change is incorrect.",query.isQueryChanged());

		query.setChanged(false);
		query.setQueryID(new QueryID("54321"));
		assertTrue("Query change is incorrect.",query.isQueryChanged());

		query.setChanged(false);
		IDisplayField field = new DisplayField();
		field.setDisplayText("123");
		field.setID("1.1");
		query.getDisplayFields().add(field);
		assertTrue("Query change is incorrect.",query.isQueryChanged());
		
		query.setChanged(false);
		field.setColumnWidth(1000);
		assertTrue("Query change is incorrect.",query.isQueryChanged());
		
		query.setChanged(false);
		IValue val = new Empty();
		val.setID("0");
		query.getConstraints().addValue(val);
		assertTrue("Query change is incorrect.",query.isQueryChanged());

		query.setChanged(false);
		query.getDisplayFields().removeAt(0);
		assertTrue("Query change is incorrect.",query.isQueryChanged());
		
		query.setChanged(false);
		field.setColumnWidth(9999);
		assertFalse("Query change is incorrect.",query.isQueryChanged());

	}

	public void testIsNewQuery() {
		query = new Query(new QueryID("12345"), null);
		assertFalse("New query test failed.",query.isUnsavedQueryID());

		query = new Query(QueryID.nullID, null);
		assertTrue("New query test failed.",query.isNullQueryID());

	}

	public void testReset() throws QueryDAOException {
		IQuery q = queryDao.getAsync(new QueryID("78"), "waaaaa");
		q.init();
		assertNotNull("Query.reset failed: value not returned.",q.getConstraints().getCriterion("1000500"));
		q.reset();
		assertNull("Query.reset failed: value returned.",q.getConstraints().getCriterion("1000500"));
		q.init();
		assertNotNull("Query.reset failed: value not returned.",q.getConstraints().getCriterion("1000500"));
	}
	
	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("QueryToolTest.xml");
		IDaoRegistry daoRegistry = (IDaoRegistry) getRuntime().getPluginInstance(IDaoRegistry.PLUGIN_ID);
		queryDao = (IQueryDAO) daoRegistry.getDAO(IQueryDAO.class.getName());
	}
	
	public void testClearTag() throws QueryDAOException, InterruptedException {
		IQuery q = queryDao.getAsync(new QueryID("78"), "waaaaa");
		q.init();
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(300);
		}
		eventCount = 0;
		q.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (e.getPropertyName().equals(IQuery.PARM_CLEAR)) {
					eventCount++;
				}
			}
		});
		assertEquals("Query.clearTag failed:",4,q.getDisplayFields().size());
		q.getConstraints().clearTag("0");
		assertEquals("Query.clearTag failed:",1,eventCount);
		assertEquals("Query.clearTag failed:",0,q.getDisplayFields().size());
	}
	
	public void testIsValueSetComplete() throws QueryDAOException {
		IQuery q = queryDao.getAsync(new QueryID("78"), "waaaaa");
		q.init();
		assertFalse("Query.isValueSetComplete failed: should return FALSE.",q.isValueSetComplete());
	}
	
	public void testGetDisplayFieldTypes() throws QueryDAOException, InterruptedException {
		IQuery q = queryDao.getAsync(new QueryID("78"), "waaaaa");
		q.init();
		synchronized (Thread.currentThread()) {
			Thread.currentThread().wait(200);
		}
		assertNotNull("Query.getDisplayFieldTypes failed: null type field list.",q.getDisplayFieldTypes());
		String[] strings = (String[])q.getDisplayFieldTypes().toArray(new String[0]);
		assertEquals("Query.getDisplayFieldTypes failed:",2,strings.length);
		assertEquals("Query.getDisplayFieldTypes failed:","P",strings[0]);
		assertEquals("Query.getDisplayFieldTypes failed:","O",strings[1]);
	}
	
	public void testSetHydrateRequested() throws QueryDAOException {
		IQuery q = queryDao.getAsync(new QueryID("78"), "waaaaa");
		q.setHydrateRequested(true);
		q.init();
		assertEquals("Query.setHydrateRequested failed:","waaaaa",q.getDisplayName());
		
	}
	
	public void testDisplayFieldRemoval() throws QueryDAOException {
		IQuery q = queryDao.getSync(new QueryID("77"));
		q.getConstraints().clearTag("2000500");
		assertEquals("Query.clearTag failed:",5,q.getDisplayFields().size());
	}
	
}
