/*
 * Created on Jul 1, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.test;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IRunEvent;
import gov.va.med.hds.querytool.IRunListener;
import gov.va.med.hds.querytool.IRunQuery;
import gov.va.med.hds.querytool.QueryID;
import gov.va.med.hds.querytool.RunQuery;
import gov.va.med.hds.querytool.dao.IQueryDAO;

import java.util.ArrayList;
import java.util.List;

/**
 * Tests for the RunQuery class
 */
public class RunQueryTest extends MAbstractTestCase {
	private IRunQuery runQuery;
	private IQueryDAO queryDAO;
	private IQuery q;
	private IRunListener listener = new IRunListener() {
		public void runEventRaised(IRunEvent event) {
			eventList.add(event);
		}
	};
	private List eventList;

	public RunQueryTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(RunQueryTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		runQuery = new RunQuery();
		runQuery.setRuntime(getRuntime());
		IDaoRegistry reg = (IDaoRegistry)getRuntime().getPluginInstance(IDaoRegistry.PLUGIN_ID);
		queryDAO = (IQueryDAO)reg.getDAO(IQueryDAO.class.getName());
		q = queryDAO.getSync(new QueryID("79"));
		runQuery.setQuery(q);
		eventList = new ArrayList();
		runQuery.addRunListener(listener);
	}

	public void testRunCancel1() {
		runQuery.runQuery();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_SUCCESS_NON_SENSITIVE,((IRunEvent)eventList.get(0)).getEventID());
		assertEquals("RunQuery.runQuery failed:",5,runQuery.getPatientCount());
		assertEquals("RunQuery.runQuery failed:",330,runQuery.getRowCount());
		eventList.clear();
		runQuery.clear();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_CANCELLED,((IRunEvent)eventList.get(0)).getEventID());
	}

	public void testRunCancel2() {
		runQuery.runQuery();
		runQuery.clear();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_CANCELLED,((IRunEvent)eventList.get(0)).getEventID());
	}
	
	public void testRunCancel3() {
		runQuery.runQuery();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		eventList.clear();
		runQuery.resumeFromPause();
		runQuery.clear();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_CANCELLED,((IRunEvent)eventList.get(0)).getEventID());
	}
	
	public void testRunHappyPath() {
		runQuery.runQuery();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		eventList.clear();
		runQuery.resumeFromPause();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_SUCCESS,((IRunEvent)eventList.get(0)).getEventID());
	}
	
	public void testRunLineLimit1() {
		runQuery.setMaxRows(10);
		runQuery.runQuery();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_EXCEEDS_MAX_LINES,((IRunEvent)eventList.get(0)).getEventID());
	}

	public void testRunLineLimit2() {
		runQuery.setMaxRows(330);
		runQuery.runQuery();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_SUCCESS_NON_SENSITIVE,((IRunEvent)eventList.get(0)).getEventID());
		eventList.clear();
		runQuery.resumeFromPause();
		while (eventList.size()<1) {}
		if (runQuery.getException()!=null) {
			runQuery.getException().printStackTrace();
			fail(runQuery.getException().getMessage());
		}
		assertEquals("RunQuery.runQuery failed:",IRunEvent.REPORT_EXCEEDS_MAX_LINES,((IRunEvent)eventList.get(0)).getEventID());
	}

}
