package gov.va.med.hds.querytool.test;

import gov.va.med.hds.querytool.ResultSaveException;
import gov.va.med.hds.querytool.ui.CantWriteFileException;
import gov.va.med.hds.querytool.ui.SaveResults;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.table.DefaultTableModel;

import junit.framework.TestCase;

/**
 * Tests for the SaveResults class.
 */
public class SaveResultsTest extends TestCase {
	private DefaultTableModel model = new DefaultTableModel();
	private File file;
	private static final char TAB = 9;
	private boolean errorOccurred;

	public SaveResultsTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(SaveResultsTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		model.addColumn("column 1");
		model.addColumn("column 2");
		model.addColumn("column 3");
		String[] row1 = {"Cell 1,1","Cell 1,2","Cell 1,3"};
		String[] row2 = {"Cell 2,1","Cell 2,2","Cell 2,3"};
		String[] row3 = {"Cell 3,1","Cell 3,2","Cell 3,3"};
		String[] row4 = new String[3];
		model.addRow(row1);
		model.addRow(row2);
		model.addRow(row3);
		model.addRow(row4);
	}

	public void testSaveReport() throws IOException, ResultSaveException, CantWriteFileException {
		file = new File("SaveResultsTest.txt");
		SaveResults.saveReport(model, file);
		BufferedReader reader = new BufferedReader(new FileReader(file));
		
		String line = reader.readLine();
		assertEquals("SaveResults.saveReport failed:","Cell 1,1"+TAB+"Cell 1,2"+TAB+"Cell 1,3",line);
		line = reader.readLine();
		assertEquals("SaveResults.saveReport failed:","Cell 2,1"+TAB+"Cell 2,2"+TAB+"Cell 2,3",line);
		line = reader.readLine();
		assertEquals("SaveResults.saveReport failed:","Cell 3,1"+TAB+"Cell 3,2"+TAB+"Cell 3,3",line);
		line = reader.readLine();
		assertEquals("SaveResults.saveReport failed:","\t\t",line);
	}
	
	public void testSaveReport_error() {
		clearFiles();
		file = null;
		errorOccurred = false;
		try {
			SaveResults.saveReport(model, file);
		}
		catch (ResultSaveException e) {
			assertNotNull(e);
			errorOccurred = true;
		}
		catch (CantWriteFileException e) {
			fail(e.getMessage());
		}
		assertTrue("SaveResults.saveReport failed: an expected exception did not occur.",errorOccurred);
		file = new File("fred");
		file.delete();
		try {
			SaveResults.saveReport(model,file);
		}
		catch (ResultSaveException e) {
			fail(e.getMessage());
		}
		catch (CantWriteFileException e) {
			fail(e.getMessage());
		}
		file = new File("fred");
		errorOccurred = false;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(file));
		}
		catch (FileNotFoundException e) {
			errorOccurred = true;
			assertNotNull(e);
		}
		assertTrue("SaveResults.saveReport failed: an expected exception did not occur.",errorOccurred);
		file = new File("fred.txt");
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(file));
		}
		catch (FileNotFoundException e) {
			fail(e.getMessage());
		}
		try {
			reader.readLine();
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
		file = new File("fred.txt");
		file.setReadOnly();
		errorOccurred = false;
		try {
			SaveResults.saveReport(model,file);
		}
		catch (ResultSaveException e) {
			fail(e.getMessage());
		}
		catch (CantWriteFileException e) {
			errorOccurred = true;
			assertNotNull(e);
		}
		assertTrue("SaveResults.saveReport failed: an expected exception did not occur.",errorOccurred);
		clearFiles();
	}
	
	private void clearFiles() {
		file = new File("fred");
		file.delete();
		file = new File("fred.txt");
		file.delete();
	}

}
