/*
 * Created on Jul 2, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.test;

import gov.va.med.hds.querytool.IPrivacyLevel;
import gov.va.med.hds.querytool.ISensitiveList;
import gov.va.med.hds.querytool.ISensitiveListItem;
import gov.va.med.hds.querytool.PrivacyLevel;
import gov.va.med.hds.querytool.SensitiveList;
import gov.va.med.hds.querytool.SensitiveListItem;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Tests the SensitiveList.
 */
public class SensitiveListTest extends MAbstractTestCase {
	private ISensitiveList list;
	private List eventList;
	private ActionListener listener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			eventList.add(e);
		}
	};

	public SensitiveListTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(SensitiveListTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		list = new SensitiveList();
		eventList = new ArrayList();
	}
	
	private void add3Items() {
		ISensitiveListItem item = new SensitiveListItem("1","fred",new PrivacyLevel(IPrivacyLevel.LEVEL_1),true);
		list.add(item);
		item = new SensitiveListItem("2","jane",new PrivacyLevel(IPrivacyLevel.LEVEL_2),false);
		list.add(item);
		item = new SensitiveListItem("3","tarzan",new PrivacyLevel(IPrivacyLevel.EMPLOYEE),true);
		list.add(item);
	}

	public void testAdd_Get_Size() {
		add3Items();
		assertEquals("SensitiveList.add failed:",3,list.getSize());
		assertEquals("SensitiveList.get failed:","1",list.getItem(0).getPatientID().toString());
		assertEquals("SensitiveList.get failed:","jane",list.getItem(1).getName());
		assertEquals("SensitiveList.get failed:","3",list.getItem(2).getPatientID().toString());
	}

	public void testCheckAll() {
		add3Items();
		list.checkAll();
		assertTrue("SensitiveList.checkAll failed: expected true.",list.getItem(0).isChecked());
		assertTrue("SensitiveList.checkAll failed: expected true.",list.getItem(1).isChecked());
		assertTrue("SensitiveList.checkAll failed: expected true.",list.getItem(2).isChecked());
	}

	public void testClearAllChecks() {
		add3Items();
		list.checkAll();
		list.clearAllChecks();
		assertFalse("SensitiveList.checkAll failed: expected false.",list.getItem(0).isChecked());
		assertFalse("SensitiveList.checkAll failed: expected false.",list.getItem(1).isChecked());
		assertFalse("SensitiveList.checkAll failed: expected false.",list.getItem(2).isChecked());
	}

	public void testClearList() {
		add3Items();
		list.clearList();
		assertEquals("SensitiveList.clearList failed:",0,list.getSize());
	}

	public void testChanging_Listener() {
		list.addActionListener(listener);
		list.setChanging(true);
		add3Items();
		assertEquals("SensitiveList.setChanging failed:",0,eventList.size());
		list.checkAll();
		assertEquals("SensitiveList.setChanging failed:",0,eventList.size());
		list.clearAllChecks();
		assertEquals("SensitiveList.setChanging failed:",0,eventList.size());
		list.clearList();
		assertEquals("SensitiveList.setChanging failed:",0,eventList.size());
		list.setChanging(false);
		assertEquals("SensitiveList.setChanging failed:",1,eventList.size());
		eventList.clear();
		add3Items();
		assertEquals("SensitiveList.setChanging failed:",3,eventList.size());
		eventList.clear();
		list.checkAll();
		assertEquals("SensitiveList.setChanging failed:",1,eventList.size());
		eventList.clear();
		list.clearAllChecks();
		assertEquals("SensitiveList.setChanging failed:",1,eventList.size());
		eventList.clear();
		list.clearList();
		assertEquals("SensitiveList.setChanging failed:",1,eventList.size());
	}

	public void testRemoveAllListeners() {
		list.addActionListener(listener);
		add3Items();
		list.removeActionListener(listener);
		eventList.clear();
		list.clearList();
		assertEquals("SensitiveList.removeActionListener failed:",0,eventList.size());
	}
	
}
