/*
 * Created on Aug 5, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.ui.test;

import gov.va.med.hds.querytool.IPrivacyLevel;
import gov.va.med.hds.querytool.ISensitiveList;
import gov.va.med.hds.querytool.ISensitiveListItem;
import gov.va.med.hds.querytool.PrivacyLevel;
import gov.va.med.hds.querytool.SensitiveListItem;
import gov.va.med.hds.querytool.ui.SensitiveAdapter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import junit.framework.TestCase;

/**
 * Tests for the SensitiveAdapter - a ListModel adapter for the SensitiveList in the
 * querytool.
 */
public class SensitiveAdapterTest extends TestCase implements ISensitiveList {
	private SensitiveAdapter adapter;
	private List list;
	private ActionListener listener;
	private List eventList;
	private Thread testThread;

	public SensitiveAdapterTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(SensitiveAdapterTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		adapter = new SensitiveAdapter();
		list = new ArrayList();
		for (int i = 0; i < 10; i++) {
			ISensitiveListItem item = new SensitiveListItem(Integer.toString(i),"n"+Integer.toString(i),new PrivacyLevel(IPrivacyLevel.EMPLOYEE),false);
			list.add(item);
		}
		eventList = new ArrayList();
		testThread = Thread.currentThread();
		adapter.setModel(this);
	}

	private void notifyThread() {
		synchronized (testThread) {
			testThread.notify();
		}
	}

	public void testFireCheckChange() throws InterruptedException {
		ListDataListener dataListener = new ListDataListener() {
			public void intervalAdded(ListDataEvent e) {
				eventList.add(e);
				assertTrue("SensitiveAdapter is not using the Swing event thread.",SwingUtilities.isEventDispatchThread());
				notifyThread();
			}

			public void intervalRemoved(ListDataEvent e) {
				eventList.add(e);
				assertTrue("SensitiveAdapter is not using the Swing event thread.",SwingUtilities.isEventDispatchThread());
				notifyThread();
			}

			public void contentsChanged(ListDataEvent e) {
				eventList.add(e);
				assertTrue("SensitiveAdapter is not using the Swing event thread.",SwingUtilities.isEventDispatchThread());
				notifyThread();
			}
		};
		adapter.addListDataListener(dataListener);
		ISensitiveListItem item = new SensitiveListItem("fred","jones",new PrivacyLevel(IPrivacyLevel.LEVEL_1),false);
		list.add(item);
		fireChange();
		synchronized (testThread) {
			testThread.wait(200);
		}
		assertEquals("SensitiveAdapter fired too many events.",1,eventList.size());
	}

	public void testGetSize() {
		assertEquals("SensitiveAdapter.getSize failed:",10,adapter.getSize());
	}

	public void testGetElementAt() {
		assertEquals("SensitiveAdapter.getSize failed:","9",((ISensitiveListItem)adapter.getElementAt(9)).getPatientID().toString());
	}

	public void testSetModel() {
		assertNotNull("SensitiveAdapter setup failed: no listener registered.",listener);
		adapter.setModel(null);
		assertNull("SensitiveAdapter.setModel failed: listener not removed.",listener);
	}

	public ISensitiveListItem getItem(int index) {
		if (index<0 || index>list.size()-1) return null;
		return (ISensitiveListItem)list.get(index);
	}

	public int getSize() {
		return list.size();
	}

	public void addActionListener(ActionListener listener) {
		this.listener = listener;
	}

	public void removeActionListener(ActionListener listener) {
		if (listener==this.listener) {
			this.listener = null;
		}
	}

	public boolean isChanging() {
		return false;
	}

	public void removeAllListeners() {}
	public void clearList() {}
	public void setChanging(boolean changing) {}
	public void checkAll() {}
	public void clearAllChecks() {}
	public void add(ISensitiveListItem item) {}
	
	private void fireChange() {
		if (listener!=null) {
			ActionEvent event = new ActionEvent(this, 0, null);
			listener.actionPerformed(event);
		}
	}

}
