package gov.va.med.hds.querytool;

import gov.va.med.hds.cd.spectrum.linkpanel.HTML;
import gov.va.med.hds.cd.spectrum.linkpanel.HTMLSelectionModel;
import gov.va.med.hds.cd.spectrum.linkpanel.IHTMLSelectionModel;

import java.awt.Font;
import java.util.Iterator;

import javax.swing.UIManager;

/**
 * Generates HTML text from the given query.
 */
public class HTMLQueryTextFactory {
	private static int INDENT = 5;
	
	public static void updateText(IHTMLSelectionModel model,IQuery q) {
		model.setChanging(true);
		model.clear();
		Font textFont = UIManager.getFont("Label.font");
		model.addHTMLText("<html><header></header><body STYLE=\"font-family:"+
		textFont.getFontName()+";"+
		"font-size:"+textFont.getSize()+";"+
		"background-color:#FFFFFF;" +
		"\">" + HTML.PAR_BEGIN);
		if (q != null) {
			getConstraintFullHTML(model, q);
		}
		else {
			model.addHTMLText("No query selected.");
		}
		model.addHTMLText(HTML.PAR_END + "</body></html>");
		model.setChanging(false);
	}
	
	private static void getConstraintFullHTML(IHTMLSelectionModel model, IQuery q) {
		Object priorTag = null;
		boolean first = true;
		for (Iterator i = q.getConstraints().iterator(); i.hasNext();) {
			IValue item = (IValue)i.next();
			if (!item.getID().equals(priorTag)) {
				if (first) {
					first = false; 
				}
				else {
					model.addHTMLText(HTML.PAR_END + HTML.PAR_BEGIN);
				}
			}
			else {
				model.addHTMLText(HTML.spaces(1));
			}
			getValueFullHTML(model, item);
			priorTag = item.getID();
		}
	}
	
	private static void getValueFullHTML(IHTMLSelectionModel model, IValue v) {
		String spacing = null;
		String result1 = new String();
		String result2 = new String();
		String style = null;
		String tag = v.getID();
		
		if ((tag.length()>2) && (tag.substring(2,4).equals("00"))) {
			spacing = "";
		}
		else {
			spacing = HTML.spaces(INDENT);
		}
		
		if ((v.getDisplayText() != null) && (v.getDisplayText().length()>0)) {
			result1 = result1 + HTML.encode(v.getDisplayText());
		}
		
		if ((v.getDefaultDisplayText() != null) && (v.getDefaultDisplayText().length()>0)) {
			if (result1.length()>0) {
				result1 = result1 + HTML.spaces(1);
			}
			result2 = result2 + v.getDefaultDisplayText();
			if ((v.getValues() != null) && (v.getValues().size()>0)) {
				style = HTMLSelectionModel.SELECTED_COLOR;
			}
			else {
				style = HTMLSelectionModel.UNSELECTED_COLOR;
			}
		}

		if (result1.length()>0) {
			model.addHTMLText(spacing + result1);
			spacing = "";
		}
		
		if (result2.length()>0) {
			if (v.isReadOnly()) {
				model.addHTMLText(spacing + result2);
			}
			else {
				if (spacing.length()>0) {
					model.addHTMLText(spacing);
				}
				model.addSelectableObject(v,result2,style);
			}
		}
	}
	
}
