/*
 * Created on Mar 14, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.ui;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.table.DefaultTableColumnModel;

/**
 * Manages the selected columns of the query display fields; provides a TableColumnModel
 * for a JTable and a ListModel for a JList.
 */
public class ColumnListModel
	extends DefaultTableColumnModel
	implements IColumnListModel {

	/**
	 * Delegate default to parent constructor.
	 */
	public ColumnListModel() {
		super();
	}

	public int getSize() {
		return super.getColumnCount();
	}

	public Object getElementAt(int index) {
		return super.getColumn(index).getIdentifier();
	}

	public void addListDataListener(ListDataListener l) {
		listenerList.add(ListDataListener.class, l);
	}

	public void removeListDataListener(ListDataListener l) {
		listenerList.remove(ListDataListener.class, l);
	}

	protected void fireColumnMoved(TableColumnModelEvent e) {
		super.fireColumnMoved(e);
		if (e.getFromIndex()>-1&&e.getToIndex()>-1&&e.getFromIndex()!=e.getToIndex()) {
			fireListChanged(e.getFromIndex(),e.getToIndex());
		}
	}

	/**
	 * Notifies ListModel listeners of the change.
	 * @param from
	 * @param to
	 */
	private void fireListChanged(int from, int to) {
		if (listenerList.getListenerCount(ListDataListener.class)>0) {
			ListDataListener[] list = (ListDataListener[])listenerList.getListeners(ListDataListener.class);
			ListDataEvent event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, from, to);
			for (int k = 0; k < list.length; k++) {
				list[k].contentsChanged(event);
			}
		}
	}

	/**
	 * Notifies ListModel listeners of the change.
	 * @param from
	 * @param to
	 */
	private void fireListAdded(int from, int to) {
		if (listenerList.getListenerCount(ListDataListener.class)>0) {
			ListDataListener[] list = (ListDataListener[])listenerList.getListeners(ListDataListener.class);
			ListDataEvent event = new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, from, to);
			for (int k = 0; k < list.length; k++) {
				list[k].intervalAdded(event);
			}
		}
	}

	/**
	 * Notifies ListModel listeners of the change.
	 * @param from
	 * @param to
	 */
	private void fireListRemoved(int from, int to) {
		if (listenerList.getListenerCount(ListDataListener.class)>0) {
			ListDataListener[] list = (ListDataListener[])listenerList.getListeners(ListDataListener.class);
			ListDataEvent event = new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, from, to);
			for (int k = 0; k < list.length; k++) {
				list[k].intervalRemoved(event);
			}
		}
	}

	protected void fireColumnAdded(TableColumnModelEvent e) {
		super.fireColumnAdded(e);
		fireListAdded(e.getFromIndex(),e.getToIndex());
	}

	protected void fireColumnRemoved(TableColumnModelEvent e) {
		super.fireColumnRemoved(e);
		fireListRemoved(e.getFromIndex(),e.getToIndex());
	}

}
