package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.linkpanel.IHTMLSelectionModel;
import gov.va.med.hds.cd.spectrum.linkpanel.ISelectionEventListener;
import gov.va.med.hds.cd.spectrum.wizard.IWizard;
import gov.va.med.hds.cd.spectrum.wizard.Wizard;
import gov.va.med.hds.querytool.ICombinedQueryCollection;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.QueryActionEvent;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class EditQuery extends JPanel implements IQueryPage {
	private static final String ROOT_HELPID = "gov.va.med.hds.querytool.EditingaReport";
	public static int LAST_PAGE = Integer.MAX_VALUE;
	private IWizard wizard = new Wizard();
	private ILogger logger;
	private IRuntimeContext runtime;
	private IQueryPageSetModel queryPageSetModel;
	private IQuery currentQuery;
	private IQueryTool queryTool;
	private WaitPageSetModel waitPageSetModel;
	private int initialPage = -1;
	private Action editCancelAction;
	private Action editCompleteSaveAction;
	private Action editCompleteNoSaveAction;
	private Action parmSelectAction;
	private ActionListener wizardListener = new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
			switch (ae.getID()) {
				case IWizard.CANCEL_REQUESTED :
					{
						if (editCancelAction!=null) {
							editCancelAction.actionPerformed(new QueryActionEvent(this, IQueryActionEvent.CANCEL));
						}
						queryPageSetModel.clearModel();
						break;
					}
				case IWizard.FINISH_REQUESTED :
					{
						if (queryPageSetModel.getCurrentPage()==null) {
							if (queryPageSetModel.isSaveSelected()) {
								if (editCompleteSaveAction!=null) {
									IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.EDIT_COMPLETE_SAVE);
									event.setNewValue(queryPageSetModel.getDisplayName());
									editCompleteSaveAction.actionPerformed((ActionEvent)event);
								}
							}
							else {
								IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.EDIT_COMPLETE_NOSAVE);
								event.setNewValue(queryPageSetModel.getDisplayName());
								if (editCompleteNoSaveAction!=null) {
									editCompleteNoSaveAction.actionPerformed((ActionEvent)event);
								}
							}
							queryPageSetModel.clearModel();
							break;
						}
					}
			}
		}
	};
	private ICombinedQueryCollection queryCollection;
	private ISelectionEventListener htmlSelectionListener = new ISelectionEventListener() {
		public void itemSelected(ActionEvent actionEvent) {
			if (actionEvent.getActionCommand().equals(IHTMLSelectionModel.ITEM_SELECTED)) {
				fireEvent(new QueryActionEvent(actionEvent.getSource(),IQueryActionEvent.EDIT_PARAMETER));
				if (parmSelectAction!=null) {
					IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.EDIT_PARAMETER);
					event.setNewValue(actionEvent.getSource());
					parmSelectAction.actionPerformed((ActionEvent)event);
				}
			}
		}
	};

	public EditQuery() {
		super();
		init();
	}

	private void init() {
		this.setLayout(new BorderLayout());
		this.add((Component)wizard, BorderLayout.CENTER);
		wizard.addActionListener(wizardListener);
		wizard.setModel(waitPageSetModel = new WaitPageSetModel());
		addHelpIds();
	}

	public void addQueryActionListener(IQueryAction l) {
		listenerList.add(IQueryAction.class, l);
	}

	public void removeQueryActionListener(IQueryAction l) {
		listenerList.remove(IQueryAction.class, l);
	}

	private void fireEvent(IQueryActionEvent event) {
		if (listenerList.getListenerCount(IQueryAction.class) > 0) {
			IQueryAction[] listeners = (IQueryAction[]) listenerList.getListeners(IQueryAction.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].actionPerformed((ActionEvent)event);
			}
		}
	}
	
	public static final void main(String args[]) {
		JFrame frame = new JFrame("EditQuery");
		EditQuery editQuery = new EditQuery();
		frame.getContentPane().add(editQuery);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
		this.logger = runtime.getLogger(this.getClass());
	}
	
	public boolean isInitialized() {
		return (runtime!=null);
	}
	
	public void initialize() {
		this.currentQuery = queryTool.getCurrentQuery();
		wizard.setModel(waitPageSetModel);
		wizard.reset();
		Thread runQuerySetBuild = new Thread(new Runnable() {
			public void run() {
				querySetBuild();
			}
		}, "Querytool setup edit");
		runQuerySetBuild.start();
	}
	
	public void setQueryCollection(ICombinedQueryCollection queryCollection) {
		this.queryCollection = queryCollection;
	}
	
	private void querySetBuild() {
		if (queryPageSetModel == null) {
			try {
				queryPageSetModel = new QueryPageSetModel();
			}
			catch (QueryDAOException ex) {
				logger.error("Error creating the query edit wizard page set.", ex);
				return;
			}
			queryPageSetModel.setRuntime(runtime);
			queryPageSetModel.getHTMLPanel().addSelectionEventListener(htmlSelectionListener);
		}
		try {
			queryPageSetModel.setModel(queryTool);
		}
		catch (QueryDAOException e) {
			logger.error("Error setting up the query edit wizard.",e);
			return;
		}
		wizard.setModel(queryPageSetModel);
		wizard.reset();
		while (wizard.isProcessingRequest()) {}
		if (initialPage>-1) {
			wizard.goToPage(initialPage);
			while (wizard.isProcessingRequest()) {}
			initialPage = -1;
		}
	}
	
	/**
	 * Sets the page the wizard should start the user on.
	 * @param page
	 */
	public void setWizardPage(int page) {
		initialPage = page;
	}
	
	public IQueryPageSetModel getQueryWizardModel() {
		return queryPageSetModel;
	}

	/**
	 * Sets the query tool model.
	 * @param queryTool
	 */
	public void setQueryModel(IQueryTool queryTool) {
		this.queryTool = queryTool;
		setQueryCollection(queryTool.getQueryCollection());
		this.currentQuery = queryTool.getCurrentQuery();
		editCancelAction = queryTool.getQueryAction(IQueryTool.EDIT_CANCEL_ACTION);
		editCompleteSaveAction = queryTool.getQueryAction(IQueryTool.EDIT_COMPLETE_SAVE_ACTION);
		editCompleteNoSaveAction = queryTool.getQueryAction(IQueryTool.EDIT_COMPLETE_NOSAVE_ACTION);
		parmSelectAction = queryTool.getQueryAction(IQueryTool.EDIT_PARM_ACTION);
	}
	
	private void addHelpIds() {
		this.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, ROOT_HELPID);
	}

	public void unInitialize() {}
	
}
