/*
 * Initially Created on Jun 4, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.ISensitiveListItem;
import gov.va.med.hds.querytool.QueryActionEvent;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Edit sensitive patient data panel
 */
public class EditSensitive extends JPanel implements IEditSensitive {
	private IQueryTool qt;
	private JButton continueButton = new JButton();
	private JButton cancelButton = new JButton();
	private ICheckListPanel checkListPanel = new CheckListPanel();
	private ISensitiveAdapter sensitivePatients;
	private JTextArea messageBox = new JTextArea();
	private JScrollPane messageScrollPane = new JScrollPane(messageBox);
	private Action selectAction;
	private ListSelectionListener selectionListener = new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e) {
			if (selectAction!=null && !e.getValueIsAdjusting()) {
				fireSelectedEvent();
			}
		}
	};
	
	public EditSensitive() {
		super();
		
		setLayout(new BorderLayout());
		
		setupMessageTop();
		setupCheckListCenter();
		setupButtonBottom();
		
	}

	private void setupButtonBottom() {
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(new EmptyBorder(5,5,5,5));
		buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(continueButton);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(cancelButton);
		buttonPanel.add(Box.createHorizontalGlue());
		add(buttonPanel, BorderLayout.SOUTH);
	}

	private void setupCheckListCenter() {
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		
		messageBox.setEditable(false);
		messageBox.setLineWrap(false);
		messageBox.setForeground(getForeground());
		messageBox.setBackground(getBackground());
		
		checkListPanel.setBorder(new EmptyBorder(3,3,3,3));
		checkListPanel.addListSelectionListener(selectionListener);

		splitPane.add((JComponent)checkListPanel, JSplitPane.LEFT);
		splitPane.add(messageScrollPane, JSplitPane.RIGHT);
		
		add(splitPane, BorderLayout.CENTER);
	}

	private void setupMessageTop() {
		JPanel messagePanel = new JPanel(new BorderLayout());
		JLabel headerLabel = new JLabel("Sensitive Patient Preview");
		headerLabel.setFont(headerLabel.getFont().deriveFont(Font.BOLD));
		messagePanel.add(headerLabel, BorderLayout.NORTH);
		JLabel headerLabel2 = new JLabel("Select the sensitive patients to include in the report.");
		messagePanel.add(headerLabel2, BorderLayout.CENTER);
		add(messagePanel, BorderLayout.NORTH);
	}
	
	public void setQueryModel(IQueryTool model) {
		qt = model;
		continueButton.setAction(qt.getQueryAction(IQueryTool.RUN_CONTINUE_ACTION));
		cancelButton.setAction(qt.getQueryAction(IQueryTool.HOME_ACTION));
		checkListPanel.setSelectAllAction(qt.getQueryAction(IQueryTool.SELECT_ALL_ACTION));
		checkListPanel.setSelectNoneAction(qt.getQueryAction(IQueryTool.SELECT_NONE_ACTION));
		selectAction = qt.getQueryAction(IQueryTool.SELECT_SENSITIVE_ACTION);
		sensitivePatients = new SensitiveAdapter();
		checkListPanel.setListModel(sensitivePatients);
	}

	public static final void main(String args[]) {
		try {
			UIManager.setLookAndFeel(new DesktopLookAndFeel(EditSensitive.class.getClassLoader()));
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		JFrame frame = new JFrame("EditSensitive");
		EditSensitive editSensitive = new EditSensitive();
		frame.getContentPane().add(editSensitive);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	/**
	 * Handles list selection events.
	 */
	protected void fireSelectedEvent() {
		ISensitiveListItem item = null;
		int[] indices = null;
		if ((indices=checkListPanel.getSelectedIndices()).length==1) {
			item = (ISensitiveListItem)qt.getRunModel().getSensitiveDataModel().getSensitivePtList().getItem(indices[0]);
		}
		
		IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.SELECT_SENSITIVE);
		event.setNewValue(item);
		selectAction.actionPerformed((ActionEvent)event);
	}

	public void initialize() {
		if (qt!=null && qt.getRunModel()!=null && qt.getRunModel().getSensitiveDataModel()!=null) {
			messageBox.setDocument(qt.getRunModel().getSensitiveDataModel().getSensitiveMessageModel());
			sensitivePatients.setModel(qt.getRunModel().getSensitiveDataModel().getSensitivePtList());
		}
	}

	public void unInitialize() {}

}
