/*
 * Initially Created on Jun 9, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.ui.editor.factory.IEditorFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * Factory for the editor factory; builds table of factories from the manifest.
 */
public class EditorFactoryFactory implements IEditorFactoryFactory {
	private IRuntimeContext runtime;
	private Map editorMap = new HashMap();

	public void setRuntime(IRuntimeContext runtime) throws FactoryException {
		this.runtime = runtime;
		IConfigurationElement[] elements = runtime.getPluginRegistry().getConfigurationElementsFor(IQuerytoolUIConstants.EDITOR_FACTORY_EXTENSION_ID);
		for (int i = 0; i < elements.length; i++) {
			String key = elements[i].getAttribute(IQuerytoolUIConstants.TYPE_ATTRIBUTE);
			String factoryName = elements[i].getAttribute(IQuerytoolUIConstants.EDITOR_FACTORY_ATTRIBUTE);
			if (key.length()>0 && factoryName.length()>0) {
				try {
					Class type = Class.forName(factoryName, true, runtime.getPluginRegistry().getPluginDescriptor(IQuerytoolUIConstants.PLUGIN_ID).getPluginClassLoader());
					editorMap.put(key, type);
				}
				catch (ClassNotFoundException e) {
					throw new FactoryException("Error creating editor factory map for query parameters.");
				}
			}
		}
	}
	
	public IEditorFactory getFactory(Object key) throws FactoryException {
		Class type = (Class)editorMap.get(key);
		if (type!=null) {
			try {
				IEditorFactory factory = (IEditorFactory)type.newInstance();
				factory.setRuntime(runtime);
				return factory;
			}
			catch (Exception e) {
				throw new FactoryException("Error creating editor factory map for query parameters.");
			}
		}
		else {
			return null;
		}
	}
	
}
