package gov.va.med.hds.querytool.ui;

import java.awt.Component;
import java.awt.LayoutManager;

import javax.swing.AbstractButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

/**
 * @author vhaislmonsos
 *
 */
public class EnablePanel extends JPanel implements IEnablePanel {
	private String title;

	/**
	 * Constructor for EnablePanel.
	 * @param layout
	 * @param isDoubleBuffered
	 */
	public EnablePanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}

	/**
	 * Constructor for EnablePanel.
	 * @param layout
	 */
	public EnablePanel(LayoutManager layout) {
		super(layout);
	}

	/**
	 * Constructor for EnablePanel.
	 * @param isDoubleBuffered
	 */
	public EnablePanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}

	/**
	 * Constructor for EnablePanel.
	 */
	public EnablePanel() {
		super();
	}

	/**
	 * @see java.awt.Component#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		if (enabled==isEnabled()) return;
		boolean firstRadioButtonFound = false;
		if (!isEnabled()) {
			super.setEnabled(enabled);
		}
		for (int i = 0; i < getComponentCount(); i++) {
			Component c = getComponent(i);
			if (enabled) {
				c.setEnabled(enabled);
				if ((!firstRadioButtonFound) && (c instanceof JRadioButton)) {
					((AbstractButton)c).doClick();
					firstRadioButtonFound = true;
				}
			}
			else {
				if ((c instanceof AbstractButton) && (((AbstractButton)c).isSelected())) {
					((AbstractButton)c).doClick();
				}
				c.setEnabled(enabled);
			}
		}
		if (!enabled) {
			super.setEnabled(enabled);
		}
	}

	/**
	 * Method setBorderText.
	 * @param text
	 */
	public void setBorderText(String text) {
		this.title = text;
		setBorderVisible(true);
	}
	
	/**
	 * Method setBorderVisible.
	 * @param visible
	 */
	public void setBorderVisible(boolean visible) {
		if (!visible) {
			setBorder(null);
		}
		else {
			setBorder(new TitledBorder(title));
		}
	}
	
	public boolean containsSelectedItems() {
		for (int i = 0; i < getComponentCount(); i++) {
			Component c = getComponent(i);
			if (c instanceof ITagComponent) {
				if (((ITagComponent)c).isSelected()) {
					return true;
				}
			}
			else if (c instanceof IEnablePanel) {
				if (((IEnablePanel)c).containsSelectedItems()) {
					return true;
				}
			}
		}
		return false;
	}
	
}
