/*
 * Created on Apr 15, 2003 by vhaislmonsos
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.QueryActionEvent;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Querytool intro panel.  Provides a configurable message that the user must click
 * okay before allowed to use the querytool.
 */
public class IntroPanel extends JPanel implements IIntroPanel {
	private JEditorPane displayPane = new JEditorPane();
	private JButton okButton = new JButton("Okay");

	public IntroPanel() {
		super();
		init();
	}
	
	public void addQueryActionListener(IQueryAction listener) {
		listenerList.add(IQueryAction.class, listener);
	}

	public void removeQueryActionListener(IQueryAction listener) {
		listenerList.remove(IQueryAction.class, listener);
	}
	
	private void init() {
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		okButton.setMnemonic('O');
		JScrollPane scrollPane = new JScrollPane(displayPane);
		displayPane.setEditable(false);
		displayPane.setMaximumSize(new Dimension(10000, 10000));
		add(scrollPane);
		add(Box.createVerticalStrut(10));
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(okButton);
		buttonPanel.add(Box.createHorizontalGlue());
		add(buttonPanel);
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireAcknowledged();
			}
		});
	}
	
	protected void fireAcknowledged() {
		if (listenerList.getListenerCount(IQueryAction.class)>0) {
			IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.HOME);
			IQueryAction[] listeners = (IQueryAction[]) listenerList.getListeners(IQueryAction.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].actionPerformed((ActionEvent)event);
			}
		}
	}
	
	public void setText(String text) {
		if (text.startsWith("<html>")) {
			displayPane.setContentType("text/html");
		}
		else {
			displayPane.setContentType("text/text");
		}
		displayPane.setText(text);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("IntroPanel");
		IntroPanel introPanel = new IntroPanel();
		frame.getContentPane().add(introPanel);
		introPanel.setText("<html><body><p><b><i>High their</i></b></p></body></html>");
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}

	public void setOkEnabled(boolean okButtonEnabled) {
		okButton.setEnabled(okButtonEnabled);
	}
	
}
