/*
 * Initially Created on Jul 14, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.querytool.IQueryTool;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;

/**
 * Manages the report progress dialog that is shown to the users during a report execution.
 */
public class ProgressDialogManager implements IProgressDialogManager {
	private static final boolean IS_MODAL = true;
	private static final String WINDOW_TITLE = "Query Execution";
	private JDialog progressDialog;
	private Thread showProgressThread;
	private JFrame parentFrame = new JFrame();
	private IReportProgress reportProgress = new ReportProgress();
	
	/**
	 * Constructor with the parent frame.
	 * @param frame
	 */
	public ProgressDialogManager(JFrame parentFrame) {
		super();
		this.parentFrame = parentFrame;
		init();
	}

	/**
	 * Standard constructor.
	 */
	public ProgressDialogManager() {
		super();
		init();
	}
	
	private void init() {
		progressDialog = new JDialog(parentFrame, WINDOW_TITLE, IS_MODAL);
		progressDialog.getContentPane().add((JComponent)reportProgress, BorderLayout.CENTER);
		progressDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	}

	private void showProgressDialog() {
		showProgressThread = new Thread(new Runnable() {
			public void run() {
				progressDialog.pack();
				int x = parentFrame.getX() + ((parentFrame.getWidth() - progressDialog.getWidth()) / 2);
				int y = parentFrame.getY() + ((parentFrame.getHeight() - progressDialog.getHeight()) / 2);

				progressDialog.setLocation(x, y);
				progressDialog.show();
			}
		}, "showQueryDialog");
		showProgressThread.start();
		while (!progressDialog.isShowing()) {}
	}
	
	private void hideProgressDialog() {
		progressDialog.hide();
	}
	
	public void setMessageText(String message) {
		reportProgress.setMessage(message);
	}

	public void setQueryModel(IQueryTool queryTool) {
		reportProgress.setQueryModel(queryTool);
	}

	public static final void main(String args[]) {
		final ProgressDialogManager manager = new ProgressDialogManager();
		manager.showProgressDialog();
		System.out.println("Last main line.");
	}
	
	public void startWaiting() {
		reportProgress.startWaiting();
	}
	
	public void stopWaiting() {
		reportProgress.stopWaiting();
	}

	public void initialize() {
		reportProgress.initialize();
		showProgressDialog();
		reportProgress.startWaiting();
	}

	public void unInitialize() {
		reportProgress.unInitialize();
		hideProgressDialog();
		reportProgress.stopWaiting();
	}
}
