package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.spectrum.bucket.BasicSlushBucket;
import gov.va.med.hds.cd.spectrum.bucket.ISlushBucket;
import gov.va.med.hds.cd.spectrum.list.AssertEvent;
import gov.va.med.hds.cd.spectrum.list.AssertList;
import gov.va.med.hds.cd.spectrum.list.IAssertListener;
import gov.va.med.hds.cd.spectrum.wizard.IWizardPage;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

/**
 * Wizard page to show the user the available query fields, selected query fields,
 * and sample table.
 */
public class QueryColumnPage extends AbstractQueryWizardPage implements IWizardPage {
	private ISlushBucket listBucket = new BasicSlushBucket();
	private AssertList inBucketList = new AssertList();
	private AssertList outBucketList = new AssertList();
	private JScrollPane inScrollPane = new JScrollPane(inBucketList);
	private JScrollPane outScrollPane = new JScrollPane(outBucketList);
	private SideButtonPanel sideButtonPanel = new SideButtonPanel();
	private TablePanel tablePanel = new TablePanel();
	private EventListenerList eventListenerList = new EventListenerList();
	private PageListener pageListener = new PageListener();
	private DefaultListModel blankModel = new DefaultListModel();
	private TableColumnModelListener columnModelListener = new TableColumnModelListener() {
		public void columnSelectionChanged(ListSelectionEvent e) {}
		public void columnAdded(TableColumnModelEvent e) {}
		public void columnRemoved(TableColumnModelEvent e) {}

		public void columnMoved(TableColumnModelEvent e) {
			if (e.getFromIndex()!=e.getToIndex()) {
				fireEvent(IQueryPageEvent.MOVE_COLUMN, e.getFromIndex(), e.getToIndex());
			}
		}
		
		public void columnMarginChanged(ChangeEvent e) {
			lastMarginEvent = e;
		}
	};
	private String errorMessage = "Query column page error.";
	private TableHeaderMouseListener tableHeaderMouseListener = new TableHeaderMouseListener();
	private ChangeEvent lastMarginEvent;
	private ListModel outListModel;

	private class TableHeaderMouseListener extends MouseInputAdapter {
		public void mouseClicked(MouseEvent e) {
			lastMarginEvent = null;
		}
		
		public void mouseReleased(MouseEvent e) {
			if (lastMarginEvent!=null) {
				fireEvent(IQueryPageEvent.COLUMN_RESIZE,-1,-1);
			}
		}
	}

	public QueryColumnPage() {
		super();
		init();
	}
	
	public QueryColumnPage(String title) {
		this();
		super.setTitle(title);
	}
	
	private class PageListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource()==sideButtonPanel.upButton) {
				int index = -1;
				if (outBucketList.getSelectedIndices().length==1) {
					index = outBucketList.getSelectedIndex();
				}
				if (index>0) {
					fireEvent(IQueryPageEvent.MOVE_UP,index,index-1);
				}
			}
			else if (e.getSource()==sideButtonPanel.downButton) {
				int index = outBucketList.getModel().getSize();
				if (outBucketList.getSelectedIndices().length==1) {
					index = outBucketList.getSelectedIndex();
				}
				if (index < outBucketList.getModel().getSize()-1) {
					fireEvent(IQueryPageEvent.MOVE_DOWN,index,index+1);
				}
			}
			else {
				if (e.getID()==ISlushBucket.SELECT_ACTION) {
					if (inBucketList.getSelectedIndices().length>0) {
						fireEvent(IQueryPageEvent.SELECT,-1,-1,inBucketList.getSelectedIndices());
					}
				}
				else if (e.getID()==ISlushBucket.REMOVE_ACTION) {
					if (outBucketList.getSelectedIndices().length>0) {
						fireEvent(IQueryPageEvent.REMOVE,-1,-1, outBucketList.getSelectedIndices());
					}
				}
				else if (e.getID()==ISlushBucket.SELECT_ALL_ACTION) {
					fireEvent(IQueryPageEvent.SELECT_ALL,-1,-1);
				}
				else if (e.getID()==ISlushBucket.REMOVE_ALL_ACTION) {
					fireEvent(IQueryPageEvent.CLEAR_ALL,-1,-1);
				}
			}
		}
	}
	
	private void init() {
		outBucketList.setClickCount(AssertList.DOUBLE_CLICK);
		inBucketList.setClickCount(AssertList.DOUBLE_CLICK);
		inBucketList.addAssertListener(new IAssertListener() {
			public void valueChanged(AssertEvent e) {
				int[] indices = inBucketList.getSelectedIndices();
				if (indices.length>0) {
					fireEvent(IQueryPageEvent.SELECT,-1,-1, indices);
				}
			}
		});
		outBucketList.addAssertListener(new IAssertListener() {
			public void valueChanged(AssertEvent e) {
				if (outBucketList.getSelectedIndices().length>0) {
					fireEvent(IQueryPageEvent.REMOVE, -1, -1, outBucketList.getSelectedIndices());
				}
			}
		});
		tablePanel.getTableHeader().addMouseListener(tableHeaderMouseListener);
		inBucketList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		listBucket.setInBucketComponent(inScrollPane);
		listBucket.setOutBucketComponent(outScrollPane);
		listBucket.addActionListener(pageListener);
		sideButtonPanel.upButton.addActionListener(pageListener);
		sideButtonPanel.downButton.addActionListener(pageListener);
		this.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.BOTH;
		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(0,0,0,0);
		c.ipadx = 0;
		c.ipady = 0;
		c.weightx = .95;
		c.weighty = .8;
		this.add((Component)listBucket,c);
		c.gridx = 1;
		c.weightx = .05;
		this.add(sideButtonPanel,c);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 2;
		c.weightx = 1.0;
		c.weighty = .2;
		this.add(tablePanel,c);
		listBucket.setInBucketLabel("Available Fields");
		listBucket.setOutBucketLabel("Selected Fields");
	}
	
	public static final void main(String args[]) {
		try {
			JFrame frame = new JFrame("QueryColumnPage");
			QueryColumnPage queryColumnPage = new QueryColumnPage();
			frame.getContentPane().add(queryColumnPage, BorderLayout.CENTER);
			frame.pack();
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.show();

		} catch (Exception ioe) {
			ioe.printStackTrace();
		}
	}
	
	public String getErrorMessage() {
		return errorMessage;
	}
	
	/**
	 * Allows the error to be set by a class managing this page.
	 * @param message
	 */
	public void setErrorMessage(String message) {
		this.errorMessage = message;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IWizardPage#isFinishAllowed()
	 */
	public boolean isFinishAllowed() {
		return true;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IWizardPage#isOkToGoNext()
	 */
	public boolean isOkToGoNext() {
		if (outListModel.getSize()>0) {
			errorMessage = null;
			return true;
		}
		else {
			errorMessage = "There are no report fields selected.\nPlease select at least one report field before continuing.";
			return false;
		}
	}

	/**
	 * Sets the slush bucket's input list model.
	 * @param model - ListModel for the input slush bucket.
	 */
	public void setInputListModel(ListModel model) {
		inBucketList.setModel(model);
	}
	
	public void clearModels() {
		inBucketList.setModel(blankModel);
		outBucketList.setModel(blankModel);
		tablePanel.setTableModel(new DefaultTableModel());
		tablePanel.setTableColumnModel(new DefaultTableColumnModel());
	}

	/**
	 * Assigns the slush bucket output list model.
	 * @param model - ListModel for the output columns
	 */
	public void setOutputListModel(ListModel model) {
		outBucketList.setModel(model);
		this.outListModel = model;
	}
	
	/**
	 * Method setTableModel.
	 * @param model - TableModel for the sample data display table.
	 */
	public void setTableModel(TableModel model) {
		tablePanel.setTableModel(model);
	}
	
	public void setTableColumnModel(TableColumnModel model) {
		tablePanel.setTableColumnModel(model);
		model.addColumnModelListener(columnModelListener);
	}

	public void addPageListener(IQueryPageListener l) {
		eventListenerList.add(IQueryPageListener.class, l);
	}
	
	public void removePageListener(IQueryPageListener l) {
		eventListenerList.remove(IQueryPageListener.class, l);
	}

	public void fireEvent(int eventID, int fromIndex, int toIndex) {
		fireEvent(eventID, fromIndex, toIndex, null);
	}
	
	public void fireEvent(int eventID, int fromIndex, int toIndex, int[] indices) {
		if (eventListenerList.getListenerCount(IQueryPageListener.class)>0) {
			IQueryPageEvent event = new QueryPageEvent(this, eventID, fromIndex, toIndex);
			event.setIndexArray(indices);
			IQueryPageListener[] listeners = (IQueryPageListener[])eventListenerList.getListeners(IQueryPageListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].pageEvent(event);
			}
		}
	}
	
	public ListSelectionModel getInSelectionModel() {
		return inBucketList.getSelectionModel();
	}

	public ListSelectionModel getOutSelectionModel() {
		return outBucketList.getSelectionModel();	
	}

	/**
	 * Sets the selected item in the output selection list.
	 * @param i
	 */
	public void setOutSelected(int i) {
		outBucketList.setSelectedIndex(i);
		outBucketList.ensureIndexIsVisible(i);
	}

}
