package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.spectrum.text.GenericDocumentFilter;
import gov.va.med.hds.cd.spectrum.wizard.IWizardPage;
import gov.va.med.hds.querytool.ICombinedQueryCollection;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.text.PlainDocument;

/**
 * Query edit final wizard page.
 */
public class QueryFinalPage extends AbstractQueryWizardPage implements IWizardPage {
	private JTextField textBox = new JTextField(80);
	private JCheckBox saveCheckBox = new JCheckBox("Save the report for later reuse",false);
	private String errorMessage;
	private ICombinedQueryCollection queryCollection;
	private boolean isNewQuery;
	private String initialDisplayName;
	private static String nameErrorString = "The report name specified is already in use.  Please enter " +
	"a different name to continue.";
	
	public QueryFinalPage() {
		super();
		init();
	}
	
	public QueryFinalPage(String title) {
		this();
		super.setTitle(title);
	}

	private void init() {
		this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		this.add(new JLabel("To use the report without saving it, click 'Finish'."));
		this.add(Box.createVerticalStrut(15));
		this.add(new JLabel("When saved, the report will appear with other reports in the report selection list."));
		this.add(new JLabel("A name is required to save the report for later reuse."));
		this.add(Box.createVerticalStrut(15));
		this.add(new JLabel("Report Name"));
		textBox.setMaximumSize(new Dimension(600,22));
		this.add(textBox);
		this.add(Box.createVerticalStrut(15));
		this.add(saveCheckBox);
		PlainDocument document = new PlainDocument();
		document.setDocumentFilter(new GenericDocumentFilter(80));
		textBox.setDocument(document);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("QueryFinalPage");
		QueryFinalPage queryFinalPage = new QueryFinalPage();
		frame.getContentPane().add(queryFinalPage, BorderLayout.CENTER);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
	public String getErrorMessage() {
		return errorMessage;
	}

	public boolean isFinishAllowed() {
		return true;
	}

	public boolean isOkToGoNext() {
		if (textBox.getText().length()<1) {
			errorMessage = "The report name is blank.  Enter a non-blank name to " +
			"continue.  If you don't want to keep this report definition, select 'Cancel'.";
			return false;
		}
		else if (isNewQuery &&
					queryCollection!=null && 
					queryCollection.containsDisplayName(textBox.getText())) {
			errorMessage = nameErrorString;
			return false;
		}
		else if (!isNewQuery && 
					!initialDisplayName.equalsIgnoreCase(textBox.getText()) &&
					queryCollection!=null &&
					queryCollection.containsDisplayName(textBox.getText())) {
			errorMessage = nameErrorString;
			return false;
		}
		else {
			errorMessage = null;
		}
		return true;
	}
	
	public void setQueryCollection(ICombinedQueryCollection queryCollection) {
		this.queryCollection = queryCollection;
	}
	
	public void setSaveSelected(boolean isSaveSelected) {
		saveCheckBox.setSelected(isSaveSelected);
	}
	
	public void setDisplayName(String displayName) {
		textBox.setText(displayName);
		initialDisplayName = displayName;
	}
	
	public boolean isSaveSelected() {
		return saveCheckBox.isSelected();
	}
	
	public String getDisplayName() {
		return textBox.getText();
	}
	
	public void setIsNewQuery(boolean isNewQuery) {
		this.isNewQuery = isNewQuery;
	}
	
}
