package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.spectrum.linkpanel.ISelectionEventListener;
import gov.va.med.hds.cd.spectrum.wizard.IWizardPage;
import gov.va.med.hds.querytool.IQuery;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

/**
 * Query constraint definition page, used to let the user select or de-select
 * constraints used by the query.
 */
public class QueryMultiPage extends JPanel implements IWizardPage {
	private JPanel specPanel = new JPanel();
	private JScrollPane scrollPane = new JScrollPane();
	private ScrollableJPanel multiPanel = new ScrollableJPanel();
	private JSplitPane contentSplit = new JSplitPane();
	private CardLayout cardLayout = new CardLayout();
	private List pageList = new ArrayList();
	private int currentPage = -1;
	private IQuery query;
	private String errorMessage = "No errors.";
	private IQueryHTMLPanel queryHTMLPanel = new QueryHTMLPanel();

	public QueryMultiPage() {
		super();
		init();
	}
	
	private void init() {
		specPanel.setLayout(new BorderLayout());
		specPanel.add(new JLabel("Click the underlined text to further specify the values to be used by the report."),BorderLayout.NORTH);
		specPanel.add((JComponent)queryHTMLPanel, BorderLayout.CENTER);
		((JComponent)queryHTMLPanel).putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, "gov.va.med.hds.querytool.asfcr");
		this.setLayout(new BorderLayout());
		multiPanel.setLayout(cardLayout);
		contentSplit.setBorder(null);
		contentSplit.setOneTouchExpandable(true);
		contentSplit.setDividerSize(10);
		contentSplit.setOrientation(JSplitPane.VERTICAL_SPLIT);
		
		contentSplit.add(specPanel, JSplitPane.RIGHT);
		contentSplit.add(scrollPane, JSplitPane.LEFT);
		this.add(contentSplit,BorderLayout.CENTER);
		String id = Integer.toString(-1);
		JPanel blankPanel = new JPanel();
		multiPanel.add(blankPanel,id);
		cardLayout.addLayoutComponent(blankPanel,id);
	}
	
	public String getErrorMessage() {
		return errorMessage;
	}

	public boolean isFinishAllowed() {
		return false;
	}

	public boolean isOkToGoNext() {
		if (currentPage>-1) {
			return isQuerySyncedWithButtons() && arePageRequirementsMet();
		}
		else {
			return true;
		}
	}
	
	private boolean arePageRequirementsMet() {
		StringBuffer requirementsString = new StringBuffer();
		List requirementsList = ((IQueryPanelItems)pageList.get(currentPage)).getRequirements();
		List components = ((IQueryPanelItems)pageList.get(currentPage)).getTagComponents();
		int missingRequirementsCount = 0;
		for (Iterator i = requirementsList.iterator(); i.hasNext();) {
			if (requirementsString.length()>0) {
				requirementsString.append("       AND\n");
			}
			int matches = 0;
			List requirements = (List)i.next();
			StringBuffer sublistString = new StringBuffer();
			for (Iterator j = requirements.iterator(); j.hasNext();) {
				if (sublistString.length()>0) {
					sublistString.append("  - or -\n");
				}
				String requirement = (String)j.next();
				ITagComponent component = getComponent(components, requirement);
				if (component!=null) {
					sublistString.append(component.getText() + "\n");
					if (component.isSelected()) {
						matches++;
					}
				}
			}
			requirementsString.append(sublistString);
			if (matches<1) {
				missingRequirementsCount++;
			}
		}
		if (missingRequirementsCount>0) {
			errorMessage = "One of the required selections needs to be made to continue:\n" + requirementsString.toString();
			return false;
		}
		else {
			errorMessage = "";
			return true;
		}
	}

	private ITagComponent getComponent(List components, String requirement) {
		for (Iterator i = components.iterator(); i.hasNext();) {
			ITagComponent component = (ITagComponent)i.next();
			if (component.getTag().equalsIgnoreCase(requirement)) {
				return component;
			}
		}
		return null;
	}

	private boolean isQuerySyncedWithButtons() {
		boolean okay = true;
		List components = ((IQueryPanelItems)pageList.get(currentPage)).getTagComponents();
		if ((components!=null) && (components.size()>0)) {
			errorMessage = "";
			for (int i = 0; i < components.size(); i++) {
				ITagComponent item = (ITagComponent)components.get(i);
				if (item.isSelected() && (item.getTag().length()>0) && (query.getConstraints().getCriterion(item.getTag())==null)) {
					errorMessage = errorMessage + item.getText() + "\n";
				}
			}
			if (errorMessage.length()>0) {
				okay = false;
				errorMessage = "The following parameters have not been updated to the database:\n" + errorMessage;
			}
		}
		return okay;
	}
	
	public void addPanelSet(IQueryPanelItems e) {
		pageList.add(e);
		String id = Integer.toString(pageList.size()-1);
		multiPanel.add(e.getPanel(),id);
		scrollPane.setViewportView(multiPanel);
		contentSplit.setDividerLocation(350);
		cardLayout.addLayoutComponent(e.getPanel(),id);
	}

	/**
	 * Method setPanelSet.
	 * @param pageNumber
	 */
	public void setPanelSet(int pageNumber) {
		if (pageNumber<0 || (pageNumber>pageList.size()-1)) {
			currentPage = -1;
		}
		else {
			currentPage = pageNumber;
		}
		cardLayout.show(multiPanel,Integer.toString(currentPage));
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.wizard.IWizardPage#getTitle()
	 */
	public String getTitle() {
		if (currentPage>-1) {
			return ((IQueryPanelItems)pageList.get(currentPage)).getTitle();
		}
		else {
			return "";
		}
	}

	public void setModel(IQuery query) {
		this.query = query;
		queryHTMLPanel.setQueryModel(query);
	}
	
	public IQueryHTMLPanel getHTMLPanel() {
		return queryHTMLPanel;
	}

	public boolean isDependencyOkay() {
		List deps = ((IQueryPanelItems)pageList.get(currentPage)).getDependencies();
		if ((deps!=null) && (deps.size()>0)) {
			boolean result = true;
			for (int i = 0; i < deps.size(); i++) {
				Object item = deps.get(i);
				if (query.getConstraints().getCriterion(item)==null) {
					result = false;
					break;
				}
			}
			return result;
		}
		else {
			return true;
		}
	}
	
	public void addParameterSelectionListener(ISelectionEventListener listener) {
		queryHTMLPanel.addSelectionEventListener(listener);
	}

	public void removeParameterSelectionListener(ISelectionEventListener listener) {
		queryHTMLPanel.removeSelectionEventListener(listener);
	}

}
