package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.wizard.AbstractPageSetModel;
import gov.va.med.hds.cd.spectrum.wizard.IWizardPage;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Implements the wizard page set model for the query wizard panels.
 */
public class QueryPageSetModel extends AbstractPageSetModel implements ActionListener, IQueryPageSetModel {
	private final static String columnPageTitle = "Create Custom Report - Field Selection";
	private final static String finalPageTitle = "Create Custom Report - Report Name";
	private List queryPages = new ArrayList();
	private List customQueryPages;
	private int currentPage = -1;
	private QueryMultiPage queryMultiPage = new QueryMultiPage();
	private QueryColumnPage queryColumnPage = new QueryColumnPage(columnPageTitle);
	private int customPageCount;
	private IQuery query;
	private boolean settingUpButtons = false;
	private boolean buttonsConnected = false;
	private List setTags = new LinkedList();
	private QueryPanelFactory panelFactory = new QueryPanelFactory();;
	private IRuntimeContext runtime;
	private QueryColumnPageManager queryColumnPageManager = new QueryColumnPageManager();
	private QueryFinalPage queryFinalPage = new QueryFinalPage();
	private PropertyChangeListener tagChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName().equals(IQuery.PARM_CLEARALL)) {
				initQueryButtonSelection(QueryPageSetModel.this.query);
			}
			else if (e.getPropertyName().equals(IQuery.PARM_ADD)) {
				IValue value = (IValue)e.getNewValue();
				parmAdd(value.getID());
			}
			else if (e.getPropertyName().equals(IQuery.PARM_CLEAR)) {
				parmClear(e.getOldValue());
			}
		}
	};

	/**
	 * Constructor for QueryPageSetModel.
	 */
	public QueryPageSetModel() throws QueryDAOException {
		super();
		init();
	}
	
	private void init() throws QueryDAOException {
		customQueryPages = panelFactory.getPanels();
		customPageCount = customQueryPages.size();
		for (int i = 0; i < customQueryPages.size(); i++) {
			QueryPanelItems e = (QueryPanelItems)customQueryPages.get(i);
			queryMultiPage.addPanelSet(e);
			queryPages.add(queryMultiPage);
		}
		queryPages.add(queryColumnPage);
		queryFinalPage.setTitle(finalPageTitle);
		queryPages.add(queryFinalPage);
	}

	public int size() {
		return queryPages.size();
	}

	public int getCurrentPageNum() {
		return currentPage;
	}

	public IWizardPage getCurrentPage() {
		IWizardPage result = null;
		
		if ((currentPage > -1) && (currentPage<queryPages.size())) {
			result = (IWizardPage)queryPages.get(currentPage);
			if (currentPage<customPageCount) {
				queryMultiPage.setPanelSet(currentPage);
			}
		}
		return result;
	}

	public void getPrevRequest() {
		getPrev();
		firePageReady();
	}
	
	private void getPrev() {
		if (!isFirstPage()) {
			while (currentPage>0) {
				currentPage--;
				if ((currentPage>-1) && (currentPage<customPageCount)) {
					queryMultiPage.setPanelSet(currentPage);
					if (queryMultiPage.isDependencyOkay()) {
						break;
					}
				}
				else {
					break;
				}
			}
		}
	}

	public void getNextRequest() {
		getNext();
		firePageReady();
	}

	private void getNext() {
		//3 states; no current page yet; a page in the middle, and a page at the end
		if (currentPage<0) {
			currentPage = 0;
			return;
		}
		IWizardPage page = getCurrentPage();
		if (page!=null) {
			if (page.isOkToGoNext()) {
				while (currentPage<queryPages.size()) {
					currentPage++;
					if ((currentPage>-1) && (currentPage<customPageCount)) {
						queryMultiPage.setPanelSet(currentPage);
						if (queryMultiPage.isDependencyOkay()) {
							break;
						}
					}
					else {
						break;
					}
				}
			}
		}
	}

	public void getFinishRequest() {
		IWizardPage currentPage = getCurrentPage();
		IWizardPage nextPage = null;
		while (currentPage!=null) {
			getNext();
			nextPage = getCurrentPage();
			if (nextPage==currentPage) {
				break;
			}
			else {
				currentPage = nextPage;
			}
		}
		firePageReady();
	}

	public boolean isFirstPage() {
		return currentPage==0;
	}

	public boolean isLastPage() {
		return (currentPage==queryPages.size()-1);
	}

	public void reset() {
		currentPage = -1;
		getNextRequest();
	}

	public void setModel(IQueryTool qt) throws QueryDAOException {
		if (this.query!=null) {
			this.query.removePropertyChangeListener(tagChangeListener);
		}
		this.query = qt.getCurrentQuery();
		queryFinalPage.setQueryCollection(qt.getQueryCollection());
		if ((customPageCount<1) || (queryColumnPage==null)) {
			throw new RuntimeException("Wizard panels not initialized; cannot set up query edit pages.");
		}
		queryMultiPage.setModel(query);
		connectButtonsToModel();
		settingUpButtons = true;
		initQueryButtonSelection(query);
		queryColumnPageManager.setAvailableColumnsModel(qt.getAvailableColumnsModel());
		queryColumnPageManager.setQueryModel(query);
		queryColumnPageManager.setPageModel(queryColumnPage);
		settingUpButtons = false;
		query.addPropertyChangeListener(tagChangeListener);
		queryFinalPage.setDisplayName(query.getDisplayName());
		queryFinalPage.setSaveSelected(true);
		queryFinalPage.setIsNewQuery(query.isNullQueryID());
	}
	
	public void clearModel() {
		queryColumnPageManager.clearModels();
		if (query!=null) {
			query.removePropertyChangeListener(tagChangeListener);
			query = null;
		}
	}

	private void initQueryButtonSelection(final IQuery query) {
		setTags.clear();
		query.getConstraints().getSetTags(setTags);
		for (Iterator i = panelFactory.getButtons().iterator(); i.hasNext();) {
			ITagComponent e = (ITagComponent) i.next();
			e.setSelected(false);
			if (setTags.contains(e.getTag())) {
				e.setSelected(true);
			}
		}
		panelFactory.initializeCheckGroups();
	}
	
	private void connectButtonsToModel() {
		if (buttonsConnected) return;
		for (Iterator i = panelFactory.getButtons().iterator(); i.hasNext();) {
			ITagComponent e = (ITagComponent) i.next();
			e.addActionListener(this);
			buttonsConnected = true;
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (settingUpButtons) return;
		ITagComponent button = (ITagComponent)e.getSource();
		if ((button.getTag()!=null) && (button.getTag().length()>0)) {
			if (button.isSelected()) {
				query.getConstraints().setTag(button.getTag());
			}
			else {
				query.getConstraints().clearTag(button.getTag());
			}
		}
	}
	
	private void parmAdd(Object itemTag) {
		if (settingUpButtons) return;
		if (panelFactory.getButtonMap().containsKey(itemTag)) {
			ITagComponent item = (ITagComponent) panelFactory.getButtonMap().get(itemTag);
			if (!item.isSelected()) {
				item.setSelected(true);
			}
		}
	}

	private void parmClear(Object tags) {
		if (settingUpButtons) return;
		List tagList = (List)tags;
		if ((tagList!=null) && (tagList.size()>0)) {
			for (Iterator i = tagList.iterator(); i.hasNext();) {
				String tag = (String) i.next();
				if (panelFactory.getButtonMap().containsKey(tag)) {
					ITagComponent item = (ITagComponent) panelFactory.getButtonMap().get(tag);
					if (item.isSelected()) {
						item.setSelected(false);
					}
				}
			}
		}
	}
	
	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
	}

	public void goToPageRequest(int pageNum) {
		if (currentPage<pageNum) {
			while (currentPage<pageNum && currentPage< queryPages.size()-1) {
				getNext();
				if (!getCurrentPage().isOkToGoNext()) {
					break;
				}
			}
		}
		else if (currentPage>pageNum) {
			while (currentPage>pageNum && currentPage>0) {
				getPrev();
			}
		}
		firePageReady();
	}

	public boolean isSaveSelected() {
		return queryFinalPage.isSaveSelected();
	}

	public String getDisplayName() {
		return queryFinalPage.getDisplayName();
	}

	public boolean isComplete() {
		return getCurrentPage()==null;
	}

	public IQueryHTMLPanel getHTMLPanel() {
		return queryMultiPage.getHTMLPanel();
	}
	
}
