package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.Box;
import javax.swing.JLabel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Query constraint page set factory.
 */
public class QueryPanelFactory implements IQueryPanelFactory {
	private static final String CHECKED_VALUE = "checked";
	private static final String FALSE_VALUE = "false";
	private static final String TITLE_ATTRIBUTE = "title";
	private static final String DEPENDENCY_ELEMENT = "dependency";
	private static final String DEFAULT_SETTING_ATTRIBUTE = "defaultSetting";
	private static final String IS_VISIBLE_ATTRIBUTE = "isVisible";
	private static final String RADIOBUTTON_ELEMENT = "radioButton";
	private static final String ELEMENT_ELEMENT = "element";
	private static final String REQUIRED_ELEMENT = "required";
	private static final String PAGE_ELEMENT = "page";
	private static final String RADIO_GROUP_ELEMENT = "radioGroup";
	private static final String GROUP_BOX_ELEMENT = "groupBox";
	private static final String CHECK_BOX_GROUP_ELEMENT = "checkBoxGroup";
	private static final String LABEL_ELEMENT = "label";
	private static final boolean FINAL_PAGE = true;
	private static final boolean NOT_FINAL_PAGE = false;
	private List panelList = new LinkedList();
	private Map allButtonMap = new HashMap();
	private List checkGroupList = new LinkedList();
	private List allButtonList = new LinkedList();
	private final String TAG = "tag";
	private final String CHECKBOX = "checkBox";

	public List getPanels() throws QueryDAOException {
		if (panelList.size()>0) return panelList;
		panelList.clear();
		allButtonMap.clear();
		allButtonList.clear();
		Document document;
		try {
			DocumentBuilderFactory factory =
				DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(QueryPanelFactory.class.getResourceAsStream("queryWizardPanel.xml"));
			document.getDocumentElement();
		}
		catch (Exception e) {
			throw new QueryDAOException(e);
		}
		if (document != null) {
			NodeList pageList = document.getElementsByTagName(PAGE_ELEMENT);
			if (pageList.getLength()>0) {
				for (int i = 0; i < pageList.getLength(); i++) {
					panelList.add(createPage(pageList.item(i),allButtonList));
				}
			}
		}
		for (Iterator i = allButtonList.iterator(); i.hasNext();) {
			ITagComponent e = (ITagComponent) i.next();
			if ((e.getTag()!=null) && (e.getTag().length()>0)) {
				allButtonMap.put(e.getTag(),e);
			}
		}
		return panelList;
	}
	
	public Collection getButtons() {
		return allButtonList;
	}

	private QueryPanelItems createPage(Node node, List buttonList) {
		List singlePageButtonList = new LinkedList();
		QueryPanelItems pageItems = new QueryPanelItems();
		pageItems.setTitle(getTitle(node));
		pageItems.setDependencies(getDepend(node.getFirstChild()));
		pageItems.setRequirements(getRequirements(node.getFirstChild()));
		EnablePanel panel = getPanel(node.getFirstChild(),singlePageButtonList, FINAL_PAGE);
		panel.add(Box.createVerticalGlue());
		pageItems.setPanel(panel);
		pageItems.getTagComponents().addAll(singlePageButtonList);
		buttonList.addAll(singlePageButtonList);
		return pageItems;
	}

	private List getRequirements(Node eNodes) {
		List result = new LinkedList();
		List subResult = null;
		Node aNode;
		Node tagNode;
		while (eNodes != null) {
			if (eNodes.getNodeName().equals(REQUIRED_ELEMENT)) {
				aNode = eNodes.getFirstChild();
				if (subResult!=null) {
					result.add(subResult);
				}
				subResult = new LinkedList();
				while (aNode!=null) {
					if (aNode.getNodeName().equals(TAG)) {
						tagNode = aNode.getFirstChild();
						subResult.add(tagNode.getNodeValue());
					}
					aNode = aNode.getNextSibling();
				}
			}
			eNodes = eNodes.getNextSibling();
		}
		if (subResult!=null) {
			result.add(subResult);
		}
		return result;
	}

	private EnablePanel getPanel(Node node, List buttons, boolean isFinalPage) {
		if (node == null) return null;
		EnablePanel result = new EnablePanel();
//		result.setBackground(defaultPanelColor);
		GridBagConstraints c = initConstraints();
		result.setLayout(new GridBagLayout());
		while (node!=null) {
			if (node.getNodeName().equals(ELEMENT_ELEMENT)) {
				Component comp = getElement(node,buttons);
				if (comp != null) {
					result.add(comp,c);
					c.gridy++;
				}
			}
			node = node.getNextSibling();
		}
		if (isFinalPage) {
			c.fill = GridBagConstraints.BOTH;
			c.weighty = 1.0;
			result.add(Box.createVerticalGlue(), c);
		}
		return result;
	}

	private GridBagConstraints initConstraints() {
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(0,0,0,0);
		c.ipadx = 0;
		c.ipady = 0;
		c.weightx = 1.0;
		c.weighty = 0.0;
		return c;
	}

	private Component getElement(Node item, List buttons) {
		if (item==null) return null;
		Component result = null;
		Node node = item.getFirstChild();
		Node textNode;
		while (node!=null) {
			if (node.getNodeName().equals(LABEL_ELEMENT)) {
				textNode = node.getFirstChild();
				result = new JLabel(textNode.getNodeValue());
			}
			else if (node.getNodeName().equals(CHECKBOX)) {
				result = getButton(node, new CheckBoxTag());
				buttons.add(result);
			}
			else if (node.getNodeName().equals(CHECK_BOX_GROUP_ELEMENT)) {
				result = getCheckBoxGroup(node,buttons);
			}
			else if (node.getNodeName().equals(GROUP_BOX_ELEMENT)) {
				result = getGroupBox(node,buttons);
			}
			else if (node.getNodeName().equals(RADIO_GROUP_ELEMENT)) {
				result = getRadioGroup(node,buttons);
			}
			node = node.getNextSibling();
		}
		return result;
	}

	private Component getRadioGroup(Node node, List buttons) {
		if (node==null) return null;
		RadioPanel newPanel = new RadioPanel();
//		newPanel.setBackground(defaultPanelColor);
		RadioPanel radioPanel = (RadioPanel)setupPanel(node, newPanel);
		Node item = null;
		
		item = node.getFirstChild();
		ITagComponent button = null;
		while (item!=null) {
			if (item.getNodeName().equals(RADIOBUTTON_ELEMENT)) {
				button = (ITagComponent)getButton(item, new RadioButtonTag());
				buttons.add(button);
				radioPanel.add((Component)button);
			}
			item = item.getNextSibling();
		}
		
		return radioPanel;
	}

	private Component getButton(Node node, ITagComponent button) {
		if (node==null) return (Component)button;
		if (node.hasAttributes()) {
			Node item;
			if ((item=node.getAttributes().getNamedItem(TAG))!=null) {
				button.setTag(item.getNodeValue());
			}
			if ((item=node.getAttributes().getNamedItem(IS_VISIBLE_ATTRIBUTE))!=null) {
				button.setVisible(!item.getNodeValue().equals(FALSE_VALUE));
			}
			if ((item=node.getAttributes().getNamedItem(DEFAULT_SETTING_ATTRIBUTE))!=null) {
				button.setSelected(item.getNodeValue().equals(CHECKED_VALUE));
			}
		}
		else {
			throw new RuntimeException("Invalid Query Wizard Panel Data:Button Attributes");
		}
		Node textNode = node.getFirstChild();
		if (textNode!=null) {
			button.setText(textNode.getNodeValue());
		}
		else {
			throw new RuntimeException("Invalid Query Wizard Panel Data:Button Label");
		}
		return (Component)button;
	}
	
	private Component getCheckBoxGroup(Node node,List buttons) {
		if (node==null) return null;
		CheckBoxPanel panel = new CheckBoxPanel();
		checkGroupList.add(panel);
		Node item = node.getFirstChild();
		ITagComponent button = null;
		while (item!=null) {
			if (item.getNodeName().equals(CHECKBOX)) {
				button = (ITagComponent)getButton(item, panel.getCheckBox());
				buttons.add(button);
				break;
			}
			item = item.getNextSibling();
		}
		if (button==null) {
			throw new RuntimeException("CheckGroup box definition is invalid.");
		}
		
		Component c = getPanel(item.getNextSibling(), buttons, NOT_FINAL_PAGE);
		if (c!=null) {
			panel.add(c);
		}
		
		return panel;
	}

	private Component getGroupBox(Node node, List buttons) {
		if (node==null) return null;
		String title = getTitle(node);
		EnablePanel panel = getPanel(node.getFirstChild(), buttons, NOT_FINAL_PAGE);
		if (title!=null) {
			panel.setBorderText(title);
			panel.setBorderVisible(true);
		}
		
		return panel;
	}
	
	private IEnablePanel setupPanel(Node node, IEnablePanel panel) {
		String title = getTitle(node);
		if ((title!=null) && (title.length()>0)) {
			panel.setBorderText(title);
		}
		
		return panel;
	}

	private List getDepend(Node node) {
		List result = new LinkedList();
		Node aNode;
		Node tagNode;
		while (node != null) {
			if (node.getNodeName().equals(DEPENDENCY_ELEMENT)) {
				aNode = node.getFirstChild();
				while (aNode!=null) {
					if (aNode.getNodeName().equals(TAG)) {
						tagNode = aNode.getFirstChild();
						result.add(tagNode.getNodeValue());
					}
					aNode = aNode.getNextSibling();
				}
			}
			node = node.getNextSibling();
		}
		return result;
	}

	private String getTitle(Node node) {
		String result = null;
		Node item = null;
		if ((item=node.getAttributes().getNamedItem(TITLE_ATTRIBUTE))!=null) {
			result = item.getNodeValue();
		}
		return result;
	}
	
	/**
	 * Method initializeCheckGroups.
	 */
	public void initializeCheckGroups() {
		for (Iterator i = checkGroupList.iterator(); i.hasNext();) {
			CheckBoxPanel panel = (CheckBoxPanel)i.next();
			if (panel.containsSelectedItems()) {
				if (!panel.getCheckBox().isSelected()) {
					panel.getCheckBox().doClick();
				}
				else {
					panel.fireUpdateCheckBox();
				}
			}
			else {
				if (panel.getCheckBox().isSelected()) {
					panel.getCheckBox().doClick();
				}
				else {
					panel.fireUpdateCheckBox();
				}
			}
		}
	}
	
	public Map getButtonMap() {
		return allButtonMap;
	}

}
