package gov.va.med.hds.querytool.ui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JRadioButton;

/**
 * Extends the EnablePanel to provide a custom radio button group that allows all
 * of the radio buttons to be turned off (the Java model does not allow this.)
 */
public class RadioPanel extends EnablePanel implements ActionListener {
	private boolean actionDisabled = false;
	private JRadioButton currentSelection;

	/**
	 * Constructor for RadioPanel.
	 */
	public RadioPanel() {
		super();
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
	}

	/**
	 * Only allows radio buttons to be added as components.
	 * @see java.awt.Container#add(java.awt.Component)
	 */
	public Component add(Component comp) {
		if (comp instanceof JRadioButton) {
			((JRadioButton)comp).addActionListener(this);
			return super.add(comp);
		}
		else {
			throw new RuntimeException("Only radio buttons should be added to the radio button panel.");
		}
	}
	
	/**
	 * Handles button events from the buttons contained by this panel, to make sure
	 * only one at a time is selected.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (actionDisabled) return;
		if (currentSelection==null) {
			currentSelection = (JRadioButton)e.getSource();
			if (currentSelection.isSelected()) {
				for (int i = 0; i < getComponentCount(); i++) {
					if (getComponent(i)!=e.getSource()) {
						if (((JRadioButton)getComponent(i)).isSelected()) {
							((JRadioButton)getComponent(i)).doClick();
							break;
						}
					}
				}
			}
			else {
				currentSelection.doClick();
			}
			currentSelection = null;
		}
	}
	
	/**
	 * Turns off one-at-a-time button handling when disabling the buttons in this
	 * panel.
	 * @see java.awt.Component#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		actionDisabled = true;
		super.setEnabled(enabled);
		actionDisabled = false;
	}

}
