/*
 * Initially Created on Jun 4, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.querytool.IQueryTool;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.border.EmptyBorder;

/**
 * Panel to view the progress of a running report.
 */
public class ReportProgress extends JPanel implements IReportProgress {
	private static final String searchingText = "Searching...";
	private boolean waiting = false;
	private Timer uiUpdateTimer;
	private IQueryTool qt;
	private JButton stopButton = new JButton();
	private DecimalFormat decimalFormat = new DecimalFormat("#,###,##0");
	private JLabel evaluatedLabel = new JLabel(decimalFormat.format(0));
	private JLabel excludedLabel = new JLabel(decimalFormat.format(0));
	private JLabel totalLabel = new JLabel(decimalFormat.format(0));
	private JLabel statusLabel = new JLabel();
	private int twizzleCount;
	private char[] twizzleArray = new char[] {'|','/','-','\\'};
	private JLabel searchingLabel = new JLabel(searchingText);
	private ActionListener timerListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			updateCounts();
		}
	};

	public ReportProgress() {
		super();
		putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.StoppingAQuery");
		setBorder(new EmptyBorder(3,3,3,3));
		uiUpdateTimer = new Timer(200, timerListener);
		uiUpdateTimer.stop();
		setLayout(new BorderLayout());
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(stopButton);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.setBorder(new EmptyBorder(5,3,3,3));
		add(buttonPanel, BorderLayout.SOUTH);
		
		JLabel exLabel = new JLabel("Executing Query...");
		exLabel.setFont(exLabel.getFont().deriveFont(Font.BOLD));
		add(exLabel, BorderLayout.NORTH);
		statusLabel.setText(null);
		
		JPanel countAndQueryDefnPanel = new JPanel();
		countAndQueryDefnPanel.setLayout(new BorderLayout());
		
		JPanel countPanel = new JPanel();
		countPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.NONE;
		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.insets = new Insets(0,0,0,0);
		c.ipadx = 10;
		c.ipady = 0;
		c.weightx = 0.0;
		c.weighty = 0.0;
		countPanel.add(searchingLabel,c);
		
		c.gridy = 1;
		countPanel.add(new JLabel("Patients Evaluated by Search:"),c);
		c.gridy = 1;
		c.gridx = 1;
		evaluatedLabel.setHorizontalAlignment(JLabel.RIGHT);
		evaluatedLabel.setPreferredSize(new Dimension(40, evaluatedLabel.getPreferredSize().height));
		countPanel.add(evaluatedLabel,c);
		
		c.gridy = 2;
		c.gridx = 0;
		countPanel.add(new JLabel("Patients Excluded by Screen:"),c);
		c.gridy = 2;
		c.gridx = 1;
		excludedLabel.setHorizontalAlignment(JLabel.RIGHT);
		excludedLabel.setPreferredSize(new Dimension(40, excludedLabel.getPreferredSize().height));
		countPanel.add(excludedLabel,c);
		
		c.gridy = 3;
		c.gridx = 0;
		countPanel.add(new JLabel("Total Records Found:"),c);
		c.gridy = 3;
		c.gridx = 1;
		totalLabel.setHorizontalAlignment(JLabel.RIGHT);
		totalLabel.setPreferredSize(new Dimension(40, totalLabel.getPreferredSize().height));
		countPanel.add(totalLabel,c);
		
		c.gridy = 4;
		c.gridx = 0;
		statusLabel.setFont(statusLabel.getFont().deriveFont(Font.BOLD));
		countPanel.add(statusLabel,c);
		
		countAndQueryDefnPanel.add(countPanel, BorderLayout.NORTH);
		this.add(countAndQueryDefnPanel, BorderLayout.CENTER);
		
	}
	
	public void setQueryModel(IQueryTool queryTool) {
		if (waiting) {
			stopWaiting();
		}
		this.qt = queryTool;
		if (qt!=null) {
			stopButton.setAction(qt.getQueryAction(IQueryTool.STOP_ACTION));
		}
	}
	
	public void startWaiting() {
		waiting = true;
		uiUpdateTimer.restart();
	}
	
	public void stopWaiting() {
		waiting = false;
		uiUpdateTimer.stop();
	}
	
	private void updateCounts() {
		if (qt==null || qt.getRunModel()==null) return;
		totalLabel.setText(decimalFormat.format(qt.getRunModel().getRowCount()));
		evaluatedLabel.setText(decimalFormat.format(qt.getRunModel().getPatientCount()));
		excludedLabel.setText(decimalFormat.format(qt.getRunModel().getScreenedCount()));
		updateTwizzle();
	}
	
	private void updateTwizzle() {
		twizzleCount = ++twizzleCount % 4;
		searchingLabel.setText(searchingText+twizzleArray[twizzleCount]);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("ReportProgress");
		ReportProgress reportProgress = null;
		reportProgress = new ReportProgress();
		frame.getContentPane().add(reportProgress);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	public void setMessage(String string) {
		searchingLabel.setText(string);
	}

	public void initialize() {}

	public void unInitialize() {}
	
}
