package gov.va.med.hds.querytool.ui;
import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.IDesktopLookAndFeelConstants;
import gov.va.med.hds.cd.spectrum.linkpanel.IHTMLSelectionModel;
import gov.va.med.hds.cd.spectrum.linkpanel.ISelectionEventListener;
import gov.va.med.hds.cd.spectrum.list.AssertEvent;
import gov.va.med.hds.cd.spectrum.list.IAssertListener;
import gov.va.med.hds.querytool.ICombinedQueryCollection;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.IQueryID;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.IQueryToolEvent;
import gov.va.med.hds.querytool.ISoftListItem;
import gov.va.med.hds.querytool.QueryActionEvent;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

public class ReportSelection extends JPanel implements IQueryMainPage {
	private JPanel buttonPanel = new JPanel();
	private JButton editButton = new JButton();
	private JButton deleteButton = new JButton();
	private JButton viewReportButton = new JButton();
	private JButton renameButton = new JButton();
	private JButton saveAsButton = new JButton();
	private JPanel contentPane = new JPanel();
	private JSplitPane contentSplit = new JSplitPane();
	private CriteriaPanel criteriaPanel;
	private ChooseSearchPanel chooseSearchPanel = new ChooseSearchPanel();
	private ICombinedQueryCollection queryCollection;
	private IQueryListModel queryListModel;
	private IQueryTool qt;
	private IQueryHTMLPanel queryHTMLPanel = new QueryHTMLPanel();
	private ISelectionEventListener selectionEventListener = new ISelectionEventListener() {
		public void itemSelected(ActionEvent actionEvent) {
			if (actionEvent.getActionCommand().equals(IHTMLSelectionModel.ITEM_SELECTED)) {
				IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.EDIT_PARAMETER);
				event.setNewValue(actionEvent.getSource());
				qt.getQueryAction(IQueryTool.EDIT_PARM_ACTION).actionPerformed((ActionEvent)event);
			}
		}
	};

	public ReportSelection() {
		super();
		init();
	}

	private void init() {
		((JComponent)queryHTMLPanel).putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, "gov.va.med.hds.querytool.asfcr");
		criteriaPanel = new CriteriaPanel((JPanel)queryHTMLPanel);
		queryHTMLPanel.addSelectionEventListener(selectionEventListener);
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		contentPane.setLayout(new BorderLayout());
		contentSplit.setDividerLocation(200);
		contentSplit.setOrientation(JSplitPane.VERTICAL_SPLIT);
		
		this.setLayout(new BorderLayout());
		this.add(buttonPanel, BorderLayout.SOUTH);
		contentSplit.add(criteriaPanel, JSplitPane.RIGHT);
		chooseSearchPanel.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, "gov.va.med.hds.querytool.sar");
		contentSplit.add(chooseSearchPanel, JSplitPane.LEFT);
		
		contentPane.add(contentSplit, BorderLayout.CENTER);
		this.add(contentPane, BorderLayout.CENTER);
		
		int m = IDesktopLookAndFeelConstants.MARGIN;
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(m, m, m, m));
		
		editButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, "gov.va.med.hds.querytool.ecpdr");
		buttonPanel.add(editButton);
		buttonPanel.add(Box.createHorizontalStrut(2 * m));
		deleteButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.deletingareport");
		buttonPanel.add(deleteButton);
		buttonPanel.add(Box.createHorizontalStrut(2 * m));
		renameButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.RenamingAReport");
		buttonPanel.add(renameButton);
		buttonPanel.add(Box.createHorizontalStrut(2 * m));
		saveAsButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.SavingAReportAs");
		buttonPanel.add(saveAsButton);
		buttonPanel.add(Box.createHorizontalGlue());
		viewReportButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.dtr");
		buttonPanel.add(viewReportButton);
		
		chooseSearchPanel.addUserSelectionListener(new IAssertListener() {
			public void valueChanged(AssertEvent e) {
				Object item = chooseSearchPanel.getSelectedValue();
				if (item!=null) {
					if (queryListModel.isCustomQuerySelection(item)) {
						qt.getQueryAction(IQueryTool.CREATE_ACTION).actionPerformed(new QueryActionEvent(this, IQueryActionEvent.CREATE_NEW));
					}
					else {
						IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.SELECT);
						event.setNewValue(item);
						qt.getQueryAction(IQueryTool.SELECT_ACTION).actionPerformed((ActionEvent)event);
					}
				}
			}
		});
		
	}

	public void addQueryActionListener(IQueryAction l) {
		listenerList.add(IQueryAction.class, l);
	}

	public void removeQueryActionListener(IQueryAction l) {
		listenerList.remove(IQueryAction.class, l);
	}

	public Object getCurrentSelection() {
		return chooseSearchPanel.getSelectedValue();
	}
	
	public int getSelectedIndex() {
		return chooseSearchPanel.getSelectedIndex();
	}
	
	public static final void main(String args[]) {
		JFrame frame = new JFrame("ReportSelection");
		ReportSelection reportSelection = new ReportSelection();
		frame.getContentPane().add(reportSelection);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	/**
	 * Sets the selected index.
	 * @param index
	 */
	public void setSelectedIndex(int index) {
		chooseSearchPanel.setSelectedIndex(index);
	}

	public void setModelQueryCollection(ICombinedQueryCollection queryCollection) {
	}

	public void setModel(IQueryTool queryTool) {
		if (this.qt!=null) {
			this.qt.removeQueryToolListener(this);
			if (queryListModel!=null) {
				queryListModel.setModel(null);
			}
			queryHTMLPanel.setQueryModel(null);
		}
		this.qt = queryTool;
		if (queryTool!=null) {
			editButton.setAction(queryTool.getQueryAction(IQueryTool.EDIT_ACTION));
			deleteButton.setAction(queryTool.getQueryAction(IQueryTool.DELETE_ACTION));
			viewReportButton.setAction(queryTool.getQueryAction(IQueryTool.RUN_ACTION));
			renameButton.setAction(queryTool.getQueryAction(IQueryTool.RENAME_ACTION));
			saveAsButton.setAction(queryTool.getQueryAction(IQueryTool.SAVEAS_ACTION));
			
			queryTool.addQueryToolListener(this);
			this.queryCollection = queryTool.getQueryCollection();
			queryListModel = new QueryListModel();
			queryListModel.setModel(queryCollection);
			chooseSearchPanel.setModel(queryListModel);
			queryHTMLPanel.setQueryModel(queryTool.getCurrentQuery());
		}
	}

	public void actionOccured(IQueryToolEvent event) {
		switch (event.getEventID()) {
			case (IQueryToolEvent.CURRENT_QUERY_SET) : {
				doSetCurrentQuery();
				break;
			}
		}
	}

	private void doSetCurrentQuery() {
		queryHTMLPanel.setQueryModel(qt.getCurrentQuery());
		if (!currentSelectionIsShown()) {
			moveSelection();
		}
	}

	private void moveSelection() {
		IQuery query = qt.getCurrentQuery();
		int index = -1;
		for (int i = 0; i < queryListModel.getSize(); i++) {
			Object object = queryListModel.getElementAt(i);
			if (object instanceof ISoftListItem) {
				ISoftListItem item = (ISoftListItem)object;
				if (item.getItem()!=null && item.getItem().equals(query)) {
					index = i;
					break;
				}
			}
		}
		
		if (index>-1) {
			chooseSearchPanel.setSelectedIndex(index);
		}
	}

	private boolean currentSelectionIsShown() {
		if (qt.getCurrentQuery()==null) {
			return true;
		}
		
		if (qt.getQueryCollection().isQueryItem(chooseSearchPanel.getSelectedValue())) {
			ISoftListItem item = (ISoftListItem)chooseSearchPanel.getSelectedValue();
			IQueryID queryID = (IQueryID)item.getID();
			String name = item.getDisplayName();
			if (qt.getCurrentQuery().getDisplayName().equalsIgnoreCase(name) && qt.getCurrentQuery().getQueryID().equals(queryID)) {
				return true;
			}
		}
		
		return false;
	}

	public IHTMLSelectionModel getQueryHTMLSelectionModel() {
		if (queryHTMLPanel!=null) {
			return queryHTMLPanel.getSelectionModel();
		}
		return null;
	}

	public void initialize() {}

	public void unInitialize() {}

}
