package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.querytool.ResultSaveException;
import gov.va.med.hds.util.io.FileUtilities;

import java.io.File;
import java.io.FileWriter;

import javax.swing.table.TableModel;

/**
 * Contains a method to save a table model to a file, tab delimited.
 */
public class SaveResults {
	public static final String DELIMITER = "\t";

	/**
	 * Saves the query results from the table model to the specified file.
	 * @param tableModel
	 * @param file
	 * @throws ResultSaveException
	 * @throws CantWriteFileException
	 */
	public static void saveReport(TableModel model, File file) throws ResultSaveException, CantWriteFileException {
		FileWriter writer = null;
		String ext = FileUtilities.getFileExtension(file);
		if (ext == null) {
			try {
				file = new File(file.getCanonicalPath() + ".txt");
			}
			catch (Exception e) {
				if (file!=null) {
					throw new ResultSaveException("Error opening file to save:"+file.toString(),e);
				}
				else {
					throw new ResultSaveException("Error opening file to save",e);
				}
			}
		}
		try {
			file.createNewFile();
		}
		catch (Exception e1) {
			throw new CantWriteFileException("No access to create file:"+file.toString());
		}
		if (!file.canWrite()) {
			throw new CantWriteFileException("No access to create file:"+file.toString());
		}
		try {
			writer = new FileWriter(file);
			boolean first;
			for (int i = 0; i < model.getRowCount(); i++) {
				first = true;
				for (int j = 0; j < model.getColumnCount(); j++) {
					if (first) {
						first = false;
					}
					else {
						writer.write(DELIMITER);
					}
					Object item = model.getValueAt(i, j);
					if (item!=null) {
						writer.write(item.toString());
					}
				}
				writer.write("\n");
			}
		}
		catch (Exception e) {
			throw new ResultSaveException("Error creating file:"+file.toString(),e);
		}
		finally {
			try {
				writer.close();
			}
			catch (Exception e) {
				if (file!=null) {
					throw new ResultSaveException("Error closing file:"+file.toString(),e);
				}
				else {
					throw new ResultSaveException("Error closing file",e);
				}
			}
		}
	}

}
