package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.IDesktopLookAndFeelConstants;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.linkpanel.IHTMLSelectionModel;
import gov.va.med.hds.clinicaldata.IPatientDemographics;
import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.QueryActionEvent;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.MouseInputAdapter;

public class ViewReport extends JPanel implements IViewReport {
	private final String warningString = "Reports are compiled for groups of patients, and do not replace" +
		" the need for clinical judgment in individual patient care.";
	private IQueryTool qt;
	private JTable detailTable;
	private PatientLine patientLineLabel = new PatientLine();
	private JTextArea detailText = new JTextArea();
	private JLabel queryNameLabel = new JLabel("<Query Name>");
	private JLabel recordCountLabel = new JLabel(getCountText(0));
	private JLabel dateTimeLabel = new JLabel("<Date Time of Query>");
	private JButton printButton = new JButton();
	private JButton exportButton = new JButton();
	private JButton returnButton = new JButton();
	private ListSelectionListener tableSelectionListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					fireSelectionChange();
				}
			}
		};
	private IPrintHelper printHelper;
	private IHTMLSelectionModel htmlSelectionModel;
	private IQueryAction sortAction;
	private SimpleDateFormat dateFormatter = new SimpleDateFormat("MMMM dd, yyyy HH:mm:ss");
	private JScrollPane tableScrollPane = new JScrollPane();
	private IRuntimeContext runtime;
	

	public ViewReport() {
		super();
		init();
	}

	private void init() {
		this.setLayout(new BorderLayout());
		this.add(setupDetailPanel(), BorderLayout.CENTER);
		this.add(setupButtonPanel(), BorderLayout.SOUTH);
		putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.dtr");
	}

	private Component setupDetailPanel() {
		JSplitPane splitPane = new JSplitPane();
		
		detailText.setEditable(false);
		detailText.setCaretPosition(0);
		
		JPanel bottomPane = new JPanel();
		bottomPane.setLayout(new BorderLayout());
		
		JScrollPane scrollPane = new JScrollPane(detailText);
		scrollPane.setColumnHeaderView(patientLineLabel);
		bottomPane.add(scrollPane,BorderLayout.CENTER);
		
		JPanel topPane = new JPanel();
		JPanel messagePane = new JPanel();
		messagePane.setLayout(new GridBagLayout());
		GridBagConstraints mc = new GridBagConstraints(0,0,1,1,.60,0.0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0);
		queryNameLabel.setFont(queryNameLabel.getFont().deriveFont(Font.BOLD));
		messagePane.add(queryNameLabel,mc);
		mc.gridx = 1;
		mc.weightx = .20;
		messagePane.add(recordCountLabel,mc);
		mc.anchor = GridBagConstraints.EAST;
		mc.gridx = 2;
		messagePane.add(dateTimeLabel,mc);
		mc.gridx = 0;
		mc.gridy = 1;
		mc.gridwidth = 3;
		mc.weightx = 1.0;
		mc.anchor = GridBagConstraints.CENTER;
		mc.fill = GridBagConstraints.BOTH;
		JLabel warningLabel = new JLabel(warningString);
		warningLabel.setBackground(Color.YELLOW);
		warningLabel.setOpaque(true);
		messagePane.add(warningLabel,mc);
		topPane.setLayout(new BorderLayout());
		topPane.add(messagePane,BorderLayout.NORTH);
		topPane.add(tableScrollPane, BorderLayout.CENTER);
		
		splitPane.setDividerLocation(200);
		splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		splitPane.add(topPane,JSplitPane.TOP);
		splitPane.add(bottomPane,JSplitPane.BOTTOM);
		
		return splitPane;
	}

	private JPanel setupButtonPanel() {
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		printButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.ptr");
		buttonPanel.add(printButton);
		buttonPanel.add(Box.createHorizontalStrut(20));
		exportButton.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.etr");
		buttonPanel.add(exportButton);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(returnButton);
 		int m = IDesktopLookAndFeelConstants.MARGIN;
 		buttonPanel.setBorder(BorderFactory.createEmptyBorder(m, m, m, m));
		
		return buttonPanel;
	}

	public void addQueryActionListener(IQueryAction l) {
		listenerList.add(IQueryAction.class, l);
	}

	public void removeQueryActionListener(IQueryAction l) {
		listenerList.remove(IQueryAction.class, l);
	}

	private void setupTable() {
		detailTable = new JTable(qt.getResultsModel(),ColumnModelFactory.getModel(qt.getCurrentQuery()));
		detailTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		detailTable.getSelectionModel().addListSelectionListener(tableSelectionListener);
		detailTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		detailTable.getTableHeader().addMouseListener(new MouseInputAdapter() {
			public void mouseClicked(MouseEvent e) {
				int col;
				if ((col=detailTable.getTableHeader().columnAtPoint(e.getPoint()))>-1) {
					fireColumnSelected(col);
				}
			}
		});
		tableScrollPane.setViewportView(detailTable);
		try {
			qt.getResultsModel().reset();
		}
		catch (QueryDAOException ex) {
			runtime.getLogger(this.getClass()).error("Error resetting the results table model.", ex);
		}
	}

	private void fireColumnSelected(int col) {
		IQueryActionEvent event = new QueryActionEvent(this, IQueryActionEvent.SORT_COLUMN);
		event.setNewValue(Integer.toString(col));
		sortAction.actionPerformed((ActionEvent)event);
	}
	
	private void fireSelectionChange() {
		if (detailTable.getSelectedRowCount()!=1) {
			patientLineLabel.clear();
			detailText.setText("");
			return;
		}
		int row = detailTable.getSelectedRow();
		if (row > -1) {
			patientLineLabel.clear();
			detailText.setText("Reading Detail from Database...");
			qt.getResultsModel().getPatientDetailRequest(row, this);
			qt.getResultsModel().getDetailRequest(row, this);
		}
	}

	public void setDetail(final String detail) {
		detailText.setText(detail);
		detailText.setCaretPosition(0);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("ViewReport");
		ViewReport viewReport = null;
		viewReport = new ViewReport();
		frame.getContentPane().add(viewReport);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}
	
	public void setPatient(IPatientDemographics patient) {
		patientLineLabel.setPatientLine(patient,false);
	}
	
	private String getCountText(int count) {
		return "(" + Integer.toString(count) + " records)";
	}
	
	public void setQueryModel(IQueryTool queryTool) {
		this.qt = queryTool;
		if (qt!=null) {
			printButton.setAction(qt.getQueryAction(IQueryTool.PRINT_ACTION));
			exportButton.setAction(qt.getQueryAction(IQueryTool.EXPORT_ACTION));
			returnButton.setAction(qt.getQueryAction(IQueryTool.HOME_ACTION));
			sortAction = (IQueryAction)qt.getQueryAction(IQueryTool.SORT_COLUMN_ACTION);
		}
	}

	public void setQueryHTMLModel(IHTMLSelectionModel model) {
		this.htmlSelectionModel = model;
	}

	public void initialize() {
		setupTable();
		queryNameLabel.setText(qt.getCurrentQuery().getDisplayName());
		dateTimeLabel.setText(dateFormatter.format(qt.getRunModel().getStartDate()));
		printHelper.setTable(detailTable);
		printHelper.setDateTimeString(dateFormatter.format(qt.getRunModel().getStartDate()));
		printHelper.setHTMLModel(htmlSelectionModel);
		patientLineLabel.clear();
		detailText.setText("");
		recordCountLabel.setText(getCountText(qt.getRunModel().getRowCount()));
	}

	public void setPrintHelper(IPrintHelper printHelper) {
		this.printHelper = printHelper;
	}

	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
	}

	public void unInitialize() {}

}
