/*
 * Initially Created on Mar 28, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui.editor.factory;

import gov.va.med.hds.cd.spectrum.TabListCellRenderer;
import gov.va.med.hds.querytool.dao.IHepCRegistryAccessDAO;
import gov.va.med.hds.querytool.dao.factory.ClinicListFactory;
import gov.va.med.hds.querytool.dao.factory.ClinicRefFactory;
import gov.va.med.hds.querytool.dao.factory.ClinicianListFactory;
import gov.va.med.hds.querytool.dao.factory.ClinicianRefFactory;
import gov.va.med.hds.querytool.dao.factory.SpecialtyRefFactory;
import gov.va.med.hds.querytool.dao.factory.TeamRefFactory;
import gov.va.med.hds.querytool.dao.factory.WardRefFactory;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.types.ClinApptPanel;
import gov.va.med.hds.querytool.ui.types.EditorException;
import gov.va.med.hds.querytool.ui.types.MultiSourceEditItem;
import gov.va.med.hds.querytool.ui.types.MultiSourceEditor;
import gov.va.med.hds.querytool.ui.types.RegistryBucketPanel;
import gov.va.med.hds.querytool.ui.types.ShortBucketPanel;
import gov.va.med.hds.querytool.ui.types.SingleBucketPanel;

/**
 * Implements an IPatientSelectionFactory to provide an editor.
 */
public class PatientSelectionFactory extends EditorFactory implements IPatientSelectionFactory {
	private String outLabel = "Selected Lists";
	private String inLabel = "Type Name or Select From List";
	private TabListCellRenderer renderer = new TabListCellRenderer();
	private MultiSourceEditItem item;
	
	public PatientSelectionFactory() {
		super();
		int[] tabs = new int[1];
		tabs[0] = 140;
		renderer.setTabs(tabs);
	}

	public IRunnableEditorPanel getEditor() throws EditorException {
		MultiSourceEditor editor = new MultiSourceEditor();
		editor.setRuntime(runtime);
		editor.setDAORegistry(daoRegistry);
		
		ClinApptPanel apptPanel = new ClinApptPanel(new ClinicListFactory(), new ClinicRefFactory());
		apptPanel.setInBucketLabel(inLabel);
		apptPanel.setOutBucketLabel(outLabel);
		apptPanel.setOutBucketCellRenderer(renderer);
		apptPanel.setRuntime(runtime);
		apptPanel.setDAORegistry(daoRegistry);
		item = new MultiSourceEditItem("c","Clinic Appointments","Clinic",apptPanel);
		editor.addPanel(item);
		
		ShortBucketPanel wardPanel = new ShortBucketPanel(new WardRefFactory());
		wardPanel.setInBucketLabelText(inLabel);
		wardPanel.setOutBucketLabelText(outLabel);
		wardPanel.setOutBucketCellRenderer(renderer);
		try {
			wardPanel.setRuntime(runtime);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		wardPanel.setDAORegistry(daoRegistry);
		item = new MultiSourceEditItem("w","Ward","Ward",wardPanel);
		editor.addPanel(item);
		
		ShortBucketPanel teamPanel = new ShortBucketPanel(new TeamRefFactory());
		teamPanel.setInBucketLabelText(inLabel);
		teamPanel.setOutBucketLabelText(outLabel);
		teamPanel.setOutBucketCellRenderer(renderer);
		try {
			teamPanel.setRuntime(runtime);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		teamPanel.setDAORegistry(daoRegistry);
		item = new MultiSourceEditItem("t","Personal/Team","Team",teamPanel);
		editor.addPanel(item);
		
		ShortBucketPanel specialtyPanel = new ShortBucketPanel(new SpecialtyRefFactory());
		specialtyPanel.setInBucketLabelText(inLabel);
		specialtyPanel.setOutBucketLabelText(outLabel);
		specialtyPanel.setOutBucketCellRenderer(renderer);
		try {
			specialtyPanel.setRuntime(runtime);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		specialtyPanel.setDAORegistry(daoRegistry);
		item = new MultiSourceEditItem("s","Specialty","Specialty",specialtyPanel);
		editor.addPanel(item);
		
		SingleBucketPanel bucketPanel = new SingleBucketPanel(new ClinicianListFactory(), new ClinicianRefFactory());
		bucketPanel.setInBucketLabel(inLabel);
		bucketPanel.setOutBucketLabel(outLabel);
		bucketPanel.setOutBucketCellRenderer(renderer);
		bucketPanel.setRuntime(runtime);
		bucketPanel.setDAORegistry(daoRegistry);
		item = new MultiSourceEditItem("p","Provider","Provider",bucketPanel);
		editor.addPanel(item);
		
		if (userAllowedToUseRegistries()) {
			RegistryBucketPanel registryPanel = new RegistryBucketPanel();
			registryPanel.setInBucketLabel(inLabel);
			registryPanel.setOutBucketLabel(outLabel);
			registryPanel.setOutBucketCellRenderer(renderer);
			try {
				registryPanel.setRuntime(runtime);
			}
			catch (Exception e) {
				throw new EditorException(e);
			}
			registryPanel.setDAORegistry(daoRegistry);
			item = new MultiSourceEditItem("r","Registry","Registry",registryPanel);
			editor.addPanel(item);
		}
		
		return editor;
	}

	private boolean userAllowedToUseRegistries() throws EditorException {
		try {
			IHepCRegistryAccessDAO hasAccessDAO = (IHepCRegistryAccessDAO)daoRegistry.getDAO(IHepCRegistryAccessDAO.class.getName());
			return hasAccessDAO.getResult();
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
	}

}
