package gov.va.med.hds.querytool.ui.internal;

import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.querytool.IQueryAction;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.IQueryToolEvent;
import gov.va.med.hds.querytool.IQueryToolListener;
import gov.va.med.hds.querytool.IValue;
import gov.va.med.hds.querytool.QueryActionEvent;
import gov.va.med.hds.querytool.ResultSaveException;
import gov.va.med.hds.querytool.ui.CantWriteFileException;
import gov.va.med.hds.querytool.ui.EditQuery;
import gov.va.med.hds.querytool.ui.EditSensitive;
import gov.va.med.hds.querytool.ui.EditorFactoryFactory;
import gov.va.med.hds.querytool.ui.IEditSensitive;
import gov.va.med.hds.querytool.ui.IEditorFactoryFactory;
import gov.va.med.hds.querytool.ui.IPrintHelper;
import gov.va.med.hds.querytool.ui.IProgressDialogManager;
import gov.va.med.hds.querytool.ui.IQueryActionEvent;
import gov.va.med.hds.querytool.ui.IQueryMainPage;
import gov.va.med.hds.querytool.ui.IQueryPage;
import gov.va.med.hds.querytool.ui.IQueryPageDispatch;
import gov.va.med.hds.querytool.ui.IQueryPages;
import gov.va.med.hds.querytool.ui.IViewReport;
import gov.va.med.hds.querytool.ui.ProgressDialogManager;
import gov.va.med.hds.querytool.ui.QueryFileFilter;
import gov.va.med.hds.querytool.ui.QueryPages;
import gov.va.med.hds.querytool.ui.ReportSelection;
import gov.va.med.hds.querytool.ui.SaveResults;
import gov.va.med.hds.querytool.ui.ViewReport;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;
import gov.va.med.hds.querytool.ui.editor.factory.IEditorFactory;
import gov.va.med.hds.querytool.ui.printables.PrintHelper;
import gov.va.med.hds.querytool.ui.types.EditorException;
import gov.va.med.hds.util.collections.ListUtil;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.List;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**
 * Implements the IQueryPageDispatch to handle query page management.
 */
public class QueryPageDispatch implements IQueryPageDispatch {
	private IQueryPages queryPages = new QueryPages();
	private IQueryMainPage queryMainPage;
	private EditQuery editQuery;
	private IRuntimeContext runtime;
	private ILogger logger;
	private IPluginDescriptor plugin;
	private IQueryTool queryTool;
	private IRunnableEditorState editorPanelState;
	private IViewReport viewReport;
	private IEditSensitive editSensitive;
	private IEditorFactoryFactory editorFactoryFactory;
	private IPrintHelper printHelper = new PrintHelper();
	private IQueryPage currentPage;
	private IQueryToolListener queryToolListener = new IQueryToolListener() {
		public void actionOccured(IQueryToolEvent event) {
			dispatchEvent(event);
		}
	};
	private IProgressDialogManager progressDialogManager;


	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin) throws Exception {
		this.runtime = runtime;
		this.plugin = plugin;
		this.logger = runtime.getLogger(this.getClass());
		queryTool = (IQueryTool)runtime.getPluginInstance(IQueryTool.PLUGIN_ID);
		while (!runtime.isPluginActive(IQueryTool.PLUGIN_ID)) {}
		queryTool.addQueryToolListener(queryToolListener);
		editorFactoryFactory = new EditorFactoryFactory();
		editorFactoryFactory.setRuntime(runtime);
		printHelper.setRuntime(runtime);
		printHelper.setQueryModel(queryTool);
		queryPages.addPage(IPrintHelper.class.getName(), (JComponent)printHelper);
		Action action = queryTool.getQueryAction(IQueryTool.HOME_ACTION);
		action.actionPerformed(new QueryActionEvent(this, IQueryActionEvent.HOME));
	}

	public void shutdown() throws Exception {
		queryTool.removeQueryToolListener(queryToolListener);
	}

	private void setCurrentPage(IQueryPage queryPage) {
		if (currentPage!=null) {
			currentPage.unInitialize();
		}
		currentPage = queryPage;
		
		if (currentPage!=null) {
			currentPage.initialize();
		}
	}

	private void goHomeState() {
		if (!queryPages.contains(IQueryMainPage.class.getName())) {
			queryMainPage = new ReportSelection();
			queryMainPage.setModel(queryTool);
			queryPages.addPage(IQueryMainPage.class.getName(), (JComponent)queryMainPage);
		}
		setCurrentPage(queryMainPage);
		queryPages.selectPage(IQueryMainPage.class.getName());
	}
	
	private void goRenameState() {
		if (editQuery==null) {
			setupEditQuery();
		}
		editQuery.setWizardPage(Integer.MAX_VALUE);
		setCurrentPage(editQuery);
		queryPages.selectPage(EditQuery.class.getName());
	}
	
	private void goEditState() {
		if (editQuery==null) {
			setupEditQuery();
		}
		setCurrentPage(editQuery);
		queryPages.selectPage(EditQuery.class.getName());
	}

	private void setupEditQuery() {
		editQuery = new EditQuery();
		editQuery.setRuntime(runtime);
		editQuery.setQueryModel(queryTool);
		queryPages.addPage(EditQuery.class.getName(), editQuery);
	}
	
	private void goViewReport() {
		if (viewReport==null) {
			viewReport = new ViewReport();
			viewReport.setQueryModel(queryTool);
			viewReport.setPrintHelper(printHelper);
			viewReport.setRuntime(runtime);
			queryPages.addPage(IViewReport.class.getName(), (JComponent)viewReport);
		}
		if (queryMainPage!=null && queryMainPage.getQueryHTMLSelectionModel()!=null) {
			viewReport.setQueryHTMLModel(queryMainPage.getQueryHTMLSelectionModel());
		}
		setCurrentPage(viewReport);
		queryPages.selectPage(IViewReport.class.getName());
	}
	
	private void doReportProgress() {
		if (progressDialogManager==null) {
			progressDialogManager = new ProgressDialogManager(PrismUtilities.getDesktop(runtime).getActiveWindow().getFrame());
			progressDialogManager.setQueryModel(queryTool);
		}
		progressDialogManager.startWaiting();
		setCurrentPage(progressDialogManager);
	}
	
	private void goEditSensitive() {
		if (editSensitive==null) {
			editSensitive = new EditSensitive();
			queryPages.addPage(IEditSensitive.class.getName(), (JComponent)editSensitive);
			editSensitive.setQueryModel(queryTool);
		}
		queryPages.selectPage(IEditSensitive.class.getName());
		setCurrentPage(editSensitive);
	}
	
	private void dispatchEvent(final IQueryToolEvent event) {
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					dispatchEvent(event);
				}
			});
			return;
		}
		switch (event.getEventID()) {
			case (IQueryToolEvent.HOME_PAGE): {
				goHomeState();
				break;
			}
			
			case (IQueryToolEvent.CURRENT_QUERY_SET) : {
				goHomeState();
				break;
			}
			
			case (IQueryToolEvent.EDIT_QUERY) : {
				goEditState();
				break;
			}
			
			case (IQueryToolEvent.CONFIRM_DELETE) : {
				if (!confirmDeletion()) return;
				IQueryAction action = (IQueryAction)queryTool.getQueryAction(IQueryTool.DELETE_CONFIRMED_ACTION);
				action.actionPerformed(new QueryActionEvent(this, IQueryActionEvent.DELETE_CONFIRMED));
				break;
			}
			
			case (IQueryToolEvent.EDIT_PARAMETER) : {
				doEditParameter(event);
				break;
			}
			
			case (IQueryToolEvent.ERROR) : {
				break;
			}
			
			case (IQueryToolEvent.EXPORT) : {
				if (viewReport!=null) {
					File file = null;
					while ((file=getSaveFile())!=null) {
						try {
							SaveResults.saveReport(queryTool.getResultsModel(), file);
							break;
						}
						catch (ResultSaveException e) {
							logger.error("Error saving query results",e);
							showExportMessage("There was an error saving the query results.");
							break;
						}
						catch (CantWriteFileException e) {
							logger.info("Error saving query results",e);
							showExportMessage("You do not have authorization to save the file in the specified location.\nSelect a location that you can save the query results.");
						}
					}
				}
				break;
			}
			
			case (IQueryToolEvent.MESSAGE) : {
				JOptionPane.showMessageDialog((Component)queryPages, queryTool.getMessage());
				break;
			}
			
			case (IQueryToolEvent.PRINT) : {
				if (viewReport!=null) {
					printHelper.doPrint();
				}
				break;
			}
			
			case (IQueryToolEvent.RENAME) : {
				goRenameState();
				break;
			}
			
			case (IQueryToolEvent.RUN_COMPLETE) : {
				goViewReport();
				break;
			}
			
			case (IQueryToolEvent.RUN_COMPLETE_NON_SENSITIVE) : {
				goEditSensitive();
				break;
			}
			
			case (IQueryToolEvent.RUN_STARTED) : {
				doReportProgress();
				break;
			}
			
			case (IQueryToolEvent.SAVE_AS) : {
				String name = null;
				while (name==null) {
					name = getNameFromDialog();
					if ((name==null) || (name.length()<1)) {
						break;
					}
					if (queryTool.getQueryCollection().containsDisplayName(name)) {
						boolean okSelected = showNameErrorDialog(name);
						name = null;
						if (!okSelected) {
							break;
						}
					}
				}
				if ((name!=null) && (name.length()>0)) {
					IQueryAction action = (IQueryAction)queryTool.getQueryAction(IQueryTool.SAVEAS_CONFIRMED_ACTION);
					IQueryActionEvent e = new QueryActionEvent(this,IQueryActionEvent.SAVE_AS_CONFIRMED);
					e.setNewValue(name);
					action.actionPerformed((ActionEvent)e);
				}
				break;
			}
			
			case (IQueryToolEvent.SELECT) : {
				break;
			}
			
			case (IQueryToolEvent.STOPPED) : {
				progressDialogManager.stopWaiting();
				progressDialogManager.setMessageText("Stopping query...");
			}
			
		}
	}

	private boolean confirmDeletion() {
		if (queryTool.getCurrentQuery()==null) return false;
		JOptionPane optionPane = new JOptionPane("Report: " + queryTool.getCurrentQuery().getDisplayName() + "\nDo you wish to delete this report?",JOptionPane.ERROR_MESSAGE,JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog = optionPane.createDialog((Component)queryPages, "Delete Report");
		dialog.show();
		Object result = optionPane.getValue();
		if ((result != null) && (((Integer)result).intValue()==JOptionPane.OK_OPTION)) {
			return true;
		}
		return false;
	}
	
	private String getNameFromDialog() {
		String name = null;
		JTextField textField = new JTextField();
		JOptionPane optionPane = new JOptionPane(textField,JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog = optionPane.createDialog((Component)queryPages, "Enter New Query Name");
		dialog.show();
		Object result = optionPane.getValue();
		if ((result != null) && (((Integer)result).intValue()==JOptionPane.OK_OPTION)) {
			name = textField.getText();
		}
		return name;
	}

	private boolean showNameErrorDialog(String name) {
		JOptionPane optionPane = new JOptionPane("The query name '"+name+"' is already used.  Enter another name or cancel.",JOptionPane.ERROR_MESSAGE,JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog = optionPane.createDialog((Component)queryPages, "SaveAs Error");
		dialog.show();
		Object result = optionPane.getValue();
		if ((result==null) || (((Integer)result).intValue()!=JOptionPane.OK_OPTION)) {
			return false;
		}
		else {
			return true;
		}
	}

	private File getSaveFile() {
		File file = null;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setApproveButtonText("Save");
		fileChooser.setDialogTitle("Save CPRS Report Data as");
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		fileChooser.setFileFilter(new QueryFileFilter());
		int val = fileChooser.showSaveDialog((Component)queryPages);
		if (val == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		}
		return file;
	}
	
	private void doEditParameter(IQueryToolEvent e) {
		IValue v = e.getValue();
		Cursor currentCursor = queryPages.getCursor();
		queryPages.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		
		//		setup modal dialog
		IRunnableEditorPanel editorPanel = null;
		try {
			editorPanel = ((IEditorFactory)editorFactoryFactory.getFactory(v.getClass().getName())).getEditor();
			editorPanel.addEditorStateChangeListener(new IRunnableEditorStateChangeListener(){
				public void stateChanged(RunnableEditorStateChangedEvent event) {
					editorPanelState=event.getState();
				}
			});
			editorPanel.setEditorState(editorPanelState);
		}
		catch (Exception e2) {
			logger.error("Error creating an editor for "+v.getClass().getName(), e2);
			return;
		}
		editorPanel.setValues(v.getValues());
		try {
			editorPanel.init();
		}
		catch (EditorException e1) {
			logger.error("Error initializing editor for " + v.getClass().getName(), e1);
			return;
		}
		
		JOptionPane optionPane = new JOptionPane(editorPanel,JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog = optionPane.createDialog((Component)queryPages, editorPanel.getWindowHeader());
		
		//		show modal dialog
		queryPages.setCursor(currentCursor);
		dialog.show();
		
		//		save values in query
		Object result = optionPane.getValue();
		if ((result != null) && (((Integer)result).intValue()==JOptionPane.OK_OPTION)) {
			List editorValues = editorPanel.getValues();
			if (!ListUtil.equals(v.getValues(), editorValues)) {
				v.setValues(editorValues);
				if (v.getValues().size()>0) {
					v.setDefaultDisplayText(editorPanel.getDisplayText());
				}
				else {
					v.setDefaultDisplayText(v.getEmptyDisplayText());
				}
			}
		}
		
		if (editorPanel!=null) {
			editorPanel.clearModels();
			editorPanel = null;
		}
	}

	public JComponent getPrintHelper() {
		return (JComponent)printHelper;
	}

	public JComponent getQueryPages() {
		return (JComponent)queryPages;
	}

	private void showExportMessage(String message) {
		JOptionPane.showMessageDialog((Component)queryPages, message,"QueryTool Export Error Message", JOptionPane.ERROR_MESSAGE);
	}

}
