/*
 * Initially Created on Jun 4, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui.printables;

import gov.va.med.hds.cd.print.IPrint;
import gov.va.med.hds.cd.print.IPrintActionEnablingListener;
import gov.va.med.hds.cd.print.IPrintActionHandler;
import gov.va.med.hds.cd.print.PrintActionEnablingEvent;
import gov.va.med.hds.cd.print.PrintHandlerException;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.linkpanel.IHTMLSelectionModel;
import gov.va.med.hds.querytool.IQueryTool;
import gov.va.med.hds.querytool.ui.IPrintHelper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.print.DocFlavor;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JTable;

/**
 * Print helper class for completed queries.
 */
public class PrintHelper extends JComponent implements IPrintHelper {
	private IRuntimeContext runtime;
	private PageSetupHandler pageSetupHandler = new PageSetupHandler();
	private PrintPreviewHandler printPreviewHandler = new PrintPreviewHandler();
	private PrintHandler printHandler = new PrintHandler();
	private JTable detailTable;
	private String dateTimeString;
	private IHTMLSelectionModel queryConstraints;
	private IQueryTool qt;
	private IPrintActionEnablingListener printEnablingListener;
	private IPrintActionEnablingListener printPreviewEnablingListener;
	private IPrintActionEnablingListener pageSetupEnablingListener;
	private PropertyChangeListener printActionListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			processPrintActionChange();
		}
	};
	private Action printAction;
	private IPrint print;

	public PrintHelper() {
		super();
	}

	private class PageSetupHandler implements IPrintActionHandler{
		public void doAction() throws PrintHandlerException {
			try {
				IPrint print = (IPrint)runtime.getPluginInstance(IPrint.PLUGIN_ID);
				print.pageSetup(DocFlavor.SERVICE_FORMATTED.PRINTABLE);
			} catch (Exception e) {
				throw new PrintHandlerException("Couldn't do a page setup from "+PageSetupHandler.class.getName(),e);
			}
		}
	}
	
	private class PrintPreviewHandler implements IPrintActionHandler{
		public void doAction() throws PrintHandlerException {
			try {
				print = (IPrint)runtime.getPluginInstance(IPrint.PLUGIN_ID);
				print.printPreview(new QueryResultTablePrintable( runtime,detailTable,queryConstraints,dateTimeString,qt.getCurrentQuery()),DocFlavor.SERVICE_FORMATTED.PRINTABLE);
			} catch (Exception e) {
				throw new PrintHandlerException("Couldn't do a print preview from "+PrintPreviewHandler.class.getName(),e);
			}
		}
	}
	
	private class PrintHandler implements IPrintActionHandler {
		public void doAction() throws PrintHandlerException {
			IPrint print;
			try {
				print = (IPrint)runtime.getPluginInstance(IPrint.PLUGIN_ID);
				print.print(new QueryResultTablePrintable( runtime,detailTable,queryConstraints,dateTimeString,qt.getCurrentQuery()),DocFlavor.SERVICE_FORMATTED.PRINTABLE);
			}
			catch (Exception e) {
				runtime.getLogger(this.getClass()).error("Error printing query results.", e);
			}
		}
	}
	
	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
		try {
			print = (IPrint)runtime.getPluginInstance(IPrint.PLUGIN_ID);
		}
		catch (Exception e) {
			runtime.getLogger(this.getClass()).error("Error in querytool starting Print plugin.", e);
		}
	}
	public void removePrintActionEnablingListener(IPrintActionEnablingListener listener) {
		printEnablingListener = null;
	}

	public void addPrintActionEnablingListener(IPrintActionEnablingListener listener) {
		printEnablingListener = listener;
		processPrintActionChange();
	}

	public void removePrintPreviewActionEnablingListener(IPrintActionEnablingListener listener) {
		printPreviewEnablingListener = null;
	}

	public void addPrintPreviewActionEnablingListener(IPrintActionEnablingListener listener) {
		printPreviewEnablingListener = listener;
		processPrintActionChange();
	}

	public void removePageSetupActionEnablingListener(IPrintActionEnablingListener listener) {
		pageSetupEnablingListener = null;
	}

	public void addPageSetupActionEnablingListener(IPrintActionEnablingListener listener) {
		pageSetupEnablingListener = listener;
		processPrintActionChange();
	}

	public void setQueryModel(IQueryTool queryTool) {
		if (this.qt!=null) {
			printAction.removePropertyChangeListener(printActionListener);
			printAction = null;
		}
		this.qt = queryTool;
		printAction = queryTool.getQueryAction(IQueryTool.PRINT_ACTION);
		printAction.addPropertyChangeListener(printActionListener);
	}

	private void processPrintActionChange() {
		if (printEnablingListener!=null) {
			printEnablingListener.enable(new PrintActionEnablingEvent(this, printAction.isEnabled(),printHandler));
		}
		
		if (printPreviewEnablingListener!=null) {
			printPreviewEnablingListener.enable(new PrintActionEnablingEvent(this, printAction.isEnabled(),printPreviewHandler));
		}
		
		if (pageSetupEnablingListener!=null) {
			pageSetupEnablingListener.enable(new PrintActionEnablingEvent(this, printAction.isEnabled(),pageSetupHandler));
		}
	}

	public void doPrint() {
		try {
			printHandler.doAction();
		}
		catch (PrintHandlerException e) {
			runtime.getLogger(this.getClass()).error("Error generating print.", e);
		}
	}

	public void setTable(JTable detailTable) {
		this.detailTable = detailTable;
	}

	public void setDateTimeString(String string) {
		this.dateTimeString = string;
	}

	public void setHTMLModel(IHTMLSelectionModel htmlSelectionModel) {
		this.queryConstraints = htmlSelectionModel;
	}
	
}
