package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.DateRange;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;
import gov.va.med.hds.util.collections.ListUtil;
import gov.va.med.hds.util.dao.IServerDateDAO;
import gov.va.med.hds.util.date.RelativeDate;
import gov.va.med.hds.util.mumps.StringUtil;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;

/**
 * Clinic appointments editor panel.
 */
public class ClinApptPanel extends JPanel implements IRunnableEditorPanel,IModelConfigurable,IConvertible {
	private static final String CLINIC_DELIMETER = "c:";
	private SingleBucketPanel bucketPanel = new SingleBucketPanel();
	private JTextField dateField = new JTextField();
	private JButton dateButton = new JButton("...");
	private String dateRange = "T:T";
	private MultiConverter converter;
	private IRuntimeContext runtime;
	private Date refDate;
	private IDaoRegistry daoRegistry;

	/**
	 * Constructor for ClinApptPanel.
	 */
	public ClinApptPanel() {
		super();
		dateButton.getAccessibleContext().setAccessibleName("Show Date Panel");
		dateField.setEditable(false);
		dateField.setPreferredSize(new Dimension(300,22));
		dateField.setOpaque(true);
		dateField.setBackground(Color.WHITE);
		dateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireDateButtonPressed();
			}
		});
		dateButton.setPreferredSize(new Dimension(22,22));
		this.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints(0,0,2,1,1.0,1.0,GridBagConstraints.WEST,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0);
		this.add(bucketPanel,c);
		c.gridy = 1;
		c.fill = GridBagConstraints.NONE;
		c.gridheight = 1;
		c.gridwidth = 1;
		c.weightx = 0.0;
		c.weighty = 0.0;
		this.add(Box.createVerticalStrut(8),c);
		c.gridy = 2;
		this.add(dateField,c);
		c.gridx = 1;
		this.add(dateButton,c);
	}

	public ClinApptPanel (IBufferedListFactory bufferFactory,IReferenceDAOFactory refFactory) {
		this();
		bucketPanel.setBufferedListFactory(bufferFactory);
		bucketPanel.setReferenceDAOFactory(refFactory);
	}

	private void fireDateButtonPressed() {
		LinkedList list = new LinkedList();
		list.add(new DateRange(refDate, dateRange));
		DateRangePanel dateRangePanel = new DateRangePanel(refDate);
		dateRangePanel.setValues(list);
		dateRangePanel.init();
		JOptionPane optionPane = new JOptionPane(dateRangePanel,JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
		JDialog dialog = optionPane.createDialog(this, "Date Range");
		
		dialog.show();
		
		//		save values in query
		Object result = optionPane.getValue();
		if ((result != null) && (((Integer)result).intValue()==JOptionPane.OK_OPTION)) {
			List dateValues = dateRangePanel.getValues();
			
			if (!ListUtil.equals(list, dateValues)) {
				String oldDateRange = dateRange;
				dateRange = dateValues.get(0).toString();
				dateField.setText(RelativeDate.getDateRangeDisplayText(refDate,dateRange));
				setOutBucketDateRanges(oldDateRange,dateRange);
			}
		}
	}
	
	private void setOutBucketDateRanges(String oldDateRange,String dateRange){
		ListModel outBucketModel= bucketPanel.getOutBucketModel();
		for(int i=0;i<outBucketModel.getSize();i++){
			MultiSourceItem msi = (MultiSourceItem)outBucketModel.getElementAt(i);
			if(msi.getKey().startsWith(ClinApptPanel.CLINIC_DELIMETER)){
				String key = msi.getKey();
				msi.setKey(StringUtil.piece(key,':',1) + ":" + StringUtil.piece(key,':',2) + ":" + dateRange);
			}
		}
		converter.setValue(dateRange);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("ClinApptPanel");
		ClinApptPanel clinApptPanel = new ClinApptPanel();
		frame.getContentPane().add(clinApptPanel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	
	}
	
	public String getDisplayText() {
		return bucketPanel.getDisplayText();
	}

	public List getValues() {
		return bucketPanel.getValues();
	}

	public void init() throws EditorException {
		setupDates();
		bucketPanel.init();
	}

	private void setupDates() {
		IServerDateDAO serverDateDAO;
		try {
			serverDateDAO = (IServerDateDAO)daoRegistry.getDAO(IServerDateDAO.class.getName());
			refDate = serverDateDAO.getDate();
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
		dateField.setText(RelativeDate.getDateRangeDisplayText(refDate, dateRange));
	}

	public void setDAORegistry(IDaoRegistry daoRegistry) {
		bucketPanel.setDAORegistry(daoRegistry);
		this.daoRegistry = daoRegistry;
	}

	public void setRuntime(IRuntimeContext runtime) throws EditorException {
		bucketPanel.setRuntime(runtime);
		this.runtime = runtime;
	}

	public void setValues(List values) {
		List result = null;
		String relativeDate = null;
		if (values.size()>0) {
			LinkedList newList = new LinkedList();
			Iterator it = values.iterator();
			while (it.hasNext()) {
				String item = (String)it.next();
				newList.add(StringUtil.piece(item,':',1));
				relativeDate = StringUtil.piece(item, ':', 2, StringUtil.length(item, ':'));
			}
			result = newList;
		}
		else {
			result = values;
		}
		if (relativeDate!=null && relativeDate.length()>0) {
			dateRange = relativeDate;
		}
		bucketPanel.setValues(result);
	}

	public void setModel(Object model) {
		bucketPanel.setModel(model);
	}

	public void setConverter(MultiConverter converter) {
		this.converter = converter;
		converter.setValue(dateRange);
		bucketPanel.setConverter(converter);
	}
	
	public void setOutBucketLabel(String text) {
		bucketPanel.setOutBucketLabel(text);
	}
	
	public void setInBucketLabel(String text) {
		bucketPanel.setInBucketLabel(text);
	}

	public String getWindowHeader() {
		return "Clinic Appointments";
	}
	
	public void setOutBucketCellRenderer(ListCellRenderer renderer) {
		bucketPanel.setOutBucketCellRenderer(renderer);
	}

	public void clearModels() {
		setModel(null);
	}

	public IRunnableEditorState getEditorState() {return null;}
	public void setEditorState(IRunnableEditorState state) {}
	public void addEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {}
	public void removeEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {}
	public void notifyEditorStateChanged(RunnableEditorStateChangedEvent e) {}

}
