/*
 * Initially Created on Jun 20, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.spectrum.bucket.BasicSlushBucket;
import gov.va.med.hds.cd.spectrum.bucket.ISlushBucket;
import gov.va.med.hds.cd.spectrum.list.AssertEvent;
import gov.va.med.hds.cd.spectrum.list.AssertList;
import gov.va.med.hds.cd.spectrum.list.IAssertListener;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.BufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.ILongListComponentListener;
import gov.va.med.hds.cd.spectrum.longlist.ILongListEvent;
import gov.va.med.hds.cd.spectrum.longlist.ILongListLookup;
import gov.va.med.hds.cd.spectrum.longlist.LongListLookup;
import gov.va.med.hds.querytool.dao.factory.ConsServAllRefFactory;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * Consult service editor; allows selection from a long list component, or ALL using the all services button.
 */
public class ConsultServiceEditor
	extends EditorPanel
	implements IRunnableEditorPanel {

	private static final String message1 = "To select an item, click it in the column on the left, then click '>'.  (You may also double click.)";
	private static final String message2 = "Repeat to add multiple items.";
	private JButton allButton = new JButton("Click to select all consult services");
	private JLabel label1 = new JLabel(message1);
	private JLabel label2 = new JLabel(message2);
	private ISlushBucket slushBucket = new BasicSlushBucket();
	private IBufferedListFactory bufferListFactory;
	private IReferenceDAOFactory refDAOFactory;
	private IReferenceDAOFactory allServicesRefFactory;
	private DefaultListModel outModel = new DefaultListModel();
	private IBufferModel inModel = new BufferModel();
	private AssertList outList = new AssertList();
	private ILongListLookup inList = new LongListLookup();
	private IDataNeededListener dataNeededListener;
	private Object allServicesKey = "";
	private Object allServicesValue = "ALL SERVICES";
	private final Dimension dimension = new Dimension(290,200);
	private boolean allServicesSelected = false;
	
	public ConsultServiceEditor() {
		super();
		this.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.insets = new Insets(0,0,0,0);
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		
		allButton.setMnemonic('C');
		this.add(label1,gbc);
		
		gbc.gridy = 1;
		this.add(label2,gbc);
		
		gbc.gridy = 2;
		this.add(Box.createVerticalStrut(10),gbc);
		
		gbc.gridy = 3;
		this.add(allButton,gbc);
		
		gbc.gridy = 4;
		this.add((JPanel)slushBucket,gbc);
		
		slushBucket.setInBucketLabel("Type Name or Select from List");
		slushBucket.setOutBucketLabel("Selected Consult Services");
		
		JScrollPane outPane = new JScrollPane(outList);
		outList.setModel(outModel);
		slushBucket.setOutBucketComponent(outPane, 'S');
		outList.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if (outList.getSelectedIndex()<0 && outModel.getSize()>0) {
					outList.setSelectedIndex(0);
				}
			}

			public void focusLost(FocusEvent e) {}
		});
		
		inList.setModel(inModel);
		slushBucket.setInBucketComponent((JComponent)inList, 'T');
		
		slushBucket.setSelectAllButtonEnabled(false);
		allButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setAllServices();
			}
		});
		
		slushBucket.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				processBucketButtonPress(e);
			}
		});
		
		outList.setClickCount(2);
		outList.addAssertListener(new IAssertListener() {
			public void valueChanged(AssertEvent e) {
				processBucketButtonPress(new ActionEvent(this, ISlushBucket.REMOVE_ACTION, null));
			}
		});
		
		inList.addLongListListener(new ILongListComponentListener() {
			public void actionPerformed(ILongListEvent e) {
				processBucketButtonPress(new ActionEvent(this, ISlushBucket.SELECT_ACTION, null));
			}
		});

		slushBucket.setPreferredBucketDimension(dimension);
	}

	public void init() throws EditorException {
		try {
			dataNeededListener = bufferListFactory.getBufferModelDataProvider(getRuntime());
			dataNeededListener.setBufferModel(inModel);
			setupAllServicesValue();
			outModel.clear();
			if (super.getValues().size()==1 && super.getValues().get(0).equals(allServicesKey)) {
				setAllServices();
			}
			else {
				Map valueMap = new HashMap();
				for (int i = 0; i < super.getValues().size(); i++) {
					valueMap.put(super.getValues().get(i), null);
				}
				Map results = refDAOFactory.getReferenceDAO(getRuntime(), getDaoRegistry()).get(valueMap);
				for (Iterator i = results.values().iterator(); i.hasNext();) {
					Map.Entry item = (Map.Entry)i.next();
					outModel.addElement(new BufferItem(item.getValue(), item.getKey()));
				}
			}
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		
	}
	
	private void setupAllServicesValue() throws EditorException {
		Map items = new HashMap();
		try {
			allServicesRefFactory.getReferenceDAO(getRuntime(), getDaoRegistry()).get(items);
			if (items.size()==1) {
				allServicesKey = items.keySet().toArray()[0];
				allServicesValue = items.values().toArray()[0];
			}
			else {
				throw new EditorException("The consult services value for the 'All' selection from the database is incorrect.");
			}
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
	}

	public String getWindowHeader() {
		return "Consult Service";
	}

	public void clearModels() {
		inModel.removeDataNeededListener(dataNeededListener);
		dataNeededListener = null;
	}

	public IRunnableEditorState getEditorState() {return null;}
	public void setEditorState(IRunnableEditorState state) {}

	/**
	 * @param bufferedListFactory
	 */
	public void setBufferedListFactory(IBufferedListFactory factory) {
		this.bufferListFactory = factory;
	}

	/**
	 * @param factory
	 */
	public void setReferenceDAOFactory(IReferenceDAOFactory factory) {
		this.refDAOFactory = factory;
	}

	public static final void main(String args[]) {
		try {
			UIManager.setLookAndFeel(new DesktopLookAndFeel(null));
		}
		catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		JFrame frame = new JFrame("ConsultServiceEditor");
		ConsultServiceEditor consultServiceEditor = new ConsultServiceEditor();
		frame.getContentPane().add(consultServiceEditor);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	/**
	 * @param factory
	 */
	public void setAllReferenceDAOFactoy(ConsServAllRefFactory factory) {
		this.allServicesRefFactory = factory;
	}
	
	private void setAllServices() {
		outModel.clear();
		outModel.addElement(new BufferItem(allServicesValue, allServicesKey));
		allServicesSelected = true;
	}

	private void processBucketButtonPress(ActionEvent e) {
		switch (e.getID()) {
			case ISlushBucket.REMOVE_ACTION : {
				allServicesSelected = false;
				if (outList.getSelectedIndices().length > 0) {
					int[] items = outList.getSelectedIndices();
					for (int i = items.length-1; i > -1; i--) {
						outModel.remove(items[i]);
					}
				}
				break;	
			}
			
			case ISlushBucket.REMOVE_ALL_ACTION : {
				allServicesSelected = false;
				outModel.clear();
				break;	
			}
			
			case ISlushBucket.SELECT_ACTION : {
				if (allServicesSelected) {
					outModel.clear();
					allServicesSelected = false;
				}
				if (inList.getSelectedValue()!=null && !outModel.contains(inList.getSelectedValue())) {
					outModel.addElement(inList.getSelectedValue());
				}
				break;	
			}
			
			default : {
			}
		}
	}
	
	public List getValues() {
		super.getValues().clear();
		StringBuffer text = new StringBuffer();
		for (int i = 0; i < outModel.getSize(); i++) {
			IBufferItem item = (IBufferItem)outModel.getElementAt(i);
			super.getValues().add(item.getId());
			if (text.length()>0) {
				text.append(" or ");
			}
			text.append(item.getValue());
		}
		setDisplayText(text.toString());
		return super.getValues();
	}

}
