package gov.va.med.hds.querytool.ui.types;
import gov.va.med.hds.querytool.DateRange;
import gov.va.med.hds.querytool.IDateRange;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.util.date.RelativeDate;
import gov.va.med.hds.util.mumps.StringUtil;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

public class DateRangePanel extends EditorPanel implements ActionListener {
	public static final String BLANK_PANEL = "blankPanel";
	public static final String SPEC_DATES = "specDatesPanel";
	public static final String SPEC_QY = "specQYPanel";
	private static final String TODAY = "T:T";
	private static final String PAST_24 = "T-1:T";
	private static final String PAST_WEEK = "T-7:T";
	private static final String PAST_MONTH = "T-30:T";
	private static final String CURRENT_QUARTER = "YFQ";
	private static final String CURRENT_FISCAL_YEAR = "YF";
	
	private JPanel topPanel = new JPanel();
	private JPanel bottomPanel = new JPanel();
	private JRadioButton todayButton = new JRadioButton();
	private JRadioButton past24Button = new JRadioButton();
	private JRadioButton pastWeekButton = new JRadioButton();
	private JRadioButton pastMonthButton = new JRadioButton();
	private JRadioButton curQButton = new JRadioButton();
	private JRadioButton curFYButton = new JRadioButton();
	private JRadioButton usingDatesButton = new JRadioButton();
	private JRadioButton usingYQButton = new JRadioButton();
	private BorderLayout borderLayout1 = new BorderLayout();
	private SpecDatesPanel specDatesPanel;
	private SpecQYPanel specQYPanel;
	private ButtonGroup buttonGroup = new ButtonGroup();
	private CardLayout cardLayout = new CardLayout();
	private JPanel blankPanel = new JPanel();
	private Date currentYear;
	private Date currentFiscal;
	private GregorianCalendar calendar = new GregorianCalendar();
	private Object currentSelection;
	private boolean displayValueGenerated = false;
	private int curMonth = -1;
	private int curYear = -1;
	private Date referenceDate;

	private DateRangePanel() {
		super();
		initPanel();
	}
	
	public DateRangePanel(Date referenceDate) {
		this();
		this.referenceDate = referenceDate;
		setupDates();
	}
	
	private void initPanel() {
		this.setLayout(borderLayout1);
		topPanel.setBorder(
			new TitledBorder(new EtchedBorder(), "Specify the Date Range"));
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
		bottomPanel.setLayout(cardLayout);
		bottomPanel.add(blankPanel, BLANK_PANEL);
		cardLayout.show(bottomPanel, BLANK_PANEL);
		todayButton.setText("Today");						//T
		past24Button.setText("Past 24 Hours");				//T-1
		pastWeekButton.setText("Past Week");				//T-7
		pastMonthButton.setText("Past Month");				//T-30
		curQButton.setText("Current Quarter");				//YFQ
		curFYButton.setText("Current Fiscal Year");			//YF
		usingDatesButton.setText("Using Specific Dates");	//
		usingYQButton.setText("Using Year or Quarter");		//
		buttonGroup.add(todayButton);
		buttonGroup.add(past24Button);
		buttonGroup.add(pastWeekButton);
		buttonGroup.add(pastMonthButton);
		buttonGroup.add(curQButton);
		buttonGroup.add(curFYButton);
		buttonGroup.add(usingDatesButton);
		buttonGroup.add(usingYQButton);

		todayButton.addActionListener(this);
		past24Button.addActionListener(this);
		pastWeekButton.addActionListener(this);
		pastMonthButton.addActionListener(this);
		curQButton.addActionListener(this);
		curFYButton.addActionListener(this);
		usingDatesButton.addActionListener(this);
		usingYQButton.addActionListener(this);

		topPanel.add(todayButton);
		topPanel.add(past24Button);
		topPanel.add(pastWeekButton);
		topPanel.add(pastMonthButton);
		topPanel.add(curQButton);
		topPanel.add(curFYButton);
		topPanel.add(Box.createVerticalStrut(10));
		topPanel.add(usingDatesButton);
		topPanel.add(usingYQButton);
		this.add(topPanel, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.SOUTH);
	}
	
	/**
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 * Handles the button click events on the main buttons.
	 */
	public void actionPerformed(ActionEvent e) {
		displayValueGenerated = false;
		if (e.getSource() == usingDatesButton) {
			cardLayout.show(bottomPanel, SPEC_DATES);
		}
		else if (e.getSource() == usingYQButton) {
			cardLayout.show(bottomPanel, SPEC_QY);
		}
		else {
			cardLayout.show(bottomPanel, BLANK_PANEL);
		}
		currentSelection = e.getSource();
	}

	public static final void main(String args[]) {
		try {
			JFrame frame = new JFrame("DateRangePanel");
			DateRangePanel dateRangePanel = new DateRangePanel(new Date());
			frame.getContentPane().add(dateRangePanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
			frame.pack();
			frame.show();
	
		}
		catch (Exception ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void init() {
		setupContent();
	}
	
	private void setupDates() {
		currentYear = referenceDate;
		specDatesPanel = new SpecDatesPanel(currentYear);
		bottomPanel.add(specDatesPanel, SPEC_DATES);
		specQYPanel = new SpecQYPanel(currentYear);
		bottomPanel.add(specQYPanel, SPEC_QY);
		currentFiscal = currentYear;
		calendar.setTime(currentYear);
		curYear = calendar.get(GregorianCalendar.YEAR);
		curMonth = calendar.get(GregorianCalendar.MONTH);	//relative to 0 (Jan=0, Dec=11)
		if (curMonth > 8) {
			calendar.add(GregorianCalendar.YEAR,1);
			currentFiscal = calendar.getTime();
		}
	}

	private void setupContent() {
		if (super.getValues().size()>0) {
			String val = ((IDateRange)super.getValues().get(0)).toString();
			if (val.compareTo(TODAY)==0) {todayButton.setSelected(true); currentSelection=todayButton;}
			else if (val.compareTo(PAST_24)==0) {past24Button.setSelected(true);currentSelection=past24Button;}
			else if (val.compareTo(PAST_WEEK)==0) {pastWeekButton.setSelected(true);currentSelection=pastWeekButton;}
			else if (val.compareTo(PAST_MONTH)==0) {pastMonthButton.setSelected(true);currentSelection=pastMonthButton;}
			else if (val.compareTo(CURRENT_QUARTER)==0) {curQButton.setSelected(true);currentSelection=curQButton;}
			else if (val.compareTo(CURRENT_FISCAL_YEAR)==0) {curFYButton.setSelected(true);currentSelection=curFYButton;}
			else if (val.startsWith("Y")) setContentForYear(val);
			else setContentForDates(val);
		}
	}

	private void setContentForYear(String val) {
		usingYQButton.setSelected(true);
		actionPerformed(new ActionEvent(usingYQButton,0,""));
		int offset = 0;
		if (val.indexOf('+')>-1) offset = Integer.parseInt(StringUtil.piece(StringUtil.piece(val,'+',2),'Q',1));
		if (val.indexOf('-')>-1) offset = -Integer.parseInt(StringUtil.piece(StringUtil.piece(val,'-',2),'Q',1));
		if (val.charAt(1)=='C') setCalendarYear(offset);
		if (val.charAt(1)=='F') setFiscalYear(offset);
		String quarterOffsetString = StringUtil.piece(val,'Q',2);
		int qtr = 0;
		if (quarterOffsetString.length()>0) {
			qtr = Integer.parseInt(quarterOffsetString);
		}
		else {
			qtr = 0;
		}
		specQYPanel.enableControls();
		if (qtr == 0) {
			specQYPanel.setForButton(false);
		}
		else {
			specQYPanel.setForButton(true);
			specQYPanel.setQuarter(qtr-1);
		}
	}

	private void setFiscalYear(int offset) {
		specQYPanel.setFiscalYearButton();
		calendar.setTime(currentFiscal);
		calendar.add(GregorianCalendar.YEAR,offset);
		int month = calendar.get(GregorianCalendar.MONTH);
		specQYPanel.setYear(calendar.getTime());
		int qindex = 0;
		switch (month) {
			case 0:  {}
			case 1:  {}
			case 2:  {qindex = 3; break;}
			case 3:  {}
			case 4:  {}
			case 5:  {qindex = 0; break;}
			case 6:  {}
			case 7:  {}
			case 8:  {qindex = 1; break;}
			case 9:  {}
			case 10: {}
			case 11: {qindex = 2; break;}
		}
		specQYPanel.setQuarter(qindex);
	}

	private void setCalendarYear(int offset) {
		specQYPanel.setCalYearButton();
		calendar.setTime(currentYear);
		calendar.add(GregorianCalendar.YEAR,offset);
		int month = calendar.get(GregorianCalendar.MONTH);
		specQYPanel.setYear(calendar.getTime());
		int qindex = 0;
		switch (month) {
			case 0:  {}
			case 1:  {}
			case 2:  {qindex = 0; break;}
			case 3:  {}
			case 4:  {}
			case 5:  {qindex = 1; break;}
			case 6:  {}
			case 7:  {}
			case 8:  {qindex = 2; break;}
			case 9:  {}
			case 10: {}
			case 11: {qindex = 3; break;}
		}
		specQYPanel.setQuarter(qindex);
	}

	private void setContentForDates(String val) {
		usingDatesButton.setSelected(true);
		actionPerformed(new ActionEvent(usingDatesButton,0,""));
		String relativeDateString = StringUtil.piece(val,':',1);
		if (relativeDateString.length()>0) {
			specDatesPanel.setNoStartDateButton(false);
			specDatesPanel.setStartDate(RelativeDate.getRelativeToDate(currentYear, relativeDateString));
		}
		else {
			specDatesPanel.setNoStartDateButton(true);
		}
		
		relativeDateString = StringUtil.piece(val,':',2);
		if (relativeDateString.length()>0) {
			specDatesPanel.setNoStopDateButton(false);
			specDatesPanel.setStopDate(RelativeDate.getRelativeToDate(currentYear, relativeDateString));
		}
		else {
			specDatesPanel.setNoStopDateButton(true);
		}
	}
	
	public List getValues() {
		String val = new String();
		if		(currentSelection==todayButton) {val = TODAY;}
		else if (currentSelection==past24Button) {val = PAST_24;}
		else if (currentSelection==pastWeekButton) {val = PAST_WEEK;}
		else if (currentSelection==pastMonthButton) {val = PAST_MONTH;}
		else if (currentSelection==curQButton) {val = CURRENT_QUARTER;}
		else if (currentSelection==curFYButton) {val = CURRENT_FISCAL_YEAR;}
		else if (currentSelection==usingDatesButton) {val = getDatesValue();}
		else if (currentSelection==usingYQButton) {val = getYQValue();}
		displayValueGenerated = true;
		super.setDisplayText(RelativeDate.getDateRangeDisplayText(currentYear, val));
		super.getValues().clear();
		super.getValues().add(new DateRange(referenceDate,val));
		return super.getValues();
	}

	private String getYQValue() {
		StringBuffer result = new StringBuffer();
		if (specQYPanel.isFiscalYearButtonSet()) {
			result.append("YF");
		}
		else {
			result.append("YC");
		}
		calendar.setTime(specQYPanel.getYear());
		int targetYear = calendar.get(GregorianCalendar.YEAR);
		int thisYear = 0;
		if ((specQYPanel.isFiscalYearButtonSet()) && (curMonth > 8)) {
			thisYear = curYear + 1;
		}
		else {
			thisYear = curYear;
		}
		int offset = targetYear - thisYear;
		if (offset > 0) result.append("+" + Integer.toString(offset));
		if (offset < 0) result.append(Integer.toString(offset));
		if (specQYPanel.isForButtonSet()) result.append("Q" + Integer.toString(specQYPanel.getQuarter() + 1));
		return result.toString();
	}

	private String getDatesValue() {
		//determine beginning date
		String startDateString = "";
		if (!specDatesPanel.isNoStartDateButtonSelected()) {
			startDateString = RelativeDate.getRelativeDateString(currentYear, specDatesPanel.getStartDate());
		}
		
		//determin ending date
		String stopDateString = "";
		if (!specDatesPanel.isNoStopDateButtonSelected()) {
			stopDateString = RelativeDate.getRelativeDateString(currentYear, specDatesPanel.getStopDate());
		}
		
		return startDateString + ":" + stopDateString;
	}

	public String getDisplayText() {
		if (!displayValueGenerated) {
			getValues();
		}
		return super.getDisplayText();
	}

	public String getWindowHeader() {
		return "Date Range";
	}

	public void setValues(List values) {
		super.setValues(values);
		setupContent();
	}

	public void clearModels() {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

}