package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;
import gov.va.med.hds.util.collections.ListUtil;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;

/**
 * @author vhaislmonsos
 *
 * Dec 2, 2002 11:06:31 PM
 */
public abstract class EditorPanel extends JPanel implements IRunnableEditorPanel {
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private List values = new ArrayList();
	private String displayText;
	private LinkedList stateChangeListeners = new LinkedList();

	/**
	 * @see gov.va.med.hds.querytool.ui.types.IRunnableEditorPanel#getValues()
	 */
	public List getValues() {
		return values;
	}
	
	/**
	 * Method setValues.
	 * @param values
	 */
	public void setValues(List values) {
		ListUtil.copy(values, this.values);
//		Collections.copy(this.values, values);
	}

	/**
	 * @see gov.va.med.hds.querytool.ui.types.IRunnableEditorPanel#getDisplayText()
	 */
	public String getDisplayText() {
		return displayText;
	}
	
	protected void setDisplayText(String text) {
		displayText = text;
	}

	/**
	 * @see gov.va.med.hds.querytool.ui.types.IRunnableEditorPanel#setRuntime(IRuntimeContext)
	 */
	public void setRuntime(IRuntimeContext runtime) throws EditorException {
		this.runtime = runtime;
		if (daoRegistry == null) {
			try {
				daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
			}
			catch (Exception e) {
				throw new EditorException(e);
			}
		}
//		init();
	}

	public abstract void init() throws EditorException;

	/**
	 * Returns the daoRegistry.
	 * @return IDaoRegistry
	 */
	protected IDaoRegistry getDaoRegistry() {
		return daoRegistry;
	}

	/**
	 * Returns the runtime.
	 * @return IRuntimeContext
	 */
	protected IRuntimeContext getRuntime() {
		return runtime;
	}

	/**
	 * @see gov.va.med.hds.querytool.ui.types.IRunnableEditorPanel#setDAORegistry(IDaoRegistry)
	 */
	public void setDAORegistry(IDaoRegistry daoRegistry) {
		this.daoRegistry = daoRegistry;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#addEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void addEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
		if(listener != null)
			stateChangeListeners.add(listener);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#removeEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void removeEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
		stateChangeListeners.remove(listener);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#notifyEditorStateChanged(gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent)
	 */
	public void notifyEditorStateChanged(RunnableEditorStateChangedEvent e) {
		Iterator it = stateChangeListeners.iterator();
		while(it.hasNext()){
			IRunnableEditorStateChangeListener listener = (IRunnableEditorStateChangeListener)it.next();
			listener.stateChanged(e);
		}
	}

}
