package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;
import gov.va.med.hds.util.mumps.StringUtil;

import java.awt.BorderLayout;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.MutableComboBoxModel;
import javax.swing.SingleSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Combines editors on a page, managed by a tabbed panel for the user.
 */
public class MultiSourceEditor extends EditorPanel {
	private JPanel labelPanel = new JPanel();
	private JLabel label1 = new JLabel("Click on the tabs to view different types of lists.  Multiple lists may be selected.");
	private JLabel label2 = new JLabel("To select lists, click items in the left box, then click '>'.  (You may also double-click to add.)");
	private JTabbedPane tabPane = new JTabbedPane(JTabbedPane.TOP);
	private Map sourceItemMap = new HashMap();
	private MutableComboBoxModel outModel = new DefaultComboBoxModel();
	private String displayText;
	private char separator = ':';
	private MultiSourceEditorState editorState = new MultiSourceEditorState(0);
	
	public MultiSourceEditor() throws EditorException {
		super();
		this.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY,"gov.va.med.hds.querytool.dpl");
		this.setLayout(new BorderLayout());
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
		labelPanel.add(label1);
		labelPanel.add(label2);
		this.add(labelPanel,BorderLayout.NORTH);
		this.add(tabPane,BorderLayout.CENTER);
		tabPane.getModel().addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				SingleSelectionModel model = (SingleSelectionModel)e.getSource();
				editorState.setActiveTabNumber(model.getSelectedIndex());
				notifyEditorStateChanged(new RunnableEditorStateChangedEvent(this,editorState));
			}
		});
	}

	public void addPanel(MultiSourceEditItem item) {
		tabPane.add(item.getTabString(),(JPanel)item.getDisplayPanel());
		sourceItemMap.put(item.getTypeID(),item);
	}
	
	public void init() throws EditorException {
		initReferences();
		initPanels();
	}
	
	private void initPanels() throws EditorException {
		MultiConverter converter = null;
		Iterator it = sourceItemMap.keySet().iterator();
		while (it.hasNext()) {
			IMultiSourceEditItem sourceItem = (IMultiSourceEditItem)sourceItemMap.get(it.next());
			if (sourceItem != null) {
				IRunnableEditorPanel panel = sourceItem.getDisplayPanel();
				((IModelConfigurable)panel).setModel(outModel);
				converter = new MultiConverter(sourceItem.getTypeID(),sourceItem.getDescr());
				((IConvertible)panel).setConverter(converter);
				sourceItem.getDisplayPanel().init();
			}
		}
	}

	private void initReferences() throws EditorException {
		Iterator it = super.getValues().iterator();
		String p1 = null;
		String p2 = null;
		String value = null;
		Map valueMapMap = new HashMap();
		Map valueMap = null;
		//Create maps of the values to pass into the reference DAO's
		while (it.hasNext()) {
			value = (String)it.next();
			p1 = StringUtil.piece(value,separator,1);
			p2 = StringUtil.piece(value,separator,2);
			if (!valueMapMap.containsKey(p1)) {
				valueMapMap.put(p1,new HashMap());
			}
			valueMap = (Map) valueMapMap.get(p1);
			valueMap.put(p2,value);
		}
		it = valueMapMap.keySet().iterator();
		Iterator it2 = null;
		LinkedList valueList = null;
		while (it.hasNext()) {
			p1 = (String)it.next();
			valueMap = (Map)valueMapMap.get(p1);
			it2 = valueMap.keySet().iterator();
			valueList = new LinkedList();
			while (it2.hasNext()) {
				String item = (String)valueMap.get(it2.next());
				String fullItem = StringUtil.piece(item,separator,2,StringUtil.length(item, separator));
				if ((StringUtil.piece(fullItem, separator, 2).length()<1) && (StringUtil.length(fullItem, separator)<3)) {
					valueList.add(StringUtil.piece(fullItem, separator, 1));
				}
				else {
					valueList.add(fullItem);
				}
			}
			IMultiSourceEditItem sourceItem = (IMultiSourceEditItem)sourceItemMap.get(p1);
			if (sourceItem != null) {
				sourceItem.getDisplayPanel().setValues(valueList);
			}
		}
	}
	
	public List getValues() {
		//Source: name and Source: name and Source:name
		List result = new LinkedList();
		StringBuffer text = new StringBuffer();
		for (int i=0;i<outModel.getSize();i++) {
			MultiSourceItem item = (MultiSourceItem)outModel.getElementAt(i);
			if (text.length()>0) {
				text.append(" and ");
			}
			text.append(item.getItemDescr());
			text.append(": ");
			text.append(item.getDescr());
			result.add(item.getKey());
		}
		displayText = text.toString();
		return result;
	}

	public String getDisplayText() {
		if ((displayText == null) || (displayText.length()<1)) {
			getValues();
		}
		return displayText;
	}

	public String getWindowHeader() {
		return "Patients";
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
		for (Iterator i = sourceItemMap.values().iterator(); i.hasNext();) {
			IMultiSourceEditItem item = (IMultiSourceEditItem)i.next();
			item.getDisplayPanel().clearModels();
		}
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return editorState;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {
		if(state != null && state instanceof MultiSourceEditorState){
			editorState = (MultiSourceEditorState)state;
			tabPane.setSelectedIndex(editorState.getActiveTabNumber());
		}
	}
	
}
