package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.spectrum.bucket.ShortListBucket;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.querytool.dao.IRegistryRefDAO;
import gov.va.med.hds.querytool.dao.QueryDataDAOException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;
import gov.va.med.hds.util.mumps.StringUtil;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.MutableComboBoxModel;

/**
 * Slush bucket that lists registries, and allows selection of a registry and whether
 * registry items should be active, inactive, or both.
 */
public class RegistryBucketPanel extends JPanel implements IRunnableEditorPanel, IModelConfigurable, IConvertible {

	public final String REGISTRY_DELIMETER = "r:";

	private ShortListBucket bucket = new ShortListBucket();
	private RegistrySelectionPanel registrySelectionPanel = new RegistrySelectionPanel();
	private IDaoRegistry daoRegistry;
	private IRuntimeContext runtime;
	private MultiConverter converter;
	private IRegistryRefDAO registryRefDAO;
	private List values = new LinkedList();
	private String displayText;
	private DefaultListModel inModel = new DefaultListModel();
	private MutableComboBoxModel outModel = new DefaultComboBoxModel();
	private static final char DELIMETER_SEPARATOR = ':';
	private int currentRegistryProperty;

	public RegistryBucketPanel() {
		super();
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(bucket);
		this.add(registrySelectionPanel);
		bucket.setOutBucketModel(outModel);
		bucket.setInBucketModel(inModel);
		currentRegistryProperty = registrySelectionPanel.getRegistrySelection();

		registrySelectionPanel.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (e.getPropertyName().equals(RegistrySelectionPanel.BUTTON_SELECTED)) {
					setRegistryPropertyOnOutModel(currentRegistryProperty, registrySelectionPanel.getRegistrySelection());
					currentRegistryProperty = registrySelectionPanel.getRegistrySelection();
				}
			}
		});
	}
	
	private void setRegistryPropertyOnOutModel(int oldProperty, int newProperty) {
		for (int i = 0; i < outModel.getSize(); i++) {
			MultiSourceItem msi = (MultiSourceItem) outModel.getElementAt(i);
			if (msi.getKey().startsWith(REGISTRY_DELIMETER)) {
				msi.setKey(msi.getKey().replaceAll(DELIMETER_SEPARATOR+Integer.toString(oldProperty),DELIMETER_SEPARATOR+Integer.toString(newProperty)));
			}
		}
		converter.setValue(Integer.toString(newProperty));
	}

	public List getValues() {
		values.clear();
		StringBuffer text = new StringBuffer();
		if ((outModel != null) && (outModel.getSize() > 0)) {
			for (int i = 0; i < outModel.getSize(); i++) {
				IBufferItem item = (BufferItem) outModel.getElementAt(i);
				values.add(item.getId());
				if (text.length() > 0) {
					text.append(" or ");
				}
				text.append(item.getValue());
			}
		}
		displayText = text.toString();
		return values;
	}

	public String getDisplayText() {
		if ((displayText == null) || (displayText.length() < 1)) {
			getValues();
			return displayText;
		} else {
			return displayText;
		}
	}

	public void setRuntime(IRuntimeContext runtime) throws PluginStartupException, EditorException {
		this.runtime = runtime;
	}

	public void setDAORegistry(IDaoRegistry daoRegistry) {
		this.daoRegistry = daoRegistry;
	}

	public void init() throws EditorException {
		if (daoRegistry == null) {
			try {
				setupDAO();
			} catch (Exception e) {
				throw new EditorException(e);
			}
		}
		try {
			registryRefDAO = (IRegistryRefDAO) daoRegistry.getDAO(IRegistryRefDAO.class.getName());
		} catch (Exception e) {
			throw new EditorException(e);
		}
		Map valueMap = new HashMap();
		for (Iterator i = values.iterator(); i.hasNext();) {
			valueMap.put(i.next(), null);
		}
		TreeMap sorted;
		try {
			sorted = registryRefDAO.get(valueMap);
		} catch (QueryDataDAOException e) {
			throw new EditorException(e);
		}

		for (Iterator i = values.iterator(); i.hasNext();) {
			String key = (String) i.next();
			String p1 = StringUtil.piece(key, DELIMETER_SEPARATOR, 1);
			String p2 = StringUtil.piece(key, DELIMETER_SEPARATOR, 2);
			String descr = null;
			if ((descr = referenceContains(sorted, p1)) != null) {
				converter.setValue(p2);
				outModel.addElement(convert(new BufferItem(descr, key)));
			}
		}
		for (Iterator i = sorted.values().iterator(); i.hasNext();) {
			Map.Entry item = (Map.Entry) i.next();
			IBufferItem bufferItem = new BufferItem(item.getValue().toString(), item.getKey().toString());
			inModel.addElement(bufferItem);
		}
		converter.setValue(Integer.toString(registrySelectionPanel.getRegistrySelection()));
	}

	private String referenceContains(Map sorted, String key) {
		for (Iterator i = sorted.values().iterator(); i.hasNext();) {
			Map.Entry item = (Map.Entry) i.next();
			if (item.getKey().equals(key)) {
				return item.getValue().toString();
			}
		}
		return null;
	}

	private Object convert(Object item) {
		if (bucket.getConverter() != null) {
			return (bucket.getConverter().convert(item));
		} else {
			return item;
		}
	}

	private void setupDAO() throws PluginNotFoundException, PluginStartupException {
		daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
	}

	public void setValues(List values) {
		this.values.addAll(values);
	}

	public String getWindowHeader() {
		return "Patient Registries";
	}

	public void setModel(Object model) {
		bucket.setOutBucketModel(model);
		outModel = (MutableComboBoxModel) model;
	}

	public void setConverter(MultiConverter converter) {
		bucket.setConverter(converter);
		this.converter = converter;
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("RegistryBucketPanel");
		RegistryBucketPanel registryBucketPanel = new RegistryBucketPanel();
		frame.getContentPane().add(registryBucketPanel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	public void setInBucketLabel(String inLabel) {
		bucket.setInBucketLabelText(inLabel);
	}

	public void setOutBucketLabel(String outLabel) {
		bucket.setOutBucketLabelText(outLabel);
	}

	public void setOutBucketCellRenderer(ListCellRenderer renderer) {
		bucket.setOutBucketCellRenderer(renderer);
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
		bucket.setInBucketModel(null);
		bucket.setOutBucketModel(null);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#addEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void addEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#removeEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void removeEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#notifyEditorStateChanged(gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent)
	 */
	public void notifyEditorStateChanged(RunnableEditorStateChangedEvent e) {
	}

}
