package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.spectrum.bucket.ShortListBucket;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.querytool.dao.INameValuePairDAO;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.MutableComboBoxModel;

/**
 * A ShortListBucket that is populated from an IReferenceDAOFactory.
 */
public class ShortBucketPanel extends ShortListBucket
	implements IRunnableEditorPanel, IConvertible, IModelConfigurable {
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private IReferenceDAOFactory refFactory;
	private List values = new LinkedList();
	private String displayText;
	private String windowHeader;

	public ShortBucketPanel() {
		super();
		setInBucketLabelText("Type Name or Select From List");
	}

	public ShortBucketPanel(IReferenceDAOFactory refFactory) {
		this.refFactory = refFactory;
	}

	public List getValues() {
		values.clear();
		ListModel model = (ListModel)getOutBucketModel();
		StringBuffer text = new StringBuffer();
		for (int i = 0; i < model.getSize(); i++) {
			IBufferItem item = (IBufferItem)model.getElementAt(i);
			values.add(item.getId());
			if (text.length()>0) {
				text.append(" or ");
			}
			text.append(item.getValue());
		}
		displayText = text.toString();
		return values;
	}

	public String getDisplayText() {
		if ((displayText==null) || (displayText.length()<1)) {
			getValues();
		}
		return displayText;
	}

	public void setRuntime(IRuntimeContext runtime)
		throws PluginStartupException, EditorException {
		this.runtime = runtime;
	}

	public void setDAORegistry(IDaoRegistry daoRegistry) {
		this.daoRegistry = daoRegistry;
	}

	public void init() throws EditorException {
		if (values==null) {
			values = new LinkedList();
		}
		Map items = new HashMap();
		TreeMap sortedItems = null;
		for (Iterator i = values.iterator(); i.hasNext();) {
			Object item = i.next();
			items.put(item,"");
		}
		try {
			INameValuePairDAO dao = refFactory.getReferenceDAO(runtime,daoRegistry);
			sortedItems = dao.get(items);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		DefaultListModel inModel = new DefaultListModel();
		IBufferItem item = null;
		for (Iterator i = sortedItems.values().iterator(); i.hasNext();) {
			Map.Entry e = (Map.Entry)i.next();
			String key = e.getKey().toString();
			String value = e.getValue().toString();
			if (value != null) {
				item = new BufferItem(value,key);
				inModel.addElement(item);
				if (itemInValues(item)) {
					((MutableComboBoxModel)this.getOutBucketModel()).addElement(convert(item));
				}
			}
		}
		setInBucketModel(inModel);
	}

	private boolean itemInValues(IBufferItem item) {
		boolean result = false;
		for (Iterator iter = values.iterator(); iter.hasNext();) {
			Object nextItem = iter.next();
			if (((String)nextItem).equals(item.getId())) {
				result = true;
				break;
			}
		}
		return result;
	}

	public void setValues(List values) {
		this.values.addAll(values);
	}

	public void setConverter(MultiConverter converter) {
		super.setConverter(converter);
	}

	public void setModel(Object model) {
		this.setOutBucketModel(model);
	}

	public String getWindowHeader() {
		return windowHeader;
	}

	public void setWindowHeader(String text) {
		windowHeader = text;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
		setInBucketModel(null);
		setOutBucketModel(null);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#addEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void addEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#removeEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void removeEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#notifyEditorStateChanged(gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent)
	 */
	public void notifyEditorStateChanged(RunnableEditorStateChangedEvent e) {
	}

}
