package gov.va.med.hds.querytool.ui.types;
import gov.va.med.hds.querytool.dao.INameValuePairDAO;
import gov.va.med.hds.querytool.dao.QueryDataDAOException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * A vertical checkbox list populated from an IShortListStrategy.
 */
public class ShortChecklistPanel extends EditorPanel {
	private CheckBoxSelection checkBoxSelectionPanel = new CheckBoxSelection();
	private Map items = new HashMap();
	private INameValuePairDAO nameValuePairDAO;
	private IShortListStrategy interfaceProvider;
	private String windowHeader;
	
	public ShortChecklistPanel() {
		add(checkBoxSelectionPanel);
	}

	public ShortChecklistPanel(IShortListStrategy interfaceProvider) {
		this();
		this.interfaceProvider = interfaceProvider;
	}
	
	public void init() throws EditorException {
		nameValuePairDAO = interfaceProvider.getDAO(super.getDaoRegistry());
		try {
			checkBoxSelectionPanel.setItems(nameValuePairDAO.get(items));
		}
		catch (QueryDataDAOException e) {
			throw new EditorException(e);
		}
		setupCheckedItems();
	}

	private void setupCheckedItems() {
		checkBoxSelectionPanel.clearSelection();
		Iterator it = super.getValues().iterator();
		while (it.hasNext()) {
			checkBoxSelectionPanel.setSelected((String)it.next(),true);
		}
	}

	public String getDisplayText() {
		StringBuffer result = new StringBuffer();
		List selected = checkBoxSelectionPanel.getSelectedItems();
		Iterator it = selected.iterator();
		boolean first = true;
		while (it.hasNext()) {
			if (first) {
				first = false;
			}
			else {
				result.append(" or ");
			}
			String key = (String)it.next();
			result.append((String)items.get(key));
		}
		super.setDisplayText(result.toString());
		
		return result.toString();
	}
	
	public List getValues() {
		super.setValues(checkBoxSelectionPanel.getSelectedItems());
		return super.getValues();
	}

	public void setValues(List values) {
		super.setValues(values);
		if (items.size()>0) {
			setupCheckedItems();
		}
	}

	public String getWindowHeader() {
		return windowHeader;
	}

	public void setWindowHeader(String text) {
		windowHeader = text;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

}
