package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.spectrum.bucket.BufferedSlushBucket;
import gov.va.med.hds.cd.spectrum.bucket.IConverter;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.BufferModel;
import gov.va.med.hds.cd.spectrum.longlist.DataRequestFactory;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.querytool.dao.INameValuePairDAO;
import gov.va.med.hds.querytool.dao.QueryDataDAOException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.ListSelectionModel;
import javax.swing.MutableComboBoxModel;

/**
 * Container for a BufferedSlushBucket, a filter combo box, and instruction text.
 */
public class SingleBucketFilterEditor extends EditorPanel {
	private IBufferedListFactory bufferedListStrategy;
	private BufferedSlushBucket slushBucket = new BufferedSlushBucket();
	private String text21 = "Repeat to add multiple items.  ";
	private String text22;
	private JLabel label1 = new JLabel("To select an item, click it in the column on the left, then click '>'. (You may also double-click.)");
	private JLabel label2 = new JLabel(text21);
	private MutableComboBoxModel outBucketModel = new DefaultComboBoxModel();
	private INameValuePairDAO referenceDAO;
	private INameValuePairDAO filterDAO;
	private IReferenceDAOFactory referenceDAOFactory;
	private IReferenceDAOFactory filterDAOFactory;
	private FilterPanel filterPanel = new FilterPanel();
	private IDataNeededListener dataProvider;
	private Map filterMap = new HashMap();
	private IConverter converter;
	private String windowHeader;

	
	private SingleBucketFilterEditor() {
		super();
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints(0,0,1,1,0.0,0.0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0);
		slushBucket.setInBucketLabelText("Type Name or Select From List");
		slushBucket.setOutBucketLabelText("Selected Providers");
		slushBucket.setOutBucketModel(outBucketModel);
		slushBucket.setOutBucketSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		filterPanel.addSelectActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (e.getID()==ActionEvent.ACTION_PERFORMED) {
					fireFilterChanged();
				}
			}
		});
		this.add(label1,c);
		c.gridy = 1;
		this.add(label2,c);
		c.gridy = 2;
		this.add(Box.createVerticalStrut(10),c);
		c.gridy = 3;
		this.add(filterPanel,c);
		c.gridy = 4;
		c.fill = GridBagConstraints.CENTER;
		this.add(slushBucket,c);
	}
	
	private void fireFilterChanged() {
		dataProvider.setFilter(((IBufferItem)filterPanel.getSelectedValue()).getId());
		((IBufferModel)slushBucket.getInBucketModel()).getDataRequest(DataRequestFactory.reset());
		while (!((IBufferModel)slushBucket.getInBucketModel()).getDataRequest(DataRequestFactory.getTop())) {}
	}

	/**
	 * Constructor SingleSlushSelector.
	 * @param clinicianListStrategy
	 * @param list
	 */
	public SingleBucketFilterEditor(IBufferedListFactory bufferedListStrategy,IReferenceDAOFactory referenceDAOFactory, IReferenceDAOFactory filterDAOFactory, boolean noFilterAllowed) {
		this();
		this.bufferedListStrategy = bufferedListStrategy;
		this.referenceDAOFactory = referenceDAOFactory;
		this.filterDAOFactory = filterDAOFactory;
	}
	
	public void init() throws EditorException {
		try {
			dataProvider = bufferedListStrategy.getBufferModelDataProvider(this.getRuntime());
			referenceDAO = referenceDAOFactory.getReferenceDAO(getRuntime(),getDaoRegistry());
			filterDAO = filterDAOFactory.getReferenceDAO(getRuntime(),getDaoRegistry());
			setupOutBucketValues();
			filterPanel.setValues(filterDAO.get(filterMap));
			dataProvider.setFilter(((IBufferItem)filterPanel.getSelectedValue()).getId());
			IBufferModel bufferModel = new BufferModel(300);
			dataProvider.setBufferModel(bufferModel);
			slushBucket.setInBucketModel(bufferModel);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
	}
	
	private void setupOutBucketValues() throws QueryDataDAOException {
		Map items = new HashMap();
		for (Iterator i = super.getValues().iterator(); i.hasNext();) {
			Object item = i.next();
			items.put(item,item);
		}
		TreeMap sortedItems = referenceDAO.get(items);
		for (Iterator i = sortedItems.values().iterator(); i.hasNext();) {
			Map.Entry e = (Map.Entry)i.next();
			outBucketModel.addElement(convert(new BufferItem(e.getValue().toString(),e.getKey().toString())));
		}
	}
	
	private Object convert(Object item) {
		if (converter!=null) {
			return converter.convert(item);
		}
		else {
			return item;
		}
	}

	/**
	 * Sets the converter.
	 * @param converter The converter to set
	 */
	public void setConverter(IConverter converter) {
		this.converter = converter;
		slushBucket.setConverter(converter);
	}

	public void setFilterInstruction(String text) {
		this.text22 = text;
		label2.setText(text21 + text22);
	}
	
	public void setFilterPrompt(String text) {
		filterPanel.setLabelText(text);
	}
	
	public String getWindowHeader() {
		return windowHeader;
	}

	public void setWindowHeader(String text) {
		windowHeader = text;
	}
	
	public void setOutBucketLabel(String text) {
		slushBucket.setOutBucketLabelText(text);
	}

	public String getDisplayText() {
		return super.getDisplayText();
	}

	public List getValues() {
		super.getValues().clear();
		String text = new String("");
		for (int i = 0; i < outBucketModel.getSize(); i++) {
			super.getValues().add(((IBufferItem)outBucketModel.getElementAt(i)).getId());
			if (text.length()>0) {
				text += " and ";
			}
			text += ((IBufferItem)outBucketModel.getElementAt(i)).getValue();
		}
		
		if (text.length()>0) {
			super.setDisplayText(text);
		}
		
		return super.getValues();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
		slushBucket.setInBucketModel(null);
		slushBucket.setOutBucketModel(null);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

}
