package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.bucket.BufferedSlushBucket;
import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.BufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.querytool.dao.INameValuePairDAO;
import gov.va.med.hds.querytool.dao.QueryDataDAOException;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener;
import gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent;

import java.awt.BorderLayout;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.MutableComboBoxModel;

/**
 * Container for a BufferedSlushBucket which maintains a ListModel for the out bucket.
 */
public class SingleBucketPanel extends JPanel implements IRunnableEditorPanel,IModelConfigurable,IConvertible {
	private IBufferedListFactory bufferedListStrategy;
	private IRuntimeContext runtime;
	private IDaoRegistry daoRegistry;
	private BufferedSlushBucket slushBucket = new BufferedSlushBucket();
	private MutableComboBoxModel outBucketModel = new DefaultComboBoxModel();
	private INameValuePairDAO referenceDAO;
	private IReferenceDAOFactory referenceDAOFactory;
	private List values;
	private IDataNeededListener dataProvider;
	private MultiConverter converter;
	private String displayText;
	private String windowHeader;

	
	public SingleBucketPanel() {
		super();
		setLayout(new BorderLayout());
		slushBucket.setInBucketLabelText("Type Name or Select From List" );
		slushBucket.setOutBucketLabelText("====");
		slushBucket.setOutBucketModel(outBucketModel);
		slushBucket.setOutBucketSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		this.add(slushBucket,BorderLayout.CENTER);
	}

	/**
	 * Constructor SingleSlushSelector.
	 * @param clinicianListStrategy
	 * @param list
	 */
	public SingleBucketPanel(IBufferedListFactory bufferedListStrategy, IReferenceDAOFactory referenceDAOFactory ,List values) {
		this();
		this.bufferedListStrategy = bufferedListStrategy;
		this.referenceDAOFactory = referenceDAOFactory;
		this.values = values;
	}

	/**
	 * Constructor SingleBucketPanel.
	 * @param clinicianListFactory
	 * @param clinicianRefFactory
	 */
	public SingleBucketPanel(IBufferedListFactory bufferFactory,IReferenceDAOFactory refFactory) {
		this();
		this.bufferedListStrategy = bufferFactory;
		this.referenceDAOFactory = refFactory;
	}

	
	public void init() throws EditorException {
		try {
			dataProvider = bufferedListStrategy.getBufferModelDataProvider(runtime);
			referenceDAO = referenceDAOFactory.getReferenceDAO(runtime,daoRegistry);
			setupInitValues();
			IBufferModel bufferModel = new BufferModel(300);
			dataProvider.setBufferModel(bufferModel);
			slushBucket.setInBucketModel(bufferModel);
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
	}

	private void setupInitValues() throws QueryDataDAOException {
		if (values == null) {
			values = new LinkedList();
		}
		Iterator it = values.iterator();
		Map valueItems = new HashMap();
		while (it.hasNext()) {
			Object item = it.next();
			valueItems.put(item,item);
		}
		TreeMap sortMap = referenceDAO.get(valueItems);
		for (Iterator i = sortMap.entrySet().iterator(); i.hasNext();) {
			Map.Entry e = (Map.Entry)((Map.Entry)i.next()).getValue();
			outBucketModel.addElement(convert(new BufferItem(e.getValue().toString(),e.getKey().toString())));
		}
	}
	
	private Object convert(Object item) {
		if (converter != null) {
			return converter.convert(item);
		}
		else {
			return item;
		}
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("SingleBucketPanel");
		SingleBucketPanel bucket = new SingleBucketPanel();
		bucket.setInBucketLabel("Hello");
		bucket.setOutBucketLabel("There");
		frame.getContentPane().add(bucket, BorderLayout.CENTER);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();

	}
	
	public void setConverter(MultiConverter converter) {
		slushBucket.setConverter(converter);
		this.converter = converter;
	}

	public void setBufferedListFactory(IBufferedListFactory bufferedListFactory) {
		this.bufferedListStrategy = bufferedListFactory;
	}

	public List getValues() {
		List result = new LinkedList();
		StringBuffer text = new StringBuffer();
		for (int i = 0; i < outBucketModel.getSize(); i++) {
			IBufferItem item = (IBufferItem)outBucketModel.getElementAt(i);
			result.add(item.getId());
			if (text.length()>0) {
				text.append(" or ");
			}
			text.append(item.getValue());
		}
		displayText = text.toString();
		return result;
	}

	public void setDAORegistry(IDaoRegistry daoRegistry) {
		this.daoRegistry = daoRegistry;
	}

	public void setReferenceDAOFactory(IReferenceDAOFactory referenceDAOFactory) {
		this.referenceDAOFactory = referenceDAOFactory;
	}

	public void setRuntime(IRuntimeContext runtime) {
		this.runtime = runtime;
	}

	public void setValues(List values) {
		this.values = values;
	}

	public void setInBucketLabel(String label) {
		slushBucket.setInBucketLabelText(label);
	}
	
	public void setOutBucketLabel(String label) {
		slushBucket.setOutBucketLabelText(label);
	}
	
	public String getDisplayText() {
		if ((displayText == null) || (displayText.length()<1)) {
			this.getValues();
		}
		return displayText;
	}

	public void setModel(Object model) {
		outBucketModel = (MutableComboBoxModel)model;
		slushBucket.setOutBucketModel(outBucketModel);
	}

	public ListModel getOutBucketModel(){
		return (ListModel)slushBucket.getOutBucketModel();
	}

	public String getWindowHeader() {
		return windowHeader;
	}

	public void setWindowHeader(String text) {
		windowHeader = text;
	}
	
	public void setOutBucketCellRenderer(ListCellRenderer renderer) {
		slushBucket.setOutBucketCellRenderer(renderer);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
		setModel(null);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#addEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void addEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#removeEditorStateChangeListener(gov.va.med.hds.querytool.ui.editor.IRunnableEditorStateChangeListener)
	 */
	public void removeEditorStateChangeListener(IRunnableEditorStateChangeListener listener) {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#notifyEditorStateChanged(gov.va.med.hds.querytool.ui.editor.RunnableEditorStateChangedEvent)
	 */
	public void notifyEditorStateChanged(RunnableEditorStateChangedEvent e) {
	}

}
