package gov.va.med.hds.querytool.ui.types;
import gov.va.med.hds.cd.spectrum.date.DateComboBox;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class SpecDatesPanel extends JPanel {
	private GridBagLayout gridBagLayout = new GridBagLayout();
	private JCheckBox noStartDateCheckBox = new JCheckBox();
	private JCheckBox noStopDateCheckBox = new JCheckBox();
	private JLabel fromLabel = new JLabel("From:");
	private JLabel throughLabel = new JLabel("Through:");
	private Date currentDate;
	private DateComboBox fromDate = new DateComboBox();
	private DateComboBox toDate = new DateComboBox();

	private SpecDatesPanel() {
		super();
	}
	
	public SpecDatesPanel(Date currentDate) {
		this();
		this.currentDate = currentDate;
		init();
	}

	private void init() {
		fromDate.setSelectedDate(currentDate);
		toDate.setSelectedDate(currentDate);
		this.setLayout(gridBagLayout);
		noStartDateCheckBox.setText("No Start Date");
		noStartDateCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setStartDateEnabled();
			}
		});
		noStopDateCheckBox.setText("No Stop Date");
		noStopDateCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setStopDateEnabled();
			}
		});

		this.add(fromLabel, 			new GridBagConstraints(0,0,1,1,0.25,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		this.add(fromDate,				new GridBagConstraints(1,0,1,1,0.50,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		this.add(noStartDateCheckBox, 	new GridBagConstraints(2,0,1,1,0.25,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		this.add(throughLabel, 		new GridBagConstraints(0,1,1,1,0.25,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		this.add(toDate,				new GridBagConstraints(1,1,1,1,0.50,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		this.add(noStopDateCheckBox, 	new GridBagConstraints(2,1,1,1,0.25,0.0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(2,2,2,2),0,0));
		setStopDateEnabled();
		setStartDateEnabled();
	}

	/**
	 * Method setStopDateEnabled.
	 */
	private void setStopDateEnabled() {
		toDate.setEnabled(!noStopDateCheckBox.isSelected());
	}

	/**
	 * Method setStartDateEnabled.
	 */
	private void setStartDateEnabled() {
		fromDate.setEnabled(!noStartDateCheckBox.isSelected());
	}

	
	public static final void main(String args[]) {
		try {
			JFrame frame = new JFrame("SpecDatesPanel");
			SpecDatesPanel specDatesPanel = new SpecDatesPanel();
			frame.getContentPane().add(specDatesPanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
			frame.pack();
			frame.show();
	
		}
		catch (Exception ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void setNoStartDateButton(boolean selected) {
		noStartDateCheckBox.setSelected(selected);
		fromDate.setEnabled(!selected);
	}
	
	public void setNoStopDateButton(boolean selected) {
		noStopDateCheckBox.setSelected(selected);
		toDate.setEnabled(!selected);
	}
	
	public boolean isNoStartDateButtonSelected() {
		return noStartDateCheckBox.isSelected();
	}
	
	public boolean isNoStopDateButtonSelected() {
		return noStopDateCheckBox.isSelected();
	}
	
	public void setStartDate(Date startDate) {
		fromDate.setSelectedDate(startDate);
	}
	
	public void setStopDate(Date stopDate) {
		toDate.setSelectedDate(stopDate);
	}
	
	public Date getStartDate() {
		return fromDate.getSelectedDate();
	}
	
	public Date getStopDate() {
		return toDate.getSelectedDate();
	}

}