package gov.va.med.hds.querytool.ui.types;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerDateModel;

public class SpecQYPanel extends JPanel {
	public static final int NO_SELECTION = -1;
	public static final int FIRST = 0;
	public static final int SECOND = 1;
	public static final int THIRD = 2;
	public static final int FOURTH = 3;
	
	private JPanel topPanel = new JPanel();
	private JPanel bottomPanel = new JPanel();
	private JPanel topLeftPanel = new JPanel();
	private JPanel topRightPanel = new JPanel();
	private JCheckBox forCheckBox = new JCheckBox();
	private JComboBox quarterComboBox = new JComboBox();
	private JRadioButton calendarYearButton = new JRadioButton();
	private JRadioButton fiscalYearButton = new JRadioButton();
	private ButtonGroup buttonGroup = new ButtonGroup();
	private SpinnerDateModel spinnerDateModel;
	private JSpinner yearSpinner;
	private JSpinner.DateEditor spinnerDateEditor;
	private String[] calQuarters = {	"1st Quarter (Jan-Mar)",
										"2nd Quarter (Apr-Jun)",
										"3rd Quarter (Jul-Sep)",
										"4th Quarter (Oct-Dec)"};
	private String[] fiscalQuarters = {"1st Quarter (Oct-Dec)",
										"2nd Quarter (Jan-Mar)",
										"3rd Quarter (Apr-Jun)",
										"4th Quarter (Jul-Sep)" };
	private ComboBoxModel comboBoxModel;
	private boolean calendarSelected = false;
	private Date currentDate;

	private SpecQYPanel() {
		super();
	}
	
	public SpecQYPanel(Date currentDate) {
		this();
		this.currentDate = currentDate;
		init();
	}

	private void init() {
		spinnerDateModel = new SpinnerDateModel(currentDate, null, null, Calendar.YEAR);
		yearSpinner = new JSpinner(spinnerDateModel);
		spinnerDateEditor = new JSpinner.DateEditor(yearSpinner, "yyyy");
		yearSpinner.setEditor(spinnerDateEditor);
		yearSpinner.setEnabled(false);
		forCheckBox.setEnabled(false);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.X_AXIS));
		topLeftPanel.setLayout(new BoxLayout(topLeftPanel, BoxLayout.Y_AXIS));
		topRightPanel.setLayout(new BoxLayout(topRightPanel, BoxLayout.Y_AXIS));

		forCheckBox.setText(" for ");
		forCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setQuarterCombo();
			}
		});
		calendarSelected = true;
		setCalendarQuarter();
		calendarYearButton.setText("Calendar Year");
		calendarYearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				enableControls();
				setCalendarQuarter();
			}
		});
		fiscalYearButton.setText("Fiscal Year");
		fiscalYearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				enableControls();
				setFiscalQuarter();
			}
		});
		buttonGroup.add(calendarYearButton);
		buttonGroup.add(fiscalYearButton);
		this.add(topPanel);
		this.add(bottomPanel);
		bottomPanel.add(forCheckBox);
		bottomPanel.add(quarterComboBox);
		topLeftPanel.add(fiscalYearButton);
		topLeftPanel.add(calendarYearButton);
		topPanel.add(topLeftPanel);
		topPanel.add(Box.createHorizontalGlue());
		topPanel.add(topRightPanel);
		topPanel.add(Box.createHorizontalGlue());
		topRightPanel.add(Box.createVerticalGlue());
		topRightPanel.add(yearSpinner);
		topRightPanel.add(Box.createVerticalGlue());
		setQuarterCombo();
	}

	/**
	 * Method enableYear.
	 */
	public void enableControls() {
		yearSpinner.setEnabled(true);
		forCheckBox.setEnabled(true);
		if (forCheckBox.isSelected()) {
			quarterComboBox.setEnabled(true);
		}
	}

	/**
	 * Method setQuarterCombo.
	 */
	private void setQuarterCombo() {
		quarterComboBox.setEnabled(forCheckBox.isSelected());
	}
	
	public static final void main(String args[]) {
		try {
			JFrame frame = new JFrame("SpecQYPanel");
			SpecQYPanel specQYPanel = new SpecQYPanel();
			frame.getContentPane().add(specQYPanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
			frame.pack();
			frame.show();
	
		}
		catch (Exception ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void setFiscalYearButton() {
		fiscalYearButton.setSelected(true);
		setFiscalQuarter();
	}
	
	public void setCalYearButton() {
		calendarYearButton.setSelected(true);
		setCalendarQuarter();
	}
	
	public void setForButton(boolean selected) {
		forCheckBox.setSelected(selected);
		setQuarterCombo();
	}
	
	public void setYear(Date year) {
		yearSpinner.setValue(year);
		yearSpinner.setModel(spinnerDateModel);
	}
	
	public void setQuarter(int quarter) {
		quarterComboBox.setSelectedIndex(quarter);
	}

	public boolean isFiscalYearButtonSet() {
		return fiscalYearButton.isSelected();
	}
	
	public boolean isCalYearButtonSet() {
		return calendarYearButton.isSelected();
	}
	
	public boolean isForButtonSet() {
		return forCheckBox.isSelected();
	}
	
	public Date getYear() {
		return spinnerDateModel.getDate();
	}
	
	public int getQuarter() {
		return quarterComboBox.getSelectedIndex();
	}
	
	/**
	 * Method setFiscalQuarter.
	 */
	private void setFiscalQuarter() {
		int index = quarterComboBox.getSelectedIndex();
		comboBoxModel = new DefaultComboBoxModel(fiscalQuarters);
		quarterComboBox.setModel(comboBoxModel);
		if ((index > -1) && calendarSelected) {
			index++;
			if (index>3) index = 0;
			quarterComboBox.setSelectedIndex(index);
		}
		calendarSelected = false;
	}

	/**
	 * Method setCalendarQuarter.
	 */
	private void setCalendarQuarter() {
		int index = quarterComboBox.getSelectedIndex();
		comboBoxModel = new DefaultComboBoxModel(calQuarters);
		quarterComboBox.setModel(comboBoxModel);
		if ((index > -1) && (!calendarSelected)) {
			index--;
			if (index<0) index = 3;
			quarterComboBox.setSelectedIndex(index);
		}
		calendarSelected = true;
	}

}