package gov.va.med.hds.querytool.ui.types;

import gov.va.med.hds.cd.spectrum.bucket.TreeSlushBucket;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.querytool.ui.editor.IRunnableEditorState;

import java.awt.BorderLayout;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.MutableComboBoxModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;

/**
 * Slush bucket that uses a JTree on the left side and a JList on the right side.
 */
public class TreeBucketEditor extends EditorPanel {
	private ITreeModelDAOFactory treeDAOFactory;
	private JPanel messagePanel = new JPanel();
	private JLabel label1 = new JLabel("To select an item, click it in the column on the left, then click '>'.");
	private JLabel label2 = new JLabel("Repeat to add multiple items.");
	private TreeSlushBucket treeBucket = new TreeSlushBucket();
	private MutableComboBoxModel outBucketModel = new DefaultComboBoxModel();
	private String displayText;
	private String windowHeader;

	private TreeBucketEditor() {
		super();
		messagePanel.setLayout(new BoxLayout(messagePanel,BoxLayout.Y_AXIS));
		messagePanel.add(label1);
		messagePanel.add(label2);
		this.setLayout(new BorderLayout());
		this.add(messagePanel,BorderLayout.NORTH);
		this.add(treeBucket,BorderLayout.CENTER);
		treeBucket.setInBucketLabelText("Service Hierarchy");
		treeBucket.setOutBucketLabelText("Selected Services");
		treeBucket.setSelectAllButtonState(false);
	}
	
	public TreeBucketEditor(ITreeModelDAOFactory treeFactory) {
		this();
		this.treeDAOFactory = treeFactory;
		this.treeBucket.setOutBucketModel(outBucketModel);
	}

	public void init() throws EditorException {
		try {
			treeBucket.setInBucketModel((treeDAOFactory.getTreeModelDAO(super.getRuntime(),super.getDaoRegistry())).getTree());
			setupValues();
		}
		catch (Exception e) {
			throw new EditorException(e);
		}
		
	}

	private void setupValues() throws EditorException {
		Iterator it = super.getValues().iterator();
		TreeModel treeModel = (TreeModel)treeBucket.getInBucketModel();
		TreeMap map = new TreeMap();
		while (it.hasNext()) {
			IBufferItem item = getItem((DefaultMutableTreeNode)treeModel.getRoot(),(String)it.next());
			if (item != null) {
				map.put(item.getValue()+item.getKey(),item);
			}
		}
		it = map.keySet().iterator();
		while (it.hasNext()) {
			outBucketModel.addElement(map.get(it.next()));
		}
	}

	private IBufferItem getItem(DefaultMutableTreeNode currentNode, String key) {
		IBufferItem result = null;
		if (currentNode == null) {
			return result;
		}
		if (((IBufferItem)currentNode.getUserObject()).getId().equals(key)) {
			result = (IBufferItem)currentNode.getUserObject();
		}
		else {
			for (int i=0;i<currentNode.getChildCount();i++) {
				if ((result=getItem((DefaultMutableTreeNode)currentNode.getChildAt(i),key))!=null) {
					i = currentNode.getChildCount();
				}
			}
		}
		return result;
	}

	public String getDisplayText() {
		if ((displayText==null) || (displayText.length()<1)) {
			this.getValues();
		}
		return displayText;
	}

	public List getValues() {
		List result = new LinkedList();
		StringBuffer text = new StringBuffer();
		for (int i = 0; i < outBucketModel.getSize(); i++) {
			IBufferItem item = (IBufferItem)outBucketModel.getElementAt(i);
			result.add(item.getId());
			if (text.length()>0) {
				text.append(" or ");
			}
			text.append(item.getValue());
		}
		displayText = text.toString();
		return result;
	}

	public String getWindowHeader() {
		return windowHeader;
	}

	public void setWindowHeader(String text) {
		windowHeader = text;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#clearModels()
	 */
	public void clearModels() {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#getEditorState()
	 */
	public IRunnableEditorState getEditorState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.querytool.ui.editor.IRunnableEditorPanel#setEditorState(gov.va.med.hds.querytool.ui.editor.IRunnableEditorState)
	 */
	public void setEditorState(IRunnableEditorState state) {

	}

}
