/*
 * Initially Created on Apr 8, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.dao.IQueryDAO;

/**
 * Interface to a collection of predefined queries and user queries.
 */
public interface ICombinedQueryCollection {
	
	/**
	 * Adds a change listener.
	 * @param l
	 */
	void addChangeListener(IQueryCollectionListener l);

	/**
	 * Returns the state of this list containing the given display name.
	 * @param displayName
	 * @return boolean
	 */
	boolean containsDisplayName(String displayName);
	
	/**
	 * Returns an item from the user query collection.
	 * @param index
	 * @return item
	 */
	ISoftListItem getUserQueryItem(int index);
	
	/**
	 * Returns an item from the predefined query collection.
	 * @param index
	 * @return item
	 */
	ISoftListItem getPredefinedQueryItem(int index);
	
	/**
	 * Returns the state of the given item.
	 * @param item
	 * @return boolean
	 */
	boolean isPredefinedItem(ISoftListItem item);
	
	/**
	 * Returns the state of the given query item.
	 * @param item
	 * @return boolean
	 */
	boolean isUserQuery(ISoftListItem item);

	/**
	 * Reloads the query in the specified item from the database, if it has changed.
	 * If the queryID is not one generated from the database, this method does nothing. 
	 * @param item
	 * @throws QueryDAOException
	 */
	void reloadChangedQuery(ISoftListItem item) throws QueryCollectionException;
	
	/**
	 * Removes a registered change listener.
	 * @param l
	 */
	void removeChangeListener(IQueryCollectionListener l);

	/**
	 * Deletes the specified item from the collection, and if it exists in the data
	 * store, removes it from there as well.
	 * @param item
	 * @throws QueryDAOException
	 */
	void removeUserItem(ISoftListItem item) throws QueryCollectionException;

	/**
	 * Sets the query DAO.
	 * @param queryDAO
	 */
	void setQueryDAO(IQueryDAO queryDAO);

	/**
	 * Sets the runtime context in this component.
	 * @param runtime
	 * @throws QueryDAOException
	 */
	void setRuntime(IRuntimeContext runtime) throws QueryCollectionException;

	/**
	 * Returns the size of the collection.
	 * @return int
	 */
	int getSize();
	
	/**
	 * Returns the size of the user query collection.
	 * @return int
	 */
	int getUserDefinedQuerySize();
	
	/**
	 * Returns the size of the predefined query collection.
	 * @return int
	 */
	int getPredefinedQuerySize();

	/**
	 * Returns true if this object is an item in the collection; returns false
	 * otherwise.
	 * @param item
	 * @return itemStatus
	 */
	boolean isQueryItem(Object item);

	/**
	 * Returns the query associated with the provided item.
	 * @param item
	 * @return query
	 * @throws QueryCollectionException
	 */
	IQuery getQuery(Object item) throws QueryCollectionException;

	/**
	 * Saves the given query if:
	 *   1.  If the query has an invalid queryID OR
	 *   2.  The current query has changed AND it's a user query.
	 * Otherwise, the call does nothing.
	 * 
	 * If the query is not in the collection, it is added to the collection as a new
	 * user query.
	 * 
	 * @param query
	 */
	void saveQuery(IQuery query) throws QueryCollectionException;
	
	/**
	 * Refreshes the model from the user and predefined lists.
	 * @throws QueryCollectionException
	 */
	void refresh() throws QueryCollectionException;

	/**
	 * Adds a query (in the user query section) without saving it to the database.
	 * @param query
	 * @throws QueryCollectionException
	 */
	void addQueryWithoutSave(IQuery query) throws QueryCollectionException;

	/**
	 * Returns the state of the given query.
	 * @param currentQuery
	 * @return queryState - is the given query a predefined query.
	 */
	boolean isPredefinedQuery(IQuery query);

	/**
	 * Removes the specified query from the collection, and if it exists in the data store,
	 * removes it from there as well.
	 * @param query
	 * @throws QueryCollectionException
	 */
	void removeUserQuery(IQuery query) throws QueryCollectionException;

}