package gov.va.med.hds.querytool;

import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;

/**
 * Interface for constraint items which are used by IQuery objects.
 */
public interface IConstraint {
	static final String ROOT = "0";
	static final String PARM_ADD = IQuery.PARM_ADD;
	static final String PARM_REMOVE = IQuery.PARM_CLEAR;
	static final String PARM_CHANGE = IQuery.PARM;
	static final String PARM_REMOVE_ALL = IQuery.PARM_CLEARALL;

	/**
	 * Add a new IValue.
	 * @param value
	 */
	void addValue(IValue value);

	/**
	 * Returns a list of "tags" in the constraint set that are selected.
	 * @param list
	 */
	void getSetTags(List list);

	/**
	 * Clears the constraint with the indicated tag, plus any below it.  The method
	 * returns a list of the tags cleared.
	 * @param item
	 * @return List
	 */
	List clearTag(Object item);

	/**
	 * Method setTag.
	 * @param item
	 */
	void setTag(Object item);

	/**
	 * Method addDataNeededListener.
	 * @param parmDataNeededListener
	 */
	void addDataNeededListener(IParmDataNeededListener parmDataNeededListener);

	/**
	 * Method removeDataNeededListener.
	 * @param parmDataNeededListener
	 */
	void removeDataNeededListener(IParmDataNeededListener parmDataNeededListener);

	/**
	 * Method addRelationship.
	 * @param text
	 */
	void addRelationship(String text);

	/**
	 * Clears all constraints.
	 */
	void clearAll();

	/**
	 * @param tag
	 * @return List
	 */
	List getCriterion(Object tag);

	/**
	 * Method clearRelationships.
	 */
	void clearRelationships();

	/**
	 * Method isValueSetComplete.
	 * @return boolean
	 */
	boolean isValueSetComplete();
	
	/**
	 * Returns a "fail fast" iterator (it does not implement the remove method.)
	 * @return Iterator
	 */
	Iterator iterator();
	
	/**
	 * Returns a list of display field types available for the selected criteria.
	 * @return List - List of display field type Objects
	 */
	List getDisplayFieldTypes();

	/**
	 * @param valueChangeListener
	 */
	void addPropertyChangeListener(PropertyChangeListener valueChangeListener);

	/**
	 * @param valueChangeListener
	 */
	void removePropertyChangeListener(PropertyChangeListener valueChangeListener);
	
}
