package gov.va.med.hds.querytool;

import java.beans.PropertyChangeListener;
import java.util.List;

/**
 * Composes a query definition, which contains a name, owner, idetifier, and a list
 * of display fields, sample data for the display fields, and report criterion.
 */
public interface IQuery {
	public static String DISP = "disp";
	public static String DISP_FIELD_ADD = "disp.add";
	public static String DISP_FIELD_REMOVE = "disp.remove";
	public static String DISP_FIELD_REMOVE_ALL = "disp.clearAll";
	public static String PARM = "parm";
	public static String PARM_ADD = "parm.addConstraint";
	public static String PARM_CLEARALL = "parm.clearAll";
	public static String PARM_CLEAR = "parm.clearConstraint";
	public static String QUERY = "query";
	public static String QUERY_DISPLAYNAME = "query.displayName";
	public static String QUERY_ID = "query.id";
	public static String QUERY_NAME = "query.name";
	public static String QUERY_ALL = "query.all";

	/**
	 * Method addDataNeededListener.
	 * @param d
	 */
	void addDataNeededListener(IDataNeededListener a);

	/**
	 * Method removeDataNeededListener.
	 * @param d
	 */
	void removeDataNeededListener(IDataNeededListener d);

	/**
	 * Returns the user context for the query.
	 * @return Object
	 */
	Object getUserContext();

	/**
	 * Gets the database query ID number.
	 * @return QueryID
	 */
	IQueryID getQueryID();

	/**
	 * Method setQueryID.
	 * @param queryID
	 */
	void setQueryID(IQueryID queryID);

	/**
	 * Returns the display name of the query.  Returns the same value as the toString
	 * function of the object.
	 * @return String
	 */
	String getDisplayName();

	/**
	 * Set the display name.
	 * @param displayName
	 */
	void setDisplayName(String displayName);

	/**
	 * Method addParameterChangeListener.
	 * @param changeListener
	 */
	void addPropertyChangeListener(PropertyChangeListener changeListener);

	/**
	 * Method removeParameterChangeListener.
	 * @param changeListener
	 */
	void removePropertyChangeListener(PropertyChangeListener changeListener);

	/**
	 * Method isQueryChanged.
	 * @return boolean
	 */
	boolean isQueryChanged();

	/**
	 * Sets the queryChanged.
	 * @param queryChanged The queryChanged to set
	 */
	void setChanged(boolean queryChanged);

	/**
	 * Method getName.
	 * @return String
	 */
	String getName();

	/**
	 * Method setName.
	 * @param name
	 */
	void setName(String name);

	/**
	 * Returns the state of this query's queryID = QueryID.newID.
	 * @return boolean
	 */
	boolean isUnsavedQueryID();

	/**
	 * Returns the state of this query's queryID = QueryID.nullID.
	 * @return boolean
	 */
	boolean isNullQueryID();

	/**
	 * Method isHydrateRequested.
	 * @return boolean
	 */
	boolean isHydrateRequested();
	
	/**
	 * Sets the hydrated status within the query.
	 * @param hydrateRequested
	 */
	void setHydrateRequested(boolean hydrateRequested);
	
	/**
	 * Method isChanging.
	 * @return boolean
	 */
	boolean isChanging();
	
	/**
	 * Method setChanging.
	 * @param changing
	 */
	void setChanging(boolean changing);

	/**
	 * Checks to see if the query is hydrated from the database, and if not,
	 * raises a data needed event.
	 */
	void init();
	
	/**
	 * Method reset.
	 */
	void reset();

	/**
	 * Method isValueSetComplete.
	 * @return boolean
	 */
	boolean isValueSetComplete();
	
	/**
	 * Returns the display field types available for the current set of
	 * parameters.
	 * @return list
	 */
	List getDisplayFieldTypes();
	
	/**
	 * Adds a finalization listener.
	 * @param listener
	 */
	void addFinalizeListener(IFinalizeListener listener);

	/**
	 * Removes a finalization listener.
	 * @param listener
	 */
	void removeFinalizeListener(IFinalizeListener listener);
	
	/**
	 * @return displayFieldList
	 */
	IDisplayFieldList getDisplayFields();
	
	/**
	 * @return constraints
	 */
	IConstraint getConstraints();

}
