package gov.va.med.hds.querytool;

import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.querytool.types.ITypeFactory;
import gov.va.med.hds.querytool.ui.IResultsTable;

import javax.swing.Action;

/**
 * Main plugin interface for the querytool.
 */
public interface IQueryTool extends IPlugin {
	static final String PLUGIN_ID = "gov.va.med.hds.querytool";
	static final String TYPES_EXTENSION_POINT_ID = "gov.va.med.hds.querytool.valueTypes";
	static final String DETAIL_DAO_MAPS_EXTENSION_POINT_ID = "gov.va.med.hds.querytool.detailDaoMaps";
	
	static final String ID_ATTRIBUTE = "id";
	static final String TYPE_ATTRIBUTE = "type";
	static final String EDITOR_FACTORY_ATTRIBUTE = "editorFactory";
	static final String INTERFACE_ATTRIBUTE = "interface";
	
	static final String QUERYACTION_HOME = "home";
	
	static final String EDIT_PARM_ACTION			= "Edit Parameter...";
	static final char EDIT_PARM_CHAR = 'P';
	
	static final String EDIT_PARM_COMPLETE_ACTION	= "edit.parm.complete";
	static final char EDIT_PARM_COMPLETE_CHAR = 'c';
	
	static final String CREATE_ACTION				= "Create Custom Query...";
	static final char CREATE_CHAR = 'C';
	
	static final String SELECT_ACTION				= "Select Query";
	static final char SELECT_CHAR = 'S';
	
	static final String UNSELECT_ACTION			= "UnSelect";
	static final char UNSELECT_CHAR = 'U';
	
	static final String RUN_ACTION				= "View Report";
	static final char RUN_CHAR = 'V';
	
	static final String HOME_ACTION				= "Back to Report Selection";
	static final char HOME_CHAR = 'B';
	
	static final String EDIT_ACTION				= "Edit...";
	static final char EDIT_CHAR = 'I';
	
	static final String EDIT_COMPLETE_SAVE_ACTION = "edit.complete.save";
	static final char EDIT_COMPLETE_SAVE_CHAR = 's';
	
	static final String EDIT_COMPLETE_NOSAVE_ACTION = "edit.complete.nosave";
	static final char EDIT_COMPLETE_NOSAVE_CHAR = 'n';
	
	static final String EDIT_CANCEL_ACTION = "edit.cancel";
	static final char EDIT_CANCEL_CHAR = 'C';
	
	static final String DELETE_ACTION				= "Delete...";
	static final char DELETE_CHAR = 'D';
	
	static final String DELETE_CONFIRMED_ACTION	= "delete.confirmed";
	static final char DELETE_CONFIRMED_CHAR = 'C';
	
	static final String SAVEAS_ACTION				= "Save As...";
	static final char SAVEAS_CHAR = 'S';
	
	static final String SAVEAS_CONFIRMED_ACTION	= "saveAs.confirmed";
	static final char SAVEAS_CONFIRMED_CHAR = 'C';
	
	static final String RENAME_ACTION				= "Rename...";
	static final char RENAME_CHAR = 'R';
	
	static final String STOP_ACTION				= "Stop";
	static final char STOP_CHAR = 'S';
	
	static final String EXPORT_ACTION				= "Export...";
	static final char EXPORT_CHAR = 'X';
	
	static final String PRINT_ACTION				= "Print...";
	static final char PRINT_CHAR = 'P';
	
	static final String RUN_COMPLETE_ACTION				= "run.complete";
	static final char RUN_COMPLETE_CHAR = 'R';
	
	static final String RUN_COMPLETE_MESSAGE_ACTION			= "run.complete.message";
	static final char RUN_COMPLETE_MESSAGE_CHAR = 'M';
	
	static final String RUN_COMPLETE_ERROR_ACTION			= "run.complete.error";
	static final char RUN_COMPLETE_ERROR_CHAR = 'E';
	
	static final String RUN_COMPLETE_NON_SENSITIVE_ACTION	= "run.complete.nonsensitive";
	static final char RUN_COMPLETE_NON_SENSITIVE_CHAR = 'N';
	
	static final String RUN_CONTINUE_ACTION				= "Continue Query";
	static final char RUN_CONTINUE_CHAR = 'C';
	
	static final String SORT_COLUMN_ACTION				= "Sort Column";
	static final char SORT_COLUMN_CHAR = 'S';
	
	static final String SELECT_ALL_ACTION					= "Select All";
	static final char SELECT_ALL_CHAR = 'A';
	
	static final String SELECT_NONE_ACTION				= "Select None";
	static final char SELECT_NONE_CHAR = 'N';
	
	static final String SELECT_SENSITIVE_ACTION				= "select.sensitive";
	static final char SELECT_SENSITIVE_CHAR = 'S';

	
	static final String SELECT_AND_EDIT_STATE				= "state.selectAndEdit";
	static final String EXECUTE_MAIN_STATE				= "state.execute.main";
	static final String EXECUTE_SENSITIVE_STATE			= "state.execute.sensitive";
	static final String SENSITIVE_EDIT_STATE				= "state.sensitive.edit";
	static final String VIEW_STATE						= "state.view";
	
	/**
	 * Returns the currentQuery that actions are taken on.
	 * @return currentQuery
	 */
	IQuery getCurrentQuery();
	
	/**
	 * Sets the currentQuery that actions are taken on.
	 * @param currentQuery
	 */
	void setCurrentQuery(IQuery currentQuery);
	
	/**
	 * Returns a query collection interface.
	 * @return
	 */
	ICombinedQueryCollection getQueryCollection();
	
	/**
	 * Gets the query execution model from the query tool.
	 * @return runQueryModel
	 */
	IRunQuery getRunModel();
	
	/**
	 * Returns the message string after a message or error event is raised.
	 * @return message
	 */
	String getMessage();
	
	/**
	 * Returns the error after an error event is raised.
	 * @return error
	 */
	Throwable getError();
	
	/**
	 * Adds an action listener.
	 * @param listener
	 */
	void addQueryToolListener(IQueryToolListener listener);
	
	/**
	 * Removes a registered listener.
	 * @param listener
	 */
	void removeQueryToolListener(IQueryToolListener listener);
	
	/**
	 * Dispatches a query tool event to listeners.
	 * @param event
	 */
	void dispatchEvent(IQueryToolEvent event);
	
	/**
	 * Dispatches a state change event.
	 * @param state
	 */
	void dispatchStateEvent(IQueryToolState state);
	
	/**
	 * Gets the querytool state from the specified key.
	 * @param stateKey
	 * @return queryToolState
	 */
	IQueryToolState getState(Object stateKey);
	
	/**
	 * Registers a query action.
	 * @param actionKey
	 * @param action
	 */
	void registerAction(Object actionKey, IQueryAction action);
	
	/**
	 * Returns a registered query action.
	 * @param actionKey
	 * @return action
	 * @throws QueryActionException
	 */
	Action getQueryAction(Object actionKey);

	/**
	 * Returns a new query instanciated from the database.
	 * @param displayName - Display name of the query.  If null, query gets a system default display name.
	 * @return newQuery
	 */
	IQuery getNewQuery(String displayName);
	
	/**
	 * Returns the type factory.
	 * @return typeFactory
	 */
	ITypeFactory getTypeFactory();
	
	/**
	 * Sets up the IRunQuery model.
	 */
	void setupRunModel();
	
	/**
	 * Sets up the report results table.
	 * @return isTableSetUp
	 */
	boolean setupResultsTable();
	
	/**
	 * Returns the completed results table model.
	 * @return resultsTableModel
	 */
	IResultsTable getResultsModel();

	/**
	 * Adds a query tool state listener.
	 * @param listener
	 */
	void addStateListener(IStateListener listener);
	
	/**
	 * Removes a registered query tool state listener.
	 * @param listener
	 */
	void removeStateListener(IStateListener listener);

	/**
	 * @return isActionAuthorized
	 * @parm actionKey
	 */
	boolean isActionAuthorized(Object actionKey);

	/**
	 * @param string
	 * @return isValidQueryName
	 */
	boolean isValidQueryName(String string);
	
	/**
	 * @param message
	 */
	void setMessage(String message);

	/**
	 * @param name
	 * @return isNameCorrectLength
	 */
	boolean isNameCorrectLength(String name);
	
	/**
	 * Returns the model that contains the IDisplayFields that are available to add to the current IDisplayFieldList
	 * in the current query.
	 * @return availableColumnsModel
	 */
	IAvailableColumnsList getAvailableColumnsModel();
	
}
