/*
 * Initially Created on Apr 3, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool;

import java.util.List;

import javax.swing.ListModel;

/**
 * Manages a collection of items that are stored as ISoftItems.
 */
public interface ISoftList extends ListModel {

	/**
	 * Returns a list of items in this collection that are changed.  What is "changed"
	 * is determined by implementers.
	 * @return list
	 */
	List getChangedItems();
	
	/**
	 * Adds an item to the collection.  If the key already exists in the collection, a
	 * SoftListException is thrown.
	 * @param key
	 * @param item
	 * @param displayString
	 */
	void addItem(Object key, Object item, String displayString) throws SoftListException;
	
	/**
	 * Removes the specified object entry in the list.  Returns the object removed if the
	 * soft reference still exists; null otherwise.
	 * @param key
	 * @return object
	 */
	Object removeItem(Object key);
	
	/**
	 * Removes the item at the specified location.  Returns the object removed if the
	 * soft reference still exists; null otherwise.
	 * @param index
	 * @return object
	 */
	Object removeAt(int index);
	
	/**
	 * Similar to addItem, except the item must exist in the list with the specified
	 * key.  It replaces item and displayString with the passed in parameters.  An 
	 * exception is thrown if the key does not exist in the list.
	 * @param key
	 * @param item
	 * @param displayString
	 */
	void updateItem(Object key, Object item, String displayString) throws SoftListException;
	
}
