package gov.va.med.hds.querytool;

import java.beans.PropertyChangeListener;
import java.util.List;

/**
 * Interface for parameter value collections.
 */
public interface IValue {
	
	/**
	 * Method getName.
	 * @return String
	 */
	String getID();

	/**
	 * Returns the "pre-amble" text to the value or constraint.
	 * @return String
	 */
	String getDisplayText();

	/**
	 * Method setID.  The ID is also known as the "tag".
	 * @param ID
	 */
	void setID(String ID);

	/**
	 * Sets the "pre-amble" text for this value or constraint.
	 * @param displayText
	 */
	void setDisplayText(String displayText);
	
	/**
	 * Method addPropertyChangeListener.
	 * @param e
	 */
	void addPropertyChangeListener(PropertyChangeListener e);
	
	/**
	 * Method removePropertyChangeListener.
	 * @param e
	 */
	void removePropertyChangeListener(PropertyChangeListener e);
	
	/**
	 * Clears all listeners registered with addPropertyChangeListener().
	 */
	void clearListeners();
	
	/**
	 * Method getValue.
	 * @return Object
	 */
	List getValues();

	/**
	 * Method setValues.
	 * @param list
	 */
	void setValues(List list);

	/**
	 * Returns the displayable string that represents the internal values
	 * for this variable.  For example, if this was a date range (t:t-7) the
	 * text displayed would be "the past week".
	 * @return String
	 */
	String getDefaultDisplayText();

	/**
	 * Method setDefaultDisplayText.
	 * @param defaultDisplayText
	 */
	void setDefaultDisplayText(String defaultDisplayText);
	
	/**
	 * Method isReadOnly.
	 * @return boolean
	 */
	boolean isReadOnly();

	/**
	 * Method setConstraintName.
	 * @param constraintName
	 */
	void setConstraintName(String constraintName);
	
	/**
	 * Method getConstraintName.
	 * @return String
	 */
	String getConstraintName();
	
	/**
	 * Method getEmptyDisplayText.
	 * @return String
	 */
	String getEmptyDisplayText();

	/**
	 * Allows the query object to specify the lock for snychronized methods for this
	 * object, so the query is not updating when someone is reading, etc.
	 * @param lock
	 */
	void setLock(Object lock);
	
	/**
	 * Method getEditorID.
	 * @return Object
	 */
	Object getEditorID();

	/**
	 * Sets the value types editor ID.
	 * @param editorID
	 */
	void setEditorID(Object editorID);
	
	/**
	 * Sets the display field category for the IValue.
	 * @param fieldCategory
	 */
	void setDisplayFieldCategory(Object fieldCategory);
	
	/**
	 * Returns the display field category for this IValue.  Used to manage display
	 * fields in a query.
	 * @return fieldCategory
	 */
	Object getDisplayFieldCategory();

}
