/*
 * Initially Created on May 29, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.dao;

import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IPatDetailReceivable;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IReportRow;

/**
 * Report results DAO interface.
 */
public interface ICompleteQueryDataDAO extends IDataAccessObject {

	/**
	 * Gets a report row from a completed report.  If this is called prematurely,
	 * return an array with size 0.
	 * @param fromIndex
	 * @param toIndex
	 * @return IReportRow[]
	 * @throws QueryDAOException
	 */
	IReportRow[] getReportRow(int fromIndex, int toIndex) throws QueryDAOException;

	/**
	 * Requests a detailed item from the server from an IReportRow.
	 * @param row
	 * @param dataUser
	 * @throws QueryDAOException
	 */
	void detailResultsRequest(IReportRow row, IDetailReceivable dataUser) throws QueryDAOException;
	
	/**
	 * Requests the patient data for the specified IReportRow.
	 * @param row
	 * @param dataUser
	 * @throws QueryDAOException
	 */
	void patientInfoRequest(IReportRow row, IPatDetailReceivable dataUser) throws QueryDAOException;

	/**
	 * Notifies the server to sort the report results be the specified IDisplayField.
	 * @param field
	 * @param ascending
	 * @throws QueryDAOException
	 */
	void setSortField(IDisplayField field, boolean ascending) throws QueryDAOException;
	
	/**
	 * Sets the query model for the query results being set up.
	 * @param query
	 */
	void setQuery(IQuery query);
	
}
