package gov.va.med.hds.querytool.dao;

import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import java.util.Map;
import java.util.TreeMap;

/**
 * Interface for a general Name-Value pair (reference lookup) DAO.
 */
public interface INameValuePairDAO extends IDataAccessObject {
	
	/**
	 * Used to build a simple map from database reference call.  If the Map is 
	 * passed in with keys (names), then the return map contains the value of 
	 * those keys only.  If the Map is passed in empty (and it is a reference
	 * set returned by *one* RPC call from the database,) then all values from 
	 * the database are returned.  The returned TreeMap has a key that is the 
	 * database value object + the database key object, with the value object 
	 * being a Map.Entry with the actual key-value objects.
	 * 
	 * key = database key object
	 * value = database value object
	 * @param items
	 * @return TreeMap
	 * @throws QueryDataDAOException
	 */
	TreeMap get(Map items) throws QueryDataDAOException;
	
	/**
	 * Implements a piece function to return the key (name) object from a 
	 * key-value object from the database.
	 * @param item
	 * @return Object
	 * @throws QueryDataDAOException
	 */
	Object getKey(Object item) throws QueryDataDAOException;
	
	/**
	 * Implements a piece function to return the value object from a key-value
	 * object from the database.
	 * @param item
	 * @return Object
	 * @throws QueryDataDAOException
	 */
	Object getValue(Object item) throws QueryDataDAOException;
}
