package gov.va.med.hds.querytool.dao;

import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryID;

import java.util.List;

/**
 * Data Access Object for an IQuery.
 */
public interface IQueryDAO extends IDataAccessObject {
	
	/**
	 * Sets the DAO as a listener for this query's data request events.
	 * @param query
	 */
	void setQuery(IQuery query);

	/**
	 * Removes the DAO as listener for this query's data request events.
	 * @param query
	 */
	void clearQuery(IQuery query);
		
	/**
	 * Returns a query initially built with a query name and ID.  This method registers
	 * the query with this DAO for dataNeeded events from the query, and will asynchronously
	 * fill the rest of the query when the query.init() method is called.
	 * @param queryID
	 * @param displayText
	 * @return IQuery
	 * @throws QueryDAOException
	 */
	IQuery getAsync(IQueryID queryID, String displayText) throws QueryDAOException;
	
	/**
	 * Returns a new query as defined by the database.
	 * @param displayText
	 * @return IQuery
	 */
	IQuery getNew(String displayText) throws QueryDAOException;

	/**
	 * Synchronous call to create a query object from the database.  It registers the
	 * query with this DAO to respond to future query dataNeeded events.
	 * @param queryID
	 * @return IQuery
	 * @throws QueryDAOException
	 */
	IQuery getSync(IQueryID queryID) throws QueryDAOException;

	/**
	 * Save the query to the database.
	 * @param query
	 * @throws QueryDAOException
	 */
	void save(IQuery query) throws QueryDAOException;

	/**
	 * Deletes the specified query from the database.
	 * @param queryID
	 * @return boolean
	 * @throws QueryDAOException
	 */
	boolean delete(IQueryID queryID) throws QueryDAOException;

	/**
	 * Gets the list of data that is used to pass into the IRunDAO to execute a 
	 * query.
	 * @param query
	 * @return List
	 */
	List getReportStrings(IQuery query);

}
