/*
 * Initially Created on Apr 8, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.IQuery;
import gov.va.med.hds.querytool.IQueryList;
import gov.va.med.hds.querytool.QueryListException;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import javax.swing.ListModel;

/**
 * Implements a list model to display predefined queries and user queries.
 */
public interface ICombinedQueryList extends ListModel {
	
	/**
	 * Returns the state of the given item.
	 * @param item
	 * @return boolean
	 */
	boolean isUserItem(Object item);
	
	/**
	 * Returns the state of the given item.
	 * @param item
	 * @return boolean
	 */
	boolean isPredefinedItem(Object item);

	/**
	 * Returns true if the list item is a real query entry; false otherwise.
	 * @param item
	 * @return boolean
	 */
	boolean isItem(Object item);

	/**
	 * Returns true if the "create new" list item is selected; false otherwise.
	 * @param item
	 * @return boolean
	 */
	boolean isCreateAction(Object item);
	
	/**
	 * Causes this model to raise a contents changed event for the entire list.
	 * @throws QueryDAOException
	 */
	void refresh() throws QueryDAOException;

	/**
	 * Adds a new user query to the list.
	 * @param newQuery
	 * @throws QueryListException
	 */
	void addUserQuery(IQuery newQuery) throws QueryListException;

	/**
	 * Sets the runtime context in this component.
	 * @param runtime
	 * @throws QueryDAOException
	 */
	void setRuntime(IRuntimeContext runtime) throws QueryDAOException;

	/**
	 * Returns the query at the specified index.
	 * @param index
	 * @return query
	 * @throws QueryDAOException
	 */
	IQuery getQueryAt(int index) throws QueryDAOException;

	/**
	 * Deletes the query at the specified index.
	 * @param object
	 * @throws QueryDAOException
	 */
	void removeQueryAt(int index) throws QueryDAOException;

	/**
	 * Asks the list to create a new IQuery, and return the index of the new query.
	 * @return index
	 * @throws QueryDAOException
	 */
	int createNewQuery() throws QueryDAOException;

	/**
	 * Returns the state of this list containing the given display name.
	 * @param displayName
	 * @return boolean
	 */
	boolean containsDisplayName(String displayName);
	
	/**
	 * Returns a list of the items that make up the user defined list.
	 * @return queryList
	 */
	IQueryList getUserDefinedList();
	
	/**
	 * Returns a list of the items that make up the predefined list.
	 * @return queryList
	 */
	IQueryList getPredefinedList();

	/**
	 * Reloads the query at the specified index, if it has changed.  If the queryID
	 * is null, this method does nothing (because presumably the query is not in the 
	 * database if it has a null ID.)
	 * @param index
	 * @throws QueryDAOException
	 */
	void reloadChangedQuery(int index) throws QueryDAOException;

	/**
	 * Saves the query at the current index, if there is a query defined and it has a 
	 * null query ID.  Does nothing otherwise.
	 * @param index
	 */
	void saveQuery(int index) throws QueryDAOException;
}