package gov.va.med.hds.querytool.ui;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import java.util.Collection;
import java.util.List;
import java.util.Map;
/**
 * @author VHAISLMONSOS
 *
 */
public interface IQueryPanelFactory {
	/**
	 * Gets the query wizard panels that allow specific selection or exclusion
	 * of query parameters.  Returns a list of IEnablePanels.
	 * @return List of IEnablePanels
	 * @throws QueryDAOException
	 */
	List getPanels() throws QueryDAOException;
	
	/**
	 * Returns a collection of all the ITagComponents (usually buttons for query
	 * parameters) generated by the getPanels method.
	 * @return Collection of ITagComponents
	 */
	Collection getButtons();
	
	/**
	 * Initializes the check box groups (contained by CheckBoxPanels which
	 * inherit from EnablePanel and implement the IEnablePanel interface.)  This
	 * method is called after the parameter buttons have been initialized.
	 */
	void initializeCheckGroups();
	
	/**
	 * Contains a map of the parameter buttons, with tag objects as the keys.
	 * @return Map of ITagComponents, with tag Objects as keys
	 */
	Map getButtonMap();
}