package gov.va.med.hds.querytool.ui;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.querytool.IDetailReceivable;
import gov.va.med.hds.querytool.IDisplayField;
import gov.va.med.hds.querytool.IPatDetailReceivable;
import gov.va.med.hds.querytool.dao.ICompleteQueryDataDAO;
import gov.va.med.hds.querytool.dao.QueryDAOException;

import javax.swing.table.TableModel;

/**
 * TableModel adapter for a report results model.
 */
public interface IResultsTable extends TableModel {
	static final int ASCENDING = 1;
	static final int DESCENDING = -1;
	
	/**
	 * Sets the column to be sorted on by column position.
	 * @param columnPosition
	 * @param direction
	 */
	void setSortColumn(int columnPosition, int direction) throws QueryDAOException;
	
	/**
	 * Gets the column position that the model is currently sorted on.
	 * @return columnPosition
	 */
	int getSortColumnPosition();
	
	/**
	 * @return sortDirection
	 */
	int getSortDirection();
	
	/**
	 * Clears the buffered items in the model.
	 */
	void reset() throws QueryDAOException;
	
	/**
	 * Sets the sorted column of the model.
	 * @param field
	 * @param ascending
	 */
	void setSortColumn(IDisplayField field, boolean ascending) throws QueryDAOException;
	
	/**
	 * Sets the completed query data DAO.
	 * @param completeQueryDataDAO
	 */
	void setDAOModel(ICompleteQueryDataDAO completeQueryDataDAO);
	
	/**
	 * @param row
	 * @param 
	 */
	void getDetailRequest(int row, IDetailReceivable detailReceivable);

	/**
	 * @param row
	 * @param 
	 */
	void getPatientDetailRequest(int row, IPatDetailReceivable patDetailReceivable);

	/**
	 * @param runtime
	 */
	void setRuntime(IRuntimeContext runtime);
	
	/**
	 * Removes the dependence on the models.
	 */
	void clearModels();

}
