package gov.va.med.hds.cd.rpclogger.test;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.test.CallFactory;
import gov.va.med.hds.util.ExceptionStackTrace;

import java.util.ArrayList;

/**
 * Model that generates RPC calls
 */
public class RpcGeneratorModel {
	/**
	 * Constructor
	 * 
	 * @param mumps
	 */
	public RpcGeneratorModel(IMumps mumps){
		this.mumps = mumps;
	}
	/**
	 * Starts a "Generator Thread" that creates and calls RPCs
	 * @throws Exception
	 */
	public void start() throws Exception{
		factory = CallFactory.getInstance();
		doGenerate = true;
		generator = new Thread(){
			private int counter = 0;
			public void run(){
				setName("Generator Thread");
				while(doGenerate){
					try {
						ICallRequest request = factory.create(counter%factory.getCount()).request;
						counter++;
						mumps.callRPC(request.getCallName(),request.getParams());
						notifyListeners(request);
						
					} catch (Exception e) {
						System.err.println(ExceptionStackTrace.getStackTrace(e));
					}
					Thread.yield();
				}
			}
			private void notifyListeners(ICallRequest request) {
				for(int i=0;i<listeners.size();i++)
					((IRpcGeneratorModelListener)listeners.get(i)).generated(request);
			}
		};
		generator.start();
	}
	/**
	 * Stops generating RPCs
	 *
	 */
	public void stop(){
		doGenerate = false;
	}
	/**
	 * Adds a listener that gets notified after each RPC is called.
	 * @param listener
	 */
	public void addListener(IRpcGeneratorModelListener listener){
		listeners.add(listener);
	}
	/**
	 * Removes the listener.
	 * @param listener
	 */
	public void removeListener(IRpcGeneratorModelListener listener){
		listeners.remove(listener);
	}
	ArrayList listeners = new ArrayList();
	CallFactory factory;
	Thread generator;
	boolean doGenerate;
	IMumps mumps;
}
