package gov.va.med.hds.cd.rpclogger.uitest;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.rpclogger.IRpcLoggerConstants;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.rpclogger.test.IRpcGeneratorModelListener;
import gov.va.med.hds.cd.rpclogger.test.RpcGeneratorModel;
import gov.va.med.hds.util.ExceptionStackTrace;

import java.util.ArrayList;

import javax.swing.AbstractListModel;
import javax.swing.JComponent;
import javax.swing.JOptionPane;

/**
 * A perspective that generates RPCs for testing purposes
 */
public class RpcGeneratorPerspective extends javax.swing.JPanel implements IVisualComponent {
	private static final int MIN_LIST_SIZE = 10;

   	/**
    * Constructor
    */
    public RpcGeneratorPerspective(){
        initComponents();
        try {
			hookupModels();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Could not hook up generator models.\n"+ExceptionStackTrace.getStackTrace(e), "RpcGeneratorPerspective Error", JOptionPane.ERROR_MESSAGE); 
		}
    }
	private void hookupModels() throws Exception {
		IPluginDescriptor desc = desktop.getRuntime().getPluginRegistry().getPluginDescriptor(IRpcLoggerConstants.PLUGIN_ID);
		IMumps mumps = ((IMumpsFactory) desktop.getRuntime().getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
		generator = new RpcGeneratorModel(mumps);
		rpcListModel = new RpcGeneratorListModel(generator);
		callList.setModel(rpcListModel);
		initMaxListLength();
	}
	private void initMaxListLength() {
		maxListLength = Integer.parseInt(displayableCallCountTextField.getText());
		rpcListModel.fixMaxListSize(maxListLength);
	}
	private RpcGeneratorListModel rpcListModel;
	private RpcGeneratorModel generator;
	private IDesktop desktop;
	private int maxListLength = 0;
	private boolean generate = false;
	private final String START_BUTTON_START_GENERATE = "Start";
	private final String START_BUTTON_STOP_GENERATE = "Stop";
	private void setCallsGeneratedCount(long count){
		callsGeneratedTextField.setText(String.valueOf(count));
		callsGeneratedTextField.validate();
	}
	private void toggleGeneration(){
		if(generate){
			startButton.setText(START_BUTTON_START_GENERATE);
			try {
				generator.stop();
				startButton.validate();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Could not stop generator\n"+ExceptionStackTrace.getStackTrace(e), "RpcGeneratorPerspective Error", JOptionPane.ERROR_MESSAGE); 
			}
			generate = false;
		}else{
			startButton.setText(START_BUTTON_STOP_GENERATE);
			startButton.validate();
			try {
				generator.start();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Could not start generator\n"+ExceptionStackTrace.getStackTrace(e), "RpcGeneratorPerspective Error", JOptionPane.ERROR_MESSAGE); 
			}
			generate = true;
		}
	}
	class RpcGeneratorListModel extends AbstractListModel implements IRpcGeneratorModelListener{
		private RpcGeneratorModel model;
		public RpcGeneratorListModel(RpcGeneratorModel model){
			this.model = model;
			model.addListener(this);
		}
		public void fixMaxListSize(int maxListLength) {
			synchronized(list){
				if(list.size() > maxListLength){
					int diff = list.size() - maxListLength;
					for(int i = 0;i<diff;i++){
						list.remove(0);
					}
					this.fireIntervalRemoved(this,0,diff);
				}
			}
		}
		public int getSize() {
			return list.size();
		}
		public Object getElementAt(int index) {
			return list.get(index);
		}
		public void generated(ICallRequest request) {
			synchronized(list){
				if(list.size() == maxListLength){
					list.remove(0);
					this.fireIntervalRemoved(this,0,0);
				}
				list.add(request.getCallName());
				this.fireIntervalAdded(this,list.size(),list.size());
				setCallsGeneratedCount(++generatedCount);
			}
		}
		public void clear(){
			int size = list.size();
			list.clear();
			this.fireIntervalRemoved(this,0,list.size());
		}
		ArrayList list = new ArrayList();
		long generatedCount = 0;
	}
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonPanel = new javax.swing.JPanel();
        startButton = new javax.swing.JToggleButton();
        clearButton = new javax.swing.JButton();
        infoPanel = new javax.swing.JPanel();
        callsGeneratedCount = new javax.swing.JLabel();
        callsGeneratedTextField = new javax.swing.JTextField();
        displayableCallCountLabel = new javax.swing.JLabel();
        displayableCallCountTextField = new javax.swing.JTextField();
        listPanel = new javax.swing.JPanel();
        listScrollPane = new javax.swing.JScrollPane();
        callList = new javax.swing.JList();

        setLayout(new java.awt.BorderLayout());

        buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        buttonPanel.setToolTipText("Execute Buttons");
        startButton.setText("Start");
        startButton.setToolTipText("Click to start generating calls.");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(startButton);

        clearButton.setText("Clear");
        clearButton.setToolTipText("Click to clear the call list");
        clearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(clearButton);

        add(buttonPanel, java.awt.BorderLayout.NORTH);

        infoPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        infoPanel.setToolTipText("Information Panel");
        callsGeneratedCount.setText("Calls Generated");
        infoPanel.add(callsGeneratedCount);

        callsGeneratedTextField.setColumns(4);
        callsGeneratedTextField.setEditable(false);
        callsGeneratedTextField.setText("0");
        callsGeneratedTextField.setToolTipText("Number of generated calls.");
        infoPanel.add(callsGeneratedTextField);

        displayableCallCountLabel.setText("Displayable Call Count");
        infoPanel.add(displayableCallCountLabel);

        displayableCallCountTextField.setColumns(4);
        displayableCallCountTextField.setText("100");
        displayableCallCountTextField.setToolTipText("Enter the number of calls displayed here.");
        displayableCallCountTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                displayableCallCountTextFieldActionPerformed(evt);
            }
        });

        infoPanel.add(displayableCallCountTextField);

        add(infoPanel, java.awt.BorderLayout.SOUTH);

        listPanel.setLayout(new java.awt.BorderLayout());

        listScrollPane.setViewportBorder(new javax.swing.border.EtchedBorder());
        callList.setToolTipText("Calls show up here");
        listScrollPane.setViewportView(callList);

        listPanel.add(listScrollPane, java.awt.BorderLayout.CENTER);

        add(listPanel, java.awt.BorderLayout.CENTER);

    }//GEN-END:initComponents

    private void displayableCallCountTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_displayableCallCountTextFieldActionPerformed
		int value = Integer.parseInt(displayableCallCountTextField.getText());
		if(value < MIN_LIST_SIZE){
			displayableCallCountTextField.setText(String.valueOf(MIN_LIST_SIZE));
			displayableCallCountTextField.validate();
		}
		initMaxListLength();
    }//GEN-LAST:event_displayableCallCountTextFieldActionPerformed

    private void clearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearButtonActionPerformed
		rpcListModel.clear();
    }//GEN-LAST:event_clearButtonActionPerformed

    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed
		toggleGeneration();
    }//GEN-LAST:event_startButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane listScrollPane;
    private javax.swing.JTextField callsGeneratedTextField;
    private javax.swing.JLabel callsGeneratedCount;
    private javax.swing.JLabel displayableCallCountLabel;
    private javax.swing.JToggleButton startButton;
    private javax.swing.JPanel infoPanel;
    private javax.swing.JTextField displayableCallCountTextField;
    private javax.swing.JButton clearButton;
    private javax.swing.JPanel listPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JList callList;
    
	public JComponent getComponent() {
		return this;
	}
	/* @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object) */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception {
	}
    // End of variables declaration//GEN-END:variables
    
}
