package gov.va.med.hds.cd.rpclogger;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

import java.awt.BorderLayout;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

class ListsPanel extends JSplitPane {
	private static final String RPC_EXT_PT = "gov.va.med.hds.cd.m.rpc";
	private static final String RPC_TAG = "rpc";
	private static final String VALUE_ATTR_TAG = "value";
	private static final String OPTION_CONTEXT_EXT_PT = "gov.va.med.hds.cd.m.optionContext";
	private static final String OPTION_CONTEXT_TAG_NAME = "optionContext";
	private static final String NAME_ATTR_TAG = "name";
	private static final String MATCH_ATTR_TAG = "match";
	private static final String VERSION_ATTR_TAG = "version";

	private IRuntimeContext runtime;
	private DefaultMutableTreeNode optionTreeNode;
	private DefaultMutableTreeNode rpcTreeNode;
	public ListsPanel(String name, IRuntimeContext runtime) {
		super();
		this.runtime = runtime;
		setName(name);
		buildModels();
		initComponents();
	}
	private void buildModels() {
		optionTreeNode = new DefaultMutableTreeNode();
		rpcTreeNode = new DefaultMutableTreeNode();
		createOptionTree();
		createRpcTree();
	}
	private void createRpcTree() {
		Map map = createRPCOptionMap(runtime.getPluginRegistry());
		createTreeFromMap(map,rpcTreeNode);
	}
	private void createOptionTree() {
		Map map = createOptionRPCMap(runtime.getPluginRegistry());
		createTreeFromMap(map,optionTreeNode);
	}
	private void createTreeFromMap(Map map,DefaultMutableTreeNode topNode) {
		Set keys = map.keySet();
		Iterator it = keys.iterator();
		DefaultMutableTreeNode branchNode;
		while (it.hasNext()) {
			String key = (String) it.next();
			branchNode = new DefaultMutableTreeNode(key,true);
		
			LinkedList values = (LinkedList) map.get(key);
			
			Iterator rpcsIt = values.iterator();
			DefaultMutableTreeNode leafNode;
			while (rpcsIt.hasNext()) {
				leafNode = new DefaultMutableTreeNode(rpcsIt.next(),false);
				branchNode.add(leafNode);
			}
			branchNode.setUserObject("("+branchNode.getChildCount()+") "+key);
			topNode.add(branchNode);
		}		
	}
	private void initComponents() {
		setLeftComponent(new UniqueOptionContextsDisplay());
		setRightComponent(new UniqueRpcsDisplay());
	}
	class UniqueOptionContextsDisplay extends JPanel {
		private static final String OPTION_CONTEXTS_LABEL = "Unique Option Contexts";
		private JTree optionsTree;

		public UniqueOptionContextsDisplay() {
			super();
			initComponents();
		}

		private void initComponents() {
			this.setLayout(new BorderLayout());
			
			add(new JLabel(OPTION_CONTEXTS_LABEL+" ("+optionTreeNode.getChildCount()+")"), BorderLayout.NORTH);
			optionsTree = new JTree(optionTreeNode);
			optionsTree.setShowsRootHandles(false);
			add(new JScrollPane(optionsTree), BorderLayout.CENTER);
		}
	}
	class UniqueRpcsDisplay extends JPanel {
		private static final String UNIQUE_RPCS_LABEL = "Unique RPCs";
		private JTree rpcTree;

		public UniqueRpcsDisplay() {
			super();
			initComponents();
		}

		private void initComponents() {
			this.setLayout(new BorderLayout());
			add(new JLabel(UNIQUE_RPCS_LABEL+" ("+rpcTreeNode.getChildCount()+")"), BorderLayout.NORTH);
			rpcTree = new JTree(rpcTreeNode);
			rpcTree.setShowsRootHandles(false);
			add(new JScrollPane(rpcTree), BorderLayout.CENTER);
		}
	}
	private Map createRPCOptionMap(IPluginRegistry registry) {
		IPluginDescriptor[] descs = registry.getPluginDescriptors();
		TreeMap map = new TreeMap();
		for (int i = 0; i < descs.length; i++) {
			String option = getOptionContext(descs[i]);
			if (option != null) {
				LinkedList rpcs = getRpcs(descs[i]);
				Iterator it = rpcs.iterator();
				while(it.hasNext()){
					Object rpc = it.next();
					if(map.containsKey(rpc)){	
						LinkedList l = (LinkedList)map.get(rpc);
						if(!listContainsString(l,option))
							l.add(option);	
					}else{
						LinkedList l = new LinkedList();
						l.add(option);
						map.put(rpc,l);
					}
				}
			}
		}
		sortMap(map);
		return map;
	}
	private boolean listContainsString(LinkedList list, String val){
		if(list != null){
		}
		Iterator it = list.iterator();
		while(it.hasNext()){
			if(it.next().equals(val))
				return true;
		}
		return false;
	}
	private Map createOptionRPCMap(IPluginRegistry registry) {
		IPluginDescriptor[] descs = registry.getPluginDescriptors();
		TreeMap map = new TreeMap();
		for (int i = 0; i < descs.length; i++) {
			String option = getOptionContext(descs[i]);
			if (option != null) {
				LinkedList rpcs = getRpcs(descs[i]);
				if (map.containsKey(option))
					appendRpcs(option, map, rpcs);
				else
					map.put(option, rpcs);
			}
		}
		sortMap(map);
		return map;
	}
	private void sortMap(TreeMap map) {
		Set keys = map.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			LinkedList values = (LinkedList) map.get(key);
			Collections.sort(values);
		}		
	}
	private void appendRpcs(String option, TreeMap map, LinkedList toAppend) {
		LinkedList rpcs = (LinkedList) map.get(option);
		if (rpcs != null) {
			Iterator it = toAppend.iterator();
			while (it.hasNext()) {
				String r = (String) it.next();
				if (!rpcs.contains(r))
					rpcs.add(r);
			}
		}
	}
	private LinkedList getRpcs(IPluginDescriptor descriptor) {
		LinkedList list = new LinkedList();
		IExtension[] exts = descriptor.getExtensions();
		if (exts != null) {
			for (int i = 0; i < exts.length; i++) {
				if (exts[i].getExtensionPointUniqueIdentifier().equals(RPC_EXT_PT)) {
					IExtension rpc = exts[i];
					IConfigurationElement[] elems = rpc.getConfigurationElements();
					for (int j = 0; j < elems.length; j++) {
						if (elems[j].getName().equals(RPC_TAG)) {
							String value = elems[j].getAttribute(VALUE_ATTR_TAG);
							list.add(value);
						}
					}
				}
			}
		}
		return list;
	}
	private String getOptionContext(IPluginDescriptor descriptor) {
		IExtension[] exts = descriptor.getExtensions();
		for (int i = 0; i < exts.length; i++) {
			if (exts[i].getExtensionPointUniqueIdentifier().equals(OPTION_CONTEXT_EXT_PT)) {
				IExtension oc = exts[i];
				IConfigurationElement[] elems = oc.getConfigurationElements();
				for (int j = 0; j < elems.length; j++) {
					if (elems[j].getName().equals(OPTION_CONTEXT_TAG_NAME)) {
						String ocName = elems[j].getAttribute(NAME_ATTR_TAG);
						String version = elems[j].getAttribute(VERSION_ATTR_TAG);
						String match = elems[j].getAttribute(MATCH_ATTR_TAG);
						return ocName + " " + "Version=" + version + " " + "Match=" + match;
					}
				}
			}
		}
		return null;
	}

}
