package gov.va.med.hds.cd.rpclogger;

import gov.va.med.hds.cd.m.RPCLogEventObject;
import gov.va.med.hds.cd.m.TimedResult;
import gov.va.med.hds.util.mumps.StringUtil;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

class RpcListWriter {
	private static final String PER_RPC_TAG = "rpc";
	private static final String ROOT_TAG = "rpclist";
	private static final String PARAM_TAG = "param";
	private static final String NAME_TAG = "name";
	private static final String RESULT_TAG = "result";
	private static final String CONTEXT_TAG = "context";
	private static final String PARAMS_TAG = "params";
	private static final String SIMPLE_TAG = "simple";
	private static final String LIST_TAG = "list";
	private static final String MAP_TAG = "map";
	private static final String SET_TAG = "set";
	private static final String LIST_ITEM_TAG = "item";
	private static final String MAP_ITEM_TAG = "item";
	private static final String MAP_ITEM_KEY_TAG = "key";
	private static final String MAP_ITEM_VALUE_TAG = "value";
	public RpcListWriter(RPCLogEventObject[] rpcList, File file) throws Exception {
		FileOutputStream fos = new FileOutputStream(file);
		write(fos, rpcList);
	}
	private void write(OutputStream os, RPCLogEventObject[] rpcList) throws Exception {
		Document document = DocumentHelper.createDocument();
		Element root = document.addElement(ROOT_TAG);
		if (rpcList != null) {
			for (int i = 0; i < rpcList.length; i++) {
				addRpc(root, rpcList[i]);
			}
		}
		XMLWriter writer = new XMLWriter(os, OutputFormat.createPrettyPrint());
		writer.write(document);
		writer.flush();
		writer.close();
	}
	private void addRpc(Element parent, RPCLogEventObject call) {
		Element elem = parent.addElement(PER_RPC_TAG);
		constructName(elem, call.request.getCallName());
		constructContext(elem, call.request.getContext());
		constructParams(elem, call.request.getParams());
		constructResult(elem, call.result);
	}
	private void constructResult(Element parent, TimedResult result) {
		Element name = parent.addElement(RESULT_TAG);
		name.addCDATA(result.result);
	}
	private void constructName(Element parent, String rpcName) {
		Element name = parent.addElement(NAME_TAG);
		name.addText(rpcName);
	}
	private void constructContext(Element parent, String context) {
		if (!context.equals("")) {
			Element contextElement = parent.addElement(CONTEXT_TAG);
			contextElement.addText(context);
		}
	}
	private void constructParams(Element parent, List params) {
		Element elem = parent.addElement(PARAMS_TAG);
		if (params != null) {
			Iterator it = params.iterator();
			while (it.hasNext())
				constructParam(it.next(), elem);
		}
	}
	private void constructParam(Object o, Element parent) {
		Element param = parent.addElement(PARAM_TAG);
		if (o instanceof List) {
			Element list = param.addElement(LIST_TAG);
			Iterator it = ((List) o).iterator();
			while (it.hasNext()) {
				Element item = list.addElement(LIST_ITEM_TAG);
				item.addCDATA(StringUtil.replaceCDATATags(it.next().toString()));
			}
		} else if (o instanceof Set) {
			Element set = param.addElement(SET_TAG);
			Iterator it = ((Set) o).iterator();
			while (it.hasNext()) {
				Element item = set.addElement(LIST_ITEM_TAG);
				item.addCDATA(StringUtil.replaceCDATATags(it.next().toString()));
			}
		} else if (o instanceof Map) {
			Element mapElem = param.addElement(MAP_TAG);
			Map map = (Map) o;
			Set set = map.keySet();
			Iterator it = set.iterator();
			while (it.hasNext()) {
				Object key = it.next();
				Object value = map.get(key);
				Element item = mapElem.addElement(MAP_ITEM_TAG);
				Element keyElem = item.addElement(MAP_ITEM_KEY_TAG);
				keyElem.addText(key.toString());
				Element valueElem = item.addElement(MAP_ITEM_VALUE_TAG);
				valueElem.addCDATA(StringUtil.replaceCDATATags(value.toString()));
			}
		} else {
			Element simple = param.addElement(SIMPLE_TAG);
			simple.addCDATA(StringUtil.replaceCDATATags(o.toString()));
		}
	}

}
