package gov.va.med.hds.cd.rpclogger;

import gov.va.med.hds.cd.m.CallRequestCacheEntry;
import gov.va.med.hds.cd.m.CallRequestCacheException;
import gov.va.med.hds.cd.m.ICallRequestCache;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.IRPCLogListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.m.RPCLogEventObject;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.util.ExceptionStackTrace;
import gov.va.med.hds.util.io.FileUtilities;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Rectangle;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.LinkedList;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.html.HTMLEditorKit;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

class RpcLogger extends JPanel {
	private static final int MIN_LIST_SIZE = 10;
	private String CALL_TRANSFORM_RESOURCE_PATH =
		"/gov/va/med/hds/cd/m/internal/xml/CallQueueRequestCacheSchemaXForm.xslt";
	private String RESOURCE_BUNDLE_NAME = "gov.va.med.hds.cd.rpclogger.RpcLogger";
	private final String ENABLE_BUTTON_ENABLE_LOGGING = "Enable Logging";
	private final String ENABLE_BUTTON_DISABLE_LOGGING = "Disable Logging";

	private RpcResultsTableModel resultsTableModel;
	private boolean enableLogging = false;
	private JScrollPane callScrollPane;
	private JSplitPane centerSplitPanel;
	private JPanel topPanel;
	private JTextField maxLogEntriesTextField;
	private JScrollPane contentScrollPane;
	private JTable callTable;
	private JEditorPane contentEditorPane;
	private JButton receivedCallIconButton;
	private JPanel centerPanel;
	private JButton saveLogButton;
	private JButton clearLogButton;
	private JLabel maxLogEntriesLabel;
	private JToggleButton enableLoggingToggleButton;
	private JPanel bottomPanel;
	private ResourceBundle resources;
	private int allowableRowCount;
	private IRuntimeContext runtime;
	private LinkedList calls = new LinkedList();
	private Transformer xformer;
	private IMumps mumps;

	public RpcLogger(String name, IRuntimeContext runtime) {
		super();
		this.runtime = runtime;
		setName(name);
		initComponents();
		try {
			hookupModels();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(
				null,
				"Could not hook up rpc logger models.\n" + ExceptionStackTrace.getStackTrace(e),
				"RpcLoggerPerspective Error",
				JOptionPane.ERROR_MESSAGE);
		}
		buildTransformer();
		resources = ResourceBundle.getBundle(RESOURCE_BUNDLE_NAME);
		setUpToolTips();
	}
	private void initComponents() {
		topPanel = new JPanel();
		receivedCallIconButton = new JButton();
		enableLoggingToggleButton = new JToggleButton();
		clearLogButton = new JButton();
		saveLogButton = new JButton();
		bottomPanel = new JPanel();
		maxLogEntriesLabel = new JLabel();
		maxLogEntriesTextField = new JTextField();
		centerPanel = new JPanel();
		centerSplitPanel = new JSplitPane();
		callScrollPane = new JScrollPane();
		callTable = new JTable();
		contentScrollPane = new JScrollPane();
		contentEditorPane = new JEditorPane();

		setLayout(new java.awt.BorderLayout());

		topPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		receivedCallIconButton.setToolTipText("Lights up when a call is being processed");
		topPanel.add(receivedCallIconButton);

		enableLoggingToggleButton.setText("Enable Logging");
		enableLoggingToggleButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				enableLoggingToggleButtonActionPerformed(evt);
			}
		});

		topPanel.add(enableLoggingToggleButton);

		clearLogButton.setText("Clear");
		clearLogButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				clearLogButtonActionPerformed(evt);
			}
		});

		topPanel.add(clearLogButton);

		saveLogButton.setText("Save Current Log");
		saveLogButton.setToolTipText("Saves the current recorded log to a file");
		saveLogButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveLogButtonActionPerformed(evt);
			}
		});

		topPanel.add(saveLogButton);

		add(topPanel, java.awt.BorderLayout.NORTH);

		bottomPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

		maxLogEntriesLabel.setText("Max Log Entries");
		bottomPanel.add(maxLogEntriesLabel);

		maxLogEntriesTextField.setColumns(4);
		maxLogEntriesTextField.setText("500");
		maxLogEntriesTextField.setToolTipText("Enter the number of maximum saved log entries");
		maxLogEntriesTextField.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				maxLogEntriesTextFieldActionPerformed(evt);
			}
		});

		bottomPanel.add(maxLogEntriesTextField);

		add(bottomPanel, java.awt.BorderLayout.SOUTH);

		centerPanel.setLayout(new java.awt.BorderLayout());

		callTable
			.setModel(
				new javax
				.swing
				.table
				.DefaultTableModel(new Object[][] { { null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }, {
				null, null, null }
		}, new String[] { "Name", "Success", "Millis" }) {
			Class[] types =
				new Class[] {
					java.lang.String.class,
					java.lang.Object.class,
					java.lang.Long.class };
			boolean[] canEdit = new boolean[] { false, false, false };

			public Class getColumnClass(int columnIndex) {
				return types[columnIndex];
			}

			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return canEdit[columnIndex];
			}
		});
		callTable.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				callTableMouseClicked(evt);
			}
		});

		callScrollPane.setViewportView(callTable);
		centerSplitPanel.setLeftComponent(callScrollPane);
		contentScrollPane.setViewportView(contentEditorPane);
		centerSplitPanel.setRightComponent(contentScrollPane);
		centerPanel.add(centerSplitPanel, java.awt.BorderLayout.CENTER);

		add(centerPanel, java.awt.BorderLayout.CENTER);
	}
	private void hookupModels() throws Exception {
		IPluginDescriptor desc =
			runtime.getPluginRegistry().getPluginDescriptor(IRpcLoggerConstants.PLUGIN_ID);
		mumps = ((IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
		resultsTableModel = new RpcResultsTableModel();
		callTable.setModel(resultsTableModel);
		ListSelectionModel selector = callTable.getSelectionModel();
		selector.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		selector.addListSelectionListener(resultsTableModel);
		initAllowableRowCount();
		fixReceiveButton();
		contentEditorPane.setEditorKit(new HTMLEditorKit());
		contentEditorPane.setContentType("text/html");
	}
	private void fixReceiveButton() {
		receivedCallIconButton.setBorderPainted(false);
		receivedCallIconButton.setBackground(Color.GRAY);
		Rectangle bounds = receivedCallIconButton.getBounds();
		bounds.width = 10;
		receivedCallIconButton.setBounds(bounds);
		receivedCallIconButton.setEnabled(false);
	}
	private void initAllowableRowCount() {
		allowableRowCount = Integer.parseInt(maxLogEntriesTextField.getText());
		resultsTableModel.fixAllowableRowCount(allowableRowCount);
	}
	private void toggleLogging() {
		if (enableLogging) {
			enableLoggingToggleButton.setText(ENABLE_BUTTON_ENABLE_LOGGING);
			try {
				resultsTableModel.stopListening();
				enableLoggingToggleButton.validate();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(
					null,
					"Could not disable logging\n" + ExceptionStackTrace.getStackTrace(e),
					"RpcLoggePerspective Error",
					JOptionPane.ERROR_MESSAGE);
			}
			enableLogging = false;
		} else {
			enableLoggingToggleButton.setText(ENABLE_BUTTON_DISABLE_LOGGING);
			enableLoggingToggleButton.validate();
			try {
				resultsTableModel.startListening();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(
					null,
					"Could not enable logging\n" + ExceptionStackTrace.getStackTrace(e),
					"RpcLoggePerspective Error",
					JOptionPane.ERROR_MESSAGE);
			}
			enableLogging = true;
		}
	}

	private void setUpToolTips() {
		maxLogEntriesTextField.setToolTipText(resources.getString("rpclogger.loglimit.tip"));
		callTable.setToolTipText(resources.getString("rpclogger.logtable.tip"));
		contentEditorPane.setToolTipText(resources.getString("rpclogger.detail.tip"));
		receivedCallIconButton.setToolTipText(resources.getString("rpclogger.receiving.tip"));
		clearLogButton.setToolTipText(resources.getString("rpclogger.clearlog.tip"));
		enableLoggingToggleButton.setToolTipText(
			resources.getString("rpclogger.enablelogging.tip"));
		callScrollPane.setToolTipText(resources.getString("rpclogger.emptyLog.tip"));
	}
	private void saveLogButtonActionPerformed(
		java.awt.event.ActionEvent evt) { //GEN-FIRST:event_saveLogButtonActionPerformed
		File file = getSaveFile();
		if (file != null) {
			try {
				getRootPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				RPCLogEventObject[] list = null;
				if (calls.size() > 0) {
					list = new RPCLogEventObject[calls.size()];
					for (int i = 0; i < calls.size(); i++)
						list[i] = (RPCLogEventObject) calls.get(i);
				}
				RpcListWriter w = new RpcListWriter(list, file);
				getRootPane().getRootPane().setCursor(null);
			} catch (Exception e) {
				getRootPane().getRootPane().setCursor(null);
				JOptionPane.showMessageDialog(
					this,
					"Could not save to the file\n" + e.getMessage(),
					"RPCLogger Save Log Message",
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private void callTableMouseClicked(java.awt.event.MouseEvent evt) {
	}

	private File getSaveFile() {
		File file = null;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setApproveButtonText("Save");
		fileChooser.setDialogTitle("Save Current Log");
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		FileFilter filter = new FileFilter() {
			public boolean accept(File f) {
				if (f == null)
					return false;
				if (f.isDirectory()) {
					return true;
				}
				String ext = FileUtilities.getFileExtension(f);
				if ((ext != null) && (ext.equals("xml"))) {
					return true;
				}
				return false;
			}
			public String getDescription() {
				return "XML Files (.xml)";
			}
		};
		fileChooser.setFileFilter(filter);
		int val = fileChooser.showSaveDialog(this);
		if (val == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		}
		return file;
	}

	private void maxLogEntriesTextFieldActionPerformed(java.awt.event.ActionEvent evt) {
		int value = Integer.parseInt(maxLogEntriesTextField.getText());
		if (value < MIN_LIST_SIZE) {
			maxLogEntriesTextField.setText(String.valueOf(MIN_LIST_SIZE));
			maxLogEntriesTextField.validate();
		}
		initAllowableRowCount();
	}

	private void clearLogButtonActionPerformed(java.awt.event.ActionEvent evt) {
		resultsTableModel.clear();
	}

	private void enableLoggingToggleButtonActionPerformed(java.awt.event.ActionEvent evt) {
		toggleLogging();
	}
	private void buildTransformer() {
		TransformerFactory factory = TransformerFactory.newInstance();
		StreamSource style =
			new StreamSource(
				RpcLoggerPerspective.class.getResourceAsStream(CALL_TRANSFORM_RESOURCE_PATH));
		xformer = null;
		try {
			xformer = factory.newTransformer(style);
		} catch (TransformerConfigurationException e) {
			JOptionPane.showMessageDialog(
				null,
				"Could not create xml transformer.\n" + ExceptionStackTrace.getStackTrace(e),
				"RpcLoggerPerspective Error",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	class RpcResultsTableModel
		extends DefaultTableModel
		implements ListSelectionListener, IRPCLogListener {
		private static final String META_TAG_LINE =
			"<META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">";

		public RpcResultsTableModel() {
			this.setColumnIdentifiers(new String[] { "Name", "Success", "Millis" });
		}
		public void startListening() throws MumpsException {
			mumps.addLogListener(this);
		}
		public void stopListening() throws MumpsException {
			mumps.removeLogListener(this);
		}
		public void callExecuted(RPCLogEventObject call) {
			receivedCallIconButton.setBackground(Color.GREEN);
			addCallToTable(call);
			receivedCallIconButton.setBackground(Color.GRAY);
		}
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return false;
		}
		synchronized public void fixAllowableRowCount(int allowableRowCount) {
			if (getRowCount() > allowableRowCount) {
				int diff = getRowCount() - allowableRowCount;
				for (int i = 0; i < diff; i++) {
					removeRow(0);
					calls.remove(0);
				}
				fireTableRowsDeleted(0, diff);
			}
		}
		synchronized public void clear() {
			int size = this.getRowCount();
			for (int i = 0; i < size; i++) {
				removeRow(0);
				calls.remove(0);
			}
			fireTableRowsDeleted(0, size - 1);
		}
		public void valueChanged(ListSelectionEvent e) {
			if (!e.getValueIsAdjusting()) {
				if (callTable.getSelectedRow() >= 0)
					displayDetail((RPCLogEventObject) getValueAt(callTable.getSelectedRow(), 0));
				else
					contentEditorPane.setText("");
			}
		}
		synchronized private void addCallToTable(RPCLogEventObject call) {
			Object[] row = new Object[3];
			row[0] = call;
			if (call.exception != null) {
				row[1] = call.exception;
				row[2] = new Integer(0);
			} else {
				row[1] = "OK";
				row[2] = new Long(call.result.elapsedTime);
			}
			if (getRowCount() > 0 && getRowCount() >= allowableRowCount) {
				this.removeRow(0);
				calls.remove(0);
			}
			addRow(row);
			calls.add(call);
			fireTableRowsInserted(getRowCount() - 1, getRowCount() - 1);
		}
		private RPCLogEventObject pendingDisplayable;
		private void displayDetail(RPCLogEventObject call) {
			pendingDisplayable = call;
			contentEditorPane.setText("Transforming this RPC please wait...");
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					try {
						synchronized (callTable) {
							if (pendingDisplayable.exception != null)
								contentEditorPane.setText(
									ExceptionStackTrace.getStackTrace(
										pendingDisplayable.exception));
							else
								contentEditorPane.setText(getDisplayText(pendingDisplayable));
						}
					} catch (Exception e) {
						JOptionPane.showMessageDialog(
							null,
							"Could not output detail text.\n"
								+ ExceptionStackTrace.getStackTrace(e),
							"RpcLoggerPerspective Error",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			});
		}
		private String getDisplayText(RPCLogEventObject call) throws Exception {
			Document document = DocumentHelper.createDocument();
			Element root = document.addElement(ICallRequestCache.CALL_CACHE_ROOT_ELEMENT_NAME);
			CallRequestCacheEntry entry = null;
			try {
				entry = new CallRequestCacheEntry(call.request, call.result);
			} catch (CallRequestCacheException e) {
				JOptionPane.showMessageDialog(
					null,
					"Could not create detail xml.\n" + ExceptionStackTrace.getStackTrace(e),
					"RpcLoggerPerspective Error",
					JOptionPane.ERROR_MESSAGE);
			}
			entry.construct(root);
			StringWriter strWriter = new StringWriter();
			XMLWriter xmlWriter = new XMLWriter(strWriter, OutputFormat.createPrettyPrint());
			try {
				xmlWriter.write(document);
			} catch (IOException e) {
				JOptionPane.showMessageDialog(
					null,
					"Could not write xml.\n" + ExceptionStackTrace.getStackTrace(e),
					"RpcLoggerPerspective Error",
					JOptionPane.ERROR_MESSAGE);
			}
			StringWriter finalOutput = new StringWriter();
			try {
				xformer.transform(
					new StreamSource(new StringReader(strWriter.toString())),
					new StreamResult(finalOutput));
			} catch (TransformerException e) {
				JOptionPane.showMessageDialog(
					null,
					"Transform failed.\n" + ExceptionStackTrace.getStackTrace(e),
					"RpcLoggerPerspective Error",
					JOptionPane.ERROR_MESSAGE);
			}
			return stripMetaTag(finalOutput.toString());
		}
		private String stripMetaTag(String data) {
			return data.replaceFirst(META_TAG_LINE, "");
		}
	}
}
