package gov.va.med.hds.cd.runtime.internal;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExecutableExtension;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginActivatingException;
import gov.va.med.hds.cd.runtime.logging.ILogger;

public class ConfigurationElement implements IConfigurationElement {

	private IExtension parent;
	private String name;
	private String value;

	private Map attributes = new HashMap();
	private List children = new LinkedList();

	public ConfigurationElement(String name) {
		this(name, null);
	}

	public ConfigurationElement(String name, String value) {
		this.name = name;
		this.value = value;
	}

	public String getAttribute(String name) {
		return getAttributeAsIs(name); // TODO_SB: do resource translation?
	}

	public String getAttributeAsIs(String name) {
		return (String) attributes.get(name);
	}

	public String[] getAttributeNames() {
		return (String[]) attributes.keySet().toArray(
			new String[attributes.size()]);
	}

	public IConfigurationElement[] getChildren() {
		return (IConfigurationElement[]) children.toArray(
			new IConfigurationElement[children.size()]);
	}

	public IConfigurationElement[] getChildren(String name) {
		List filteredChildren = new LinkedList();
		for (Iterator i = children.iterator(); i.hasNext();) {
			IConfigurationElement element = (IConfigurationElement) i.next();
			if (element.getName().equals(name)) {
				filteredChildren.add(element);
			}
		}
		return (IConfigurationElement[]) filteredChildren.toArray(
			new IConfigurationElement[filteredChildren.size()]);
	}

	public IExtension getDeclaringExtension() {
		return parent;
	}

	public void setDeclaringExtension(IExtension parent) {
		this.parent = parent;
		for (Iterator i = children.iterator(); i.hasNext();) {
			ConfigurationElement e = (ConfigurationElement) i.next();
			e.setDeclaringExtension(parent);
		}
	}

	public String getName() {
		return name;
	}

	public String getValue() {
		return getValueAsIs(); // TODO_SB: do resource translation?
	}

	public String getValueAsIs() {
		return value;
	}

	public void addAttribute(String name, String value) {
		attributes.put(name, value);
	}

	public void removeAttribute(String name) {
		attributes.remove(name);
	}

	public void addChild(ConfigurationElement configurationElement) {
		children.add(configurationElement);
		configurationElement.setDeclaringExtension(getDeclaringExtension());
	}

	public void removeChild(ConfigurationElement configurationElement) {
		children.remove(configurationElement);
	}

	public void setValue(String value) {
		this.value = value;
	}

	public Object createExecutableExtension(String propertyName)
		throws Exception {
		return createExecutableExtension(propertyName, null);
	}

	public Object createExecutableExtension(String propertyName, Object data)
		throws Exception {
		IPluginDescriptor descr =
			getDeclaringExtension().getDeclaringPluginDescriptor();
		String pluginID = descr.getUniqueIdentifier();
		Version pluginVersion = descr.getVersion();
		IRuntimeContext runtime = descr.getRuntimeContext();
		ILogger log = runtime.getLogger(getClass());
		log.debug("entering createExecutableExtension("+ propertyName + "," + data +")");
		
		if (!runtime.isPluginActive(pluginID, pluginVersion) && descr.getClassName() != null) {
			try {
				runtime.getPluginInstance(pluginID, pluginVersion);
			} catch (PluginActivatingException e) {
				// NOOP: this is OK, the plugin activation has already been triggered
			}
		}
		String className = getAttribute(propertyName);
		log.debug("creating executable extension from " + descr + " " + propertyName + "=" + className);
		
		ClassLoader loader = descr.getPluginClassLoader();
		Class clazz = Class.forName(className, true, loader);
		Object result = clazz.newInstance();
		if (result instanceof IExecutableExtension) {
			((IExecutableExtension) result).setInitializationData(
				runtime,
				this,
				propertyName,
				data);
		}
		
		log.debug("exiting createExecutableExtension("+ propertyName + "," + data +")");
		return result;
	}
}
