package gov.va.med.hds.cd.runtime.internal;

import java.util.HashMap;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class DelegatingHandler extends DefaultHandler {

	private Map handlers = new HashMap();
	private String currentHandlerId;

	public void addContentHandler(String id, ContentHandler handler) {
		handlers.put(id, handler);
	}

	public void removeContentHandler(String id) {
		handlers.remove(id);
	}

	public void setCurrentHandler(String id) {
		if (id != null && !handlers.containsKey(id))
			throw new IllegalArgumentException();
		currentHandlerId = id;
	}

	public String getCurrentHandlerId() {
		return currentHandlerId;
	}

	public ContentHandler getCurrentHandler() {
		return (ContentHandler) handlers.get(currentHandlerId);
	}

	public void characters(char[] ch, int start, int length) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().characters(ch, start, length);
	}

	public void endDocument() throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().endDocument();
	}

	public void endElement(String namespaceURI, String localName, String qName)
		throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().endElement(namespaceURI, localName, qName);
	}

	public void endPrefixMapping(String prefix) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().endPrefixMapping(prefix);
	}

	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().ignorableWhitespace(ch, start, length);
	}

	public void processingInstruction(String target, String data) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().processingInstruction(target, data);
	}

	public void setDocumentLocator(Locator locator) {
		if (getCurrentHandler() != null) getCurrentHandler().setDocumentLocator(locator);
	}

	public void skippedEntity(String name) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().skippedEntity(name);
	}

	public void startDocument() throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().startDocument();
	}

	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
		throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().startElement(namespaceURI, localName, qName, atts);
	}

	public void startPrefixMapping(String prefix, String uri) throws SAXException {
		if (getCurrentHandler() != null) getCurrentHandler().startPrefixMapping(prefix, uri);
	}
}
