package gov.va.med.hds.cd.runtime.internal;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.VersionComparator;
import gov.va.med.hds.cd.runtime.VersionMatchCondition;

public class PluginDescriptorMap {

	private static final VersionComparator VERSION_COMPARATOR =
		new VersionComparator();

	private Map plugins = new HashMap();
	private Map valueMap = new HashMap();

	public void put(IPluginDescriptor plugin, Object value) {
		valueMap.put(plugin, value);
		getVersions(plugin.getUniqueIdentifier()).add(plugin.getVersion());
	}

	public void remove(IPluginDescriptor plugin) {
		valueMap.remove(plugin);
		SortedSet versions = (SortedSet) plugins.get(plugin.getUniqueIdentifier());
		if (versions == null) return;
		versions.remove(plugin.getVersion());
		if (versions.isEmpty()) plugins.remove(plugin.getUniqueIdentifier());
	}

	public boolean contains(IPluginDescriptor p) {
		return valueMap.containsKey(p);
	}

	public boolean contains(String pluginId) {
		return plugins.containsKey(pluginId);
	}

	public boolean contains(String pluginId, Version version) {
		return contains(pluginId, version, VersionMatchCondition.PERFECT);
	}

	public boolean contains(
		String pluginId,
		Version version,
		VersionMatchCondition condition) {
		if (!contains(pluginId))
			return false;
		SortedSet versionMap = getVersions(pluginId);
		if (condition.equals(VersionMatchCondition.PERFECT)) {
			return versionMap.contains(version);
		} else {
			return !getMatchingVersions(pluginId, version, condition).isEmpty();
		}
	}

	public Object get(IPluginDescriptor p) {
		return valueMap.get(p);
	}

	public Set keySet() {
		return valueMap.keySet();
	}

	private SortedSet getVersions(String pluginId) {
		SortedSet versions = (SortedSet) plugins.get(pluginId);
		if (versions == null) {
			versions = new TreeSet(VERSION_COMPARATOR);
			plugins.put(pluginId, versions);
		}
		return versions;
	}

	private SortedSet getMatchingVersions(
		String pluginId,
		Version v,
		VersionMatchCondition matchCondition) {
		if (matchCondition.equals(VersionMatchCondition.GREATER_OR_EQUAL)) {
			return getVersions(pluginId).tailSet(v);
		}
		if (matchCondition.equals(VersionMatchCondition.COMPATIBLE)) {
			Version highestCompatibleVersionExclusive =
				new Version(v.getMajorComponent() + 1, 0, 0);
			return getVersions(pluginId).subSet(
				v,
				highestCompatibleVersionExclusive);
		}
		if (matchCondition.equals(VersionMatchCondition.EQUIVALENT)) {
			Version highestEquivalentVersionExclusive =
				new Version(
					v.getMajorComponent(),
					v.getMinorComponent() + 1,
					0);
			return getVersions(pluginId).subSet(
				v,
				highestEquivalentVersionExclusive);
		}
		if (matchCondition.equals(VersionMatchCondition.PERFECT)) {
			Version highestPerfectVersionExclusive =
				new Version(
					v.getMajorComponent(),
					v.getMinorComponent(),
					v.getServiceComponent() + 1);
			return getVersions(pluginId).subSet(
				v,
				highestPerfectVersionExclusive);
		}
		return new TreeSet(VERSION_COMPARATOR);
	}

	public int size() {
		return valueMap.size();
	}

	public boolean isEmpty() {
		return size() == 0;
	}

	public void clear() {
		valueMap.clear();
		plugins.clear();
	}
	
	public IPluginDescriptor[] getPluginDescriptors() {
		return (IPluginDescriptor[]) valueMap.keySet().toArray(new IPluginDescriptor[size()]);
	}
}
