package gov.va.med.hds.cd.runtime.internal;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IRuntimeConstants;
import gov.va.med.hds.cd.runtime.InvalidExtensionException;
import gov.va.med.hds.cd.runtime.InvalidExtensionPointException;
import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.InvalidPrerequisiteException;

public class PluginValidation implements IRuntimeConstants {
	public static void validatePluginAttributes(String id, String name, Version version)
		throws InvalidPluginDescriptorException {

		if (id == null || id.length() == 0) {
			throw new InvalidPluginDescriptorException(
				"expected '" + ID_ATTRIBUTE + "' attribute in '" + PLUGIN_ELEMENT + "' element");
		}

		if (name == null || name.length() == 0) {
			throw new InvalidPluginDescriptorException(
				"expected '" + NAME_ATTRIBUTE + "' attribute in '" + PLUGIN_ELEMENT + "' element");
		}

		if (version == null) {
			throw new InvalidPluginDescriptorException(
				"expected '"
					+ VERSION_ATTRIBUTE
					+ "' attribute in '"
					+ PLUGIN_ELEMENT
					+ "' element");
		}
	}

	public static void validatePrerequisiteAttributes(String id, Version version, String match)
		throws InvalidPrerequisiteException {
		if (id == null || id.length() == 0) {
			throw new InvalidPrerequisiteException(
				"expected '"
					+ PLUGIN_ELEMENT
					+ "' attribute in '"
					+ PREREQUISITE_ELEMENT
					+ "' element");
		}

		if (version == null) {
			throw new InvalidPrerequisiteException(
				"expected '"
					+ VERSION_ATTRIBUTE
					+ "' attribute in '"
					+ PREREQUISITE_ELEMENT
					+ "' element");
		}

		if (match != null) {
			if (match.length() == 0 || (!GREATER_OR_EQUAL_MATCH_CONDITION.equals(match)
				&& !COMPATIBLE_MATCH_CONDITION.equals(match)
				&& !EQUIVALENT_MATCH_CONDITION.equals(match)
				&& !PERFECT_MATCH_CONDITION.equals(match))) {
				throw new InvalidPrerequisiteException(
					"expected '"
						+ MATCH_ATTRIBUTE
						+ "' attribute in '"
						+ PREREQUISITE_ELEMENT
						+ "' element to have value '"
						+ GREATER_OR_EQUAL_MATCH_CONDITION
						+ "','"
						+ COMPATIBLE_MATCH_CONDITION
						+ "','"
						+ EQUIVALENT_MATCH_CONDITION
						+ "', or'"
						+ PERFECT_MATCH_CONDITION
						+ "'");
			}
		}
	}

	public static void validateExtensionPointAttributes(String id, String name)
		throws InvalidExtensionPointException {
		if (id == null || id.length() == 0) {
			throw new InvalidExtensionPointException(
				"expected '"
					+ ID_ATTRIBUTE
					+ "' attribute in '"
					+ EXTENSION_POINT_ELEMENT
					+ "' element");
		}

		if (name == null || name.length() == 0) {
			throw new InvalidExtensionPointException(
				"expected '"
					+ NAME_ATTRIBUTE
					+ "' attribute in '"
					+ EXTENSION_POINT_ELEMENT
					+ "' element");
		}

	}

	public static void validateExtensionAttributes(String extensionPointId)
		throws InvalidExtensionException {
		if (extensionPointId == null || extensionPointId.length() == 0) {
			throw new InvalidExtensionException(
				"expected '"
					+ EXTENDS_ATTRIBUTE
					+ "' attribute in '"
					+ EXTENSION_ELEMENT
					+ "' element");
		}
	}

}
