package gov.va.med.hds.cd.runtime.internal;

import java.net.URL;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.util.Hashtable;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeConstants;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;


class RuntimeURLHandlerFactory implements URLStreamHandlerFactory {
    private static final String PLUGIN_ID = IRuntimeConstants.PLUGIN_ID +
        ".urlHandlers";
    private static final String PROTOCOL = "protocol"; //$NON-NLS-1$
    private static final String HANDLER = "class"; //$NON-NLS-1$
    private static IRuntimeContext runtime = null;
    private static RuntimeURLHandlerFactory instance = null;
    private static Hashtable handlers = new Hashtable();

    public synchronized static void init(ILogger logger) {
        if (instance == null)
            try {
                instance = new RuntimeURLHandlerFactory();
            } catch (Error e) {
                logger.warning("URLStreamHandlerFactory already registered", e);
            }
    }

    private RuntimeURLHandlerFactory() throws Error {
        URL.setURLStreamHandlerFactory(this);
    }

    public synchronized static void register(IConfigurationElement ce) {
        String protocol = ce.getAttribute(PROTOCOL);
        String className = ce.getAttribute(HANDLER);

        if ((protocol != null) && (protocol.length() > 0) &&
                (className != null) && (className.length() > 0) &&
                !handlers.containsKey(protocol))
            handlers.put(protocol, ce);
    }


    public synchronized static void registerAll(IRuntimeContext runtime,
        ILogger logger) {
        init(logger);

        if (instance != null) {
            IPluginRegistry r = runtime.getPluginRegistry();
            IConfigurationElement[] ce = r.getConfigurationElementsFor
                (IRuntimeConstants.UNIQUE_HANDLER_PLUGIN_EXTENSION_POINT_ID);

            for (int i = 0; i < ce.length; i++) {
                register(ce[i]);
            }
        }
    }


    public URLStreamHandler createURLStreamHandler(String protocol) {
        synchronized (handlers) {
            Object result = handlers.get(protocol);

            if (result == null)
                return null;

            if (result instanceof IConfigurationElement)
                try {
                    IConfigurationElement ce = (IConfigurationElement) result;
                    URLStreamHandler h = (URLStreamHandler) ce.createExecutableExtension(HANDLER);
                    handlers.put(protocol, h);
                    result = h;
                } catch (Exception e) {
                    result = null;
                    handlers.remove(protocol);
                }

            if (result instanceof URLStreamHandler)
                return (URLStreamHandler) result;
            else {
                handlers.remove(protocol);

                return null;
            }
        }
    }
}
