package gov.va.med.hds.cd.runtime.logging.internal;

import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.Level;

/**
 * TODO_SB: Provide description for "SimpleStandardErrorLogger".
 */
public class SimpleStandardErrorLogger implements ILogger {

	private static final String FATAL = "FATAL: ";
	private static final String ERROR = "ERROR: ";
	private static final String WARNING = "WARNING: ";

	public void fatal(String message) {
		System.err.print(FATAL);
		System.err.println(message);
	}

	public void fatal(String message, Throwable t) {
		System.err.print(FATAL);
		System.err.println(message);
		t.printStackTrace();
	}

	public void error(String message) {
		System.err.print(ERROR);
		System.err.println(message);
	}

	public void error(String message, Throwable t) {
		System.err.print(ERROR);
		System.err.println(message);
		t.printStackTrace();
	}

	public void warning(String message) {
		System.err.print(WARNING);
		System.err.println(message);
	}

	public void warning(String message, Throwable t) {
		System.err.print(WARNING);
		System.err.println(message);
		t.printStackTrace();
	}

	public void info(String message) {
		// NOOP
	}

	public void info(String message, Throwable t) {
		// NOOP
	}

	public void debug(String message) {
		// NOOP
	}

	public void debug(String message, Throwable t) {
		// NOOP
	}

	public void log(Level level, String message) {
		if (Level.FATAL.equals(level)) {
			fatal(message);
		} else if (Level.ERROR.equals(level)) {
			error(message);
		} else if (Level.WARNING.equals(level)) {
			warning(message);
		}
	}

	public void log(Level level, String message, Throwable t) {
		if (Level.FATAL.equals(level)) {
			fatal(message, t);
		} else if (Level.ERROR.equals(level)) {
			error(message, t);
		} else if (Level.WARNING.equals(level)) {
			warning(message, t);
		}
	}

	public void addHandler(IHandler logHandler) {
		// NOOP
	}

	public void removeHandler(IHandler logHandler) {
		// NOOP
	}

	public void setUseParentHandlers(boolean b) {
		// NOOP
	}

	/* @see gov.va.med.hds.cd.runtime.logging.ILogger#getUseParentHandlers() */
	public boolean getUseParentHandlers() {
		// TODO_SB: Auto-generated method stub
		return false;
	}
}
