package gov.va.med.hds.cd.runtime.logging.log4j.internal;

import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Properties;

import org.apache.log4j.Appender;
import org.apache.log4j.Layout;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.config.PropertySetter;
import org.apache.log4j.helpers.LogLog;
import org.apache.log4j.helpers.OptionConverter;
import org.apache.log4j.or.RendererMap;
import org.apache.log4j.spi.AppenderAttachable;
import org.apache.log4j.spi.Configurator;
import org.apache.log4j.spi.ErrorHandler;
import org.apache.log4j.spi.Filter;
import org.apache.log4j.spi.LoggerFactory;
import org.apache.log4j.spi.LoggerRepository;
import org.apache.log4j.spi.RendererSupport;

import gov.va.med.hds.cd.runtime.IConfigurationElement;

/**
 * Use this class to initialize the log4j environment using an <code>IConfigurationElement</code>
 * tree.
 */
public class ConfigurationElementLogConfigurator {

	static final String LOGGING_TAG = "logging";
	static final String RENDERER_TAG = "renderer";
	static final String APPENDER_TAG = "appender";
	static final String APPENDER_REF_TAG = "appender-ref";
	static final String PARAM_TAG = "param";
	static final String LAYOUT_TAG = "layout";
	static final String CATEGORY = "category";
	static final String LOGGER = "logger";
	static final String LOGGER_REF = "logger-ref";
	static final String CATEGORY_FACTORY_TAG = "categoryFactory";
	static final String NAME_ATTR = "name";
	static final String CLASS_ATTR = "class";
	static final String VALUE_ATTR = "value";
	static final String ROOT_TAG = "root";
	static final String ROOT_REF = "root-ref";
	static final String LEVEL_TAG = "level";
	static final String PRIORITY_TAG = "priority";
	static final String FILTER_TAG = "filter";
	static final String ERROR_HANDLER_TAG = "errorHandler";
	static final String REF_ATTR = "ref";
	static final String ADDITIVITY_ATTR = "additivity";
	static final String THRESHOLD_ATTR = "threshold";
	static final String CONFIG_DEBUG_ATTR = "configDebug";
	static final String INTERNAL_DEBUG_ATTR = "debug";
	static final String RENDERING_CLASS_ATTR = "renderingClass";
	static final String RENDERED_CLASS_ATTR = "renderedClass";

	static final String EMPTY_STR = "";
	static final Class[] ONE_STRING_PARAM = new Class[] { String.class };
	
	private Hashtable appenderBag;
	private Properties props;
	private LoggerRepository repository;
	private IConfigurationElement rootElement;

	public ConfigurationElementLogConfigurator() {
		appenderBag = new Hashtable();
	}

	/**
	 * Used internally to parse appenders by IDREF name.
	 */
	protected Appender findAppenderByName(String appenderName) {
		Appender appender = (Appender) appenderBag.get(appenderName);

		if (appender != null) {
			return appender;
		} else {
			IConfigurationElement element = null;
			// Doesn't work on DOM Level 1 :
			// Element element = doc.getElementById(appenderName);
			element.getChildren();

			// Endre's hack:
//			Element element = null;
//			NodeList list = doc.getElementsByTagName("appender");
//			for (int t = 0; t < list.getLength(); t++) {
//				Node node = list.item(t);
//				NamedNodeMap map = node.getAttributes();
//				Node attrNode = map.getNamedItem("name");
//				if (appenderName.equals(attrNode.getNodeValue())) {
//					element = (Element) node;
//					break;
//				}
//			}
			// Hack finished.

			if (element == null) {
				LogLog.error("No appender named [" + appenderName + "] could be found.");
				return null;
			} else {
				appender = parseAppender(element);
				appenderBag.put(appenderName, appender);
				return appender;
			}
		}
	}
	
	protected IConfigurationElement getElementsByTagName() {
		return null;
	}
	
	/**
	 * Used internally to parse appenders by IDREF element.
	 */
	protected Appender findAppenderByReference(IConfigurationElement appenderRef) {
		String appenderName = subst(appenderRef.getAttribute(REF_ATTR));
		return findAppenderByName(appenderName);
	}
	/**
	 * Used internally to parse an appender element.
	 */
	protected Appender parseAppender(IConfigurationElement appenderElement) {
		String className = subst(appenderElement.getAttribute(CLASS_ATTR));
		LogLog.debug("Class name: [" + className + ']');
		try {
			Object instance = appenderElement.createExecutableExtension(CLASS_ATTR);
			Appender appender = (Appender) instance;
			PropertySetter propSetter = new PropertySetter(appender);

			appender.setName(subst(appenderElement.getAttribute(NAME_ATTR)));

			IConfigurationElement[] children = appenderElement.getChildren();
			for (int loop = 0; loop < children.length; loop++) {
				IConfigurationElement currentElement = children[loop];

				// Parse appender parameters 
				if (currentElement.getName().equals(PARAM_TAG)) {
					setParameter(currentElement, propSetter);
				}
				// Set appender layout
				else if (currentElement.getName().equals(LAYOUT_TAG)) {
					appender.setLayout(parseLayout(currentElement));
				}
				// Add filters
				else if (currentElement.getName().equals(FILTER_TAG)) {
					parseFilters(currentElement, appender);
				} else if (currentElement.getName().equals(ERROR_HANDLER_TAG)) {
					parseErrorHandler(currentElement, appender);
				} else if (currentElement.getName().equals(APPENDER_REF_TAG)) {
					String refName = subst(currentElement.getAttribute(REF_ATTR));
					if (appender instanceof AppenderAttachable) {
						AppenderAttachable aa = (AppenderAttachable) appender;
						LogLog.debug(
							"Attaching appender named ["
								+ refName
								+ "] to appender named ["
								+ appender.getName()
								+ "].");
						aa.addAppender(findAppenderByReference(currentElement));
					} else {
						LogLog.error(
							"Requesting attachment of appender named ["
								+ refName
								+ "] to appender named ["
								+ appender.getName()
								+ "] which does not implement org.apache.log4j.spi.AppenderAttachable.");
					}
				}
			}
			propSetter.activate();
			return appender;
		}
		/* Yes, it's ugly.  But all of these exceptions point to the same
		   problem: we can't create an Appender */
		catch (Exception oops) {
			LogLog.error("Could not create an Appender. Reported error follows.", oops);
			return null;
		}
	}

	/**
		 Used internally to parse an {@link ErrorHandler} element.
	 */
	protected void parseErrorHandler(IConfigurationElement element, Appender appender) {
		ErrorHandler eh =
			(ErrorHandler) OptionConverter.instantiateByClassName(
				subst(element.getAttribute(CLASS_ATTR)),
				org.apache.log4j.spi.ErrorHandler.class,
				null);

		if (eh != null) {
			eh.setAppender(appender);

			PropertySetter propSetter = new PropertySetter(eh);
			IConfigurationElement[] children = element.getChildren();
			for (int loop = 0; loop < children.length; loop++) {
				IConfigurationElement currentElement = children[loop];
				String tagName = currentElement.getName();
				if (tagName.equals(PARAM_TAG)) {
					setParameter(currentElement, propSetter);
				} else if (tagName.equals(APPENDER_REF_TAG)) {
					eh.setBackupAppender(findAppenderByReference(currentElement));
				} else if (tagName.equals(LOGGER_REF)) {
					String loggerName = currentElement.getAttribute(REF_ATTR);
					Logger logger = repository.getLogger(loggerName);
					eh.setLogger(logger);
				} else if (tagName.equals(ROOT_REF)) {
					Logger root = repository.getRootLogger();
					eh.setLogger(root);
				}
			}
			propSetter.activate();
			appender.setErrorHandler(eh);
		}
	}

	/**
		 Used internally to parse a filter element.
	 */
	protected void parseFilters(IConfigurationElement element, Appender appender) {
		String clazz = subst(element.getAttribute(CLASS_ATTR));
		Filter filter = (Filter) OptionConverter.instantiateByClassName(clazz, Filter.class, null);

		if (filter != null) {
			PropertySetter propSetter = new PropertySetter(filter);
			IConfigurationElement[] children = element.getChildren();
			for (int loop = 0; loop < children.length; loop++) {
				IConfigurationElement currentElement = children[loop];
				String tagName = currentElement.getName();
				if (tagName.equals(PARAM_TAG)) {
					setParameter(currentElement, propSetter);
				}
			}
			propSetter.activate();
			LogLog.debug(
				"Adding filter of type ["
					+ filter.getClass()
					+ "] to appender named ["
					+ appender.getName()
					+ "].");
			appender.addFilter(filter);
		}
	}

	/**
		 Used internally to parse an category element.
	*/
	protected void parseCategory(IConfigurationElement loggerElement) {
		// Create a new org.apache.log4j.Category object from the <category> element.
		String catName = subst(loggerElement.getAttribute(NAME_ATTR));

		Logger cat;

		String className = subst(loggerElement.getAttribute(CLASS_ATTR));

		if (EMPTY_STR.equals(className)) {
			LogLog.debug("Retreiving an instance of org.apache.log4j.Logger.");
			cat = repository.getLogger(catName);
		} else {
			LogLog.debug("Desired logger sub-class: [" + className + ']');
			try {
				Class clazz = loggerElement.createExecutableExtension(CLASS_ATTR).getClass();
				Method getInstanceMethod = clazz.getMethod("getLogger", ONE_STRING_PARAM);
				cat = (Logger) getInstanceMethod.invoke(null, new Object[] { catName });
			} catch (Exception oops) {
				LogLog.error(
					"Could not retrieve category [" + catName + "]. Reported error follows.",
					oops);
				return;
			}
		}

		// Setting up a category needs to be an atomic operation, in order
		// to protect potential log operations while category
		// configuration is in progress.
		synchronized (cat) {
			boolean additivity =
				OptionConverter.toBoolean(subst(loggerElement.getAttribute(ADDITIVITY_ATTR)), true);

			LogLog.debug("Setting [" + cat.getName() + "] additivity to [" + additivity + "].");
			cat.setAdditivity(additivity);
			parseChildrenOfLoggerElement(loggerElement, cat, false);
		}
	}

	/**
		 Used internally to parse the category factory element.
	*/
	protected void parseCategoryFactory(IConfigurationElement factoryElement) {
		String className = subst(factoryElement.getAttribute(CLASS_ATTR));

		if (EMPTY_STR.equals(className)) {
			LogLog.error("Category Factory tag " + CLASS_ATTR + " attribute not found.");
			LogLog.debug("No Category Factory configured.");
		} else {
			LogLog.debug("Desired category factory: [" + className + ']');
			Object catFactory =
				OptionConverter.instantiateByClassName(className, LoggerFactory.class, null);
			PropertySetter propSetter = new PropertySetter(catFactory);

			IConfigurationElement currentElement = null;
			IConfigurationElement[] children = factoryElement.getChildren();
			for (int loop = 0; loop < children.length; loop++) {
				currentElement = children[loop];
				if (currentElement.getName().equals(PARAM_TAG)) {
					setParameter(currentElement, propSetter);
				}
			}
		}
	}

	/**
		 Used internally to parse the roor category element.
	*/
	protected void parseRoot(IConfigurationElement rootElement) {
		Logger root = repository.getRootLogger();
		// category configuration needs to be atomic
		synchronized (root) {
			parseChildrenOfLoggerElement(rootElement, root, true);
		}
	}

	/**
		 Used internally to parse the children of a category element.
	*/
	protected void parseChildrenOfLoggerElement(
		IConfigurationElement catElement,
		Logger cat,
		boolean isRoot) {

		PropertySetter propSetter = new PropertySetter(cat);

		IConfigurationElement[] children = catElement.getChildren();
		for (int loop = 0; loop < children.length; loop++) {
			IConfigurationElement currentElement = children[loop];
			String tagName = currentElement.getName();

			if (tagName.equals(APPENDER_REF_TAG)) {
				IConfigurationElement appenderRef = currentElement;
				Appender appender = findAppenderByReference(appenderRef);
				String refName = subst(appenderRef.getAttribute(REF_ATTR));
				if (appender != null)
					LogLog.debug(
						"Adding appender named ["
							+ refName
							+ "] to category ["
							+ cat.getName()
							+ "].");
				else
					LogLog.debug("Appender named [" + refName + "] not found.");

				cat.addAppender(appender);

			} else if (tagName.equals(LEVEL_TAG)) {
				parseLevel(currentElement, cat, isRoot);
			} else if (tagName.equals(PRIORITY_TAG)) {
				parseLevel(currentElement, cat, isRoot);
			} else if (tagName.equals(PARAM_TAG)) {
				setParameter(currentElement, propSetter);
			}
		}
		propSetter.activate();
	}

	/**
		 Used internally to parse a layout element.
	*/
	protected Layout parseLayout(IConfigurationElement layout_element) {
		String className = subst(layout_element.getAttribute(CLASS_ATTR));
		LogLog.debug("Parsing layout of class: \"" + className + "\"");
		try {
			Object instance = layout_element.createExecutableExtension(CLASS_ATTR);
			Layout layout = (Layout) instance;
			PropertySetter propSetter = new PropertySetter(layout);

			IConfigurationElement[] params = layout_element.getChildren();
			for (int loop = 0; loop < params.length; loop++) {
				IConfigurationElement currentElement = params[loop];
				String tagName = currentElement.getName();
				if (tagName.equals(PARAM_TAG)) {
					setParameter(currentElement, propSetter);
				}
			}

			propSetter.activate();
			return layout;
		} catch (Exception oops) {
			LogLog.error("Could not create the Layout. Reported error follows.", oops);
			return null;
		}
	}

	protected void parseRenderer(IConfigurationElement element) {
		String renderingClass = subst(element.getAttribute(RENDERING_CLASS_ATTR));
		String renderedClass = subst(element.getAttribute(RENDERED_CLASS_ATTR));
		if (repository instanceof RendererSupport) {
			RendererMap.addRenderer((RendererSupport) repository, renderedClass, renderingClass);
		}
	}

	/**
	 * Used internally to parse a level element.
	 */
	protected void parseLevel(IConfigurationElement element, Logger logger, boolean isRoot) {
		String catName = logger.getName();
		if (isRoot) {
			catName = "root";
		}

		String priStr = subst(element.getAttribute(VALUE_ATTR));
		LogLog.debug("Level value for " + catName + " is  [" + priStr + "].");

		if (Configurator.INHERITED.equalsIgnoreCase(priStr) || Configurator.NULL.equalsIgnoreCase(priStr)) {
			if (isRoot) {
				LogLog.error("Root level cannot be inherited. Ignoring directive.");
			} else {
				logger.setLevel(null);
			}
		} else {
			String className = subst(element.getAttribute(CLASS_ATTR));
			if (EMPTY_STR.equals(className)) {
				logger.setLevel(OptionConverter.toLevel(priStr, Level.DEBUG));
			} else {
				LogLog.debug("Desired Level sub-class: [" + className + ']');
				try {
					Class clazz = element.createExecutableExtension(CLASS_ATTR).getClass();
					Method toLevelMethod = clazz.getMethod("toLevel", ONE_STRING_PARAM);
					Level pri = (Level) toLevelMethod.invoke(null, new Object[] { priStr });
					logger.setLevel(pri);
				} catch (Exception oops) {
					LogLog.error(
						"Could not create level [" + priStr + "]. Reported error follows.",
						oops);
					return;
				}
			}
		}
		LogLog.debug(catName + " level set to " + logger.getLevel());
	}

	protected void setParameter(IConfigurationElement elem, PropertySetter propSetter) {
		String name = subst(elem.getAttribute(NAME_ATTR));
		String value = (elem.getAttribute(VALUE_ATTR));
		value = subst(OptionConverter.convertSpecialChars(value));
		propSetter.setProperty(name, value);
	}

	/**
		 Configure log4j using a <code>configuration</code> element as
		 defined in the log4j.dtd. 
	
	*/
	static public void configure(IConfigurationElement element) {
		ConfigurationElementLogConfigurator configurator =
			new ConfigurationElementLogConfigurator();
		configurator.doConfigure(element, LogManager.getLoggerRepository());
	}

	/**
	 * Configure by taking in a configuration element. 
	 */
	public void doConfigure(IConfigurationElement element, LoggerRepository repository) {
		this.repository = repository;
		parse(element);
	}

	/**
		 Used internally to configure the log4j framework by parsing a DOM
		 tree of XML elements based on <a
		 href="doc-files/log4j.dtd">log4j.dtd</a>.
	   
	*/
	protected void parse(IConfigurationElement element) {
		String rootElementName = element.getName();
		if (!rootElementName.equals(LOGGING_TAG)) {
			LogLog.error("element is not a <" + LOGGING_TAG + "> element.");
			return;
		}
		rootElement = element;

		String debugAttrib = subst(element.getAttribute(INTERNAL_DEBUG_ATTR));

		LogLog.debug("debug attribute= \"" + debugAttrib + "\".");
		// if the log4j.dtd is not specified in the XML file, then the
		// "debug" attribute is returned as the empty string.
		if (!debugAttrib.equals(EMPTY_STR) && !debugAttrib.equals("null")) {
			LogLog.setInternalDebugging(OptionConverter.toBoolean(debugAttrib, true));
		} else {
			LogLog.debug("Ignoring " + INTERNAL_DEBUG_ATTR + " attribute.");
		}

		String confDebug = subst(element.getAttribute(CONFIG_DEBUG_ATTR));
		if (!confDebug.equals(EMPTY_STR) && !confDebug.equals("null")) {
			LogLog.warn("The \"" + CONFIG_DEBUG_ATTR + "\" attribute is deprecated.");
			LogLog.warn("Use the \"" + INTERNAL_DEBUG_ATTR + "\" attribute instead.");
			LogLog.setInternalDebugging(OptionConverter.toBoolean(confDebug, true));
		}

		String thresholdStr = subst(element.getAttribute(THRESHOLD_ATTR));
		LogLog.debug("Threshold =\"" + thresholdStr + "\".");
		if (!EMPTY_STR.equals(thresholdStr) && !"null".equals(thresholdStr)) {
			repository.setThreshold(thresholdStr);
		}

		//Hashtable appenderBag = new Hashtable(11);

		/* Building Appender objects, placing them in a local namespace
		   for future reference */

		// First configure each category factory under the root element.
		// Category factories need to be configured before any of
		// categories they support.
		//
		String tagName = null;
		IConfigurationElement currentElement = null;
		IConfigurationElement[] children = element.getChildren();
		for (int loop = 0; loop < children.length; loop++) {
			currentElement = children[loop];
			tagName = currentElement.getName();
			if (tagName.equals(CATEGORY_FACTORY_TAG)) {
				parseCategoryFactory(currentElement);
			}
		}

		for (int loop = 0; loop < children.length; loop++) {
			currentElement = children[loop];
			tagName = currentElement.getName();

			if (tagName.equals(CATEGORY) || tagName.equals(LOGGER)) {
				parseCategory(currentElement);
			} else if (tagName.equals(ROOT_TAG)) {
				parseRoot(currentElement);
			} else if (tagName.equals(RENDERER_TAG)) {
				parseRenderer(currentElement);
			}
		}
	}

	protected String subst(String value) {
		if (value == null) return EMPTY_STR;
		return value;
//		try {
//			return OptionConverter.substVars(value, props);
//		} catch (IllegalArgumentException e) {
//			LogLog.warn("Could not perform variable substitution.", e);
//			return value;
//		}
	}
}
