package gov.va.med.hds.cd.runtime.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import gov.va.med.hds.cd.runtime.IRuntimeConstants;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.internal.DOMPluginReader;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginRegistry;
import gov.va.med.hds.cd.runtime.internal.RuntimeUtilities;
import gov.va.med.hds.util.ExceptionStackTrace;

public class DOMRegistryReader implements IRegistryReader, IRuntimeConstants {

	private static final Logger log = Logger.getLogger(DOMRegistryReader.class.getName());

	private DOMPluginReader pluginReader;
	private IRuntimeContext runtime;

	public DOMRegistryReader(IRuntimeContext runtime) {
		this.runtime = runtime;
		this.pluginReader = new DOMPluginReader(runtime);
	}

	public void readPluginDescriptors(PluginRegistry registry, InputStream inputStream) {
		readPluginDescriptors(registry, new InputSource(inputStream));
	}

	public void readPluginDescriptors(PluginRegistry registry, Reader reader) {
		readPluginDescriptors(registry, new InputSource(reader));
	}

	public PluginRegistry readPluginRegistry(Reader reader) {
		PluginRegistry registry = new PluginRegistry(runtime);
		readPluginDescriptors(registry, reader);
		return registry;
	}

	public PluginRegistry readPluginRegistry(InputStream inputStream) {
		PluginRegistry registry = new PluginRegistry(runtime);
		readPluginDescriptors(registry, inputStream);
		return registry;
	}

	public void readPluginDescriptors(PluginRegistry registry, InputSource inputSource) {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			
			builder = factory.newDocumentBuilder();
			builder.setErrorHandler(new BuilderErrorHandler());
			Document document = builder.parse(inputSource);
			readPluginDescriptors(registry, document.getDocumentElement().getChildNodes());
		} catch (ParserConfigurationException e) {
			log.severe(ExceptionStackTrace.getStackTrace(e));
		} catch (SAXException e) {
			log.severe(ExceptionStackTrace.getStackTrace(e));
		} catch (IOException e) {
			log.severe(ExceptionStackTrace.getStackTrace(e));
		}
	}

	private void readPluginDescriptors(PluginRegistry registry, NodeList pluginElements) {
		for (int i = 0; i < pluginElements.getLength(); i++) {
			if (!(pluginElements.item(i) instanceof Element)) continue;
			try {
				PluginDescriptor plugin =
					pluginReader.readPlugin((Element) pluginElements.item(i), getClass().getClassLoader());
				if (!RuntimeUtilities.isRuntimePlugin(plugin)) registry.add(plugin);
			} catch (InvalidPluginDescriptorException e) {
				log.warning(ExceptionStackTrace.getStackTrace(e));
			}
		}
	}
	
	private static class BuilderErrorHandler implements ErrorHandler {
		public void warning(SAXParseException exception) throws SAXException {
		}
		public void error(SAXParseException exception) throws SAXException {
			throw exception;
		}
		public void fatalError(SAXParseException exception) throws SAXException {
			throw exception;
		}
	}
}
