package gov.va.med.hds.cd.runtime.test;

import java.io.StringReader;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import junit.framework.TestCase;

import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.internal.IPluginReader;

public abstract class InvalidPluginReaderTest extends TestCase {
    private LogRecord logRecord;

    public InvalidPluginReaderTest(String name) {
        super(name);
    }

	abstract protected IPluginReader createPluginReader();

    protected void setUp() throws Exception {
        super.setUp();
        logRecord = null;
        IPluginReader reader = createPluginReader();
        Logger.getLogger(reader.getClass().getName()).setUseParentHandlers(false);
        Logger.getLogger(reader.getClass().getName()).addHandler(new TestHandler());
    }

    public void testNotWellFormed() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin>afasdaweweaf"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        }
    }

    public void testMissingIdAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin name=\"\" version=\"1\" class=\"\"/>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        } 
    }

    public void testMissingNameAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"\" version=\"1\" class=\"\"/>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        } 
    }

    public void testMissingVersionAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"\" name=\"\" class=\"\"/>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        } 
    }

    public void testPrerequisiteMissingPluginAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
                                               "<prerequisites>" + "<requires version=\"1.0\"/>" +
                                               "</prerequisites>" + "</plugin>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        }
    }

    public void testPrerequisiteMissingVersionAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
                                               "<prerequisites>" + "<requires plugin=\"test\"/>" +
                                               "</prerequisites>" + "</plugin>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        } 
    }

	public void testPrerequisiteInvalidMatchAttribute() {
		IPluginReader reader = createPluginReader();
		try {
			reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
											   "<prerequisites>" + "<requires plugin=\"test\" version=\"1.0\" match=\"invalid\"/>" +
											   "</prerequisites>" + "</plugin>"), getClass().getClassLoader());
			fail("expected " + InvalidPluginDescriptorException.class.getName());
		} catch (InvalidPluginDescriptorException e) {
		} 
	}

    public void testExtensionPointMissingIdAttribute() {
        IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
                                               "<extension-points>" +
                                               "<extension-point name=\"test\"/>" +
                                               "</extension-points>" + "</plugin>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        } 
    }

    public void testExtensionPointMissingNameAttribute() {
       IPluginReader reader = createPluginReader();
        try {
            reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
                                               "<extension-points>" +
                                               "<extension-point id=\"test\"/>" +
                                               "</extension-points>" + "</plugin>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        }
    }

    public void testExtensionMissingExtendsAttribute() {
        try {
            IPluginReader reader = createPluginReader();
            reader.readPlugin(new StringReader("<plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">" +
                                               "<extensions>" +
                                               "<extension id=\"test\" name=\"test\"/>" +
                                               "</extensions>" + "</plugin>"), getClass().getClassLoader());
            fail("expected " + InvalidPluginDescriptorException.class.getName());
        } catch (InvalidPluginDescriptorException e) {
        }
    }

    class TestHandler extends Handler {
        public void publish(LogRecord record) {
            logRecord = record;
        }

        public void flush() {
        }

        public void close() {
        }
    }
}
