package gov.va.med.hds.cd.runtime.test;

import java.io.StringReader;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import junit.framework.TestCase;


public class InvalidPluginRegistryReaderTest extends TestCase {

	private static final String RUNTIME_PACAKGE = "gov.va.med.hds.cd.runtime";

	private static final SimpleFormatter formatter = new SimpleFormatter();
	private LogRecord logRecord;

	private IRegistryReader reader;

	public InvalidPluginRegistryReaderTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		logRecord = null;
		Logger.getLogger(RUNTIME_PACAKGE).setUseParentHandlers(false);
		Logger.getLogger(RUNTIME_PACAKGE).addHandler(new TestHandler());
		reader = createRegistryReader();
	}

	protected IRegistryReader createRegistryReader() {
		return new DOMRegistryReader(null);
	}

	public void testNotWellFormed() {
		reader.readPluginRegistry(new StringReader("<registry>afasdaweweaf"));
		assertNotNull(logRecord);
		assertEquals(Level.SEVERE, logRecord.getLevel());
	}

	public void testMissingIdAttribute() {
		reader.readPluginRegistry(new StringReader("<registry><plugin name=\"\" version=\"1.0\" class=\"\"/></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testMissingNameAttribute() {
		reader.readPluginRegistry(new StringReader("<registry><plugin id=\"\" version=\"1.0\" class=\"\"/></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testMissingVersionAttribute() {
		reader.readPluginRegistry(new StringReader("<registry><plugin id=\"\" name=\"\" class=\"\"/></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testPrerequisiteMissingPluginAttribute() {
		reader.readPluginRegistry(new StringReader(
					"<registry><plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">"
						+ "<prerequisites>"
						+ "<requires version=\"1.0\"/>"
						+ "</prerequisites>"
						+ "</plugin></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testPrerequisiteMissingVersionAttribute() {
		reader.readPluginRegistry(new StringReader(
					"<registry><plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">"
						+ "<prerequisites>"
						+ "<requires plugin=\"test\"/>"
						+ "</prerequisites>"
						+ "</plugin></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testExtensionPointMissingIdAttribute() {
		reader.readPluginRegistry(	new StringReader(
					"<registry><plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">"
						+ "<extension-points>"
						+ "<extension-point name=\"test\"/>"
						+ "</extension-points>"
						+ "</plugin></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testExtensionPointMissingNameAttribute() {
		reader.readPluginRegistry(new StringReader(
					"<registry><plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">"
						+ "<extension-points>"
						+ "<extension-point id=\"test\"/>"
						+ "</extension-points>"
						+ "</plugin></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	public void testExtensionMissingExtendsAttribute() {
		reader.readPluginRegistry(new StringReader(
					"<registry><plugin id=\"test\" name=\"test\" version=\"1.0\" class=\"test\">"
						+ "<extensions>"
							+ "<extension id=\"test\" name=\"test\"/>"
						+ "</extensions>"
						+ "</plugin></registry>"));
		assertNotNull(logRecord);
		assertEquals(formatter.format(logRecord), Level.WARNING, logRecord.getLevel());
	}

	class TestHandler extends Handler {
		public void publish(LogRecord record) {
			logRecord = record;
		}

		public void flush() {
		}

		public void close() {
		}
	}
}
