package gov.va.med.hds.cd.runtime.test;

import junit.framework.TestCase;

import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;

public class Log4JLoggingTest extends TestCase {
	
	private ILoggingCapable logging;
	private LoggingEvent logEvent;
	private ILogRecord record;
	
	public Log4JLoggingTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		logging = createLoggingImplementation();
		configureLog4J();
	}
	
	private void configureLog4J() {
		LogManager.resetConfiguration();
		Logger.getRootLogger().addAppender(new TestAppender());
	}

	protected ILoggingCapable createLoggingImplementation() {
		return new Log4JLogging();
	}

	public void testFatalLog() {
		logging.getLogger(getClass()).fatal("A fatal message!");
		assertNotNull(logEvent);
		assertEquals(getClass().getName(), logEvent.getLoggerName());
		assertEquals(Level.FATAL, logEvent.getLevel());
		assertEquals("A fatal message!", logEvent.getMessage());
	}

	public void testErrorLog() {
		logging.getLogger(getClass()).error("An error message!");
		assertNotNull(logEvent);
		assertEquals(getClass().getName(), logEvent.getLoggerName());
		assertEquals(Level.ERROR, logEvent.getLevel());
		assertEquals("An error message!", logEvent.getMessage());
	}

	public void testWarningLog() {
		logging.getLogger(getClass()).warning("A warning message!");
		assertNotNull(logEvent);
		assertEquals(getClass().getName(), logEvent.getLoggerName());
		assertEquals(Level.WARN, logEvent.getLevel());
		assertEquals("A warning message!", logEvent.getMessage());
	}

	public void testInfoLog() {
		logging.getLogger(getClass()).info("An informational message!");
		assertNotNull(logEvent);
		assertEquals(getClass().getName(), logEvent.getLoggerName());
		assertEquals(Level.INFO, logEvent.getLevel());
		assertEquals("An informational message!", logEvent.getMessage());
	}

	public void testDebugLog() {
		logging.getLogger(getClass()).debug("A debug message!");
		assertNotNull(logEvent);
		assertEquals(getClass().getName(), logEvent.getLoggerName());
		assertEquals(Level.DEBUG, logEvent.getLevel());
		assertEquals("A debug message!", logEvent.getMessage());
	}

	public void testHandlerAdapter() {
		logging.getLogger(getClass()).addHandler(new IHandler() {
			public void publish(ILogRecord logRecord) {
				record = logRecord;
			}

			public void close() {
				// NOOP
			}
		});
		logging.getLogger(getClass()).debug("ftang");
		assertNotNull(record);
		assertEquals(gov.va.med.hds.cd.runtime.logging.Level.DEBUG, record.getLevel());
	}

	private class TestAppender extends AppenderSkeleton {

		protected void append(LoggingEvent arg0) {
			logEvent = arg0;
		}

		public boolean requiresLayout() {
			return false;
		}

		public void close() {
			// NOOP
		}
	}
}
