package gov.va.med.hds.cd.runtime.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.VersionMatchCondition;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptor;
import gov.va.med.hds.cd.runtime.internal.PluginDescriptorMap;

public class PluginDescriptorMapTest extends TestCase {

	IPluginDescriptor p1;
	IPluginDescriptor p12;
	IPluginDescriptor p123;
	IPluginDescriptor p123s;
	IPluginDescriptor p2 ;
	IPluginDescriptor p3;

	public PluginDescriptorMapTest(String name) {
		super(name);
	}

	private IPluginDescriptor createPluginDescriptor(String id, String version, String name) {
		return new PluginDescriptor(null, getClass().getClassLoader(), id, name, new Version(version));
	}

	protected void setUp() throws Exception {
		super.setUp();
		p1 = createPluginDescriptor("one", "1", "One");
		p12 = createPluginDescriptor("one", "1.2", "One Point Two");
		p123 = createPluginDescriptor("one", "1.2.3", "One Point Two Point Three");
		p123s = createPluginDescriptor("one", "1.2.3.s", "One Point Two Point Three SSSS");
		p2 = createPluginDescriptor("two", "2", "Two");
		p3 = createPluginDescriptor("three", "3", "Three");
	}

	public void testCreate() {
		PluginDescriptorMap m = new PluginDescriptorMap();
		assertTrue(m.isEmpty());
		assertEquals(0, m.size());
	}

	public void testOneInMap() {	
		PluginDescriptorMap m = new PluginDescriptorMap();
		Integer fiftySix = new Integer(56);
		m.put(p1, fiftySix);
		assertFalse(m.isEmpty());
		assertTrue(m.contains(p1));
		assertEquals(fiftySix, m.get(p1));
		assertEquals(1, m.size());
	}
	
	public void testMultipleIdsInMap() {
		PluginDescriptorMap m = new PluginDescriptorMap();
		m.put(p1, "qui-gonn");
		m.put(p2, "obi-wan");
		m.put(p3, "anakin");
		assertEquals("qui-gonn", m.get(p1));
		assertEquals("obi-wan", m.get(p2));
		assertEquals("anakin", m.get(p3));
		assertTrue(m.contains(p1));
		assertTrue(m.contains(p2));
		assertTrue(m.contains(p3));
		assertEquals(3, m.size());
		assertFalse(m.isEmpty());
	}
	
	public void testContains() {
		PluginDescriptorMap m = new PluginDescriptorMap();
		m.put(p1, "yoda");
		m.put(p12, "dooku");
		m.put(p3, "anakin");
		assertTrue(m.contains("one"));
		assertFalse(m.contains("two"));
		assertTrue(m.contains("three"));
		assertTrue(m.contains("one", new Version("1")));
		assertTrue(m.contains("one", new Version("1.2")));
		assertFalse(m.contains("one", new Version("1.2.3")));
	}
	
	public void testContainsMatchCondition() {
		PluginDescriptorMap m = new PluginDescriptorMap();
		m.put(p1, "yoda");
		m.put(p12, "dooku");
		m.put(p123s, "obi-wan");
		m.put(p3, "anakin");
		assertTrue(m.contains("one", new Version("0.4"), VersionMatchCondition.GREATER_OR_EQUAL));
		assertFalse(m.contains("one", new Version("1.2.4"), VersionMatchCondition.GREATER_OR_EQUAL));
		assertTrue(m.contains("three", new Version("3"), VersionMatchCondition.PERFECT));
		assertFalse(m.contains("one", new Version("1.2.0.sfgerter"), VersionMatchCondition.PERFECT));
		assertFalse(m.contains("one", new Version("0.4"), VersionMatchCondition.COMPATIBLE));
		assertTrue(m.contains("one", new Version("1.1"), VersionMatchCondition.COMPATIBLE));
		assertFalse(m.contains("one", new Version("1.1"), VersionMatchCondition.EQUIVALENT));
		assertTrue(m.contains("one", new Version("1.2.1"), VersionMatchCondition.EQUIVALENT));
		assertFalse(m.contains("one", new Version("1.2.2"), VersionMatchCondition.PERFECT));
		assertTrue(m.contains("one", new Version("1.2.3.s"), VersionMatchCondition.PERFECT));
	}
}
