package gov.va.med.hds.cd.runtime.test;

import junit.framework.Assert;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExecutableExtension;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;

public class RuntimeTest extends RuntimeTestCase {

	public RuntimeTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("runtimeTestRegistry.xml");
	}

	public void testCreateExecutableExtensionActivation()
		throws PluginStartupException, PluginNotFoundException {
		Object one = getRuntime().getPluginInstance("testPluginOne");
		assertTrue(getRuntime().isPluginActive("testPluginOne"));
		assertTrue(getRuntime().isPluginActive("testPluginTwo"));
		assertEquals(1, TestPlugin1.instanceCount);
		assertEquals(1, TestPlugin2.instanceCount);
		assertEquals(one, getRuntime().getPluginInstance("testPluginOne"));
		Object two = getRuntime().getPluginInstance("testPluginTwo");
		assertEquals(two, getRuntime().getPluginInstance("testPluginTwo"));
		assertEquals(1, TestPlugin1.instanceCount);
		assertEquals(1, TestPlugin2.instanceCount);
	}

	public void testSharedPluginActivation() throws PluginStartupException, PluginNotFoundException {
		getRuntime().getPluginInstance("testPluginThree");
		assertTrue(getRuntime().isPluginActive("testPluginThree"));
		assertTrue(getRuntime().isPluginActive("testPluginFour"));
		assertTrue(getRuntime().isPluginActive("sharedPlugin"));
		assertEquals(1, Shared.instanceCount);
	}

	public void testSelfExtension()
		throws PluginStartupException, PluginNotFoundException {
		getRuntime().getPluginInstance("selfExtendingPlugin");
	}

	public static class TestPlugin1 extends AbstractTestPlugin {

		public static int instanceCount = 0;

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
			IConfigurationElement[] foos =
				runtime.getPluginRegistry().getConfigurationElementsFor(
					plugin.getUniqueIdentifier() + ".foo");
			for (int i = 0; i < foos.length; i++) {
				Object foo = foos[i].createExecutableExtension("class");
//				System.out.println(
//					plugin + " created foo: " + foo.getClass().getName());
			}
			instanceCount++;
		}
	}

	public static class TestPlugin2 extends AbstractTestPlugin {

		public static int instanceCount = 0;

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
			instanceCount++;
		}

	}

	public static class TestPlugin3 extends AbstractTestPlugin {

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
		}
	}

	public static class TestPlugin4 extends AbstractTestPlugin {

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
		}
	}

	public static class SelfExtending extends AbstractTestPlugin {

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
			IConfigurationElement[] barneys =
				runtime.getPluginRegistry().getConfigurationElementsFor(
					"selfExtendingPlugin.barney");
			for (int i = 0; i < barneys.length; i++) {
				Object barney = barneys[i].createExecutableExtension("class");
//				System.out.println(
//					"SelfExtending.startup.created barney: "
//						+ barney.getClass().getName());
			}
		}
	}

	public static class Shared extends AbstractTestPlugin {

		public static int instanceCount = 0;

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			super.startup(runtime, plugin);
			instanceCount++;
		}
	}

	public static class MockFoo implements IExecutableExtension {

		/* @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object) */
		public void setInitializationData(
			IRuntimeContext runtime,
			IConfigurationElement config,
			String propertyName,
			Object data)
			throws Exception {
//			System.out.println("MockFoo.setInitializationData()");
		}
	}

	public static class MockBarney implements IExecutableExtension {

		/* @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object) */
		public void setInitializationData(
			IRuntimeContext runtime,
			IConfigurationElement config,
			String propertyName,
			Object data)
			throws Exception {
//			System.out.println("MockBarney.setInitializationData()");
			//			Object self = runtime.getPluginInstance("selfExtendingPlugin");
		}
	}

	public static class AbstractTestPlugin implements IPlugin {

		private IPluginDescriptor plugin;

		/* @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor) */
		public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
			throws Exception {
			this.plugin = plugin;
//			System.out.println(plugin + " startup()");
			Assert.assertFalse(runtime.isPluginActive(plugin.getUniqueIdentifier(), plugin.getVersion()));
		}

		/* @see gov.va.med.hds.cd.runtime.IPlugin#shutdown() */
		public void shutdown() throws Exception {
//			System.out.println(plugin + " shutdown()");
		}
	}
}
