package gov.va.med.hds.cd.runtime.test;

import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;

import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;
import gov.va.med.hds.cd.runtime.logging.IHandler;
import gov.va.med.hds.cd.runtime.logging.ILogRecord;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.util.PluginManifestValidator;

public abstract class RuntimeTestCase extends TestCase {
	public static final String RUNTIME_PACKAGE = "gov.va.med.hds.cd";

	protected ILogger log;
	private IHandler handler;
	private TestRuntime runtime;

	public RuntimeTestCase(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		runtime = new TestRuntime();
		log = runtime.getLogger(RUNTIME_PACKAGE);
		log.addHandler(handler = new IHandler() {
			public void publish(ILogRecord logRecord) {
				fail(logRecord.getMessage());
			}
			public void close() {
			}
		});
	}

	protected void tearDown() throws Exception {
		if (runtime != null) {
			runtime.shutdown();
			runtime = null;
		}
		log.removeHandler(handler);
		super.tearDown();
	}

	public void setRuntime(TestRuntime runtime) {
		if (this.runtime != null)
			this.runtime.shutdown();
		this.runtime = runtime;
	}

	public TestRuntime getRuntime() {
		return runtime;
	}

	protected void parseRegistryResource(String resourceName) throws InvalidPluginDescriptorException, IOException {
			validateRegistryResource(getClass().getResourceAsStream(resourceName));
			parseRegistryResource(getClass().getResourceAsStream(resourceName));
	}

	public void parseRegistryResource(InputStream resourceStream) {
		runtime.readPluginDescriptors(resourceStream);
	}

	public void validateRegistryResource(InputStream resourceStream) throws InvalidPluginDescriptorException, IOException {
		PluginManifestValidator validator = new PluginManifestValidator();
		validator.validate(resourceStream);
	}
}
