package gov.va.med.hds.cd.runtime.test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import gov.va.med.hds.cd.mum.boot.utils.ILoadableTreeData;
import gov.va.med.hds.cd.mum.boot.utils.TreeDataFactory;
import gov.va.med.hds.cd.runtime.logging.Level;
public class TestRuntime extends gov.va.med.hds.cd.runtime.internal.Runtime {
	
	private static class BuilderErrorHandler implements ErrorHandler {
		public void error(SAXParseException exception) throws SAXException {
			throw exception;
		}
		public void fatalError(SAXParseException exception) throws SAXException {
			throw exception;
		}
		public void warning(SAXParseException exception) throws SAXException {
		}
	}

	private static final String DEFAULT_REGISTRY_FILENAME = "xml/registry.xml";
	private static final String DEFAULT_SETTINGS_FILENAME = "xml/settings.xml";

	public static void main(String[] args) {
		try {
			TestRuntime runtime = new TestRuntime();
			runtime.readSettings(new FileInputStream(DEFAULT_SETTINGS_FILENAME));
			runtime.readPluginDescriptors(new FileInputStream(DEFAULT_REGISTRY_FILENAME));
			runtime.run(args);
		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	private IRegistryReader registryReader;

	public TestRuntime() {
		super();
		this.registryReader = createRegistryReader();
	}

	protected IRegistryReader createRegistryReader() {
		return new DOMRegistryReader(this);
	}

	public void readPluginDescriptors(InputStream inputStream) {
		readPluginDescriptors(inputStream, true);
	}

	public void readPluginDescriptors(InputStream inputStream, boolean warnOfUnresolved) {
		registryReader.readPluginDescriptors(registry, inputStream);
		registerURLHandlers();
		if (warnOfUnresolved)
			logUnresolvedPlugins(Level.WARNING);
	}

	public void readSettings(InputStream inputStream) {
		try {
			ILoadableTreeData xml = TreeDataFactory.newSAXTreeData();
			xml.load(inputStream);
			readSettings(xml);
		} catch (Exception e) {
			getLogger(getClass()).warning("Error reading settings", e);
		}
	}
	
	public synchronized void shutdown() {
		cleanUp();
	}
}
