package gov.va.med.hds.cd.runtime.test;

import java.io.StringReader;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeConstants;
import gov.va.med.hds.cd.runtime.internal.RuntimeCoreDescriptor;
import gov.va.med.hds.cd.runtime.internal.RuntimeDescriptor;

public class ValidPluginRegistryReaderTest extends TestCase implements IRuntimeConstants {

	protected static final String VALID_REGISTRY_XML = 
	"<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
	"<registry>" +
		"<plugin name=\"Prism\" id=\"gov.va.med.hds.cd.prism\" version=\"0.1\" provider-name=\"CPRS Re\">" +
			"<library name=\"prism.jar\"/>" +
			"<prerequisites>" +
				"<requires plugin=\"gov.va.med.hds.cd.runtime\" version=\"" + RuntimeDescriptor.VERSION.toString() + "\"/>" +
			"</prerequisites>" +
			"<extensions>" +
				"<extension extends=\"gov.va.med.hds.cd.runtime.mainPlugin\">" +
					"<!-- some content that is validated by MainEntryPoint.xsd -->" +
					"<mainEntryPoint class=\"gov.va.med.hds.cd.ui.Prism\"/>" +
					"<root>" +
						"<child1/>" +
						"<child2>" +
							"<child21/>" +
						"</child2>" +
						"<child3/>" +
					"</root>" +
				"</extension>" +
			"</extensions>" +
		"</plugin>" +
	"</registry>\n";

	private IPluginRegistry registry;

	protected void setUp() throws Exception {
		super.setUp();
		StringReader sr = new StringReader(VALID_REGISTRY_XML);
		IRegistryReader reader = createRegistryReader();
		registry = reader.readPluginRegistry(sr);
	}

	protected IRegistryReader createRegistryReader() {
		return new DOMRegistryReader(null);
	}

	public ValidPluginRegistryReaderTest(String name) {
		super(name);
	}
	
	public void testReadRegistry() {
		assertNotNull(registry);
		
		assertEquals(3, registry.getPluginDescriptors().length);
		assertEquals(0, registry.getPluginDescriptors("some.test.id").length);
		assertNotNull(registry.getPluginDescriptors(PLUGIN_ID));
		assertEquals(1, registry.getPluginDescriptors(PLUGIN_ID).length);
		assertNotNull(registry.getPluginDescriptors(RuntimeCoreDescriptor.PLUGIN_ID));
		assertEquals(1, registry.getPluginDescriptors(RuntimeCoreDescriptor.PLUGIN_ID).length);
		assertNotNull(registry.getPluginDescriptors("gov.va.med.hds.cd.prism"));
		assertEquals(1, registry.getPluginDescriptors("gov.va.med.hds.cd.prism").length);
		
		assertNull(registry.getPluginDescriptor("some.test.id"));
		assertNull(registry.getPluginDescriptor(PLUGIN_ID, new Version(RuntimeDescriptor.VERSION.getMajorComponent() + 1,0,0)));
		assertNotNull(registry.getPluginDescriptor(PLUGIN_ID, RuntimeDescriptor.VERSION));
		assertNotNull(registry.getPluginDescriptor(PLUGIN_ID));
	}
}
