package gov.va.med.hds.cd.runtime.util;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.xerces.jaxp.SAXParserFactoryImpl;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import gov.va.med.hds.cd.runtime.InvalidPluginDescriptorException;

public class PluginManifestValidator {

	private static final String JAXP_SCHEMA_LANGUAGE =
		"http://java.sun.com/xml/jaxp/properties/schemaLanguage";

	private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";

	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

	private static final String PLUGIN_MANIFEST_SCHEMA = "pluginManifest.xsd";
	private static final String REGISTRY_SCHEMA = "registry.xsd";

	public static void main(String[] args) {
		if (args.length != 1) {
			return;
		}
		PluginManifestValidator validator = new PluginManifestValidator();
		try {
			validator.validate(new FileInputStream(args[0]));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private SAXParser parser;

	public PluginManifestValidator() {
		SAXParserFactory factory = new SAXParserFactoryImpl();
		factory.setNamespaceAware(true);
		factory.setValidating(true);
		try {
			parser = factory.newSAXParser();
			parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void validate(InputStream inputStream) throws InvalidPluginDescriptorException, IOException {
		try {
			parser.setProperty(
							JAXP_SCHEMA_SOURCE, getClass().getClassLoader().getResourceAsStream(PLUGIN_MANIFEST_SCHEMA));
			getClass().getClassLoader().getResourceAsStream(PLUGIN_MANIFEST_SCHEMA).close();
			parser.parse(inputStream, new ParseErrorHandler());
		} catch (SAXException e) {
			throw new InvalidPluginDescriptorException(e);
		} catch (IOException e) {
			throw e;
		}
	}

	private class ParseErrorHandler extends DefaultHandler {

		public void error(SAXParseException exception) throws SAXException {
			throw exception;
		}

		public void fatalError(SAXParseException exception) throws SAXException {
			throw exception;
		}

	}
}
