package gov.va.med.hds.cd.runtime;

/**
 * A <code>PluginActivatingException</code> is thrown by the runtime when access to an
 * active plugin instance is requested but the plugin has not yet completed its activation
 * sequence.
 * 
 * This is most likely to occur in {@link gov.va.med.hds.cd.runtime.IPlugin#startup(IRuntimeContext, IPluginDescriptor)} implementations that
 * are invoked indirectly through {@link gov.va.med.hds.cd.runtime.IConfigurationElement#createExecutableExtension(String)} by 
 * another plugin's {@link gov.va.med.hds.cd.runtime.IPlugin#startup(IRuntimeContext, IPluginDescriptor)} method.
 * 
 * Avoiding this exception can generally be accomplised by delaying initialization of as much
 * plugin state as possible until it is asked for via the plugin's API.  This is referred to
 * as lazy initialization.
 * 
 * @see gov.va.med.hds.cd.runtime.IRuntimeContext#getPluginInstance(String)
 * @see gov.va.med.hds.cd.runtime.IRuntimeContext#getPluginInstance(String, Version)
 **/
public class PluginActivatingException extends PluginStartupException {
	private static final String MESSAGE = " plugin instance is still activating and is unavailable";
	
	/**
	 * Constructs a PluginActivatingException with the plugin descriptor this exception corresponds to.
	 * @param plugin the <code>IPluginDescriptor</code> of the plugin that is still activating
	 * at the time this exception was thrown.
	 */
	public PluginActivatingException(
		IPluginDescriptor plugin) {
		super(plugin + MESSAGE);
		this.plugin = plugin;
	}

}
