package gov.va.med.hds.cd.runtime;

import java.util.EventObject;

/**
 * PluginEvent is used to notify interested parties that changes in
 * plugin activation state have occurred. 
 * 
 * @see gov.va.med.hds.cd.runtime.IPluginActivationListener 
 * @see gov.va.med.hds.cd.runtime.IRuntimeContext
 */
public class PluginEvent extends EventObject {

	private IPluginDescriptor pluginDescriptor;

	/**
	 * Constructs a new <code>PluginEvent</code> object
	 * @param source The object that fired the event.
	 * @param p the <code>IPluginDescriptor</code> of the plugin
	 * this event corresponds to.
	 */
	public PluginEvent(Object source, IPluginDescriptor p) {
		super(source);
		this.pluginDescriptor = p;
	}

	/**
	 * Gets the <code>IPluginDescriptor</code> of the plugin this event
	 * corresponds to.
	 * 
	 * @return the <code>IPluginDescriptor</code> of the plugin
	 * this event corresponds to
	 */
	public IPluginDescriptor getPluginDescriptor() {
		return pluginDescriptor;
	}

}
