package gov.va.med.hds.cd.runtime;

import gov.va.med.hds.cd.mum.Version;

/**
 * Thrown when the runtime cannot find a registered plugin with the specified
 * identifier.
 */
public class PluginNotFoundException extends Exception {
	/**
	 * Constructs a new <code>PluginNotFoundException</code> with the specified
	 * plugin.
	 * @param pluginId the unique identifier of the requested plugin
	 */
	public PluginNotFoundException(String pluginId) {
		super("plugin '" + pluginId + "' has no registered versions with the runtime");
	}

	/**
	 * Constructs a new <code>PluginNotFoundException</code> with the specified
	 * plugin and version.
	 * @param pluginId the unique identifier of the requested plugin
	 * @param version the version of the requested plugin
	 * @see Version
	 */
	public PluginNotFoundException(String pluginId, Version version) {
		this(pluginId, version, VersionMatchCondition.PERFECT);
	}

	/**
	 * Constructs a new <code>PluginNotFoundException</code> with the specified
	 * plugin, version, and matchCondition
	 * @param pluginId the unique identifier of the requested plugin
	 * @param version the version of the requested plugin
	 * @param matchCondition the match condition that was specifed in order to find a matching plugin
	 * @see Version
	 * @see VersionMatchCondition
	 **/ 
	public PluginNotFoundException(
		String pluginId,
		Version version,
		VersionMatchCondition matchCondition) {
		super("no " + matchCondition + " matches of plugin '" + pluginId + " v" + version + "' are registered with the runtime");
	}
}
