package gov.va.med.hds.cd.runtime;

/**
 * A <code>PluginStartupException</code> is thrown to indicate that the runtime
 * was unable to startup a plugin.
 */
public class PluginStartupException extends Exception {

	private static final String MESSAGE = "unable to startup plugin";

	protected IPluginDescriptor plugin;

	protected PluginStartupException(String message) {
		super(message);
	}

	/**
	 * Constructs a new <code>PluginStartupException</code> with the specified plugin and cause.
	 * @param plugin the descriptor that did not
	 * startup
	 * @param cause the cause 
	 */
	public PluginStartupException(IPluginDescriptor plugin, Throwable cause) {
		super(MESSAGE + " '" + plugin + "'", cause);
		this.plugin = plugin;
	}
	
	/**
	 * Constructs a new <code>PluginStartupException</code> with the specified plugin and a detail
	 * message.
	 * @param pluginId the unique identifier of the plugin that did not startup
	 * @param message the detail message.
	 */
	public PluginStartupException(IPluginDescriptor plugin, String message) {
		super(MESSAGE + " '" + plugin + "': " + message);
		this.plugin = plugin;
	}
	
	/**
	 * Returns the descriptor of the plugin that was unable to startup.
	 * @return the unique identifier of the plugin that did not startup
	 */
	public IPluginDescriptor getPlugin() {
		return plugin;
	}
}
