package gov.va.med.hds.cd.runtime.logging;

/**
 * An <code>IHandler</code> object takes log messages from a Logger and exports them. 
 * 
 * It might for example, write them to a console or write them to a file, 
 * or send them to a network logging service, or forward them to an OS log,
 * or whatever.
 * 
 * @see gov.va.med.hds.cd.runtime.logging.ILogger
 * @see gov.va.med.hds.cd.runtime.logging.ILogRecord
 */
public interface IHandler {
	/**
	 * Publish an <code>ILogRecord</code>.
	 * 
	 * The logging request was made initially to an <code>ILogger object, which
	 * initialized the <code>ILogRecord</code> and forwarded it here. 
	 * The <code>IHandler</code> is responsible for formatting the message, 
	 * when and if necessary. The formatting should include localization. 
	 * 
	 * @param logRecord
	 */
	void publish(ILogRecord logRecord);
	/**
	 * Close the IHandler and free all associated resources.
	 */
	void close();
}
