package gov.va.med.hds.cd.runtime.logging;

/**
 * An interface that describes a class that is capable of providing logging
 * functionality.  Most logging operations are done through this interface.
 */
public interface ILoggingCapable {
	/**
	 * Same as calling getLogger(c.getName()).
	 * @param c the Class object whose name will be used to retrive a logger.
	 * @return a reference to the logger with the fully qualified name of the specified class.
	 */
	ILogger getLogger(Class c);
	/**
	 * Retrieve a logger by name.
	 * @param name A name for the logger. This should be a dot-separated name
	 * and should normally be based on the package name or class name of the
	 * subsystem, such as java.net or javax.swing
	 * @return a reference to the logger with the specified name.
	 */
	ILogger getLogger(String name);
	/**
	 * Retrieve the root logger.
	 * @return a reference to the root logger.
	 */
	ILogger getRootLogger();	
}
