package gov.va.med.hds.cd.security.test;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.cd.security.IUserInfo;
import gov.va.med.hds.cd.security.SecurityException;

import java.security.PrivilegedAction;
import java.util.Set;

import javax.security.auth.Subject;

/**
 * Tests a mock security plugin that succeeds logon
 */
public class SecuritySuccessTest extends RuntimeTestCase {
	public SecuritySuccessTest(String arg0) {
		super(arg0);
	}
	protected void setUp() throws Exception {
		super.setUp();
		parseRegistryResource("MockRegistrySuccess.xml");
		securityPlugin= (ISecurity) getRuntime().getPluginInstance(ISecurity.PLUGIN_ID);
	}
	public void testSimpleAuthentication() throws Exception {
		securityPlugin.login();
		securityPlugin.logout();
	}
	public void testSimpleActionAuthorization() throws Exception, SecurityException {
		securityPlugin.login();
		Subject.doAs(securityPlugin.getSubject(), new PrivilegedAction(){
			public Object run() {
				return null;
			}
		});
		securityPlugin.logout();
	}
	public void testUserInfoFromLogin() throws Exception{
		IUserInfo uinfo = securityPlugin.login();
		IUserInfo uinfo2 = securityPlugin.getUserInfo();
		assertEquals("MockFirst",uinfo.getFirstName());
		assertEquals("MockLast",uinfo.getLastName());
		assertEquals("MockMiddle",uinfo.getMiddleName());
		assertEquals("MockM.D.",uinfo.getTitle());
		assertEquals(uinfo,uinfo2);
		
		assertEquals("MockFirst",uinfo2.getFirstName());
		assertEquals("MockLast",uinfo2.getLastName());
		assertEquals("MockMiddle",uinfo2.getMiddleName());
		assertEquals("MockM.D.",uinfo2.getTitle());

		securityPlugin.logout();
	}
	public void testFindUserInfoPrincipal() throws Exception{
		securityPlugin.login();
		
		Subject subject = securityPlugin.getSubject();
		Set principals = subject.getPrincipals(IUserInfo.class);
		assertEquals(1,principals.size());
		IUserInfo uinfo = (IUserInfo)principals.iterator().next();

		assertEquals("MockFirst",uinfo.getFirstName());
		assertEquals("MockLast",uinfo.getLastName());
		assertEquals("MockMiddle",uinfo.getMiddleName());
		assertEquals("MockM.D.",uinfo.getTitle());
		
		securityPlugin.logout();
	}
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	private ISecurity securityPlugin= null;
}
