package gov.va.med.hds.cd.security;

import java.security.PrivilegedAction;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginException;

/**
 * Abstracts the concept of a security plugin.
 */
public interface ISecurity {
	/**
	 * Plugin id for plugging into the runtme registry
	 */
	public static final String PLUGIN_ID = "gov.va.med.hds.cd.security";
	/**
	 * Security plugin extension name
	 **/
	public static final String PLUGIN_EXTENSION_NAME = "gov.va.med.hds.cd.security.plugin";
	/**
	 * Class attribute name 
	 */
	public static final String CLASS_ATTRIBUTE_NAME = "classFactory";
	/**
	 * Plugin element name
	 */
	public static final String PLUGIN_ELEMENT_NAME = "plugin";

	/**
	 * Logs into the security system.
	 * @throws LoginException
	 */
	public IUserInfo login() throws LoginException;

	/**
	 * Logs out of the security system.
	 * @throws LoginException
	 */
	public void logout() throws LoginException;

	/**
	 * Gets the Subject containing user identities
	 * @return Subject
	 */
	public Subject getSubject();

	/**
	 * Gets general user information associated with this login.
	 * To get specific identity information associated with the user
	 * for a specific login connection walk through the Principals in
	 * the Subject obtained from getSubject() to find that information.
	 * 
	 * @return IUserInfo
	 * @throws CouldNotCreateUserInfoException
	 */
	public IUserInfo getUserInfo() throws CouldNotCreateUserInfoException;

	/**
	 * Executes a privileged security action on the internally registered subject.
	 * 
	 * @param action action to be performed.
	 */
	public void doAs(PrivilegedAction action);
}
