/*
 * Created on Apr 21, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.signature;

import java.awt.Frame;
import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JOptionPane;

import gov.va.med.hds.cd.prism.DesktopCloseVetoException;
import gov.va.med.hds.cd.prism.DesktopEvent;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.IDesktopListener;
import gov.va.med.hds.cd.prism.IDesktopWindow;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.dashboard.items.IDashboardItemTypes;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.signature.dao.ISignatureDAO;
import gov.va.med.hds.signature.ui.SignatureList;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class SignaturePlugin
	implements IPlugin, ISignaturePlugin, IDesktopListener {
	private IRuntimeContext runtime;
	private Map signableItems = Collections.synchronizedMap(new Hashtable());

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IPlugin#startup(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IPluginDescriptor)
	 */
	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
		throws Exception {
		this.runtime = runtime;
		PrismUtilities.getDesktop(runtime).addDesktopListener(this);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IPlugin#shutdown()
	 */
	public void shutdown() throws Exception {
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignaturePlugin#getSignableItem(java.lang.String)
	 */
	public IItemEntry getSignableItem(String id) {
		if (signableItems.isEmpty()) getSignableItems();
		return (IItemEntry) signableItems.get(id);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.ISignaturePlugin#iterator()
	 */
	public Collection values() {
		if (signableItems.isEmpty()) getSignableItems();
		return signableItems.values();
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopClosing(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopClosing(DesktopEvent e)
		throws DesktopCloseVetoException {
		boolean hasPending = false;
		Iterator signables = values().iterator();
		while (signables.hasNext()) {
			IItemEntry itemType = (IItemEntry) signables.next();
			try {
				ISignatureDAO dao =
					(ISignatureDAO) itemType.getItemDAOInstance();
				if (0 < dao.getSignatureCandidates().size()) {
					hasPending = true;
					break;
				}
			} catch (Exception ex) {
				runtime.getLogger(this.getClass()).error(
					ex.getLocalizedMessage(),
					ex);
			}
		}
		if (hasPending) {
			IDesktop desktop = PrismUtilities.getDesktop(runtime);
			IDesktopWindow activeWindow = desktop.getActiveWindow();
			Frame parent = null;
			if (activeWindow != null)
				parent = activeWindow.getFrame();
			Object[] options = { "Sign", "Exit" };
			int reply =
				JOptionPane.showOptionDialog(
					parent,
					"There are documents or orders you have selected for signing.\n"
						+ "To sign these items click the '"
						+ options[0]
						+ "' button to open the Sign List perspective.",
					"Outstanding Transactions",
					JOptionPane.DEFAULT_OPTION,
					JOptionPane.WARNING_MESSAGE,
					null,
					options,
					options[0]);
			if (reply == 0)
      {
        activeWindow.setActivePerspective(SignatureList.PERSPECTIVE_ID);
        throw new DesktopCloseVetoException(
          "Pending items on Sign List",
          e);
      }
		}
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.prism.IDesktopListener#desktopOpening(gov.va.med.hds.cd.prism.DesktopEvent)
	 */
	public void desktopOpening(DesktopEvent e) {
	}

	private void getSignableItems() {
		try {
			IDashboardItemTypes itemTypes =
				(IDashboardItemTypes) runtime.getPluginInstance(
					IDashboardItemTypes.PLUGIN_ID);
			for (Iterator items = itemTypes.itemIterator(); items.hasNext();) {
				IItemEntry item = (IItemEntry) items.next();
				if (item.requiresSignature())
					signableItems.put(item.getID(), item);
			}
		} catch (Exception ex) {
			runtime.getLogger(getClass()).error(
				"Unable to get instance of "
					+ IDashboardItemTypes.class.getName(),
				ex);
		}
	}

	public void desktopClosed(DesktopEvent e) {
		// TODO_SB: Auto-generated method stub
		
	}

}
