/*
 * Created on Jun 4, 2003
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 */
package gov.va.med.hds.signature.dao.m;

import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;

import gov.va.med.hds.cd.daoregistry.IDataAccessObject;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.signature.ISignatureCoreConstants;
import gov.va.med.hds.signature.dao.ISignatureDAO;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 */
public class SignatureDAO implements IDataAccessObject, ISignatureDAO
{
  protected IRuntimeContext runtime;
  protected IMumps mumps = null;
  protected ILogger logger = null;

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.dao.ISignatureDAO#getSignatureCandidates()
	 */
	public List getSignatureCandidates()
	{
		// Not implemented
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.dao.ISignatureDAO#getItem(java.lang.Object)
	 */
	public Object getItem(Object id)
	{
    // Not implemented
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.signature.dao.ISignatureDAO#sign(java.lang.Object)
	 */
	public void sign(Object signature) throws SignatureException
	{
    boolean success = true;
    StringBuffer unsuccessful = new StringBuffer();
    try
    {
      String sigCode = mumps.encrypt(signature.toString());
      List params = new ArrayList(1);
      params.add(sigCode);
      if (!("1".equals(mumps.callRPC(VERIFY_SIG, params))))
      { 
        success = false;
        unsuccessful.append("\n\tYou have entered an incorrect Electronic Signature Code...Try again!");
      }
    }
    catch (Exception e)
    {
      logger.error(e.getLocalizedMessage(), e);
      unsuccessful.append("\n" + e.getMessage());
      success = false;
    }
    if (!success)
      throw new SignatureException(unsuccessful.toString());
}

	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data)
		throws Exception
	{
    this.runtime = runtime;
    this.logger = runtime.getLogger(this.getClass());

  	IPluginDescriptor desc = runtime.getPluginRegistry().getPluginDescriptor(ISignatureCoreConstants.PLUGIN_ID);
  	mumps = ((IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
	}
}
