package gov.va.med.hds.signature.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
//import java.awt.FocusTraversalPolicy;
//import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.help.IHelp;
import gov.va.med.hds.cd.prism.IDesktop;
import gov.va.med.hds.cd.prism.PrismUtilities;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveListener;
import gov.va.med.hds.cd.prism.perspective.IPerspectiveReference;
import gov.va.med.hds.cd.prism.perspective.PerspectiveEvent;
import gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.signature.ISignaturePlugin;
import gov.va.med.hds.signature.dao.ISignatureDAO;

/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public class SignatureList extends JPanel implements IVisualComponent, IPerspectiveListener {
	private static final String INTRO_HELP_ID = "gov.va.med.hds.signature.ui.intro";
	public final static String PERSPECTIVE_ID = "dashboard.signatureList";
	private final String VALID_SIGNATURE_DAO = "gov.va.med.hds.signature.dao.ValidSignature";

	protected DHTMLPane dhtmlPane;
	protected JPasswordField signature;
	protected JButton signButton;

	protected ISignaturePlugin plugin;
	protected Collection signableItems;
	protected List actionableItems = new ArrayList();

	private IRuntimeContext runtime = null;
	private IDesktop desktop = null;
	private ISecurity security;
	private ISignatureDAO validSignature;

	private final String urlBase = "cdtp:///gov/va/med/hds/signature/SignList";


	public void signature_actionPerformed(ActionEvent e) {
		class doSign implements Runnable {
			SignatureList parent;
			public doSign(SignatureList parent) {
				this.parent = parent;
			}
			public void run() {
				security.doAs(new SignAction(parent, signableItems, signature, runtime, validSignature));
				signature.setText("");
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						try {
							dhtmlPane.reload();
						} catch (IOException ioe) {
							runtime.getLogger(this.getClass()).error(ioe.getLocalizedMessage(), ioe);
						}
					}
				});
			}
		}
		if ("Sign".equals(e.getActionCommand()))
			SwingUtilities.invokeLater(new doSign(this));
	}

	public void signature_CaretChanged(CaretEvent e) {
    int sigLen = ((JPasswordField)e.getSource()).getPassword().length;
		signButton.setEnabled((0 < sigLen));
		getRootPane().setDefaultButton(signButton);
	}

	private void maybeReloadPane(IPerspectiveReference perspective) {
		signature.requestFocusInWindow();
		String perspectiveId = perspective.getDescriptor().getId();
		if (perspectiveId.endsWith(PERSPECTIVE_ID)) {
			URL currentURL = dhtmlPane.getPage();
			if (null != currentURL) {
				String path = currentURL.getPath();
				if (path.endsWith("SignList"))
					SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						try {
							dhtmlPane.reload();
						} catch (IOException ioe) {
							runtime.getLogger(this.getClass()).error(ioe.getLocalizedMessage(), ioe);
						}
					}
				});
			}
		}
	}

	public void perspectiveActivated(PerspectiveEvent e) {
//		boolean root = isFocusCycleRoot();
//		FocusTraversalPolicy ftp = getFocusTraversalPolicy();
		JRootPane rp = getRootPane();
		if (rp != null)
			rp.setDefaultButton(signButton);
		signature.requestFocusInWindow();
		signature.setEnabled(0 < getSignableItemCount());

		IPerspectiveReference perspective = e.getPerspective();
		maybeReloadPane(perspective);
	}

	public void perspectiveClosed(PerspectiveEvent e) {
	}

	public void perspectiveOpened(PerspectiveEvent e) {
	}

	/* @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String, java.lang.Object) */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception {
		this.runtime = runtime;
		this.desktop = PrismUtilities.getDesktop(runtime);
		this.plugin = (ISignaturePlugin) runtime.getPluginInstance(ISignaturePlugin.PLUGIN_ID);
		this.signableItems = plugin.values();

		desktop.getPerspectiveEventSource().addPerspectiveListener(this);

		security = (ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
		IDaoRegistry daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
		validSignature = (ISignatureDAO) daoRegistry.getDAO(VALID_SIGNATURE_DAO);
		if (null == validSignature)
			throw new Exception("No Data Access Object for " + VALID_SIGNATURE_DAO);

		setLayout(new BorderLayout());

		add(new JLabel("Enter your electronic signature code to sign the items below that are checked."), BorderLayout.NORTH);
		JScrollPane scroller = new JScrollPane();
		dhtmlPane = new DHTMLPane(runtime);
		dhtmlPane.setContentType("text/html");
		try {
			dhtmlPane.setPage(urlBase);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		scroller.getViewport().add(dhtmlPane);
		add(scroller, BorderLayout.CENTER);

//		GridBagLayout gridBag = new GridBagLayout();

		JPanel bottomPane = new JPanel();
		bottomPane.setLayout(new FlowLayout(FlowLayout.RIGHT));

		Component leader = Box.createHorizontalStrut(100);
		JLabel label = new JLabel("Signature Code");
		signature = new JPasswordField(25);
		signature.addCaretListener(new SignatureKeyCaretAdapter(this));
		signature.setEnabled(0 < getSignableItemCount());
		signButton = new JButton("Sign");
		signButton.addActionListener(new SignatureActionEventAdapter(this));
		signButton.setEnabled((0 != signature.getPassword().length));

		bottomPane.add(leader);
		bottomPane.add(label);
		bottomPane.add(signature);
		bottomPane.add(signButton);

		add(bottomPane, BorderLayout.SOUTH);
		dhtmlPane.setVisible(true);
		addHelpIds();
	}
	
	private void addHelpIds() {
		this.putClientProperty(IHelp.HELPID_CLIENT_PROPERTY_KEY, INTRO_HELP_ID);
	}

	/* @see gov.va.med.hds.cd.prism.visualcomponent.IVisualComponent#getComponent() */
	public JComponent getComponent() {
		return this;
	}

	public void perspectiveClosing(PerspectiveEvent e) {
	}

	public void perspectiveOpening(PerspectiveEvent e) {
	}

	private int getSignableItemCount() {
		List candidates = new ArrayList();
		Iterator signableTypes = signableItems.iterator();
		while (signableTypes.hasNext()) {
			ISignatureDAO signDAO;
			try {
				signDAO = (ISignatureDAO) ((IItemEntry) signableTypes.next()).getItemDAOInstance();
			} catch (Exception e) {
				continue;
			}
			candidates.addAll(signDAO.getSignatureCandidates());
		}
		return candidates.size();
	}
}

class SignAction implements PrivilegedAction {
	private Object signature;
	private SignatureList pane;
	private Iterator signableTypes;
	private ILogger logger;
	private ISignatureDAO validSignature;

	SignAction(SignatureList pane, Collection signableTypes, Object signature, IRuntimeContext runtime, ISignatureDAO validSignature) {
		this.signature = signature;
		this.pane = pane;
		this.signableTypes = signableTypes.iterator();
		this.logger = runtime.getLogger(this.getClass());
		this.validSignature = validSignature;
	}

	public Object run() {
		boolean successful = true;
		StringBuffer failureMsg = new StringBuffer();
		if (signature instanceof JPasswordField) {
			JPasswordField signField = (JPasswordField) signature;
			try {
				validSignature.sign(new String(signField.getPassword()));
				while (signableTypes.hasNext()) {
					ISignatureDAO signDAO = (ISignatureDAO) ((IItemEntry) signableTypes.next()).getItemDAOInstance();
					List candidates = signDAO.getSignatureCandidates();
					if (0 != candidates.size())
						signDAO.sign(new String(signField.getPassword()));
				}
			} catch (Exception e) {
				logger.debug(e.getLocalizedMessage(), e);
				failureMsg.append(e.getLocalizedMessage());
				successful = false;
			}
			if (!successful)
				JOptionPane.showMessageDialog(pane, "One or more items were not signed: " + failureMsg.toString(), "Signing Warning", JOptionPane.WARNING_MESSAGE);
		}
		return new Boolean(successful);
	}
}

class SignatureActionEventAdapter implements ActionListener {
	SignatureList adaptee;

	SignatureActionEventAdapter(SignatureList adaptee) {
		this.adaptee = adaptee;
	}

	public void actionPerformed(ActionEvent e) {
		adaptee.signature_actionPerformed(e);
	}
}

class SignatureKeyCaretAdapter implements CaretListener {
	SignatureList adaptee;

  SignatureKeyCaretAdapter(SignatureList adaptee) {
		this.adaptee = adaptee;
	}

  public void caretUpdate(CaretEvent e)
  {
    adaptee.signature_CaretChanged(e);
  }

}
