package gov.va.med.hds.signature.ui.scriptlet;

import javax.swing.JToggleButton;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptlet;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptletEvent;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.signature.ISignableItem;
import gov.va.med.hds.signature.ISignatureEventSource;
import gov.va.med.hds.signature.ISignaturePlugin;
import gov.va.med.hds.signature.ISignatureStatusChangeListener;
import gov.va.med.hds.signature.dao.ISignatureDAO;
/**
 * @author vhaislschras
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @version 
 */
public class Signature implements IScriptlet {
	/*
	 * Public scriptlet methods
	 */
	public static final String SIGNATURE = "signature";
	private IRuntimeContext runtime;
	private ILogger logger;
	private IWorkspace workspace;
	private ISignaturePlugin plugin;
	private ISignatureEventSource signatureEventSource;
	
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptlet#init(IScriptletEvent)
	 */
	public void init(IScriptletEvent event) {
		this.runtime = (IRuntimeContext) event.getEnvironment();
		this.logger = runtime.getLogger(this.getClass());
		try {
			plugin =
				(ISignaturePlugin) runtime.getPluginInstance(
					ISignaturePlugin.PLUGIN_ID);
			workspace =
				(IWorkspace) runtime.getPluginInstance(IWorkspace.PLUGIN_ID);
			signatureEventSource = (ISignatureEventSource) workspace.getScope(IWorkspaceConstants.SESSION_SCOPE).getModelSpace().getModel(ISignatureEventSource.MODEL_ID);
		} catch (PluginStartupException e) {
			logger.error(e.getLocalizedMessage(), e);
		} catch (PluginNotFoundException e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}
	public void signature(IScriptletEvent event, String id, String itemType) {
		int suffixIdx = itemType.lastIndexOf('~');
		if (-1 != suffixIdx)
			itemType = itemType.substring(0, suffixIdx);
		ISignatureDAO dao = null;
		try {
			dao =
				(ISignatureDAO) plugin
					.getSignableItem(itemType)
					.getItemDAOInstance();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}
		String itemId = id.split("_")[1];
		Element ckBox = event.getElement();
		// Get the current state of the checkbox
		AttributeSet attrs = ckBox.getAttributes();
		JToggleButton.ToggleButtonModel model =
			(JToggleButton.ToggleButtonModel) attrs.getAttribute(
				StyleConstants.ModelAttribute);
		// Just getting one item
		ISignableItem item = (ISignableItem) dao.getItem(itemId);
		item.setReadyToSign(model.isSelected());
		fireChangeNotification(item);
	}
	private void fireChangeNotification(ISignableItem item) {
		signatureEventSource.fireSignatureStatusChange(item, (item.isReadyToSign()
		? ISignatureStatusChangeListener.SIGNATURE_SIGNABLE
		: ISignatureStatusChangeListener.SIGNATURE_NOT_SIGNABLE));
	}
}
