package gov.va.med.hds.signature.ui.servlet;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.logging.ILogger;

import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;

//import gov.va.med.hds.dashboard.DashPatient;
//import gov.va.med.hds.dashboard.DashPatientList;
//import gov.va.med.hds.dashboard.DashboardPlugin;
//import gov.va.med.hds.dashboard.UnsignedItem;

import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.ServletWriter;
import gov.va.med.hds.clinicaldata.ui.PatientLine;
import gov.va.med.hds.dashboard.items.IItemEntry;
import gov.va.med.hds.signature.ISignableItem;
import gov.va.med.hds.signature.ISignaturePatient;
import gov.va.med.hds.signature.ISignaturePlugin;
import gov.va.med.hds.signature.dao.ISignatureDAO;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 *
 */
public class SignList implements IServlet
{
  private final static int MAX_TITLE_WIDTH = 115;
  private IRuntimeContext runtime;
//  private List signableItems;
  private ILogger logger;
  private ISignaturePlugin plugin = null;

  private ServletWriter out;
  
  /* (non-Javadoc)
   * @see gov.va.med.hds.cd.url.IServlet#service(IServletRequest, IServletResponse)
   */
  public void service(IServletRequest request, IServletResponse response)
      throws IOException
  {
    runtime = request.getRuntimeContext();
    this.logger = runtime.getLogger(this.getClass());
    try
    {
      plugin = (ISignaturePlugin)runtime
        .getPluginInstance(ISignaturePlugin.PLUGIN_ID);
    }
    catch (PluginStartupException pse)
    {
      logger.error(pse.getLocalizedMessage(), pse);
    }
    catch (PluginNotFoundException pnfe)
    {
      logger.error(pnfe.getLocalizedMessage(), pnfe);
    }
    
    
    out = response.getWriter();
    response.setContentType("text/html");
    
    String[] scriptlets = {
      "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse",
      "gov.va.med.hds.signature.ui.scriptlet.Signature"
    };
    
    out.write("<html><head>");
    out.write(htmlHeader(scriptlets));
    out.write("</head><body>");
    out.write(htmlBody(signableItems()));
    out.write("</body></html>");
  }

  protected String htmlHeader(String[] scriptlets)
  {
    StringBuffer html = new StringBuffer();
    for (int s=0; s<scriptlets.length; s++)
      html.append("<meta name='scriptlet' content='" + scriptlets[s] + "'>");
    html.append("<link rel='stylesheet' type='text/css' href='signature.css'>");

    return html.toString();
  }

  protected String htmlBody(List items)
  {
    StringBuffer html = new StringBuffer();
    String lastPatient = "";
    String lastType = "";
    if (0 < items.size())
    {
      Iterator i = items.iterator();
      while (i.hasNext())
      {
        ISignableItem item = (ISignableItem) i.next();
        ISignaturePatient patient = (ISignaturePatient)item.getSignaturePatient();
        if (!(lastPatient.equals(patient.getPatientId())))
        {
          if (0 != lastPatient.length())
            html.append("</table></div>");
          html.append("<div class='signbox'>");
          PatientLine patLine = patient.getPatientLine();
          html.append(patLine.getPatientLineHTML());
          lastPatient = (String)patient.getPatientId();
          lastType = "";
        }
        if (!(lastType.equals(item.getTypeID())))
        {
          if (0 < lastType.length())
            html.append("</table>");
          html.append("<table cellspacing='1' cellpadding='0'>");      
          String signableType = "";
          String itemTypeID = item.getTypeID();
          // Some types have programmatically added suffixes
          // that should be used for the signableType.
          int suffixIdx = itemTypeID.lastIndexOf('~');
          if (-1 != suffixIdx)
            signableType = itemTypeID.substring(suffixIdx+1);
          else
          {
            Iterator itemTypes = plugin.values().iterator();
            while (itemTypes.hasNext())
            {
              IItemEntry itemType = (IItemEntry)itemTypes.next();
              if (itemType.getID().equals(item.getTypeID()))
                signableType = itemType.getName();
            }
          }
          html.append("<tr><td><b>&nbsp; " + signableType + "</b></td></tr>");
          lastType = item.getTypeID();
        }
        String id = item.getID();
        html.append("<tr><td><table cellpadding='2'>");
        html.append("<tr><td width='1%'>&nbsp;&nbsp;&nbsp;");
        html.append("<input type='checkbox' checked='true' id='ck_" + id + "' " +
          "onchange=\"signature('ck_" + id + "', '" + item.getTypeID() + "')\" label=''>");
        html.append("</td><td>");
        String title = item.getShortTitle();
        int titleLen = title.length();
        html.append(out.escapeEntities(title.substring(0,Math.min(MAX_TITLE_WIDTH, titleLen)),false) +
          (50 < titleLen ? "..." : ""));
        html.append("</td></tr></table></td></tr>");
      }
      html.append("</table></div>");
    }
    return html.toString();
  }
  
  private List signableItems()
  {
    List candidates = new ArrayList();
    if (null != plugin)
    {
      Iterator signables = plugin.values().iterator();
      while (signables.hasNext())
			{
				IItemEntry itemType = (IItemEntry)signables.next();
				try
				{
					ISignatureDAO dao = (ISignatureDAO)itemType.getItemDAOInstance();
          candidates.addAll(dao.getSignatureCandidates());
				}
				catch (Exception e)
				{
          logger.error(e.getLocalizedMessage(),e);
				}
			}
      Collections.sort(candidates, ascendingID);
    }
    return candidates;    
  }

  public static Comparator ascendingID = new Comparator(){
    public int compare(Object obj1, Object obj2)
    {
      if (!(obj1 instanceof ISignableItem) || null == obj1)
        return -1;
      if (!(obj1 instanceof ISignableItem) || null == obj1)
        return 1;
      
      ISignableItem item1 = (ISignableItem)obj1;
      ISignableItem item2 = (ISignableItem)obj2;
      
      String name1 = item1.getSignaturePatient().toString();
      String id1 = item1.getSignaturePatient().getPatientId().toString();
      String objId1 = item1.getTypeID();
      String name2 = item2.getSignaturePatient().toString();
      String id2 = item2.getSignaturePatient().getPatientId().toString();
      String objId2 = item2.getTypeID();

      String compare1 = name1 + id1 + objId1;
      String compare2 = name2 + id2 + objId2;
      
      return compare1.compareTo(compare2);
    }
  };

}
