package gov.va.med.hds.cd.spectrum.itest;

import java.awt.event.KeyEvent;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.JFCTestHelper;
import junit.extensions.jfcunit.eventdata.KeyEventData;
import junit.extensions.jfcunit.eventdata.MouseEventData;

import gov.va.med.hds.cd.spectrum.list.AssertEvent;
import gov.va.med.hds.cd.spectrum.list.AssertList;
import gov.va.med.hds.cd.spectrum.list.IAssertListener;

/**
 * @author vhaislmonsos
 *
 * Mar 6, 2003 1:17:10 PM
 */
public class AssertListTest extends JFCTestCase {
	private JFCTestHelper helper = new JFCTestHelper();
	private JFrame f1;
	private int mouseCount;
	private int enterCount;
	private AssertList assertList;

	/**
	 * Constructor for AssertListTest.
	 * @param arg0
	 */
	public AssertListTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AssertListTest.class);
	}

	public void testEnter() throws Exception {
		AssertList list = (AssertList)JFCTestHelper.findComponent(AssertList.class, f1, 0);
		assertNotNull("AssertList <enter> test error: component not found.",list);
		helper.sendKeyAction(new KeyEventData(this, list, KeyEvent.VK_ENTER));
		assertEquals("AssertList <enter> test error:",1,enterCount);
		assertEquals("AssertList <enter> test error:",1,assertList.getSelectedIndices().length);
	}
	
	public void testSingleClick() throws Exception {
		AssertList list = (AssertList)JFCTestHelper.findComponent(AssertList.class, f1, 0);
		assertNotNull("AssertList <enter> test error: component not found.",list);
		helper.enterClickAndLeave(new MouseEventData(this, list, 1));
		assertEquals("AssertList mouse click test error:",1,mouseCount);
		assertEquals("AssertList mouse click test error:",1,assertList.getSelectedIndices().length);
	}
	
	public void testDoubleClick() throws Exception {
		AssertList list = (AssertList)JFCTestHelper.findComponent(AssertList.class, f1, 0);
		list.setClickCount(AssertList.DOUBLE_CLICK);
		assertNotNull("AssertList <enter> test error: component not found.",list);
		helper.enterClickAndLeave(new MouseEventData(this, list, 2));
		assertEquals("AssertList mouse click test error:",1,mouseCount);
		assertEquals("AssertList mouse click test error:",1,assertList.getSelectedIndices().length);
	}

	private void enterPressed() {
		enterCount++;
	}
	
	private void mouseClick() {
		mouseCount++;
	}
	
	protected void tearDown() throws Exception {
		JFCTestHelper.cleanUp(this);
		JFCTestHelper.disposeWindow(f1,this);
		super.tearDown();
	}
	
	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		enterCount = 0;
		mouseCount = 0;
		f1 = new JFrame("Enter Test");
//		f1.getContentPane().setLayout(new GridBagLayout());
		assertList = new AssertList();
		DefaultListModel model = new DefaultListModel();
		model.addElement(new String("one"));
		model.addElement(new String("two"));
		model.addElement(new String("three"));
		model.addElement(new String("four"));
		assertList.setModel(model);
		assertList.setSelectedIndex(0);
		assertList.addAssertListener(new IAssertListener() {
			public void valueChanged(AssertEvent e) {
				if (e.getEventID()==AssertEvent.MOUSE_CLICK) {
					mouseClick();
				}
				else if (e.getEventID()==AssertEvent.ENTER_PRESSED) {
					enterPressed();
				}
				else {
					fail("AssertList event failed: wrong eventID " + Integer.toString(e.getEventID()));
				}
			}
		});
		f1.getContentPane().add(assertList);
		f1.pack();
		f1.show();
		flushAWT();
	}

}
