package gov.va.med.hds.cd.spectrum.itest;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.BufferListDataEvent;
import gov.va.med.hds.cd.spectrum.longlist.BufferModel;
import gov.va.med.hds.cd.spectrum.longlist.BufferModelException;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.DataRequest;
import gov.va.med.hds.cd.spectrum.longlist.DataRequestFactory;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.IListPositionListener;

/**
 * Tests for the buffer model, used by the long list lookup component.
 */
public class BufferModelTest extends TestCase implements IDataNeededListener,ListDataListener,IListPositionListener {
	private IBufferModel bufferModel;
	private IDataNeededEvent event;
	private int dataSize = 0;
	private boolean delayedCall = false;
	private ListDataEvent listDataEvent;
	private int eventFired;
	private Thread testThread;
	private IBufferItem[] dataItems;
	private BufferListDataEvent positionEvent;

	public BufferModelTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(BufferModelTest.class);
	}
	
	protected void setUp() throws Exception {
		super.setUp();
		bufferModel = new BufferModel(300);
		bufferModel.addDataNeededListener(this);
		bufferModel.addListDataListener(this);
		dataSize = 0;
		delayedCall = false;
		event = null;
		listDataEvent = null;
		testThread = Thread.currentThread();
		eventFired = -1;
	}
	
	//Test BufferModel where the data size exceeds the maximum buffer size
	//
	public void testGetTop_large() throws BufferModelException, InterruptedException {
		setDataSize(10000);
		bufferModel.setDefaultCallSize(30);
		while (!bufferModel.getDataRequest(DataRequestFactory.getTop())){};
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",30,bufferObjects.length);
		assertEquals("BufferModel.getTop failed:",30,bufferModel.getSize());
		assertEquals("BufferModel.getTop failed:",0,listDataEvent.getIndex0());
		assertEquals("BufferModel.getTop failed:",29,listDataEvent.getIndex1());
		Object item = bufferObjects[0];
		//0,1,10,...,10,100,1000,1001,1002,1003,4,5,6,7,8,9,101,1010,1,2,3,4,5,6,7,8,9
		//0 1  2     11  12  13   14   15   16  ...
		assertEquals("BufferModel.getTop failed:","*0*",item.toString());
		assertEquals("BufferModel.getTop failed:","*0*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getTop failed:","*100*",bufferModel.getElementAt(12).toString());
		assertEquals("BufferModel.getTop failed:","*1000*",bufferModel.getElementAt(13).toString());
		item = bufferObjects[29];
		assertEquals("BufferModel.getTop failed:","*1015*",item.toString());
		assertEquals("BufferModel.getTop failed:","*1015*",bufferModel.getElementAt(29).toString());
		assertTrue("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetBottom_large() throws BufferModelException, InterruptedException {
		setDataSize(10000);
		bufferModel.setDefaultCallSize(20);
		while (!bufferModel.getDataRequest(DataRequestFactory.getBottom())){};
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getBottom failed:",20,bufferObjects.length);
		assertEquals("BufferModel.getBottom failed:",20,bufferModel.getSize());
		assertEquals("BufferModel.getBottom failed:",0,listDataEvent.getIndex0());
		assertEquals("BufferModel.getBottom failed:",19,listDataEvent.getIndex1());
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getBottom failed:","*9981*",item.toString());//*9990*
		assertEquals("BufferModel.getBottom failed:","*9981*",bufferModel.getElementAt(0).toString());
		item = bufferObjects[19];
		assertEquals("BufferModel.getBottom failed:","*9999*",item.toString());
		assertEquals("BufferModel.getBottom failed:","*9999*",bufferModel.getElementAt(19).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext1_large() throws BufferModelException, InterruptedException {
		//Next after top
		setDataSize(10000);
		bufferModel.setDefaultCallSize(30);
		while (!bufferModel.getDataRequest(DataRequestFactory.getTop())){};
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		assertEquals("BufferModel.getNext failed:","*0*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*1015*",bufferModel.getElementAt(29).toString());
		while (!bufferModel.getDataRequest(DataRequestFactory.getNext(50))){};
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		assertEquals("BufferModel.getNext failed:","*1034*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*1060*",bufferModel.getElementAt(29).toString());
		Object[] bufferObjects = bufferModel.getBufferFull();
		Object item = bufferObjects[30];
		assertEquals("BufferModel.getNext failed:","*1016*",item.toString());
		item = bufferObjects[79];
		assertEquals("BufferModel.getNext failed:","*1060*",item.toString());
		assertEquals("BufferModel.getNext failed:",80,bufferObjects.length);
		assertNotNull("BufferModel.getNext failed:",listDataEvent);
		assertEquals("BufferModel.getNext failed:",30,bufferModel.getSize());
		assertEquals("BufferModel.getNext failed:",0,listDataEvent.getIndex0());
		assertEquals("BufferModel.getNext failed:",29,listDataEvent.getIndex1());
		assertEquals("BufferModel.getNext failed:","*1034*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*1060*",bufferModel.getElementAt(29).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext2_large() throws BufferModelException, InterruptedException {
		//Next in the middle of the data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		IBufferItem searchItem = new BufferItem("*9800*","");
		listDataEvent = null;
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,searchItem,100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*9800*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*9890*",item.toString());
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		assertEquals("BufferModel.getNext failed:",100,bufferModel.getSize());
		assertEquals("BufferModel.getNext failed:",0,listDataEvent.getIndex0());
		assertEquals("BufferModel.getNext failed:",99,listDataEvent.getIndex1());
		assertEquals("BufferModel.getNext failed:","*9800*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9890*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(2))){}
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		assertEquals("BufferModel.getNext failed:","*9802*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9891*",bufferModel.getElementAt(98).toString());
		assertEquals("BufferModel.getNext failed:","*9892*",bufferModel.getElementAt(99).toString());
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",102,bufferObjects.length);
		item = bufferObjects[0];
		assertEquals("BufferModel.getNext failed:","*9800*",item.toString());
		item = bufferObjects[101];
		assertEquals("BufferModel.getNext failed:","*9892*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext3_large() throws BufferModelException, InterruptedException {
		//Next with the same call size as the remaining data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		IBufferItem search = new BufferItem("*9909*","");
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,search,100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getNext failed:","*9909*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getNext failed:","*9999*",item.toString());
		assertEquals("BufferModel.getNext failed:","*9909*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9999*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(1))){}
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",100,bufferObjects.length);
		item = bufferObjects[0];
		assertEquals("BufferModel.getNext failed:","*9909*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getNext failed:","*9999*",item.toString());
		assertEquals("BufferModel.getNext failed:","*9909*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9999*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext4_large() throws BufferModelException, InterruptedException {
		//Next with request less than remaining in the data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(20);
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*9950*","")))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",20,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getNext failed:","*9950*",item.toString());
		item = bufferObjects[19];
		assertEquals("BufferModel.getNext failed:","*9968*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext())){}
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",40,bufferObjects.length);
		item = bufferObjects[20];
		assertEquals("BufferModel.getNext failed:","*9969*",item.toString());
		item = bufferObjects[39];
		assertEquals("BufferModel.getNext failed:","*9986*",item.toString());
		assertEquals("BufferModel.getNext failed:","*9969*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9986*",bufferModel.getElementAt(19).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext())){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getNext failed:",54,bufferObjects.length);
		item = bufferObjects[40];
		assertEquals("BufferModel.getNext failed:","*9987*",item.toString());
		item = bufferObjects[53];
		assertEquals("BufferModel.getNext failed:","*9999*",item.toString());
		assertEquals("BufferModel.getNext failed:","*9981*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getNext failed:","*9999*",bufferModel.getElementAt(19).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetPrev1_large() throws BufferModelException, InterruptedException {
		//Prev after bottom
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getBottom())){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getPrev failed:","*9909*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getPrev failed:","*9999*",item.toString());
		assertEquals("BufferModel.getPrev failed:","*9909*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getPrev failed:","*9999*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetPrev2_large() throws BufferModelException, InterruptedException {
		//Prev in the middle of the data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		IBufferItem searchItem = new BufferItem("*5000*","");
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,searchItem,100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",100,bufferObjects.length);
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(10))){}
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",110,bufferObjects.length);
		Object item = bufferObjects[109];
		assertEquals("BufferModel.getPrev failed:","*5090*",item.toString());
		item = bufferObjects[0];
		assertEquals("BufferModel.getPrev failed:","*4993*",item.toString());
		assertEquals("BufferModel.getPrev failed:","*4993*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getPrev failed:","*5081*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetPrev3_large() throws BufferModelException, InterruptedException {
		//Prev with the same call size as the remaining data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		Object search = new BufferItem("*108*","");
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,search,100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",100,bufferObjects.length);
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(100))){}
		listDataEvent = null;
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",200,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getPrev failed:","*0*",item.toString());
		item = bufferObjects[199];
		assertEquals("BufferModel.getPrev failed:","*117*",item.toString());
		assertEquals("BufferModel.getPrev failed:","*0*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getPrev failed:","*1079*",bufferModel.getElementAt(99).toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetPrev4_large() throws BufferModelException, InterruptedException {
		//Prev with request less than remaining in the data elements
		setDataSize(10000);
		bufferModel.setDefaultCallSize(100);
		while (!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*1034*",""),100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",100,bufferObjects.length);
		listDataEvent = null;
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getPrev failed:",150,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getPrev failed:","*0*",item.toString());
		item = bufferObjects[49];
		assertEquals("BufferModel.getPrev failed:","*1033*",item.toString());
		assertEquals("BufferModel.getPrev failed:","*0*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel.getPrev failed:","*1079*",bufferModel.getElementAt(99).toString());
		assertTrue("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

//	//Test BufferModel where the data size is the same as the maximum buffer size
	public void testGetTop_medium() throws BufferModelException, InterruptedException {
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*0*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*187*",item.toString());
		assertTrue("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetBottom_medium() throws BufferModelException, InterruptedException {
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getBottom())){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*279*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*99*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext1_medium() throws BufferModelException, InterruptedException {
		//Next after top
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(50))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",150,bufferObjects.length);
		Object item = bufferObjects[149];
		assertEquals("BufferModel.getTop failed:","*232*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext2_medium() throws BufferModelException, InterruptedException {
		//Next in the middle of the data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		Object searchItem = new BufferItem("*137*","");
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,searchItem,100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*137*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*231*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext3_medium() throws BufferModelException, InterruptedException {
		//Next with the same call size as the remaining data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		Object search = new BufferItem("*279*","");
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,search,100))){}
		synchronized(testThread) {
			testThread.wait(6000);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*279*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*99*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetNext4_medium() throws BufferModelException, InterruptedException {
		//Next with request less than remaining in the data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*54*",""),100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",50,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*54*",item.toString());
		item = bufferObjects[49];
		assertEquals("BufferModel.getTop failed:","*99*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetPrev1_medium() throws BufferModelException, InterruptedException {
		//Prev after bottom
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getBottom())){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertNotNull("BufferModel test failed:No event generated.",listDataEvent);
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getBottom failed:",100,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getBottom failed:","*279*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getBottom failed:","*99*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}

	public void testGetPrev2_medium() throws BufferModelException, InterruptedException {
		//Prev in the middle of the data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*233*",""),100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",200,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*138*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*232*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetPrev3_medium() throws BufferModelException, InterruptedException {
		//Prev with the same call size as the remaining data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*188*",""),100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",100,bufferObjects.length);
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",200,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*0*",item.toString());
		item = bufferObjects[99];
		assertEquals("BufferModel.getTop failed:","*187*",item.toString());
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetPrev4_medium() throws BufferModelException, InterruptedException {
		//Prev with request less than remaining in the data elements
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(new DataRequest(IBufferModel.SEARCH_REQUEST,new BufferItem("*138*",""),100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!bufferModel.getDataRequest(DataRequestFactory.getPrev(100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",150,bufferObjects.length);
		Object item = bufferObjects[0];
		assertEquals("BufferModel.getTop failed:","*0*",item.toString());
		item = bufferObjects[49];
		assertEquals("BufferModel.getTop failed:","*137*",item.toString());
		assertTrue("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertFalse("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testGetFull_medium() throws BufferModelException, InterruptedException {
		//Get full buffer
		setDataSize(300);
		bufferModel.setDefaultCallSize(100);
		while(!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(100))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		Object[] bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",300,bufferObjects.length);
		while(!bufferModel.getDataRequest(DataRequestFactory.getNext(1))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		bufferObjects = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",300,bufferObjects.length);
		assertFalse("BufferModel.isAtTop failed.",bufferModel.isAtTop());
		assertTrue("BufferModel.isAtBottom failed.",bufferModel.isAtBottom());
	}
	
	public void testDuplicatesForward() throws InterruptedException {
		setDataSize(300);
		bufferModel.setDefaultCallSize(30);
		while(!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		Object[] items = bufferModel.getBufferFull();
		synchronized(testThread) {
			testThread.wait(500);
		}
		bufferModel.setDefaultCallSize(5);
		while(!(bufferModel.getDataRequest(DataRequestFactory.getSearch(new BufferItem("*100*",""))))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!(bufferModel.getDataRequest(DataRequestFactory.getSearch(new BufferItem("*10*",""))))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		items = bufferModel.getBufferFull();
		assertEquals("BufferModel duplicates forward test failed:",30,items.length);
		assertEquals("BufferModel duplicates forward test failed:",5,bufferModel.getSize());
		assertEquals("BufferModel duplicates forward test failed:","*10*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel duplicates forward test failed:","*10*",bufferModel.getElementAt(4).toString());
		while(!(bufferModel.getDataRequest(DataRequestFactory.getSearch(new BufferItem("*10*","15"))))){}
		//Gets *10*(10),*10*(11),*10*(12),*10*(13),*10*(14)
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertEquals("BufferModel duplicates forward test failed:","*10*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel duplicates forward test failed:","14",((IBufferItem)bufferModel.getElementAt(0)).getId());
		assertEquals("BufferModel duplicates forward test failed:","*10*",bufferModel.getElementAt(4).toString());
		assertEquals("BufferModel duplicates forward test failed:","18",((IBufferItem)bufferModel.getElementAt(4)).getId());
		bufferModel.setDefaultCallSize(10);
		while(!(bufferModel.getDataRequest(DataRequestFactory.getNext()))){}
		//Gets *10*(15),*10*(16),*10*(17),*10*(18),*10*(19),*100*(100),*101*(101),*102*(102),*103*(103),*104*(104)
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertEquals("BufferModel duplicates forward test failed:","*10*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel duplicates forward test failed:","*108*",bufferModel.getElementAt(9).toString());
	}
	
	public void testDuplicatesBackward() throws InterruptedException {
		setDataSize(300);
		bufferModel.setDefaultCallSize(30);
		while(!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		Object[] items = bufferModel.getBufferFull();
		synchronized(testThread) {
			testThread.wait(500);
		}
		while(!(bufferModel.getDataRequest(DataRequestFactory.getSearch(new BufferItem("*100*",""))))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		items = bufferModel.getBufferFull();
		assertEquals("BufferModel duplicates backward test failed:",42,items.length);
		assertEquals("BufferModel duplicates backward test failed:","*100*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel duplicates backward test failed:","*129*",bufferModel.getElementAt(29).toString());
//		bufferModel.setDefaultCallSize(5);
		while(!(bufferModel.getDataRequest(DataRequestFactory.getPrev(5)))){}
		//Gets *10*(19),*10*(18),*10*(17),*10*(16),*10*(15)
		synchronized(testThread) {
			Thread.yield();
			testThread.wait(1000);
		}
		items = bufferModel.getBufferFull();
		assertEquals("BufferModel duplicates backward test failed:",42,items.length);
		assertEquals("BufferModel duplicates backward test failed:",30,bufferModel.getSize());
		assertEquals("BufferModel duplicates backward test failed:","*10*",bufferModel.getElementAt(0).toString());
		assertEquals("BufferModel duplicates backward test failed:","15",((IBufferItem)bufferModel.getElementAt(0)).getId());
		assertEquals("BufferModel duplicates backward test failed:","*10*",bufferModel.getElementAt(4).toString());
		assertEquals("BufferModel duplicates backward test failed:","19",((IBufferItem)bufferModel.getElementAt(4)).getId());
	}

	//
	//Test the BuferModel timeout
	public void testTimeout() throws BufferModelException, InterruptedException {
		setDataSize(300);
		delayedCall = true;
		bufferModel.setDefaultCallSize(100);
		boolean goodCall;
		while(!(goodCall=bufferModel.getDataRequest(DataRequestFactory.getTop()))){}
		synchronized(testThread) {
			Thread.yield();
			testThread.wait(7000);
		}
		assertTrue("BufferModel.getTop return value invalid.",goodCall);
		Object[] items = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",0,items.length);
		items = bufferModel.getBufferFull();
		assertEquals("BufferModel.getTop failed:",0,items.length);
	}
	
	public void testMultipleCalls() throws BufferModelException, InterruptedException {
		setDataSize(300);
		delayedCall = true;
		bufferModel.setDefaultCallSize(100);
		boolean goodCall;
		while(!(goodCall= bufferModel.getDataRequest(DataRequestFactory.getTop()))){}
		boolean badCall = 	bufferModel.getDataRequest(DataRequestFactory.getBottom());
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertTrue("BufferModel.getTop return value invalid.",goodCall);
		assertFalse("BufferModel.getBottom return value invalid.",badCall);
	}
	
	public void testListPositionListener() throws InterruptedException {
		setDataSize(300);
		bufferModel.setDefaultCallSize(20);
		bufferModel.addListPositionListener(this);
		positionEvent = null;
		while (!(bufferModel.getDataRequest(DataRequestFactory.getTop()))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertNotNull("BufferModel positionUpdated event not generated.",positionEvent);

		positionEvent = null;
		while (!(bufferModel.getDataRequest(DataRequestFactory.getBottom()))){}
		synchronized(testThread) {
			testThread.wait(500);
		}
		assertNotNull("BufferModel positionUpdated event not generated.",positionEvent);
	}
	
	public void positionUpdated(BufferListDataEvent e) {
		this.positionEvent = e;
	}
	
	private void setDataSize(int size) {
		dataSize = size;
		dataItems = new IBufferItem[size];
		for (int i = 0; i < dataItems.length; i++) {
			if ((i>10) && (i<20)) {
				dataItems[i] = new BufferItem("*10*",Integer.toString(i));
			}
			else {
				dataItems[i] = new BufferItem("*"+Integer.toString(i)+"*",Integer.toString(i));
			}
		}
		Arrays.sort(dataItems);
	}
		
	private class RunGetData implements Runnable {
		private DataNeededEvent event;
		
		/**
		 * Constructor runGetData.
		 * @param event
		 */
		public RunGetData(DataNeededEvent event) {
			this.event = event;
		}

		/**
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			getData(event);
		}

	}
	
	public void getDataRequest(DataNeededEvent event) throws DataProviderException {
		Thread getDataThread = new Thread(new RunGetData(event));
		getDataThread.start();
	}
	
	private void getData(DataNeededEvent e) {
		Object[] items;
		List list = new LinkedList();
		if (delayedCall) {
//			System.out.println("Data provider sleeping...");
			try {
				Thread.sleep(6000);
			}
			catch (InterruptedException exception) {
			}
//			System.out.println("Data provider awake...");
		}
		int start;
		if (e.getStartingAt()==null) {
			if (e.getDirection()==IBufferModel.FORWARD_DIRECTION) {
				start = 0;
			}
			else {
				start = dataItems.length-1;
			}
		}
		else {
			start = Arrays.binarySearch(dataItems,e.getStartingAt());
			if (!e.isDataSearch()) {
				start += e.getDirection();
			}
			if (start < 0) {
				start = (-1*start)-1;
			}
			if (start==dataItems.length) {
				start = -1;
			}
		}
		if (start > -1) {
			for (int i = 0; i < e.getLength(); i++) {
				int index = start+(e.getDirection()*i);
				if ((index>-1) && (index<dataItems.length)) {
					list.add(dataItems[index]);
				}
				else {
					break;
				}
			}
		}
		items = list.toArray();
		bufferModel.setData(items,e.getLength());
	}
	
	
	public void setBufferModel(IBufferModel bufferModel) {
		this.bufferModel = bufferModel;
	}

	public void setFilter(Object filter) {
	}

	public void contentsChanged(ListDataEvent e) {
		this.positionEvent = (BufferListDataEvent)e;
		processData(e,0);
	}

	public void intervalAdded(ListDataEvent e) {
		this.positionEvent = (BufferListDataEvent)e;
		processData(e,1);
	}

	public void intervalRemoved(ListDataEvent e) {
		this.positionEvent = (BufferListDataEvent)e;
		processData(e,2);
	}
	
	private void processData(ListDataEvent e, int eventNum) {
		listDataEvent = e;
		eventFired = eventNum;
		synchronized(testThread) {
			testThread.notify();
		}
	}

}
