/*
 * Initially Created on Jun 5, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.cd.spectrum.itest;

import gov.va.med.hds.cd.spectrum.checklist.CheckList;
import gov.va.med.hds.cd.spectrum.checklist.ICheckEvent;
import gov.va.med.hds.cd.spectrum.checklist.ICheckable;
import gov.va.med.hds.cd.spectrum.checklist.ICheckableListener;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JList;

import junit.extensions.jfcunit.eventdata.KeyEventData;
import junit.extensions.jfcunit.eventdata.MouseEventData;

/**
 * JFC Tests for the CheckList component.
 */
public class CheckListTest extends AbstractJFCTestCase {
	private CheckList checkList;
	private DefaultListModel listModel;
	private Thread testThread;
	private static final int NO_MODIFIER = 0;
	private ICheckable[] items;

	public CheckListTest(String name) {
		super(name);
	}
	
	private class ListItem implements ICheckable {
		boolean checked;
		String text = "";

		public ListItem() {
			super();
		}
		
		public ListItem(String text) {
			super();
			this.text = text;
		}

		public boolean isChecked() {
			return checked;
		}

		public void setChecked(boolean checked) {
			this.checked = checked;
		}
		
		public String toString() {
			return text;
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(CheckListTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		listModel = new DefaultListModel();
		listModel.addElement(new ListItem("Line 1"));
		listModel.addElement(new ListItem("Entry 2"));
		listModel.addElement(new ListItem("String 3"));
		listModel.addElement(new ListItem("Item 4"));
		listModel.addElement(new ListItem("Thing 5"));
		listModel.addElement(new ListItem("Object 6"));
		listModel.addElement(new ListItem("Entity 7"));
		checkList = new CheckList();
		checkList.setModel(listModel);
		
		frame = new JFrame("CheckListTest");
		frame.getContentPane().add((JComponent)checkList);
		((JComponent)checkList).setPreferredSize(new Dimension(200,200));
		frame.pack();
		frame.show();
		flushAWT();
		testThread = Thread.currentThread();
	}
	
	public void testMouseClick() throws Exception {
		MouseEventData data = new MouseEventData(this, (JList)checkList, 1);
		data.setPosition(MouseEventData.NORTH_WEST);
		helper.enterClickAndLeave(data);
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("CheckList mouse click failed.",1,checkList.getSelectedIndices().length);
		assertEquals("CheckList mouse click failed.",0,checkList.getSelectedIndices()[0]);
		assertTrue("CheckList mouse click failed.",((ICheckable)listModel.getElementAt(0)).isChecked());
	}
	
	public void testMultipleSelectCheck() throws Exception {
		KeyEventData data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_DOWN, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_DOWN, KeyEvent.SHIFT_DOWN_MASK, 0);
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		
		assertEquals("CheckList multiple select failed:",3,checkList.getSelectedIndices().length);
		for (int i = 0; i < listModel.getSize(); i++) {
			assertFalse("CheckList multiple select failed: checked should be FALSE.",((ICheckable)listModel.getElementAt(i)).isChecked());
		}
		
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_SPACE, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		
		assertEquals("CheckList multiple select failed:",3,checkList.getSelectedIndices().length);
		for (int i = 0; i < 3; i++) {
			assertTrue("CheckList multiple select failed: checked should be TRUE.",((ICheckable)listModel.getElementAt(i)).isChecked());
		}
		for (int i = 3; i < listModel.getSize(); i++) {
			assertFalse("CheckList multiple select failed: checked should be FALSE.",((ICheckable)listModel.getElementAt(i)).isChecked());
		}
		
	}

	public void testCheckedEvent() throws Exception {
		checkList.getCheckableModel().addCheckableListener(new ICheckableListener() {
			public void checkChanged(ICheckEvent event) {
				items = event.getChangedItems();
			}
		});
			
		KeyEventData data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_DOWN, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_DOWN, KeyEvent.SHIFT_DOWN_MASK, 0);
		helper.sendKeyAction(data);
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_SPACE, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		
		assertEquals("CheckList event failed:",3,items.length);
		assertEquals("CheckList event failed:","Line 1",items[0].toString());
		assertEquals("CheckList event failed:","Entry 2",items[1].toString());
		assertEquals("CheckList event failed:","String 3",items[2].toString());
		
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_UP, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_SPACE, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_DOWN, KeyEvent.SHIFT_DOWN_MASK, 0);
		helper.sendKeyAction(data);
		helper.sendKeyAction(data);
		data = new KeyEventData(this, (JComponent)checkList, KeyEvent.VK_SPACE, NO_MODIFIER, 0);
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(100);
		}
		
		assertEquals("CheckList event failed:",1,items.length);
		assertEquals("CheckList event failed:","Line 1",items[0].toString());
	}
	
	public void testCtrlSelect() throws Exception {
		MouseEventData data = new MouseEventData(this, (JList)checkList, 1, MouseEvent.BUTTON1_MASK,false,10,MouseEventData.CUSTOM,new Point(25,100));
		helper.enterClickAndLeave(data);
		
		data = new MouseEventData(this, (JList)checkList, 1, MouseEvent.BUTTON1_MASK+MouseEvent.CTRL_MASK,false,10,MouseEventData.CUSTOM,new Point(25,5));
		helper.enterClickAndLeave(data);
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("CheckList mouse click failed.",2,checkList.getSelectedIndices().length);
		for (int i = 0;i<checkList.getModel().getSize(); i++) {
			assertFalse("CheckList mouse CTRL clicks failed: element is checked.",((ICheckable)checkList.getModel().getElementAt(i)).isChecked());
		}
	}
	
	public void testShiftSelect() throws Exception {
		MouseEventData data = new MouseEventData(this, (JList)checkList, 1, MouseEvent.BUTTON1_MASK,false,10,MouseEventData.CUSTOM,new Point(25,100));
		helper.enterClickAndLeave(data);
		
		data = new MouseEventData(this, (JList)checkList, 1, MouseEvent.BUTTON1_MASK+MouseEvent.SHIFT_MASK,false,10,MouseEventData.CUSTOM,new Point(25,5));
		helper.enterClickAndLeave(data);
		Thread.yield();
		synchronized (testThread) {
			testThread.wait(100);
		}
		assertEquals("CheckList mouse click failed.",5,checkList.getSelectedIndices().length);
		for (int i = 0;i<checkList.getModel().getSize(); i++) {
			assertFalse("CheckList mouse SHIFT clicks failed: element is checked.",((ICheckable)checkList.getModel().getElementAt(i)).isChecked());
		}
	}
	
}
