package gov.va.med.hds.cd.spectrum.itest;

import javax.swing.event.ListDataListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.spectrum.longlist.BufferModelException;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.DataRequest;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;
import gov.va.med.hds.cd.spectrum.longlist.IListPositionListener;
import gov.va.med.hds.cd.spectrum.test.DummyDataProvider;

/**
 * @author vhaislmonsos
 *
 * Dec 4, 2002 10:20:15 PM
 */
public class DummyDataProviderTest extends TestCase implements IBufferModel {
	private IDataNeededListener dataListener;
	private Object[] items;
	private int sizeExpected;

	/**
	 * Constructor for DummyDataProviderTest.
	 * @param arg0
	 */
	public DummyDataProviderTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(DummyDataProviderTest.class);
	}
	
	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		dataListener = new DummyDataProvider();
		dataListener.setBufferModel(this);
		items = null;
		sizeExpected = -1;
	}
	
	public void testGetTop() throws DataProviderException, InterruptedException {
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getTop failed: wrong number of items returned:",100,items.length);
		assertEquals("DummyDataProvider.getTop failed:","This is item #00000",items[0].toString());
		assertEquals("DummyDataProvider.getTop failed:","This is item #00099",items[99].toString());
	}

	public void testGetBottom() throws DataProviderException, InterruptedException {
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getTop failed: wrong number of items returned:",100,items.length);
		for (int i = 0; i< items.length; i++) {
			assertEquals("DummeyItemProvider.getTop failed:","This is item #" + Integer.toString(19999 - i),items[i].toString());
		}
	}

	public void testGetNext() throws DataProviderException, InterruptedException {
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		Object item = items[99];
		event = new DataNeededEvent(this,item,IBufferModel.FORWARD_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:","This is item #00100",items[0].toString());
		assertEquals("DummyDataProvider.getNext failed:","This is item #00199",items[99].toString());
		
		event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,100);
		dataListener.getDataRequest(event); //gets data 19999-19900
		Thread.yield();
		Thread.sleep(250);
		event = new DataNeededEvent(this,items[90],IBufferModel.FORWARD_DIRECTION,100); //items[90] s/b data 19909
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:",90,items.length);
		assertEquals("DummyDataProvider.getNext failed:","This is item #19910",items[0].toString());
		assertEquals("DummyDataProvider.getNext failed:","This is item #19999",items[89].toString());

		event = new DataNeededEvent(this,items[0],IBufferModel.FORWARD_DIRECTION,2); //items[0] s/b data 19910
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:",2,items.length);
		assertEquals("DummyDataProvider.getNext failed:","This is item #19912",items[1].toString());
	}
	
	public void testGetPrev() throws DataProviderException, InterruptedException {
		DataNeededEvent event = new DataNeededEvent(this,null,IBufferModel.REVERSE_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		Object item = items[99];	//s/b 19900
		event = new DataNeededEvent(this,item,IBufferModel.REVERSE_DIRECTION,100);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:",100,items.length);
		assertEquals("DummyDataProvider.getNext failed:","This is item #19899",items[0].toString());
		assertEquals("DummyDataProvider.getNext failed:","This is item #19800",items[99].toString());
		
		event = new DataNeededEvent(this,null,IBufferModel.FORWARD_DIRECTION,100);
		dataListener.getDataRequest(event); //gets data 0-99
		Thread.yield();
		Thread.sleep(250);
		event = new DataNeededEvent(this,items[90],IBufferModel.REVERSE_DIRECTION,100); //items[90] s/b data 00090
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:",90,items.length);
		assertEquals("DummyDataProvider.getNext failed:","This is item #00089",items[0].toString());
		assertEquals("DummyDataProvider.getNext failed:","This is item #00000",items[89].toString());

		event = new DataNeededEvent(this,items[0],IBufferModel.REVERSE_DIRECTION,2); //items[0] s/b data 00090
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getNext failed:",2,items.length);
		assertEquals("DummyDataProvider.getNext failed:","This is item #00087",items[1].toString());
	}
	
	public void testGetElement() throws DataProviderException, InterruptedException {
		Object item = new String("This is item #");
		DataNeededEvent event = new DataNeededEvent(this,item,IBufferModel.FORWARD_DIRECTION,2,true);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getElement failed:",2,items.length);
		assertEquals("DummyDataProvider.getElement failed:","This is item #00000",items[0].toString());
		assertEquals("DummyDataProvider.getElement failed:","This is item #00001",items[1].toString());
		
		item = new String("This is item #0101010101010");
		event = new DataNeededEvent(this,item,IBufferModel.FORWARD_DIRECTION,100,true);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getElement failed:",100,items.length);
		assertEquals("DummyDataProvider.getElement failed:","This is item #01011",items[0].toString());
		assertEquals("DummyDataProvider.getElement failed:","This is item #01110",items[99].toString());
		
		item = new String("This is item #19990");
		event = new DataNeededEvent(this,item,IBufferModel.FORWARD_DIRECTION,100,true);
		dataListener.getDataRequest(event);
		Thread.yield();
		Thread.sleep(250);
		assertEquals("DummyDataProvider.getElement failed:",10,items.length);
		assertEquals("DummyDataProvider.getElement failed:","This is item #19990",items[0].toString());
		assertEquals("DummyDataProvider.getElement failed:","This is item #19999",items[9].toString());
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getBottom()
	 */
	public boolean getBottom() throws BufferModelException {
		return true;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getBufferFull()
	 */
	public Object[] getBufferFull() {
		return null;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getInitSize()
	 */
	public int getDefaultCallSize() {
		return -1;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getMaxSize()
	 */
	public int getCapacity() {
		return -1;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtBottom()
	 */
	public boolean isAtBottom() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isAtTop()
	 */
	public boolean isAtTop() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isBoundryKnown()
	 */
	public boolean isBoundryKnown() {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setInitSize(int)
	 */
	public void setDefaultCallSize(int initSize) {
	}

	/**
	 * @see javax.swing.ListModel#addListDataListener(ListDataListener)
	 */
	public void addListDataListener(ListDataListener l) {
	}

	/**
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		return null;
	}

	/**
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return 0;
	}

	/**
	 * @see javax.swing.ListModel#removeListDataListener(ListDataListener)
	 */
	public void removeListDataListener(ListDataListener l) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addDataNeededListener(IDataNeededListener)
	 */
	public void addDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListDataListener(IListPositionListener)
	 */
	public void addListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeDataNeededListener(IDataNeededListener)
	 */
	public void removeDataNeededListener(IDataNeededListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListDataListener(IListPositionListener)
	 */
	public void removeListDataListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#reset()
	 */
	public boolean reset() {
		return true;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#getDataRequest(DataRequest)
	 */
	public boolean getDataRequest(DataRequest dataRequest) {
		return false;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#setData(Object[], int)
	 */
	public void setData(Object[] items, int sizeExpected) {
		this.items = items;
		this.sizeExpected = sizeExpected;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#addListPositionListener(IListPositionListener)
	 */
	public void addListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#removeListPositionListener(IListPositionListener)
	 */
	public void removeListPositionListener(IListPositionListener listener) {
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModel#isCallInProgress()
	 */
	public boolean isCallInProgress() {
		return false;
	}

}
