/*
 * Initially Created on Mar 25, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.cd.spectrum.itest;

import java.awt.event.KeyEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;

import junit.extensions.jfcunit.eventdata.KeyEventData;

import gov.va.med.hds.cd.spectrum.longlist.ILongListComponentListener;
import gov.va.med.hds.cd.spectrum.longlist.ILongListEvent;
import gov.va.med.hds.cd.spectrum.longlist.ISearchTextBox;
import gov.va.med.hds.cd.spectrum.longlist.SearchTextBox;

/**
 * Tests th SearchTextBox used in the LongListLookup component.
 */
public class SearchTextBoxTest extends AbstractJFCTestCase {
	private Thread testThread;
	private ISearchTextBox textBox;
	private ILongListComponentListener listener = new ILongListComponentListener() {
		public void actionPerformed(ILongListEvent e) {
			event = e;
		}
	};
	private ILongListEvent event;
	private static final int NO_MODIFIER = 0;

	/**
	 * Constructor for SearchTextBoxTest.
	 * @param name
	 */
	public SearchTextBoxTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(SearchTextBoxTest.class);
	}

	/*
	 * @see JFCTestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		testThread = Thread.currentThread();
		frame = new JFrame("SearchTextBoxTest");
		textBox = new SearchTextBox();
		frame.getContentPane().add((JComponent)textBox);
		frame.pack();
		frame.show();
		flushAWT();
	}

	public void testSpecialKeys() throws Exception {
		textBox.addLongListListener(listener);
		textBox.setText("testSpecialKeys");
		
		testNullEvent(KeyEvent.VK_RIGHT, "right arrow");
		testNullEvent(KeyEvent.VK_LEFT, "left arrow");
		testNullEvent(KeyEvent.VK_HOME, "home");
		testNullEvent(KeyEvent.VK_END, "end");
		
		testEvent(KeyEvent.VK_X, NO_MODIFIER, "x", ILongListEvent.SEARCH);
		testEvent(KeyEvent.VK_DOWN, NO_MODIFIER, "down arrow", ILongListEvent.DOWN);
		testEvent(KeyEvent.VK_UP, NO_MODIFIER, "up arrow", ILongListEvent.UP);
		testEvent(KeyEvent.VK_PAGE_UP, NO_MODIFIER, "page up", ILongListEvent.PAGE_UP);
		testEvent(KeyEvent.VK_PAGE_DOWN, NO_MODIFIER, "page down", ILongListEvent.PAGE_DOWN);
		testEvent(KeyEvent.VK_ENTER, NO_MODIFIER, "enter", ILongListEvent.SELECT);
		
		testEvent(KeyEvent.VK_HOME, KeyEvent.CTRL_MASK, "ctrl-home", ILongListEvent.HOME);
		testEvent(KeyEvent.VK_END, KeyEvent.CTRL_MASK, "ctrl-end", ILongListEvent.END);
		
		testNoEvent(KeyEvent.VK_TAB, KeyEvent.SHIFT_MASK, "shift-tab");
		testNoEvent(KeyEvent.VK_TAB, NO_MODIFIER, "tab");
		testNoEvent(KeyEvent.VK_A, KeyEvent.ALT_MASK, "alt-A");
		
	}

	private void testNoEvent(int keyID, int modifier, String keyName) throws Exception {
		KeyEventData data = new KeyEventData(this, (JComponent)textBox, keyID, modifier, 0);
		event = null;
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(550);
		}
		assertNull("SearchTextBox '" + keyName + "' key raised an unexpected event.",event);
	}
	
	private void testEvent(int keyID, int modifier, String keyName, int eventID) throws Exception {
		KeyEventData data = new KeyEventData(this, (JComponent)textBox, keyID, modifier, 0);
		event = null;
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(550);
		}
		assertNotNull("SearchTextBox '" + keyName + "' key did not raise an event.",event);
		assertEquals("SearchTextBox incorrect eventID for '" + keyName + "' key:",eventID,event.getID());
	}

	private void testNullEvent(int keyID, String keyName) throws Exception {
		KeyEventData data = new KeyEventData(this, (JComponent)textBox, keyID);
		event = null;
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(550);
		}
		assertNull("SearchTextBox '" + keyName + "' key raised an event.",event);
	}
	
	public void testAddingInvalidCharacter() throws Exception {
		KeyEventData data = new KeyEventData(this, (JComponent)textBox, KeyEvent.VK_CIRCUMFLEX);
		textBox.addLongListListener(listener);
		event = null;
		helper.sendKeyAction(data);
		synchronized (testThread) {
			testThread.wait(250);
		}
		assertEquals("SearchTextBox accepted a '^' in error.", 0, textBox.getText().length());
		assertNull("SearchTextBox threw an event for an invalid character.", event);
	}
	
	public void testEntryLength() throws Exception {
		textBox.setText("01234567890123456789012345678901234567890123456789012345678901234567890123456789");
		KeyEventData data = new KeyEventData(this, (JComponent)textBox, KeyEvent.VK_0);
		helper.sendKeyAction(data);
		helper.sendKeyAction(data);
		assertEquals("SearchTextBox accepted too many characters.", 80, textBox.getText().length());
	}

}
