package gov.va.med.hds.cd.spectrum.test;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.spectrum.date.CalendarTableModel;
import gov.va.med.hds.util.CalendarMath;

public class CalendarTableModelTest extends TestCase implements TableModelListener {

	private boolean tableChanged = false;

	public CalendarTableModelTest(String name) {
		super(name);
	}

	public void testCreate() {
		Calendar anchor = new GregorianCalendar(1975, Calendar.JULY, 23);
		
		CalendarTableModel model = new CalendarTableModel(anchor);
		assertEquals(anchor, model.getAnchorDate());
		
		assertEquals(7, model.getColumnCount());
		assertEquals(6, model.getRowCount());
		
		for (int col = 0; col < 7; ++col) {
			assertEquals(Calendar.class, model.getColumnClass(col));
		}
		
		for (int row = 0; row < 6; ++row) {
			for (int col = 0; col < 7; ++col) {
				assertFalse(model.isCellEditable(row, col));
				Calendar date = (Calendar) model.getValueAt(row, col);
				assertTrue(CalendarMath.isSameEra(anchor, date));
				assertTrue(CalendarMath.isSameYear(anchor, date));
				if (isPreviousMonth(row, col)) {
					assertEquals(anchor.get(Calendar.MONTH) - 1, date.get(Calendar.MONTH));
				} else if (isNextMonth(row, col)) {
					assertEquals(anchor.get(Calendar.MONTH) + 1, date.get(Calendar.MONTH));
				} else {
					assertEquals(anchor.get(Calendar.MONTH), date.get(Calendar.MONTH));
				}
				assertEquals(getDate(row, col), date.get(Calendar.DATE));
			}
		}
	}
	
	public void testTableModelEvents() {
		Calendar date = new GregorianCalendar(1975, Calendar.JULY, 23);
		CalendarTableModel model = new CalendarTableModel(date);
		model.addTableModelListener(this);
		assertFalse(tableChanged);
		date.add(Calendar.DATE, 5);
		model.setAnchorDate(date);
		assertFalse(tableChanged);
		date.add(Calendar.MONTH, 5);
		model.setAnchorDate(date);
		assertTrue(tableChanged);
	}
	
	private boolean isPreviousMonth(int row, int col) {
		return row == 0 && col < 2;
	}
	
	private boolean isNextMonth(int row, int col) {
		return row >= 5 || (row == 4 && col >= 5);
	}

	private int getDate(int row, int col) {
		int index = row * 7 + col;
		int julyStartIndex = 2;
		if (index < julyStartIndex) return 29 + index;
		else if (index >= julyStartIndex && index < julyStartIndex + 31) return index - julyStartIndex + 1;
		else return index - 32;
	}

	public void tableChanged(TableModelEvent e) {
		tableChanged = true;
	}
}
