package gov.va.med.hds.cd.spectrum.test;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.spectrum.date.DefaultDateSelectionModel;
import gov.va.med.hds.cd.spectrum.date.IDateSelectionModel;
import gov.va.med.hds.util.CalendarMath;

public class DateSelectionModelTest extends TestCase implements ChangeListener {

	private boolean dateChanged = false;

	public DateSelectionModelTest(String name) {
		super(name);
	}

	public void testCreate() {
		IDateSelectionModel model = new DefaultDateSelectionModel();
		Calendar today = Calendar.getInstance();
		assertTrue(CalendarMath.isSameDate(today, model.getSelectedDate()));
	
		Calendar date = new GregorianCalendar(1975, Calendar.JULY, 23);
		model = new DefaultDateSelectionModel(date);
		assertEquals(date, model.getSelectedDate());
	}

	public void testChangeEvents() {
		IDateSelectionModel model = new DefaultDateSelectionModel(new GregorianCalendar(1975, Calendar.JULY, 23));
		model.addChangeListener(this);
		
		Calendar today = Calendar.getInstance();
		assertFalse(dateChanged);
		model.setSelectedDate(today);
		assertTrue(dateChanged);
		assertEquals(today, model.getSelectedDate());
		
		dateChanged = false;
		assertFalse(dateChanged);
		model.setSelectedDate(today);
		assertFalse(dateChanged);
		assertEquals(today, model.getSelectedDate());
		
		dateChanged = false;
		assertFalse(dateChanged);
		model.setSelectedDate(null);
		assertTrue(dateChanged);
		assertNull(model.getSelectedDate());
	}

	public void stateChanged(ChangeEvent e) {
		dateChanged = true;
	}
}
