package gov.va.med.hds.cd.spectrum.test;

import gov.va.med.hds.cd.spectrum.longlist.BufferItem;
import gov.va.med.hds.cd.spectrum.longlist.DataNeededEvent;
import gov.va.med.hds.cd.spectrum.longlist.DataProviderException;
import gov.va.med.hds.cd.spectrum.longlist.IBufferItem;
import gov.va.med.hds.cd.spectrum.longlist.IBufferModel;
import gov.va.med.hds.cd.spectrum.longlist.IDataNeededListener;

/**
 * @author vhaislmonsos
 *
 * Dec 4, 2002 10:04:00 PM
 */
public class DummyDataProvider implements IDataNeededListener {
	private int size = 20000;
	private String text = "This is item #";
	private Object[] items = new Object[20000];
	private IBufferModel bufferModel;

	/**
	 * Constructor for DummyDataProvider.
	 */
	public DummyDataProvider() {
		super();
		for (int i=0; i<items.length; i++) {
			items[i] = new BufferItem((text + integerString(i)),Integer.toString(i));
		}
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModelDataProvider#getBottom(int)
	 */
	private Object[] getTop(int count) throws DataProviderException {
		Object[] result = new Object[count];
		for (int i=0; i < count; i++) {
			result[i] = items[i];
		}
		printNotify(result.length);
		return result;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModelDataProvider#getTop(int)
	 */
	private Object[] getBottom(int count) throws DataProviderException {
		Object[] result = new Object[count];
		for (int i=0;i<count;i++) {
			result[i] = items[size-1-i];
		}
		printNotify(result.length);
		return result;
	}

	/**
	 * Method integerString.
	 * @param i
	 * @return String
	 */
	private static String integerString(int i) {
		String intString = Integer.toString(i);
		int count = intString.length();
		while (count++ < 5) {
			intString = "0" + intString;
		}
		return intString;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModelDataProvider#getElement(Object)
	 */
	private Object[] getElement(Object item, int count) throws DataProviderException {
		Object[] result = null;
		int location = -1;
		
		int top = 0;
		int bottom = items.length-1;
		int middle = 0;
		while (top < bottom) {
			middle = (bottom + top) / 2;
			int compare = item.toString().compareTo(items[middle].toString());
			if (compare==0) {
				location = middle;
				top = bottom;
			}
			else if (compare<0) {
				bottom = middle;
			}
			else {
				top = middle+1;
			}
		}
		if (location == -1) {
			location = top;
		}

		int size = 0;
		if (location < items.length) {
			if ((location + count) > items.length) {
				size = items.length - location;
			}
			else {
				size = count;
			}
		}
		else {
			size = 0;
		}
		result = new Object[size];
		for (int i=0;i<size;i++) {
			result[i] = items[i+location];
		}
		printNotify(result.length);
		return result;
	}
	
	/**
	 * Method printNotify.
	 */
	private void printNotify(int length) {
//		System.out.println("RPC for :" + Integer.toString(length));
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IBufferModelDataProvider#getNext(Object, int, int)
	 */
	private Object[] getNext(Object startItem, int direction, int count) throws DataProviderException {
		int startLocation = Integer.parseInt(((IBufferItem)startItem).getId().toString());
		Object[] result = null;
		
		if (startLocation < 0) {
			throw new DataProviderException("Item not in list.");
		}
		
		int endLocation = 0;
		startLocation += direction;
		if ((startLocation < items.length) && (startLocation > -1)) {
			endLocation = startLocation + ((count-1)*direction);
		
			if (endLocation > items.length-1) {
				endLocation = items.length-1;
			}
			else if (endLocation < 0) {
				endLocation = 0;
			}
			
			count = Math.abs(startLocation - endLocation) + 1;
			result = new Object[count];
			for (int i=0;i<count;i++) {
				result[i] = items[startLocation + (i*direction)];
			}
			
			printNotify(count);
		}
		else {
			result = new Object[0];
		}
		return result;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IDataNeededListener#getData(DataNeededEvent)
	 */
	public void getDataRequest(DataNeededEvent event) throws DataProviderException {
		final DataNeededEvent e = event;
		Thread getDataThread = new Thread(new Runnable() {
			public void run() {
				try {
					getData(e);
				}
				catch (DataProviderException e) {
				}
			}
		});
		getDataThread.start();
	}
	
	private void getData(DataNeededEvent event) throws DataProviderException {
		Object[] items = null;
		if (event.isDataSearch()) {
			items = getElement(event.getStartingAt(),event.getLength());
			if (items.length<1) {
				items = getNext(event.getStartingAt(),IBufferModel.REVERSE_DIRECTION,event.getLength());
			}
			if (items.length<1) {
				event.setDataSearch(false);
				event.setStartingAt(null);
				getDataRequest(event);
			}
			else {
				bufferModel.setData(items, event.getLength());
			}
		}
		else {
			if (event.getStartingAt()==null) {
				//Get TOP or BOTTOM
				if (event.getDirection()>0) {
					//Get TOP
					items = getTop(event.getLength());
					bufferModel.setData(items, event.getLength());
				}
				else {
					//Get BOTTOM
					items = getBottom(event.getLength());
					bufferModel.setData(items, event.getLength());
				}
			}
			else {
				//Get middle part somewhere
				items = getNext(event.getStartingAt(),event.getDirection(),event.getLength());
				
				//Get Next
				if (event.getDirection()>0) {
					bufferModel.setData(items, event.getLength());
				}
				//Get Previous
				else {
					bufferModel.setData(items, event.getLength());
				}
			}
		}
	}

	/**
	 * Returns the bufferModel.
	 * @return BufferModel
	 */
	public IBufferModel getBufferModel() {
		return bufferModel;
	}

	/**
	 * Sets the bufferModel.
	 * @param bufferModel The bufferModel to set
	 */
	public void setBufferModel(IBufferModel bufferModel) {
		this.bufferModel = bufferModel;
		this.bufferModel.addDataNeededListener(this);
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IDataNeededListener#setFilter(String)
	 */
	public void setFilter(Object filter) {
	}

}
