/*
 * Initially Created on Mar 24, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.cd.spectrum.test;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.spectrum.longlist.ILongListComponentListener;
import gov.va.med.hds.cd.spectrum.longlist.ILongListEvent;
import gov.va.med.hds.cd.spectrum.longlist.IScrollBarModel;
import gov.va.med.hds.cd.spectrum.longlist.ScrollBarModel;

/**
 * Tests for the ScrollBarModel.
 */
public class ScrollBarModelTest extends TestCase {
	private IScrollBarModel model;
	private int eventCount;
	private ChangeListener changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			eventCount++;
		}
	};
	private ILongListComponentListener compListener = new ILongListComponentListener() {
		public void actionPerformed(ILongListEvent e) {
			eventCount++;
		}
	};

	/**
	 * Constructor for ScrollBarModelTest.
	 * @param arg0
	 */
	public ScrollBarModelTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(ScrollBarModelTest.class);
	}

	public void testGetMinimum() {
		assertEquals("ScrollBarModel.getMinimum failed:",0,model.getMinimum());
	}

	public void testSetMinimum() {
		model.setMinimum(2);
		assertEquals("ScrollBarModel.setMinimum failed:",0,model.getMinimum());
	}

	public void testGetMaximum() {
		assertEquals("ScrollBarModel.getMaximum failed:",100,model.getMaximum());
	}

	public void testSetMaximum() {
		model.setMaximum(1000);
		assertEquals("ScrollBarModel.setMaximum failed:",100,model.getMaximum());
	}

	public void testGetValue() {
		assertEquals("ScrollBarModel.getValue failed:",50,model.getValue());
	}

	public void testSetValue() {
		model.setValue(1000);
		assertEquals("ScrollBarModel.setValue failed:",50,model.getValue());
	}

	public void testSetValueIsAdjusting() {
		assertFalse("ScrollBarModel.getValueIsAdjusting failed:",model.getValueIsAdjusting());
		model.setValueIsAdjusting(true);
		assertTrue("ScrollBarModel.setValueIsAdjusting failed:",model.getValueIsAdjusting());
	}

	public void testGetExtent() {
		assertEquals("ScrollBarModel.getExtent failed:",10,model.getExtent());
	}

	public void testSetExtent() {
		model.setExtent(10000);
		assertEquals("ScrollBarModel.setExtent failed:",10,model.getExtent());
	}

	public void testSetRangeProperties() {
		model.setRangeProperties(234892, 23489, 29080, 23748929, true);
		assertEquals("ScrollBarModel.setRangeProperties failed:",100,model.getMaximum());
		assertEquals("ScrollBarModel.setRangeProperties failed:",0,model.getMinimum());
		assertEquals("ScrollBarModel.setRangeProperties failed:",50,model.getValue());
		assertTrue("ScrollBarModel.setRangeProperties failed:",model.getValueIsAdjusting());
	}

	public void testAddChangeListener() {
		eventCount = 0;
		model.addChangeListener(changeListener);
		model.setNewValue(30);
		assertEquals("ScrollBarModel.addChangeListener failed:",1,eventCount);
	}

	public void testRemoveChangeListener() {
		testAddChangeListener();
		eventCount = 0;
		model.removeChangeListener(changeListener);
		model.setNewValue(10);
		assertEquals("ScrollBarModel.removeChangeListener failed:",0,eventCount);
	}

	public void testAddLongListListener() {
		model.addLongListListener(compListener);
		eventCount = 0;
		model.setValue(99);
		assertEquals("ScrollBarModel.addLongListListener failed:",1,eventCount);
	}

	public void testRemoveLongListListener() {
		testAddLongListListener();
		model.removeLongListListener(compListener);
		eventCount = 0;
		model.setValue(28);
		assertEquals("ScrollBarModel.removeLongListListener failed:",0,eventCount);
	}

	public void testSetNewValue() {
		model.setNewValue(44);
		assertEquals("ScrollBarModel.setNewValue failed:",44,model.getValue());
	}

	protected void setUp() throws Exception {
		super.setUp();
		model = new ScrollBarModel();
	}

}
