package gov.va.med.hds.cd.spectrum;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.MouseInputAdapter;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalComboBoxUI;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 * @author vhaislschras
 *
 * @version 
 */

public class ButtonComboBox extends JComboBox
{
  private ButtonComboBoxUI ui;
  public ButtonComboBox()
  {
    initButtonComboBox();
  }
  
//  public ButtonComboBox(List items)
//  {
//    initButtonComboBox();
//    initItemList(items);
//  }
  
  public ButtonComboBox(Vector items) {
      super(items);
      initButtonComboBox();
  }
  
  private void initButtonComboBox()
  {
    MetalLookAndFeel.setCurrentTheme(new ButtonComboTheme());
    setUI((ui = new ButtonComboBoxUI()));
    setForeground(Color.white);
    setBackground(new Color(109, 114, 178));
    setEditable(false);
    setBorder(BorderFactory.createEmptyBorder());
  }

//  public void initItemList(List items)
//  {
//    DefaultComboBoxModel model = (DefaultComboBoxModel)getModel();
//    if (null != items)
//    {
//      for (Iterator iter = items.iterator(); iter.hasNext();)
//      {
//        Object item = iter.next();
//        // If this item is already in the list, this will ensure
//        // it only appears once
//        removeItem(item);
//        // Add it directly to the model so that it appends to the end.
//        model.addElement(item);
//      }
//    }
//  }

  public Border getActiveBorder()
  {
    return (null != ui ? ui.getActiveBorder() : null);
  }
  
  public Border getInactiveBorder()
  {
    return (null != ui ? ui.getInactiveBorder() : null);
  }
  
  public JButton getArrowButton()
  {
    return (null != ui ? ui.getArrowButton() : null);
  }
    
  class ButtonComboTheme extends DefaultMetalTheme
  {
    public ColorUIResource getControlInfo()
    {
      return getWhite();
    }
  }

  /* (non-Javadoc)
   * @see javax.swing.JComboBox#setEditable(boolean)
   */
  public void setEditable(boolean aFlag)
  {
    super.setEditable(false);
  }

  /* (non-Javadoc)
   * @see javax.swing.JComboBox#addItem(Object)
   */
  public void addItem(Object anObject)
  {
    // Ensure the object is not already in the list
    super.removeItem(anObject);
    // Always put it at the front of the list.
    ((DefaultComboBoxModel)getModel()).insertElementAt(anObject, 0);
    getModel().setSelectedItem(anObject);
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    Object selectedItem = getSelectedItem();
    return (null != selectedItem ? selectedItem.toString() : "No selection");
    //return super.toString();
  }

}

class ButtonComboBoxUI extends MetalComboBoxUI
{
  protected Border activeBorder;
  protected Border inactiveBorder;
  
  ButtonComboBoxUI()
  {
    super();
  }

  public Border getActiveBorder()  
  {
    return activeBorder;
  }
  
  public Border getInactiveBorder()
  {
    return inactiveBorder;
  }
  
  public JButton getArrowButton()
  {
    return arrowButton;
  }
  
  protected JButton createArrowButton()
  {
    JButton button = super.createArrowButton();
    activeBorder = button.getBorder();
    Insets in = button.getInsets();
    inactiveBorder = BorderFactory.createEmptyBorder(in.top, in.left, in.bottom, in.right);
    button.setBorder(inactiveBorder);
    button.addMouseListener(new Mousing());
    return button;
  }
  
  private class Mousing extends MouseInputAdapter
  {
    public void mouseEntered(MouseEvent e)
    {
      super.mouseEntered(e);
      if (e.getSource() instanceof JComponent)
      {
        JComponent source = (JComponent)e.getSource();
        source.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        source.setBorder(activeBorder);
      }
    }
    
    public void mouseExited(MouseEvent e)
    {
      super.mouseExited(e);
      if (e.getSource() instanceof JComponent)
      {
        JComponent source = (JComponent)e.getSource();
        Component parent = SwingUtilities.getRoot(source);
        parent.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        source.setBorder(inactiveBorder);
        
        // A kludge to notify the parent of mouse events.  The button
        // component consumes the event so it doesn't go up the hierarchy, but
        // I want it to.
        Component container = source.getParent();
        container.dispatchEvent(SwingUtilities.convertMouseEvent(source, e, container));
      }
    }
    
    public void mouseClicked(MouseEvent e)
    {
      super.mouseExited(e);
      if (e.getSource() instanceof JComponent)
      {
        JComponent source = (JComponent)e.getSource();
        Component parent = SwingUtilities.getRoot(source);
        parent.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      }
    }
    
  }
  
}
