
package gov.va.med.hds.cd.spectrum;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.Timer;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.PlainDocument;

import gov.va.med.hds.cd.spectrum.text.GenericDocumentFilter;

public class DoubleLookupListBox extends JPanel {
	
    private MyJTextField textField;
    private JList listBox;
    private ListModel listModel;
    private JScrollPane scrollPane;
    private String lastCompletedText = "";
    private boolean isPopup;
    private JMenu popupWindow;

    public DoubleLookupListBox(JRootPane rp, ListModel l) {
        this(rp, l, true);
    }

    public DoubleLookupListBox(JRootPane rp, ListModel l, boolean popup) {
        isPopup = popup;

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        textField = new MyJTextField();
        Dimension h = textField.getPreferredSize();
        h.width = 10000;
        textField.setMaximumSize(h);
        textField.addKeyListener(new KeypressListener());
        textField.addFocusListener(new FocusListener() {
                public void focusGained(FocusEvent e) {};
                public void focusLost(FocusEvent e) {
                    if (!e.isTemporary()) doneWithField();
                } });
        add(textField);

        listBox = new UnfocusableList(listModel = l);
        listBox.setPrototypeCellValue("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        listBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listBox.addListSelectionListener(new ListListener());
        scrollPane = new JScrollPane(listBox);

        if (isPopup) {
            System.err.println
                ("Warning! popup lookup list boxes aren't fully supported yet!");
            JMenuBar bar = new JMenuBar();
            popupWindow = new JMenu("Popup");
            popupWindow.add(scrollPane);
            bar.add(popupWindow);
            bar.setMaximumSize(new Dimension(1000,0));
            bar.setMinimumSize(new Dimension(0,0));
            bar.setPreferredSize(new Dimension(1,0));
            add(bar);
            KeyEventRouter router = new KeyEventRouter();
            router.addRouting(popupWindow, textField);
            rp.addKeyListener(router);
            listBox.addMouseListener(new MouseAdapter() {
                    public void mouseClicked(MouseEvent e) {
                        doneWithField();
                        fireActionPerformed();
                    } });
        } else {
            add(Box.createVerticalStrut(2));
            add(scrollPane);
            listBox.addMouseListener(new MouseAdapter() {
                    public void mouseClicked(MouseEvent e) {
                        maybeFireActionPerformed(e);
                    } });
        }

        add(Box.createVerticalStrut(2));
        //add(new JTextField("bogus"));
    }

    public String getCurrentItem() { return textField.getText(); }
    public void clearCurrentItem() { textField.setText(""); }

    private int clickCountToFire = 2;
    public void setClickCountToFire(int count) { clickCountToFire = count; }
    public int  getClickCountToFire() { return clickCountToFire; }
    private void maybeFireActionPerformed(MouseEvent e) {
        if (e.getClickCount() == clickCountToFire)
            fireActionPerformed();
    }

    public void autoCompleteCurrentText(boolean suppressAppend) {
        String text = textField.getText();
        if (text.equalsIgnoreCase(lastCompletedText)) return;

        int pos = findItem(text);
        scrollPane.getVerticalScrollBar().setValue(listBox.indexToLocation(pos).y);

        listBox.clearSelection();

        if (suppressAppend)
            return;

        String foundItem = listModel.getElementAt(pos).toString();
		if (foundItem.toUpperCase().startsWith(text.toUpperCase())) {
			textField.setText(foundItem);
			textField.select(text.length(), foundItem.length());
		}
        lastCompletedText = text;
    }

    private int findItem(String item) {
        int beg = 0;
        int end = listModel.getSize() - 1;
        int mid, cmp;
        while (beg < end) {
            mid = (beg + end) / 2;
            cmp = item.compareToIgnoreCase(listModel.getElementAt(mid).toString());
            if (cmp < 0) // item less than element at mid
                end = mid;
            else if (cmp > 0)
                beg = mid + 1;
            else
                return mid;
        }
        return beg;
    }

    public void doneWithField() {
        if (popupWindow != null) popupWindow.setPopupMenuVisible(false);
        String text = textField.getText();
        int len = text.length();
        textField.select(len, len);
        textField.setCaretPosition(len);
    }

    private void page(int direction) {
        JScrollBar sb = scrollPane.getVerticalScrollBar();
        int selectedIndex = listBox.getSelectedIndex();
        if (selectedIndex == -1) {
            sb.setValue(sb.getValue() +
                        direction*sb.getBlockIncrement(direction));
        } else {
            selectedIndex += (direction * sb.getBlockIncrement(direction) /
                              listBox.getFixedCellHeight());
            if (selectedIndex < 0) selectedIndex = 0;
            else if (selectedIndex >= listModel.getSize())
                selectedIndex = listModel.getSize() - 1;
            listBox.setSelectedIndex(selectedIndex);
            listBox.ensureIndexIsVisible(selectedIndex);
        }
    }

    private void arrow(int direction) {
        int selectedIndex = listBox.getSelectedIndex();
        if (selectedIndex < 0)
            selectedIndex = listBox.getFirstVisibleIndex();
        else {
            selectedIndex += direction;
            if (selectedIndex >= listModel.getSize())
                selectedIndex = listModel.getSize() - 1;
        }

        listBox.setSelectedIndex(selectedIndex);
        listBox.ensureIndexIsVisible(selectedIndex);
    }

    HashSet actionListeners = null;
    public void addActionListener(ActionListener l) {
        if (actionListeners == null) actionListeners = new HashSet();
        actionListeners.add(l);
    }
    public void removeActionListener(ActionListener l) {
        if (actionListeners != null)
            actionListeners.remove(l);
    }
    public void fireActionPerformed() {
        if (actionListeners == null || actionListeners.size() == 0) return;
        ActionEvent e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                        textField.getText());
        Iterator i = actionListeners.iterator();
        while (i.hasNext())
            ((ActionListener) i.next()).actionPerformed(e);
    }


    private interface KeyEventDestination {
        public void dispatchKeyEvent(KeyEvent e);
    }

    private class KeyEventRouter implements KeyListener {
        HashMap popupRouters = new HashMap();
        public void addRouting(JMenu menu, KeyEventDestination dest) {
            popupRouters.put(menu, dest);
        }
        public void keyTyped(KeyEvent e) { routeKeyEvent(e); }
        public void keyPressed(KeyEvent e) { routeKeyEvent(e); }
        public void keyReleased(KeyEvent e) { routeKeyEvent(e); }
        public void routeKeyEvent(KeyEvent e) {
            Iterator i = popupRouters.entrySet().iterator();
            Map.Entry entry;
            JMenu m;
            while (i.hasNext()) {
                entry = (Map.Entry) i.next();
                m = (JMenu) entry.getKey();
                if (m.isPopupMenuVisible()) {
                    KeyEventDestination dest =
                        (KeyEventDestination) entry.getValue();
                    dest.dispatchKeyEvent(e);
                    return;
                }
            }
        }
    }

    private class KeypressListener implements KeyListener, ActionListener {
        private Timer timer;
        private boolean suppressAppend = false;

        public KeypressListener() {
            timer = new Timer(Integer.MAX_VALUE, this);
            timer.setRepeats(false);
            timer.setInitialDelay(100); // one-tenth of a second
        }

        public void keyTyped(KeyEvent e) {}
        public void keyPressed(KeyEvent e) {
            if (popupWindow != null &&
                e.getKeyCode() != KeyEvent.VK_ENTER &&
                !popupWindow.isPopupMenuVisible()) {
                //popupWindow.setPopupMenuVisible(true);
                popupWindow.doClick(0);
            }
            switch (e.getKeyCode()) {
            case KeyEvent.VK_ENTER:
                fireActionPerformed();
            case KeyEvent.VK_TAB:
                timer.stop();
                doneWithField();
                break;

            case KeyEvent.VK_UP:
                timer.stop();
                arrow(-1);
                break;

            case KeyEvent.VK_DOWN:
                timer.stop();
                arrow(+1);
                break;

            case KeyEvent.VK_DELETE:
                timer.stop();
                break;

            case KeyEvent.VK_BACK_SPACE:
                if (textField.getSelectedText() != null)
                    textField.replaceSelection("");
                suppressAppend = true;
                timer.restart();
                break;

            case KeyEvent.VK_PAGE_UP:
                timer.stop();
                page(-1);
                break;

            case KeyEvent.VK_PAGE_DOWN:
                timer.stop();
                page(+1);
                break;
                
			case KeyEvent.VK_END:
				if (e.getModifiers() == KeyEvent.CTRL_MASK) {
					timer.stop();
					listBox.setSelectedIndex(listModel.getSize()-1);
					listBox.ensureIndexIsVisible(listModel.getSize()-1);
				}
				break;

			case KeyEvent.VK_HOME:
				if (e.getModifiers() == KeyEvent.CTRL_MASK) {
					timer.stop();
					listBox.setSelectedIndex(0);
					listBox.ensureIndexIsVisible(0);
				}
				break;

			case KeyEvent.VK_SHIFT:
				break;
				
            default:
                suppressAppend = false;
                timer.restart();
            }
        }
        public void keyReleased(KeyEvent e) {}

        public void actionPerformed(ActionEvent e) {
            autoCompleteCurrentText(suppressAppend);
        }
    }

    private class ListListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
        	if (listBox.getSelectedValue()!=null) {
            	textField.setText(listBox.getSelectedValue().toString());
        	}
            textField.selectAll();
        }
    }

    private class DropDownMenuBar extends JMenuBar {
        MyJTextField f;
        public DropDownMenuBar(MyJTextField f) { this.f=f; }
        protected void processKeyEvent(KeyEvent e) {
//            System.out.println("Menubar.processKeyEvent");
            f.dispatchKeyEvent(e);
        }
    }

    private class MyJTextField extends JTextField implements KeyEventDestination {
		public MyJTextField() {
			super();
			PlainDocument doc = new PlainDocument();
			doc.setDocumentFilter(new GenericDocumentFilter());
			setDocument(doc);
		}

        public void dispatchKeyEvent(KeyEvent e) {
            processKeyEvent(e);
            e.consume();
        }

        public void keyTyped(KeyEvent e) { dispatchKeyEvent(e); }
        public void keyPressed(KeyEvent e) { dispatchKeyEvent(e); }
        public void keyReleased(KeyEvent e) { dispatchKeyEvent(e); }
    }

    private class UnfocusableList extends JList {
        public UnfocusableList(ListModel l) { super(l); }
        public boolean isFocusable() { return false; }
    }


    public static final void main(String args[]) {
        if (args.length != 1) {
            System.out.println();
            System.out.println("Usage: java gov.va.med.hds.swingEx.DoubleLookupListBox listFileName");
            System.out.println();
            System.out.println("    where listFileName is the name of a text file containing the");
            System.out.println("    items to display in the list, one item per line");
            System.out.println();
            System.exit(0);
        }


        try {
            JFrame frame = new JFrame("Double Lookup List Box Test");
            frame.getContentPane().add
                (new DoubleLookupListBox(frame.getRootPane(),
                                         //new CachedLongList()),
                                         new FileListModel(args[0])),
                 BorderLayout.CENTER);
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

            frame.pack();
            frame.show();



        } catch (Exception ioe) {
            ioe.printStackTrace();
        }
    }

	public static class FileListModel implements ListModel {
		ArrayList list = new ArrayList();

		public FileListModel(String filename) throws IOException {
			this(new File(filename));
		}

		public FileListModel(File f) throws IOException {
			BufferedReader r = new BufferedReader(new FileReader(f));
			String item;
			while ((item = r.readLine()) != null)
				list.add(item);

			Collections.sort(list);
		}

		public int getSize() {
			return list.size();
		}
		public Object getElementAt(int index) {
			return list.get(index);
		}
		public void addListDataListener(ListDataListener l) {
		}
		public void removeListDataListener(ListDataListener l) {
		}
	}
	
	public Object[] getSelectedItems() {
		return listBox.getSelectedValues();
	}

	public void setPrototypeCellValue(Object value) {
		listBox.setPrototypeCellValue(value);
	}

	public int getFixedCellHeight() {
		return listBox.getFixedCellHeight();
	}

	public int getFixedCellWidth() {
		return listBox.getFixedCellWidth();
	}

	public void setFixedCellHeight(int height) {
		listBox.setFixedCellHeight(height);
	}

	public void setFixedCellWidth(int width) {
		listBox.setFixedCellWidth(width);
	}

}
