package gov.va.med.hds.cd.spectrum;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;
import java.awt.geom.Rectangle2D;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class GradientPanel extends JPanel {

	private Color backgroundColor = new Color(0, 0, 128);
	private Color foregroundColor = new Color(255, 255, 255);
	private JSeparator separator = new JSeparator();
	private JLabel titleLabel;
	private JLabel subTitleLabel;
	private JPanel titlePanel = new JPanel();
	private int direction = 0;
	public static int LEFT_TO_RIGHT = 0;
	public static int RIGHT_TO_LEFT = 1;

	/**
	 * Constructor for GradientPanel.
	 * @param layout
	 * @param isDoubleBuffered
	 */
	public GradientPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}

	/**
	 * Constructor for GradientPanel.
	 * @param layout
	 */
	public GradientPanel(LayoutManager layout) {
		super(layout);
	}

	/**
	 * Constructor for GradientPanel.
	 * @param isDoubleBuffered
	 */
	public GradientPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}

	/**
	 * Constructor for GradientPanel.
	 */
	public GradientPanel() {
		super();
	}

	/**
	 * Constructor for GradientPanel.
	 **/
	public GradientPanel(String title) {
		super();
		initTitle(title, "");
	}

	public GradientPanel(
		String title,
		String subTitle,
		Color foregroundColor,
		Color backgroundColor,
		int direction) {
		super();
		this.foregroundColor = foregroundColor;
		this.backgroundColor = backgroundColor;
		this.direction = direction;
		initTitle(title, subTitle);
	}

	/**
	 * Method initTitle.
	 * @param title
	 */
	private void initTitle(String title, String subTitle) {
		titleLabel = new JLabel(title);
		titleLabel.setForeground(foregroundColor);
		titleLabel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
		titleLabel.setFont(getFont().deriveFont(Font.BOLD));
		titleLabel.setOpaque(false);
		titlePanel.setLayout(new BoxLayout(titlePanel, BoxLayout.Y_AXIS));
		titlePanel.add(titleLabel);
		titlePanel.setOpaque(false);
		if (subTitle != "") {
			subTitleLabel = new JLabel(subTitle);
			subTitleLabel.setFont(getFont().deriveFont(Font.ITALIC));
			subTitleLabel.setBorder(
				BorderFactory.createEmptyBorder(5, 10, 0, 0));
			titlePanel.setBorder(
				BorderFactory.createEmptyBorder(5, 0, 5, 0));
			titlePanel.add(subTitleLabel);
			titlePanel.add(Box.createVerticalGlue());
		}
		setLayout(new BorderLayout());
		add(titlePanel, BorderLayout.WEST);
		add(separator, BorderLayout.SOUTH);
	}

	public void setTitle(String title) {
		titleLabel.setText(title);
	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		Color c1 =
			(direction == LEFT_TO_RIGHT
				? backgroundColor
				: new Color(
					backgroundColor.getRed(),
					backgroundColor.getGreen(),
					backgroundColor.getBlue(),
					0));
		Color c2 =
			(direction == LEFT_TO_RIGHT
				? new Color(
					backgroundColor.getRed(),
					backgroundColor.getGreen(),
					backgroundColor.getBlue(),
					0)
				: backgroundColor);

		GradientPaint gp = new GradientPaint(0, 0, c1, getWidth(), 0, c2);
		g2.setPaint(gp);
		Rectangle2D rect =
			new Rectangle2D.Double(0, 0, getWidth(), getHeight());
		g2.fill(rect);
		paintChildren(g);
	}
}
