package gov.va.med.hds.cd.spectrum;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import gov.va.med.hds.cd.mum.boot.plaf.desktop.DefaultDesktopTheme;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.DesktopLookAndFeel;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.IDesktopTheme;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.LookAndFeelUtilities;
import gov.va.med.hds.cd.mum.boot.plaf.desktop.WindowsDesktopTheme;
import gov.va.med.hds.cd.spectrum.theme.AquaTheme;
import gov.va.med.hds.cd.spectrum.theme.CharcoalTheme;
import gov.va.med.hds.cd.spectrum.theme.ContrastTheme;
import gov.va.med.hds.cd.spectrum.theme.EmeraldTheme;
import gov.va.med.hds.cd.spectrum.theme.RubyTheme;

public class LookAndFeelMenu extends JMenu {

	public static final String DEFAULT_MENU_TEXT = "Look&Feel";

	public LookAndFeelMenu() {
		super(DEFAULT_MENU_TEXT);
		UIManager.LookAndFeelInfo[] lnfs = UIManager.getInstalledLookAndFeels();
		createLookAndFeelMenuItems(lnfs);
		addSeparator();
		desktopThemeMenu = createDesktopThemeMenu();
		metalThemeMenu = createMetalThemeMenu();
		if (isLookAndFeelDesktop())
			add(desktopThemeMenu);
		else
			add(metalThemeMenu);
	}

	private JMenu createMetalThemeMenu() {
		createMetalThemes();

		metalThemeMenu = new JMenu("Themes");
		ActionListener al = new MetalThemeActionListener();
		ButtonGroup themeGroup = new ButtonGroup();
		for (Iterator i = metalThemeMap.keySet().iterator(); i.hasNext();) {
			String themeName = (String) i.next();
			JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem(themeName);
			menuItem.addActionListener(al);
			metalThemeMenu.add(menuItem);
			themeGroup.add(menuItem);
			if (isCurrentTheme(themeName)) {
				menuItem.setSelected(true);
			}
		}
		if (!areThemesSupported())
			metalThemeMenu.setEnabled(false);

		return metalThemeMenu;
	}

	private JMenu createDesktopThemeMenu() {
		createDesktopThemes();

		desktopThemeMenu = new JMenu("Themes");
		ActionListener al = new DesktopThemeActionListener();
		ButtonGroup themeGroup = new ButtonGroup();
		for (Iterator i = desktopThemeMap.keySet().iterator(); i.hasNext();) {
			String themeName = (String) i.next();
			JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem(themeName);
			menuItem.addActionListener(al);
			desktopThemeMenu.add(menuItem);
			themeGroup.add(menuItem);
			if (isCurrentTheme(themeName)) {
				menuItem.setSelected(true);
			}
		}
		if (!areThemesSupported())
			desktopThemeMenu.setEnabled(false);

		return desktopThemeMenu;
	}

	private boolean isCurrentTheme(String themeName) {
		if (isLookAndFeelMetal()
			&& currentMetalTheme != null
			&& (themeName.equals(currentMetalTheme.getName())
				|| (themeName.equals("Default")
					&& currentMetalTheme.getClass().equals(DefaultMetalTheme.class)))) {
			return true;
		} else if (
			isLookAndFeelDesktop()
				&& currentDesktopTheme != null
				&& (themeName.equals(currentDesktopTheme.getName())
					|| (themeName.equals("Default")
						&& currentDesktopTheme.getClass().equals(DefaultDesktopTheme.class)))) {
			return true;
		} else {
			return false;
		}
	}

	private boolean areThemesSupported() {
		return isLookAndFeelMetal() || isLookAndFeelDesktop();
	}

	private boolean isLookAndFeelDesktop() {
		return UIManager.getLookAndFeel().getClass().getName().equals(
			DesktopLookAndFeel.class.getName());
	}

	private boolean isLookAndFeelMetal() {
		return UIManager.getLookAndFeel().getClass().getName().equals(
			UIManager.getCrossPlatformLookAndFeelClassName());
	}

	private void createMetalThemes() {
		metalThemeMap.put("Default", currentMetalTheme = new DefaultMetalTheme());
		MetalTheme theme = new AquaTheme();
		metalThemeMap.put(theme.getName(), theme);
		theme = new CharcoalTheme();
		metalThemeMap.put(theme.getName(), theme);
		theme = new ContrastTheme();
		metalThemeMap.put(theme.getName(), theme);
		theme = new EmeraldTheme();
		metalThemeMap.put(theme.getName(), theme);
		theme = new RubyTheme();
		metalThemeMap.put(theme.getName(), theme);
	}

	private void createDesktopThemes() {
		desktopThemeMap.put("Default", currentDesktopTheme = new DefaultDesktopTheme());
		IDesktopTheme theme = new WindowsDesktopTheme();
		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DefaultDesktopSmallTheme();
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DefaultDesktopLargeTheme();
		//		desktopThemeMap.put(theme.getName(), theme);
		//		IDesktopTheme theme = new DesktopMetalThemeAdapter(new DefaultMetalTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new AquaTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new AquaTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new CharcoalTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new ContrastTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new EmeraldTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
		//		theme = new DesktopMetalThemeAdapter(new RubyTheme());
		//		desktopThemeMap.put(theme.getName(), theme);
	}

	private void createLookAndFeelMenuItems(UIManager.LookAndFeelInfo[] lnfs) {
		ActionListener al = new LookAndFeelActionListener();
		ButtonGroup group = new ButtonGroup();
		for (int i = 0; i < lnfs.length; ++i) {
			JRadioButtonMenuItem item = new JRadioButtonMenuItem(lnfs[i].getName());
			if (lnfs[i].getName().equals(UIManager.getLookAndFeel().getName())) {
				item.setSelected(true);
			}
			item.addActionListener(al);
			this.add(item);
			group.add(item);
		}
	}

	private Map metalThemeMap = new TreeMap();
	private JMenu metalThemeMenu;
	private MetalTheme currentMetalTheme;

	private Map desktopThemeMap = new TreeMap();
	private JMenu desktopThemeMenu;
	private IDesktopTheme currentDesktopTheme;

	private class LookAndFeelActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			updateLookAndFeel(e);
			updateThemeMenu();
		}

		public void updateLookAndFeel(ActionEvent e) {
			try {
				UIManager.LookAndFeelInfo[] lnfs = UIManager.getInstalledLookAndFeels();
				JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
				for (int i = 0; i < lnfs.length; ++i) {
					if (item.getText().equals(lnfs[i].getName())) {
						UIManager.setLookAndFeel(lnfs[i].getClassName());
						LookAndFeelUtilities.updateFrames();
						break;
					}
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		private void updateThemeMenu() {
			desktopThemeMenu.setEnabled(isLookAndFeelDesktop());
			metalThemeMenu.setEnabled(isLookAndFeelMetal());
			if (isLookAndFeelDesktop()) {
				if (metalThemeMenu.getParent() != null)
					remove(metalThemeMenu);
				add(desktopThemeMenu);
			} else if (isLookAndFeelMetal()) {
				if (desktopThemeMenu.getParent() != null)
					remove(desktopThemeMenu);
				add(metalThemeMenu);
			}
			revalidate();
			repaint();
		}
	}

	private class MetalThemeActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			try {
				JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
				MetalTheme theme = (MetalTheme) metalThemeMap.get(item.getText());
				currentMetalTheme = theme;
				MetalLookAndFeel.setCurrentTheme(theme);
				UIManager.setLookAndFeel(UIManager.getLookAndFeel()); // silly, but Swing wants it
				LookAndFeelUtilities.updateFrames();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

	}

	private class DesktopThemeActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			try {
				JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
				IDesktopTheme theme = (IDesktopTheme) desktopThemeMap.get(item.getText());
				currentDesktopTheme = theme;
				DesktopLookAndFeel.setCurrentTheme(theme);
				UIManager.setLookAndFeel(UIManager.getLookAndFeel()); // silly, but Swing wants it
				LookAndFeelUtilities.updateFrames();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

	}
}
