package gov.va.med.hds.cd.spectrum;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
public class OutlineRenderer extends DefaultTreeCellRenderer {
	private ImageIcon leafIcon;
	private ImageIcon folderClosedIcon;
	private ImageIcon folderOpenIcon;
	private Font plainFont = null;
	private Font boldFont = null;
	private boolean useBold = true;
	public static final int USE_DEFAULT_ICONS = 0;
	public static final int USE_OUTLINE_ICONS = 1;
	public static final int SUPPRESS_ICONS = 2;
	
	public OutlineRenderer() {
	}
	
	public OutlineRenderer(int iconStyle) {
		
		switch (iconStyle) {
			case USE_DEFAULT_ICONS : 
				;
				break;
			case USE_OUTLINE_ICONS : {
				leafIcon = new ImageIcon(OutlineRenderer.class.getResource("leaficon.gif"));
				folderClosedIcon =
					new ImageIcon(OutlineRenderer.class.getResource("folderClosed.gif"));
				folderOpenIcon =
					new ImageIcon(OutlineRenderer.class.getResource("folderOpen.gif"));
				setLeafIcon(leafIcon);
				setClosedIcon(folderClosedIcon);
				setOpenIcon(folderOpenIcon);
			} break;
			case SUPPRESS_ICONS : {
				setLeafIcon(null);
				setClosedIcon(null);
				setOpenIcon(null);
			} break;
		}
	}

	public OutlineRenderer(
		ImageIcon leafIcon,
		ImageIcon folerClosedIcon,
		ImageIcon folderOpenIcon) {
		
		setLeafIcon(leafIcon);
		setClosedIcon(folderClosedIcon);
		setOpenIcon(folderOpenIcon);
	}

	private Font getFont(boolean bold) {

		if (!isUseBold())
			return plainFont;

		if (plainFont == null) {
			Font defaultFont = getFont();
			if (defaultFont == null)
				return null;
			plainFont = defaultFont.deriveFont(Font.PLAIN);
			boldFont = defaultFont.deriveFont(Font.BOLD);
		}

		return (bold ? boldFont : plainFont);

	}

	public Component getTreeCellRendererComponent(
		JTree outlineTree,
		Object value,
		boolean sel,
		boolean expanded,
		boolean leaf,
		int row,
		boolean hasFocus) {

		super.getTreeCellRendererComponent(
			outlineTree,
			value,
			sel,
			expanded,
			leaf,
			row,
			hasFocus);
		Font f = getFont(!leaf);
		setFont(f);
		UIManager.put("ToolTip.font", f);

		setToolTipText(getText());
		setBackground(outlineTree.getBackground());
		return this;
	}

	public String getToolTipText(MouseEvent event) {
		String text = getText();
		if (text == null || text.length() == 0)
			return null;

		Dimension currSize = getSize();
		Dimension minSize = getMinimumSize();
		if (minSize.width > currSize.width) {
			return text;
		} else
			return null;
	}

	public Rectangle getTextRectangle(Graphics g) {

		String text = getText();
		Icon icon = (isEnabled()) ? getIcon() : getDisabledIcon();

		if ((icon == null) && (text == null)) {
			return null;
		}

		Rectangle paintIconR = new Rectangle();
		Rectangle paintTextR = new Rectangle();
		Rectangle paintViewR = new Rectangle();
		Insets paintViewInsets = new Insets(0, 0, 0, 0);

		paintViewInsets = getInsets(paintViewInsets);
		paintViewR.x = paintViewInsets.left;
		paintViewR.y = paintViewInsets.top;
		paintViewR.width =
			getWidth() - (paintViewInsets.left + paintViewInsets.right);
		paintViewR.height =
			getHeight() - (paintViewInsets.top + paintViewInsets.bottom);

		if (g == null) {
			return null;
		}
		String clippedText =
			SwingUtilities.layoutCompoundLabel(
				(JComponent) this,
				g.getFontMetrics(),
				text,
				icon,
				getVerticalAlignment(),
				getHorizontalAlignment(),
				getVerticalTextPosition(),
				getHorizontalTextPosition(),
				paintViewR,
				paintIconR,
				paintTextR,
				getIconTextGap());

		return paintTextR;
	}
	private void debug(String message) {
		System.out.println(message);
	}

	/**
	 * Returns the useBold.
	 * @return boolean
	 */
	public boolean isUseBold() {
		return useBold;
	}

	/**
	 * Sets the useBold.
	 * @param useBold The useBold to set
	 */
	public void setUseBold(boolean useBold) {
		this.useBold = useBold;
	}

}
