package gov.va.med.hds.cd.spectrum;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * The <code>OutlineTree</code> extends the standard <code>JTree</code> to pop
 * up a tool tip when the viewport is too small to display a node in the tree,
 * and the mouse hovers over a particular node.
 * 
 * The current incarnation of </code>OutlineTree<code> also supports drag and
 * drop.  TODO: move drag and drop support into subclass of OutlineTree?
 */
public class OutlineTree extends JTree {

	private static final int VERTICAL_TOOLTIP_MARGIN = 7;
	private static final int HORIZONTAL_TOOLTIP_MARGIN = 4;

	/** Variables needed for DnD */
	private DragSource dragSource = null;

	/**
	 * Constructor for OutlineTree.
	 */
	public OutlineTree() {
		super();
	}

	/**
	 * Constructor for OutlineTree.
	 * @param value
	 */
	public OutlineTree(Object[] value) {
		super(value);
	}

	/**
	 * Constructor for OutlineTree.
	 * @param value
	 */
	public OutlineTree(Vector value) {
		super(value);
	}

	/**
	 * Constructor for OutlineTree.
	 * @param value
	 */
	public OutlineTree(Hashtable value) {
		super(value);
	}

	/**
	 * Constructor for OutlineTree.
	 * @param root
	 */
	public OutlineTree(TreeNode root) {
		super(root);
	}

	/**
	 * Constructor for OutlineTree.
	 * @param root
	 * @param asksAllowsChildren
	 */
	public OutlineTree(TreeNode root, boolean asksAllowsChildren) {
		super(root, asksAllowsChildren);
	}

	/**
	 * Constructor for OutlineTree.
	 * @param newModel
	 */
	public OutlineTree(TreeModel newModel) {
		super(newModel);
	}

	public void initializeDnD(
		TreeSelectionListener tsl,
		DragGestureListener dgl,
		DropTargetListener dtl,
		DragSourceListener dsl) {

		addTreeSelectionListener(tsl);

		dragSource = DragSource.getDefaultDragSource();
		dragSource.addDragSourceListener(dsl);
		
		DragGestureRecognizer dgr =
			dragSource.createDefaultDragGestureRecognizer(
				this,
				DnDConstants.ACTION_COPY_OR_MOVE,
				dgl);

		/* 
		 * mask right click from dragGestures (avoid confusion with popup)
		 */
		dgr.setSourceActions(dgr.getSourceActions() & ~InputEvent.BUTTON3_MASK);

		/* First argument:  Component to associate the target with
		 * Second argument: DropTargetListener 
		*/
		DropTarget dropTarget = new DropTarget(this, dtl);
	}

	public Point getToolTipLocation(MouseEvent event) {
		if (event != null) {
			Point p = event.getPoint();
			int selRow = getRowForLocation(p.x, p.y);
			TreeCellRenderer r = getCellRenderer();

			if (selRow != -1 && r != null) {
				TreePath path = getPathForRow(selRow);
				Object lastPath = path.getLastPathComponent();
				Component rComponent =
					r.getTreeCellRendererComponent(
						this,
						lastPath,
						isRowSelected(selRow),
						isExpanded(selRow),
						getModel().isLeaf(lastPath),
						selRow,
						true);

				if (rComponent instanceof OutlineRenderer) {
					Rectangle pathBounds = getPathBounds(path);
					Rectangle textBounds =
						((OutlineRenderer) rComponent).getTextRectangle(
							getGraphics());
					return new Point(
						pathBounds.x + textBounds.x - HORIZONTAL_TOOLTIP_MARGIN,
						pathBounds.y + textBounds.y + VERTICAL_TOOLTIP_MARGIN);
				}
			}
		}
		return null;
	}

	public void setLineStyle(String lineStyle) {
		putClientProperty("JTree.lineStyle", lineStyle);
	}

	/**
	 * Returns the dragSource.
	 * @return DragSource
	 */
	public DragSource getDragSource() {
		return dragSource;
	}

}
