

package gov.va.med.hds.cd.spectrum;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListDataListener;


//import com.wildcrest.jeditlist.JEditList;


public class SlushBucket extends JPanel {

    DoubleLookupListBox longList;
    JList selectedItems;
    DefaultListModel selectedItemModel;
    JButton addItem, removeItem, addAllItems, removeAllItems;


    public SlushBucket(ListModel l) {
        setLayout(new BorderLayout());

        Component buttonPanel = buildButtonPanel();
        makeLongList(l);
        makeSelectedList();

        add(longList, BorderLayout.WEST);
        add(buttonPanel, BorderLayout.CENTER);
        add(new JScrollPane(selectedItems), BorderLayout.EAST);
        setQuick(true);
    }

    public void setQuick(boolean quick) {
        clickCountToRemove = (quick ? 1 : 2);
        longList.setClickCountToFire(clickCountToRemove);
    }

    private int clickCountToRemove = 2;

    private void makeLongList(ListModel l) {
        longList = new DoubleLookupListBox(null, l, false);
        ActionListener listener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    addSelectedItem(); }};

        longList.addActionListener(listener);
        addItem.addActionListener(listener);
    }

    private int ignoreIndex = -1;
    private boolean watchDrag = false;

    private void makeSelectedList() {
        selectedItemModel = new DefaultListModel();
        selectedItems = new DragReorderList(selectedItemModel);
        selectedItems.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        selectedItems.setPrototypeCellValue("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        selectedItems.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    maybeFireItemRemoved(e);
                } });
    }
    private void maybeFireItemRemoved(MouseEvent e) {
        if (e.getClickCount() == clickCountToRemove)
            removeSelectedItem();
    }



    private Component buildButtonPanel() {

        Container oPanel = new JPanel();
        GridBagLayout oLayout = new GridBagLayout();
        oPanel.setLayout(oLayout);
        GridBagConstraints oConst;
        JLabel topFiller = new JLabel();
        topFiller.setText(" ");
        oPanel.add(topFiller);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =0;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oLayout.setConstraints(topFiller, oConst);

        addItem = new JButton(">");
        oPanel.add(addItem);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =1;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oConst.insets.top =5;
        oConst.insets.left =5;
        oConst.insets.right =5;
        oConst.insets.bottom =5;
        oLayout.setConstraints(addItem, oConst);

        addAllItems = new JButton(">>");
        addAllItems.setEnabled(false);
        oPanel.add(addAllItems);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =2;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oConst.insets.left =5;
        oConst.insets.right =5;
        oConst.insets.bottom =5;
        oLayout.setConstraints(addAllItems, oConst);

        removeItem = new JButton("<");
        oPanel.add(removeItem);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =3;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oConst.insets.left =5;
        oConst.insets.right =5;
        oConst.insets.bottom =5;
        oLayout.setConstraints(removeItem, oConst);

        removeAllItems = new JButton("<<");
        oPanel.add(removeAllItems);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =4;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oConst.insets.left =5;
        oConst.insets.right =5;
        oConst.insets.bottom =5;
        oLayout.setConstraints(removeAllItems, oConst);

        JLabel bottomFiller = new JLabel();
        bottomFiller.setText(" ");
        oPanel.add(bottomFiller);
        oConst = new GridBagConstraints();
        oConst.gridx =0;
        oConst.gridy =5;
        oConst.weightx =1.0;
        oConst.weighty =1.0;
        oConst.fill =1;
        oLayout.setConstraints(bottomFiller, oConst);


        removeItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeSelectedItem(); }});

        removeAllItems.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeAllItems(); }});

        return oPanel;
    }

    private void addSelectedItem() {
        String selectedItem = longList.getCurrentItem();
        if (selectedItem == null || selectedItem.length() == 0) return;

        if (!selectedItemModel.contains(selectedItem)) {
            selectedItemModel.addElement(selectedItem);
            longList.clearCurrentItem();
        }
    }
    private void removeSelectedItem() {
        String selectedItem = (String) selectedItems.getSelectedValue();
        if (selectedItem == null || selectedItem.length() == 0) return;

        selectedItemModel.removeElement(selectedItem);
        ignoreIndex = selectedItems.getSelectedIndex();
    }
    private void removeAllItems() {
        selectedItemModel.removeAllElements();
    }

    public class DragReorderList extends JList {
        DefaultListModel listModel;
        int    from = -1;
        public DragReorderList(DefaultListModel m) {
            super(m);
            listModel = m;
            addMouseListener(new MouseAdapter() {
                    public void mousePressed(MouseEvent m) {
                        from = getSelectedIndex();
                    } });
            addMouseMotionListener(new MouseMotionAdapter() {
                    public void mouseDragged(MouseEvent m) {
                        int to  = getSelectedIndex();
                        if (to == from) return;

                        Object fromObj = listModel.getElementAt(from);
                        Object toObj   = listModel.getElementAt(to);
                        listModel.setElementAt(fromObj, to);
                        listModel.setElementAt(toObj, from);

                        from = to;
                    }
                });
        }
    }


    public static final void main(String args[]) {
        if (args.length != 1) {
            System.out.println("Usage: java gov.va.med.hds.swingEx.SlushBucket listFileName");
            System.out.println("    where listFileName is the name of a text file containing the");
            System.out.println("    items to display in the list, one item per line");
            System.exit(0);
        }


        try {
            JFrame frame = new JFrame("Slush Bucket Test");
            frame.getContentPane().add
                (new SlushBucket(new FileListModel(args[0])),
                 BorderLayout.CENTER);
            frame.pack();
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            frame.show();

        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    public static class FileListModel implements ListModel {
        ArrayList list = new ArrayList();

        public FileListModel(String filename) throws IOException {
            this(new File(filename));
        }

        public FileListModel(File f) throws IOException {
            BufferedReader r = new BufferedReader(new FileReader(f));
            String item;
            while ((item = r.readLine()) != null)
                list.add(item);

            Collections.sort(list);
        }

        public int getSize() { return list.size(); }
        public Object getElementAt(int index) { return list.get(index); }
        public void addListDataListener(ListDataListener l) {}
        public void removeListDataListener(ListDataListener l) {}
    }

}
