package gov.va.med.hds.cd.spectrum.bucket;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.EventListenerList;

public abstract class AbstractSlushBucket extends JPanel {
	public static final int SINGLE_SELECTION_MODE = 0;
	public static final int MULTIPLE_SELECTION_MODE = 1;
	public static final int SELECT_ACTION = 0;
	public static final int SELECT_ALL_ACTION = 1;
	public static final int REMOVE_ACTION = 2;
	public static final int REMOVE_ALL_ACTION = 3;

	
	private JButton selectAllButton = new JButton();
	private JButton removeAllButton = new JButton();
	private JPanel outBucket = new JPanel();
	private JPanel inBucket = new JPanel();
	private JButton selectButton = new JButton();
	private JButton removeButton = new JButton();
	private EventListenerList actionEventListeners = new EventListenerList();
	private JLabel inBucketLabel = new JLabel("In Bucket Label");
	private JLabel outBucketLabel = new JLabel("Out Bucket Label");
	private IConverter converter;
	private final Dimension dimension = new Dimension(290,200);

	public AbstractSlushBucket() {
		init();
	}
	private void init() {
		setAccessibleItems();
		outBucket.setPreferredSize(dimension);
		inBucket.setPreferredSize(dimension);
		this.setLayout(new GridBagLayout());
		this.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		inBucket.setLayout(new BorderLayout());
		outBucket.setLayout(new BorderLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(2,2,2,2);
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		this.add(inBucketLabel,gbc);
		gbc.gridx = 2;
		this.add(outBucketLabel,gbc);
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridheight = 7;
		gbc.weightx = 0.5;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		this.add(inBucket,gbc);
		gbc.gridx = 2;
		this.add(outBucket,gbc);
		gbc.gridy = 1;
		gbc.gridx = 1;
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		this.add(Box.createVerticalStrut(20),gbc);
		gbc.gridy = 2;
		this.add(selectButton,gbc);
		gbc.gridy = 3;
		this.add(removeButton,gbc);
		gbc.gridy = 4;
		this.add(Box.createVerticalStrut(10),gbc);
		gbc.gridy = 5;
		this.add(selectAllButton,gbc);
		gbc.gridy = 6;
		this.add(removeAllButton,gbc);
		gbc.gridy = 7;
		this.add(Box.createVerticalStrut(20),gbc);

		selectAllButton.setText(">>");
		selectAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireSelectAllLocal();
			}
		});

		removeAllButton.setText("<<");
		removeAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireRemoveAllLocal();
			}
		});
		removeAllButton.setEnabled(false);
		
		selectButton.setText(">");
		selectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireSelectLocal();
			}
		});
		removeButton.setText("<");
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireRemoveLocal();
			}
		});
		removeButton.setEnabled(false);
		
		Dimension minSize = selectAllButton.getMinimumSize();
		Dimension maxSize = selectAllButton.getMaximumSize();
		Dimension prefSize = selectAllButton.getPreferredSize();
		
		selectAllButton.setPreferredSize(prefSize);
		selectAllButton.setMinimumSize(minSize);
		selectAllButton.setMaximumSize(maxSize);
		removeAllButton.setPreferredSize(prefSize);
		removeAllButton.setMinimumSize(minSize);
		removeAllButton.setMaximumSize(maxSize);
		selectButton.setPreferredSize(prefSize);
		selectButton.setMinimumSize(minSize);
		selectButton.setMaximumSize(maxSize);
		removeButton.setPreferredSize(prefSize);
		removeButton.setMinimumSize(minSize);
		removeButton.setMaximumSize(maxSize);
	}
	
	private void setAccessibleItems() {
		selectButton.getAccessibleContext().setAccessibleName("Select");
		selectAllButton.getAccessibleContext().setAccessibleName("Select all");
		removeButton.getAccessibleContext().setAccessibleName("Remove");
		removeAllButton.getAccessibleContext().setAccessibleName("Remove all");
		
		selectButton.setToolTipText(selectButton.getAccessibleContext().getAccessibleName());
		selectAllButton.setToolTipText(selectAllButton.getAccessibleContext().getAccessibleName());
		removeButton.setToolTipText(removeButton.getAccessibleContext().getAccessibleName());
		removeAllButton.setToolTipText(removeAllButton.getAccessibleContext().getAccessibleName());
	}

	public void fireAction(int actionId) {
		if ((actionEventListeners != null) && (actionEventListeners.getListenerCount() > 0)) {
			EventListener[] list = actionEventListeners.getListeners(ActionListener.class);
			ActionEvent event = new ActionEvent(this,actionId,"");
			for (int i=0;i<list.length;i++) {
				ActionListener a = (ActionListener)list[i];
				a.actionPerformed(event);
			}
		}
	}
	
	protected void setInBucket(JComponent inBucket) {
		this.inBucket.removeAll();
		this.inBucket.add(inBucket,BorderLayout.CENTER);
	}
	
	protected void setOutBucket(JComponent outBucket) {
		this.outBucket.removeAll();
		this.outBucket.add(outBucket,BorderLayout.CENTER);
	}
	
	private void fireSelectLocal() {
		fireAction(SELECT_ACTION);
		fireSelect();
	}
	
	public abstract void fireSelect();

	private void fireSelectAllLocal() {
		fireAction(SELECT_ALL_ACTION);
		fireSelectAll();
	}
	
	public abstract void fireSelectAll();


	private void fireRemoveLocal() {
		fireAction(REMOVE_ACTION);
		fireRemove();
	}

	public abstract void fireRemove();
	
	private void fireRemoveAllLocal() {
		fireAction(REMOVE_ALL_ACTION);
		fireRemoveAll();
	}
	
	public abstract void fireRemoveAll();

	public void setRemoveAllButtonState(boolean active) {
		removeAllButton.setEnabled(active);
	}
	public void setRemoveButtonState(boolean active) {
		removeButton.setEnabled(active);
	}
	public void setSelectAllButtonState(boolean active) {
		selectAllButton.setEnabled(active);
	}
	public void setRemoveButtonsState(boolean state){
		removeButton.setEnabled(state);
		removeAllButton.setEnabled(state);
	}

	public IConverter getConverter() {
		return converter;
	}

	public abstract void setInBucketModel(Object model);
	
	public abstract void setOutBucketModel(Object listModel);
	
	public abstract void setOutBucketSelectionMode(int selectionMode);
	
	public abstract Object getInBucketModel();
	
	public abstract Object getOutBucketModel();

	public void setInBucketLabelText(String text) {
		inBucketLabel.setText(text)	;
	}
	
	public void setOutBucketLabelText(String text) {
		outBucketLabel.setText(text);
	}
	
	public String getInBucketLabelText() {
		return inBucketLabel.getText();
	}
	
	public String getOutBucketLabelText() {
		return outBucketLabel.getText();
	}
	
	public void addActionListener(ActionListener listener) {
		actionEventListeners.add(ActionListener.class,listener);
	}

	public void removeActionListener(ActionListener listener) {
		actionEventListeners.remove(ActionListener.class,listener);
	}
	
	public abstract Object[] getInBucketSelectedValues();

	public abstract Object[] getOutBucketSelectedValues();

	/**
	 * Method setConverter.
	 * @param converter
	 */
	public void setConverter(IConverter converter) {
		this.converter = converter;
	}
	
	public abstract void setInBucketSelectionMode(int selectionMode);
	
	protected JPanel getInBucketContainer() {
		return inBucket;
	}
	
	protected JPanel getOutBucketContainer() {
		return outBucket;
	}
	
	public void setPreferredBucketDimension(Dimension dim) {
		outBucket.setPreferredSize(dim);
		inBucket.setPreferredSize(dim);
	}
	
}