package gov.va.med.hds.cd.spectrum.bucket;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.EventListener;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.EventListenerList;

/**
 * Implements basic slush bucket functionality.
 */
public class BasicSlushBucket extends JPanel implements ISlushBucket {
	private JButton selectAllButton = new JButton();
	private JButton removeAllButton = new JButton();
	private JPanel outBucket = new JPanel();
	private JPanel inBucket = new JPanel();
	private JButton selectButton = new JButton();
	private JButton removeButton = new JButton();
	private EventListenerList actionEventListeners = new EventListenerList();
	private JLabel inBucketLabel = new JLabel("In Bucket Label");
	private JLabel outBucketLabel = new JLabel("Out Bucket Label");
	private final Dimension dimension = new Dimension(400,200);
	private JComponent inComponent;
	private JComponent outComponent;

	
	public BasicSlushBucket() {
		super();
		init();
	}

	public void fireAction(int actionId) {
		if ((actionEventListeners != null) && (actionEventListeners.getListenerCount() > 0)) {
			EventListener[] list = actionEventListeners.getListeners(ActionListener.class);
			ActionEvent event = new ActionEvent(this,actionId,"");
			for (int i=0;i<list.length;i++) {
				ActionListener a = (ActionListener)list[i];
				a.actionPerformed(event);
			}
		}
	}

	public void setRemoveAllButtonEnabled(boolean enabled) {
		removeAllButton.setEnabled(enabled);
	}

	public void setSelectAllButtonEnabled(boolean enabled) {
		selectAllButton.setEnabled(enabled);
	}

	public void setInBucketComponent(JComponent c) {
		this.inBucket.removeAll();
		this.inBucket.add(c,BorderLayout.CENTER);
		inBucketLabel.setLabelFor(c);
		inComponent = c;
		if (inComponent instanceof JScrollPane) {
			inComponent = (JComponent)((JScrollPane)inComponent).getViewport().getView();
		}
	}

	public void setInBucketLabel(String labelText) {
		inBucketLabel.setText(labelText);
	}

	public void setOutBucketLabel(String labelText) {
		outBucketLabel.setText(labelText);
	}

	public void addActionListener(ActionListener listener) {
		actionEventListeners.add(ActionListener.class,listener);
	}

	public void removeActionListener(ActionListener listener) {
		actionEventListeners.remove(ActionListener.class,listener);
	}

	public void setPreferredBucketDimension(Dimension dim) {
		outBucket.setPreferredSize(dim);
		inBucket.setPreferredSize(dim);
	}

	private void init() {
		setAccessibleItems();
		inBucketLabel.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if (inComponent!=null) {
					inComponent.requestFocusInWindow();
				}
			}

			public void focusLost(FocusEvent e) {}
		});
		outBucketLabel.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if (outComponent!=null) {
					outComponent.requestFocusInWindow();
				}
			}

			public void focusLost(FocusEvent e) {}
		});
		outBucket.setPreferredSize(dimension);
		inBucket.setPreferredSize(dimension);
		this.setLayout(new GridBagLayout());
		this.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		inBucket.setLayout(new BorderLayout());
		outBucket.setLayout(new BorderLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(2,2,2,2);
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		this.add(inBucketLabel,gbc);
		gbc.gridx = 2;
		this.add(outBucketLabel,gbc);
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridheight = 7;
		gbc.weightx = 0.5;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		this.add(inBucket,gbc);
		gbc.gridx = 2;
		this.add(outBucket,gbc);
		gbc.gridy = 1;
		gbc.gridx = 1;
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		this.add(Box.createVerticalStrut(20),gbc);
		gbc.gridy = 2;
		this.add(selectButton,gbc);
		gbc.gridy = 3;
		this.add(removeButton,gbc);
		gbc.gridy = 4;
		this.add(Box.createVerticalStrut(10),gbc);
		gbc.gridy = 5;
		this.add(selectAllButton,gbc);
		gbc.gridy = 6;
		this.add(removeAllButton,gbc);
		gbc.gridy = 7;
		this.add(Box.createVerticalStrut(20),gbc);

		selectAllButton.setText(">>");
		selectAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireAction(SELECT_ALL_ACTION);
			}
		});
		
		removeAllButton.setText("<<");
		removeAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireAction(REMOVE_ALL_ACTION);
			}
		});
		
		selectButton.setText(">");
		selectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireAction(SELECT_ACTION);
			}
		});
		removeButton.setText("<");
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fireAction(REMOVE_ACTION);
			}
		});
		
		Dimension minSize = selectAllButton.getMinimumSize();
		Dimension maxSize = selectAllButton.getMaximumSize();
		Dimension prefSize = selectAllButton.getPreferredSize();
		
		selectAllButton.setPreferredSize(prefSize);
		selectAllButton.setMinimumSize(minSize);
		selectAllButton.setMaximumSize(maxSize);
		removeAllButton.setPreferredSize(prefSize);
		removeAllButton.setMinimumSize(minSize);
		removeAllButton.setMaximumSize(maxSize);
		selectButton.setPreferredSize(prefSize);
		selectButton.setMinimumSize(minSize);
		selectButton.setMaximumSize(maxSize);
		removeButton.setPreferredSize(prefSize);
		removeButton.setMinimumSize(minSize);
		removeButton.setMaximumSize(maxSize);
	}
	
	private void setAccessibleItems() {
		selectButton.getAccessibleContext().setAccessibleName("Select");
		selectAllButton.getAccessibleContext().setAccessibleName("Select all");
		removeButton.getAccessibleContext().setAccessibleName("Remove");
		removeAllButton.getAccessibleContext().setAccessibleName("Remove all");
		selectButton.setToolTipText(selectButton.getAccessibleContext().getAccessibleName());
		selectAllButton.setToolTipText(selectAllButton.getAccessibleContext().getAccessibleName());
		removeButton.setToolTipText(removeButton.getAccessibleContext().getAccessibleName());
		removeAllButton.setToolTipText(removeAllButton.getAccessibleContext().getAccessibleName());
	}
	
	public void setRemoveButtonEnabled(boolean enabled) {
		removeButton.setEnabled(enabled);
	}
	
	public void setSelectButtonEnabled(boolean enabled) {
		selectButton.setEnabled(enabled);
	}

	public void setInBucketComponent(JComponent c, char mnemonic) {
		setInBucketComponent(c);
		setInBucketMnemonic(mnemonic);
	}

	public void setOutBucketComponent(JComponent c, char mnemonic) {
		setOutBucketComponent(c);
		setOutBucketMnemonic(mnemonic);
	}

	public void setOutBucketComponent(JComponent c) {
		this.outBucket.removeAll();
		this.outBucket.add(c,BorderLayout.CENTER);
		outBucketLabel.setLabelFor(c);
		outComponent = c;
		if (outComponent instanceof JScrollPane) {
			outComponent = (JComponent)((JScrollPane)outComponent).getViewport().getView();
		}
	}

	public void setInBucketMnemonic(char mnemonic) {
		inBucketLabel.setDisplayedMnemonic(mnemonic);
	}

	public void setOutBucketMnemonic(char mnemonic) {
		outBucketLabel.setDisplayedMnemonic(mnemonic);
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("BasicSlushBucket");
		BasicSlushBucket basicSlushBucket = new BasicSlushBucket();
		frame.getContentPane().add(basicSlushBucket);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		frame.pack();
		frame.show();
	}

	public void setSelectAction(Action action) {
		selectButton.setAction(action);
	}

	public void setSelectAllAction(Action action) {
		selectAllButton.setAction(action);
	}

	public void setRemoveAction(Action action) {
		removeButton.setAction(action);
	}

	public void setRemoveAllAction(Action action) {
		removeAllButton.setAction(action);
	}

}
