package gov.va.med.hds.cd.spectrum.bucket;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.ListModel;
import javax.swing.MutableComboBoxModel;

import gov.va.med.hds.cd.spectrum.DoubleLookupListBox;

/**
 * Slush bucket where the right hand bucket is a JList connected to a ListModel, and the
 * left hand bucket is a DoubleLookupListBox with a ListModel.
 */
public class ShortListBucket extends AbstractListBucket {
	private DoubleLookupListBox inBucket;
	private ListModel inBucketModel;
	private ActionListener selectListener = new SelectActionListener();
	
	public ShortListBucket() {
		super();
		inBucketModel = new DefaultListModel();
		inBucket = new DoubleLookupListBox(this.getInBucketContainer().getRootPane(),inBucketModel,false);
		super.setInBucket(inBucket);
		inBucket.addActionListener(selectListener);
	}
	
	private class SelectActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			fireSelect();
		}
	}

	public void fireSelect() {
		Object[] items = getInBucketSelectedValues();
		MutableComboBoxModel outModel = (MutableComboBoxModel)super.getOutBucketModel();
		for (int i = 0; i < items.length; i++) {
			Object item = convert(items[i]);
			if (!outBucketContains(outModel,item)) {
				outModel.addElement(item);
			}
		}
	}

	private boolean outBucketContains(MutableComboBoxModel outModel, Object item) {
		boolean result = false;
		for (int i = 0; i < outModel.getSize(); i++) {
			Object tempItem = outModel.getElementAt(i);
			if (item.equals(tempItem)) {
				result = true;
				break;
			}
		}
		return result;
	}

	protected Object convert(Object item) {
		if (super.getConverter()!=null) {
			return super.getConverter().convert(item);
		}
		else {
			return item;
		}
	}

	public void fireSelectAll() {
		MutableComboBoxModel outModel = (MutableComboBoxModel)getOutBucketModel();
		for (int i = 0; i < inBucketModel.getSize(); i++) {
			Object newItem = convert(inBucketModel.getElementAt(i));
			if (!outBucketContains(outModel,newItem)) {
				outModel.addElement(newItem);
			}
		}
	}

	public void setInBucketModel(Object model) {
		if (model!=null) {
			inBucketModel = (ListModel)model;
		}
		else {
			inBucketModel = new DefaultListModel();
		}
		if (inBucket!=null) {
			inBucket.removeActionListener(selectListener);
		}
		inBucket = new DoubleLookupListBox(this.getInBucketContainer().getRootPane(),inBucketModel,false);
		inBucket.addActionListener(selectListener);
		super.setInBucket(inBucket);
	}

	public Object getInBucketModel() {
		return inBucketModel;
	}

	public Object[] getInBucketSelectedValues() {
		return inBucket.getSelectedItems();
	}

	public void setInBucketSelectionMode(int selectionMode) {
	}

	public static final void main(String args[]) {
		JFrame frame = new JFrame("ShortListBucket");
		ShortListBucket bucket = new ShortListBucket();
		DefaultListModel model = new DefaultListModel();
		model.addElement("Item 1");
		model.addElement("Item 2");
		model.addElement("Item 3");
		bucket.setInBucketModel(model);
		bucket.setOutBucketModel(new DefaultComboBoxModel());
		frame.getContentPane().add(bucket, BorderLayout.CENTER);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
}
