package gov.va.med.hds.cd.spectrum.bucket;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.MutableComboBoxModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;


public class TreeSlushBucket extends AbstractListBucket {
	public static final int SINGLE_SELECTION_MODE = 0;
	public static final int MULTIPLE_SELECTION_MODE = 1;
	
	private JScrollPane inBucket;
	private JTree inBucketTree = new JTree();

	public TreeSlushBucket() {
		super();
		init();
	}
	
	private void init() {
		inBucket = new JScrollPane(inBucketTree);
		super.setInBucket(inBucket);
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#fireSelect()
	 */
	public void fireSelect() {
		fireAction(SELECT_ACTION);
		Object item = null;
		DefaultMutableTreeNode mItem = null;
		MutableComboBoxModel model = (MutableComboBoxModel)super.getOutBucketModel();
		TreePath[] paths = inBucketTree.getSelectionModel().getSelectionPaths();
		if (paths != null) {
			for (int i=0;i<paths.length;i++) {
				Object[] items = paths[i].getPath();
				mItem = (DefaultMutableTreeNode)items[items.length-1];
				item = convert(mItem.getUserObject());
				if (!modelContains(model,item)) {
					model.addElement(item);
				}
			}
		}
	}

	private Object convert(Object item) {
		Object newItem = item;
		if (super.getConverter() != null) {
			newItem = super.getConverter().convert(item);
		}
		return newItem;
	}
	
	private boolean modelContains(MutableComboBoxModel model, Object item) {
		boolean result = false;
		for (int i=0;i<model.getSize();i++) {
			if ((model.getElementAt(i)).equals(item)) {
				result = true;
				break;
			}
		}
		return result;
	}
	
	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#fireSelectAll()
	 */
	public void fireSelectAll() {
		MutableComboBoxModel model = (MutableComboBoxModel)super.getOutBucketModel();
		while (model.getSize()>0) {
			model.removeElementAt(0);
		}
		addElement(inBucketTree.getModel().getRoot(),model);
	}

	private void addElement(Object nodeObject,MutableComboBoxModel outBucketModel) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodeObject;
		outBucketModel.addElement(node.getUserObject());
		for (int i=0;i<node.getChildCount();i++) {
			addElement(node.getChildAt(i),outBucketModel);
		}
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#getInBucketModel()
	 */
	public Object getInBucketModel() {
		return inBucketTree.getModel();
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#getInBucketSelectedValues()
	 */
	public Object[] getInBucketSelectedValues() {
		return null;
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#setInBucketModel(Object)
	 */
	public void setInBucketModel(Object model) {
		inBucketTree.setModel((TreeModel)model);
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.bucket.AbstractSlushBucket#setInBucketSelectionMode(int)
	 */
	public void setInBucketSelectionMode(int selectionMode) {
		if (selectionMode == MULTIPLE_SELECTION_MODE) {
			inBucketTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
			setSelectAllButtonState(true);
		}
		else {
			inBucketTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			setSelectAllButtonState(false);
		}
	}

}