package gov.va.med.hds.cd.spectrum.checklist;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.accessibility.AccessibleContext;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;

public class CheckList extends JList {
	private ICheckableModel checkModel = new CheckableModel();
	private ListCellRenderer checkBoxRenderer = new CheckBoxRenderer();
	private int verticalClickSpace;

	public CheckList() {
		super();
		init();
	}
	
	private void init() {
		setCellRenderer(checkBoxRenderer);
		verticalClickSpace = UIManager.getIcon("CheckBoxMenuItem.checkIcon").getIconWidth() + ((JCheckBox)checkBoxRenderer).getIconTextGap();

		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent me) {
				int selectedIndex = locationToIndex(me.getPoint());
				if (selectedIndex < 0)
					return;
				if (me.getPoint().getX()<verticalClickSpace) {
					ICheckable item = (ICheckable)getModel().getElementAt(selectedIndex);
					item.setChecked(!item.isChecked());
					ICheckable[] items = new ICheckable[1];
					items[0] = item;
					getAccessibleContext().firePropertyChange(AccessibleContext.ACCESSIBLE_VISIBLE_DATA_PROPERTY, null, null);
					checkModel.fireCheckChangeEvent(CheckList.this, items);
				}
				repaint();
			}
		});
		getInputMap().put(KeyStroke.getKeyStroke(' '), "TOGGLE");
		getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
			"TOGGLE");
		final Action selectAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				int[] indexes = getSelectedIndices();
				if (indexes == null || indexes.length == 0)
					return;

				List items = new ArrayList();
				ICheckable item = (ICheckable)getModel().getElementAt(indexes[0]);
				boolean newChecked = !item.isChecked();

				for (int i = 0; i < indexes.length; i++) {
					ICheckable box = (ICheckable)getModel().getElementAt(indexes[i]);
					if (box.isChecked()!=newChecked) {
						box.setChecked(newChecked);
						items.add(box);
					}
				}
				if (items.size()>0) {
					getAccessibleContext().firePropertyChange(AccessibleContext.ACCESSIBLE_VISIBLE_DATA_PROPERTY, null, null);
					checkModel.fireCheckChangeEvent(CheckList.this, (ICheckable[])items.toArray(new ICheckable[0]));
				}
				repaint();
			}
		};
		getActionMap().put("TOGGLE", selectAction);

	}

	public static void main(String args[]) {
		JFrame frame = new JFrame("CheckList");
		CheckList checkList = new CheckList();
		DefaultListModel model = new DefaultListModel();
		checkList.setModel(model);
		frame.getContentPane().add(checkList);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}

	class CheckBoxRenderer extends JCheckBox implements ListCellRenderer {

		public CheckBoxRenderer() {
			setBorderPaintedFlat(true);
		}

		public Component getListCellRendererComponent(
							JList list,
							Object value,
							int index,
							boolean isSelected,
							boolean cellHasFocus) {

			ICheckable item = (ICheckable)value;
			setSelected(item.isChecked());
			setText(item.toString());
			setBackground(isSelected ? list.getSelectionBackground() : list.getBackground());
			setForeground(isSelected ? list.getSelectionForeground(): list.getForeground());

			return this;
		}

	}
	
	public ICheckableModel getCheckableModel() {
		return checkModel;
	}
}
