package gov.va.med.hds.cd.spectrum.date;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.Action;
import javax.swing.JFormattedTextField;
import javax.swing.border.Border;
import javax.swing.text.DateFormatter;

public class BasicDateComboBoxEditor implements IDateComboBoxEditor {

	public static final DateFormat DEFAULT_FORMAT = new SimpleDateFormat("MMM dd, yyyy");

	private DateFormat dateFormat = DEFAULT_FORMAT;
	protected BorderlessFormattedTextField editor;
	private Calendar tempDate = Calendar.getInstance();

	public BasicDateComboBoxEditor() {
        this(DEFAULT_FORMAT);
    }
    
    public BasicDateComboBoxEditor(DateFormat format) {
		editor = new BorderlessFormattedTextField(format);
		editor.addPropertyChangeListener(new EditorPropertyChangeListener());
	}

	public Component getEditorComponent() {
		return editor;
	}

	public void setDate(Calendar date) {
		if (date == null) {
			editor.setValue(null);
			return;
		}
		editor.setValue(date.getTime());
		editor.selectAll();
	}

	public Calendar getDate() {
		Date value = (Date) editor.getValue();
		if (value == null)
			return null;
		tempDate.setTime(value);
		return tempDate;
	}

	public DateFormat getDateFormat() {
		return dateFormat;
	}

	public void setDateFormat(DateFormat format) {
		dateFormat = format;
        editor.setFormat(format);
	}

	public void selectAll() {
		editor.selectAll();
		editor.requestFocus();
	}

	public void addActionListener(ActionListener l) {
		editor.addActionListener(l);
	}

	public void removeActionListener(ActionListener l) {
		editor.removeActionListener(l);
	}

	protected void fireActionEvent(ActionEvent e) {
		ActionListener[] listeners = editor.getActionListeners();
		for (int i = 0; i < listeners.length; i++) {
			listeners[i].actionPerformed(e);
		}
	}

	private static class BorderlessFormattedTextField extends JFormattedTextField {
		public BorderlessFormattedTextField(DateFormat format) {
			super(new InadjustableDateFormatter(format));
			setBorder(null);
			resetKeyboardActions();
		}

		public void setBorder(Border b) {
		}
		
		public Dimension getMinimumSize() {
			return getPreferredSize();
		}
        
        public void setFormat(DateFormat format) {
            setFormatter(new InadjustableDateFormatter(format));
        }

	}

	static class InadjustableDateFormatter extends DateFormatter {
		
		public InadjustableDateFormatter(DateFormat dateFormat) {
			super(dateFormat);
		}

		protected Action[] getActions() {
			return null;
		}

	}

	private class EditorPropertyChangeListener implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			if (!"value".equals(evt.getPropertyName()))
				return;
			final ActionEvent e = new ActionEvent(BasicDateComboBoxEditor.this, 0, "");
			fireActionEvent(e);
		}
	}
}
