package gov.va.med.hds.cd.spectrum.date;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.Calendar;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import gov.va.med.hds.util.CalendarMath;

/**
 * 
 * TODO_SB: Provide description for "CalendarTable".
 */
public class CalendarTable extends JTable {

	private Calendar today = Calendar.getInstance();

	/**
	 * 
	 * TODO_SB: Provide description for "CalendarTable" constructor
	 *
	 */
	public CalendarTable() {
		this(Calendar.getInstance());
	}

	/**
	 * 
	 * TODO_SB: Provide description for "CalendarTable" constructor
	 * @param initialDate
	 */
	public CalendarTable(Calendar initialDate) {
		super(new CalendarTableModel(initialDate));

		getTableHeader().setReorderingAllowed(false);
		setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		configureColumnSizes();

		setCellSelectionEnabled(true);
		getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		getColumnModel().getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		setDefaultRenderer(Calendar.class, new CalendarTableCellRenderer());
	}

	public void setAnchorDate(Calendar anchorDate) {
		((CalendarTableModel) getModel()).setAnchorDate(anchorDate);
	}
	
	public Calendar getAnchorDate() {
		return ((CalendarTableModel) getModel()).getAnchorDate();
	}

	public Calendar getSelectedDate() {
		int row = getSelectedRow();
		int col = getSelectedColumn();
		if (row == -1 || col == -1) return null;
		return (Calendar) getValueAt(row, col);
	}

	public void setSelectedDate(Calendar date) {
		if (date == null) {
			clearSelection();
		} else {
			Calendar first = (Calendar) getValueAt(0, 0);
			Calendar last = (Calendar) getValueAt(getRowCount() - 1, getColumnCount() - 1);
			if (!CalendarMath.isSameMonth(getAnchorDate(), date)) setAnchorDate(date);
			int[] indices = getSelectionIndices(date);
			setRowSelectionInterval(indices[0], indices[0]);
			setColumnSelectionInterval(indices[1], indices[1]);
		}
	}

	private int[] getSelectionIndices(Calendar date) {
		int rowCount = getRowCount();
		int columnCount = getColumnCount();
		for (int i = 0; i < rowCount; ++i) {
			for (int j = 0; j < columnCount; ++j) {
				if (CalendarMath.isSameDate((Calendar) getValueAt(i, j), date))
					return new int[] { i, j };
			}
		}
		return new int[] { -1, -1 };
	}

	protected void configureColumnSizes() {
		TableColumn column = null;
		int maxHeaderWidth = 0;
		for (int i = 0; i < getColumnCount(); i++) {
			column = getColumnModel().getColumn(i);
			maxHeaderWidth =
				Math.max(
					maxHeaderWidth,
					getFontMetrics(getFont()).stringWidth((String) column.getHeaderValue()));
		}
		for (int i = 0; i < getColumnCount(); i++) {
			column = getColumnModel().getColumn(i);
			column.setMinWidth(maxHeaderWidth + 8);
			column.setPreferredWidth(maxHeaderWidth + 8);
		}
	}

	private Calendar getToday() {
		return today;
	}

	private class CalendarTableCellRenderer extends DefaultTableCellRenderer {

		public CalendarTableCellRenderer() {
			super();
			setHorizontalAlignment(SwingConstants.CENTER);
		}

		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
			Calendar c = (Calendar) value;
			JLabel label =
				(JLabel) super.getTableCellRendererComponent(
					table,
					new Integer(c.get(Calendar.DAY_OF_MONTH)),
					isSelected,
					hasFocus,
					row,
					column);
			if (CalendarMath.isSameDate(getToday(), c)) {
				if (isSelected) {
					label.setForeground(getTodayColor());
					label.setBackground(table.getSelectionBackground());
				} else {
					label.setForeground(table.getForeground());
					label.setBackground(getTodayColor());
				}
				label.setFont(label.getFont().deriveFont(Font.BOLD));
			} else {
				if (isSelected) {
					label.setForeground(table.getSelectionForeground());
					label.setBackground(table.getSelectionBackground());
				} else {
					label.setForeground(table.getForeground());
					if (!CalendarMath.isSameMonth(getAnchorDate(), c)) {
						label.setBackground(Color.LIGHT_GRAY);
					} else {
						label.setBackground(Color.WHITE);
					}
				}
			}

			label.setBorder(noFocusBorder);

			return label;
		}

		private Color getTodayColor() {
			return Color.YELLOW;
		}
	}
}
