package gov.va.med.hds.cd.spectrum.date;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.Serializable;
import java.util.Calendar;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

class DateChooserDialog extends JDialog {

	private Calendar initialDate;
	private DateChooser chooserPane;

	public DateChooserDialog(
		Component c,
		String title,
		boolean modal,
		DateChooser chooserPane,
		ActionListener okListener,
		ActionListener cancelListener)
		throws HeadlessException {
		super(JOptionPane.getFrameForComponent(c), title, modal);

		this.chooserPane = chooserPane;

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(chooserPane, BorderLayout.CENTER);

		/*
		 * Create Lower button panel
		 */
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.CENTER));

		JButton todayButton = createTodayButton();
		buttonPane.add(todayButton);

		JButton okButton = createOKButton(okListener);
		buttonPane.add(okButton);

		final JButton cancelButton = createCancelButton(cancelListener);
		buttonPane.add(cancelButton);
		
		contentPane.add(buttonPane, BorderLayout.SOUTH);

		registerEscapeKeyToCancel(cancelButton);

		if (JDialog.isDefaultLookAndFeelDecorated()) {
			boolean supportsWindowDecorations =
				UIManager.getLookAndFeel().getSupportsWindowDecorations();
			if (supportsWindowDecorations) {
				getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);
			}
		}
		applyComponentOrientation(((c == null) ? getRootPane() : c).getComponentOrientation());

		pack();
		setLocationRelativeTo(c);
	}

	private void registerEscapeKeyToCancel(final JButton cancelButton) {
		Action cancelKeyAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				cancelButton.doClick();
				//				((AbstractButton) e.getSource()).fireActionPerformed(e);
			}
		};
		KeyStroke cancelKeyStroke = KeyStroke.getKeyStroke((char) KeyEvent.VK_ESCAPE, false);
		InputMap inputMap = cancelButton.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionMap = cancelButton.getActionMap();
		if (inputMap != null && actionMap != null) {
			inputMap.put(cancelKeyStroke, "cancel");
			actionMap.put("cancel", cancelKeyAction);
		}
	}

	private JButton createOKButton(ActionListener okListener) {
		JButton okButton = new JButton("OK");
		getRootPane().setDefaultButton(okButton);
		okButton.setActionCommand("OK");
		if (okListener != null) {
			okButton.addActionListener(okListener);
		}
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				hide();
			}
		});
		return okButton;
	}

	private JButton createCancelButton(ActionListener cancelListener) {
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		if (cancelListener != null) {
			cancelButton.addActionListener(cancelListener);
		}
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				hide();
			}
		});
		return cancelButton;
	}

	private JButton createTodayButton() {
		JButton todayButton = new JButton("Today");
		todayButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				today();
			}
		});
		//			int mnemonic = UIManager.getInt("ColorChooser.resetMnemonic", -1);
		//			if (mnemonic != -1) {
		//				resetButton.setMnemonic(mnemonic);
		//			}
		return todayButton;
	}

	public void show() {
		initialDate = chooserPane.getDate();
		super.show();
	}

	public void today() {
		chooserPane.setDate(Calendar.getInstance());
	}

	static class Closer extends WindowAdapter implements Serializable {
		public void windowClosing(WindowEvent e) {
			Window w = e.getWindow();
			w.hide();
		}
	}

	static class DisposeOnClose extends ComponentAdapter implements Serializable {
		public void componentHidden(ComponentEvent e) {
			Window w = (Window) e.getComponent();
			w.dispose();
		}
	}
}
