package gov.va.med.hds.cd.spectrum.date;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.accessibility.Accessible;
import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleState;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.ActionMapUIResource;

import gov.va.med.hds.util.CalendarMath;
import gov.va.med.hds.util.date.FlexibleDateFormat;

public class DateComboBox extends JComponent implements Accessible {

	static class AltAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			if (comboBox.isEnabled()) {
				comboBox.setPopupVisible(!comboBox.isPopupVisible());
			}
		}
	}

	static class DownAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			if (comboBox.isEnabled() && comboBox.isShowing()) {
				if (!comboBox.isPopupVisible()) {
					comboBox.setPopupVisible(true);
				}
			}
		}
	}

	private class EditorActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			editorChanged();
		}
	}

	private class EditorFocusListener extends FocusAdapter implements Runnable {
        
		/**
		 * This will make the comboBox fire an ActionEvent if the editor
		 * value is different from the selected item in the model.  
		 * This allows for the entering of data in the combo box editor and
		 * sends notification when tabbing or clicking out of focus.
		 */
		public void focusLost(FocusEvent e) { 
            if (e.isTemporary()) return;
            SwingUtilities.invokeLater(this);   
        }
        
        public void run() {
            maybeFireEditorChanged();
        }
        
        private void maybeFireEditorChanged() {

			IDateComboBoxEditor editor = getEditor();
			Calendar date = editor.getDate();
            Calendar oldDate = getSelectedCalendar();
            
            if ((oldDate != null || date != null) &&
                !CalendarMath.isSameDate(oldDate, date)) {
                editorChanged();
            }
		}

	}

	static class EnterAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			if (!comboBox.isEnabled()) {
				return;
			}
			if (comboBox.isPopupVisible()) {
				comboBox.setPopupVisible(false);
			} else {
				// Call the default button binding.
				// This is a pretty messy way of passing an event through
				// to the root pane.
				JRootPane root = SwingUtilities.getRootPane(comboBox);
				if (root != null) {
					InputMap im = root.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
					ActionMap am = root.getActionMap();
					if (im != null && am != null) {
						Object obj = im.get(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
						if (obj != null) {
							Action action = am.get(obj);
							if (action != null) {
								action.actionPerformed(e);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * This listener hides the popup when the focus is lost.  It also repaints
	 * when focus is gained or lost.
	 *
	 * This public inner class should be treated as protected. Instantiate it
	 * only within subclasses of <code>BasicComboBoxUI</code>.
	 */
	private class FocusHandler implements FocusListener {
		public void focusGained(FocusEvent e) {
			hasFocus = true;
			repaint();

			// Notify assistive technologies that the combo box
			// gained focus.
			if (DateComboBox.this instanceof Accessible) {
				AccessibleContext ac = ((Accessible) DateComboBox.this).getAccessibleContext();
				if (ac != null) {
					ac.firePropertyChange(
						AccessibleContext.ACCESSIBLE_STATE_PROPERTY,
						null,
						AccessibleState.FOCUSED);
				}
			}
		}

		public void focusLost(FocusEvent e) {
			hasFocus = false;
			// GES, 980818:
			// Note that the second check here is a workaround to bug
			// 4168483.  There is a bogus focusLost sent to the
			// ComboBox with isTemporary false when a mediumweight menu 
			// is popped up.  Until this is fixed in AWT, we make the
			// tradeoff of not popping down mediumweight popups when
			// the combobox loses focus.  Although this means that the
			// combobox does not remove such menus when you tab out,
			// it is seen as more desirable than the alternative which 
			// is that mediumweight combobox menus dissappear immediately
			// on popup, rendering them completely unusable.
			if (!e.isTemporary()) {
				setPopupVisible(false);
			}
			repaint();

			// Notify assistive technologies that the combo box
			// lost focus.
			if (DateComboBox.this instanceof Accessible) {
				AccessibleContext ac = ((Accessible) DateComboBox.this).getAccessibleContext();
				if (ac != null) {
					ac.firePropertyChange(
						AccessibleContext.ACCESSIBLE_STATE_PROPERTY,
						AccessibleState.FOCUSED,
						null);
				}
			}
		}
	}

	class HidePopupAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			if (isEnabled()) {
				setPopupVisible(false);
			}
		}

		public boolean isEnabled() {
			return isPopupVisible();
		}
	}

	/**
	* The interface that defines an <code>IKeySelectionManager</code>. To
	* qualify as a <code>KeySelectionManager</code>, the class needs to
	* implement the method that identifies the list index given a character and
	* the combo box data model.
	*/
	public interface IKeySelectionManager {
		/** Given <code>aKey</code> and the model, returns the row
			*  that should become selected. Return -1 if no match was
			*  found. 
			*
			* @param  aKey  a char value, usually indicating a keyboard key that
			*               was pressed
			* @param aModel a ComboBoxModel -- the component's data model, containing
			*               the list of selectable items 
			* @return an int equal to the selected row, where 0 is the
			*         first item and -1 is none. 
			*/
		int selectionForKey(char aKey, ComboBoxModel aModel);
	}

	/**
	 * This listener checks to see if the key event isn't a navigation key.  If
	 * it finds a key event that wasn't a navigation key it dispatches it to
	 * DateComboBox.selectWithKeyChar() so that it can do type-ahead.
	 *
	 * This public inner class should be treated as protected. 
	 * Instantiate it only within subclasses of 
	 * <code>BasicComboBoxUI</code>.
	 */
	private class KeyHandler extends KeyAdapter {

		boolean isTypeAheadKey(KeyEvent e) {
			return !e.isAltDown() && !e.isControlDown() && !e.isMetaDown();
		}
		public void keyPressed(KeyEvent e) {
			if (isEnabled() && !isNavigationKey(e.getKeyCode()) && isTypeAheadKey(e)) {
				if (selectWithKeyChar(e.getKeyChar())) {
					e.consume();
				}
			}
		}
	}

	static class NavigationalAction extends AbstractAction {
		int keyCode;

		NavigationalAction(int keyCode) {
			this.keyCode = keyCode;
		}

		public void actionPerformed(ActionEvent ev) {
			DateComboBox comboBox = (DateComboBox) ev.getSource();
			//			int index = getNextIndex(comboBox);
			//			if (index >= 0 && index < comboBox.getItemCount()) {
			//				comboBox.setSelectedIndex(index);
			//			}
		}

		//		int getNextIndex(DateComboBox comboBox) {
		//			switch (keyCode) {
		//				//				case KeyEvent.VK_PAGE_UP :
		//				//					int listHeight = comboBox.getMaximumRowCount();
		//				//					int index = comboBox.getSelectedIndex() - listHeight;
		//				//					return (index < 0 ? 0 : index);
		//				//				case KeyEvent.VK_PAGE_DOWN :
		//				//					listHeight = comboBox.getMaximumRowCount();
		//				//					index = comboBox.getSelectedIndex() + listHeight;
		//				//					int max = comboBox.getItemCount();
		//				//					return (index < max ? index : max - 1);
		//				//				case KeyEvent.VK_HOME :
		//				//					return 0;
		//				//				case KeyEvent.VK_END :
		//				//					return comboBox.getItemCount() - 1;
		//				default :
		//					return comboBox.getSelectedIndex();
		//			}
		//		}
	}

	/**
	 * This listener watches for bound properties that have changed in the
	 * combo box. 
	 * <p>
	 * Subclasses which wish to listen to combo box property changes should
	 * call the superclass methods to ensure that the combo box ui correctly 
	 * handles property changes.
	 * <p>
	 * This public inner class should be treated as protected. 
	 * Instantiate it only within subclasses of 
	 * <code>BasicComboBoxUI</code>.
	 * 
	 * @see #createPropertyChangeListener
	 */
	private class PropertyChangeHandler implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent e) {
			String propertyName = e.getPropertyName();
			DateComboBox comboBox = (DateComboBox) e.getSource();

			if (propertyName.equals("selectionModel")) {
				IDateSelectionModel newModel = (IDateSelectionModel) e.getNewValue();
				IDateSelectionModel oldModel = (IDateSelectionModel) e.getOldValue();

				if (oldModel != null && selectionModelListener != null) {
					oldModel.removeChangeListener(selectionModelListener);
				}

				if (newModel != null && selectionModelListener != null) {
					newModel.addChangeListener(selectionModelListener);
				}

				if (getEditor() != null && isEditable()) {
					setSelectionSource(this);
					getEditor().setDate(getSelectedCalendar());
					setSelectionSource(null);
				}

				comboBox.revalidate();
				comboBox.repaint();
			} else if (propertyName.equals("editor") && comboBox.isEditable()) {
				removeEditor();
				addEditor();
				comboBox.revalidate();
				comboBox.repaint();
			} else if (propertyName.equals("editable")) {
				if (comboBox.isEditable()) {
					comboBox.setRequestFocusEnabled(false);
					addEditor();
				} else {
					comboBox.setRequestFocusEnabled(true);
					removeEditor();
				}

				updateToolTipTextForChildren();

				comboBox.revalidate();
				comboBox.repaint();
			} else if (propertyName.equals("enabled")) {
				boolean enabled = comboBox.isEnabled();
				if (editor != null)
					editor.getEditorComponent().setEnabled(enabled);
				if (pickerButton != null)
					pickerButton.setEnabled(enabled);
				comboBox.repaint();
			} else if (propertyName.equals("font")) {
				label.setFont(getFont());
				if (editor != null) {
					editor.getEditorComponent().setFont(comboBox.getFont());
				}
				comboBox.validate();
			} else if (propertyName.equals(JComponent.TOOL_TIP_TEXT_KEY)) {
				updateToolTipTextForChildren();
			}
		}
	}

	/**
	 * This listener watches for changes in the <code>IDateSelectionListener</code>.
	 *
	 * @see #createSelectionListener
	 */
	private class SelectionModelChangeHandler implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			if (getSelectedCalendar() != null) {
				label.setText(dateFormat.format(getSelectedDate()));
			} else {
				label.setText("");
			}
			if (getEditor() != null) {
				if (getSelectionSource() != getEditor()) {
					getEditor().setDate(getSelectedCalendar());
				}
			}
			repaint();
		}
	}

	// Same as the AltAction except that it doesn't invoke if
	// the space key is pressed in the editable text portion.
	static class SpaceAction extends AltAction {
		public void actionPerformed(ActionEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			if (!comboBox.isEditable()) {
				super.actionPerformed(e);
			}
		}
	}

	static class UpAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			if (comboBox.isEnabled()) {
				if (comboBox.isPopupVisible()) {
					//					ui.selectPreviousPossibleValue();
				}
			}
		}
	}

	private static final String CALENDAR_ICON = "DateComboIcon16.gif";
	private static Action endAction = new NavigationalAction(KeyEvent.VK_END);

	private static Action homeAction = new NavigationalAction(KeyEvent.VK_HOME);
	private static Action pgDownAction = new NavigationalAction(KeyEvent.VK_PAGE_DOWN);
	private static Action pgUpAction = new NavigationalAction(KeyEvent.VK_PAGE_UP);

	private Action action;
	private String actionCommand = "comboBoxChanged";
	private PropertyChangeListener actionPropertyChangeListener;

	private DateFormat dateFormat;

	private boolean editable = true;

	private IDateComboBoxEditor editor;
	private ActionListener editorActionListener;

	private FocusListener editorFocusListener;
	private boolean firingActionEvent = false;
	private FocusListener focusListener;

	private boolean hasFocus = false;

	private KeyListener keyListener;

	private IKeySelectionManager keySelectionManager = null;

	// This label is used to draw the current date when the combo box is not editable
	private JLabel label;

	//	The button that invokes the popup.
	private JButton pickerButton;

	// The implementation of IDateComboPopup that is used to show the popup.
	private IDateComboPopup popup;
	private KeyListener popupKeyListener;

	// Listeners that the DateComboPopup produces.
	private MouseListener popupMouseListener;
	private MouseMotionListener popupMouseMotionListener;
	private PropertyChangeListener propertyChangeListener;

	private Calendar selectedDateReminder = null;
	private Object selectionSource;

	private IDateSelectionModel selectionModel;
	private ChangeListener selectionModelListener;

	public DateComboBox() {
		this(new DefaultDateSelectionModel());
	}

	public DateComboBox(Calendar date) {
		this(new DefaultDateSelectionModel(date));
	}

	public DateComboBox(Date date) {
		this(new DefaultDateSelectionModel(date));
	}

	public DateComboBox(IDateSelectionModel model) {
		super();
		this.selectionModel = model;
		init();
	}

	/** 
	 * Adds an <code>ActionListener</code>. 
	 * <p>
	 * The <code>ActionListener</code> will receive an <code>ActionEvent</code>
	 * when a selection has been made. If the combo box is editable, then
	 * an <code>ActionEvent</code> will be fired when editing has stopped.
	 *
	 * @param l  the <code>ActionListener</code> that is to be notified
	 * @see #setSelectedItem
	 */
	public void addActionListener(ActionListener l) {
		listenerList.add(ActionListener.class, l);
	}

	private void addEditor() {
		Component editorComponent = getEditor().getEditorComponent();
		if (editorComponent != null) {
			if (editor instanceof Accessible) {
				AccessibleContext ac = ((Accessible) editor).getAccessibleContext();
				if (ac != null) {
					ac.setAccessibleParent(DateComboBox.this);
				}
			}

			if (focusListener != null) {
				editor.getEditorComponent().addFocusListener(focusListener);
			}

			installEditorListeners();

			setSelectionSource(this);
			getEditor().setDate(getSelectedCalendar());
			setSelectionSource(null);
			
			editorComponent.setEnabled(isEnabled());
			if (getFont() != null) editorComponent.setFont(getFont());
			
			add(editorComponent, BorderLayout.CENTER);

			revalidate();
			repaint();
		}
	}

	private void configureComboBox() {
		setRequestFocusEnabled(true);
		setOpaque(true);
		setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		setLayout(new BorderLayout());

		add(this.label, BorderLayout.CENTER);
		add(this.pickerButton, BorderLayout.EAST);
		if (isEditable())
			addEditor();
	}

	private ActionMap createActionMap() {
		ActionMap map = new ActionMapUIResource();

		map.put("hidePopup", new HidePopupAction());
		map.put("pageDownPassThrough", pgDownAction);
		map.put("pageUpPassThrough", pgUpAction);
		map.put("homePassThrough", homeAction);
		map.put("endPassThrough", endAction);
		map.put("selectNext", new DownAction());
		map.put("togglePopup", new AltAction());
		map.put("spacePopup", new SpaceAction());
		map.put("selectPrevious", new UpAction());
		map.put("enterPressed", new EnterAction());

		return map;
	}

	protected DateFormat createDateFormat() {
        //return new SimpleDateFormat("dd MMM, yyyy");
		return new FlexibleDateFormat();
	}

	/**
	 * Creates the default editor that will be used in editable combo boxes.  
	 * A default editor will be used only if an editor has not been 
	 * explicitly set with <code>setEditor</code>.
	 *
	 * @return a <code>ComboBoxEditor</code> used for the combo box
	 * @see javax.swing.DateComboBox#setEditor
	 */
	protected IDateComboBoxEditor createEditor() {
		return new BasicDateComboBoxEditor(getDateFormat());
	}

	/**
	 * Creates a <code>FocusListener</code> which will be added to the combo
	 * box. If this method returns null then it will not be added to the combo
	 * box.
	 *
	 * @return an instance of a <code>FocusListener</code> or null
	 */
	protected FocusListener createFocusListener() {
		return new FocusHandler();
	}

	/**
	 * Creates a <code>KeyListener</code> which will be added to the combo box.
	 * If this method returns null then it will not be added to the combo box.
	 * 
	 * @return an instance <code>KeyListener</code> or null
	 */
	protected KeyListener createKeyListener() {
		return new KeyHandler();
	}

	/**
	 * Creates the popup portion of the combo box.
	 *
	 * @return an instance of <code>ComboPopup</code>
	 * @see ComboPopup
	 */
	protected IDateComboPopup createPopup() {
		PopupDateChooser popup = new PopupDateChooser(this);
		popup.getAccessibleContext().setAccessibleParent(this);
		if (keySelectionManager == null)
			keySelectionManager = popup;
		return popup;
	}

	/**
	 * Creates a <code>PropertyChangeListener</code> which will be added to the
	 * combo box. If this method returns null then it will not be added to the
	 * combo box.
	 * @return an instance of a <code>PropertyChangeListener</code> or null
	 */
	protected PropertyChangeListener createPropertyChangeListener() {
		return new PropertyChangeHandler();
	}

	/**
	 * Creates a change listener which will be added to the
	 * <code>IDateSelectionModel</code>. If this method returns null then it will not
	 * be added to the selection model.
	 *
	 * @return an instance of a <code>ChangeListener</code> or null
	 */
	protected ChangeListener createSelectionModelListener() {
		return new SelectionModelChangeHandler();
	}

	private void setSelectionSource(Object source) {
		this.selectionSource = source;
	}

	private Object getSelectionSource() {
		return selectionSource;
	}

	private void editorChanged() {
		if (getSelectionSource() == this) return;
//		System.out.println("DateComboBox.editorChanged()");
		Calendar newItem = getEditor().getDate();
		setPopupVisible(false);
		setSelectionSource(getEditor());
		getSelectionModel().setSelectedDate(newItem);
		setSelectionSource(null);
		String oldCommand = getActionCommand();
		setActionCommand("comboBoxEdited");
		fireActionEvent();
		setActionCommand(oldCommand);
	}

	/**
	 * Notifies all listeners that have registered interest for notification on
	 * this event type.
	 *  
	 * @see EventListenerList
	 */
	protected void fireActionEvent() {
		if (!firingActionEvent) {
			// Set flag to ensure that an infinite loop is not created
			firingActionEvent = true;
			ActionEvent e = null;
			// Guaranteed to return a non-null array
			Object[] listeners = listenerList.getListenerList();
			long mostRecentEventTime = EventQueue.getMostRecentEventTime();
			int modifiers = 0;
			AWTEvent currentEvent = EventQueue.getCurrentEvent();
			if (currentEvent instanceof InputEvent) {
				modifiers = ((InputEvent) currentEvent).getModifiers();
			} else if (currentEvent instanceof ActionEvent) {
				modifiers = ((ActionEvent) currentEvent).getModifiers();
			}
			// Process the listeners last to first, notifying
			// those that are interested in this event
			for (int i = listeners.length - 2; i >= 0; i -= 2) {
				if (listeners[i] == ActionListener.class) {
					// Lazily create the event:
					if (e == null)
						e =
							new ActionEvent(
								this,
								ActionEvent.ACTION_PERFORMED,
								getActionCommand(),
								mostRecentEventTime,
								modifiers);
					((ActionListener) listeners[i + 1]).actionPerformed(e);
				}
			}
			firingActionEvent = false;
		}
	}

	/** 
	  * Returns the action command that is included in the event sent to
	  * action listeners.
	  *
	  * @return  the string containing the "command" that is sent
	  *          to action listeners.
	  */
	private String getActionCommand() {
		return actionCommand;
	}

	public DateFormat getDateFormat() {
		return dateFormat;
	}

	public IDateComboBoxEditor getEditor() {
		return editor;
	}

	/**
	  * Returns the current selected item.
	  * <p>
	  * If the combo box is editable, then this value may not have been added
	  * to the combo box with <code>addItem</code>, <code>insertItemAt</code> 
	  * or the data constructors.
	  * 
	  * @return the current selected Object
	  * @see #setSelectedItem
	  */
	public Calendar getSelectedCalendar() {
		return getSelectionModel().getSelectedDate();
	}

	public Date getSelectedDate() {
		Calendar c = getSelectedCalendar();
		if (c == null)
			return null;
		return c.getTime();
	}

	public IDateSelectionModel getSelectionModel() {
		return selectionModel;
	}

	/** 
	 * Causes the combo box to close its popup window.
	 * @see #setPopupVisible
	 */
	public void hidePopup() {
		setPopupVisible(false);
	}

	private void init() {
		installComponents();
		installListeners();
		installKeyboardActions();
	}

	private void installAncestorListener() {
		addAncestorListener(new AncestorListener() {
			public void ancestorAdded(AncestorEvent event) {
				hidePopup();
			}
			public void ancestorMoved(AncestorEvent event) {
				if (event.getSource() != DateComboBox.this)
					hidePopup();
			}
			public void ancestorRemoved(AncestorEvent event) {
				hidePopup();
			}
		});
	}

	private void installComponents() {
		installDateFormat();
		installLabel();
		installPopup();
		installPickerButton();
		installEditor();
		configureComboBox();
	}

	private void installDateFormat() {
		if (getDateFormat() == null)
			setDateFormat(createDateFormat());
	}

	private void installEditor() {
		if (getEditor() == null) {
			setEditor(createEditor());
		}
	}

	private void installEditorListeners() {
		if (editorFocusListener == null) {
			editorFocusListener = new EditorFocusListener();
		}
		editor.getEditorComponent().addFocusListener(editorFocusListener);

//		if (editorActionListener == null) {
//			editorActionListener = new EditorActionListener();
//		}
//		getEditor().addActionListener(editorActionListener);
	}

	private void installKeyboardActions() {
		SwingUtilities.replaceUIInputMap(
			this,
			JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,
			(InputMap) UIManager.get("ComboBox.ancestorInputMap"));

		ActionMap am = createActionMap();
		if (am != null) {
			SwingUtilities.replaceUIActionMap(this, am);
		}
	}

	private void installLabel() {
		label = new JLabel();
		label.setOpaque(true);
		label.setFont(getFont());
		label.setBackground(UIManager.getColor("ComboBox.background"));
		label.setForeground(UIManager.getColor("ComboBox.foreground"));
	}

	private void installListeners() {
		installAncestorListener();
		if ((propertyChangeListener = createPropertyChangeListener()) != null) {
			addPropertyChangeListener(propertyChangeListener);
		}
		if ((keyListener = createKeyListener()) != null) {
			addKeyListener(keyListener);
		}
		if ((focusListener = createFocusListener()) != null) {
			addFocusListener(focusListener);
		}
		if ((popupMouseListener = popup.getMouseListener()) != null) {
			addMouseListener(popupMouseListener);
		}
		if ((popupMouseMotionListener = popup.getMouseMotionListener()) != null) {
			addMouseMotionListener(popupMouseMotionListener);
		}
		if ((popupKeyListener = popup.getKeyListener()) != null) {
			addKeyListener(popupKeyListener);
		}

		if (getSelectionModel() != null) {
			if ((selectionModelListener = createSelectionModelListener()) != null) {
				getSelectionModel().addChangeListener(selectionModelListener);
			}
		}
	}

	private void installPickerButton() {
		pickerButton = new JButton(new ImageIcon(DateComboBox.class.getResource(CALENDAR_ICON)));
		pickerButton.setPreferredSize(new Dimension(17, 17));
		pickerButton.setEnabled(isEnabled());
		pickerButton.setFocusable(false);
		pickerButton.setRequestFocusEnabled(false);
		pickerButton.addMouseListener(popup.getMouseListener());
		pickerButton.addMouseMotionListener(popup.getMouseMotionListener());
		pickerButton.resetKeyboardActions();
	}

	private void installPopup() {
		this.popup = createPopup();
	}

	/**
	 * Returns true if the <code>DateComboBox</code> is editable.
	 * By default, a combo box is not editable.
	 * 
	 * @return true if the <code>DateComboBox</code> is editable, else false
	 */
	public boolean isEditable() {
		return editable;
	}

	public boolean isFocusable() {
		return !isEditable();
	}

	/**
	 * Returns whether or not the supplied keyCode maps to a key that is used
	 * for navigation.  This is used for optimizing key input by only passing
	 * non- navigation keys to the type-ahead mechanism.  Subclasses should
	 * override this if they change the navigation keys.
	 */
	protected boolean isNavigationKey(int keyCode) {
		return keyCode == KeyEvent.VK_UP || keyCode == KeyEvent.VK_DOWN ||
		// This is horrible, but necessary since these aren't 
		// supported until JDK 1.2
		keyCode == KeyStroke.getKeyStroke("KP_UP").getKeyCode()
			|| keyCode == KeyStroke.getKeyStroke("KP_DOWN").getKeyCode();
	}

	/** 
	 * Determines the visibility of the popup.
	 *
	 * @return true if the popup is visible, otherwise returns false
	 */
	public boolean isPopupVisible() {
		return popup.isVisible();
	}

	/**
	 * Handles <code>KeyEvent</code>s, looking for the Tab key.
	 * If the Tab key is found, the popup window is closed.
	 *
	 * @param e  the <code>KeyEvent</code> containing the keyboard
	 *		key that was pressed  
	 */
	public void processKeyEvent(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_TAB) {
			hidePopup();
		}
		super.processKeyEvent(e);
	}

	/**
	 * Removes an <code>ActionListener</code>.
	 *
	 * @param l  the <code>ActionListener</code> to remove
	 */
	public void removeActionListener(ActionListener l) {
		listenerList.remove(ActionListener.class, l);
	}

	private void removeEditor() {
		Component editorComponent = getEditor().getEditorComponent();
		if (editorComponent != null) {
			if (focusListener != null) {
				editor.getEditorComponent().removeFocusListener(focusListener);
			}

			if (editorFocusListener != null) {
				editor.getEditorComponent().removeFocusListener(editorFocusListener);
			}

			if (editorActionListener != null) {
				getEditor().removeActionListener(editorActionListener);
			}
			remove(editorComponent);
		}
		if (getSelectedCalendar() != null) {
			label.setText(dateFormat.format(getSelectedDate()));
		} else {
			label.setText("");
		}
		add(label, BorderLayout.CENTER);
	}

	private void selectedDateChanged() {
		Calendar oldSelection = selectedDateReminder;
		Calendar newSelection = selectionModel.getSelectedDate();
		if (oldSelection == null || !oldSelection.equals(newSelection)) {
			selectedDateReminder = selectionModel.getSelectedDate();
			if (getSelectionSource() != this) {
				fireActionEvent();
			}
		}
	}

	/**
	* Selects the list item that corresponds to the specified keyboard
	* character and returns true, if there is an item corresponding
	* to that character.  Otherwise, returns false.
	*
	* @param keyChar a char, typically this is a keyboard key
	*			typed by the user
	*/
	private boolean selectWithKeyChar(char keyChar) {
		int index;

		//		index = keySelectionManager.selectionForKey(keyChar, getModel());
		//		if (index != -1) {
		//			setSelectedIndex(index);
		//			return true;
		//		} else
		return false;
	}

	/** 
	 * Sets the action command that should be included in the event
	 * sent to action listeners.
	 *
	 * @param aCommand  a string containing the "command" that is sent
	 *                  to action listeners; the same listener can then
	 *                  do different things depending on the command it
	 *                  receives
	 */
	private void setActionCommand(String aCommand) {
		actionCommand = aCommand;
	}

	public void setDateFormat(DateFormat format) {
		DateFormat oldDateFormat = this.dateFormat;
		this.dateFormat = format;
        if (editor != null)
            editor.setDateFormat(format);
		firePropertyChange("dateFormat", oldDateFormat, this.dateFormat);
	}

	/**
	 * Determines whether the <code>DateComboBox</code> field is editable.
	 * An editable <code>DateComboBox</code> allows the user to type into the
	 * field or selected an item from the list to initialize the field,
	 * after which it can be edited. (The editing affects only the field,
	 * the list item remains intact.) A non editable <code>DateComboBox</code> 
	 * displays the selected item in the field,
	 * but the selection cannot be modified.
	 *
	 * @param aFlag a boolean value, where true indicates that the
	 *			field is editable
	 * 
	 * @beaninfo
	 *        bound: true
	 *    preferred: true
	 *  description: If true, the user can type a new value in the combo box.
	 */
	public void setEditable(boolean aFlag) {
		boolean oldFlag = editable;
		editable = aFlag;
		firePropertyChange("editable", oldFlag, editable);
	}

	/**
	* Sets the editor used to paint and edit the selected item in the 
	* <code>DateComboBox</code> field.  The editor is used only if the
	* receiving <code>DateComboBox</code> is editable. If not editable,
	* the combo box uses the renderer to paint the selected item.
	*  
	* @param anEditor  the <code>ComboBoxEditor</code> that
	*			displays the selected item
	* @see #setRenderer
	* @beaninfo
	*     bound: true
	*    expert: true
	*  description: The editor that combo box uses to edit the current value
	*/
	public void setEditor(IDateComboBoxEditor anEditor) {
		IDateComboBoxEditor oldEditor = editor;

		if (editor != null) {
			editor.removeActionListener(editorActionListener);
		}
		editor = anEditor;
		if (editor != null) {
			editor.addActionListener(editorActionListener);
		}
		firePropertyChange("editor", oldEditor, editor);
	}

	/**
	 * Sets the visibility of the popup.
	 */
	public void setPopupVisible(boolean v) {
		if (v) {
			popup.showPopup();
		} else {
			popup.hidePopup();
		}
	}

	/** 
	 * Sets the selected item in the combo box display area to the object in 
	 * the argument.
	 * If <code>anObject</code> is in the list, the display area shows 
	 * <code>anObject</code> selected.
	 * <p>
	 * If <code>anObject</code> is <i>not</i> in the list and the combo box is
	 * uneditable, it will not change the current selection. For editable 
	 * combo boxes, the selection will change to <code>anObject</code>.
	 * <p>
	 * If this constitutes a change in the selected item, 
	 * <code>ItemListener</code>s added to the combo box will be notified with
	 * one or two <code>ItemEvent</code>s.
	 * If there is a current selected item, an <code>ItemEvent</code> will be
	 * fired and the state change will be <code>ItemEvent.DESELECTED</code>. 
	 * If <code>anObject</code> is in the list and is not currently selected
	 * then an <code>ItemEvent</code> will be fired and the state change will 
	 * be <code>ItemEvent.SELECTED</code>.
	 * <p>
	 * <code>ActionListener</code>s added to the combo box will be notified
	 * with an <code>ActionEvent</code> when this method is called.
	 *
	 * @param anObject  the list object to select; use <code>null</code> to
						clear the selection
	 * @beaninfo
	 *    preferred:   true
	 *    description: Sets the selected item in the DateComboBox.
	 */
	public void setSelectedDate(Calendar date) {
		Calendar oldSelection = selectedDateReminder;
		if (oldSelection == null || !CalendarMath.isSameDate(oldSelection, date)) {


			// Must toggle the state of this flag since this method
			// call may result in ListDataEvents being fired.
			setSelectionSource(this);
			getSelectionModel().setSelectedDate(date);
			setSelectionSource(null);

			if (selectedDateReminder != getSelectionModel().getSelectedDate()) {
				// in case a users implementation of IDateSelectionModel
				// doesn't fire a ChangeEvent when the selection
				// changes.
				selectedDateChanged();
			}
		}
		fireActionEvent();
	}

	public void setSelectedDate(Date date) {
		if (date == null) {
			setSelectedDate((Calendar) null);
		} else {
			Calendar c = Calendar.getInstance();
			c.setTime(date);
			setSelectedDate(c);
		}
	}

	public void setSelectionModel(IDateSelectionModel model) {
		IDateSelectionModel oldModel = this.selectionModel;
		if (oldModel != null) {
			oldModel.removeChangeListener(selectionModelListener);
		}
		this.selectionModel = model;
		model.addChangeListener(selectionModelListener);

		// set the current selected item.
		selectedDateReminder = model.getSelectedDate();

		firePropertyChange("selectionModel", oldModel, model);
	}

	/** 
	 * Causes the combo box to display its popup window.
	 * @see #setPopupVisible
	 */
	public void showPopup() {
		setPopupVisible(true);
	}

	// Syncronizes the ToolTip text for the components within the combo box to be the 
	// same value as the combo box ToolTip text.
	private void updateToolTipTextForChildren() {
		Component[] children = getComponents();
		for (int i = 0; i < children.length; ++i) {
			if (children[i] instanceof JComponent) {
				((JComponent) children[i]).setToolTipText(getToolTipText());
			}
		}
	}

	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

}
