package gov.va.med.hds.cd.spectrum.date;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

/**
 * TODO_SB: Provide description for "DefaultDateSelectionModel".
 */
public class DefaultDateSelectionModel implements IDateSelectionModel, Serializable {
	/**
	 * Only one <code>ChangeEvent</code> is needed per model instance
	 * since the event's only (read-only) state is the source property.
	 * The source of events generated here is always "this".
	 */
	protected transient ChangeEvent changeEvent = null;

	protected EventListenerList listenerList = new EventListenerList();

	private Calendar selectedDate;

	/**
	 * Creates a <code>DefaultDateSelectionModel</code> with the
	 * current date set to <code>Color.white</code>.  This is
	 * the default constructor.
	 */
	public DefaultDateSelectionModel() {
		selectedDate = Calendar.getInstance();
	}

	public DefaultDateSelectionModel(Date date) {
		Calendar c = Calendar.getInstance();
		c.setTime(date);
		selectedDate = c;
	}

	/**
	 * Creates a <code>DefaultColorSelectionModel</code> with the
	 * current color set to <code>color</code>, which should be
	 * non-<code>null</code>.  Note that setting the color to
	 * <code>null</code> is undefined and may have unpredictable
	 * results.
	 *
	 * @param date the new <code>Calendar</code>
	 */
	public DefaultDateSelectionModel(Calendar date) {
		selectedDate = date;
	}

	/**
     * Adds a <code>ChangeListener</code> to the model.
     *
     * @param l the <code>ChangeListener</code> to be added
     */
    public void addChangeListener(ChangeListener l) {
		listenerList.add(ChangeListener.class, l);
    }

    /**
     * Removes a <code>ChangeListener</code> from the model.
     * 
     * @param l the <code>ChangeListener</code> to be removed
     */
    public void removeChangeListener(ChangeListener l) {
		listenerList.remove(ChangeListener.class, l);
    }

	/**
	 * Returns the selected <code>Calendar</code>.
	 *
	 * @return the selected <code>Calendar</code>
	 */
	public Calendar getSelectedDate() {
		return selectedDate;
	}

	/**
	* Sets the selected date to <code>date</code>.
	* This method fires a state changed event if it sets the
	* current date to a new date;
	* if the new date is the same as the current date,
	* no event is fired.
	*
	* @param color the new <code>Color</code>
	*/
	public void setSelectedDate(Calendar date) {
		if (selectedDate != null && selectedDate.equals(date)) return;
		selectedDate = date;
		fireStateChanged();
	}

	/**
	 * Returns an array of all the <code>ChangeListener</code>s added
	 * to this <code>DefaultColorSelectionModel</code> with
	 * <code>addChangeListener</code>.
	 *
	 * @return all of the <code>ChangeListener</code>s added, or an empty
	 *         array if no listeners have been added
	 */
	public ChangeListener[] getChangeListeners() {
		return (ChangeListener[]) listenerList.getListeners(ChangeListener.class);
	}

	/**
	 * Runs each <code>ChangeListener</code>'s <code>stateChanged</code> method.
	 * @see EventListenerList
	 */
	protected void fireStateChanged() {
		Object[] listeners = listenerList.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ChangeListener.class) {
				if (changeEvent == null) {
					changeEvent = new ChangeEvent(this);
				}
				((ChangeListener) listeners[i + 1]).stateChanged(changeEvent);
			}
		}
	}
}
