package gov.va.med.hds.cd.spectrum.date;

import java.util.Calendar;

import javax.swing.event.ChangeListener;

/**
 * A model that supports selecting a date.
 */
public interface IDateSelectionModel {
   	/**
	 * Adds <code>listener</code> as a listener to changes in the model.
	 * @param listener the <code>ChangeListener</code> to be added
	 */
	void addChangeListener(ChangeListener listener);
   	/**
	 * Removes <code>listener</code> as a listener to changes in the model.
	 * @param listener the <code>ChangeListener</code> to be removed
	 */
	void removeChangeListener(ChangeListener listener);
	/**
	 * Returns the selected <code>Calendar</code> which should be
	 * non-<code>null</code>.
	 *
	 * @return  the selected <code>Calendar</code>
	 * @see     #setSelectedDate
	 */
	Calendar getSelectedDate();
	/**
	 * Sets the selected date to <code>date</code>.
	 * Note that setting the date to <code>null</code>
	 * is undefined and may have unpredictable results.
	 * This method fires a state changed event if it sets the
	 * current color to a new non-<code>null</code> date.
	 *
	 * @param date the new <code>Calendar</code>
	 * @see   #getSelectedDate
	 * @see   #addChangeListener
	 */
	void setSelectedDate(Calendar date);
}
