package gov.va.med.hds.cd.spectrum.date;

import java.util.Calendar;
import java.util.Date;

import javax.swing.AbstractListModel;

import gov.va.med.hds.util.CalendarMath;

public class MonthListModel extends AbstractListModel {

	private Calendar startDate;
	private Calendar[] days;

	public MonthListModel() {
		this(Calendar.getInstance());
	}

	public MonthListModel(Date startDate) {
		setStartDate(startDate);
	}

	public MonthListModel(Calendar startDate) {
		setStartDate(startDate);
	}

	public void setStartDate(Date startDate) {
		Calendar d = Calendar.getInstance();
		d.setTime(startDate);
		setStartDate(d);
	}

	public void setStartDate(Calendar startDate) {
		if (this.startDate != null && isInMonth(startDate)) return;
		this.startDate = startDate;
		refreshDays();
	}

	public Calendar getStartDate() {
		return startDate;
	}

	public Object getElementAt(int index) {
		if (index < 0 || index >= days.length)
			throw new IndexOutOfBoundsException();
		return days[index];
	}

	public int getSize() {
		if (days == null)
			return 0;
		return days.length;
	}

	private void refreshDays() {
		if (days != null && isInMonth(startDate))
			return;

		int oldSize = getSize();
		days = new Calendar[getDaysInMonth()];
		if (oldSize < getSize()) {
			fireIntervalAdded(this, oldSize, getSize() - 1);
		} else if (oldSize > getSize()) {
			fireIntervalRemoved(this, 0, getSize() - 1);
		}

		for (int day = 1; day <= getDaysInMonth(); day++) {
			Calendar c = (Calendar) startDate.clone();
			c.set(Calendar.DAY_OF_MONTH, day);
			days[day - 1] = c;
		}
		fireContentsChanged(this, 0, getSize() - 1);
	}

	private int getDaysInMonth() {
		return startDate.getActualMaximum(Calendar.DAY_OF_MONTH);
	}

	private boolean isInMonth(Calendar c) {
		return CalendarMath.isSameMonth(startDate, c);
	}
}
