package gov.va.med.hds.cd.spectrum.date;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;
import java.util.Vector;

import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleState;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.Popup;
import javax.swing.PopupFactory;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

public class PopupDateChooser
	extends JPanel
	implements IDateComboPopup, DateComboBox.IKeySelectionManager {

	private DateComboBox comboBox;

	private DateChooser dateChooser;
	private JPanel buttonPanel;
	private JButton todayButton;
	private JButton noneButton;

	//	private boolean hasEntered = false;
	private boolean updatingSelection = false;

	private transient Component invoker;
	private transient Popup popup;
	private transient Frame frame;
	private int desiredLocationX, desiredLocationY;

	protected PropertyChangeListener propertyChangeListener;
	protected ChangeListener comboBoxSelectionListener;

	protected MouseMotionListener mouseMotionListener;
	protected MouseListener mouseListener;
	protected KeyListener keyListener;

	private MouseListener dateChooserMouseListener;
	private MouseMotionListener dateChooserMouseMotionListener;
	private ChangeListener dateChooserSelectionListener;
	private PropertyChangeListener dateChooserPropertyChangeListener;

	private transient MouseGrabber mouseGrabber = null;

	public PopupDateChooser(DateComboBox comboBox) {
		super();
		this.comboBox = comboBox;

		setFocusTraversalKeysEnabled(false);

		installComponents();
		installListeners();
		installKeyboardActions();
	}

	private void installComponents() {
		configureDateChooser();
		configureButtonPanel();
		configurePopup();
	}

	private void installListeners() {
		installDateChooserListeners();
		if (mouseGrabber == null) {
			mouseGrabber = new MouseGrabber();
			addPropertyChangeListener(mouseGrabber);
		}
		installComboBoxListeners();
	}

	private void installKeyboardActions() {
	}

	/**
	 * Creates a date selection listener that watches for selection changes in
	 * the combo box's date selection model.  If this method returns null then
	 * it will not be added to the combo box list.
	 *
	 * @return an instance of a <code>ChangeListener</code> or null
	 */
	protected ChangeListener createComboBoxSelectionListener() {
		return null;
	}

	/**
	 * Creates a <code>PropertyChangeListener</code> which will be added to
	 * the combo box. If this method returns null then it will not
	 * be added to the combo box.
	 * 
	 * @return an instance of a <code>PropertyChangeListener</code> or null
	 */
	protected PropertyChangeListener createPropertyChangeListener() {
		return new PropertyChangeHandler();
	}
	
	/**
	 * Creates a date selection listener that watches for selection changes in
	 * the popup's date chooser.  If this method returns null then it will not
	 * be added to the popup list.
	 *
	 * @return an instance of a <code>ChangeListener</code> or null
	 */
	protected ChangeListener createDateChooserSelectionListener() {
		return null;
	}

	/**
	 * Creates a mouse listener that watches for mouse events in
	 * the popup's list. If this method returns null then it will
	 * not be added to the combo box.
	 *
	 * @return an instance of a <code>MouseListener</code> or null
	 */
	protected MouseListener createDateChooserMouseListener() {
		return new DateChooserMouseHandler();
	}

	/**
	 * Creates a mouse motion listener that watches for mouse motion 
	 * events in the popup's list. If this method returns null then it will
	 * not be added to the combo box.
	 *
	 * @return an instance of a <code>MouseMotionListener</code> or null
	 */
	protected MouseMotionListener createDateChooserMouseMotionListener() {
//		return new DateChooserMouseMotionHandler();
		return null;
	}

	/**
	 * Creates a property change listener that watches for property changes in
	 * the popup's date picker. If this method returns null then it will not be
	 * added to the combo box.
	 *
	 * @return an instance of a <code>PropertyChangeListener</code> or null
	 */
	protected PropertyChangeListener createDateChooserPropertyChangeListener() {
		//		return null;
		return new DateChooserPropertyChangeListener();
	}

	/**
	 * This method adds the necessary listeners to the DateComboBox.
	 */
	protected void installComboBoxListeners() {
		if ((propertyChangeListener = createPropertyChangeListener()) != null) {
			comboBox.addPropertyChangeListener(propertyChangeListener);
		}
		installComboBoxSelectionListeners(comboBox.getSelectionModel());
	}

	/** 
	  * Installs the listeners on the combo box selection model. Any listeners installed
	  * on the combo box model should be removed in 
	  * <code>uninstallComboBoxModelListeners</code>.
	  *
	  * @param model The combo box model to install listeners
	  * @see #uninstallComboBoxModelListeners
	  */
	protected void installComboBoxSelectionListeners(IDateSelectionModel model) {
		if (model != null && (comboBoxSelectionListener = createComboBoxSelectionListener()) != null) {
			model.addChangeListener(comboBoxSelectionListener);
		}
	}

	protected void installDateChooserListeners() {
		if ((dateChooserMouseListener = createDateChooserMouseListener()) != null) {
			dateChooser.addMouseListener(dateChooserMouseListener);
			todayButton.addMouseListener(dateChooserMouseListener);
			noneButton.addMouseListener(dateChooserMouseListener);
		}
		if ((dateChooserMouseMotionListener = createDateChooserMouseMotionListener()) != null) {
			dateChooser.addMouseMotionListener(dateChooserMouseMotionListener);
		}
		if ((dateChooserSelectionListener = createDateChooserSelectionListener()) != null) {
			dateChooser.getSelectionModel().addChangeListener(dateChooserSelectionListener);
		}
		if ((dateChooserPropertyChangeListener = createDateChooserPropertyChangeListener())
			!= null) {
			dateChooser.addPropertyChangeListener(dateChooserPropertyChangeListener);
		}
	}

	/** 
	 * Removes the listeners from the combo box model
	 *
	 * @param model The combo box model to install listeners
	 * @see #installComboBoxModelListeners
	 */
	protected void uninstallComboBoxSelectionListeners(IDateSelectionModel model) {
		if (model != null && comboBoxSelectionListener != null) {
			model.removeChangeListener(comboBoxSelectionListener);
		}
	}

	/**
	  *  Adds a <code>PopupMenu</code> listener.
	  *
	  *  @param l  the <code>PopupMenuListener</code> to add
	  */
	public void addPopupMenuListener(PopupMenuListener l) {
		listenerList.add(PopupMenuListener.class, l);
	}

	/**
	 * Removes a <code>PopupMenu</code> listener.
	 *
	 * @param l  the <code>PopupMenuListener</code> to remove
	 */
	public void removePopupMenuListener(PopupMenuListener l) {
		listenerList.remove(PopupMenuListener.class, l);
	}

	/**
	 * Displays the popup menu at the position x,y in the coordinate space of
	 * the component invoker.
	 *
	 * @param invoker the component in whose space the popup menu is to appear
	 * @param x the x coordinate in invoker's coordinate space at which the
	 * popup menu is to be displayed
	 * @param y the y coordinate in invoker's coordinate space at which the
	 * popup menu is to be displayed
	 */
	public void show(Component invoker, int x, int y) {
		setInvoker(invoker);
		Frame newFrame = getFrame(invoker);
		if (newFrame != frame) {
			// Use the invoker's frame so that events 
			// are propagated properly
			if (newFrame != null) {
				this.frame = newFrame;
				if (popup != null) {
					setVisible(false);
				}
			}
		}
		Point invokerOrigin;
		if (invoker != null) {
			invokerOrigin = invoker.getLocationOnScreen();
			setLocation(invokerOrigin.x + x, invokerOrigin.y + y);
		} else {
			setLocation(x, y);
		}
		setVisible(true);
	}

	/**
	 * Sets the location of the upper left corner of the
	 * popup menu using x, y coordinates.
	 *
	 * @param x the x coordinate of the popup's new position
	 * @param y the y coordinate of the popup's new position
	 * @beaninfo
	 * description: The location of the popup menu.
	 */
	public void setLocation(int x, int y) {
		int oldX = desiredLocationX;
		int oldY = desiredLocationY;

		desiredLocationX = x;
		desiredLocationY = y;
		if (popup != null && (x != oldX || y != oldY)) {
			popup = getPopup();
		}
	}

	private static Frame getFrame(Component c) {
		Component w = c;

		while (!(w instanceof Frame) && (w != null)) {
			w = w.getParent();
		}
		return (Frame) w;
	}

	/**
	 * Overridden to unconditionally return false.
	 */
	public boolean isFocusable() {
		return false;
	}

	/**
	 * Returns true if the <code>MouseEvent</code> is considered a popup trigger
	 * by the <code>JPopupMenu</code>'s currently installed UI.
	 *
	 * @return true if the mouse event is a popup trigger
	 */
	public boolean isPopupTrigger(MouseEvent e) {
		return (
			(e.getID() == MouseEvent.MOUSE_RELEASED)
				&& ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0));
	}

	public void showPopup() {
		updatingSelection = true;
		dateChooser.setDate(comboBox.getSelectedCalendar());
		updatingSelection = false;
		Point location = getPopupLocation();
		show(comboBox, location.x, location.y);
	}

	public void hidePopup() {
		setVisible(false);
		comboBox.repaint();
	}

	/**
	 * Returns a <code>Popup</code> instance. If the current <code>popup</code>
	 * is non-null, this will invoke <code>dispose</code> of it, and then
	 * <code>show</code> the new one.
	 * 
	 * <p> This does NOT fire any events, it is up the caller to dispatch the
	 * necessary events.
	 */
	private Popup getPopup() {
		Popup oldPopup = popup;

		if (oldPopup != null) {
			oldPopup.hide();
		}
		PopupFactory popupFactory = PopupFactory.getSharedInstance();

		// adjust the location of the popup
		Point p = adjustPopupLocationToFitScreen(desiredLocationX, desiredLocationY);
		desiredLocationX = p.x;
		desiredLocationY = p.y;

		Popup newPopup = popupFactory.getPopup(invoker, this, desiredLocationX, desiredLocationY);

		newPopup.show();
		return newPopup;
	}

	/**
	 * Calculates the upper left location of the Popup.
	 */
	private Point getPopupLocation() {
		Point popupLocation = getBounds().getLocation();
		popupLocation.y += comboBox.getHeight();
		return popupLocation;
	}

	/**
	 * Returns true if the popup menu is visible (currently
	 * being displayed).
	 */
	public boolean isVisible() {
		if (popup != null)
			return true;
		else
			return false;
	}

	/**
		 * Sets the visibility of the popup menu.
		 * 
		 * @param b true to make the popup visible, or false to
		 *          hide it
		 * @beaninfo
		 *           bound: true
		 *     description: Makes the popup visible
		 */
	public void setVisible(boolean b) {
		// Is it a no-op?
		if (b == isVisible())
			return;

		if (b) {
			firePopupMenuWillBecomeVisible();
			popup = getPopup();
			firePropertyChange("visible", Boolean.FALSE, Boolean.TRUE);
		} else if (popup != null) {
			firePopupMenuWillBecomeInvisible();
			popup.hide();
			popup = null;
			firePropertyChange("visible", Boolean.TRUE, Boolean.FALSE);
		}
		if (accessibleContext != null) {
			if (b) {
				accessibleContext.firePropertyChange(
					AccessibleContext.ACCESSIBLE_STATE_PROPERTY,
					null,
					AccessibleState.VISIBLE);
			} else {
				accessibleContext.firePropertyChange(
					AccessibleContext.ACCESSIBLE_STATE_PROPERTY,
					AccessibleState.VISIBLE,
					null);
			}
		}
	}

	/**
	 * Returns the component which is the 'invoker' of this popup.
	 *
	 * @return the <code>Component</code> in which the popup is displayed
	 */
	public Component getInvoker() {
		return this.invoker;
	}

	/**
	 * Sets the invoker of this popup -- the component in which the popup is to
	 * be displayed.
	 *
	 * @param invoker the <code>Component</code> in which the popup is displayed
	 * @beaninfo description: The invoking component for the popup expert: true
	 */
	public void setInvoker(Component invoker) {
		this.invoker = invoker;
		invalidate();
	}

	private Point adjustPopupLocationToFitScreen(int xposition, int yposition) {
		Point p = new Point(xposition, yposition);

		if (GraphicsEnvironment.isHeadless())
			return p;

		Toolkit toolkit = getToolkit();
		Rectangle screenBounds;
		Insets screenInsets;
		GraphicsConfiguration gc = null;
		// Try to find GraphicsConfiguration, that includes mouse
		// pointer position
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice[] gd = ge.getScreenDevices();
		for (int i = 0; i < gd.length; i++) {
			if (gd[i].getType() == GraphicsDevice.TYPE_RASTER_SCREEN) {
				GraphicsConfiguration dgc = gd[i].getDefaultConfiguration();
				if (dgc.getBounds().contains(p)) {
					gc = dgc;
					break;
				}
			}
		}

		// If not found and we have invoker, ask invoker about his gc
		if (gc == null && getInvoker() != null) {
			gc = getInvoker().getGraphicsConfiguration();
		}

		if (gc != null) {
			// If we have GraphicsConfiguration use it to get
			// screen bounds and insets
			screenInsets = toolkit.getScreenInsets(gc);
			screenBounds = gc.getBounds();
		} else {
			// If we don't have GraphicsConfiguration use primary screen
			// and empty insets
			screenInsets = new Insets(0, 0, 0, 0);
			screenBounds = new Rectangle(toolkit.getScreenSize());
		}

		int scrWidth = screenBounds.width - Math.abs(screenInsets.left + screenInsets.right);
		int scrHeight = screenBounds.height - Math.abs(screenInsets.top + screenInsets.bottom);

		Dimension size;

		size = PopupDateChooser.this.getPreferredSize();

		if ((p.x + size.width) > screenBounds.x + scrWidth)
			p.x = screenBounds.x + scrWidth - size.width;

		if ((p.y + size.height) > screenBounds.y + scrHeight)
			p.y = screenBounds.y + scrHeight - size.height;

		/* Change is made to the desired (X,Y) values, when the
			  PopupMenu is too tall OR too wide for the screen
		*/
		if (p.x < screenBounds.x)
			p.x = screenBounds.x;
		if (p.y < screenBounds.y)
			p.y = screenBounds.y;

		return p;
	}

	public MouseListener getMouseListener() {
		if (mouseListener == null) {
			mouseListener = createMouseListener();
		}
		return mouseListener;
	}

	public MouseMotionListener getMouseMotionListener() {
		if (mouseMotionListener == null) {
			mouseMotionListener = createMouseMotionListener();
		}
		return mouseMotionListener;

	}

	public KeyListener getKeyListener() {
		if (keyListener == null) {
			keyListener = createKeyListener();
		}
		return keyListener;
	}

	public int selectionForKey(char aKey, ComboBoxModel aModel) {
		return 0;
	}

	/**
	 * Creates a listener 
	 * that will watch for mouse-press and release events on the combo box.
	 * 
	 * <strong>Warning:</strong>
	 * When overriding this method, make sure to maintain the existing
	 * behavior.
	 *
	 * @return a <code>MouseListener</code> which will be added to 
	 * the combo box or null
	 */
	protected MouseListener createMouseListener() {
		return new InvocationMouseHandler();
	}

	/**
	 * Creates the mouse motion listener which will be added to the combo
	 * box.
	 *
	 * <strong>Warning:</strong>
	 * When overriding this method, make sure to maintain the existing
	 * behavior.
	 *
	 * @return a <code>MouseMotionListener</code> which will be added to
	 *         the combo box or null
	 */
	protected MouseMotionListener createMouseMotionListener() {
		return new InvocationMouseMotionHandler();
	}

	/**
	 * Creates the key listener that will be added to the combo box. If 
	 * this method returns null then it will not be added to the combo box.
	 *
	 * @return a <code>KeyListener</code> or null
	 */
	protected KeyListener createKeyListener() {
		return null;
	}

	/**
	 * Notifies <code>PopupMenuListener</code>s that this popup menu will 
	 * become visible.
	 */
	protected void firePopupMenuWillBecomeVisible() {
		Object[] listeners = listenerList.getListenerList();
		PopupMenuEvent e = null;
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == PopupMenuListener.class) {
				if (e == null)
					e = new PopupMenuEvent(this);
				((PopupMenuListener) listeners[i + 1]).popupMenuWillBecomeVisible(e);
			}
		}
	}

	/**
	 * Notifies <code>PopupMenuListener</code>s that this popup menu will 
	 * become invisible.
	 */
	protected void firePopupMenuWillBecomeInvisible() {
		Object[] listeners = listenerList.getListenerList();
		PopupMenuEvent e = null;
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == PopupMenuListener.class) {
				if (e == null)
					e = new PopupMenuEvent(this);
				((PopupMenuListener) listeners[i + 1]).popupMenuWillBecomeInvisible(e);
			}
		}
	}

	/**
	 * Notifies <code>PopupMenuListeners</code> that this popup menu is 
	 * cancelled.
	 */
	protected void firePopupMenuCanceled() {
		Object[] listeners = listenerList.getListenerList();
		PopupMenuEvent e = null;
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == PopupMenuListener.class) {
				if (e == null)
					e = new PopupMenuEvent(this);
				((PopupMenuListener) listeners[i + 1]).popupMenuCanceled(e);
			}
		}
	}

	private void configureDateChooser() {
		dateChooser = new DateChooser();
	}

	private void configurePopup() {
		setOpaque(false);
		setDoubleBuffered(true);
		setFocusable(false);

		setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));

		setLayout(new BorderLayout());

		add(dateChooser, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);
	}

	private void configureButtonPanel() {
		todayButton = new JButton(new TodayAction());
		noneButton = new JButton(new NoneAction());
		buttonPanel = new JPanel();
		buttonPanel.add(todayButton);
		buttonPanel.add(noneButton);
	}

	/**
	 * Makes the popup visible if it is hidden and makes it hidden if it is
	 * visible.
	 */
	private void togglePopup() {
		if (isVisible()) {
			hidePopup();
		} else {
			showPopup();
		}
	}

	private MouseEvent convertMouseEvent(MouseEvent e) {
		Point convertedPoint =
			SwingUtilities.convertPoint((Component) e.getSource(), e.getPoint(), dateChooser);
		MouseEvent newEvent =
			new MouseEvent(
				(Component) e.getSource(),
				e.getID(),
				e.getWhen(),
				e.getModifiers(),
				convertedPoint.x,
				convertedPoint.y,
				e.getClickCount(),
				e.isPopupTrigger());
		return newEvent;
	}

	private class TodayAction extends AbstractAction {
		public TodayAction() {
			super();
			putValue(Action.NAME, "Today");
		}

		public void actionPerformed(ActionEvent e) {
			dateChooser.setDate(Calendar.getInstance());
		}
	}

	private class NoneAction extends AbstractAction {
		public NoneAction() {
			super();
			putValue(Action.NAME, "None");
		}

		public void actionPerformed(ActionEvent e) {
			dateChooser.setDate((Calendar) null);
		}
	}

	/**
	 * A listener to be registered upon the combo box
	 * (<em>not</em> its popup menu)
	 * to handle mouse events
	 * that affect the state of the popup menu.
	 * The main purpose of this listener is to make the popup menu
	 * appear and disappear.
	 * This listener also helps
	 * with click-and-drag scenarios by setting the selection if the mouse was
	 * released over the list during a drag.
	 *
	 * <p>
	 * <strong>Warning:</strong>
	 * We recommend that you <em>not</em> 
	 * create subclasses of this class.
	 * If you absolutely must create a subclass,
	 * be sure to invoke the superclass
	 * version of each method.
	 *
	 * @see BasicComboPopup#createMouseListener
	 */
	protected class InvocationMouseHandler extends MouseAdapter {
		/**
		 * Responds to mouse-pressed events on the combo box.
		 *
		 * @param e the mouse-press event to be handled
		 */
		public void mousePressed(MouseEvent e) {
			if (!SwingUtilities.isLeftMouseButton(e) || !comboBox.isEnabled())
				return;

			if (comboBox.isEditable()) {
				Component comp = comboBox.getEditor().getEditorComponent();
				if ((!(comp instanceof JComponent))
					|| ((JComponent) comp).isRequestFocusEnabled()) {
					comp.requestFocus();
				}
			} else if (comboBox.isRequestFocusEnabled()) {
				comboBox.requestFocus();
			}
			togglePopup();
		}

		/**
		 * Responds to the user terminating
		 * a click or drag that began on the combo box.
		 *
		 * @param e the mouse-release event to be handled
		 */
		public void mouseReleased(MouseEvent e) {
			Component source = (Component) e.getSource();
			Dimension size = source.getSize();
			Rectangle bounds = new Rectangle(0, 0, size.width - 1, size.height - 1);
			if (!bounds.contains(e.getPoint())) {
				MouseEvent newEvent = convertMouseEvent(e);
				Point location = newEvent.getPoint();
				Rectangle r = new Rectangle();
				dateChooser.computeVisibleRect(r);
				if (r.contains(location)) {
//					comboBox.setSelectedIndex(dateChooser.getSelectedIndex());
				}
				comboBox.setPopupVisible(false);
			}
		}
	}

	/**
	 * This listener watches for dragging and updates the current selection in the
	 * list if it is dragging over the list.
	 */
	protected class InvocationMouseMotionHandler extends MouseMotionAdapter {
		public void mouseDragged(MouseEvent e) {
			if (isVisible()) {
				MouseEvent newEvent = convertMouseEvent(e);
				Rectangle r = new Rectangle();
				dateChooser.computeVisibleRect(r);

				if (newEvent.getPoint().y >= r.y && newEvent.getPoint().y <= r.y + r.height - 1) {
					Point location = newEvent.getPoint();
					if (r.contains(location)) {
						updateSelectionForEvent(newEvent);
					}
				}
			}
		}
	}

	/**
	 * This listener watches for bound properties that have changed in the combo
	 * box.
	 * <p> Subclasses which wish to listen to combo box property changes should
	 * call the superclass methods to ensure that the combo popup correctly
	 * handles property changes.
	 * 
	 * @see #createPropertyChangeListener
	 */
	protected class PropertyChangeHandler implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent e) {
			DateComboBox comboBox = (DateComboBox) e.getSource();
			String propertyName = e.getPropertyName();

			if (propertyName.equals("selectionModel")) {
				IDateSelectionModel oldModel = (IDateSelectionModel) e.getOldValue();
				IDateSelectionModel newModel = (IDateSelectionModel) e.getNewValue();
				uninstallComboBoxSelectionListeners(oldModel);
				installComboBoxSelectionListeners(newModel);

				if (!updatingSelection) {
//					dateChooser.setSelectionModel(newModel);
					if (isVisible()) {
						hidePopup();
					}
				}
			}
		}
	}

	/**
	 * This listener hides the popup when the mouse is released in the date
	 * picker.
	 */
	protected class DateChooserMouseHandler extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
		}
		public void mouseReleased(MouseEvent anEvent) {
			updatingSelection = true;
			comboBox.setSelectedDate(dateChooser.getDate());
			updatingSelection = false;
			comboBox.setPopupVisible(false);
		}
	}

	/**
	 * This listener changes the selected item as you move the mouse over date
	 * picker. The selection change is not committed to the model, this is for
	 * user feedback only.
	 */
//	protected class DateChooserMouseMotionHandler extends MouseMotionAdapter {
//		public void mouseMoved(MouseEvent anEvent) {
//			updateSelectionForEvent(anEvent);
//		}
//	}

	protected class DateChooserPropertyChangeListener implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			if (!"model".equals(evt.getPropertyName()))
				return;
			dateChooser.getSelectionModel().addChangeListener(dateChooserSelectionListener);
		}
	}

	protected void updateSelectionForEvent(MouseEvent anEvent) {
		Point p =
			SwingUtilities.convertPoint(
				(Component) anEvent.getSource(),
				anEvent.getPoint(),
				dateChooser);
//		int index = dateChooser.locationToIndex(anEvent.getPoint());
//		if (dateChooser.getSelectedIndex() != index) {
//			dateChooser.setSelectedIndex(index);
//		}
	}

	private static Window getWindow(Component c) {
		Component w = c;
		while (!(w instanceof Window) && (w != null)) {
			w = w.getParent();
		}
		return (Window) w;
	}

	private class MouseGrabber
		implements
			MouseListener,
			MouseMotionListener,
			MouseWheelListener,
			WindowListener,
			ComponentListener,
			PropertyChangeListener {

		Vector grabbed = new Vector();
		private void requestAddGrab(Component invoker) {
			Window ancestor = getWindow(invoker);
			grabContainer(ancestor, PopupDateChooser.this);
		}

		private void requestRemoveGrab() {
			ungrabContainers();
		}

		private void cancelPopupMenu() {
			ungrabContainers();
			setVisible(false);
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if (!"visible".equals(evt.getPropertyName()))
				return;
			if (Boolean.TRUE.equals(evt.getNewValue())) {
				requestAddGrab(getInvoker());
			} else {
				requestRemoveGrab();
			}
		}

		void grabContainer(Container c, Component excluded) {
			if (c == excluded)
				return;
			Component ignore = null;
			if (c instanceof java.awt.Window) {
				((java.awt.Window) c).addWindowListener(this);
				((java.awt.Window) c).addComponentListener(this);
				grabbed.addElement(c);
			} else if (c instanceof JRootPane) {
				// Only add listeners to the glass pane if it listeners,
				// otherwise if the glass pane is visible no mouse move
				// events will come to us.
				ignore = ((JRootPane) c).getGlassPane();
				if (ignore.getMouseListeners().length > 0
					|| ignore.getMouseMotionListeners().length > 0
					|| ignore.getMouseWheelListeners().length > 0) {
					ignore = null;
				}
			}
			synchronized (c.getTreeLock()) {
				int ncomponents = c.getComponentCount();
				Component[] component = c.getComponents();
				for (int i = 0; i < ncomponents; i++) {
					Component comp = component[i];
					if (!comp.isVisible())
						continue;
					if (comp != ignore) {
						comp.addMouseListener(this);
						comp.addMouseMotionListener(this);
						comp.addMouseWheelListener(this);
						grabbed.addElement(comp);
					}
					if (comp instanceof Container) {
						Container cont = (Container) comp;
						grabContainer(cont, excluded);
					}
				}
			}

		}

		void ungrabContainers() {
			int i, c;
			Component cpn;
			for (i = 0, c = grabbed.size(); i < c; i++) {
				cpn = (Component) grabbed.elementAt(i);
				if (cpn instanceof java.awt.Window) {
					((java.awt.Window) cpn).removeWindowListener(this);
					((java.awt.Window) cpn).removeComponentListener(this);
				} else {
					cpn.removeMouseListener(this);
					cpn.removeMouseMotionListener(this);
					cpn.removeMouseWheelListener(this);
				}
			}
			grabbed = new Vector();
		}

		public void mousePressed(MouseEvent e) {
			cancelPopupMenu();
		}

		public void mouseReleased(MouseEvent e) {
			//			MenuSelectionManager.defaultManager().processMouseEvent(e);
		}
		public void mouseEntered(MouseEvent e) {
			// MenuSelectionManager.defaultManager().processMouseEvent(e);
		}
		public void mouseExited(MouseEvent e) {
			// MenuSelectionManager.defaultManager().processMouseEvent(e);
		}
		public void mouseMoved(MouseEvent e) {
			// MenuSelectionManager.defaultManager().processMouseEvent(e);
		}
		public void mouseDragged(MouseEvent e) {
			//			MenuSelectionManager.defaultManager().processMouseEvent(e);
		}
		public void mouseClicked(MouseEvent e) {
		}
		public void mouseWheelMoved(MouseWheelEvent e) {
			cancelPopupMenu();
		}
		public void componentResized(ComponentEvent e) {
			cancelPopupMenu();
		}
		public void componentMoved(ComponentEvent e) {
			cancelPopupMenu();
		}
		public void componentShown(ComponentEvent e) {
			cancelPopupMenu();
		}
		public void componentHidden(ComponentEvent e) {
			cancelPopupMenu();
		}
		public void windowOpened(WindowEvent e) {
		}
		public void windowClosing(WindowEvent e) {
			cancelPopupMenu();
		}
		public void windowClosed(WindowEvent e) {
			cancelPopupMenu();
		}
		public void windowIconified(WindowEvent e) {
			cancelPopupMenu();
		}
		public void windowDeiconified(WindowEvent e) {
		}
		public void windowActivated(WindowEvent e) {
			cancelPopupMenu();
		}
		public void windowDeactivated(WindowEvent e) {
			cancelPopupMenu();
		}
	}
}
