package gov.va.med.hds.cd.spectrum.dhtml;

import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.AbstractButton;
import javax.swing.ButtonModel;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.FormView;
import javax.swing.text.html.HTML;

import gov.va.med.hds.cd.spectrum.date.DateComboBox;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptletEvent;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ScriptletException;
import gov.va.med.hds.cd.spectrum.text.GenericDocumentFilter;

/**
 * Customizations to the objects displayed in an HTML form.
 */
class CustomFormView extends FormView {

    public CustomFormView(Element elem) {
        super(elem);
    }

    /** Create and customize a form component.
     */
    protected Component createComponent() {
        // get the form component
        Component result = createCustomComponent();
        
        if (result == null)
            result = super.createComponent();

        customizeComponent(result);

        maybeInstallListeners(getElement());

        return result;
    }
    
    private Component createCustomComponent() {
        AttributeSet attr = getElement().getAttributes();
        HTML.Tag t = (HTML.Tag) 
            attr.getAttribute(StyleConstants.NameAttribute);
        if (t != HTML.Tag.INPUT) return null;
        
        String type = (String) attr.getAttribute(HTML.Attribute.TYPE);
        if (type.equalsIgnoreCase("button"))
        return new ButtonInputComponent(attr);        
        
        if (!type.equals("text")) return null;

        String customType = (String) attr.getAttribute("customtype");
        if (customType == null || customType.length() == 0) return null;
        
        if (customType.equalsIgnoreCase("absdate")) try {
            Document model = (Document) attr.getAttribute(StyleConstants.ModelAttribute);  
            return new DateInputComponent(model);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return null;
    }

    private void customizeComponent(Component result) {
        AttributeSet attr = getElement().getAttributes();
        if (result instanceof AbstractButton) {
            String mnemonic = (String) attr.getAttribute("mnemonic");
            if (mnemonic != null)
                ((AbstractButton) result).setMnemonic(mnemonic.charAt(0));
        }
        
        // if the component is a radio button or checkbox, make its
        // background transparent so the background color of the HTML
        // document can show through.
        if (result instanceof JToggleButton) {
            ((AbstractButton) result).setBorder(null);
            ((AbstractButton) result).setMargin(new Insets(0, 0, 0, 0));
            //((AbstractButton) result).setFocusPainted(true);
            String caption = (String) attr.getAttribute("label");
            if (caption != null) {
                ((AbstractButton) result).setText(caption);
                ((AbstractButton) result).setOpaque(false);
            }
            result.setBackground(null);
            
        } else if (result instanceof JTextField) {
            String maxLen = (String) attr.getAttribute("maxlen");
            if (maxLen != null || maxLen.length() > 0) {
                
                Document doc = ((JTextField) result).getDocument();
                if (doc instanceof AbstractDocument) {
                    int len = -1;
                    try {
                        len = Integer.parseInt(maxLen);
                    } catch (NumberFormatException nfe) {}
                    ((AbstractDocument) doc).setDocumentFilter
                        (new GenericDocumentFilter(len));
                }
            }
        }
    }
    
    private void maybeInstallListeners(Element element) {
        AttributeSet attr = getElement().getAttributes();
        String onChange = (String) attr.getAttribute("onchange");
        if (onChange == null || onChange.length() == 0)
            return;

        Object model = attr.getAttribute(StyleConstants.ModelAttribute);
        if (model == null) {
            System.out.println("model is null");
        }

        if (model instanceof JToggleButton.ToggleButtonModel)
            ((JToggleButton.ToggleButtonModel) model).addChangeListener(
                new CheckBoxChangeListener(
                    (JToggleButton.ToggleButtonModel) model,
                    onChange));
    }

    private class CheckBoxChangeListener implements ChangeListener, Runnable {
        private JToggleButton.ToggleButtonModel model;
        private String scriptlet;
        private boolean lastValue;

        public CheckBoxChangeListener(
            JToggleButton.ToggleButtonModel model,
            String scriptlet) {
            this.model = model;
            this.scriptlet = scriptlet;
            lastValue = model.isSelected();
        }
        public void stateChanged(ChangeEvent e) {
            boolean currentValue = model.isSelected();
            if (currentValue == lastValue)
                return;
            lastValue = currentValue;

            SwingUtilities.invokeLater(this);
        }
        public void run() {
            invokeScript();
        }
        private void invokeScript() {
            try {
                DHTMLEditorKit.runScriptlet(
                    scriptlet,
                    IScriptletEvent.VALUE_CHANGED,
                    getElement());
            } catch (ScriptletException e) {
                e.printStackTrace();
            }
        }
    }
    
    protected static final DateFormat DATE_FIELD_FORMAT = 
            new SimpleDateFormat("yyyyMMdd");
    
    private class DateInputComponent extends DateComboBox implements ActionListener {    
        
        private Document model;
        private ComboBoxModel comboBoxModel;
            
        public DateInputComponent(Document model) {
            super(new Date());
            this.model = model;

            addActionListener(this);
            setAlignmentY(1.0f);

            AttributeSet attr = getElement().getAttributes();
            String valueString = (String) attr.getAttribute(HTML.Attribute.VALUE);
            Date value = parseDate(valueString);
            setSelectedDate(value);
        }

        public void actionPerformed(ActionEvent arg0) {
            Date value = getSelectedDate();
            String textValue = 
                (value == null ? "" : DATE_FIELD_FORMAT.format(value));
            try {
                model.remove(0, model.getLength());
                model.insertString(0, textValue, null);
            } catch (BadLocationException ble) { }
        }
    }
    
    private static DateFormat[] DEFAULT_DATE_FORMATS =
        {
            new SimpleDateFormat("yyyyMMdd"),
            DateFormat.getDateInstance(DateFormat.LONG),
            DateFormat.getDateInstance(DateFormat.MEDIUM),
            DateFormat.getDateInstance(DateFormat.SHORT),
            DateFormat.getDateInstance(DateFormat.FULL)
        };

    private Date parseDate(String date) {
        if ((date == null) || (date.length() == 0))
            return null;

        for (int i = 0; i < DEFAULT_DATE_FORMATS.length; i++)
            try {
                return DEFAULT_DATE_FORMATS[i].parse(date);
            } catch (Exception e) {
            }

        return null;
    }
    
    
    
    private class ButtonInputComponent
        extends JButton
        implements ActionListener, Runnable {

        private String onClick;

        public ButtonInputComponent(AttributeSet attr) {
            super();
            setAlignmentY(1.0f);

            String value = (String) attr.getAttribute(HTML.Attribute.VALUE);
            if (value == null)
                value = "OK";
            setText(value);

            Object model = attr.getAttribute(StyleConstants.ModelAttribute);
            if (model != null)
                setModel((ButtonModel) model);

            onClick = (String) attr.getAttribute("onclick");

            addActionListener(this);
        }

        public void actionPerformed(ActionEvent arg0) {
            SwingUtilities.invokeLater(this);
        }

        public void run() {
            invokeScript();
        }

        private void invokeScript() {
            try {
                DHTMLEditorKit.runScriptlet(
                    onClick,
                    IScriptletEvent.MOUSE_CLICK,
                    getElement());
            } catch (ScriptletException e) {
                e.printStackTrace();
            }
        }
    }
        
        
}