package gov.va.med.hds.cd.spectrum.dhtml;

import java.util.EventListener;
import java.util.EventObject;

import javax.swing.event.DocumentEvent;
import javax.swing.event.UndoableEditEvent;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;
import javax.swing.text.html.HTML.Tag;
import javax.swing.text.html.HTMLEditorKit.ParserCallback;

public class DHTMLDocument extends HTMLDocument {

    public interface DHTMLEventListener extends EventListener {
        public void documentLoaded(EventObject event);
    }

    public DHTMLDocument() {
        super();
    }

    public DHTMLDocument(StyleSheet styles) {
        super(styles);
    }

    public DHTMLDocument(Content c, StyleSheet styles) {
        super(c, styles);
    }

    /** Set the value of an attribute on an element. */
    public void setAttribute(Element element, Object key, Object value) {
        try {
            writeLock();
            DefaultDocumentEvent changes =
                new DefaultDocumentEvent(
                    element.getStartOffset(),
                    1,
                    DocumentEvent.EventType.CHANGE);
            AttributeSet sCopy = element.getAttributes().copyAttributes();
            MutableAttributeSet attr =
                (MutableAttributeSet) element.getAttributes();
            changes.addEdit(new AttributeUndoableEdit(element, sCopy, false));
            attr.removeAttribute(key);
            if (value != null)
                attr.addAttribute(key, value);
            changes.end();
            fireChangedUpdate(changes);
            fireUndoableEditUpdate(new UndoableEditEvent(this, changes));
        } finally {
            writeUnlock();
        }
    }

    public void addDHTMLEventListener(DHTMLEventListener l) {
        listenerList.add(DHTMLEventListener.class, l);
    }
    public void removeDHTMLEventListener(DHTMLEventListener l) {
        listenerList.remove(DHTMLEventListener.class, l);
    }

    protected void fireDocumentLoaded() {
        //System.out.println("Document loaded: " + this);

        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        EventObject e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == DHTMLEventListener.class) {
                // Lazily create the event:
                if (e == null)
                    e = new EventObject(this);
                ((DHTMLEventListener) listeners[i + 1]).documentLoaded(e);
            }
        }
    }

    private void maybeFireDocumentLoaded() {
        if (parsingState == DOCUMENT_DONE) {
            parsingState = NOT_PARSING;
            fireDocumentLoaded();
        }
    }

    private static final int NOT_PARSING = 0;
    private static final int IN_DOCUMENT = 1;
    private static final int DOCUMENT_DONE = 2;

    private volatile int parsingState = NOT_PARSING;

    public ParserCallback getReader(
        int pos,
        int popDepth,
        int pushDepth,
        Tag insertTag) {
            
        //System.out.println("in getReader 1");
        return new HTMLReaderWatcher(
            super.getReader(pos, popDepth, pushDepth, insertTag));
    }

    public ParserCallback getReader(int pos) {
        //System.out.println("in getReader 2");
        return new HTMLReaderWatcher(super.getReader(pos));
    }

    private class HTMLReaderWatcher extends HTMLEditorKit.ParserCallback {
        private HTMLEditorKit.ParserCallback delegate;

        public HTMLReaderWatcher(HTMLEditorKit.ParserCallback delegate) {
            this.delegate = delegate;
        }

        public void flush() throws BadLocationException {
            delegate.flush();
            maybeFireDocumentLoaded();
        }

        public void handleComment(char[] data, int pos) {
            delegate.handleComment(data, pos);
        }

        public void handleEndOfLineString(String eol) {
            delegate.handleEndOfLineString(eol);
        }

        public void handleEndTag(Tag t, int pos) {
            delegate.handleEndTag(t, pos);
            if (t == Tag.HTML) {
                //System.out.println("Saw </HTML> tag.");
                parsingState = DOCUMENT_DONE;
            }
        }

        public void handleError(String errorMsg, int pos) {
            delegate.handleError(errorMsg, pos);
        }

        public void handleSimpleTag(Tag t, MutableAttributeSet a, int pos) {
            delegate.handleSimpleTag(t, a, pos);
        }

        public void handleStartTag(Tag t, MutableAttributeSet a, int pos) {
            delegate.handleStartTag(t, a, pos);
            if (t == Tag.HTML) {
                //System.out.println("Saw <HTML> tag.");
                parsingState = IN_DOCUMENT;
            }
        }

        public void handleText(char[] data, int pos) {
            delegate.handleText(data, pos);
        }
    }

}
