package gov.va.med.hds.cd.spectrum.dhtml;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptletEvent;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Registry;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.RuntimeScriptletRegistry;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ScriptletException;

public class DHTMLPane extends JEditorPane {

    private DHTMLEditorKit editorKit;

    public DHTMLPane(IRuntimeContext runtime) {
        this(RuntimeScriptletRegistry.getRegistry(runtime), runtime);
    }
    public DHTMLPane(Registry scriptletRegistry, Object envObj) {
        super();

        // set default appearance/behavior (clients can change)
        setBackground(Color.white);
        //setContentType("text/html");
        setEditable(false);

        editorKit = new DHTMLEditorKit(scriptletRegistry, envObj, this);
        setEditorKitForContentType("text/html", editorKit);

        addHyperlinkListener(new HyperlinkHandler());
        addMouseListener(new ViewSourceMouseListener());
        
        setFocusTraversalPolicy(new HTMLFocusTraversalPolicy(this));
    }
    private static URL blankURL = null;
    
    public void setBlankPage() {
        try {
            if (blankURL == null)
                blankURL = DHTMLPane.class.getResource("blank.html");
            super.setPage(blankURL);
        } catch (Exception e) {}
    }

    public void setPage(String url) throws IOException {
        setBlankPage();  // why do I have to do this?
        super.setPage(url);
    }

    public void setPage(URL page) throws IOException {
        setBlankPage();  // why do I have to do this?
        super.setPage(page);
    }
    
    public void reload() throws IOException {
        URL u = getPage();
        if (u != null) {
            setBlankPage();
            super.setPage(u);
        }
    }
    
    public Object runScriptlet(
        String command,
        int type,
        Element sourceElement)
        throws ScriptletException, IllegalArgumentException {

        Document doc = getDocument();
        if (!(doc instanceof DHTMLDocument))
            throw new IllegalArgumentException(
                "Not currently displaying a DHTML document");

        if (sourceElement != null && doc != sourceElement.getDocument())
            throw new IllegalArgumentException(
                "The specified source element not contained in currently " +
                "displayed document");

        return DHTMLEditorKit.runScriptlet(
            command,
            type,
            (DHTMLDocument) doc,
            sourceElement);
    }

    private void showHTMLSource() {
        JTextPane jtp = new JTextPane();
        jtp.setText(getText());
        JScrollPane jsp = new JScrollPane(jtp);
        jsp.setPreferredSize(new Dimension(500, 500));
        JOptionPane.showMessageDialog(null, jsp, "View Source", JOptionPane.INFORMATION_MESSAGE);
    }

    private boolean shouldIgnoreMouseListener(Class c) {
        return (c == javax.swing.text.html.HTMLEditorKit.LinkController.class);
    }

    public synchronized void addMouseListener(MouseListener l) {
        if (l != null && !shouldIgnoreMouseListener(l.getClass()))
            super.addMouseListener(l);
    }

    public synchronized void addMouseMotionListener(MouseMotionListener l) {
        if (l != null && !shouldIgnoreMouseListener(l.getClass()))
            super.addMouseMotionListener(l);
    }


    private class HyperlinkHandler implements HyperlinkListener {

        public void hyperlinkUpdate(HyperlinkEvent e) {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED &&
                !isMetaLink(e.getSourceElement())) {    
                String s = e.getDescription();
                if (s != null && s.trim().startsWith("scriptlet:")) {
                    try {
                        // dispatch dhtml actions
                        runScriptlet(
                            s.substring(10),
                            IScriptletEvent.MOUSE_CLICK,
                            e.getSourceElement());
                    } catch (ScriptletException se) {
                        se.printStackTrace();
                    }
                } else {
                    if (e instanceof HTMLFrameHyperlinkEvent) {
                        HTMLFrameHyperlinkEvent evt =
                            (HTMLFrameHyperlinkEvent) e;
                        HTMLDocument doc = (HTMLDocument) getDocument();
                        doc.processHTMLFrameHyperlinkEvent(evt);
                    } else {
                        try {
                            setPage(e.getURL());
                        } catch (Throwable t) {
                            t.printStackTrace();
                        }
                    }
                }
            }
        }

        private boolean isMetaLink(Element element) {
            AttributeSet attrs = element.getAttributes();
            Object elemName = attrs.getAttribute(StyleConstants.NameAttribute);
            return (elemName == HTML.Tag.LINK);
        }
    }
    
    private class ViewSourceMouseListener implements MouseListener {
        private static final int mask =
            MouseEvent.ALT_DOWN_MASK
                | MouseEvent.CTRL_DOWN_MASK
                | MouseEvent.SHIFT_DOWN_MASK;
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON3 &&
                (e.getModifiersEx() & mask) == mask)
                showHTMLSource();
        }
        public void mousePressed(MouseEvent e) {
        }
        public void mouseReleased(MouseEvent e) {
        }
        public void mouseEntered(MouseEvent e) {
        }
        public void mouseExited(MouseEvent e) {
        }        
    }


}
