package gov.va.med.hds.cd.spectrum.dhtml;

import java.awt.Graphics;
import java.awt.Shape;
import java.net.URL;

import javax.swing.SwingUtilities;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.html.BlockView;

import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Replace;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ScriptletEnvironment;

public class DeferredDivView extends BlockView {

    public static final String DEFERRED_SRC_ATTRIBUTE = "deferredsrc";

    private static int DEFERRED = 0;
    private static int LOADING = 1;
    private static int READY = 2;

    private volatile int state = DEFERRED;

    public DeferredDivView(Element elem, int axis) {
        super(elem, axis);
    }

    public void paint(Graphics g, Shape allocation) {
        maybeStartLoad();
        super.paint(g, allocation);
    }
    private final void maybeStartLoad() {
        if (state == DEFERRED) {
            boolean needToLoad = false;
            synchronized (this) {
                if (state == DEFERRED) {
                    state = LOADING;
                    needToLoad = true;
                }
            }
            if (needToLoad) {
                LoadingThread t = new LoadingThread();
                t.start();
            }
        }
    }

    private String ERROR_MSG =
        "<font color='red'><b>Error encountered</b></font>";
    
    private class LoadingThread extends Thread {
        public void run() {
            Object parent = getParent();
            Element elem = getElement();
            DHTMLDocument document = (DHTMLDocument) elem.getDocument();

            try {
                AttributeSet attr = elem.getAttributes();
                String src =
                    (String) attr.getAttribute(DEFERRED_SRC_ATTRIBUTE);
                URL base = document.getBase();
                String content = Replace.getHTMLBody(base, src);

                Object currentParent = getParent();
                if (parent == currentParent)
                    SwingUtilities.invokeLater
                        (new ReplacingTask(document, elem, content, false));
                else
                    System.out.println("parent changed.");                
            } catch (Exception e) {
                System.out.println("Got Exception "+e);
                e.printStackTrace();
                SwingUtilities.invokeLater
                    (new ReplacingTask(document, elem, ERROR_MSG, true));
            }
        }
    }
    
    private class ReplacingTask implements Runnable {
        private DHTMLDocument document;
        private Element elem;
        private String content;
        private boolean append;
        
        public ReplacingTask(DHTMLDocument document, Element elem, 
                             String content, boolean append) {
            this.document = document;
            this.elem = elem;
            this.content = content;
            this.append = append;    
        }
        public void run() {
            try {
                if (append)
                    document.insertBeforeEnd(elem, content);
                else
                    document.setInnerHTML(elem, content);
                    
                ScriptletEnvironment env =
                    (ScriptletEnvironment) document.getProperty(
                        DHTMLEditorKit.SCRIPTLET_ENVIRONMENT_PROPERTY);
                env.fireSubdocumentLoaded(elem);
                    
            } catch (Exception e) { }
        }
    }

}
