package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.text.AttributeSet;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.ElementIterator;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.CSS;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;

import gov.va.med.hds.cd.spectrum.dhtml.DHTMLDocument;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLEditorKit;

public class ExpandCollapse implements IScriptlet {

    public static final String TAG_NAME = "collapser";
    private int uniqueID = 0;

    private DHTMLDocument document;

    public void init(IScriptletEvent event) {
        initializeImageURLs();

        document = event.getDocument();

        HTML.Tag collapserTag = new HTML.UnknownTag(TAG_NAME);
        ArrayList collapserElementList = new ArrayList();
        ElementIterator iterator = new ElementIterator(document);
        Element elem;
        Object tagname;
        while ((elem = iterator.next()) != null) {
            tagname =
                elem.getAttributes().getAttribute(
                    StyleConstants.NameAttribute);
            if (collapserTag.equals(tagname))
                collapserElementList.add(elem);
        }

        Iterator elements = collapserElementList.iterator();
        
        while (elements.hasNext()) {
            elem = (Element) elements.next();
            String expandID =
                String.valueOf(elem.getAttributes().getAttribute("forid"));
            String imageID = "expansionImage_" + uniqueID++;
            String replacement =
                findAndReplace(EXPANDER_TEMPLATE, "expandID", expandID);
            replacement = findAndReplace(replacement, "imageID", imageID);
            
            Element target = document.getElement(expandID);
            boolean collapsed = isCollapsed(target);
            replacement = findAndReplace
                (replacement, "IMG_URL", collapsed ? PLUS_URL : MINUS_URL);
            try {
                document.setOuterHTML(elem, replacement);
                new ImageUpdater(imageID, expandID);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void expandCollapse(String id) {
        if (id == null || id.length() == 0)
            return;

        Element e = document.getElement(id);
        if (e == null)
            return;

        boolean isCollapsed = isCollapsed(e);
        Object o = (isCollapsed ? "inline" : "none");
        DHTMLEditorKit.setAttribute(e, CSS.Attribute.DISPLAY, o);
        fireExpansionEvent(document, e, id, isCollapsed);
    }

    private class ImageUpdater implements ExpansionListener {
        private String imageID;
        private String expansionID;
        private Element elem = null;
        public ImageUpdater(String imageID, String expansionID) {
            this.imageID = imageID;
            this.expansionID = expansionID;

            addExpansionListener(this, expansionID);
        }

        public void elementChanged(ExpansionEvent e) {
            if (!expansionID.equals(e.getID()))
                return;
            if (elem == null)
                elem = e.getDocument().getElement(imageID);
            if (elem != null)
                DHTMLEditorKit.setAttribute(
                    elem,
                    HTML.Attribute.SRC,
                    (e.isExpanded() ? MINUS_URL : PLUS_URL));
        }
    }

    private boolean isCollapsed(Element elem) {
        return isDisplayNone(elem);
    }
    
    public static boolean isDisplayNone(Element elem) {
        if (elem == null) return false;
        AttributeSet attrs = elem.getAttributes();
        if (attrs == null) return false;
        
        Object o = null;
        boolean isSetLocally = attrs.isDefined(CSS.Attribute.DISPLAY);
        if (isSetLocally) {
            o = attrs.getAttribute(CSS.Attribute.DISPLAY);
        } else {
            Object kind =
                elem.getAttributes().getAttribute(
                    StyleConstants.NameAttribute);
            if (kind instanceof HTML.Tag) {
                Document doc = elem.getDocument();
                if (doc instanceof HTMLDocument)
                    o = ((HTMLDocument) doc).getStyleSheet().getRule(
                        (HTML.Tag) kind,
                        elem).getAttribute(
                        CSS.Attribute.DISPLAY);
            }
        }
        return (o != null && "none".equalsIgnoreCase(String.valueOf(o)));
    }

    private String findAndReplace(String text, String find, String replace) {
        if (text == null)
            return null;
        else
            return text.replaceAll("\\Q" + find + "\\E", replace);
    }

    private void initializeImageURLs() {
        if (PLUS_URL == null) {
            URL u = ExpandCollapse.class.getResource("BtnPlus.png");
            if (u != null)
                PLUS_URL = String.valueOf(u);
        }
        if (MINUS_URL == null) {
            URL u = ExpandCollapse.class.getResource("BtnMinus.png");
            if (u != null)
                MINUS_URL = String.valueOf(u);
        }
    }

    private String PLUS_URL = null;
    private String MINUS_URL = null;
    private String EXPANDER_TEMPLATE =
        "<a border=0 href=\"scriptlet:expandCollapse('expandID')\">"
            + "<img id=\"imageID\" align=\"absmiddle\" border=0 "
            + "hspace=1 vspace=1 width=\"14\" height=\"14\" src=\"IMG_URL\">"
            + "</a>";

    private class ExpansionEvent {
        private HTMLDocument document;
        private Element element;
        private String id;
        private boolean isExpanded;

        public ExpansionEvent(
            HTMLDocument doc,
            Element e,
            String i,
            boolean expanded) {
            document = doc;
            element = e;
            id = i;
            isExpanded = expanded;
        }
        public HTMLDocument getDocument() {
            return document;
        }
        public Element getElement() {
            return element;
        }
        public String getID() {
            return id;
        }
        public boolean isExpanded() {
            return isExpanded;
        }
    }

    private interface ExpansionListener {
        public void elementChanged(ExpansionEvent e);
    }

    private Map EXPANSION_LISTENERS = new HashMap();

    private void addExpansionListener(
        ExpansionListener l,
        String expansionID) {
        Set listeners = (Set) EXPANSION_LISTENERS.get(expansionID);
        if (listeners == null) {
            listeners = new HashSet();
            EXPANSION_LISTENERS.put(expansionID, listeners);
        }
        listeners.add(l);
    }
    private void removeExpansionListener(
        ExpansionListener l,
        String expansionID) {
        Set listeners = (Set) EXPANSION_LISTENERS.get(expansionID);
        if (listeners != null)
            listeners.remove(l);
    }
    private void fireExpansionEvent(
        HTMLDocument doc,
        Element e,
        String id,
        boolean isExpanded) {
        Set listeners = (Set) EXPANSION_LISTENERS.get(id);
        if (listeners == null || listeners.isEmpty())
            return;
        ExpansionEvent event = new ExpansionEvent(doc, e, id, isExpanded);
        Iterator i = listeners.iterator();
        while (i.hasNext())
             ((ExpansionListener) i.next()).elementChanged(event);
    }

}
