package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;

class Interpreter {

    private SymbolTable symbolTable;

    public Interpreter(SymbolTable symbolTable) {
        this.symbolTable = symbolTable;
    }

    public Object exec(String command, IScriptletEvent event)
        throws ScriptletException {
        Object result = null;
        try {
            StreamTokenizer tok =
                new StreamTokenizer(new StringReader(command));
            tok.ordinaryChar('/'); // don't use absurd comment indicator

            int token;
            while ((token = tok.nextToken()) != StreamTokenizer.TT_EOF) {
                // we're looking for a word, naming a method.
                if (token != StreamTokenizer.TT_WORD)
                    continue;
                String methodName = tok.sval;
                ArrayList args = new ArrayList();

                // if the next token is a '(' character, build the parameter list.
                token = tok.nextToken();
                if (token != '(')
                    tok.pushBack();
                else {
                    ARG : while (token != ')') {
                        switch (token = tok.nextToken()) {
                            case '"' :
                            case '\'' : // quoted string
                                args.add(new String(tok.sval));
                                break;

                            case StreamTokenizer.TT_NUMBER :
                                args.add(new Double(tok.nval));
                                break;

                            case StreamTokenizer.TT_WORD :
                                if ("true".equals(tok.sval))
                                    args.add(Boolean.TRUE);
                                else if ("false".equals(tok.sval))
                                    args.add(Boolean.FALSE);
                                else
                                    throw new ScriptletException(
                                        "Syntax error in '" + command + "'");
                                break;
                            
                            case ')':
                                // this scriptlet takes no arguments
                                break ARG;

                            default :
                                throw new ScriptletException(
                                    "Syntax error in '" + command + "'");
                        }
                        token = tok.nextToken();
                        if (token != ',' && token != ')')
                            throw new ScriptletException(
                                "Syntax error in '" + command + "'");
                    }
                }

                result = exec(methodName, event, args.toArray());
                if (result instanceof Throwable)
                    throw new ScriptletException(
                        "Exception '"
                            + result
                            + "' thrown by method "
                            + methodName,
                        (Throwable) result);
            }
        } catch (IOException e) {
        }
        return result;
    }

    public Object exec(
        String methodName,
        IScriptletEvent event,
        Object[] args)
        throws ScriptletException {

        ScriptletAction a = symbolTable.getAction(methodName);

        if (a == null)
            throw new ScriptletException(
                "Syntax error: No method named '" + methodName + "'");

        return a.invoke(event, args);
    }

}
