package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.util.HashMap;
import java.util.Map;


public class Registry {

    private Map scriptletClassLoaderMap;
    private Map scriptletClassNameMap;
    private Map scriptletClassMap;

    /** Create a new. empty registry.
     */
    public Registry() {
        scriptletClassLoaderMap = new HashMap();
        scriptletClassNameMap = new HashMap();
        scriptletClassMap = new HashMap();
    }

    /** Register a scriplet.
     * 
     * @param id The id of the scriptlet. (This is the value that people would
     * reference in the &lt;meta&gt; tag of an HTML document.)
     * @param className The name of the java class which contains the scriptlet
     * behaviors.  The named class must implement {@link IScriptlet}.
     * @throws IllegalArgumentException if a scriptlet has already been
     * registered with the given id.
     */
    public void registerScriptlet(String id, String className)
        throws IllegalArgumentException {
        registerScriptlet(id, className, getClass().getClassLoader());
    }

    /** Register a scriplet.
     * 
     * @param id The id of the scriptlet. (This is the value that people would
     * reference in the &lt;meta&gt; tag of an HTML document.)
     * @param className The name of the java class which contains the scriptlet
     * behaviors.  The named class must implement {@link IScriptlet}.
     * @param classLoader the classloader to use when loading the class
     * @throws IllegalArgumentException if a scriptlet has already been
     * registered with the given id.
     */
    public void registerScriptlet(String id, String className, ClassLoader classLoader)
        throws IllegalArgumentException {

        String oldClassName = (String) scriptletClassMap.get(id);
        
        if (oldClassName == null) {
            scriptletClassNameMap.put(id, className);
            scriptletClassLoaderMap.put(id, classLoader);
        }
            
        else if (!oldClassName.equals(className)) 
            throw new IllegalArgumentException(
                "ID already registered: '" + id + "'");
    }
    
    /** Get a scriptlet with the given id.
     * 
     * @return a {@link ScriptletClass} object with the given id, or
     * <code>null</code> if an appropriate object could not be created, either
     * because: <ul>
     * <li>No scriptlet has been registered with the given id, or
     * <li>Fatal errors were encountered when attempting to create the
     * registered scriptlet object.
     * </ul> */
    ScriptletClass getScriptletClass(String id) {

        ScriptletClass result = (ScriptletClass) scriptletClassMap.get(id);

        if (result == null) {
            try {
                String className = (String) scriptletClassNameMap.get(id);
                ClassLoader classLoader = (ClassLoader) scriptletClassLoaderMap.get(id);
                if (className != null && classLoader != null)
                    result = new ScriptletClass(id, className, classLoader);
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (result != null)
                // on success, store the object in our class map.
                scriptletClassMap.put(id, result);
            else
                // on failure, delete this id from our id map so we don't
                // attempt to create one in the future. 
                scriptletClassNameMap.remove(id);
        }

        return result;
    }

}
