package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;

public class RuntimeScriptletRegistry extends Registry {

    /** Maps IRuntimeContext objects to their appropriate registries */
    private static Map REGISTRIES = Collections.synchronizedMap(new HashMap());

    /** Get the registry for a given runtime context */
    public static RuntimeScriptletRegistry getRegistry(IRuntimeContext runtime) {
        RuntimeScriptletRegistry result = (RuntimeScriptletRegistry) REGISTRIES.get(runtime);
        if (result == null) {
            result = new RuntimeScriptletRegistry(runtime);
            REGISTRIES.put(runtime, result);
        }
        return result;
    }


    private IRuntimeContext runtime;
   

    private RuntimeScriptletRegistry(IRuntimeContext runtime) {
        this.runtime = runtime;
        init();
    }

    private void init() {
        IPluginRegistry pluginRegistry = runtime.getPluginRegistry();
        IConfigurationElement[] classes =
            pluginRegistry.getConfigurationElementsFor(
                IScriptlet.SCRIPTLET_EXTENSION_POINT_ID);

        for (int i = 0; i < classes.length; i++) {
            String id = classes[i].getAttribute(IScriptlet.ID_NAME_ATTRIBUTE);
            String className =
                classes[i].getAttribute(IScriptlet.CLASS_NAME_ATTRIBUTE);
            if (className == null || className.length() == 0)
                className = id;
            ClassLoader classLoader = classes[i].getDeclaringExtension()
                .getDeclaringPluginDescriptor().getPluginClassLoader();
                
            try {
                super.registerScriptlet(id, className, classLoader);
            } catch (Exception e) {
                // log an error
            }          
        }
    }
}
