package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

class ScriptletClass {

    private String id;
    private Class scriptletClass;
    private List methods;

    ScriptletClass(String id, String className, ClassLoader classLoader)
        throws ClassNotFoundException, IllegalArgumentException {

        // could throw ClassNotFoundException
        this.scriptletClass = Class.forName(className, true, classLoader);
        this.id = id;

        // ensure that the named class implements IScriptlet.
        if (!IScriptlet.class.isAssignableFrom(scriptletClass))
            throw new IllegalArgumentException(
                "Not a scriptlet class: '" + className + "'");

        // create an appropriate list of ScriptletMethod objects.
        this.methods = new ArrayList();
        Method[] methods = scriptletClass.getDeclaredMethods();
        for (int i = 0; i < methods.length; i++)
            maybeAddMethod(ScriptletMethod.get(this, methods[i]));
    }

    String getID() {
        return id;
    }

    private void maybeAddMethod(ScriptletMethod m) {
        if (m != null)
            methods.add(m);
    }



    /** import the actions for this scriptlet into the given symbol table.
     */
    IScriptlet doImport(SymbolTable symbolTable, IScriptletEvent loadEvent) {

        // instantiate the scriptlet class, and initialize it with
        // the load event.
        IScriptlet instance = null;
        try {
            instance = (IScriptlet) scriptletClass.newInstance();
            instance.init(loadEvent);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }

        // create actions that implement each of our scriptlet methods.
        Iterator i = methods.iterator();
        while (i.hasNext()) {
            ScriptletMethod m = (ScriptletMethod) i.next();
            symbolTable.addAction(
                m.getID(),
                m.getName(),
                m.getActionFor(instance));
        }
        
        return instance;
    }
}