package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.ElementIterator;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.HTML;

import gov.va.med.hds.cd.spectrum.dhtml.DHTMLDocument;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;

public class ScriptletEnvironment {

    static final String META_TAG_NAME = "scriptlet";

    private Registry registry;
    private DHTMLDocument document;
    private DHTMLPane pane;
    private Object environment;
    private SymbolTable symbolTable;
    private Interpreter interpreter;

    public ScriptletEnvironment(
        Registry registry,
        DHTMLDocument document,
        DHTMLPane pane,
        Object envObject) {

        this.registry = registry;
        this.document = document;
        this.pane = pane;
        this.environment = envObject;
        
        ScriptletEvent event =
            buildEvent(IScriptletEvent.DOCUMENT_LOADED, null);
        this.symbolTable = new SymbolTable(registry, event);
        init();
        
        this.interpreter = new Interpreter(symbolTable);
    }

    public Object exec(String command, int type, Element elem)
        throws ScriptletException {
        return interpreter.exec(command, buildEvent(type, elem));
    }

    private ScriptletEvent buildEvent(int type, Element elem) {
        return new ScriptletEvent(type, elem, environment, document, pane);
    }

    private void init() {
        ElementIterator iterator = new ElementIterator(document);
        Element elem;
        while ((elem = iterator.next()) != null) {
            if (HTML.Tag.META
                == elem.getAttributes().getAttribute(
                    StyleConstants.NameAttribute))
                parseMetaTag(elem);
        }
        symbolTable.importComplete();
    }

    private void parseMetaTag(Element elem) {
        AttributeSet attrs = elem.getAttributes();
        String name = (String) attrs.getAttribute(HTML.Attribute.NAME);
        if (!META_TAG_NAME.equals(name))
            return;

        String value = (String) attrs.getAttribute(HTML.Attribute.CONTENT);
        if (value == null || value.length() == 0)
            return;

        symbolTable.doImport(value);
    }
    
    public void fireSubdocumentLoaded(Element elementToReplace) {
        symbolTable.fireSubdocumentLoaded(
            buildEvent(IScriptletEvent.SUBDOCUMENT_LOADED, elementToReplace));
    }

}
