package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

class ScriptletMethod {

    private ScriptletClass parent;
    private Method method;
    private String name;
    private String id;
    private boolean useEvent;

    private ScriptletMethod(
        ScriptletClass parent,
        Method method,
        boolean useEvent) {

        this.parent = parent;
        this.method = method;
        this.name = method.getName();
        this.id = parent.getID() + "." + name;
        this.useEvent = useEvent;
    }

    public static ScriptletMethod get(ScriptletClass parent, Method method) {

        // only expose public methods.
        if (!Modifier.isPublic(method.getModifiers()))
            return null;
        
        // don't expose the "init" method, which is required by IScriptlet:
        //     public void init(IScriptletEvent event);
        if ("init".equals(method.getName()))
            return null;

        // see if the first parameter is an IScriptletEvent 
        Class[] params = method.getParameterTypes();
        boolean shouldUseEvent =
            (params.length > 0 && params[0] == IScriptletEvent.class);

        // only expose methods whose remaining parameters are Strings/numbers
        for (int i = (shouldUseEvent ? 1 : 0); i < params.length; i++)
            if (LEGAL_ARG_TYPES.indexOf(params[i]) == -1)
                return null;

        return new ScriptletMethod(parent, method, shouldUseEvent);
    }

    private static final List LEGAL_ARG_TYPES =
        Collections.unmodifiableList(
            Arrays.asList(
                new Object[] {
                    String.class,
                    Boolean.TYPE,
                    Double.TYPE,
                    Float.TYPE,
                    Long.TYPE,
                    Integer.TYPE,
                    Short.TYPE }));

    public String getID() {
        return id;
    }
    public String getName() {
        return name;
    }
    public ScriptletAction getActionFor(IScriptlet instance) {
        return new ScriptletAction(instance, method, useEvent);
    }

}
