package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

class SymbolTable {

    Registry registry;
    Map scriptletActions;
    List scriptletInstances;
    Set attemptedImports;
    IScriptletEvent loadEvent;
    boolean initialImportComplete;

    SymbolTable(Registry registry, IScriptletEvent docLoadEvent) {
        this.registry = registry;
        scriptletActions = new HashMap();
        scriptletInstances = new LinkedList();
        attemptedImports = new HashSet();
        loadEvent = docLoadEvent;
        initialImportComplete = false;
    }

    void doImport(String id) {
        if (attemptedImports.contains(id))
            // if we've already attempted to import this scriptlet, don't
            // try again.
            return;
        
        attemptedImports.add(id);      

        ScriptletClass scriptletClass = registry.getScriptletClass(id);
        if (scriptletClass == null) {
            System.out.println("Could not import scriptlet: '" + id + "'");
            return;
        }

        IScriptlet instance = scriptletClass.doImport(this, loadEvent);
        if (instance != null)
            scriptletInstances.add(instance);
    }
    
    void importComplete() {
        initialImportComplete = true;
    }

    void addAction(String id, String name, ScriptletAction a) {
        // store the action under its fully qualified name.
        addAction(id, a);
        
        // on-the-fly imports should only be stored under their fully
        // qualified names, so we'll stop here if the initial import
        // is complete.
        if (initialImportComplete)
            return;
        
        // store the action under its simple name.    
        addAction(name, a);

        // store the action under its partially qualified name.
        int pos = id.lastIndexOf('.');
        if (pos > 0) pos = id.lastIndexOf('.', pos-1);
        if (pos != -1) scriptletActions.put(id.substring(pos+1), a);
    }

    void addAction(String name, ScriptletAction a) {
        ScriptletAction aliasedAction =
            (ScriptletAction) scriptletActions.get(name);
        if (aliasedAction == null)
            // store the action by this name, if it is unambiguous.
            scriptletActions.put(name, a);
        else if (!(aliasedAction instanceof AmbiguousAction) &&
                 !aliasedAction.equals(a))
            // if the name is ambiguous, store an ambiguous action in its place.
            scriptletActions.put(name, new AmbiguousAction(name));
    }

    ScriptletAction getAction(String s) {
        // look up the named scriptlet action in our list of imported actions.
        ScriptletAction result = (ScriptletAction) scriptletActions.get(s);
        
        // if we didn't find the named action, see if we can identify the
        // owning scriptlet, and attempt to import it on the fly
        if (result == null) {
            int pos = s.lastIndexOf('.');
            if (pos != -1) {
                String id = s.substring(0, pos);                
                doImport(id);
                result = (ScriptletAction) scriptletActions.get(s);
            }
        }
        return result;
    }

    private class AmbiguousAction extends ScriptletAction {
        private String name;
        public AmbiguousAction(String name) {
            super(null, null, false);
            this.name = name;
        }
        public Object invoke(IScriptletEvent event, Object[] args) {
            return new Exception("Ambiguous reference: '" + name + "'");
        }
    }
    
    public void fireSubdocumentLoaded(IScriptletEvent event) {
        Set handledScriptlets = new HashSet();
        Iterator i = scriptletInstances.iterator();
        while (i.hasNext()) {
            IScriptlet scriptlet = (IScriptlet) i.next();
            scriptlet.init(event);
        }
    }

}
