package gov.va.med.hds.cd.spectrum.dhtml.scriptlet;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.io.IOException;
import java.net.URL;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.spectrum.dhtml.DHTMLPane;

public class Window implements IScriptlet {
    
    public static final String PARENT_CLIENT_PROPERTY = "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window:parent";

    public void init(IScriptletEvent event) {
    }

    public void reload(IScriptletEvent event) throws IOException {
        event.getPane().reload();
    }

    public void openWindow(
        IScriptletEvent event,
        String url,
        String title,
        double width,
        double height,
        boolean modal)
        throws IOException {

        openWindow(
            (IRuntimeContext) event.getEnvironment(),
            event.getPane(),
            new URL(event.getDocument().getBase(), url),
            title,
            width,
            height,
            modal);
        }
        
    public static void openWindow(
            IRuntimeContext runtime,
            JComponent parent,
            URL url,
            String title,
            double width,
            double height,
            boolean modal)
            throws IOException {

        DHTMLPane openedPane = new DHTMLPane(runtime);
        openedPane.putClientProperty(PARENT_CLIENT_PROPERTY, parent);
        openedPane.setPage(url);

        title = unescapeEntities(title);

        Container c = parent;
        while (c != null
            && !(c instanceof Frame)
            && !(c instanceof Dialog)
            && c.getParent() != null)
            c = c.getParent();

        JDialog dialog = null;
        if (c instanceof Frame)
            dialog = new JDialog((Frame) c, title, true);
        else if (c instanceof Dialog)
            dialog = new JDialog((Dialog) c, title, true);
        else
            dialog = new JDialog((Frame) null, title, true);

        c = parent;
        if (c != null && c.getParent() instanceof JViewport)
            // if the parent is enclosed in a scroll pane, use the
            // dimensions of the scroll pane for relative sizing.
            c = c.getParent();

        dialog.getContentPane().add(new JScrollPane(openedPane));

        dialog.setSize(
            resolveDimension(width, c == null ? -1 : c.getWidth()),
            resolveDimension(height, c == null ? -1 : c.getHeight()));

        if (c != null)
            dialog.setLocationRelativeTo(c);

        // this will block if "modal" is true.
        dialog.show();
    }

    private static int resolveDimension(double size, int parentSize) {
        if (size > 1)
            return (int) size;
        else if (size > 0 && parentSize > 1)
            return (int) (size * parentSize);
        else
            return 300;
    }
    


    private static String unescapeEntities(String value) {
        value = findAndReplace(value, "&lt;",   "<");
        value = findAndReplace(value, "&gt;",   ">");
        value = findAndReplace(value, "&quot;", "\"");
        value = findAndReplace(value, "&apos;", "'");
        value = findAndReplace(value, "&amp;",  "&");
        return value;
    }
    
    private static String findAndReplace(String s, String find, String replace) {
        if (s.indexOf(find) == -1) return s;
        return s.replaceAll("\\Q" + find + "\\E", replace);
    }
    

    public void closeWindow(IScriptletEvent event) {     
        Container c = event.getPane();
        while (c != null) {
            if (c instanceof Dialog) {
                Dialog d = (Dialog) c;
                if (d.isModal()) d.dispose();
                return;
            }            
            
            c = c.getParent();
        }
    }
}
