package gov.va.med.hds.cd.spectrum.linkpanel;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.event.EventListenerList;

/**
 * Selection model used by the HTMLSelectionPanel.
 */
public class HTMLSelectionModel implements IHTMLSelectionModel {
	private EventListenerList eventListenerList = new EventListenerList();
	private int itemCount = 0;
	private Map objectMap = new HashMap();
	private List printList = new ArrayList();
	private boolean changing = false;
	
	public class ObjectID {
		private int id = 0;
		
		private ObjectID() {
		}
		
		public ObjectID(int id) {
			this.id = id;
		}
		
		public boolean equals(Object obj) {
			return (obj instanceof ObjectID)
				&& (id == ((ObjectID) obj).id);
		}

		public int hashCode() {
			return id;
		}
		
		/**
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			return Integer.toString(id);
		}

	}
	
	public class HTMLObjectItem {
		private ObjectID objectID= null;
		private Object item = null;
		private String descr = null;
		private String styleText = null;

		/**
		 * Constructor for ObjectItem.
		 */
		public HTMLObjectItem(int id) {
			super();
			ObjectID newID = new ObjectID(id);
			this.objectID = newID;
		}
		
		/**
		 * Returns the descr.
		 * @return String
		 */
		public String getDescr() {
			return descr;
		}

		/**
		 * Returns the styleText.
		 * @return String
		 */
		public String getStyleText() {
			return styleText;
		}

		/**
		 * Returns the item.
		 * @return Object
		 */
		public Object getItem() {
			return item;
		}

		/**
		 * Sets the descr.
		 * @param descr The descr to set
		 */
		public void setDescr(String descr) {
			this.descr = descr;
		}

		/**
		 * Sets the styleText.
		 * @param styleText The styleText to set
		 */
		public void setStyleText(String styleText) {
			this.styleText = styleText;
		}

		/**
		 * Sets the item.
		 * @param item The item to set
		 */
		public void setItem(Object item) {
			this.item = item;
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			String result = new String();
//			result = 	"<font size=\"+1\">"+
//						"<a href=\"http://." + 
//						objectID.toString() + 
//						".\" " + 
//						styleText +
//						">";
			result = 	"<a href=\"http://." + 
						objectID.toString() + 
						".\" " + 
						styleText +
						">";
			if ((descr != null) && (descr.length()>0)) {
				result = result + HTML.encode(descr);
			}
			else {
				if (item != null) {
					result = result + HTML.encode(item.toString());
				}
			}
//			result = result + "</a></font>";
			result = result + "</a>";
			return result;
		}
		
		/**
		 * Returns the objectID.
		 * @return ObjectID
		 */
		public ObjectID getObjectID() {
			return objectID;
		}

	}

	/**
	 * Creates a sort of HTML document, except it is used to display any HTML
	 * handled by the JEditPane
	 * and provide an implementation to return an object when a hyperlink is
	 * selected.  These special hyperlinks are implememted by this model, and 
	 * return a reference to the object when the hyperlink is selected.  Thus, 
	 * to setup these special hyperlinks, pass in any HTML data, then where you
	 * want a special hyperlink, call one of the addSelectableObject methods to
	 * add the object and insert the hyperlink in one call.  Then add any more
	 * HTML text, other objects, and so on.
	 * 
	 * Events
	 * 
	 * Events are generated when the user selects an item, or when the document is 
	 * updated (this includes the clear command.)
	 */
	public HTMLSelectionModel() {
		super();
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#addHTMLText(String)
	 */
	public void addHTMLText(String in) {
		if ((in != null) && (in.length() > 0)) {
			printList.add(in);
			fireTextChanged();
		}
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#addSelectableObject(Object)
	 */
	public int addSelectableObject(Object o) {
		return addSelectableObject(o, o.toString(),"");
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#addSelectableObject(Object, String, String)
	 */
	public int addSelectableObject(Object o,String description,String style) {
		HTMLObjectItem oi = new HTMLObjectItem(++itemCount);
		oi.setDescr(description);
		oi.setStyleText(style);
		oi.setItem(o);
		printList.add(oi);
		objectMap.put(oi.getObjectID(), oi);
		fireTextChanged();
		return oi.getObjectID().hashCode();
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#addEventListener(ISelectionEventListener)
	 */
	public void addEventListener(ISelectionEventListener el) {
		eventListenerList.add(ISelectionEventListener.class, el);
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#removeEventListener(ISelectionEventListener)
	 */
	public void removeEventListener(ISelectionEventListener el) {
		eventListenerList.remove(ISelectionEventListener.class, el);
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#clear()
	 */
	public void clear() {
		printList.clear();
		objectMap.clear();
		fireTextChanged();
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#getHTML()
	 */
	public String getHTML() {
		String result = new String();
		Iterator it = printList.iterator();
		while (it.hasNext()) {
			result = result + it.next().toString();
		}
		return result;
	}
	
	private static final String FRONT_BOUND_FONT_STR = "font-family:";
	private static final String REAR_BOUND_FONT_STR = "background-color:";
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#getHTML(java.awt.Font)
	 */
	public String getHTML(Font newFont) {
		
		String original = getHTML();
		int frontIdx = original.indexOf(FRONT_BOUND_FONT_STR);
		int rearIdx = original.indexOf(REAR_BOUND_FONT_STR);
		if(frontIdx == -1 || frontIdx == 1 || rearIdx == -1)
			return original;
		
		String newFontStr = "font-family:"+newFont.getFontName()+";font-size:"+newFont.getSize()+";"+"background-color:";
		String toReplace =  original.substring(frontIdx-1,rearIdx+REAR_BOUND_FONT_STR.length());
		return original.replaceAll(toReplace,newFontStr);
	}
	
	/**
	 * Method getItem.
	 * @param id
	 * @return HTMLObjectItem
	 */
	private HTMLObjectItem getItem(int id) {
		ObjectID oid = new ObjectID(id);
		return (HTMLObjectItem) objectMap.get(oid);
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#setDescription(String, int)
	 */
	public void setDescription(String descr, int id) {
		HTMLObjectItem item = getItem(id);
		item.setDescr(descr);
		fireTextChanged();
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#setStyle(String, int)
	 */
	public void setStyle(String style, int id) {
		HTMLObjectItem item = getItem(id);
		item.setStyleText(style);
		fireTextChanged();
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#setObject(Object, int)
	 */
	public void setObject(Object object, int id) {
		HTMLObjectItem item = getItem(id);
		item.setItem(object);
		fireTextChanged();
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#getDescription(int)
	 */
	public String getDescription(int id) {
		HTMLObjectItem item = getItem(id);
		return item.getDescr();
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#getStyle(int)
	 */
	public String getStyle(int id) {
		HTMLObjectItem item = getItem(id);
		return item.getStyleText();
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#getObject(int)
	 */
	public Object getObject(int id) {
		HTMLObjectItem item = getItem(id);
		return item.getItem();
	}

	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#fireObjectSelected(int)
	 */
	public void fireObjectSelected(int objectID) {
		HTMLObjectItem item = getItem(objectID);
		if (item != null) {
			notifyListeners(new ActionEvent(item.getItem(),objectID,ITEM_SELECTED));
		}
	}

	/**
	 * Method fireTextChanged.
	 */
	private void fireTextChanged() {
		if (!changing) {
			notifyListeners(new ActionEvent(this, -1, TEXT_CHANGED));
		}
	}
	
	/**
	 * Method notifyListeners.
	 * @param ae
	 */
	private void notifyListeners(ActionEvent ae) {
		if (eventListenerList.getListenerCount() > 0) {
			ISelectionEventListener[] listeners = (ISelectionEventListener[])eventListenerList.getListeners(ISelectionEventListener.class);
			for (int i=0; i<listeners.length; i++) {
				listeners[i].itemSelected(ae);
			}
		}
	}
	
	/**
	 * @see gov.va.med.hds.querytool.IHTMLSelectionModel#setChanging(boolean)
	 */
	public synchronized void setChanging(boolean isChanging) {
		boolean oldChanging = this.changing;
		this.changing = isChanging;
		if (oldChanging && !isChanging) {
			fireTextChanged();
		}
	}

}
