package gov.va.med.hds.cd.spectrum.linkpanel;
import gov.va.med.hds.cd.spectrum.dhtml.HTMLFocusTraversalPolicy;
import gov.va.med.hds.util.mumps.StringUtil;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;

public class HTMLSelectionPanel extends JPanel {
	private JEditorPane editorPane = new JEditorPane();
	private BorderLayout borderLayout1 = new BorderLayout();
	private IHTMLSelectionModel model = null;
	private JScrollPane scrollPane = new JScrollPane();
	private ISelectionEventListener textChangedEventListener = new ISelectionEventListener() {
		public void itemSelected(ActionEvent ae) {
			if (ae.getActionCommand().equals(IHTMLSelectionModel.TEXT_CHANGED)) {
				fireTextUpdated();
			}
		}
	};
	private LinkListener linkListener = new LinkListener();
	
	public HTMLSelectionPanel() {
		super();
		jbInit();

	}
	
	public HTMLSelectionPanel(IHTMLSelectionModel model) {
		this();
		setModel(model);
	}
	
	private class LinkListener implements HyperlinkListener, MouseListener{
		private URL potentialLink = null;
		public void hyperlinkUpdate(HyperlinkEvent e) {
			if(e.getEventType()==HyperlinkEvent.EventType.ENTERED){
				potentialLink = e.getURL();
			}else if(e.getEventType()==HyperlinkEvent.EventType.EXITED){
				potentialLink = null;
			}else if(e.getEventType()==HyperlinkEvent.EventType.ACTIVATED){
                potentialLink = null;
                editorPane_hyperlinkEvent(e.getURL().getHost());
            }
		}
		public void mouseClicked(MouseEvent e) {}
		public void mousePressed(MouseEvent e) {
            if (potentialLink != null) {
                e.consume();
                editorPane_hyperlinkEvent(potentialLink.getHost());
            }
        }
		public void mouseReleased(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
	}
	
	private void jbInit() {
		this.setLayout(borderLayout1);
		editorPane.setEditorKit(new HTMLEditorKit());
		editorPane.setContentType("text/html");
		editorPane.setEditable(false);
		editorPane.setText("");
		editorPane.addHyperlinkListener(linkListener);
		editorPane.addMouseListener(linkListener);
		editorPane.setFocusTraversalPolicy(new HTMLFocusTraversalPolicy(editorPane));
		scrollPane.getViewport().add(editorPane, null);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		this.add(scrollPane, BorderLayout.CENTER);
	}

	void editorPane_hyperlinkEvent(String data) {
		data = StringUtil.piece(data, '.', 2);
		int id = Integer.parseInt(data);
		if (id > 0) {
			model.fireObjectSelected(id);
		}
	}
	
	public void setModel(IHTMLSelectionModel model) {
		if (this.model!=null) {
			this.model.removeEventListener(textChangedEventListener);
		}
		this.model = model;
		if (this.model != null) {
			model.addEventListener(textChangedEventListener);
		}
		else {
			editorPane.setText("");
		}
		fireTextUpdated();
	}
	
	private void fireTextUpdated() {
		editorPane.setText(model.getHTML());
		editorPane.setCaretPosition(0);
	}
	
	public void setDefaultText(String text) {
		editorPane.setText(text);
		editorPane.setCaretPosition(0);
	}
	
	public static final void main(String args[]) {
		JFrame frame = new JFrame("HTMLSelectionPanel");
		HTMLSelectionPanel hTMLSelectionPanel = new HTMLSelectionPanel();
		frame.getContentPane().add(hTMLSelectionPanel, BorderLayout.CENTER);
		hTMLSelectionPanel.setDefaultText("<html><head><title></title></head><meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\"><body>My little test.&nbsp;<a href=\"test\">Link1</a>&nbsp;<a href=\"test2\">Link2</a>&nbsp;</body></html>");
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
}
