package gov.va.med.hds.cd.spectrum.linkpanel;

import java.awt.Font;

/**
 * Selection model interface.
 */
public interface IHTMLSelectionModel {
	public static final String ITEM_SELECTED = "0";
	public static final String TEXT_CHANGED = "1";
	public static final String SELECTED_COLOR = "style=\"color: #800080\"";
	public static final String UNSELECTED_COLOR = "style=\"color: #0000FF\"";

	/**
	 * Add HTML formatted text to the HTML document.  Text is not screened, and is
	 * added directly to the document.
	 * @param in
	 */
	void addHTMLText(String in);
	
	/**
	 * Adds the object to the model as a selectable object whose link is formed
	 * by calling Object.toString().  Use the setSelectableObjectTextColor to set
	 * the link color.  To change the display text, use the other add method to
	 * add the object with	 a given display name or call setDisplayName.
	 * @param o
	 */
	int addSelectableObject(Object o);

	/**
	 * Similar to the other similarly named method, except allows setting of other
	 * variables at the same time.
	 * @param object: object to add to selectable collection
	 * @param description: display text to show as a link
	 * @param styleText: HTML color to set the link to.
	 */
	int addSelectableObject(Object o,String description,String style);

	/**
	 * Method addEventListener.
	 * @param el
	 */
	void addEventListener(ISelectionEventListener el);

	/**
	 * Method removeEventListener.
	 * @param el
	 */
	void removeEventListener(ISelectionEventListener el);

	/**
	 * Method clear.
	 */
	void clear();

	/**
	 * Method getHTML.
	 * @return String
	 */
	String getHTML();
	
	/**
	 * Method getHTML is the same as getHTML except it replaces the font
	 * in the HTML with the new font
	 * @param newFont
	 * @return
	 */
	String getHTML(Font newFont);
	
	/**
	 * Method setDescription.
	 * @param descr
	 * @param id
	 */
	void setDescription(String descr, int id);
	
	/**
	 * Method setStyle.
	 * @param style
	 * @param id
	 */
	void setStyle(String style, int id);
	
	/**
	 * Method setObject.
	 * @param object
	 * @param id
	 */
	void setObject(Object object, int id);

	/**
	 * Method getDescription.
	 * @param id
	 * @return String
	 */
	String getDescription(int id);
	
	/**
	 * Method getStyle.
	 * @param id
	 * @return String
	 */
	String getStyle(int id);
	
	/**
	 * Method getObject.
	 * @param id
	 * @return Object
	 */
	Object getObject(int id);

	/**
	 * Method fireObjectSelected.
	 * @param objectID
	 */
	void fireObjectSelected(int objectID);

	/**
	 * Method setChanging.
	 * @param b
	 */
	void setChanging(boolean isChanging);

}
