package gov.va.med.hds.cd.spectrum.longlist;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.EventListener;

import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Extends a JList to use an IBufferModel ListModel for long list lookups.
 */
public class BufferedListLookupBox extends JPanel {
	public static int MIN_HEIGHT = 77;
	public static int MIN_WIDTH = 80;
	public static int TOP = Integer.MIN_VALUE;
	public static int BOTTOM = Integer.MAX_VALUE;
	private JTextField textField = new JTextField();
	private JList itemList = new JList();
	private JScrollBar scrollBar = new JScrollBar();
//	private JScrollPane scrollPane;
	private IBufferModel bufferModel;
	private int position = -1;
	private int pendingPosition = TOP;
	private KeyInputListener keyInputListener = new KeyInputListener();
	private EventListenerList eventListeners = new EventListenerList();
	private DefaultBoundedRangeModel rangeModel;
	private boolean programChangingThumb = true;
	private int oldThumbValue = 0;
	private int rangeModelMidValue;
	private int visibleRowCount;
	private boolean falseSelection;
	
	public BufferedListLookupBox() {
		super();
		init();
	}
	
	public BufferedListLookupBox(IBufferModel bufferModel) {
		this();
		try {
			setBufferModel(bufferModel);
		}
		catch (BufferModelException e) {
			throw new RuntimeException(e);
		}
	}
	
	private void init() {
		itemList.addComponentListener(new ListComponentListener());
		setLayout(new BorderLayout());
		scrollBar.setFocusable(false);
		itemList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		itemList.setPrototypeCellValue("1");
		itemList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (position != -1) {
					//Selection changed from a typed character
					int x = position;
					position = -1;
					falseSelection = true;
					itemList.setSelectedIndex(x);
				}
				else {
					if (!falseSelection) {
						syncText(true);
						int i;
						if ((i=itemList.getSelectedIndex())>-1) {
							setThumb(i);
						}
					}
					else {
						falseSelection = false;
					}
				}
			}
		});
		rangeModel = new DefaultBoundedRangeModel(0,10,0,300);
		oldThumbValue = 0;
		rangeModelMidValue = rangeModel.getMaximum()/2;
		rangeModel.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (!programChangingThumb) {
					userThumbChange();
				}
			}
		});
		rangeModel.setValueIsAdjusting(false);
		programChangingThumb = false;
		scrollBar.setModel(rangeModel);
		itemList.addKeyListener(keyInputListener);
		textField.addKeyListener(keyInputListener);
		itemList.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				fireMouseScroll(e);
			}
		});
		itemList.addMouseListener(new MouseInputListener());
//		scrollPane = new JScrollPane(itemList);
//		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
//		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		Dimension textDim = new Dimension(200,21);
		textField.setPreferredSize(textDim);
		textField.setMaximumSize(new Dimension(10000,21));
		textField.setMinimumSize(new Dimension(40,21));
		add(textField,BorderLayout.NORTH);
		add(scrollBar,BorderLayout.EAST);
//		add(scrollPane,BorderLayout.CENTER);
		add(itemList, BorderLayout.CENTER);
		super.setMinimumSize(new Dimension(MIN_WIDTH, MIN_HEIGHT));
		super.setPreferredSize(new Dimension(MIN_WIDTH, MIN_HEIGHT));
		itemList.setLayoutOrientation(JList.VERTICAL);
	}

	/**
	 * Method processChange.
	 * @param e
	 */
	private void userThumbChange() {
		//When clicking the up or down arrows (or holding them down), getValueIsAdjusting
		// is always FALSE.  If the slider is at either end and the corresponding
		// button is pressed, an event is NOT generated.
		//When clicking and holding in the gutter or sliding the thumb, 
		// getValueIsAdjusting is always TRUE.
		//This change listener always fires when the mouse button is pressed.
		//This change listener always fires a final event when the mouse button is
		//released, and getValueIsAdjusting is always false.
		setTextFocus();
		int thumbValue = rangeModel.getValue();
		if (!rangeModel.getValueIsAdjusting()) {
			int diff = thumbValue - oldThumbValue;
			setThumb(itemList.getSelectedIndex());
			oldThumbValue = rangeModel.getValue();
			if (thumbValue==0) {
				moveTop();
			}
			else if (thumbValue>=rangeModel.getMaximum()-rangeModel.getExtent()) {
				moveBottom();
			}
			else if (diff > 0) {
				if (diff > 1) {
					movePageDown();
				}
				else {
					movePosition(diff);
				}
			}
			else if (diff < 0) {
				if (diff < -1) {
					movePageUp();
				}
				else {
					movePosition(diff);
				}
			}
		}
	}

	/**
	 * Method setTextFocus.
	 */
	private void setTextFocus() {
		if (!textField.isFocusOwner()) {
			textField.requestFocus();
		}
	}
	
	private void fireMouseScroll(MouseWheelEvent e) {
		setTextFocus();
		movePosition(e.getWheelRotation());
	}
	
	public void addListSelectionListener(ListSelectionListener listener) {
		eventListeners.add(ListSelectionListener.class,listener);
	}
	
	public void removeListSelectionListener(ListSelectionListener listener) {
		eventListeners.remove(ListSelectionListener.class,listener);
	}
	
	private class ListComponentListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			visibleRowCount = visibleRowCount();
			if (bufferModel != null) {
				bufferModel.setDefaultCallSize(visibleRowCount);
				if (!bufferModel.isCallInProgress()) {
					resizeSmallList();
				}
				else {
					SwingUtilities.invokeLater(new ResizeDelegate(1000));
				}
			}
		}
	}
	
	private void resizeSmallList() {
		itemList.setVisibleRowCount(visibleRowCount);
		while(!bufferModel.getDataRequest(DataRequestFactory.getSearch(bufferModel.getElementAt(0)))){}
		pendingPosition = 0;
	}
		
	private class ResizeDelegate implements Runnable {
		private int tryCount = 0;
		
		public ResizeDelegate() {
		}
		
		public ResizeDelegate(int tries) {
			tryCount = tries;
		}
		
		public void run() {
			if (tryCount < 1) {
				return;
			}
			else {
				if (bufferModel.isCallInProgress()) {
					SwingUtilities.invokeLater(new ResizeDelegate(--tryCount));
				}
				else {
					resizeSmallList();
				}
			}
		}
	}
	
	private class MouseInputListener extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			setTextFocus();
			if (e.getClickCount()>1) {
				int index = itemList.locationToIndex(e.getPoint());
				if (index > -1) {
					setListIndex(index);
					syncText(true);
					fireItemSelected();
				}
			}
		}
		
	}
	
	private class ListPositionListener implements IListPositionListener {
		/**
		 * @see javax.swing.event.ListDataListener#contentsChanged(ListDataEvent)
		 */
		public void contentsChanged(ListDataEvent e) {
			fireDataUpdated((BufferListDataEvent)e);
		}

		/**
		 * @see javax.swing.event.ListDataListener#intervalAdded(ListDataEvent)
		 */
		public void intervalAdded(ListDataEvent e) {
			fireDataUpdated((BufferListDataEvent)e);
		}

		/**
		 * @see javax.swing.event.ListDataListener#intervalRemoved(ListDataEvent)
		 */
		public void intervalRemoved(ListDataEvent e) {
			fireDataUpdated((BufferListDataEvent)e);
		}

		/**
		 * @see gov.va.med.hds.cd.spectrum.IListPositionListener#positionUpdated(BufferListDataEvent)
		 */
		public void positionUpdated(BufferListDataEvent e) {
			listPositionChanged(e.getIndex0());
		}

	}
	
	private class KeyInputListener extends KeyAdapter implements ActionListener {
		private Timer timer;
		
		public KeyInputListener() {
			timer = new Timer(Integer.MAX_VALUE, this);
			timer.setRepeats(false);
			timer.setInitialDelay(250);
		}
		
		public void keyPressed(KeyEvent e) {
			//The list box should handle all PageUp/PageDown/Up/Down/Ctrl+Home/Ctrl+End
			//key events
			setTextFocus();
			switch (e.getKeyCode()) {
				case KeyEvent.VK_DOWN: {
					movePosition(IBufferModel.FORWARD_DIRECTION);
					syncText(true);
					timer.stop();
					e.consume();
					break;
				}
				case KeyEvent.VK_UP: {
					movePosition(IBufferModel.REVERSE_DIRECTION);
					syncText(true);
					timer.stop();
					e.consume();
					break;
				}
				case KeyEvent.VK_PAGE_UP: {
					movePageUp();
					syncText(true);
					timer.stop();
					e.consume();
					break;
				}
				case KeyEvent.VK_PAGE_DOWN: {
					movePageDown();
					syncText(true);
					timer.stop();
					e.consume();
					break;
				}
				case KeyEvent.VK_HOME: {
					if (e.getModifiers() == KeyEvent.CTRL_MASK) {
						moveTop();
						syncText(true);
						e.consume();
						timer.stop();
					}
					break;
				}
				case KeyEvent.VK_END: {
					if (e.getModifiers() == KeyEvent.CTRL_MASK) {
						moveBottom();
						syncText(true);
						e.consume();
						timer.stop();
					}
					break;
				}
				case KeyEvent.VK_ENTER: {
					if (!itemList.isSelectionEmpty()) {
						fireItemSelected();
					}
					e.consume();
					timer.stop();
					break;
				}
				case KeyEvent.VK_LEFT: {
					timer.stop();
					break;
				}
				case KeyEvent.VK_RIGHT: {
					timer.stop();
					break;
				}
				case KeyEvent.VK_ALT: {
					timer.stop();
					break;
				}
				case KeyEvent.VK_TAB: {
					timer.stop();
					break;
				}
				case KeyEvent.VK_SHIFT: {
					timer.restart();
					break;
				}
//				case KeyEvent.VK_DELETE: {}
//				case KeyEvent.VK_BACK_SPACE: {
//					if (itemList.isFocusOwner()) {
//						position = itemList.getSelectedIndex();
//						textField.requestFocus();
//						textField.setText(textField.getText());
//						e.consume();
//					}
//					else {
//						timer.restart();
//					}
//					break;
//				}
				default: {
					if (e.getSource()==itemList) {
						position = itemList.getSelectedIndex();
						if ((e.getKeyCode()<128) && (e.getKeyCode()>31)) {
							textField.setText(""+e.getKeyChar());
						}
					}
					timer.restart();
				}
			}
		}

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			search();
		}

	}
	
	/**
	 * Method goEnd.
	 */
	private void moveBottom() {
		bufferModel.getDataRequest(DataRequestFactory.getBottom(visibleRowCount));
		pendingPosition = visibleRowCount-1;
	}

	/**
	 * Method goHome.
	 */
	private void moveTop() {
		bufferModel.getDataRequest(DataRequestFactory.getTop(visibleRowCount));
		pendingPosition = 0;
	}

	private void movePageUp() {
		int index = itemList.getSelectedIndex();
		
		if (index < 0) {
			index = 0;
		}
		
		index -= visibleRowCount;
		
		if (index < 0) {
			int delta = 0 - index;
			bufferModel.getDataRequest(DataRequestFactory.getPrev(delta));
			index = 0;
		}

		pendingPosition = 0;
		setListIndex(index);
	}

	private void movePageDown() {
		int index = itemList.getSelectedIndex();
		
		if (index < 0) {
			index = 0;
		}
		
		index += visibleRowCount;
		if (index > bufferModel.getSize()-1) {
			int delta = index - bufferModel.getSize() + 1;
			bufferModel.getDataRequest(DataRequestFactory.getNext(delta));
		}
		
		pendingPosition = visibleRowCount-1;
		setListIndex(index);
	}

	private int visibleRowCount() {
		int result = 1;
		if (itemList.getFixedCellHeight()>0) {
			result = itemList.getHeight()/itemList.getFixedCellHeight();
		}
		
		if (result < 1) {
			result = 1;
		}
		return result;
	}

	/**
	 * Method reposition.
	 */
	private void search() {
		bufferModel.getDataRequest(DataRequestFactory.getSearch(textField.getText()));
		pendingPosition = 0;
	}
	
	private void selectDiffText() {
		if (bufferModel.getSize()>0) {
			String typedText = new String(textField.getText());
			String itemText = bufferModel.getElementAt(0).toString();
			if (itemText.toUpperCase().startsWith(typedText.toUpperCase())) {
				textField.setText(itemText);
				textField.setSelectionStart(typedText.length());
			}
		}
	}
	
	private void syncText(boolean selectAllText) {
		if (bufferModel.getSize()>0) {
			if (itemList.isSelectionEmpty()) {
				itemList.setSelectedIndex(0);
			}
			setThumb(itemList.getSelectedIndex());
			textField.setText(itemList.getSelectedValue().toString());
			if (selectAllText) {
				textField.setSelectionStart(0);
				textField.setSelectionEnd(textField.getText().length());
			}
		}
	}
	
	private void movePosition(int dir) {
		position = -1;
		int index = itemList.getSelectedIndex();
		if (index > -1) {
			index += dir;
			if (index < 0) {
				bufferModel.getDataRequest(DataRequestFactory.getPrev(1));
				setListIndex(0);
				pendingPosition = 0;
			}
			else if (index > bufferModel.getSize()-1) {
				bufferModel.getDataRequest(DataRequestFactory.getNext(1));
				setListIndex(bufferModel.getSize()-1);
				pendingPosition = visibleRowCount-1;
			}
			else {
				setListIndex(index);
			}
		}
	}
	
	private void setListIndex(int i) {
		if ((i>-1) && (i<bufferModel.getSize())) {
			itemList.setSelectedIndex(i);
			itemList.ensureIndexIsVisible(i);
			setThumb(i);
		}
	}
	
	private void setThumb(int i) {
		programChangingThumb = true;
		if ((i==0) && bufferModel.isAtTop()) {
			//Put at top
			rangeModel.setValue(0);
		}
		else if ((i==bufferModel.getSize()-1) && bufferModel.isAtBottom()) {
			//Put at bottom
			rangeModel.setValue(rangeModel.getMaximum()-rangeModel.getExtent());
		}
		else {
			//Put in center
			rangeModel.setValue(rangeModelMidValue);
		}
		oldThumbValue = rangeModel.getValue();
		programChangingThumb = false;
	}
	
	/**
	 * Method fireItemSelected.
	 */
	private void fireItemSelected() {
		syncText(true);
		int index = itemList.getSelectedIndex();
		if (eventListeners.getListenerCount()>0) {
			EventListener[] listeners = eventListeners.getListeners(ListSelectionListener.class);
			for (int i = 0; i < listeners.length; i++) {
				((ListSelectionListener)listeners[i]).valueChanged(new ListSelectionEvent(this,index,index,false));
			}
		}
	}

	/**
	 * Returns the bufferModel.
	 * @return BufferModel
	 */
	public IBufferModel getBufferModel() {
		return bufferModel;
	}

	/**
	 * Method getSelected.
	 * @return Object
	 */
	public Object getSelectedValue() {
		return itemList.getSelectedValue();
	}

	/**
	 * Sets the bufferModel.
	 * @param bufferModel The bufferModel to set
	 */
	public void setBufferModel(IBufferModel bufferModel) throws BufferModelException {
		this.bufferModel = bufferModel;
		itemList.setModel(bufferModel);
		bufferModel.addListPositionListener(new ListPositionListener());
		while(!this.bufferModel.getDataRequest(DataRequestFactory.reset())){}
		while(!this.bufferModel.getDataRequest(DataRequestFactory.getTop())){}
	}

	private void listPositionChanged(int i) {
		itemList.setSelectedIndex(i);
		itemList.ensureIndexIsVisible(i);
		setThumb(i);
	}
	
	private void fireDataUpdated(BufferListDataEvent e) {
		if (pendingPosition == TOP) {
			setListIndex(0);
		}
		else if (pendingPosition == BOTTOM) {
			setListIndex(bufferModel.getSize()-1);
		}
		else {
			setListIndex(pendingPosition);
		}
		if (e.getUpdateType()==IBufferModel.SEARCH_REQUEST) {
			selectDiffText();
		}
		else {
			syncText(true);
		}
	}
	
	public static final void main(String args[]) {
		try {
			JFrame frame = new JFrame("BufferedListLookupBox");
			BufferedListLookupBox list = new BufferedListLookupBox();
			list.init();
			frame.getContentPane().add(list, BorderLayout.CENTER);
			frame.pack();
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.show();

		} catch (Exception ioe) {
			ioe.printStackTrace();
		}
	}
	
}
