package gov.va.med.hds.cd.spectrum.longlist;

import javax.swing.ListModel;

/**
 * Long list buffer model interface.
 */
public interface IBufferModel extends ListModel {
	public static final int FORWARD_DIRECTION = 1;
	public static final int REVERSE_DIRECTION = -1;
	public static final int NO_REQUEST = -1;
	public static final int BOTTOM_REQUEST = 0;
	public static final int SEARCH_REQUEST = 1;
	public static final int TOP_REQUEST = 2;
	public static final int NEXT_REQUEST = 3;
	public static final int PREV_REQUEST = 4;
	public static final int RESET_REQUEST = 5;
	
	/**
	 * Adds listeners to data needed event.  If a user asks this object for data which it
	 * doesn't have, it raises an IDataNeededEvent.
	 * @param listener
	 */
	void addDataNeededListener(IDataNeededListener listener);
	
	/**
	 * Removes the specified registered listener.
	 * @param listener
	 */
	void removeDataNeededListener(IDataNeededListener listener);
	
	/**
	 * Adds position listeners.  This model raises these events when the requested element
	 * is already in the ListModel, and specifies the position of that element in the list.
	 * @param listener
	 */
	void addListPositionListener(IListPositionListener listener);
		
	/**
	 * Removes the specified registered listener.
	 * @param listener
	 */
	void removeListPositionListener(IListPositionListener listener);
		
	/**
	 * Returns the maximum capacity of this buffer model, i.e., the buffer size this model
	 * maintains.
	 * @return int
	 */
	int getCapacity();
	
	/**
	 * Returns the atBottom indicator.  The model knows it is at the end of a long list
	 * if a request for data at a certain point forward returns less than the expected
	 * number of requested elements.
	 * @return boolean
	 */
	boolean isAtBottom();
	
	/**
	 * Returns the atTop indicator.  The model knows it is at the beginning of a long list
	 * if a request for data at a certain point backwards returns less than the expect
	 * number of elements.
	 * @return boolean
	 */
	boolean isAtTop();
	
	/**
	 * Returns the state of a pending call in progress.  If a call is in progress, the
	 * model discards any requests until the current call is complete.  It does not cache
	 * requests.
	 * @return boolean
	 */
	boolean isCallInProgress();
	
	/**
	 * Returns the buffer (in array form) that this model contains.
	 * @return Object[]
	 */
	Object[] getBufferFull();
	
	/**
	 * Sets the default size for data requests that do not contain a call size.
	 * @param defaultCallSize
	 */
	void setDefaultCallSize(int defaultCallSize);

	/**
	 * Returns the default call size used when a data request is made to the model without
	 * specifying the size of the request.
	 * @return int
	 */
	int getDefaultCallSize();
	
	/**
	 * This is the main data request method.  It spawns a new thread and returns immediately
	 * to the caller, and processes the request on the thread.
	 * @param dataRequest
	 * @return boolean
	 */
	boolean getDataRequest(DataRequest dataRequest);

	/**
	 * Data needed listeners use this method to supply requested data.
	 * @param items
	 * @param sizeExpected
	 */
	void setData(Object[] items, int sizeExpected);
	
}
