/*
 * Created on Mar 19, 2003 by vhaislmonsos
 */
package gov.va.med.hds.cd.spectrum.longlist;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * Visual component that uses an IBufferModel to display a long list lookup.  This is a
 * test box and list with a scroll bar.  Cursor/mouse actions move the selected item
 * through the list, and typing in the text box causes the component to call the model
 * to search and find the item typed.
 */
public class LongListLookup extends JPanel implements ILongListLookup {
	private ISearchTextBox textBox = new SearchTextBox();
	private ISearchScrollBar scrollBar = new SearchScrollBar();
	private ISearchList list = new SearchList();
	private IBufferModel bufferModel;
	private ILongListComponentListener componentListener = new ILongListComponentListener() {
		public void actionPerformed(ILongListEvent e) {
			fireComponentEvent(e);
		}
	};
	private class ListComponentListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			if (bufferModel != null) {
				bufferModel.setDefaultCallSize(list.getVisibleRowSize());
				((JList)list).setVisibleRowCount(list.getVisibleRowSize());
				while (!bufferModel.getDataRequest(DataRequestFactory.reset())) {}
				while (!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
			}
		}
	}
	private boolean isSearch = false;
	private ListDataListener listDataListener = new ListDataListener() {
		public void intervalAdded(ListDataEvent e) {
			updateThis();
		}

		public void intervalRemoved(ListDataEvent e) {
			updateThis();
		}

		public void contentsChanged(ListDataEvent e) {
			updateThis();
		}
	};
	private IListPositionListener positionListener = new IListPositionListener() {
		public void intervalAdded(ListDataEvent e) {}
		public void intervalRemoved(ListDataEvent e) {}
		public void contentsChanged(ListDataEvent e) {}
		
		public void positionUpdated(BufferListDataEvent e) {
			list.setSelectedRow(e.getIndex0());
			updateThis();
		}

	};
	
	private void updateText() {
		if (list.getSelectedRow()>-1) {
			String bufferText = bufferModel.getElementAt(list.getSelectedRow()).toString();
			String boxText = textBox.getText();
			if (isSearch) {
				if (bufferText.toLowerCase().startsWith(boxText.toLowerCase()) && boxText.length()>0) {
					textBox.setText(bufferText);
					textBox.getCaret().setDot(textBox.getText().length());
					textBox.getCaret().moveDot(boxText.length());
				}
			}
			else {
				textBox.setText(bufferText);
				selectText();
			}
		}
	}

	private void selectText() {
		textBox.getCaret().setDot(textBox.getText().length());
		textBox.getCaret().moveDot(0);
	}

	private void fireComponentEvent(ILongListEvent e) {
		isSearch = false;
		if (!textBox.isFocusOwner()) {
			textBox.requestFocusInWindow();
		}
		if (bufferModel.getSize()<1) return;
		switch (e.getID()) {
			case ILongListEvent.UP: {
				if (ensureSelectedRow()<0) {
					updateThis();
					break;
				} 
				if (list.getSelectedRow()==0) {
					bufferModel.getDataRequest(DataRequestFactory.getPrev(1));
				}
				else {
					list.setSelectedRow(list.getSelectedRow()-1);
					updateThis();
				}
				break;
			}
			case ILongListEvent.DOWN: {
				if (ensureSelectedRow()<0) {
					updateThis();
					break;
				} 
				if (list.getSelectedRow()==list.getVisibleRowSize()-1) {
					bufferModel.getDataRequest(DataRequestFactory.getNext(1));
				}
				else {
					if (list.getSelectedRow()<bufferModel.getSize()-1) {
						list.setSelectedRow(list.getSelectedRow()+1);
					}
					updateThis();
				}
				break;
			}
			case ILongListEvent.HOME: {
				list.setSelectedRow(0);
				bufferModel.getDataRequest(DataRequestFactory.getTop());
				break;
			}
			case ILongListEvent.END: {
				list.setSelectedRow(bufferModel.getSize()-1);
				bufferModel.getDataRequest(DataRequestFactory.getBottom());
				break;
			}
			case ILongListEvent.PAGE_DOWN: {
				if (ensureSelectedRow()<0) {
					updateThis();
					break;
				} 
				if (list.getSelectedRow()==bufferModel.getSize()-1) {
					bufferModel.getDataRequest(DataRequestFactory.getNext(list.getVisibleRowSize()));
				}
				list.setSelectedRow(bufferModel.getSize()-1);
				updateThis();
				break;
			}
			case ILongListEvent.PAGE_UP: {
				if (ensureSelectedRow()<0) {
					updateThis();
					break;
				}
				if (list.getSelectedRow()==0) {
					bufferModel.getDataRequest(DataRequestFactory.getPrev(list.getVisibleRowSize()));
				}
				list.setSelectedRow(0);
				updateThis();
				break;
			}
			case ILongListEvent.SEARCH: {
				list.setSelectedRow(0);
				bufferModel.getDataRequest(DataRequestFactory.getSearch(textBox.getText()));
				isSearch = true;
				break;
			}
			case ILongListEvent.SELECT: {
				updateThis();
				fireEvent(ILongListEvent.SELECT, list.getSelectedValue());
				break;
			}
			case ILongListEvent.SELECT_CHANGE: {
				updateThis();
				break;
			}
		}
	}
	
	private void fireEvent(int eventID, Object parameter) {
		if (listenerList.getListenerCount(ILongListComponentListener.class)>0) {
			ILongListEvent e = new LongListEvent(this);
			e.setID(eventID);
			e.setParameter(parameter);
			ILongListComponentListener[] list = (ILongListComponentListener[])listenerList.getListeners(ILongListComponentListener.class);
			for (int i = 0; i < list.length; i++) {
				list[i].actionPerformed(e);
			}
		}
	}

	private int ensureSelectedRow() {
		int result = list.getSelectedRow();
		if (result<0) {
			list.setSelectedRow(0);
		}
		return result;
	}

	private void init() {
		addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				textBox.requestFocusInWindow();
			}

			public void focusLost(FocusEvent e) {}
		});
		setLayout(new BorderLayout());
		add((Component)textBox, BorderLayout.NORTH);
		add((Component)scrollBar, BorderLayout.EAST);
		add((Component)list, BorderLayout.CENTER);
		textBox.addLongListListener(componentListener);
		scrollBar.addLongListListener(componentListener);
		list.addLongListListener(componentListener);
		((JComponent)list).setBorder(new BevelBorder(BevelBorder.LOWERED));
		list.setFocusable(false);
		scrollBar.setFocusable(false);
		addComponentListener(new ListComponentListener());
	}
	
	private void updateThis() {
		updateText();
		updateThumb();
	}

	private void updateThumb() {
		if (list.getSelectedRow()==0 && bufferModel.isAtTop()) {
			scrollBar.setNewValue(ISearchScrollBar.TOP);
		}
		else if (list.getSelectedRow()==bufferModel.getSize()-1 && bufferModel.isAtBottom()) {
			scrollBar.setNewValue(ISearchScrollBar.BOTTOM);
		}
		else {
			scrollBar.setNewValue(ISearchScrollBar.MIDDLE);
		}
	}

	public LongListLookup() {
		super();
		init();
	}
	
	public static void main(String[] args) {
		JFrame frame = new JFrame("LongListLookup");
		LongListLookup lll = new LongListLookup();
		frame.getContentPane().add(lll);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}

	/**
	 * Sets the model used by this component.
	 * @param bufferModel
	 */
	public void setModel(IBufferModel bufferModel) {
		if (this.bufferModel!=null) {
			this.bufferModel.removeListDataListener(listDataListener);
			this.bufferModel.removeListPositionListener(positionListener);
			list.setModel(new DefaultListModel());
		}
		this.bufferModel = bufferModel;
		if (bufferModel!=null) {
			list.setModel(bufferModel);
			bufferModel.setDefaultCallSize(list.getVisibleRowSize());
			bufferModel.addListDataListener(listDataListener);
			bufferModel.addListPositionListener(positionListener);
			while (!bufferModel.getDataRequest(DataRequestFactory.reset())){}
			while (!bufferModel.getDataRequest(DataRequestFactory.getTop())){}
		}
	}
	
	public void addLongListListener(ILongListComponentListener l) {
		listenerList.add(ILongListComponentListener.class, l);
	}

	public void removeLongListListener(ILongListComponentListener l) {
		listenerList.remove(ILongListComponentListener.class, l);
	}
	
	/**
	 * Returns the current selection if one exists; null otherwise.
	 * @return Object
	 */
	public Object getSelectedValue() {
		if (list.getSelectedRow()>-1) {
			return list.getSelectedValue();
		}
		else {
			return null;
		}
	}

	/**
	 * Returns the model used by this list.
	 * @return IBufferModel
	 */
	public IBufferModel getModel() {
		return bufferModel;
	}

	public void setPrototypeCellValue(Object value) {
		list.setPrototypeCellValue(value);
	}

	public int getFixedCellHeight() {
		return list.getFixedCellHeight();
	}

	public int getFixedCellWidth() {
		return list.getFixedCellWidth();
	}

	public void setFixedCellHeight(int height) {
		list.setFixedCellHeight(height);
	}

	public void setFixedCellWidth(int width) {
		list.setFixedCellWidth(width);
	}
	
}
