/*
 * Created on Mar 24, 2003 by vhaislmonsos
 */
package gov.va.med.hds.cd.spectrum.longlist;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;

/**
 * The list component for the long list component.
 */
public class SearchList extends JList implements ISearchList {

	/**
	 * Delegates to parent class.
	 * @param dataModel
	 */
	public SearchList(ListModel dataModel) {
		super(dataModel);
		init();
	}

	/**
	 * Delegates to parent class.
	 */
	public SearchList() {
		super();
		init();
	}
	
	private void init() {
		addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				fireMouseMove(e);
			}
		});
		super.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}

			public void mouseReleased(MouseEvent e) {
				fireMouseClick(e);
			}

		});
		setPrototypeCellValue("test TEXT");
	}

	private void fireMouseClick(MouseEvent e) {
		if (e.getButton()==MouseEvent.BUTTON1) {
			int pos;
			if ((pos=this.locationToIndex(e.getPoint()))>-1) {
				setSelectedIndex(pos);
				if (e.getClickCount()==2) {
					fireEvent(ILongListEvent.SELECT, getSelectedValue());
				}
				else {
					fireEvent(ILongListEvent.SELECT_CHANGE, null);
				}
			}
		}
	}

	private void fireMouseMove(MouseWheelEvent e) {
		if (e.getWheelRotation()>0) {
			fireEvent(ILongListEvent.DOWN, null);
		}
		else if (e.getWheelRotation()<0) {
			fireEvent(ILongListEvent.UP, null);
		}
	}

	private void fireEvent(int eventID, Object parameter) {
		if (listenerList.getListenerCount(ILongListComponentListener.class)>0) {
			ILongListEvent e = new LongListEvent(this);
			e.setID(eventID);
			e.setParameter(parameter);
			ILongListComponentListener[] listeners = (ILongListComponentListener[])listenerList.getListeners(ILongListComponentListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].actionPerformed(e);
			}
		}
	}

	public int getVisibleRowSize() {
		int result = 1;
		if (getFixedCellHeight()>0) {
			result = getHeight()/getFixedCellHeight();
		}
		
		if (result < 1) {
			result = 1;
		}
		return result;
	}
	
	public void addLongListListener(ILongListComponentListener l) {
		listenerList.add(ILongListComponentListener.class, l);
	}

	public void removeLongListListener(ILongListComponentListener l) {
		listenerList.remove(ILongListComponentListener.class, l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.JList#setSelectionModel(javax.swing.ListSelectionModel)
	 * 
	 * Overridden to ignore changes to the selection model.
	 */
	public void setSelectionModel(ListSelectionModel selectionModel) {}

	public static void main(String[] args) {
		JFrame frame = new JFrame("SearchList");
		SearchList searchList = new SearchList();
		searchList.addLongListListener(new ILongListComponentListener() {
			public void actionPerformed(ILongListEvent e) {
				System.out.println(e.getID());
				System.out.println(e.getParameter());
			}
		});
		DefaultComboBoxModel model = new DefaultComboBoxModel();
		model.addElement(new String("line 1"));
		model.addElement(new String("String 2"));
		searchList.setModel(model);
		frame.getContentPane().add(searchList);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}

	public int getSelectedRow() {
		return getSelectedIndex();
	}

	public void setSelectedRow(int row) {
		setSelectedIndex(row);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JList#setSelectionMode(int)
	 * 
	 * Overridden to keep the selection mode to single selection.
	 */
	public void setSelectionMode(int selectionMode) {}

}
