/*
 * Initially Created on Mar 20, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.cd.spectrum.longlist;

import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JFrame;
import javax.swing.JScrollBar;

/**
 * Scroll bar convenience class to provide events to the ILongListLookup.
 */
public class SearchScrollBar extends JScrollBar implements ISearchScrollBar {
	private IScrollBarModel scrollBarModel = new ScrollBarModel();

	/**
	 * Delegates to parent class.
	 * @param orientation
	 * @param value
	 * @param extent
	 * @param min
	 * @param max
	 */
	public SearchScrollBar(
		int orientation,
		int value,
		int extent,
		int min,
		int max) {
		super(orientation, value, extent, min, max);
		init();
	}

	/**
	 * Delegates to parent class.
	 * @param orientation
	 */
	public SearchScrollBar(int orientation) {
		super(orientation);
		init();
	}

	/**
	 * Delegates to parent class.
	 */
	public SearchScrollBar() {
		super();
		init();
	}
	
	private void init() {
		setModel(scrollBarModel);
		super.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				if (!e.isConsumed()) {
					if (e.getWheelRotation()>0) {
						fireEvent(ILongListEvent.UP);
					}
					else if (e.getWheelRotation() < 0) {
						fireEvent(ILongListEvent.DOWN);
					}
				}
			}
		});
	}

	private void fireEvent(int eventID) {
		ILongListEvent event = new LongListEvent(this);
		event.setID(eventID);
		if (listenerList.getListenerCount(ILongListComponentListener.class)>0) {
			ILongListComponentListener[] listeners = (ILongListComponentListener[])listenerList.getListeners(ILongListComponentListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].actionPerformed(event);
			}
		}
	}

	public void addLongListListener(ILongListComponentListener l) {
		listenerList.add(ILongListComponentListener.class, l);
		scrollBarModel.addLongListListener(l);
	}
	
	public void removeLongListListener(ILongListComponentListener l) {
		listenerList.remove(ILongListComponentListener.class, l);
		scrollBarModel.removeLongListListener(l);
	}
	
	public static void main(String[] args) {
		JFrame frame = new JFrame("SearchScrollBar");
		SearchScrollBar searchScrollBar = new SearchScrollBar();
		searchScrollBar.addLongListListener(new ILongListComponentListener() {
			public void actionPerformed(ILongListEvent e) {
				System.out.println(e.getID());
				System.out.println(e.getParameter());
			}
		});
		frame.getContentPane().add(searchScrollBar);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
	public void setNewValue(int newValue) {
		scrollBarModel.setNewValue(newValue);
	}
}
