/*
 * Created on Mar 19, 2003 by vhaislmonsos
 */
package gov.va.med.hds.cd.spectrum.longlist;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.swing.text.PlainDocument;

import gov.va.med.hds.cd.spectrum.text.GenericDocumentFilter;

/**
 * Extends SearchTextBox to forward keys not handled here.
 */
public class SearchTextBox extends JTextField implements ISearchTextBox {
	private Timer timer;
	private int timerTime;
	private ActionListener timerListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			fireSearchEvent(e);
		}
	};

	/**
	 * Delegates to parent class.
	 */
	public SearchTextBox() {
		super();
		init();
	}

	/**
	 * Delegates to parent class.
	 * @param text
	 */
	public SearchTextBox(String text) {
		super(text);
		init();
	}
	
	public static void main(String[] args) {
		JFrame frame = new JFrame("SearchTextBox");
		SearchTextBox searchTextBox = new SearchTextBox();
		searchTextBox.setText("test...");
		searchTextBox.addLongListListener(new ILongListComponentListener() {
			public void actionPerformed(ILongListEvent e) {
				System.out.println(e.getID());
				System.out.println(e.getParameter());
			}
		});
		frame.getContentPane().add(searchTextBox);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
	private void init() {
		PlainDocument doc = new PlainDocument();
		doc.setDocumentFilter(new GenericDocumentFilter(80));
		setDocument(doc);
		timer = new Timer(500, timerListener);
		timer.stop();
		timer.setRepeats(false);
		super.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				if (!e.isConsumed()) {
					if (e.getWheelRotation()>0) {
						fireEvent(ILongListEvent.DOWN, null);
					}
					else {
						fireEvent(ILongListEvent.UP, null);
					}
				}
			}
		});
	}
	
	protected void processKeyEvent(KeyEvent e) {
		if (e.getID()==KeyEvent.KEY_PRESSED && !e.isConsumed() && !e.isAltDown()) {
			switch (e.getKeyCode()) {
				case KeyEvent.VK_ENTER: {
					e.consume();
					timer.stop();
					fireEvent(ILongListEvent.SELECT, null);
					break;
				}
				case KeyEvent.VK_UP: {
					e.consume();
					timer.stop();
					fireEvent(ILongListEvent.UP, null);
					break;
				}
				case KeyEvent.VK_DOWN: {
					e.consume();
					timer.stop();
					fireEvent(ILongListEvent.DOWN, null);
					break;
				}
				case KeyEvent.VK_PAGE_UP: {
					e.consume();
					timer.stop();
					fireEvent(ILongListEvent.PAGE_UP, null);
					break;
				}
				case KeyEvent.VK_PAGE_DOWN: {
					e.consume();
					timer.stop();
					fireEvent(ILongListEvent.PAGE_DOWN, null);
					break;
				}
				case KeyEvent.VK_HOME: {
					if (e.getModifiers() == KeyEvent.CTRL_MASK) {
						e.consume();
						timer.stop();
						fireEvent(ILongListEvent.HOME, null);
					}
					break;
				}
				case KeyEvent.VK_END: {
					if (e.getModifiers() == KeyEvent.CTRL_MASK) {
						e.consume();
						timer.stop();
						fireEvent(ILongListEvent.END, null);
					}
					break;
				}
				case KeyEvent.VK_SHIFT: {
					break;
				}
				case KeyEvent.VK_TAB: {
					if (e.isShiftDown()) {
						this.transferFocusBackward();
					}
					else {
						this.transferFocus();
					}
					break;
				}
				case KeyEvent.VK_LEFT:
				case KeyEvent.VK_RIGHT: {
					timer.stop();
					break;
				}
				default: {
					timer.restart();
				}
			}
		}
		super.processKeyEvent(e);
	}
	
	private void fireSearchEvent(ActionEvent e) {
		fireEvent(ILongListEvent.SEARCH, getText());
	}

	private void fireEvent(int eventID, Object parameter) {
		ILongListEvent event = new LongListEvent(this);
		event.setID(eventID);
		event.setParameter(parameter);
		if (listenerList.getListenerCount(ILongListComponentListener.class)>0) {
			ILongListComponentListener[] listeners = (ILongListComponentListener[])listenerList.getListeners(ILongListComponentListener.class);
			for (int i = 0; i < listeners.length; i++) {
				listeners[i].actionPerformed(event);
			}
		}
	}

	public void addLongListListener(ILongListComponentListener l) {
		listenerList.add(ILongListComponentListener.class, l);
	}
	
	public void removeLongListListener(ILongListComponentListener l) {
		listenerList.remove(ILongListComponentListener.class, l);
	}
	
	/**
	 * Returns the number of milliseconds the component waits before firing a search event.
	 * @return int
	 */
	public int getTimerTime() {
		return timerTime;
	}

	/**
	 * Sets the number of milliseconds the component waits before firing a search event.
	 * @param timerTime
	 */
	public void setTimerTime(int timerTime) {
		this.timerTime = timerTime;
	}

}
