/*
 * Created on Feb 28, 2003 1:15:09 PM
 */
package gov.va.med.hds.cd.spectrum.text;

/**
 *
 * @author Joel Russell
 *
 * Department of Veterans Affairs
 * OI Field Office - Salt Lake City
 * Health Data Systems
 * 
 **/
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.EventListenerList;

import gov.va.med.hds.cd.spectrum.longlist.LongListLookup;

public class LongListField extends JPanel {

	private Toolkit toolkit;
	private int maxLength, minLength;
	private StringField stringField;
	private JButton button;
	private String title;
	private Object initialValue;
	private LongListLookup lookupBox;
	private EventListenerList listeners = new EventListenerList();

	public LongListField(
		int minLength,
		int maxLength,
		int columns,
		String title,
		LongListLookup lookupBox,
		Object initialValue) {
		super();
		stringField = new StringField(minLength, maxLength, columns);
		toolkit = Toolkit.getDefaultToolkit();
		this.minLength = minLength;
		this.maxLength = maxLength;
		this.title = title;
		this.lookupBox = lookupBox;
		this.initialValue = initialValue;
		initGUI();
	}

	private void initGUI() {
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		button = new JButton("Browse...");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				doShowList();
			}
		});
		stringField.setValue(initialValue.toString());
		add(stringField);
		add(Box.createRigidArea(new Dimension(5, button.getHeight())));
		add(button);
	}

	private void setComponentSize(JComponent component, Dimension dimension) {
		component.setMaximumSize(dimension);
		component.setMinimumSize(dimension);
		component.setPreferredSize(dimension);
	}

	private void doShowList() {
		int result =
			JOptionPane.showOptionDialog(
				getRootPane(),
				lookupBox,
				"Choose an Item",
				JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.PLAIN_MESSAGE,
				null,
				null,
				initialValue);
		if (result == JOptionPane.OK_OPTION) {
			setValue(lookupBox.getSelectedValue().toString());
			fireItemSelected(lookupBox.getSelectedValue());
		}
	}

	public String getValue() {
		return stringField.getValue();
	}

	public void setValue(String value) {
		stringField.setValue(value);
	}

	private void fireItemSelected(Object selectedItem) {

		EventListener[] sl;

		sl = listeners.getListeners(
				ILongListFieldSelectionListener.class);

		if (sl.length == 0) {
			return;
		}

		for (int i = 0; i < sl.length; ++i) {

			ILongListFieldSelectionListener listener =
				(ILongListFieldSelectionListener) sl[i];
			listener.itemSelected(selectedItem);
		}
		
	}
	
	public void addSelectionListener(ILongListFieldSelectionListener ll) {
		listeners.add(ILongListFieldSelectionListener.class, ll);
	}

	public void removeSelectionListener(ILongListFieldSelectionListener ll) {
		listeners.remove(ILongListFieldSelectionListener.class, ll);	
	}
	
	public void addStringFieldActionListener(ActionListener al) {
		stringField.addActionListener(al);
	}
	
	public void removeStringFieldActionListener(ActionListener al) {
		stringField.removeActionListener(al);
	}
}