package gov.va.med.hds.cd.spectrum.text;

import java.awt.Toolkit;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

public class StringField extends JTextField {

	private	Toolkit	toolkit;
    private	int	maxLength,
	    				minLength;
	
	
	public StringField(int minLength, int maxLength, int columns) {
		super(columns);
		toolkit = Toolkit.getDefaultToolkit();
		this.minLength = minLength;
		this.maxLength = maxLength;
	}

	public String getValue() {
		return getText();
	}

	public void setValue(String value) {
		setText(value);
	}

	protected Document createDefaultModel() {
		return new LimitedLengthDocument();
	}
	
	protected class LimitedLengthDocument extends PlainDocument {
	    
	    public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
			// first, screen confrols, and non-Unicode characters
			char[] source = str.toCharArray();
			char[] result = new char[source.length];
			int j = 0;

			for (int i = 0; i < result.length; i++) {
				if ((Character.isDefined(source[i]))&&(!(Character.isISOControl(source[i]))))
					result[j++] = source[i];
				else {
					toolkit.beep();
					System.err.println("insertString: " + source[i]);
				}
			}
			str = new String(result, 0, j);
	
	        if (str == null) {
	            return;
	        }
	
	        int strLen = str.length();
	
	        if (strLen == 0) {
	            return;
	        }
	
	        int len = getLength();
	        
	        if (strLen + len > maxLength) {
	            toolkit.beep();
	
	            str = str.substring(0, maxLength - len);    // or return;
	        }
	        super.insertString(offs, str, a);
	    }
	}

	/**
	 * Returns the maxLength.
	 * @return int
	 */
	public int getMaxLength() {
		return maxLength;
	}

	/**
	 * Returns the minLength.
	 * @return int
	 */
	public int getMinLength() {
		return minLength;
	}

	/**
	 * Sets the maxLength.
	 * @param maxLength The maxLength to set
	 */
	public void setMaxLength(int maxLength) {
		this.maxLength = maxLength;
	}

	/**
	 * Sets the minLength.
	 * @param minLength The minLength to set
	 */
	public void setMinLength(int minLength) {
		this.minLength = minLength;
	}

}