package gov.va.med.hds.cd.spectrum.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.event.EventListenerList;

/**
 * Convenience class for creating a wizard page set, which takes care of the event
 * listers.  Inheriting classes need only worry about calling firePageReady when 
 * the next requested page is ready.
 *
 */
public abstract class AbstractPageSetModel implements IPageSetModel {
	private EventListenerList eventListeners = new EventListenerList();

	/**
	 * Constructor for AbstractPageSetModel.
	 */
	public AbstractPageSetModel() {
		super();
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IPageSetModel#addPageGeneratedListener(ActionListener)
	 */
	public void addPageGeneratedListener(ActionListener actionListener) {
		eventListeners.add(ActionListener.class,actionListener);
	}

	/**
	 * @see gov.va.med.hds.cd.spectrum.IPageSetModel#removePageGeneratedListener(ActionListener)
	 */
	public void removePageGeneratedListener(ActionListener actionListener) {
		eventListeners.remove(ActionListener.class,actionListener);
	}
	
	protected EventListenerList getEventListenerList() {
		return eventListeners;
	}

	protected void firePageReady() {
		if (eventListeners.getListenerCount() > 0) {
			EventListener[] listeners = eventListeners.getListeners(ActionListener.class);
			for (int i = 0; i < listeners.length; i++) {
				ActionListener listener = (ActionListener)listeners[i];
				listener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,null));
			}
		}
	}
	
}
