package gov.va.med.hds.cd.spectrum.wizard;

import java.awt.event.ActionListener;

/**
 * Base wizard page set model interface.
 */
public interface IPageSetModel {
	/**
	 * Adds a listener that gets notified when a page is generated by this model.
	 * @param actionListener
	 */
	void addPageGeneratedListener(ActionListener actionListener);
	
	/**
	 * Removes the listener for page generated events.
	 * @param actionListener
	 */
	void removePageGeneratedListener(ActionListener actionListener);
	
	/**
	 * The total number of pages provided by the model.
	 * @return int
	 */
	int size();
	
	/**
	 * The page number (0 based) of the page returned by getCurrentPage().
	 * @return int
	 */
	int getCurrentPageNum();
	
	/**
	 * Returns the current page generated by the model.
	 * @return IWizardPage
	 */
	IWizardPage getCurrentPage();
	
	/**
	 * Asks the model for the previous page.
	 */
	void getPrevRequest();
	
	/**
	 * Asks the model for the next page.
	 */
	void getNextRequest();
	
	/**
	 * Asks the model to complete default processing on all pages (similar to
	 * calling getNextRequest() repeatedly.)  The model eventually generates a 
	 * page generated request.  After this, if the getCurrentPage() call returns
	 * a null, this call was successful.  If the call returns a non-null page,
	 * then there is an unresolved error on that page, and the message can be 
	 * retrieved by calling getErrorMessage on the returned page object.
	 */
	void getFinishRequest();
	
	/**
	 * Returns a boolean to indicate if the current page is the first page.
	 * @return boolean
	 */
	boolean isFirstPage();
	
	/**
	 * Returns a boolean to indicate if the current page is the last page.
	 * @return boolean
	 */
	boolean isLastPage();

	/**
	 * Tells the page model to reset the pages and supply the first valid page for
	 * the set.
	 */
	void reset();

	/**
	 * Goes to the specified page number in the wizard model.
	 * @param pageNum
	 */
	void goToPageRequest(int pageNum);

	/**
	 * Returns true if the wizard page set model is complete and valid.  False
	 * otherwise.
	 * @return
	 */
	boolean isComplete();
}
