/*
 * Initially Created on Mar 18, 2003 by vhaislmonsos
 *
 */
package gov.va.med.hds.cd.spectrum.wizard;

import java.awt.event.ActionListener;

/**
 * Wizard panel interface.
 */
public interface IWizard {
	public static final int CANCEL_REQUESTED = 1;
	public static final int BACK_REQUESTED = 2;
	public static final int NEXT_REQUESTED = 3;
	public static final int FINISH_REQUESTED = 4;
	public static final int RESET_REQUESTED = 5;
	public static final int COMPLETED = 6;
	public static final int LAST_PAGE = Integer.MAX_VALUE;
	
	/**
	 * Sets the page set model used by the IWizard.
	 * @param model
	 */
	void setModel(final IPageSetModel model);
	
	/**
	 * Returns the model used by the IWizard.
	 * @return
	 */
	IPageSetModel getModel();
	
	/**
	 * Tells the IWizard to go to the next page; equivalent to the user pressing 
	 * the "Next" button.  If the page has errors, the wizard does not proceed past
	 * the page with the errors.
	 */
	void goNext();
	
	/**
	 * Programmatically the same as the user pressing the "Finish" button.
	 */
	void goFinish();
	
	/**
	 * Programmatically the same as the user pressing the "Back" button.
	 */
	void goBack();
	
	/**
	 * Programmatically the same as the user pressing the "Cancel" button.
	 */
	void goCancel();
	
	/**
	 * Delegates to the model for processing.
	 * @param page
	 */
	void goToPage(int page);
	
	/**
	 * Adds an action listener to IWizard events.
	 * @param al
	 */
	void addActionListener(ActionListener al);
	
	/**
	 * Removes an action listener to IWizard events.
	 * @param al
	 */
	void removeActionListener(ActionListener al);
	
	/**
	 * Sets the height of the IWizard buttons.
	 * @param height
	 */
	void setButtonHeight(int height);
	
	/**
	 * If a wizard page model exists, the wizard calls the reset method on the model,
	 * which provides the first page when finished.
	 */
	public abstract void reset();
	
	/**
	 * Returns the state of the component.
	 * @return boolean
	 */
	public boolean isProcessingRequest();

}