package gov.va.med.hds.cd.spectrum.wizard;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.event.EventListenerList;

public class Wizard extends JPanel implements IWizard {
	private JPanel contextPanel = new JPanel();
	private JPanel buttonPanel = new JPanel();
	private JButton cancelButton = new JButton();
	private JButton backButton = new JButton();
	private JButton nextButton = new JButton();
	private JButton finishButton = new JButton();
	private JPanel contentCollectionPanel = new JPanel();
	private JPanel statusPanel = new JPanel();
	private JPanel separatorPanel = new JPanel();
	private JPanel borderLinePanel = new JPanel();
	private CardLayout cardLayout = new CardLayout();
	private JLabel statusText = new JLabel();
	private JLabel progressLabel = new JLabel();
	private JProgressBar progressBar = new JProgressBar();
	private EventListenerList eventListeners = new EventListenerList();
	private IPageSetModel model;
	private boolean pendingRequest = false;
	private IWizardPage currentPage;
	private int panelNumber = 0;
	private ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				firePageAvailable();
			}
		};

	public Wizard() {
		init();
	}

	public void setModel(final IPageSetModel model) {
		if (this.model!=null) {
			this.model.removePageGeneratedListener(actionListener);
		}
		this.model = model;
		model.addPageGeneratedListener(actionListener);
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					resetProgressBar(model);
				}
			});
		}
		else {
			resetProgressBar(model);
		}
	}
	
	private void resetProgressBar(IPageSetModel model) {
		progressBar.setMinimum(0);
		progressBar.setMaximum(model.size()-1);
	}
	
	public IPageSetModel getModel() {
		return model;
	}

	public synchronized void goNext() {
		if (pendingRequest) return;
		notifyListeners(NEXT_REQUESTED);
		if (readyForNext()) {
			model.getNextRequest();
		}
	}

	private boolean readyForNext() {
		if ((currentPage!=null) && (!currentPage.isOkToGoNext())) {
			displayErrorDialog(currentPage.getErrorMessage());
			return false;
		}
		else {
			pendingRequest = true;
			updateButtons();
			return true;
		}
	}

	private void displayErrorDialog(String message) {
		JOptionPane optionPane = new JOptionPane(message,JOptionPane.INFORMATION_MESSAGE);
		JDialog dialog = optionPane.createDialog(this, "Input Needed");
		dialog.show();
	}

	public synchronized void goFinish() {
		if (pendingRequest) return;
		if (readyForNext()) {
			model.getFinishRequest();
		}
		notifyListeners(FINISH_REQUESTED);
	}
	
	public synchronized void goBack() {
		if (pendingRequest) return;
		notifyListeners(BACK_REQUESTED);
		if (readyForNext()) {
			model.getPrevRequest();
		}
	}
	
	public synchronized void goCancel() {
		notifyListeners(CANCEL_REQUESTED);
		pendingRequest = false;
		updateButtons();
	}
	
	public synchronized void addActionListener(ActionListener al) {
		eventListeners.add(ActionListener.class, al);
	}
	
	public synchronized void removeActionListener(ActionListener al) {
		eventListeners.remove(ActionListener.class, al);
	}
	
	public void setButtonHeight(int height) {
		cancelButton.setMinimumSize(new Dimension(80, height));
		cancelButton.setMaximumSize(new Dimension(80, height));
		cancelButton.setPreferredSize(new Dimension(80, height));
		
		backButton.setMinimumSize(new Dimension(80, height));
		backButton.setMaximumSize(new Dimension(80, height));
		backButton.setPreferredSize(new Dimension(80, height));
		
		nextButton.setMinimumSize(new Dimension(80, height));
		nextButton.setMaximumSize(new Dimension(80, height));
		nextButton.setPreferredSize(new Dimension(80, height));
		
		finishButton.setMinimumSize(new Dimension(80, height));
		finishButton.setMaximumSize(new Dimension(80, height));
		finishButton.setPreferredSize(new Dimension(80, height));
		
	}
	
	private synchronized void firePageAvailable() {
		if (!pendingRequest) return;
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					firePageAvailable();
				}
			});
			return;
		}
		if (contentCollectionPanel.getComponentCount()>0) {
			Component c = contentCollectionPanel.getComponent(0);
			cardLayout.removeLayoutComponent(c);
			contentCollectionPanel.removeAll();
		}
		currentPage = model.getCurrentPage();
		if (currentPage != null) {
			contentCollectionPanel.add((Component)currentPage,Integer.toString(panelNumber+1));
			cardLayout.addLayoutComponent((Component)currentPage,Integer.toString(panelNumber+1));
			cardLayout.first(contentCollectionPanel);
			progressBar.setValue(model.getCurrentPageNum());
			statusText.setText(currentPage.getTitle());
		}
		pendingRequest = false;
		updateButtons();
	}
	
	private void notifyListeners(int actionSelected) {
		if (eventListeners.getListenerCount(ActionListener.class) > 0) {
			EventListener[] listeners = eventListeners.getListeners(ActionListener.class);
			ActionEvent ae = new ActionEvent(this, actionSelected,"");
			for (int i = 0; i < listeners.length; i++) {
				ActionListener listener = (ActionListener)listeners[i];
				listener.actionPerformed(ae);
			}
		}
	}
	
	private void init() {
		this.setLayout(new BorderLayout());
		contextPanel.setLayout(new BorderLayout());
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
		cancelButton.setText("Cancel");
		cancelButton.setMnemonic('C');
		backButton.setText("< Back");
		backButton.setMnemonic('B');
		nextButton.setText("Next >");
		nextButton.setMnemonic('N');
		finishButton.setText("Finish");
		finishButton.setMnemonic('i');
		contentCollectionPanel.setLayout(cardLayout);
		statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
//		statusPanel.setBackground(new Color(52, 88, 154)); //Dark blue
		statusPanel.setBackground(new Color(168, 178, 216));  //Light blue
//		statusPanel.setBackground(new Color(201, 201, 209));  //Gray
		statusPanel.setMinimumSize(new Dimension(112, 20));
		statusPanel.setMaximumSize(new Dimension(65636, 40));
		statusPanel.setPreferredSize(new Dimension(252, 24));
		statusPanel.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
		separatorPanel.setLayout(new BoxLayout(separatorPanel,BoxLayout.X_AXIS));
		separatorPanel.setMinimumSize(new Dimension(14, 14));
		separatorPanel.setPreferredSize(new Dimension(14, 14));
		borderLinePanel.setBorder(
			BorderFactory.createLineBorder(Color.black, 2));
		borderLinePanel.setMaximumSize(new Dimension(32767, 2));
		borderLinePanel.setMinimumSize(new Dimension(14, 2));
		borderLinePanel.setPreferredSize(new Dimension(14, 2));
		statusText.setText("Status Text");
		progressLabel.setText("Progress: ");
		progressLabel.setLabelFor(progressBar);
		progressBar.setMaximumSize(new Dimension(100, 16));
		progressBar.setMinimumSize(new Dimension(70, 16));
		progressBar.setPreferredSize(new Dimension(70, 16));
		progressBar.setMaximum(99);
		progressBar.setValue(50);
		contextPanel.add(statusPanel, BorderLayout.NORTH);
		statusPanel.add(Box.createHorizontalStrut(5));
		statusPanel.add(statusText);
		statusPanel.add(Box.createHorizontalGlue());
		statusPanel.add(progressLabel);
		statusPanel.add(progressBar, null);
		contextPanel.add(contentCollectionPanel, BorderLayout.CENTER);
		separatorPanel.add(borderLinePanel, null);
		contextPanel.add(separatorPanel, BorderLayout.SOUTH);
		this.add(contextPanel, BorderLayout.CENTER);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(cancelButton, null);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(backButton, null);
		buttonPanel.add(nextButton, null);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(finishButton, null);
		buttonPanel.add(Box.createHorizontalStrut(20));
		this.add(buttonPanel, BorderLayout.SOUTH);
		
		nextButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				goNext();
			}
		});
		
		backButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				goBack();
			}
		});
		
		finishButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				goFinish();
			}
		});
		
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				goCancel();
			}
		});
		
	}
	
	private void updateButtons() {
		if (pendingRequest) {
			backButton.setEnabled(false);
			nextButton.setEnabled(false);
			finishButton.setEnabled(false);
		}
		else {
			backButton.setEnabled(!model.isFirstPage());
			nextButton.setEnabled(!model.isLastPage());
			finishButton.setEnabled((currentPage != null) ? currentPage.isFinishAllowed():true);
		}
	}
	
	public static final void main(String args[]) {
		JFrame frame = new JFrame("Wizard");
		Wizard wizard = new Wizard();
		frame.getContentPane().add(wizard);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.show();
	}
	
	/**
	 * If a wizard page model exists, the wizard calls the reset method on the model,
	 * which provides the first page when finished.
	 */
	public synchronized void reset() {
		if (pendingRequest) return;
		pendingRequest = true;
		updateButtons();
		notifyListeners(RESET_REQUESTED);
		model.reset();
	}

	public synchronized void goToPage(int page) {
		if (pendingRequest) return;
		if (readyForNext()) {
			model.goToPageRequest(page);
		}
	}

	public boolean isProcessingRequest() {
		return pendingRequest;
	}

}