package gov.va.med.hds.task.data.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import gov.va.med.hds.task.data.ITask;
import gov.va.med.hds.task.data.TaskDataAccessException;
import gov.va.med.hds.task.data.TaskID;

public class AbstractTask implements ITask {

    boolean isDirty;
    TaskID taskID;
    Object patientID;
    Object creatorID;
    Date creationDate;
    String subject;
    String priority;
    Date dueDate;
    Date completionDate;
    Object completingUserID;
    Date deletionDate;
    Object deletingUserID;
    List linkedItems;

    public TaskID getID() {
        return taskID;
    }

    public synchronized void setID(TaskID id) throws IllegalStateException {
        if (taskID != null)
            throw new IllegalStateException("Task ID is already set");
        taskID = id;
    }

    public Object getPatientID() {
        return patientID;
    }

    public Object getCreatingUserID() {
        return creatorID;
    }

    public boolean creatingUserMatches(Object userID) {
        return creatorID != null && creatorID.equals(userID);
    }
    
    public Date getCreationDate() {
        return creationDate;
    }

    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        if (!areEqual(subject, this.subject)) {
            this.subject = subject;
            isDirty = true;
        }
    }

    public String getPriority() {
        return priority;
    }

    public void setPriority(String priority) {
        if (!areEqual(priority, this.priority)) {
            this.priority = priority;
            isDirty = true;
        }
    }

    public Date getDueDate() {
        return dueDate;
    }

    public void setDueDate(Date dueDate) {
        if (!areEqual(dueDate, this.dueDate)) {
            this.dueDate = dueDate;
            isDirty = true;
        }
    }

    public Date getCompletionDate() {
        return completionDate;
    }

    public Object getCompletingUserID() {
        return completingUserID;
    }

    public Date getDeletionDate() {
        return deletionDate;
    }

    public Object getDeletingUserID() {
        return deletingUserID;
    }

    public void markTaskComplete(Date when, Object userID) {
        if (areEqual(when, completionDate)
            && areEqual(userID, completingUserID))
            return;

        completionDate = when;
        completingUserID = userID;
        isDirty = true;
    }

    public void markTaskIncomplete() {
        if (completionDate != null || completingUserID != null)
            isDirty = true;
        completionDate = null;
        completingUserID = null;
    }

    public void markTaskDeleted(Date when, Object userID) {
        deletionDate = when;
        deletingUserID = userID;
        isDirty = true;
    }

    public List getLinkedItems() {
        return linkedItems != null ? linkedItems : Collections.EMPTY_LIST;
    }

    public void addLinkedItem(String itemType, String itemID) {
        if (linkedItems == null)
            linkedItems = new ArrayList();

        AbstractLinkedItem item = new AbstractLinkedItem();
        item.itemType = itemType;
        item.itemID = itemID;

        if (!linkedItems.contains(item)) {
            linkedItems.add(item);
            isDirty = true;
        }
    }

    public void removeLinkedItem(String itemType, String itemID) {
        AbstractLinkedItem item = new AbstractLinkedItem();
        item.itemType = itemType;
        item.itemID = itemID;
        
        if (linkedItems != null && linkedItems.contains(item)) {
            linkedItems.remove(item);
            isDirty = true;
        }
    }

    public boolean isDirty() {
        return isDirty;
    }

    public void save() throws TaskDataAccessException {
    }

    static boolean areEqual(Object a, Object b) {
        if (a == b)
            return true;
        if (a != null)
            return a.equals(b);
        if (b != null)
            return b.equals(a);
        return false;
    }

}
