package gov.va.med.hds.task.data.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;

import gov.va.med.hds.cd.m.IMumpsUserInfo;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.task.data.ITask;
import gov.va.med.hds.task.data.ITaskDAO;
import gov.va.med.hds.task.data.TaskDataAccessException;
import gov.va.med.hds.task.data.TaskID;

public class MemoryTaskDAO implements ITaskDAO {

	static Map taskCache = Collections.synchronizedMap(new HashMap());
	;
	String[] PRIORITIES = { "Low", "Medium", "High" };
	IRuntimeContext runtime;
    CurrentDateProvider currDate = new CurrentDateProvider();
    TaskComparator comparator = new TaskComparator(PRIORITIES, currDate);

	private static long nextIDNumber = 0;

	private static synchronized TaskID getNextID() {
		return new TaskID(new Long(++nextIDNumber));
	}

	public String[] getDefinedTaskPriorities() throws TaskDataAccessException {
		return PRIORITIES;
	}

	public synchronized ITask getTask(Object taskID) throws TaskDataAccessException {
		taskID = resolveTaskID(taskID);

		return (ITask) taskCache.get(taskID);
	}

	protected TaskID resolveTaskID(Object taskID) {
		if (!(taskID instanceof TaskID))
			taskID = new TaskID(taskID);
		return (TaskID) taskID;
	}

	public synchronized void saveTask(ITask task) throws TaskDataAccessException {
		if (task == null)
			return;

		TaskID id = task.getID();
		if (id == null)
			try {
				task.setID(id = getNextID());
			} catch (IllegalStateException ise) {
				id = task.getID();
			}

		taskCache.put(id, task);
	}

	public synchronized List getTasksForPatient(Object patientID) throws TaskDataAccessException {
		if (patientID == null)
			return null;

		List result = new ArrayList();
		Iterator i = taskCache.values().iterator();
		while (i.hasNext()) {
			ITask task = (ITask) i.next();
			if (patientID.equals(task.getPatientID()) && task.getDeletionDate() == null)
				result.add(task);
		}
		Collections.sort(result, comparator);

		return result;
	}

	public List getTasks(List taskIDs) throws TaskDataAccessException {
		List result = new ArrayList();
		Iterator i = taskIDs.iterator();
		while (i.hasNext()) {
			Object taskID = resolveTaskID(i.next());
			Object task = getTask(taskID);
			if (task != null)
				result.add(task);
		}
		Collections.sort(result, comparator);
		return result;
	}

	public ITask createNewTask(Object patientID, Object creatorID) {
		if (patientID == null)
			throw new NullPointerException();

		AbstractTask result = newTaskObject();
		result.patientID = patientID;

		creatorID = resolveUser(creatorID);

		result.creatorID = creatorID;
		result.creationDate = result.dueDate = currDate.now();
		return result;
	}

	protected AbstractTask newTaskObject() {
		return new AbstractTask();
	}

	public void setInitializationData(
		IRuntimeContext runtime,
		IConfigurationElement config,
		String propertyName, Object data)
		throws Exception {
		this.runtime = runtime;
	}

	static String resolveUser(Object userID) {
		if (userID instanceof Subject)
			try {
				Subject subject = (Subject) userID;
				Set principals = subject.getPrincipals(IMumpsUserInfo.class);
				IMumpsUserInfo uinfo = (IMumpsUserInfo) principals.iterator().next();
				return uinfo.getDuz();
			} catch (Exception e) {
			}
		if (userID == null)
			return null;
		else
			return userID.toString();
	}

}
