package gov.va.med.hds.task.data.internal;

import java.util.Comparator;
import java.util.Date;

import gov.va.med.hds.task.data.ITask;

public class TaskComparator implements Comparator {
    
    private String[] priorities;
    private CurrentDateProvider currDate;
    
    public TaskComparator(String[] priorities, CurrentDateProvider currDate) {
        this.priorities = priorities;
        this.currDate = currDate;
    }

    public int compare(Object o1, Object o2) {
        try {
            ITask task1 = (ITask) o1;
            ITask task2 = (ITask) o2;
            
            // first, compare roughly by due date, ascending
            int due1 = getDueDateBucket(task1.getDueDate());
            int due2 = getDueDateBucket(task2.getDueDate());
            if (due1 != due2)
                return due1 - due2;
             
            // next, compare by priority, descending
            int priority1 = getPriorityPos(task1.getPriority());
            int priority2 = getPriorityPos(task2.getPriority());
            if (priority1 != priority2)
                return priority2 - priority1;
                 
            // finally, sort by subject, ascending
            String subject1 = task1.getSubject();
            String subject2 = task2.getSubject();
            if (subject1 == subject2) return 0;
            if (subject1 == null) return -1;
            if (subject2 == null) return  1;
            return subject1.compareToIgnoreCase(subject2);
            
        } catch (Exception e) {
            return 0;
        }
    }
    
    private static final int PAST = -1;
    private static final int PRESENT = 0;
    private static final int FUTURE = 1;
    private static final int UNSET = 2;
    private static final long DAY_MILLIS = 
        24L /*hours*/ * 60 /*minutes*/ * 60 /*seconds*/ * 1000 /*millis*/;
    
    private int getDueDateBucket(Date dueDate) {
        if (dueDate == null)
            return UNSET;
            
        long time = dueDate.getTime();
        long now = currDate.now().getTime();
        long delta = time - now;
        int days = (int) (delta / DAY_MILLIS);
        
        if (days > 0) return FUTURE;
        if (days == 0 && delta > 0) return PRESENT;
        return PAST;
    }
    
    private int getPriorityPos(String priority) {
        if (priority == null) 
            return -1;
            
        for (int i = 0;   i < priorities.length;   i++)
            if (priority.equalsIgnoreCase(priorities[i]))
                return i;
                
        return -1;
    }

}
