package gov.va.med.hds.task.internal;

import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;

import gov.va.med.hds.cd.ccow.IContextModule;
import gov.va.med.hds.cd.daoregistry.IDaoRegistry;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.security.ISecurity;
import gov.va.med.hds.cd.workspace.IWorkspace;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.task.TaskChangeEvent;
import gov.va.med.hds.task.ITaskChangeListener;
import gov.va.med.hds.task.ITaskPlugin;
import gov.va.med.hds.task.data.ILinkedItem;
import gov.va.med.hds.task.data.INameListDAO;
import gov.va.med.hds.task.data.ITaskDAO;
import gov.va.med.hds.task.data.TaskDataAccessException;
import gov.va.med.hds.task.ui.Link;

public class TaskEditorPlugin implements IPlugin, ITaskPlugin {

	public static final String EXTENSION_POINT_ID =
		PLUGIN_ID + ".linkableItem";


	private LinkedList linkableItemTypes;
	private StringBuffer htmlHeaders;
    private IRuntimeContext runtime;
    private IContextModule contextPlugin;
    private ISecurity securityPlugin;
    private ITaskDAO taskDAO;
    private INameListDAO nameDAO;
    private ILogger logger;
    private TaskEventSource eventDispatcher;

	public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
		throws Exception {

        this.runtime = runtime;
		IPluginRegistry registry = runtime.getPluginRegistry();
		IConfigurationElement[] extensions =
			registry.getConfigurationElementsFor(EXTENSION_POINT_ID);

		linkableItemTypes = new LinkedList();
		htmlHeaders = new StringBuffer();

		for (int i = 0; i < extensions.length; i++)
			try {
				LinkableItemType type = new LinkableItemType(extensions[i]);
				linkableItemTypes.add(type);
				String headerContent = type.getHtmlHeaderContent();
				if (htmlHeaders.indexOf(headerContent) == -1)
					htmlHeaders.append(headerContent);
			} catch (Exception e) {
				getLogger().error("Unable to initialize linkable item type", e);
			}
        
        securityPlugin = (ISecurity) runtime.getPluginInstance(ISecurity.PLUGIN_ID);
           
		try {
			contextPlugin =
				(IContextModule) runtime.getPluginInstance(
					IContextModule.PLUGIN_ID);
		} catch (Exception e) {
            // don't worry if the context plugin can't be initialized - we
            // just won't show "Go To Chart" hyperlinks.
		}
		
		eventDispatcher = new TaskEventSource();
		
		IWorkspace workspace =
			(IWorkspace) runtime.getPluginInstance(IWorkspace.PLUGIN_ID);
		workspace.getScope(IWorkspaceConstants.SESSION_SCOPE).getMulticaster().addSource(
			TaskChangeEvent.class,
			eventDispatcher);
	}

	public void shutdown() throws Exception {
	}

	public String getTypeForID(String id) {
		Iterator i = linkableItemTypes.iterator();
		while (i.hasNext()) {
			LinkableItemType type = (LinkableItemType) i.next();
			if (type.matchesID(id))
				return type.getType();
		}
		return null;
	}

	public URL getURLForItem(
		ILinkedItem item,
		String patientID,
		boolean collapse) {
		return getURLForItem(
			item.getItemType(),
			item.getItemID(),
			patientID,
			collapse);
	}

	public URL getURLForItem(
		String itemType,
		String itemID,
		String patientID,
		boolean collapse) {
		Iterator i = linkableItemTypes.iterator();
		while (i.hasNext()) {
			LinkableItemType type = (LinkableItemType) i.next();

			if (type.getType().equalsIgnoreCase(itemType)) {
				URL result = type.getURL(itemID, patientID, collapse);
				if (result != null)
					return result;
			}
		}
		return null;
	}

	public String getHTMLHeaders() {
		return htmlHeaders.toString();
	}

	public String getLinkToTaskURL(
		String linkedItemName,
		String linkedItemType,
		String linkedItemID,
		Object patientID) {

		return Link.getLinkToTaskURL(
			linkedItemName,
			linkedItemType,
			linkedItemID,
			patientID);
	}

	public IContextModule getContextPlugin() {
		return contextPlugin;
	}

	public INameListDAO getNameDAO() throws TaskDataAccessException {
        if (nameDAO == null) try {
            IDaoRegistry daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
            nameDAO = (INameListDAO) daoRegistry.getDAO(INameListDAO.class.getName());
        } catch (Exception e) {
            throw new TaskDataAccessException(e.getMessage());
        }
		return nameDAO;
	}

	public ITaskDAO getTaskDAO() throws TaskDataAccessException {
        if (taskDAO == null) try {
            IDaoRegistry daoRegistry = (IDaoRegistry) runtime.getPluginInstance(IDaoRegistry.PLUGIN_ID);
            taskDAO = (ITaskDAO) daoRegistry.getDAO(ITaskDAO.class.getName());
        } catch (Exception e) {
            throw new TaskDataAccessException(e.getMessage());
        }
		return taskDAO;
	}
    
    public Object getCurrentUser() {
        return securityPlugin.getSubject();
    }

    private ILogger getLogger() {
        if (logger == null)
            logger = runtime.getLogger(TaskEditorPlugin.class);
        return logger;
    }
    
    public void fireTaskEvent(TaskChangeEvent e) {
    	eventDispatcher.fireTaskEvent(e);
    }

	public class TaskEventSource {
		private LinkedList listenerList;
		
		TaskEventSource() {
			listenerList = new LinkedList();
		}
		
		public void addTaskChangeListener(ITaskChangeListener l) {
			synchronized (listenerList) {
				listenerList.add(l);
			}
		}

		public void removeTaskChangeListener(ITaskChangeListener l) {
			synchronized (listenerList) {
				listenerList.remove(l);
			}
		}
		
		public void fireTaskEvent(TaskChangeEvent e) {
			synchronized (listenerList) {
				Iterator i = listenerList.iterator();
				while (i.hasNext()) 
					fireTaskEvent((ITaskChangeListener) i.next(), e);
			}
		}
		
		private void fireTaskEvent(ITaskChangeListener l, TaskChangeEvent e) {
			switch (e.getType()) {
				case TaskChangeEvent.ADDED:   l.taskAdded(e);   break;
				case TaskChangeEvent.CHANGED: l.taskChanged(e); break;
				case TaskChangeEvent.DELETED: l.taskDeleted(e); break;
			}
		}
	}

}
