package gov.va.med.hds.task.ui;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Replace;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;
import gov.va.med.hds.task.data.ILinkedItem;
import gov.va.med.hds.task.data.ITask;
import gov.va.med.hds.task.data.TaskDataAccessException;
import gov.va.med.hds.task.data.TaskID;
import gov.va.med.hds.task.internal.TaskEditorPlugin;


public class Display implements IServlet {
    static final String TASK_END_COMMENT = "<!--END_TASK ";
    static final String TASK_START_COMMENT = "<!--START_TASK ";
    static final String TASK_ID = "taskID";
    static final String PATIENT_ID = "patientID";
    static final String EDIT_TASK_ID = "editTaskID";
    static final String NEW_TASK_ID = "new";
    static final String BRIEF_FORMAT = "brief";
    static final String INNER_FORMAT = "inner";
    static final String LINKED_ITEM_ID = "linkedItemID";
    static final String LINKED_ITEM_TYPE = "linkedItemType";
    static final String SHOW_LINK = "showLink";
    static final String SHOW_ALL_FOR_PATIENT = "showAllForPt";
    static final String FORCE_SHOW_DELETED = "showDeleted";
    
    static IRuntimeContext runtime = null;
    static ILogger logger = null;
    static TaskEditorPlugin taskPlugin = null;
    ServletWriter out;

    public void service(IServletRequest request, IServletResponse response)
        throws IOException
    {
        init(request.getRuntimeContext());

        response.setContentType("text/html");

        out = response.getWriter();

        writeHTMLHeader(request, response);

        try {
            writeTasks(request, response);
        } catch (TaskDataAccessException e) {
            writeErrorMessage(e);
        }

        writeHTMLFooter(request, response);
    }


    static void init(IRuntimeContext runtime) throws IOException {
        Display.runtime = runtime;
        if (taskPlugin == null) 
            try {
                taskPlugin =
                    (TaskEditorPlugin) runtime.getPluginInstance(
                        TaskEditorPlugin.PLUGIN_ID);
            } catch (Exception e) {
                getLogger().error("Could not retrieve the task editor plugin component", e);
                throw new IOException("Could not retrieve the task editor plugin component");
            }
    }
    
    static ILogger getLogger() {
        if (logger == null)
            logger = runtime.getLogger(Display.class);
        return logger;
    }


    private void writeHTMLHeader(IServletRequest request,
        IServletResponse response) {
        out.write("<html>" + "<head>" +
            "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse'>" +
            "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Replace'>" +
            "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Alert'>" +
            "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window'>" +
            "<meta name='scriptlet' content='gov.va.med.hds.task.ui.Edit'>" +
            "<link rel='stylesheet' type='text/css' href='style.css'>");
        out.write(taskPlugin.getHTMLHeaders());
        out.write("</head>" + "<body>");
    }


    private void writeHTMLFooter(IServletRequest request,
        IServletResponse response) {
        out.write("</body></html>");
    }


    private void writeErrorMessage(TaskDataAccessException e) {
        getLogger().error(e.getMessage(), e);
        writeErrorMessage(e.getMessage());
    }


    private void writeErrorMessage(String message) {
        out.print("<h1>Error Retrieving Tasks</h1>");
        out.print("A problem was encountered: ");
        out.printAsHTML(message);
    }


    private void writeTasks(IServletRequest request, IServletResponse response)
        throws TaskDataAccessException
    {
        String editedTaskID = request.getParameter(EDIT_TASK_ID);
        String patientID = request.getParameter(PATIENT_ID);
        String linkedItemID = request.getParameter(LINKED_ITEM_ID);
        String linkedItemType = request.getParameter(LINKED_ITEM_TYPE);
        boolean brief = request.getParameter(BRIEF_FORMAT) != null;
        boolean inner = request.getParameter(INNER_FORMAT) != null;
        boolean showAllForPatient = 
            request.getParameter(SHOW_ALL_FOR_PATIENT) != null;
        boolean forceShowDeleted = 
            request.getParameter(FORCE_SHOW_DELETED) != null;

        if (request.getParameter(SHOW_LINK) != null)
            showLinkedItems(request.getParameterList(SHOW_LINK), patientID, brief);
        else if (showAllForPatient)
            writeTasksForPatient(patientID, brief);
        else if (NEW_TASK_ID.equals(editedTaskID))
            writeNewEditableTask(patientID, linkedItemType, linkedItemID);
        else if (editedTaskID != null)
            writeEditableTask(editedTaskID, linkedItemType, linkedItemID);
        else 
            writeTasks(patientID, request.getParameterList(TASK_ID), brief, inner, forceShowDeleted);
    }


    private void writeNewEditableTask(Object patientID, String linkedItemType,
        String linkedItemID) throws TaskDataAccessException {
        ITask newTask = taskPlugin.getTaskDAO().createNewTask(patientID, getCurrentUserID());
        writeTask(newTask, true, false, false, false, linkedItemType, linkedItemID);
    }


    private void writeEditableTask(Object taskID, String linkedItemType,
        String linkedItemID) throws TaskDataAccessException
    {
        ITask task = taskPlugin.getTaskDAO().getTask(taskID);

        if (task == null)
            writeErrorMessage("Task not found.");
        else
            writeTask(task, true, false, false, false, linkedItemType, linkedItemID);
    }


    private void writeTasksForPatient(Object patientID, boolean brief)
        throws TaskDataAccessException
    {   
        if (!brief)
            writeHeaderLinks(patientID, false);
            
        writeTasks(patientID, taskPlugin.getTaskDAO().getTasksForPatient(patientID), brief, false, false);
    }

    protected void writeHeaderLinks(Object patientID, boolean includeShowAll) {
        if (patientID != null) {
            out.print("<table class='linkheader'><tr><td align='left'>");
            writeCreateTaskLink(patientID);
            if (includeShowAll) {
                out.print("</td><td align='right'>");
                writeShowAllLink(patientID);
            }
            out.print("</td></tr></table>");
        }        
    }

    protected void writeCreateTaskLink(Object patientID) {
        out.print("<a href=\"scriptlet:openWindow('Display"
            + ("?" + EDIT_TASK_ID + "=" + NEW_TASK_ID)
            + ("&" + PATIENT_ID + "="));
        out.printAsURL(patientID.toString());
        out.print("', 'Create Task', 600, 200, true); reload()\">New Task</a>");
    }


    protected void writeShowAllLink(Object patientID) {
        out.print("<a href=\"Display"
            + ("?" + SHOW_ALL_FOR_PATIENT)
            + ("&" + PATIENT_ID + "="));
        out.printAsURL(patientID.toString());
        out.print("\">Show all open tasks</a>");
    }


    private void writeTasks(Object patientID, String[] taskIDs, boolean brief, boolean inner, boolean showDeleted)
        throws TaskDataAccessException
    {
        if (!brief && !inner)
            writeHeaderLinks(patientID, true);
        
        List tasks = taskPlugin.getTaskDAO().getTasks(Arrays.asList(taskIDs));
        writeTasks(patientID, tasks, brief, inner, showDeleted);
    }


    private void writeTasks(Object patientID, List list, boolean brief, boolean inner, boolean showDeleted) {        
        Iterator i = list.iterator();

        while (i.hasNext()) {
            ITask task = (ITask) i.next();
            writeTask(task, false, brief, inner, showDeleted, null, null);
        }
    }


    private void writeTask(ITask task, boolean editing, boolean brief, boolean inner, boolean showDeleted,
        String linkedItemType, String linkedItemID) {
        TaskID taskID = task.getID();
        Object patientID = task.getPatientID();
        boolean newTask = (editing && (taskID == null));

        String id = (newTask ? NEW_TASK_ID
                             : out.escapeEntities(String.valueOf(taskID), true));
        
        if (task.getDeletionDate() != null) {
            if (showDeleted) {
                getLogger().warning("Attempting to display deleted task " + id + " - not yet implemented");
            } else {
                out.print("<div id='deletedTask"+id+"' class='displaynone'>");
                out.print("<!-- deleted task --></div>");
            }
            return;
        }

        out.print(TASK_START_COMMENT + id + " -->\n");

        if (inner)
            out.print("<div class='resultbox'>");
        else if (editing)
            out.print("<div class='editbox'>");
        else
            out.print("<div id='taskDiv" + id + "'><div class='resultbox'>");

        out.print("<form action='Edit' method='POST' name='task" + id +
            "'><table border=0 class='resultheader' cellpadding=1 cellspacing=1>" +
            "    <tr>");

        if (!editing)
            out.print("<td width='1%'><collapser forID='task" + id + "'></td>");

        out.print("      <td align='left'><b><a name='task" + id + "'></a>");

        if (editing) {
            out.print("Subject: ");
            printInput("text", Edit.SUBJECT, task.getSubject(), "maxLen='98'");
            printHidden("Save", "1");
            printHidden(PATIENT_ID, task.getPatientID());
            printHidden(TASK_ID, taskID);
        } else
            out.printAsHTML(task.getSubject());

        out.print("</b></td>" + "      <td align='right'>Due: ");

        if (editing)
            printInput("text", Edit.DUE, formatDueDateField(task.getDueDate()), "customType='absdate'");
        else
            out.printAsHTML(formatDueDate(task.getDueDate()));

        out.print("</td>" + "    </tr>" + "  </table>" + "  <div id='task" +
            id + "'");

        if (!editing)
            out.print(" class='displaynone'");

        out.print(">" + "    <div class='resultbody'>" +
            "<p style='margin-left: 10; margin-top: 0'>");

        String priority = task.getPriority();

        if (editing) {
            out.print("Priority: ");
            printPrioritySelect(priority);
            out.print("<br>");
        } else if (priority != null) {
            out.print("Priority: ");
            out.printAsHTML(priority);
            out.print("<br>");
        }

        Object creatingUserID = task.getCreatingUserID();

        if (creatingUserID != null)
            try {
                String clinicianID = creatingUserID.toString();
                HashMap m = new HashMap();
                m.put(clinicianID, clinicianID);
                taskPlugin.getNameDAO().get(m);

                String clinicianName = (String) m.get(clinicianID);

                if ((clinicianName != null) && (clinicianName.length() > 0)) {
                    out.print("Entered by: ");
                    out.printAsHTML(clinicianName);
                }
            } catch (TaskDataAccessException e) {
                getLogger().error("Unable to retrieve clinician name", e);
            }

        if (editing && (linkedItemID != null)) {
            printHidden(Edit.LINKED_ITEM_ID, linkedItemID);
            printHidden(Edit.LINKED_ITEM_TYPE, linkedItemType);
        }

        List linkedItemList = task.getLinkedItems();

        if (linkedItemList != null || linkedItemID != null) {
            writeLinkedItems(linkedItemType, linkedItemID, linkedItemList, patientID, editing || brief);
        }

        out.write("</div></div>");

        if (editing || !brief) {
            out.write(
                "  <table border=0 class='resultfooter' cellpadding=1 cellspacing=1>" +
                "    <tr>" + "      <td>&nbsp;");

            if (editing) {
                printInput("submit", "Save", "Save", "mnemonic='s'");
                out.print("&nbsp;");
                printInput("button", "Cancel", "Cancel", "onclick='closeWindow()' mnemonic='c'");
                if (!newTask && task.creatingUserMatches(getCurrentUserID())) {
                    out.print("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
                    printInput("button", "Delete", "Delete", 
                        "onclick='confirmDelete(\"" + id + "\", true);' mnemonic='d'");
                }
            } else {
                out.write("<a href=\"");
                out.write(buildEditURI(patientID, id));
                out.write("\">Edit Task</a>&nbsp;&nbsp;&nbsp;");
                if (taskPlugin.getContextPlugin() != null) {
                    out.write("<a href=\"");
                    out.write(taskPlugin.getContextPlugin().getCcowHref(patientID));
                    out.write("\">Go to Chart</a>");
                }
            }

            out.write("&nbsp;&nbsp;&nbsp;</td><td align='right'>");

            boolean completed = (task.getCompletionDate() != null);

            if (editing)
                printInput("checkbox", Edit.COMPLETE, null,
                    (completed ? "checked label='Complete'" : "label='Complete'"));
            else {
                String script = "onChange=\"setTaskCompletion('" + id +
                    "')\" label='Complete' ";
                printInput("checkbox", Edit.COMPLETE, null,
                    script + (completed ? "checked" : ""));
            }

            out.print("&nbsp;</td></tr></table>");
        }

        out.print("</form></div>");
        
        if (!inner && !editing)
            out.print("</div>");

        out.print(TASK_END_COMMENT + id + " -->\n");
    }


    protected String buildEditURI(Object patientID, String taskID) {
        patientID = urlEnc(patientID.toString());

        StringBuffer result = new StringBuffer();
        result.append("scriptlet:openWindow('Display");
        result.append("?").append(EDIT_TASK_ID).append("=").append(taskID);
        result.append("&").append(PATIENT_ID).append("=").append(patientID);
        result.append("', 'Edit Task', 600, 200, true); ");
        
        result.append("replaceInnerURL('taskDiv").append(taskID).append("', ");
        result.append("'Display?inner&").append(TASK_ID).append("=").append(taskID);
        result.append("')");
        
        return result.toString();
    }
    

    protected static final DateFormat DUE_DATE_FORMAT = 
                new SimpleDateFormat("yyyyMMdd");
    private static final DateFormat DATE_FORMATTER = DateFormat.getDateInstance();

    private String formatDueDate(Date d) {
        if (d == null)
            return "none";

        if (d.getTime() < System.currentTimeMillis())
            return "Now";
        else

            return DATE_FORMATTER.format(d);
    }


    private String formatDueDateField(Date d) {
        return ((d == null) ? null : DUE_DATE_FORMAT.format(d));
    }

    private int linkedItemNum = 0;
    

    private void writeLinkedItems(String extraItemType, String extraItemID,
                                  List linkedItemList, Object patientID,
                                  boolean brief) {                                      
        boolean hasItems = false;

        if (patientID != null)
            patientID = urlEnc(patientID.toString());
            
        StringBuffer url = new StringBuffer();
        url.append("Display?" + PATIENT_ID + "=");
        url.append(patientID);
        if (brief) url.append("&brief");
        
        if (extraItemType != null && extraItemID != null) {
            hasItems = true;
            url.append("&").append(SHOW_LINK).append('=')
               .append(urlEnc(extraItemType)).append("%2f%2f")
               .append(urlEnc(extraItemID));
        }
        
        if (linkedItemList != null) {
            Iterator i = linkedItemList.iterator();
            while (i.hasNext()) {
                ILinkedItem linkedItem = (ILinkedItem) i.next();
                if (linkedItem.getItemType() == null) continue;
                hasItems = true;
                url.append("&").append(SHOW_LINK).append('=')
                   .append(urlEnc(linkedItem.getItemType())).append("%2f%2f")
                   .append(urlEnc(linkedItem.getItemID()));
            }
        }
        
        if (hasItems) {
            String u = url.toString();
            out.print("<div deferredSrc='" + u + "'><i>Retrieving linked items...</i></div>");
        }
    }
    
    private String urlEnc(String s) {
        if (s == null) return "";
        try {
            return URLEncoder.encode(s, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            // "can't happen"
            return s;
        }
    }

    private void showLinkedItems(String[] linkedItemIDs, String patientID, boolean brief) {
        for (int i = 0; i < linkedItemIDs.length; i++) {
            String fullID = linkedItemIDs[i];
            int pos = fullID.indexOf("//");
            if (pos == -1) continue; // TODO : error handling?
            String linkedItemType = fullID.substring(0, pos);
            String linkedItemID = fullID.substring(pos+2);
            showLinkedItem(patientID, linkedItemType, linkedItemID, brief);
        }
    }
    
    private void showLinkedItem(
        String patientID,
        String linkedItemType,
        String linkedItemID,
        boolean collapse) {

        try {
            URL u = taskPlugin.getURLForItem(linkedItemType, linkedItemID, patientID, collapse);
            if (u != null) {
                String result = Replace.getHTMLBody(u);
                out.print(result);
            }
        } catch (IOException e) {
            getLogger().error("Unable to display linked item "+linkedItemID, e);
        }
    }


    private Object getCurrentUserID() {
        return taskPlugin.getCurrentUser();
    }


    private void printHidden(String name, Object value) {
        if (value != null)
            printInput("hidden", name, value, null);
    }


    private void printInput(String type, String name, Object value, Object extra) {
        out.print("<input type='");
        out.print(type);
        out.print("' name='");
        out.print(name);
        out.print("'");

        if (value != null) {
            out.print(" value=\"");
            out.printAsHTML(value.toString());
            out.print("\"");
        }

        if (extra != null) {
            out.print(" ");
            out.print(extra);
        }

        out.print(">");
    }


    private void printPrioritySelect(Object value) {
        String[] priorities;

        try {
            priorities = taskPlugin.getTaskDAO().getDefinedTaskPriorities();
        } catch (TaskDataAccessException e) {
            getLogger().error("Unable to retrieve defined task priorities.", e);
            return;
        }

        out.print("<select name='");
        out.print(Edit.PRIORITY);
        out.print("'>");

        for (int i = 0; i < priorities.length; i++) {
            String p = priorities[i];
            out.print("<option value='");
            out.printAsHTML(p);
            out.print("'");

            if ((value != null) && p.equalsIgnoreCase(value.toString()))
                out.print(" selected");

            out.print(">");
            out.printAsHTML(p);
            out.print("</option>");
        }

        out.print("</select>");
    }
}
