package gov.va.med.hds.task.ui;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JOptionPane;
import javax.swing.JToggleButton;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptlet;
import gov.va.med.hds.cd.spectrum.dhtml.scriptlet.IScriptletEvent;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;
import gov.va.med.hds.task.TaskChangeEvent;
import gov.va.med.hds.task.data.ITask;
import gov.va.med.hds.task.data.TaskDataAccessException;
import gov.va.med.hds.task.internal.TaskEditorPlugin;


public class Edit implements IServlet, IScriptlet {
    static final String CANCEL = "Cancel";
    static final String SAVE = "Save";
    static final String PATIENT_ID = Display.PATIENT_ID;
    static final String TASK_ID = "taskID";
    static final String LINKED_ITEM_ID = "linkedItemID";
    static final String LINKED_ITEM_TYPE = "linkedItemType";
    static final String SUBJECT = "subject";
    static final String PRIORITY = "priority";
    static final String DUE = "due";
    static final String COMPLETE = "complete";
    static final String LINK_ONLY = "linkOnly";
    
    static IRuntimeContext runtime;
    static ILogger logger;
    static TaskEditorPlugin taskPlugin;

    public void service(IServletRequest request, IServletResponse response)
        throws IOException
    {
        init(request.getRuntimeContext());
        int preSize = request.getParameters().size();
        request.parsePostedFormData();
        int postSize = request.getParameters().size();

        if ((request.getParameter(CANCEL) != null) ||
                (request.getParameter(CANCEL + ".x") != null))
            handleCancel(request, response);
        else if ((request.getParameter(SAVE) != null) ||
                (request.getParameter(SAVE + ".x") != null))
            handleSave(request, response);
    }


    private void init(IRuntimeContext runtime) throws IOException {
        Edit.runtime = runtime;
        if (taskPlugin == null) 
            try {
                taskPlugin =
                    (TaskEditorPlugin) runtime.getPluginInstance(
                        TaskEditorPlugin.PLUGIN_ID);
            } catch (Exception e) {
                getLogger().error("Could not retrieve the task editor plugin component", e);
                throw new IOException("Could not retrieve the task editor plugin component");
            }
    }
    
    private static ILogger getLogger() {
        if (logger == null)
            logger = runtime.getLogger(Edit.class);
        return logger;
    }


    private void redirectToDisplay(IServletRequest request,
        IServletResponse response, String message, String onload)
        throws IOException
    {
        response.setContentType("text/html");

        ServletWriter out = response.getWriter();
        out.print("<html><head>" +
            "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window'>" +
            "<meta name='scriptlet' content='gov.va.med.hds.task.ui.Edit'>" +
            "</head><body onload=\"");
        if (onload != null)
            out.print(onload + "; ");
        out.print("closeWindow()\">");
        out.printAsHTML(message);
        out.print("</body></html>");
    }


    private void handleCancel(IServletRequest request, IServletResponse response)
        throws IOException
    {
        redirectToDisplay(request, response, "Editing cancelled.", null);
    }


    private void showError(IServletResponse response, String message) {
        response.setContentType("text/html");

        ServletWriter out = response.getWriter();
        out.print("<html><body><h1>Error Encountered</h1>");
        out.printAsHTML(message);
        out.print("</body></html>");
    }


    private Object getCurrentUserID() {
        return taskPlugin.getCurrentUser();
    }


    private void handleSave(IServletRequest request, IServletResponse response)
        throws IOException
    {
        try {
            saveTask(request, response);
        } catch (TaskDataAccessException e) {
            getLogger().error("Unable to save task.", e);
            showError(response, "Unable to save task.");
        }
    }


    private void saveTask(IServletRequest request, IServletResponse response)
        throws TaskDataAccessException, IOException
    {
        ITask task = null;
        Object patientID = null;
        Object currentUserID = getCurrentUserID();
        String taskID = request.getParameter(TASK_ID);
        boolean creatingTask = false;
        boolean changedCompletionDate = false;

        if (taskID == null || taskID.trim().length() == 0) {
            patientID = request.getParameter(PATIENT_ID);
            task = taskPlugin.getTaskDAO().createNewTask(patientID, currentUserID);
            creatingTask = true;
        } else {
            task = taskPlugin.getTaskDAO().getTask(taskID);

            if (task == null)
                throw new TaskDataAccessException("No such task");

            patientID = task.getPatientID();
        }

        if (request.getParameter(LINK_ONLY) == null) {
            String subj = request.getParameter(SUBJECT);
            if (subj == null || subj.trim().length() == 0) 
                subj = "No subject";
            task.setSubject(subj);
            task.setPriority(request.getParameter(PRIORITY));

            String dueDate = request.getParameter(DUE);

            if ((dueDate == null) || (dueDate.length() == 0))
                task.setDueDate(null);
            else {
                Date d = parseDate(dueDate);

                if (d != null)
                    task.setDueDate(d);
            }

            if (request.getParameter(COMPLETE) == null) {
                if (task.getCompletionDate() != null) {
                    task.markTaskIncomplete();
                    changedCompletionDate = true;
                }
            } else if (task.getCompletionDate() == null) {
                task.markTaskComplete(new Date(), currentUserID);
                changedCompletionDate = true;
            }
        }

        String linkedItemID = request.getParameter(LINKED_ITEM_ID);
        String linkedItemType = request.getParameter(LINKED_ITEM_TYPE);

        if ((linkedItemID != null) && (linkedItemType != null))
            task.addLinkedItem(linkedItemType, linkedItemID);

        taskPlugin.getTaskDAO().saveTask(task);
        
        taskID = task.getID().toString();                    
        String onLoad = "fireTaskChange(" +
            "'" + taskID + "', " +
            (creatingTask ? "true" : "false") +
            ")";         

        // The save was successful.
        redirectToDisplay(request, response, "Task saved.", onLoad);
    }

    private static DateFormat[] DEFAULT_DATE_FORMATS = {
        new SimpleDateFormat("yyyyMMdd"),
        DateFormat.getDateInstance(DateFormat.LONG),
        DateFormat.getDateInstance(DateFormat.MEDIUM),
        DateFormat.getDateInstance(DateFormat.SHORT),
        DateFormat.getDateInstance(DateFormat.FULL)
    };

    private Date parseDate(String date) {
        if ((date == null) || (date.length() == 0))
            return null;

        for (int i = 0; i < DEFAULT_DATE_FORMATS.length; i++)
            try {
                return DEFAULT_DATE_FORMATS[i].parse(date);
            } catch (Exception e) {
                getLogger().error("Unable to parse date.", e);
            }

        return null;
    }


    public void init(IScriptletEvent event) {}


    public void setTaskCompletion(IScriptletEvent event, String taskID) {
        try {
            init((IRuntimeContext) event.getEnvironment());

            ITask task = taskPlugin.getTaskDAO().getTask(taskID);

            Element checkBoxElem = event.getElement();
            AttributeSet attr = checkBoxElem.getAttributes();
            JToggleButton.ToggleButtonModel model = (JToggleButton.ToggleButtonModel) attr.getAttribute(StyleConstants.ModelAttribute);

            if (model.isSelected())
                task.markTaskComplete(new Date(), getCurrentUserID());
            else
                task.markTaskIncomplete();

            //TODO: we'll want to batch these up and save them later
            taskPlugin.getTaskDAO().saveTask(task);
            fireChangeNotification(
                (IRuntimeContext) event.getEnvironment(),
                task.getID(),
                TaskChangeEvent.CHANGED);
        } catch (Exception tde) {
            getLogger().error("Unable to set task completion.", tde);
        }
    }
    
    public void confirmDelete(IScriptletEvent event, String taskID, boolean closeWindow) {
        try {
            init((IRuntimeContext) event.getEnvironment());
            
            ITask task = taskPlugin.getTaskDAO().getTask(taskID);
            String subject = task.getSubject();
            String[] message = {
                "Are you certain you want to delete the task,",
                "    '" + subject + "'",
                "from the task list for this patient?  Once",
                "deleted, the task cannot be restored."
            };

            int choice =
                JOptionPane.showConfirmDialog(
                    event.getPane(),
                    message,
                    "Delete Task?",
                    JOptionPane.OK_CANCEL_OPTION);
            if (choice != JOptionPane.OK_OPTION)
                return;

            task.markTaskDeleted(new Date(), getCurrentUserID());
            taskPlugin.getTaskDAO().saveTask(task);
            fireChangeNotification(
                (IRuntimeContext) event.getEnvironment(),
                task.getID(),
                TaskChangeEvent.DELETED);
            
            if (closeWindow)
                event.getPane().runScriptlet(
                    "gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window.closeWindow()",
                    event.getType(),
                    event.getElement());

        } catch (Exception tde) {
            getLogger().error("Unable to delete task.", tde);
        }        
    }
    
    public void fireTaskChange(
        IScriptletEvent event,
        String taskID,
        boolean created) {
            
        fireChangeNotification(
            (IRuntimeContext) event.getEnvironment(),
            taskID,
            created ? TaskChangeEvent.ADDED : TaskChangeEvent.CHANGED);
    }
    

    
	private void fireChangeNotification(
		IRuntimeContext runtime,
		Object taskID,
		int type) {

		if (taskID == null)
			return;

		try {
			init(runtime);
			ITask task = taskPlugin.getTaskDAO().getTask(taskID);
			TaskChangeEvent e = new TaskChangeEvent(taskPlugin, task, type);
			taskPlugin.fireTaskEvent(e);
		} catch (Exception e) {
			getLogger().warning("Error notifying task listeners.", e);
		}
	}
}
