package gov.va.med.hds.task.ui;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IServletRequest;
import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;
import gov.va.med.hds.task.internal.TaskEditorPlugin;
import gov.va.med.hds.util.io.StreamUtilities;

public class Link implements IServlet {
    static final String TASK_ID = "taskID";
    static final String PATIENT_ID = "patientID";
    static final String EDIT_TASK_ID = "editTaskID";
    static final String NEW_TASK_ID = "new";
    static final String LINKED_ITEM_ID = "linkedItemID";
    static final String LINKED_ITEM_TYPE = "linkedItemType";
    
    static IRuntimeContext runtime = null;
    static ILogger logger = null;
    static TaskEditorPlugin taskPlugin = null;
    ServletWriter out;

    public void service(IServletRequest request, IServletResponse response)
        throws IOException {
        String linkedItemType = request.getParameter(LINKED_ITEM_TYPE);
        String linkedItemID = request.getParameter(LINKED_ITEM_ID);

        if ((linkedItemType == null) || (linkedItemID == null)) {
            showError(response, "No linked item was specified.");

            return;
        }

        String patientID = request.getParameter(PATIENT_ID);

        init(request.getRuntimeContext());

        if (!printLinkOptions(request,
            response,
            patientID,
            linkedItemType,
            linkedItemID))
            response.performInternalRedirect(
                getLinkToNewTaskURI(patientID, linkedItemType, linkedItemID));
    }

    private boolean printLinkOptions(
        IServletRequest request,
        IServletResponse response,
        String patientID,
        String linkedItemType,
        String linkedItemID)
        throws IOException {
        List linkableTasks = getTasks(request, patientID);

        if ((linkableTasks == null) || (linkableTasks.size() == 0))
            return false;

        response.setContentType("text/html");
        out = response.getWriter();
        writeHTMLHeader();
        out.print("<h1>Link Item to Task</h1>");

        out.print("<p>To which task would you like to link this item?<br>&nbsp;</p>");
        out.print("<table width='100%' cellpadding=0 cellspacing=0>");
        printLinkOption(
            out,
            getLinkToNewTaskURI(patientID, linkedItemType, linkedItemID),
            "<div class='spacebelow'>Link to new task</div>");

        Iterator i = linkableTasks.iterator();

        while (i.hasNext())
            printLinkOption(
                out,
                patientID,
                linkedItemType,
                linkedItemID,
                (LinkableTask) i.next());

        out.print("</table>");
        out.print("<form method=get action='Edit'>" +
            "<input type=hidden name=Cancel value=true>" +
            "<input type=Submit name=Cancel value=Cancel mnemonic='c'>" +
            "</form>");
        out.print("</body></html>");

        return true;
    }

    private void writeHTMLHeader() {
        out.write(
            "<html>"
                + "<head>"
                + "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.ExpandCollapse'>"
                + "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Replace'>"
                + "<meta name='scriptlet' content='gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Alert'>"
                + "<meta name='scriptlet' content='gov.va.med.hds.task.ui.Edit'>"
                + "<link rel='stylesheet' type='text/css' href='style.css'>"
                + "<style>"
                + "     .resultbox {margin:0 10 10 10; width:100%; border-color:#708090; border-style:solid; border-width:1 } "
                + "     .spacebelow { margin:0 10 10 10 }"
                + "     .padimg {margin: 4 0 0 10 }"
                + "</style>");
        out.write(taskPlugin.getHTMLHeaders());
        out.write("</head><body>");
    }

    private void printLinkOption(
        ServletWriter out,
        String patientID,
        String linkedItemType,
        String linkedItemID,
        LinkableTask linkableTask) {
        String linkUri =
            getLinkToExistingTaskURI(
                patientID,
                linkedItemType,
                linkedItemID,
                linkableTask.urlEncodedTaskID);
        printLinkOption(out, linkUri, linkableTask.html);
    }

    private void printLinkOption(
        ServletWriter out,
        String linkURI,
        String linkHTML) {
        out.print("<tr><td valign='top' width='1%'>");
        out.print("<a href='" + linkURI + "'>");
        out.print("<img hspace='5' vspace='4' border='0' src='radioButton.png' width='12' height='12'>");
        out.print("</a></td><td>");
        out.print(linkHTML);
        out.print("</td></tr>\n");
    }

    private class LinkableTask {
        String urlEncodedTaskID;
        String html;
    }

    private List getTasks(IServletRequest request, String patientID)
        throws IOException {
        String subURI =
            "Display?brief&showAllForPt&" + PATIENT_ID + "=" + urlEncode(patientID);
        URL u = new URL(request.getURL(), subURI);
        URLConnection conn = u.openConnection();
        byte[] rawTaskData =
            StreamUtilities.convertStreamToByteArray(
                conn.getInputStream(),
                true);
        String encoding = conn.getContentEncoding();
        String taskData = null;

        if (encoding != null)
            try {
                taskData = new String(rawTaskData, encoding);
            } catch (UnsupportedEncodingException e) {
            }

        if (taskData == null)
            taskData = new String(rawTaskData);

        List result = new LinkedList();
        int beg = 0;
        int end;

        while ((beg = taskData.indexOf(Display.TASK_START_COMMENT, beg))
            != -1) {
            end = taskData.indexOf(Display.TASK_END_COMMENT, beg);

            if (end == -1)
                break;

            LinkableTask t = new LinkableTask();
            t.html = taskData.substring(beg, end);
            beg = end;

            int idEnd =
                t.html.indexOf(' ', Display.TASK_START_COMMENT.length());
            t.urlEncodedTaskID =
                t.html.substring(Display.TASK_START_COMMENT.length(), idEnd);
            result.add(t);
        }

        return result;
    }

    private String getLinkToNewTaskURI(
        String patientID,
        String linkedItemType,
        String linkedItemID)
        throws UnsupportedEncodingException {
        patientID = urlEncode(patientID);
        linkedItemType = urlEncode(linkedItemType);
        linkedItemID = urlEncode(linkedItemID);

        StringBuffer result = new StringBuffer();
        result.append("Display");
        result.append("?").append(Display.PATIENT_ID).append("=").append(
            patientID);
        result.append("&").append(Display.EDIT_TASK_ID).append("=").append(
            Display.NEW_TASK_ID);
        result.append("&").append(Display.LINKED_ITEM_TYPE).append(
            "=").append(
            linkedItemType);
        result.append("&").append(Display.LINKED_ITEM_ID).append("=").append(
            linkedItemID);

        return result.toString();
    }

    private String urlEncode(String value) {
        try {
            return URLEncoder.encode(value, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return value; // can't happen?
        }
    }

    private String getLinkToExistingTaskURI(
        String patientID,
        String linkedItemType,
        String linkedItemID,
        String urlEncodedTaskID) {
            
        patientID = urlEncode(patientID);
        linkedItemType = urlEncode(linkedItemType);
        linkedItemID = urlEncode(linkedItemID);

        StringBuffer result = new StringBuffer();
        result.append("Edit?").append(Edit.SAVE);
        result.append("&").append(Edit.LINK_ONLY);
        result.append("&").append(Edit.PATIENT_ID).append("=").append(patientID);
        result.append("&").append(Edit.LINKED_ITEM_TYPE).append("=").append(
            linkedItemType);
        result.append("&").append(Edit.LINKED_ITEM_ID).append("=").append(
            linkedItemID);
        result.append("&").append(Edit.TASK_ID).append("=").append(
            urlEncodedTaskID);

        return result.toString();
    }

    private void init(IRuntimeContext runtime) throws IOException {
        Link.runtime = runtime;
        if (taskPlugin == null) 
            try {
                taskPlugin =
                    (TaskEditorPlugin) runtime.getPluginInstance(
                        TaskEditorPlugin.PLUGIN_ID);
            } catch (Exception e) {
                getLogger().error("Could not retrieve the task editor plugin component", e);
                throw new IOException("Could not retrieve the task editor plugin component");
            }
    }
    
    private static ILogger getLogger() {
        if (logger == null)
            logger = runtime.getLogger(Link.class);
        return logger;
    }

    private void showError(IServletResponse response, String message) {
        response.setContentType("text/html");

        ServletWriter out = response.getWriter();
        out.print("<html><body><h1>Error Encountered</h1>");
        out.printAsHTML(message);
        out.print("</body></html>");
    }
    
    
    
    public static String getLinkToTaskURL(String linkedItemName, String linkedItemType, String linkedItemID, Object patientID) {
        StringBuffer result = new StringBuffer();
        String patient_id = "";
        if (patientID != null) patient_id = patientID.toString();
        
        result.append("scriptlet: gov.va.med.hds.cd.spectrum.dhtml.scriptlet.Window.openWindow('");
        
        result.append("cdtp:///gov/va/med/hds/task/Link?linkedItemType=");
        result.append(linkedItemType);
        try {
            result.append("&linkedItemID=").append(URLEncoder.encode(linkedItemID, "UTF-8"));
            result.append("&patientID=").append(URLEncoder.encode(patient_id, "UTF-8"));
            result.append("','Link &quot;").append(dblEscapeHTMLEntities(linkedItemName))
                .append("&quot; to Task',0.85,0.85,true)");
        } catch (UnsupportedEncodingException e) { 
            // "can't happen"
        }
        return result.toString();
    }
    
    private static String dblEscapeHTMLEntities(String value) {
        value = escapeHTMLEntities(value);
        value = escapeHTMLEntities(value);
        value = fixControlChars(value);
        return value;
    }

	private static String escapeHTMLEntities(String value) {
        StringTokenizer tok = new StringTokenizer(value, "<>&'\"", true);
        StringBuffer result = new StringBuffer();
        String token;
        while (tok.hasMoreTokens()) {
            token = tok.nextToken();
            if      ("<".equals(token))  result.append("&lt;");
            else if (">".equals(token))  result.append("&gt;");
            else if ("&".equals(token))  result.append("&amp;");
            else if ("'".equals(token))  result.append("&apos;");
            else if ("\"".equals(token)) result.append("&quot;");
            else                         result.append(token);
        }
        return result.toString();
    }
    
	private static String fixControlChars(String value) {
		for (char i = 0;   i < ' ';   i++)
			value = value.replace(i, ' ');
		return value;
	}
}
