package gov.va.med.hds.cd.url.internal;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.url.IContentProvider;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.URLPluginException;
import gov.va.med.hds.util.io.HTMLUtil;

public class CDServletContentProvider implements IContentProvider {

	private IRuntimeContext runtime;
	private Class servletClass;

	public CDServletContentProvider(Class servletClass) {
		super();
		this.servletClass = servletClass;
	}

	public void init(IRuntimeContext runtime) {
		this.runtime = runtime;
	}

	protected IRuntimeContext getRuntime() {
		return runtime;
	}

	public URLConnection connect(IRuntimeContext runtime, URL u, String id)
		throws IOException {

		String fullPath = u.getPath();
		if (fullPath.startsWith("///"))
			fullPath = fullPath.substring(2);

		String extraPath = null;
		if (fullPath.length() > id.length())
			extraPath = fullPath.substring(id.length());

		Map parameters = HTMLUtil.parseQueryParameters(u.getQuery());
		return openConnection(u, id, extraPath, parameters);
	}

	protected URLConnection openConnection(
		URL u,
		String id,
		String extraPath,
		Map parameters)
		throws IOException {

		IServlet servlet = createServlet(u, id, extraPath, parameters);
		return new CDServletURLConnection(
			servlet,
			runtime,
			u,
			id,
			extraPath,
			parameters);
	}

	protected IServlet createServlet(
		URL u,
		String id,
		String extraPath,
		Map parameters)
		throws IOException {

		if (servletClass != null)
			try {
				return (IServlet) servletClass.newInstance();
			} catch (Exception e) {
				throw new URLPluginException(
					"Unable to create servlet '"
						+ servletClass.getName()
						+ "'");
			}

		throw new URLPluginException("Unsupported url '" + u + "'");
	}

}
