package gov.va.med.hds.cd.url.internal;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.util.io.HTMLUtil;
import gov.va.med.hds.util.io.StreamUtilities;

public class CDServletRequest
    implements gov.va.med.hds.cd.url.IServletRequest {

    private IRuntimeContext runtimeContext;
    private URL url;
    private String id, extraPath;
    private Map parameters;
    private InputStream requestBody;

    public CDServletRequest(
        IRuntimeContext runtimeContext,
        URL url,
        String id,
        String extraPath,
        Map parameters) {
        this.runtimeContext = runtimeContext;
        this.url = url;
        this.id = id;
        this.extraPath = extraPath;
        this.parameters =
            (parameters == null ? Collections.EMPTY_MAP : parameters);
        this.requestBody = null;
    }

    public IRuntimeContext getRuntimeContext() {
        return runtimeContext;
    }

    public URL getURL() {
        return url;
    }

    public String getID() {
        return id;
    }

    public String getExtraPath() {
        return extraPath;
    }

    public Map getParameters() {
        return parameters;
    }

    public String getParameter(String name) {
        return (String) parameters.get(name);
    }

    public String[] getParameterList(String name) {
        String[] result = (String[]) parameters.get(name + "[]");
        if (result == null)
            return EMPTY_STRING_ARRAY;
        else
            return result;
    }

    private static final String[] EMPTY_STRING_ARRAY = new String[0];

    public InputStream getInputStream() throws IOException {
        return requestBody;
    }

    void setInputStream(InputStream requestBody) {
        this.requestBody = requestBody;
    }

    public void parsePostedFormData() throws IOException {
        InputStream inputStream = getInputStream();
        if (inputStream == null)
            // no data was posted to this servlet - do nothing.
            return;

        String data =
            new String(StreamUtilities.convertStreamToByteArray(inputStream));
        Map newParams = new HashMap(parameters);
        newParams.putAll(HTMLUtil.parseQueryParameters(data));
        parameters = Collections.unmodifiableMap(newParams);
    }

}
