package gov.va.med.hds.cd.url.internal;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.url.IServletResponse;
import gov.va.med.hds.cd.url.ServletWriter;


public class CDServletResponse implements IServletResponse {

    private OutputStream outputStream;
    private ServletWriter writer;
    private HashMap responseHeaders;
    private volatile boolean headersDone;
    private URL base;

    public CDServletResponse(OutputStream out, URL base) {
        this.outputStream = out;
        this.writer = null;
        this.responseHeaders = new HashMap();
        this.headersDone = false;
        this.base = base;
        setContentType("text/html"); // default
    }

    public OutputStream getOutputStream() {
        return outputStream;
    }

    public ServletWriter getWriter() {
        if (writer == null)
            writer = new ServletWriter(getOutputStream());

        return writer;
    }

    synchronized void close() throws IOException {
        if (!headersDone)
            commitHeaders();

        if (writer != null) {
            writer.flush();
            writer.close();
        } else {
            outputStream.flush();
            outputStream.close();
        }
    }

    synchronized Map getResponseHeaders() {
        while (!headersDone)
            try {
                wait();
            } catch (InterruptedException ie) {
            }

        return responseHeaders;
    }

    public void setContentType(String type) {
        setHeader("content-type", type);
    }

    public void setHeader(String name, String value) {
        if (headersDone)
            throw new IllegalStateException("Response headers already committed");

        responseHeaders.put(name.toLowerCase(), value);
    }

    public synchronized void commitHeaders() {
        headersDone = true;
        notifyAll();
    }
    
    public void performInternalRedirect(String location)
        throws IllegalStateException, IOException {
        if (headersDone)
            throw new IllegalStateException("Headers already committed");
        
        // this is not a very good implementation.
        URL u = new URL(base, location);
        URLConnection conn = u.openConnection();
        conn.connect();
        setContentType(conn.getContentType());
        commitHeaders();
        InputStream in = conn.getInputStream();
        byte [] buf = new byte[2048];
        int bytesRead;
        while ((bytesRead = in.read(buf)) > 0)
            outputStream.write(buf, 0, bytesRead);
    }

}
