/*
 * Created on Jun 5, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.cd.url.internal;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.url.IContentProvider;
import gov.va.med.hds.cd.url.IServlet;
import gov.va.med.hds.cd.url.IURLPlugin;
import gov.va.med.hds.cd.url.URLPluginException;

/**
 * @author vhaisltumad
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ProviderExtension implements Comparable {

	private IRuntimeContext runtime;

	private String pluginID;

	private String pathID;

	private String pathPrefix;

	private String pathClassName;

	private String contentClassName;

	private String contentPackageName;

	private String contentPackagePrefix;

	private ClassLoader classLoader;

	private IContentProvider classPathProvider;

	private Map providers;

	private static final Object NO_PROVIDER_AVAILABLE = new Object();

	public ProviderExtension(
		IRuntimeContext runtime,
		IConfigurationElement ce)
		throws URLPluginException {

		IPluginDescriptor plugin =
			ce.getDeclaringExtension().getDeclaringPluginDescriptor();
		pluginID = plugin.getUniqueIdentifier();

		// find the ID of this extension point
		pathID = filterEmpty(ce.getAttribute(IURLPlugin.ID_NAME_ATTRIBUTE));
		if (pathID == null)
			throw new URLPluginException(
				"The plugin "
					+ plugin.getUniqueIdentifier()
					+ " failed to provide an ID for a URL content provider.");

		// look up the named class / package.
		contentClassName =
			filterEmpty(ce.getAttribute(IURLPlugin.CLASS_NAME_ATTRIBUTE));
		contentPackageName =
			filterEmpty(ce.getAttribute(IURLPlugin.PACKAGE_NAME_ATTRIBUTE));

		// only allow the user to register one attribute or the other.
		if (contentClassName != null && contentPackageName != null)
			// should we throw an exception here, or just log an error?
			throw new URLPluginException(
				"The URL content provider with id "
					+ pathID
					+ " in plugin "
					+ plugin.getUniqueIdentifier()
					+ " contained values for both "
					+ IURLPlugin.CLASS_NAME_ATTRIBUTE
					+ " and "
					+ IURLPlugin.PACKAGE_NAME_ATTRIBUTE
					+ ". Only one or the other may be used.");

		if (contentClassName != null) {
			int lastDotPos = contentClassName.lastIndexOf('.');
			contentPackageName = contentClassName.substring(0, lastDotPos);
			pathClassName = "/" + pathID.replace('.', '/');

			lastDotPos = pathID.lastIndexOf('.');
			if (lastDotPos == -1)
				pathPrefix = "/";
			else {
				String pathIDPrefix = pathID.substring(0, lastDotPos);
				pathPrefix = "/" + pathIDPrefix.replace('.', '/') + "/";
			}

		} else {
			// if the user did not provide a class name, then this is a
			// package content provider.
			pathPrefix = "/" + pathID.replace('.', '/') + "/";
			pathClassName = null;

			// if no package was named, assume it is the same as the content ID.
			if (contentPackageName == null)
				contentPackageName = pathID;
		}

		// compute the package prefix that this provider will use to find resources
		contentPackagePrefix =
			"/" + contentPackageName.replace('.', '/') + "/";

		// store the class loader of the given configuration element                  
		classLoader = plugin.getPluginClassLoader();

		classPathProvider = null;
		providers = new HashMap();
		this.runtime = runtime;
	}

	private final String filterEmpty(String s) {
		if (s == null || s.length() > 0)
			return s;
		else
			return null;
	}

	private boolean namesClass(String remainder) {
		return remainder != null && remainder.indexOf('.') == -1;
	}

	/** Returns true if this provider extension might be capable of locating
	 * a content provider for the given path.
	 * 
	 * @param path a path to test
	 * @return true if this provider extension might be able to handle a
	 * request for the given path; false otherwise
	 */
	public boolean matches(String path) {
		if (!path.startsWith(pathPrefix))
			return false;

		int len = pathPrefix.length();
		if (contentClassName != null)
			if (path.indexOf('.', len) == -1)
				return path.equals(pathClassName);

		return true;
	}

	/** 
	 * 
	 */
	public String translatePathToID(String path) {
		if (!matches(path))
			return null;
            
        if (pathClassName != null && pathClassName.equals(path))
            return "/" + contentClassName.replace('.', '/');

		int prefixLen = pathPrefix.length();
		String remainder = path.substring(prefixLen);
		return contentPackagePrefix + remainder;
	}

	/**
	 * 
	 */
	public IContentProvider getContentProvider(String path)
		throws IOException {
		if (!matches(path))
			return null;

		int prefixLen = pathPrefix.length();
		String remainder = path.substring(prefixLen);
		if (remainder.indexOf('.') != -1)
			return getClassPathProvider(remainder);
		else
			return getDynamicProvider(path, remainder);
	}

	/**
	 * 
	 */
	private IContentProvider getClassPathProvider(String remainder)
		throws IOException {
		synchronized (this) {
			if (classPathProvider == null) {
				classPathProvider = new ClasspathProvider(classLoader);
				classPathProvider.init(runtime);
			}
		}

		if (!validResourceName(remainder))
			return null;

		String resourcePath = contentPackagePrefix + remainder;
		if (classLoader.getResource(resourcePath.substring(1)) == null)
			return null;
		else
			return classPathProvider;
	}

	protected static final String[] VALID_SUFFIXES =
		{
			".css",
			".png",
			".jpg",
			".gif",
			".xml",
			".xsl",
			".xsd",
			".htm",
			".html",
			".txt" };

	private boolean validResourceName(String resourcePath) {
		resourcePath = resourcePath.toLowerCase();
		for (int i = VALID_SUFFIXES.length; i-- > 0;)
			if (resourcePath.endsWith(VALID_SUFFIXES[i]))
				return true;

		return false;
	}

	/**
	 * 
	 */
	private IContentProvider getDynamicProvider(String path, String remainder)
		throws IOException {
		synchronized (providers) {
			// check to see if we already know the answer to this question.
			Object result = providers.get(path);
			if (result == NO_PROVIDER_AVAILABLE)
				return null;
			else if (result instanceof IContentProvider)
				return (IContentProvider) result;

			try {
				result = buildDynamicProvider(path, remainder);
				return (IContentProvider) result;
			} finally {
				providers.put(
					path,
					result == null ? NO_PROVIDER_AVAILABLE : result);
			}
		}
	}

	/** Create a dynamic content provider (IServlet or IContentProvider)
	 */
	private IContentProvider buildDynamicProvider(
		String path,
		String remainder)
		throws URLPluginException {

		// Calculate the name of the class that would be used to satisfy the
		// given request 
		String className;

		if (contentClassName != null) {
			// if this provider only provides a single class, but a different
			// class is being requested, return null.
			if (path.equals(pathClassName))
				className = contentClassName;
			else
				return null;

		} else {
			className = contentPackageName + "." + remainder.replace('/', '.');
		}

		try {
			IContentProvider result;

			// Look up the class
			Class c = classLoader.loadClass(className);

			// if the class is an IContentProvider, create and return a new
			// instance of it
			if (IContentProvider.class.isAssignableFrom(c)) {
				result = (IContentProvider) c.newInstance();
				result.init(runtime);
				return result;
			}

			// if the class is an IServlet, wrap it with a
			// DefaultContentProvider    
			else if (IServlet.class.isAssignableFrom(c)) {
				result = new CDServletContentProvider(c);
				result.init(runtime);
				return result;
			}

			// Otherwise, the class is not fit to be an IContentProvider.
			else
				throw new URLPluginException(
					"Invalid content provider class: '" + className + "'");

		} catch (URLPluginException upe) {
			throw upe;

		} catch (ClassNotFoundException cnfe) {
			throw new URLPluginException(
				"Missing content provider class: '" + className + "'");

		} catch (Throwable t) {
			throw new URLPluginException();
		}
	}

	/** Sorts provider extensions in descending order.
	 * 
	 * Provider extensions at the front of a sorted list should be preferred
	 * over provider extenstions at the end of a sorted list. 
	 */
	public int compareTo(Object o) {
		ProviderExtension that = (ProviderExtension) o;

		// first, compare the path prefixes.
		int result = that.pathPrefix.length() - this.pathPrefix.length();
		if (result != 0)
			return result;

		// next, compare the path IDs.
		result = 0 - this.pathID.compareTo(that.pathID);
		if (result != 0)
			return result;

		// finally, compare the plugin IDs.
		return this.pluginID.compareTo(that.pluginID);
	}
}
