package gov.va.med.hds.cd.url.internal;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPlugin;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.url.IContentProvider;
import gov.va.med.hds.cd.url.IURLPlugin;
import gov.va.med.hds.cd.url.URLPluginException;
import gov.va.med.hds.cd.url.internal.cdtp.Handler;

public class URLPlugin implements IPlugin, IURLPlugin {

    private IRuntimeContext runtime;
    private List providerExtensions;

    public URLPlugin() {
    }

    public void startup(IRuntimeContext runtime, IPluginDescriptor plugin)
        throws Exception {
        this.runtime = runtime;

        init();
        Handler.install(runtime, this);
    }

    public void shutdown() throws Exception {
    }

    
    /** Find all the extensions to our extension point that plugins have
     * declared, and add them to our class name map.
     */
    private void init() throws URLPluginException {
        IPluginRegistry pluginRegistry = runtime.getPluginRegistry();
        IConfigurationElement[] ce =
            pluginRegistry.getConfigurationElementsFor(
                IURLPlugin.CONTENT_PROVIDER_EXTENSION_POINT_ID);
                
        Map maxVersions = getMaxVersions(ce);

        providerExtensions = new LinkedList();
        for (int i = 0; i < ce.length; i++)
            if (isMaxVersion(maxVersions, ce[i]))
                providerExtensions.add(new ProviderExtension(runtime, ce[i]));
        
        Collections.sort(providerExtensions);    
        providerExtensions = Collections.unmodifiableList(providerExtensions);
    }

	private boolean isMaxVersion(Map maxVersions, IConfigurationElement ce) {
		IPluginDescriptor plugin = ce.getDeclaringExtension().getDeclaringPluginDescriptor();
		String pluginID = plugin.getUniqueIdentifier();
		Version version = plugin.getVersion();
		Version maxVersion = (Version) maxVersions.get(pluginID);
		return version.isPerfect(maxVersion);
	}
    


    private Map getMaxVersions(IConfigurationElement[] ce) {
        HashMap result = new HashMap();
        for (int i = 0; i < ce.length; i++) {
            IPluginDescriptor plugin = ce[i].getDeclaringExtension().getDeclaringPluginDescriptor();
			String pluginID = plugin.getUniqueIdentifier();
            Version version = plugin.getVersion();
            Version maxVersion = (Version) result.get(pluginID);
            if (maxVersion == null || version.isGreaterThan(maxVersion))
                result.put(pluginID, version);
		}
        
		return result;
	}
    
    

	public URLConnection openConnection(URL u) throws IOException {
        String path = u.getPath();
        
        List possibleProviders = new LinkedList();
        Iterator i = providerExtensions.iterator();
        while (i.hasNext()) {
            ProviderExtension ext = (ProviderExtension) i.next();
            if (ext.matches(path))
                possibleProviders.add(ext);
        }

        IOException ioe = null;        
        i = possibleProviders.iterator();
        while (i.hasNext()) try {
            ProviderExtension ext = (ProviderExtension) i.next();
            IContentProvider provider = ext.getContentProvider(path);
            if (provider == null)
                continue;
            
            String pathID = ext.translatePathToID(path);
            return provider.connect(runtime, u, pathID);
            
        } catch (IOException ioException) {
            if (ioe == null) 
                ioe = ioException;
        }
        
        if (ioe != null)
            throw ioe;
        else
            throw new URLPluginException("No content provider exists for the URL " + u);
    }

}
