package gov.va.med.hds.cd.url.test;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.Random;

import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.test.RuntimeTestCase;
import gov.va.med.hds.util.io.StreamUtilities;

public class ContentProviderTest extends RuntimeTestCase {

    public ContentProviderTest(String arg0)
        throws PluginStartupException, PluginNotFoundException {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
        parseRegistryResource("registry.xml");
        try {
        	getRuntime().getPluginInstance("gov.va.med.hds.cd.url.test.mockplugin");
        } catch (PluginStartupException e) {
        }
//        getRuntime().run(new String[] { "-m", "gov.va.med.hds.cd.url.test.mockplugin" });        
    }

    static final String BASE = "cdtp:///gov/va/med/hds/cd/url/test";
    static final String URL1 = BASE + "/Dummy?some&query=string";
    static final String URL2 = BASE + "/registry.xml";
    static final String URL3 = BASE + "/DummyContentProvider?foo=bar";

    public void testParseURL() throws IOException {
        URL base = new URL(URL1);
        URL derived = new URL(base, "registry.xml");
        URL expected = new URL(URL2);
        assertEquals(expected.toString(), derived.toString());
    }

    public void testURLConnection() throws IOException {
        URL u = new URL(URL1);
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(u.toString(), result);
    }

    public void testClasspathConnection() throws IOException {
        URL u = new URL(URL2);
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));

        String expected =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    getClass().getResourceAsStream("registry.xml"),
                    true));
        assertEquals(expected, result);
    }
    
    public void testPluginVersioning() throws IOException {
        URL u = new URL(URL3);
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(u.toString(), result);
    }

    public static final String SYNC_SERVLET_URL = BASE + "/DummyServletProvider";
    public static final String ASYNC_SERVLET_URL = SYNC_SERVLET_URL + "?async";

    public void testSyncServletProvider() throws IOException {
        URL u = new URL(SYNC_SERVLET_URL);
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(SyncDummyServlet.EXPECTED_OUTPUT, result);
        assertEquals(
            SyncDummyServlet.EXPECTED_CONTENT_TYPE,
            conn.getContentType());
    }

    public void testAsyncServletProvider() throws IOException {
        URL u = new URL(ASYNC_SERVLET_URL);
        URLConnection conn = u.openConnection();
        conn.setDoOutput(true);
        conn.getOutputStream().write("body=TheBody".getBytes());
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(AsyncDummyServlet.EXPECTED_OUTPUT, result);
        assertEquals(
            AsyncDummyServlet.EXPECTED_CONTENT_TYPE,
            conn.getContentType());
    }

    public static final String SYNC_SERVLET_URL2 = BASE + "/SyncDummyServlet";
    public static final String ASYNC_SERVLET_URL2 = BASE + "/AsyncDummyServlet";

    public void testSyncServlet() throws IOException {
        URL u = new URL(SYNC_SERVLET_URL2);
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(SyncDummyServlet.EXPECTED_OUTPUT, result);
        assertEquals(
            SyncDummyServlet.EXPECTED_CONTENT_TYPE,
            conn.getContentType());
    }

    public void testAsyncServlet() throws IOException {
        URL u = new URL(ASYNC_SERVLET_URL2);
        URLConnection conn = u.openConnection();
        conn.setDoOutput(true);
        conn.getOutputStream().write("body=TheBody".getBytes());
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(AsyncDummyServlet.EXPECTED_OUTPUT, result);
        assertEquals(
            AsyncDummyServlet.EXPECTED_CONTENT_TYPE,
            conn.getContentType());
    }

    public void testServletClasspathConnection() throws IOException {
        URL base = new URL(ASYNC_SERVLET_URL2);
        URL u = new URL(base, "registry.xml");
        URLConnection conn = u.openConnection();
        String result =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));

        String expected =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    getClass().getResourceAsStream("registry.xml"),
                    true));
        assertEquals(expected, result);
    }
    

    static final String ECHO_SERVLET_URL = BASE + "/AsyncEchoServlet";
    static final int ECHO_LENGTH = 10000;
    static final String ECHO_CHARS = 
        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789._";
    
    public void testPostServlet() throws IOException {
        URL u = new URL(ECHO_SERVLET_URL);
        URLConnection conn = u.openConnection();

        StringBuffer buf = new StringBuffer();
        Random r = new Random();
        for (int i = ECHO_LENGTH;   i-- > 0; )
            buf.append(ECHO_CHARS.charAt(r.nextInt() & 63));
        String originalContent = buf.toString();

        conn.setDoOutput(true);
        PrintWriter out = 
            new PrintWriter(new OutputStreamWriter(conn.getOutputStream()));
        out.print("contentType=application%2foctet-stream&echo=");
        out.print(originalContent);
        out.flush();
        out.close();
        
        String contentType = conn.getContentType();
        assertEquals("application/octet-stream", contentType);
        
        String contentReturned =
            new String(
                StreamUtilities.convertStreamToByteArray(
                    conn.getInputStream()));
        assertEquals(originalContent, contentReturned);
    }
}
