package gov.va.med.hds.cd.url;

import java.io.IOException;

/**
 * Interface for an embedded servlet.
 * 
 * The <code>IServlet</code> interface is modeled after the Java Servlets API
 * in the J2EE platform.  It gives plugin developers the power to build
 * applications using the familiar web application paradigm, without requiring
 * a web server.
 * 
 * Unlike traditional web applications, applications written with
 * <code>IServlet</code> objects are not exposed to clients via the HTTP
 * protocol - they are only accessible within the desktop runtime environment.
 * 
 * Like traditional servlets, <code>IServlet</code> objects can generate
 * various types of multimedia content by appropriately setting the content
 * type on the response object.
 * 
 * After you write a class that implements <code>IServlet</code>, you must
 * register it via the <code>contentProvider</code> extension point for it to
 * be found and invoked by the embedded servlet container.
 */

public interface IServlet {


    /**
     * Called by the embedded servlet container to allow the servlet to respond
     * to a request.
     *  
     * Servlets typically run in a multithreaded environment, and may run
     * concurrently with other servlets and other user interface
     * components.  Servlet developers must be aware to synchronize access to
     * any shared resources such as files, network connections, and as well as
     * the servlet's static variables.
     * 
     * @param request the {@link IServletRequest} object that contains the
     * client's request
     * @param response the {@link IServletResponse} object that contains the
     * servlet's response
     * @throws IOException if an exception occurs that interferes with the
     * servlet's normal operation
     */
    public void service(IServletRequest request, IServletResponse response)
        throws IOException;

}
