package gov.va.med.hds.cd.url;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;

import gov.va.med.hds.util.io.HTMLUtil;

/** An object that servlets can use to send textual data to the client.  
 * 
 * This class extends the standard Java <code>PrintWriter</code> class with
 * additional methods that facilitate the writing of HTML and XML.
 */
public class ServletWriter extends PrintWriter {

    public ServletWriter(Writer out) {
        super(out);
    }

    public ServletWriter(Writer out, boolean autoFlush) {
        super(out, autoFlush);
    }

    public ServletWriter(OutputStream out) {
        super(out);
    }

    public ServletWriter(OutputStream out, boolean autoFlush) {
        super(out, autoFlush);
    }

    /** Prints a URL-encoded version of the given string.
     * 
     * For example, <pre>printAsURL("foo bar/baz")</pre> would print
     * <pre>"foo+bar%2fbaz"</pre> (without the quotes, of course).  
     * @see java.net.URLEncoder
	 */
    public void printAsURL(String s) {
        super.print(HTMLUtil.urlEncode(s));
    }
    

    /** Escapes HTML entities in the given string, and prints the result.
     * 
     * This method is ideal when you have an arbitrary string you would like
     * to appear verbatim in an HTML document.  For example,
     * <pre>printAsHTML ("special chars: <>&\"")</pre> would print
     * <pre>"special chars: &lt;&gt;&amp;&quot;"</pre> (without the outer
     * quotes, of course).
	 */
    public void printAsHTML(String s) {
        print(escapeEntities(s, false));
    }
    

    /** Escapes XML entities in the given string, and prints the result. 
     * 
     * This method is ideal when you have an arbitrary string you would like to
     * appear verbatim in an XML attribute. For example, 
     * <pre>printAsXML("special chars: <>&\"'")</pre> would print
     * <pre>"special chars: &lt;&gt;&amp;&quot;&apos;"</pre> (without the outer
     * quotes, of course).
     */
    public void printAsXML(String s) {
        print(escapeEntities(s, true));
    }



    /** Escape HTML and/or XML entities in a String.
     * 
     * @param value the <code>String</code> value to escape.
     * @param xml <code>true</code> to generate XML-compliant output,
     * <code>false</code> to generate output suitable for HTML.
     * @return String an escaped version of the original value.
     */
    public String escapeEntities(String value, boolean xml) {
        return HTMLUtil.escapeEntities(value, xml);
    }

}
