package gov.va.med.hds.util.m;

import gov.va.med.hds.cd.m.ICallRequest;
import gov.va.med.hds.cd.m.IMumps;
import gov.va.med.hds.cd.m.IMumpsFactory;
import gov.va.med.hds.cd.m.IMumpsListener;
import gov.va.med.hds.cd.m.MumpsException;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.util.dao.ISiteDAO;
import gov.va.med.hds.util.dao.SiteInfoRequestException;
import gov.va.med.hds.util.mumps.StringUtil;

/**
 * Data Access Object that gets site specific information
 */
public class SiteDAO implements ISiteDAO {
    private static final char PIECE_DELIMETER = '^';
	private static final int DIVISION_PIECE_NUMBER = 2;
    private static final int STATION_ID_PIECE_NUMBER = 3;
	private static final int INTEGRATION_PIECE_NUMBER = 4;
	
	private static final String SITE_RPC_KEY = "SITE INFO";
	private static final int TIME_TO_WAIT_FOR_RPC = 5000;
	private String divisionName = null;
    private String primaryStationID = null;
	private String integrationName = null;
	private boolean resultsReceived = false;
	private SiteInfoRequestException callException = null;
	/* (non-Javadoc)
	 * @see gov.va.med.hds.cd.runtime.IExecutableExtension#setInitializationData(gov.va.med.hds.cd.runtime.IRuntimeContext, gov.va.med.hds.cd.runtime.IConfigurationElement, java.lang.String)
	 */
	public void setInitializationData(IRuntimeContext runtime, IConfigurationElement config, String propertyName, Object data) throws Exception {
		IPluginDescriptor desc = runtime.getPluginRegistry().getPluginDescriptor(IUtilMConstants.PLUGIN_ID);
		IMumps mumps = ((IMumpsFactory) runtime.getPluginInstance(IMumpsFactory.PLUGIN_ID)).create(desc);
		mumps.callRPC(SITE_RPC_KEY, new IMumpsListener(){
			public void CallFailed(ICallRequest request, MumpsException e){
				callException = new SiteInfoRequestException("Call "+request.getCallName()+" failed",e);
				resultsReceived = true;
				synchronized (SiteDAO.this){
					SiteDAO.this.notify();
				}					
			}
			public void CallSucceeded(ICallRequest request, String result){
				divisionName = StringUtil.piece(result,PIECE_DELIMETER,DIVISION_PIECE_NUMBER);
                primaryStationID = StringUtil.piece(result,PIECE_DELIMETER,STATION_ID_PIECE_NUMBER);
				integrationName = StringUtil.piece(result,PIECE_DELIMETER,INTEGRATION_PIECE_NUMBER);
				resultsReceived = true;
				synchronized (SiteDAO.this){
					SiteDAO.this.notify();
				}					
			}
		});
	}
	/* (non-Javadoc)
	 * @see gov.va.med.hds.util.dao.ISiteDAO#getDivisionName()
	 */
	public String getDivisionName() throws SiteInfoRequestException {
		checkResultsReceived();
		return divisionName;
	}
    /* (non-Javadoc)
     * @see gov.va.med.hds.util.dao.ISiteDAO#getPrimaryStationID()
     */
    public String getPrimaryStationID() throws SiteInfoRequestException {
        checkResultsReceived();
        return primaryStationID;
    }
	/* (non-Javadoc)
	 * @see gov.va.med.hds.util.dao.ISiteDAO#getIntegrationName()
	 */
	public String getIntegrationName() throws SiteInfoRequestException {
		checkResultsReceived();
		return integrationName;
	}
	/* (non-Javadoc)
	 * @see gov.va.med.hds.util.dao.ISiteDAO#getSiteName()
	 */
	synchronized private void checkResultsReceived() throws SiteInfoRequestException {
		if(!resultsReceived){
			try {
				this.wait(TIME_TO_WAIT_FOR_RPC);
			} catch (InterruptedException e) {
				throw new SiteInfoRequestException(e);
			}
		}
		if(callException != null)
			throw callException;
		if(divisionName == null || integrationName==null)
			throw new SiteInfoRequestException("Call "+SITE_RPC_KEY+" timed out");
	}
}
