package gov.va.med.hds.util.test;

import gov.va.med.hds.util.date.RelativeDate;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

import junit.framework.TestCase;

/**
 * RelativeDate tests.
 */
public class RelativeDateTest extends TestCase {

	/**
	 * Constructor for RelativeDateTest.
	 * @param arg0
	 */
	public RelativeDateTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(RelativeDateTest.class);
	}

	public void testGetDateRangeDisplayText1() {
		Date now = new Date();
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(now);
		SimpleDateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy");
		assertEquals("RelativeDate.getDateRangeDisplayText failed:","through " + dateFormat.format(now),RelativeDate.getDateRangeDisplayText(now, ":T"));
		assertEquals("RelativeDate.getDateRangeDisplayText failed:","from " + dateFormat.format(now),RelativeDate.getDateRangeDisplayText(now, "T:"));
		assertEquals("RelativeDate.getDateRangeDisplayText failed:","during the Past 24 Hours",RelativeDate.getDateRangeDisplayText(now, "T-1:T"));
		assertEquals("RelativeDate.getDateRangeDisplayText failed:","during the Past Week",RelativeDate.getDateRangeDisplayText(now, "T-7:T"));
		cal.setTime(now);
		cal.add(GregorianCalendar.DATE,-17);
		String pastString = dateFormat.format(cal.getTime());
		cal.add(GregorianCalendar.DATE,20);
		String futureString = dateFormat.format(cal.getTime());
		assertEquals("RelativeDate.getDateRangeDisplayText failed:","from " + pastString + " through " + futureString,RelativeDate.getDateRangeDisplayText(now, "T-17:T+3"));
		
	}

	public void testGetRelativeDateString() {
		Date sampleDate = new Date();
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(sampleDate);
		cal.add(GregorianCalendar.DATE, -7);
		String result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T-7",result);
		cal.add(GregorianCalendar.DATE, +14);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T+7",result);
		cal.add(GregorianCalendar.DATE, -7);
		cal.add(GregorianCalendar.DATE, +100);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T+100",result);
		cal.add(GregorianCalendar.DATE, -200);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T-100",result);
		cal.add(GregorianCalendar.DATE, +100);
		cal.add(GregorianCalendar.DATE, +1000);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T+1000",result);
		cal.add(GregorianCalendar.DATE, -2000);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T-1000",result);
		cal.add(GregorianCalendar.DATE, +11000);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T+10000",result);
		cal.add(GregorianCalendar.DATE, -20000);
		result = RelativeDate.getRelativeDateString(sampleDate, cal.getTime());
		assertEquals("RelativeDate.getRelativeDateString failed:","T-10000",result);
		
	}

	public void testGetNumberOfDaysInYear() {
		assertEquals("RelativeDate.getNumberOfDaysInYear failed:",365,RelativeDate.getNumberOfDaysInYear(2003));
		assertEquals("RelativeDate.getNumberOfDaysInYear failed:",366,RelativeDate.getNumberOfDaysInYear(2000));
		assertEquals("RelativeDate.getNumberOfDaysInYear failed:",366,RelativeDate.getNumberOfDaysInYear(1996));
	}

	public void testGetRelativeToDate() {
		Date today = new Date();
		
		String todayRelative = "T";
		
		String yesterday = "T-1";
		String lastWeek  = "T-7";
		String lastMonth = "T-30";
		String lastYear  = "T-730";
		
		String tomorrow  = "T+1";
		String nextWeek  = "T+7";
		String nextMonth = "T+30";
		String nextYear  = "T+730";

		Date date = new Date();
		assertTrue("RelativeDate.getRelativeToDate failed for a past date.",today.after(RelativeDate.getRelativeToDate(date,yesterday)));
		assertTrue("RelativeDate.getRelativeToDate failed for a past date.",today.after(RelativeDate.getRelativeToDate(date, lastWeek)));
		assertTrue("RelativeDate.getRelativeToDate failed for a past date.",today.after(RelativeDate.getRelativeToDate(date, lastMonth)));
		assertTrue("RelativeDate.getRelativeToDate failed for a past date.",today.after(RelativeDate.getRelativeToDate(date, lastYear)));
		
		assertTrue("RelativeDate.getRelativeToDate failed for a future date.",today.before(RelativeDate.getRelativeToDate(date, tomorrow)));
		assertTrue("RelativeDate.getRelativeToDate failed for a future date.",today.before(RelativeDate.getRelativeToDate(date, nextWeek)));
		assertTrue("RelativeDate.getRelativeToDate failed for a future date.",today.before(RelativeDate.getRelativeToDate(date, nextMonth)));
		assertTrue("RelativeDate.getRelativeToDate failed for a future date.",today.before(RelativeDate.getRelativeToDate(date, nextYear)));
		
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(today);
		int todayYear = cal.get(GregorianCalendar.YEAR);
		int todayMonth = cal.get(GregorianCalendar.MONTH);
		int todayDate = cal.get(GregorianCalendar.DAY_OF_MONTH);
		cal.setTime(RelativeDate.getRelativeToDate(date, todayRelative));
		assertEquals("RelativeDate.getRelativeToDate failed:",todayYear,cal.get(GregorianCalendar.YEAR));
		assertEquals("RelativeDate.getRelativeToDate failed:",todayMonth,cal.get(GregorianCalendar.MONTH));
		assertEquals("RelativeDate.getRelativeToDate failed:",todayDate,cal.get(GregorianCalendar.DAY_OF_MONTH));
	}

	public void testGetRelativeTextForForwardYears() {
		String relativeDate = "YC+1Q2";
		Date refDate = new Date();
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy");
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(refDate);
		calendar.add(GregorianCalendar.YEAR,1);
		assertEquals("during 2nd Quarter "+dateFormat.format(calendar.getTime()),RelativeDate.getDateRangeDisplayText(refDate,relativeDate));
	}
	
}
