package gov.va.med.hds.util.test;

import gov.va.med.hds.util.mumps.StringUtil;
import java.util.GregorianCalendar;
import junit.framework.TestCase;

/**
 * Test for the StringUtil class.
 */
public class StringUtilTest extends TestCase {

	/**
	 * Constructor for StringUtilTest.
	 * @param name
	 */
	public StringUtilTest(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(StringUtilTest.class);
	}

	public void testPieceWithEmptyString() {
		String empty = "";
		String p = StringUtil.piece(empty,'^',1);
		assertEquals("StringUtil.piece failed: Should have empty string:","",p);
		p = StringUtil.piece(empty,'*',10);
		assertEquals("StringUtil.piece failed: Should have empty string:","",p);
	}

	public void testPieceWithoutMatch() {
		String data = "1234^5678";
		String p = StringUtil.piece(data,'*',2);
		assertEquals("StringUtil.piece failed: Should have empty string:","",p);
		p = StringUtil.piece(data,'^',10);
		assertEquals("StringUtil.piece failed: Should have empty string:","",p);
		p = StringUtil.piece(data,'^',3);
		assertEquals("StringUtil.piece failed: Should have empty string:","",p);
	}
	
	public void testPieceMatch() {
		String data = "abcd%efgh%ijkl%mnop%qrst%uvwx%yz";
		String p = StringUtil.piece(data, '*', 1);
		assertEquals("StringUtil.piece failed: Wrong result string:",data,p);
		p = StringUtil.piece(data,'%',1);
		assertEquals("StringUtil.piece failed: Wrong result string:","abcd",p);
		p = StringUtil.piece(data,'%',3);
		assertEquals("StringUtil.piece failed: Wrong result string:","ijkl",p);
		p = StringUtil.piece(data,'%',7);
		assertEquals("StringUtil.piece failed: Wrong result string:","yz",p);
		data = "abcd%efgh%ijkl%mn^op%qrst%uvwx%yz";
		p = StringUtil.piece(StringUtil.piece(data, '^', 1), '%', 4);
		assertEquals("StringUtil.piece failed: Wrong result string:","mn",p);
	}
	
	public void testLength() {
		int p = StringUtil.length("abcd^efgh^ijkl^mnop^qrst^uvwx^yz", '^');		
		assertEquals("StringUtil.length failed: Wrong result string:", 7, p);
		p = StringUtil.length("^^^^^^", '^');		
		assertEquals("StringUtil.length failed: Wrong result string:", 7, p);
		p = StringUtil.length("abcd", '^');		
		assertEquals("StringUtil.length failed: Wrong result string:", 1, p);
		p = StringUtil.length("", '^');		
		assertEquals("StringUtil.length failed: Wrong result string:", 1, p);
		p = StringUtil.length("^abcd^", '^');		
		assertEquals("StringUtil.length failed: Wrong result string:", 3, p);
	}

	public void testSplit() {
		String [] pieces = new String [7];
		String data = "abcd^efgh^ijkl^mnop^qrst^uvwx^yz";
		pieces = StringUtil.split(data, '^');
		assertEquals("StringUtil.split failed: Wrong result string:", "abcd", pieces[0]);
		assertEquals("StringUtil.split failed: Wrong result string:", "efgh", pieces[1]);
		assertEquals("StringUtil.split failed: Wrong result string:", "ijkl", pieces[2]);
		assertEquals("StringUtil.split failed: Wrong result string:", "mnop", pieces[3]);
		assertEquals("StringUtil.split failed: Wrong result string:", "qrst", pieces[4]);
		assertEquals("StringUtil.split failed: Wrong result string:", "uvwx", pieces[5]);
		assertEquals("StringUtil.split failed: Wrong result string:", "yz", pieces[6]);
		data = "abcd^efgh^ijkl^^^^";
		pieces = StringUtil.split(data, '^');
		assertEquals("StringUtil.split failed: Wrong result string:", "abcd", pieces[0]);
		assertEquals("StringUtil.split failed: Wrong result string:", "efgh", pieces[1]);
		assertEquals("StringUtil.split failed: Wrong result string:", "ijkl", pieces[2]);
		assertEquals("StringUtil.split failed: Wrong result string:", "", pieces[3]);
		assertEquals("StringUtil.split failed: Wrong result string:", "", pieces[4]);
		assertEquals("StringUtil.split failed: Wrong result string:", "", pieces[5]);
		assertEquals("StringUtil.split failed: Wrong result string:", "", pieces[6]);
		data = "abcd^efgh^ijkl^mnop^qrst^uvwx^yz";
		pieces = StringUtil.split(data, '\t');
		assertEquals("StringUtil.split failed: Wrong result string:", data, pieces[0]);
	}
		
	public void testMultiplePiece() {
		String data = "abcd^efgh^ijkl^mnop^qrst^uvwx^yz";
		String p = StringUtil.piece(data, '*', 1, 3);
		assertEquals("StringUtil.piece failed: Wrong result string:",data,p);
		p = StringUtil.piece(data,'^',1,3);
		assertEquals("StringUtil.piece failed: Wrong result string:","abcd^efgh^ijkl",p);
		p = StringUtil.piece(data,'^',3,4);
		assertEquals("StringUtil.piece failed: Wrong result string:","ijkl^mnop",p);
		p = StringUtil.piece(data,'^',7,7);
		assertEquals("StringUtil.piece failed: Wrong result string:","yz",p);
//		data = "abcd^efgh^^^^^";
//		p = StringUtil.piece(data,'^',3,5);
//		assertEquals("StringUtil.piece failed: Wrong result string:","^^",p);
	}
	
	public void testSetPiece() {
		String data = "abcd^efgh^ijkl^mnop^qrst^uvwx^yz";
		String p = StringUtil.setPiece(data, '*', 3, "foo");
		assertEquals("StringUtil.piece failed: Wrong result string:",data + '*' + '*' + "foo",p);
		p = StringUtil.setPiece(data, '^', 3, "foo");
		assertEquals("StringUtil.piece failed: Wrong result string:","abcd^efgh^foo^mnop^qrst^uvwx^yz",p);
		p = StringUtil.setPiece(data, '^', 10, "foo");
		assertEquals("StringUtil.piece failed: Wrong result string:","abcd^efgh^ijkl^mnop^qrst^uvwx^yz^^^foo",p);
		data = "";
		p = StringUtil.setPiece(data,'^',7,"");
		assertEquals("StringUtil.piece failed: Wrong result string:","^^^^^^",p);
	}

	public void testFMDateToDate() {
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(StringUtil.fmDateToDate("3000101.212121"));
		assertEquals("StringUtil.fmDateToDate failed:",1,calendar.get(GregorianCalendar.DAY_OF_MONTH));
		assertEquals("StringUtil.fmDateToDate failed:",2000,calendar.get(GregorianCalendar.YEAR));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.JANUARY,calendar.get(GregorianCalendar.MONTH));
		
		assertEquals("StringUtil.fmDateToDate failed:",9,calendar.get(GregorianCalendar.HOUR));
		assertEquals("StringUtil.fmDateToDate failed:",21,calendar.get(GregorianCalendar.MINUTE));
		assertEquals("StringUtil.fmDateToDate failed:",21,calendar.get(GregorianCalendar.SECOND));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.PM,calendar.get(GregorianCalendar.AM_PM));

		calendar.setTime(StringUtil.fmDateToDate("2901231.235959"));
		assertEquals("StringUtil.fmDateToDate failed:",31,calendar.get(GregorianCalendar.DAY_OF_MONTH));
		assertEquals("StringUtil.fmDateToDate failed:",1990,calendar.get(GregorianCalendar.YEAR));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.DECEMBER,calendar.get(GregorianCalendar.MONTH));
		
		assertEquals("StringUtil.fmDateToDate failed:",11,calendar.get(GregorianCalendar.HOUR));
		assertEquals("StringUtil.fmDateToDate failed:",59,calendar.get(GregorianCalendar.MINUTE));
		assertEquals("StringUtil.fmDateToDate failed:",59,calendar.get(GregorianCalendar.SECOND));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.PM,calendar.get(GregorianCalendar.AM_PM));
		
		calendar.setTime(StringUtil.fmDateToDate("3020201.09"));
		assertEquals("StringUtil.fmDateToDate failed:",1,calendar.get(GregorianCalendar.DAY_OF_MONTH));
		assertEquals("StringUtil.fmDateToDate failed:",2002,calendar.get(GregorianCalendar.YEAR));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.FEBRUARY,calendar.get(GregorianCalendar.MONTH));
		
		assertEquals("StringUtil.fmDateToDate failed:",9,calendar.get(GregorianCalendar.HOUR));
		assertEquals("StringUtil.fmDateToDate failed:",0,calendar.get(GregorianCalendar.MINUTE));
		assertEquals("StringUtil.fmDateToDate failed:",0,calendar.get(GregorianCalendar.SECOND));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.AM,calendar.get(GregorianCalendar.AM_PM));
		
		calendar.setTime(StringUtil.fmDateToDate("3030731.120001"));
		assertEquals("StringUtil.fmDateToDate failed:",31,calendar.get(GregorianCalendar.DAY_OF_MONTH));
		assertEquals("StringUtil.fmDateToDate failed:",2003,calendar.get(GregorianCalendar.YEAR));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.JULY,calendar.get(GregorianCalendar.MONTH));
		
		assertEquals("StringUtil.fmDateToDate failed:",0,calendar.get(GregorianCalendar.HOUR));
		assertEquals("StringUtil.fmDateToDate failed:",0,calendar.get(GregorianCalendar.MINUTE));
		assertEquals("StringUtil.fmDateToDate failed:",1,calendar.get(GregorianCalendar.SECOND));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.PM,calendar.get(GregorianCalendar.AM_PM));
		
		calendar.setTime(StringUtil.fmDateToDate("3021231.24"));
		assertEquals("StringUtil.fmDateToDate failed:",31,calendar.get(GregorianCalendar.DAY_OF_MONTH));
		assertEquals("StringUtil.fmDateToDate failed:",2002,calendar.get(GregorianCalendar.YEAR));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.DECEMBER,calendar.get(GregorianCalendar.MONTH));
		
		assertEquals("StringUtil.fmDateToDate failed:",11,calendar.get(GregorianCalendar.HOUR));
		assertEquals("StringUtil.fmDateToDate failed:",59,calendar.get(GregorianCalendar.MINUTE));
		assertEquals("StringUtil.fmDateToDate failed:",59,calendar.get(GregorianCalendar.SECOND));
		assertEquals("StringUtil.fmDateToDate failed:",GregorianCalendar.PM,calendar.get(GregorianCalendar.AM_PM));
		
	}
	
	public void testToMixedCase() {
		String s1 = "12762891437892";
		assertEquals("StringUtil.toMixedCase failed:","12762891437892",StringUtil.toMixedCase(s1));
		s1 = "ThIs iS JeSt A tEsT.";
		assertEquals("StringUtil.toMixedCase failed:","This Is Jest A Test.",StringUtil.toMixedCase(s1));
		s1 = "ALLUPPERCASESHTUFF!@#$%^&*()_+=-0987654321`~";
		assertEquals("StringUtil.toMixedCase failed:","Alluppercaseshtuff!@#$%^&*()_+=-0987654321`~",StringUtil.toMixedCase(s1));
		s1 = "(all in the <family";
		assertEquals("StringUtil.toMixedCase failed:","(All In The <Family",StringUtil.toMixedCase(s1));
	}
	
	public void testDiffStartIgnoreCase() {
		String s1 = "123abcdEFGH";
		String s2 = "123abcdEFGH";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",-1,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "aBcDeFgH";
		s2 = "AbCdEfGh";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",-1,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "AbCdEfGh";
		s2 = "aBcDeFgH";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",-1,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "AbCdEfGhI";
		s2 = "aBcDeFgH";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",8,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "aBcDeFgHi";
		s2 = "AbCdEfGh";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",8,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "aBcDeFgH";
		s2 = "AbCdEfGhI";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",8,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "AbCdEfGh";
		s2 = "aBcDeFgHi";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",8,StringUtil.diffStartIgnoreCase(s1, s2));

		s1 = "123abcd";
		s2 = "123abcdEFGH";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",7,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "123abcdEFGH";
		s2 = "123abcd";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",7,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "1";
		s2 = "123abcd";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",1,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "";
		s2 = "123abcd";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",0,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "123abcd";
		s2 = "";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",0,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "123abcd";
		s2 = "qweuioweriuo";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",0,StringUtil.diffStartIgnoreCase(s1, s2));
		
		s1 = "qweuioweriuo";
		s2 = "123abcd";
		assertEquals("StringUtil.diffStartIgnoreCase failed:",0,StringUtil.diffStartIgnoreCase(s1, s2));
		
	}
	
}
