package gov.va.med.hds.util.collections;

/**
 * ArrayList based implementation of IFifo, The FIFO is 
 * initialized based on the intitial size passed into 
 * the constructor. The max size of the container never
 * changes.
 */
public class Fifo implements IFifo {
	private static final int EMPTY_INDEX = -1;
	private static final int MIN_SIZE = 1;
	private Object[] list = null;
	private int first=EMPTY_INDEX,last=EMPTY_INDEX;
	
	
	/**
	 * Maximum size constructor
	 * @param initialSize fixed max size of the queue
	 */
	public Fifo(int initialSize) {
		list = new Object[Math.max(initialSize,MIN_SIZE)];
	}
	/**
	 * @see gov.va.med.hds.common.fifo.IFifo#add(Object)
	 */
	public void add(Object o) throws FifoException{
		if(first == EMPTY_INDEX){
			list[++first] = o;
			++last;
		}
		else if(last < list.length-1){
			list[++last] = o;
		}
		else if(last - first < list.length-1){
			packListToTop();
			add(o);
		}
		else {
			throw new FifoException("Fifo is full. Can't add");
		}
	}

	/**
	 * @see gov.va.med.hds.common.fifo.IFifo#pop()
	 */
	public Object pop() throws FifoException {
		Object result = null;
		if(first == EMPTY_INDEX)
			throw new FifoException("Fifo is empty");
		else if(last==first)
		{
			result = list[first];
			first = EMPTY_INDEX;
			last = EMPTY_INDEX;
		}
		else{
			result = list[first++];
		}	
		return result;
	}

	/**
	 * @see gov.va.med.hds.common.fifo.IFifo#getCount()
	 */
	public long getCount() {
		if(first == EMPTY_INDEX)
			return 0;
		else if(first == last)
			return 1;
		else return last-first;
	}
	private void packListToTop() {
		int count = last-first+1;
		for(int i=0;i<count;i++)
			list[i] = list[i+first];
		first = 0;
		last = count-1;
	}
}
