package gov.va.med.hds.util.collections;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * @author vhaislmonsos
 *
 * Dec 13, 2002 1:13:03 PM
 */
public class ListUtil {

	/**
	 * Similar to Collections.copy, but the destination list should <i>not</i> be
	 * null, and it clobbers the destination lists items.
	 * @param source
	 * @param dest
	 */
	public static void copy(List source, List dest) {
		dest.clear();
		Iterator it = source.iterator();
		while (it.hasNext()) {
			dest.add(it.next());
		}
	}
	
	/**
	 * Compares the items in the list after sorting each list (every item in the 
	 * list should implement the Comparable interface.)
	 * @param list1
	 * @param list2
	 * @return boolean - true if the items in the lists match; false otherwise.
	 */
	public static boolean equals(List list1, List list2) {
		boolean result = false;
    
    // Check for null conditions.
    // If both are null they are equal
    if (null == list1 && null == list2){
      return true;
    }
    // If only one is null they are not equal
    if (null == list1 || null == list2){
      return result;
    }
    
    // There are two valid lists
		if (list1.size() != list2.size()) {
			return result;
		}
		result = true;
		Collections.sort(list1);
		Collections.sort(list2);
		Iterator it1 = list1.iterator();
		Iterator it2 = list2.iterator();
		while (it1.hasNext()) {
			if (((Comparable)it1.next()).compareTo((Comparable)it2.next())!=0) {
				result = false;
				break;
			}
		}
		
		return result;
	}

}
