package gov.va.med.hds.util.mumps;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * @author vhaislmonsos
 *
 * Oct 29, 2002 8:12:24 AM
 */
public class StringUtil {
	private static final char[] charSet = new char[] {' ','\'',',','-','.','/','^','(','<'};
	public static final char DEFAULT_SEPARATOR = '^';
	public static final String DEFAULT_SEPARATOR_REGEXP = "^";
	public static final byte[] cr_lf = new byte[]{0x0d, 0x0a};
	public static final String CR_LF = new String(cr_lf);
	
	static
	{
		Arrays.sort(charSet);
	}
	
	public static String piece(String string, char separator, int pieceNumber) {
		int	i = 1, 
				start = 0,
				next = string.indexOf(separator);
		String result = new String("");
				
		if (string.length() < 1) {
			return result;
		}
		
		while ((i < pieceNumber) && (next != -1)) {
			i++;
			start = next + 1;
			next = string.indexOf(separator, start);
		}
		
		if (next == -1) {
			next = string.length();
		}
		
		if (i >= pieceNumber) {
			result = new String(string.substring(start, next));
		}
		
		return result;
	}
	
	/**
	 * Method piece gets range of separator pieces from lowerBound to upperBound
	 **/
	public static String piece(String string, char separator, int lowerBound, int upperBound) {
		String [] pieces;

		if (lowerBound > upperBound) {
			return "";
		}
		
		//if string doesn't contain separator
		if (string.indexOf(separator) == -1) {
			if (lowerBound > 1)
				return "";
			if (lowerBound == 1)
				return string;
		}
		
		if (length(string, separator) < lowerBound)
			return "";
		
		pieces = split(string, separator);

		String result = "";
		
		for (int i=lowerBound; i < upperBound + 1; i++) {
			result = result + (i == lowerBound ? "" : String.valueOf(separator)) + pieces[i-1];
		}
		return result;
	}
	
	/**
	 * Method splits string into separator-pieces, returns String [] of pieces
	 * analogous to String.split, but not subject to limitations of regex and
	 * does NOT ignore trailing null-string pieces
	 **/
	public static String [] split(String string, char separator) {
		int pieceCount = length(string, separator);
		String [] pieces = new String [pieceCount];
		
		for (int i=0; i < pieceCount; i++) {
			pieces[i] = piece(string, separator, i + 1);
		}
		return pieces;
	}
	
	/**
	 * Method analogous to the two argument form of the MUMPS $LENGTH function
	 * finds the number of delimiter-pieces in the input string
	 **/
	public static int length(String string, char separator) {
		int pieceCount = 1;
		int index = string.indexOf(separator);
		
		if (index == -1)
			return pieceCount;
			
		while (index != -1) {
			pieceCount++;
			index = string.indexOf(separator, index + 1);
		}
		return pieceCount;
	}
	
	/**
	 * Method setPiece implements MUMPS set-positional
	 * (i.e., SET $P(string,separator,pieceNbr)=stringVal)
	 **/
	public static String setPiece(String string, char separator, int pieceNbr, String stringVal) {
		ArrayList pieceArray = new ArrayList();

		if (string.indexOf(separator) == -1) {
			pieceArray.add(0, string);
			for (int i=1; i < pieceNbr; i++) {
				pieceArray.add(i, "");
			}
			pieceArray.set(pieceNbr-1, stringVal);
		} else {
			String [] pieces;
			pieces = split(string, separator);
			for (int i=0; i < pieces.length; i++) {
				pieceArray.add(i, pieces[i]);
			}
			if (pieceArray.size() < pieceNbr) {
				for (int i=pieceArray.size(); i < pieceNbr; i++) {
					pieceArray.add(i, "");
				}
			}
			pieceArray.set(pieceNbr - 1, stringVal);
		}

		String result = "";
		
		for (int i=0; i < pieceArray.size(); i++) {
			result = result + (i == 0 ? "" : String.valueOf(separator)) + pieceArray.get(i);
		}
		return result;
	}
	
	/**
	 * Replaces placeholder tags in data with XML CDATA tags
	 * @param data
	 * @return String
	 */
	public static String restoreCDATATags(String data){
		String result = data.replaceAll("\\Q"+CDATA_START_PLACEHOLDER_TAG+"\\E",CDATA_START_TAG).replaceAll("\\Q"+CDATA_END_PLACEHOLDER_TAG+"\\E",CDATA_END_TAG);
		return result;
	}
	/**
	 * Replaces XML CDATA tags with place holder tags.
	 * This is useful when working with nested XML strings which my have nested CDATA tags.
	 * @param data
	 * @return String
	 */
	public static String replaceCDATATags(String data) {
		String result =	data.replaceAll("\\Q"+CDATA_START_TAG+"\\E",CDATA_START_PLACEHOLDER_TAG).replaceAll("\\Q"+CDATA_END_TAG+"\\E",CDATA_END_PLACEHOLDER_TAG);
		return result;
	}
	

	private static final String CDATA_START_TAG= "<![CDATA[";
	private static final String CDATA_START_PLACEHOLDER_TAG= "<!CDATASTART[";
	private static final String CDATA_END_TAG= "]]>";
	private static final String CDATA_END_PLACEHOLDER_TAG= "]CDATAEND]>";

	public static Date fmDateToDate(String filemanString) {
		String sDate,sTime = null;
		sDate = StringUtil.piece(filemanString, '.', 1);
		sTime = StringUtil.piece(filemanString, '.', 2);
		int year = Integer.parseInt(sDate.substring(0, 3))+1700;
		int month = Integer.parseInt(sDate.substring(3, 5))-1;
		int day = Integer.parseInt(sDate.substring(5, 7));
		if (sTime.equals("24")) {
			sTime = "235959";
		}
		if (sTime.length()<6) {
			while (sTime.length()<6) sTime += "0";
			DecimalFormat format = new DecimalFormat("000000");
			sTime = format.format(Integer.parseInt(sTime));
		}
		int hour = Integer.parseInt(sTime.substring(0, 2));
		int minute = Integer.parseInt(sTime.substring(2, 4));
		int second = Integer.parseInt(sTime.substring(4, 6));
		GregorianCalendar calendar = new GregorianCalendar(year,month,day,hour,minute,second); 
		return calendar.getTime();
//		function FMDateTimeToDateTime(ADateTime: TFMDateTime): TDateTime;
//		{ converts a Fileman date/time (type double) to a Delphi date/time }
//		var
//		  ADate, ATime: TDateTime;
//		  DatePart, TimePart: string;
//		begin
//		  DatePart := Piece(FloatToStrF(ADateTime, ffFixed, 14, 6), '.', 1);
//		  TimePart := Piece(FloatToStrF(ADateTime, ffFixed, 14, 6), '.', 2) + '000000';
//		  if Length(DatePart) <> 7 then raise EFMDateTimeError.Create('Invalid Fileman Date');
//		  if Copy(TimePart, 1, 2) = '24' then TimePart := '23595959';
//		  ADate := EncodeDate(StrToInt(Copy(DatePart, 1, 3)) + 1700,
//							  StrToInt(Copy(DatePart, 4, 2)),
//							  StrToInt(Copy(DatePart, 6, 2)));
//		  ATime := EncodeTime(StrToInt(Copy(TimePart, 1, 2)),
//							  StrToInt(Copy(TimePart, 3, 2)),
//							  StrToInt(Copy(TimePart, 5, 2)), 0);
//		  Result := ADate + ATime;
//		end;
	}
	
	public static String toMixedCase(String string) {
//Original Delphi code from ORFn.pas
//		function MixedCase(const x: string): string;
//		var
//		  i: integer;
//		begin
//		  Result := x;
//		  for i := 2 to Length(x) do
//			if (not (x[i-1] in [' ','''',',','-','.','/','^'])) and (x[i] in ['A'..'Z'])
//			  then Result[i] := Chr(Ord(x[i]) + 32);
//		end;
		if (string.length()<1) {
			return string;
		}
		String result = new String(string.toUpperCase());
		char[] newChars = result.toCharArray();
		newChars[0] = result.charAt(0);
		for (int i = 1; i < newChars.length; i++) {
			if ( (Arrays.binarySearch(charSet,newChars[i-1])<0) &&
					Character.isLetter(newChars[i]) )	{
				newChars[i] = Character.toLowerCase(newChars[i]);
			}
		}
		return String.copyValueOf(newChars);
	}
	
	/**
	 * Returns the position where 2 string begin to differ, ignoring case.  Both strings
	 * must be non-null.  If there are no differences, returns -1; otherwise, returns the starting
	 * differing position.  Note: if the strings are different length and the longer string starts with
	 * the entire shorter string, then the difference start position is shorterString.length().
	 * @param string1
	 * @param string2
	 * @return int
	 */
	public static int diffStartIgnoreCase(String s1, String s2) {
		int maxLength = Math.min(s1.length(), s2.length());
		if (maxLength<1) return 0;
		String t1 = s1.toLowerCase();
		String t2 = s2.toLowerCase();
		for (int i = 0; i < maxLength; i++) {
			if (t1.charAt(i)!=t2.charAt(i)) {
				return i;
			}
		}
		if (t1.length()==t2.length()) {
			return -1;
		}
		else {
			return maxLength;
		}
	}
}
