library OSUtils;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

{%File '..\..\..\..\..\..\..\..\..\..\..\Shared\DelphiJNI\PAS\JNI_MD.INC'}

uses
  Windows, Messages, ActiveX,
  JNI in '..\..\..\..\..\..\..\..\..\..\..\Shared\DelphiJNI\PAS\JNI.pas';

{$R *.res}

procedure Java_gov_va_med_hds_util_os_OSUtils_setForegroundWindow(
    PEnv: PJNIEnv; Obj: JObject; value: JLong);
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  handle: HWND;

begin
  handle := HWND(value);
  SetForegroundWindow(handle);
end;

procedure Java_gov_va_med_hds_util_os_OSUtils_restoreWindowIfMinimized(
    PEnv: PJNIEnv; Obj: JObject; value: JLong);
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  handle: HWND;
  owner: HWND;

begin
  handle := HWND(value);
  repeat
    owner := GetWindow(handle, GW_OWNER);
    if(owner <> 0) then
      handle := owner;
  until(owner = 0);
  if(IsIconic(handle)) then
    ShowWindow(handle, SW_RESTORE)
end;

procedure Java_gov_va_med_hds_util_os_OSUtils_setAndRestoreForegroundWindow(
    PEnv: PJNIEnv; Obj: JObject; value: JLong);
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
begin
  Java_gov_va_med_hds_util_os_OSUtils_restoreWindowIfMinimized(PEnv, Obj, value);
  Java_gov_va_med_hds_util_os_OSUtils_setForegroundWindow(PEnv, Obj, value);
end;

procedure Java_gov_va_med_hds_util_os_OSUtils_setStayOnTop(
    PEnv: PJNIEnv; Obj: JObject; value: JLong; stayOnTop: JBoolean);
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  handle: HWND;
  flag: HWND;

begin
  handle := HWND(value);
  if(stayOnTop = JNI_TRUE) then
    flag := HWND_TOPMOST
  else
    flag := HWND_NOTOPMOST;
  SetWindowPos(handle, flag, 0, 0, 0, 0, SWP_NOMOVE or
          SWP_NOSIZE or SWP_NOACTIVATE);
end;

procedure Java_gov_va_med_hds_util_os_OSUtils_displayAndStayOnTop(
    PEnv: PJNIEnv; Obj: JObject; value: JLong);
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
begin
  Java_gov_va_med_hds_util_os_OSUtils_setAndRestoreForegroundWindow(PEnv, Obj, value);
  Java_gov_va_med_hds_util_os_OSUtils_setStayOnTop(PEnv, Obj, value, JNI_TRUE);
end;

function Java_gov_va_med_hds_util_os_OSUtils_createGUID(
    PEnv: PJNIEnv; Obj: JObject): JString;
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  ClassID: TCLSID;
  P: PWideChar;
  GUID: String;
  JVM: TJNIEnv;

begin
  CoCreateGuid(ClassID);
  StringFromCLSID(ClassID, P);
  GUID := P;
  CoTaskMemFree(P);
  JVM := TJNIEnv.Create(PEnv);
  try
    Result := JVM.StringToJString(PAnsiChar(GUID));
  finally
    JVM.free;
  end;
end;

type
  PWindowInfo = ^TWindowInfo;
  TWindowInfo = record
    handle: HWND;
    text: String;
  end;

function WindowSearch(Handle: HWND; info: Pointer): BOOL; stdcall;
var
  Buffer: array[0..255] of char;
  Title: String;
  len: Integer;

begin
  len := GetWindowText(Handle, @Buffer, sizeof(Buffer));
  Buffer[len] := char(0);
  SetString(Title, Buffer, len);
  if(Title = PWindowInfo(info)^.text) then
  begin
    Result := False;
    PWindowInfo(info)^.handle := Handle;
  end
  else
    Result := True;
end;

function Java_gov_va_med_hds_util_os_OSUtils_findWindowByTitle(
    PEnv: PJNIEnv; Obj: JObject; text: JString): JLong;
    {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  Info: TWindowInfo;
  JVM: TJNIEnv;

begin
  JVM := TJNIEnv.Create(PEnv);
  try
    Info.handle := 0;
    Info.text := JVM.JStringToString(text);
    EnumWindows(@WindowSearch, Longint(@Info));
    Result := Info.handle;
  finally
    JVM.free;
  end;
end;


exports
  Java_gov_va_med_hds_util_os_OSUtils_setForegroundWindow,
  Java_gov_va_med_hds_util_os_OSUtils_restoreWindowIfMinimized,
  Java_gov_va_med_hds_util_os_OSUtils_setAndRestoreForegroundWindow,
  Java_gov_va_med_hds_util_os_OSUtils_setStayOnTop,
  Java_gov_va_med_hds_util_os_OSUtils_displayAndStayOnTop,
  Java_gov_va_med_hds_util_os_OSUtils_createGUID,
  Java_gov_va_med_hds_util_os_OSUtils_findWindowByTitle;

begin
end.
