/*
 * Created on Apr 10, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.va.med.hds.util.os;

import java.awt.Dialog;
import java.awt.Frame;

import javax.swing.SwingUtilities;

/**
 * General routines used to access Windows APIs
 */
public class OSUtils {

    /**
     * Calls the Windows API SetForegroundWindow, to allow a java application to 
     * bring a window to the front, even a window in another application, provided
     * the window handle is known
     * @param handle
     */
    public static native void setForegroundWindow(long handle);

    /**
     * Restores a window that has been minimized.
     * @param handle
     */
    public static native void restoreWindowIfMinimized(long handle);

    /**
     * Combines calls to restoreWindowIfMinimized and setForegroundWindow.  Runs faster than
     * seperate calls since the native boundry is crossed only once.
     * @param handle
     */
    public static native void setAndRestoreForegroundWindow(long handle);

    /**
     * Sets a window's stay on top status
     * @param handle - the handle of the windoe
     * @param stayOnTop - turns the stay on top status on or off
     */
    public static native void setStayOnTop(long handle, boolean stayOnTop);

    /**
     * Combines setAndRestoreForegroundWindow and setStayOnTop(handle, true)
     * @param handle
     */
    public static native void displayAndStayOnTop(long handle);

    public static long getParentWindowHandle(Dialog dialog) {
		Frame parent =
			(Frame) SwingUtilities.getAncestorOfClass(Frame.class, dialog);
        return getWindowHandle(parent);
    }

    public static long getWindowHandle(Frame frame) {
        String text = createGUID();
		long result;
        String oldTitle = frame.getTitle();
        frame.setTitle(text);
        try {
            result = findWindowByTitle(text);
        } finally {
            frame.setTitle(oldTitle);
        }
        return result;
    }

	/**
     * Returns a Windows GUID
     */
    public static native String createGUID();
	
    private static native long findWindowByTitle(String text);

    static {
        System.loadLibrary("OSUtils");
    }

}
