package gov.va.med.hds.cd.workspace.internal;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;


/** 
 * Implementation of the InvocationHandler which handles the basic
 * object methods.
 */
class DefaultInvoker implements InvocationHandler {

	public Object invoke(Object proxy, Method method, Object[] args)
		throws Throwable {

		if (method.getDeclaringClass() == Object.class) {
			String methodName = method.getName();
			if (methodName.equals("hashCode")) {
				return proxyHashCode(proxy);
			} else if (methodName.equals("equals")) {
				return proxyEquals(proxy, args[0]);
			} else if (methodName.equals("toString")) {
				return proxyToString(proxy);
			}
		}

		// Although listener methods are supposed to be void, we 

		// allow for any return type here and produce null/0/false

		// as appropriate.

		return DefaultInvoker.nullValueOf(method.getReturnType());
	}

	protected Integer proxyHashCode(Object proxy) {
		return new Integer(System.identityHashCode(proxy));
	}

	protected Boolean proxyEquals(Object proxy, Object other) {
		return (proxy == other ? Boolean.TRUE : Boolean.FALSE);
	}

	protected String proxyToString(Object proxy) {
		return proxy.getClass().getName()
			+ '@'
			+ Integer.toHexString(proxy.hashCode());
	}

	private final static Character char_0 = new Character((char) 0);
	private final static Byte byte_0 = new Byte((byte) 0);

	private final static Object nullValueOf(Class rt) {
		if (!rt.isPrimitive()) {
			return null;
		} else if (rt == void.class) {
			return null;
		} else if (rt == boolean.class) {
			return Boolean.FALSE;
		} else if (rt == char.class) {
			return DefaultInvoker.char_0;
		} else {
			// this will convert to any other kind of number
			return DefaultInvoker.byte_0;
		}
	}

}