package gov.va.med.hds.cd.workspace.internal;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class ModelRegistry {

	private Map modelsById = new HashMap();
	private Map modelsByScope = new HashMap();

	public void addModelDescriptor(IModelDescriptor model) {
		if (modelsById.containsKey(model.getId())) return;
		modelsById.put(model.getId(), model);
		addDescriptorByScope(model);
	}

	private void addDescriptorByScope(IModelDescriptor model) {
		List models = (List) modelsByScope.get(model.getScope());
		if (models == null) {
			models = new LinkedList();
			modelsByScope.put(model.getScope(), models);
		}
		models.add(model);
	}

	public void removeModelDescriptor(IModelDescriptor model) {
		modelsById.remove(model.getId());
	}
	
	public boolean contains(String modelId) {
		return modelsById.containsKey(modelId);
	}
	
	public IModelDescriptor getModelDescriptor(String modelId) {
		return (IModelDescriptor) modelsById.get(modelId);
	}
	
	public IModelDescriptor[] getModelDescriptorsInScope(String scopeId) {
		List models = (List) modelsByScope.get(scopeId);
		if (models == null || models.isEmpty()) return new IModelDescriptor[0];
		return (IModelDescriptor[]) models.toArray(new IModelDescriptor[models.size()]);
	}
}
