package gov.va.med.hds.cd.workspace.internal;

import java.text.MessageFormat;

import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.InvalidConfigurationElementException;
import gov.va.med.hds.cd.runtime.logging.ILogger;
import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;

public class ModelRegistryReader implements IWorkspaceConstants {
	
	private ILogger log;
	
	public ModelRegistryReader(ILoggingCapable logging) {
		log = logging.getLogger(getClass());
	}
	
	public ModelRegistry readModelRegistry(IPluginRegistry plugins) {
		ModelRegistry modelRegistry = new ModelRegistry();
		IConfigurationElement[] modelElements = plugins.getConfigurationElementsFor(UNIQUE_MODEL_EXTENSION_POINT_ID);
		for (int i = 0; i < modelElements.length; i++) {
			try {
				ModelDescriptor model = readModelDescriptor(modelElements[i]);
				modelRegistry.addModelDescriptor(model);
			} catch (InvalidConfigurationElementException e) {
				// TODO_SB: Auto-generated catch block
				e.printStackTrace();
			}
		}
		return modelRegistry;
	}

	private ModelDescriptor readModelDescriptor(IConfigurationElement element) throws InvalidConfigurationElementException {
		ModelDescriptor model = new ModelDescriptor(element);
		IConfigurationElement[] children = element.getChildren();
		for (int i = 0; i < children.length; i++) {
			readChildElement(children[i], model);
		}
		return model;
	}

	private void readChildElement(IConfigurationElement element, ModelDescriptor model) throws InvalidConfigurationElementException {
		if (SOURCE_CONFIGURATION_ELEMENT.equals(element.getName())) {
			readEventSourceDescriptor(element, model);
		} else if (LISTENER_CONFIGURATION_ELEMENT.equals(element.getName())) {
			readEventListenerDescriptor(element, model);
		} else {
			Object[] args = new Object[] { element.getName(), MODEL_CONFIGURATION_ELEMENT, UNIQUE_MODEL_EXTENSION_POINT_ID, element.getDeclaringExtension().getDeclaringPluginDescriptor() };
			String message = MessageFormat.format("unrecognized child element ''{0}'' of ''{1}'' element in extension to extension point ''{2}'' provided by plugin ''{3}''", args);
			throw new InvalidConfigurationElementException(message);
		}
	}
	
	private void readEventSourceDescriptor(IConfigurationElement source, ModelDescriptor model) {
		String eventClassName = source.getAttribute(EVENT_ATTRIBUTE);
		model.addSourceOfEvent(eventClassName);
	}
	
	private void readEventListenerDescriptor(IConfigurationElement listener, ModelDescriptor model) {
		String eventClassName = listener.getAttribute(EVENT_ATTRIBUTE);
		model.addListenerOfEvent(eventClassName);
	}
}
