package gov.va.med.hds.cd.workspace.test;

import java.beans.PropertyChangeEvent;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IExtension;
import gov.va.med.hds.cd.runtime.IExtensionPoint;
import gov.va.med.hds.cd.runtime.IPluginDescriptor;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.VersionMatchCondition;
import gov.va.med.hds.cd.runtime.internal.ConfigurationElement;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;
import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.cd.workspace.ListenerOfEventDescriptor;
import gov.va.med.hds.cd.workspace.SourceOfEventDescriptor;
import gov.va.med.hds.cd.workspace.internal.ModelRegistry;
import gov.va.med.hds.cd.workspace.internal.ModelRegistryReader;

public class ModelRegistryReaderTest extends TestCase implements IWorkspaceConstants {

	public ModelRegistryReaderTest(String name) {
		super(name);
	}

	private IConfigurationElement[] createValidModelElements() {
		ConfigurationElement[] models = new ConfigurationElement[3];
		models[0] = new ConfigurationElement(MODEL_CONFIGURATION_ELEMENT);
		models[0].addAttribute(ID_ATTRIBUTE, "model1");
		models[0].addAttribute(SCOPE_ATTRIBUTE, "scope1");
		models[0].addAttribute(CLASS_ATTRIBUTE, "some.class.name.that.is.a.model.1");
		models[1] = new ConfigurationElement(MODEL_CONFIGURATION_ELEMENT);
		models[1].addAttribute(ID_ATTRIBUTE, "model2");
		models[1].addAttribute(SCOPE_ATTRIBUTE, "scope2");
		models[1].addAttribute(CLASS_ATTRIBUTE, "some.class.name.that.is.a.model.2");
		ConfigurationElement listener = new ConfigurationElement(LISTENER_CONFIGURATION_ELEMENT);
		listener.addAttribute(EVENT_ATTRIBUTE, PropertyChangeEvent.class.getName());
		models[1].addChild(listener);
		models[2] = new ConfigurationElement(MODEL_CONFIGURATION_ELEMENT);
		models[2].addAttribute(ID_ATTRIBUTE, "model3");
		models[2].addAttribute(SCOPE_ATTRIBUTE, "scope3");
		models[2].addAttribute(CLASS_ATTRIBUTE, "some.class.name.that.is.a.model.3");
		ConfigurationElement source = new ConfigurationElement(SOURCE_CONFIGURATION_ELEMENT);
		source.addAttribute(EVENT_ATTRIBUTE, TelephoneEvent.class.getName());
		models[2].addChild(source);
		return models;
	}

	public void testReadValidModelRegistry() {
		ModelRegistryReader reader = new ModelRegistryReader(new Log4JLogging());
		ModelRegistry registry = reader.readModelRegistry(new MockPluginRegistry(createValidModelElements()));
		assertTrue(registry.contains("model1"));
		assertEquals("model1", registry.getModelDescriptor("model1").getId());
		assertEquals("scope1", registry.getModelDescriptor("model1").getScope());
		assertEquals("some.class.name.that.is.a.model.1", registry.getModelDescriptor("model1").getClassName());
		assertTrue(registry.contains("model2"));
		assertEquals("model2", registry.getModelDescriptor("model2").getId());
		assertEquals("scope2", registry.getModelDescriptor("model2").getScope());
		assertEquals("some.class.name.that.is.a.model.2", registry.getModelDescriptor("model2").getClassName());
		ListenerOfEventDescriptor[] listenerOfs = registry.getModelDescriptor("model2").getListenerOfEventDescriptors();
		assertEquals(1, listenerOfs.length);
		assertEquals(PropertyChangeEvent.class.getName(), listenerOfs[0].getEventClassName());
		assertTrue(registry.contains("model3"));
		assertEquals("model3", registry.getModelDescriptor("model3").getId());
		assertEquals("scope3", registry.getModelDescriptor("model3").getScope());
		assertEquals("some.class.name.that.is.a.model.3", registry.getModelDescriptor("model3").getClassName());
		SourceOfEventDescriptor[] sourceOfs = registry.getModelDescriptor("model3").getSourceOfEventDescriptors();
		assertEquals(1, sourceOfs.length);
		assertEquals(TelephoneEvent.class.getName(), sourceOfs[0].getEventClassName());
	}

	private class MockPluginRegistry implements IPluginRegistry {

		private IConfigurationElement[] elements;
		
		public MockPluginRegistry(IConfigurationElement[] elements) {
			this.elements = elements;
		}

		public IConfigurationElement[] getConfigurationElementsFor(String extensionPointId) {
			return elements;
		}

		public IConfigurationElement[] getConfigurationElementsFor(String pluginId, String extensionPointName) {
			return null;
		}

		public IConfigurationElement[] getConfigurationElementsFor(String pluginId, String extensionPointName, String extensionId) {
			return null;
		}

		public IExtension getExtension(String extensionPointId, String extensionId) {
			return null;
		}

		public IExtension getExtension(String pluginId, String extensionPointName, String extensionId) {
			return null;
		}

		public IExtension[] getExtensions(String extensionPointId) {
			return null;
		}

		public IExtension[] getExtensions(String pluginId, String extensionPointName) {
			return null;
		}

		public IExtensionPoint getExtensionPoint(String extensionPointId) {
			return null;
		}

		public IExtensionPoint getExtensionPoint(String pluginId, String extensionPointName) {
			return null;
		}

		public IExtensionPoint[] getExtensionPoints() {
			return null;
		}

		public IPluginDescriptor getPluginDescriptor(String pluginId) {
			return null;
		}

		public IPluginDescriptor getPluginDescriptor(String pluginId, Version version) {
			return null;
		}

		public IPluginDescriptor getPluginDescriptor(String pluginId, Version version, VersionMatchCondition matchCondition) {
			return null;
		}

		public IPluginDescriptor[] getPluginDescriptors() {
			return null;
		}

		public IPluginDescriptor[] getPluginDescriptors(String pluginId) {
			return null;
		}
	}
}
