package gov.va.med.hds.cd.workspace.test;

import gov.va.med.hds.cd.workspace.IWorkspaceConstants;
import gov.va.med.hds.cd.workspace.ListenerOfEventDescriptor;
import gov.va.med.hds.cd.workspace.SourceOfEventDescriptor;
import gov.va.med.hds.cd.workspace.internal.IModelDescriptor;
import gov.va.med.hds.cd.workspace.internal.ModelRegistry;

import junit.framework.TestCase;

public class ModelRegistryTest
	extends TestCase
	implements IWorkspaceConstants {

	private MockModelDescriptor model1 =
		new MockModelDescriptor("model1", SESSION_SCOPE);
	private MockModelDescriptor model2 =
		new MockModelDescriptor("model2", PATIENT_SCOPE);
	private MockModelDescriptor model3 =
		new MockModelDescriptor("model3", PATIENT_SCOPE);
	private MockModelDescriptor model4 =
		new MockModelDescriptor("model4", SESSION_SCOPE);
	private MockModelDescriptor model5 =
		new MockModelDescriptor("model5", PATIENT_SCOPE);
	private MockModelDescriptor model6 =
		new MockModelDescriptor("model6", PERSPECTIVE_SCOPE);

	public ModelRegistryTest(String name) {
		super(name);
	}

	public void testModelRegistry() {
		ModelRegistry registry = new ModelRegistry();
		registry.addModelDescriptor(model1);
		registry.addModelDescriptor(model2);
		registry.addModelDescriptor(model3);
		registry.addModelDescriptor(model4);
		registry.addModelDescriptor(model5);
		registry.addModelDescriptor(model6);
		assertTrue(registry.contains(model1.getId()));
		assertTrue(registry.contains(model2.getId()));
		assertTrue(registry.contains(model3.getId()));
		assertTrue(registry.contains(model4.getId()));
		assertTrue(registry.contains(model5.getId()));
		assertTrue(registry.contains(model6.getId()));
		IModelDescriptor[] modelsInSessionScope = registry.getModelDescriptorsInScope(SESSION_SCOPE);
		assertTrue(contains(modelsInSessionScope, model1));
		assertTrue(contains(modelsInSessionScope, model4));
		IModelDescriptor[] modelsInPatientScope = registry.getModelDescriptorsInScope(PATIENT_SCOPE);
		assertTrue(contains(modelsInPatientScope, model2));
		assertTrue(contains(modelsInPatientScope, model3));
		assertTrue(contains(modelsInPatientScope, model5));
		IModelDescriptor[] modelsInPerspectiveScope = registry.getModelDescriptorsInScope(PERSPECTIVE_SCOPE);
		assertTrue(contains(modelsInPerspectiveScope, model6));
	}

	private boolean contains(IModelDescriptor descriptors[], IModelDescriptor descriptor) {
		for (int i = 0; i < descriptors.length; i++) {
			if (descriptors[i] == descriptor) return true;
		}
		return false;
	}

	private class MockModelDescriptor implements IModelDescriptor {

		private String id;
		private String scope;

		public MockModelDescriptor(String id, String scope) {
			this.id = id;
			this.scope = scope;
		}

		public String getId() {
			return id;
		}

		public String getScope() {
			return scope;
		}

		public String getClassName() {
			return null;
		}

		public SourceOfEventDescriptor[] getSourceOfEventDescriptors() {
			return null;
		}

		public ListenerOfEventDescriptor[] getListenerOfEventDescriptors() {
			return null;
		}

		public Object createModel() throws Exception {
			return null;
		}

	}
}
