package gov.va.med.hds.cd.workspace.test;

import java.util.EventListener;

import junit.framework.TestCase;

import gov.va.med.hds.cd.runtime.logging.ILoggingCapable;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;
import gov.va.med.hds.cd.workspace.IModelSpace;
import gov.va.med.hds.cd.workspace.IMulticaster;
import gov.va.med.hds.cd.workspace.IWorkspaceScope;
import gov.va.med.hds.cd.workspace.InvalidEventListenerException;
import gov.va.med.hds.cd.workspace.InvalidEventSourceException;
import gov.va.med.hds.cd.workspace.internal.Multicaster;

public class MulticasterTest extends TestCase {

	private TelephoneEvent telephoneEvent;
	private boolean telephoneRang = false;
	private boolean telephoneAnswered = false;
	private ILoggingCapable logging = new Log4JLogging();

	protected void setUp() throws Exception {
		super.setUp();
	}

	public MulticasterTest(String name) {
		super(name);
	}

	public void testForwardEvent()
		throws InvalidEventSourceException, InvalidEventListenerException {
		Telephone source1 = new Telephone();
		ITelephoneListener listener = new TelephoneListener();
		Multicaster caster = new Multicaster(logging, new MockWorkspaceScope("testForwardEvent"));
		caster.addSource(TelephoneEvent.class, source1);
		caster.addListener(TelephoneEvent.class, listener);
		source1.fireRingEvent();
		assertEquals(source1, telephoneEvent.getSource());
		assertTrue(telephoneRang);
		Telephone source2 = new Telephone();
		caster.addSource(TelephoneEvent.class, source2);
		source2.fireAnsweredEvent();
		assertEquals(source2, telephoneEvent.getSource());
		assertTrue(telephoneAnswered);
	}

	public void testRemoveSource()
		throws InvalidEventSourceException, InvalidEventListenerException {
		Telephone source = new Telephone();
		ITelephoneListener listener = new TelephoneListener();
		Multicaster caster = new Multicaster(logging, new MockWorkspaceScope("testRemoveSource"));
		caster.addSource(TelephoneEvent.class, source);
		caster.addListener(TelephoneEvent.class, listener);
		caster.removeSource(TelephoneEvent.class, source);
		source.fireRingEvent();
		assertNull(telephoneEvent);
		assertFalse(telephoneRang);
	}

	public void testRemoveListener() throws InvalidEventSourceException, InvalidEventListenerException {
		Telephone source = new Telephone();
		ITelephoneListener listener = new TelephoneListener();
		Multicaster caster = new Multicaster(logging, new MockWorkspaceScope("testRemoveListener"));
		caster.addSource(TelephoneEvent.class, source);
		caster.addListener(TelephoneEvent.class, listener);
		caster.removeListener(TelephoneEvent.class, listener);
		source.fireRingEvent();
		assertNull(telephoneEvent);
		assertFalse(telephoneRang);
	}

	public void testInvalidSource() {
		Multicaster caster = new Multicaster(logging, new MockWorkspaceScope("testInvalidSource"));
		try {
			caster.addSource(TelephoneEvent.class, this);
			fail("expected invalid event source exception");
		} catch (InvalidEventSourceException e) {
		}
	}

	public void testInvalidListener() {
		Multicaster caster = new Multicaster(logging, new MockWorkspaceScope("testInvalidListener"));
		try {
			caster.addListener(TelephoneEvent.class, new EventListener() {});
			fail("expected invalid event listener exception");
		} catch (InvalidEventListenerException e) {
		}
	}

	private class MockWorkspaceScope implements IWorkspaceScope {

		private String id;

		public MockWorkspaceScope(String id) {
			this.id = id;
		}

		public String getId() {
			return id;
		}

		public IMulticaster getMulticaster() {
			return null;
		}

		public IModelSpace getModelSpace() {
			return null;
		}
	}

	private class TelephoneListener implements ITelephoneListener {

		/* @see gov.va.med.hds.cd.workspace.test.ITelephoneListener#telephoneRang(gov.va.med.hds.cd.workspace.test.TelephoneEvent) */
		public void telephoneRang(TelephoneEvent e) {
			telephoneEvent = e;
			telephoneRang = true;
		}

		/* @see gov.va.med.hds.cd.workspace.test.ITelephoneListener#telephoneAnswered(gov.va.med.hds.cd.workspace.test.TelephoneEvent) */
		public void telephoneAnswered(TelephoneEvent e) {
			telephoneEvent = e;
			telephoneAnswered = true;
		}

		/* @see gov.va.med.hds.cd.workspace.test.ITelephoneListener#telephoneHungUp(gov.va.med.hds.cd.workspace.test.TelephoneEvent) */
		public void telephoneHungUp(TelephoneEvent e) {
		}
	}
}
