package gov.va.med.hds.cd.workspace.test;

import junit.framework.TestCase;

import gov.va.med.hds.cd.mum.Version;
import gov.va.med.hds.cd.runtime.IConfigurationElement;
import gov.va.med.hds.cd.runtime.IPluginActivationListener;
import gov.va.med.hds.cd.runtime.IPluginRegistry;
import gov.va.med.hds.cd.runtime.IRuntimeContext;
import gov.va.med.hds.cd.runtime.PluginNotFoundException;
import gov.va.med.hds.cd.runtime.PluginStartupException;
import gov.va.med.hds.cd.runtime.logging.internal.Log4JLogging;
import gov.va.med.hds.cd.runtime.test.MockPluginRegistry;
import gov.va.med.hds.cd.workspace.IWorkspaceListener;
import gov.va.med.hds.cd.workspace.WorkspaceEvent;
import gov.va.med.hds.cd.workspace.internal.Workspace;

public class WorkspaceTest extends TestCase implements IWorkspaceListener {
	
	private WorkspaceEvent event;
	
	public WorkspaceTest(String name) {
		super(name);
	}

	public void testAddRemoveScopes() throws Exception {
		Workspace workspace = new Workspace();
		workspace.startup(new MockRuntime(), null);
		workspace.addWorkspaceListener(this);
		assertNull(workspace.getScope("a.test.scope"));
		workspace.addScope("a.test.scope");
		assertEquals("a.test.scope", event.getScope().getId());
		event = null;
		assertNotNull(workspace.getScope("a.test.scope"));
		workspace.removeScope("a.test.scope");
		assertEquals("a.test.scope", event.getScope().getId());
		assertNull(workspace.getScope("a.test.scope"));
	}
	
	public void workspaceScopeAdded(WorkspaceEvent e) {
		event = e;
	}

	public void workspaceScopeRemoved(WorkspaceEvent e) {
		event = e;
	}
	
	private static class MockRuntime extends Log4JLogging implements IRuntimeContext {

		public IConfigurationElement[] getSettings() {
			return null;
		}

		public IPluginRegistry getPluginRegistry() {
			MockPluginRegistry mockRegistry = new MockPluginRegistry();
			mockRegistry.setExpectedGetConfigurationElementsForStringCalls(1);
			mockRegistry.setupGetConfigurationElementsForString(new IConfigurationElement[0]);
			return mockRegistry;
		}

		public boolean isPluginActive(String pluginId) {
			return false;
		}

		public boolean isPluginActive(String pluginId, Version version) {
			return false;
		}

		public Object getPluginInstance(String pluginId) throws PluginStartupException, PluginNotFoundException {
			return null;
		}

		public Object getPluginInstance(String pluginId, Version version) throws PluginStartupException, PluginNotFoundException {
			return null;
		}

		public void addPluginActivationListener(IPluginActivationListener l) {
		}

		public void removePluginActivationListener(IPluginActivationListener l) {
		}
	}
}
