package junit.extensions.jfcunit.eventdata;

import org.w3c.dom.Element;

import junit.extensions.xml.XMLException;
import junit.extensions.xml.elements.AbstractTagHandler;
import junit.extensions.xml.XMLTagResourceBundle;
import junit.extensions.xml.IXMLTestCase;
import junit.extensions.jfcunit.xml.JFCXMLConstants;
import junit.extensions.jfcunit.xml.JFCXMLTestCase;

/**
 * This class will handle the processing of &lt;click&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class ClickTagHandler extends AbstractTagHandler implements JFCXMLConstants {
    /**
     * Constructor for ClickTagHandler.
     *
     * @param element     The element to be processed
     * @param testCase    The IXMLTestCase that uses this element
     */
    public ClickTagHandler(Element element, IXMLTestCase testCase) {
        super(element, testCase);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // Only one of the following is required.
        checkRequiredAttribute(TYPE);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        BaseEventDataTagHandler th = (BaseEventDataTagHandler)
            XMLTagResourceBundle.getTagHandler(getElement(), getTestCase(), getType());
        ((JFCXMLTestCase)getTestCase()).getHelper().enterClickAndLeave(th.getEventData());
    }

    /**
     * Returns the value of the TYPE attribute for this element.
     * @return String  The value of the TYPE attribute.
     */
    protected String getType() {
        return getString(TYPE);
    }

}