package junit.extensions.jfcunit.eventdata;

import java.awt.event.MouseEvent;
import junit.extensions.jfcunit.TestHelper;

import javax.swing.SwingConstants;

/**
 * An interface defining the default values of the common attributes used in all EventDataContainer classes.
 * Package level access is enough for this class.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public interface EventDataConstants {
    /**
     * Identifier for the mouse position at the center of the {@link java.awt.Component}.
     */
    public static final int CENTER = SwingConstants.CENTER;

    /**
     * Identifier for the mouse position at the center of the top edge {@link java.awt.Component}.
     */
    public static final int NORTH = SwingConstants.NORTH;

    /**
     * Identifier for the mouse position at the top right corner of the {@link java.awt.Component}.
     */
    public static final int NORTH_EAST = SwingConstants.NORTH_EAST;

    /**
     * Identifier for the mouse position at the center of the right edge of the {@link java.awt.Component}.
     */
    public static final int EAST = SwingConstants.EAST;

    /**
     * Identifier for the mouse position at the bottom right corner of the {@link java.awt.Component}.
     */
    public static final int SOUTH_EAST = SwingConstants.SOUTH_EAST;

    /**
     * Identifier for the mouse position at the center of the bottom edge of the {@link java.awt.Component}.
     */
    public static final int SOUTH = SwingConstants.SOUTH;

    /**
     * Identifier for the mouse position at the bottom left corner of the {@link java.awt.Component}.
     */
    public static final int SOUTH_WEST = SwingConstants.SOUTH_WEST;

    /**
     * Identifier for the mouse position at the center of the left edge of the {@link java.awt.Component}.
     */
    public static final int WEST = SwingConstants.WEST;

    /**
     * Identifier for the mouse position at the top left corner of the {@link java.awt.Component}.
     */
    public static final int NORTH_WEST = SwingConstants.NORTH_WEST;

    /**
     * Identifier for the mouse position at a point in the component specified by the user.
     * Skip other SwingConstants in case they are needed in future.
     */
    public static final int CUSTOM = 12;

    /**
     * Identifier for the mouse position at a point in the component specified by
     * percentage of width and height. The point argument should
     * contain the percentages. For example: to get  a position
     * in the center the arguments to the event for position and point
     * would be <code>PERCENT, new {@link java.awt.Point}(50, 50)</code>.
     */
    public static final int PERCENT = 13;

    /**
     * Identifier for the mouse position at a point specified by the offset.
     */
    public static final int OFFSET = 14;

    /**
     * String description of the positions.
     */
    public static final String POSITIONSTRINGS[] =
        { "center",
          "north",
          "northeast",
          "east",
          "southeast",
          "south",
          "southwest",
          "west",
          "northwest",
          "",
          "",
          "",
          "custom",
          "percent",
          "offset" };

    /**
     * Invalid text offset
     */
    public static final int INVALID_TEXT_OFFSET = -1;

    /**
     * Default value specifying the position of the mouse relative to the {@link java.awt.Component}.
     */
    public static final int DEFAULT_POSITION = CENTER;

    /**
     * Default value specifying the number of clicks for the {@link MouseEvent}s.
     */
    public static final int DEFAULT_NUMBEROFCLICKS = 1;

    /**
     * Default value specifying whether the {@link MouseEvent} being fired
     * would trigger a popup or not.
     */
    public static final boolean DEFAULT_ISPOPUPTRIGGER = false;

    /**
     * Default value specifying the modifier key values that need to be passed onto the {@link MouseEvent}.
     */
    public static final int DEFAULT_MOUSE_MODIFIERS = MouseEvent.BUTTON1_MASK;

    /**
     * Default value specifying the modifiers key values that need to be passed by {@link java.awt.event.KeyEvent}s.
     */
    public static final int DEFAULT_KEY_MODIFIERS = 0;

    /**
     * Default value specifying the wait time in ms between each event.
     */
    public static final long DEFAULT_SLEEPTIME = TestHelper.DEFAULTSLEEP;

    /**
     * Default value specifying the hold time in ms before ejecting a event
     * from the event manager.
     */
    public static final long DEFAULT_HOLDTIME = 100L;

    /* ---------------- JSpinner support ------------------------*/
    /**
     * Identifier used to refer to an invalid sub-component (used only for {@link javax.swing.JSpinner})
     */
    public static final int INVALID_SUBCOMPONENT = -1;

    /**
     * Identifier used to refer to the up-arrow sub-component (used only for {@link javax.swing.JSpinner})
     */
    public static final int UP_ARROW_SUBCOMPONENT = 1;

    /**
     * Identifier used to refer to the down-arrow sub-component (used only for {@link javax.swing.JSpinner})
     */
    public static final int DOWN_ARROW_SUBCOMPONENT = 2;

    /**
     * Identifier used to refer to the editor sub-component (used only for {@link javax.swing.JSpinner})
     */
    public static final int EDITOR_SUBCOMPONENT = 3;

    /* ---------------- Wheel support ------------------------*/
    /**
     * The amount to scroll by default.
     */
    public static final int DEFAULT_SCROLL_AMOUNT = 3;

    /**
     * The amount to scroll by default.
     */
    public static final int DEFAULT_WHEEL_ROTATION = 1;
}