package junit.extensions.jfcunit.eventdata;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.TestHelper;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.JButton;
import javax.swing.text.JTextComponent;

/**
 * Data container class that holds all the data necessary for jfcUnit to fire mouse events.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class JSpinnerMouseEventData extends AbstractMouseEventData {

    /**
     * The Component on which to trigger the event.
     */
    private Component comp;

    /**
     * The Source on which to trigger the event.
     */
    private Component source;

    /**
     * The subcomponent type to send click.
     */
    private int subComponent = INVALID_SUBCOMPONENT;

    /**
     * Offset into the text component.
     */
    private int offset = INVALID_TEXT_OFFSET;

    /** Class for spinner component */
    private static Class spinnerClass = null;

    /**
     * Use reflection to keep 1.2 and 1.3 jdk compiling.
     */
    static {
        try {
            spinnerClass = Class.forName("javax.swing.JSpinner");
        } catch (Exception e) {
            spinnerClass = null;
        }
    }

    /**
     * Default Constructor
     */
    public JSpinnerMouseEventData() {
        super();
        setValid(false);
    }

    /**
     * Constructor - Defualt click cell
     *
     * @param _testCase The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp     The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent) {
        this(_testCase, _comp, _subComponent, DEFAULT_NUMBEROFCLICKS);
    }

    /**
     * Constructor
     *
     * @param _testCase The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp     The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                 Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, DEFAULT_MOUSE_MODIFIERS);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _sleepTime
     *                  The wait time in ms between each event.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, long _sleepTime) {
        this(_testCase, _comp, _subComponent, DEFAULT_NUMBEROFCLICKS, _sleepTime);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                  Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, int _modifiers) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, _modifiers, DEFAULT_ISPOPUPTRIGGER);
    }

    /**
     * Constructor
     *
     * @param _testCase The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp     The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                 Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _isPopupTrigger
     *                 boolean specifying whether this event will show a popup.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, boolean _isPopupTrigger) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, getDefaultModifiers(_isPopupTrigger), _isPopupTrigger);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                  Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _sleepTime
     *                  The wait time in ms between each event.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, long _sleepTime) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, DEFAULT_MOUSE_MODIFIERS, DEFAULT_ISPOPUPTRIGGER, _sleepTime);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                  Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     * @param _isPopupTrigger
     *                  boolean specifying whether this event will show a popup.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, int _modifiers, boolean _isPopupTrigger) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, _modifiers, _isPopupTrigger, DEFAULT_SLEEPTIME);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                  Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _isPopupTrigger
     *                  boolean specifying whether this event will show a popup.
     * @param _sleepTime
     *                  The wait time in ms between each event.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, boolean _isPopupTrigger, long _sleepTime) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, getDefaultModifiers(_isPopupTrigger), _isPopupTrigger, _sleepTime);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                   Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     * @param _isPopupTrigger
     *                   boolean specifying whether this event will show a popup.
     * @param _sleepTime
     *                   The wait time in ms between each event.
     */
    public JSpinnerMouseEventData(JFCTestCase _testCase, Component _comp, int _subComponent, int _numberOfClicks, int _modifiers, boolean _isPopupTrigger, long _sleepTime) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, _modifiers, _isPopupTrigger, _sleepTime, DEFAULT_POSITION, null, INVALID_TEXT_OFFSET);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                   Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     * @param _isPopupTrigger
     *                   boolean specifying whether this event will show a popup.
     * @param _sleepTime
     *                   The wait time in ms between each event.
     * @param _position  The relative mouse position within the cell.
     */
    public JSpinnerMouseEventData(
        JFCTestCase _testCase,
        Component _comp,
        int _subComponent,
        int _numberOfClicks,
        int _modifiers,
        boolean _isPopupTrigger,
        long _sleepTime,
        int _position) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, _modifiers, _isPopupTrigger, _sleepTime, _position, null, INVALID_TEXT_OFFSET);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                   Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     * @param _isPopupTrigger
     *                   boolean specifying whether this event will show a popup.
     * @param _sleepTime
     *                   The wait time in ms between each event.
     * @param _referencePoint     The CUSTOM mouse position within the cell.
     */
    public JSpinnerMouseEventData(
        JFCTestCase _testCase,
        Component _comp,
        int _subComponent,
        int _numberOfClicks,
        int _modifiers,
        boolean _isPopupTrigger,
        long _sleepTime,
        Point _referencePoint) {
        this(_testCase, _comp, _subComponent, _numberOfClicks, _modifiers, _isPopupTrigger, _sleepTime, CUSTOM, _referencePoint, INVALID_TEXT_OFFSET);
    }

    /**
     * Constructor
     *
     * @param _testCase  The JFCTestCase on whose thread <code>awtSleep()</code> has to be invoked.
     * @param _comp      The component on which to trigger the event.
     * @param _subComponent Sub-component of the JSpinner.
     * @param _numberOfClicks
     *                   Number of clicks in the MouseEvent (1 for single-click, 2 for double clicks)
     * @param _modifiers The modifier key values that need to be passed onto the event.
     * @param _isPopupTrigger
     *                   boolean specifying whether this event will show a popup.
     * @param _sleepTime
     *                   The wait time in ms between each event.
     * @param _position  The relative mouse position within the cell.
     * @param _referencePoint
     *                                   If _position is CUSTOM then the point is a offset from
     *                                   the location of the component. If the _position is PERCENT
     *                                   then the location is a percentage offset of the hight and width.
     *                                   Otherwise, the _referencePoint is unused.
     * @param _offset    The offset into the text.
     */
    public JSpinnerMouseEventData(
        JFCTestCase _testCase,
        Component _comp,
        int _subComponent,
        int _numberOfClicks,
        int _modifiers,
        boolean _isPopupTrigger,
        long _sleepTime,
        int _position,
        Point _referencePoint,
        int _offset) {
        setTestCase(_testCase);
        setSource(_comp);
        setSubComponent(_subComponent);
        setNumberOfClicks(_numberOfClicks);
        setModifiers(_modifiers);
        setPopupTrigger(_isPopupTrigger);
        setSleepTime(_sleepTime);
        setPosition(_position);
        setReferencePoint(_referencePoint);
        setOffset(_offset);
        setValid(true);
    }

    /**
     * Set the sub-component.
     * @param _subComponent SubComponent
     *                      UP_ARROW_COMPONENT,
     *                      DOWN_ARROW_COMPONENT,
     *                      EDITOR_COMPONENT
     */
    public void setSubComponent(int _subComponent) {
        this.subComponent = _subComponent;
    }

    /**
     * Get the sub-component.
     * @return _subComponent Component constant.
     */
    public int getSubComponent() {
        return subComponent;
    }

    /**
     * Set the attribute value
     *
     * @param _comp   The new value of the attribute
     */
    public void setSource(Component _comp) {
        source = _comp;
    }

    /**
     * Get the attribute value
     *
     * @return Component    The value of the attribute
     * NOTE: This is left as a Component so that the
     *       class may still compile under jdk1.3
     *       and below where JSpinner is not implemented.
     */
    public Component getSource() {
        return source;
    }

    /**
     * The component on which the event has to be fired.
     *
     * @return The component
     */
    public Component getComponent() {
        // by default, the component is the same as the source
        return comp;
    }

    /**
     * The component on which the event has to be fired.
     *
     * @param _comp The component
     */
    public void setComponent(Component _comp) {
        // by default, the component is the same as the source
        comp = _comp;
    }

    /**
     * Set the offset into the text.
     *
     * @param _offset The offset into the text.
     */
    public void setOffset(int _offset) {
        offset = _offset;
    }

    /**
     * Get the offset into the text.
     *
     * @return int offset into the text.
     */
    public int getOffset() {
        return offset;
    }

    /**
     * Prepare the component for firing the mouse event.
     *
     * @return boolean true if the component is ready.
     */
    public boolean prepareComponent() {
        if (!isValidForProcessing(getSource()) || spinnerClass == null) {
            return false;
        }

        JFCTestCase testCase = getTestCase();
        if (testCase != null) {
            testCase.pauseAWT();
        }
        int index = 0;
        Component c = null;
        Rectangle bounds = null;
        switch (getSubComponent()) {
            case DOWN_ARROW_SUBCOMPONENT :
                c = TestHelper.findComponent(JButton.class, (Container) getSource(), 1);
                bounds = c.getBounds();
                bounds.setLocation(c.getLocationOnScreen());
                setLocationOnScreen(calculatePoint(bounds));
                break;
            case UP_ARROW_SUBCOMPONENT :
                c = TestHelper.findComponent(JButton.class, (Container) getSource(), 0);
                bounds = c.getBounds();
                bounds.setLocation(c.getLocationOnScreen());
                setLocationOnScreen(calculatePoint(bounds));
                break;
            case EDITOR_SUBCOMPONENT :
                try {
                    c = (Component) spinnerClass.getMethod("getEditor", new Class[0]).invoke(getSource(), new Object[0]);

                    Class defEditor = Class.forName("javax.swing.JSpinner$DefaultEditor");
                    // If we have a DefaultEditor then we know how to get the
                    // text field component.
                    if (defEditor.isInstance(c)) {
                        c = (JTextComponent) defEditor.getMethod("getTextField", new Class[0]).invoke(c, new Object[0]);
                    }
                    if (getPosition() == OFFSET && c instanceof JTextComponent) {
                        setPosition(EAST);
                        Point p = calculatePoint(((JTextComponent) c).modelToView(offset));
                        Point screen = c.getLocationOnScreen();
                        screen.translate(p.x, p.y);
                        setLocationOnScreen(screen);

                    } else {
                        Point p = calculatePoint(c.getBounds());
                        Point screen = c.getLocationOnScreen();
                        screen.translate(p.x, p.y);
                        setLocationOnScreen(screen);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
        }
        setComponent(c);

        requestFocus(c);

        return true;
    }

    /**
     * Consume the event. This method always
     * returns false (see canConsume).
     *
     * @param ae AWTEvent to be consumed.
     * @return boolean true if the event was consumed.
     */
    public boolean consume(AWTEvent ae) {
        return false;
    }

    /**
     * Returns true if the event can be consumed by
     * this instnace of event data. However, since
     * the JSpinner class is a composite of Buttons
     * and JFormattedTextField, no events can be issued,
     * on the JSpinner. Thus this call always returns false.
     *
     * @param ae Event to be consumed.
     * @return true if the event was consumed.
     */
    public boolean canConsume(AWTEvent ae) {
        return false;
    }

    /**
     * Compare to event data and deterime if they are equal.
     *
     * @param o Object to be compared.
     * @return true if the events are the same.
     */
    public boolean equals(Object o) {
        if (!(o instanceof JSpinnerMouseEventData)) {
            return false;
        }
        JSpinnerMouseEventData data = (JSpinnerMouseEventData) o;
        return super.equals(o) && data.getSource() == getSource() && data.getSubComponent() == getSubComponent() && data.getOffset() == getOffset();
    }
}