package junit.extensions.jfcunit.finder;

import java.awt.Component;
import java.awt.Dialog;

/**
 * Class for checking if the ({@link java.awt.Dialog}) component being searched for has been found
 * The pattern syntax can be found at the Jakarta RegExp API Documentation in {@link org.apache.regexp.RE}.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class DialogFinder extends AbstractWindowFinder {
    /**
     * Constructor accepting all arguments needed to filter component.
     *
     * @param str    The desired pattern for the title of the component.
     */
    public DialogFinder(String str) {
        super(str);
    }

    /**
     * Constructor accepting all arguments needed to filter {@link java.awt.Component}.
     *
     * @param str                The desired pattern for the title of the component.
     * @param caseIndependent    Whether the match should be case independent (true) or not (false)
     */
    public DialogFinder(String str, boolean caseIndependent) {
        super(str, caseIndependent);
    }

    /**
     * Method that returns true if the given {@link java.awt.Component} matches the search
     * criteria.
     *
     * @param comp   The component to test
     * @return true if this component is a match
     */
    public boolean testComponent(final Component comp) {
        return (comp != null && isValidForProcessing(comp, Dialog.class) && checkIfTitleMatches(((Dialog) comp).getTitle()));
    }
}