package junit.extensions.jfcunit.finder;

import java.awt.Component;

import javax.swing.JLabel;

/**
 * Class for checking if the component being searched for has been found
 * The pattern syntax can be found at the Jakarta RegExp API Documentation in {@link org.apache.regexp.RE}.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class JLabelFinder extends Finder {
    /**
     * The text of the component.
     */
    private String label;

    /**
     * A boolean specifying whether the filtration is case insensitive.
     */
    private boolean caseIndependent = false;

    /**
     * Constructor accepting all arguments needed to filter component.
     *
     * @param str    The desired pattern for the text of the component.
     */
    public JLabelFinder(String str) {
        this(str, false);
    }

    /**
     * Constructor accepting all arguments needed to filter component.
     *
     * @param str                The desired pattern for the text of the component.
     * @param caseIndependent    Whether the match should be case independent (true) or not (false)
     */
    public JLabelFinder(String str, boolean caseIndependent) {
        label = str;
        this.caseIndependent = caseIndependent;
        createPatternMatcher(label, caseIndependent);
    }

    /**
     * Method that returns true if the given component matches the search
     * criteria
     *
     * @param comp   The component to test
     * @return true if this component is a match
     */
    public boolean testComponent(final Component comp) {
        return (comp != null && isValidForProcessing(comp, JLabel.class) && matchPattern(((JLabel) comp).getText(), label));
    }
}
