package junit.extensions.jfcunit.finder;

import java.awt.Component;

/**
 * This is a 'breakaway' from the ComponentFinder in which the component to be
 * tested is assumed to have a name and so the name passed in is compared to the comp.name
 * The pattern syntax can be found at the Jakarta RegExp API Documentation in {@link org.apache.regexp.RE}.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class NamedComponentFinder extends Finder {
    /**
     * The type of the component.
     */
    private Class compCls = null;

    /**
     * The pattern for the name of the component.
     */
    private String name = null;

    /**
     * A boolean specifying whether the filtration is case insensitive.
     */
    private boolean caseIndependent = false;

    /**
     * Constructor accepting all arguments needed to filter component.
     *
     * @param cls    The desired type of the component.
     * @param str    The desired pattern for the name of the component.
     */
    public NamedComponentFinder(Class cls, String str) {
        this(cls, str, false);
    }

    /**
     * Constructor accepting all arguments needed to filter component.
     *
     * @param cls                The desired type of the component.
     * @param str                The desired pattern for the name of the component.
     * @param caseIndependent    Whether the match should be case independent (true) or not (false)
     */
    public NamedComponentFinder(Class cls, String str, boolean caseIndependent) {
        compCls = cls;
        name = str;
        this.caseIndependent = caseIndependent;
        createPatternMatcher(name, caseIndependent);
    }

    /**
     * Method that returns true if the given component matches the search
     * criteria.
     *
     * @param comp   The component to test.
     * @return true if this component is a match.
     */
    public boolean testComponent(Component comp) {
        return (comp != null && (compCls == null || isValidForProcessing(comp, compCls)) && matchPattern(comp.getName(), name));
    }
}
