package junit.extensions.jfcunit.xml;

import org.w3c.dom.Element;

import junit.extensions.xml.elements.AbstractTagHandler;
import junit.extensions.xml.XMLException;
import junit.extensions.xml.IXMLTestCase;
import junit.extensions.jfcunit.JFCTestCase;

/**
 * This class will handle the processing of &lt;awtthread&gt; element.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class AWTThreadTagHandler extends AbstractTagHandler implements  JFCXMLConstants {
    /**
     * Constructor for AWTThreadTagHandler.
     *
     * @param element     The element to be processed
     * @param testCase    The IXMLTestCase that uses this element
     */
    public AWTThreadTagHandler(Element element, IXMLTestCase testCase) {
        super(element, testCase);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // check the element tag name
        checkElementTagName(AWTTHREAD);

        // reqd attribute: action
        checkRequiredAttribute(ACTION);

        if (SLEEP.equals(getString(ACTION))) {
            checkRequiredAttribute(DURATION);
        }
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        Element element = getElement();
        String action = getString(ACTION);
        if (FLUSH.equals(action)) {
            ((JFCTestCase)getTestCase()).flushAWT();
        } else if (PAUSE.equals(action)) {
            ((JFCTestCase)getTestCase()).pauseAWT();
        } else if (RESUME.equals(action)) {
            ((JFCTestCase)getTestCase()).resumeAWT();
        } else if (SLEEP.equals(action)) {
            ((JFCTestCase)getTestCase()).sleep(getSleepTime());
        }
    }

    /**
     * Returns the value of the SLEEPTIME attribute for this element, DEFAULT_SLEEPTIME if nothing
     * was specified.
     * @return long    The sleepTime for this element.
     */
    protected long getSleepTime() {
        return getLong(SLEEPTIME, DEFAULT_SLEEPTIME);
    }
}
