package junit.extensions.jfcunit.xml;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import junit.extensions.xml.IXMLTestSuite;
import junit.extensions.xml.XMLTestSuite;
import junit.extensions.xml.XMLUtil;

/**
 * This class will handle the processing of &lt;suite&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class SuiteTagHandler extends junit.extensions.xml.elements.SuiteTagHandler implements JFCXMLConstants {

    /**
     * Default constructor.
     *
     * @param element      The element to be processed
     * @param testSuite    The JFCXMLTestSuite that uses this element
     */
    public SuiteTagHandler(Element element, IXMLTestSuite testSuite) {
        super(element, testSuite);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        // Get the children and add to the suite.
        NodeList children = getElement().getChildNodes();
        Element child;
        for (int i = 0; i < children.getLength(); i++) {
            if (children.item(i) instanceof Element) {
                child = (Element) children.item(i);
                String name = child.getTagName();
                if (SUITE.equals(name)) {
                    getTestSuite().addTest(new XMLTestSuite(getTestSuite().getFileName(), child));
                } else if (TEST.equals(name)) {
                    getTestSuite().addTest(new JFCXMLTestCase(getTestSuite().getFileName(), child));
                } else if (FILE.equals(name)) {
                    getTestSuite().addFile(XMLUtil.getAttribute(child, NAME));
                }
            }
        }
    }

}