package junit.extensions.jfcunit.xml;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import junit.extensions.jfcunit.JFCTestCase;
import junit.extensions.jfcunit.JFCTestHelper;
import junit.extensions.jfcunit.RobotTestHelper;

import junit.extensions.xml.XMLException;
import junit.extensions.xml.XMLTagResourceBundle;
import junit.extensions.xml.XMLUtil;
import junit.extensions.xml.IXMLTestCase;
import junit.extensions.xml.elements.AbstractTagHandler;

/**
 * This class will handle the processing of &lt;test&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class TestTagHandler extends AbstractTagHandler implements JFCXMLConstants {
    /**
     * Constructor for TestTagHandler.
     *
     * @param element     The element to be processed
     * @param testCase    The IXMLTestCase that uses this element
     */
    public TestTagHandler(Element element, IXMLTestCase testCase) {
        super(element, testCase);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // check the element tag name
        checkElementTagName(TEST);

        // TODO: Other validations
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        // for each test, the found objects map on the test case has to be reset
        IXMLTestCase tc = getTestCase();
        tc.clearFoundObjects();

        Element element = getElement();

        // TODO: Need to support a helperClass attribute which might use a totally new sub-class of TestHelper...
        if (tc instanceof JFCTestCase) {
            boolean robot = XMLUtil.getBooleanAttributeValue(element, ROBOT);
            if (robot) {
                ((JFCTestCase)tc).setHelper(new RobotTestHelper());
            } else {
                ((JFCTestCase)tc).setHelper(new JFCTestHelper());
            }
        }
        tc.setName(getName());

        // now we are ready to handle each individual node/element
        // Get the children and add to the suite.
        NodeList children = getElement().getChildNodes();
        Element child;
        for (int i = 0; i < children.getLength(); i++) {
            if (children.item(i) instanceof Element) {
                child = (Element) children.item(i);
                String name = child.getTagName();

                XMLTagResourceBundle.getTagHandler(child, getTestCase(), name).processElement();
                // TODO: window/dialog close using testHelper
                // TODO: More asserts (on different fields of the JComponents)
            }
        }
    }

    /**
     * Returns the value of the NAME attribute for this element.
     * @return String  The value of the NAME attribute.
     */
    private String getName() {
        return getString(NAME);
    }
}