package junit.extensions.xml;

import java.io.InputStream;

import org.w3c.dom.Element;

import junit.framework.TestSuite;

/**
 * Test Case for running XML Script based testing.
 *
 * @author Kevin Wilson
 */
public class XMLTestSuite extends TestSuite implements IXMLTestSuite, XMLConstants {
    /**
     * The name of the test xml script file - found from the classpath.
     */
    private String fileName;

    /**
     * A no-argument constructor to support junit 3.8
     */
    public XMLTestSuite() {
    }

    /**
     * A constructor accepting the name of the test script file.
     *
     * @param  fileName    The name of the test script file (found from the classpath)
     * @exception   Exception   An instance of java.lang.Exception can be thrown.
     */
    public XMLTestSuite(String fileName) throws Exception {
        this(fileName, XMLUtil.parse(XMLUtil.readFileFromClasspath(fileName)).getDocumentElement());
    }

    /**
     * A constructor accepting the name of the test script file.
     *
     * @param  name      A dummy name for the file
     * @param  stream    The contents of the test script file
     * @exception   Exception   An instance of java.lang.Exception can be thrown.
     */
    public XMLTestSuite(String name, InputStream stream) throws Exception {
        this(name, XMLUtil.parse(stream).getDocumentElement());
    }

    /**
     * A constructor accepting the name of the test script file and the element to be processed.
     *
     * @param  fileName    The name of the test script file (found from the classpath)
     * @param  element     The element to be processed.
     * @exception   Exception   An instance of java.lang.Exception can be thrown.
     */
    public XMLTestSuite(String fileName, Element element) throws Exception {
        super(fileName + ":" + XMLUtil.getPath(element));
        this.fileName = fileName;
        XMLTagResourceBundle.getTagHandler(element, this, SUITE).processElement();
    }

    /**
     * Returns the filename.
     *
     * @return String   The name of the test xml script file
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * Add the given XML file to the TestSuite.
     *
     * @param  xmlFileName    The name of the test script file to be added.
     * @exception   Exception   can be thrown.
     */
    public void addFile(String xmlFileName) throws Exception {
        addTest(new XMLTestSuite(xmlFileName));
    }
}
