package junit.extensions.xml.elements;

import org.w3c.dom.Element;

import junit.extensions.xml.XMLException;
import junit.extensions.xml.XMLTestCase;
import junit.extensions.xml.XMLConstants;
import junit.framework.Assert;

/**
 * This class will handle the processing of &lt;assertnotnull&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class AssertNotNullTagHandler extends AbstractTagHandler implements XMLConstants {
    /**
     * Constructor for AssertNotNullTagHandler.
     *
     * @param element     The element to be processed
     * @param testCase    The XMLTestCase that uses this element
     */
    public AssertNotNullTagHandler(Element element, XMLTestCase testCase) {
        super(element, testCase);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // check the element tag name
        checkElementTagName(ASSERTNOTNULL);

        // reqd attribute: refid
        checkRequiredAttribute(REFID);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        Assert.assertNotNull(getMessage(), getTestCase().getFoundObject(getRefId()));
    }

    /**
     * Returns the value of the MESSAGE attribute for this element.
     * @return String  The value of the MESSAGE attribute.
     */
    protected String getMessage() {
        return getString(MESSAGE);
    }

    /**
     * Returns the value of the REFID attribute for this element.
     * @return String  The value of the REFID attribute.
     */
    private String getRefId() {
        return getString(REFID);
    }
}
