package junit.extensions.xml.elements;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import junit.extensions.xml.XMLException;
import junit.extensions.xml.XMLTestCase;
import junit.extensions.xml.IXMLTestSuite;
import junit.extensions.xml.XMLTestSuite;
import junit.extensions.xml.XMLConstants;
import junit.extensions.xml.XMLUtil;

/**
 * This class will handle the processing of &lt;suite&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class SuiteTagHandler extends AbstractTagHandler implements XMLConstants {

    /** testSuite is the test suite for this test. */
    private IXMLTestSuite testSuite;

    /**
     * Default constructor.
     *
     * @param element      The element to be processed
     * @param testSuite    The XMLTestSuite that uses this element
     */
    public SuiteTagHandler(Element element, IXMLTestSuite testSuite) {
        super(element, null);
        this.testSuite = testSuite;
    }

    /**
     * Returns the test suite passed in the constructor.
     * @return Test suite.
     */
    public IXMLTestSuite getTestSuite() {
        return testSuite;
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // check the element tag name
        checkElementTagName(SUITE);

        // TODO: Other validations
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        // Get the children and add to the suite.
        NodeList children = getElement().getChildNodes();
        Element child;
        for (int i = 0; i < children.getLength(); i++) {
            if (children.item(i) instanceof Element) {
                child = (Element) children.item(i);
                String name = child.getTagName();
                if (SUITE.equals(name)) {
                    testSuite.addTest(new XMLTestSuite(testSuite.getFileName(), child));
                } else if (TEST.equals(name)) {
                    testSuite.addTest(new XMLTestCase(testSuite.getFileName(), child));
                } else if (FILE.equals(name)) {
                    testSuite.addFile(XMLUtil.getAttribute(child, NAME));
                }
            }
        }
    }
}
