package junit.extensions.xml.elements;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import junit.extensions.xml.XMLException;
import junit.extensions.xml.XMLTestCase;
import junit.extensions.xml.XMLConstants;
import junit.extensions.xml.XMLTagResourceBundle;

/**
 * This class will handle the processing of &lt;test&gt; nodes.
 *
 * @author <a href="mailto:vraravam@thoughtworks.com">Vijay Aravamudhan : ThoughtWorks Inc.</a>
 */
public class TestTagHandler extends AbstractTagHandler implements XMLConstants {
    /**
     * Constructor for TestTagHandler.
     *
     * @param element     The element to be processed
     * @param testCase    The XMLTestCase that uses this element
     */
    public TestTagHandler(Element element, XMLTestCase testCase) {
        super(element, testCase);
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#validateElement()
     */
    public void validateElement() throws XMLException {
        // do the default validations from the super class
        super.validateElement();

        // check the element tag name
        checkElementTagName(TEST);

        // TODO: Other validations
    }

    /**
     * @see junit.extensions.xml.elements.AbstractTagHandler#processElement()
     */
    public void processElement() throws Exception {
        validateElement();

        // for each test, the found objects map on the test case has to be reset
        getTestCase().clearFoundObjects();

        Element element = getElement();

        getTestCase().setName(getName());

        // now we are ready to handle each individual node/element
        // Get the children and add to the suite.
        NodeList children = getElement().getChildNodes();
        Element child;
        for (int i = 0; i < children.getLength(); i++) {
            if (children.item(i) instanceof Element) {
                child = (Element) children.item(i);
                String name = child.getTagName();

                XMLTagResourceBundle.getTagHandler(child, getTestCase(), name).processElement();
            }
        }
    }

    /**
     * Returns the value of the NAME attribute for this element.
     * @return String  The value of the NAME attribute.
     */
    private String getName() {
        return getString(NAME);
    }
}
