/**
 * Title:        TestExtensions
 * Description:  This class tests the extensions of JUnit
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestExtensions.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestExtensions.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:53:38  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.extensions;


import junit.framework.*;
import junit.extensions.*;


/**
 * A test case testing the extensions to the testing framework.
 */

public class TestExtensions extends TestCase
{
  /**
   * helper class.
   */

  static class TornDown extends TestSetup
  {
    boolean _tornDown = false;

    /**
     * constructor.
     * 
     * @param   aTest   a test
     */
		
    TornDown (Test aTest)
    {
      super (aTest);
    }


    /**
     * sets the tornDown flag to true if this method has been called.
     */

    protected void tearDown ()
    {
      _tornDown= true;
    }
  }


  /**
   * constructor.
   *
   * @param     aName     a test name
   */

  public TestExtensions (String aName)
  {
    super (aName);
  }


  /**
   * creates a failure during a test
   */

  public void testRunningErrorInTestSetup ()
  {
    TestCase  test     = new TestCase ("failure")
                         {
                           public void runTest ()
                           {
                             fail();
                           }
                         };
    TestSetup  wrapper = new TestSetup  (test);
    TestResult result  = new TestResult ();
 
    wrapper.run (result);
    assert (!result.wasSuccessful ());
  }


  /**
   * creates an error and a failure during a test.
   */

  public void testRunningErrorsInTestSetup ()
  {
    TestCase   failure = new TestCase ("failure")
                         {
                           public void runTest ()
                           {
                             fail();
                           }
                         };
    TestCase   error   = new TestCase ("error")
                         {
                           public void runTest ()
                           {
                             throw new Error ();
                           }
                         };
    TestSuite  suite   = new TestSuite ();

    suite.addTest (failure);
    suite.addTest (error);
		
    TestSetup  wrapper = new TestSetup  (suite);
    TestResult result  = new TestResult ();

    wrapper.run(result);

    assertEquals (1, result.failureCount ());
    assertEquals (1, result.errorCount   ());
  }


  /**
   * creates a failure during a test so that the tearDown method is not 
   * called.
   */

  public void testSetupErrorDontTearDown ()
  {
    WasRun     test    = new WasRun   ("");
    TornDown   wrapper = new TornDown (test)
                         {
                           public void setUp ()
                           {
                             fail ();
                           }
                         };
    TestResult result  = new TestResult ();

    wrapper.run (result);

    assert (!wrapper._tornDown);
  }


  /**
   * tests a failure during setup.
   */

  public void testSetupErrorInTestSetup ()
  {
    WasRun      test    = new WasRun ("");
    TestSetup   wrapper = new TestSetup (test)
                          {
                            public void setUp ()
                            {
                              fail();
                            }
                          };
    TestResult result   = new TestResult ();

    wrapper.run (result);

    assert (!test._wasRun);
    assert (!result.wasSuccessful ());
  }
}
