/**
 * Title:        TestListenerTest
 * Description:  Test the test listener
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestListenerTest.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestListenerTest.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:22  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import junit.framework.*;


/**
 * Tests the test listener.
 */

public class TestListenerTest extends TestCase implements TestListener
{
  private TestResult _result;
  private int        _startCount;
  private int        _endCount;
  private int        _failureCount;
  private int        _errorCount;


  /**
   * constructor.
   *
   * @param     aName    a test case name
   */

  public TestListenerTest (String aName)
  {
    super (aName);
  }


  /**
   * adds an error.
   *
   * @param     aTest       a test
   * @param     aThrowable  an error
   */

  public void addError (Test aTest, Throwable aThrowable)
  {
    _errorCount++;
  }


  /**
   * adds a failure.
   *
   * @param     aTest       a test
   * @param     aThrowable  a failure
   */

  public void addFailure (Test aTest, Throwable aThrowable)
  {
    _failureCount++;
  }


  /**
   * adds a failure.
   *
   * @param     aTest       a test
   * @param     anError     an <code>AssertionFailedError</code>
   */

  public void addFailure (Test aTest, AssertionFailedError anError)
  {
    _failureCount++;
  }


  /**
   * reports that a test ended.
   *
   * @param     aTest       a test
   */

  public void endTest (Test aTest)
  {
    _endCount++;
  }


  /**
   * sets up the fixture.
   */

  protected void setUp ()
  {
    _result = new TestResult ();
    _result.addListener (this);

    _startCount   = 0;
    _endCount     = 0;
    _failureCount = 0;
  }


  /**
   * starts a test.
   *
   * @param   aTest  a test
   */

  public void startTest (Test aTest)
  {
    _startCount++;
  }


  /**
   * tests an error.
   */

  public void testError ()
  {
    TestCase test = new TestCase ("noop")
                    {
                      public void runTest ()
                      {
                        throw new Error ();
                      }
                    };

    test.run (_result);

    assertEquals (1, _errorCount);
    assertEquals (1, _endCount);
  }


  /**
   * tests a failure.
   */

  public void testFailure ()
  {
    TestCase test = new TestCase ("noop")
                    {
                      public void runTest ()
                      {
                        fail ();
                      }
                    };

    test.run (_result);

    assertEquals (1, _failureCount);
    assertEquals (1, _endCount);
  }


  /**
   * tests a success.
   */

  public void testStartStop ()
  {
    TestCase test = new TestCase ("noop")
                    {
                      public void runTest ()
                      {
                      }
                    };

    test.run (_result);

    assertEquals (1, _startCount);
    assertEquals (1, _endCount);
  }
}
