/**
 * Title:        TestPrivate
 * Description:  A class containing tests for accessing private fields/methods
 * Copyright:    Copyright (c) 2000 Andreas Heilwagen
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Andreas Heilwagen. Andreas Heilwagen also grant you the right to
 *   distribute the Software and Software Derivative Works.
 *
 *   Andreas Heilwagen licenses the Software to you on an "AS IS" basis, without
 *   warranty of any kind. Andreas Heilwagen HEREBY EXPRESSLY DISCLAIMS ALL
 *   WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, NON
 *   INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. You are solely
 *   responsible for determining the appropriateness of using the Software and
 *   assume all risks associated with the use and distribution of this Software,
 *   including but not limited to the risks of program errors, damage to or loss
 *   of data, programs or equipment, and unavailability or interruption of
 *   operations. ANDREAS HEILWAGEN WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR
 *   FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC
 *   CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF ANDREAS
 *   HEILWAGEN HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Andreas 
 *   Heilwagen will not be liable for the loss of, or damage to, your records or
 *   data, or any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Andreas Heilwagen assumes no
 *   liability for any claim that may arise regarding the Software or Software
 *   Derivatives, and 2) that disclaims all warranties, both express and implied,
 *   from Andreas Heilwagen regarding the Software and Software Derivatives. (If
 *   you include this Agreement with any distribution of the Software and
 *   Software Derivatives you will have met this requirement). You agree that you
 *   will not delete any copyright notices in the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Andreas Heilwagen. Except for the licenses granted to you in
 *   the second paragraph above, no other licenses are granted hereunder, by
 *   estoppel, implication or otherwise.
 *
 * @author Andreas Heilwagen
 *
 * $Id: TestPrivate.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestPrivate.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:31  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import java.io.*;

import junit.framework.*;
import junitx.framework.*;

import tests.junitx.framework.proxy.pkg.*;


/**
 * tests access to a package private class in another package
 * using a <code>TestProxy</code>.
 */

public class TestPrivate extends PrivateTestCase
{
  public TestPrivate (String aName)
  {
    super (aName);
  }


  /**
   * test if the class name of an instance of a package private class in
   * another package can be identified in this package.
   */

  public void testPkgPrivClassName ()
  {
    PkgPublicClass pubClass = new PkgPublicClass     ();
    Object         privObj  = pubClass.getPrivateObj ();

    assertEquals (privObj.getClass ().toString (),
                  "class tests.junitx.framework.proxy.pkg.PkgPrivateClass");
  }


  /**
   * test access to a package public class
   */

  public void testPkgPubClass ()
  {
    PkgPublicClass publicClass = new PkgPublicClass ();

    assert (publicClass.knock ());
  }


  /**
   * test instantiation of a package private class as well as accessing
   * a private method. Here the default constructor is beeing used.
   */

  public void testPkgPrivClassAccess ()
  throws TestAccessException
  {
    Object privateClass = newInstance ("tests.junitx.framework.proxy.pkg.PkgPrivateClass", NOARGS);

    assert (asBoolean (invoke (privateClass, "knock", NOARGS)));
  }


  /**
   * test instantiation of a package private class as well as accessing
   * a private method. Here the default constructor is beeing used, but
   * it is identified using a method key.
   */

  public void testPkgPrivClassAccessWithKey ()
  throws TestAccessException
  {
    Object privateClass = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                              "", NOARGS);
    assert (asBoolean (invoke (privateClass, "knock", NOARGS)));
  }


  /**
   * test access to a private field in a package private class.
   */

  public void testPkgPrivClassPrivFieldAccess ()
  throws TestAccessException
  {
    Object privateClass = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                              "", NOARGS);

    assert (getBoolean (privateClass, "_foo"));
  }


  /**
   * test instantiation of a package private class as well as accessing
   * a private method. Here a private constructor is beeing used.
   */

  public void testPkgPrivClassPrivConstructor ()
  throws TestAccessException
  {
    try
    {
      Object[] args         = { new Boolean (false) };
      Object   privateClass = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                                  "_boolean", args);
      assert (!asBoolean (invoke (privateClass, "knock", NOARGS)));
    }
    catch (TestAccessException e)
    {
      assert (e.getReason () instanceof IllegalAccessException);
    }
  }


  /**
   * test instantiation of a package private class as well as accessing
   * a private method. Here a public constructor is beeing used, locating
   * it using a method key.
   */

  public void testPkgPrivClassPubConstructorWithKey ()
  throws TestAccessException
  {
    Object[] args         = { new Boolean (false) };
    Object   privateClass = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                                  "_java.lang.Boolean", args);
    assert (!asBoolean (invoke (privateClass, "knock", NOARGS)));
  }


  /**
   * invoke a private method of a package private class getting a 
   * package private class argument.
   */

  public void testPkgPrivClassPkgPrivArgMethod ()
  throws TestAccessException
  {
    Object[] args1         = { new Boolean (false) };
    Object[] args2         = { new Boolean (true)  };
    Object   privateClass1 = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                                  "_java.lang.Boolean", args1);
    Object   privateClass2 = newInstanceWithKey ("tests.junitx.framework.proxy.pkg.PkgPrivateClass",
                                                  "_java.lang.Boolean", args2);
    Object[] copyArg      = { privateClass1 };

    assert ( asBoolean (invoke (privateClass2, "knock", NOARGS)));
    invoke (privateClass2, "copyFrom", copyArg);
    assert (!asBoolean (invoke (privateClass2, "knock", NOARGS)));
  }
}
