/**
 * Title:        TestProxiedInverter
 * Description:  Tests for accessing a class using a TestProxy
 * Copyright:    Copyright (c) 2000 Andreas Heilwagen
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Andreas Heilwagen. Andreas Heilwagen also grant you the right to
 *   distribute the Software and Software Derivative Works.
 *
 *   Andreas Heilwagen licenses the Software to you on an "AS IS" basis, without
 *   warranty of any kind. Andreas Heilwagen HEREBY EXPRESSLY DISCLAIMS ALL
 *   WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, NON
 *   INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. You are solely
 *   responsible for determining the appropriateness of using the Software and
 *   assume all risks associated with the use and distribution of this Software,
 *   including but not limited to the risks of program errors, damage to or loss
 *   of data, programs or equipment, and unavailability or interruption of
 *   operations. ANDREAS HEILWAGEN WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR
 *   FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC
 *   CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF ANDREAS
 *   HEILWAGEN HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Andreas 
 *   Heilwagen will not be liable for the loss of, or damage to, your records or
 *   data, or any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Andreas Heilwagen assumes no
 *   liability for any claim that may arise regarding the Software or Software
 *   Derivatives, and 2) that disclaims all warranties, both express and implied,
 *   from Andreas Heilwagen regarding the Software and Software Derivatives. (If
 *   you include this Agreement with any distribution of the Software and
 *   Software Derivatives you will have met this requirement). You agree that you
 *   will not delete any copyright notices in the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Andreas Heilwagen. Except for the licenses granted to you in
 *   the second paragraph above, no other licenses are granted hereunder, by
 *   estoppel, implication or otherwise.
 *
 * @author Andreas Heilwagen
 *
 * $Id: TestProxiedInverter.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestProxiedInverter.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:35  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import java.io.*;

import junit.framework.*;
import junitx.framework.*;

import tests.junitx.framework.proxy.Inverter;


/**
 * This test cases checks accessing a public class in another package
 * using a proxy.
 */

public class TestProxiedInverter extends PrivateTestCase
{
  public TestProxiedInverter (String aName)
  {
    super (aName);
  }


  /**
   * test accessing a private field of a public class in another
   * package.
   */

  public void testGetFieldObjValue ()
  throws TestAccessException
  {
    Inverter inv = new Inverter ();
    Integer  val = (Integer)(get (inv, "_value"));

    assertEquals (asInt (val), 0);
  }


  /**
   * test access of a private field in a class in another package.
   * The tested class is public, the field has a simple data type.
   */

  public void testGetFieldIntValue ()
  throws TestAccessException
  {
    Inverter inv = new Inverter ();
    assertEquals (getInt (inv, "_value"), 0);
  }


  /**
   * tests invoking a public method on a public class in another
   * package. The invoked method only has arguments types derived
   * from Class.
   */

  public void testSetFieldValue ()
  {
    Inverter inv  = new Inverter ();
    Object[] args = { new Integer (1) };

    try
    {
      invoke (inv, "set", args);
    } 
    catch (Exception e)
    {
      System.out.println ("testSetValue caught exception: " + e);
    }

    assertEquals (inv.get (), 0);
  }


  public void testSetFieldIntValue ()
  throws TestAccessException
  {
    Inverter inv   = new Inverter ();
    Object[] args  = { new Integer (1) };

    invokeWithKey (new Inverter (), "setSimple_int", args);
    assertEquals (inv.get (), 1);
  }
}
