/**
 * Title:        TestSimpleTypes
 * Description:  Test for accessing private fields of simple data types
 * Copyright:    Copyright (c) 2000 Andreas Heilwagen
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Andreas Heilwagen. Andreas Heilwagen also grant you the right to
 *   distribute the Software and Software Derivative Works.
 *
 *   Andreas Heilwagen licenses the Software to you on an "AS IS" basis, without
 *   warranty of any kind. Andreas Heilwagen HEREBY EXPRESSLY DISCLAIMS ALL
 *   WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, NON
 *   INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. You are solely
 *   responsible for determining the appropriateness of using the Software and
 *   assume all risks associated with the use and distribution of this Software,
 *   including but not limited to the risks of program errors, damage to or loss
 *   of data, programs or equipment, and unavailability or interruption of
 *   operations. ANDREAS HEILWAGEN WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR
 *   FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC
 *   CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF ANDREAS
 *   HEILWAGEN HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Andreas 
 *   Heilwagen will not be liable for the loss of, or damage to, your records or
 *   data, or any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Andreas Heilwagen assumes no
 *   liability for any claim that may arise regarding the Software or Software
 *   Derivatives, and 2) that disclaims all warranties, both express and implied,
 *   from Andreas Heilwagen regarding the Software and Software Derivatives. (If
 *   you include this Agreement with any distribution of the Software and
 *   Software Derivatives you will have met this requirement). You agree that you
 *   will not delete any copyright notices in the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Andreas Heilwagen. Except for the licenses granted to you in
 *   the second paragraph above, no other licenses are granted hereunder, by
 *   estoppel, implication or otherwise.
 *
 * @author Andreas Heilwagen
 *
 * $Id: TestSimpleTypes.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestSimpleTypes.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:44  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import junit.framework.*;
import junitx.framework.*;

import tests.junitx.framework.proxy.SimpleTypes;


/**
 * test the access to simple type private fields in a public class
 * of another package. Especially the <code>TestProxy</code> and
 * <code>TestCase</code> class will be tested by this.
 */

public class TestSimpleTypes extends PrivateTestCase
{
  SimpleTypes types = new SimpleTypes ();


  public TestSimpleTypes (String aName)
  {
    super (aName);
  }


  public void testGetShortAsBoolean ()
  {
    try
    {
      assert (getBoolean (types, "_short"));
    }
    catch (TestAccessException e)
    {
      assert (e.getReason () instanceof IllegalArgumentException);
    }
  }


  public void testGetBoolean ()
  throws TestAccessException
  {
    assert (getBoolean (types, "_boolean"));
  }


  public void testGetByte ()
  throws TestAccessException
  {
    assertEquals (getByte (types, "_byte"), 1);
  }


  public void testGetChar ()
  throws TestAccessException
  {
    assertEquals (getChar (types, "_char"), 'x');
  }


  public void testGetDouble ()
  throws TestAccessException
  {
    assert (getDouble (types, "_double") == (double)3);
  }


  public void testGetFloat ()
  throws TestAccessException
  {
    assert (getFloat (types, "_float") == (float)4);
  }


  public void testGetInt ()
  throws TestAccessException
  {
    assertEquals (getInt (types, "_int"), 5);
  }


  public void testGetLong ()
  throws TestAccessException
  {
    assertEquals (getLong (types, "_long"), 6);
  }


  public void testGetShort ()
  throws TestAccessException
  {
    assertEquals (getShort (types, "_short"), 7);
  }
}
