/**
 * Title:        TestTest
 * Description:  A test case testing the testing framework
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestTest.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestTest.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:53  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import java.util.Vector;

import junit.framework.*;


/**
 * A test case testing the testing framework.
 */

public class TestTest extends TestCase
{
  /**
   * inner test class.
   */
	
  static class TornDown extends TestCase
  {
    boolean _tornDown= false;

 
    /**
     * constructor.
     *
     * @param   aName    a test case name
     */
		
    TornDown (String aName)
    {
      super (aName);
    }


    /**
     * tears down the fixture.
     */

    protected void tearDown ()
    {
      _tornDown = true;
    }


    /**
     * runs a test throwing an error.
     */

    protected void runTest ()
    {
      throw new Error ();
    }
  }


  /**
   * constructor.
   *
   * @param     aName   a test case name
   */

  public TestTest (String aName)
  {
    super (aName);
  }


  /**
   * tests assertEquals.
   */

  public void testAssertEquals ()
  {
    Object o = new Object ();

    assertEquals (o, o);
  }


  /**
   * tests assertEquals with null values.
   */

  public void testAssertEqualsNull ()
  {
    assertEquals (null, null);
  }


  /**
   * tests assertNull.
   */

  public void testAssertNull ()
  {
    TestCase succeeds = new TestCase ("isnull")
                        {
                          protected void runTest ()
                          {
                            assertNull (null);
                          }
                        };

     verifySuccess (succeeds);
  }


  /**
   * tests a failure of assertNull.
   */

  public void testAssertNullNotEqualsNull ()
  {
    TestCase fails = new TestCase("fails")
                     {
                       protected void runTest ()
                       {
                         assertEquals (null, new Object ());
                       }
                     };

    verifyFailure (fails);
  }


  /**
   * tests assertSame.
   */

  public void testAssertSame ()
  {
    Object o = new Object ();

    assertSame (o, o);
  }


  /**
   * tests assertSame failure.
   */

  public void testAssertSameFails ()
  {
    TestCase assertSame = new TestCase ("assertSame")
                          {
                            protected void runTest ()
                            {
                              assertSame (new Integer (1), new Integer (1));
                            }
                          };

    verifyFailure (assertSame);
  }


  /**
   * tests toString.
   *
   * This test wins the award for twisted snake tail eating while writing self
   * tests. And you thought those weird anonymous inner classes were bad...
   */

  public void testCaseToString()
  {
    assertEquals ("testCaseToString(tests.junitx.framework.TestTest)", toString ());
  }


  /**
   * tests throwing an error.
   */

  public void testError ()
  {
    TestCase error = new TestCase ("error")
                     {
                       protected void runTest ()
                       {
                         throw new Error ();
                       }
                     };

    verifyError (error);
  }


  /**
   * tests a failure.
   */

  public void testFail ()
  {
    TestCase failure = new TestCase ("fail")
                       {
                         protected void runTest ()
                         {
                           fail ();
                         }
                       };

     verifyFailure(failure);
  }


  /** 
   * tests assertNotNull.
   */

  public void testFailAssertNotNull ()
  {
    TestCase failure = new TestCase ("fails")
                       {
                         protected void runTest ()
                         {
                           assertNotNull (null);
                         }
                       };

    verifyFailure (failure);
  }


  /**
   * tests a failure.
   */

  public void testFailure ()
  {
    TestCase failure = new TestCase ("failure")
                       {
                         protected void runTest ()
                         {
                           assert (false);
                         }
                       };

    verifyFailure (failure);
  }


  /**
   * tests an AssertionFailedError.
   */

  public void testFailureError ()
  {
    try
    {
      fail ();
    } 
    catch (AssertionFailedError e)
    {
      return;
    }
    fail ();
  }


  /**
   * tests failing tearDown() method.
   */

  public void testRunAndTearDownFails ()
  {
    TornDown fails = new TornDown ("fails")
                     {
                       protected void tearDown ()
                       {
                         super.tearDown  ();
                         throw new Error ();
                       }

                       protected void runTest ()
                       {
                         throw new Error ();
                       }
                     };

    verifyError (fails);
    assert      (fails._tornDown);
  }


  /**
   * tests failure of setUp method.
   */

  public void testSetupFails ()
  {
    TestCase fails = new TestCase ("success")
                     {
                       protected void setUp ()
                       {
                         throw new Error ();
                       }

                       protected void runTest ()
                       {
                       }
                     };

    verifyError (fails);
  }


  /**
   * tests assertNotNull.
   */

  public void testSucceedAssertNotNull ()
  {
    assertNotNull (new Object ());
  }


  /**
   * tests assert.
   */

  public void testSuccess ()
  {
    TestCase success = new TestCase ("success")
                       {
                         protected void runTest ()
                         {
                           assert (true);
                         }
                       };

    verifySuccess (success);
  }


  /**
   * tests tearDown method after error.
   */

  public void testTearDownAfterError ()
  {
    TornDown fails = new TornDown ("fails");

    verifyError (fails);
    assert      (fails._tornDown);
  }


  /**
   * tests tearDown failure.
   */

  public void testTearDownFails ()
  {
    TestCase fails = new TestCase ("success")
                     {
                       protected void tearDown ()
                       {
                         throw new Error ();
                       }

                       protected void runTest ()
                       {
                       }
                     };

    verifyError (fails);
  }


  /**
   * tests tearDown after failure of setUp method.
   */

  public void testTearDownSetupFails ()
  {
    TornDown fails = new TornDown ("fails")
                     {
                       protected void setUp ()
                       {
                         throw new Error ();
                       }
                     };

    verifyError (fails);
    assert      (!fails._tornDown);
  }


  /**
   * tests a failed test.
   */

  public void testWasNotSuccessful ()
  {
    TestCase failure  = new TestCase ("fail")
                        {
                          protected void runTest ()
                          {
                            fail ();
                          }
                        };
    TestResult result = failure.run();

    assert ( result.runCount     () == 1);
    assert ( result.failureCount () == 1);
    assert ( result.errorCount   () == 0);
    assert (!result.wasSuccessful());
  }


  /**
   * tests if a test has been run.
   */

  public void testWasRun ()
  {
    WasRun test = new WasRun ("");

    test.run ();

    assert (test._wasRun);
  }


  /**
   * tests a successful test.
   */

  public void testWasSuccessful ()
  {
    TestCase success  = new TestCase ("success")
                        {
                          protected void runTest ()
                          {
                            assert (true);
                          }
                        };
    TestResult result = success.run ();

    assert (result.runCount      () == 1);
    assert (result.failureCount  () == 0);
    assert (result.errorCount    () == 0);
    assert (result.wasSuccessful ());
  }


  /**
   * verifies that a test has thrown an error.
   *
   * @param    aTest    a test
   */

  private void verifyError (TestCase aTest)
  {
    TestResult result = aTest.run ();

    assert (result.runCount     () == 1);
    assert (result.failureCount () == 0);
    assert (result.errorCount   () == 1);
  }


  /**
   * verifies that a test has thrown a failure
   *
   * @param    aTest    a test
   */

  private void verifyFailure (TestCase aTest)
  {
    TestResult result = aTest.run ();

    assert (result.runCount     () == 1);
    assert (result.failureCount () == 1);
    assert (result.errorCount   () == 0);
  }


  /**
   * verifies that a test has been successful
   *
   * @param    aTest    a test
   */

  private void verifySuccess (TestCase aTest)
  {
    TestResult result = aTest.run ();

    assert (result.runCount     () == 1);
    assert (result.failureCount () == 0);
    assert (result.errorCount   () == 0);
  }
}
