/**
 * Title:        TestTestProxy
 * Description:  Tests for accessing inherited methods
 * Copyright:    Copyright (c) 2000 Andreas Heilwagen
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Andreas Heilwagen. Andreas Heilwagen also grant you the right to
 *   distribute the Software and Software Derivative Works.
 *
 *   Andreas Heilwagen licenses the Software to you on an "AS IS" basis, without
 *   warranty of any kind. Andreas Heilwagen HEREBY EXPRESSLY DISCLAIMS ALL
 *   WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, NON
 *   INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. You are solely
 *   responsible for determining the appropriateness of using the Software and
 *   assume all risks associated with the use and distribution of this Software,
 *   including but not limited to the risks of program errors, damage to or loss
 *   of data, programs or equipment, and unavailability or interruption of
 *   operations. ANDREAS HEILWAGEN WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR
 *   FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC
 *   CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF ANDREAS
 *   HEILWAGEN HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Andreas 
 *   Heilwagen will not be liable for the loss of, or damage to, your records or
 *   data, or any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Andreas Heilwagen assumes no
 *   liability for any claim that may arise regarding the Software or Software
 *   Derivatives, and 2) that disclaims all warranties, both express and implied,
 *   from Andreas Heilwagen regarding the Software and Software Derivatives. (If
 *   you include this Agreement with any distribution of the Software and
 *   Software Derivatives you will have met this requirement). You agree that you
 *   will not delete any copyright notices in the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Andreas Heilwagen. Except for the licenses granted to you in
 *   the second paragraph above, no other licenses are granted hereunder, by
 *   estoppel, implication or otherwise.
 *
 * @author Andreas Heilwagen
 *
 * $Id: TestTestProxy.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestTestProxy.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:54:59  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import junit.framework.*;
import junitx.framework.*;


/**
 * This tests check if we can access inherited methods and if the
 * constructors are all found.
 */

public class TestTestProxy extends PrivateTestCase
{
  public TestTestProxy (String aName)
  {
    super (aName);
  }


  /**
   * An inner class extended by another class.
   */

  public class ParentClass
  {
    String _last = null;


    /**
     * a default constructor.
     */

    public ParentClass ()
    {
    }


    /**
     * a constructor which takes an argument.
     *
     * @param   aValue   an int value
     */

    public ParentClass (int aValue)
    {
    }


    /**
     * a public method.
     */

    public void publicParentMethod ()
    {
      _last = "publicParentMethod";
    }


    /**
     * a public method which takes arguments.
     *
     * @param    aValue  an int value
     */

    public int publicParentMethodInt (int aValue)
    {
      _last = "publicParentMethodInt_" + aValue;

      return 0;
    }


    /**
     * a protected method.
     */

    protected void protectedParentMethod ()
    {
      _last = "protectedParentMethod";
    }


    /**
     * a private method.
     */

    private void privateParentMethod ()
    {
      _last = "privateParentMethod";
    }


    /**
     * @return the last method
     */

    public String lastMethod ()
    {
      return _last;
    }
  }


  /**
   * An inner class which extends the previous class in order to check
   * which methods we can access.
   */

  public class ChildClass extends ParentClass
  {
    String _last = null;


    /**
     * a default constructor.
     */

    public ChildClass ()
    {
    }


    /**
     * a constructor which takes an argument.
     *
     * @param   aValue   an int value
     */

    public ChildClass (int aValue)
    {
    }


    /**
     * a public method.
     */

    public void publicChildMethod ()
    {
      _last = "publicChildMethod";
    }


    /**
     * a public method which takes arguments.
     *
     * @param    aValue  an int value
     */

    public int publicChildMethodInt (int aValue)
    {
      _last = "publicChildMethodInt_" + aValue;

      return 0;
    }


    /**
     * a protected method.
     */

    protected void protectedChildMethod ()
    {
      _last = "protectedChildMethod";
    }


    /**
     * a private method.
     */

    private void privateChildMethod ()
    {
      _last = "privateChildMethod";
    }


    /**
     * @return the last method
     */

    public String lastMethod ()
    {
      return _last;
    }
  }


  /*
   * creates a test suite.
   * 
   * @return a test suite
   */

  public static Test suite ()
  {
    return new TestSuite (tests.junitx.framework.TestTestProxy.class);
  }


  /**
   * test if ParentClass() can be accessed.
   */

  public void testParentConstructor ()
  throws TestAccessException
  {
    newInstanceWithKey ("tests.junitx.framework.TestTestProxy$ParentClass",
                        "_tests.junitx.framework.TestTestProxy", new Object[] { this });
  }


  /**
   * test if ParentClass(int) can be accessed.
   */

  public void testParentConstructorInt ()
  throws TestAccessException
  {
    newInstanceWithKey ("tests.junitx.framework.TestTestProxy$ParentClass",
                        "_tests.junitx.framework.TestTestProxy_int", new Object[] { this, new Integer (3) });
  }


  /**
   * test if publicParentMethod() can be accessed.
   */

  public void testPublicParentMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ParentClass (), "publicParentMethod", NOARGS);
  }


  /**
   * test if publicParentMethodInt(int) can be accessed.
   */

  public void testPublicParentMethodInt ()
  throws TestAccessException
  {
    invokeWithKey (new ParentClass (), "publicParentMethodInt_int",
                         new Object[] { new Integer (1) });
  }


  /**
   * test if protectedParentMethod() can be accessed.
   */

  public void testProtectedParentMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ParentClass (), "protectedParentMethod", NOARGS);
  }


  /**
   * test if privateParentMethod() can be accessed.
   */

  public void testPrivateParentMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ParentClass (), "privateParentMethod", NOARGS);
  }


  /**
   * test if lastMethod() can be accessed.
   */

  public void testLastMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ParentClass (), "lastMethod", NOARGS);
  }


  /**
   * test if ChildClass() can be accessed.
   */

  public void testChildConstructor ()
  throws TestAccessException
  {
    newInstanceWithKey ("tests.junitx.framework.TestTestProxy$ChildClass",
                        "_tests.junitx.framework.TestTestProxy", new Object[] { this });
  }


  /**
   * test if ChildClass(int) can be accessed.
   */

  public void testChildConstructorInt ()
  throws TestAccessException
  {
    newInstanceWithKey ("tests.junitx.framework.TestTestProxy$ChildClass",
                        "_tests.junitx.framework.TestTestProxy_int", new Object[] { this, new Integer (3) });
  }


  /**
   * test if publicChildMethod() can be accessed.
   */

  public void testPublicChildMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "publicChildMethod", NOARGS);
  }


  /**
   * test if publicChildMethodInt(int) can be accessed.
   */

  public void testPublicChildMethodInt ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "publicChildMethodInt_int",
                         new Object[] { new Integer (1) });
  }


  /**
   * test if protectedChildMethod() can be accessed.
   */

  public void testProtectedChildMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "protectedChildMethod", NOARGS);
  }


  /**
   * test if privateChildMethod() can be accessed.
   */

  public void testPrivateChildMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "privateChildMethod", NOARGS);
  }


  /**
   * test if lastMethod() can be accessed.
   */

  public void testChildLastMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "lastMethod", NOARGS);
  }


  /**
   * test if publicParentMethod() can be accessed from ChildClass.
   */

  public void testChildPublicParentMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "publicParentMethod", NOARGS);
  }


  /**
   * test if publicParentMethodInt(int) can be accessed from ChildClass.
   */

  public void testChildPublicParentMethodInt ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "publicParentMethodInt_int",
                         new Object[] { new Integer (1) });
  }


  /**
   * test if protectedParentMethod() can be accessed from ChildClass.
   */

  public void testChildProtectedParentMethod ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "protectedParentMethod", NOARGS);
  }


  /**
   * test if privateParentMethod() can be accessed from ChildClass.
   */

  public void testChildPrivateParentMethod ()
  throws TestAccessException
  {
    try
    {
      invokeWithKey (new ChildClass (), "privateParentMethod", NOARGS);
      fail ("private methods are not inheritable!");
    }
    catch (TestAccessException e)
    {
    }
  }


  /**
   * test if clone() can be accessed from ChildClass.
   */

  public void testChildClone ()
  throws TestAccessException
  {
    invokeWithKey (new ChildClass (), "toString", NOARGS);
  }


  /**
   * dumps the keys of all three classes.
   */
/*
  public void testDumpKeys ()
  throws TestAccessException
  {
    dumpKeys ("tests.junitx.framework.TestTestProxy");
    dumpKeys ("tests.junitx.framework.TestTestProxy$ChildClass");
    dumpKeys ("tests.junitx.framework.TestTestProxy$ChildClass");
  }*/
}
