/**
 * Title:        TestThreads
 * Description:  This class shows that tests accross threads do not work yet
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestThreads.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestThreads.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.3  2001/02/12 07:17:11  aheilwag
 * renamed thread-Variable to enable compilation with JDK 1.2
 *
 * Revision 1.2  2000/11/10 19:55:04  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework;


import java.util.Vector;

import junit.framework.*;


/**
 * A test case testing the running of tests across multiple threads.
 * Demonstrates that running a test case across different threads doesn't work
 * yet.
 */

public class TestThreads extends TestCase
{
  /**
   * constructor.
   *
   * @param      aTest     a test case
   */
	
  public TestThreads (String aName)
  {
    super (aName);
  }


  /**
   * read the source luke.
   */

  public void testRemote ()
  {
    Thread thread = new Thread ()
                    {
                      public void run ()
                      {
                        verifyResults ();
                      }
                    };

    thread.start ();

    try
    {
      thread.join ();
    }
    catch (InterruptedException e)
    {
      fail ("interrupted test");
    }
  }


  /**
   * read the source luke.
   */

  public void testRunInThread ()
  {
    final Thread thread  = new Thread ()
                           {
                             public void run ()
                             {
                               verifyResults ();
                             }
                           };
    TestCase runInThread = new TestCase ("runInThread")
                           {
                             protected void runTest ()
                             {
                               final Thread testThread = new Thread ()
                               {
                                 public void run ()
                                 {
                                   Assert.fail ();
                                 }
                               };

                               testThread.start ();

                               try
                               {
                                 testThread.join ();
                               }
                               catch (InterruptedException e)
                               {
                                 fail ("interrupted test");
                               }
                             }
                           };

    TestResult result = runInThread.run ();

    assert (result.runCount     () == 1);
    assert (result.failureCount () == 1);
    assert (result.errorCount   () == 0);
  }


  /**
   * verify that the test failed.
   */

  public void verifyResults ()
  {
    fail ("verify failed");
  }
}
