/**
 * Title:        TestProxy
 * Description:  A package local proxy to access private classes
 * Copyright:    Copyright (c) 2000 Andreas Heilwagen
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Andreas Heilwagen. Andreas Heilwagen also grant you the right to
 *   distribute the Software and Software Derivative Works.
 *
 *   Andreas Heilwagen licenses the Software to you on an "AS IS" basis, without
 *   warranty of any kind. Andreas Heilwagen HEREBY EXPRESSLY DISCLAIMS ALL
 *   WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, NON
 *   INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. You are solely
 *   responsible for determining the appropriateness of using the Software and
 *   assume all risks associated with the use and distribution of this Software,
 *   including but not limited to the risks of program errors, damage to or loss
 *   of data, programs or equipment, and unavailability or interruption of
 *   operations. ANDREAS HEILWAGEN WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR
 *   FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC
 *   CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF ANDREAS
 *   HEILWAGEN HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Andreas 
 *   Heilwagen will not be liable for the loss of, or damage to, your records or
 *   data, or any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Andreas Heilwagen assumes no
 *   liability for any claim that may arise regarding the Software or Software
 *   Derivatives, and 2) that disclaims all warranties, both express and implied,
 *   from Andreas Heilwagen regarding the Software and Software Derivatives. (If
 *   you include this Agreement with any distribution of the Software and
 *   Software Derivatives you will have met this requirement). You agree that you
 *   will not delete any copyright notices in the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Andreas Heilwagen. Except for the licenses granted to you in
 *   the second paragraph above, no other licenses are granted hereunder, by
 *   estoppel, implication or otherwise.
 *
 * @author Andreas Heilwagen
 *
 * $Id: TestProxy.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestProxy.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:55:30  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.framework.proxy;


import java.lang.reflect.*;

import junit.framework.*;
import junitx.framework.*;


/**
 * This proxy enables access to package private classes as well as
 * protected and private methods and fields of any class in this
 * package.
 */

public class TestProxy extends junitx.framework.TestProxy
{
  /**
   * creates a new instance of the proxied class.
   *
   * This method only supports constructors without simple data types
   * as their arguments. Use <code>newInstanceWithKey</code> otherwise.
   *
   * @param     anArgList           argument array for the constructor
   *
   * @return a new instance of the proxied class or null
   *
   * @exception TestAccessException if the class cannot be instantiated
   */

  public Object newInstance (Object[] anArgList)
  throws TestAccessException
  {
    try
    {
      return getProxiedClass ().getConstructor (anArgList).newInstance (anArgList);
    }
    catch (Exception e)
    {
      throw new TestAccessException ("could not instantiate " + getTestedClassName (), e);
    }
  }


  /**
   * creates a new instance of the proxied class.
   *
   * This method supports constructors with simple data types as their
   * arguments. Use <code>newInstance</code> for maintenability if the
   * constructor has only object arguments. See
   * <code>junit.framework.TestProxy.invokeWithKey</code> for documentation
   * on using method keys.
   *
   * @param     aConstructorKey     a constructor key
   * @param     anArgList           argument array for the constructor
   *
   * @return a new instance of the proxied class or null
   *
   * @exception TestAccessException if the class cannot be instantiated
   */

  public Object newInstanceWithKey (String aConstructorKey, Object[] anArgList)
  throws TestAccessException
  {
    try
    {
      return getProxiedClass ().getConstructor (aConstructorKey).newInstance (anArgList);
    }
    catch (Exception e)
    {
      throw new TestAccessException ("could not instantiate " + getTestedClassName (), e);
    }
  }
}
