/**
 * Title:        TestMoney
 * Description:  A test suite for the production.money package
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestMoney.java,v 1.2 2001/12/14 11:27:26 aheilwag Exp $
 * 
 * $Log: TestMoney.java,v $
 * Revision 1.2  2001/12/14 11:27:26  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:57:09  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.money;


import junit.framework.*;

import tests.junitx.money.setup.*;


/**
 * A test suite for the production.money package.
 */

public class TestMoney extends TestCase
{
  private Money    _12CHF;
  private Money    _14CHF;
  private Money    _7USD;
  private Money    _21USD;

  private MoneyBag _mb1;
  private MoneyBag _mb2;


  /**
   * constructor
   *
   * @param   aName   a name for the test suite
   */

  public TestMoney (String aName)
  {
    super (aName);
  }


  /**
   * main entry point
   *
   * @param   args     commandline arguments
   */

  public static void main (String args[])
  {
    junit.textui.TestRunner.run (TestMoney.class);
  }


  /**
   * set up fixture
   */

  protected void setUp ()
  {
    _12CHF = new Money (12, "CHF");
    _14CHF = new Money (14, "CHF");
    _7USD  = new Money ( 7, "USD");
    _21USD = new Money (21, "USD");

    _mb1   = new MoneyBag (_12CHF, _7USD);
    _mb2   = new MoneyBag (_14CHF, _21USD);
  }


  /**
   * tests multiplication.
   *
   * {[12 CHF][7 USD]} *2 == {[24 CHF][14 USD]}
   */

  public void testBagMultiply ()
  {
    Money bag[] = { new Money (24, "CHF"), new Money (14, "USD") };

    assertEquals (new MoneyBag (bag), _mb1.multiply (2)); 
    assertEquals (_mb1.multiply (1), _mb1);
    assert       (_mb1.multiply (0).isNull ());
  }


  /**
   * tests negation.
   *
   * {[12 CHF][7 USD]} negate == {[-12 CHF][-7 USD]}
   */

  public void testBagNegate ()
  {
    Money bag[] = { new Money (-12, "CHF"), new Money (-7, "USD") };

    assertEquals (new MoneyBag (bag), _mb1.negate ());
  }


  /**
   * test adding to a bag.
   *
   * {[12 CHF][7 USD]} + [14 CHF] == {[26 CHF][7 USD]}
   */

  public void testBagSimpleAdd ()
  {
    Money bag[] = { new Money (26, "CHF"), new Money (7, "USD") };
 
    assertEquals (new MoneyBag (bag), _mb1.add (_14CHF));
  }


  /**
   * test subtraction.
   *
   * {[12 CHF][7 USD]} - {[14 CHF][21 USD] == {[-2 CHF][-14 USD]}
   */

  public void testBagSubtract ()
  {
    Money bag[] = { new Money (-2, "CHF"), new Money (-14, "USD") };

    assertEquals (new MoneyBag (bag), _mb1.subtract (_mb2));
  }


  /**
   * test addition.
   *
   * {[12 CHF][7 USD]} + {[14 CHF][21 USD]} == {[26 CHF][28 USD]}
   */

  public void testBagSumAdd ()
  {
    Money bag[] = { new Money (26, "CHF"), new Money (28, "USD") };

    assertEquals (new MoneyBag (bag), _mb1.add (_mb2));
  }


  /**
   * test isNull()
   */

  public void testIsNull ()
  {
    Money bag[] = { new Money (0, "CHF"), new Money (0, "USD") };

    assert (_mb1.subtract (_mb1).isNull ()); 
    assert (new MoneyBag (bag).isNull ());
  }


  /**
   * test addition.
   *
   * [12 CHF] + [7 USD] == {[12 CHF][7 USD]}
   */

  public void testMixedSimpleAdd ()
  {
    Money bag[] = { _12CHF, _7USD };

    assertEquals (new MoneyBag (bag), _12CHF.add (_7USD));
  }


  /**
   * test equals() for money bag.
   */

  public void testMoneyBagEquals ()
  {
    assert       (!_mb1.equals (null)); 
    assertEquals (_mb1, _mb1);

    MoneyBag equal = new MoneyBag (new Money (12, "CHF"), new Money (7, "USD"));

    assert ( _mb1.equals   (equal));
    assert (!_mb1.equals   (_12CHF));
    assert (!_12CHF.equals (_mb1));
    assert (!_mb1.equals   (_mb2));
  }


  /**
   * test hash function for money bag
   */
 
  public void testMoneyBagHash ()
  {
    MoneyBag equal = new MoneyBag (new Money (12, "CHF"), new Money (7, "USD"));

    assertEquals (_mb1.hashCode (), equal.hashCode ());
  }


  /**
   * test equals() for Money.
   */

  public void testMoneyEquals ()
  {
    assert (!_12CHF.equals (null)); 

    Money equalMoney = new Money (12, "CHF");

    assertEquals ( _12CHF, _12CHF);
    assertEquals ( _12CHF, equalMoney);
    assertEquals ( _12CHF.hashCode (), equalMoney.hashCode ());
    assert       (!_12CHF.equals (_14CHF));
  }


  /**
   * test hash function for money.
   */

  public void testMoneyHash ()
  {
    Money equal = new Money (12, "CHF");

    assert       (!_12CHF.equals (null)); 
    assertEquals ( _12CHF.hashCode (), equal.hashCode ());
  }


  /**
   * first normalization test.
   */

  public void testNormalize ()
  {
    Money    bag[]      = { new Money (26, "CHF"),
                            new Money (28, "CHF"),
                            new Money ( 6, "CHF") };
    MoneyBag moneyBag   = new MoneyBag (bag);
    Money    expected[] = { new Money (60, "CHF") };

    // note: expected is still a MoneyBag

    MoneyBag expectedBag = new MoneyBag (expected);

    assertEquals (expectedBag, moneyBag);
  }


  /**
   * second normalization test
   *
   * {[12 CHF][7 USD]} - [12 CHF] == [7 USD]
   */

  public void testNormalize2 ()
  {
    Money expected = new Money (7, "USD");

    assertEquals (expected, _mb1.subtract (_12CHF));
  }


  /**
   * third normalization test.
   *
   * {[12 CHF][7 USD]} - {[12 CHF][3 USD]} == [4 USD]
   */

  public void testNormalize3 ()
  {
    Money    s1[]     = { new Money (12, "CHF"), new Money (3, "USD") };
    MoneyBag ms1      = new MoneyBag (s1);
    Money    expected = new Money    (4, "USD");
 
    assertEquals (expected, _mb1.subtract (ms1));
  }


  /**
   * fourth normalization test.
   *
   * [12 CHF] - {[12 CHF][3 USD]} == [-3 USD]
   */

  public void testNormalize4 ()
  {
    Money    s1[]     = { new Money (12, "CHF"), new Money (3, "USD") };
    MoneyBag ms1      = new MoneyBag (s1);
    Money    expected = new Money    (-3, "USD");

    assertEquals (expected, _12CHF.subtract (ms1));
  }


  /**
   * test printing.
   */

  public void testPrint ()
  {
    assertEquals ("[12 CHF]", _12CHF.toString ());
  }


  /**
   * test addition.
   *
   * [12 CHF] + [14 CHF] == [26 CHF]
   */

  public void testSimpleAdd ()
  {
    Money expected = new Money (26, "CHF");

    assertEquals (expected, _12CHF.add (_14CHF));
  }


  /**
   * test simple bag addition
   *
   * [14 CHF] + {[12 CHF][7 USD]} == {[26 CHF][7 USD]}
   */

  public void testSimpleBagAdd ()
  {
    Money    bag[]    = { new Money (26, "CHF"), new Money (7, "USD") };
    MoneyBag expected = new MoneyBag (bag);

    assertEquals (expected, _14CHF.add (_mb1));
  }


  /**
   * test multiplication.
   *
   * [14 CHF] *2 == [28 CHF]
   */

  public void testSimpleMultiply ()
  {
    Money expected = new Money (28, "CHF");

    assertEquals (expected, _14CHF.multiply (2));
  }


  /**
   * test negation.
   *
   * [14 CHF] negate == [-14 CHF]
   */

  public void testSimpleNegate ()
  {
    Money expected = new Money (-14, "CHF");

    assertEquals (expected, _14CHF.negate ());
  }


  /**
   * test subtraction
   *
   * [14 CHF] - [12 CHF] == [2 CHF]
   */

  public void testSimpleSubtract ()
  {
    Money expected = new Money (2, "CHF");

    assertEquals (expected, _14CHF.subtract (_12CHF));
  }
}
