/**
 * Title:        Money
 * Description:  A simple money
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: Money.java,v 1.2 2001/12/14 11:27:27 aheilwag Exp $
 * 
 * $Log: Money.java,v $
 * Revision 1.2  2001/12/14 11:27:27  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:57:37  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.money.setup;


/**
 * A simple Money.
 *
 */

public class Money implements IMoney
{
  private int    _amount;
  private String _currency;


  /**
   * constructs a money from the given amount and currency.
   *
   * @param     anAmount    an integer value
   * @param     aCurreny    a currency string
   */

  public Money (int anAmount, String aCurrency)
  {
    _amount   = anAmount;
    _currency = aCurrency;
  }


  /**
   * adds a money to this money. Forwards the request to the addMoney helper.
   *
   * @param     someMoney    some money
   *
   * @return    resulting money
   */

  public IMoney add (IMoney someMoney)
  {
    return someMoney.addMoney (this);
  }


  /**
   * adds a money to this money.
   *
   * @param     someMoney    some money
   *
   * @return    resulting money
   */

  public IMoney addMoney (Money someMoney)
  {
    if (someMoney.currency ().equals (currency ()))
      return new Money (amount () + someMoney.amount (), currency ());

    return new MoneyBag (this, someMoney);
  }


  /**
   * adds a money bag to this money.
   *
   * @param     aMoneyBag   a money bag
   *
   * @return    resulting money
   */

  public IMoney addMoneyBag (MoneyBag aMoneyBag)
  {
    return aMoneyBag.addMoney (this);
  }


  /**
   * @return amount of money
   */

  public int amount ()
  {
    return _amount;
  }

  
  /**
   * @return currency
   */

  public String currency ()
  {
    return _currency;
  }


  /**
   * checks if two monies are equal.
   *
   * @param   anObject    somebody implementing <code>IMoney</code>
   *
   * @return  true if equal
   */

  public boolean equals (Object anObject)
  {
    if (isNull ())
      if (anObject instanceof IMoney)
        return ((IMoney)anObject).isNull ();

    if (anObject instanceof Money)
    {
      Money aMoney = (Money)anObject;

      return aMoney.currency ().equals (currency ())
         && (amount () == aMoney.amount ());
    }
    return false;
  }


  /**
   * @return hashcode for this money
   */

  public int hashCode ()
  {
    return _currency.hashCode () + _amount;
  }


  /**
   * @return true if money is 0
   */

  public boolean isNull ()
  {
    return amount () == 0;
  }


  /**
   * multiplies this money with a factor
   *
   * @param   aFactor    an integer factor
   *
   * @return  resulting money
   */

  public IMoney multiply (int aFactor)
  {
    return new Money (amount() * aFactor, currency ());
  }


  /**
   * negate the current money
   *
   * @return the resulting money
   */

  public IMoney negate ()
  {
    return new Money (-amount (), currency ());
  }


  /**
   * subtract some money from this money
   *
   * @param    someMoney    some money
   *
   * @return   the resulting money
   */
 
  public IMoney subtract (IMoney someMoney)
  {
    return add (someMoney.negate ());
  }


  /**
   * @return string representation for this money
   */

  public String toString ()
  {
    StringBuffer buffer = new StringBuffer ("[");

    buffer.append (amount   ());
    buffer.append (" ");
    buffer.append (currency ());
    buffer.append ("]");

    return buffer.toString ();
  }
}
