/**
 * Title:        TestVector
 * Description:  A sample test case illustrating the test of the Vector class
 * Copyright:    Copyright (c) 2000 Kent Beck and Erich Gamma
 *
 *   Permission to reproduce and create derivative works from the Software 
 *   ("Software Derivative Works") is hereby granted to you under the copyrights
 *   of Kent Beck and Erich Gamma. Kent Beck and Erich Gamma also grant you 
 *   the right to distribute the Software and Software Derivative Works.
 *
 *   Kent Beck and Erich Gamma licenses the Software to you on an "AS IS" basis,
 *   without warranty of any kind. Kent Beck and Erich Gamma HEREBY
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS, EITHER EXPRESS OR
 *   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR 
 *   CONDITIONS OF MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A 
 *   PARTICULAR PURPOSE. You are solely responsible for determining the 
 *   appropriateness of using the Software and assume all risks associated with
 *   the use and distribution of this Software, including but not limited to 
 *   the risks of program errors, damage to or loss of data, programs or 
 *   equipment, and unavailability or interruption of operations. KENT BECK AND
 *   ERICH GAMMA WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, 
 *   INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY ECONOMIC CONSEQUENTIAL DAMAGES 
 *   (INCLUDING LOST PROFITS OR SAVINGS), EVEN IF KENT BECK AND ERICH GAMMA HAD
 *   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. Kent Beck and Erich Gamma 
 *   will not be liable for the loss of, or damage to, your records or data, or
 *   any damages claimed by you based on a third party claim.
 *
 *   You agree to distribute the Software and any Software Derivatives under a
 *   license agreement that: 1) is sufficient to notify all licensees of the
 *   Software and Software Derivatives that Kent Beck and Erich Gamma assumes
 *   no liability for any claim that may arise regarding the Software or
 *   Software Derivatives, and 2) that disclaims all warranties, both express
 *   and implied, from Kent Beck and Erich Gamma regarding the Software and
 *   Software Derivatives. (If you include this Agreement with any distribution
 *   of the Software and Software Derivatives you will have met this
 *   requirement). You agree that you will not delete any copyright notices in
 *   the Software.
 *
 *   This Agreement is the exclusive statement of your rights in the Software
 *   as provided by Kent Beck and Erich Gamma. Except for the licenses
 *   granted to you in the second paragraph above, no other licenses are granted
 *   hereunder, by estoppel, implication or otherwise.
 *
 * @author Kent Beck
 * @author Erich Gamma
 *
 * $Id: TestVector.java,v 1.2 2001/12/14 11:27:27 aheilwag Exp $
 * 
 * $Log: TestVector.java,v $
 * Revision 1.2  2001/12/14 11:27:27  aheilwag
 * changes to the JUnit test cases moved to JUnitX and JUnitX now beeing a
 * module on top of JUnit
 *
 * Revision 1.1  2001/12/14 09:30:40  aheilwag
 * moved test cases from junit to junitx
 *
 * Revision 1.2  2000/11/10 19:58:06  aheilwag
 * added cvs tags
 *
 */


package tests.junitx.samples;


import junit.framework.*;
import java.util.Vector;


/**
 * A sample test case, testing <code>java.util.Vector</code>.
 */

public class TestVector extends TestCase
{
  protected Vector _empty;
  protected Vector _full;


  /**
   * constructor.
   *
   * @param     aName     a test name
   */

  public TestVector (String aName)
  {
    super (aName);
  }


  /**
   * creates a test suite.
   * 
   * @return a test suite
   */

  public static Test suite ()
  {
    return new TestSuite (TestVector.class);
  }


  /**
   * main entry point
   *
   * @param   args   commandline arguments
   */

  public static void main  (String[] args)
  {
    junit.textui.TestRunner.run (suite ());
  }


  /**
   * sets up the fixture.
   */

  protected void setUp ()
  {
    _empty = new Vector ();
    _full  = new Vector ();

    _full.addElement (new Integer (1));
    _full.addElement (new Integer (2));
    _full.addElement (new Integer (3));
  }


  /**
   * tests the capacity.
   */

  public void testCapacity ()
  {
    int size = _full.size (); 

    for (int i = 0; i < 100; i++)
      _full.addElement (new Integer (i));
    
    assert (_full.size () == 100 + size);
  }


  /**
   * tests cloning.
   */

  public void testClone ()
  {
    Vector clone = (Vector)_full.clone (); 

    assert (clone.size () == _full.size ());
    assert (clone.contains (new Integer (1)));
  }


  /**
   * tests contains.
   */

  public void testContains ()
  {
    assert (_full.contains   (new Integer (1)));  
    assert (!_empty.contains (new Integer (1)));
  }


  /**
   * tests elementAt.
   */

  public void testElementAt ()
  {
    Integer i = (Integer)_full.elementAt (0);

    assert (i.intValue () == 1);
    try
    { 
      Integer j = (Integer)_full.elementAt (_full.size ());
    }
    catch (ArrayIndexOutOfBoundsException e)
    {
      return;
    }
    fail ("Should raise an ArrayIndexOutOfBoundsException");
  }


  /**
   * tests removeAllElements.
   */

  public void testRemoveAll ()
  {
    _full.removeAllElements  ();
    _empty.removeAllElements ();

    assert (_full.isEmpty  ());
    assert (_empty.isEmpty ()); 
  }

 
  /**
   * tests removeElement.
   */

  public void testRemoveElement ()
  {
    _full.removeElement (new Integer (3));

    assert (!_full.contains (new Integer (3)) ); 
  }
}
